@@ -49,7 +49,6 @@ static const int ConnectTimeout = 60 * 1000;
 static const int KeepAliveInterval = 30 * 1000;
 static const int RateControlTimerDelay = 2000;
 static const int MinimalHeaderSize = 48;
-static const int FullHeaderSize = 68;
 static const char ProtocolId[] = "BitTorrent protocol";
 static const char ProtocolIdSize = 19;
 
@@ -386,15 +385,15 @@ bool PeerWireClient::canTransferMore() const
         || !outgoingBuffer.isEmpty() || !pendingBlocks.isEmpty();
 }
 
-void PeerWireClient::connectToHostImplementation(const QString &hostName,
-                                                 quint16 port, OpenMode openMode)
+void PeerWireClient::connectToHost(const QHostAddress &address,
+                                   quint16 port, OpenMode openMode)
 
 {
     setOpenMode(openMode);
-    socket.connectToHost(hostName, port, openMode);
+    socket.connectToHost(address, port, openMode);
 }
 
-void PeerWireClient::diconnectFromHostImplementation()
+void PeerWireClient::diconnectFromHost()
 {
     socket.disconnectFromHost();
 }
