@@ -50,8 +50,6 @@
 #  include "qabstractanimation.h"
 #endif
 
-QT_BEGIN_HEADER
-
 QT_BEGIN_NAMESPACE
 
 
@@ -86,6 +84,4 @@ private:
 
 QT_END_NAMESPACE
 
-QT_END_HEADER
-
 #endif // QANIMATIONSTATE_H
