@@ -352,7 +352,7 @@ void ConclusionPage::setVisible(bool visible)
 
 void ConclusionPage::printButtonClicked()
 {
-#ifndef QT_NO_PRINTER
+#if !defined(QT_NO_PRINTER) && !defined(QT_NO_PRINTDIALOG)
     QPrinter printer;
     QPrintDialog dialog(&printer, this);
     if (dialog.exec())
