@@ -197,7 +197,7 @@ void TextEdit::setupFileActions()
 #ifndef QT_NO_PRINTER
     a = new QAction(QIcon::fromTheme("document-print", QIcon(rsrcPath + "/fileprint.png")),
                     tr("&Print..."), this);
-    a->setPriority(QAction::LowPriority);    
+    a->setPriority(QAction::LowPriority);
     a->setShortcut(QKeySequence::Print);
     connect(a, SIGNAL(triggered()), this, SLOT(filePrint()));
     tb->addAction(a);
@@ -436,8 +436,7 @@ bool TextEdit::maybeSave()
 {
     if (!textEdit->document()->isModified())
         return true;
-    if (fileName.startsWith(QLatin1String(":/")))
-        return true;
+
     QMessageBox::StandardButton ret;
     ret = QMessageBox::warning(this, tr("Application"),
                                tr("The document has been modified.\n"
@@ -485,6 +484,8 @@ bool TextEdit::fileSave()
 {
     if (fileName.isEmpty())
         return fileSaveAs();
+    if (fileName.startsWith(QStringLiteral(":/")))
+        return fileSaveAs();
 
     QTextDocumentWriter writer(fileName);
     bool success = writer.write(textEdit->document());
@@ -511,7 +512,7 @@ bool TextEdit::fileSaveAs()
 
 void TextEdit::filePrint()
 {
-#ifndef QT_NO_PRINTER
+#if !defined(QT_NO_PRINTER) && !defined(QT_NO_PRINTDIALOG)
     QPrinter printer(QPrinter::HighResolution);
     QPrintDialog *dlg = new QPrintDialog(&printer, this);
     if (textEdit->textCursor().hasSelection())
@@ -525,7 +526,7 @@ void TextEdit::filePrint()
 
 void TextEdit::filePrintPreview()
 {
-#ifndef QT_NO_PRINTER
+#if !defined(QT_NO_PRINTER) && !defined(QT_NO_PRINTDIALOG)
     QPrinter printer(QPrinter::HighResolution);
     QPrintPreviewDialog preview(&printer, this);
     connect(&preview, SIGNAL(paintRequested(QPrinter*)), SLOT(printPreview(QPrinter*)));
