@@ -3,7 +3,7 @@
 ** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
-** This file is part of the QtGui module of the Qt Toolkit.
+** This file is part of the qmake application of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
@@ -44,7 +44,6 @@
 #include "meta.h"
 #include <qdir.h>
 #include <qregexp.h>
-#include <qcryptographichash.h>
 #include <qdebug.h>
 #include <stdlib.h>
 #include <time.h>
@@ -113,8 +112,8 @@ GBuildMakefileGenerator::write()
         /* this is for bulding an INTEGRITY application.
          * generate the .int integrate file and the .gpj INTEGRITY Application
          * project file, then go on with regular files */
-        t << "#!gbuild" << "\n";
-        t << "[INTEGRITY Application]" << "\n";
+        t << "#!gbuild\n";
+        t << "[INTEGRITY Application]\n";
         t << "\t:binDirRelative=.\n";
         t << "\t-o " << strtarget << "\n";
         t << intname << "\n";
@@ -125,24 +124,24 @@ GBuildMakefileGenerator::write()
         QFile f(intname);
         f.open(QIODevice::WriteOnly | QIODevice::Text | QIODevice::Truncate);
         QTextStream ti(&f);
-        ti << "# This is a file automatically generated by qmake" << "\n";
-        ti << "# Modifications will be lost next time you run qmake" << "\n";
-        ti << "Kernel" << "\n";
-        ti << "\tFilename\tDynamicDownload" << "\n";
-        ti << "EndKernel" << "\n" << "\n";
-        ti << "AddressSpace" << "\n";
+        ti << "# This is a file automatically generated by qmake\n";
+        ti << "# Modifications will be lost next time you run qmake\n";
+        ti << "Kernel\n";
+        ti << "\tFilename\tDynamicDownload\n";
+        ti << "EndKernel\n\n";
+        ti << "AddressSpace\n";
         ti << "\tName\t" << strtarget << "\n";
-        ti << "\tFilename\t" << strtarget << "_app" << "\n";
-        ti << "\tMemoryPoolSize\t0x100000" << "\n";
-        ti << "\tLanguage\tC++" << "\n";
+        ti << "\tFilename\t" << strtarget << "_app\n";
+        ti << "\tMemoryPoolSize\t0x100000\n";
+        ti << "\tLanguage\tC++\n";
         /* FIXME : heap size is huge to be big enough for every example
          * it should probably be tailored for each example, btu there is no
          * good way to guess that */
-        ti << "\tHeapSize\t0x00D00000" << "\n";
-        ti << "\tTask\tInitial" << "\n";
-        ti << "\t\tStackSize\t0x30000" << "\n";
-        ti << "\tEndTask" << "\n";
-        ti << "EndAddressSpace" << "\n";
+        ti << "\tHeapSize\t0x00D00000\n";
+        ti << "\tTask\tInitial\n";
+        ti << "\t\tStackSize\t0x30000\n";
+        ti << "\tEndTask\n";
+        ti << "EndAddressSpace\n";
         ti.flush();
 
         /* change current project file to <projectname>_app.gpj and continue
@@ -220,19 +219,19 @@ GBuildMakefileGenerator::write()
     filename.remove(qmake_getpwd());
 
     //HEADER
-    t << "#!gbuild" << "\n";
+    t << "#!gbuild\n";
 
     /* find the architecture out of the compiler name */
     if (filename.endsWith("projects.gpj")) {
         primaryTarget.remove(0, 5);
         t << "macro QT_BUILD_DIR=%expand_path(.)\n";
         t << "macro __OS_DIR=" << project->values("INTEGRITY_DIR").first() << "\n";
-        t << "primaryTarget=" << primaryTarget << "_integrity.tgt" << "\n";
+        t << "primaryTarget=" << primaryTarget << "_integrity.tgt\n";
         t << "customization=util/integrity/qt.bod\n";
     }
     /* project type */
     if (project->first("TEMPLATE") == "app") {
-        t << "[Program]" << "\n";
+        t << "[Program]\n";
         if (isnativebin) {
             t << "\t:binDir=bin\n";
             t << "\t-o " << strtarget << "\n";
@@ -241,16 +240,16 @@ GBuildMakefileGenerator::write()
             t << "\t-o " << strtarget << "_app\n";
         }
     } else if (project->first("TEMPLATE") == "lib") {
-        t << "[Library]" << "\n";
-        t << "\t:binDir=lib" << "\n";
-        t << "\t-o lib" << strtarget << ".a" << "\n";
+        t << "[Library]\n";
+        t << "\t:binDir=lib\n";
+        t << "\t-o lib" << strtarget << ".a\n";
     } else if (project->first("TEMPLATE") == "subdirs")
-        t << "[Project]" << "\n";
+        t << "[Project]\n";
     else
         t << project->first("TEMPLATE") << "\n";
 
     /* compilations options */
-    t << "\t:sourceDir=." << "\n";
+    t << "\t:sourceDir=.\n";
 
     t << "\t:outputDir=work" << relpath << "\n";
     if (filename.endsWith("projects.gpj")) {
@@ -261,7 +260,7 @@ GBuildMakefileGenerator::write()
         const ProStringList &l = project->values("QMAKE_CXXFLAGS");
         for (ProStringList::ConstIterator it = l.begin(); it != l.end(); ++it) {
             if ((*it).startsWith("-"))
-                t << "\n" << "\t" << (*it);
+                t << "\n\t" << (*it);
             else
                 t << " " << (*it);
         }
@@ -287,7 +286,7 @@ GBuildMakefileGenerator::write()
             const ProStringList &l = project->values(src[i]);
             for (ProStringList::ConstIterator it = l.begin(); it != l.end(); ++it) {
                 if ((*it).startsWith("-"))
-                    t << "\n" << "\t" << (*it);
+                    t << "\n\t" << (*it);
                 else
                     t << " " << (*it);
             }
@@ -418,6 +417,7 @@ QString GBuildMakefileGenerator::writeOne(QString filename, QString pathtoremove
 bool
 GBuildMakefileGenerator::openOutput(QFile &file, const QString &build) const
 {
+    Q_UNUSED(build)
     debug_msg(1, "file is %s", file.fileName().toLatin1().constData());
     QFileInfo fi(file);
     if (fi.filePath().isEmpty())
