@@ -106,20 +106,8 @@ UnixMakefileGenerator::init()
         return; /* subdirs is done */
     }
 
-    //If the TARGET looks like a path split it into DESTDIR and the resulting TARGET
-    if(!project->isEmpty("TARGET")) {
+    if (!project->isEmpty("TARGET"))
         project->values("TARGET") = escapeFilePaths(project->values("TARGET"));
-        ProString targ = unescapeFilePath(project->first("TARGET"));
-        int slsh = qMax(targ.lastIndexOf('/'), targ.lastIndexOf(Option::dir_sep));
-        if(slsh != -1) {
-            if(project->isEmpty("DESTDIR"))
-                project->values("DESTDIR").append("");
-            else if(project->first("DESTDIR").right(1) != Option::dir_sep)
-                project->values("DESTDIR") = ProStringList(project->first("DESTDIR") + Option::dir_sep);
-            project->values("DESTDIR") = ProStringList(project->first("DESTDIR") + targ.left(slsh+1));
-            project->values("TARGET") = ProStringList(targ.mid(slsh+1));
-        }
-    }
 
     project->values("QMAKE_ORIG_TARGET") = project->values("TARGET");
     project->values("QMAKE_ORIG_DESTDIR") = project->values("DESTDIR");
@@ -142,6 +130,7 @@ UnixMakefileGenerator::init()
     project->values("QMAKE_L_FLAG")
             << (project->isActiveConfig("rvct_linker") ? "--userlibpath "
               : project->isActiveConfig("armcc_linker") ? "-L--userlibpath="
+              : project->isActiveConfig("ti_linker") ? "--search_path="
               : "-L");
     ProStringList ldadd;
     if(!project->isEmpty("QMAKE_LIBDIR")) {
@@ -153,7 +142,7 @@ UnixMakefileGenerator::init()
         }
     }
     ldadd += project->values("QMAKE_LIBDIR_FLAGS");
-    if (project->isActiveConfig("macx")) {
+    if (project->isActiveConfig("mac")) {
         if (!project->isEmpty("QMAKE_FRAMEWORKPATH")) {
             const ProStringList &fwdirs = project->values("QMAKE_FRAMEWORKPATH");
             for (int i = 0; i < fwdirs.size(); ++i)
@@ -255,13 +244,13 @@ UnixMakefileGenerator::init()
 
         const ProKey runComp("QMAKE_RUN_" + compiler);
         if(project->isEmpty(runComp))
-            project->values(runComp).append("$(" + compiler + ") " + compile_flag + " -o $obj $src");
+            project->values(runComp).append("$(" + compiler + ") " + compile_flag + " " + var("QMAKE_CC_O_FLAG") + "$obj $src");
         const ProKey runCompImp("QMAKE_RUN_" + compiler + "_IMP");
         if(project->isEmpty(runCompImp))
-            project->values(runCompImp).append("$(" + compiler + ") " + compile_flag + " -o \"$@\" \"$<\"");
+            project->values(runCompImp).append("$(" + compiler + ") " + compile_flag + " " + var("QMAKE_CC_O_FLAG") + "\"$@\" \"$<\"");
     }
 
-    if(project->isActiveConfig("macx") && !project->isEmpty("TARGET") && !project->isActiveConfig("compile_libtool") &&
+    if (project->isActiveConfig("mac") && !project->isEmpty("TARGET") && !project->isActiveConfig("compile_libtool") &&
        ((project->isActiveConfig("build_pass") || project->isEmpty("BUILDS")))) {
         ProString bundle;
         if(project->isActiveConfig("bundle") && !project->isEmpty("QMAKE_BUNDLE_EXTENSION")) {
@@ -490,10 +479,12 @@ UnixMakefileGenerator::findLibraries()
                 } else if(opt.startsWith("-l")) {
                     if (project->isActiveConfig("rvct_linker") || project->isActiveConfig("armcc_linker")) {
                         (*it) = "lib" + opt.mid(2) + ".so";
+                    } else if (project->isActiveConfig("ti_linker")) {
+                        (*it) = opt.mid(2);
                     } else {
                         stub = opt.mid(2);
                     }
-                } else if (target_mode == TARG_MACX_MODE && opt.startsWith("-framework")) {
+                } else if (target_mode == TARG_MAC_MODE && opt.startsWith("-framework")) {
                     if (opt.length() == 10)
                         ++it;
                     // Skip
@@ -607,11 +598,11 @@ UnixMakefileGenerator::processPrlFiles()
                             break;
                         }
                     }
-                } else if (target_mode == TARG_MACX_MODE && opt.startsWith("-F")) {
+                } else if (target_mode == TARG_MAC_MODE && opt.startsWith("-F")) {
                     QMakeLocalFileName f(opt.right(opt.length()-2));
                     if(!frameworkdirs.contains(f))
                         frameworkdirs.insert(fwidx++, f);
-                } else if (target_mode == TARG_MACX_MODE && opt.startsWith("-framework")) {
+                } else if (target_mode == TARG_MAC_MODE && opt.startsWith("-framework")) {
                     if(opt.length() > 11)
                         opt = opt.mid(11);
                     else
@@ -638,7 +629,7 @@ UnixMakefileGenerator::processPrlFiles()
             ProStringList &prl_libs = project->values("QMAKE_CURRENT_PRL_LIBS");
             if(!prl_libs.isEmpty()) {
                 for(int prl = 0; prl < prl_libs.size(); ++prl)
-                    l.insert(lit+prl+1, prl_libs.at(prl));
+                    l.insert(lit+prl+1, escapeFilePath(prl_libs.at(prl).toQString()));
                 prl_libs.clear();
             }
         }
@@ -650,7 +641,7 @@ UnixMakefileGenerator::processPrlFiles()
                 ProKey arch("default");
                 ProString opt = l.at(lit).trimmed();
                 if(opt.startsWith("-")) {
-                    if (target_mode == TARG_MACX_MODE && opt.startsWith("-Xarch")) {
+                    if (target_mode == TARG_MAC_MODE && opt.startsWith("-Xarch")) {
                         if (opt.length() > 7) {
                             arch = opt.mid(7).toKey();
                             opt = l.at(++lit);
@@ -658,7 +649,7 @@ UnixMakefileGenerator::processPrlFiles()
                     }
 
                     if (opt.startsWith(libArg) ||
-                       (target_mode == TARG_MACX_MODE && opt.startsWith("-F"))) {
+                       (target_mode == TARG_MAC_MODE && opt.startsWith("-F"))) {
                         if(!lflags[arch].contains(opt))
                             lflags[arch].append(opt);
                     } else if(opt.startsWith("-l") || opt == "-pthread") {
@@ -666,12 +657,12 @@ UnixMakefileGenerator::processPrlFiles()
                         if (lflags[arch].contains(opt))
                             lflags[arch].removeAll(opt);
                         lflags[arch].append(opt);
-                    } else if (target_mode == TARG_MACX_MODE && opt.startsWith("-framework")) {
+                    } else if (target_mode == TARG_MAC_MODE && opt.startsWith("-framework")) {
                         if(opt.length() > 11)
                             opt = opt.mid(11);
                         else {
                             opt = l.at(++lit);
-                            if (target_mode == TARG_MACX_MODE && opt.startsWith("-Xarch"))
+                            if (target_mode == TARG_MAC_MODE && opt.startsWith("-Xarch"))
                                 opt = l.at(++lit); // The user has done the right thing and prefixed each part
                         }
                         bool found = false;
@@ -724,7 +715,7 @@ UnixMakefileGenerator::defaultInstall(const QString &t)
     if(t != "target" || project->first("TEMPLATE") == "subdirs")
         return QString();
 
-    bool bundle = false;
+    enum { NoBundle, SolidBundle, SlicedBundle } bundle = NoBundle;
     const QString root = "$(INSTALL_ROOT)";
     ProStringList &uninst = project->values(ProKey(t + ".uninstall"));
     QString ret, destdir = project->first("DESTDIR").toQString();
@@ -740,7 +731,7 @@ UnixMakefileGenerator::defaultInstall(const QString &t)
     const ProStringList &targets = project->values(ProKey(t + ".targets"));
     if(!project->isEmpty("QMAKE_BUNDLE")) {
         target = project->first("QMAKE_BUNDLE").toQString();
-        bundle = true;
+        bundle = project->isActiveConfig("sliced_bundle") ? SlicedBundle : SolidBundle;
     } else if(project->first("TEMPLATE") == "app") {
         target = "$(QMAKE_TARGET)";
     } else if(project->first("TEMPLATE") == "lib") {
@@ -765,7 +756,7 @@ UnixMakefileGenerator::defaultInstall(const QString &t)
         uninst.append("-$(DEL_FILE) \"" + dst + "\"");
     }
 
-    if(!bundle && project->isActiveConfig("compile_libtool")) {
+    if (bundle == NoBundle && project->isActiveConfig("compile_libtool")) {
         QString src_targ = target;
         if(src_targ == "$(TARGET)")
             src_targ = "$(TARGETL)";
@@ -778,23 +769,36 @@ UnixMakefileGenerator::defaultInstall(const QString &t)
         QString src_targ = target;
         if(!destdir.isEmpty())
             src_targ = Option::fixPathToTargetOS(destdir + target, false);
-        QString dst_targ = filePrefixRoot(root, fileFixify(targetdir + target, FileFixifyAbsolute));
-        if(bundle) {
-            if(!ret.isEmpty())
-                ret += "\n\t";
-            ret += "$(DEL_FILE) -r \"" + dst_targ + "\"\n\t";
+        QString plain_targ = filePrefixRoot(root, fileFixify(targetdir + target, FileFixifyAbsolute));
+        QString dst_targ = plain_targ;
+        if (bundle != NoBundle) {
+            QString suffix;
+            if (project->first("TEMPLATE") == "lib")
+                suffix = "/Versions/" + project->first("QMAKE_FRAMEWORK_VERSION") + "/$(TARGET)";
+            else
+                suffix = "/" + project->first("QMAKE_BUNDLE_LOCATION") + "/$(QMAKE_TARGET)";
+            dst_targ += suffix;
+            if (bundle == SolidBundle) {
+                if (!ret.isEmpty())
+                    ret += "\n\t";
+                ret += "$(DEL_FILE) -r \"" + plain_targ + "\"\n\t";
+            } else {
+                src_targ += suffix;
+            }
         }
         if(!ret.isEmpty())
             ret += "\n\t";
 
         QString copy_cmd("-");
-        if (bundle)
-            copy_cmd += "$(INSTALL_DIR)";
-        else if (project->first("TEMPLATE") == "lib" && project->isActiveConfig("staticlib"))
-            copy_cmd += "$(INSTALL_FILE)";
-        else
-            copy_cmd += "$(INSTALL_PROGRAM)";
-        copy_cmd += " \"" + src_targ + "\" \"" + dst_targ + "\"";
+        if (bundle == SolidBundle) {
+            copy_cmd += "$(INSTALL_DIR) \"" + src_targ + "\" \"" + plain_targ + "\"";
+        } else if (project->first("TEMPLATE") == "lib" && project->isActiveConfig("staticlib")) {
+            copy_cmd += "$(INSTALL_FILE) \"" + src_targ + "\" \"" + dst_targ + "\"";
+        } else {
+            if (bundle == SlicedBundle)
+                ret += mkdir_p_asstring("\"`dirname \"" + dst_targ + "\"`\"", false) + "\n\t";
+            copy_cmd += "$(INSTALL_PROGRAM) \"" + src_targ + "\" \"" + dst_targ + "\"";
+        }
         if(project->first("TEMPLATE") == "lib" && !project->isActiveConfig("staticlib")
            && project->values(ProKey(t + ".CONFIG")).indexOf("fix_rpath") != -1) {
             if(!project->isEmpty("QMAKE_FIX_RPATH")) {
@@ -814,26 +818,42 @@ UnixMakefileGenerator::defaultInstall(const QString &t)
         if(project->first("TEMPLATE") == "lib" && project->isActiveConfig("staticlib")) {
             if(!project->isEmpty("QMAKE_RANLIB"))
                 ret += QString("\n\t$(RANLIB) \"") + dst_targ + "\"";
-        } else if(!project->isActiveConfig("debug") && !project->isActiveConfig("nostrip") && !project->isEmpty("QMAKE_STRIP")) {
+        } else if (!project->isActiveConfig("debug_info") && !project->isActiveConfig("nostrip")
+                   && !project->isEmpty("QMAKE_STRIP")) {
             ret += "\n\t-$(STRIP)";
-            if(project->first("TEMPLATE") == "lib" && !project->isEmpty("QMAKE_STRIPFLAGS_LIB"))
-                ret += " " + var("QMAKE_STRIPFLAGS_LIB");
-            else if(project->first("TEMPLATE") == "app" && !project->isEmpty("QMAKE_STRIPFLAGS_APP"))
-                ret += " " + var("QMAKE_STRIPFLAGS_APP");
-            if(bundle)
-                ret = " \"" + dst_targ + "/Contents/MacOS/$(QMAKE_TARGET)\"";
-            else
-                ret += " \"" + dst_targ + "\"";
+            if (project->first("TEMPLATE") == "lib") {
+                if (!project->isEmpty("QMAKE_STRIPFLAGS_LIB"))
+                    ret += " " + var("QMAKE_STRIPFLAGS_LIB");
+            } else if (project->first("TEMPLATE") == "app") {
+                if (!project->isEmpty("QMAKE_STRIPFLAGS_APP"))
+                    ret += " " + var("QMAKE_STRIPFLAGS_APP");
+            }
+            ret += " \"" + dst_targ + "\"";
         }
         if(!uninst.isEmpty())
             uninst.append("\n\t");
-        if(bundle)
-            uninst.append("-$(DEL_FILE) -r \"" + dst_targ + "\"");
+        if (bundle == SolidBundle)
+            uninst.append("-$(DEL_FILE) -r \"" + plain_targ + "\"");
         else
             uninst.append("-$(DEL_FILE) \"" + dst_targ + "\"");
+        if (bundle == SlicedBundle) {
+            int dstlen = project->first("DESTDIR").length();
+            foreach (const ProString &src, project->values("QMAKE_BUNDLED_FILES")) {
+                QString file = unescapeFilePath(src.toQString()).mid(dstlen);
+                QString dst = filePrefixRoot(root, fileFixify(targetdir + file, FileFixifyAbsolute));
+                if (!ret.isEmpty())
+                    ret += "\n\t";
+                ret += mkdir_p_asstring("\"`dirname \"" + dst + "\"`\"", false) + "\n\t";
+                ret += "-$(DEL_FILE) \"" + dst + "\"\n\t"; // Can't overwrite symlinks to directories
+                ret += "-$(INSTALL_DIR) " + src + " \"" + dst + "\""; // Use cp -R to copy symlinks
+                if (!uninst.isEmpty())
+                    uninst.append("\n\t");
+                uninst.append("-$(DEL_FILE) \"" + dst + "\"");
+            }
+        }
         if(!links.isEmpty()) {
             for(int i = 0; i < links.size(); ++i) {
-                if (target_mode == TARG_UNIX_MODE || target_mode == TARG_MACX_MODE) {
+                if (target_mode == TARG_UNIX_MODE || target_mode == TARG_MAC_MODE) {
                     QString link = Option::fixPathToTargetOS(destdir + links[i], false);
                     int lslash = link.lastIndexOf(Option::dir_sep);
                     if(lslash != -1)
