@@ -63,6 +63,7 @@ const char _GUIDLexYaccFiles[]         = "{E12AE0D2-192F-4d59-BD23-7D3FA58D3183}
 const char _GUIDTranslationFiles[]     = "{639EADAA-A684-42e4-A9AD-28FC9BCB8F7C}";
 const char _GUIDFormFiles[]            = "{99349809-55BA-4b9d-BF79-8FDBB0286EB3}";
 const char _GUIDExtraCompilerFiles[]   = "{E0D8C965-CC5F-43d7-AD63-FAEF0BBC0F85}";
+const char _GUIDDeploymentFiles[]      = "{D9D6E243-F8AF-46E4-B9FD-80ECBC20BA3E}";
 QT_END_NAMESPACE
 
 #ifdef Q_OS_WIN32
@@ -77,6 +78,8 @@ struct DotNetCombo {
     const char *regKey;
 } dotNetCombo[] = {
 #ifdef Q_OS_WIN64
+    {NET2013, "MSVC.NET 2013 (12.0)", "Software\\Wow6432Node\\Microsoft\\VisualStudio\\12.0\\Setup\\VC\\ProductDir"},
+    {NET2013, "MSVC.NET 2013 Express Edition (12.0)", "Software\\Wow6432Node\\Microsoft\\VCExpress\\12.0\\Setup\\VC\\ProductDir"},
     {NET2012, "MSVC.NET 2012 (11.0)", "Software\\Wow6432Node\\Microsoft\\VisualStudio\\11.0\\Setup\\VC\\ProductDir"},
     {NET2012, "MSVC.NET 2012 Express Edition (11.0)", "Software\\Wow6432Node\\Microsoft\\VCExpress\\11.0\\Setup\\VC\\ProductDir"},
     {NET2010, "MSVC.NET 2010 (10.0)", "Software\\Wow6432Node\\Microsoft\\VisualStudio\\10.0\\Setup\\VC\\ProductDir"},
@@ -88,6 +91,8 @@ struct DotNetCombo {
     {NET2003, "MSVC.NET 2003 (7.1)", "Software\\Wow6432Node\\Microsoft\\VisualStudio\\7.1\\Setup\\VC\\ProductDir"},
     {NET2002, "MSVC.NET 2002 (7.0)", "Software\\Wow6432Node\\Microsoft\\VisualStudio\\7.0\\Setup\\VC\\ProductDir"},
 #else
+    {NET2013, "MSVC.NET 2013 (12.0)", "Software\\Microsoft\\VisualStudio\\12.0\\Setup\\VC\\ProductDir"},
+    {NET2013, "MSVC.NET 2013 Express Edition (12.0)", "Software\\Microsoft\\VCExpress\\12.0\\Setup\\VC\\ProductDir"},
     {NET2012, "MSVC.NET 2012 (11.0)", "Software\\Microsoft\\VisualStudio\\11.0\\Setup\\VC\\ProductDir"},
     {NET2012, "MSVC.NET 2012 Express Edition (11.0)", "Software\\Microsoft\\VCExpress\\11.0\\Setup\\VC\\ProductDir"},
     {NET2010, "MSVC.NET 2010 (10.0)", "Software\\Microsoft\\VisualStudio\\10.0\\Setup\\VC\\ProductDir"},
@@ -119,17 +124,17 @@ DotNET which_dotnet_version()
     // Fallback to .NET 2002
     current_version = NET2002;
 
-    QStringList warnPath;
+    const DotNetCombo *lowestInstalledVersion = 0;
     QHash<DotNET, QString> installPaths;
     int installed = 0;
     int i = 0;
     for(; dotNetCombo[i].version; ++i) {
         QString path = qt_readRegistryKey(HKEY_LOCAL_MACHINE, dotNetCombo[i].regKey);
         if (!path.isEmpty() && installPaths.value(dotNetCombo[i].version) != path) {
-            installPaths.insert(dotNetCombo[i].version, path);
+            lowestInstalledVersion = &dotNetCombo[i];
+            installPaths.insert(lowestInstalledVersion->version, path);
             ++installed;
-            current_version = dotNetCombo[i].version;
-                        warnPath += QString("%1").arg(dotNetCombo[i].versionStr);
+            current_version = lowestInstalledVersion->version;
         }
     }
 
@@ -138,35 +143,22 @@ DotNET which_dotnet_version()
 
     // More than one version installed, search directory path
     QString paths = qgetenv("PATH");
-    QStringList pathlist = paths.toLower().split(";");
-
-    i = installed = 0;
-    for(; dotNetCombo[i].version; ++i) {
-        QString productPath = qt_readRegistryKey(HKEY_LOCAL_MACHINE, dotNetCombo[i].regKey).toLower();
-                if (productPath.isEmpty())
-                        continue;
-        QStringList::iterator it;
-        for(it = pathlist.begin(); it != pathlist.end(); ++it) {
-            if((*it).contains(productPath)) {
-                ++installed;
+    const QStringList pathlist = paths.split(QLatin1Char(';'));
+    foreach (const QString &path, pathlist) {
+        for (i = 0; dotNetCombo[i].version; ++i) {
+            const QString productPath = installPaths.value(dotNetCombo[i].version);
+            if (productPath.isEmpty())
+                continue;
+            if (path.startsWith(productPath, Qt::CaseInsensitive)) {
                 current_version = dotNetCombo[i].version;
-                warnPath += QString("%1 in path").arg(dotNetCombo[i].versionStr);
-                                break;
+                return current_version;
             }
         }
     }
-        switch(installed) {
-        case 1:
-                break;
-        case 0:
-                warn_msg(WarnLogic, "Generator: MSVC.NET: Found more than one version of Visual Studio, but"
-                                 " none in your path! Fallback to lowest version (%s)", warnPath.join(", ").toLatin1().data());
-                break;
-        default:
-                warn_msg(WarnLogic, "Generator: MSVC.NET: Found more than one version of Visual Studio in"
-                                 " your path! Fallback to lowest version (%s)", warnPath.join(", ").toLatin1().data());
-                break;
-        }
+
+    warn_msg(WarnLogic, "Generator: MSVC.NET: Found more than one version of Visual Studio, but"
+                        " none in your PATH. Falling back to lowest version (%s)",
+                        qPrintable(lowestInstalledVersion->versionStr));
 
     return current_version;
 #endif
@@ -183,6 +175,8 @@ const char _slnHeader100[]      = "Microsoft Visual Studio Solution File, Format
                                   "\n# Visual Studio 2010";
 const char _slnHeader110[]      = "Microsoft Visual Studio Solution File, Format Version 12.00"
                                   "\n# Visual Studio 2012";
+const char _slnHeader120[]      = "Microsoft Visual Studio Solution File, Format Version 12.00"
+                                  "\n# Visual Studio 2013";
                                   // The following UUID _may_ change for later servicepacks...
                                   // If so we need to search through the registry at
                                   // HKEY_LOCAL_MACHINE\SOFTWARE\Microsoft\VisualStudio\7.0\Projects
@@ -196,13 +190,14 @@ const char _slnProjectMid[]     = "\") = ";
 const char _slnProjectEnd[]     = "\nEndProject";
 const char _slnGlobalBeg[]      = "\nGlobal";
 const char _slnGlobalEnd[]      = "\nEndGlobal";
-const char _slnSolutionConf[]   = "\n\tGlobalSection(SolutionConfiguration) = preSolution"
-                                  "\n\t\tConfigName.0 = Debug|Win32"
-                                  "\n\t\tConfigName.1 = Release|Win32"
+const char _slnSolutionConf[]   = "\n\tGlobalSection(SolutionConfigurationPlatforms) = preSolution"
+                                  "\n\t\tDebug|Win32 = Debug|Win32"
+                                  "\n\t\tRelease|Win32 = Release|Win32"
                                   "\n\tEndGlobalSection";
-const char _slnProjDepBeg[]     = "\n\tGlobalSection(ProjectDependencies) = postSolution";
-const char _slnProjDepEnd[]     = "\n\tEndGlobalSection";
-const char _slnProjConfBeg[]    = "\n\tGlobalSection(ProjectConfiguration) = postSolution";
+
+const char _slnProjDepBeg[]     = "\n\tProjectSection(ProjectDependencies) = postProject";
+const char _slnProjDepEnd[]     = "\n\tEndProjectSection";
+const char _slnProjConfBeg[]    = "\n\tGlobalSection(ProjectConfigurationPlatforms) = postSolution";
 const char _slnProjRelConfTag1[]= ".Release|%1.ActiveCfg = Release|";
 const char _slnProjRelConfTag2[]= ".Release|%1.Build.0 = Release|";
 const char _slnProjDbgConfTag1[]= ".Debug|%1.ActiveCfg = Debug|";
@@ -333,7 +328,7 @@ QUuid VcprojGenerator::getProjectUUID(const QString &filename)
     // If none, create one based on the MD5 of absolute project path
     if(uuid.isNull() || !filename.isEmpty()) {
         QString abspath = Option::fixPathToLocalOS(filename.isEmpty()?project->first("QMAKE_MAKEFILE").toQString():filename);
-        QByteArray digest = QCryptographicHash::hash(abspath.toUtf8(), QCryptographicHash::Md5);
+        QByteArray digest = QCryptographicHash::hash(abspath.toUtf8(), QCryptographicHash::Sha1);
         memcpy((unsigned char*)(&uuid), digest.constData(), sizeof(QUuid));
         validUUID = !uuid.isNull();
         uuid.data4[0] = (uuid.data4[0] & 0x3F) | 0x80; // UV_DCE variant
@@ -441,7 +436,7 @@ ProStringList VcprojGenerator::collectDependencies(QMakeProject *proj, QHash<QSt
                 if (!qmake_setpwd(dir))
                     fprintf(stderr, "Cannot find directory: %s", dir.toLatin1().constData());
             }
-            Option::output_dir = Option::globals->shadowedPath(QDir::cleanPath(fi.absoluteFilePath()));
+            Option::output_dir = Option::globals->shadowedPath(QDir::cleanPath(dir));
             if (tmp_proj.read(fn)) {
                 // Check if all requirements are fulfilled
                 if (!tmp_proj.isEmpty("QMAKE_FAILED_REQUIREMENTS")) {
@@ -506,7 +501,7 @@ ProStringList VcprojGenerator::collectDependencies(QMakeProject *proj, QHash<QSt
                     // We want to store it as the .lib name.
                     if (newDep->target.endsWith(".dll"))
                         newDep->target = newDep->target.left(newDep->target.length()-3) + "lib";
-                    projGuids.insert(val.first, newDep->target);
+                    projGuids.insert(newDep->orig_target, newDep->target);
 
                     if (val.second.size()) {
                         const ProStringList depends = val.second;
@@ -571,10 +566,6 @@ ProStringList VcprojGenerator::collectDependencies(QMakeProject *proj, QHash<QSt
 #endif
                     solution_cleanup.append(newDep);
                     solution_depends.insert(newDep->target, newDep);
-                    t << _slnProjectBeg << _slnMSVCvcprojGUID << _slnProjectMid
-                        << "\"" << newDep->orig_target << "\", \"" << newDep->vcprojFile
-                        << "\", \"" << newDep->uuid << "\"";
-                    t << _slnProjectEnd;
                 }
 nextfile:
                 qmake_setpwd(oldpwd);
@@ -595,6 +586,9 @@ void VcprojGenerator::writeSubDirs(QTextStream &t)
     }
 
     switch(which_dotnet_version()) {
+    case NET2013:
+        t << _slnHeader120;
+        break;
     case NET2012:
         t << _slnHeader110;
         break;
@@ -622,8 +616,6 @@ void VcprojGenerator::writeSubDirs(QTextStream &t)
     QHash<QString, VcsolutionDepend*> solution_depends;
     QList<VcsolutionDepend*> solution_cleanup;
 
-    QString oldpwd = qmake_getpwd();
-
     // Make sure that all temp projects are configured
     // for release so that the depends are created
     // without the debug <lib>dxxx.lib name mangling
@@ -636,6 +628,30 @@ void VcprojGenerator::writeSubDirs(QTextStream &t)
     QHash<QString, ProStringList> subdirProjectLookup;
     collectDependencies(project, profileLookup, projGuids, extraSubdirs, solution_depends, solution_cleanup, t, subdirProjectLookup);
 
+    // write out projects
+    for (QList<VcsolutionDepend*>::Iterator it = solution_cleanup.begin(); it != solution_cleanup.end(); ++it) {
+        t << _slnProjectBeg << _slnMSVCvcprojGUID << _slnProjectMid
+            << "\"" << (*it)->orig_target << "\", \"" << (*it)->vcprojFile
+            << "\", \"" << (*it)->uuid << "\"";
+
+        debug_msg(1, "Project %s has dependencies: %s", (*it)->target.toLatin1().constData(), (*it)->dependencies.join(" ").toLatin1().constData());
+
+        bool hasDependency = false;
+        for (QStringList::iterator dit = (*it)->dependencies.begin();  dit != (*it)->dependencies.end(); ++dit) {
+            if (VcsolutionDepend *vc = solution_depends[*dit]) {
+                if (!hasDependency) {
+                    hasDependency = true;
+                    t << _slnProjDepBeg;
+                }
+                t << "\n\t\t" << vc->uuid << " = " << vc->uuid;
+            }
+        }
+        if (hasDependency)
+            t << _slnProjDepEnd;
+
+        t << _slnProjectEnd;
+    }
+
     t << _slnGlobalBeg;
 
     QHashIterator<VcsolutionDepend *, QStringList> extraIt(extraSubdirs);
@@ -651,7 +667,9 @@ void VcprojGenerator::writeSubDirs(QTextStream &t)
         }
     }
     QString slnConf = _slnSolutionConf;
-    if (!project->isEmpty("CE_SDK") && !project->isEmpty("CE_ARCH")) {
+    if (!project->isEmpty("VCPROJ_ARCH")) {
+        slnConf.replace(QString("|Win32"), "|" + project->first("VCPROJ_ARCH"));
+    } else if (!project->isEmpty("CE_SDK") && !project->isEmpty("CE_ARCH")) {
         QString slnPlatform = QString("|") + project->values("CE_SDK").join(' ') + " (" + project->first("CE_ARCH") + ")";
         slnConf.replace(QString("|Win32"), slnPlatform);
     } else if (is64Bit) {
@@ -659,26 +677,18 @@ void VcprojGenerator::writeSubDirs(QTextStream &t)
     }
     t << slnConf;
 
-    t << _slnProjDepBeg;
-
     // Restore previous after_user_var options
     Option::globals->postcmds = old_after_vars;
 
-    // Figure out dependencies
-    for(QList<VcsolutionDepend*>::Iterator it = solution_cleanup.begin(); it != solution_cleanup.end(); ++it) {
-        int cnt = 0;
-        for(QStringList::iterator dit = (*it)->dependencies.begin();  dit != (*it)->dependencies.end(); ++dit) {
-            if(VcsolutionDepend *vc = solution_depends[*dit])
-                t << "\n\t\t" << (*it)->uuid << "." << cnt++ << " = " << vc->uuid;
-        }
-    }
-    t << _slnProjDepEnd;
     t << _slnProjConfBeg;
     for(QList<VcsolutionDepend*>::Iterator it = solution_cleanup.begin(); it != solution_cleanup.end(); ++it) {
         QString platform = is64Bit ? "x64" : "Win32";
         QString xplatform = platform;
-        if (!project->isEmpty("CE_SDK") && !project->isEmpty("CE_ARCH"))
+        if (!project->isEmpty("VCPROJ_ARCH")) {
+            xplatform = project->first("VCPROJ_ARCH").toQString();
+        } else if (!project->isEmpty("CE_SDK") && !project->isEmpty("CE_ARCH")) {
             xplatform = project->values("CE_SDK").join(' ') + " (" + project->first("CE_ARCH") + ")";
+        }
         if (!project->isHostBuild())
             platform = xplatform;
         t << "\n\t\t" << (*it)->uuid << QString(_slnProjDbgConfTag1).arg(xplatform) << platform;
@@ -856,10 +866,18 @@ void VcprojGenerator::initProject()
     initFormFiles();
     initResourceFiles();
     initExtraCompilerOutputs();
+    if (vcProject.Configuration.WinRT) {
+        if (vcProject.Configuration.WinPhone
+                && vcProject.Configuration.ConfigurationType == typeApplication)
+            initWMAppManifest();
+    }
 
     // Own elements -----------------------------
     vcProject.Name = unescapeFilePath(project->first("QMAKE_ORIG_TARGET").toQString());
     switch(which_dotnet_version()) {
+    case NET2013:
+        vcProject.Version = "13.00";
+        break;
     case NET2012:
         vcProject.Version = "12.00";
         break;
@@ -887,7 +905,9 @@ void VcprojGenerator::initProject()
     }
 
     vcProject.Keyword = project->first("VCPROJ_KEYWORD").toQString();
-    if (project->isHostBuild() || project->isEmpty("CE_SDK") || project->isEmpty("CE_ARCH")) {
+    if (!project->isEmpty("VCPROJ_ARCH")) {
+        vcProject.PlatformName = project->first("VCPROJ_ARCH").toQString();
+    } else if (project->isHostBuild() || project->isEmpty("CE_SDK") || project->isEmpty("CE_ARCH")) {
         vcProject.PlatformName = (is64Bit ? "x64" : "Win32");
     } else {
         vcProject.PlatformName = project->values("CE_SDK").join(' ') + " (" + project->first("CE_ARCH") + ")";
@@ -914,9 +934,10 @@ void VcprojGenerator::initConfiguration()
     if(projectTarget == StaticLib)
         initLibrarianTool();
     else {
-        conf.linker.GenerateDebugInformation = isDebug ? _True : _False;
+        conf.linker.GenerateDebugInformation = project->isActiveConfig("debug_info") ? _True : _False;
         initLinkerTool();
     }
+    initManifestTool();
     initResourceTool();
     initIDLTool();
 
@@ -947,11 +968,19 @@ void VcprojGenerator::initConfiguration()
             conf.PrimaryOutput.append(project->first("TARGET_VERSION_EXT").toQString());
     }
 
+    if (conf.CompilerVersion >= NET2012) {
+        conf.WinRT = project->isActiveConfig("winrt");
+        if (conf.WinRT)
+            conf.WinPhone = project->isActiveConfig("winphone");
+    }
+
     conf.Name = project->values("BUILD_NAME").join(' ');
     if (conf.Name.isEmpty())
         conf.Name = isDebug ? "Debug" : "Release";
     conf.ConfigurationName = conf.Name;
-    if (project->isHostBuild() || project->isEmpty("CE_SDK") || project->isEmpty("CE_ARCH")) {
+    if (!project->isEmpty("VCPROJ_ARCH")) {
+        conf.Name += "|" + project->first("VCPROJ_ARCH");
+    } else if (project->isHostBuild() || project->isEmpty("CE_SDK") || project->isEmpty("CE_ARCH")) {
         conf.Name += (is64Bit ? "|x64" : "|Win32");
     } else {
         conf.Name += "|" + project->values("CE_SDK").join(' ') + " (" + project->first("CE_ARCH") + ")";
@@ -976,17 +1005,14 @@ void VcprojGenerator::initConfiguration()
     initCustomBuildTool();
     initPreBuildEventTools();
     initPostBuildEventTools();
-    // Only deploy for CE projects
-    if (!project->isHostBuild() && !project->isEmpty("CE_SDK") && !project->isEmpty("CE_ARCH"))
+    // Only deploy for CE and WinRT projects
+    if ((!project->isHostBuild() && !project->isEmpty("CE_SDK") && !project->isEmpty("CE_ARCH"))
+            || conf.WinRT)
         initDeploymentTool();
     initPreLinkEventTools();
 
-    // Set definite values in both configurations
-    if (isDebug) {
-        conf.compiler.PreprocessorDefinitions.removeAll("NDEBUG");
-    } else {
+    if (!isDebug)
         conf.compiler.PreprocessorDefinitions += "NDEBUG";
-    }
 }
 
 void VcprojGenerator::initCompilerTool()
@@ -1001,7 +1027,6 @@ void VcprojGenerator::initCompilerTool()
         conf.compiler.Optimization = optimizeDisabled;
     }
     conf.compiler.AssemblerListingLocation = placement ;
-    conf.compiler.ProgramDataBaseFileName = ".\\" ;
     conf.compiler.ObjectFile = placement ;
     conf.compiler.ExceptionHandling = ehNone;
     // PCH
@@ -1042,6 +1067,19 @@ void VcprojGenerator::initLibrarianTool()
     conf.librarian.AdditionalOptions += project->values("QMAKE_LIBFLAGS").toQStringList();
 }
 
+void VcprojGenerator::initManifestTool()
+{
+    VCManifestTool &tool = vcProject.Configuration.manifestTool;
+    const ProString tmplt = project->first("TEMPLATE");
+    if ((tmplt == "vclib"
+         && !project->isActiveConfig("embed_manifest_dll")
+         && !project->isActiveConfig("static"))
+        || (tmplt == "vcapp"
+            && !project->isActiveConfig("embed_manifest_exe"))) {
+        tool.EmbedManifest = _False;
+    }
+}
+
 void VcprojGenerator::initLinkerTool()
 {
     VCConfiguration &conf = vcProject.Configuration;
@@ -1061,10 +1099,6 @@ void VcprojGenerator::initLinkerTool()
 
     conf.linker.OutputFile = "$(OutDir)\\";
     conf.linker.OutputFile += project->first("MSVCPROJ_TARGET").toQString();
-
-    if(project->isActiveConfig("dll")){
-        conf.linker.parseOptions(project->values("QMAKE_LFLAGS_QT_DLL"));
-    }
 }
 
 void VcprojGenerator::initResourceTool()
@@ -1133,11 +1167,19 @@ void VcprojGenerator::initPostBuildEventTools()
 void VcprojGenerator::initDeploymentTool()
 {
     VCConfiguration &conf = vcProject.Configuration;
-    QString targetPath = project->values("deploy.path").join(' ');
-    if (targetPath.isEmpty())
-        targetPath = QString("%CSIDL_PROGRAM_FILES%\\") + project->first("TARGET");
-    if (targetPath.endsWith("/") || targetPath.endsWith("\\"))
-        targetPath.chop(1);
+    QString targetPath;
+    if (conf.WinRT) {
+        vcProject.DeploymentFiles.Name = "Deployment Files";
+        vcProject.DeploymentFiles.ParseFiles = _False;
+        vcProject.DeploymentFiles.Filter = "deploy";
+        vcProject.DeploymentFiles.Guid = _GUIDDeploymentFiles;
+    } else {
+        targetPath = project->values("deploy.path").join(' ');
+        if (targetPath.isEmpty())
+            targetPath = QString("%CSIDL_PROGRAM_FILES%\\") + project->first("TARGET");
+        if (targetPath.endsWith("/") || targetPath.endsWith("\\"))
+            targetPath.chop(1);
+    }
 
     // Only deploy Qt libs for shared build
     if (!project->values("QMAKE_QT_DLL").isEmpty()) {
@@ -1155,48 +1197,55 @@ void VcprojGenerator::initDeploymentTool()
                     continue;
                 dllName.replace(QLatin1String(".lib") , QLatin1String(".dll"));
                 QFileInfo info(dllName);
-                conf.deployment.AdditionalFiles += info.fileName()
-                                                + "|" + QDir::toNativeSeparators(info.absolutePath())
-                                                + "|" + targetPath
-                                                + "|0;";
+                if (conf.WinRT) {
+                    QString absoluteFilePath(QDir::toNativeSeparators(info.absoluteFilePath()));
+                    vcProject.DeploymentFiles.addFile(absoluteFilePath);
+                } else {
+                    conf.deployment.AdditionalFiles += info.fileName()
+                                                    + "|" + QDir::toNativeSeparators(info.absolutePath())
+                                                    + "|" + targetPath
+                                                    + "|0;";
+                }
             }
         }
     }
 
-    // C-runtime deployment
-    QString runtime = project->values("QT_CE_C_RUNTIME").join(QLatin1Char(' '));
-    if (!runtime.isEmpty() && (runtime != QLatin1String("no"))) {
-        QString runtimeVersion = QLatin1String("msvcr");
-        ProString mkspec = project->first("QMAKESPEC");
+    if (!conf.WinRT) {
+        // C-runtime deployment
+        QString runtime = project->values("QT_CE_C_RUNTIME").join(QLatin1Char(' '));
+        if (!runtime.isEmpty() && (runtime != QLatin1String("no"))) {
+            QString runtimeVersion = QLatin1String("msvcr");
+            ProString mkspec = project->first("QMAKESPEC");
 
-        if (!mkspec.isEmpty()) {
-            if (mkspec.endsWith("2008"))
-                runtimeVersion.append("90");
-            else
-                runtimeVersion.append("80");
-            if (project->isActiveConfig("debug"))
-                runtimeVersion.append("d");
-            runtimeVersion.append(".dll");
+            if (!mkspec.isEmpty()) {
+                if (mkspec.endsWith("2008"))
+                    runtimeVersion.append("90");
+                else
+                    runtimeVersion.append("80");
+                if (project->isActiveConfig("debug"))
+                    runtimeVersion.append("d");
+                runtimeVersion.append(".dll");
 
-            if (runtime == "yes") {
-                // Auto-find C-runtime
-                QString vcInstallDir = qgetenv("VCINSTALLDIR");
-                if (!vcInstallDir.isEmpty()) {
-                    vcInstallDir += "\\ce\\dll\\";
-                    vcInstallDir += project->values("CE_ARCH").join(QLatin1Char(' '));
-                    if (!QFileInfo(vcInstallDir + QDir::separator() + runtimeVersion).exists())
-                        runtime.clear();
-                    else
-                        runtime = vcInstallDir;
+                if (runtime == "yes") {
+                    // Auto-find C-runtime
+                    QString vcInstallDir = qgetenv("VCINSTALLDIR");
+                    if (!vcInstallDir.isEmpty()) {
+                        vcInstallDir += "\\ce\\dll\\";
+                        vcInstallDir += project->values("CE_ARCH").join(QLatin1Char(' '));
+                        if (!QFileInfo(vcInstallDir + QDir::separator() + runtimeVersion).exists())
+                            runtime.clear();
+                        else
+                            runtime = vcInstallDir;
+                    }
                 }
             }
-        }
 
-        if (!runtime.isEmpty() && runtime != QLatin1String("yes")) {
-            conf.deployment.AdditionalFiles += runtimeVersion
-                                            + "|" + QDir::toNativeSeparators(runtime)
-                                            + "|" + targetPath
-                                            + "|0;";
+            if (!runtime.isEmpty() && runtime != QLatin1String("yes")) {
+                conf.deployment.AdditionalFiles += runtimeVersion
+                        + "|" + QDir::toNativeSeparators(runtime)
+                        + "|" + targetPath
+                        + "|0;";
+            }
         }
     }
 
@@ -1204,14 +1253,16 @@ void VcprojGenerator::initDeploymentTool()
     foreach (const ProString &item, project->values("DEPLOYMENT")) {
         // get item.path
         QString devicePath = project->first(ProKey(item + ".path")).toQString();
-        if (devicePath.isEmpty())
-            devicePath = targetPath;
-        // check if item.path is relative (! either /,\ or %)
-        if (!(devicePath.at(0) == QLatin1Char('/')
-            || devicePath.at(0) == QLatin1Char('\\')
-            || devicePath.at(0) == QLatin1Char('%'))) {
-            // create output path
-            devicePath = Option::fixPathToLocalOS(QDir::cleanPath(targetPath + QLatin1Char('\\') + devicePath));
+        if (!conf.WinRT) {
+            if (devicePath.isEmpty())
+                devicePath = targetPath;
+            // check if item.path is relative (! either /,\ or %)
+            if (!(devicePath.at(0) == QLatin1Char('/')
+                || devicePath.at(0) == QLatin1Char('\\')
+                || devicePath.at(0) == QLatin1Char('%'))) {
+                // create output path
+                devicePath = Option::fixPathToLocalOS(QDir::cleanPath(targetPath + QLatin1Char('\\') + devicePath));
+            }
         }
         // foreach d in item.files
         foreach (const ProString &src, project->values(ProKey(item + ".files"))) {
@@ -1236,16 +1287,27 @@ void VcprojGenerator::initDeploymentTool()
             // foreach dirIterator-entry in d
             while(iterator.hasNext()) {
                 iterator.next();
-                QString absoluteItemPath = Option::fixPathToLocalOS(QFileInfo(iterator.filePath()).absolutePath());
-                // Identify if it is just another subdir
-                int diffSize = absoluteItemPath.size() - pathSize;
-                // write out rules
-                conf.deployment.AdditionalFiles += iterator.fileName()
-                    + "|" + absoluteItemPath
-                    + "|" + itemDevicePath + (diffSize ? (absoluteItemPath.right(diffSize)) : QLatin1String(""))
-                    + "|0;";
+                if (conf.WinRT) {
+                    QString absoluteItemFilePath = Option::fixPathToLocalOS(QFileInfo(iterator.filePath()).absoluteFilePath());
+                    vcProject.DeploymentFiles.addFile(absoluteItemFilePath);
+                } else {
+                    QString absoluteItemPath = Option::fixPathToLocalOS(QFileInfo(iterator.filePath()).absolutePath());
+                    // Identify if it is just another subdir
+                    int diffSize = absoluteItemPath.size() - pathSize;
+                    // write out rules
+                    conf.deployment.AdditionalFiles += iterator.fileName()
+                            + "|" + absoluteItemPath
+                            + "|" + itemDevicePath + (diffSize ? (absoluteItemPath.right(diffSize)) : QLatin1String(""))
+                            + "|0;";
+                }
             }
         }
+
+        if (conf.WinRT) {
+            vcProject.DeploymentFiles.Project = this;
+            vcProject.DeploymentFiles.Config = &(vcProject.Configuration);
+            vcProject.DeploymentFiles.CustomBuild = none;
+        }
     }
 }
 
@@ -1309,7 +1371,8 @@ void VcprojGenerator::initGeneratedFiles()
     vcProject.GeneratedFiles.addFiles(project->values("GENERATED_SOURCES"));
     vcProject.GeneratedFiles.addFiles(project->values("GENERATED_FILES"));
     vcProject.GeneratedFiles.addFiles(project->values("IDLSOURCES"));
-    vcProject.GeneratedFiles.addFiles(project->values("RES_FILE"));
+    if (project->values("RC_FILE").isEmpty())
+        vcProject.GeneratedFiles.addFiles(project->values("RES_FILE"));
     vcProject.GeneratedFiles.addFiles(project->values("QMAKE_IMAGE_COLLECTION"));   // compat
     if(!extraCompilerOutputs.isEmpty())
         vcProject.GeneratedFiles.addFiles(extraCompilerOutputs.keys());
@@ -1494,6 +1557,68 @@ void VcprojGenerator::initExtraCompilerOutputs()
     }
 }
 
+void VcprojGenerator::initWMAppManifest()
+{
+    if (!project->isActiveConfig("autogen_wmappmanifest"))
+        return;
+
+    // autogen_wmappmanifest
+    QFile file(Option::output_dir + "\\WMAppManifest.xml");
+    if (!file.open(QFile::WriteOnly))
+        return;
+
+    QTextStream stream(&file);
+
+    QString productID = project->first("PRODUCTID").toQString();
+    QString target = project->first("TARGET").toQString();
+    QString author = project->first("AUTHOR").toQString();
+    QString publisher = project->first("PUBLISHER").toQString();
+    QString publisherID = project->first("PUBLISHERID").toQString();
+    QString description = project->first("DESCRIPTION").toQString();
+
+    if (author.isEmpty())
+        author = "Qt";
+    if (publisher.isEmpty())
+        publisher = "Qt";
+    if (productID.isEmpty())
+        productID = QUuid::createUuid().toString();
+    if (publisherID.isEmpty())
+        publisherID = QUuid::createUuid().toString();
+
+    stream << "<?xml version=\"1.0\" encoding=\"utf-8\"?>\n"
+           << "<Deployment xmlns=\"http://schemas.microsoft.com/windowsphone/2012/deployment\" AppPlatformVersion=\"8.0\">\n"
+           << "  <DefaultLanguage xmlns=\"\" code=\"en-US\"/>\n"
+           << "  <App xmlns=\"\" ProductID=\"" << productID << "\" Title=\"" << target
+           << "\" RuntimeType=\"Modern Native\" Version=\"1.0.0.0\""
+           << " Genre=\"apps.normal\"  Author=\"" << author
+           << "\" Description=\"" << description << "\" Publisher=\"" << publisher
+           << "\" PublisherID=\"" << publisherID << "\">\n"
+           << "    <IconPath IsRelative=\"true\" IsResource=\"false\">ApplicationIcon.png</IconPath>\n"
+           << "    <Capabilities>\n"
+           << "      <Capability Name=\"ID_CAP_NETWORKING\" />\n"
+           << "      <Capability Name=\"ID_CAP_MEDIALIB_AUDIO\" />\n"
+           << "      <Capability Name=\"ID_CAP_MEDIALIB_PLAYBACK\" />\n"
+           << "    </Capabilities>\n"
+           << "    <Tasks>\n"
+           << "      <DefaultTask Name=\"_default\" ImagePath=\"" << target << ".exe\" ImageParams=\"\" />\n"
+           << "    </Tasks>\n"
+           << "    <Tokens>\n"
+           << "      <PrimaryToken TokenID=\"" << target << "Token\" TaskName=\"_default\">\n"
+           << "        <TemplateType5>\n"
+           << "          <Count>0</Count>\n"
+           << "          <Title>" << target << "</Title>\n"
+           << "        </TemplateType5>\n"
+           << "      </PrimaryToken>\n"
+           << "    </Tokens>\n"
+           << "    <ScreenResolutions>\n"
+           << "      <ScreenResolution Name=\"ID_RESOLUTION_WVGA\" />\n"
+           << "      <ScreenResolution Name=\"ID_RESOLUTION_WXGA\" />\n"
+           << "      <ScreenResolution Name=\"ID_RESOLUTION_HD720P\" />\n"
+           << "    </ScreenResolutions>\n"
+           << "  </App>\n"
+           << "</Deployment>\n";
+}
+
 void VcprojGenerator::initOld()
 {
     // $$QMAKE.. -> $$MSVCPROJ.. -------------------------------------
