@@ -41,6 +41,7 @@
 
 #include "qmakeparser.h"
 
+#include "qmakevfs.h"
 #include "ioutils.h"
 using namespace QMakeInternal;
 
@@ -142,18 +143,19 @@ void QMakeParser::initialize()
     statics.strLITERAL_WHITESPACE = QLatin1String("LITERAL_WHITESPACE");
 }
 
-QMakeParser::QMakeParser(ProFileCache *cache, QMakeParserHandler *handler)
+QMakeParser::QMakeParser(ProFileCache *cache, QMakeVfs *vfs, QMakeParserHandler *handler)
     : m_cache(cache)
     , m_handler(handler)
+    , m_vfs(vfs)
 {
     // So that single-threaded apps don't have to call initialize() for now.
     initialize();
 }
 
-ProFile *QMakeParser::parsedProFile(const QString &fileName, bool cache)
+ProFile *QMakeParser::parsedProFile(const QString &fileName, ParseFlags flags)
 {
     ProFile *pro;
-    if (cache && m_cache) {
+    if ((flags & ParseUseCache) && m_cache) {
         ProFileCache::Entry *ent;
 #ifdef PROPARSER_THREAD_SAFE
         QMutexLocker locker(&m_cache->mutex);
@@ -182,7 +184,7 @@ ProFile *QMakeParser::parsedProFile(const QString &fileName, bool cache)
             locker.unlock();
 #endif
             pro = new ProFile(fileName);
-            if (!read(pro)) {
+            if (!read(pro, flags)) {
                 delete pro;
                 pro = 0;
             } else {
@@ -203,7 +205,7 @@ ProFile *QMakeParser::parsedProFile(const QString &fileName, bool cache)
         }
     } else {
         pro = new ProFile(fileName);
-        if (!read(pro)) {
+        if (!read(pro, flags)) {
             delete pro;
             pro = 0;
         }
@@ -228,26 +230,16 @@ void QMakeParser::discardFileFromCache(const QString &fileName)
         m_cache->discardFile(fileName);
 }
 
-bool QMakeParser::read(ProFile *pro)
+bool QMakeParser::read(ProFile *pro, ParseFlags flags)
 {
-    QFile file(pro->fileName());
-    if (!file.open(QIODevice::ReadOnly)) {
-        if (m_handler && IoUtils::exists(pro->fileName()))
+    QString content;
+    QString errStr;
+    if (!m_vfs->readFile(pro->fileName(), &content, &errStr)) {
+        if (m_handler && ((flags & ParseReportMissing) || m_vfs->exists(pro->fileName())))
             m_handler->message(QMakeParserHandler::ParserIoError,
-                               fL1S("Cannot read %1: %2").arg(pro->fileName(), file.errorString()));
+                               fL1S("Cannot read %1: %2").arg(pro->fileName(), errStr));
         return false;
     }
-
-    QByteArray bcont = file.readAll();
-    if (bcont.startsWith("\xef\xbb\xbf")) {
-        // UTF-8 BOM will cause subtle errors
-        m_handler->message(QMakeParserHandler::ParserIoError,
-                           fL1S("Unexpected UTF-8 BOM in %1").arg(pro->fileName()));
-        return false;
-    }
-    QString content(QString::fromLocal8Bit(bcont));
-    bcont.clear();
-    file.close();
     return read(pro, content, 1, FullGrammar);
 }
 
@@ -298,12 +290,12 @@ bool QMakeParser::read(ProFile *pro, const QString &in, int line, SubGrammar gra
     // Worst-case size calculations:
     // - line marker adds 1 (2-nl) to 1st token of each line
     // - empty assignment "A=":2 =>
-    //   TokHashLiteral(1) + hash(2) + len(1) + "A"(1) + TokAssign(1) +
-    //   TokValueTerminator(1) == 7 (8)
+    //   TokHashLiteral(1) + hash(2) + len(1) + "A"(1) + TokAssign(1) + 0(1) +
+    //   TokValueTerminator(1) == 8 (9)
     // - non-empty assignment "A=B C":5 =>
-    //   TokHashLiteral(1) + hash(2) + len(1) + "A"(1) + TokAssign(1) +
+    //   TokHashLiteral(1) + hash(2) + len(1) + "A"(1) + TokAssign(1) + 2(1) +
     //   TokLiteral(1) + len(1) + "B"(1) +
-    //   TokLiteral(1) + len(1) + "C"(1) + TokValueTerminator(1) == 13 (14)
+    //   TokLiteral(1) + len(1) + "C"(1) + TokValueTerminator(1) == 14 (15)
     // - variable expansion: "$$f":3 =>
     //   TokVariable(1) + hash(2) + len(1) + "f"(1) = 5
     // - function expansion: "$$f()":5 =>
@@ -1024,7 +1016,6 @@ void QMakeParser::finalizeCall(ushort *&tokPtr, ushort *uc, ushort *ptr, int arg
             m_tmp.setRawData((QChar *)uc + 4, nlen);
             const QString *defName;
             ushort defType;
-            uchar nest;
             if (m_tmp == statics.strfor) {
                 if (m_invert || m_operator == OrOperator) {
                     // '|' could actually work reasonably, but qmake does nonsense here.
@@ -1101,13 +1092,20 @@ void QMakeParser::finalizeCall(ushort *&tokPtr, ushort *uc, ushort *ptr, int arg
                 parseError(fL1S("%1(function) requires one literal argument.").arg(*defName));
                 return;
             } else if (m_tmp == statics.strreturn) {
-                if (argc > 1) {
-                    parseError(fL1S("return() requires zero or one argument."));
-                    bogusTest(tokPtr);
-                    return;
+                if (m_blockstack.top().nest & NestFunction) {
+                    if (argc > 1) {
+                        parseError(fL1S("return() requires zero or one argument."));
+                        bogusTest(tokPtr);
+                        return;
+                    }
+                } else {
+                    if (*uce != TokFuncTerminator) {
+                        parseError(fL1S("Top-level return() requires zero arguments."));
+                        bogusTest(tokPtr);
+                        return;
+                    }
                 }
                 defType = TokReturn;
-                nest = NestFunction;
                 goto ctrlstm2;
             } else if (m_tmp == statics.strnext) {
                 defType = TokNext;
@@ -1120,15 +1118,14 @@ void QMakeParser::finalizeCall(ushort *&tokPtr, ushort *uc, ushort *ptr, int arg
                     bogusTest(tokPtr);
                     return;
                 }
-                nest = NestLoop;
-              ctrlstm2:
-                if (m_invert) {
-                    parseError(fL1S("Unexpected NOT operator in front of %1().").arg(m_tmp));
+                if (!(m_blockstack.top().nest & NestLoop)) {
+                    parseError(fL1S("Unexpected %1().").arg(m_tmp));
                     bogusTest(tokPtr);
                     return;
                 }
-                if (!(m_blockstack.top().nest & nest)) {
-                    parseError(fL1S("Unexpected %1().").arg(m_tmp));
+              ctrlstm2:
+                if (m_invert) {
+                    parseError(fL1S("Unexpected NOT operator in front of %1().").arg(m_tmp));
                     bogusTest(tokPtr);
                     return;
                 }
