@@ -54,10 +54,10 @@ QT_BEGIN_NAMESPACE
 EvalHandler Option::evalHandler;
 QMakeGlobals *Option::globals;
 ProFileCache *Option::proFileCache;
+QMakeVfs *Option::vfs;
 QMakeParser *Option::parser;
 
 //convenience
-const char *Option::application_argv0 = 0;
 QString Option::prf_ext;
 QString Option::prl_ext;
 QString Option::libtool_ext;
@@ -82,7 +82,6 @@ char Option::field_sep;
 Option::QMAKE_MODE Option::qmake_mode = Option::QMAKE_GENERATE_NOTHING;
 
 //all modes
-QStringList Option::qmake_args;
 int Option::warn_level = WarnLogic | WarnDeprecated;
 int Option::debug_level = 0;
 QFile Option::output;
@@ -318,13 +317,11 @@ Option::parseCommandLine(QStringList &args, QMakeCmdLineParserState &state)
 int
 Option::init(int argc, char **argv)
 {
-    Option::application_argv0 = 0;
     Option::prf_ext = ".prf";
     Option::pro_ext = ".pro";
     Option::field_sep = ' ';
 
     if(argc && argv) {
-        Option::application_argv0 = argv[0];
         QString argv0 = argv[0];
         if(Option::qmake_mode == Option::QMAKE_GENERATE_NOTHING)
             Option::qmake_mode = default_mode(argv0);
@@ -341,6 +338,7 @@ Option::init(int argc, char **argv)
             QDir currentDir = QDir::current();
 #ifdef Q_OS_WIN
             QStringList paths = QString::fromLocal8Bit(pEnv).split(QLatin1String(";"));
+            paths.prepend(QLatin1String("."));
 #else
             QStringList paths = QString::fromLocal8Bit(pEnv).split(QLatin1String(":"));
 #endif
@@ -349,7 +347,8 @@ Option::init(int argc, char **argv)
                     continue;
                 QString candidate = currentDir.absoluteFilePath(*p + QLatin1Char('/') + argv0);
 #ifdef Q_OS_WIN
-                candidate += ".exe";
+                if (!candidate.endsWith(QLatin1String(".exe")))
+                    candidate += QLatin1String(".exe");
 #endif
                 if (QFile::exists(candidate)) {
                     globals->qmake_abslocation = candidate;
@@ -436,7 +435,7 @@ Option::init(int argc, char **argv)
             return ret;
             //return ret == QMAKE_CMDLINE_SHOW_USAGE ? usage(argv[0]) : false;
         }
-        Option::qmake_args = args;
+        globals->qmake_args = args;
     }
     globals->commitCommandLineArguments(cmdstate);
     globals->debugLevel = Option::debug_level;
@@ -532,12 +531,11 @@ Option::fixString(QString string, uchar flags)
         string = QDir::cleanPath(string);
     }
 
-    bool localSep = (flags & Option::FixPathToLocalSeparators) != 0;
-    bool targetSep = (flags & Option::FixPathToTargetSeparators) != 0;
-    bool normalSep = (flags & Option::FixPathToNormalSeparators) != 0;
-
     // either none or only one active flag
-    Q_ASSERT(localSep + targetSep + normalSep <= 1);
+    Q_ASSERT(((flags & Option::FixPathToLocalSeparators) != 0) +
+             ((flags & Option::FixPathToTargetSeparators) != 0) +
+             ((flags & Option::FixPathToNormalSeparators) != 0) <= 1);
+
     //fix separators
     if (flags & Option::FixPathToNormalSeparators) {
         string = string.replace('\\', '/');
