@@ -46,7 +46,6 @@
 
 #ifndef QT_NO_CONCURRENT
 
-QT_BEGIN_HEADER
 QT_BEGIN_NAMESPACE
 
 namespace QtPrivate {
@@ -64,7 +63,6 @@ public:
 }
 
 QT_END_NAMESPACE
-QT_END_HEADER
 
 #endif // QT_NO_CONCURRENT
 
