@@ -49,7 +49,6 @@
 #include <QtConcurrent/qtconcurrentfilterkernel.h>
 #include <QtConcurrent/qtconcurrentfunctionwrappers.h>
 
-QT_BEGIN_HEADER
 QT_BEGIN_NAMESPACE
 
 
@@ -261,7 +260,6 @@ OutputSequence blockingFiltered(Iterator begin, Iterator end, KeepFunctor keep)
 #endif // Q_QDOC
 
 QT_END_NAMESPACE
-QT_END_HEADER
 
 #endif // QT_NO_CONCURRENT
 
