@@ -52,7 +52,6 @@
 
 #include <iterator>
 
-QT_BEGIN_HEADER
 QT_BEGIN_NAMESPACE
 
 
@@ -60,8 +59,6 @@ QT_BEGIN_NAMESPACE
 
 namespace QtConcurrent {
 
-    using std::advance;
-
 /*
     The BlockSizeManager class manages how many iterations a thread should
     reserve and process at a time. This is done by measuring the time spent
@@ -303,7 +300,6 @@ public:
 #endif //Q_QDOC
 
 QT_END_NAMESPACE
-QT_END_HEADER
 
 #endif // QT_NO_CONCURRENT
 
