@@ -507,7 +507,7 @@ void QUnifiedTimer::uninstallAnimationDriver(QAnimationDriver *d)
 }
 
 /*!
-    Returns true if \a d is the currently installed animation driver
+    Returns \c true if \a d is the currently installed animation driver
     and is not the default animation driver (which can never be uninstalled).
 */
 bool QUnifiedTimer::canUninstallAnimationDriver(QAnimationDriver *d)
@@ -827,8 +827,8 @@ void QAnimationDriver::start()
 {
     Q_D(QAnimationDriver);
     if (!d->running) {
-        emit started();
         d->running = true;
+        emit started();
     }
 }
 
@@ -837,8 +837,8 @@ void QAnimationDriver::stop()
 {
     Q_D(QAnimationDriver);
     if (d->running) {
-        emit stopped();
         d->running = false;
+        emit stopped();
     }
 }
 
