@@ -45,15 +45,12 @@
 
 #include <QtCore/qgenericatomic.h>
 
-QT_BEGIN_HEADER
-
 QT_BEGIN_NAMESPACE
 
 #if 0
 // silence syncqt warnings
 QT_END_NAMESPACE
-QT_END_HEADER
-
+#pragma qt_sync_skip_header_check
 #pragma qt_sync_stop_processing
 #endif
 
@@ -117,8 +114,8 @@ template <typename T> struct QAtomicOps : QBasicAtomicOps<sizeof(T)>
 template<> template<typename T> inline
 bool QBasicAtomicOps<4>::ref(T &_q_value) Q_DECL_NOTHROW
 {
-    register T originalValue;
-    register T newValue;
+    T originalValue;
+    T newValue;
     do {
         originalValue = _q_value;
         newValue = originalValue + 1;
@@ -129,8 +126,8 @@ bool QBasicAtomicOps<4>::ref(T &_q_value) Q_DECL_NOTHROW
 template<> template <typename T> inline
 bool QBasicAtomicOps<4>::deref(T &_q_value) Q_DECL_NOTHROW
 {
-    register T originalValue;
-    register T newValue;
+    T originalValue;
+    T newValue;
     do {
         originalValue = _q_value;
         newValue = originalValue - 1;
@@ -141,7 +138,7 @@ bool QBasicAtomicOps<4>::deref(T &_q_value) Q_DECL_NOTHROW
 template<> template <typename T> inline
 bool QBasicAtomicOps<4>::testAndSetRelaxed(T &_q_value, T expectedValue, T newValue) Q_DECL_NOTHROW
 {
-    register T originalValue;
+    T originalValue;
     do {
         originalValue = _q_value;
         if (originalValue != expectedValue)
@@ -167,20 +164,28 @@ __asm T QBasicAtomicOps<4>::fetchAndStoreRelaxed(T &_q_value, T newValue) Q_DECL
 template<> template <typename T> inline
 T QBasicAtomicOps<4>::fetchAndStoreRelaxed(T &_q_value, T newValue) Q_DECL_NOTHROW
 {
+#if defined(__thumb__)
+    T originalValue;
+    do {
+        originalValue = _q_value;
+    } while (_q_cmpxchg(originalValue, newValue, &_q_value) != 0);
+    return originalValue;
+#else
     T originalValue;
     asm volatile("swp %0,%2,[%3]"
                  : "=&r"(originalValue), "=m" (_q_value)
                  : "r"(newValue), "r"(&_q_value)
                  : "cc", "memory");
     return originalValue;
+#endif
 }
 #endif // Q_CC_RVCT
 
 template<> template <typename T> inline
 T QBasicAtomicOps<4>::fetchAndAddRelaxed(T &_q_value, typename QAtomicAdditiveType<T>::AdditiveT valueToAdd) Q_DECL_NOTHROW
 {
-    register T originalValue;
-    register T newValue;
+    T originalValue;
+    T newValue;
     do {
         originalValue = _q_value;
         newValue = originalValue + valueToAdd;
@@ -190,6 +195,4 @@ T QBasicAtomicOps<4>::fetchAndAddRelaxed(T &_q_value, typename QAtomicAdditiveTy
 
 QT_END_NAMESPACE
 
-QT_END_HEADER
-
 #endif // QATOMIC_ARMV5_H
