@@ -89,7 +89,7 @@
 #include <stdlib.h>
 #include <ctype.h>
 #include <locale.h>
-#if defined (_XOPEN_UNIX) && !defined(Q_OS_QNX) && !defined(Q_OS_OSF) && !defined(Q_OS_LINUX_ANDROID)
+#if defined (_XOPEN_UNIX) && !defined(Q_OS_QNX) && !defined(Q_OS_OSF) && !defined(Q_OS_ANDROID)
 # include <langinfo.h>
 #endif
 
@@ -99,9 +99,9 @@ Q_GLOBAL_STATIC_WITH_ARGS(QMutex, textCodecsMutex, (QMutex::Recursive));
 QMutex *qTextCodecsMutex() { return textCodecsMutex(); }
 
 #if !defined(QT_USE_ICU)
-static char qtolower(register char c)
+static char qtolower(char c)
 { if (c >= 'A' && c <= 'Z') return c + 0x20; return c; }
-static bool qisalnum(register char c)
+static bool qisalnum(char c)
 { return (c >= '0' && c <= '9') || ((c | 0x20) >= 'a' && (c | 0x20) <= 'z'); }
 
 bool qTextCodecNameMatch(const char *n, const char *h)
@@ -600,9 +600,6 @@ QTextCodec* QTextCodec::codecForMib(int mib)
 */
 QList<QByteArray> QTextCodec::availableCodecs()
 {
-#ifdef QT_USE_ICU
-    return QIcuCodec::availableCodecs();
-#else
     QMutexLocker locker(textCodecsMutex());
 
     QCoreGlobalData *globalData = QCoreGlobalData::instance();
@@ -616,8 +613,11 @@ QList<QByteArray> QTextCodec::availableCodecs()
         codecs += globalData->allCodecs.at(i)->aliases();
     }
 
-    return codecs;
+#ifdef QT_USE_ICU
+    codecs += QIcuCodec::availableCodecs();
 #endif
+
+    return codecs;
 }
 
 /*!
@@ -828,8 +828,8 @@ QString QTextCodec::toUnicode(const QByteArray& a) const
 }
 
 /*!
-    Returns true if the Unicode character \a ch can be fully encoded
-    with this codec; otherwise returns false.
+    Returns \c true if the Unicode character \a ch can be fully encoded
+    with this codec; otherwise returns \c false.
 */
 bool QTextCodec::canEncode(QChar ch) const
 {
@@ -1043,20 +1043,24 @@ QString QTextDecoder::toUnicode(const QByteArray &ba)
 QTextCodec *QTextCodec::codecForHtml(const QByteArray &ba, QTextCodec *defaultCodec)
 {
     // determine charset
-    int pos;
-    QTextCodec *c = 0;
-
-    c = QTextCodec::codecForUtfText(ba, c);
+    QTextCodec *c = QTextCodec::codecForUtfText(ba, 0);
     if (!c) {
         QByteArray header = ba.left(512).toLower();
-        if ((pos = header.indexOf("http-equiv=")) != -1) {
-            if ((pos = header.lastIndexOf("meta ", pos)) != -1) {
-                pos = header.indexOf("charset=", pos) + int(strlen("charset="));
-                if (pos != -1) {
-                    int pos2 = header.indexOf('\"', pos+1);
-                    QByteArray cs = header.mid(pos, pos2-pos);
-                    //            qDebug("found charset: %s", cs.data());
-                    c = QTextCodec::codecForName(cs);
+        int pos = header.indexOf("meta ");
+        if (pos != -1) {
+            pos = header.indexOf("charset=", pos);
+            if (pos != -1) {
+                pos += qstrlen("charset=");
+
+                int pos2 = pos;
+                // The attribute can be closed with either """, "'", ">" or "/",
+                // none of which are valid charset characters.
+                while (++pos2 < header.size()) {
+                    char ch = header.at(pos2);
+                    if (ch == '\"' || ch == '\'' || ch == '>') {
+                        c = QTextCodec::codecForName(header.mid(pos, pos2 - pos));
+                        return c ? c : defaultCodec;
+                    }
                 }
             }
         }
