@@ -78,6 +78,7 @@ struct QLibrarySettings
     QLibrarySettings();
     QScopedPointer<QSettings> settings;
 #ifdef QT_BOOTSTRAPPED
+    bool haveEffectiveSourcePaths;
     bool haveEffectivePaths;
     bool havePaths;
 #endif
@@ -99,8 +100,10 @@ public:
     static bool haveGroup(QLibraryInfo::PathGroup group)
     {
         QLibrarySettings *ls = qt_library_settings();
-        return ls ? (group == QLibraryInfo::EffectivePaths
-                     ? ls->haveEffectivePaths : ls->havePaths) : false;
+        return ls ? (group == QLibraryInfo::EffectiveSourcePaths
+                     ? ls->haveEffectiveSourcePaths
+                     : group == QLibraryInfo::EffectivePaths
+                       ? ls->haveEffectivePaths : ls->havePaths) : false;
     }
 #endif
     static QSettings *configuration()
@@ -122,7 +125,12 @@ QLibrarySettings::QLibrarySettings()
         // This code needs to be in the regular library, as otherwise a qt.conf that
         // works for qmake would break things for dynamically built Qt tools.
         QStringList children = settings->childGroups();
+#ifdef QT_BOOTSTRAPPED
+        haveEffectiveSourcePaths = children.contains(QLatin1String("EffectiveSourcePaths"));
+        haveEffectivePaths = haveEffectiveSourcePaths || children.contains(QLatin1String("EffectivePaths"));
+#else
         haveEffectivePaths = children.contains(QLatin1String("EffectivePaths"));
+#endif
         // Backwards compat: an existing but empty file is claimed to contain the Paths section.
         havePaths = !haveEffectivePaths || children.contains(QLatin1String("Paths"));
 #ifndef QT_BOOTSTRAPPED
@@ -130,6 +138,9 @@ QLibrarySettings::QLibrarySettings()
             settings.reset(0);
 #else
     } else {
+#ifdef QT_BOOTSTRAPPED
+        haveEffectiveSourcePaths = false;
+#endif
         haveEffectivePaths = false;
         havePaths = false;
 #endif
@@ -210,7 +221,7 @@ QLibraryInfo::QLibraryInfo()
 QString
 QLibraryInfo::licensee()
 {
-    const char *str = QT_CONFIGURE_LICENSEE;
+    const char * volatile str = QT_CONFIGURE_LICENSEE;
     return QString::fromLocal8Bit(str);
 }
 
@@ -223,7 +234,7 @@ QLibraryInfo::licensee()
 QString
 QLibraryInfo::licensedProducts()
 {
-    const char *str = QT_CONFIGURE_LICENSED_PRODUCTS;
+    const char * volatile str = QT_CONFIGURE_LICENSED_PRODUCTS;
     return QString::fromLatin1(str);
 }
 
@@ -242,7 +253,7 @@ QLibraryInfo::buildDate()
 
 /*!
     \since 5.0
-    Returns true if this build of Qt was built with debugging enabled, or
+    Returns \c true if this build of Qt was built with debugging enabled, or
     false if it was built in release mode.
 */
 bool
@@ -291,6 +302,7 @@ static const struct {
     { "Sysroot", "" },
     { "HostPrefix", "" },
     { "HostBinaries", "bin" },
+    { "HostLibraries", "lib" },
     { "HostData", "." },
     { "TargetSpec", "" },
     { "HostSpec", "" },
@@ -308,7 +320,7 @@ QLibraryInfo::location(LibraryLocation loc)
     QString ret = rawLocation(loc, FinalPaths);
 
     // Automatically prepend the sysroot to target paths
-    if (loc < SysrootPath || loc > LastHostPath) {
+    if ((loc < SysrootPath || loc > LastHostPath) && qt_sysrootify_prefix[12] == 'y') {
         QString sysroot = rawLocation(SysrootPath, FinalPaths);
         if (!sysroot.isEmpty() && ret.length() > 2 && ret.at(1) == QLatin1Char(':')
             && (ret.at(2) == QLatin1Char('/') || ret.at(2) == QLatin1Char('\\')))
@@ -333,9 +345,12 @@ QLibraryInfo::rawLocation(LibraryLocation loc, PathGroup group)
     // and qt.conf with that section is present, use it, otherwise fall back to
     // FinalPaths. For FinalPaths, use qt.conf if present and contains not only
     // [EffectivePaths], otherwise fall back to builtins.
+    // EffectiveSourcePaths falls back to EffectivePaths.
     if (!QLibraryInfoPrivate::haveGroup(group)
-        && (group == FinalPaths
-            || !(group = FinalPaths, QLibraryInfoPrivate::haveGroup(FinalPaths))))
+        && !(group == EffectiveSourcePaths
+             && (group = EffectivePaths, QLibraryInfoPrivate::haveGroup(group)))
+        && !(group == EffectivePaths
+             && (group = FinalPaths, QLibraryInfoPrivate::haveGroup(group))))
 #elif !defined(QT_NO_SETTINGS)
     if (!QLibraryInfoPrivate::configuration())
 #endif
@@ -367,6 +382,7 @@ QLibraryInfo::rawLocation(LibraryLocation loc, PathGroup group)
             QSettings *config = QLibraryInfoPrivate::configuration();
             config->beginGroup(QLatin1String(
 #ifdef QT_BOOTSTRAPPED
+                   group == EffectiveSourcePaths ? "EffectiveSourcePaths" :
                    group == EffectivePaths ? "EffectivePaths" :
 #endif
                                              "Paths"));
@@ -426,7 +442,11 @@ QLibraryInfo::rawLocation(LibraryLocation loc, PathGroup group)
                     QCFType<CFURLRef> urlRef = CFBundleCopyBundleURL(bundleRef);
                     if (urlRef) {
                         QCFString path = CFURLCopyFileSystemPath(urlRef, kCFURLPOSIXPathStyle);
+#ifdef Q_OS_MACX
                         return QDir::cleanPath(QString(path) + QLatin1String("/Contents/") + ret);
+#else
+                        return QDir::cleanPath(QString(path) + QLatin1Char('/') + ret); // iOS
+#endif
                     }
                 }
 #endif
@@ -478,6 +498,8 @@ QT_END_NAMESPACE
 #  include <stdio.h>
 #  include <stdlib.h>
 
+#include "private/qcoreapplication_p.h"
+
 extern const char qt_core_interpreter[] __attribute__((section(".interp")))
     = ELF_INTERPRETER;
 
@@ -500,8 +522,8 @@ void qt_core_boilerplate()
     QT_PREPEND_NAMESPACE(qDumpCPUFeatures)();
 
 #ifdef QT_EVAL
-    extern void qt_core_eval_init(uint);
-    qt_core_eval_init(1);
+    extern void qt_core_eval_init(QCoreApplicationPrivate::Type);
+    qt_core_eval_init(QCoreApplicationPrivate::Tty);
 #endif
 
     exit(0);
