@@ -49,6 +49,8 @@
 #ifndef QT_BOOTSTRAPPED
 #include "qcoreapplication.h"
 #include "qthread.h"
+#include "qloggingcategory.h"
+#include "private/qloggingregistry_p.h"
 #endif
 #ifdef Q_OS_WIN
 #include <qt_windows.h>
@@ -57,6 +59,10 @@
 #include <slog2.h>
 #endif
 
+#ifdef Q_OS_ANDROID
+#include <android/log.h>
+#endif
+
 #include <stdio.h>
 
 QT_BEGIN_NAMESPACE
@@ -72,14 +78,41 @@ static bool isFatal(QtMsgType msgType)
     if (msgType == QtFatalMsg)
         return true;
 
-    if (msgType == QtWarningMsg) {
-        static bool fatalWarnings = qEnvironmentVariableIsSet("QT_FATAL_WARNINGS");
+    if (msgType == QtWarningMsg || msgType == QtCriticalMsg) {
+        static bool fatalWarnings = !qEnvironmentVariableIsEmpty("QT_FATAL_WARNINGS");
         return fatalWarnings;
     }
 
     return false;
 }
 
+#ifdef Q_OS_WIN
+
+// Do we have stderr for QDebug? - Either there is a console or we are running
+// with redirected stderr.
+#  if !defined(Q_OS_WINCE) && !defined(Q_OS_WINRT)
+static inline bool hasStdErr()
+{
+    if (GetConsoleWindow())
+        return true;
+    STARTUPINFO info;
+    GetStartupInfo(&info);
+    return (info.dwFlags & STARTF_USESTDHANDLES) && info.hStdError
+        && info.hStdError != INVALID_HANDLE_VALUE;
+}
+#  endif // !Q_OS_WINCE && !Q_OS_WINRT
+
+Q_CORE_EXPORT bool qWinLogToStderr()
+{
+#  if !defined(Q_OS_WINCE) && !defined(Q_OS_WINRT)
+    static const bool result = hasStdErr();
+    return result;
+#  else
+    return false;
+#  endif
+}
+#endif // Q_OS_WIN
+
 /*!
     \class QMessageLogContext
     \inmodule QtCore
@@ -110,11 +143,7 @@ static bool isFatal(QtMsgType msgType)
     \sa QMessageLogContext, qDebug(), qWarning(), qCritical(), qFatal()
 */
 
-#if defined(Q_OS_WIN) && defined(QT_BUILD_CORE_LIB)
-// defined in qcoreapplication_win.cpp
-extern bool usingWinMain;
-#endif
-
+#ifdef Q_OS_WIN
 static inline void convert_to_wchar_t_elided(wchar_t *d, size_t space, const char *s) Q_DECL_NOEXCEPT
 {
     size_t len = qstrlen(s);
@@ -128,6 +157,7 @@ static inline void convert_to_wchar_t_elided(wchar_t *d, size_t space, const cha
         *d++ = *s++;
     *d++ = 0;
 }
+#endif
 
 #if !defined(QT_NO_EXCEPTIONS)
 /*!
@@ -140,7 +170,7 @@ static void qEmergencyOut(QtMsgType msgType, const char *msg, va_list ap) Q_DECL
 {
     char emergency_buf[256] = { '\0' };
     emergency_buf[sizeof emergency_buf - 1] = '\0';
-#if defined(Q_OS_WIN) && defined(QT_BUILD_CORE_LIB) && defined(Q_OS_WINCE) \
+#if defined(Q_OS_WIN) && defined(QT_BUILD_CORE_LIB) && (defined(Q_OS_WINCE) || defined(Q_OS_WINRT)) \
     || defined(Q_CC_MSVC) && defined(QT_DEBUG) && defined(_DEBUG) && defined(_CRT_ERROR)
     wchar_t emergency_bufL[sizeof emergency_buf];
 #endif
@@ -149,25 +179,23 @@ static void qEmergencyOut(QtMsgType msgType, const char *msg, va_list ap) Q_DECL
         qvsnprintf(emergency_buf, sizeof emergency_buf - 1, msg, ap);
 
 #if defined(Q_OS_WIN) && defined(QT_BUILD_CORE_LIB)
-# ifdef Q_OS_WINCE
+# if defined(Q_OS_WINCE) || defined(Q_OS_WINRT)
     convert_to_wchar_t_elided(emergency_bufL, sizeof emergency_buf, emergency_buf);
     OutputDebugStringW(emergency_bufL);
 # else
-    if (usingWinMain) {
-        OutputDebugStringA(emergency_buf);
-    } else {
-        fprintf(stderr, "%s", emergency_buf);
+    if (qWinLogToStderr()) {
+        fprintf(stderr, "%s\n", emergency_buf);
         fflush(stderr);
+    } else {
+        OutputDebugStringA(emergency_buf);
     }
 # endif
 #else
-    fprintf(stderr, "%s", emergency_buf);
+    fprintf(stderr, "%s\n", emergency_buf);
     fflush(stderr);
 #endif
 
-    if (msgType == QtFatalMsg
-            || (msgType == QtWarningMsg
-                && qEnvironmentVariableIsSet("QT_FATAL_WARNINGS"))) {
+    if (isFatal(msgType)) {
 #if defined(Q_CC_MSVC) && defined(QT_DEBUG) && defined(_DEBUG) && defined(_CRT_ERROR)
         // get the current report mode
         int reportMode = _CrtSetReportMode(_CRT_ERROR, _CRTDBG_MODE_WNDW);
@@ -497,7 +525,7 @@ Q_AUTOTEST_EXPORT QByteArray qCleanupFuncinfo(QByteArray info)
         templatecount = 1;
         --pos;
         while (pos && templatecount) {
-            register char c = info.at(pos);
+            char c = info.at(pos);
             if (c == '>')
                 ++templatecount;
             else if (c == '<')
@@ -521,9 +549,15 @@ static const char functionTokenC[] = "%{function}";
 static const char pidTokenC[] = "%{pid}";
 static const char appnameTokenC[] = "%{appname}";
 static const char threadidTokenC[] = "%{threadid}";
+static const char ifCategoryTokenC[] = "%{if-category}";
+static const char ifDebugTokenC[] = "%{if-debug}";
+static const char ifWarningTokenC[] = "%{if-warning}";
+static const char ifCriticalTokenC[] = "%{if-critical}";
+static const char ifFatalTokenC[] = "%{if-fatal}";
+static const char endifTokenC[] = "%{endif}";
 static const char emptyTokenC[] = "";
 
-static const char defaultPattern[] = "%{message}";
+static const char defaultPattern[] = "%{if-category}%{category}: %{endif}%{message}";
 
 
 struct QMessagePattern {
@@ -607,6 +641,10 @@ void QMessagePattern::setPattern(const QString &pattern)
     tokens = new const char*[lexemes.size() + 1];
     tokens[lexemes.size()] = 0;
 
+    bool nestedIfError = false;
+    bool inIf = false;
+    QString error;
+
     for (int i = 0; i < lexemes.size(); ++i) {
         const QString lexeme = lexemes.at(i);
         if (lexeme.startsWith(QLatin1String("%{"))
@@ -630,23 +668,29 @@ void QMessagePattern::setPattern(const QString &pattern)
                 tokens[i] = appnameTokenC;
             else if (lexeme == QLatin1String(threadidTokenC))
                 tokens[i] = threadidTokenC;
-            else {
+
+#define IF_TOKEN(LEVEL) \
+            else if (lexeme == QLatin1String(LEVEL)) { \
+                if (inIf) \
+                    nestedIfError = true; \
+                tokens[i] = LEVEL; \
+                inIf = true; \
+            }
+            IF_TOKEN(ifCategoryTokenC)
+            IF_TOKEN(ifDebugTokenC)
+            IF_TOKEN(ifWarningTokenC)
+            IF_TOKEN(ifCriticalTokenC)
+            IF_TOKEN(ifFatalTokenC)
+#undef IF_TOKEN
+            else if (lexeme == QLatin1String(endifTokenC)) {
+                tokens[i] = endifTokenC;
+                if (!inIf && !nestedIfError)
+                    error += QStringLiteral("QT_MESSAGE_PATTERN: %{endif} without an %{if-*}\n");
+                inIf = false;
+            } else {
                 tokens[i] = emptyTokenC;
-
-                QString error = QStringLiteral("QT_MESSAGE_PATTERN: Unknown placeholder %1\n")
+                error += QStringLiteral("QT_MESSAGE_PATTERN: Unknown placeholder %1\n")
                         .arg(lexeme);
-
-#if defined(Q_OS_WINCE)
-                OutputDebugString(reinterpret_cast<const wchar_t*>(error.utf16()));
-                continue;
-#elif defined(Q_OS_WIN) && defined(QT_BUILD_CORE_LIB)
-                if (usingWinMain) {
-                    OutputDebugString(reinterpret_cast<const wchar_t*>(error.utf16()));
-                    continue;
-                }
-#endif
-                fprintf(stderr, "%s", error.toLocal8Bit().constData());
-                fflush(stderr);
             }
         } else {
             char *literal = new char[lexeme.size() + 1];
@@ -656,6 +700,24 @@ void QMessagePattern::setPattern(const QString &pattern)
             tokens[i] = literal;
         }
     }
+    if (nestedIfError)
+        error += QStringLiteral("QT_MESSAGE_PATTERN: %{if-*} cannot be nested\n");
+    else if (inIf)
+        error += QStringLiteral("QT_MESSAGE_PATTERN: missing %{endif}\n");
+    if (!error.isEmpty()) {
+#if defined(Q_OS_WINCE) || defined(Q_OS_WINRT)
+        OutputDebugString(reinterpret_cast<const wchar_t*>(error.utf16()));
+        if (0)
+#elif defined(Q_OS_WIN) && defined(QT_BUILD_CORE_LIB)
+        if (!qWinLogToStderr()) {
+            OutputDebugString(reinterpret_cast<const wchar_t*>(error.utf16()));
+        } else
+#endif
+        {
+            fprintf(stderr, "%s", error.toLocal8Bit().constData());
+            fflush(stderr);
+        }
+    }
     literals = new const char*[literalsVar.size() + 1];
     literals[literalsVar.size()] = 0;
     memcpy(literals, literalsVar.constData(), literalsVar.size() * sizeof(const char*));
@@ -735,10 +797,16 @@ Q_CORE_EXPORT QString qMessageFormatString(QtMsgType type, const QMessageLogCont
     if (pattern->tokens[0] == 0)
         return message;
 
+    bool skip = false;
+
     // we do not convert file, function, line literals to local encoding due to overhead
     for (int i = 0; pattern->tokens[i] != 0; ++i) {
         const char *token = pattern->tokens[i];
-        if (token == messageTokenC) {
+        if (token == endifTokenC) {
+            skip = false;
+        } else if (skip) {
+            // do nothing
+        } else if (token == messageTokenC) {
             message.append(str);
         } else if (token == categoryTokenC) {
             message.append(QLatin1String(context.category));
@@ -770,6 +838,17 @@ Q_CORE_EXPORT QString qMessageFormatString(QtMsgType type, const QMessageLogCont
             message.append(QLatin1String("0x"));
             message.append(QString::number(qlonglong(QThread::currentThread()->currentThread()), 16));
 #endif
+        } else if (token == ifCategoryTokenC) {
+            if (!context.category || (strcmp(context.category, "default") == 0))
+                skip = true;
+#define HANDLE_IF_TOKEN(LEVEL)  \
+        } else if (token == if##LEVEL##TokenC) { \
+            skip = type != Qt##LEVEL##Msg;
+        HANDLE_IF_TOKEN(Debug)
+        HANDLE_IF_TOKEN(Warning)
+        HANDLE_IF_TOKEN(Critical)
+        HANDLE_IF_TOKEN(Fatal)
+#undef HANDLE_IF_TOKEN
         } else {
             message.append(QLatin1String(token));
         }
@@ -787,6 +866,25 @@ Q_CORE_EXPORT QtMsgHandler qInstallMsgHandler(QtMsgHandler);
 static QtMsgHandler msgHandler = 0;                // pointer to debug handler (without context)
 static QtMessageHandler messageHandler = 0;         // pointer to debug handler (with context)
 
+#ifdef Q_OS_ANDROID
+static void android_default_message_handler(QtMsgType type,
+                                  const QMessageLogContext &context,
+                                  const QString &message)
+{
+    android_LogPriority priority;
+    switch (type) {
+    case QtDebugMsg: priority = ANDROID_LOG_DEBUG; break;
+    case QtWarningMsg: priority = ANDROID_LOG_WARN; break;
+    case QtCriticalMsg: priority = ANDROID_LOG_ERROR; break;
+    case QtFatalMsg: priority = ANDROID_LOG_FATAL; break;
+    };
+
+    __android_log_print(priority, "Qt", "%s:%d (%s): %s",
+                        context.file, context.line,
+                        context.function, qPrintable(message));
+}
+#endif //Q_OS_ANDROID
+
 /*!
     \internal
 */
@@ -796,10 +894,7 @@ static void qDefaultMessageHandler(QtMsgType type, const QMessageLogContext &con
     QString logMessage = qMessageFormatString(type, context, buf);
 
 #if defined(Q_OS_WIN) && defined(QT_BUILD_CORE_LIB)
-#if !defined(Q_OS_WINCE)
-    if (usingWinMain)
-#endif
-    {
+    if (!qWinLogToStderr()) {
         OutputDebugString(reinterpret_cast<const wchar_t *>(logMessage.utf16()));
         return;
     }
@@ -807,6 +902,14 @@ static void qDefaultMessageHandler(QtMsgType type, const QMessageLogContext &con
 
 #if defined(QT_USE_SLOG2)
     slog2_default_handler(type, logMessage.toLocal8Bit().constData());
+#elif defined(Q_OS_ANDROID)
+    static bool logToAndroid = qEnvironmentVariableIsEmpty("QT_ANDROID_PLAIN_LOG");
+    if (logToAndroid) {
+        android_default_message_handler(type, context, logMessage);
+    } else {
+        fprintf(stderr, "%s", logMessage.toLocal8Bit().constData());
+        fflush(stderr);
+    }
 #else
     fprintf(stderr, "%s", logMessage.toLocal8Bit().constData());
     fflush(stderr);
@@ -819,11 +922,21 @@ static void qDefaultMessageHandler(QtMsgType type, const QMessageLogContext &con
 static void qDefaultMsgHandler(QtMsgType type, const char *buf)
 {
     QMessageLogContext emptyContext;
-    qDefaultMessageHandler(type, emptyContext, QLatin1String(buf));
+    qDefaultMessageHandler(type, emptyContext, QString::fromLocal8Bit(buf));
 }
 
 static void qt_message_print(QtMsgType msgType, const QMessageLogContext &context, const QString &message)
 {
+#ifndef QT_BOOTSTRAPPED
+    // qDebug, qWarning, ... macros do not check whether category is enabled
+    if (!context.category || (strcmp(context.category, "default") == 0)) {
+        if (QLoggingCategory *defaultCategory = QLoggingCategory::defaultCategory()) {
+            if (!defaultCategory->isEnabled(msgType))
+                return;
+        }
+    }
+#endif
+
     if (!msgHandler)
         msgHandler = qDefaultMsgHandler;
     if (!messageHandler)
@@ -841,21 +954,19 @@ static void qt_message_print(QtMsgType msgType, const QMessageLogContext &contex
 static void qt_message_fatal(QtMsgType, const QMessageLogContext &context, const QString &message)
 {
 #if defined(Q_CC_MSVC) && defined(QT_DEBUG) && defined(_DEBUG) && defined(_CRT_ERROR)
-        wchar_t contextFileL[256];
-        // we probably should let the compiler do this for us, by
-        // declaring QMessageLogContext::file to be const wchar_t * in
-        // the first place, but the #ifdefery above is very complex
-        // and we wouldn't be able to change it later on...
-        convert_to_wchar_t_elided(contextFileL, sizeof contextFileL / sizeof *contextFileL, context.file);
+    wchar_t contextFileL[256];
+    // we probably should let the compiler do this for us, by declaring QMessageLogContext::file to
+    // be const wchar_t * in the first place, but the #ifdefery above is very complex  and we
+    // wouldn't be able to change it later on...
+    convert_to_wchar_t_elided(contextFileL, sizeof contextFileL / sizeof *contextFileL,
+                              context.file);
     // get the current report mode
     int reportMode = _CrtSetReportMode(_CRT_ERROR, _CRTDBG_MODE_WNDW);
     _CrtSetReportMode(_CRT_ERROR, reportMode);
 
-        int ret = _CrtDbgReportW(_CRT_ERROR, contextFileL,
-                             context.line, _CRT_WIDE(QT_VERSION_STR),
-                             reinterpret_cast<const wchar_t *> (
-                                 message.utf16()));
-    if (ret == 0  && reportMode & _CRTDBG_MODE_WNDW)
+    int ret = _CrtDbgReportW(_CRT_ERROR, contextFileL, context.line, _CRT_WIDE(QT_VERSION_STR),
+                             reinterpret_cast<const wchar_t *>(message.utf16()));
+    if ((ret == 0) && (reportMode & _CRTDBG_MODE_WNDW))
         return; // ignore
     else if (ret == 1)
         _CrtDbgBreak();
@@ -1000,6 +1111,7 @@ void qErrnoWarning(int code, const char *msg, ...)
     \table
     \header \li Placeholder \li Description
     \row \li \c %{appname} \li QCoreApplication::applicationName()
+    \row \li \c %{category} \li Logging category
     \row \li \c %{file} \li Path to source file
     \row \li \c %{function} \li Function
     \row \li \c %{line} \li Line in source file
@@ -1009,6 +1121,15 @@ void qErrnoWarning(int code, const char *msg, ...)
     \row \li \c %{type} \li "debug", "warning", "critical" or "fatal"
     \endtable
 
+    You can also use conditionals on the type of the message using \c %{if-debug},
+    \c %{if-warning}, \c %{if-critical} or \c %{if-fatal} followed by an \c %{endif}.
+    What is inside the \c %{if-*} and \c %{endif} will only be printed if the type matches.
+
+    Example:
+    \code
+    QT_MESSAGE_PATTERN="[%{if-debug}D%{endif}%{if-warning}W%{endif}%{if-critical}C%{endif}%{if-fatal}F%{endif}] %{file}:%{line} - %{message}"
+    \endcode
+
     The default \a pattern is "%{message}".
 
     The \a pattern can also be changed at runtime by setting the QT_MESSAGE_PATTERN
@@ -1048,6 +1169,7 @@ void qSetMessagePattern(const QString &pattern)
         qMessagePattern()->setPattern(pattern);
 }
 
+
 /*!
     Copies context information from \a logContext into this QMessageLogContext
     \internal
@@ -1101,7 +1223,7 @@ void QMessageLogContext::copy(const QMessageLogContext &logContext)
     \internal
 
     Constructs a QMessageLogContext with for file \a fileName at line
-    \a lineNumber, in function \a functionName, and category \a CategoryName.
+    \a lineNumber, in function \a functionName, and category \a categoryName.
 */
 
 QT_END_NAMESPACE
