@@ -44,8 +44,6 @@
 
 #include <QtCore/qdir.h>
 
-QT_BEGIN_HEADER
-
 QT_BEGIN_NAMESPACE
 
 
@@ -91,6 +89,4 @@ Q_DECLARE_OPERATORS_FOR_FLAGS(QDirIterator::IteratorFlags)
 
 QT_END_NAMESPACE
 
-QT_END_HEADER
-
 #endif
