@@ -50,8 +50,6 @@
 #error qfile.h must be included before any header file that defines open
 #endif
 
-QT_BEGIN_HEADER
-
 QT_BEGIN_NAMESPACE
 
 class QTemporaryFile;
@@ -154,6 +152,4 @@ private:
 
 QT_END_NAMESPACE
 
-QT_END_HEADER
-
 #endif // QFILE_H
