@@ -47,8 +47,6 @@
 #include <QtCore/qshareddata.h>
 #include <QtCore/qmetatype.h>
 
-QT_BEGIN_HEADER
-
 QT_BEGIN_NAMESPACE
 
 
@@ -86,6 +84,7 @@ public:
     void setFile(const QFile &file);
     void setFile(const QDir &dir, const QString &file);
     bool exists() const;
+    static bool exists(const QString &file);
     void refresh();
 
     QString filePath() const;
@@ -159,6 +158,4 @@ QT_END_NAMESPACE
 
 Q_DECLARE_METATYPE(QFileInfo)
 
-QT_END_HEADER
-
 #endif // QFILEINFO_H
