@@ -256,6 +256,23 @@ bool QFSFileEnginePrivate::nativeFlush()
     return fh ? flushFh() : fd != -1;
 }
 
+/*!
+    \internal
+    \since 5.1
+*/
+bool QFSFileEnginePrivate::nativeSyncToDisk()
+{
+    Q_Q(QFSFileEngine);
+#if defined(_POSIX_SYNCHRONIZED_IO) && _POSIX_SYNCHRONIZED_IO > 0
+    const int ret = fdatasync(nativeHandle());
+#else
+    const int ret = fsync(nativeHandle());
+#endif
+    if (ret != 0)
+        q->setError(QFile::WriteError, qt_error_string(errno));
+    return ret == 0;
+}
+
 /*!
     \internal
 */
@@ -388,6 +405,12 @@ bool QFSFileEngine::copy(const QString &newName)
     return ret;
 }
 
+bool QFSFileEngine::renameOverwrite(const QString &newName)
+{
+    // On Unix, rename() overwrites.
+    return rename(newName);
+}
+
 bool QFSFileEngine::rename(const QString &newName)
 {
     Q_D(QFSFileEngine);
