@@ -125,16 +125,15 @@ QT_BEGIN_NAMESPACE
     The environment of the calling process can be obtained using
     QProcessEnvironment::systemEnvironment().
 
-    On Unix systems, the variable names are case-sensitive. For that reason,
-    this class will not touch the names of the variables. Note as well that
+    On Unix systems, the variable names are case-sensitive. Note that the
     Unix environment allows both variable names and contents to contain arbitrary
-    binary data (except for the NUL character), but this is not supported by
-    QProcessEnvironment. This class only supports names and values that are
-    encodable by the current locale settings (see QTextCodec::codecForLocale).
+    binary data (except for the NUL character). QProcessEnvironment will preserve
+    such variables, but does not support manipulating variables whose names or
+    values are not encodable by the current locale settings (see
+    QTextCodec::codecForLocale).
 
-    On Windows, the variable names are case-insensitive. Therefore,
-    QProcessEnvironment will always uppercase the names and do case-insensitive
-    comparisons.
+    On Windows, the variable names are case-insensitive, but case-preserving.
+    QProcessEnvironment behaves accordingly.
 
     On Windows CE, the concept of environment does not exist. This class will
     keep the values set for compatibility with other platforms, but the values
@@ -250,13 +249,13 @@ QProcessEnvironment &QProcessEnvironment::operator=(const QProcessEnvironment &o
 /*!
     \fn bool QProcessEnvironment::operator !=(const QProcessEnvironment &other) const
 
-    Returns true if this and the \a other QProcessEnvironment objects are different.
+    Returns \c true if this and the \a other QProcessEnvironment objects are different.
 
     \sa operator==()
 */
 
 /*!
-    Returns true if this and the \a other QProcessEnvironment objects are equal.
+    Returns \c true if this and the \a other QProcessEnvironment objects are equal.
 
     Two QProcessEnvironment objects are considered equal if they have the same
     set of key=value pairs. The comparison of keys is done case-sensitive on
@@ -266,17 +265,24 @@ QProcessEnvironment &QProcessEnvironment::operator=(const QProcessEnvironment &o
 */
 bool QProcessEnvironment::operator==(const QProcessEnvironment &other) const
 {
-    return d == other.d || (d && other.d && d->hash == other.d->hash);
+    if (d == other.d)
+        return true;
+    if (d && other.d) {
+        QProcessEnvironmentPrivate::OrderedMutexLocker locker(d, other.d);
+        return d->hash == other.d->hash;
+    }
+    return false;
 }
 
 /*!
-    Returns true if this QProcessEnvironment object is empty: that is
+    Returns \c true if this QProcessEnvironment object is empty: that is
     there are no key=value pairs set.
 
     \sa clear(), systemEnvironment(), insert()
 */
 bool QProcessEnvironment::isEmpty() const
 {
+    // Needs no locking, as no hash nodes are accessed
     return d ? d->hash.isEmpty() : true;
 }
 
@@ -295,18 +301,18 @@ void QProcessEnvironment::clear()
 }
 
 /*!
-    Returns true if the environment variable of name \a name is found in
+    Returns \c true if the environment variable of name \a name is found in
     this QProcessEnvironment object.
 
-    On Windows, variable names are case-insensitive, so the key is converted
-    to uppercase before searching. On other systems, names are case-sensitive
-    so no trasformation is applied.
 
     \sa insert(), value()
 */
 bool QProcessEnvironment::contains(const QString &name) const
 {
-    return d ? d->hash.contains(d->prepareName(name)) : false;
+    if (!d)
+        return false;
+    QProcessEnvironmentPrivate::MutexLocker locker(d);
+    return d->hash.contains(d->prepareName(name));
 }
 
 /*!
@@ -314,10 +320,6 @@ bool QProcessEnvironment::contains(const QString &name) const
     into this QProcessEnvironment object. If that variable already existed,
     it is replaced by the new value.
 
-    On Windows, variable names are case-insensitive, so this function always
-    uppercases the variable name before inserting. On other systems, names
-    are case-sensitive, so no transformation is applied.
-
     On most systems, inserting a variable with no contents will have the
     same effect for applications as if the variable had not been set at all.
     However, to guarantee that there are no incompatibilities, to remove a
@@ -327,7 +329,8 @@ bool QProcessEnvironment::contains(const QString &name) const
 */
 void QProcessEnvironment::insert(const QString &name, const QString &value)
 {
-    // d detaches from null
+    // our re-impl of detach() detaches from null
+    d.detach(); // detach before prepareName()
     d->hash.insert(d->prepareName(name), d->prepareValue(value));
 }
 
@@ -336,16 +339,15 @@ void QProcessEnvironment::insert(const QString &name, const QString &value)
     QProcessEnvironment object. If that variable did not exist before,
     nothing happens.
 
-    On Windows, variable names are case-insensitive, so the key is converted
-    to uppercase before searching. On other systems, names are case-sensitive
-    so no trasformation is applied.
 
     \sa contains(), insert(), value()
 */
 void QProcessEnvironment::remove(const QString &name)
 {
-    if (d)
+    if (d) {
+        d.detach(); // detach before prepareName()
         d->hash.remove(d->prepareName(name));
+    }
 }
 
 /*!
@@ -353,10 +355,6 @@ void QProcessEnvironment::remove(const QString &name)
     \a name and returns its value. If the variable is not found in this object,
     then \a defaultValue is returned instead.
 
-    On Windows, variable names are case-insensitive, so the key is converted
-    to uppercase before searching. On other systems, names are case-sensitive
-    so no trasformation is applied.
-
     \sa contains(), insert(), remove()
 */
 QString QProcessEnvironment::value(const QString &name, const QString &defaultValue) const
@@ -364,6 +362,7 @@ QString QProcessEnvironment::value(const QString &name, const QString &defaultVa
     if (!d)
         return defaultValue;
 
+    QProcessEnvironmentPrivate::MutexLocker locker(d);
     QProcessEnvironmentPrivate::Hash::ConstIterator it = d->hash.constFind(d->prepareName(name));
     if (it == d->hash.constEnd())
         return defaultValue;
@@ -376,17 +375,20 @@ QString QProcessEnvironment::value(const QString &name, const QString &defaultVa
     each environment variable that is set. The environment variable's name
     and its value are separated by an equal character ('=').
 
-    The QStringList contents returned by this function are suitable for use
-    with the QProcess::setEnvironment function. However, it is recommended
-    to use QProcess::setProcessEnvironment instead since that will avoid
-    unnecessary copying of the data.
+    The QStringList contents returned by this function are suitable for
+    presentation.
+    Use with the QProcess::setEnvironment function is not recommended due to
+    potential encoding problems under Unix, and worse performance.
 
     \sa systemEnvironment(), QProcess::systemEnvironment(), QProcess::environment(),
         QProcess::setEnvironment()
 */
 QStringList QProcessEnvironment::toStringList() const
 {
-    return d ? d->toList() : QStringList();
+    if (!d)
+        return QStringList();
+    QProcessEnvironmentPrivate::MutexLocker locker(d);
+    return d->toList();
 }
 
 /*!
@@ -397,7 +399,10 @@ QStringList QProcessEnvironment::toStringList() const
 */
 QStringList QProcessEnvironment::keys() const
 {
-    return d ? d->keys() : QStringList();
+    if (!d)
+        return QStringList();
+    QProcessEnvironmentPrivate::MutexLocker locker(d);
+    return d->keys();
 }
 
 /*!
@@ -412,7 +417,8 @@ void QProcessEnvironment::insert(const QProcessEnvironment &e)
     if (!e.d)
         return;
 
-    // d detaches from null
+    // our re-impl of detach() detaches from null
+    QProcessEnvironmentPrivate::MutexLocker locker(e.d);
     d->insert(*e.d);
 }
 
@@ -458,6 +464,9 @@ void QProcessPrivate::Channel::clear()
     the program you want to run as arguments to start(). Arguments
     are supplied as individual strings in a QStringList.
 
+    Alternatively, you can set the program to run with setProgram()
+    and setArguments(), and then call start() or open().
+
     For example, the following code snippet runs the analog clock
     example in the Fusion style on X11 platforms by passing strings
     containing "-style" and "fusion" as two items in the list of
@@ -524,7 +533,11 @@ void QProcessPrivate::Channel::clear()
     MergedChannels before starting the process to activative
     this feature. You also have the option of forwarding the output of
     the running process to the calling, main process, by passing
-    ForwardedChannels as the argument.
+    ForwardedChannels as the argument. It is also possible to forward
+    only one of the output channels - typically one would use
+    ForwardedErrorChannel, but ForwardedOutputChannel also exists.
+    Note that using channel forwarding is typically a bad idea in GUI
+    applications - you should present errors graphically instead.
 
     Certain processes need special environment settings in order to
     operate. You can set environment variables for your process by
@@ -595,8 +608,8 @@ void QProcessPrivate::Channel::clear()
 /*!
     \enum QProcess::ProcessChannelMode
 
-    This enum describes the process channel modes of QProcess. Pass
-    one of these values to setProcessChannelMode() to set the
+    This enum describes the process output channel modes of QProcess.
+    Pass one of these values to setProcessChannelMode() to set the
     current read channel mode.
 
     \value SeparateChannels QProcess manages the output of the
@@ -616,6 +629,17 @@ void QProcessPrivate::Channel::clear()
     writes to its standard output and standard error will be written
     to the standard output and standard error of the main process.
 
+    \value ForwardedErrorChannel QProcess manages the standard output
+    of the running process, but forwards its standard error onto the
+    main process. This reflects the typical use of command line tools
+    as filters, where the standard output is redirected to another
+    process or a file, while standard error is printed to the console
+    for diagnostic purposes.
+    (This value was introduced in Qt 5.2.)
+
+    \value ForwardedOutputChannel Complementary to ForwardedErrorChannel.
+    (This value was introduced in Qt 5.2.)
+
     \note Windows intentionally suppresses output from GUI-only
     applications to inherited consoles.
     This does \e not apply to output redirected to files or pipes.
@@ -627,6 +651,26 @@ void QProcessPrivate::Channel::clear()
     \sa setProcessChannelMode()
 */
 
+/*!
+    \enum QProcess::InputChannelMode
+    \since 5.2
+
+    This enum describes the process input channel modes of QProcess.
+    Pass one of these values to setInputChannelMode() to set the
+    current write channel mode.
+
+    \value ManagedInputChannel QProcess manages the input of the running
+    process. This is the default input channel mode of QProcess.
+
+    \value ForwardedInputChannel QProcess forwards the input of the main
+    process onto the running process. The child process reads its standard
+    input from the same source as the main process.
+    Note that the main process must not try to read its standard input
+    while the child process is running.
+
+    \sa setInputChannelMode()
+*/
+
 /*!
     \enum QProcess::ProcessError
 
@@ -718,9 +762,11 @@ void QProcessPrivate::Channel::clear()
     \fn void QProcess::finished(int exitCode, QProcess::ExitStatus exitStatus)
 
     This signal is emitted when the process finishes. \a exitCode is the exit
-    code of the process, and \a exitStatus is the exit status.  After the
-    process has finished, the buffers in QProcess are still intact. You can
-    still read any data that the process may have written before it finished.
+    code of the process (only valid for normal exits), and \a exitStatus is
+    the exit status.
+    After the process has finished, the buffers in QProcess are still intact.
+    You can still read any data that the process may have written before it
+    finished.
 
     \sa exitStatus()
 */
@@ -753,6 +799,7 @@ QProcessPrivate::QProcessPrivate()
 {
     processChannel = QProcess::StandardOutput;
     processChannelMode = QProcess::SeparateChannels;
+    inputChannelMode = QProcess::ManagedInputChannel;
     processError = QProcess::UnknownError;
     processState = QProcess::NotRunning;
     pid = 0;
@@ -807,8 +854,7 @@ void QProcessPrivate::cleanup()
         pid = 0;
     }
     if (processFinishedNotifier) {
-        processFinishedNotifier->setEnabled(false);
-        qDeleteInEventHandler(processFinishedNotifier);
+        delete processFinishedNotifier;
         processFinishedNotifier = 0;
     }
 
@@ -818,33 +864,28 @@ void QProcessPrivate::cleanup()
     dying = false;
 
     if (stdoutChannel.notifier) {
-        stdoutChannel.notifier->setEnabled(false);
-        qDeleteInEventHandler(stdoutChannel.notifier);
+        delete stdoutChannel.notifier;
         stdoutChannel.notifier = 0;
     }
     if (stderrChannel.notifier) {
-        stderrChannel.notifier->setEnabled(false);
-        qDeleteInEventHandler(stderrChannel.notifier);
+        delete stderrChannel.notifier;
         stderrChannel.notifier = 0;
     }
     if (stdinChannel.notifier) {
-        stdinChannel.notifier->setEnabled(false);
-        qDeleteInEventHandler(stdinChannel.notifier);
+        delete stdinChannel.notifier;
         stdinChannel.notifier = 0;
     }
     if (startupSocketNotifier) {
-        startupSocketNotifier->setEnabled(false);
-        qDeleteInEventHandler(startupSocketNotifier);
+        delete startupSocketNotifier;
         startupSocketNotifier = 0;
     }
     if (deathNotifier) {
-        deathNotifier->setEnabled(false);
-        qDeleteInEventHandler(deathNotifier);
+        delete deathNotifier;
         deathNotifier = 0;
     }
 #ifdef Q_OS_WIN
     if (notifier) {
-        qDeleteInEventHandler(notifier);
+        delete notifier;
         notifier = 0;
     }
 #endif
@@ -1018,6 +1059,7 @@ bool QProcessPrivate::_q_processDied()
         return false;
 #endif
 #ifdef Q_OS_WIN
+    drainOutputPipes();
     if (processFinishedNotifier)
         processFinishedNotifier->setEnabled(false);
 #endif
@@ -1113,12 +1155,8 @@ void QProcessPrivate::closeWriteChannel()
     qDebug("QProcessPrivate::closeWriteChannel()");
 #endif
     if (stdinChannel.notifier) {
-        extern void qDeleteInEventHandler(QObject *o);
-        stdinChannel.notifier->setEnabled(false);
-        if (stdinChannel.notifier) {
-            qDeleteInEventHandler(stdinChannel.notifier);
-            stdinChannel.notifier = 0;
-        }
+        delete stdinChannel.notifier;
+        stdinChannel.notifier = 0;
     }
 #ifdef Q_OS_WIN
     // ### Find a better fix, feeding the process little by little
@@ -1216,6 +1254,34 @@ void QProcess::setProcessChannelMode(ProcessChannelMode mode)
     d->processChannelMode = mode;
 }
 
+/*!
+    \since 5.2
+
+    Returns the channel mode of the QProcess standard input channel.
+
+    \sa setInputChannelMode(), InputChannelMode
+*/
+QProcess::InputChannelMode QProcess::inputChannelMode() const
+{
+    Q_D(const QProcess);
+    return d->inputChannelMode;
+}
+
+/*!
+    \since 5.2
+
+    Sets the channel mode of the QProcess standard intput
+    channel to the \a mode specified.
+    This mode will be used the next time start() is called.
+
+    \sa inputChannelMode(), InputChannelMode
+*/
+void QProcess::setInputChannelMode(InputChannelMode mode)
+{
+    Q_D(QProcess);
+    d->inputChannelMode = mode;
+}
+
 /*!
     Returns the current read channel of the QProcess.
 
@@ -1305,6 +1371,10 @@ void QProcess::closeWriteChannel()
     object will be in read-only mode (calling write() will result in
     error).
 
+    To make the process read EOF right away, pass nullDevice() here.
+    This is cleaner than using closeWriteChannel() before writing any
+    data, because it can be set up prior to starting the process.
+
     If the file \a fileName does not exist at the moment start() is
     called or is not readable, starting the process will fail.
 
@@ -1328,6 +1398,10 @@ void QProcess::setStandardInputFile(const QString &fileName)
     read channel is closed: reading from it using read() will always
     fail, as will readAllStandardOutput().
 
+    To discard all standard output from the process, pass nullDevice()
+    here. This is more efficient than simply never reading the standard
+    output, as no QProcess buffers are filled.
+
     If the file \a fileName doesn't exist at the moment start() is
     called, it will be created. If it cannot be created, the starting
     will fail.
@@ -1513,8 +1587,8 @@ void QProcess::close()
 
 /*! \reimp
 
-   Returns true if the process is not running, and no more data is available
-   for reading; otherwise returns false.
+   Returns \c true if the process is not running, and no more data is available
+   for reading; otherwise returns \c false.
 */
 bool QProcess::atEnd() const
 {
@@ -1661,8 +1735,8 @@ QProcessEnvironment QProcess::processEnvironment() const
     Blocks until the process has started and the started() signal has
     been emitted, or until \a msecs milliseconds have passed.
 
-    Returns true if the process was started successfully; otherwise
-    returns false (if the operation timed out or if an error
+    Returns \c true if the process was started successfully; otherwise
+    returns \c false (if the operation timed out or if an error
     occurred).
 
     This function can operate without an event loop. It is
@@ -1679,10 +1753,10 @@ QProcessEnvironment QProcess::processEnvironment() const
 bool QProcess::waitForStarted(int msecs)
 {
     Q_D(QProcess);
-    if (d->processState == QProcess::Running)
-        return true;
+    if (d->processState == QProcess::Starting)
+        return d->waitForStarted(msecs);
 
-    return d->waitForStarted(msecs);
+    return d->processState == QProcess::Running;
 }
 
 /*! \reimp
@@ -1724,7 +1798,7 @@ bool QProcess::waitForBytesWritten(int msecs)
     Blocks until the process has finished and the finished() signal
     has been emitted, or until \a msecs milliseconds have passed.
 
-    Returns true if the process finished; otherwise returns false (if
+    Returns \c true if the process finished; otherwise returns \c false (if
     the operation timed out, if an error occurred, or if this QProcess
     is already finished).
 
@@ -1946,35 +2020,101 @@ void QProcess::start(const QString &program, const QStringList &arguments, OpenM
         return;
     }
 
+    d->program = program;
+    d->arguments = arguments;
+
+    d->start(mode);
+}
+
+/*!
+    \since 5.1
+    \overload
+
+    Starts the program set by setProgram() with arguments set by setArguments().
+    The OpenMode is set to \a mode.
+
+    This method is a convenient alias to open().
+
+    \sa open(), setProgram(), setArguments()
+ */
+void QProcess::start(OpenMode mode)
+{
+    Q_D(QProcess);
+    if (d->processState != NotRunning) {
+        qWarning("QProcess::start: Process is already running");
+        return;
+    }
+    if (d->program.isEmpty()) {
+        qWarning("QProcess::start: program not set");
+        return;
+    }
+
+    d->start(mode);
+}
+
+/*!
+    Starts the program set by setProgram() in a new process, if none is already
+    running, passing the command line arguments set by setArguments(). The OpenMode
+    is set to \a mode.
+
+    The QProcess object will immediately enter the Starting state. If the
+    process starts successfully, QProcess will emit started(); otherwise,
+    error() will be emitted. If the QProcess object is already running a
+    process, a warning may be printed at the console, the function will return false,
+    and the existing process will continue running.
+
+    \note Processes are started asynchronously, which means the started()
+    and error() signals may be delayed. Call waitForStarted() to make
+    sure the process has started (or has failed to start) and those signals
+    have been emitted. In this regard, a true return value merly means the process
+    was correcty initialized, not that the program was actually started.
+
+*/
+bool QProcess::open(OpenMode mode)
+{
+    Q_D(QProcess);
+    if (d->processState != NotRunning) {
+        qWarning("QProcess::start: Process is already running");
+        return false;
+    }
+    if (d->program.isEmpty()) {
+        qWarning("QProcess::start: program not set");
+        return false;
+    }
+
+    d->start(mode);
+    return true;
+}
+
+void QProcessPrivate::start(QIODevice::OpenMode mode)
+{
+    Q_Q(QProcess);
 #if defined QPROCESS_DEBUG
     qDebug() << "QProcess::start(" << program << ',' << arguments << ',' << mode << ')';
 #endif
 
-    d->outputReadBuffer.clear();
-    d->errorReadBuffer.clear();
+    outputReadBuffer.clear();
+    errorReadBuffer.clear();
 
-    if (d->stdinChannel.type != QProcessPrivate::Channel::Normal)
-        mode &= ~WriteOnly;     // not open for writing
-    if (d->stdoutChannel.type != QProcessPrivate::Channel::Normal &&
-        (d->stderrChannel.type != QProcessPrivate::Channel::Normal ||
-         d->processChannelMode == MergedChannels))
-        mode &= ~ReadOnly;      // not open for reading
+    if (stdinChannel.type != QProcessPrivate::Channel::Normal)
+        mode &= ~QIODevice::WriteOnly;     // not open for writing
+    if (stdoutChannel.type != QProcessPrivate::Channel::Normal &&
+        (stderrChannel.type != QProcessPrivate::Channel::Normal ||
+         processChannelMode == QProcess::MergedChannels))
+        mode &= ~QIODevice::ReadOnly;      // not open for reading
     if (mode == 0)
-        mode = Unbuffered;
-    QIODevice::open(mode);
+        mode = QIODevice::Unbuffered;
+    q->QIODevice::open(mode);
 
-    d->stdinChannel.closed = false;
-    d->stdoutChannel.closed = false;
-    d->stderrChannel.closed = false;
+    stdinChannel.closed = false;
+    stdoutChannel.closed = false;
+    stderrChannel.closed = false;
 
-    d->program = program;
-    d->arguments = arguments;
-
-    d->exitCode = 0;
-    d->exitStatus = NormalExit;
-    d->processError = QProcess::UnknownError;
-    d->errorString.clear();
-    d->startProcess();
+    exitCode = 0;
+    exitStatus = QProcess::NormalExit;
+    processError = QProcess::UnknownError;
+    errorString.clear();
+    startProcess();
 }
 
 
@@ -2073,6 +2213,24 @@ QString QProcess::program() const
     return d->program;
 }
 
+/*!
+    \since 5.1
+
+    Set the \a program to use when starting the process.
+    That function must be call before open()
+
+    \sa start(), setArguments(), program()
+*/
+void QProcess::setProgram(const QString &program)
+{
+    Q_D(QProcess);
+    if (d->processState != NotRunning) {
+        qWarning("QProcess::setProgram: Process is already running");
+        return;
+    }
+    d->program = program;
+}
+
 /*!
     Returns the command line arguments the process was last started with.
 
@@ -2084,6 +2242,24 @@ QStringList QProcess::arguments() const
     return d->arguments;
 }
 
+/*!
+    \since 5.1
+
+    Set the \a arguments to pass to the called program when starting the process.
+    That function must be call before  open()
+
+    \sa start(), setProgram(), arguments()
+*/
+void QProcess::setArguments(const QStringList &arguments)
+{
+    Q_D(QProcess);
+    if (d->processState != NotRunning) {
+        qWarning("QProcess::setProgram: Process is already running");
+        return;
+    }
+    d->arguments = arguments;
+}
+
 /*!
     Attempts to terminate the process.
 
@@ -2122,6 +2298,8 @@ void QProcess::kill()
 
 /*!
     Returns the exit code of the last process that finished.
+
+    This value is not valid unless exitStatus() returns NormalExit.
 */
 int QProcess::exitCode() const
 {
@@ -2188,8 +2366,8 @@ int QProcess::execute(const QString &program)
 
 /*!
     Starts the program \a program with the arguments \a arguments in a
-    new process, and detaches from it. Returns true on success;
-    otherwise returns false. If the calling process exits, the
+    new process, and detaches from it. Returns \c true on success;
+    otherwise returns \c false. If the calling process exits, the
     detached process will continue to live.
 
     Note that arguments that contain spaces are not passed to the
@@ -2222,8 +2400,8 @@ bool QProcess::startDetached(const QString &program,
 
 /*!
     Starts the program \a program with the given \a arguments in a
-    new process, and detaches from it. Returns true on success;
-    otherwise returns false. If the calling process exits, the
+    new process, and detaches from it. Returns \c true on success;
+    otherwise returns \c false. If the calling process exits, the
     detached process will continue to live.
 
     \note Arguments that contain spaces are not passed to the
@@ -2264,7 +2442,7 @@ bool QProcess::startDetached(const QString &program)
 }
 
 QT_BEGIN_INCLUDE_NAMESPACE
-#if defined(Q_OS_MAC) && !defined(Q_OS_IOS)
+#if defined(Q_OS_MACX)
 # include <crt_externs.h>
 # define environ (*_NSGetEnviron())
 #elif defined(Q_OS_WINCE) || defined(Q_OS_IOS)
@@ -2323,6 +2501,25 @@ QStringList QProcess::systemEnvironment()
     \sa QProcess::systemEnvironment()
 */
 
+/*!
+    \since 5.2
+
+    \brief The null device of the operating system.
+
+    The returned file path uses native directory separators.
+
+    \sa QProcess::setStandardInputFile(), QProcess::setStandardOutputFile(),
+        QProcess::setStandardErrorFile()
+*/
+QString QProcess::nullDevice()
+{
+#ifdef Q_OS_WIN
+    return QStringLiteral("\\\\.\\NUL");
+#else
+    return QStringLiteral("/dev/null");
+#endif
+}
+
 /*!
     \typedef Q_PID
     \relates QProcess
