@@ -121,17 +121,34 @@ static const int errorBufferMax = 512;
 
 static int qt_qprocess_deadChild_pipe[2];
 static struct sigaction qt_sa_old_sigchld_handler;
-static void qt_sa_sigchld_handler(int signum)
+static void qt_sa_sigchld_sigaction(int signum, siginfo_t *info, void *context)
 {
+    // *Never* use the info or contect variables in this function
+    // (except for passing them to the next signal in the chain).
+    // We cannot be sure if another library or if the application
+    // installed a signal handler for SIGCHLD without SA_SIGINFO
+    // and fails to pass the arguments to us. If they do that,
+    // these arguments contain garbage and we'd most likely crash.
+
     qt_safe_write(qt_qprocess_deadChild_pipe[1], "", 1);
 #if defined (QPROCESS_DEBUG)
     fprintf(stderr, "*** SIGCHLD\n");
 #endif
 
-    // load it as volatile
-    void (*oldAction)(int) = ((volatile struct sigaction *)&qt_sa_old_sigchld_handler)->sa_handler;
-    if (oldAction && oldAction != SIG_IGN)
-        oldAction(signum);
+    // load as volatile
+    volatile struct sigaction *vsa = &qt_sa_old_sigchld_handler;
+
+    if (qt_sa_old_sigchld_handler.sa_flags & SA_SIGINFO) {
+        void (*oldAction)(int, siginfo_t *, void *) = vsa->sa_sigaction;
+
+        if (oldAction)
+            oldAction(signum, info, context);
+    } else {
+        void (*oldAction)(int) = vsa->sa_handler;
+
+        if (oldAction && oldAction != SIG_IGN)
+            oldAction(signum);
+    }
 }
 
 static inline void add_fd(int &nfds, int fd, fd_set *fdset)
@@ -197,10 +214,16 @@ QProcessManager::QProcessManager()
 
     // set up the SIGCHLD handler, which writes a single byte to the dead
     // child pipe every time a child dies.
+
     struct sigaction action;
-    memset(&action, 0, sizeof(action));
-    action.sa_handler = qt_sa_sigchld_handler;
-    action.sa_flags = SA_NOCLDSTOP;
+    // use the old handler as template, i.e., preserve the signal mask
+    // otherwise the original signal handler might be interrupted although it
+    // was marked to never be interrupted
+    ::sigaction(SIGCHLD, NULL, &action);
+    action.sa_sigaction = qt_sa_sigchld_sigaction;
+    // set the SA_SIGINFO flag such that we can use the three argument handler
+    // function
+    action.sa_flags = SA_NOCLDSTOP | SA_SIGINFO;
     ::sigaction(SIGCHLD, &action, &qt_sa_old_sigchld_handler);
 
     processManagerInstance = this;
@@ -225,7 +248,7 @@ QProcessManager::~QProcessManager()
 
     struct sigaction currentAction;
     ::sigaction(SIGCHLD, 0, &currentAction);
-    if (currentAction.sa_handler == qt_sa_sigchld_handler) {
+    if (currentAction.sa_sigaction == qt_sa_sigchld_sigaction) {
         ::sigaction(SIGCHLD, &qt_sa_old_sigchld_handler, 0);
     }
 
@@ -328,16 +351,18 @@ void QProcessManager::unlock()
     mutex.unlock();
 }
 
-static void qt_create_pipe(int *pipe)
+static int qt_create_pipe(int *pipe)
 {
     if (pipe[0] != -1)
         qt_safe_close(pipe[0]);
     if (pipe[1] != -1)
         qt_safe_close(pipe[1]);
-    if (qt_safe_pipe(pipe) != 0) {
+    int pipe_ret = qt_safe_pipe(pipe);
+    if (pipe_ret != 0) {
         qWarning("QProcessPrivate::createPipe: Cannot create pipe %p: %s",
                  pipe, qPrintable(qt_error_string(errno)));
     }
+    return pipe_ret;
 }
 
 void QProcessPrivate::destroyPipe(int *pipe)
@@ -374,10 +399,11 @@ bool QProcessPrivate::createChannel(Channel &channel)
 
     if (channel.type == Channel::Normal) {
         // we're piping this channel to our own process
-        qt_create_pipe(channel.pipe);
+        if (qt_create_pipe(channel.pipe) != 0)
+            return false;
 
         // create the socket notifiers
-        if (threadData->eventDispatcher) {
+        if (threadData->hasEventDispatcher()) {
             if (&channel == &stdinChannel) {
                 channel.notifier = new QSocketNotifier(channel.pipe[1],
                                                        QSocketNotifier::Write, q);
@@ -458,7 +484,8 @@ bool QProcessPrivate::createChannel(Channel &channel)
             Q_ASSERT(sink->pipe[0] == INVALID_Q_PIPE && sink->pipe[1] == INVALID_Q_PIPE);
 
             Q_PIPE pipe[2] = { -1, -1 };
-            qt_create_pipe(pipe);
+            if (qt_create_pipe(pipe) != 0)
+                return false;
             sink->pipe[0] = pipe[0];
             source->pipe[1] = pipe[1];
 
@@ -468,7 +495,7 @@ bool QProcessPrivate::createChannel(Channel &channel)
 }
 
 QT_BEGIN_INCLUDE_NAMESPACE
-#if defined(Q_OS_MAC) && !defined(Q_OS_IOS)
+#if defined(Q_OS_MACX)
 # include <crt_externs.h>
 # define environ (*_NSGetEnviron())
 #else
@@ -548,12 +575,17 @@ void QProcessPrivate::startProcess()
     // Initialize pipes
     if (!createChannel(stdinChannel) ||
         !createChannel(stdoutChannel) ||
-        !createChannel(stderrChannel))
+        !createChannel(stderrChannel) ||
+        qt_create_pipe(childStartedPipe) != 0 ||
+        qt_create_pipe(deathPipe) != 0) {
+        processError = QProcess::FailedToStart;
+        q->setErrorString(qt_error_string(errno));
+        emit q->error(processError);
+        cleanup();
         return;
-    qt_create_pipe(childStartedPipe);
-    qt_create_pipe(deathPipe);
+    }
 
-    if (threadData->eventDispatcher) {
+    if (threadData->hasEventDispatcher()) {
         startupSocketNotifier = new QSocketNotifier(childStartedPipe[0],
                                                     QSocketNotifier::Read, q);
         QObject::connect(startupSocketNotifier, SIGNAL(activated(int)),
@@ -608,8 +640,10 @@ void QProcessPrivate::startProcess()
     // Duplicate the environment.
     int envc = 0;
     char **envp = 0;
-    if (environment.d.constData())
+    if (environment.d.constData()) {
+        QProcessEnvironmentPrivate::MutexLocker locker(environment.d);
         envp = _q_dupEnvironment(environment.d.constData()->hash, &envc);
+    }
 
     // Encode the working directory if it's non-empty, otherwise just pass 0.
     const char *workingDirPtr = 0;
@@ -796,19 +830,29 @@ pid_t QProcessPrivate::spawnChild(const char *workingDir, char **argv, char **en
                   ? i : SPAWN_FDCLOSED;
     }
 
+    if (inputChannelMode == QProcess::ManagedInputChannel)
+        fd_map[0] = stdinChannel.pipe[0];
+    else
+        fd_map[0] = QT_FILENO(stdin);
+
     switch (processChannelMode) {
     case QProcess::ForwardedChannels:
-        fd_map[0] = stdinChannel.pipe[0];
         fd_map[1] = QT_FILENO(stdout);
         fd_map[2] = QT_FILENO(stderr);
         break;
+    case QProcess::ForwardedOutputChannel:
+        fd_map[1] = QT_FILENO(stdout);
+        fd_map[2] = stderrChannel.pipe[1];
+        break;
+    case QProcess::ForwardedErrorChannel:
+        fd_map[1] = stdoutChannel.pipe[1];
+        fd_map[2] = QT_FILENO(stderr);
+        break;
     case QProcess::MergedChannels:
-        fd_map[0] = stdinChannel.pipe[0];
         fd_map[1] = stdoutChannel.pipe[1];
         fd_map[2] = stdoutChannel.pipe[1];
         break;
     case QProcess::SeparateChannels:
-        fd_map[0] = stdinChannel.pipe[0];
         fd_map[1] = stdoutChannel.pipe[1];
         fd_map[2] = stderrChannel.pipe[1];
         break;
@@ -834,17 +878,19 @@ void QProcessPrivate::execChild(const char *workingDir, char **path, char **argv
 
     Q_Q(QProcess);
 
-    // copy the stdin socket (without closing on exec)
-    qt_safe_dup2(stdinChannel.pipe[0], fileno(stdin), 0);
+    // copy the stdin socket if asked to (without closing on exec)
+    if (inputChannelMode != QProcess::ForwardedInputChannel)
+        qt_safe_dup2(stdinChannel.pipe[0], fileno(stdin), 0);
 
     // copy the stdout and stderr if asked to
     if (processChannelMode != QProcess::ForwardedChannels) {
-        qt_safe_dup2(stdoutChannel.pipe[1], fileno(stdout), 0);
+        if (processChannelMode != QProcess::ForwardedOutputChannel)
+            qt_safe_dup2(stdoutChannel.pipe[1], fileno(stdout), 0);
 
         // merge stdout and stderr if asked to
         if (processChannelMode == QProcess::MergedChannels) {
             qt_safe_dup2(fileno(stdout), fileno(stderr), 0);
-        } else {
+        } else if (processChannelMode != QProcess::ForwardedErrorChannel) {
             qt_safe_dup2(stderrChannel.pipe[1], fileno(stderr), 0);
         }
     }
@@ -1001,9 +1047,9 @@ static int select_msecs(int nfds, fd_set *fdread, fd_set *fdwrite, int timeout)
     if (timeout < 0)
         return qt_safe_select(nfds, fdread, fdwrite, 0, 0);
 
-    struct timeval tv;
+    struct timespec tv;
     tv.tv_sec = timeout / 1000;
-    tv.tv_usec = (timeout % 1000) * 1000;
+    tv.tv_nsec = (timeout % 1000) * 1000 * 1000;
     return qt_safe_select(nfds, fdread, fdwrite, 0, &tv);
 }
 
@@ -1020,6 +1066,41 @@ static int qt_timeout_value(int msecs, int elapsed)
     return timeout < 0 ? 0 : timeout;
 }
 
+#ifdef Q_OS_BLACKBERRY
+// The BlackBerry event dispatcher uses bps_get_event. Unfortunately, already registered
+// socket notifiers are disabled by a call to select. This is to rearm the standard streams.
+static int bb_select(QProcessPrivate *process, int nfds, fd_set *fdread, fd_set *fdwrite, int timeout)
+{
+    bool stdoutEnabled = false;
+    bool stderrEnabled = false;
+    bool stdinEnabled = false;
+
+    if (process->stdoutChannel.notifier && process->stdoutChannel.notifier->isEnabled()) {
+        stdoutEnabled = true;
+        process->stdoutChannel.notifier->setEnabled(false);
+    }
+    if (process->stderrChannel.notifier && process->stderrChannel.notifier->isEnabled()) {
+        stderrEnabled = true;
+        process->stderrChannel.notifier->setEnabled(false);
+    }
+    if (process->stdinChannel.notifier && process->stdinChannel.notifier->isEnabled()) {
+        stdinEnabled = true;
+        process->stdinChannel.notifier->setEnabled(false);
+    }
+
+    const int ret = select_msecs(nfds, fdread, fdwrite, timeout);
+
+    if (stdoutEnabled)
+        process->stdoutChannel.notifier->setEnabled(true);
+    if (stderrEnabled)
+        process->stderrChannel.notifier->setEnabled(true);
+    if (stdinEnabled)
+        process->stdinChannel.notifier->setEnabled(true);
+
+    return ret;
+}
+#endif // Q_OS_BLACKBERRY
+
 bool QProcessPrivate::waitForStarted(int msecs)
 {
     Q_Q(QProcess);
@@ -1080,7 +1161,11 @@ bool QProcessPrivate::waitForReadyRead(int msecs)
             add_fd(nfds, stdinChannel.pipe[1], &fdwrite);
 
         int timeout = qt_timeout_value(msecs, stopWatch.elapsed());
+#ifdef Q_OS_BLACKBERRY
+        int ret = bb_select(this, nfds + 1, &fdread, &fdwrite, timeout);
+#else
         int ret = select_msecs(nfds + 1, &fdread, &fdwrite, timeout);
+#endif
         if (ret < 0) {
             break;
         }
@@ -1152,8 +1237,12 @@ bool QProcessPrivate::waitForBytesWritten(int msecs)
         if (!writeBuffer.isEmpty() && stdinChannel.pipe[1] != -1)
             add_fd(nfds, stdinChannel.pipe[1], &fdwrite);
 
-	int timeout = qt_timeout_value(msecs, stopWatch.elapsed());
-	int ret = select_msecs(nfds + 1, &fdread, &fdwrite, timeout);
+        int timeout = qt_timeout_value(msecs, stopWatch.elapsed());
+#ifdef Q_OS_BLACKBERRY
+        int ret = bb_select(this, nfds + 1, &fdread, &fdwrite, timeout);
+#else
+        int ret = select_msecs(nfds + 1, &fdread, &fdwrite, timeout);
+#endif
         if (ret < 0) {
             break;
         }
@@ -1219,8 +1308,12 @@ bool QProcessPrivate::waitForFinished(int msecs)
         if (!writeBuffer.isEmpty() && stdinChannel.pipe[1] != -1)
             add_fd(nfds, stdinChannel.pipe[1], &fdwrite);
 
-	int timeout = qt_timeout_value(msecs, stopWatch.elapsed());
-	int ret = select_msecs(nfds + 1, &fdread, &fdwrite, timeout);
+        int timeout = qt_timeout_value(msecs, stopWatch.elapsed());
+#ifdef Q_OS_BLACKBERRY
+        int ret = bb_select(this, nfds + 1, &fdread, &fdwrite, timeout);
+#else
+        int ret = select_msecs(nfds + 1, &fdread, &fdwrite, timeout);
+#endif
         if (ret < 0) {
             break;
         }
@@ -1339,10 +1432,15 @@ bool QProcessPrivate::startDetached(const QString &program, const QStringList &a
 
     // To catch the startup of the child
     int startedPipe[2];
-    qt_safe_pipe(startedPipe);
+    if (qt_safe_pipe(startedPipe) != 0)
+        return false;
     // To communicate the pid of the child
     int pidPipe[2];
-    qt_safe_pipe(pidPipe);
+    if (qt_safe_pipe(pidPipe) != 0) {
+        qt_safe_close(startedPipe[0]);
+        qt_safe_close(startedPipe[1]);
+        return false;
+    }
 
     pid_t childPid = fork();
     if (childPid == 0) {
