@@ -62,15 +62,6 @@
 
 //#define DEBUG_RESOURCE_MATCH
 
-#if defined(Q_OS_VXWORKS)
-#  if defined(m_data)
-#    undef m_data
-#  endif
-#  if defined(m_len)
-#    undef m_len
-#  endif
-#endif
-
 QT_BEGIN_NAMESPACE
 
 
@@ -460,7 +451,7 @@ QString QResource::absoluteFilePath() const
 }
 
 /*!
-    Returns true if the resource really exists in the resource hierarchy,
+    Returns \c true if the resource really exists in the resource hierarchy,
     false otherwise.
 
 */
@@ -475,7 +466,7 @@ bool QResource::isValid() const
 /*!
     \fn bool QResource::isFile() const
 
-    Returns true if the resource represents a file and thus has data
+    Returns \c true if the resource represents a file and thus has data
     backing it, false if it represents a directory.
 
     \sa isDir()
@@ -483,7 +474,7 @@ bool QResource::isValid() const
 
 
 /*!
-    Returns true if the resource represents a file and the data backing it
+    Returns \c true if the resource represents a file and the data backing it
     is in a compressed format, false otherwise.
 
     \sa data(), isFile()
@@ -526,7 +517,7 @@ const uchar *QResource::data() const
 }
 
 /*!
-    Returns true if the resource represents a directory and thus may have
+    Returns \c true if the resource represents a directory and thus may have
     children() in it, false if it represents a file.
 
     \sa isFile()
@@ -674,7 +665,7 @@ int QResourceRoot::findNode(const QString &_path, const QLocale &locale) const
             qDebug() << "   " << child+j << " :: " << name(child+j);
         }
 #endif
-        const uint h = qt_hash(segment.toString());
+        const uint h = qt_hash(segment);
 
         //do the binary search for the hash
         int l = 0, r = child_count-1;
@@ -1053,8 +1044,8 @@ static QString qt_resource_fixResourceRoot(QString r) {
    \fn bool QResource::registerResource(const QString &rccFileName, const QString &mapRoot)
 
    Registers the resource with the given \a rccFileName at the location in the
-   resource tree specified by \a mapRoot, and returns true if the file is
-   successfully opened; otherwise returns false.
+   resource tree specified by \a mapRoot, and returns \c true if the file is
+   successfully opened; otherwise returns \c false.
 
    \sa unregisterResource()
 */
@@ -1084,9 +1075,9 @@ QResource::registerResource(const QString &rccFilename, const QString &resourceR
   \fn bool QResource::unregisterResource(const QString &rccFileName, const QString &mapRoot)
 
   Unregisters the resource with the given \a rccFileName at the location in
-  the resource tree specified by \a mapRoot, and returns true if the
+  the resource tree specified by \a mapRoot, and returns \c true if the
   resource is successfully unloaded and no references exist for the
-  resource; otherwise returns false.
+  resource; otherwise returns \c false.
 
   \sa registerResource()
 */
@@ -1121,8 +1112,8 @@ QResource::unregisterResource(const QString &rccFilename, const QString &resourc
    \since 4.3
 
    Registers the resource with the given \a rccData at the location in the
-   resource tree specified by \a mapRoot, and returns true if the file is
-   successfully opened; otherwise returns false.
+   resource tree specified by \a mapRoot, and returns \c true if the file is
+   successfully opened; otherwise returns \c false.
 
    \warning The data must remain valid throughout the life of any QFile
    that may reference the resource data.
@@ -1156,8 +1147,8 @@ QResource::registerResource(const uchar *rccData, const QString &resourceRoot)
   \since 4.3
 
   Unregisters the resource with the given \a rccData at the location in the
-  resource tree specified by \a mapRoot, and returns true if the resource is
-  successfully unloaded and no references exist into the resource; otherwise returns false.
+  resource tree specified by \a mapRoot, and returns \c true if the resource is
+  successfully unloaded and no references exist into the resource; otherwise returns \c false.
 
   \sa registerResource()
 */
