@@ -47,8 +47,6 @@
 #include <QtCore/qstring.h>
 #include <QtCore/qscopedpointer.h>
 
-QT_BEGIN_HEADER
-
 QT_BEGIN_NAMESPACE
 QT_END_NAMESPACE
 
@@ -198,6 +196,4 @@ QT_END_NAMESPACE
 
 #endif // QT_NO_SETTINGS
 
-QT_END_HEADER
-
 #endif // QSETTINGS_H
