@@ -45,8 +45,6 @@
 #include <QtCore/qdir.h>
 #include <QtCore/QScopedPointer>
 
-QT_BEGIN_HEADER
-
 QT_BEGIN_NAMESPACE
 
 
@@ -79,6 +77,4 @@ private:
 
 QT_END_NAMESPACE
 
-QT_END_HEADER
-
 #endif // QTEMPORARYDIR_H
