@@ -44,10 +44,9 @@
 #ifndef QT_NO_TEMPORARYFILE
 
 #include "qplatformdefs.h"
+#include "private/qtemporaryfile_p.h"
 #include "private/qfile_p.h"
-#include "private/qfsfileengine_p.h"
 #include "private/qsystemerror_p.h"
-#include "private/qfilesystemengine_p.h"
 
 #if !defined(Q_OS_WIN)
 #include "private/qcore_unix_p.h"       // overrides QT_OPEN
@@ -161,7 +160,14 @@ static bool createFileFromTemplate(NativeFileHandle &file,
             return true;
 
         DWORD err = GetLastError();
-        if (err != ERROR_FILE_EXISTS) {
+        if (err == ERROR_ACCESS_DENIED) {
+            DWORD attributes = GetFileAttributes((const wchar_t *)path.constData());
+            if (attributes == INVALID_FILE_ATTRIBUTES) {
+                // Potential write error (read-only parent directory, etc.).
+                error = QSystemError(err, QSystemError::NativeError);
+                return false;
+            } // else file already exists as a directory.
+        } else if (err != ERROR_FILE_EXISTS) {
             error = QSystemError(err, QSystemError::NativeError);
             return false;
         }
@@ -218,36 +224,6 @@ static bool createFileFromTemplate(NativeFileHandle &file,
 }
 
 //************* QTemporaryFileEngine
-class QTemporaryFileEngine : public QFSFileEngine
-{
-    Q_DECLARE_PRIVATE(QFSFileEngine)
-public:
-    QTemporaryFileEngine(const QString &file, bool fileIsTemplate = true)
-        : QFSFileEngine(), filePathIsTemplate(fileIsTemplate),
-          filePathWasTemplate(fileIsTemplate)
-    {
-        Q_D(QFSFileEngine);
-        d->fileEntry = QFileSystemEntry(file);
-
-        if (!filePathIsTemplate)
-            QFSFileEngine::setFileName(file);
-    }
-
-    ~QTemporaryFileEngine();
-
-    bool isReallyOpen();
-    void setFileName(const QString &file);
-    void setFileTemplate(const QString &fileTemplate);
-
-    bool open(QIODevice::OpenMode flags);
-    bool remove();
-    bool rename(const QString &newName);
-    bool close();
-
-    bool filePathIsTemplate;
-    bool filePathWasTemplate;
-};
-
 QTemporaryFileEngine::~QTemporaryFileEngine()
 {
     QFSFileEngine::close();
@@ -398,6 +374,12 @@ bool QTemporaryFileEngine::rename(const QString &newName)
     return QFSFileEngine::rename(newName);
 }
 
+bool QTemporaryFileEngine::renameOverwrite(const QString &newName)
+{
+    QFSFileEngine::close();
+    return QFSFileEngine::renameOverwrite(newName);
+}
+
 bool QTemporaryFileEngine::close()
 {
     // Don't close the file, just seek to the front.
@@ -407,19 +389,6 @@ bool QTemporaryFileEngine::close()
 }
 
 //************* QTemporaryFilePrivate
-class QTemporaryFilePrivate : public QFilePrivate
-{
-    Q_DECLARE_PUBLIC(QTemporaryFile)
-
-protected:
-    QTemporaryFilePrivate();
-    ~QTemporaryFilePrivate();
-
-    QAbstractFileEngine *engine() const;
-
-    bool autoRemove;
-    QString templateName;
-};
 
 QTemporaryFilePrivate::QTemporaryFilePrivate() : autoRemove(true)
 {
@@ -440,7 +409,7 @@ QAbstractFileEngine *QTemporaryFilePrivate::engine() const
     return fileEngine;
 }
 
-static QString defaultTemplateName()
+QString QTemporaryFilePrivate::defaultTemplateName()
 {
     QString baseName;
 #if defined(QT_BUILD_CORE_LIB)
@@ -506,7 +475,7 @@ QTemporaryFile::QTemporaryFile()
     : QFile(*new QTemporaryFilePrivate)
 {
     Q_D(QTemporaryFile);
-    d->templateName = defaultTemplateName();
+    d->templateName = QTemporaryFilePrivate::defaultTemplateName();
 }
 
 QTemporaryFile::QTemporaryFile(const QString &templateName)
@@ -529,7 +498,7 @@ QTemporaryFile::QTemporaryFile()
     : QFile(*new QTemporaryFilePrivate, 0)
 {
     Q_D(QTemporaryFile);
-    d->templateName = defaultTemplateName();
+    d->templateName = QTemporaryFilePrivate::defaultTemplateName();
 }
 
 /*!
@@ -565,7 +534,7 @@ QTemporaryFile::QTemporaryFile(QObject *parent)
     : QFile(*new QTemporaryFilePrivate, parent)
 {
     Q_D(QTemporaryFile);
-    d->templateName = defaultTemplateName();
+    d->templateName = QTemporaryFilePrivate::defaultTemplateName();
 }
 
 /*!
@@ -618,7 +587,7 @@ QTemporaryFile::~QTemporaryFile()
 */
 
 /*!
-   Returns true if the QTemporaryFile is in auto remove
+   Returns \c true if the QTemporaryFile is in auto remove
    mode. Auto-remove mode will automatically delete the filename from
    disk upon destruction. This makes it very easy to create your
    QTemporaryFile object on the stack, fill it with data, read from
