@@ -54,8 +54,6 @@
 #error qtextstream.h must be included before any header file that defines Status
 #endif
 
-QT_BEGIN_HEADER
-
 QT_BEGIN_NAMESPACE
 
 
@@ -194,6 +192,7 @@ public:
 
 private:
     Q_DISABLE_COPY(QTextStream)
+    friend class QDebugStateSaverPrivate;
 
     QScopedPointer<QTextStreamPrivate> d_ptr;
 };
@@ -284,6 +283,4 @@ inline QTextStreamManipulator qSetRealNumberPrecision(int precision)
 
 QT_END_NAMESPACE
 
-QT_END_HEADER
-
 #endif // QTEXTSTREAM_H
