@@ -60,8 +60,6 @@
 
 #include <qt_windows.h>
 
-QT_BEGIN_HEADER
-
 QT_BEGIN_NAMESPACE
 
 
@@ -117,6 +115,4 @@ private:
 
 QT_END_NAMESPACE
 
-QT_END_HEADER
-
 #endif // QWINDOWSPIPEREADER_P_H
