@@ -57,8 +57,6 @@
 #include <qt_windows.h>
 #include <qqueue.h>
 
-QT_BEGIN_HEADER
-
 QT_BEGIN_NAMESPACE
 
 class QWinIoCompletionPort;
@@ -112,6 +110,4 @@ private:
 
 QT_END_NAMESPACE
 
-QT_END_HEADER
-
 #endif // QWINOVERLAPPEDIONOTIFIER_P_H
