@@ -91,6 +91,7 @@ QT_BEGIN_NAMESPACE
 //detects the deletion of the source model
 void QAbstractProxyModelPrivate::_q_sourceModelDestroyed()
 {
+    invalidatePersistentIndexes();
     model = QAbstractItemModelPrivate::staticEmptyModel();
 }
 
@@ -147,6 +148,15 @@ void QAbstractProxyModel::setSourceModel(QAbstractItemModel *sourceModel)
     }
 }
 
+/*!
+    Clears the roleNames of this proxy model.
+*/
+void QAbstractProxyModel::resetInternalData()
+{
+    Q_D(QAbstractProxyModel);
+    d->roleNames = d->model->roleNames();
+}
+
 /*!
     Returns the model that contains the data that is available through the proxy model.
 */
@@ -262,8 +272,7 @@ QVariant QAbstractProxyModel::headerData(int section, Qt::Orientation orientatio
  */
 QMap<int, QVariant> QAbstractProxyModel::itemData(const QModelIndex &proxyIndex) const
 {
-    Q_D(const QAbstractProxyModel);
-    return d->model->itemData(mapToSource(proxyIndex));
+    return QAbstractItemModel::itemData(proxyIndex);
 }
 
 /*!
@@ -289,8 +298,7 @@ bool QAbstractProxyModel::setData(const QModelIndex &index, const QVariant &valu
  */
 bool QAbstractProxyModel::setItemData(const QModelIndex &index, const QMap< int, QVariant >& roles)
 {
-    Q_D(QAbstractProxyModel);
-    return d->model->setItemData(mapToSource(index), roles);
+    return QAbstractItemModel::setItemData(index, roles);
 }
 
 /*!
