@@ -69,7 +69,9 @@ QT_BEGIN_NAMESPACE
     QJsonObject is an implicitly shared class, and shares the data with the document
     it has been created from as long as it is not being modified.
 
-    You can convert the array to and from text based JSON through QJsonDocument.
+    You can convert the object to and from text based JSON through QJsonDocument.
+
+    \sa {JSON Support in Qt}, {JSON Save Game Example}
 */
 
 /*!
@@ -104,7 +106,7 @@ QT_BEGIN_NAMESPACE
 
 
 /*!
-    Constructs an empty JSON object
+    Constructs an empty JSON object.
 
     \sa isEmpty()
  */
@@ -245,7 +247,7 @@ bool QJsonObject::isEmpty() const
 /*!
     Returns a QJsonValue representing the value for the key \a key.
 
-    The returned QJsonValue is \c Undefined, if the key does not exist.
+    The returned QJsonValue is QJsonValue::Undefined if the key does not exist.
 
     \sa QJsonValue, QJsonValue::isUndefined()
  */
@@ -266,7 +268,7 @@ QJsonValue QJsonObject::value(const QString &key) const
 
     This does the same as value().
 
-    The returned QJsonValue is \c Undefined, if the key does not exist.
+    The returned QJsonValue is QJsonValue::Undefined if the key does not exist.
 
     \sa value(), QJsonValue, QJsonValue::isUndefined()
  */
@@ -281,7 +283,7 @@ QJsonValue QJsonObject::operator [](const QString &key) const
     The return value is of type QJsonValueRef, a helper class for QJsonArray
     and QJsonObject. When you get an object of type QJsonValueRef, you can
     use it as if it were a reference to a QJsonValue. If you assign to it,
-    the assignment will apply to the character in the QJsonArray of QJsonObject
+    the assignment will apply to the element in the QJsonArray or QJsonObject
     from which you got the reference.
 
     \sa value()
@@ -301,13 +303,13 @@ QJsonValueRef QJsonObject::operator [](const QString &key)
 /*!
     Inserts a new item with the key \a key and a value of \a value.
 
-    If there is already an item with the key \a key then that item's value
+    If there is already an item with the key \a key, then that item's value
     is replaced with \a value.
 
     Returns an iterator pointing to the inserted item.
 
     If the value is QJsonValue::Undefined, it will cause the key to get removed
-    from the object. The returned iterator will then point to end()
+    from the object. The returned iterator will then point to end().
 
     \sa remove(), take(), QJsonObject::iterator, end()
  */
@@ -317,9 +319,10 @@ QJsonObject::iterator QJsonObject::insert(const QString &key, const QJsonValue &
         remove(key);
         return end();
     }
+    QJsonValue val = value;
 
     bool latinOrIntValue;
-    int valueSize = QJsonPrivate::Value::requiredStorage(value, &latinOrIntValue);
+    int valueSize = QJsonPrivate::Value::requiredStorage(val, &latinOrIntValue);
 
     bool latinKey = QJsonPrivate::useCompressed(key);
     int valueOffset = sizeof(QJsonPrivate::Entry) + QJsonPrivate::qStringSize(key, latinKey);
@@ -335,16 +338,21 @@ QJsonObject::iterator QJsonObject::insert(const QString &key, const QJsonValue &
     if (keyExists)
         ++d->compactionCounter;
 
-    o->reserveSpace(requiredSize, pos, 1, keyExists);
+    uint off = o->reserveSpace(requiredSize, pos, 1, keyExists);
+    if (!off)
+        return end();
 
     QJsonPrivate::Entry *e = o->entryAt(pos);
-    e->value.type = value.t;
+    e->value.type = val.t;
     e->value.latinKey = latinKey;
     e->value.latinOrIntValue = latinOrIntValue;
-    e->value.value = QJsonPrivate::Value::valueToStore(value, (char *)e - (char *)o + valueOffset);
+    e->value.value = QJsonPrivate::Value::valueToStore(val, (char *)e - (char *)o + valueOffset);
     QJsonPrivate::copyString((char *)(e + 1), key, latinKey);
     if (valueSize)
-        QJsonPrivate::Value::copyData(value, (char *)e + valueOffset, latinOrIntValue);
+        QJsonPrivate::Value::copyData(val, (char *)e + valueOffset, latinOrIntValue);
+
+    if (d->compactionCounter > 32u && d->compactionCounter >= unsigned(o->length) / 2u)
+        compact();
 
     return iterator(this, pos);
 }
@@ -376,7 +384,7 @@ void QJsonObject::remove(const QString &key)
 
     Returns a QJsonValue containing the value referenced by \a key.
     If \a key was not contained in the object, the returned QJsonValue
-    is Undefined.
+    is QJsonValue::Undefined.
 
     \sa insert(), remove(), QJsonValue
  */
@@ -416,7 +424,7 @@ bool QJsonObject::contains(const QString &key) const
 }
 
 /*!
-    Returns \c true if \a other is equal to this object
+    Returns \c true if \a other is equal to this object.
  */
 bool QJsonObject::operator==(const QJsonObject &other) const
 {
@@ -441,7 +449,7 @@ bool QJsonObject::operator==(const QJsonObject &other) const
 }
 
 /*!
-    Returns \c true if \a other is not equal to this object
+    Returns \c true if \a other is not equal to this object.
  */
 bool QJsonObject::operator!=(const QJsonObject &other) const
 {
@@ -495,7 +503,7 @@ QJsonObject::iterator QJsonObject::find(const QString &key)
 */
 
 /*!
-    Returns an const iterator pointing to the item with key \a key in the
+    Returns a const iterator pointing to the item with key \a key in the
     map.
 
     If the map contains no item with key \a key, the function
@@ -588,7 +596,7 @@ QJsonObject::const_iterator QJsonObject::constFind(const QString &key) const
     should use QJsonObject::const_iterator. It is generally good practice to
     use QJsonObject::const_iterator on a non-const QJsonObject as well, unless you
     need to change the QJsonObject through the iterator. Const iterators are
-    slightly faster, and improves code readability.
+    slightly faster, and improve code readability.
 
     The default QJsonObject::iterator constructor creates an uninitialized
     iterator. You must initialize it using a QJsonObject function like
@@ -598,7 +606,7 @@ QJsonObject::const_iterator QJsonObject::constFind(const QString &key) const
     Multiple iterators can be used on the same object. Existing iterators will however
     become dangling once the object gets modified.
 
-    \sa QJsonObject::const_iterator
+    \sa QJsonObject::const_iterator, {JSON Support in Qt}, {JSON Save Game Example}
 */
 
 /*! \typedef QJsonObject::iterator::difference_type
@@ -608,8 +616,8 @@ QJsonObject::const_iterator QJsonObject::constFind(const QString &key) const
 
 /*! \typedef QJsonObject::iterator::iterator_category
 
-  A synonym for \e {std::bidirectional_iterator_tag} indicating
-  this iterator is a bidirectional iterator.
+    A synonym for \e {std::bidirectional_iterator_tag} indicating
+    this iterator is a bidirectional iterator.
 */
 
 /*! \typedef QJsonObject::iterator::reference
@@ -658,7 +666,7 @@ QJsonObject::const_iterator QJsonObject::constFind(const QString &key) const
     The return value is of type QJsonValueRef, a helper class for QJsonArray
     and QJsonObject. When you get an object of type QJsonValueRef, you can
     use it as if it were a reference to a QJsonValue. If you assign to it,
-    the assignment will apply to the character in the QJsonArray of QJsonObject
+    the assignment will apply to the element in the QJsonArray or QJsonObject
     from which you got the reference.
 
     \sa key(), operator*()
@@ -673,7 +681,7 @@ QJsonObject::const_iterator QJsonObject::constFind(const QString &key) const
     The return value is of type QJsonValueRef, a helper class for QJsonArray
     and QJsonObject. When you get an object of type QJsonValueRef, you can
     use it as if it were a reference to a QJsonValue. If you assign to it,
-    the assignment will apply to the character in the QJsonArray of QJsonObject
+    the assignment will apply to the element in the QJsonArray or QJsonObject
     from which you got the reference.
 
     \sa key()
@@ -775,6 +783,8 @@ QJsonObject::const_iterator QJsonObject::constFind(const QString &key) const
 /*!
     \class QJsonObject::const_iterator
     \inmodule QtCore
+    \ingroup json
+    \since 5.0
     \brief The QJsonObject::const_iterator class provides an STL-style const iterator for QJsonObject.
 
     QJsonObject::const_iterator allows you to iterate over a QJsonObject.
@@ -782,7 +792,7 @@ QJsonObject::const_iterator QJsonObject::constFind(const QString &key) const
     over it, you must use QJsonObject::iterator instead. It is generally
     good practice to use QJsonObject::const_iterator on a non-const QJsonObject as
     well, unless you need to change the QJsonObject through the iterator.
-    Const iterators are slightly faster and improves code
+    Const iterators are slightly faster and improve code
     readability.
 
     The default QJsonObject::const_iterator constructor creates an
@@ -793,7 +803,7 @@ QJsonObject::const_iterator QJsonObject::constFind(const QString &key) const
     Multiple iterators can be used on the same object. Existing iterators
     will however become dangling if the object gets modified.
 
-    \sa QJsonObject::iterator
+    \sa QJsonObject::iterator, {JSON Support in Qt}, {JSON Save Game Example}
 */
 
 /*! \typedef QJsonObject::const_iterator::difference_type
@@ -803,8 +813,8 @@ QJsonObject::const_iterator QJsonObject::constFind(const QString &key) const
 
 /*! \typedef QJsonObject::const_iterator::iterator_category
 
-  A synonym for \e {std::bidirectional_iterator_tag} indicating
-  this iterator is a bidirectional iterator.
+    A synonym for \e {std::bidirectional_iterator_tag} indicating
+    this iterator is a bidirectional iterator.
 */
 
 /*! \typedef QJsonObject::const_iterator::reference
@@ -1028,7 +1038,7 @@ void QJsonObject::setValueAt(int i, const QJsonValue &val)
     insert(e->key(), val);
 }
 
-#ifndef QT_NO_DEBUG_STREAM
+#if !defined(QT_NO_DEBUG_STREAM) && !defined(QT_JSON_READONLY)
 QDebug operator<<(QDebug dbg, const QJsonObject &o)
 {
     if (!o.o) {
