@@ -42,9 +42,11 @@
 #include "qcoreapplication.h"
 #include "qcoreapplication_p.h"
 
+#ifndef QT_NO_QOBJECT
 #include "qabstracteventdispatcher.h"
 #include "qcoreevent.h"
 #include "qeventloop.h"
+#endif
 #include "qcorecmdlineargs_p.h"
 #include <qdatastream.h>
 #include <qdebug.h>
@@ -52,13 +54,16 @@
 #include <qfile.h>
 #include <qfileinfo.h>
 #include <qhash.h>
+#include <qmutex.h>
 #include <private/qprocess_p.h>
 #include <qstandardpaths.h>
 #include <qtextcodec.h>
+#ifndef QT_NO_QOBJECT
 #include <qthread.h>
 #include <qthreadpool.h>
 #include <qthreadstorage.h>
 #include <private/qthread_p.h>
+#endif
 #include <qelapsedtimer.h>
 #include <qlibraryinfo.h>
 #include <qvarlengtharray.h>
@@ -66,6 +71,7 @@
 #include <private/qfunctions_p.h>
 #include <private/qlocale_p.h>
 
+#ifndef QT_NO_QOBJECT
 #if defined(Q_OS_UNIX)
 #  if defined(Q_OS_BLACKBERRY)
 #    include "qeventdispatcher_blackberry_p.h"
@@ -78,10 +84,10 @@
 #    include "qeventdispatcher_unix_p.h"
 #  endif
 #endif
-
 #ifdef Q_OS_WIN
 #  include "qeventdispatcher_win_p.h"
 #endif
+#endif // QT_NO_QOBJECT
 
 #ifdef Q_OS_MAC
 #  include "qcore_mac_p.h"
@@ -91,14 +97,18 @@
 
 #ifdef Q_OS_UNIX
 #  include <locale.h>
+#  include <unistd.h>
 #endif
 
 #ifdef Q_OS_VXWORKS
 #  include <taskLib.h>
 #endif
 
+#include <algorithm>
+
 QT_BEGIN_NAMESPACE
 
+#ifndef QT_NO_QOBJECT
 class QMutexUnlocker
 {
 public:
@@ -113,6 +123,7 @@ private:
 
     QMutex *mtx;
 };
+#endif
 
 #if defined(Q_OS_WIN) || defined(Q_OS_MAC)
 extern QString qAppFileName();
@@ -136,18 +147,21 @@ QString QCoreApplicationPrivate::macMenuBarName()
 #endif
 QString QCoreApplicationPrivate::appName() const
 {
-    static QString applName;
+    QString applicationName;
 #ifdef Q_OS_MAC
-    applName = macMenuBarName();
+    applicationName = macMenuBarName();
 #endif
-    if (applName.isEmpty() && argv[0]) {
+    if (applicationName.isEmpty() && argv[0]) {
         char *p = strrchr(argv[0], '/');
-        applName = QString::fromLocal8Bit(p ? p + 1 : argv[0]);
+        applicationName = QString::fromLocal8Bit(p ? p + 1 : argv[0]);
     }
-    return applName;
+
+    return applicationName;
 }
 #endif
 
+QString *QCoreApplicationPrivate::cachedApplicationFilePath = 0;
+
 bool QCoreApplicationPrivate::checkInstance(const char *function)
 {
     bool b = (QCoreApplication::self != 0);
@@ -183,19 +197,47 @@ void QCoreApplicationPrivate::processCommandLineArguments()
 
 // Support for introspection
 
+#ifndef QT_NO_QOBJECT
 QSignalSpyCallbackSet Q_CORE_EXPORT qt_signal_spy_callback_set = { 0, 0, 0, 0 };
 
 void qt_register_signal_spy_callbacks(const QSignalSpyCallbackSet &callback_set)
 {
     qt_signal_spy_callback_set = callback_set;
 }
+#endif
 
 extern "C" void Q_CORE_EXPORT qt_startup_hook()
 {
 }
 
+typedef QList<QtStartUpFunction> QStartUpFuncList;
+Q_GLOBAL_STATIC(QStartUpFuncList, preRList)
 typedef QList<QtCleanUpFunction> QVFuncList;
 Q_GLOBAL_STATIC(QVFuncList, postRList)
+#ifndef QT_NO_QOBJECT
+static QBasicMutex globalPreRoutinesMutex;
+#endif
+
+/*!
+    \internal
+
+    Adds a global routine that will be called from the QCoreApplication
+    constructor. The public API is Q_COREAPP_STARTUP_FUNCTION.
+*/
+void qAddPreRoutine(QtStartUpFunction p)
+{
+    QStartUpFuncList *list = preRList();
+    if (!list)
+        return;
+    // Due to C++11 parallel dynamic initialization, this can be called
+    // from multiple threads.
+#ifndef QT_NO_THREAD
+    QMutexLocker locker(&globalPreRoutinesMutex);
+#endif
+    if (QCoreApplication::instance())
+        p();
+    list->prepend(p); // in case QCoreApplication is re-created, see qt_call_pre_routines
+}
 
 void qAddPostRoutine(QtCleanUpFunction p)
 {
@@ -213,6 +255,21 @@ void qRemovePostRoutine(QtCleanUpFunction p)
     list->removeAll(p);
 }
 
+static void qt_call_pre_routines()
+{
+    QStartUpFuncList *list = preRList();
+    if (!list)
+        return;
+#ifndef QT_NO_THREAD
+    QMutexLocker locker(&globalPreRoutinesMutex);
+#endif
+    // Unlike qt_call_post_routines, we don't empty the list, because
+    // Q_COREAPP_STARTUP_FUNCTION is a macro, so the user expects
+    // the function to be executed every time QCoreApplication is created.
+    for (int i = 0; i < list->count(); ++i)
+        list->at(i)();
+}
+
 void Q_CORE_EXPORT qt_call_post_routines()
 {
     QVFuncList *list = 0;
@@ -230,12 +287,15 @@ void Q_CORE_EXPORT qt_call_post_routines()
 }
 
 
+// initialized in qcoreapplication and in qtextstream autotest when setlocale is called.
+static bool qt_locale_initialized = false;
+
+#ifndef QT_NO_QOBJECT
+
 // app starting up if false
 bool QCoreApplicationPrivate::is_app_running = false;
  // app closing down if true
 bool QCoreApplicationPrivate::is_app_closing = false;
-// initialized in qcoreapplication and in qtextstream autotest when setlocale is called.
-static bool qt_locale_initialized = false;
 
 Q_CORE_EXPORT uint qGlobalPostedEventsCount()
 {
@@ -243,14 +303,17 @@ Q_CORE_EXPORT uint qGlobalPostedEventsCount()
     return currentThreadData->postEventList.size() - currentThreadData->postEventList.startOffset;
 }
 
-QCoreApplication *QCoreApplication::self = 0;
 QAbstractEventDispatcher *QCoreApplicationPrivate::eventDispatcher = 0;
-uint QCoreApplicationPrivate::attribs = (1 << Qt::AA_SynthesizeMouseForUnhandledTouchEvents);
 
 #ifdef Q_OS_UNIX
 Qt::HANDLE qt_application_thread_id = 0;
 #endif
 
+#endif // QT_NO_QOBJECT
+
+QCoreApplication *QCoreApplication::self = 0;
+uint QCoreApplicationPrivate::attribs = (1 << Qt::AA_SynthesizeMouseForUnhandledTouchEvents);
+
 struct QCoreApplicationData {
     QCoreApplicationData() {
 #ifndef QT_NO_LIBRARY
@@ -261,13 +324,13 @@ struct QCoreApplicationData {
 #ifndef QT_NO_LIBRARY
         delete app_libpaths;
 #endif
-
+#ifndef QT_NO_QOBJECT
         // cleanup the QAdoptedThread created for the main() thread
-       if (QCoreApplicationPrivate::theMainThread) {
-           QThreadData *data = QThreadData::get2(QCoreApplicationPrivate::theMainThread);
-           QCoreApplicationPrivate::theMainThread = 0;
-           data->deref(); // deletes the data and the adopted thread
-       }
+        if (QCoreApplicationPrivate::theMainThread) {
+            QThreadData *data = QThreadData::get2(QCoreApplicationPrivate::theMainThread);
+            data->deref(); // deletes the data and the adopted thread
+        }
+#endif
     }
 
 #ifdef Q_OS_BLACKBERRY
@@ -309,48 +372,66 @@ struct QCoreApplicationData {
 
 Q_GLOBAL_STATIC(QCoreApplicationData, coreappdata)
 
+#ifndef QT_NO_QOBJECT
 static bool quitLockRefEnabled = true;
+#endif
 
 QCoreApplicationPrivate::QCoreApplicationPrivate(int &aargc, char **aargv, uint flags)
-    : QObjectPrivate()
-    , argc(aargc)
+    :
+#ifndef QT_NO_QOBJECT
+      QObjectPrivate(),
+#endif
+      argc(aargc)
     , argv(aargv)
 #ifdef Q_OS_WIN
     , origArgc(aargc)
     , origArgv(new char *[aargc])
 #endif
-    , application_type(0)
+    , application_type(QCoreApplicationPrivate::Tty)
+#ifndef QT_NO_QOBJECT
     , in_exec(false)
     , aboutToQuitEmitted(false)
     , threadData_clean(false)
+#else
+    , q_ptr(0)
+#endif
 {
     app_compile_version = flags & 0xffffff;
     static const char *const empty = "";
     if (argc == 0 || argv == 0) {
         argc = 0;
-        argv = (char **)&empty; // ouch! careful with QCoreApplication::argv()!
+        argv = (char **)&empty;
     }
+#ifdef Q_OS_WIN
+    std::copy(argv, argv + argc, origArgv);
+#endif
+
+#ifndef QT_NO_QOBJECT
     QCoreApplicationPrivate::is_app_closing = false;
 
-#if defined(Q_OS_UNIX)
+#  if defined(Q_OS_UNIX)
     qt_application_thread_id = QThread::currentThreadId();
-#elif defined(Q_OS_WIN)
-    qCopy(argv, argv + argc, origArgv);
-#endif
+#  endif
 
     // note: this call to QThread::currentThread() may end up setting theMainThread!
     if (QThread::currentThread() != theMainThread)
         qWarning("WARNING: QApplication was not created in the main() thread.");
+#endif
 }
 
 QCoreApplicationPrivate::~QCoreApplicationPrivate()
 {
+#ifndef QT_NO_QOBJECT
     cleanupThreadData();
+#endif
 #ifdef Q_OS_WIN
     delete [] origArgv;
 #endif
+    QCoreApplicationPrivate::clearApplicationFilePath();
 }
 
+#ifndef QT_NO_QOBJECT
+
 void QCoreApplicationPrivate::cleanupThreadData()
 {
     if (threadData && !threadData_clean) {
@@ -397,6 +478,10 @@ void QCoreApplicationPrivate::createEventDispatcher()
 #endif
 }
 
+void QCoreApplicationPrivate::eventDispatcherReady()
+{
+}
+
 QThread *QCoreApplicationPrivate::theMainThread = 0;
 QThread *QCoreApplicationPrivate::mainThread()
 {
@@ -423,6 +508,8 @@ void QCoreApplicationPrivate::checkReceiverThread(QObject *receiver)
 }
 #endif
 
+#endif // QT_NO_QOBJECT
+
 void QCoreApplicationPrivate::appendApplicationPathToLibraryPaths()
 {
 #ifndef QT_NO_LIBRARY
@@ -513,9 +600,7 @@ void QCoreApplicationPrivate::initLocale()
     Note that some arguments supplied by the user may have been
     processed and removed by QCoreApplication.
 
-    In cases where command line arguments need to be obtained using the
-    argv() function, you must convert them from the local string encoding
-    using QString::fromLocal8Bit().
+    For more advanced command line option handling, create a QCommandLineParser.
 
     \section1 Locale Settings
 
@@ -544,13 +629,18 @@ void QCoreApplicationPrivate::initLocale()
     \internal
  */
 QCoreApplication::QCoreApplication(QCoreApplicationPrivate &p)
+#ifdef QT_NO_QOBJECT
+    : d_ptr(&p)
+#else
     : QObject(p, 0)
+#endif
 {
     init();
     // note: it is the subclasses' job to call
     // QCoreApplicationPrivate::eventDispatcher->startingUp();
 }
 
+#ifndef QT_NO_QOBJECT
 /*!
     Flushes the platform specific event queues.
 
@@ -567,6 +657,7 @@ void QCoreApplication::flush()
     if (self && self->d_func()->eventDispatcher)
         self->d_func()->eventDispatcher->flush();
 }
+#endif
 
 /*!
     Constructs a Qt kernel application. Kernel applications are
@@ -587,16 +678,23 @@ QCoreApplication::QCoreApplication(int &argc, char **argv
                                    , int _internal
 #endif
                                    )
+#ifdef QT_NO_QOBJECT
+    : d_ptr(new QCoreApplicationPrivate(argc, argv, _internal))
+#else
     : QObject(*new QCoreApplicationPrivate(argc, argv, _internal))
+#endif
 {
     init();
+#ifndef QT_NO_QOBJECT
     QCoreApplicationPrivate::eventDispatcher->startingUp();
+#endif
 }
 
 
 // ### move to QCoreApplicationPrivate constructor?
 void QCoreApplication::init()
 {
+    d_ptr->q_ptr = this;
     Q_D(QCoreApplication);
 
     QCoreApplicationPrivate::initLocale();
@@ -604,9 +702,10 @@ void QCoreApplication::init()
     Q_ASSERT_X(!self, "QCoreApplication", "there should be only one application object");
     QCoreApplication::self = this;
 
+#ifndef QT_NO_QOBJECT
     // use the event dispatcher created by the app programmer (if any)
     if (!QCoreApplicationPrivate::eventDispatcher)
-        QCoreApplicationPrivate::eventDispatcher = d->threadData->eventDispatcher;
+        QCoreApplicationPrivate::eventDispatcher = d->threadData->eventDispatcher.load();
     // otherwise we create one
     if (!QCoreApplicationPrivate::eventDispatcher)
         d->createEventDispatcher();
@@ -618,26 +717,35 @@ void QCoreApplication::init()
     }
 
     d->threadData->eventDispatcher = QCoreApplicationPrivate::eventDispatcher;
+    d->eventDispatcherReady();
+#endif
 
 #ifndef QT_NO_LIBRARY
     if (coreappdata()->app_libpaths)
         d->appendApplicationPathToLibraryPaths();
 #endif
 
+#ifndef QT_NO_QOBJECT
 #if defined(Q_OS_UNIX) && !(defined(QT_NO_PROCESS))
     // Make sure the process manager thread object is created in the main
     // thread.
     QProcessPrivate::initializeProcessManager();
 #endif
+#endif
 
 #ifdef QT_EVAL
-    extern void qt_core_eval_init(uint);
+    extern void qt_core_eval_init(QCoreApplicationPrivate::Type);
     qt_core_eval_init(d->application_type);
 #endif
 
     d->processCommandLineArguments();
 
+    qt_call_pre_routines();
     qt_startup_hook();
+
+#ifndef QT_NO_QOBJECT
+    QCoreApplicationPrivate::is_app_running = true; // No longer starting up.
+#endif
 }
 
 /*!
@@ -648,8 +756,10 @@ QCoreApplication::~QCoreApplication()
     qt_call_post_routines();
 
     self = 0;
+#ifndef QT_NO_QOBJECT
     QCoreApplicationPrivate::is_app_closing = true;
     QCoreApplicationPrivate::is_app_running = false;
+#endif
 
 #if !defined(QT_NO_THREAD)
     // Synchronize and stop the global thread pool threads.
@@ -663,10 +773,12 @@ QCoreApplication::~QCoreApplication()
         globalThreadPool->waitForDone();
 #endif
 
+#ifndef QT_NO_QOBJECT
     d_func()->threadData->eventDispatcher = 0;
     if (QCoreApplicationPrivate::eventDispatcher)
         QCoreApplicationPrivate::eventDispatcher->closingDown();
     QCoreApplicationPrivate::eventDispatcher = 0;
+#endif
 
 #ifndef QT_NO_LIBRARY
     delete coreappdata()->app_libpaths;
@@ -697,8 +809,8 @@ void QCoreApplication::setAttribute(Qt::ApplicationAttribute attribute, bool on)
 }
 
 /*!
-  Returns true if attribute \a attribute is set;
-  otherwise returns false.
+  Returns \c true if attribute \a attribute is set;
+  otherwise returns \c false.
 
   \sa setAttribute()
  */
@@ -708,18 +820,20 @@ bool QCoreApplication::testAttribute(Qt::ApplicationAttribute attribute)
 }
 
 
+#ifndef QT_NO_QOBJECT
+
 /*!
     \property QCoreApplication::quitLockEnabled
 
-    Returns true if the use of the QEventLoopLocker feature can cause the
-    application to quit, otherwise returns false.
+    Returns \c true if the use of the QEventLoopLocker feature can cause the
+    application to quit, otherwise returns \c false.
 
     \sa QEventLoopLocker
 */
 
 /*!
-    Returns true if the use of the QEventLoopLocker feature can cause the
-    application to quit, otherwise returns false.
+    Returns \c true if the use of the QEventLoopLocker feature can cause the
+    application to quit, otherwise returns \c false.
 
     \sa QEventLoopLocker
  */
@@ -777,7 +891,7 @@ bool QCoreApplication::notifyInternal(QObject *receiver, QEvent *event)
   For certain types of events (e.g. mouse and key events),
   the event will be propagated to the receiver's parent and so on up to
   the top-level object if the receiver is not interested in the event
-  (i.e., it returns false).
+  (i.e., it returns \c false).
 
   There are five different ways that events can be processed;
   reimplementing this virtual function is just one of them. All five
@@ -834,7 +948,7 @@ bool QCoreApplicationPrivate::sendThroughApplicationEventFilters(QObject *receiv
     if (receiver->d_func()->threadData == this->threadData && extraData) {
         // application event filters are only called for objects in the GUI thread
         for (int i = 0; i < extraData->eventFilters.size(); ++i) {
-            register QObject *obj = extraData->eventFilters.at(i);
+            QObject *obj = extraData->eventFilters.at(i);
             if (!obj)
                 continue;
             if (obj->d_func()->threadData != threadData) {
@@ -853,7 +967,7 @@ bool QCoreApplicationPrivate::sendThroughObjectEventFilters(QObject *receiver, Q
     Q_Q(QCoreApplication);
     if (receiver != q && receiver->d_func()->extraData) {
         for (int i = 0; i < receiver->d_func()->extraData->eventFilters.size(); ++i) {
-            register QObject *obj = receiver->d_func()->extraData->eventFilters.at(i);
+            QObject *obj = receiver->d_func()->extraData->eventFilters.at(i);
             if (!obj)
                 continue;
             if (obj->d_func()->threadData != receiver->d_func()->threadData) {
@@ -885,8 +999,8 @@ bool QCoreApplicationPrivate::notify_helper(QObject *receiver, QEvent * event)
 }
 
 /*!
-  Returns true if an application object has not been created yet;
-  otherwise returns false.
+  Returns \c true if an application object has not been created yet;
+  otherwise returns \c false.
 
   \sa closingDown()
 */
@@ -897,8 +1011,8 @@ bool QCoreApplication::startingUp()
 }
 
 /*!
-  Returns true if the application objects are being destroyed;
-  otherwise returns false.
+  Returns \c true if the application objects are being destroyed;
+  otherwise returns \c false.
 
   \sa startingUp()
 */
@@ -916,7 +1030,7 @@ bool QCoreApplication::closingDown()
     You can call this function occasionally when your program is busy
     performing a long operation (e.g. copying a file).
 
-    In event you are running a local loop which calls this function
+    In the event that you are running a local loop which calls this function
     continuously, without an event loop, the
     \l{QEvent::DeferredDelete}{DeferredDelete} events will
     not be processed. This can affect the behaviour of widgets,
@@ -934,9 +1048,9 @@ bool QCoreApplication::closingDown()
 void QCoreApplication::processEvents(QEventLoop::ProcessEventsFlags flags)
 {
     QThreadData *data = QThreadData::current();
-    if (!data->eventDispatcher)
+    if (!data->hasEventDispatcher())
         return;
-    data->eventDispatcher->processEvents(flags);
+    data->eventDispatcher.load()->processEvents(flags);
 }
 
 /*!
@@ -958,11 +1072,11 @@ void QCoreApplication::processEvents(QEventLoop::ProcessEventsFlags flags)
 void QCoreApplication::processEvents(QEventLoop::ProcessEventsFlags flags, int maxtime)
 {
     QThreadData *data = QThreadData::current();
-    if (!data->eventDispatcher)
+    if (!data->hasEventDispatcher())
         return;
     QElapsedTimer start;
     start.start();
-    while (data->eventDispatcher->processEvents(flags & ~QEventLoop::WaitForMoreEvents)) {
+    while (data->eventDispatcher.load()->processEvents(flags & ~QEventLoop::WaitForMoreEvents)) {
         if (start.elapsed() > maxtime)
             break;
     }
@@ -1161,13 +1275,14 @@ void QCoreApplication::postEvent(QObject *receiver, QEvent *event, int priority)
     data->canWait = false;
     locker.unlock();
 
-    if (data->eventDispatcher)
-        data->eventDispatcher->wakeUp();
+    QAbstractEventDispatcher* dispatcher = data->eventDispatcher.loadAcquire();
+    if (dispatcher)
+        dispatcher->wakeUp();
 }
 
 /*!
   \internal
-  Returns true if \a event was compressed away (possibly deleted) and should not be added to the list.
+  Returns \c true if \a event was compressed away (possibly deleted) and should not be added to the list.
 */
 bool QCoreApplication::compressEvent(QEvent *event, QObject *receiver, QPostEventList *postedEvents)
 {
@@ -1282,8 +1397,8 @@ void QCoreApplicationPrivate::sendPostedEvents(QObject *receiver, int event_type
             }
 
             --data->postEventList.recursion;
-            if (!data->postEventList.recursion && !data->canWait && data->eventDispatcher)
-                data->eventDispatcher->wakeUp();
+            if (!data->postEventList.recursion && !data->canWait && data->hasEventDispatcher())
+                data->eventDispatcher.load()->wakeUp();
 
             // clear the global list, i.e. remove everything that was
             // delivered.
@@ -1346,7 +1461,7 @@ void QCoreApplicationPrivate::sendPostedEvents(QObject *receiver, int event_type
         // first, we diddle the event so that we can deliver
         // it, and that no one will try to touch it later.
         pe.event->posted = false;
-        QScopedPointer<QEvent> e(pe.event);
+        QEvent *e = pe.event;
         QObject * r = pe.receiver;
 
         --r->d_func()->postedEvents;
@@ -1364,8 +1479,10 @@ void QCoreApplicationPrivate::sendPostedEvents(QObject *receiver, int event_type
         };
         MutexUnlocker unlocker(locker);
 
+        QScopedPointer<QEvent> event_deleter(e); // will delete the event (with the mutex unlocked)
+
         // after all that work, it's time to deliver the event.
-        QCoreApplication::sendEvent(r, e.data());
+        QCoreApplication::sendEvent(r, e);
 
         // careful when adding anything below this point - the
         // sendEvent() call might invalidate any invariants this
@@ -1570,6 +1687,8 @@ void QCoreApplication::quit()
   \sa quit()
 */
 
+#endif // QT_NO_QOBJECT
+
 #ifndef QT_NO_TRANSLATION
 /*!
     Adds the translation file \a translationFile to the list of
@@ -1591,7 +1710,7 @@ void QCoreApplication::quit()
     generated by Qt Designer provide a \c retranslateUi() function that can be
     called.
 
-    The function returns true on success and false on failure.
+    The function returns \c true on success and false on failure.
 
     \sa removeTranslator(), translate(), QTranslator::load(), {Dynamic Translation}
 */
@@ -1611,8 +1730,11 @@ bool QCoreApplication::installTranslator(QTranslator *translationFile)
         return false;
 #endif
 
+#ifndef QT_NO_QOBJECT
     QEvent ev(QEvent::LanguageChange);
     QCoreApplication::sendEvent(self, &ev);
+#endif
+
     return true;
 }
 
@@ -1621,7 +1743,7 @@ bool QCoreApplication::installTranslator(QTranslator *translationFile)
     translation files used by this application. (It does not delete the
     translation file from the file system.)
 
-    The function returns true on success and false on failure.
+    The function returns \c true on success and false on failure.
 
     \sa installTranslator(), translate(), QObject::tr()
 */
@@ -1634,10 +1756,12 @@ bool QCoreApplication::removeTranslator(QTranslator *translationFile)
         return false;
     QCoreApplicationPrivate *d = self->d_func();
     if (d->translators.removeAll(translationFile)) {
+#ifndef QT_NO_QOBJECT
         if (!self->closingDown()) {
             QEvent ev(QEvent::LanguageChange);
             QCoreApplication::sendEvent(self, &ev);
         }
+#endif
         return true;
     }
     return false;
@@ -1748,8 +1872,35 @@ bool QCoreApplicationPrivate::isTranslatorInstalled(QTranslator *translator)
            && QCoreApplication::self->d_func()->translators.contains(translator);
 }
 
+#else
+
+QString QCoreApplication::translate(const char *context, const char *sourceText,
+                                    const char *disambiguation, int n)
+{
+    Q_UNUSED(context)
+    Q_UNUSED(disambiguation)
+    QString ret = QString::fromUtf8(sourceText);
+    if (n >= 0)
+        ret.replace(QLatin1String("%n"), QString::number(n));
+    return ret;
+}
+
 #endif //QT_NO_TRANSLATE
 
+// Makes it possible to point QCoreApplication to a custom location to ensure
+// the directory is added to the patch, and qt.conf and deployed plugins are
+// found from there. This is for use cases in which QGuiApplication is
+// instantiated by a library and not by an application executable, for example,
+// Active X servers.
+
+void QCoreApplicationPrivate::setApplicationFilePath(const QString &path)
+{
+    if (QCoreApplicationPrivate::cachedApplicationFilePath)
+        *QCoreApplicationPrivate::cachedApplicationFilePath = path;
+    else
+        QCoreApplicationPrivate::cachedApplicationFilePath = new QString(path);
+}
+
 /*!
     Returns the directory that contains the application executable.
 
@@ -1805,20 +1956,28 @@ QString QCoreApplication::applicationFilePath()
     }
 
     QCoreApplicationPrivate *d = self->d_func();
-    if (!d->cachedApplicationFilePath.isNull())
-        return d->cachedApplicationFilePath;
+
+    static char *procName = d->argv[0];
+    if (qstrcmp(procName, d->argv[0]) != 0) {
+        // clear the cache if the procname changes, so we reprocess it.
+        QCoreApplicationPrivate::clearApplicationFilePath();
+        procName = d->argv[0];
+    }
+
+    if (QCoreApplicationPrivate::cachedApplicationFilePath)
+        return *QCoreApplicationPrivate::cachedApplicationFilePath;
 
 #if defined(Q_OS_WIN)
-    d->cachedApplicationFilePath = QFileInfo(qAppFileName()).filePath();
-    return d->cachedApplicationFilePath;
+    QCoreApplicationPrivate::setApplicationFilePath(QFileInfo(qAppFileName()).filePath());
+    return *QCoreApplicationPrivate::cachedApplicationFilePath;
 #elif defined(Q_OS_BLACKBERRY)
     if (!arguments().isEmpty()) { // args is never empty, but the navigator can change behaviour some day
         QFileInfo fileInfo(arguments().at(0));
         const bool zygotized = fileInfo.exists();
         if (zygotized) {
             // Handle the zygotized case:
-            d->cachedApplicationFilePath = QDir::cleanPath(fileInfo.absoluteFilePath());
-            return d->cachedApplicationFilePath;
+            QCoreApplicationPrivate::setApplicationFilePath(QDir::cleanPath(fileInfo.absoluteFilePath()));
+            return *QCoreApplicationPrivate::cachedApplicationFilePath;
         }
     }
 
@@ -1826,8 +1985,7 @@ QString QCoreApplication::applicationFilePath()
     const size_t maximum_path = static_cast<size_t>(pathconf("/",_PC_PATH_MAX));
     char buff[maximum_path+1];
     if (_cmdname(buff)) {
-        d->cachedApplicationFilePath = QDir::cleanPath(QString::fromLocal8Bit(buff));
-        return d->cachedApplicationFilePath;
+        QCoreApplicationPrivate::setApplicationFilePath(QDir::cleanPath(QString::fromLocal8Bit(buff)));
     } else {
         qWarning("QCoreApplication::applicationFilePath: _cmdname() failed");
         // _cmdname() won't fail, but just in case, fallback to the old method
@@ -1835,60 +1993,67 @@ QString QCoreApplication::applicationFilePath()
         QStringList executables = dir.entryList(QDir::Executable | QDir::Files);
         if (!executables.empty()) {
             //We assume that there is only one executable in the folder
-            d->cachedApplicationFilePath = dir.absoluteFilePath(executables.first());
-            return d->cachedApplicationFilePath;
-        } else {
-            return QString();
+            QCoreApplicationPrivate::setApplicationFilePath(dir.absoluteFilePath(executables.first()));
         }
     }
+    return *QCoreApplicationPrivate::cachedApplicationFilePath;
 #elif defined(Q_OS_MAC)
     QString qAppFileName_str = qAppFileName();
     if(!qAppFileName_str.isEmpty()) {
         QFileInfo fi(qAppFileName_str);
-        d->cachedApplicationFilePath = fi.exists() ? fi.canonicalFilePath() : QString();
-        return d->cachedApplicationFilePath;
+        if (fi.exists()) {
+            QCoreApplicationPrivate::setApplicationFilePath(fi.canonicalFilePath());
+            return *QCoreApplicationPrivate::cachedApplicationFilePath;
+        }
+        return QString();
     }
 #endif
 #if defined( Q_OS_UNIX )
-#  ifdef Q_OS_LINUX
+#  if defined(Q_OS_LINUX) && (!defined(Q_OS_ANDROID) || defined(Q_OS_ANDROID_NO_SDK))
     // Try looking for a /proc/<pid>/exe symlink first which points to
     // the absolute path of the executable
     QFileInfo pfi(QString::fromLatin1("/proc/%1/exe").arg(getpid()));
     if (pfi.exists() && pfi.isSymLink()) {
-        d->cachedApplicationFilePath = pfi.canonicalFilePath();
-        return d->cachedApplicationFilePath;
+        QCoreApplicationPrivate::setApplicationFilePath(pfi.canonicalFilePath());
+        return *QCoreApplicationPrivate::cachedApplicationFilePath;
     }
 #  endif
+    if (!arguments().isEmpty()) {
+        QString argv0 = QFile::decodeName(arguments().at(0).toLocal8Bit());
+        QString absPath;
 
-    QString argv0 = QFile::decodeName(arguments().at(0).toLocal8Bit());
-    QString absPath;
+        if (!argv0.isEmpty() && argv0.at(0) == QLatin1Char('/')) {
+            /*
+              If argv0 starts with a slash, it is already an absolute
+              file path.
+            */
+            absPath = argv0;
+        } else if (argv0.contains(QLatin1Char('/'))) {
+            /*
+              If argv0 contains one or more slashes, it is a file path
+              relative to the current directory.
+            */
+            absPath = QDir::current().absoluteFilePath(argv0);
+        } else {
+            /*
+              Otherwise, the file path has to be determined using the
+              PATH environment variable.
+            */
+            absPath = QStandardPaths::findExecutable(argv0);
+        }
 
-    if (!argv0.isEmpty() && argv0.at(0) == QLatin1Char('/')) {
-        /*
-          If argv0 starts with a slash, it is already an absolute
-          file path.
-        */
-        absPath = argv0;
-    } else if (argv0.contains(QLatin1Char('/'))) {
-        /*
-          If argv0 contains one or more slashes, it is a file path
-          relative to the current directory.
-        */
-        absPath = QDir::current().absoluteFilePath(argv0);
-    } else {
-        /*
-          Otherwise, the file path has to be determined using the
-          PATH environment variable.
-        */
-        absPath = QStandardPaths::findExecutable(argv0);
+        absPath = QDir::cleanPath(absPath);
+
+        QFileInfo fi(absPath);
+        if (fi.exists()) {
+            QCoreApplicationPrivate::setApplicationFilePath(fi.canonicalFilePath());
+            return *QCoreApplicationPrivate::cachedApplicationFilePath;
+        }
     }
 
-    absPath = QDir::cleanPath(absPath);
-
-    QFileInfo fi(absPath);
-    d->cachedApplicationFilePath = fi.exists() ? fi.canonicalFilePath() : QString();
-    return d->cachedApplicationFilePath;
+    return QString();
 #endif
+    Q_UNREACHABLE();
 }
 
 /*!
@@ -1898,7 +2063,7 @@ QString QCoreApplication::applicationFilePath()
 */
 qint64 QCoreApplication::applicationPid()
 {
-#if defined(Q_OS_WIN32) || defined(Q_OS_WINCE)
+#if defined(Q_OS_WIN)
     return GetCurrentProcessId();
 #elif defined(Q_OS_VXWORKS)
     return (pid_t) taskIdCurrent;
@@ -1934,7 +2099,7 @@ qint64 QCoreApplication::applicationPid()
     As a result of this, the string given by arguments().at(0) might not be
     the program name on Windows, depending on how the application was started.
 
-    \sa applicationFilePath()
+    \sa applicationFilePath(), QCommandLineParser
 */
 
 QStringList QCoreApplication::arguments()
@@ -1949,7 +2114,7 @@ QStringList QCoreApplication::arguments()
     char ** const av = self->d_func()->argv;
     list.reserve(ac);
 
-#ifdef Q_OS_WIN
+#if defined(Q_OS_WIN) && !defined(Q_OS_WINRT)
     // On Windows, it is possible to pass Unicode arguments on
     // the command line. To restore those, we split the command line
     // and filter out arguments that were deleted by derived application
@@ -1971,7 +2136,7 @@ QStringList QCoreApplication::arguments()
     const QStringList allArguments = qWinCmdArgs(cmdline);
     Q_ASSERT(allArguments.size() == origArgc);
     for (int i = 0; i < origArgc; ++i)
-        if (qFind(av, avEnd, origArgv[i]) != avEnd)
+        if (std::find(av, avEnd, origArgv[i]) != avEnd)
             list.push_back(allArguments.at(i));
 
 #else
@@ -1999,9 +2164,23 @@ QStringList QCoreApplication::arguments()
     \sa organizationDomain, applicationName
 */
 
+/*!
+  \fn void QCoreApplication::organizationNameChanged()
+  \internal
+
+  While not useful from C++ due to how organizationName is normally set once on
+  startup, this is still needed for QML so that bindings are reevaluated after
+  that initial change.
+*/
 void QCoreApplication::setOrganizationName(const QString &orgName)
 {
+    if (coreappdata()->orgName == orgName)
+        return;
     coreappdata()->orgName = orgName;
+#ifndef QT_NO_QOBJECT
+    if (QCoreApplication::self)
+        emit QCoreApplication::self->organizationNameChanged();
+#endif
 }
 
 QString QCoreApplication::organizationName()
@@ -2027,9 +2206,21 @@ QString QCoreApplication::organizationName()
 
     \sa organizationName, applicationName, applicationVersion
 */
+/*!
+  \fn void QCoreApplication::organizationDomainChanged()
+  \internal
+
+  Primarily for QML, see organizationNameChanged.
+*/
 void QCoreApplication::setOrganizationDomain(const QString &orgDomain)
 {
+    if (coreappdata()->orgDomain == orgDomain)
+        return;
     coreappdata()->orgDomain = orgDomain;
+#ifndef QT_NO_QOBJECT
+    if (QCoreApplication::self)
+        emit QCoreApplication::self->organizationDomainChanged();
+#endif
 }
 
 QString QCoreApplication::organizationDomain()
@@ -2047,11 +2238,23 @@ QString QCoreApplication::organizationDomain()
 
     If not set, the application name defaults to the executable name (since 5.0).
 
-    \sa organizationName, organizationDomain, applicationVersion, applicationFilePath
+    \sa organizationName, organizationDomain, applicationVersion, applicationFilePath()
+*/
+/*!
+  \fn void QCoreApplication::applicationNameChanged()
+  \internal
+
+  Primarily for QML, see organizationNameChanged.
 */
 void QCoreApplication::setApplicationName(const QString &application)
 {
+    if (coreappdata()->application == application)
+        return;
     coreappdata()->application = application;
+#ifndef QT_NO_QOBJECT
+    if (QCoreApplication::self)
+        emit QCoreApplication::self->applicationNameChanged();
+#endif
 }
 
 QString QCoreApplication::applicationName()
@@ -2078,9 +2281,21 @@ Q_CORE_EXPORT QString qt_applicationName_noFallback()
 
     \sa applicationName, organizationName, organizationDomain
 */
+/*!
+  \fn void QCoreApplication::applicationVersionChanged()
+  \internal
+
+  Primarily for QML, see organizationNameChanged.
+*/
 void QCoreApplication::setApplicationVersion(const QString &version)
 {
+    if (coreappdata()->applicationVersion == version)
+        return;
     coreappdata()->applicationVersion = version;
+#ifndef QT_NO_QOBJECT
+    if (QCoreApplication::self)
+        emit QCoreApplication::self->applicationVersionChanged();
+#endif
 }
 
 QString QCoreApplication::applicationVersion()
@@ -2229,6 +2444,8 @@ void QCoreApplication::removeLibraryPath(const QString &path)
 
 #endif //QT_NO_LIBRARY
 
+#ifndef QT_NO_QOBJECT
+
 /*!
     Installs an event filter \a filterObj for all native events
     received by the application in the main thread.
@@ -2285,8 +2502,8 @@ void QCoreApplication::removeNativeEventFilter(QAbstractNativeEventFilter *filte
 }
 
 /*!
-    This function returns true if there are pending events; otherwise
-    returns false. Pending events can be either from the window
+    This function returns \c true if there are pending events; otherwise
+    returns \c false. Pending events can be either from the window
     system or posted events using postEvent().
 
     \sa QAbstractEventDispatcher::hasPendingEvents()
@@ -2324,6 +2541,33 @@ void QCoreApplication::setEventDispatcher(QAbstractEventDispatcher *eventDispatc
     mainThread->setEventDispatcher(eventDispatcher);
 }
 
+#endif // QT_NO_QOBJECT
+
+/*!
+    \macro Q_COREAPP_STARTUP_FUNCTION(QtStartUpFunction ptr)
+    \since 5.1
+    \relates QCoreApplication
+    \reentrant
+
+    Adds a global function that will be called from the QCoreApplication
+    constructor. This macro is normally used to initialize libraries
+    for program-wide functionality, without requiring the application to
+    call into the library for initialization.
+
+    The function specified by \a ptr should take no arguments and should
+    return nothing. For example:
+
+    \snippet code/src_corelib_kernel_qcoreapplication.cpp 3
+
+    Note that the startup function will run at the end of the QCoreApplication constructor,
+    before any GUI initialization. If GUI code is required in the function,
+    use a timer (or a queued invocation) to perform the initialization later on,
+    from the event loop.
+
+    If QCoreApplication is deleted and another QCoreApplication is created,
+    the startup function will be invoked again.
+*/
+
 /*!
     \fn void qAddPostRoutine(QtCleanUpFunction ptr)
     \relates QCoreApplication
