@@ -61,9 +61,9 @@ QT_BEGIN_NAMESPACE
     QEvents, and sends the translated events to \l{QObject}s.
 
     In general, events come from the underlying window system
-    (spontaneous() returns true), but it is also possible to manually
+    (spontaneous() returns \c true), but it is also possible to manually
     send events using QCoreApplication::sendEvent() and
-    QCoreApplication::postEvent() (spontaneous() returns false).
+    QCoreApplication::postEvent() (spontaneous() returns \c false).
 
     QObjects receive events by having their QObject::event() function
     called. The function can be reimplemented in subclasses to
@@ -98,12 +98,13 @@ QT_BEGIN_NAMESPACE
     \value ActionChanged                    An action has been changed (QActionEvent).
     \value ActionRemoved                    An action has been removed (QActionEvent).
     \value ActivationChange                 A widget's top-level window activation state has changed.
-    \value ApplicationActivate              The application has been made available to the user.
-    \value ApplicationActivated             This enum has been deprecated. Use ApplicationActivate instead.
-    \value ApplicationDeactivate            The application has been suspended, and is unavailable to the user.
+    \value ApplicationActivate              This enum has been deprecated. Use ApplicationStateChange instead.
+    \value ApplicationActivated             This enum has been deprecated. Use ApplicationStateChange instead.
+    \value ApplicationDeactivate            This enum has been deprecated. Use ApplicationStateChange instead.
     \value ApplicationFontChange            The default application font has changed.
     \value ApplicationLayoutDirectionChange The default application layout direction has changed.
     \value ApplicationPaletteChange         The default application palette has changed.
+    \value ApplicationStateChange           The state of the application has changed.
     \value ApplicationWindowIconChange      The application's icon has changed.
     \value ChildAdded                       An object gets a child (QChildEvent).
     \value ChildPolished                    A widget child gets polished (QChildEvent).
@@ -264,7 +265,9 @@ QT_BEGIN_NAMESPACE
     \omitvalue Style
     \omitvalue StyleAnimationUpdate
     \omitvalue ZeroTimerEvent
+    \omitvalue ApplicationActivate
     \omitvalue ApplicationActivated
+    \omitvalue ApplicationDeactivate
     \omitvalue ApplicationDeactivated
     \omitvalue MacGLWindowChange
     \omitvalue MacGLClearDrawable
@@ -379,8 +382,8 @@ QEvent::~QEvent()
 /*!
     \fn bool QEvent::spontaneous() const
 
-    Returns true if the event originated outside the application (a
-    system event); otherwise returns false.
+    Returns \c true if the event originated outside the application (a
+    system event); otherwise returns \c false.
 
     The return value of this function is not defined for paint events.
 */
@@ -484,7 +487,7 @@ QTimerEvent::~QTimerEvent()
     added or removed.
 
     In both cases you can only rely on the child being a QObject (or,
-    if QObject::isWidgetType() returns true, a QWidget). This is
+    if QObject::isWidgetType() returns \c true, a QWidget). This is
     because in the QEvent::ChildAdded case the child is not yet fully
     constructed; in the QEvent::ChildRemoved case it might have
     already been destructed.
@@ -521,21 +524,21 @@ QChildEvent::~QChildEvent()
 /*!
     \fn bool QChildEvent::added() const
 
-    Returns true if type() is QEvent::ChildAdded; otherwise returns
+    Returns \c true if type() is QEvent::ChildAdded; otherwise returns
     false.
 */
 
 /*!
     \fn bool QChildEvent::removed() const
 
-    Returns true if type() is QEvent::ChildRemoved; otherwise returns
+    Returns \c true if type() is QEvent::ChildRemoved; otherwise returns
     false.
 */
 
 /*!
     \fn bool QChildEvent::polished() const
 
-    Returns true if type() is QEvent::ChildPolished; otherwise returns
+    Returns \c true if type() is QEvent::ChildPolished; otherwise returns
     false.
 */
 
