@@ -42,6 +42,8 @@
 #ifndef QFUNCTIONS_NACL_H
 #define QFUNCTIONS_NACL_H
 
+#include <QtCore/qglobal.h>
+
 #ifdef Q_OS_NACL
 
 #include <sys/types.h>
@@ -52,8 +54,6 @@
 #define PTHREAD_CANCEL_ENABLE 2
 #define PTHREAD_INHERIT_SCHED 3
 
-QT_BEGIN_HEADER
-
 QT_BEGIN_NAMESPACE
 
 
@@ -69,7 +69,7 @@ int pthread_cancel(pthread_t thread);
 
 int pthread_attr_setinheritsched(pthread_attr_t *attr,
     int inheritsched);
-int pthread_attr_getinheritsched(const pthread_attr_t *attr, 
+int pthread_attr_getinheritsched(const pthread_attr_t *attr,
     int *inheritsched);
 
 // event dispatcher, select
@@ -89,8 +89,6 @@ int select(int nfds, fd_set * readfds, fd_set * writefds, fd_set * errorfds, str
 
 QT_END_NAMESPACE
 
-QT_END_HEADER
-
 #endif //Q_OS_NACL
 
 #endif //QFUNCTIONS_NACL_H
