@@ -45,13 +45,13 @@
 #include <QtCore/qobjectdefs.h>
 #include <QtCore/qvariant.h>
 
-QT_BEGIN_HEADER
-
 QT_BEGIN_NAMESPACE
 
 
 template <typename T> class QList;
 
+#define Q_METAMETHOD_INVOKE_MAX_ARGS 10
+
 class Q_CORE_EXPORT QMetaMethod
 {
 public:
@@ -148,8 +148,8 @@ public:
     static inline QMetaMethod fromSignal(Func signal)
     {
         typedef QtPrivate::FunctionPointer<Func> SignalType;
-        reinterpret_cast<typename SignalType::Object *>(0)->qt_check_for_QOBJECT_macro(
-                    *reinterpret_cast<typename SignalType::Object *>(0));
+        Q_STATIC_ASSERT_X(QtPrivate::HasQ_OBJECT_Macro<typename SignalType::Object>::Value,
+                          "No Q_OBJECT in the class with the signal");
         return fromSignalImpl(&SignalType::Object::staticMetaObject,
                               reinterpret_cast<void **>(&signal));
     }
@@ -274,6 +274,4 @@ Q_DECLARE_TYPEINFO(QMetaClassInfo, Q_MOVABLE_TYPE);
 
 QT_END_NAMESPACE
 
-QT_END_HEADER
-
 #endif // QMETAOBJECT_H
