@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2012 Digia Plc and/or its subsidiary(-ies).
+** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
 ** This file is part of the QtCore module of the Qt Toolkit.
@@ -90,7 +90,7 @@ Q_CORE_EXPORT bool isBuiltinType(const QByteArray &type)
 } // namespace QtPrivate
 
 // copied from qmetaobject.cpp
-static inline const QMetaObjectPrivate *priv(const uint* data)
+static inline Q_DECL_UNUSED const QMetaObjectPrivate *priv(const uint* data)
 { return reinterpret_cast<const QMetaObjectPrivate*>(data); }
 
 class QMetaMethodBuilderPrivate
@@ -510,7 +510,7 @@ QMetaMethodBuilder QMetaObjectBuilder::addSignal(const QByteArray& signature)
 {
     int index = d->methods.size();
     d->methods.append(QMetaMethodBuilderPrivate
-        (QMetaMethod::Signal, signature, QByteArray("void"), QMetaMethod::Protected));
+        (QMetaMethod::Signal, signature, QByteArray("void"), QMetaMethod::Public));
     return QMetaMethodBuilder(this, index);
 }
 
@@ -1075,8 +1075,14 @@ int QMetaObjectBuilder::indexOfClassInfo(const QByteArray& name)
     \brief The QMetaStringTable class can generate a meta-object string table at runtime.
 */
 
-QMetaStringTable::QMetaStringTable()
-    : m_index(0) {}
+QMetaStringTable::QMetaStringTable(const QByteArray &className)
+    : m_index(0)
+    , m_className(className)
+{
+    const int index = enter(m_className);
+    Q_ASSERT(index == 0);
+    Q_UNUSED(index);
+}
 
 // Enters the given value into the string table (if it hasn't already been
 // entered). Returns the index of the string.
@@ -1106,30 +1112,45 @@ int QMetaStringTable::blobSize() const
     return size;
 }
 
+static void writeString(char *out, int i, const QByteArray &str,
+                        const int offsetOfStringdataMember, int &stringdataOffset)
+{
+    int size = str.size();
+    qptrdiff offset = offsetOfStringdataMember + stringdataOffset
+            - i * sizeof(QByteArrayData);
+    const QByteArrayData data =
+        Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(size, offset);
+
+    memcpy(out + i * sizeof(QByteArrayData), &data, sizeof(QByteArrayData));
+
+    memcpy(out + offsetOfStringdataMember + stringdataOffset, str.constData(), size);
+    out[offsetOfStringdataMember + stringdataOffset + size] = '\0';
+
+    stringdataOffset += size + 1;
+}
+
 // Writes strings to string data struct.
 // The struct consists of an array of QByteArrayData, followed by a char array
 // containing the actual strings. This format must match the one produced by
 // moc (see generator.cpp).
-void QMetaStringTable::writeBlob(char *out)
+void QMetaStringTable::writeBlob(char *out) const
 {
     Q_ASSERT(!(reinterpret_cast<quintptr>(out) & (preferredAlignment()-1)));
 
     int offsetOfStringdataMember = m_entries.size() * sizeof(QByteArrayData);
     int stringdataOffset = 0;
-    for (int i = 0; i < m_entries.size(); ++i) {
-        const QByteArray &str = m_entries.key(i);
-        int size = str.size();
-        qptrdiff offset = offsetOfStringdataMember + stringdataOffset
-                - i * sizeof(QByteArrayData);
-        const QByteArrayData data =
-            Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(size, offset);
 
-        memcpy(out + i * sizeof(QByteArrayData), &data, sizeof(QByteArrayData));
+    // qt_metacast expects the first string in the string table to be the class name.
+    writeString(out, /*index*/0, m_className, offsetOfStringdataMember, stringdataOffset);
 
-        memcpy(out + offsetOfStringdataMember + stringdataOffset, str.constData(), size);
-        out[offsetOfStringdataMember + stringdataOffset + size] = '\0';
+    for (Entries::ConstIterator it = m_entries.constBegin(), end = m_entries.constEnd();
+         it != end; ++it) {
+        const int i = it.value();
+        if (i == 0)
+            continue;
+        const QByteArray &str = it.key();
 
-        stringdataOffset += size + 1;
+        writeString(out, i, str, offsetOfStringdataMember, stringdataOffset);
     }
 }
 
@@ -1270,8 +1291,7 @@ static int buildMetaObject(QMetaObjectBuilderPrivate *d, char *buf,
     // Reset the current data position to just past the QMetaObjectPrivate.
     dataIndex = MetaObjectPrivateFieldCount;
 
-    QMetaStringTable strings;
-    strings.enter(d->className);
+    QMetaStringTable strings(d->className);
 
     // Output the class infos,
     Q_ASSERT(!buf || dataIndex == pmeta->classInfoData);
@@ -2005,8 +2025,7 @@ void QMetaMethodBuilder::setTag(const QByteArray& value)
 /*!
     Returns the access specification of this method (private, protected,
     or public).  The default value is QMetaMethod::Public for methods,
-    slots, and constructors.  The default value is QMetaMethod::Protected
-    for signals.
+    slots, signals and constructors.
 
     \sa setAccess()
 */
@@ -2145,7 +2164,7 @@ QByteArray QMetaPropertyBuilder::type() const
 }
 
 /*!
-    Returns true if this property has a notify signal; false otherwise.
+    Returns \c true if this property has a notify signal; false otherwise.
 
     \sa notifySignal(), setNotifySignal(), removeNotifySignal()
 */
@@ -2206,7 +2225,7 @@ void QMetaPropertyBuilder::removeNotifySignal()
 }
 
 /*!
-    Returns true if this property is readable; otherwise returns false.
+    Returns \c true if this property is readable; otherwise returns \c false.
     The default value is true.
 
     \sa setReadable(), isWritable()
@@ -2221,7 +2240,7 @@ bool QMetaPropertyBuilder::isReadable() const
 }
 
 /*!
-    Returns true if this property is writable; otherwise returns false.
+    Returns \c true if this property is writable; otherwise returns \c false.
     The default value is true.
 
     \sa setWritable(), isReadable()
@@ -2236,8 +2255,8 @@ bool QMetaPropertyBuilder::isWritable() const
 }
 
 /*!
-    Returns true if this property can be reset to a default value; otherwise
-    returns false.  The default value is false.
+    Returns \c true if this property can be reset to a default value; otherwise
+    returns \c false.  The default value is false.
 
     \sa setResettable()
 */
@@ -2251,7 +2270,7 @@ bool QMetaPropertyBuilder::isResettable() const
 }
 
 /*!
-    Returns true if this property is designable; otherwise returns false.
+    Returns \c true if this property is designable; otherwise returns \c false.
     This default value is false.
 
     \sa setDesignable(), isScriptable(), isStored()
@@ -2266,7 +2285,7 @@ bool QMetaPropertyBuilder::isDesignable() const
 }
 
 /*!
-    Returns true if the property is scriptable; otherwise returns false.
+    Returns \c true if the property is scriptable; otherwise returns \c false.
     This default value is true.
 
     \sa setScriptable(), isDesignable(), isStored()
@@ -2281,7 +2300,7 @@ bool QMetaPropertyBuilder::isScriptable() const
 }
 
 /*!
-    Returns true if the property is stored; otherwise returns false.
+    Returns \c true if the property is stored; otherwise returns \c false.
     This default value is false.
 
     \sa setStored(), isDesignable(), isScriptable()
@@ -2296,7 +2315,7 @@ bool QMetaPropertyBuilder::isStored() const
 }
 
 /*!
-    Returns true if the property is editable; otherwise returns false.
+    Returns \c true if the property is editable; otherwise returns \c false.
     This default value is false.
 
     \sa setEditable(), isDesignable(), isScriptable(), isStored()
@@ -2311,7 +2330,7 @@ bool QMetaPropertyBuilder::isEditable() const
 }
 
 /*!
-    Returns true if this property is designated as the \c USER
+    Returns \c true if this property is designated as the \c USER
     property, i.e., the one that the user can edit or that is
     significant in some other way.  Otherwise it returns
     false.  This default value is false.
@@ -2328,7 +2347,7 @@ bool QMetaPropertyBuilder::isUser() const
 }
 
 /*!
-    Returns true if the property has a C++ setter function that
+    Returns \c true if the property has a C++ setter function that
     follows Qt's standard "name" / "setName" pattern. Designer and uic
     query hasStdCppSet() in order to avoid expensive
     QObject::setProperty() calls. All properties in Qt [should] follow
@@ -2346,8 +2365,8 @@ bool QMetaPropertyBuilder::hasStdCppSet() const
 }
 
 /*!
-    Returns true if the property is an enumerator or flag type;
-    otherwise returns false.  This default value is false.
+    Returns \c true if the property is an enumerator or flag type;
+    otherwise returns \c false.  This default value is false.
 
     \sa setEnumOrFlag()
 */
@@ -2361,7 +2380,7 @@ bool QMetaPropertyBuilder::isEnumOrFlag() const
 }
 
 /*!
-    Returns true if the property is constant; otherwise returns false.
+    Returns \c true if the property is constant; otherwise returns \c false.
     The default value is false.
 */
 bool QMetaPropertyBuilder::isConstant() const
@@ -2374,7 +2393,7 @@ bool QMetaPropertyBuilder::isConstant() const
 }
 
 /*!
-    Returns true if the property is final; otherwise returns false.
+    Returns \c true if the property is final; otherwise returns \c false.
     The default value is false.
 */
 bool QMetaPropertyBuilder::isFinal() const
@@ -2601,7 +2620,7 @@ QByteArray QMetaEnumBuilder::name() const
 }
 
 /*!
-    Returns true if this enumerator is used as a flag; otherwise returns
+    Returns \c true if this enumerator is used as a flag; otherwise returns
     false.
 
     \sa setIsFlag()
