@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2012 Digia Plc and/or its subsidiary(-ies).
+** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
 ** This file is part of the QtCore module of the Qt Toolkit.
@@ -363,8 +363,8 @@ void QMimeData::setUrls(const QList<QUrl> &urls)
 }
 
 /*!
-    Returns true if the object can return a list of urls; otherwise
-    returns false.
+    Returns \c true if the object can return a list of urls; otherwise
+    returns \c false.
 
     URLs correspond to the MIME type \c text/uri-list.
 
@@ -402,8 +402,8 @@ void QMimeData::setText(const QString &text)
 }
 
 /*!
-    Returns true if the object can return plain text (MIME type \c
-    text/plain); otherwise returns false.
+    Returns \c true if the object can return plain text (MIME type \c
+    text/plain); otherwise returns \c false.
 
     \sa setText(), text(), hasHtml(), hasFormat()
 */
@@ -438,8 +438,8 @@ void QMimeData::setHtml(const QString &html)
 }
 
 /*!
-    Returns true if the object can return HTML (MIME type \c
-    text/html); otherwise returns false.
+    Returns \c true if the object can return HTML (MIME type \c
+    text/html); otherwise returns \c false.
 
     \sa setHtml(), html(), hasFormat()
 */
@@ -452,8 +452,8 @@ bool QMimeData::hasHtml() const
     Returns a QVariant storing a QImage if the object can return an
     image; otherwise returns a null variant.
 
-    A QVariant is used because QMimeData belongs to the \l QtCore
-    library, whereas QImage belongs to \l QtGui. To convert the
+    A QVariant is used because QMimeData belongs to the Qt Core
+    module, whereas QImage belongs to Qt GUI. To convert the
     QVariant to a QImage, simply use qvariant_cast(). For example:
 
     \snippet code/src_corelib_kernel_qmimedata.cpp 5
@@ -469,8 +469,8 @@ QVariant QMimeData::imageData() const
 /*!
     Sets the data in the object to the given \a image.
 
-    A QVariant is used because QMimeData belongs to the \l QtCore
-    library, whereas QImage belongs to \l QtGui. The conversion
+    A QVariant is used because QMimeData belongs to the Qt Core
+    module, whereas QImage belongs to Qt GUI. The conversion
     from QImage to QVariant is implicit. For example:
 
     \snippet code/src_corelib_kernel_qmimedata.cpp 6
@@ -484,7 +484,7 @@ void QMimeData::setImageData(const QVariant &image)
 }
 
 /*!
-    Returns true if the object can return an image; otherwise returns
+    Returns \c true if the object can return an image; otherwise returns
     false.
 
     \sa setImageData(), imageData(), hasFormat()
@@ -499,8 +499,8 @@ bool QMimeData::hasImage() const
     color (MIME type \c application/x-color); otherwise returns a
     null variant.
 
-    A QVariant is used because QMimeData belongs to the \l QtCore
-    library, whereas QColor belongs to \l QtGui. To convert the
+    A QVariant is used because QMimeData belongs to the Qt Core
+    module, whereas QColor belongs to Qt GUI. To convert the
     QVariant to a QColor, simply use qvariant_cast(). For example:
 
     \snippet code/src_corelib_kernel_qmimedata.cpp 7
@@ -528,8 +528,8 @@ void QMimeData::setColorData(const QVariant &color)
 
 
 /*!
-    Returns true if the object can return a color (MIME type \c
-    application/x-color); otherwise returns false.
+    Returns \c true if the object can return a color (MIME type \c
+    application/x-color); otherwise returns \c false.
 
     \sa setColorData(), colorData(), hasFormat()
 */
@@ -572,8 +572,8 @@ void QMimeData::setData(const QString &mimeType, const QByteArray &data)
 }
 
 /*!
-    Returns true if the object can return data for the MIME type
-    specified by \a mimeType; otherwise returns false.
+    Returns \c true if the object can return data for the MIME type
+    specified by \a mimeType; otherwise returns \c false.
 
     For the most common types of data, you can call the higher-level
     functions hasText(), hasHtml(), hasUrls(), hasImage(), and
