@@ -143,7 +143,7 @@ key_t QSharedMemoryPrivate::handle()
 /*!
     \internal
     Creates the unix file if needed.
-    returns true if the unix file was created.
+    returns \c true if the unix file was created.
 
     -1 error
      0 already existed
@@ -151,9 +151,6 @@ key_t QSharedMemoryPrivate::handle()
   */
 int QSharedMemoryPrivate::createUnixKeyFile(const QString &fileName)
 {
-    if (QFile::exists(fileName))
-        return 0;
-
     int fd = qt_safe_open(QFile::encodeName(fileName).constData(),
             O_EXCL | O_CREAT | O_RDWR, 0640);
     if (-1 == fd) {
