@@ -181,17 +181,16 @@ QSocketNotifier::QSocketNotifier(qintptr socket, Type type, QObject *parent)
     : QObject(*new QSocketNotifierPrivate, parent)
 {
     Q_D(QSocketNotifier);
-    if (socket < 0)
-        qWarning("QSocketNotifier: Invalid socket specified");
     d->sockfd = socket;
     d->sntype = type;
     d->snenabled = true;
 
-    if (!d->threadData->eventDispatcher) {
+    if (socket < 0)
+        qWarning("QSocketNotifier: Invalid socket specified");
+    else if (!d->threadData->eventDispatcher.load())
         qWarning("QSocketNotifier: Can only be used with threads started with QThread");
-    } else {
-        d->threadData->eventDispatcher->registerSocketNotifier(this);
-    }
+    else
+        d->threadData->eventDispatcher.load()->registerSocketNotifier(this);
 }
 
 /*!
@@ -239,7 +238,7 @@ QSocketNotifier::Type QSocketNotifier::type() const
 }
 
 /*!
-    Returns true if the notifier is enabled; otherwise returns false.
+    Returns \c true if the notifier is enabled; otherwise returns \c false.
 
     \sa setEnabled()
 */
@@ -273,12 +272,12 @@ void QSocketNotifier::setEnabled(bool enable)
         return;
     d->snenabled = enable;
 
-    if (!d->threadData->eventDispatcher) // perhaps application/thread is shutting down
+    if (!d->threadData->eventDispatcher.load()) // perhaps application/thread is shutting down
         return;
     if (d->snenabled)
-        d->threadData->eventDispatcher->registerSocketNotifier(this);
+        d->threadData->eventDispatcher.load()->registerSocketNotifier(this);
     else
-        d->threadData->eventDispatcher->unregisterSocketNotifier(this);
+        d->threadData->eventDispatcher.load()->unregisterSocketNotifier(this);
 }
 
 
