@@ -135,7 +135,7 @@ QWinEventNotifier::QWinEventNotifier(HANDLE hEvent, QObject *parent)
  : QObject(*new QWinEventNotifierPrivate(hEvent, false), parent)
 {
     Q_D(QWinEventNotifier);
-    QAbstractEventDispatcher *eventDispatcher = d->threadData->eventDispatcher;
+    QAbstractEventDispatcher *eventDispatcher = d->threadData->eventDispatcher.load();
     if (!eventDispatcher) {
         qWarning("QWinEventNotifier: Can only be used with threads started with QThread");
     } else {
@@ -183,7 +183,7 @@ HANDLE  QWinEventNotifier::handle() const
 }
 
 /*!
-    Returns true if the notifier is enabled; otherwise returns false.
+    Returns \c true if the notifier is enabled; otherwise returns \c false.
 
     \sa setEnabled()
 */
@@ -208,7 +208,7 @@ void QWinEventNotifier::setEnabled(bool enable)
         return;
     d->enabled = enable;
 
-    QAbstractEventDispatcher *eventDispatcher = d->threadData->eventDispatcher;
+    QAbstractEventDispatcher *eventDispatcher = d->threadData->eventDispatcher.load();
     if (!eventDispatcher) // perhaps application is shutting down
         return;
 
