@@ -45,7 +45,6 @@
 #include <QtCore/qshareddata.h>
 #include <QtCore/qstring.h>
 
-QT_BEGIN_HEADER
 QT_BEGIN_NAMESPACE
 
 class QMimeTypePrivate;
@@ -112,6 +111,4 @@ protected:
 Q_DECLARE_SHARED(QMimeType)
 
 QT_END_NAMESPACE
-QT_END_HEADER
-
 #endif   // QMIMETYPE_H
