@@ -1,7 +1,7 @@
-
 /****************************************************************************
 **
 ** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
+** Copyright (C) 2013 Intel Corporation
 ** Contact: http://www.qt-project.org/legal
 **
 ** This file is part of the QtCore module of the Qt Toolkit.
@@ -64,6 +64,7 @@
 #include <qjsondocument.h>
 #include <qjsonvalue.h>
 #include "qelfparser_p.h"
+#include "qmachparser_p.h"
 
 QT_BEGIN_NAMESPACE
 
@@ -79,8 +80,6 @@ QT_BEGIN_NAMESPACE
 #  define QT_NO_DEBUG_PLUGIN_CHECK
 #endif
 
-static QBasicMutex qt_library_mutex;
-
 /*!
     \class QLibrary
     \inmodule QtCore
@@ -182,8 +181,6 @@ static QBasicMutex qt_library_mutex;
 */
 
 
-#if defined(Q_OS_UNIX) && !defined(Q_OS_MAC)
-
 static long qt_find_pattern(const char *s, ulong s_len,
                              const char *pattern, ulong p_len)
 {
@@ -226,11 +223,11 @@ static long qt_find_pattern(const char *s, ulong s_len,
   we can get the verification data without have to actually load the library.
   This lets us detect mismatches more safely.
 
-  Returns false if version information is not present, or if the
+  Returns \c false if version information is not present, or if the
                 information could not be read.
   Returns  true if version information is present and successfully read.
 */
-static bool qt_unix_query(const QString &library, QLibraryPrivate *lib)
+static bool findPatternUnloaded(const QString &library, QLibraryPrivate *lib)
 {
     QFile file(library);
     if (!file.open(QIODevice::ReadOnly)) {
@@ -255,11 +252,12 @@ static bool qt_unix_query(const QString &library, QLibraryPrivate *lib)
     }
 
     /*
-       ELF binaries on GNU, have .qplugin sections.
+       ELF and Mach-O binaries with GCC have .qplugin sections.
     */
     bool hasMetaData = false;
     long pos = 0;
-    const char pattern[] = "QTMETADATA  ";
+    char pattern[] = "qTMETADATA  ";
+    pattern[0] = 'Q'; // Ensure the pattern "QTMETADATA" is not found in this library should QPluginLoader ever encounter it.
     const ulong plen = qstrlen(pattern);
 #if defined (Q_OF_ELF) && defined(Q_CC_GNU)
     int r = QElfParser().parse(filedata, fdlen, library, lib, &pos, &fdlen);
@@ -276,6 +274,26 @@ static bool qt_unix_query(const QString &library, QLibraryPrivate *lib)
             pos += rel;
         hasMetaData = true;
     }
+#elif defined (Q_OF_MACH_O)
+    {
+        QString errorString;
+        int r = QMachOParser::parse(filedata, fdlen, library, &errorString, &pos, &fdlen);
+        if (r == QMachOParser::NotSuitable) {
+            if (qt_debug_component())
+                qWarning("QMachOParser: %s", qPrintable(errorString));
+            if (lib)
+                lib->errorString = errorString;
+            return false;
+        }
+        // even if the metadata section was not found, the Mach-O parser will
+        // at least return the boundaries of the right architecture
+        long rel = qt_find_pattern(filedata + pos, fdlen, pattern, plen);
+        if (rel < 0)
+            pos = -1;
+        else
+            pos += rel;
+        hasMetaData = true;
+    }
 #else
     pos = qt_find_pattern(filedata, fdlen, pattern, plen);
     if (pos > 0)
@@ -302,8 +320,6 @@ static bool qt_unix_query(const QString &library, QLibraryPrivate *lib)
     return ret;
 }
 
-#endif // Q_OS_UNIX && !Q_OS_MAC
-
 static void installCoverageTool(QLibraryPrivate *libPrivate)
 {
 #ifdef __COVERAGESCANNER__
@@ -339,46 +355,143 @@ static void installCoverageTool(QLibraryPrivate *libPrivate)
 #endif
 }
 
-typedef QMap<QString, QLibraryPrivate*> LibraryMap;
+class QLibraryStore
+{
+public:
+    inline ~QLibraryStore();
+    static inline QLibraryPrivate *findOrCreate(const QString &fileName, const QString &version);
+    static inline void releaseLibrary(QLibraryPrivate *lib);
 
-struct LibraryData {
+    static inline void cleanup();
+
+private:
+    static inline QLibraryStore *instance();
+
+    // all members and instance() are protected by qt_library_mutex
+    typedef QMap<QString, QLibraryPrivate*> LibraryMap;
     LibraryMap libraryMap;
-    QSet<QLibraryPrivate*> loadedLibs;
 };
 
-Q_GLOBAL_STATIC(LibraryData, libraryData)
+static QBasicMutex qt_library_mutex;
+static QLibraryStore *qt_library_data = 0;
+static bool qt_library_data_once;
 
-static LibraryMap *libraryMap()
+QLibraryStore::~QLibraryStore()
 {
-    LibraryData *data = libraryData();
-    return data ? &data->libraryMap : 0;
+    qt_library_data = 0;
+}
+
+inline void QLibraryStore::cleanup()
+{
+    QLibraryStore *data = qt_library_data;
+    if (!data)
+        return;
+
+    // find any libraries that are still loaded but have a no one attached to them
+    LibraryMap::Iterator it = data->libraryMap.begin();
+    for (; it != data->libraryMap.end(); ++it) {
+        QLibraryPrivate *lib = it.value();
+        if (lib->libraryRefCount.load() == 1) {
+            if (lib->libraryUnloadCount.load() > 0) {
+                Q_ASSERT(lib->pHnd);
+                lib->libraryUnloadCount.store(1);
+#ifdef __GLIBC__
+                // glibc has a bug in unloading from global destructors
+                // see https://bugzilla.novell.com/show_bug.cgi?id=622977
+                // and http://sourceware.org/bugzilla/show_bug.cgi?id=11941
+                lib->unload(QLibraryPrivate::NoUnloadSys);
+#else
+                lib->unload();
+#endif
+            }
+            delete lib;
+            it.value() = 0;
+        }
+    }
+
+    if (qt_debug_component()) {
+        // dump all objects that remain
+        foreach (QLibraryPrivate *lib, data->libraryMap) {
+            if (lib)
+                qDebug() << "On QtCore unload," << lib->fileName << "was leaked, with"
+                         << lib->libraryRefCount.load() << "users";
+        }
+    }
+
+    delete data;
+}
+
+static void qlibraryCleanup()
+{
+    QLibraryStore::cleanup();
+}
+Q_DESTRUCTOR_FUNCTION(qlibraryCleanup)
+
+// must be called with a locked mutex
+QLibraryStore *QLibraryStore::instance()
+{
+    if (Q_UNLIKELY(!qt_library_data_once && !qt_library_data)) {
+        // only create once per process lifetime
+        qt_library_data = new QLibraryStore;
+        qt_library_data_once = true;
+    }
+    return qt_library_data;
+}
+
+inline QLibraryPrivate *QLibraryStore::findOrCreate(const QString &fileName, const QString &version)
+{
+    QMutexLocker locker(&qt_library_mutex);
+    QLibraryStore *data = instance();
+
+    // check if this library is already loaded
+    QLibraryPrivate *lib = 0;
+    if (Q_LIKELY(data))
+        lib = data->libraryMap.value(fileName);
+    if (!lib)
+        lib = new QLibraryPrivate(fileName, version);
+
+    // track this library
+    if (Q_LIKELY(data))
+        data->libraryMap.insert(fileName, lib);
+
+    lib->libraryRefCount.ref();
+    return lib;
+}
+
+inline void QLibraryStore::releaseLibrary(QLibraryPrivate *lib)
+{
+    QMutexLocker locker(&qt_library_mutex);
+    QLibraryStore *data = instance();
+
+    if (lib->libraryRefCount.deref()) {
+        // still in use
+        return;
+    }
+
+    // no one else is using
+    Q_ASSERT(lib->libraryUnloadCount.load() == 0);
+
+    if (Q_LIKELY(data)) {
+        QLibraryPrivate *that = data->libraryMap.take(lib->fileName);
+        Q_ASSERT(lib == that);
+        Q_UNUSED(that);
+    }
+    delete lib;
 }
 
 QLibraryPrivate::QLibraryPrivate(const QString &canonicalFileName, const QString &version)
     : pHnd(0), fileName(canonicalFileName), fullVersion(version), instance(0),
       loadHints(0),
-      libraryRefCount(1), libraryUnloadCount(0), pluginState(MightBeAPlugin)
-{ libraryMap()->insert(canonicalFileName, this); }
+      libraryRefCount(0), libraryUnloadCount(0), pluginState(MightBeAPlugin)
+{ }
 
 QLibraryPrivate *QLibraryPrivate::findOrCreate(const QString &fileName, const QString &version)
 {
-    QMutexLocker locker(&qt_library_mutex);
-    if (QLibraryPrivate *lib = libraryMap()->value(fileName)) {
-        lib->libraryRefCount.ref();
-        return lib;
-    }
-
-    return new QLibraryPrivate(fileName, version);
+    return QLibraryStore::findOrCreate(fileName, version);
 }
 
 QLibraryPrivate::~QLibraryPrivate()
 {
-    LibraryMap * const map = libraryMap();
-    if (map) {
-        QLibraryPrivate *that = map->take(fileName);
-        Q_ASSERT(this == that);
-	Q_UNUSED(that);
-    }
 }
 
 QFunctionPointer QLibraryPrivate::resolve(const char *symbol)
@@ -391,9 +504,10 @@ QFunctionPointer QLibraryPrivate::resolve(const char *symbol)
 
 bool QLibraryPrivate::load()
 {
-    libraryUnloadCount.ref();
-    if (pHnd)
+    if (pHnd) {
+        libraryUnloadCount.ref();
         return true;
+    }
     if (fileName.isEmpty())
         return false;
 
@@ -403,32 +517,27 @@ bool QLibraryPrivate::load()
     if (ret) {
         //when loading a library we add a reference to it so that the QLibraryPrivate won't get deleted
         //this allows to unload the library at a later time
-        if (LibraryData *lib = libraryData()) {
-            lib->loadedLibs += this;
-            libraryRefCount.ref();
-        }
-
+        libraryUnloadCount.ref();
+        libraryRefCount.ref();
         installCoverageTool(this);
     }
 
     return ret;
 }
 
-bool QLibraryPrivate::unload()
+bool QLibraryPrivate::unload(UnloadFlag flag)
 {
     if (!pHnd)
         return false;
-    if (!libraryUnloadCount.deref()) { // only unload if ALL QLibrary instance wanted to
+    if (libraryUnloadCount.load() > 0 && !libraryUnloadCount.deref()) { // only unload if ALL QLibrary instance wanted to
         delete inst.data();
-        if  (unload_sys()) {
+        if (flag == NoUnloadSys || unload_sys()) {
             if (qt_debug_component())
-                qWarning() << "QLibraryPrivate::unload succeeded on" << fileName;
+                qWarning() << "QLibraryPrivate::unload succeeded on" << fileName
+                           << (flag == NoUnloadSys ? "(faked)" : "");
             //when the library is unloaded, we release the reference on it so that 'this'
             //can get deleted
-            if (LibraryData *lib = libraryData()) {
-                if (lib->loadedLibs.remove(this))
-                    libraryRefCount.deref();
-            }
+            libraryRefCount.deref();
             pHnd = 0;
             instance = 0;
         }
@@ -439,9 +548,7 @@ bool QLibraryPrivate::unload()
 
 void QLibraryPrivate::release()
 {
-    QMutexLocker locker(&qt_library_mutex);
-    if (!libraryRefCount.deref())
-        delete this;
+    QLibraryStore::releaseLibrary(this);
 }
 
 bool QLibraryPrivate::loadPlugin()
@@ -463,8 +570,8 @@ bool QLibraryPrivate::loadPlugin()
 }
 
 /*!
-    Returns true if \a fileName has a valid suffix for a loadable
-    library; otherwise returns false.
+    Returns \c true if \a fileName has a valid suffix for a loadable
+    library; otherwise returns \c false.
 
     \table
     \header \li Platform \li Valid suffixes
@@ -538,41 +645,18 @@ bool QLibrary::isLibrary(const QString &fileName)
 
 }
 
-#if defined (Q_OS_WIN) && defined(_MSC_VER) && _MSC_VER >= 1400  && !defined(Q_CC_INTEL)
-#define QT_USE_MS_STD_EXCEPTION 1
-const char* qt_try_versioninfo(void *pfn, bool *exceptionThrown)
-{
-    *exceptionThrown = false;
-    const char *szData = 0;
-    typedef const char * (*VerificationFunction)();
-    VerificationFunction func = reinterpret_cast<VerificationFunction>(pfn);
-    __try {
-        if(func)
-            szData =  func();
-    } __except(EXCEPTION_EXECUTE_HANDLER) {
-        *exceptionThrown = true;
-    }
-    return szData;
-}
-#endif
-
 typedef const char * (*QtPluginQueryVerificationDataFunction)();
 
-bool qt_get_metadata(QtPluginQueryVerificationDataFunction pfn, QLibraryPrivate *priv, bool *exceptionThrown)
+static bool qt_get_metadata(QtPluginQueryVerificationDataFunction pfn, QLibraryPrivate *priv)
 {
-    *exceptionThrown = false;
     const char *szData = 0;
     if (!pfn)
         return false;
-#ifdef QT_USE_MS_STD_EXCEPTION
-    szData = qt_try_versioninfo((void *)pfn, exceptionThrown);
-    if (*exceptionThrown)
-        return false;
-#else
+
     szData = pfn();
-#endif
     if (!szData)
         return false;
+
     QJsonDocument doc = QLibraryPrivate::fromRawMetaData(szData);
     if (doc.isNull())
         return false;
@@ -610,81 +694,15 @@ void QLibraryPrivate::updatePluginState()
     }
 #endif
 
-#if defined(Q_OS_UNIX) && !defined(Q_OS_MAC)
     if (!pHnd) {
-        // use unix shortcut to avoid loading the library
-        success = qt_unix_query(fileName, this);
-    } else
-#endif
-    {
-        bool retryLoadLibrary = false;    // Only used on Windows with MS compiler.(false in other cases)
-        do {
-            bool temporary_load = false;
-#ifdef Q_OS_WIN
-            HMODULE hTempModule = 0;
-#endif
-            if (!pHnd) {
-#ifdef Q_OS_WIN
-                DWORD dwFlags = (retryLoadLibrary) ? 0: DONT_RESOLVE_DLL_REFERENCES;
-                //avoid 'Bad Image' message box
-                UINT oldmode = SetErrorMode(SEM_FAILCRITICALERRORS|SEM_NOOPENFILEERRORBOX);
-                hTempModule = ::LoadLibraryEx((wchar_t*)QDir::toNativeSeparators(fileName).utf16(), 0, dwFlags);
-                SetErrorMode(oldmode);
-#else
-                temporary_load =  load_sys();
-#endif
-            }
-            QtPluginQueryVerificationDataFunction getMetaData = NULL;
-
-            bool exceptionThrown = false;
-            bool ret = false;
-#ifdef Q_OS_WIN
-            if (hTempModule) {
-                getMetaData = (QtPluginQueryVerificationDataFunction)
-#ifdef Q_OS_WINCE
-                    ::GetProcAddress(hTempModule, L"qt_plugin_query_metadata")
-#else
-                    ::GetProcAddress(hTempModule, "qt_plugin_query_metadata")
-#endif
-                        ;
-            } else
-#endif
-            {
-                getMetaData = (QtPluginQueryVerificationDataFunction) resolve("qt_plugin_query_metadata");
-            }
-
-            if (getMetaData)
-                ret = qt_get_metadata(getMetaData, this, &exceptionThrown);
-
-            if (!exceptionThrown) {
-                if (!ret) {
-                    if (temporary_load)
-                        unload_sys();
-                } else {
-                    success = true;
-                }
-                retryLoadLibrary = false;
-            }
-#ifdef QT_USE_MS_STD_EXCEPTION
-            else {
-                // An exception was thrown when calling qt_plugin_query_verification_data().
-                // This usually happens when plugin is compiled with the /clr compiler flag,
-                // & will only work if the dependencies are loaded & DLLMain() is called.
-                // LoadLibrary() will do this, try once with this & if it fails don't load.
-                retryLoadLibrary = !retryLoadLibrary;
-            }
-#endif
-#ifdef Q_OS_WIN
-            if (hTempModule) {
-                BOOL ok = ::FreeLibrary(hTempModule);
-                if (ok) {
-                    hTempModule = 0;
-                }
-
-            }
-#endif
-        } while (retryLoadLibrary);  // Will be 'false' in all cases other than when an
-        // exception is thrown(will happen only when using a MS compiler)
+        // scan for the plugin metadata without loading
+        success = findPatternUnloaded(fileName, this);
+    } else {
+        // library is already loaded (probably via QLibrary)
+        // simply get the target function and call it.
+        QtPluginQueryVerificationDataFunction getMetaData = NULL;
+        getMetaData = (QtPluginQueryVerificationDataFunction) resolve("qt_plugin_query_metadata");
+        success = qt_get_metadata(getMetaData, this);
     }
 
     if (!success) {
@@ -728,8 +746,8 @@ void QLibraryPrivate::updatePluginState()
 }
 
 /*!
-    Loads the library and returns true if the library was loaded
-    successfully; otherwise returns false. Since resolve() always
+    Loads the library and returns \c true if the library was loaded
+    successfully; otherwise returns \c false. Since resolve() always
     calls this function before resolving any symbols it is not
     necessary to call it explicitly. In some situations you might want
     the library loaded in advance, in which case you would use this
@@ -748,8 +766,8 @@ bool QLibrary::load()
 }
 
 /*!
-    Unloads the library and returns true if the library could be
-    unloaded; otherwise returns false.
+    Unloads the library and returns \c true if the library could be
+    unloaded; otherwise returns \c false.
 
     This happens automatically on application termination, so you
     shouldn't normally need to call this function.
@@ -772,7 +790,7 @@ bool QLibrary::unload()
 }
 
 /*!
-    Returns true if the library is loaded; otherwise returns false.
+    Returns \c true if the library is loaded; otherwise returns \c false.
 
     \sa load()
  */
