@@ -71,6 +71,7 @@ QT_BEGIN_NAMESPACE
 
 bool qt_debug_component();
 
+class QLibraryStore;
 class QLibraryPrivate
 {
 public:
@@ -82,12 +83,14 @@ public:
 #endif
     pHnd;
 
+    enum UnloadFlag { UnloadSys, NoUnloadSys };
+
     QString fileName, qualifiedFileName;
     QString fullVersion;
 
     bool load();
     bool loadPlugin(); // loads and resolves instance
-    bool unload();
+    bool unload(UnloadFlag flag = UnloadSys);
     void release();
     QFunctionPointer resolve(const char *);
 
@@ -95,9 +98,6 @@ public:
     static QStringList suffixes_sys(const QString &fullVersion);
     static QStringList prefixes_sys();
 
-    static QVector<QStaticPlugin> staticPlugins();
-
-
     QPointer<QObject> inst;
     QtPluginInstanceFunction instance;
     QJsonObject metaData;
@@ -124,11 +124,13 @@ private:
     bool unload_sys();
     QFunctionPointer resolve_sys(const char *);
 
+    /// counts how many QLibrary or QPluginLoader are attached to us, plus 1 if it's loaded
     QAtomicInt libraryRefCount;
+    /// counts how many times load() or loadPlugin() were called
     QAtomicInt libraryUnloadCount;
 
     enum { IsAPlugin, IsNotAPlugin, MightBeAPlugin } pluginState;
-    friend class QLibraryPrivateHasFriends;
+    friend class QLibraryStore;
 };
 
 QT_END_NAMESPACE
