@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2012 Digia Plc and/or its subsidiary(-ies).
+** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
 ** This file is part of the QtCore module of the Qt Toolkit.
@@ -73,6 +73,8 @@
 #include <QtCore/qmetaobject.h>
 #include <qdebug.h>
 
+#include <algorithm>
+
 QT_BEGIN_NAMESPACE
 
 /*!
@@ -167,7 +169,7 @@ QT_BEGIN_NAMESPACE
 
     \brief whether animations are enabled
 
-    The default value of this property is true.
+    The default value of this property is \c true.
 
     \sa QAbstractTransition::addAnimation()
 */
@@ -438,7 +440,7 @@ QList<QAbstractState*> QStateMachinePrivate::computeStatesToExit(const QList<QAb
         }
     }
     QList<QAbstractState*> statesToExit_sorted = statesToExit.toList();
-    qSort(statesToExit_sorted.begin(), statesToExit_sorted.end(), stateExitLessThan);
+    std::sort(statesToExit_sorted.begin(), statesToExit_sorted.end(), stateExitLessThan);
     return statesToExit_sorted;
 }
 
@@ -541,7 +543,7 @@ QList<QAbstractState*> QStateMachinePrivate::computeStatesToEnter(const QList<QA
     }
 
     QList<QAbstractState*> statesToEnter_sorted = statesToEnter.toList();
-    qSort(statesToEnter_sorted.begin(), statesToEnter_sorted.end(), stateEntryLessThan);
+    std::sort(statesToEnter_sorted.begin(), statesToEnter_sorted.end(), stateEntryLessThan);
     return statesToEnter_sorted;
 }
 
@@ -860,8 +862,8 @@ bool QStateMachinePrivate::isInFinalState(QAbstractState* s) const
 
 /*!
   \internal
-  Returns true if the given state has saved the value of the given property,
-  otherwise returns false.
+  Returns \c true if the given state has saved the value of the given property,
+  otherwise returns \c false.
 */
 bool QStateMachinePrivate::hasRestorable(QAbstractState *state, QObject *object,
                                          const QByteArray &propertyName) const
@@ -2262,8 +2264,8 @@ int QStateMachine::postDelayedEvent(QEvent *event, int delay)
   \threadsafe
 
   Cancels the delayed event identified by the given \a id. The id should be a
-  value returned by a call to postDelayedEvent(). Returns true if the event
-  was successfully cancelled, otherwise returns false.
+  value returned by a call to postDelayedEvent(). Returns \c true if the event
+  was successfully cancelled, otherwise returns \c false.
 
   \sa postDelayedEvent()
 */
