@@ -46,8 +46,6 @@
 #include <QtCore/qatomic.h>
 #include <new>
 
-QT_BEGIN_HEADER
-
 QT_BEGIN_NAMESPACE
 
 
@@ -211,6 +209,4 @@ typedef QMutex QBasicMutex;
 
 QT_END_NAMESPACE
 
-QT_END_HEADER
-
 #endif // QMUTEX_H
