@@ -42,11 +42,14 @@
 #include "qplatformdefs.h"
 #include "qmutex.h"
 #include "qstring.h"
+#include "qelapsedtimer.h"
 
 #ifndef QT_NO_THREAD
 #include "qatomic.h"
 #include "qmutex_p.h"
 #include <errno.h>
+#include <sys/time.h>
+#include <time.h>
 
 #if defined(Q_OS_VXWORKS) && defined(wakeup)
 #undef wakeup
@@ -64,7 +67,7 @@ QMutexPrivate::QMutexPrivate()
     : wakeup(false)
 {
     report_error(pthread_mutex_init(&mutex, NULL), "QMutex", "mutex init");
-    report_error(pthread_cond_init(&cond, NULL), "QMutex", "cv init");
+    qt_initialize_pthread_cond(&cond, "QMutex");
 }
 
 QMutexPrivate::~QMutexPrivate()
@@ -81,12 +84,8 @@ bool QMutexPrivate::wait(int timeout)
         if (timeout < 0) {
             errorCode = pthread_cond_wait(&cond, &mutex);
         } else {
-            struct timeval tv;
-            gettimeofday(&tv, 0);
             timespec ti;
-            ti.tv_nsec = (tv.tv_usec + (timeout % 1000) * 1000) * 1000;
-            ti.tv_sec = tv.tv_sec + (timeout / 1000) + (ti.tv_nsec / 1000000000);
-            ti.tv_nsec %= 1000000000;
+            qt_abstime_for_timeout(&ti, timeout);
             errorCode = pthread_cond_timedwait(&cond, &mutex, &ti);
         }
         if (errorCode) {
