@@ -46,8 +46,6 @@
 
 #include <limits.h>
 
-QT_BEGIN_HEADER
-
 QT_BEGIN_NAMESPACE
 
 
@@ -88,6 +86,9 @@ public:
     bool isFinished() const;
     bool isRunning() const;
 
+    void requestInterruption();
+    bool isInterruptionRequested() const;
+
     void setStackSize(uint stackSize);
     uint stackSize() const;
 
@@ -164,6 +165,4 @@ private:
 
 QT_END_NAMESPACE
 
-QT_END_HEADER
-
 #endif // QTHREAD_H
