@@ -142,12 +142,15 @@ public:
     QThreadPrivate(QThreadData *d = 0);
     ~QThreadPrivate();
 
+    void setPriority(QThread::Priority prio);
+
     mutable QMutex mutex;
     QAtomicInt quitLockRef;
 
     bool running;
     bool finished;
     bool isInFinish; //when in QThreadPrivate::finish
+    bool interruptionRequested;
 
     bool exited;
     int returnCode;
@@ -216,29 +219,63 @@ public:
 
 class QThreadData
 {
-    QAtomicInt _ref;
-
 public:
     QThreadData(int initialRefCount = 1);
     ~QThreadData();
 
     static QThreadData *current();
+    static void clearCurrentThreadData();
     static QThreadData *get2(QThread *thread)
     { Q_ASSERT_X(thread != 0, "QThread", "internal error"); return thread->d_func()->data; }
 
 
     void ref();
     void deref();
+    inline bool hasEventDispatcher() const
+    { return eventDispatcher.load() != 0; }
 
-    QThread *thread;
-    Qt::HANDLE threadId;
-    bool quitNow;
+    bool canWaitLocked()
+    {
+        QMutexLocker locker(&postEventList.mutex);
+        return canWait;
+    }
+
+    // This class provides per-thread (by way of being a QThreadData
+    // member) storage for qFlagLocation()
+    class FlaggedDebugSignatures
+    {
+        static const uint Count = 2;
+
+        uint idx;
+        const char* locations[Count];
+
+    public:
+        FlaggedDebugSignatures() : idx(0)
+        { std::fill_n(locations, Count, static_cast<char*>(0)); }
+
+        void store(const char* method)
+        { locations[idx++ % Count] = method; }
+
+        bool contains(const char *method) const
+        { return std::find(locations, locations + Count, method) != locations + Count; }
+    };
+
+private:
+    QAtomicInt _ref;
+
+public:
     int loopLevel;
-    QAbstractEventDispatcher *eventDispatcher;
+
     QStack<QEventLoop *> eventLoops;
     QPostEventList postEventList;
-    bool canWait;
+    QThread *thread;
+    Qt::HANDLE threadId;
+    QAtomicPointer<QAbstractEventDispatcher> eventDispatcher;
     QVector<void *> tls;
+    FlaggedDebugSignatures flaggedSignatures;
+
+    bool quitNow;
+    bool canWait;
     bool isAdopted;
 };
 
