@@ -96,6 +96,11 @@ Q_DESTRUCTOR_FUNCTION(qt_free_tls)
 /*
     QThreadData
 */
+void QThreadData::clearCurrentThreadData()
+{
+    TlsSetValue(qt_current_thread_data_tls_index, 0);
+}
+
 QThreadData *QThreadData::current()
 {
     qt_create_tls();
@@ -177,7 +182,7 @@ void qt_watch_adopted_thread(const HANDLE adoptedThreadHandle, QThread *qthread)
             qt_adopted_thread_handles.prepend(qt_adopted_thread_wakeup);
         }
 
-        CreateThread(0, 0, qt_adopted_thread_watcher_function, 0, 0, &qt_adopted_thread_watcher_id);
+        CloseHandle(CreateThread(0, 0, qt_adopted_thread_watcher_function, 0, 0, &qt_adopted_thread_watcher_id));
     } else {
         SetEvent(qt_adopted_thread_wakeup);
     }
@@ -203,7 +208,11 @@ DWORD WINAPI qt_adopted_thread_watcher_function(LPVOID)
         qt_adopted_thread_watcher_mutex.unlock();
 
         DWORD ret = WAIT_TIMEOUT;
-        int loops = (handlesCopy.count() / MAXIMUM_WAIT_OBJECTS) + 1, offset, count;
+        int count;
+        int offset;
+        int loops = handlesCopy.size() / MAXIMUM_WAIT_OBJECTS;
+        if (handlesCopy.size() % MAXIMUM_WAIT_OBJECTS)
+            ++loops;
         if (loops == 1) {
             // no need to loop, no timeout
             offset = 0;
@@ -301,8 +310,9 @@ void qt_set_thread_name(HANDLE threadId, LPCSTR threadName)
 
 void QThreadPrivate::createEventDispatcher(QThreadData *data)
 {
-    data->eventDispatcher = new QEventDispatcherWin32;
-    data->eventDispatcher->startingUp();
+    QEventDispatcherWin32 *theEventDispatcher = new QEventDispatcherWin32;
+    data->eventDispatcher.storeRelease(theEventDispatcher);
+    theEventDispatcher->startingUp();
 }
 
 #ifndef QT_NO_THREAD
@@ -323,8 +333,8 @@ unsigned int __stdcall QT_ENSURE_STACK_ALIGNED_FOR_SSE QThreadPrivate::start(voi
         data->quitNow = thr->d_func()->exited;
     }
 
-    if (data->eventDispatcher) // custom event dispatcher set?
-        data->eventDispatcher->startingUp();
+    if (data->eventDispatcher.load()) // custom event dispatcher set?
+        data->eventDispatcher.load()->startingUp();
     else
         createEventDispatcher(data);
 
@@ -359,7 +369,7 @@ void QThreadPrivate::finish(void *arg, bool lockAnyway)
     QThreadStorageData::finish(tls_data);
     locker.relock();
 
-    QAbstractEventDispatcher *eventDispatcher = d->data->eventDispatcher;
+    QAbstractEventDispatcher *eventDispatcher = d->data->eventDispatcher.load();
     if (eventDispatcher) {
         d->data->eventDispatcher = 0;
         locker.unlock();
@@ -371,6 +381,7 @@ void QThreadPrivate::finish(void *arg, bool lockAnyway)
     d->running = false;
     d->finished = true;
     d->isInFinish = false;
+    d->interruptionRequested = false;
 
     if (!d->waiters) {
         CloseHandle(d->handle);
@@ -440,6 +451,7 @@ void QThread::start(Priority priority)
     d->finished = false;
     d->exited = false;
     d->returnCode = 0;
+    d->interruptionRequested = false;
 
     /*
       NOTE: we create the thread in the suspended state, set the
@@ -582,55 +594,49 @@ void QThread::setTerminationEnabled(bool enabled)
     }
 }
 
-void QThread::setPriority(Priority priority)
+// Caller must hold the mutex
+void QThreadPrivate::setPriority(QThread::Priority threadPriority)
 {
-    Q_D(QThread);
-    QMutexLocker locker(&d->mutex);
-    if (!d->running) {
-        qWarning("QThread::setPriority: Cannot set priority, thread is not running");
-        return;
-    }
-
     // copied from start() with a few modifications:
 
     int prio;
-    d->priority = priority;
-    switch (d->priority) {
-    case IdlePriority:
+    priority = threadPriority;
+    switch (priority) {
+    case QThread::IdlePriority:
         prio = THREAD_PRIORITY_IDLE;
         break;
 
-    case LowestPriority:
+    case QThread::LowestPriority:
         prio = THREAD_PRIORITY_LOWEST;
         break;
 
-    case LowPriority:
+    case QThread::LowPriority:
         prio = THREAD_PRIORITY_BELOW_NORMAL;
         break;
 
-    case NormalPriority:
+    case QThread::NormalPriority:
         prio = THREAD_PRIORITY_NORMAL;
         break;
 
-    case HighPriority:
+    case QThread::HighPriority:
         prio = THREAD_PRIORITY_ABOVE_NORMAL;
         break;
 
-    case HighestPriority:
+    case QThread::HighestPriority:
         prio = THREAD_PRIORITY_HIGHEST;
         break;
 
-    case TimeCriticalPriority:
+    case QThread::TimeCriticalPriority:
         prio = THREAD_PRIORITY_TIME_CRITICAL;
         break;
 
-    case InheritPriority:
+    case QThread::InheritPriority:
     default:
         qWarning("QThread::setPriority: Argument cannot be InheritPriority");
         return;
     }
 
-    if (!SetThreadPriority(d->handle, prio)) {
+    if (!SetThreadPriority(handle, prio)) {
         qErrnoWarning("QThread::setPriority: Failed to set thread priority");
     }
 }
