@@ -49,7 +49,6 @@
 
 #ifndef QT_NO_THREAD
 
-QT_BEGIN_HEADER
 QT_BEGIN_NAMESPACE
 
 
@@ -84,10 +83,11 @@ public:
     void releaseThread();
 
     bool waitForDone(int msecs = -1);
+
+    void clear();
 };
 
 QT_END_NAMESPACE
-QT_END_HEADER
 
 #endif // QT_NO_THREAD
 
