@@ -44,8 +44,6 @@
 
 #include <QtCore/qarraydataops.h>
 
-QT_BEGIN_HEADER
-
 QT_BEGIN_NAMESPACE
 
 template <class T>
@@ -219,6 +217,4 @@ namespace std
     }
 }
 
-QT_END_HEADER
-
 #endif // include guard
