@@ -66,8 +66,6 @@
 #endif
 
 
-QT_BEGIN_HEADER
-
 QT_BEGIN_NAMESPACE
 
 
@@ -175,6 +173,15 @@ private:
     typedef QTypedArrayData<char> Data;
 
 public:
+    enum Base64Option {
+        Base64Encoding = 0,
+        Base64UrlEncoding = 1,
+
+        KeepTrailingEquals = 0,
+        OmitTrailingEquals = 2
+    };
+    Q_DECLARE_FLAGS(Base64Options, Base64Option)
+
     inline QByteArray();
     QByteArray(const char *, int size = -1);
     QByteArray(int size, char c);
@@ -319,7 +326,8 @@ public:
     qulonglong toULongLong(bool *ok = 0, int base = 10) const;
     float toFloat(bool *ok = 0) const;
     double toDouble(bool *ok = 0) const;
-    QByteArray toBase64() const;
+    QByteArray toBase64(Base64Options options) const;
+    QByteArray toBase64() const; // ### Qt6 merge with previous
     QByteArray toHex() const;
     QByteArray toPercentEncoding(const QByteArray &exclude = QByteArray(),
                                  const QByteArray &include = QByteArray(),
@@ -341,7 +349,8 @@ public:
     static QByteArray number(qulonglong, int base = 10);
     static QByteArray number(double, char f = 'g', int prec = 6);
     static QByteArray fromRawData(const char *, int size);
-    static QByteArray fromBase64(const QByteArray &base64);
+    static QByteArray fromBase64(const QByteArray &base64, Base64Options options);
+    static QByteArray fromBase64(const QByteArray &base64); // ### Qt6 merge with previous
     static QByteArray fromHex(const QByteArray &hexEncoded);
     static QByteArray fromPercentEncoding(const QByteArray &pctEncoded, char percent = '%');
 
@@ -375,7 +384,7 @@ public:
     bool isNull() const;
 
     inline QByteArray(QByteArrayDataPtr dd)
-        : d(reinterpret_cast<Data *>(dd.ptr))
+        : d(static_cast<Data *>(dd.ptr))
     {
     }
 
@@ -394,6 +403,8 @@ public:
     inline DataPtr &data_ptr() { return d; }
 };
 
+Q_DECLARE_OPERATORS_FOR_FLAGS(QByteArray::Base64Options)
+
 inline QByteArray::QByteArray(): d(Data::sharedNull()) { }
 inline QByteArray::~QByteArray() { if (!d->ref.deref()) Data::deallocate(d); }
 inline int QByteArray::size() const
@@ -610,8 +621,6 @@ Q_DECLARE_SHARED(QByteArray)
 
 QT_END_NAMESPACE
 
-QT_END_HEADER
-
 #ifdef QT_USE_QSTRINGBUILDER
 #include <QtCore/qstring.h>
 #endif
