@@ -54,6 +54,8 @@
 #include "qunicodetables_p.h"
 #include "qunicodetables.cpp"
 
+#include <algorithm>
+
 QT_BEGIN_NAMESPACE
 
 #define FLAG(x) (1 << (x))
@@ -113,7 +115,7 @@ QT_BEGIN_NAMESPACE
     operating on the full range of Unicode characters, not just for the ASCII
     range. They all return true if the character is a certain type of character;
     otherwise they return false. These classification functions are
-    isNull() (returns true if the character is '\\0'), isPrint()
+    isNull() (returns \c true if the character is '\\0'), isPrint()
     (true if the character is any sort of printable character,
     including whitespace), isPunct() (any sort of punctation),
     isMark() (Unicode Mark), isLetter() (a letter), isNumber() (any
@@ -121,7 +123,7 @@ QT_BEGIN_NAMESPACE
     isDigit() (decimal digits). All of these are wrappers around
     category() which return the Unicode-defined category of each
     character. Some of these also calculate the derived properties
-    (for example isSpace() returns true if the character is of category
+    (for example isSpace() returns \c true if the character is of category
     Separator_* or an exceptional code point from Other_Control category).
 
     QChar also provides direction(), which indicates the "natural"
@@ -262,6 +264,132 @@ QT_BEGIN_NAMESPACE
     \sa category()
 */
 
+/*!
+    \enum QChar::Script
+    \since 5.1
+
+    This enum type defines the Unicode script property values.
+
+    For details about the Unicode script property values see
+    \l{http://www.unicode.org/reports/tr24/}{Unicode Standard Annex #24}.
+
+    In order to conform to C/C++ naming conventions "Script_" is prepended
+    to the codes used in the Unicode Standard.
+
+    \value Script_Unknown    For unassigned, private-use, noncharacter, and surrogate code points.
+    \value Script_Inherited  For characters that may be used with multiple scripts
+                             and that inherit their script from the preceding characters.
+                             These include nonspacing marks, enclosing marks,
+                             and zero width joiner/non-joiner characters.
+    \value Script_Common     For characters that may be used with multiple scripts
+                             and that do not inherit their script from the preceding characters.
+
+    \value Script_Latin
+    \value Script_Greek
+    \value Script_Cyrillic
+    \value Script_Armenian
+    \value Script_Hebrew
+    \value Script_Arabic
+    \value Script_Syriac
+    \value Script_Thaana
+    \value Script_Devanagari
+    \value Script_Bengali
+    \value Script_Gurmukhi
+    \value Script_Gujarati
+    \value Script_Oriya
+    \value Script_Tamil
+    \value Script_Telugu
+    \value Script_Kannada
+    \value Script_Malayalam
+    \value Script_Sinhala
+    \value Script_Thai
+    \value Script_Lao
+    \value Script_Tibetan
+    \value Script_Myanmar
+    \value Script_Georgian
+    \value Script_Hangul
+    \value Script_Ethiopic
+    \value Script_Cherokee
+    \value Script_CanadianAboriginal
+    \value Script_Ogham
+    \value Script_Runic
+    \value Script_Khmer
+    \value Script_Mongolian
+    \value Script_Hiragana
+    \value Script_Katakana
+    \value Script_Bopomofo
+    \value Script_Han
+    \value Script_Yi
+    \value Script_OldItalic
+    \value Script_Gothic
+    \value Script_Deseret
+    \value Script_Tagalog
+    \value Script_Hanunoo
+    \value Script_Buhid
+    \value Script_Tagbanwa
+    \value Script_Coptic
+    \value Script_Limbu
+    \value Script_TaiLe
+    \value Script_LinearB
+    \value Script_Ugaritic
+    \value Script_Shavian
+    \value Script_Osmanya
+    \value Script_Cypriot
+    \value Script_Braille
+    \value Script_Buginese
+    \value Script_NewTaiLue
+    \value Script_Glagolitic
+    \value Script_Tifinagh
+    \value Script_SylotiNagri
+    \value Script_OldPersian
+    \value Script_Kharoshthi
+    \value Script_Balinese
+    \value Script_Cuneiform
+    \value Script_Phoenician
+    \value Script_PhagsPa
+    \value Script_Nko
+    \value Script_Sundanese
+    \value Script_Lepcha
+    \value Script_OlChiki
+    \value Script_Vai
+    \value Script_Saurashtra
+    \value Script_KayahLi
+    \value Script_Rejang
+    \value Script_Lycian
+    \value Script_Carian
+    \value Script_Lydian
+    \value Script_Cham
+    \value Script_TaiTham
+    \value Script_TaiViet
+    \value Script_Avestan
+    \value Script_EgyptianHieroglyphs
+    \value Script_Samaritan
+    \value Script_Lisu
+    \value Script_Bamum
+    \value Script_Javanese
+    \value Script_MeeteiMayek
+    \value Script_ImperialAramaic
+    \value Script_OldSouthArabian
+    \value Script_InscriptionalParthian
+    \value Script_InscriptionalPahlavi
+    \value Script_OldTurkic
+    \value Script_Kaithi
+    \value Script_Batak
+    \value Script_Brahmi
+    \value Script_Mandaic
+    \value Script_Chakma
+    \value Script_MeroiticCursive
+    \value Script_MeroiticHieroglyphs
+    \value Script_Miao
+    \value Script_Sharada
+    \value Script_SoraSompeng
+    \value Script_Takri
+
+    \omitvalue ScriptCount
+
+    \sa script()
+*/
+
 /*!
     \enum QChar::Direction
 
@@ -378,6 +506,7 @@ QT_BEGIN_NAMESPACE
     \value CarriageReturn
     \value Space
     \value Nbsp Non-breaking space.
+    \value SoftHyphen
     \value ReplacementCharacter The character shown when a font has no glyph
            for a certain codepoint. A special question mark character is often
            used. Codecs use this codepoint when input data cannot be
@@ -468,8 +597,8 @@ QT_BEGIN_NAMESPACE
 /*!
     \fn bool QChar::isNull() const
 
-    Returns true if the character is the Unicode character 0x0000
-    ('\\0'); otherwise returns false.
+    Returns \c true if the character is the Unicode character 0x0000
+    ('\\0'); otherwise returns \c false.
 */
 
 /*!
@@ -491,8 +620,8 @@ QT_BEGIN_NAMESPACE
 /*!
     \fn bool QChar::isPrint() const
 
-    Returns true if the character is a printable character; otherwise
-    returns false. This is any character not of category Other_*.
+    Returns \c true if the character is a printable character; otherwise
+    returns \c false. This is any character not of category Other_*.
 
     Note that this gives no indication of whether the character is
     available in a particular font.
@@ -502,8 +631,8 @@ QT_BEGIN_NAMESPACE
     \overload
     \since 5.0
 
-    Returns true if the UCS-4-encoded character specified by \a ucs4 is
-    a printable character; otherwise returns false.
+    Returns \c true if the UCS-4-encoded character specified by \a ucs4 is
+    a printable character; otherwise returns \c false.
     This is any character not of category Other_*.
 
     Note that this gives no indication of whether the character is
@@ -524,9 +653,9 @@ bool QChar::isPrint(uint ucs4)
 /*!
     \fn bool QChar::isSpace() const
 
-    Returns true if the character is a separator character
+    Returns \c true if the character is a separator character
     (Separator_* categories or certain code points from Other_Control category);
-    otherwise returns false.
+    otherwise returns \c false.
 */
 
 /*!
@@ -534,9 +663,9 @@ bool QChar::isPrint(uint ucs4)
     \overload
     \since 5.0
 
-    Returns true if the UCS-4-encoded character specified by \a ucs4 is
+    Returns \c true if the UCS-4-encoded character specified by \a ucs4 is
     a separator character (Separator_* categories or certain code points
-    from Other_Control category); otherwise returns false.
+    from Other_Control category); otherwise returns \c false.
 */
 
 /*!
@@ -555,8 +684,8 @@ bool QT_FASTCALL QChar::isSpace_helper(uint ucs4)
 /*!
     \fn bool QChar::isMark() const
 
-    Returns true if the character is a mark (Mark_* categories);
-    otherwise returns false.
+    Returns \c true if the character is a mark (Mark_* categories);
+    otherwise returns \c false.
 
     See QChar::Category for more information regarding marks.
 */
@@ -565,8 +694,8 @@ bool QT_FASTCALL QChar::isSpace_helper(uint ucs4)
     \overload
     \since 5.0
 
-    Returns true if the UCS-4-encoded character specified by \a ucs4 is
-    a mark (Mark_* categories); otherwise returns false.
+    Returns \c true if the UCS-4-encoded character specified by \a ucs4 is
+    a mark (Mark_* categories); otherwise returns \c false.
 */
 bool QChar::isMark(uint ucs4)
 {
@@ -581,16 +710,16 @@ bool QChar::isMark(uint ucs4)
 /*!
     \fn bool QChar::isPunct() const
 
-    Returns true if the character is a punctuation mark (Punctuation_*
-    categories); otherwise returns false.
+    Returns \c true if the character is a punctuation mark (Punctuation_*
+    categories); otherwise returns \c false.
 */
 
 /*!
     \overload
     \since 5.0
 
-    Returns true if the UCS-4-encoded character specified by \a ucs4 is
-    a punctuation mark (Punctuation_* categories); otherwise returns false.
+    Returns \c true if the UCS-4-encoded character specified by \a ucs4 is
+    a punctuation mark (Punctuation_* categories); otherwise returns \c false.
 */
 bool QChar::isPunct(uint ucs4)
 {
@@ -609,16 +738,16 @@ bool QChar::isPunct(uint ucs4)
 /*!
     \fn bool QChar::isSymbol() const
 
-    Returns true if the character is a symbol (Symbol_* categories);
-    otherwise returns false.
+    Returns \c true if the character is a symbol (Symbol_* categories);
+    otherwise returns \c false.
 */
 
 /*!
     \overload
     \since 5.0
 
-    Returns true if the UCS-4-encoded character specified by \a ucs4 is
-    a symbol (Symbol_* categories); otherwise returns false.
+    Returns \c true if the UCS-4-encoded character specified by \a ucs4 is
+    a symbol (Symbol_* categories); otherwise returns \c false.
 */
 bool QChar::isSymbol(uint ucs4)
 {
@@ -634,8 +763,8 @@ bool QChar::isSymbol(uint ucs4)
 /*!
     \fn bool QChar::isLetter() const
 
-    Returns true if the character is a letter (Letter_* categories);
-    otherwise returns false.
+    Returns \c true if the character is a letter (Letter_* categories);
+    otherwise returns \c false.
 */
 
 /*!
@@ -643,8 +772,8 @@ bool QChar::isSymbol(uint ucs4)
     \overload
     \since 5.0
 
-    Returns true if the UCS-4-encoded character specified by \a ucs4 is
-    a letter (Letter_* categories); otherwise returns false.
+    Returns \c true if the UCS-4-encoded character specified by \a ucs4 is
+    a letter (Letter_* categories); otherwise returns \c false.
 */
 
 /*!
@@ -665,8 +794,8 @@ bool QT_FASTCALL QChar::isLetter_helper(uint ucs4)
 /*!
     \fn bool QChar::isNumber() const
 
-    Returns true if the character is a number (Number_* categories,
-    not just 0-9); otherwise returns false.
+    Returns \c true if the character is a number (Number_* categories,
+    not just 0-9); otherwise returns \c false.
 
     \sa isDigit()
 */
@@ -676,8 +805,8 @@ bool QT_FASTCALL QChar::isLetter_helper(uint ucs4)
     \overload
     \since 5.0
 
-    Returns true if the UCS-4-encoded character specified by \a ucs4 is
-    a number (Number_* categories, not just 0-9); otherwise returns false.
+    Returns \c true if the UCS-4-encoded character specified by \a ucs4 is
+    a number (Number_* categories, not just 0-9); otherwise returns \c false.
 
     \sa isDigit()
 */
@@ -698,8 +827,8 @@ bool QT_FASTCALL QChar::isNumber_helper(uint ucs4)
 /*!
     \fn bool QChar::isLetterOrNumber() const
 
-    Returns true if the character is a letter or number (Letter_* or
-    Number_* categories); otherwise returns false.
+    Returns \c true if the character is a letter or number (Letter_* or
+    Number_* categories); otherwise returns \c false.
 */
 
 /*!
@@ -707,8 +836,8 @@ bool QT_FASTCALL QChar::isNumber_helper(uint ucs4)
     \overload
     \since 5.0
 
-    Returns true if the UCS-4-encoded character specified by \a ucs4 is
-    a letter or number (Letter_* or Number_* categories); otherwise returns false.
+    Returns \c true if the UCS-4-encoded character specified by \a ucs4 is
+    a letter or number (Letter_* or Number_* categories); otherwise returns \c false.
 */
 
 /*!
@@ -732,8 +861,8 @@ bool QT_FASTCALL QChar::isLetterOrNumber_helper(uint ucs4)
 /*!
     \fn bool QChar::isDigit() const
 
-    Returns true if the character is a decimal digit
-    (Number_DecimalDigit); otherwise returns false.
+    Returns \c true if the character is a decimal digit
+    (Number_DecimalDigit); otherwise returns \c false.
 
     \sa isNumber()
 */
@@ -743,8 +872,8 @@ bool QT_FASTCALL QChar::isLetterOrNumber_helper(uint ucs4)
     \overload
     \since 5.0
 
-    Returns true if the UCS-4-encoded character specified by \a ucs4 is
-    a decimal digit (Number_DecimalDigit); otherwise returns false.
+    Returns \c true if the UCS-4-encoded character specified by \a ucs4 is
+    a decimal digit (Number_DecimalDigit); otherwise returns \c false.
 
     \sa isNumber()
 */
@@ -753,7 +882,7 @@ bool QT_FASTCALL QChar::isLetterOrNumber_helper(uint ucs4)
     \fn bool QChar::isNonCharacter() const
     \since 5.0
 
-    Returns true if the QChar is a non-character; false otherwise.
+    Returns \c true if the QChar is a non-character; false otherwise.
 
     Unicode has a certain number of code points that are classified
     as "non-characters:" that is, they can be used for internal purposes
@@ -765,14 +894,14 @@ bool QT_FASTCALL QChar::isLetterOrNumber_helper(uint ucs4)
 /*!
     \fn bool QChar::isHighSurrogate() const
 
-    Returns true if the QChar is the high part of a UTF16 surrogate
+    Returns \c true if the QChar is the high part of a UTF16 surrogate
     (for example if its code point is in range [0xd800..0xdbff]); false otherwise.
 */
 
 /*!
     \fn bool QChar::isLowSurrogate() const
 
-    Returns true if the QChar is the low part of a UTF16 surrogate
+    Returns \c true if the QChar is the low part of a UTF16 surrogate
     (for example if its code point is in range [0xdc00..0xdfff]); false otherwise.
 */
 
@@ -780,7 +909,7 @@ bool QT_FASTCALL QChar::isLetterOrNumber_helper(uint ucs4)
     \fn bool QChar::isSurrogate() const
     \since 5.0
 
-    Returns true if the QChar contains a code point that is in either
+    Returns \c true if the QChar contains a code point that is in either
     the high or the low part of the UTF-16 surrogate range
     (for example if its code point is in range [0xd800..0xdfff]); false otherwise.
 */
@@ -790,7 +919,7 @@ bool QT_FASTCALL QChar::isLetterOrNumber_helper(uint ucs4)
     \overload
     \since 5.0
 
-    Returns true if the UCS-4-encoded character specified by \a ucs4
+    Returns \c true if the UCS-4-encoded character specified by \a ucs4
     is a non-character; false otherwise.
 
     Unicode has a certain number of code points that are classified
@@ -804,7 +933,7 @@ bool QT_FASTCALL QChar::isLetterOrNumber_helper(uint ucs4)
     \fn static bool QChar::isHighSurrogate(uint ucs4)
     \overload
 
-    Returns true if the UCS-4-encoded character specified by \a ucs4
+    Returns \c true if the UCS-4-encoded character specified by \a ucs4
     is the high part of a UTF16 surrogate
     (for example if its code point is in range [0xd800..0xdbff]); false otherwise.
 */
@@ -813,7 +942,7 @@ bool QT_FASTCALL QChar::isLetterOrNumber_helper(uint ucs4)
     \fn static bool QChar::isLowSurrogate(uint ucs4)
     \overload
 
-    Returns true if the UCS-4-encoded character specified by \a ucs4
+    Returns \c true if the UCS-4-encoded character specified by \a ucs4
     is the low part of a UTF16 surrogate
     (for example if its code point is in range [0xdc00..0xdfff]); false otherwise.
 */
@@ -823,7 +952,7 @@ bool QT_FASTCALL QChar::isLetterOrNumber_helper(uint ucs4)
     \overload
     \since 5.0
 
-    Returns true if the UCS-4-encoded character specified by \a ucs4
+    Returns \c true if the UCS-4-encoded character specified by \a ucs4
     contains a code point that is in either the high or the low part of the
     UTF-16 surrogate range (for example if its code point is in range [0xd800..0xdfff]);
     false otherwise.
@@ -832,7 +961,7 @@ bool QT_FASTCALL QChar::isLetterOrNumber_helper(uint ucs4)
 /*!
     \fn static bool QChar::requiresSurrogates(uint ucs4)
 
-    Returns true if the UCS-4-encoded character specified by \a ucs4
+    Returns \c true if the UCS-4-encoded character specified by \a ucs4
     can be split into the high and low parts of a UTF16 surrogate
     (for example if its code point is greater than or equals to 0x10000);
     false otherwise.
@@ -940,8 +1069,8 @@ QChar::Joining QChar::joining(uint ucs4)
 /*!
     \fn bool QChar::hasMirrored() const
 
-    Returns true if the character should be reversed if the text
-    direction is reversed; otherwise returns false.
+    Returns \c true if the character should be reversed if the text
+    direction is reversed; otherwise returns \c false.
 
     A bit faster equivalent of (ch.mirroredChar() != ch).
 
@@ -952,8 +1081,8 @@ QChar::Joining QChar::joining(uint ucs4)
     \overload
     \since 5.0
 
-    Returns true if the UCS-4-encoded character specified by \a ucs4
-    should be reversed if the text direction is reversed; otherwise returns false.
+    Returns \c true if the UCS-4-encoded character specified by \a ucs4
+    should be reversed if the text direction is reversed; otherwise returns \c false.
 
     A bit faster equivalent of (QChar::mirroredChar(ucs4) != ucs4).
 
@@ -969,7 +1098,7 @@ bool QChar::hasMirrored(uint ucs4)
 /*!
     \fn bool QChar::isLower() const
 
-    Returns true if the character is a lowercase letter, for example
+    Returns \c true if the character is a lowercase letter, for example
     category() is Letter_Lowercase.
 
     \sa isUpper(), toLower(), toUpper()
@@ -980,7 +1109,7 @@ bool QChar::hasMirrored(uint ucs4)
     \overload
     \since 5.0
 
-    Returns true if the UCS-4-encoded character specified by \a ucs4
+    Returns \c true if the UCS-4-encoded character specified by \a ucs4
     is a lowercase letter, for example category() is Letter_Lowercase.
 
     \sa isUpper(), toLower(), toUpper()
@@ -989,7 +1118,7 @@ bool QChar::hasMirrored(uint ucs4)
 /*!
     \fn bool QChar::isUpper() const
 
-    Returns true if the character is an uppercase letter, for example
+    Returns \c true if the character is an uppercase letter, for example
     category() is Letter_Uppercase.
 
     \sa isLower(), toUpper(), toLower()
@@ -1000,7 +1129,7 @@ bool QChar::hasMirrored(uint ucs4)
     \overload
     \since 5.0
 
-    Returns true if the UCS-4-encoded character specified by \a ucs4
+    Returns \c true if the UCS-4-encoded character specified by \a ucs4
     is an uppercase letter, for example category() is Letter_Uppercase.
 
     \sa isLower(), toUpper(), toLower()
@@ -1009,7 +1138,7 @@ bool QChar::hasMirrored(uint ucs4)
 /*!
     \fn bool QChar::isTitleCase() const
 
-    Returns true if the character is a titlecase letter, for example
+    Returns \c true if the character is a titlecase letter, for example
     category() is Letter_Titlecase.
 
     \sa isLower(), toUpper(), toLower(), toTitleCase()
@@ -1020,7 +1149,7 @@ bool QChar::hasMirrored(uint ucs4)
     \overload
     \since 5.0
 
-    Returns true if the UCS-4-encoded character specified by \a ucs4
+    Returns \c true if the UCS-4-encoded character specified by \a ucs4
     is a titlecase letter, for example category() is Letter_Titlecase.
 
     \sa isLower(), toUpper(), toLower(), toTitleCase()
@@ -1158,6 +1287,27 @@ unsigned char QChar::combiningClass(uint ucs4)
     return (unsigned char) qGetProp(ucs4)->combiningClass;
 }
 
+/*!
+    \fn QChar::Script QChar::script() const
+    \since 5.1
+
+    Returns the Unicode script property value for this character.
+*/
+
+/*!
+    \overload
+    \since 5.1
+
+    Returns the Unicode script property value for the character specified in
+    its UCS-4-encoded form as \a ucs4.
+*/
+QChar::Script QChar::script(uint ucs4)
+{
+    if (ucs4 > LastValidCodePoint)
+        return QChar::Script_Unknown;
+    return (QChar::Script) qGetProp(ucs4)->script;
+}
+
 /*!
     \fn QChar::UnicodeVersion QChar::unicodeVersion() const
 
@@ -1436,8 +1586,8 @@ QDataStream &operator>>(QDataStream &in, QChar &chr)
 
     \relates QChar
 
-    Returns true if \a c1 and \a c2 are the same Unicode character;
-    otherwise returns false.
+    Returns \c true if \a c1 and \a c2 are the same Unicode character;
+    otherwise returns \c false.
 */
 
 /*!
@@ -1445,8 +1595,8 @@ QDataStream &operator>>(QDataStream &in, QChar &chr)
 
     \relates QChar
 
-    Returns true if \a c1 and \a c2 are not the same Unicode
-    character; otherwise returns false.
+    Returns \c true if \a c1 and \a c2 are not the same Unicode
+    character; otherwise returns \c false.
 */
 
 /*!
@@ -1454,8 +1604,8 @@ QDataStream &operator>>(QDataStream &in, QChar &chr)
 
     \relates QChar
 
-    Returns true if the numeric Unicode value of \a c1 is less than
-    or equal to that of \a c2; otherwise returns false.
+    Returns \c true if the numeric Unicode value of \a c1 is less than
+    or equal to that of \a c2; otherwise returns \c false.
 */
 
 /*!
@@ -1463,8 +1613,8 @@ QDataStream &operator>>(QDataStream &in, QChar &chr)
 
     \relates QChar
 
-    Returns true if the numeric Unicode value of \a c1 is greater than
-    or equal to that of \a c2; otherwise returns false.
+    Returns \c true if the numeric Unicode value of \a c1 is greater than
+    or equal to that of \a c2; otherwise returns \c false.
 */
 
 /*!
@@ -1472,8 +1622,8 @@ QDataStream &operator>>(QDataStream &in, QChar &chr)
 
     \relates QChar
 
-    Returns true if the numeric Unicode value of \a c1 is less than
-    that of \a c2; otherwise returns false.
+    Returns \c true if the numeric Unicode value of \a c1 is less than
+    that of \a c2; otherwise returns \c false.
 */
 
 /*!
@@ -1481,8 +1631,8 @@ QDataStream &operator>>(QDataStream &in, QChar &chr)
 
     \relates QChar
 
-    Returns true if the numeric Unicode value of \a c1 is greater than
-    that of \a c2; otherwise returns false.
+    Returns \c true if the numeric Unicode value of \a c1 is greater than
+    that of \a c2; otherwise returns \c false.
 */
 
 
@@ -1530,6 +1680,8 @@ struct UCS2Pair {
     ushort u2;
 };
 
+inline bool operator<(const UCS2Pair &ligature1, const UCS2Pair &ligature2)
+{ return ligature1.u1 < ligature2.u1; }
 inline bool operator<(ushort u1, const UCS2Pair &ligature)
 { return u1 < ligature.u1; }
 inline bool operator<(const UCS2Pair &ligature, ushort u1)
@@ -1540,6 +1692,8 @@ struct UCS2SurrogatePair {
     UCS2Pair p2;
 };
 
+inline bool operator<(const UCS2SurrogatePair &ligature1, const UCS2SurrogatePair &ligature2)
+{ return QChar::surrogateToUcs4(ligature1.p1.u1, ligature1.p1.u2) < QChar::surrogateToUcs4(ligature2.p1.u1, ligature2.p1.u2); }
 inline bool operator<(uint u1, const UCS2SurrogatePair &ligature)
 { return u1 < QChar::surrogateToUcs4(ligature.p1.u1, ligature.p1.u2); }
 inline bool operator<(const UCS2SurrogatePair &ligature, uint u1)
@@ -1572,13 +1726,13 @@ static uint inline ligatureHelper(uint u1, uint u2)
     ushort length = *ligatures++;
     if (QChar::requiresSurrogates(u1)) {
         const UCS2SurrogatePair *data = reinterpret_cast<const UCS2SurrogatePair *>(ligatures);
-        const UCS2SurrogatePair *r = qBinaryFind(data, data + length, u1);
-        if (r != data + length)
+        const UCS2SurrogatePair *r = std::lower_bound(data, data + length, u1);
+        if (r != data + length && QChar::surrogateToUcs4(r->p1.u1, r->p1.u2) == u1)
             return QChar::surrogateToUcs4(r->p2.u1, r->p2.u2);
     } else {
         const UCS2Pair *data = reinterpret_cast<const UCS2Pair *>(ligatures);
-        const UCS2Pair *r = qBinaryFind(data, data + length, ushort(u1));
-        if (r != data + length)
+        const UCS2Pair *r = std::lower_bound(data, data + length, ushort(u1));
+        if (r != data + length && r->u1 == ushort(u1))
             return r->u2;
     }
 
@@ -1592,10 +1746,10 @@ static void composeHelper(QString *str, QChar::UnicodeVersion version, int from)
     if (from < 0 || s.length() - from < 2)
         return;
 
-    int starter = 0; // starter position
     uint stcode = 0; // starter code point
-    int next = -1;
-    int lastCombining = 0;
+    int starter = -1; // starter position
+    int next = -1; // to prevent i == next
+    int lastCombining = 255; // to prevent combining > lastCombining
 
     int pos = from;
     while (pos < s.length()) {
@@ -1619,8 +1773,7 @@ static void composeHelper(QString *str, QChar::UnicodeVersion version, int from)
         }
 
         int combining = p->combiningClass;
-        if (i == next || combining > lastCombining) {
-            Q_ASSERT(starter >= from);
+        if ((i == next || combining > lastCombining) && starter >= from) {
             // allowed to form ligature with S
             uint ligature = ligatureHelper(stcode, uc);
             if (ligature) {
@@ -1738,4 +1891,65 @@ static void canonicalOrderHelper(QString *str, QChar::UnicodeVersion version, in
     }
 }
 
+// returns true if the text is in a desired Normalization Form already; false otherwise.
+// sets lastStable to the position of the last stable code point
+static bool normalizationQuickCheckHelper(QString *str, QString::NormalizationForm mode, int from, int *lastStable)
+{
+    Q_STATIC_ASSERT(QString::NormalizationForm_D == 0);
+    Q_STATIC_ASSERT(QString::NormalizationForm_C == 1);
+    Q_STATIC_ASSERT(QString::NormalizationForm_KD == 2);
+    Q_STATIC_ASSERT(QString::NormalizationForm_KC == 3);
+
+    enum { NFQC_YES = 0, NFQC_NO = 1, NFQC_MAYBE = 3 };
+
+    const ushort *string = reinterpret_cast<const ushort *>(str->constData());
+    int length = str->length();
+
+    // this avoids one out of bounds check in the loop
+    while (length > from && QChar::isHighSurrogate(string[length - 1]))
+        --length;
+
+    uchar lastCombining = 0;
+    for (int i = from; i < length; ++i) {
+        int pos = i;
+        uint uc = string[i];
+        if (uc < 0x80) {
+            // ASCII characters are stable code points
+            lastCombining = 0;
+            *lastStable = pos;
+            continue;
+        }
+
+        if (QChar::isHighSurrogate(uc)) {
+            ushort low = string[i + 1];
+            if (!QChar::isLowSurrogate(low)) {
+                // treat surrogate like stable code point
+                lastCombining = 0;
+                *lastStable = pos;
+                continue;
+            }
+            ++i;
+            uc = QChar::surrogateToUcs4(uc, low);
+        }
+
+        const QUnicodeTables::Properties *p = qGetProp(uc);
+
+        if (p->combiningClass < lastCombining && p->combiningClass > 0)
+            return false;
+
+        const uchar check = (p->nfQuickCheck >> (mode << 1)) & 0x03;
+        if (check != NFQC_YES)
+            return false; // ### can we quick check NFQC_MAYBE ?
+
+        lastCombining = p->combiningClass;
+        if (lastCombining == 0)
+            *lastStable = pos;
+    }
+
+    if (length != str->length()) // low surrogate parts at the end of text
+        *lastStable = str->length() - 1;
+
+    return true;
+}
+
 QT_END_NAMESPACE
