@@ -55,8 +55,6 @@
 
 #include <QtCore/qatomic.h>
 
-QT_BEGIN_HEADER
-
 QT_BEGIN_NAMESPACE
 
 
@@ -288,6 +286,4 @@ inline void QFreeList<T, ConstantsType>::release(int id)
 
 QT_END_NAMESPACE
 
-QT_END_HEADER
-
 #endif // QFREELIST_P_H
