@@ -395,7 +395,7 @@ void QMapDataBase::freeData(QMapDataBase *d)
     differences are:
 
     \list
-    \li QHash provides faster lookups than QMap. (See \l{Algorithmic
+    \li QHash provides average faster lookups than QMap. (See \l{Algorithmic
        Complexity} for details.)
     \li When iterating over a QHash, the items are arbitrarily ordered.
        With QMap, the items are always sorted by key.
@@ -516,6 +516,15 @@ void QMapDataBase::freeData(QMapDataBase *d)
     \sa clear()
 */
 
+/*!
+    \fn QMap::QMap(QMap<Key, T> &&other)
+
+    Move-constructs a QMap instance, making it point at the same
+    object that \a other was pointing to.
+
+    \since 5.2
+*/
+
 /*! \fn QMap::QMap(const QMap<Key, T> &other)
 
     Constructs a copy of \a other.
@@ -538,6 +547,16 @@ void QMapDataBase::freeData(QMapDataBase *d)
     \sa toStdMap()
 */
 
+/*! \fn QMap::QMap(std::initializer_list<std::pair<Key,T> > list)
+    \since 5.1
+
+    Constructs a map with a copy of each of the elements in the
+    initializer list \a list.
+
+    This function is only available if the program is being
+    compiled in C++11 mode.
+*/
+
 /*! \fn std::map<Key, T> QMap::toStdMap() const
 
     Returns an STL map equivalent to this QMap.
@@ -557,6 +576,14 @@ void QMapDataBase::freeData(QMapDataBase *d)
     Assigns \a other to this map and returns a reference to this map.
 */
 
+/*!
+    \fn QMap<Key, T> &QMap::operator=(QMap<Key, T> &&other)
+
+    Move-assigns \a other to this QMap instance.
+
+    \since 5.2
+*/
+
 /*! \fn void QMap::swap(QMap<Key, T> &other)
     \since 4.8
 
@@ -573,7 +600,7 @@ void QMapDataBase::freeData(QMapDataBase *d)
 
 /*! \fn bool QMap::operator==(const QMap<Key, T> &other) const
 
-    Returns true if \a other is equal to this map; otherwise returns
+    Returns \c true if \a other is equal to this map; otherwise returns
     false.
 
     Two maps are considered equal if they contain the same (key,
@@ -587,8 +614,8 @@ void QMapDataBase::freeData(QMapDataBase *d)
 
 /*! \fn bool QMap::operator!=(const QMap<Key, T> &other) const
 
-    Returns true if \a other is not equal to this map; otherwise
-    returns false.
+    Returns \c true if \a other is not equal to this map; otherwise
+    returns \c false.
 
     Two maps are considered equal if they contain the same (key,
     value) pairs.
@@ -609,7 +636,7 @@ void QMapDataBase::freeData(QMapDataBase *d)
 /*!
     \fn bool QMap::isEmpty() const
 
-    Returns true if the map contains no items; otherwise returns
+    Returns \c true if the map contains no items; otherwise returns
     false.
 
     \sa size()
@@ -629,8 +656,8 @@ void QMapDataBase::freeData(QMapDataBase *d)
 
     \internal
 
-    Returns true if the map's internal data isn't shared with any
-    other map object; otherwise returns false.
+    Returns \c true if the map's internal data isn't shared with any
+    other map object; otherwise returns \c false.
 
     \sa detach()
 */
@@ -679,8 +706,8 @@ void QMapDataBase::freeData(QMapDataBase *d)
 
 /*! \fn bool QMap::contains(const Key &key) const
 
-    Returns true if the map contains an item with key \a key;
-    otherwise returns false.
+    Returns \c true if the map contains an item with key \a key;
+    otherwise returns \c false.
 
     \sa count(), QMultiMap::contains()
 */
@@ -869,6 +896,62 @@ void QMapDataBase::freeData(QMapDataBase *d)
     \sa constBegin(), end()
 */
 
+/*! \fn const Key &QMap::firstKey() const
+    \since 5.2
+
+    Returns a reference to the smallest key in the map.
+    This function assumes that the map is not empty.
+
+    This executes in \l{constant time}.
+
+    \sa lastKey(), first(), isEmpty()
+*/
+
+/*! \fn const Key &QMap::lastKey() const
+    \since 5.2
+
+    Returns a reference to the largest key in the map.
+    This function assumes that the map is not empty.
+
+    This executes in \l{logarithmic time}.
+
+    \sa firstKey(), last(), isEmpty()
+*/
+
+/*! \fn T &QMap::first()
+    \since 5.2
+
+    Returns a reference to the first value in the map, that is the value mapped
+    to the smallest key. This function assumes that the map is not empty.
+
+    When unshared (or const version is called), this executes in \l{constant time}.
+
+    \sa last(), firstKey(), isEmpty()
+*/
+
+/*! \fn const T &QMap::first() const
+    \since 5.2
+
+    \overload
+*/
+
+/*! \fn T &QMap::last()
+    \since 5.2
+
+    Returns a reference to the last value in the map, that is the value mapped
+    to the largest key. This function assumes that the map is not empty.
+
+    When unshared (or const version is called), this executes in \l{logarithmic time}.
+
+    \sa first(), lastKey(), isEmpty()
+*/
+
+/*! \fn const T &QMap::last() const
+    \since 5.2
+
+    \overload
+*/
+
 /*! \fn QMap::iterator QMap::erase(iterator pos)
 
     Removes the (key, value) pair pointed to by the iterator \a pos
@@ -971,6 +1054,35 @@ void QMapDataBase::freeData(QMapDataBase *d)
     \sa insertMulti()
 */
 
+/*! \fn QMap::iterator QMap::insert(const_iterator pos, const Key &key, const T &value)
+    \overload
+    \since 5.1
+    Inserts a new item with the key \a key and value \a value and with hint \a pos
+    suggesting where to do the insert.
+
+    If constBegin() is used as hint it indicates that the \a key is less than any key in the map
+    while constEnd() suggests that the \a key is (strictly) larger than any key in the map.
+    Otherwise the hint should meet the condition (\a pos - 1).key() < \a key <= pos.key().
+    If the hint \a pos is wrong it is ignored and a regular insert is done.
+
+    If there is already an item with the key \a key, that item's value
+    is replaced with \a value.
+
+    If there are multiple items with the key \a key, then exactly one of them
+    is replaced with \a value.
+
+    If the hint is correct and the map is unshared, the insert executes in amortized \l{constant time}.
+
+    When creating a map from sorted data inserting the largest key first with constBegin()
+    is faster than inserting in sorted order with constEnd(), since constEnd() - 1 (which is needed
+    to check if the hint is valid) needs \l{logarithmic time}.
+
+    \b {Note:} Be careful with the hint. Providing an iterator from an older shared instance might
+    crash but there is also a risk that it will silently corrupt both the map and the \a pos map.
+
+    \sa insertMulti()
+*/
+
 /*! \fn QMap::iterator QMap::insertMulti(const Key &key, const T &value)
 
     Inserts a new item with the key \a key and a value of \a value.
@@ -983,6 +1095,26 @@ void QMapDataBase::freeData(QMapDataBase *d)
     \sa insert(), values()
 */
 
+/*! \fn QMap::iterator QMap::insertMulti(const_iterator pos, const Key &key, const T &value)
+    \overload
+    \since 5.1
+    Inserts a new item with the key \a key and value \a value and with hint \a pos
+    suggesting where to do the insert.
+
+    If constBegin() is used as hint it indicates that the \a key is less than any key in the map
+    while constEnd() suggests that the \a key is larger than any key in the map.
+    Otherwise the hint should meet the condition (\a pos - 1).key() < \a key <= pos.key().
+    If the hint \a pos is wrong it is ignored and a regular insertMulti is done.
+
+    If there is already an item with the same key in the map, this function will simply create a new one.
+
+    \b {Note:} Be careful with the hint. Providing an iterator from an older shared instance might
+    crash but there is also a risk that it will silently corrupt both the map and the \a pos map.
+
+    \sa insert()
+*/
+
+
 /*! \fn QMap<Key, T> &QMap::unite(const QMap<Key, T> &other)
 
     Inserts all the items in the \a other map into this map. If a
@@ -1100,6 +1232,11 @@ void QMapDataBase::freeData(QMapDataBase *d)
     items from the map, iterators that point to the removed items
     will become dangling iterators.
 
+    \warning Iterators on implicitly shared containers do not work
+    exactly like STL-iterators. You should avoid copying a container
+    while iterators are active on that container. For more information,
+    read \l{Implicit sharing iterator problem}.
+
     \sa QMap::const_iterator, QMutableMapIterator
 */
 
@@ -1188,8 +1325,8 @@ void QMapDataBase::freeData(QMapDataBase *d)
     \fn bool QMap::iterator::operator==(const iterator &other) const
     \fn bool QMap::iterator::operator==(const const_iterator &other) const
 
-    Returns true if \a other points to the same item as this
-    iterator; otherwise returns false.
+    Returns \c true if \a other points to the same item as this
+    iterator; otherwise returns \c false.
 
     \sa operator!=()
 */
@@ -1198,8 +1335,8 @@ void QMapDataBase::freeData(QMapDataBase *d)
     \fn bool QMap::iterator::operator!=(const iterator &other) const
     \fn bool QMap::iterator::operator!=(const const_iterator &other) const
 
-    Returns true if \a other points to a different item than this
-    iterator; otherwise returns false.
+    Returns \c true if \a other points to a different item than this
+    iterator; otherwise returns \c false.
 
     \sa operator==()
 */
@@ -1318,6 +1455,11 @@ void QMapDataBase::freeData(QMapDataBase *d)
     items from the map, iterators that point to the removed items
     will become dangling iterators.
 
+    \warning Iterators on implicitly shared containers do not work
+    exactly like STL-iterators. You should avoid copying a container
+    while iterators are active on that container. For more information,
+    read \l{Implicit sharing iterator problem}.
+
     \sa QMap::iterator, QMapIterator
 */
 
@@ -1400,16 +1542,16 @@ void QMapDataBase::freeData(QMapDataBase *d)
 
 /*! \fn bool QMap::const_iterator::operator==(const const_iterator &other) const
 
-    Returns true if \a other points to the same item as this
-    iterator; otherwise returns false.
+    Returns \c true if \a other points to the same item as this
+    iterator; otherwise returns \c false.
 
     \sa operator!=()
 */
 
 /*! \fn bool QMap::const_iterator::operator!=(const const_iterator &other) const
 
-    Returns true if \a other points to a different item than this
-    iterator; otherwise returns false.
+    Returns \c true if \a other points to a different item than this
+    iterator; otherwise returns \c false.
 
     \sa operator==()
 */
@@ -1576,6 +1718,16 @@ void QMapDataBase::freeData(QMapDataBase *d)
     Constructs an empty map.
 */
 
+/*! \fn QMultiMap::QMultiMap(std::initializer_list<std::pair<Key,T> > list)
+    \since 5.1
+
+    Constructs a multi map with a copy of each of the elements in the
+    initializer list \a list.
+
+    This function is only available if the program is being
+    compiled in C++11 mode.
+*/
+
 /*! \fn QMultiMap::QMultiMap(const QMap<Key, T> &other)
 
     Constructs a copy of \a other (which can be a QMap or a
@@ -1609,6 +1761,23 @@ void QMapDataBase::freeData(QMapDataBase *d)
     \sa replace()
 */
 
+/*! \fn QMultiMap::iterator QMultiMap::insert(QMap<Key, T>::const_iterator pos, const Key &key, const T &value)
+
+    \since 5.1
+    Inserts a new item with the key \a key and value \a value and with hint \a pos
+    suggesting where to do the insert.
+
+    If constBegin() is used as hint it indicates that the \a key is less than any key in the map
+    while constEnd() suggests that the \a key is larger than any key in the map.
+    Otherwise the hint should meet the condition (\a pos - 1).key() < \a key <= pos.key().
+    If the hint \a pos is wrong it is ignored and a regular insert is done.
+
+    If there is already an item with the same key in the map, this function will simply create a new one.
+
+    \b {Note:} Be careful with the hint. Providing an iterator from an older shared instance might
+    crash but there is also a risk that it will silently corrupt both the map and the \a pos map.
+*/
+
 /*! \fn QMultiMap &QMultiMap::operator+=(const QMultiMap &other)
 
     Inserts all the items in the \a other map into this map and
@@ -1630,8 +1799,8 @@ void QMapDataBase::freeData(QMapDataBase *d)
     \fn bool QMultiMap::contains(const Key &key, const T &value) const
     \since 4.3
 
-    Returns true if the map contains an item with key \a key and
-    value \a value; otherwise returns false.
+    Returns \c true if the map contains an item with key \a key and
+    value \a value; otherwise returns \c false.
 
     \sa QMap::contains()
 */
