@@ -54,7 +54,7 @@
 #      include <intrin.h>
 #    endif
 #  endif
-#elif defined(Q_OS_LINUX) && defined(__arm__)
+#elif defined(Q_OS_LINUX) && (defined(Q_PROCESSOR_ARM) || defined(QT_COMPILER_SUPPORTS_IWMMXT) || defined(QT_COMPILER_SUPPORTS_NEON))
 #include "private/qcore_unix_p.h"
 
 // the kernel header definitions for HWCAP_*
@@ -159,7 +159,7 @@ static int maxBasicCpuidSupported()
 #if defined(Q_CC_GNU)
     qintptr tmp1;
 
-# ifdef Q_PROCESSOR_X86_32
+# if Q_PROCESSOR_X86 < 5
     // check if the CPUID instruction is supported
     long cpuid_supported;
     asm ("pushf\n"
@@ -240,14 +240,14 @@ inline quint64 _xgetbv(__int64) { return 0; }
 #endif
 static void xgetbv(uint in, uint &eax, uint &edx)
 {
-#ifdef Q_OS_WIN
-    quint64 result = _xgetbv(in);
-    eax = result;
-    edx = result >> 32;
-#elif defined(Q_CC_GNU)
+#if defined(Q_CC_GNU)
     asm (".byte 0x0F, 0x01, 0xD0" // xgetbv instruction
         : "=a" (eax), "=d" (edx)
         : "c" (in));
+#elif defined(Q_OS_WIN)
+    quint64 result = _xgetbv(in);
+    eax = result;
+    edx = result >> 32;
 #endif
 }
 
