@@ -43,6 +43,8 @@
 #include <qset.h>
 #include <qregularexpression.h>
 
+#include <algorithm>
+
 QT_BEGIN_NAMESPACE
 
 /*! \typedef QStringListIterator
@@ -222,8 +224,8 @@ QT_BEGIN_NAMESPACE
     If \a cs is \l Qt::CaseSensitive (the default), the string comparison
     is case sensitive; otherwise the comparison is case insensitive.
 
-    Sorting is performed using Qt's qSort() algorithm,
-    which operates in \l{linear-logarithmic time}, i.e. O(\e{n} log \e{n}).
+    Sorting is performed using the STL's std::sort() algorithm,
+    which averages \l{linear-logarithmic time}, i.e. O(\e{n} log \e{n}).
 
     If you want to sort your strings in an arbitrary order, consider
     using the QMap class. For example, you could use a QMap<QString,
@@ -231,8 +233,6 @@ QT_BEGIN_NAMESPACE
     being lower-case versions of the strings, and the values being the
     strings), or a QMap<int, QString> to sort the strings by some
     integer index.
-
-    \sa qSort()
 */
 
 static inline bool caseInsensitiveLessThan(const QString &s1, const QString &s2)
@@ -243,9 +243,9 @@ static inline bool caseInsensitiveLessThan(const QString &s1, const QString &s2)
 void QtPrivate::QStringList_sort(QStringList *that, Qt::CaseSensitivity cs)
 {
     if (cs == Qt::CaseSensitive)
-        qSort(that->begin(), that->end());
+        std::sort(that->begin(), that->end());
     else
-        qSort(that->begin(), that->end(), caseInsensitiveLessThan);
+        std::sort(that->begin(), that->end(), caseInsensitiveLessThan);
 }
 
 
@@ -283,8 +283,8 @@ QStringList QtPrivate::QStringList_filter(const QStringList *that, const QString
 /*!
     \fn bool QStringList::contains(const QString &str, Qt::CaseSensitivity cs) const
 
-    Returns true if the list contains the string \a str; otherwise
-    returns false. The search is case insensitive if \a cs is
+    Returns \c true if the list contains the string \a str; otherwise
+    returns \c false. The search is case insensitive if \a cs is
     Qt::CaseInsensitive; the search is case sensitive by default.
 
     \sa indexOf(), lastIndexOf(), QString::contains()
@@ -320,7 +320,7 @@ QStringList QtPrivate::QStringList_filter(const QStringList *that, const QRegExp
 #endif
 
 #ifndef QT_BOOTSTRAPPED
-#ifndef QT_NO_REGEXP
+#ifndef QT_NO_REGULAREXPRESSION
 /*!
     \fn QStringList QStringList::filter(const QRegularExpression &re) const
     \overload
@@ -338,7 +338,7 @@ QStringList QtPrivate::QStringList_filter(const QStringList *that, const QRegula
     }
     return res;
 }
-#endif // QT_NO_REGEXP
+#endif // QT_NO_REGULAREXPRESSION
 #endif // QT_BOOTSTRAPPED
 
 /*!
@@ -395,7 +395,7 @@ void QtPrivate::QStringList_replaceInStrings(QStringList *that, const QRegExp &r
 #endif
 
 #ifndef QT_BOOTSTRAPPED
-#ifndef QT_NO_REGEXP
+#ifndef QT_NO_REGULAREXPRESSION
 /*!
     \fn QStringList &QStringList::replaceInStrings(const QRegularExpression &re, const QString &after)
     \overload
@@ -424,7 +424,7 @@ void QtPrivate::QStringList_replaceInStrings(QStringList *that, const QRegularEx
     for (int i = 0; i < that->size(); ++i)
         (*that)[i].replace(re, after);
 }
-#endif // QT_NO_REGEXP
+#endif // QT_NO_REGULAREXPRESSION
 #endif // QT_BOOTSTRAPPED
 
 /*!
@@ -619,7 +619,7 @@ int QtPrivate::QStringList_lastIndexOf(const QStringList *that, QRegExp &rx, int
 #endif
 
 #ifndef QT_BOOTSTRAPPED
-#ifndef QT_NO_REGEXP
+#ifndef QT_NO_REGULAREXPRESSION
 /*!
     \fn int QStringList::indexOf(const QRegularExpression &re, int from) const
     \overload
@@ -676,7 +676,7 @@ int QtPrivate::QStringList_lastIndexOf(const QStringList *that, const QRegularEx
     }
     return -1;
 }
-#endif // QT_NO_REGEXP
+#endif // QT_NO_REGULAREXPRESSION
 #endif // QT_BOOTSTRAPPED
 
 /*!
@@ -738,7 +738,8 @@ int QtPrivate::QStringList_removeDuplicates(QStringList *that)
 
     Construct a list from a std::initializer_list given by \a args.
 
-    This constructor is only enabled if the compiler supports C++0x
+    This constructor is only enabled if the compiler supports C++11 initializer
+    lists.
 */
 
 
