@@ -44,7 +44,7 @@
 #include "qunicodetables_p.h"
 #include "qvarlengtharray.h"
 
-#include <private/harfbuzz-shaper.h>
+#include "qharfbuzz_p.h"
 
 #define FLAG(x) (1 << (x))
 
@@ -603,28 +603,35 @@ Q_CORE_EXPORT void initCharAttributes(const ushort *string, int length,
     if (options & WhiteSpaces)
         getWhiteSpaces(string, length, attributes);
 
-    if (!items || numItems <= 0)
-        return;
     if (!qt_initcharattributes_default_algorithm_only) {
+        if (!items || numItems <= 0)
+            return;
+
         QVarLengthArray<HB_ScriptItem, 64> scriptItems;
         scriptItems.reserve(numItems);
         int start = 0;
+        HB_Script startScript = script_to_hbscript(items[start].script);
+        if (Q_UNLIKELY(startScript == HB_Script_Inherited))
+            startScript = HB_Script_Common;
         for (int i = start + 1; i < numItems; ++i) {
-            if (items[i].script == items[start].script)
+            HB_Script script = script_to_hbscript(items[i].script);
+            if (Q_LIKELY(script == startScript || script == HB_Script_Inherited))
                 continue;
+            Q_ASSERT(items[i].position > items[start].position);
             HB_ScriptItem item;
             item.pos = items[start].position;
             item.length = items[i].position - items[start].position;
-            item.script = (HB_Script)items[start].script;
+            item.script = startScript;
             item.bidiLevel = 0; // unused
             scriptItems.append(item);
             start = i;
+            startScript = script;
         }
         if (items[start].position + 1 < length) {
             HB_ScriptItem item;
             item.pos = items[start].position;
             item.length = length - items[start].position;
-            item.script = (HB_Script)items[start].script;
+            item.script = startScript;
             item.bidiLevel = 0; // unused
             scriptItems.append(item);
         }
@@ -635,6 +642,51 @@ Q_CORE_EXPORT void initCharAttributes(const ushort *string, int length,
     }
 }
 
+
+// ----------------------------------------------------------------------------
+//
+// The Unicode script property. See http://www.unicode.org/reports/tr24/ (some very old version)
+//
+// ----------------------------------------------------------------------------
+
+Q_CORE_EXPORT void initScripts(const ushort *string, int length, uchar *scripts)
+{
+    int sor = 0;
+    int eor = -1;
+    uchar script = QChar::Script_Common;
+    for (int i = 0; i < length; ++i) {
+        eor = i;
+        uint ucs4 = string[i];
+        if (QChar::isHighSurrogate(ucs4) && i + 1 < length) {
+            ushort low = string[i + 1];
+            if (QChar::isLowSurrogate(low)) {
+                ucs4 = QChar::surrogateToUcs4(ucs4, low);
+                ++i;
+            }
+        }
+
+        const QUnicodeTables::Properties *prop = QUnicodeTables::properties(ucs4);
+
+        if (Q_LIKELY(prop->script == script || prop->script == QChar::Script_Inherited))
+            continue;
+
+        // Never break between a combining mark (gc= Mc, Mn or Me) and its base character.
+        // Thus, a combining mark — whatever its script property value is — should inherit
+        // the script property value of its base character.
+        static const int test = (FLAG(QChar::Mark_NonSpacing) | FLAG(QChar::Mark_SpacingCombining) | FLAG(QChar::Mark_Enclosing));
+        if (Q_UNLIKELY(FLAG(prop->category) & test))
+            continue;
+
+        while (sor < eor)
+            scripts[sor++] = script;
+
+        script = prop->script;
+    }
+    eor = length;
+    while (sor < eor)
+        scripts[sor++] = script;
+}
+
 } // namespace QUnicodeTools
 
 QT_END_NAMESPACE
