@@ -56,7 +56,7 @@
 #include <initializer_list>
 #endif
 
-QT_BEGIN_HEADER
+#include <algorithm>
 
 QT_BEGIN_NAMESPACE
 
@@ -140,6 +140,10 @@ public:
     void replace(int i, const T &t);
     void remove(int i);
     void remove(int i, int n);
+    inline void removeFirst() { Q_ASSERT(!isEmpty()); erase(d->begin()); }
+    inline void removeLast();
+    inline T takeFirst() { Q_ASSERT(!isEmpty()); T r = first(); removeFirst(); return r; }
+    inline T takeLast()  { Q_ASSERT(!isEmpty()); T r = last(); removeLast(); return r; }
 
     QVector<T> &fill(const T &t, int size = -1);
 
@@ -148,6 +152,11 @@ public:
     bool contains(const T &t) const;
     int count(const T &t) const;
 
+    // QList compatibility
+    void removeAt(int i) { remove(i); }
+    int length() const { return size(); }
+    T takeAt(int i) { T t = at(i); remove(i); return t; }
+
     // STL-style
     typedef typename Data::iterator iterator;
     typedef typename Data::const_iterator const_iterator;
@@ -183,7 +192,7 @@ public:
     inline const T &last() const { Q_ASSERT(!isEmpty()); return *(end()-1); }
     inline bool startsWith(const T &t) const { return !isEmpty() && first() == t; }
     inline bool endsWith(const T &t) const { return !isEmpty() && last() == t; }
-    QVector<T> mid(int pos, int length = -1) const;
+    QVector<T> mid(int pos, int len = -1) const;
 
     T value(int i) const;
     T value(int i, const T &defaultValue) const;
@@ -200,8 +209,8 @@ public:
     typedef int size_type;
     inline void push_back(const T &t) { append(t); }
     inline void push_front(const T &t) { prepend(t); }
-    void pop_back() { Q_ASSERT(!isEmpty()); erase(d->end() - 1); }
-    void pop_front() { Q_ASSERT(!isEmpty()); erase(d->begin()); }
+    void pop_back() { removeLast(); }
+    void pop_front() { removeFirst(); }
     inline bool empty() const
     { return d->size == 0; }
     inline T& front() { return first(); }
@@ -225,18 +234,22 @@ public:
     static QVector<T> fromList(const QList<T> &list);
 
     static inline QVector<T> fromStdVector(const std::vector<T> &vector)
-    { QVector<T> tmp; tmp.reserve(int(vector.size())); qCopy(vector.begin(), vector.end(), std::back_inserter(tmp)); return tmp; }
+    { QVector<T> tmp; tmp.reserve(int(vector.size())); std::copy(vector.begin(), vector.end(), std::back_inserter(tmp)); return tmp; }
     inline std::vector<T> toStdVector() const
-    { std::vector<T> tmp; tmp.reserve(size()); qCopy(constBegin(), constEnd(), std::back_inserter(tmp)); return tmp; }
+    { std::vector<T> tmp; tmp.reserve(size()); std::copy(constBegin(), constEnd(), std::back_inserter(tmp)); return tmp; }
 private:
     friend class QRegion; // Optimization for QRegion::rects()
 
     void reallocData(const int size, const int alloc, QArrayData::AllocationOptions options = QArrayData::Default);
+    void reallocData(const int sz) { reallocData(sz, d->alloc); }
     void freeData(Data *d);
     void defaultConstruct(T *from, T *to);
     void copyConstruct(const T *srcFrom, const T *srcTo, T *dstFrom);
     void destruct(T *from, T *to);
-
+    bool isValidIterator(const iterator &i) const
+    {
+        return (i <= d->end()) && (d->begin() <= i);
+    }
     class AlignmentDummy { Data header; T array[1]; };
 };
 
@@ -396,7 +409,8 @@ QVector<T> &QVector<T>::operator=(const QVector<T> &v)
 template <typename T>
 QVector<T>::QVector(int asize)
 {
-    if (Q_LIKELY(asize)) {
+    Q_ASSERT_X(asize >= 0, "QVector::QVector", "Size must be greater than or equal to 0.");
+    if (Q_LIKELY(asize > 0)) {
         d = Data::allocate(asize);
         d->size = asize;
         defaultConstruct(d->begin(), d->end());
@@ -408,7 +422,8 @@ QVector<T>::QVector(int asize)
 template <typename T>
 QVector<T>::QVector(int asize, const T &t)
 {
-    if (asize) {
+    Q_ASSERT_X(asize >= 0, "QVector::QVector", "Size must be greater than or equal to 0.");
+    if (asize > 0) {
         d = Data::allocate(asize);
         d->size = asize;
         T* i = d->end();
@@ -491,7 +506,7 @@ void QVector<T>::reallocData(const int asize, const int aalloc, QArrayData::Allo
             }
             x->capacityReserved = d->capacityReserved;
         } else {
-            Q_ASSERT(d->alloc == aalloc); // resize, without changing allocation size
+            Q_ASSERT(int(d->alloc) == aalloc); // resize, without changing allocation size
             Q_ASSERT(isDetached());       // can be done only on detached d
             Q_ASSERT(x == d);             // in this case we do not need to allocate anything
             if (asize <= d->size) {
@@ -528,7 +543,7 @@ void QVector<T>::reallocData(const int asize, const int aalloc, QArrayData::Allo
 template<typename T>
 Q_OUTOFLINE_TEMPLATE T QVector<T>::value(int i) const
 {
-    if (i < 0 || i >= d->size) {
+    if (uint(i) >= uint(d->size)) {
         return T();
     }
     return d->begin()[i];
@@ -536,7 +551,7 @@ Q_OUTOFLINE_TEMPLATE T QVector<T>::value(int i) const
 template<typename T>
 Q_OUTOFLINE_TEMPLATE T QVector<T>::value(int i, const T &defaultValue) const
 {
-    return ((i < 0 || i >= d->size) ? defaultValue : d->begin()[i]);
+    return uint(i) >= uint(d->size) ? defaultValue : d->begin()[i];
 }
 
 template <typename T>
@@ -555,9 +570,26 @@ void QVector<T>::append(const T &t)
     ++d->size;
 }
 
+template <typename T>
+void QVector<T>::removeLast()
+{
+    Q_ASSERT(!isEmpty());
+    Q_ASSERT(d->alloc);
+
+    if (!d->ref.isShared()) {
+        --d->size;
+        if (QTypeInfo<T>::isComplex)
+            (d->data() + d->size)->~T();
+    } else {
+        reallocData(d->size - 1);
+    }
+}
+
 template <typename T>
 typename QVector<T>::iterator QVector<T>::insert(iterator before, size_type n, const T &t)
 {
+    Q_ASSERT_X(isValidIterator(before),  "QVector::insert", "The specified iterator argument 'before' is invalid");
+
     int offset = std::distance(d->begin(), before);
     if (n != 0) {
         const T copy(t);
@@ -591,6 +623,9 @@ typename QVector<T>::iterator QVector<T>::insert(iterator before, size_type n, c
 template <typename T>
 typename QVector<T>::iterator QVector<T>::erase(iterator abegin, iterator aend)
 {
+    Q_ASSERT_X(isValidIterator(abegin), "QVector::erase", "The specified iterator argument 'abegin' is invalid");
+    Q_ASSERT_X(isValidIterator(aend), "QVector::erase", "The specified iterator argument 'aend' is invalid");
+
     const int itemsToErase = aend - abegin;
 
     if (!itemsToErase)
@@ -604,6 +639,7 @@ typename QVector<T>::iterator QVector<T>::erase(iterator abegin, iterator aend)
 
     // FIXME we could do a proper realloc, which copy constructs only needed data.
     // FIXME we ara about to delete data maybe it is good time to shrink?
+    // FIXME the shrink is also an issue in removeLast, that is just a copy + reduce of this.
     if (d->alloc) {
         detach();
         abegin = d->begin() + itemsUntouched;
@@ -613,7 +649,7 @@ typename QVector<T>::iterator QVector<T>::erase(iterator abegin, iterator aend)
             iterator moveEnd = d->end();
             while (moveBegin != moveEnd) {
                 if (QTypeInfo<T>::isComplex)
-                    abegin->~T();
+                    static_cast<T *>(abegin)->~T();
                 new (abegin++) T(*moveBegin++);
             }
             if (abegin < d->end()) {
@@ -741,17 +777,17 @@ int QVector<T>::count(const T &t) const
 }
 
 template <typename T>
-Q_OUTOFLINE_TEMPLATE QVector<T> QVector<T>::mid(int pos, int length) const
+Q_OUTOFLINE_TEMPLATE QVector<T> QVector<T>::mid(int pos, int len) const
 {
-    if (length < 0)
-        length = size() - pos;
-    if (pos == 0 && length == size())
+    if (len < 0)
+        len = size() - pos;
+    if (pos == 0 && len == size())
         return *this;
-    if (pos + length > size())
-        length = size() - pos;
+    if (pos + len > size())
+        len = size() - pos;
     QVector<T> copy;
-    copy.reserve(length);
-    for (int i = pos; i < pos + length; ++i)
+    copy.reserve(len);
+    for (int i = pos; i < pos + len; ++i)
         copy += at(i);
     return copy;
 }
@@ -813,6 +849,4 @@ Q_TEMPLATE_EXTERN template class Q_CORE_EXPORT QVector<QPoint>;
 
 QT_END_NAMESPACE
 
-QT_END_HEADER
-
 #endif // QVECTOR_H
