@@ -288,7 +288,8 @@ QDBusAbstractInterface::QDBusAbstractInterface(QDBusAbstractInterfacePrivate &d,
     if (d.isValid &&
         d.connection.isConnected()
         && !d.service.isEmpty()
-        && !d.service.startsWith(QLatin1Char(':')))
+        && !d.service.startsWith(QLatin1Char(':'))
+        && d.connectionPrivate()->mode != QDBusConnectionPrivate::PeerMode)
         d_func()->connection.connect(QLatin1String(DBUS_SERVICE_DBUS), // service
                                      QString(), // path
                                      QLatin1String(DBUS_INTERFACE_DBUS), // interface
@@ -313,7 +314,8 @@ QDBusAbstractInterface::QDBusAbstractInterface(const QString &service, const QSt
     if (d_func()->isValid &&
         d_func()->connection.isConnected()
         && !service.isEmpty()
-        && !service.startsWith(QLatin1Char(':')))
+        && !service.startsWith(QLatin1Char(':'))
+        && d_func()->connectionPrivate()->mode != QDBusConnectionPrivate::PeerMode)
         d_func()->connection.connect(QLatin1String(DBUS_SERVICE_DBUS), // service
                                      QString(), // path
                                      QLatin1String(DBUS_INTERFACE_DBUS), // interface
@@ -331,7 +333,7 @@ QDBusAbstractInterface::~QDBusAbstractInterface()
 }
 
 /*!
-    Returns true if this is a valid reference to a remote object. It returns false if
+    Returns \c true if this is a valid reference to a remote object. It returns \c false if
     there was an error during the creation of this interface (for instance, if the remote
     application does not exist).
 
@@ -340,7 +342,13 @@ QDBusAbstractInterface::~QDBusAbstractInterface()
 */
 bool QDBusAbstractInterface::isValid() const
 {
-    return !d_func()->currentOwner.isEmpty();
+    Q_D(const QDBusAbstractInterface);
+    /* We don't retrieve the owner name for peer connections */
+    if (d->connectionPrivate() && d->connectionPrivate()->mode == QDBusConnectionPrivate::PeerMode) {
+        return d->isValid;
+    } else {
+        return !d->currentOwner.isEmpty();
+    }
 }
 
 /*!
@@ -509,11 +517,11 @@ QDBusPendingCall QDBusAbstractInterface::asyncCallWithArgumentList(const QString
     object \a receiver. If an error occurs, the \a errorMethod
     on object \a receiver is called instead.
 
-    This function returns true if the queueing succeeds. It does
+    This function returns \c true if the queueing succeeds. It does
     not indicate that the executed call succeeded. If it fails,
     the \a errorMethod is called. If the queueing failed, this
-    function returns false and no slot will be called.
- 
+    function returns \c false and no slot will be called.
+
     The \a returnMethod must have as its parameters the types returned
     by the function call. Optionally, it may have a QDBusMessage
     parameter as its last or only parameter.  The \a errorMethod must
@@ -559,7 +567,7 @@ bool QDBusAbstractInterface::callWithCallback(const QString &method,
     the remote function or any errors emitted by it are delivered
     to the \a slot slot on object \a receiver.
 
-    This function returns true if the queueing succeeded: it does
+    This function returns \c true if the queueing succeeded: it does
     not indicate that the call succeeded. If it failed, the slot
     will be called with an error message. lastError() will not be
     set under those circumstances.
@@ -609,9 +617,22 @@ void QDBusAbstractInterface::disconnectNotify(const QMetaMethod &signal)
         return;
 
     QDBusConnectionPrivate *conn = d->connectionPrivate();
-    if (conn)
-        conn->disconnectRelay(d->service, d->path, d->interface,
-                              this, signal);
+    if (conn && signal.isValid() && !isSignalConnected(signal))
+        return conn->disconnectRelay(d->service, d->path, d->interface,
+                                     this, signal);
+    if (!conn)
+        return;
+
+    // wildcard disconnecting, we need to figure out which of our signals are
+    // no longer connected to anything
+    const QMetaObject *mo = metaObject();
+    int midx = QObject::staticMetaObject.methodCount();
+    const int end = mo->methodCount();
+    for ( ; midx < end; ++midx) {
+        QMetaMethod mm = mo->method(midx);
+        if (mm.methodType() == QMetaMethod::Signal && !isSignalConnected(mm))
+            conn->disconnectRelay(d->service, d->path, d->interface, this, mm);
+    }
 }
 
 /*!
