@@ -57,7 +57,6 @@
 #include <QtCore/qvector.h>
 #endif
 
-QT_BEGIN_HEADER
 QT_BEGIN_NAMESPACE
 
 #ifndef QT_NO_DBUS
@@ -75,6 +74,5 @@ QT_BEGIN_NAMESPACE
 #endif // QT_NO_DBUS
 
 QT_END_NAMESPACE
-QT_END_HEADER
 
 #endif
