@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2012 Digia Plc and/or its subsidiary(-ies).
+** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
 ** This file is part of the QtDBus module of the Qt Toolkit.
@@ -620,7 +620,7 @@ bool QDBusMessage::isReplyRequired() const
 
 /*!
     Sets whether the message will be replied later (if \a enable is
-    true) or if an automatic reply should be generated by QtDBus
+    true) or if an automatic reply should be generated by Qt D-Bus
     (if \a enable is false).
 
     In D-Bus, all method calls must generate a reply to the caller, unless the
@@ -639,7 +639,7 @@ void QDBusMessage::setDelayedReply(bool enable) const
 
 /*!
     Returns the delayed reply flag, as set by setDelayedReply(). By default, this
-    flag is false, which means QtDBus will generate automatic replies
+    flag is false, which means Qt D-Bus will generate automatic replies
     when necessary.
 */
 bool QDBusMessage::isDelayedReply() const
@@ -674,7 +674,7 @@ void QDBusMessage::setAutoStartService(bool enable)
 
 /*!
     Returns the auto start flag, as set by setAutoStartService(). By default, this
-    flag is true, which means QtDBus will auto start a service, if it is
+    flag is true, which means Qt D-Bus will auto start a service, if it is
     not running already.
 
     \sa setAutoStartService()
@@ -744,8 +744,8 @@ QDBusMessage::MessageType QDBusMessage::type() const
     for errors, signals, and return values as well as calls whose
     return values are not necessary.
 
-    Returns true if the message was queued successfully;
-    otherwise returns false.
+    Returns \c true if the message was queued successfully;
+    otherwise returns \c false.
 
     \sa QDBusConnection::send()
 */
