@@ -75,9 +75,10 @@ private:
         QByteArray name;
         QVarLengthArray<int, 4> inputTypes;
         QVarLengthArray<int, 4> outputTypes;
+        QByteArray rawReturnType;
         int flags;
     };
-    
+
     struct Property {
         QByteArray typeName;
         QByteArray signature;
@@ -92,14 +93,14 @@ private:
     QMap<QByteArray, Method> signals_;
     QMap<QByteArray, Method> methods;
     QMap<QByteArray, Property> properties;
-    
+
     const QDBusIntrospection::Interface *data;
     QString interface;
 
     Type findType(const QByteArray &signature,
                   const QDBusIntrospection::Annotations &annotations,
                   const char *direction = "Out", int id = -1);
-    
+
     void parseMethods();
     void parseSignals();
     void parseProperties();
@@ -258,7 +259,7 @@ void QDBusMetaObjectGenerator::parseMethods()
             mm.inputTypes.append(type.id);
 
             mm.parameterNames.append(arg.name.toLatin1());
-            
+
             prototype.append(type.name);
             prototype.append(',');
         }
@@ -276,6 +277,9 @@ void QDBusMetaObjectGenerator::parseMethods()
 
             mm.outputTypes.append(type.id);
 
+            if (i == 0 && type.id == -1) {
+                mm.rawReturnType = type.name;
+            }
             if (i != 0) {
                 // non-const ref parameter
                 mm.parameterNames.append(arg.name.toLatin1());
@@ -331,7 +335,7 @@ void QDBusMetaObjectGenerator::parseSignals()
             mm.inputTypes.append(type.id);
 
             mm.parameterNames.append(arg.name.toLatin1());
-            
+
             prototype.append(type.name);
             prototype.append(',');
         }
@@ -344,7 +348,7 @@ void QDBusMetaObjectGenerator::parseSignals()
             prototype.append(')');
 
         // meta method flags
-        mm.flags = AccessProtected | MethodSignal | MethodScriptable;
+        mm.flags = AccessPublic | MethodSignal | MethodScriptable;
 
         // add
         signals_.insert(QMetaObject::normalizedSignature(prototype), mm);
@@ -361,7 +365,7 @@ void QDBusMetaObjectGenerator::parseProperties()
         Type type = findType(p.type.toLatin1(), p.annotations);
         if (type.id == QVariant::Invalid)
             continue;
-        
+
         QByteArray name = p.name.toLatin1();
         mp.signature = p.type.toLatin1();
         mp.type = type.id;
@@ -441,8 +445,7 @@ void QDBusMetaObjectGenerator::write(QDBusMetaObject *obj)
         data_size += 2 + mm.inputTypes.count() + mm.outputTypes.count();
     idata.resize(data_size + 1);
 
-    QMetaStringTable strings;
-    strings.enter(className.toLatin1());
+    QMetaStringTable strings(className.toLatin1());
 
     int offset = header->methodData;
     int parametersOffset = offset + header->methodCount * 5;
@@ -471,10 +474,14 @@ void QDBusMetaObjectGenerator::write(QDBusMetaObject *obj)
                 int type;
                 QByteArray typeName;
                 if (i < 0) { // Return type
-                    if (!mm.outputTypes.isEmpty())
+                    if (!mm.outputTypes.isEmpty()) {
                         type = mm.outputTypes.first();
-                    else
+                        if (type == -1) {
+                            type = IsUnresolvedType | strings.enter(mm.rawReturnType);
+                        }
+                    } else {
                         type = QMetaType::Void;
+                    }
                 } else if (i < mm.inputTypes.size()) {
                     type = mm.inputTypes.at(i);
                 } else {
@@ -564,7 +571,7 @@ void QDBusMetaObjectGenerator::writeWithoutXml(const QString &interface)
 
     char *stringdata = new char[name.length() + 1];
     stringdata[name.length()] = '\0';
-    
+
     d.data = reinterpret_cast<uint*>(header);
     d.relatedMetaObjects = 0;
     d.static_metacall = 0;
@@ -615,7 +622,7 @@ QDBusMetaObject *QDBusMetaObject::createMetaObject(const QString &interface, con
     if (we)
         return we;
     // still nothing?
-    
+
     if (parsed.isEmpty()) {
         // object didn't return introspection
         we = new QDBusMetaObject;
@@ -627,7 +634,7 @@ QDBusMetaObject *QDBusMetaObject::createMetaObject(const QString &interface, con
         // merge all interfaces
         it = parsed.constBegin();
         QDBusIntrospection::Interface merged = *it.value().constData();
- 
+
         for (++it; it != end; ++it) {
             merged.annotations.unite(it.value()->annotations);
             merged.methods.unite(it.value()->methods);
