@@ -47,8 +47,6 @@
 
 #ifndef QT_NO_DBUS
 
-QT_BEGIN_HEADER
-
 QT_BEGIN_NAMESPACE
 
 
@@ -93,7 +91,5 @@ int qDBusRegisterMetaType(
 
 QT_END_NAMESPACE
 
-QT_END_HEADER
-
 #endif // QT_NO_DBUS
 #endif
