@@ -3,7 +3,7 @@
 ** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
-** This file is part of the FOO module of the Qt Toolkit.
+** This file is part of the QtDBus module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
@@ -197,7 +197,7 @@ QDBusUnixFileDescriptor::~QDBusUnixFileDescriptor()
 */
 
 /*!
-    Returns true if this Unix file descriptor is valid. A valid Unix file
+    Returns \c true if this Unix file descriptor is valid. A valid Unix file
     descriptor is not -1.
 
     \sa fileDescriptor()
@@ -231,11 +231,11 @@ int QDBusUnixFileDescriptor::fileDescriptor() const
 // qdoc documentation is generated on Unix
 
 /*!
-    Returns true if Unix file descriptors are supported on this platform. In
-    other words, this function returns true if this is a Unix platform.
+    Returns \c true if Unix file descriptors are supported on this platform. In
+    other words, this function returns \c true if this is a Unix platform.
 
     Note that QDBusUnixFileDescriptor continues to operate even if this
-    function returns false. The only difference is that the
+    function returns \c false. The only difference is that the
     QDBusUnixFileDescriptor objects will always be in the isValid() == false
     state and fileDescriptor() will always return -1. The class will not
     consume any operating system resources.
