@@ -53,8 +53,6 @@
 #include <QtCore/qstringlist.h>
 #endif
 
-QT_BEGIN_HEADER
-
 QT_BEGIN_NAMESPACE
 
 
@@ -112,6 +110,9 @@ public:
         Format_RGB888,
         Format_RGB444,
         Format_ARGB4444_Premultiplied,
+        Format_RGBX8888,
+        Format_RGBA8888,
+        Format_RGBA8888_Premultiplied,
 #if 0
         // reserved for future use
         Format_RGB15,
@@ -141,6 +142,11 @@ public:
     explicit QImage(const QString &fileName, const char *format = 0);
 
     QImage(const QImage &);
+#ifdef Q_COMPILER_RVALUE_REFS
+    inline QImage(QImage &&other)
+        : QPaintDevice(), d(0)
+    { qSwap(d, other.d); }
+#endif
     ~QImage();
 
     QImage &operator=(const QImage &);
@@ -321,6 +327,7 @@ inline void QImage::setPixel(const QPoint &pt, uint index_or_rgb) { setPixel(pt.
 # pragma GCC diagnostic push
 # pragma GCC diagnostic ignored "-Wdeprecated-declarations"
 #elif defined(Q_CC_MSVC)
+# pragma warning(push)
 # pragma warning(disable: 4996)
 #endif
 
@@ -397,7 +404,7 @@ inline void QImage::setText(const char* key, const char* lang, const QString &s)
 #if defined(Q_CC_GNU) && !defined(Q_CC_INTEL) && (__GNUC__ * 100 + __GNUC_MINOR__ >= 406)
 # pragma GCC diagnostic pop
 #elif defined(Q_CC_MSVC)
-# pragma warning(default: 4996)
+# pragma warning(pop)
 #endif
 
 inline int QImage::numColors() const
@@ -430,6 +437,4 @@ Q_GUI_EXPORT QDebug operator<<(QDebug, const QImage &);
 
 QT_END_NAMESPACE
 
-QT_END_HEADER
-
 #endif // QIMAGE_H
