@@ -52,7 +52,7 @@
 
     Call setDevice() to assign a device to the handler, and
     setFormat() to assign a format to it. One QImageIOHandler may
-    support more than one image format. canRead() returns true if an
+    support more than one image format. canRead() returns \c true if an
     image can be read from the device, and read() and write() return
     true if reading or writing an image was completed successfully.
 
@@ -187,11 +187,15 @@
     return this handler.
 
     The json metadata file for the plugin needs to contain information
-    about the image formats the plugins supports. For a jpeg plugin, this
-    could e.g. look as follows:
+    about the image formats the plugins supports, together with the
+    corresponding MIME types (one for each format). For a jpeg plugin, this
+    could, for example, look as follows:
 
     \code
-    { "Keys": [ "jpg", "jpeg" ] }
+    {
+      "Keys": [ "jpg", "jpeg" ],
+      "MimeTypes": [ "image/jpeg", "image/jpeg" ]
+    }
     \endcode
 
     Different plugins can support different capabilities. For example,
@@ -342,7 +346,7 @@ QByteArray QImageIOHandler::format() const
     \fn bool QImageIOHandler::read(QImage *image)
 
     Read an image from the device, and stores it in \a image.
-    Returns true if the image is successfully read; otherwise returns
+    Returns \c true if the image is successfully read; otherwise returns
     false.
 
     For image formats that support incremental loading, and for animation
@@ -355,10 +359,10 @@ QByteArray QImageIOHandler::format() const
 /*!
     \fn bool QImageIOHandler::canRead() const
 
-    Returns true if an image can be read from the device (i.e., the
+    Returns \c true if an image can be read from the device (i.e., the
     image format is supported, the device can be read from and the
     initial header information suggests that the image can be read);
-    otherwise returns false.
+    otherwise returns \c false.
 
     When reimplementing canRead(), make sure that the I/O device
     (device()) is left in its original state (e.g., by using peek()
@@ -379,10 +383,10 @@ QByteArray QImageIOHandler::name() const
 }
 
 /*!
-    Writes the image \a image to the assigned device. Returns true on
-    success; otherwise returns false.
+    Writes the image \a image to the assigned device. Returns \c true on
+    success; otherwise returns \c false.
 
-    The default implementation does nothing, and simply returns false.
+    The default implementation does nothing, and simply returns \c false.
 */
 bool QImageIOHandler::write(const QImage &image)
 {
@@ -415,8 +419,8 @@ QVariant QImageIOHandler::option(ImageOption option) const
 }
 
 /*!
-    Returns true if the QImageIOHandler supports the option \a option;
-    otherwise returns false. For example, if the QImageIOHandler
+    Returns \c true if the QImageIOHandler supports the option \a option;
+    otherwise returns \c false. For example, if the QImageIOHandler
     supports the \l Size option, supportsOption(Size) must return
     true.
 
@@ -433,7 +437,7 @@ bool QImageIOHandler::supportsOption(ImageOption option) const
     the sequence number of the current image in the animation. If
     this function is called before any image is read(), -1 is
     returned. The number of the first image in the sequence is 0.
-    
+
     If the image format does not support animation, 0 is returned.
 
     \sa read()
@@ -461,7 +465,7 @@ QRect QImageIOHandler::currentImageRect() const
     not support animation, or if it is unable to determine the number
     of images, 0 is returned.
 
-    The default implementation returns 1 if canRead() returns true;
+    The default implementation returns 1 if canRead() returns \c true;
     otherwise 0 is returned.
 */
 int QImageIOHandler::imageCount() const
@@ -473,7 +477,7 @@ int QImageIOHandler::imageCount() const
    For image formats that support animation, this function jumps to the
    next image.
 
-   The default implementation does nothing, and returns false.
+   The default implementation does nothing, and returns \c false.
 */
 bool QImageIOHandler::jumpToNextImage()
 {
@@ -485,7 +489,7 @@ bool QImageIOHandler::jumpToNextImage()
    whose sequence number is \a imageNumber. The next call to read() will
    attempt to read this image.
 
-   The default implementation does nothing, and returns false.
+   The default implementation does nothing, and returns \c false.
 */
 bool QImageIOHandler::jumpToImage(int imageNumber)
 {
@@ -537,23 +541,33 @@ QImageIOPlugin::~QImageIOPlugin()
 
 /*! \fn QImageIOPlugin::capabilities(QIODevice *device, const QByteArray &format) const
 
-    Returns the capabilities on the plugin, based on the data in \a
-    device and the format \a format. For example, if the
-    QImageIOHandler supports the BMP format, and the data in the
-    device starts with the characters "BM", this function should
-    return \l CanRead. If \a format is "bmp" and the handler supports
-    both reading and writing, this function should return \l CanRead |
-    \l CanWrite.
+    Returns the capabilities of the plugin, based on the data in \a
+    device and the format \a format.  If \a device is \c 0, it should
+    simply report whether the format can be read or written.  Otherwise,
+    it should attempt to determine whether the given format (or any
+    format supported by the plugin if \a format is empty) can be read
+    from or written to \a device.  It should do this without changing
+    the state of \a device (typically by using QIODevice::peek()).
+
+    For example, if the QImageIOPlugin supports the BMP format, \a format
+    is either empty or \c "bmp", and the data in the device starts with the
+    characters \c "BM", this function should return \l CanRead. If \a format
+    is \c "bmp", \a device is \c 0 and the handler supports both reading and
+    writing, this function should return \l CanRead | \l CanWrite.
+
+    Format names are always given in lower case.
 */
 
 /*!
     \fn QImageIOHandler *QImageIOPlugin::create(QIODevice *device, const QByteArray &format) const
 
     Creates and returns a QImageIOHandler subclass, with \a device
-    and \a format set. The \a format must come from the list returned by keys().
-    Format names are case sensitive.
+    and \a format set. The \a format must come from the values listed
+    in the \c "Keys" entry in the plugin metadata, or be empty.  If it is
+    empty, the data in \a device must have been recognized by the
+    capabilities() method (with a likewise empty format).
 
-    \sa keys()
+    Format names are always given in lower case.
 */
 
 #endif // QT_NO_IMAGEFORMATPLUGIN
