@@ -46,8 +46,6 @@
 #include <QtCore/qfactoryinterface.h>
 #include <QtCore/qscopedpointer.h>
 
-QT_BEGIN_HEADER
-
 QT_BEGIN_NAMESPACE
 
 
@@ -142,6 +140,4 @@ Q_DECLARE_OPERATORS_FOR_FLAGS(QImageIOPlugin::Capabilities)
 
 QT_END_NAMESPACE
 
-QT_END_HEADER
-
 #endif // QIMAGEIOHANDLER_H
