@@ -150,6 +150,8 @@
 #include <private/qgifhandler_p.h>
 #endif
 
+#include <algorithm>
+
 QT_BEGIN_NAMESPACE
 
 #ifndef QT_NO_IMAGEFORMATPLUGIN
@@ -190,34 +192,35 @@ struct _qt_BuiltInFormatStruct
 {
     _qt_BuiltInFormatType type;
     const char *extension;
+    const char *mimeType;
 };
 
 static const _qt_BuiltInFormatStruct _qt_BuiltInFormats[] = {
 #ifndef QT_NO_IMAGEFORMAT_PNG
-    {_qt_PngFormat, "png"},
+    {_qt_PngFormat, "png", "image/png"},
 #endif
 #ifndef QT_NO_IMAGEFORMAT_JPEG
-    {_qt_JpgFormat, "jpg"},
-    {_qt_JpegFormat, "jpeg"},
+    {_qt_JpgFormat, "jpg", "image/jpeg"},
+    {_qt_JpegFormat, "jpeg", "image/jpeg"},
 #endif
 #ifdef QT_BUILTIN_GIF_READER
-    {_qt_GifFormat, "gif"},
+    {_qt_GifFormat, "gif", "image/gif"},
 #endif
 #ifndef QT_NO_IMAGEFORMAT_BMP
-    {_qt_BmpFormat, "bmp"},
+    {_qt_BmpFormat, "bmp", "image/bmp"},
 #endif
 #ifndef QT_NO_IMAGEFORMAT_PPM
-    {_qt_PpmFormat, "ppm"},
-    {_qt_PgmFormat, "pgm"},
-    {_qt_PbmFormat, "pbm"},
+    {_qt_PpmFormat, "ppm", "image/x-portable-pixmap"},
+    {_qt_PgmFormat, "pgm", "image/x-portable-graymap"},
+    {_qt_PbmFormat, "pbm", "image/x-portable-bitmap"},
 #endif
 #ifndef QT_NO_IMAGEFORMAT_XBM
-    {_qt_XbmFormat, "xbm"},
+    {_qt_XbmFormat, "xbm", "image/x-xbitmap"},
 #endif
 #ifndef QT_NO_IMAGEFORMAT_XPM
-    {_qt_XpmFormat, "xpm"},
+    {_qt_XpmFormat, "xpm", "image/x-xpixmap"},
 #endif
-    {_qt_NoFormat, ""}
+    {_qt_NoFormat, "", ""}
 };
 
 static QImageIOHandler *createReadHandlerHelper(QIODevice *device,
@@ -310,7 +313,7 @@ static QImageIOHandler *createReadHandlerHelper(QIODevice *device,
                     QImageIOPlugin *plugin = qobject_cast<QImageIOPlugin *>(l->instance(i));
                     if (plugin && plugin->capabilities(device, testFormat) & QImageIOPlugin::CanRead) {
 #ifdef QIMAGEREADER_DEBUG
-                        qDebug() << "QImageReader::createReadHandler: the" << keys.at(i) << "plugin can read this format";
+                        qDebug() << "QImageReader::createReadHandler: the" << keyMap.keys().at(i) << "plugin can read this format";
 #endif
                         handler = plugin->create(device, testFormat);
                         break;
@@ -391,7 +394,7 @@ static QImageIOHandler *createReadHandlerHelper(QIODevice *device,
                 if (plugin && plugin->capabilities(device, QByteArray()) & QImageIOPlugin::CanRead) {
                     handler = plugin->create(device, testFormat);
 #ifdef QIMAGEREADER_DEBUG
-                    qDebug() << "QImageReader::createReadHandler: the" << keys.at(i) << "plugin can read this data";
+                    qDebug() << "QImageReader::createReadHandler: the" << keyMap.keys().at(i) << "plugin can read this data";
 #endif
                     break;
                 }
@@ -758,8 +761,8 @@ void QImageReader::setAutoDetectImageFormat(bool enabled)
 }
 
 /*!
-    Returns true if image format autodetection is enabled on this image
-    reader; otherwise returns false. By default, autodetection is enabled.
+    Returns \c true if image format autodetection is enabled on this image
+    reader; otherwise returns \c false. By default, autodetection is enabled.
 
     \sa setAutoDetectImageFormat()
 */
@@ -1086,7 +1089,7 @@ QColor QImageReader::backgroundColor() const
 /*!
     \since 4.1
 
-    Returns true if the image format supports animation;
+    Returns \c true if the image format supports animation;
     otherwise, false is returned.
 
     \sa QMovie::supportedFormats()
@@ -1101,15 +1104,15 @@ bool QImageReader::supportsAnimation() const
 }
 
 /*!
-    Returns true if an image can be read for the device (i.e., the
+    Returns \c true if an image can be read for the device (i.e., the
     image format is supported, and the device seems to contain valid
-    data); otherwise returns false.
+    data); otherwise returns \c false.
 
     canRead() is a lightweight function that only does a quick test to
     see if the image data is valid. read() may still return false
-    after canRead() returns true, if the image data is corrupt.
+    after canRead() returns \c true, if the image data is corrupt.
 
-    For images that support animation, canRead() returns false when
+    For images that support animation, canRead() returns \c false when
     all frames have been read.
 
     \sa read(), supportedImageFormats()
@@ -1146,7 +1149,7 @@ QImage QImageReader::read()
     \overload
 
     Reads an image from the device into \a image, which must point to a
-    QImage. Returns true on success; otherwise, returns false.
+    QImage. Returns \c true on success; otherwise, returns \c false.
 
     If \a image has same format and size as the image data that is about to be
     read, this function may not need to allocate a new image before
@@ -1390,7 +1393,7 @@ QString QImageReader::errorString() const
 /*!
     \since 4.2
 
-    Returns true if the reader supports \a option; otherwise returns
+    Returns \c true if the reader supports \a option; otherwise returns
     false.
 
     Different image formats support different options. Call this function to
@@ -1445,6 +1448,10 @@ QByteArray QImageReader::imageFormat(QIODevice *device)
 void supportedImageHandlerFormats(QFactoryLoader *loader,
                                   QImageIOPlugin::Capability cap,
                                   QSet<QByteArray> *result);
+
+void supportedImageHandlerMimeTypes(QFactoryLoader *loader,
+                                  QImageIOPlugin::Capability cap,
+                                  QSet<QByteArray> *result);
 #endif
 
 /*!
@@ -1453,18 +1460,17 @@ void supportedImageHandlerFormats(QFactoryLoader *loader,
     By default, Qt can read the following formats:
 
     \table
-    \header \li Format \li Description
-    \row    \li BMP    \li Windows Bitmap
-    \row    \li GIF    \li Graphic Interchange Format (optional)
-    \row    \li JPG    \li Joint Photographic Experts Group
-    \row    \li JPEG   \li Joint Photographic Experts Group
-    \row    \li PNG    \li Portable Network Graphics
-    \row    \li PBM    \li Portable Bitmap
-    \row    \li PGM    \li Portable Graymap
-    \row    \li PPM    \li Portable Pixmap
-    \row    \li XBM    \li X11 Bitmap
-    \row    \li XPM    \li X11 Pixmap
-    \row    \li SVG    \li Scalable Vector Graphics
+    \header \li Format \li MIME type                    \li Description
+    \row    \li BMP    \li image/bmp                    \li Windows Bitmap
+    \row    \li GIF    \li image/gif                    \li Graphic Interchange Format (optional)
+    \row    \li JPG    \li image/jpeg                   \li Joint Photographic Experts Group
+    \row    \li PNG    \li image/png                    \li Portable Network Graphics
+    \row    \li PBM    \li image/x-portable-bitmap      \li Portable Bitmap
+    \row    \li PGM    \li image/x-portable-graymap     \li Portable Graymap
+    \row    \li PPM    \li image/x-portable-pixmap      \li Portable Pixmap
+    \row    \li XBM    \li image/x-xbitmap              \li X11 Bitmap
+    \row    \li XPM    \li image/x-xpixmap              \li X11 Pixmap
+    \row    \li SVG    \li image/svg+xml                \li Scalable Vector Graphics
     \endtable
 
     Reading and writing SVG files is supported through the \l{Qt SVG} module.
@@ -1490,8 +1496,35 @@ QList<QByteArray> QImageReader::supportedImageFormats()
     for (QSet<QByteArray>::ConstIterator it = formats.constBegin(); it != formats.constEnd(); ++it)
         sortedFormats << *it;
 
-    qSort(sortedFormats);
+    std::sort(sortedFormats.begin(), sortedFormats.end());
     return sortedFormats;
 }
 
+/*!
+    Returns the list of MIME types supported by QImageReader.
+
+    Note that the QApplication instance must be created before this function is
+    called.
+
+    \sa supportedImageFormats(), QImageWriter::supportedMimeTypes()
+*/
+
+QList<QByteArray> QImageReader::supportedMimeTypes()
+{
+    QSet<QByteArray> mimeTypes;
+    for (int i = 0; i < _qt_NumFormats; ++i)
+        mimeTypes << _qt_BuiltInFormats[i].mimeType;
+
+#ifndef QT_NO_IMAGEFORMATPLUGIN
+    supportedImageHandlerMimeTypes(loader(), QImageIOPlugin::CanRead, &mimeTypes);
+#endif // QT_NO_IMAGEFORMATPLUGIN
+
+    QList<QByteArray> sortedMimeTypes;
+    for (QSet<QByteArray>::ConstIterator it = mimeTypes.constBegin(); it != mimeTypes.constEnd(); ++it)
+        sortedMimeTypes << *it;
+
+    std::sort(sortedMimeTypes.begin(), sortedMimeTypes.end());
+    return sortedMimeTypes;
+}
+
 QT_END_NAMESPACE
