@@ -58,7 +58,7 @@
     object.  Pass either a file name or a device pointer, and the
     image format to QImageWriter's constructor. You can then set
     several options, such as the gamma level (by calling setGamma())
-    and quality (by calling setQuality()). canWrite() returns true if
+    and quality (by calling setQuality()). canWrite() returns \c true if
     QImageWriter can write the image (i.e., the image format is
     supported and the device is open for writing). Call write() to
     write the image to the device.
@@ -100,6 +100,7 @@
 #include <qfile.h>
 #include <qfileinfo.h>
 #include <qimageiohandler.h>
+#include <qjsonarray.h>
 #include <qset.h>
 #include <qvariant.h>
 
@@ -122,6 +123,8 @@
 #include <private/qgifhandler_p.h>
 #endif
 
+#include <algorithm>
+
 QT_BEGIN_NAMESPACE
 
 #ifndef QT_NO_IMAGEFORMATPLUGIN
@@ -242,6 +245,8 @@ class QImageWriterPrivate
 public:
     QImageWriterPrivate(QImageWriter *qq);
 
+    bool canWriteHelper();
+
     // device
     QByteArray format;
     QIODevice *device;
@@ -274,11 +279,33 @@ QImageWriterPrivate::QImageWriterPrivate(QImageWriter *qq)
     compression = 0;
     gamma = 0.0;
     imageWriterError = QImageWriter::UnknownError;
-    errorString = QT_TRANSLATE_NOOP(QImageWriter, QLatin1String("Unknown error"));
+    errorString = QLatin1String(QT_TRANSLATE_NOOP(QImageWriter, "Unknown error"));
 
     q = qq;
 }
 
+bool QImageWriterPrivate::canWriteHelper()
+{
+    if (!device) {
+        imageWriterError = QImageWriter::DeviceError;
+        errorString = QLatin1String(QT_TRANSLATE_NOOP(QImageWriter, "Device is not set"));
+        return false;
+    }
+    if (!device->isOpen())
+        device->open(QIODevice::WriteOnly);
+    if (!device->isWritable()) {
+        imageWriterError = QImageWriter::DeviceError;
+        errorString = QLatin1String(QT_TRANSLATE_NOOP(QImageWriter, "Device not writable"));
+        return false;
+    }
+    if (!handler && (handler = createWriteHandlerHelper(device, format)) == 0) {
+        imageWriterError = QImageWriter::UnsupportedFormatError;
+        errorString = QLatin1String(QT_TRANSLATE_NOOP(QImageWriter, "Unsupported image format"));
+        return false;
+    }
+    return true;
+}
+
 /*!
     Constructs an empty QImageWriter object. Before writing, you must
     call setFormat() to set an image format, then setDevice() or
@@ -551,33 +578,27 @@ void QImageWriter::setText(const QString &key, const QString &text)
 }
 
 /*!
-    Returns true if QImageWriter can write the image; i.e., the image
+    Returns \c true if QImageWriter can write the image; i.e., the image
     format is supported and the assigned device is open for reading.
 
     \sa write(), setDevice(), setFormat()
 */
 bool QImageWriter::canWrite() const
 {
-    if (d->device && !d->handler && (d->handler = createWriteHandlerHelper(d->device, d->format)) == 0) {
-        d->imageWriterError = QImageWriter::UnsupportedFormatError;
-        d->errorString = QT_TRANSLATE_NOOP(QImageWriter,
-                                           QLatin1String("Unsupported image format"));
-        return false;
+    if (QFile *file = qobject_cast<QFile *>(d->device)) {
+        const bool remove = !file->isOpen() && !file->exists();
+        const bool result = d->canWriteHelper();
+        if (!result && remove)
+            file->remove();
+        return result;
     }
-    if (d->device && !d->device->isOpen())
-        d->device->open(QIODevice::WriteOnly);
-    if (!d->device || !d->device->isWritable()) {
-        d->imageWriterError = QImageWriter::DeviceError;
-        d->errorString = QT_TRANSLATE_NOOP(QImageWriter,
-                                           QLatin1String("Device not writable"));
-        return false;
-    }
-    return true;
+
+    return d->canWriteHelper();
 }
 
 /*!
     Writes the image \a image to the assigned device or file
-    name. Returns true on success; otherwise returns false. If the
+    name. Returns \c true on success; otherwise returns \c false. If the
     operation fails, you can call error() to find the type of error
     that occurred, or errorString() to get a human readable
     description of the error.
@@ -628,7 +649,7 @@ QString QImageWriter::errorString() const
 /*!
     \since 4.2
 
-    Returns true if the writer supports \a option; otherwise returns
+    Returns \c true if the writer supports \a option; otherwise returns
     false.
 
     Different image formats support different options. Call this function to
@@ -646,8 +667,7 @@ bool QImageWriter::supportsOption(QImageIOHandler::ImageOption option) const
 {
     if (!d->handler && (d->handler = createWriteHandlerHelper(d->device, d->format)) == 0) {
         d->imageWriterError = QImageWriter::UnsupportedFormatError;
-        d->errorString = QT_TRANSLATE_NOOP(QImageWriter,
-                                           QLatin1String("Unsupported image format"));
+        d->errorString = QLatin1String(QT_TRANSLATE_NOOP(QImageWriter, "Unsupported image format"));
         return false;
     }
 
@@ -677,6 +697,26 @@ void supportedImageHandlerFormats(QFactoryLoader *loader,
             result->insert(key);
     }
 }
+
+void supportedImageHandlerMimeTypes(QFactoryLoader *loader,
+                                    QImageIOPlugin::Capability cap,
+                                    QSet<QByteArray> *result)
+{
+    QList<QJsonObject> metaDataList = loader->metaData();
+
+    const int pluginCount = metaDataList.size();
+    for (int i = 0; i < pluginCount; ++i) {
+        const QJsonObject metaData = metaDataList.at(i).value(QStringLiteral("MetaData")).toObject();
+        const QJsonArray keys = metaData.value(QStringLiteral("Keys")).toArray();
+        const QJsonArray mimeTypes = metaData.value(QStringLiteral("MimeTypes")).toArray();
+        QImageIOPlugin *plugin = qobject_cast<QImageIOPlugin *>(loader->instance(i));
+        const int keyCount = keys.size();
+        for (int k = 0; k < keyCount; ++k) {
+            if (plugin && (plugin->capabilities(0, keys.at(k).toString().toLatin1()) & cap) != 0)
+                result->insert(mimeTypes.at(k).toString().toLatin1());
+        }
+    }
+}
 #endif // QT_NO_IMAGEFORMATPLUGIN
 
 /*!
@@ -685,16 +725,15 @@ void supportedImageHandlerFormats(QFactoryLoader *loader,
     By default, Qt can write the following formats:
 
     \table
-    \header \li Format \li Description
-    \row    \li BMP    \li Windows Bitmap
-    \row    \li JPG    \li Joint Photographic Experts Group
-    \row    \li JPEG   \li Joint Photographic Experts Group
-    \row    \li PNG    \li Portable Network Graphics
-    \row    \li PBM    \li Portable Bitmap
-    \row    \li PGM    \li Portable Graymap
-    \row    \li PPM    \li Portable Pixmap
-    \row    \li XBM    \li X11 Bitmap
-    \row    \li XPM    \li X11 Pixmap
+    \header \li Format \li MIME type                    \li Description
+    \row    \li BMP    \li image/bmp                    \li Windows Bitmap
+    \row    \li JPG    \li image/jpeg                   \li Joint Photographic Experts Group
+    \row    \li PNG    \li image/png                    \li Portable Network Graphics
+    \row    \li PBM    \li image/x-portable-bitmap      \li Portable Bitmap
+    \row    \li PGM    \li image/x-portable-graymap     \li Portable Graymap
+    \row    \li PPM    \li image/x-portable-pixmap      \li Portable Pixmap
+    \row    \li XBM    \li image/x-xbitmap              \li X11 Bitmap
+    \row    \li XPM    \li image/x-xpixmap              \li X11 Pixmap
     \endtable
 
     Reading and writing SVG files is supported through the \l{Qt SVG} module.
@@ -726,9 +765,6 @@ QList<QByteArray> QImageWriter::supportedImageFormats()
 #ifndef QT_NO_IMAGEFORMAT_JPEG
     formats << "jpg" << "jpeg";
 #endif
-#ifdef QT_BUILTIN_GIF_READER
-    formats << "gif";
-#endif
 
 #ifndef QT_NO_IMAGEFORMATPLUGIN
     supportedImageHandlerFormats(loader(), QImageIOPlugin::CanWrite, &formats);
@@ -738,8 +774,52 @@ QList<QByteArray> QImageWriter::supportedImageFormats()
     for (QSet<QByteArray>::ConstIterator it = formats.constBegin(); it != formats.constEnd(); ++it)
         sortedFormats << *it;
 
-    qSort(sortedFormats);
+    std::sort(sortedFormats.begin(), sortedFormats.end());
     return sortedFormats;
 }
 
+/*!
+    Returns the list of MIME types supported by QImageWriter.
+
+    Note that the QApplication instance must be created before this function is
+    called.
+
+    \sa supportedImageFormats(), QImageReader::supportedMimeTypes()
+*/
+QList<QByteArray> QImageWriter::supportedMimeTypes()
+{
+    QSet<QByteArray> mimeTypes;
+#ifndef QT_NO_IMAGEFORMAT_BMP
+    mimeTypes << "image/bmp";
+#endif
+#ifndef QT_NO_IMAGEFORMAT_PPM
+    mimeTypes << "image/x-portable-bitmap";
+    mimeTypes << "image/x-portable-graymap";
+    mimeTypes << "image/x-portable-pixmap";
+#endif
+#ifndef QT_NO_IMAGEFORMAT_XBM
+    mimeTypes << "image/x-xbitmap";
+#endif
+#ifndef QT_NO_IMAGEFORMAT_XPM
+    mimeTypes << "image/x-xpixmap";
+#endif
+#ifndef QT_NO_IMAGEFORMAT_PNG
+    mimeTypes << "image/png";
+#endif
+#ifndef QT_NO_IMAGEFORMAT_JPEG
+    mimeTypes << "image/jpeg";
+#endif
+
+#ifndef QT_NO_IMAGEFORMATPLUGIN
+    supportedImageHandlerMimeTypes(loader(), QImageIOPlugin::CanWrite, &mimeTypes);
+#endif // QT_NO_IMAGEFORMATPLUGIN
+
+    QList<QByteArray> sortedMimeTypes;
+    for (QSet<QByteArray>::ConstIterator it = mimeTypes.constBegin(); it != mimeTypes.constEnd(); ++it)
+        sortedMimeTypes << *it;
+
+    std::sort(sortedMimeTypes.begin(), sortedMimeTypes.end());
+    return sortedMimeTypes;
+}
+
 QT_END_NAMESPACE
