@@ -39,7 +39,7 @@
 **
 ****************************************************************************/
 
-/*! 
+/*!
     \class QMovie
 
     \inmodule QtGui
@@ -48,7 +48,7 @@
     with QImageReader.
 
     This class is used to show simple animations without sound. If you want
-    to display video and media content, use the \l{Phonon Module}{Phonon}
+    to display video and media content, use the \l{Qt Multimedia}
     multimedia framework instead.
 
     First, create a QMovie object by passing either the name of a file or a
@@ -150,7 +150,7 @@
     copy of the frame.
 */
 
-/*! 
+/*!
     \fn void QMovie::stateChanged(QMovie::MovieState state)
 
     This signal is emitted every time the state of the movie changes. The new
@@ -203,23 +203,23 @@ public:
     inline QFrameInfo(bool endMark)
         : pixmap(QPixmap()), delay(QMOVIE_INVALID_DELAY), endMark(endMark)
     { }
-    
+
     inline QFrameInfo()
         : pixmap(QPixmap()), delay(QMOVIE_INVALID_DELAY), endMark(false)
     { }
-    
+
     inline QFrameInfo(const QPixmap &pixmap, int delay)
         : pixmap(pixmap), delay(delay), endMark(false)
     { }
-    
+
     inline bool isValid()
     {
         return endMark || !(pixmap.isNull() && (delay == QMOVIE_INVALID_DELAY));
     }
-    
+
     inline bool isEndMarker()
     { return endMark; }
-    
+
     static inline QFrameInfo endMarker()
     { return QFrameInfo(true); }
 };
@@ -777,8 +777,8 @@ QImage QMovie::currentImage() const
 }
 
 /*!
-    Returns true if the movie is valid (e.g., the image data is readable and
-    the image format is supported); otherwise returns false.
+    Returns \c true if the movie is valid (e.g., the image data is readable and
+    the image format is supported); otherwise returns \c false.
 */
 bool QMovie::isValid() const
 {
@@ -819,7 +819,7 @@ int QMovie::currentFrameNumber() const
 }
 
 /*!
-    Jumps to the next frame. Returns true on success; otherwise returns false.
+    Jumps to the next frame. Returns \c true on success; otherwise returns \c false.
 */
 bool QMovie::jumpToNextFrame()
 {
@@ -828,8 +828,8 @@ bool QMovie::jumpToNextFrame()
 }
 
 /*!
-    Jumps to frame number \a frameNumber. Returns true on success; otherwise
-    returns false.
+    Jumps to frame number \a frameNumber. Returns \c true on success; otherwise
+    returns \c false.
 */
 bool QMovie::jumpToFrame(int frameNumber)
 {
