@@ -50,8 +50,6 @@
 #include <QtCore/qlist.h>
 #include <QtGui/qimagereader.h>
 
-QT_BEGIN_HEADER
-
 QT_BEGIN_NAMESPACE
 
 
@@ -146,8 +144,6 @@ private:
 
 QT_END_NAMESPACE
 
-QT_END_HEADER
-
 #endif // QT_NO_MOVIE
 
 #endif // QMOVIE_H
