@@ -50,8 +50,6 @@
 #include <QtGui/qimage.h>
 #include <QtGui/qtransform.h>
 
-QT_BEGIN_HEADER
-
 QT_BEGIN_NAMESPACE
 
 
@@ -245,6 +243,4 @@ Q_GUI_EXPORT QDebug operator<<(QDebug, const QPixmap &);
 
 QT_END_NAMESPACE
 
-QT_END_HEADER
-
 #endif // QPIXMAP_H
