@@ -54,8 +54,6 @@
 #include <QtGui/qpixmap.h>
 #include <QtCore/qatomic.h>
 
-QT_BEGIN_HEADER
-
 QT_BEGIN_NAMESPACE
 
 
@@ -132,6 +130,7 @@ public:
 protected:
 
     void setSerialNumber(int serNo);
+    void setDetachNumber(int detNo);
     int w;
     int h;
     int d;
@@ -158,6 +157,4 @@ extern bool qt_xForm_helper(const QTransform&, int, int, int, uchar*, int, int,
 
 QT_END_NAMESPACE
 
-QT_END_HEADER
-
 #endif // QPLATFORMPIXMAP_H
