@@ -1,6 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2012 Digia Plc and/or its subsidiary(-ies).
+** Copyright (C) 2013 Samuel Gaist <samuel.gaist@edeltech.ch>
+** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
@@ -280,6 +281,15 @@ void setup_qt(QImage& image, png_structp png_ptr, png_infop info_ptr, QSize scal
             image.setColorCount(2);
             image.setColor(1, qRgb(0,0,0));
             image.setColor(0, qRgb(255,255,255));
+            if (png_get_tRNS(png_ptr, info_ptr, &trans_alpha, &num_trans, &trans_color_p) && trans_color_p) {
+                const int g = trans_color_p->gray;
+                // the image has white in the first position of the color table,
+                // black in the second. g is 0 for black, 1 for white.
+                if (g == 0)
+                    image.setColor(1, qRgba(0, 0, 0, 0));
+                else if (g == 1)
+                    image.setColor(0, qRgba(255, 255, 255, 0));
+            }
         } else if (bit_depth == 16 && png_get_valid(png_ptr, info_ptr, PNG_INFO_tRNS)) {
             png_set_expand(png_ptr);
             png_set_strip_16(png_ptr);
@@ -406,11 +416,16 @@ static void read_image_scaled(QImage *outImage, png_structp png_ptr, png_infop i
                               QPngHandlerPrivate::AllocatedMemoryPointers &amp, QSize scaledSize)
 {
 
-    png_uint_32 width;
-    png_uint_32 height;
-    int bit_depth;
-    int color_type;
+    png_uint_32 width = 0;
+    png_uint_32 height = 0;
+    png_int_32 offset_x = 0;
+    png_int_32 offset_y = 0;
+
+    int bit_depth = 0;
+    int color_type = 0;
+    int unit_type = PNG_OFFSET_PIXEL;
     png_get_IHDR(png_ptr, info_ptr, &width, &height, &bit_depth, &color_type, 0, 0, 0);
+    png_get_oFFs(png_ptr, info_ptr, &offset_x, &offset_y, &unit_type);
     uchar *data = outImage->bits();
     int bpl = outImage->bytesPerLine();
 
@@ -470,6 +485,10 @@ static void read_image_scaled(QImage *outImage, png_structp png_ptr, png_infop i
 
     outImage->setDotsPerMeterX((png_get_x_pixels_per_meter(png_ptr,info_ptr)*oxsz)/ixsz);
     outImage->setDotsPerMeterY((png_get_y_pixels_per_meter(png_ptr,info_ptr)*oysz)/iysz);
+
+    if (unit_type == PNG_OFFSET_PIXEL)
+        outImage->setOffset(QPoint(offset_x*oxsz/ixsz, offset_y*oysz/iysz));
+
 }
 
 #if defined(Q_C_CALLBACKS)
@@ -550,7 +569,6 @@ bool Q_INTERNAL_WIN_NO_THROW QPngHandlerPrivate::readPngHeader()
     return true;
 }
 
-
 bool Q_INTERNAL_WIN_NO_THROW QPngHandlerPrivate::readPngImage(QImage *outImage)
 {
     if (state == Error)
@@ -583,11 +601,16 @@ bool Q_INTERNAL_WIN_NO_THROW QPngHandlerPrivate::readPngImage(QImage *outImage)
     if (doScaledRead) {
         read_image_scaled(outImage, png_ptr, info_ptr, amp, scaledSize);
     } else {
-        png_uint_32 width;
-        png_uint_32 height;
-        int bit_depth;
-        int color_type;
+        png_uint_32 width = 0;
+        png_uint_32 height = 0;
+        png_int_32 offset_x = 0;
+        png_int_32 offset_y = 0;
+
+        int bit_depth = 0;
+        int color_type = 0;
+        int unit_type = PNG_OFFSET_PIXEL;
         png_get_IHDR(png_ptr, info_ptr, &width, &height, &bit_depth, &color_type, 0, 0, 0);
+        png_get_oFFs(png_ptr, info_ptr, &offset_x, &offset_y, &unit_type);
         uchar *data = outImage->bits();
         int bpl = outImage->bytesPerLine();
         amp.row_pointers = new png_bytep[height];
@@ -601,6 +624,9 @@ bool Q_INTERNAL_WIN_NO_THROW QPngHandlerPrivate::readPngImage(QImage *outImage)
         outImage->setDotsPerMeterX(png_get_x_pixels_per_meter(png_ptr,info_ptr));
         outImage->setDotsPerMeterY(png_get_y_pixels_per_meter(png_ptr,info_ptr));
 
+        if (unit_type == PNG_OFFSET_PIXEL)
+            outImage->setOffset(QPoint(offset_x, offset_y));
+
         // sanity check palette entries
         if (color_type == PNG_COLOR_TYPE_PALETTE && outImage->format() == QImage::Format_Indexed8) {
             int color_table_size = outImage->colorCount();
