@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2012 Digia Plc and/or its subsidiary(-ies).
+** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
@@ -153,6 +153,14 @@ QT_BEGIN_NAMESPACE
     \sa QWidget, {fowler}{GUI Design Handbook: Cursors}
 */
 
+/*!
+    \fn QCursor &QCursor::operator=(QCursor &&other)
+
+    Move-assigns \a other to this QCursor instance.
+
+    \since 5.2
+*/
+
 /*!
     \fn QPoint QCursor::pos(const QScreen *screen)
 
