@@ -65,8 +65,6 @@
 #include "private/qshortcutmap_p.h"
 #include <qicon.h>
 
-QT_BEGIN_HEADER
-
 QT_BEGIN_NAMESPACE
 
 class QPlatformIntegration;
@@ -85,14 +83,16 @@ public:
     ~QGuiApplicationPrivate();
 
     void createPlatformIntegration();
-    void createEventDispatcher();
-    void setEventDispatcher(QAbstractEventDispatcher *eventDispatcher);
+    void createEventDispatcher() Q_DECL_OVERRIDE;
+    void eventDispatcherReady() Q_DECL_OVERRIDE;
 
     virtual void notifyLayoutDirectionChange();
     virtual void notifyActiveWindowChange(QWindow *previous);
 
     virtual bool shouldQuit();
 
+    bool shouldQuitInternal(const QWindowList &processedWindows);
+
     static Qt::KeyboardModifiers modifier_buttons;
     static Qt::MouseButtons mouse_buttons;
 
@@ -109,7 +109,7 @@ public:
     static QAbstractEventDispatcher *qt_qpa_core_dispatcher()
     {
         if (QCoreApplication::instance())
-            return QCoreApplication::instance()->d_func()->threadData->eventDispatcher;
+            return QCoreApplication::instance()->d_func()->threadData->eventDispatcher.load();
         else
             return 0;
     }
@@ -128,6 +128,7 @@ public:
 
     static void processActivatedEvent(QWindowSystemInterfacePrivate::ActivatedWindowEvent *e);
     static void processWindowStateChangedEvent(QWindowSystemInterfacePrivate::WindowStateChangedEvent *e);
+    static void processWindowScreenChangedEvent(QWindowSystemInterfacePrivate::WindowScreenChangedEvent *e);
 
     static void processWindowSystemEvent(QWindowSystemInterfacePrivate::WindowSystemEvent *e);
 
@@ -148,6 +149,10 @@ public:
     static void processTabletEnterProximityEvent(QWindowSystemInterfacePrivate::TabletEnterProximityEvent *e);
     static void processTabletLeaveProximityEvent(QWindowSystemInterfacePrivate::TabletLeaveProximityEvent *e);
 
+#ifndef QT_NO_GESTURES
+    static void processGestureEvent(QWindowSystemInterfacePrivate::GestureEvent *e);
+#endif
+
     static void processPlatformPanelEvent(QWindowSystemInterfacePrivate::PlatformPanelEvent *e);
 #ifndef QT_NO_CONTEXTMENU
     static void processContextMenuEvent(QWindowSystemInterfacePrivate::ContextMenuEvent *e);
@@ -191,6 +196,8 @@ public:
     static void updateBlockedStatus(QWindow *window);
     virtual bool isWindowBlocked(QWindow *window, QWindow **blockingWindow = 0) const;
 
+    static bool synthesizeMouseFromTouchEventsEnabled();
+
     static Qt::MouseButtons buttons;
     static ulong mousePressTime;
     static Qt::MouseButton mousePressButton;
@@ -201,6 +208,7 @@ public:
     static bool tabletState;
     static QWindow *tabletPressTarget;
     static QWindow *currentMouseWindow;
+    static Qt::ApplicationState applicationState;
 
 #ifndef QT_NO_CLIPBOARD
     static QClipboard *qt_clipboard;
@@ -230,12 +238,10 @@ public:
 
 #ifndef QT_NO_SESSIONMANAGER
     QSessionManager *session_manager;
-    QString session_id;
-    QString session_key;
     bool is_session_restored;
     bool is_saving_session;
-    void commitData(QSessionManager& sm);
-    void saveState(QSessionManager& sm);
+    void commitData();
+    void saveState();
 #endif
 
     struct ActiveTouchPointsKey {
@@ -269,6 +275,10 @@ public:
     // hook reimplemented in QApplication to apply the QStyle function on the QIcon
     virtual QPixmap applyQIconStyleHelper(QIcon::Mode, const QPixmap &basePixmap) const { return basePixmap; }
 
+    static QRect applyWindowGeometrySpecification(const QRect &windowGeometry, const QWindow *window);
+
+    static void setApplicationState(Qt::ApplicationState state);
+
 protected:
     virtual void notifyThemeChanged();
 #ifndef QT_NO_DRAGANDDROP
@@ -293,6 +303,4 @@ Q_GUI_EXPORT bool operator==(const QGuiApplicationPrivate::ActiveTouchPointsKey
 
 QT_END_NAMESPACE
 
-QT_END_HEADER
-
 #endif // QGUIAPPLICATION_P_H
