@@ -61,8 +61,6 @@
 #include <qpa/qplatformintegration.h>
 #include <private/qguiapplication_p.h>
 
-QT_BEGIN_HEADER
-
 QT_BEGIN_NAMESPACE
 
 class QInputMethodPrivate : public QObjectPrivate
@@ -86,11 +84,10 @@ public:
     bool objectAcceptsInputMethod(QObject *object);
 
     QTransform inputItemTransform;
+    QRectF inputRectangle;
     QPlatformInputContext *testContext;
 };
 
 QT_END_NAMESPACE
 
-QT_END_HEADER
-
 #endif
