@@ -89,6 +89,14 @@ static void qt_palette_from_color(QPalette &pal, const QColor &button)
                       whiteBrush, buttonBrush, buttonBrush);
 }
 
+/*!
+    \fn QPalette &QPalette::operator=(QPalette &&other)
+
+    Move-assigns \a other to this QPalette instance.
+
+    \since 5.2
+*/
+
 /*!
    \fn const QColor &QPalette::color(ColorRole role) const
 
@@ -755,8 +763,8 @@ void QPalette::setBrush(ColorGroup cg, ColorRole cr, const QBrush &b)
 /*!
     \since 4.2
 
-    Returns true if the ColorGroup \a cg and ColorRole \a cr has been
-    set previously on this palette; otherwise returns false.
+    Returns \c true if the ColorGroup \a cg and ColorRole \a cr has been
+    set previously on this palette; otherwise returns \c false.
 
     \sa setBrush()
 */
@@ -787,8 +795,8 @@ void QPalette::detach()
 /*!
     \fn bool QPalette::operator!=(const QPalette &p) const
 
-    Returns true (slowly) if this palette is different from \a p;
-    otherwise returns false (usually quickly).
+    Returns \c true (slowly) if this palette is different from \a p;
+    otherwise returns \c false (usually quickly).
 
     \note The current ColorGroup is not taken into account when
     comparing palettes
@@ -797,8 +805,8 @@ void QPalette::detach()
 */
 
 /*!
-    Returns true (usually quickly) if this palette is equal to \a p;
-    otherwise returns false (slowly).
+    Returns \c true (usually quickly) if this palette is equal to \a p;
+    otherwise returns \c false (slowly).
 
     \note The current ColorGroup is not taken into account when
     comparing palettes
@@ -821,8 +829,8 @@ bool QPalette::operator==(const QPalette &p) const
 /*!
     \fn bool QPalette::isEqual(ColorGroup cg1, ColorGroup cg2) const
 
-    Returns true (usually quickly) if color group \a cg1 is equal to
-    \a cg2; otherwise returns false.
+    Returns \c true (usually quickly) if color group \a cg1 is equal to
+    \a cg2; otherwise returns \c false.
 */
 bool QPalette::isEqual(QPalette::ColorGroup group1, QPalette::ColorGroup group2) const
 {
@@ -1002,9 +1010,9 @@ QDataStream &operator>>(QDataStream &s, QPalette &p)
 #endif //QT_NO_DATASTREAM
 
 /*!
-    Returns true if this palette and \a p are copies of each other,
+    Returns \c true if this palette and \a p are copies of each other,
     i.e. one of them was created as a copy of the other and neither
-    was subsequently modified; otherwise returns false. This is much
+    was subsequently modified; otherwise returns \c false. This is much
     stricter than equality.
 
     \sa operator=(), operator==()
@@ -1096,6 +1104,42 @@ void QPalette::setColorGroup(ColorGroup cg, const QBrush &foreground, const QBru
     setBrush(cg, ToolTipText, toolTipText);
 }
 
+Q_GUI_EXPORT QPalette qt_fusionPalette()
+{
+    QColor backGround(239, 235, 231);
+    QColor light = backGround.lighter(150);
+    QColor mid(backGround.darker(130));
+    QColor midLight = mid.lighter(110);
+    QColor base = Qt::white;
+    QColor disabledBase(backGround);
+    QColor dark = backGround.darker(150);
+    QColor darkDisabled = QColor(209, 200, 191).darker(110);
+    QColor text = Qt::black;
+    QColor hightlightedText = Qt::white;
+    QColor disabledText = QColor(190, 190, 190);
+    QColor button = backGround;
+    QColor shadow = dark.darker(135);
+    QColor disabledShadow = shadow.lighter(150);
+
+    QPalette fusionPalette(Qt::black,backGround,light,dark,mid,text,base);
+    fusionPalette.setBrush(QPalette::Midlight, midLight);
+    fusionPalette.setBrush(QPalette::Button, button);
+    fusionPalette.setBrush(QPalette::Shadow, shadow);
+    fusionPalette.setBrush(QPalette::HighlightedText, hightlightedText);
+
+    fusionPalette.setBrush(QPalette::Disabled, QPalette::Text, disabledText);
+    fusionPalette.setBrush(QPalette::Disabled, QPalette::WindowText, disabledText);
+    fusionPalette.setBrush(QPalette::Disabled, QPalette::ButtonText, disabledText);
+    fusionPalette.setBrush(QPalette::Disabled, QPalette::Base, disabledBase);
+    fusionPalette.setBrush(QPalette::Disabled, QPalette::Dark, darkDisabled);
+    fusionPalette.setBrush(QPalette::Disabled, QPalette::Shadow, disabledShadow);
+
+    fusionPalette.setBrush(QPalette::Active, QPalette::Highlight, QColor(48, 140, 198));
+    fusionPalette.setBrush(QPalette::Inactive, QPalette::Highlight, QColor(48, 140, 198));
+    fusionPalette.setBrush(QPalette::Disabled, QPalette::Highlight, QColor(145, 141, 126));
+    return fusionPalette;
+}
+
 #ifndef QT_NO_DEBUG_STREAM
 QDebug operator<<(QDebug dbg, const QPalette &)
 {
