@@ -46,8 +46,6 @@
 #include <QtGui/qcolor.h>
 #include <QtGui/qbrush.h>
 
-QT_BEGIN_HEADER
-
 QT_BEGIN_NAMESPACE
 
 
@@ -176,11 +174,12 @@ private:
     void detach();
 
     QPalettePrivate *d;
+    struct Data {
+        uint current_group : 4;
+        uint resolve_mask : 28;
+    };
     union {
-        struct {
-            uint current_group : 4;
-            uint resolve_mask : 28;
-        } data;
+        Data data;
         quint32 for_faster_swapping_dont_use;
     };
     friend Q_GUI_EXPORT QDataStream &operator<<(QDataStream &s, const QPalette &p);
@@ -210,6 +209,4 @@ Q_GUI_EXPORT QDebug operator<<(QDebug, const QPalette &);
 
 QT_END_NAMESPACE
 
-QT_END_HEADER
-
 #endif // QPALETTE_H
