@@ -58,8 +58,6 @@
 #include <qpa/qplatformscreen.h>
 #include <QtGui/QCursor>
 
-QT_BEGIN_HEADER
-
 QT_BEGIN_NAMESPACE
 
 
@@ -106,6 +104,4 @@ private:
 
 QT_END_NAMESPACE
 
-QT_END_HEADER
-
 #endif // QPLATFORMCURSOR_H
