@@ -54,14 +54,12 @@
 #include <QtCore/qplugin.h>
 #include <QtCore/qfactoryinterface.h>
 
-QT_BEGIN_HEADER
-
 QT_BEGIN_NAMESPACE
 
 
 class QPlatformIntegration;
 
-#define QPlatformIntegrationFactoryInterface_iid "org.qt-project.Qt.QPA.QPlatformIntegrationFactoryInterface.5.1"
+#define QPlatformIntegrationFactoryInterface_iid "org.qt-project.Qt.QPA.QPlatformIntegrationFactoryInterface.5.2"
 
 class Q_GUI_EXPORT QPlatformIntegrationPlugin : public QObject
 {
@@ -70,11 +68,10 @@ public:
     explicit QPlatformIntegrationPlugin(QObject *parent = 0);
     ~QPlatformIntegrationPlugin();
 
-    virtual QPlatformIntegration *create(const QString &key, const QStringList &paramList) = 0;
+    virtual QPlatformIntegration *create(const QString &key, const QStringList &paramList);
+    virtual QPlatformIntegration *create(const QString &key, const QStringList &paramList, int &argc, char **argv);
 };
 
 QT_END_NAMESPACE
 
-QT_END_HEADER
-
 #endif // QPLATFORMINTEGRATIONPLUGIN_H
