@@ -41,11 +41,13 @@
 
 #include "qplatformwindow.h"
 #include "qplatformwindow_p.h"
+#include "qplatformscreen.h"
 
 #include <private/qguiapplication_p.h>
 #include <qpa/qwindowsysteminterface.h>
 #include <QtGui/qwindow.h>
 #include <QtGui/qscreen.h>
+#include <private/qwindow_p.h>
 
 QT_BEGIN_NAMESPACE
 
@@ -165,7 +167,7 @@ bool QPlatformWindow::isExposed() const
 }
 
 /*!
-    Returns true if the window should appear active from a style perspective.
+    Returns \c true if the window should appear active from a style perspective.
 
     This function can make platform-specific isActive checks, such as checking
     if the QWindow is embedded in an active native window.
@@ -176,7 +178,7 @@ bool QPlatformWindow::isActive() const
 }
 
 /*!
-    Returns true if the window is a descendant of an embedded non-Qt window.
+    Returns \c true if the window is a descendant of an embedded non-Qt window.
     Example of an embedded non-Qt window is the parent window of an in-process QAxServer.
 
     If \a parentWindow is nonzero, only check if the window is embedded in the
@@ -197,7 +199,13 @@ bool QPlatformWindow::isEmbedded(const QPlatformWindow *parentWindow) const
 */
 QPoint QPlatformWindow::mapToGlobal(const QPoint &pos) const
 {
-    return pos;
+    const QPlatformWindow *p = this;
+    QPoint result = pos;
+    while (p) {
+        result += p->geometry().topLeft();
+        p = p->parent();
+    }
+    return result;
 }
 
 /*!
@@ -209,7 +217,13 @@ QPoint QPlatformWindow::mapToGlobal(const QPoint &pos) const
 */
 QPoint QPlatformWindow::mapFromGlobal(const QPoint &pos) const
 {
-    return pos;
+    const QPlatformWindow *p = this;
+    QPoint result = pos;
+    while (p) {
+        result -= p->geometry().topLeft();
+        p = p->parent();
+    }
+    return result;
 }
 
 /*!
@@ -411,8 +425,7 @@ bool QPlatformWindow::startSystemResize(const QPoint &pos, Qt::Corner corner)
 
 void QPlatformWindow::setFrameStrutEventsEnabled(bool enabled)
 {
-    if (enabled)
-        qWarning("This plugin does not support frame strut events.");
+    Q_UNUSED(enabled) // Do not warn as widgets enable it by default causing warnings with XCB.
 }
 
 /*!
@@ -448,6 +461,97 @@ QString QPlatformWindow::formatWindowTitle(const QString &title, const QString &
     return fullTitle;
 }
 
+/*!
+    Reimplement this method to set whether the window demands attention
+    (for example, by flashing the taskbar icon) depending on \a enabled.
+
+    \sa isAlertState()
+    \since 5.1
+*/
+
+void QPlatformWindow::setAlertState(bool enable)
+{
+    Q_UNUSED(enable)
+}
+
+/*!
+    Reimplement this method return whether the window is in
+    an alert state.
+
+    \sa setAlertState()
+    \since 5.1
+*/
+
+bool QPlatformWindow::isAlertState() const
+{
+    return false;
+}
+
+// Return the effective screen for the initial geometry of a window. In a
+// multimonitor-setup, try to find the right screen by checking the transient
+// parent or the mouse cursor for parentless windows (cf QTBUG-34204,
+// QDialog::adjustPosition()).
+static inline const QScreen *effectiveScreen(const QWindow *window)
+{
+    if (!window)
+        return QGuiApplication::primaryScreen();
+    const QScreen *screen = window->screen();
+    if (!screen)
+        return QGuiApplication::primaryScreen();
+    const QList<QScreen *> siblings = screen->virtualSiblings();
+    if (siblings.size() > 1) {
+        const QPoint referencePoint = window->transientParent() ? window->transientParent()->geometry().center() : QCursor::pos();
+        foreach (const QScreen *sibling, siblings)
+            if (sibling->geometry().contains(referencePoint))
+                return sibling;
+    }
+    return screen;
+}
+
+/*!
+    Helper function to get initial geometry on windowing systems which do not
+    do smart positioning and also do not provide a means of centering a
+    transient window w.r.t. its parent. For example this is useful on Windows
+    and MacOS but not X11, because an X11 window manager typically tries to
+    layout new windows to optimize usage of the available desktop space.
+    However if the given window already has geometry which the application has
+    initialized, it takes priority.
+*/
+QRect QPlatformWindow::initialGeometry(const QWindow *w,
+    const QRect &initialGeometry, int defaultWidth, int defaultHeight)
+{
+    QRect rect(initialGeometry);
+    if (rect.isNull()) {
+        QSize minimumSize = w->minimumSize();
+        if (minimumSize.width() > 0 || minimumSize.height() > 0) {
+            rect.setSize(minimumSize);
+        } else {
+            rect.setWidth(defaultWidth);
+            rect.setHeight(defaultHeight);
+        }
+    }
+    if (w->isTopLevel() && qt_window_private(const_cast<QWindow*>(w))->positionAutomatic
+        && w->type() != Qt::Popup) {
+        if (const QScreen *screen = effectiveScreen(w)) {
+            const QRect availableGeometry = screen->availableGeometry();
+            // Center unless the geometry ( + unknown window frame) is too large for the screen).
+            if (rect.height() < (availableGeometry.height() * 8) / 9
+                && rect.width() < (availableGeometry.width() * 8) / 9) {
+                const QWindow *tp = w->transientParent();
+                if (tp) {
+                    // A transient window should be centered w.r.t. its transient parent.
+                    rect.moveCenter(tp->geometry().center());
+                } else {
+                    // Center the window on the screen.  (Only applicable on platforms
+                    // which do not provide a better way.)
+                    rect.moveCenter(availableGeometry.center());
+                }
+            }
+        }
+    }
+    return rect;
+}
+
 /*!
     \class QPlatformWindow
     \since 4.8
