@@ -55,8 +55,6 @@
 
 #include <QtCore/qrect.h>
 
-QT_BEGIN_HEADER
-
 QT_BEGIN_NAMESPACE
 
 class QPlatformWindowPrivate
@@ -67,6 +65,4 @@ public:
 
 QT_END_NAMESPACE
 
-QT_END_HEADER
-
 #endif // QPLATFORMWINDOW_P_H
