@@ -317,13 +317,16 @@ QKeySequence::SequenceMatch QShortcutMap::state()
 /*! \internal
     Uses ShortcutOverride event to see if any widgets want to override
     the event. If not, uses nextState(QKeyEvent) to check for a grabbed
-    Shortcut, and dispatchEvent() is found an identical.
+    Shortcut, and dispatchEvent() is found and identical.
     \sa nextState, dispatchEvent
 */
 bool QShortcutMap::tryShortcutEvent(QObject *o, QKeyEvent *e)
 {
     Q_D(QShortcutMap);
 
+    if (e->key() == Qt::Key_unknown)
+        return false;
+
     bool wasAccepted = e->isAccepted();
     bool wasSpontaneous = e->spont;
     if (d->currentState == QKeySequence::NoMatch) {
