@@ -119,7 +119,7 @@ public:
     the size of the alpha buffer; the size of the depth and stencil buffers;
     and number of samples per pixel for multisampling. In addition, the format
     contains surface configuration parameters such as OpenGL profile and
-    version for rendering, whether or not enable stereo buffers, and swap
+    version for rendering, whether or not to enable stereo buffers, and swap
     behaviour.
 */
 
@@ -254,7 +254,7 @@ QSurfaceFormat::~QSurfaceFormat()
 /*!
     \fn bool QSurfaceFormat::stereo() const
 
-    Returns true if stereo buffering is enabled; otherwise returns
+    Returns \c true if stereo buffering is enabled; otherwise returns
     false. Stereo buffering is disabled by default.
 
     \sa setStereo()
@@ -325,7 +325,7 @@ void QSurfaceFormat::setOption(QSurfaceFormat::FormatOptions opt)
 }
 
 /*!
-    Returns true if format option \a opt is set; otherwise returns false.
+    Returns \c true if format option \a opt is set; otherwise returns \c false.
 
     \sa setOption()
 */
@@ -337,7 +337,7 @@ bool QSurfaceFormat::testOption(QSurfaceFormat::FormatOptions opt) const
 /*!
     Set the minimum depth buffer size to \a size.
 
-    \sa depthBufferSize(), setDepth(), depth()
+    \sa depthBufferSize()
 */
 void QSurfaceFormat::setDepthBufferSize(int size)
 {
@@ -350,7 +350,7 @@ void QSurfaceFormat::setDepthBufferSize(int size)
 /*!
     Returns the depth buffer size.
 
-    \sa setDepthBufferSize(), setDepth(), depth()
+    \sa setDepthBufferSize()
 */
 int QSurfaceFormat::depthBufferSize() const
 {
@@ -361,7 +361,7 @@ int QSurfaceFormat::depthBufferSize() const
     Set the swap \a behavior of the surface.
 
     The swap behavior specifies whether single, double, or triple
-    buffering is desired. The default, SwapBehavior::DefaultSwapBehavior,
+    buffering is desired. The default, DefaultSwapBehavior,
     gives the default swap behavior of the platform.
 */
 void QSurfaceFormat::setSwapBehavior(SwapBehavior behavior)
@@ -383,7 +383,7 @@ QSurfaceFormat::SwapBehavior QSurfaceFormat::swapBehavior() const
 }
 
 /*!
-    Returns true if the alpha buffer size is greater than zero.
+    Returns \c true if the alpha buffer size is greater than zero.
 
     This means that the surface might be used with per pixel
     translucency effects.
@@ -396,7 +396,7 @@ bool QSurfaceFormat::hasAlpha() const
 /*!
     Set the preferred stencil buffer size to \a size bits.
 
-    \sa stencilBufferSize(), setStencil(), stencil()
+    \sa stencilBufferSize()
 */
 void QSurfaceFormat::setStencilBufferSize(int size)
 {
@@ -409,7 +409,7 @@ void QSurfaceFormat::setStencilBufferSize(int size)
 /*!
     Returns the stencil buffer size in bits.
 
-    \sa stencil(), setStencil(), setStencilBufferSize()
+    \sa setStencilBufferSize()
 */
 int QSurfaceFormat::stencilBufferSize() const
 {
@@ -583,7 +583,31 @@ int QSurfaceFormat::minorVersion() const
 }
 
 /*!
-    Returns true if all the options of the two QSurfaceFormat objects
+    Returns a QPair<int, int> representing the OpenGL version.
+
+    Useful for version checks, for example format.version() >= qMakePair(3, 2)
+*/
+QPair<int, int> QSurfaceFormat::version() const
+{
+    return qMakePair(d->major, d->minor);
+}
+
+/*!
+    Sets the desired \a major and \a minor OpenGL versions.
+
+    The default version is 2.0.
+*/
+void QSurfaceFormat::setVersion(int major, int minor)
+{
+    if (d->minor != minor || d->major != major) {
+        detach();
+        d->minor = minor;
+        d->major = major;
+    }
+}
+
+/*!
+    Returns \c true if all the options of the two QSurfaceFormat objects
     \a a and \a b are equal.
 
     \relates QSurfaceFormat
@@ -605,8 +629,8 @@ bool operator==(const QSurfaceFormat& a, const QSurfaceFormat& b)
 }
 
 /*!
-    Returns false if all the options of the two QSurfaceFormat objects
-    \a a and \a b are equal; otherwise returns true.
+    Returns \c false if all the options of the two QSurfaceFormat objects
+    \a a and \a b are equal; otherwise returns \c true.
 
     \relates QSurfaceFormat
 */
