@@ -139,6 +139,16 @@ QTouchDevice::Capabilities QTouchDevice::capabilities() const
     return d->caps;
 }
 
+/*!
+    Returns the maximum number of simultaneous touch points (fingers) that
+    can be detected.
+    \since 5.2
+  */
+int QTouchDevice::maximumTouchPoints() const
+{
+    return d->maxTouchPoints;
+}
+
 /*!
     Returns the touch device name.
 
@@ -168,6 +178,15 @@ void QTouchDevice::setCapabilities(Capabilities caps)
     d->caps = caps;
 }
 
+/*!
+  Sets the maximum number of simultaneous touchpoints \a max
+  supported by the device and its driver.
+  */
+void QTouchDevice::setMaximumTouchPoints(int max)
+{
+    d->maxTouchPoints = max;
+}
+
 /*!
   Sets the \a name (a unique identifier) for the device. In most systems it is
   enough to leave this unset and keep the default empty name. This identifier
