@@ -56,8 +56,6 @@
 #include <QtCore/qobject.h>
 #include <QtGui/qtouchdevice.h>
 
-QT_BEGIN_HEADER
-
 QT_BEGIN_NAMESPACE
 
 
@@ -66,12 +64,14 @@ class QTouchDevicePrivate
 public:
     QTouchDevicePrivate()
         : type(QTouchDevice::TouchScreen),
-          caps(QTouchDevice::Position)
+          caps(QTouchDevice::Position),
+          maxTouchPoints(1)
     { }
 
     QTouchDevice::DeviceType type;
     QTouchDevice::Capabilities caps;
     QString name;
+    int maxTouchPoints;
 
     static void registerDevice(QTouchDevice *dev);
     static bool isRegistered(QTouchDevice *dev);
@@ -79,6 +79,4 @@ public:
 
 QT_END_NAMESPACE
 
-QT_END_HEADER
-
 #endif // QTOUCHDEVICE_P_H
