@@ -62,8 +62,6 @@
 #include <QtCore/QEventLoop>
 #include <QtGui/QVector2D>
 
-QT_BEGIN_HEADER
-
 QT_BEGIN_NAMESPACE
 
 class QMimeData;
@@ -105,8 +103,8 @@ public:
                                        quint32 nativeModifiers,
                                        const QString& text = QString(), bool autorep = false,
                                        ushort count = 1);
-    static void handleWheelEvent(QWindow *w, const QPointF & local, const QPointF & global, QPoint pixelDelta, QPoint angleDelta, Qt::KeyboardModifiers mods = Qt::NoModifier);
-    static void handleWheelEvent(QWindow *w, ulong timestamp, const QPointF & local, const QPointF & global, QPoint pixelDelta, QPoint angleDelta, Qt::KeyboardModifiers mods = Qt::NoModifier);
+    static void handleWheelEvent(QWindow *w, const QPointF & local, const QPointF & global, QPoint pixelDelta, QPoint angleDelta, Qt::KeyboardModifiers mods = Qt::NoModifier, Qt::ScrollPhase phase = Qt::ScrollUpdate);
+    static void handleWheelEvent(QWindow *w, ulong timestamp, const QPointF & local, const QPointF & global, QPoint pixelDelta, QPoint angleDelta, Qt::KeyboardModifiers mods = Qt::NoModifier, Qt::ScrollPhase phase = Qt::ScrollUpdate);
 
     // Wheel event compatibility functions. Will be removed: do not use.
     static void handleWheelEvent(QWindow *w, const QPointF & local, const QPointF & global, int d, Qt::Orientation o, Qt::KeyboardModifiers mods = Qt::NoModifier);
@@ -132,13 +130,17 @@ public:
     static void handleTouchCancelEvent(QWindow *w, QTouchDevice *device, Qt::KeyboardModifiers mods = Qt::NoModifier);
     static void handleTouchCancelEvent(QWindow *w, ulong timestamp, QTouchDevice *device, Qt::KeyboardModifiers mods = Qt::NoModifier);
 
-    static void handleGeometryChange(QWindow *w, const QRect &newRect);
-    static void handleCloseEvent(QWindow *w);
+    static void handleGeometryChange(QWindow *w, const QRect &newRect, const QRect &oldRect = QRect());
+    static void handleCloseEvent(QWindow *w, bool *accepted = 0);
     static void handleEnterEvent(QWindow *w, const QPointF &local = QPointF(), const QPointF& global = QPointF());
     static void handleLeaveEvent(QWindow *w);
     static void handleEnterLeaveEvent(QWindow *enter, QWindow *leave, const QPointF &local = QPointF(), const QPointF& global = QPointF());
-    static void handleWindowActivated(QWindow *w);
+    static void handleWindowActivated(QWindow *w, Qt::FocusReason r = Qt::OtherFocusReason);
+
     static void handleWindowStateChanged(QWindow *w, Qt::WindowState newState);
+    static void handleWindowScreenChanged(QWindow *w, QScreen *newScreen);
+
+    static void handleApplicationStateChanged(Qt::ApplicationState newState);
 
     static void handleExposeEvent(QWindow *tlw, const QRegion &region);
 
@@ -160,6 +162,7 @@ public:
     static void handleThemeChange(QWindow *tlw);
 
     static void handleFileOpenEvent(const QString& fileName);
+    static void handleFileOpenEvent(const QUrl &url);
 
     static void handleTabletEvent(QWindow *w, ulong timestamp, bool down, const QPointF &local, const QPointF &global,
                                   int device, int pointerType, qreal pressure, int xTilt, int yTilt,
@@ -174,23 +177,37 @@ public:
     static void handleTabletLeaveProximityEvent(ulong timestamp, int device, int pointerType, qint64 uid);
     static void handleTabletLeaveProximityEvent(int device, int pointerType, qint64 uid);
 
+#ifndef QT_NO_GESTURES
+    static void handleGestureEvent(QWindow *window,  ulong timestamp, Qt::NativeGestureType type,
+                                   QPointF &local, QPointF &global);
+    static void handleGestureEventWithRealValue(QWindow *window,  ulong timestamp, Qt::NativeGestureType type,
+                                                qreal value, QPointF &local, QPointF &global);
+    static void handleGestureEventWithSequenceIdAndValue(QWindow *window, ulong timestamp,Qt::NativeGestureType type,
+                                                         ulong sequenceId, quint64 value, QPointF &local, QPointF &global);
+#endif // QT_NO_GESTURES
+
     static void handlePlatformPanelEvent(QWindow *w);
 #ifndef QT_NO_CONTEXTMENU
     static void handleContextMenuEvent(QWindow *w, bool mouseTriggered,
                                        const QPoint &pos, const QPoint &globalPos,
                                        Qt::KeyboardModifiers modifiers);
 #endif
+#ifndef QT_NO_WHATSTHIS
+    static void handleEnterWhatsThisEvent();
+#endif
 
     // For event dispatcher implementations
     static bool sendWindowSystemEvents(QEventLoop::ProcessEventsFlags flags);
     static void setSynchronousWindowsSystemEvents(bool enable);
     static void flushWindowSystemEvents();
+    static void deferredFlushWindowSystemEvents();
     static int windowSystemEventsQueued();
-
-private:
-    static bool sendWindowSystemEventsImplementation(QEventLoop::ProcessEventsFlags flags);
 };
 
+#ifndef QT_NO_DEBUG_STREAM
+Q_GUI_EXPORT QDebug operator<<(QDebug dbg, const QWindowSystemInterface::TouchPoint &p);
+#endif
+
 QT_END_NAMESPACE
-QT_END_HEADER
+
 #endif // QWINDOWSYSTEMINTERFACE_H
