@@ -58,8 +58,7 @@
 #include <QPointer>
 #include <QMutex>
 #include <QList>
-
-QT_BEGIN_HEADER
+#include <QWaitCondition>
 
 QT_BEGIN_NAMESPACE
 
@@ -90,7 +89,14 @@ public:
         TabletEnterProximity = UserInputEvent | 0x15,
         TabletLeaveProximity = UserInputEvent | 0x16,
         PlatformPanel = UserInputEvent | 0x17,
-        ContextMenu = UserInputEvent | 0x18
+        ContextMenu = UserInputEvent | 0x18,
+        EnterWhatsThisMode = UserInputEvent | 0x19,
+#ifndef QT_NO_GESTURES
+        Gesture = UserInputEvent | 0x1a,
+#endif
+        ApplicationStateChanged = 0x19,
+        FlushEvents = 0x20,
+        WindowScreenChanged = 0x21
     };
 
     class WindowSystemEvent {
@@ -104,18 +110,21 @@ public:
 
     class CloseEvent : public WindowSystemEvent {
     public:
-        explicit CloseEvent(QWindow *w)
-            : WindowSystemEvent(Close), window(w) { }
+        explicit CloseEvent(QWindow *w, bool *a = 0)
+            : WindowSystemEvent(Close), window(w), accepted(a)
+            { }
         QPointer<QWindow> window;
+        bool *accepted;
     };
 
     class GeometryChangeEvent : public WindowSystemEvent {
     public:
-        GeometryChangeEvent(QWindow *tlw, const QRect &newGeometry)
-            : WindowSystemEvent(GeometryChange), tlw(tlw), newGeometry(newGeometry)
+        GeometryChangeEvent(QWindow *tlw, const QRect &newGeometry, const QRect &oldGeometry)
+            : WindowSystemEvent(GeometryChange), tlw(tlw), newGeometry(newGeometry), oldGeometry(oldGeometry)
         { }
         QPointer<QWindow> tlw;
         QRect newGeometry;
+        QRect oldGeometry;
     };
 
     class EnterEvent : public WindowSystemEvent {
@@ -138,10 +147,11 @@ public:
 
     class ActivatedWindowEvent : public WindowSystemEvent {
     public:
-        explicit ActivatedWindowEvent(QWindow *activatedWindow)
-            : WindowSystemEvent(ActivatedWindow), activated(activatedWindow)
+        explicit ActivatedWindowEvent(QWindow *activatedWindow, Qt::FocusReason r)
+            : WindowSystemEvent(ActivatedWindow), activated(activatedWindow), reason(r)
         { }
         QPointer<QWindow> activated;
+        Qt::FocusReason reason;
     };
 
     class WindowStateChangedEvent : public WindowSystemEvent {
@@ -154,6 +164,32 @@ public:
         Qt::WindowState newState;
     };
 
+    class WindowScreenChangedEvent : public WindowSystemEvent {
+    public:
+        WindowScreenChangedEvent(QWindow *w, QScreen *s)
+            : WindowSystemEvent(WindowScreenChanged), window(w), screen(s)
+        { }
+
+        QPointer<QWindow> window;
+        QPointer<QScreen> screen;
+    };
+
+    class ApplicationStateChangedEvent : public WindowSystemEvent {
+    public:
+        ApplicationStateChangedEvent(Qt::ApplicationState newState)
+            : WindowSystemEvent(ApplicationStateChanged), newState(newState)
+        { }
+
+        Qt::ApplicationState newState;
+    };
+
+    class FlushEventsEvent : public WindowSystemEvent {
+    public:
+        FlushEventsEvent()
+            : WindowSystemEvent(FlushEvents)
+        { }
+    };
+
     class UserEvent : public WindowSystemEvent {
     public:
         UserEvent(QWindow * w, ulong time, EventType t)
@@ -186,14 +222,15 @@ public:
     class WheelEvent : public InputEvent {
     public:
         WheelEvent(QWindow *w, ulong time, const QPointF & local, const QPointF & global, QPoint pixelD, QPoint angleD, int qt4D, Qt::Orientation qt4O,
-                   Qt::KeyboardModifiers mods)
-            : InputEvent(w, time, Wheel, mods), pixelDelta(pixelD), angleDelta(angleD), qt4Delta(qt4D), qt4Orientation(qt4O), localPos(local), globalPos(global) { }
+                   Qt::KeyboardModifiers mods, Qt::ScrollPhase phase = Qt::ScrollUpdate)
+            : InputEvent(w, time, Wheel, mods), pixelDelta(pixelD), angleDelta(angleD), qt4Delta(qt4D), qt4Orientation(qt4O), localPos(local), globalPos(global), phase(phase) { }
         QPoint pixelDelta;
         QPoint angleDelta;
         int qt4Delta;
         Qt::Orientation qt4Orientation;
         QPointF localPos;
         QPointF globalPos;
+        Qt::ScrollPhase phase;
     };
 
     class KeyEvent : public InputEvent {
@@ -287,9 +324,12 @@ public:
     class FileOpenEvent : public WindowSystemEvent {
     public:
         FileOpenEvent(const QString& fileName)
-            : WindowSystemEvent(FileOpen), fileName(fileName)
+            : WindowSystemEvent(FileOpen), url(QUrl::fromLocalFile(fileName))
         { }
-        QString fileName;
+        FileOpenEvent(const QUrl &url)
+            : WindowSystemEvent(FileOpen), url(url)
+        { }
+        QUrl url;
     };
 
     class TabletEvent : public InputEvent {
@@ -362,14 +402,32 @@ public:
     };
 #endif
 
+#ifndef QT_NO_GESTURES
+    class GestureEvent : public InputEvent {
+    public:
+        GestureEvent(QWindow *window, ulong time, Qt::NativeGestureType type, QPointF pos, QPointF globalPos)
+            : InputEvent(window, time, Gesture, Qt::NoModifier), type(type), pos(pos), globalPos(globalPos),
+              realValue(0), sequenceId(0), intValue(0) { }
+        Qt::NativeGestureType type;
+        QPointF pos;
+        QPointF globalPos;
+        // Mac
+        qreal realValue;
+        // Windows
+        ulong sequenceId;
+        quint64 intValue;
+    };
+#endif
+
     class WindowSystemEventList {
         QList<WindowSystemEvent *> impl;
         mutable QMutex mutex;
     public:
         WindowSystemEventList() : impl(), mutex() {}
-        ~WindowSystemEventList()
-        { const QMutexLocker locker(&mutex); qDeleteAll(impl); impl.clear(); }
+        ~WindowSystemEventList() { clear(); }
 
+        void clear()
+        { const QMutexLocker locker(&mutex); qDeleteAll(impl); impl.clear(); }
         void prepend(WindowSystemEvent *e)
         { const QMutexLocker locker(&mutex); impl.prepend(e); }
         WindowSystemEvent *takeFirstOrReturnNull()
@@ -400,13 +458,13 @@ public:
             const QMutexLocker locker(&mutex);
             for (int i = 0; i < impl.size(); ++i) {
                 if (impl.at(i) == e) {
-                    impl.removeAt(i);
+                    delete impl.takeAt(i);
                     break;
                 }
             }
         }
     private:
-        Q_DISABLE_COPY(WindowSystemEventList);
+        Q_DISABLE_COPY(WindowSystemEventList)
     };
 
     static WindowSystemEventList windowSystemEventQueue;
@@ -421,10 +479,12 @@ public:
     static QElapsedTimer eventTime;
     static bool synchronousWindowsSystemEvents;
 
+    static QWaitCondition eventsFlushed;
+    static QMutex flushEventMutex;
+
     static QList<QTouchEvent::TouchPoint> convertTouchPoints(const QList<QWindowSystemInterface::TouchPoint> &points, QEvent::Type *type);
 };
 
-QT_END_HEADER
 QT_END_NAMESPACE
 
 #endif // QWINDOWSYSTEMINTERFACE_P_H
