@@ -68,7 +68,7 @@ QT_BEGIN_NAMESPACE
 /*!
     \fn QVector2D::QVector2D()
 
-    Constructs a null vector, i.e. with coordinates (0, 0, 0).
+    Constructs a null vector, i.e. with coordinates (0, 0).
 */
 
 /*!
@@ -124,8 +124,8 @@ QVector2D::QVector2D(const QVector4D& vector)
 /*!
     \fn bool QVector2D::isNull() const
 
-    Returns true if the x and y coordinates are set to 0.0,
-    otherwise returns false.
+    Returns \c true if the x and y coordinates are set to 0.0,
+    otherwise returns \c false.
 */
 
 /*!
@@ -160,6 +160,25 @@ QVector2D::QVector2D(const QVector4D& vector)
     \sa y(), setX()
 */
 
+/*! \fn float &QVector2D::operator[](int i)
+    \since 5.2
+
+    Returns the component of the vector at index position \a i
+    as a modifiable reference.
+
+    \a i must be a valid index position in the vector (i.e., 0 <= \a i
+    < 2).
+*/
+
+/*! \fn float QVector2D::operator[](int i) const
+    \since 5.2
+
+    Returns the component of the vector at index position \a i.
+
+    \a i must be a valid index position in the vector (i.e., 0 <= \a i
+    < 2).
+*/
+
 /*!
     Returns the length of the vector from the origin.
 
@@ -228,6 +247,39 @@ void QVector2D::normalize()
     yp = float(double(yp) / len);
 }
 
+/*!
+    \since 5.1
+
+    Returns the distance from this vertex to a point defined by
+    the vertex \a point.
+
+    \sa distanceToLine()
+*/
+float QVector2D::distanceToPoint(const QVector2D& point) const
+{
+    return (*this - point).length();
+}
+
+/*!
+    \since 5.1
+
+    Returns the distance that this vertex is from a line defined
+    by \a point and the unit vector \a direction.
+
+    If \a direction is a null vector, then it does not define a line.
+    In that case, the distance from \a point to this vertex is returned.
+
+    \sa distanceToPoint()
+*/
+float QVector2D::distanceToLine
+        (const QVector2D& point, const QVector2D& direction) const
+{
+    if (direction.isNull())
+        return (*this - point).length();
+    QVector2D p = point + dotProduct(*this - point, direction) * direction;
+    return (*this - p).length();
+}
+
 /*!
     \fn QVector2D &QVector2D::operator+=(const QVector2D &vector)
 
@@ -283,7 +335,7 @@ float QVector2D::dotProduct(const QVector2D& v1, const QVector2D& v2)
     \fn bool operator==(const QVector2D &v1, const QVector2D &v2)
     \relates QVector2D
 
-    Returns true if \a v1 is equal to \a v2; otherwise returns false.
+    Returns \c true if \a v1 is equal to \a v2; otherwise returns \c false.
     This operator uses an exact floating-point comparison.
 */
 
@@ -291,7 +343,7 @@ float QVector2D::dotProduct(const QVector2D& v1, const QVector2D& v2)
     \fn bool operator!=(const QVector2D &v1, const QVector2D &v2)
     \relates QVector2D
 
-    Returns true if \a v1 is not equal to \a v2; otherwise returns false.
+    Returns \c true if \a v1 is not equal to \a v2; otherwise returns \c false.
     This operator uses an exact floating-point comparison.
 */
 
@@ -366,7 +418,7 @@ float QVector2D::dotProduct(const QVector2D& v1, const QVector2D& v2)
     \fn bool qFuzzyCompare(const QVector2D& v1, const QVector2D& v2)
     \relates QVector2D
 
-    Returns true if \a v1 and \a v2 are equal, allowing for a small
+    Returns \c true if \a v1 and \a v2 are equal, allowing for a small
     fuzziness factor for floating-point comparisons; false otherwise.
 */
 
