@@ -144,8 +144,8 @@ QVector3D::QVector3D(const QVector4D& vector)
 /*!
     \fn bool QVector3D::isNull() const
 
-    Returns true if the x, y, and z coordinates are set to 0.0,
-    otherwise returns false.
+    Returns \c true if the x, y, and z coordinates are set to 0.0,
+    otherwise returns \c false.
 */
 
 /*!
@@ -196,6 +196,25 @@ QVector3D::QVector3D(const QVector4D& vector)
     \sa z(), setX(), setY()
 */
 
+/*! \fn float &QVector3D::operator[](int i)
+    \since 5.2
+
+    Returns the component of the vector at index position \a i
+    as a modifiable reference.
+
+    \a i must be a valid index position in the vector (i.e., 0 <= \a i
+    < 3).
+*/
+
+/*! \fn float QVector3D::operator[](int i) const
+    \since 5.2
+
+    Returns the component of the vector at index position \a i.
+
+    \a i must be a valid index position in the vector (i.e., 0 <= \a i
+    < 3).
+*/
+
 /*!
     Returns the normalized unit vector form of this vector.
 
@@ -347,6 +366,19 @@ QVector3D QVector3D::normal
     return crossProduct((v2 - v1), (v3 - v1)).normalized();
 }
 
+/*!
+    \since 5.1
+
+    Returns the distance from this vertex to a point defined by
+    the vertex \a point.
+
+    \sa distanceToPlane(), distanceToLine()
+*/
+float QVector3D::distanceToPoint(const QVector3D& point) const
+{
+    return (*this - point).length();
+}
+
 /*!
     Returns the distance from this vertex to a plane defined by
     the vertex \a plane and a \a normal unit vector.  The \a normal
@@ -406,7 +438,7 @@ float QVector3D::distanceToLine
     \fn bool operator==(const QVector3D &v1, const QVector3D &v2)
     \relates QVector3D
 
-    Returns true if \a v1 is equal to \a v2; otherwise returns false.
+    Returns \c true if \a v1 is equal to \a v2; otherwise returns \c false.
     This operator uses an exact floating-point comparison.
 */
 
@@ -414,7 +446,7 @@ float QVector3D::distanceToLine
     \fn bool operator!=(const QVector3D &v1, const QVector3D &v2)
     \relates QVector3D
 
-    Returns true if \a v1 is not equal to \a v2; otherwise returns false.
+    Returns \c true if \a v1 is not equal to \a v2; otherwise returns \c false.
     This operator uses an exact floating-point comparison.
 */
 
@@ -492,7 +524,7 @@ float QVector3D::distanceToLine
     \fn bool qFuzzyCompare(const QVector3D& v1, const QVector3D& v2)
     \relates QVector3D
 
-    Returns true if \a v1 and \a v2 are equal, allowing for a small
+    Returns \c true if \a v1 and \a v2 are equal, allowing for a small
     fuzziness factor for floating-point comparisons; false otherwise.
 */
 
