@@ -45,8 +45,6 @@
 #include <QtCore/qpoint.h>
 #include <QtCore/qmetatype.h>
 
-QT_BEGIN_HEADER
-
 QT_BEGIN_NAMESPACE
 
 
@@ -81,6 +79,9 @@ public:
     void setY(float y);
     void setZ(float z);
 
+    float &operator[](int i);
+    float operator[](int i) const;
+
     float length() const;
     float lengthSquared() const;
 
@@ -99,6 +100,7 @@ public:
     static QVector3D normal
         (const QVector3D& v1, const QVector3D& v2, const QVector3D& v3);
 
+    float distanceToPoint(const QVector3D& point) const;
     float distanceToPlane(const QVector3D& plane, const QVector3D& normal) const;
     float distanceToPlane(const QVector3D& plane1, const QVector3D& plane2, const QVector3D& plane3) const;
     float distanceToLine(const QVector3D& point, const QVector3D& direction) const;
@@ -161,6 +163,18 @@ inline void QVector3D::setX(float aX) { xp = aX; }
 inline void QVector3D::setY(float aY) { yp = aY; }
 inline void QVector3D::setZ(float aZ) { zp = aZ; }
 
+inline float &QVector3D::operator[](int i)
+{
+    Q_ASSERT(uint(i) < 3u);
+    return *(&xp + i);
+}
+
+inline float QVector3D::operator[](int i) const
+{
+    Q_ASSERT(uint(i) < 3u);
+    return *(&xp + i);
+}
+
 inline QVector3D &QVector3D::operator+=(const QVector3D &vector)
 {
     xp += vector.xp;
@@ -276,6 +290,4 @@ Q_GUI_EXPORT QDataStream &operator>>(QDataStream &, QVector3D &);
 
 QT_END_NAMESPACE
 
-QT_END_HEADER
-
 #endif
