@@ -259,7 +259,7 @@ namespace {
 }
 
 /*!
-    Creates the buffer object in the OpenGL server.  Returns true if
+    Creates the buffer object in the OpenGL server.  Returns \c true if
     the object was created; false otherwise.
 
     This function must be called with a current QOpenGLContext.
@@ -294,7 +294,7 @@ bool QOpenGLBuffer::create()
 }
 
 /*!
-    Returns true if this buffer has been created; false otherwise.
+    Returns \c true if this buffer has been created; false otherwise.
 
     \sa create(), destroy()
 */
@@ -316,11 +316,13 @@ void QOpenGLBuffer::destroy()
         d->guard->free();
         d->guard = 0;
     }
+    delete d->funcs;
+    d->funcs = 0;
 }
 
 /*!
     Reads the \a count bytes in this buffer starting at \a offset
-    into \a data.  Returns true on success; false if reading from
+    into \a data.  Returns \c true on success; false if reading from
     the buffer is not supported.  Buffer reading is not supported
     under OpenGL/ES.
 
@@ -401,7 +403,7 @@ void QOpenGLBuffer::allocate(const void *data, int count)
 
 /*!
     Binds the buffer associated with this object to the current
-    OpenGL context.  Returns false if binding was not possible, usually because
+    OpenGL context.  Returns \c false if binding was not possible, usually because
     type() is not supported on this OpenGL implementation.
 
     The buffer must be bound to the same QOpenGLContext current when create()
@@ -531,7 +533,7 @@ void *QOpenGLBuffer::map(QOpenGLBuffer::Access access)
 
 /*!
     Unmaps the buffer after it was mapped into the application's
-    memory space with a previous call to map().  Returns true if
+    memory space with a previous call to map().  Returns \c true if
     the unmap succeeded; false otherwise.
 
     It is assumed that this buffer has been bound to the current context,
