@@ -56,8 +56,6 @@
 
 #include "qopenglengineshadermanager_p.h"
 
-QT_BEGIN_HEADER
-
 QT_BEGIN_NAMESPACE
 
 
@@ -240,7 +238,7 @@ static const char* const qopenglslPositionWithRadialGradientBrushVertexShader =
     uniform   mediump vec2      halfViewportSize; \n\
     uniform   highp   mat3      brushTransform; \n\
     uniform   highp   vec2      fmp; \n\
-    uniform   highp   vec3      bradius; \n\
+    uniform   mediump vec3      bradius; \n\
     varying   highp   float     b; \n\
     varying   highp   vec2      A; \n\
     void setPosition(void) \n\
@@ -266,7 +264,7 @@ static const char* const qopenglslRadialGradientBrushSrcFragmentShader = "\n\
     uniform   highp   float     sqrfr; \n\
     varying   highp   float     b; \n\
     varying   highp   vec2      A; \n\
-    uniform   highp   vec3      bradius; \n\
+    uniform   mediump vec3      bradius; \n\
     lowp vec4 srcPixel() \n\
     { \n\
         highp float c = sqrfr-dot(A, A); \n\
@@ -523,6 +521,4 @@ static const char* const qopenglslRgbMaskFragmentShaderPass2 = "\n\
 
 QT_END_NAMESPACE
 
-QT_END_HEADER
-
 #endif // GLGC_SHADER_SOURCE_H
