@@ -238,7 +238,7 @@ void QOpenGLFramebufferObjectFormat::setMipmap(bool enabled)
 }
 
 /*!
-    Returns true if mipmapping is enabled.
+    Returns \c true if mipmapping is enabled.
 
     \sa setMipmap()
 */
@@ -320,8 +320,8 @@ GLenum QOpenGLFramebufferObjectFormat::internalTextureFormat() const
 }
 
 /*!
-    Returns true if all the options of this framebuffer object format
-    are the same as \a other; otherwise returns false.
+    Returns \c true if all the options of this framebuffer object format
+    are the same as \a other; otherwise returns \c false.
 */
 bool QOpenGLFramebufferObjectFormat::operator==(const QOpenGLFramebufferObjectFormat& other) const
 {
@@ -332,8 +332,8 @@ bool QOpenGLFramebufferObjectFormat::operator==(const QOpenGLFramebufferObjectFo
 }
 
 /*!
-    Returns false if all the options of this framebuffer object format
-    are the same as \a other; otherwise returns true.
+    Returns \c false if all the options of this framebuffer object format
+    are the same as \a other; otherwise returns \c true.
 */
 bool QOpenGLFramebufferObjectFormat::operator!=(const QOpenGLFramebufferObjectFormat& other) const
 {
@@ -425,6 +425,17 @@ void QOpenGLFramebufferObjectPrivate::init(QOpenGLFramebufferObject *, const QSi
     if (!funcs.hasOpenGLFeature(QOpenGLFunctions::Framebuffers))
         return;
 
+
+    // Fall back to using a normal non-msaa FBO if we don't have support for MSAA
+    if (!funcs.hasOpenGLExtension(QOpenGLExtensions::FramebufferMultisample)
+            || !funcs.hasOpenGLExtension(QOpenGLExtensions::FramebufferBlit)) {
+        samples = 0;
+    }
+
+    GLint maxSamples;
+    glGetIntegerv(GL_MAX_SAMPLES, &maxSamples);
+    samples = qBound(0, int(samples), int(maxSamples));
+
     size = sz;
     target = texture_target;
     // texture dimensions
@@ -473,25 +484,11 @@ void QOpenGLFramebufferObjectPrivate::init(QOpenGLFramebufferObject *, const QSi
         color_buffer = 0;
     } else {
         mipmap = false;
-        GLint maxSamples;
-        glGetIntegerv(GL_MAX_SAMPLES, &maxSamples);
-
-        samples = qBound(0, int(samples), int(maxSamples));
-
         funcs.glGenRenderbuffers(1, &color_buffer);
         funcs.glBindRenderbuffer(GL_RENDERBUFFER, color_buffer);
-        if (funcs.hasOpenGLExtension(QOpenGLExtensions::FramebufferMultisample) && samples > 0) {
-            funcs.glRenderbufferStorageMultisample(GL_RENDERBUFFER, samples,
-                internal_format, size.width(), size.height());
-        } else {
-            samples = 0;
-            funcs.glRenderbufferStorage(GL_RENDERBUFFER, internal_format,
-                size.width(), size.height());
-        }
-
+        funcs.glRenderbufferStorageMultisample(GL_RENDERBUFFER, samples, internal_format, size.width(), size.height());
         funcs.glFramebufferRenderbuffer(GL_FRAMEBUFFER, GL_COLOR_ATTACHMENT0,
                                              GL_RENDERBUFFER, color_buffer);
-
         QT_CHECK_GLERROR();
         valid = checkFramebufferStatus(ctx);
 
@@ -880,7 +877,7 @@ QOpenGLFramebufferObject::~QOpenGLFramebufferObject()
 /*!
     \fn bool QOpenGLFramebufferObject::isValid() const
 
-    Returns true if the framebuffer object is valid.
+    Returns \c true if the framebuffer object is valid.
 
     The framebuffer can become invalid if the initialization process
     fails, the user attaches an invalid buffer to the framebuffer
@@ -906,7 +903,7 @@ bool QOpenGLFramebufferObject::isValid() const
 
     Switches rendering from the default, windowing system provided
     framebuffer to this framebuffer object.
-    Returns true upon success, false otherwise.
+    Returns \c true upon success, false otherwise.
 
     \sa release()
 */
@@ -934,7 +931,7 @@ bool QOpenGLFramebufferObject::bind()
 
     Switches rendering back to the default, windowing system provided
     framebuffer.
-    Returns true upon success, false otherwise.
+    Returns \c true upon success, false otherwise.
 
     \sa bind()
 */
@@ -1012,23 +1009,28 @@ QOpenGLFramebufferObjectFormat QOpenGLFramebufferObject::format() const
 
 Q_GUI_EXPORT QImage qt_gl_read_framebuffer(const QSize &size, bool alpha_format, bool include_alpha)
 {
-    QImage img(size, (alpha_format && include_alpha) ? QImage::Format_ARGB32_Premultiplied
-                                                     : QImage::Format_RGB32);
     int w = size.width();
     int h = size.height();
 
+    while (glGetError());
+
+#if Q_BYTE_ORDER == Q_LITTLE_ENDIAN
+    QImage img(size, (alpha_format && include_alpha) ? QImage::Format_ARGB32_Premultiplied
+                                                     : QImage::Format_RGB32);
 #ifdef QT_OPENGL_ES
     GLint fmt = GL_BGRA_EXT;
 #else
     GLint fmt = GL_BGRA;
 #endif
-    while (glGetError());
     glReadPixels(0, 0, w, h, fmt, GL_UNSIGNED_BYTE, img.bits());
-    if (glGetError()) {
-        glReadPixels(0, 0, w, h, GL_RGBA, GL_UNSIGNED_BYTE, img.bits());
-        img = img.rgbSwapped();
-    }
-    return img.mirrored();
+    if (!glGetError())
+        return img.mirrored();
+#endif
+
+    QImage rgbaImage(size, (alpha_format && include_alpha) ? QImage::Format_RGBA8888_Premultiplied
+                                                           : QImage::Format_RGBX8888);
+    glReadPixels(0, 0, w, h, GL_RGBA, GL_UNSIGNED_BYTE, rgbaImage.bits());
+    return rgbaImage.mirrored();
 }
 
 /*!
@@ -1064,11 +1066,10 @@ QImage QOpenGLFramebufferObject::toImage() const
 
 /*!
     \fn bool QOpenGLFramebufferObject::bindDefault()
-    \internal
 
     Switches rendering back to the default, windowing system provided
     framebuffer.
-    Returns true upon success, false otherwise.
+    Returns \c true upon success, false otherwise.
 
     \sa bind(), release()
 */
@@ -1092,8 +1093,8 @@ bool QOpenGLFramebufferObject::bindDefault()
 /*!
     \fn bool QOpenGLFramebufferObject::hasOpenGLFramebufferObjects()
 
-    Returns true if the OpenGL \c{GL_EXT_framebuffer_object} extension
-    is present on this system; otherwise returns false.
+    Returns \c true if the OpenGL \c{GL_EXT_framebuffer_object} extension
+    is present on this system; otherwise returns \c false.
 */
 bool QOpenGLFramebufferObject::hasOpenGLFramebufferObjects()
 {
@@ -1153,7 +1154,7 @@ void QOpenGLFramebufferObject::setAttachment(QOpenGLFramebufferObject::Attachmen
 }
 
 /*!
-    Returns true if the framebuffer object is currently bound to a context,
+    Returns \c true if the framebuffer object is currently bound to a context,
     otherwise false is returned.
 */
 
@@ -1167,8 +1168,8 @@ bool QOpenGLFramebufferObject::isBound() const
 /*!
     \fn bool QOpenGLFramebufferObject::hasOpenGLFramebufferBlit()
 
-    Returns true if the OpenGL \c{GL_EXT_framebuffer_blit} extension
-    is present on this system; otherwise returns false.
+    Returns \c true if the OpenGL \c{GL_EXT_framebuffer_blit} extension
+    is present on this system; otherwise returns \c false.
 
     \sa blitFramebuffer()
 */
