@@ -116,6 +116,7 @@ public:
 
     qreal dpmx;
     qreal dpmy;
+    qreal devicePixelRatio;
 
     bool flipped;
 
@@ -178,6 +179,7 @@ QOpenGLPaintDevicePrivate::QOpenGLPaintDevicePrivate(const QSize &sz)
     , ctx(QOpenGLContext::currentContext())
     , dpmx(qt_defaultDpiX() * 100. / 2.54)
     , dpmy(qt_defaultDpiY() * 100. / 2.54)
+    , devicePixelRatio(1.0)
     , flipped(false)
     , engine(0)
 {
@@ -248,6 +250,14 @@ void QOpenGLPaintDevice::setSize(const QSize &size)
     d_ptr->size = size;
 }
 
+/*!
+    Sets the device pixel ratio for the paint device to \a devicePixelRatio.
+*/
+void QOpenGLPaintDevice::setDevicePixelRatio(qreal devicePixelRatio)
+{
+    d_ptr->devicePixelRatio = devicePixelRatio;
+}
+
 /*!
     \reimp
 */
@@ -275,6 +285,8 @@ int QOpenGLPaintDevice::metric(QPaintDevice::PaintDeviceMetric metric) const
         return qRound(d_ptr->dpmx * 0.0254);
     case PdmPhysicalDpiY:
         return qRound(d_ptr->dpmy * 0.0254);
+    case PdmDevicePixelRatio:
+        return d_ptr->devicePixelRatio;
     default:
         qWarning("QOpenGLPaintDevice::metric() - metric %d not known", metric);
         return 0;
@@ -336,7 +348,7 @@ void QOpenGLPaintDevice::setPaintFlipped(bool flipped)
 }
 
 /*!
-    Returns true if painting is flipped around the Y-axis.
+    Returns \c true if painting is flipped around the Y-axis.
 
     \sa setPaintFlipped()
 */
