@@ -70,7 +70,8 @@ enum EngineMode {
     ImageDrawingMode,
     TextDrawingMode,
     BrushDrawingMode,
-    ImageArrayDrawingMode
+    ImageArrayDrawingMode,
+    ImageOpacityArrayDrawingMode
 };
 
 QT_BEGIN_NAMESPACE
@@ -157,7 +158,8 @@ public:
     void setRenderTextActive(bool);
 
     bool isNativePaintingActive() const;
-    bool supportsTransformations(QFontEngine *, const QTransform &) const { return true; }
+    bool requiresPretransformedGlyphPositions(QFontEngine *, const QTransform &) const { return false; }
+    bool shouldDrawCachedGlyphs(QFontEngine *, const QTransform &) const;
 
 private:
     Q_DISABLE_COPY(QOpenGL2PaintEngineEx)
@@ -235,6 +237,7 @@ public:
     void setBrush(const QBrush& brush);
     void transferMode(EngineMode newMode);
     bool prepareForDraw(bool srcPixelsAreOpaque); // returns true if the program has changed
+    bool prepareForCachedGlyphDraw(const QFontEngineGlyphCache &cache);
     inline void useSimpleShader();
     inline GLuint location(const QOpenGLEngineShaderManager::Uniform uniform) {
         return shaderManager->getUniformLocation(uniform);
