@@ -43,6 +43,8 @@
 #include "qopenglpaintengine_p.h"
 #include "private/qopenglengineshadersource_p.h"
 #include "qopenglextensions_p.h"
+#include <qrgb.h>
+#include <private/qdrawhelper_p.h>
 
 QT_BEGIN_NAMESPACE
 
@@ -120,7 +122,7 @@ void QOpenGLTextureGlyphCache::createTextureData(int width, int height)
     m_textureResource->m_width = width;
     m_textureResource->m_height = height;
 
-    if (m_type == QFontEngineGlyphCache::Raster_RGBMask) {
+    if (m_type == QFontEngineGlyphCache::Raster_RGBMask || m_type == QFontEngineGlyphCache::Raster_ARGB) {
         QVarLengthArray<uchar> data(width * height * 4);
         for (int i = 0; i < data.size(); ++i)
             data[i] = 0;
@@ -147,6 +149,9 @@ void QOpenGLTextureGlyphCache::resizeTextureData(int width, int height)
         return;
     }
 
+    GLint oldFbo;
+    glGetIntegerv(GL_FRAMEBUFFER_BINDING, &oldFbo);
+
     int oldWidth = m_textureResource->m_width;
     int oldHeight = m_textureResource->m_height;
 
@@ -265,7 +270,7 @@ void QOpenGLTextureGlyphCache::resizeTextureData(int width, int height)
     glDeleteTextures(1, &tmp_texture);
     glDeleteTextures(1, &oldTexture);
 
-    funcs.glBindFramebuffer(GL_FRAMEBUFFER, ctx->d_func()->current_fbo);
+    funcs.glBindFramebuffer(GL_FRAMEBUFFER, (GLuint)oldFbo);
 
     if (pex != 0) {
         glViewport(0, 0, pex->width, pex->height);
@@ -298,15 +303,20 @@ void QOpenGLTextureGlyphCache::fillTexture(const Coord &c, glyph_t glyph, QFixed
         return;
     }
 
-    QImage mask = textureMapForGlyph(glyph, subPixelPosition);
+    QImage mask;
+
+    if (m_current_fontengine->hasInternalCaching()) {
+        QImage *alphaMap = m_current_fontengine->lockedAlphaMapForGlyph(glyph, subPixelPosition, QFontEngine::Format_None);
+        if (!alphaMap || alphaMap->isNull())
+            return;
+        mask = alphaMap->copy();
+        m_current_fontengine->unlockAlphaMapForGlyph();
+    } else {
+        mask = textureMapForGlyph(glyph, subPixelPosition);
+    }
     const int maskWidth = mask.width();
     const int maskHeight = mask.height();
 
-#if defined(QT_OPENGL_ES_2)
-    QOpenGLExtensions extensions(ctx);
-    bool hasBGRA = extensions.hasOpenGLExtension(QOpenGLExtensions::BGRATextureFormat);
-#endif
-
     if (mask.format() == QImage::Format_Mono) {
         mask = mask.convertToFormat(QImage::Format_Indexed8);
         for (int y = 0; y < maskHeight; ++y) {
@@ -314,32 +324,40 @@ void QOpenGLTextureGlyphCache::fillTexture(const Coord &c, glyph_t glyph, QFixed
             for (int x = 0; x < maskWidth; ++x)
                 src[x] = -src[x]; // convert 0 and 1 into 0 and 255
         }
-    } else if (mask.format() == QImage::Format_RGB32) {
-        // Make the alpha component equal to the average of the RGB values.
-        // This is needed when drawing sub-pixel antialiased text on translucent targets.
-        for (int y = 0; y < maskHeight; ++y) {
-            quint32 *src = (quint32 *) mask.scanLine(y);
-            for (int x = 0; x < maskWidth; ++x) {
-                uchar r = src[x] >> 16;
-                uchar g = src[x] >> 8;
-                uchar b = src[x];
-                quint32 avg = (quint32(r) + quint32(g) + quint32(b) + 1) / 3; // "+1" for rounding.
-
-#if defined(QT_OPENGL_ES_2)
-                if (!hasBGRA) {
-                    // Reverse bytes to match GL_RGBA
-                    src[x] = (avg << 24) | (quint32(r) << 0) | (quint32(g) << 8) | (quint32(b) << 16);
-                } else
+    } else if (mask.depth() == 32) {
+        if (mask.format() == QImage::Format_RGB32
+            // We need to make the alpha component equal to the average of the RGB values.
+            // This is needed when drawing sub-pixel antialiased text on translucent targets.
+#if defined(QT_OPENGL_ES_2) || Q_BYTE_ORDER == Q_BIG_ENDIAN
+            || mask.format() == QImage::Format_ARGB32_Premultiplied
 #endif
-                    src[x] = (src[x] & 0x00ffffff) | (avg << 24);
+            ) {
+            for (int y = 0; y < maskHeight; ++y) {
+                QRgb *src = (QRgb *) mask.scanLine(y);
+                for (int x = 0; x < maskWidth; ++x) {
+                    int r = qRed(src[x]);
+                    int g = qGreen(src[x]);
+                    int b = qBlue(src[x]);
+                    int avg;
+                    if (mask.format() == QImage::Format_RGB32)
+                        avg = (r + g + b + 1) / 3; // "+1" for rounding.
+                    else // Format_ARGB_Premultiplied
+                        avg = qAlpha(src[x]);
+
+                    src[x] = qRgba(r, g, b, avg);
+#if defined(QT_OPENGL_ES_2) || Q_BYTE_ORDER == Q_BIG_ENDIAN
+                    // swizzle the bits to accommodate for the GL_RGBA upload.
+                    src[x] = ARGB2RGBA(src[x]);
+#endif
+                }
             }
         }
     }
 
     glBindTexture(GL_TEXTURE_2D, m_textureResource->m_texture);
-    if (mask.format() == QImage::Format_RGB32) {
-#if defined(QT_OPENGL_ES_2)
-        glTexSubImage2D(GL_TEXTURE_2D, 0, c.x, c.y, maskWidth, maskHeight, hasBGRA ? GL_BGRA_EXT : GL_RGBA, GL_UNSIGNED_BYTE, mask.bits());
+    if (mask.depth() == 32) {
+#if defined(QT_OPENGL_ES_2) || Q_BYTE_ORDER == Q_BIG_ENDIAN
+        glTexSubImage2D(GL_TEXTURE_2D, 0, c.x, c.y, maskWidth, maskHeight, GL_RGBA, GL_UNSIGNED_BYTE, mask.bits());
 #else
         glTexSubImage2D(GL_TEXTURE_2D, 0, c.x, c.y, maskWidth, maskHeight, GL_BGRA, GL_UNSIGNED_BYTE, mask.bits());
 #endif
@@ -369,7 +387,9 @@ void QOpenGLTextureGlyphCache::fillTexture(const Coord &c, glyph_t glyph, QFixed
         } else {
 #endif
             glTexSubImage2D(GL_TEXTURE_2D, 0, c.x, c.y, maskWidth, maskHeight, GL_ALPHA, GL_UNSIGNED_BYTE, mask.bits());
-//        }
+#if 0
+        }
+#endif
     }
 }
 
