@@ -201,10 +201,12 @@ static inline qint64 qCross(const QPodPoint &u, const QPodPoint &v)
     return qint64(u.x) * qint64(v.y) - qint64(u.y) * qint64(v.x);
 }
 
+#ifdef Q_TRIANGULATOR_DEBUG
 static inline qint64 qDot(const QPodPoint &u, const QPodPoint &v)
 {
     return qint64(u.x) * qint64(v.x) + qint64(u.y) * qint64(v.y);
 }
+#endif
 
 // Return positive value if 'p' is to the right of the line 'v1'->'v2', negative if left of the
 // line and zero if exactly on the line.
@@ -249,13 +251,6 @@ static inline QIntersectionPoint qIntersectionPoint(const QPodPoint &point)
     return p;
 }
 
-static inline QIntersectionPoint qIntersectionPoint(int x, int y)
-{
-    // upperLeft = (x, y), xOffset = 0/1, yOffset = 0/1.
-    QIntersectionPoint p = {{x, y}, {0, 1}, {0, 1}};
-    return p;
-}
-
 static QIntersectionPoint qIntersectionPoint(const QPodPoint &u1, const QPodPoint &u2, const QPodPoint &v1, const QPodPoint &v2)
 {
     QIntersectionPoint result = {{0, 0}, {0, 0}, {0, 0}};
@@ -347,7 +342,7 @@ bool QIntersectionPoint::operator == (const QIntersectionPoint &other) const
     return upperLeft == other.upperLeft && xOffset == other.xOffset && yOffset == other.yOffset;
 }
 
-// Returns true if this point is on the infinite line passing through 'u' and 'v'.
+// Returns \c true if this point is on the infinite line passing through 'u' and 'v'.
 bool QIntersectionPoint::isOnLine(const QPodPoint &u, const QPodPoint &v) const
 {
     // TODO: Make code path for coordinates with more than 21 bits.
@@ -370,7 +365,7 @@ bool QIntersectionPoint::isOnLine(const QPodPoint &u, const QPodPoint &v) const
 
     if (((q.x < 0) == (q.y < 0)) != ((p.x < 0) == (p.y < 0)))
         return false; // 'p + offset' and 'q' pass through different quadrants.
-    
+
     // Move all coordinates into the first quadrant.
     quint64 nx, ny;
     if (p.x < 0)
@@ -991,7 +986,7 @@ void QTriangulator<T>::ComplexToSimple::initEdges()
     if (first != m_edges.size())
         m_edges.last().to = m_edges.at(first).from;
     for (int i = 0; i < m_edges.size(); ++i) {
-        m_edges.at(i).originallyPointingUp = m_edges.at(i).pointingUp = 
+        m_edges.at(i).originallyPointingUp = m_edges.at(i).pointingUp =
             m_parent->m_vertices.at(m_edges.at(i).to) < m_parent->m_vertices.at(m_edges.at(i).from);
     }
 }
@@ -1646,7 +1641,7 @@ QTriangulator<T>::ComplexToSimple::DebugDialog::DebugDialog(ComplexToSimple *par
     QDataBuffer<QPodPoint> &vertices = m_parent->m_parent->m_vertices;
     if (vertices.isEmpty())
         return;
-    
+
     int minX, maxX, minY, maxY;
     minX = maxX = vertices.at(0).x;
     minY = maxY = vertices.at(0).y;
@@ -1671,7 +1666,7 @@ void QTriangulator<T>::ComplexToSimple::DebugDialog::paintEvent(QPaintEvent *)
     QDataBuffer<QPodPoint> &vertices = m_parent->m_parent->m_vertices;
     if (vertices.isEmpty())
         return;
-    
+
     qreal halfPointSize = qMin(m_window.width(), m_window.height()) / 300.0;
     p.setWindow(m_window.toRect());
 
@@ -1725,7 +1720,7 @@ void QTriangulator<T>::ComplexToSimple::DebugDialog::paintEvent(QPaintEvent *)
         }
         u += q;
         v += q;
-        p.drawLine(u.x, u.y, v.x, v.y);        
+        p.drawLine(u.x, u.y, v.x, v.y);
     }
 }
 
