@@ -119,64 +119,6 @@ QBezier QBezier::getSubRange(qreal t0, qreal t1) const
     return result;
 }
 
-static inline int quadraticRoots(qreal a, qreal b, qreal c,
-                                 qreal *x1, qreal *x2)
-{
-    if (qFuzzyIsNull(a)) {
-        if (qFuzzyIsNull(b))
-            return 0;
-        *x1 = *x2 = (-c / b);
-        return 1;
-    } else {
-        const qreal det = b * b - 4 * a * c;
-        if (qFuzzyIsNull(det)) {
-            *x1 = *x2 = -b / (2 * a);
-            return 1;
-        }
-        if (det > 0) {
-            if (qFuzzyIsNull(b)) {
-                *x2 = qSqrt(-c / a);
-                *x1 = -(*x2);
-                return 2;
-            }
-            const qreal stableA = b / (2 * a);
-            const qreal stableB = c / (a * stableA * stableA);
-            const qreal stableC = -1 - qSqrt(1 - stableB);
-            *x2 = stableA * stableC;
-            *x1 = (stableA * stableB) / stableC;
-            return 2;
-        } else
-            return 0;
-    }
-}
-
-static inline bool findInflections(qreal a, qreal b, qreal c,
-                                   qreal *t1 , qreal *t2, qreal *tCups)
-{
-    qreal r1 = 0, r2 = 0;
-
-    short rootsCount = quadraticRoots(a, b, c, &r1, &r2);
-
-    if (rootsCount >= 1) {
-        if (r1 < r2) {
-            *t1 = r1;
-            *t2 = r2;
-        } else {
-            *t1 = r2;
-            *t2 = r1;
-        }
-        if (!qFuzzyIsNull(a))
-            *tCups = qreal(0.5) * (-b / a);
-        else
-            *tCups = 2;
-
-        return true;
-    }
-
-    return false;
-}
-
-
 void QBezier::addToPolygon(QPolygonF *polygon, qreal bezier_flattening_threshold) const
 {
     QBezier beziers[10];
@@ -531,34 +473,6 @@ static QDebug operator<<(QDebug dbg, const QBezier &bz)
 }
 #endif
 
-static inline void splitBezierAt(const QBezier &bez, qreal t,
-                                 QBezier *left, QBezier *right)
-{
-    left->x1 = bez.x1;
-    left->y1 = bez.y1;
-
-    left->x2 = bez.x1 + t * ( bez.x2 - bez.x1 );
-    left->y2 = bez.y1 + t * ( bez.y2 - bez.y1 );
-
-    left->x3 = bez.x2 + t * ( bez.x3 - bez.x2 ); // temporary holding spot
-    left->y3 = bez.y2 + t * ( bez.y3 - bez.y2 ); // temporary holding spot
-
-    right->x3 = bez.x3 + t * ( bez.x4 - bez.x3 );
-    right->y3 = bez.y3 + t * ( bez.y4 - bez.y3 );
-
-    right->x2 = left->x3 + t * ( right->x3 - left->x3);
-    right->y2 = left->y3 + t * ( right->y3 - left->y3);
-
-    left->x3 = left->x2 + t * ( left->x3 - left->x2 );
-    left->y3 = left->y2 + t * ( left->y3 - left->y2 );
-
-    left->x4 = right->x1 = left->x3 + t * (right->x2 - left->x3);
-    left->y4 = right->y1 = left->y3 + t * (right->y2 - left->y3);
-
-    right->x4 = bez.x4;
-    right->y4 = bez.y4;
-}
-
 qreal QBezier::length(qreal error) const
 {
     qreal length = qreal(0.0);
