@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2012 Digia Plc and/or its subsidiary(-ies).
+** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
@@ -604,7 +604,10 @@ SrcOverScaleFunc qScaleFunctions[QImage::NImageFormats][QImage::NImageFormats] =
         0,      // Format_ARGB8555_Premultiplied,
         0,      // Format_RGB888,
         0,      // Format_RGB444,
-        0       // Format_ARGB4444_Premultiplied,
+        0,      // Format_ARGB4444_Premultiplied,
+        0,      // Format_RGBX8888,
+        0,      // Format_RGBA8888,
+        0       // Format_RGBA8888_Premultiplied,
     },
     {   // Format_Mono
         0,      // Format_Invalid,
@@ -622,7 +625,10 @@ SrcOverScaleFunc qScaleFunctions[QImage::NImageFormats][QImage::NImageFormats] =
         0,      // Format_ARGB8555_Premultiplied,
         0,      // Format_RGB888,
         0,      // Format_RGB444,
-        0       // Format_ARGB4444_Premultiplied,
+        0,      // Format_ARGB4444_Premultiplied,
+        0,      // Format_RGBX8888,
+        0,      // Format_RGBA8888,
+        0       // Format_RGBA8888_Premultiplied,
     },
     {   // Format_MonoLSB
         0,      // Format_Invalid,
@@ -640,7 +646,10 @@ SrcOverScaleFunc qScaleFunctions[QImage::NImageFormats][QImage::NImageFormats] =
         0,      // Format_ARGB8555_Premultiplied,
         0,      // Format_RGB888,
         0,      // Format_RGB444,
-        0       // Format_ARGB4444_Premultiplied,
+        0,      // Format_ARGB4444_Premultiplied,
+        0,      // Format_RGBX8888,
+        0,      // Format_RGBA8888,
+        0       // Format_RGBA8888_Premultiplied,
     },
     {   // Format_Indexed8
         0,      // Format_Invalid,
@@ -658,7 +667,10 @@ SrcOverScaleFunc qScaleFunctions[QImage::NImageFormats][QImage::NImageFormats] =
         0,      // Format_ARGB8555_Premultiplied,
         0,      // Format_RGB888,
         0,      // Format_RGB444,
-        0       // Format_ARGB4444_Premultiplied,
+        0,      // Format_ARGB4444_Premultiplied,
+        0,      // Format_RGBX8888,
+        0,      // Format_RGBA8888,
+        0       // Format_RGBA8888_Premultiplied,
     },
     {   // Format_RGB32
         0,      // Format_Invalid,
@@ -676,7 +688,10 @@ SrcOverScaleFunc qScaleFunctions[QImage::NImageFormats][QImage::NImageFormats] =
         0,      // Format_ARGB8555_Premultiplied,
         0,      // Format_RGB888,
         0,      // Format_RGB444,
-        0       // Format_ARGB4444_Premultiplied,
+        0,      // Format_ARGB4444_Premultiplied,
+        0,      // Format_RGBX8888,
+        0,      // Format_RGBA8888,
+        0       // Format_RGBA8888_Premultiplied,
     },
     {   // Format_ARGB32
         0,      // Format_Invalid,
@@ -694,7 +709,10 @@ SrcOverScaleFunc qScaleFunctions[QImage::NImageFormats][QImage::NImageFormats] =
         0,      // Format_ARGB8555_Premultiplied,
         0,      // Format_RGB888,
         0,      // Format_RGB444,
-        0       // Format_ARGB4444_Premultiplied,
+        0,      // Format_ARGB4444_Premultiplied,
+        0,      // Format_RGBX8888,
+        0,      // Format_RGBA8888,
+        0       // Format_RGBA8888_Premultiplied,
     },
     {   // Format_ARGB32_Premultiplied
         0,      // Format_Invalid,
@@ -712,7 +730,10 @@ SrcOverScaleFunc qScaleFunctions[QImage::NImageFormats][QImage::NImageFormats] =
         0,      // Format_ARGB8555_Premultiplied,
         0,      // Format_RGB888,
         0,      // Format_RGB444,
-        0       // Format_ARGB4444_Premultiplied,
+        0,      // Format_ARGB4444_Premultiplied,
+        0,      // Format_RGBX8888,
+        0,      // Format_RGBA8888,
+        0       // Format_RGBA8888_Premultiplied,
     },
     {   // Format_RGB16
         0,      // Format_Invalid,
@@ -730,7 +751,10 @@ SrcOverScaleFunc qScaleFunctions[QImage::NImageFormats][QImage::NImageFormats] =
         0,      // Format_ARGB8555_Premultiplied,
         0,      // Format_RGB888,
         0,      // Format_RGB444,
-        0       // Format_ARGB4444_Premultiplied,
+        0,      // Format_ARGB4444_Premultiplied,
+        0,      // Format_RGBX8888,
+        0,      // Format_RGBA8888,
+        0       // Format_RGBA8888_Premultiplied,
     },
     {   // Format_ARGB8565_Premultiplied
         0,      // Format_Invalid,
@@ -748,7 +772,10 @@ SrcOverScaleFunc qScaleFunctions[QImage::NImageFormats][QImage::NImageFormats] =
         0,      // Format_ARGB8555_Premultiplied,
         0,      // Format_RGB888,
         0,      // Format_RGB444,
-        0       // Format_ARGB4444_Premultiplied,
+        0,      // Format_ARGB4444_Premultiplied,
+        0,      // Format_RGBX8888,
+        0,      // Format_RGBA8888,
+        0       // Format_RGBA8888_Premultiplied,
     },
     {   // Format_RGB666
         0,      // Format_Invalid,
@@ -766,7 +793,10 @@ SrcOverScaleFunc qScaleFunctions[QImage::NImageFormats][QImage::NImageFormats] =
         0,      // Format_ARGB8555_Premultiplied,
         0,      // Format_RGB888,
         0,      // Format_RGB444,
-        0       // Format_ARGB4444_Premultiplied,
+        0,      // Format_ARGB4444_Premultiplied,
+        0,      // Format_RGBX8888,
+        0,      // Format_RGBA8888,
+        0       // Format_RGBA8888_Premultiplied,
     },
     {   // Format_ARGB6666_Premultiplied
         0,      // Format_Invalid,
@@ -784,7 +814,10 @@ SrcOverScaleFunc qScaleFunctions[QImage::NImageFormats][QImage::NImageFormats] =
         0,      // Format_ARGB8555_Premultiplied,
         0,      // Format_RGB888,
         0,      // Format_RGB444,
-        0       // Format_ARGB4444_Premultiplied,
+        0,      // Format_ARGB4444_Premultiplied,
+        0,      // Format_RGBX8888,
+        0,      // Format_RGBA8888,
+        0       // Format_RGBA8888_Premultiplied,
     },
     {   // Format_RGB555
         0,      // Format_Invalid,
@@ -802,7 +835,10 @@ SrcOverScaleFunc qScaleFunctions[QImage::NImageFormats][QImage::NImageFormats] =
         0,      // Format_ARGB8555_Premultiplied,
         0,      // Format_RGB888,
         0,      // Format_RGB444,
-        0       // Format_ARGB4444_Premultiplied,
+        0,      // Format_ARGB4444_Premultiplied,
+        0,      // Format_RGBX8888,
+        0,      // Format_RGBA8888,
+        0       // Format_RGBA8888_Premultiplied,
     },
     {   // Format_ARGB8555_Premultiplied
         0,      // Format_Invalid,
@@ -820,7 +856,10 @@ SrcOverScaleFunc qScaleFunctions[QImage::NImageFormats][QImage::NImageFormats] =
         0,      // Format_ARGB8555_Premultiplied,
         0,      // Format_RGB888,
         0,      // Format_RGB444,
-        0       // Format_ARGB4444_Premultiplied,
+        0,      // Format_ARGB4444_Premultiplied,
+        0,      // Format_RGBX8888,
+        0,      // Format_RGBA8888,
+        0       // Format_RGBA8888_Premultiplied,
     },
     {   // Format_RGB888
         0,      // Format_Invalid,
@@ -838,7 +877,10 @@ SrcOverScaleFunc qScaleFunctions[QImage::NImageFormats][QImage::NImageFormats] =
         0,      // Format_ARGB8555_Premultiplied,
         0,      // Format_RGB888,
         0,      // Format_RGB444,
-        0       // Format_ARGB4444_Premultiplied,
+        0,      // Format_ARGB4444_Premultiplied,
+        0,      // Format_RGBX8888,
+        0,      // Format_RGBA8888,
+        0       // Format_RGBA8888_Premultiplied,
     },
     {   // Format_RGB444
         0,      // Format_Invalid,
@@ -856,7 +898,10 @@ SrcOverScaleFunc qScaleFunctions[QImage::NImageFormats][QImage::NImageFormats] =
         0,      // Format_ARGB8555_Premultiplied,
         0,      // Format_RGB888,
         0,      // Format_RGB444,
-        0       // Format_ARGB4444_Premultiplied,
+        0,      // Format_ARGB4444_Premultiplied,
+        0,      // Format_RGBX8888,
+        0,      // Format_RGBA8888,
+        0       // Format_RGBA8888_Premultiplied,
     },
     {   // Format_ARGB4444_Premultiplied
         0,      // Format_Invalid,
@@ -874,7 +919,85 @@ SrcOverScaleFunc qScaleFunctions[QImage::NImageFormats][QImage::NImageFormats] =
         0,      // Format_ARGB8555_Premultiplied,
         0,      // Format_RGB888,
         0,      // Format_RGB444,
-        0       // Format_ARGB4444_Premultiplied,
+        0,      // Format_ARGB4444_Premultiplied,
+        0,      // Format_RGBX8888,
+        0,      // Format_RGBA8888,
+        0       // Format_RGBA8888_Premultiplied,
+    },
+    {   // Format_RGBX8888
+        0,      // Format_Invalid,
+        0,      // Format_Mono,
+        0,      // Format_MonoLSB,
+        0,      // Format_Indexed8,
+        0,      // Format_RGB32,
+        0,      // Format_ARGB32,
+        0,      // Format_ARGB32_Premultiplied,
+        0,      // Format_RGB16,
+        0,      // Format_ARGB8565_Premultiplied,
+        0,      // Format_RGB666,
+        0,      // Format_ARGB6666_Premultiplied,
+        0,      // Format_RGB555,
+        0,      // Format_ARGB8555_Premultiplied,
+        0,      // Format_RGB888,
+        0,      // Format_RGB444,
+        0,      // Format_ARGB4444_Premultiplied,
+#if Q_BYTE_ORDER == Q_LITTLE_ENDIAN
+        qt_scale_image_rgb32_on_rgb32,          // Format_RGBX8888,
+        0,      // Format_RGBA8888,
+        qt_scale_image_argb32_on_argb32,        // Format_RGBA8888_Premultiplied,
+#else
+        0,      // Format_RGBX8888,
+        0,      // Format_RGBA8888,
+        0       // Format_RGBA8888_Premultiplied,
+#endif
+    },
+    {   // Format_RGBA8888
+        0,      // Format_Invalid,
+        0,      // Format_Mono,
+        0,      // Format_MonoLSB,
+        0,      // Format_Indexed8,
+        0,      // Format_RGB32,
+        0,      // Format_ARGB32,
+        0,      // Format_ARGB32_Premultiplied,
+        0,      // Format_RGB16,
+        0,      // Format_ARGB8565_Premultiplied,
+        0,      // Format_RGB666,
+        0,      // Format_ARGB6666_Premultiplied,
+        0,      // Format_RGB555,
+        0,      // Format_ARGB8555_Premultiplied,
+        0,      // Format_RGB888,
+        0,      // Format_RGB444,
+        0,      // Format_ARGB4444_Premultiplied,
+        0,      // Format_RGBX8888,
+        0,      // Format_RGBA8888,
+        0       // Format_RGBA8888_Premultiplied,
+    },
+    {   // Format_RGBA8888_Premultiplied
+        0,      // Format_Invalid,
+        0,      // Format_Mono,
+        0,      // Format_MonoLSB,
+        0,      // Format_Indexed8,
+        0,      // Format_RGB32,
+        0,      // Format_ARGB32,
+        0,      // Format_ARGB32_Premultiplied,
+        0,      // Format_RGB16,
+        0,      // Format_ARGB8565_Premultiplied,
+        0,      // Format_RGB666,
+        0,      // Format_ARGB6666_Premultiplied,
+        0,      // Format_RGB555,
+        0,      // Format_ARGB8555_Premultiplied,
+        0,      // Format_RGB888,
+        0,      // Format_RGB444,
+        0,      // Format_ARGB4444_Premultiplied,
+#if Q_BYTE_ORDER == Q_LITTLE_ENDIAN
+        qt_scale_image_rgb32_on_rgb32,          // Format_RGBX8888,
+        0,      // Format_RGBA8888,
+        qt_scale_image_argb32_on_argb32,        // Format_RGBA8888_Premultiplied,
+#else
+        0,      // Format_RGBX8888,
+        0,      // Format_RGBA8888,
+        0       // Format_RGBA8888_Premultiplied,
+#endif
     }
 };
 
@@ -896,7 +1019,10 @@ SrcOverBlendFunc qBlendFunctions[QImage::NImageFormats][QImage::NImageFormats] =
         0,      // Format_ARGB8555_Premultiplied,
         0,      // Format_RGB888,
         0,      // Format_RGB444,
-        0       // Format_ARGB4444_Premultiplied,
+        0,      // Format_ARGB4444_Premultiplied,
+        0,      // Format_RGBX8888,
+        0,      // Format_RGBA8888,
+        0       // Format_RGBA8888_Premultiplied,
     },
     {   // Format_Mono
         0,      // Format_Invalid,
@@ -914,7 +1040,10 @@ SrcOverBlendFunc qBlendFunctions[QImage::NImageFormats][QImage::NImageFormats] =
         0,      // Format_ARGB8555_Premultiplied,
         0,      // Format_RGB888,
         0,      // Format_RGB444,
-        0       // Format_ARGB4444_Premultiplied,
+        0,      // Format_ARGB4444_Premultiplied,
+        0,      // Format_RGBX8888,
+        0,      // Format_RGBA8888,
+        0       // Format_RGBA8888_Premultiplied,
     },
     {   // Format_MonoLSB
         0,      // Format_Invalid,
@@ -932,7 +1061,10 @@ SrcOverBlendFunc qBlendFunctions[QImage::NImageFormats][QImage::NImageFormats] =
         0,      // Format_ARGB8555_Premultiplied,
         0,      // Format_RGB888,
         0,      // Format_RGB444,
-        0       // Format_ARGB4444_Premultiplied,
+        0,      // Format_ARGB4444_Premultiplied,
+        0,      // Format_RGBX8888,
+        0,      // Format_RGBA8888,
+        0       // Format_RGBA8888_Premultiplied,
     },
     {   // Format_Indexed8
         0,      // Format_Invalid,
@@ -950,7 +1082,10 @@ SrcOverBlendFunc qBlendFunctions[QImage::NImageFormats][QImage::NImageFormats] =
         0,      // Format_ARGB8555_Premultiplied,
         0,      // Format_RGB888,
         0,      // Format_RGB444,
-        0       // Format_ARGB4444_Premultiplied,
+        0,      // Format_ARGB4444_Premultiplied,
+        0,      // Format_RGBX8888,
+        0,      // Format_RGBA8888,
+        0       // Format_RGBA8888_Premultiplied,
     },
     {   // Format_RGB32
         0,      // Format_Invalid,
@@ -968,7 +1103,10 @@ SrcOverBlendFunc qBlendFunctions[QImage::NImageFormats][QImage::NImageFormats] =
         0,      // Format_ARGB8555_Premultiplied,
         0,      // Format_RGB888,
         0,      // Format_RGB444,
-        0       // Format_ARGB4444_Premultiplied,
+        0,      // Format_ARGB4444_Premultiplied,
+        0,      // Format_RGBX8888,
+        0,      // Format_RGBA8888,
+        0       // Format_RGBA8888_Premultiplied,
     },
     {   // Format_ARGB32
         0,      // Format_Invalid,
@@ -986,7 +1124,10 @@ SrcOverBlendFunc qBlendFunctions[QImage::NImageFormats][QImage::NImageFormats] =
         0,      // Format_ARGB8555_Premultiplied,
         0,      // Format_RGB888,
         0,      // Format_RGB444,
-        0       // Format_ARGB4444_Premultiplied,
+        0,      // Format_ARGB4444_Premultiplied,
+        0,      // Format_RGBX8888,
+        0,      // Format_RGBA8888,
+        0       // Format_RGBA8888_Premultiplied,
     },
     {   // Format_ARGB32_Premultiplied
         0,      // Format_Invalid,
@@ -1004,7 +1145,10 @@ SrcOverBlendFunc qBlendFunctions[QImage::NImageFormats][QImage::NImageFormats] =
         0,      // Format_ARGB8555_Premultiplied,
         0,      // Format_RGB888,
         0,      // Format_RGB444,
-        0       // Format_ARGB4444_Premultiplied,
+        0,      // Format_ARGB4444_Premultiplied,
+        0,      // Format_RGBX8888,
+        0,      // Format_RGBA8888,
+        0       // Format_RGBA8888_Premultiplied,
     },
     {   // Format_RGB16
         0,      // Format_Invalid,
@@ -1022,7 +1166,10 @@ SrcOverBlendFunc qBlendFunctions[QImage::NImageFormats][QImage::NImageFormats] =
         0,      // Format_ARGB8555_Premultiplied,
         0,      // Format_RGB888,
         0,      // Format_RGB444,
-        0       // Format_ARGB4444_Premultiplied,
+        0,      // Format_ARGB4444_Premultiplied,
+        0,      // Format_RGBX8888,
+        0,      // Format_RGBA8888,
+        0       // Format_RGBA8888_Premultiplied,
     },
     {   // Format_ARGB8565_Premultiplied
         0,      // Format_Invalid,
@@ -1040,7 +1187,10 @@ SrcOverBlendFunc qBlendFunctions[QImage::NImageFormats][QImage::NImageFormats] =
         0,      // Format_ARGB8555_Premultiplied,
         0,      // Format_RGB888,
         0,      // Format_RGB444,
-        0       // Format_ARGB4444_Premultiplied,
+        0,      // Format_ARGB4444_Premultiplied,
+        0,      // Format_RGBX8888,
+        0,      // Format_RGBA8888,
+        0       // Format_RGBA8888_Premultiplied,
     },
     {   // Format_RGB666
         0,      // Format_Invalid,
@@ -1058,7 +1208,10 @@ SrcOverBlendFunc qBlendFunctions[QImage::NImageFormats][QImage::NImageFormats] =
         0,      // Format_ARGB8555_Premultiplied,
         0,      // Format_RGB888,
         0,      // Format_RGB444,
-        0       // Format_ARGB4444_Premultiplied,
+        0,      // Format_ARGB4444_Premultiplied,
+        0,      // Format_RGBX8888,
+        0,      // Format_RGBA8888,
+        0       // Format_RGBA8888_Premultiplied,
     },
     {   // Format_ARGB6666_Premultiplied
         0,      // Format_Invalid,
@@ -1076,7 +1229,10 @@ SrcOverBlendFunc qBlendFunctions[QImage::NImageFormats][QImage::NImageFormats] =
         0,      // Format_ARGB8555_Premultiplied,
         0,      // Format_RGB888,
         0,      // Format_RGB444,
-        0       // Format_ARGB4444_Premultiplied,
+        0,      // Format_ARGB4444_Premultiplied,
+        0,      // Format_RGBX8888,
+        0,      // Format_RGBA8888,
+        0       // Format_RGBA8888_Premultiplied,
     },
     {   // Format_RGB555
         0,      // Format_Invalid,
@@ -1094,7 +1250,10 @@ SrcOverBlendFunc qBlendFunctions[QImage::NImageFormats][QImage::NImageFormats] =
         0,      // Format_ARGB8555_Premultiplied,
         0,      // Format_RGB888,
         0,      // Format_RGB444,
-        0       // Format_ARGB4444_Premultiplied,
+        0,      // Format_ARGB4444_Premultiplied,
+        0,      // Format_RGBX8888,
+        0,      // Format_RGBA8888,
+        0       // Format_RGBA8888_Premultiplied,
     },
     {   // Format_ARGB8555_Premultiplied
         0,      // Format_Invalid,
@@ -1112,7 +1271,10 @@ SrcOverBlendFunc qBlendFunctions[QImage::NImageFormats][QImage::NImageFormats] =
         0,      // Format_ARGB8555_Premultiplied,
         0,      // Format_RGB888,
         0,      // Format_RGB444,
-        0       // Format_ARGB4444_Premultiplied,
+        0,      // Format_ARGB4444_Premultiplied,
+        0,      // Format_RGBX8888,
+        0,      // Format_RGBA8888,
+        0       // Format_RGBA8888_Premultiplied,
     },
     {   // Format_RGB888
         0,      // Format_Invalid,
@@ -1130,7 +1292,10 @@ SrcOverBlendFunc qBlendFunctions[QImage::NImageFormats][QImage::NImageFormats] =
         0,      // Format_ARGB8555_Premultiplied,
         0,      // Format_RGB888,
         0,      // Format_RGB444,
-        0       // Format_ARGB4444_Premultiplied,
+        0,      // Format_ARGB4444_Premultiplied,
+        0,      // Format_RGBX8888,
+        0,      // Format_RGBA8888,
+        0       // Format_RGBA8888_Premultiplied,
     },
     {   // Format_RGB444
         0,      // Format_Invalid,
@@ -1148,7 +1313,10 @@ SrcOverBlendFunc qBlendFunctions[QImage::NImageFormats][QImage::NImageFormats] =
         0,      // Format_ARGB8555_Premultiplied,
         0,      // Format_RGB888,
         0,      // Format_RGB444,
-        0       // Format_ARGB4444_Premultiplied,
+        0,      // Format_ARGB4444_Premultiplied,
+        0,      // Format_RGBX8888,
+        0,      // Format_RGBA8888,
+        0       // Format_RGBA8888_Premultiplied,
     },
     {   // Format_ARGB4444_Premultiplied
         0,      // Format_Invalid,
@@ -1166,7 +1334,85 @@ SrcOverBlendFunc qBlendFunctions[QImage::NImageFormats][QImage::NImageFormats] =
         0,      // Format_ARGB8555_Premultiplied,
         0,      // Format_RGB888,
         0,      // Format_RGB444,
-        0       // Format_ARGB4444_Premultiplied,
+        0,      // Format_ARGB4444_Premultiplied,
+        0,      // Format_RGBX8888,
+        0,      // Format_RGBA8888,
+        0       // Format_RGBA8888_Premultiplied,
+    },
+    {   // Format_RGBX8888
+        0,      // Format_Invalid,
+        0,      // Format_Mono,
+        0,      // Format_MonoLSB,
+        0,      // Format_Indexed8,
+        0,      // Format_RGB32,
+        0,      // Format_ARGB32,
+        0,      // Format_ARGB32_Premultiplied,
+        0,      // Format_RGB16,
+        0,      // Format_ARGB8565_Premultiplied,
+        0,      // Format_RGB666,
+        0,      // Format_ARGB6666_Premultiplied,
+        0,      // Format_RGB555,
+        0,      // Format_ARGB8555_Premultiplied,
+        0,      // Format_RGB888,
+        0,      // Format_RGB444,
+        0,      // Format_ARGB4444_Premultiplied,
+#if Q_BYTE_ORDER == Q_LITTLE_ENDIAN
+        qt_blend_rgb32_on_rgb32,        // Format_RGBX8888,
+        0,      // Format_RGBA8888,
+        qt_blend_argb32_on_argb32,      // Format_RGBA8888_Premultiplied,
+#else
+        0,      // Format_RGBX8888,
+        0,      // Format_RGBA8888,
+        0       // Format_RGBA8888_Premultiplied,
+#endif
+    },
+    {   // Format_RGBA8888
+        0,      // Format_Invalid,
+        0,      // Format_Mono,
+        0,      // Format_MonoLSB,
+        0,      // Format_Indexed8,
+        0,      // Format_RGB32,
+        0,      // Format_ARGB32,
+        0,      // Format_ARGB32_Premultiplied,
+        0,      // Format_RGB16,
+        0,      // Format_ARGB8565_Premultiplied,
+        0,      // Format_RGB666,
+        0,      // Format_ARGB6666_Premultiplied,
+        0,      // Format_RGB555,
+        0,      // Format_ARGB8555_Premultiplied,
+        0,      // Format_RGB888,
+        0,      // Format_RGB444,
+        0,      // Format_ARGB4444_Premultiplied,
+        0,      // Format_RGBX8888,
+        0,      // Format_RGBA8888,
+        0       // Format_RGBA8888_Premultiplied,
+    },
+    {   // Format_RGBA8888_Premultiplied
+        0,      // Format_Invalid,
+        0,      // Format_Mono,
+        0,      // Format_MonoLSB,
+        0,      // Format_Indexed8,
+        0,      // Format_RGB32,
+        0,      // Format_ARGB32,
+        0,      // Format_ARGB32_Premultiplied,
+        0,      // Format_RGB16,
+        0,      // Format_ARGB8565_Premultiplied,
+        0,      // Format_RGB666,
+        0,      // Format_ARGB6666_Premultiplied,
+        0,      // Format_RGB555,
+        0,      // Format_ARGB8555_Premultiplied,
+        0,      // Format_RGB888,
+        0,      // Format_RGB444,
+        0,      // Format_ARGB4444_Premultiplied,
+#if Q_BYTE_ORDER == Q_LITTLE_ENDIAN
+        qt_blend_rgb32_on_rgb32,        // Format_RGBX8888,
+        0,      // Format_RGBA8888,
+        qt_blend_argb32_on_argb32,      // Format_RGBA8888_Premultiplied,
+#else
+        0,      // Format_RGBX8888,
+        0,      // Format_RGBA8888,
+        0       // Format_RGBA8888_Premultiplied,
+#endif
     }
 };
 
@@ -1187,7 +1433,10 @@ SrcOverTransformFunc qTransformFunctions[QImage::NImageFormats][QImage::NImageFo
         0,      // Format_ARGB8555_Premultiplied,
         0,      // Format_RGB888,
         0,      // Format_RGB444,
-        0       // Format_ARGB4444_Premultiplied,
+        0,      // Format_ARGB4444_Premultiplied,
+        0,      // Format_RGBX8888,
+        0,      // Format_RGBA8888,
+        0       // Format_RGBA8888_Premultiplied,
     },
     {   // Format_Mono
         0,      // Format_Invalid,
@@ -1205,7 +1454,10 @@ SrcOverTransformFunc qTransformFunctions[QImage::NImageFormats][QImage::NImageFo
         0,      // Format_ARGB8555_Premultiplied,
         0,      // Format_RGB888,
         0,      // Format_RGB444,
-        0       // Format_ARGB4444_Premultiplied,
+        0,      // Format_ARGB4444_Premultiplied,
+        0,      // Format_RGBX8888,
+        0,      // Format_RGBA8888,
+        0       // Format_RGBA8888_Premultiplied,
     },
     {   // Format_MonoLSB
         0,      // Format_Invalid,
@@ -1223,7 +1475,10 @@ SrcOverTransformFunc qTransformFunctions[QImage::NImageFormats][QImage::NImageFo
         0,      // Format_ARGB8555_Premultiplied,
         0,      // Format_RGB888,
         0,      // Format_RGB444,
-        0       // Format_ARGB4444_Premultiplied,
+        0,      // Format_ARGB4444_Premultiplied,
+        0,      // Format_RGBX8888,
+        0,      // Format_RGBA8888,
+        0       // Format_RGBA8888_Premultiplied,
     },
     {   // Format_Indexed8
         0,      // Format_Invalid,
@@ -1241,7 +1496,10 @@ SrcOverTransformFunc qTransformFunctions[QImage::NImageFormats][QImage::NImageFo
         0,      // Format_ARGB8555_Premultiplied,
         0,      // Format_RGB888,
         0,      // Format_RGB444,
-        0       // Format_ARGB4444_Premultiplied,
+        0,      // Format_ARGB4444_Premultiplied,
+        0,      // Format_RGBX8888,
+        0,      // Format_RGBA8888,
+        0       // Format_RGBA8888_Premultiplied,
     },
     {   // Format_RGB32
         0,      // Format_Invalid,
@@ -1259,7 +1517,10 @@ SrcOverTransformFunc qTransformFunctions[QImage::NImageFormats][QImage::NImageFo
         0,      // Format_ARGB8555_Premultiplied,
         0,      // Format_RGB888,
         0,      // Format_RGB444,
-        0       // Format_ARGB4444_Premultiplied,
+        0,      // Format_ARGB4444_Premultiplied,
+        0,      // Format_RGBX8888,
+        0,      // Format_RGBA8888,
+        0       // Format_RGBA8888_Premultiplied,
     },
     {   // Format_ARGB32
         0,      // Format_Invalid,
@@ -1277,7 +1538,10 @@ SrcOverTransformFunc qTransformFunctions[QImage::NImageFormats][QImage::NImageFo
         0,      // Format_ARGB8555_Premultiplied,
         0,      // Format_RGB888,
         0,      // Format_RGB444,
-        0       // Format_ARGB4444_Premultiplied,
+        0,      // Format_ARGB4444_Premultiplied,
+        0,      // Format_RGBX8888,
+        0,      // Format_RGBA8888,
+        0       // Format_RGBA8888_Premultiplied,
     },
     {   // Format_ARGB32_Premultiplied
         0,      // Format_Invalid,
@@ -1295,7 +1559,10 @@ SrcOverTransformFunc qTransformFunctions[QImage::NImageFormats][QImage::NImageFo
         0,      // Format_ARGB8555_Premultiplied,
         0,      // Format_RGB888,
         0,      // Format_RGB444,
-        0       // Format_ARGB4444_Premultiplied,
+        0,      // Format_ARGB4444_Premultiplied,
+        0,      // Format_RGBX8888,
+        0,      // Format_RGBA8888,
+        0       // Format_RGBA8888_Premultiplied,
     },
     {   // Format_RGB16
         0,      // Format_Invalid,
@@ -1313,7 +1580,10 @@ SrcOverTransformFunc qTransformFunctions[QImage::NImageFormats][QImage::NImageFo
         0,      // Format_ARGB8555_Premultiplied,
         0,      // Format_RGB888,
         0,      // Format_RGB444,
-        0       // Format_ARGB4444_Premultiplied,
+        0,      // Format_ARGB4444_Premultiplied,
+        0,      // Format_RGBX8888,
+        0,      // Format_RGBA8888,
+        0       // Format_RGBA8888_Premultiplied,
     },
     {   // Format_ARGB8565_Premultiplied
         0,      // Format_Invalid,
@@ -1331,7 +1601,10 @@ SrcOverTransformFunc qTransformFunctions[QImage::NImageFormats][QImage::NImageFo
         0,      // Format_ARGB8555_Premultiplied,
         0,      // Format_RGB888,
         0,      // Format_RGB444,
-        0       // Format_ARGB4444_Premultiplied,
+        0,      // Format_ARGB4444_Premultiplied,
+        0,      // Format_RGBX8888,
+        0,      // Format_RGBA8888,
+        0       // Format_RGBA8888_Premultiplied,
     },
     {   // Format_RGB666
         0,      // Format_Invalid,
@@ -1349,7 +1622,10 @@ SrcOverTransformFunc qTransformFunctions[QImage::NImageFormats][QImage::NImageFo
         0,      // Format_ARGB8555_Premultiplied,
         0,      // Format_RGB888,
         0,      // Format_RGB444,
-        0       // Format_ARGB4444_Premultiplied,
+        0,      // Format_ARGB4444_Premultiplied,
+        0,      // Format_RGBX8888,
+        0,      // Format_RGBA8888,
+        0       // Format_RGBA8888_Premultiplied,
     },
     {   // Format_ARGB6666_Premultiplied
         0,      // Format_Invalid,
@@ -1367,7 +1643,10 @@ SrcOverTransformFunc qTransformFunctions[QImage::NImageFormats][QImage::NImageFo
         0,      // Format_ARGB8555_Premultiplied,
         0,      // Format_RGB888,
         0,      // Format_RGB444,
-        0       // Format_ARGB4444_Premultiplied,
+        0,      // Format_ARGB4444_Premultiplied,
+        0,      // Format_RGBX8888,
+        0,      // Format_RGBA8888,
+        0       // Format_RGBA8888_Premultiplied,
     },
     {   // Format_RGB555
         0,      // Format_Invalid,
@@ -1385,7 +1664,10 @@ SrcOverTransformFunc qTransformFunctions[QImage::NImageFormats][QImage::NImageFo
         0,      // Format_ARGB8555_Premultiplied,
         0,      // Format_RGB888,
         0,      // Format_RGB444,
-        0       // Format_ARGB4444_Premultiplied,
+        0,      // Format_ARGB4444_Premultiplied,
+        0,      // Format_RGBX8888,
+        0,      // Format_RGBA8888,
+        0       // Format_RGBA8888_Premultiplied,
     },
     {   // Format_ARGB8555_Premultiplied
         0,      // Format_Invalid,
@@ -1403,7 +1685,10 @@ SrcOverTransformFunc qTransformFunctions[QImage::NImageFormats][QImage::NImageFo
         0,      // Format_ARGB8555_Premultiplied,
         0,      // Format_RGB888,
         0,      // Format_RGB444,
-        0       // Format_ARGB4444_Premultiplied,
+        0,      // Format_ARGB4444_Premultiplied,
+        0,      // Format_RGBX8888,
+        0,      // Format_RGBA8888,
+        0       // Format_RGBA8888_Premultiplied,
     },
     {   // Format_RGB888
         0,      // Format_Invalid,
@@ -1421,7 +1706,10 @@ SrcOverTransformFunc qTransformFunctions[QImage::NImageFormats][QImage::NImageFo
         0,      // Format_ARGB8555_Premultiplied,
         0,      // Format_RGB888,
         0,      // Format_RGB444,
-        0       // Format_ARGB4444_Premultiplied,
+        0,      // Format_ARGB4444_Premultiplied,
+        0,      // Format_RGBX8888,
+        0,      // Format_RGBA8888,
+        0       // Format_RGBA8888_Premultiplied,
     },
     {   // Format_RGB444
         0,      // Format_Invalid,
@@ -1439,7 +1727,10 @@ SrcOverTransformFunc qTransformFunctions[QImage::NImageFormats][QImage::NImageFo
         0,      // Format_ARGB8555_Premultiplied,
         0,      // Format_RGB888,
         0,      // Format_RGB444,
-        0       // Format_ARGB4444_Premultiplied,
+        0,      // Format_ARGB4444_Premultiplied,
+        0,      // Format_RGBX8888,
+        0,      // Format_RGBA8888,
+        0       // Format_RGBA8888_Premultiplied,
     },
     {   // Format_ARGB4444_Premultiplied
         0,      // Format_Invalid,
@@ -1457,7 +1748,85 @@ SrcOverTransformFunc qTransformFunctions[QImage::NImageFormats][QImage::NImageFo
         0,      // Format_ARGB8555_Premultiplied,
         0,      // Format_RGB888,
         0,      // Format_RGB444,
-        0       // Format_ARGB4444_Premultiplied,
+        0,      // Format_ARGB4444_Premultiplied,
+        0,      // Format_RGBX8888,
+        0,      // Format_RGBA8888,
+        0       // Format_RGBA8888_Premultiplied,
+    },
+    {   // Format_RGBX8888
+        0,      // Format_Invalid,
+        0,      // Format_Mono,
+        0,      // Format_MonoLSB,
+        0,      // Format_Indexed8,
+        0,      // Format_RGB32,
+        0,      // Format_ARGB32,
+        0,      // Format_ARGB32_Premultiplied,
+        0,      // Format_RGB16,
+        0,      // Format_ARGB8565_Premultiplied,
+        0,      // Format_RGB666,
+        0,      // Format_ARGB6666_Premultiplied,
+        0,      // Format_RGB555,
+        0,      // Format_ARGB8555_Premultiplied,
+        0,      // Format_RGB888,
+        0,      // Format_RGB444,
+        0,      // Format_ARGB4444_Premultiplied,
+#if Q_BYTE_ORDER == Q_LITTLE_ENDIAN
+        qt_transform_image_rgb32_on_rgb32,        // Format_RGBX8888,
+        0,      // Format_RGBA8888,
+        qt_transform_image_argb32_on_argb32,      // Format_RGBA8888_Premultiplied,
+#else
+        0,      // Format_RGBX8888,
+        0,      // Format_RGBA8888,
+        0       // Format_RGBA8888_Premultiplied,
+#endif
+    },
+    {   // Format_RGBA8888
+        0,      // Format_Invalid,
+        0,      // Format_Mono,
+        0,      // Format_MonoLSB,
+        0,      // Format_Indexed8,
+        0,      // Format_RGB32,
+        0,      // Format_ARGB32,
+        0,      // Format_ARGB32_Premultiplied,
+        0,      // Format_RGB16,
+        0,      // Format_ARGB8565_Premultiplied,
+        0,      // Format_RGB666,
+        0,      // Format_ARGB6666_Premultiplied,
+        0,      // Format_RGB555,
+        0,      // Format_ARGB8555_Premultiplied,
+        0,      // Format_RGB888,
+        0,      // Format_RGB444,
+        0,      // Format_ARGB4444_Premultiplied,
+        0,      // Format_RGBX8888,
+        0,      // Format_RGBA8888,
+        0       // Format_RGBA8888_Premultiplied,
+    },
+    {   // Format_RGBA8888_Premultiplied
+        0,      // Format_Invalid,
+        0,      // Format_Mono,
+        0,      // Format_MonoLSB,
+        0,      // Format_Indexed8,
+        0,      // Format_RGB32,
+        0,      // Format_ARGB32,
+        0,      // Format_ARGB32_Premultiplied,
+        0,      // Format_RGB16,
+        0,      // Format_ARGB8565_Premultiplied,
+        0,      // Format_RGB666,
+        0,      // Format_ARGB6666_Premultiplied,
+        0,      // Format_RGB555,
+        0,      // Format_ARGB8555_Premultiplied,
+        0,      // Format_RGB888,
+        0,      // Format_RGB444,
+        0,      // Format_ARGB4444_Premultiplied,
+#if Q_BYTE_ORDER == Q_LITTLE_ENDIAN
+        qt_transform_image_rgb32_on_rgb32,        // Format_RGBX8888,
+        0,      // Format_RGBA8888,
+        qt_transform_image_argb32_on_argb32,      // Format_RGBA8888_Premultiplied,
+#else
+        0,      // Format_RGBX8888,
+        0,      // Format_RGBA8888,
+        0       // Format_RGBA8888_Premultiplied,
+#endif
     }
 };
 
