@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2012 Digia Plc and/or its subsidiary(-ies).
+** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
@@ -529,7 +529,10 @@ MemRotateFunc qMemRotateFunctions[QImage::NImageFormats][3] =
     { 0, 0, 0 },      // Format_ARGB8555_Premultiplied,
     { 0, 0, 0 },      // Format_RGB888,
     { 0, 0, 0 },      // Format_RGB444,
-    { 0, 0, 0 }       // Format_ARGB4444_Premultiplied,
+    { 0, 0, 0 },      // Format_ARGB4444_Premultiplied,
+    { qt_memrotate90_32, qt_memrotate180_32, qt_memrotate270_32 },      // Format_RGBX8888,
+    { qt_memrotate90_32, qt_memrotate180_32, qt_memrotate270_32 },      // Format_RGBA8888,
+    { qt_memrotate90_32, qt_memrotate180_32, qt_memrotate270_32 }       // Format_RGBA8888_Premultiplied,
 };
 
 QT_END_NAMESPACE
