@@ -71,7 +71,7 @@ static const struct {
     {125, 176}, // B6
     {88, 125}, // B7
     {62, 88}, // B8
-    {33, 62}, // B9
+    {44, 62}, // B9
     {163, 229}, // C5E
     {105, 241}, // US Common
     {110, 220}, // DLE
@@ -133,7 +133,7 @@ QPagedPaintDevice::~QPagedPaintDevice()
   \value B6 125 x 176 mm
   \value B7 88 x 125 mm
   \value B8 62 x 88 mm
-  \value B9 33 x 62 mm
+  \value B9 44 x 62 mm
   \value B10 31 x 44 mm
   \value C5E 163 x 229 mm
   \value Comm10E 105 x 241 mm, U.S. Common 10 Envelope
@@ -162,14 +162,14 @@ QPagedPaintDevice::~QPagedPaintDevice()
 /*!
   Sets the size of the a page to \a size.
 
-  \sa setPageSizeMM
+  \sa setPageSizeMM()
   */
 void QPagedPaintDevice::setPageSize(PageSize size)
 {
     if (size >= Custom)
         return;
     d->pageSize = size;
-    d->pageSizeMM = QSizeF(pageSizes[A4].width, pageSizes[A4].height);
+    d->pageSizeMM = QSizeF(pageSizes[size].width, pageSizes[size].height);
 }
 
 /*!
@@ -205,7 +205,7 @@ QSizeF QPagedPaintDevice::pageSizeMM() const
   The margins are purely a hint to the drawing method. They don't affect the
   coordinate system or clipping.
 
-  \sa margins
+  \sa margins()
   */
 void QPagedPaintDevice::setMargins(const Margins &margins)
 {
@@ -215,7 +215,7 @@ void QPagedPaintDevice::setMargins(const Margins &margins)
 /*!
   returns the current margins of the paint device. The default is 0.
 
-  \sa setMargins
+  \sa setMargins()
   */
 QPagedPaintDevice::Margins QPagedPaintDevice::margins() const
 {
