@@ -44,8 +44,6 @@
 
 #include <QtGui/qpaintdevice.h>
 
-QT_BEGIN_HEADER
-
 QT_BEGIN_NAMESPACE
 
 #if defined(B0)
@@ -90,6 +88,4 @@ protected:
 
 QT_END_NAMESPACE
 
-QT_END_HEADER
-
 #endif
