@@ -74,11 +74,11 @@ QTextItemIntCopy::QTextItemIntCopy(const QTextItem &item)
     char *glyphLayoutData = new char[size];
     QGlyphLayout glyphs(glyphLayoutData, m_item.glyphs.numGlyphs);
     memcpy(glyphs.offsets, m_item.glyphs.offsets, m_item.glyphs.numGlyphs * sizeof(QFixedPoint));
-    memcpy(glyphs.glyphs, m_item.glyphs.glyphs, m_item.glyphs.numGlyphs * sizeof(HB_Glyph));
+    memcpy(glyphs.glyphs, m_item.glyphs.glyphs, m_item.glyphs.numGlyphs * sizeof(glyph_t));
     memcpy(glyphs.advances_x, m_item.glyphs.advances_x, m_item.glyphs.numGlyphs * sizeof(QFixed));
     memcpy(glyphs.advances_y, m_item.glyphs.advances_y, m_item.glyphs.numGlyphs * sizeof(QFixed));
     memcpy(glyphs.justifications, m_item.glyphs.justifications, m_item.glyphs.numGlyphs * sizeof(QGlyphJustification));
-    memcpy(glyphs.attributes, m_item.glyphs.attributes, m_item.glyphs.numGlyphs * sizeof(HB_GlyphAttributes));
+    memcpy(glyphs.attributes, m_item.glyphs.attributes, m_item.glyphs.numGlyphs * sizeof(QGlyphAttributes));
     m_item.glyphs = glyphs;
 
     m_font = *m_item.f;
@@ -89,9 +89,9 @@ QTextItemIntCopy::QTextItemIntCopy(const QTextItem &item)
 
 QTextItemIntCopy::~QTextItemIntCopy()
 {
-    delete m_item.chars;
-    delete m_item.logClusters;
-    delete m_item.glyphs.data();
+    delete [] m_item.chars;
+    delete [] m_item.logClusters;
+    delete [] m_item.glyphs.data();
     if (!m_item.fontEngine->ref.deref())
         delete m_item.fontEngine;
 }
@@ -1731,12 +1731,12 @@ void QPainterReplayer::process(const QPaintBufferCommand &cmd)
 #endif
         painter->setClipRegion(region, Qt::ClipOperation(cmd.extra));
         break; }
-        
+
 #if !defined(QT_NO_RAWFONT)
     case QPaintBufferPrivate::Cmd_DrawStaticText: {
-            
+
             QVariantList variants(d->variants.at(cmd.offset).value<QVariantList>());
-            
+
             QFont font = variants.at(0).value<QFont>();
 
             QVector<quint32> glyphIndexes;
@@ -1752,7 +1752,7 @@ void QPainterReplayer::process(const QPaintBufferCommand &cmd)
             QRawFont rawFont;
             QRawFontPrivate *rawFontD = QRawFontPrivate::get(rawFont);
             QFontPrivate *fontD = QFontPrivate::get(font);
-            rawFontD->fontEngine = fontD->engineForScript(QUnicodeTables::Common);
+            rawFontD->fontEngine = fontD->engineForScript(QChar::Script_Common);
             rawFontD->fontEngine->ref.ref();
 
             QGlyphRun glyphs;
