@@ -47,8 +47,6 @@
 #include <QtCore/qscopedpointer.h>
 #include <QtGui/qpainter.h>
 
-QT_BEGIN_HEADER
-
 QT_BEGIN_NAMESPACE
 
 
@@ -198,7 +196,7 @@ public:
         Windows,
         QuickDraw, CoreGraphics, MacPrinter,
         QWindowSystem,
-        PostScript,
+        PostScript,   // ### Qt 6: Remove, update documentation
         OpenGL,
         Picture,
         SVG,
@@ -249,13 +247,6 @@ private:
     friend class QFontEngineBox;
     friend class QFontEngineMac;
     friend class QFontEngineWin;
-#ifndef QT_NO_QWS_QPF
-    friend class QFontEngineQPF1;
-#endif
-#ifndef QT_NO_QWS_QPF2
-    friend class QFontEngineQPF;
-#endif
-    friend class QPSPrintEngine;
     friend class QMacPrintEngine;
     friend class QMacPrintEnginePrivate;
     friend class QFontEngineQPA;
@@ -348,6 +339,4 @@ Q_DECLARE_OPERATORS_FOR_FLAGS(QPaintEngine::DirtyFlags)
 
 QT_END_NAMESPACE
 
-QT_END_HEADER
-
 #endif // QPAINTENGINE_H
