@@ -61,8 +61,6 @@
 #include <private/qvectorpath_p.h>
 
 
-QT_BEGIN_HEADER
-
 QT_BEGIN_NAMESPACE
 
 
@@ -164,7 +162,7 @@ public:
         IsEmulationEngine = 0x02    // If set, this object is a QEmulationEngine.
     };
     virtual uint flags() const {return 0;}
-    virtual bool supportsTransformations(QFontEngine *fontEngine, const QTransform &m) const;
+    virtual bool requiresPretransformedGlyphPositions(QFontEngine *fontEngine, const QTransform &m) const;
     virtual bool shouldDrawCachedGlyphs(QFontEngine *fontEngine, const QTransform &m) const;
 
 protected:
@@ -192,6 +190,4 @@ public:
 
 QT_END_NAMESPACE
 
-QT_END_HEADER
-
 #endif
