@@ -49,8 +49,6 @@
 #include <QtCore/qvector.h>
 #include <QtCore/qscopedpointer.h>
 
-QT_BEGIN_HEADER
-
 QT_BEGIN_NAMESPACE
 
 
@@ -375,6 +373,4 @@ Q_GUI_EXPORT QDebug operator<<(QDebug, const QPainterPath &);
 
 QT_END_NAMESPACE
 
-QT_END_HEADER
-
 #endif // QPAINTERPATH_H
