@@ -45,6 +45,7 @@
 #include <private/qdatabuffer_p.h>
 #include <private/qnumeric_p.h>
 #include <qmath.h>
+#include <algorithm>
 
 /**
   The algorithm is as follows:
@@ -713,8 +714,7 @@ class QKdPointFinder
 {
 public:
     QKdPointFinder(int point, const QPathSegments &segments, QKdPointTree &tree)
-        : m_point(point)
-        , m_result(-1)
+        : m_result(-1)
         , m_segments(&segments)
         , m_tree(&tree)
     {
@@ -759,7 +759,6 @@ public:
     }
 
 private:
-    int m_point;
     qreal pointComponents[2];
     int m_result;
     const QPathSegments *m_segments;
@@ -826,7 +825,7 @@ void QWingedEdge::intersectAndAdd()
             }
         }
 
-        qSort(intersections.data(), intersections.data() + intersections.size());
+        std::sort(intersections.data(), intersections.data() + intersections.size());
 
         int first = m_segments.segmentAt(i).va;
         int second = m_segments.segmentAt(i).vb;
@@ -1038,16 +1037,6 @@ qreal QWingedEdge::delta(int vertex, int a, int b) const
         return result;
 }
 
-static inline QPointF midPoint(const QWingedEdge &list, int ei)
-{
-    const QPathEdge *ep = list.edge(ei);
-    Q_ASSERT(ep);
-
-    const QPointF a = *list.vertex(ep->first);
-    const QPointF b = *list.vertex(ep->second);
-    return a + 0.5 * (b - a);
-}
-
 QWingedEdge::TraversalStatus QWingedEdge::findInsertStatus(int vi, int ei) const
 {
     const QPathVertex *vp = vertex(vi);
@@ -1663,7 +1652,7 @@ bool QPathClipper::doClip(QWingedEdge &list, ClipperMode mode)
     for (int i = 0; i < list.vertexCount(); ++i)
         y_coords << list.vertex(i)->y;
 
-    qSort(y_coords.begin(), y_coords.end());
+    std::sort(y_coords.begin(), y_coords.end());
     y_coords.resize(qRemoveDuplicates(y_coords.begin(), y_coords.end(), fuzzyCompare) - y_coords.begin());
 
 #ifdef QDEBUG_CLIPPER
@@ -1839,7 +1828,7 @@ bool QPathClipper::handleCrossingEdges(QWingedEdge &list, qreal y, ClipperMode m
     QVector<QCrossingEdge> crossings = findCrossings(list, y);
 
     Q_ASSERT(!crossings.isEmpty());
-    qSort(crossings.begin(), crossings.end());
+    std::sort(crossings.begin(), crossings.end());
 
     int windingA = 0;
     int windingB = 0;
