@@ -38,9 +38,13 @@
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
+
+#include "qpdf_p.h"
+
+#ifndef QT_NO_PDF
+
 #include "qplatformdefs.h"
 #include <qdebug.h>
-#include "qpdf_p.h"
 #include <qfile.h>
 #include <qtemporaryfile.h>
 #include <private/qmath_p.h>
@@ -1455,6 +1459,9 @@ int QPdfEngine::metric(QPaintDevice::PaintDeviceMetric metricType) const
     case QPaintDevice::PdmDepth:
         val = 32;
         break;
+    case QPaintDevice::PdmDevicePixelRatio:
+        val = 1;
+        break;
     default:
         qWarning("QPdfWriter::metric: Invalid metric command");
         return 0;
@@ -1462,7 +1469,12 @@ int QPdfEngine::metric(QPaintDevice::PaintDeviceMetric metricType) const
     return val;
 }
 
+static inline QSizeF pageSizeToPostScriptPoints(const QSizeF &pageSizeMM)
+{
 #define Q_MM(n) int((n * 720 + 127) / 254)
+    return QSizeF(Q_MM(pageSizeMM.width()), Q_MM(pageSizeMM.height()));
+#undef Q_MM
+}
 
 QPdfEnginePrivate::QPdfEnginePrivate()
     : clipEnabled(false), allClipped(false), hasPen(true), hasBrush(false), simplePen(false),
@@ -1470,11 +1482,10 @@ QPdfEnginePrivate::QPdfEnginePrivate()
       fullPage(false), embedFonts(true),
       landscape(false),
       grayscale(false),
-      paperSize(Q_MM(210), Q_MM(297)), // A4
+      paperSize(pageSizeToPostScriptPoints(QSizeF(210, 297))), // A4
       leftMargin(10), topMargin(10), rightMargin(10), bottomMargin(10) // ~3.5 mm
 {
     resolution = 1200;
-    postscript = false;
     currentObject = 1;
     currentPage = 0;
     stroker.stream = 0;
@@ -1484,6 +1495,11 @@ QPdfEnginePrivate::QPdfEnginePrivate()
     stream = new QDataStream;
 }
 
+void QPdfEnginePrivate::setPaperSize(const QSizeF &pageSizeMM)
+{
+    paperSize = pageSizeToPostScriptPoints(pageSizeMM);
+}
+
 bool QPdfEngine::begin(QPaintDevice *pdev)
 {
     Q_D(QPdfEngine);
@@ -1503,7 +1519,6 @@ bool QPdfEngine::begin(QPaintDevice *pdev)
         d->ownsDevice = true;
     }
 
-    d->postscript = false;
     d->currentObject = 1;
 
     d->currentPage = new QPdfPage;
@@ -1634,7 +1649,7 @@ void QPdfEnginePrivate::writeInfo()
     xprintf("\n/Creator ");
     printString(creator);
     xprintf("\n/Producer ");
-    printString(QString::fromLatin1("Qt " QT_VERSION_STR " (C) 2012 Digia Plc and/or its subsidiary(-ies)"));
+    printString(QString::fromLatin1("Qt " QT_VERSION_STR));
     QDateTime now = QDateTime::currentDateTime().toUTC();
     QTime t = now.time();
     QDate d = now.date();
@@ -2489,8 +2504,8 @@ void QPdfEnginePrivate::drawTextItem(const QPointF &p, const QTextItemInt &ti)
     QFontEngine::FaceId face_id = fe->faceId();
     bool noEmbed = false;
     if (face_id.filename.isEmpty()
-        || (!postscript && ((fe->fsType & 0x200) /* bitmap embedding only */
-                            || (fe->fsType == 2) /* no embedding allowed */))) {
+        || fe->fsType & 0x200 /* bitmap embedding only */
+        || fe->fsType == 2 /* no embedding allowed */) {
         *currentPage << "Q\n";
         q->QPaintEngine::drawTextItem(p, ti);
         *currentPage << "q\n";
@@ -2511,6 +2526,10 @@ void QPdfEnginePrivate::drawTextItem(const QPointF &p, const QTextItemInt &ti)
 
     qreal size = ti.fontEngine->fontDef.pixelSize;
 
+#if defined(Q_OS_WIN)
+    size = (ti.fontEngine->ascent() + ti.fontEngine->descent()).toReal();
+#endif
+
     QVarLengthArray<glyph_t> glyphs;
     QVarLengthArray<QFixedPoint> positions;
     QTransform m = QTransform::fromTranslate(p.x(), p.y());
@@ -2621,5 +2640,6 @@ void QPdfEnginePrivate::newPage()
                  << "q q\n";
 }
 
-
 QT_END_NAMESPACE
+
+#endif // QT_NO_PDF
