@@ -52,6 +52,11 @@
 //
 // We mean it.
 //
+
+#include <QtCore/qglobal.h>
+
+#ifndef QT_NO_PDF
+
 #include "QtGui/qmatrix.h"
 #include "QtCore/qstring.h"
 #include "QtCore/qvector.h"
@@ -221,6 +226,7 @@ public:
 
     QRect paperRect() const;
     QRect pageRect() const;
+    void setPaperSize(const QSizeF &pageSizeMM);
 
     int width() const {
         QRect r = paperRect();
@@ -244,7 +250,6 @@ public:
 
     void newPage();
 
-    bool postscript;
     int currentObject;
 
     QPdfPage* currentPage;
@@ -328,5 +333,7 @@ void QPdfEngine::setResolution(int resolution)
 
 QT_END_NAMESPACE
 
+#endif // QT_NO_PDF
+
 #endif // QPDF_P_H
 
