@@ -42,14 +42,15 @@
 #ifndef QPDFWRITER_H
 #define QPDFWRITER_H
 
+#include <QtCore/qglobal.h>
+
+#ifndef QT_NO_PDF
+
 #include <QtCore/qobject.h>
 #include <QtGui/qpagedpaintdevice.h>
 
-QT_BEGIN_HEADER
-
 QT_BEGIN_NAMESPACE
 
-
 class QIODevice;
 class QPdfWriterPrivate;
 
@@ -85,6 +86,6 @@ private:
 
 QT_END_NAMESPACE
 
-QT_END_HEADER
+#endif // QT_NO_PDF
 
 #endif
