@@ -45,8 +45,6 @@
 #include <QtGui/qcolor.h>
 #include <QtGui/qbrush.h>
 
-QT_BEGIN_HEADER
-
 QT_BEGIN_NAMESPACE
 
 
@@ -139,6 +137,4 @@ Q_GUI_EXPORT QDebug operator<<(QDebug, const QPen &);
 
 QT_END_NAMESPACE
 
-QT_END_HEADER
-
 #endif // QPEN_H
