@@ -56,8 +56,6 @@
 #include <QtGui/qwindow.h>
 #include <QtGui/qregion.h>
 
-QT_BEGIN_HEADER
-
 QT_BEGIN_NAMESPACE
 
 
@@ -95,6 +93,4 @@ private:
 
 QT_END_NAMESPACE
 
-QT_END_HEADER
-
 #endif // QPLATFORMBACKINGSTORE_H
