@@ -50,12 +50,6 @@
 #include <QtCore/qpoint.h>
 #include <QtCore/qrect.h>
 
-#if defined(Q_OS_VXWORKS) && defined(m_type)
-#  undef m_type
-#endif
-
-QT_BEGIN_HEADER
-
 QT_BEGIN_NAMESPACE
 
 
@@ -389,6 +383,4 @@ inline QTransform operator -(const QTransform &a, qreal n)
 
 QT_END_NAMESPACE
 
-QT_END_HEADER
-
 #endif // QTRANSFORM_H
