@@ -55,7 +55,6 @@
 #include "qthread.h"
 #include "qthreadstorage.h"
 
-#include <private/qunicodetables_p.h>
 #include "qfont_p.h"
 #include <private/qfontengine_p.h>
 #include <private/qpainter_p.h>
@@ -192,8 +191,8 @@ QFontPrivate::QFontPrivate(const QFontPrivate &other)
 
 QFontPrivate::~QFontPrivate()
 {
-    if (engineData)
-        engineData->ref.deref();
+    if (engineData && !engineData->ref.deref())
+        delete engineData;
     engineData = 0;
     if (scFont && scFont != this)
         scFont->ref.deref();
@@ -207,11 +206,12 @@ extern QMutex *qt_fontdatabase_mutex();
 QFontEngine *QFontPrivate::engineForScript(int script) const
 {
     QMutexLocker locker(qt_fontdatabase_mutex());
-    if (script >= QUnicodeTables::Inherited)
-        script = QUnicodeTables::Common;
-    if (engineData && engineData->fontCache != QFontCache::instance()) {
+    if (script <= QChar::Script_Latin)
+        script = QChar::Script_Common;
+    if (engineData && engineData->fontCacheId != QFontCache::instance()->id()) {
         // throw out engineData that came from a different thread
-        engineData->ref.deref();
+        if (!engineData->ref.deref())
+            delete engineData;
         engineData = 0;
     }
     if (!engineData || !QT_FONT_ENGINE_FROM_DATA(engineData, script))
@@ -317,17 +317,20 @@ void QFontPrivate::resolve(uint mask, const QFontPrivate *other)
 
 
 QFontEngineData::QFontEngineData()
-    : ref(1), fontCache(QFontCache::instance())
+    : ref(0), fontCacheId(QFontCache::instance()->id())
 {
-    memset(engines, 0, QUnicodeTables::ScriptCount * sizeof(QFontEngine *));
+    memset(engines, 0, QChar::ScriptCount * sizeof(QFontEngine *));
 }
 
 QFontEngineData::~QFontEngineData()
 {
-    for (int i = 0; i < QUnicodeTables::ScriptCount; ++i) {
-        if (engines[i])
-            engines[i]->ref.deref();
-        engines[i] = 0;
+    Q_ASSERT(ref.load() == 0);
+    for (int i = 0; i < QChar::ScriptCount; ++i) {
+        if (engines[i]) {
+            if (!engines[i]->ref.deref())
+                delete engines[i];
+            engines[i] = 0;
+        }
     }
 }
 
@@ -352,7 +355,7 @@ QFontEngineData::~QFontEngineData()
     attributes, or if no matching font exists, Qt will use the closest
     matching installed font. The attributes of the font that is
     actually used are retrievable from a QFontInfo object. If the
-    window system provides an exact match exactMatch() returns true.
+    window system provides an exact match exactMatch() returns \c true.
     Use QFontMetrics to get measurements, e.g. the pixel length of a
     string using QFontMetrics::width().
 
@@ -533,6 +536,14 @@ QFontEngineData::~QFontEngineData()
     \sa Weight
 */
 
+/*!
+    \fn QFont &QFont::operator=(QFont &&other)
+
+    Move-assigns \a other to this QFont instance.
+
+    \since 5.2
+*/
+
 /*!
     \fn QString QFont::rawName() const
 
@@ -636,8 +647,8 @@ QFont::QFont(QFontPrivate *data)
 void QFont::detach()
 {
     if (d->ref.load() == 1) {
-        if (d->engineData)
-            d->engineData->ref.deref();
+        if (d->engineData && !d->engineData->ref.deref())
+            delete d->engineData;
         d->engineData = 0;
         if (d->scFont && d->scFont != d.data())
             d->scFont->ref.deref();
@@ -1038,7 +1049,7 @@ int QFont::pixelSize() const
 /*!
   \fn bool QFont::italic() const
 
-    Returns true if the style() of the font is not QFont::StyleNormal
+    Returns \c true if the style() of the font is not QFont::StyleNormal
 
     \sa setItalic(), style()
 */
@@ -1128,8 +1139,8 @@ void QFont::setWeight(int weight)
 /*!
     \fn bool QFont::bold() const
 
-    Returns true if weight() is a value greater than
-   \l{Weight}{QFont::Normal}; otherwise returns false.
+    Returns \c true if weight() is a value greater than
+   \l{Weight}{QFont::Normal}; otherwise returns \c false.
 
     \sa weight(), setBold(), QFontInfo::bold()
 */
@@ -1147,7 +1158,7 @@ void QFont::setWeight(int weight)
 */
 
 /*!
-    Returns true if underline has been set; otherwise returns false.
+    Returns \c true if underline has been set; otherwise returns \c false.
 
     \sa setUnderline()
 */
@@ -1174,7 +1185,7 @@ void QFont::setUnderline(bool enable)
 }
 
 /*!
-    Returns true if overline has been set; otherwise returns false.
+    Returns \c true if overline has been set; otherwise returns \c false.
 
     \sa setOverline()
 */
@@ -1200,7 +1211,7 @@ void QFont::setOverline(bool enable)
 }
 
 /*!
-    Returns true if strikeout has been set; otherwise returns false.
+    Returns \c true if strikeout has been set; otherwise returns \c false.
 
     \sa setStrikeOut()
 */
@@ -1227,7 +1238,7 @@ void QFont::setStrikeOut(bool enable)
 }
 
 /*!
-    Returns true if fixed pitch has been set; otherwise returns false.
+    Returns \c true if fixed pitch has been set; otherwise returns \c false.
 
     \sa setFixedPitch(), QFontInfo::fixedPitch()
 */
@@ -1255,7 +1266,7 @@ void QFont::setFixedPitch(bool enable)
 }
 
 /*!
-  Returns true if kerning should be used when drawing text with this font.
+  Returns \c true if kerning should be used when drawing text with this font.
 
   \sa setKerning()
 */
@@ -1660,14 +1671,14 @@ void QFont::setRawMode(bool enable)
 }
 
 /*!
-    Returns true if a window system font exactly matching the settings
+    Returns \c true if a window system font exactly matching the settings
     of this font is available.
 
     \sa QFontInfo
 */
 bool QFont::exactMatch() const
 {
-    QFontEngine *engine = d->engineForScript(QUnicodeTables::Common);
+    QFontEngine *engine = d->engineForScript(QChar::Script_Common);
     Q_ASSERT(engine != 0);
     return (d->rawMode
             ? engine->type() != QFontEngine::Box
@@ -1675,7 +1686,7 @@ bool QFont::exactMatch() const
 }
 
 /*!
-    Returns true if this font is equal to \a f; otherwise returns
+    Returns \c true if this font is equal to \a f; otherwise returns
     false.
 
     Two QFonts are considered equal if their font attributes are
@@ -1703,7 +1714,7 @@ bool QFont::operator==(const QFont &f) const
 
 /*!
     Provides an arbitrary comparison of this font and font \a f.
-    All that is guaranteed is that the operator returns false if both
+    All that is guaranteed is that the operator returns \c false if both
     fonts are equal and that (f1 \< f2) == !(f2 \< f1) if the fonts
     are not equal.
 
@@ -1739,8 +1750,8 @@ bool QFont::operator<(const QFont &f) const
 
 
 /*!
-    Returns true if this font is different from \a f; otherwise
-    returns false.
+    Returns \c true if this font is different from \a f; otherwise
+    returns \c false.
 
     Two QFonts are considered to be different if their font attributes
     are different. If rawMode() is enabled for both fonts, only the
@@ -1762,7 +1773,7 @@ QFont::operator QVariant() const
 }
 
 /*!
-    Returns true if this font and \a f are copies of each other, i.e.
+    Returns \c true if this font and \a f are copies of each other, i.e.
     one of them was created as a copy of the other and neither has
     been modified since. This is much stricter than equality.
 
@@ -1774,8 +1785,8 @@ bool QFont::isCopyOf(const QFont & f) const
 }
 
 /*!
-    Returns true if raw mode is used for font name matching; otherwise
-    returns false.
+    Returns \c true if raw mode is used for font name matching; otherwise
+    returns \c false.
 
     \sa setRawMode(), rawName()
 */
@@ -2363,7 +2374,7 @@ QFontInfo &QFontInfo::operator=(const QFontInfo &fi)
 */
 QString QFontInfo::family() const
 {
-    QFontEngine *engine = d->engineForScript(QUnicodeTables::Common);
+    QFontEngine *engine = d->engineForScript(QChar::Script_Common);
     Q_ASSERT(engine != 0);
     return engine->fontDef.family;
 }
@@ -2378,7 +2389,7 @@ QString QFontInfo::family() const
 */
 QString QFontInfo::styleName() const
 {
-    QFontEngine *engine = d->engineForScript(QUnicodeTables::Common);
+    QFontEngine *engine = d->engineForScript(QChar::Script_Common);
     Q_ASSERT(engine != 0);
     return engine->fontDef.styleName;
 }
@@ -2390,7 +2401,7 @@ QString QFontInfo::styleName() const
 */
 int QFontInfo::pointSize() const
 {
-    QFontEngine *engine = d->engineForScript(QUnicodeTables::Common);
+    QFontEngine *engine = d->engineForScript(QChar::Script_Common);
     Q_ASSERT(engine != 0);
     return qRound(engine->fontDef.pointSize);
 }
@@ -2402,7 +2413,7 @@ int QFontInfo::pointSize() const
 */
 qreal QFontInfo::pointSizeF() const
 {
-    QFontEngine *engine = d->engineForScript(QUnicodeTables::Common);
+    QFontEngine *engine = d->engineForScript(QChar::Script_Common);
     Q_ASSERT(engine != 0);
     return engine->fontDef.pointSize;
 }
@@ -2414,7 +2425,7 @@ qreal QFontInfo::pointSizeF() const
 */
 int QFontInfo::pixelSize() const
 {
-    QFontEngine *engine = d->engineForScript(QUnicodeTables::Common);
+    QFontEngine *engine = d->engineForScript(QChar::Script_Common);
     Q_ASSERT(engine != 0);
     return engine->fontDef.pixelSize;
 }
@@ -2426,7 +2437,7 @@ int QFontInfo::pixelSize() const
 */
 bool QFontInfo::italic() const
 {
-    QFontEngine *engine = d->engineForScript(QUnicodeTables::Common);
+    QFontEngine *engine = d->engineForScript(QChar::Script_Common);
     Q_ASSERT(engine != 0);
     return engine->fontDef.style != QFont::StyleNormal;
 }
@@ -2438,7 +2449,7 @@ bool QFontInfo::italic() const
 */
 QFont::Style QFontInfo::style() const
 {
-    QFontEngine *engine = d->engineForScript(QUnicodeTables::Common);
+    QFontEngine *engine = d->engineForScript(QChar::Script_Common);
     Q_ASSERT(engine != 0);
     return (QFont::Style)engine->fontDef.style;
 }
@@ -2450,7 +2461,7 @@ QFont::Style QFontInfo::style() const
 */
 int QFontInfo::weight() const
 {
-    QFontEngine *engine = d->engineForScript(QUnicodeTables::Common);
+    QFontEngine *engine = d->engineForScript(QChar::Script_Common);
     Q_ASSERT(engine != 0);
     return engine->fontDef.weight;
 
@@ -2459,8 +2470,8 @@ int QFontInfo::weight() const
 /*!
     \fn bool QFontInfo::bold() const
 
-    Returns true if weight() would return a value greater than
-    QFont::Normal; otherwise returns false.
+    Returns \c true if weight() would return a value greater than
+    QFont::Normal; otherwise returns \c false.
 
     \sa weight(), QFont::bold()
 */
@@ -2515,7 +2526,7 @@ bool QFontInfo::strikeOut() const
 */
 bool QFontInfo::fixedPitch() const
 {
-    QFontEngine *engine = d->engineForScript(QUnicodeTables::Common);
+    QFontEngine *engine = d->engineForScript(QChar::Script_Common);
     Q_ASSERT(engine != 0);
 #ifdef Q_OS_MAC
     if (!engine->fontDef.fixedPitchComputed) {
@@ -2539,13 +2550,13 @@ bool QFontInfo::fixedPitch() const
 */
 QFont::StyleHint QFontInfo::styleHint() const
 {
-    QFontEngine *engine = d->engineForScript(QUnicodeTables::Common);
+    QFontEngine *engine = d->engineForScript(QChar::Script_Common);
     Q_ASSERT(engine != 0);
     return (QFont::StyleHint) engine->fontDef.styleHint;
 }
 
 /*!
-    Returns true if the font is a raw mode font; otherwise returns
+    Returns \c true if the font is a raw mode font; otherwise returns
     false.
 
     If it is a raw mode font, all other functions in QFontInfo will
@@ -2560,14 +2571,14 @@ bool QFontInfo::rawMode() const
 }
 
 /*!
-    Returns true if the matched window system font is exactly the same
-    as the one specified by the font; otherwise returns false.
+    Returns \c true if the matched window system font is exactly the same
+    as the one specified by the font; otherwise returns \c false.
 
     \sa QFont::exactMatch()
 */
 bool QFontInfo::exactMatch() const
 {
-    QFontEngine *engine = d->engineForScript(QUnicodeTables::Common);
+    QFontEngine *engine = d->engineForScript(QChar::Script_Common);
     Q_ASSERT(engine != 0);
     return (d->rawMode
             ? engine->type() != QFontEngine::Box
@@ -2627,45 +2638,18 @@ void QFontCache::cleanup()
 }
 #endif // QT_NO_THREAD
 
+QBasicAtomicInt font_cache_id = Q_BASIC_ATOMIC_INITIALIZER(1);
+
 QFontCache::QFontCache()
     : QObject(), total_cost(0), max_cost(min_cost),
-      current_timestamp(0), fast(false), timer_id(-1)
+      current_timestamp(0), fast(false), timer_id(-1),
+      m_id(font_cache_id.fetchAndAddRelaxed(1))
 {
 }
 
 QFontCache::~QFontCache()
 {
     clear();
-    {
-        EngineDataCache::ConstIterator it = engineDataCache.constBegin(),
-                                 end = engineDataCache.constEnd();
-        while (it != end) {
-            if (it.value()->ref.load() == 0)
-                delete it.value();
-            else
-                FC_DEBUG("QFontCache::~QFontCache: engineData %p still has refcount %d",
-                         it.value(), it.value()->ref.load());
-            ++it;
-        }
-    }
-    EngineCache::ConstIterator it = engineCache.constBegin(),
-                         end = engineCache.constEnd();
-    while (it != end) {
-        if (--it.value().data->cache_count == 0) {
-            if (it.value().data->ref.load() == 0) {
-                FC_DEBUG("QFontCache::~QFontCache: deleting engine %p key=(%d / %g %g %d %d %d)",
-                         it.value().data, it.key().script, it.key().def.pointSize,
-                         it.key().def.pixelSize, it.key().def.weight, it.key().def.style,
-                         it.key().def.fixedPitch);
-
-                delete it.value().data;
-            } else {
-                FC_DEBUG("QFontCache::~QFontCache: engine = %p still has refcount %d",
-                         it.value().data, it.value().data->ref.load());
-            }
-        }
-        ++it;
-    }
 }
 
 void QFontCache::clear()
@@ -2675,33 +2659,56 @@ void QFontCache::clear()
                                  end = engineDataCache.end();
         while (it != end) {
             QFontEngineData *data = it.value();
-            for (int i = 0; i < QUnicodeTables::ScriptCount; ++i) {
+            for (int i = 0; i < QChar::ScriptCount; ++i) {
                 if (data->engines[i]) {
-                    data->engines[i]->ref.deref();
+                    if (!data->engines[i]->ref.deref()) {
+                        Q_ASSERT(engineCacheCount.value(data->engines[i]) == 0);
+                        delete data->engines[i];
+                    }
                     data->engines[i] = 0;
                 }
             }
+            if (!data->ref.deref()) {
+                delete data;
+            } else {
+                FC_DEBUG("QFontCache::clear: engineData %p still has refcount %d",
+                         data, data->ref.load());
+            }
             ++it;
         }
     }
 
-    for (EngineCache::Iterator it = engineCache.begin(), end = engineCache.end();
-         it != end; ++it) {
-        if (it->data->ref.load() == 0) {
-            delete it->data;
-            it->data = 0;
-        }
-    }
+    engineDataCache.clear();
 
-    for (EngineCache::Iterator it = engineCache.begin(), end = engineCache.end();
-         it != end; ++it) {
-        if (it->data && it->data->ref.load() == 0) {
-            delete it->data;
-            it->data = 0;
+
+    bool mightHaveEnginesLeftForCleanup;
+    do {
+        mightHaveEnginesLeftForCleanup = false;
+        for (EngineCache::Iterator it = engineCache.begin(), end = engineCache.end();
+             it != end; ++it) {
+            QFontEngine *engine = it.value().data;
+            if (engine) {
+                const int cacheCount = --engineCacheCount[engine];
+                Q_ASSERT(cacheCount >= 0);
+                if (!engine->ref.deref()) {
+                    Q_ASSERT(cacheCount == 0);
+                    mightHaveEnginesLeftForCleanup = engine->type() == QFontEngine::Multi;
+                    delete engine;
+                } else if (cacheCount == 0) {
+                    FC_DEBUG("QFontCache::clear: engine %p still has refcount %d",
+                             engine, engine->ref.load());
+                }
+                it.value().data = 0;
+            }
         }
-    }
+    } while (mightHaveEnginesLeftForCleanup);
 
     engineCache.clear();
+    engineCacheCount.clear();
+
+
+    total_cost = 0;
+    max_cost = min_cost;
 }
 
 
@@ -2717,8 +2724,17 @@ QFontEngineData *QFontCache::findEngineData(const QFontDef &def) const
 
 void QFontCache::insertEngineData(const QFontDef &def, QFontEngineData *engineData)
 {
+#ifdef QFONTCACHE_DEBUG
     FC_DEBUG("QFontCache: inserting new engine data %p", engineData);
+    if (engineDataCache.contains(def)) {
+        FC_DEBUG("   QFontCache already contains engine data %p for key=(%g %g %d %d %d)",
+                 engineDataCache.value(def), def.pointSize,
+                 def.pixelSize, def.weight, def.style, def.fixedPitch);
+    }
+#endif
+    Q_ASSERT(!engineDataCache.contains(def));
 
+    engineData->ref.ref();
     engineDataCache.insert(def, engineData);
     increaseCost(sizeof(QFontEngineData));
 }
@@ -2742,13 +2758,22 @@ void QFontCache::updateHitCountAndTimeStamp(Engine &value)
     FC_DEBUG("QFontCache: found font engine\n"
              "  %p: timestamp %4u hits %3u ref %2d/%2d, type '%s'",
              value.data, value.timestamp, value.hits,
-             value.data->ref.load(), value.data->cache_count,
+             value.data->ref.load(), engineCacheCount.value(value.data),
              value.data->name());
 }
 
 void QFontCache::insertEngine(const Key &key, QFontEngine *engine, bool insertMulti)
 {
-    FC_DEBUG("QFontCache: inserting new engine %p", engine);
+#ifdef QFONTCACHE_DEBUG
+    FC_DEBUG("QFontCache: inserting new engine %p, refcount %d", engine, engine->ref.load());
+    if (!insertMulti && engineCache.contains(key)) {
+        FC_DEBUG("   QFontCache already contains engine %p for key=(%g %g %d %d %d)",
+                 engineCache.value(key).data, key.def.pointSize,
+                 key.def.pixelSize, key.def.weight, key.def.style, key.def.fixedPitch);
+    }
+#endif
+
+    engine->ref.ref();
 
     Engine data(engine);
     data.timestamp = ++current_timestamp;
@@ -2757,12 +2782,9 @@ void QFontCache::insertEngine(const Key &key, QFontEngine *engine, bool insertMu
         engineCache.insertMulti(key, data);
     else
         engineCache.insert(key, data);
-
     // only increase the cost if this is the first time we insert the engine
-    if (engine->cache_count == 0)
+    if (++engineCacheCount[engine] == 1)
         increaseCost(engine->cache_cost);
-
-    ++engine->cache_count;
 }
 
 void QFontCache::increaseCost(uint cost)
@@ -2826,12 +2848,9 @@ void QFontCache::timerEvent(QTimerEvent *)
         EngineDataCache::ConstIterator it = engineDataCache.constBegin(),
                                       end = engineDataCache.constEnd();
         for (; it != end; ++it) {
-#ifdef QFONTCACHE_DEBUG
             FC_DEBUG("    %p: ref %2d", it.value(), int(it.value()->ref.load()));
 
-#endif // QFONTCACHE_DEBUG
-
-            if (it.value()->ref.load() != 0)
+            if (it.value()->ref.load() != 1)
                 in_use_cost += engine_data_cost;
         }
     }
@@ -2844,11 +2863,11 @@ void QFontCache::timerEvent(QTimerEvent *)
         for (; it != end; ++it) {
             FC_DEBUG("    %p: timestamp %4u hits %2u ref %2d/%2d, cost %u bytes",
                      it.value().data, it.value().timestamp, it.value().hits,
-                     it.value().data->ref.load(), it.value().data->cache_count,
+                     it.value().data->ref.load(), engineCacheCount.value(it.value().data),
                      it.value().data->cache_cost);
 
             if (it.value().data->ref.load() != 0)
-                in_use_cost += it.value().data->cache_cost / it.value().data->cache_count;
+                in_use_cost += it.value().data->cache_cost / engineCacheCount.value(it.value().data);
         }
 
         // attempt to make up for rounding errors
@@ -2894,29 +2913,26 @@ void QFontCache::timerEvent(QTimerEvent *)
         FC_DEBUG("  CLEAN engine data:");
 
         // clean out all unused engine data
-        EngineDataCache::Iterator it = engineDataCache.begin(),
-                                 end = engineDataCache.end();
-        while (it != end) {
-            if (it.value()->ref.load() != 0) {
+        EngineDataCache::Iterator it = engineDataCache.begin();
+        while (it != engineDataCache.end()) {
+            if (it.value()->ref.load() == 1) {
+                FC_DEBUG("    %p", it.value());
+                decreaseCost(sizeof(QFontEngineData));
+                it.value()->ref.deref();
+                delete it.value();
+                it = engineDataCache.erase(it);
+            } else {
                 ++it;
-                continue;
             }
-
-            EngineDataCache::Iterator rem = it++;
-
-            decreaseCost(sizeof(QFontEngineData));
-
-            FC_DEBUG("    %p", rem.value());
-
-            delete rem.value();
-            engineDataCache.erase(rem);
         }
     }
 
+    FC_DEBUG("  CLEAN engine:");
+
     // clean out the engine cache just enough to get below our new max cost
-    uint current_cost;
+    bool cost_decreased;
     do {
-        current_cost = total_cost;
+        cost_decreased = false;
 
         EngineCache::Iterator it = engineCache.begin(),
                              end = engineCache.end();
@@ -2924,49 +2940,46 @@ void QFontCache::timerEvent(QTimerEvent *)
         uint oldest = ~0u;
         uint least_popular = ~0u;
 
-        for (; it != end; ++it) {
-            if (it.value().data->ref.load() != 0)
+        EngineCache::Iterator jt = end;
+
+        for ( ; it != end; ++it) {
+            if (it.value().data->ref.load() != engineCacheCount.value(it.value().data))
                 continue;
 
-            if (it.value().timestamp < oldest &&
-                 it.value().hits <= least_popular) {
+            if (it.value().timestamp < oldest && it.value().hits <= least_popular) {
                 oldest = it.value().timestamp;
                 least_popular = it.value().hits;
+                jt = it;
             }
         }
 
-        FC_DEBUG("    oldest %u least popular %u", oldest, least_popular);
-
-        for (it = engineCache.begin(); it != end; ++it) {
-            if (it.value().data->ref.load() == 0 &&
-                 it.value().timestamp == oldest &&
-                 it.value().hits == least_popular)
-                break;
-        }
-
+        it = jt;
         if (it != end) {
             FC_DEBUG("    %p: timestamp %4u hits %2u ref %2d/%2d, type '%s'",
                      it.value().data, it.value().timestamp, it.value().hits,
-                     it.value().data->ref.load(), it.value().data->cache_count,
+                     it.value().data->ref.load(), engineCacheCount.value(it.value().data),
                      it.value().data->name());
 
-            if (--it.value().data->cache_count == 0) {
-                FC_DEBUG("    DELETE: last occurrence in cache");
-
-                decreaseCost(it.value().data->cache_cost);
-                delete it.value().data;
-            } else {
-                /*
-                  this particular font engine is in the cache multiple
-                  times...  set current_cost to zero, so that we can
-                  keep looping to get rid of all occurrences
-                */
-                current_cost = 0;
+            QFontEngine *fontEngine = it.value().data;
+            // get rid of all occurrences
+            it = engineCache.begin();
+            while (it != engineCache.end()) {
+                if (it.value().data == fontEngine) {
+                    fontEngine->ref.deref();
+                    it = engineCache.erase(it);
+                } else {
+                    ++it;
+                }
             }
+            // and delete the last occurrence
+            Q_ASSERT(fontEngine->ref.load() == 0);
+            decreaseCost(fontEngine->cache_cost);
+            delete fontEngine;
+            engineCacheCount.remove(fontEngine);
 
-            engineCache.erase(it);
+            cost_decreased = true;
         }
-    } while (current_cost != total_cost && total_cost > max_cost);
+    } while (cost_decreased && total_cost > max_cost);
 }
 
 
