@@ -57,7 +57,6 @@
 #include "QtCore/qmap.h"
 #include "QtCore/qobject.h"
 #include "QtCore/qstringlist.h"
-#include <private/qunicodetables_p.h>
 #include <QtGui/qfontdatabase.h>
 #include "private/qfixed_p.h"
 
@@ -141,9 +140,12 @@ public:
     ~QFontEngineData();
 
     QAtomicInt ref;
-    QFontCache *fontCache;
+    const int fontCacheId;
 
-    QFontEngine *engines[QUnicodeTables::ScriptCount];
+    QFontEngine *engines[QChar::ScriptCount];
+
+private:
+    Q_DISABLE_COPY(QFontEngineData)
 };
 
 
@@ -194,9 +196,8 @@ private:
 };
 
 
-class QFontCache : public QObject
+class Q_AUTOTEST_EXPORT QFontCache : public QObject
 {
-    Q_OBJECT
 public:
     // note: these static functions work on a per-thread basis
     static QFontCache *instance();
@@ -205,9 +206,10 @@ public:
     QFontCache();
     ~QFontCache();
 
+    int id() const { return m_id; }
+
     void clear();
-    // universal key structure.  QFontEngineDatas and QFontEngines are cached using
-    // the same keys
+
     struct Key {
         Key() : script(0), screen(0) { }
         Key(const QFontDef &d, int c, int s = 0)
@@ -246,13 +248,14 @@ public:
 
     typedef QMap<Key,Engine> EngineCache;
     EngineCache engineCache;
+    QHash<QFontEngine *, int> engineCacheCount;
 
     QFontEngine *findEngine(const Key &key);
 
     void updateHitCountAndTimeStamp(Engine &value);
     void insertEngine(const Key &key, QFontEngine *engine, bool insertMulti = false);
 
-    private:
+private:
     void increaseCost(uint cost);
     void decreaseCost(uint cost);
     void timerEvent(QTimerEvent *event);
@@ -262,6 +265,7 @@ public:
     uint current_timestamp;
     bool fast;
     int timer_id;
+    const int m_id;
 };
 
 Q_GUI_EXPORT int qt_defaultDpiX();
