@@ -48,15 +48,16 @@
 #include "qmutex.h"
 #include "qfile.h"
 #include "qfileinfo.h"
-#include "private/qunicodetables_p.h"
 #include "qfontengine_p.h"
 #include <qpa/qplatformintegration.h>
 
 #include <QtGui/private/qguiapplication_p.h>
 #include <qpa/qplatformfontdatabase.h>
+#include <qpa/qplatformtheme.h>
 
 #include <stdlib.h>
 #include <limits.h>
+#include <algorithm>
 
 
 // #define QFONTDATABASE_DEBUG
@@ -315,9 +316,6 @@ struct  QtFontFamily
     QtFontFamily(const QString &n)
         :
         fixedPitch(false),
-#if !defined(QWS) && defined(Q_OS_MAC)
-        fixedPitchComputed(false),
-#endif
         name(n), count(0), foundries(0)
         , bogusWritingSystems(false)
         , askedForFallback(false)
@@ -331,9 +329,6 @@ struct  QtFontFamily
     }
 
     bool fixedPitch : 1;
-#if !defined(QWS) && defined(Q_OS_MAC)
-    bool fixedPitchComputed : 1;
-#endif
 
     QString name;
     QStringList aliases;
@@ -345,21 +340,10 @@ struct  QtFontFamily
     bool askedForFallback;
     unsigned char writingSystems[QFontDatabase::WritingSystemsCount];
 
+    bool matchesFamilyName(const QString &familyName) const;
     QtFontFoundry *foundry(const QString &f, bool = false);
 };
 
-#if !defined(QWS) && defined(Q_OS_MAC)
-inline static void qt_mac_get_fixed_pitch(QtFontFamily *f)
-{
-    if(f && !f->fixedPitchComputed) {
-        QFontMetrics fm(f->name);
-        f->fixedPitch = fm.width(QLatin1Char('i')) == fm.width(QLatin1Char('m'));
-        f->fixedPitchComputed = true;
-    }
-}
-#endif
-
-
 QtFontFoundry *QtFontFamily::foundry(const QString &f, bool create)
 {
     if (f.isNull() && count == 1)
@@ -384,131 +368,9 @@ QtFontFoundry *QtFontFamily::foundry(const QString &f, bool create)
     return foundries[count++];
 }
 
-// ### copied to tools/makeqpf/qpf2.cpp
-
-// see the Unicode subset bitfields in the MSDN docs
-static int requiredUnicodeBits[QFontDatabase::WritingSystemsCount][2] = {
-        // Any,
-    { 127, 127 },
-        // Latin,
-    { 0, 127 },
-        // Greek,
-    { 7, 127 },
-        // Cyrillic,
-    { 9, 127 },
-        // Armenian,
-    { 10, 127 },
-        // Hebrew,
-    { 11, 127 },
-        // Arabic,
-    { 13, 127 },
-        // Syriac,
-    { 71, 127 },
-    //Thaana,
-    { 72, 127 },
-    //Devanagari,
-    { 15, 127 },
-    //Bengali,
-    { 16, 127 },
-    //Gurmukhi,
-    { 17, 127 },
-    //Gujarati,
-    { 18, 127 },
-    //Oriya,
-    { 19, 127 },
-    //Tamil,
-    { 20, 127 },
-    //Telugu,
-    { 21, 127 },
-    //Kannada,
-    { 22, 127 },
-    //Malayalam,
-    { 23, 127 },
-    //Sinhala,
-    { 73, 127 },
-    //Thai,
-    { 24, 127 },
-    //Lao,
-    { 25, 127 },
-    //Tibetan,
-    { 70, 127 },
-    //Myanmar,
-    { 74, 127 },
-        // Georgian,
-    { 26, 127 },
-        // Khmer,
-    { 80, 127 },
-        // SimplifiedChinese,
-    { 126, 127 },
-        // TraditionalChinese,
-    { 126, 127 },
-        // Japanese,
-    { 126, 127 },
-        // Korean,
-    { 56, 127 },
-        // Vietnamese,
-    { 0, 127 }, // same as latin1
-        // Other,
-    { 126, 127 },
-        // Ogham,
-    { 78, 127 },
-        // Runic,
-    { 79, 127 },
-        // Nko,
-    { 14, 127 },
-};
-
-#define SimplifiedChineseCsbBit 18
-#define TraditionalChineseCsbBit 20
-#define JapaneseCsbBit 17
-#define KoreanCsbBit 21
-
-QList<QFontDatabase::WritingSystem> qt_determine_writing_systems_from_truetype_bits(quint32 unicodeRange[4], quint32 codePageRange[2])
+bool QtFontFamily::matchesFamilyName(const QString &familyName) const
 {
-    QList<QFontDatabase::WritingSystem> writingSystems;
-    bool hasScript = false;
-
-    int i;
-    for(i = 0; i < QFontDatabase::WritingSystemsCount; i++) {
-        int bit = requiredUnicodeBits[i][0];
-        int index = bit/32;
-        int flag =  1 << (bit&31);
-        if (bit != 126 && unicodeRange[index] & flag) {
-            bit = requiredUnicodeBits[i][1];
-            index = bit/32;
-
-            flag =  1 << (bit&31);
-            if (bit == 127 || unicodeRange[index] & flag) {
-                writingSystems.append(QFontDatabase::WritingSystem(i));
-                hasScript = true;
-                // qDebug("font %s: index=%d, flag=%8x supports script %d", familyName.latin1(), index, flag, i);
-            }
-        }
-    }
-    if(codePageRange[0] & (1 << SimplifiedChineseCsbBit)) {
-        writingSystems.append(QFontDatabase::SimplifiedChinese);
-        hasScript = true;
-        //qDebug("font %s supports Simplified Chinese", familyName.latin1());
-    }
-    if(codePageRange[0] & (1 << TraditionalChineseCsbBit)) {
-        writingSystems.append(QFontDatabase::TraditionalChinese);
-        hasScript = true;
-        //qDebug("font %s supports Traditional Chinese", familyName.latin1());
-    }
-    if(codePageRange[0] & (1 << JapaneseCsbBit)) {
-        writingSystems.append(QFontDatabase::Japanese);
-        hasScript = true;
-        //qDebug("font %s supports Japanese", familyName.latin1());
-    }
-    if(codePageRange[0] & (1 << KoreanCsbBit)) {
-        writingSystems.append(QFontDatabase::Korean);
-        hasScript = true;
-        //qDebug("font %s supports Korean", familyName.latin1());
-    }
-    if (!hasScript)
-        writingSystems.append(QFontDatabase::Symbol);
-
-    return writingSystems;
+    return name.compare(familyName, Qt::CaseInsensitive) == 0 || aliases.contains(familyName, Qt::CaseInsensitive);
 }
 
 
@@ -601,50 +463,44 @@ QtFontFamily *QFontDatabasePrivate::family(const QString &f, bool create)
 
 
 static const int scriptForWritingSystem[] = {
-    QUnicodeTables::Common, // Any
-    QUnicodeTables::Latin, // Latin
-    QUnicodeTables::Greek, // Greek
-    QUnicodeTables::Cyrillic, // Cyrillic
-    QUnicodeTables::Armenian, // Armenian
-    QUnicodeTables::Hebrew, // Hebrew
-    QUnicodeTables::Arabic, // Arabic
-    QUnicodeTables::Syriac, // Syriac
-    QUnicodeTables::Thaana, // Thaana
-    QUnicodeTables::Devanagari, // Devanagari
-    QUnicodeTables::Bengali, // Bengali
-    QUnicodeTables::Gurmukhi, // Gurmukhi
-    QUnicodeTables::Gujarati, // Gujarati
-    QUnicodeTables::Oriya, // Oriya
-    QUnicodeTables::Tamil, // Tamil
-    QUnicodeTables::Telugu, // Telugu
-    QUnicodeTables::Kannada, // Kannada
-    QUnicodeTables::Malayalam, // Malayalam
-    QUnicodeTables::Sinhala, // Sinhala
-    QUnicodeTables::Thai, // Thai
-    QUnicodeTables::Lao, // Lao
-    QUnicodeTables::Tibetan, // Tibetan
-    QUnicodeTables::Myanmar, // Myanmar
-    QUnicodeTables::Georgian, // Georgian
-    QUnicodeTables::Khmer, // Khmer
-    QUnicodeTables::Common, // SimplifiedChinese
-    QUnicodeTables::Common, // TraditionalChinese
-    QUnicodeTables::Common, // Japanese
-    QUnicodeTables::Hangul, // Korean
-    QUnicodeTables::Common, // Vietnamese
-    QUnicodeTables::Common, // Yi
-    QUnicodeTables::Common, // Tagalog
-    QUnicodeTables::Common, // Hanunoo
-    QUnicodeTables::Common, // Buhid
-    QUnicodeTables::Common, // Tagbanwa
-    QUnicodeTables::Common, // Limbu
-    QUnicodeTables::Common, // TaiLe
-    QUnicodeTables::Common, // Braille
-    QUnicodeTables::Common, // Symbol
-    QUnicodeTables::Ogham,  // Ogham
-    QUnicodeTables::Runic, // Runic
-    QUnicodeTables::Nko // Nko
+    QChar::Script_Common, // Any
+    QChar::Script_Latin, // Latin
+    QChar::Script_Greek, // Greek
+    QChar::Script_Cyrillic, // Cyrillic
+    QChar::Script_Armenian, // Armenian
+    QChar::Script_Hebrew, // Hebrew
+    QChar::Script_Arabic, // Arabic
+    QChar::Script_Syriac, // Syriac
+    QChar::Script_Thaana, // Thaana
+    QChar::Script_Devanagari, // Devanagari
+    QChar::Script_Bengali, // Bengali
+    QChar::Script_Gurmukhi, // Gurmukhi
+    QChar::Script_Gujarati, // Gujarati
+    QChar::Script_Oriya, // Oriya
+    QChar::Script_Tamil, // Tamil
+    QChar::Script_Telugu, // Telugu
+    QChar::Script_Kannada, // Kannada
+    QChar::Script_Malayalam, // Malayalam
+    QChar::Script_Sinhala, // Sinhala
+    QChar::Script_Thai, // Thai
+    QChar::Script_Lao, // Lao
+    QChar::Script_Tibetan, // Tibetan
+    QChar::Script_Myanmar, // Myanmar
+    QChar::Script_Georgian, // Georgian
+    QChar::Script_Khmer, // Khmer
+    QChar::Script_Han, // SimplifiedChinese
+    QChar::Script_Han, // TraditionalChinese
+    QChar::Script_Han, // Japanese
+    QChar::Script_Hangul, // Korean
+    QChar::Script_Latin, // Vietnamese
+    QChar::Script_Common, // Symbol
+    QChar::Script_Ogham,  // Ogham
+    QChar::Script_Runic, // Runic
+    QChar::Script_Nko // Nko
 };
 
+Q_STATIC_ASSERT(sizeof(scriptForWritingSystem) / sizeof(scriptForWritingSystem[0]) == QFontDatabase::WritingSystemsCount);
+
 int qt_script_for_writing_system(QFontDatabase::WritingSystem writingSystem)
 {
     return scriptForWritingSystem[writingSystem];
@@ -708,9 +564,9 @@ struct QtFontDesc
     int familyIndex;
 };
 
-static void match(int script, const QFontDef &request,
-                  const QString &family_name, const QString &foundry_name, int force_encoding_id,
-                  QtFontDesc *desc, const QList<int> &blacklistedFamilies = QList<int>());
+static int match(int script, const QFontDef &request,
+                 const QString &family_name, const QString &foundry_name, int force_encoding_id,
+                 QtFontDesc *desc, const QList<int> &blacklisted);
 
 static void initFontDef(const QtFontDesc &desc, const QFontDef &request, QFontDef *fontDef, bool multi)
 {
@@ -749,9 +605,8 @@ static void getEngineData(const QFontPrivate *d, const QFontDef &def)
         // create a new one
         d->engineData = new QFontEngineData;
         QFontCache::instance()->insertEngineData(def, d->engineData);
-    } else {
-        d->engineData->ref.ref();
     }
+    d->engineData->ref.ref();
 }
 
 static QStringList familyList(const QFontDef &req)
@@ -954,9 +809,6 @@ unsigned int bestFoundry(int script, unsigned int score, int styleStrategy,
             EncodingMismatch    = 0x0002
         };
         if (pitch != '*') {
-#if !defined(QWS) && defined(Q_OS_MAC)
-            qt_mac_get_fixed_pitch(const_cast<QtFontFamily*>(family));
-#endif
             if ((pitch == 'm' && !family->fixedPitch)
                 || (pitch == 'p' && family->fixedPitch))
                 this_score += PitchMismatch;
@@ -988,19 +840,7 @@ static bool matchFamilyName(const QString &familyName, QtFontFamily *f)
 {
     if (familyName.isEmpty())
         return true;
-
-    if (f->name.compare(familyName, Qt::CaseInsensitive) == 0)
-        return true;
-
-    QStringList::const_iterator it = f->aliases.constBegin();
-    while (it != f->aliases.constEnd()) {
-        if ((*it).compare(familyName, Qt::CaseInsensitive) == 0)
-            return true;
-
-        ++it;
-    }
-
-    return false;
+    return f->matchesFamilyName(familyName);
 }
 
 /*!
@@ -1008,11 +848,12 @@ static bool matchFamilyName(const QString &familyName, QtFontFamily *f)
 
     Tries to find the best match for a given request and family/foundry
 */
-static void match(int script, const QFontDef &request,
-                  const QString &family_name, const QString &foundry_name, int force_encoding_id,
-                  QtFontDesc *desc, const QList<int> &blacklistedFamilies)
+static int match(int script, const QFontDef &request,
+                 const QString &family_name, const QString &foundry_name, int force_encoding_id,
+                 QtFontDesc *desc, const QList<int> &blacklistedFamilies)
 {
     Q_UNUSED(force_encoding_id);
+    int result = -1;
 
     QtFontStyle::Key styleKey;
     styleKey.style = request.style;
@@ -1043,6 +884,8 @@ static void match(int script, const QFontDef &request,
 
     load(family_name, script);
 
+    const size_t writingSystem = std::find(scriptForWritingSystem, scriptForWritingSystem + QFontDatabase::WritingSystemsCount, script) - scriptForWritingSystem;
+
     QFontDatabasePrivate *db = privateDb();
     for (int x = 0; x < db->count; ++x) {
         if (blacklistedFamilies.contains(x))
@@ -1057,19 +900,9 @@ static void match(int script, const QFontDef &request,
         if (family_name.isEmpty())
             load(test.family->name, script);
 
-        uint score_adjust = 0;
-
-        bool supported = (script == QUnicodeTables::Common);
-        for (int ws = 1; !supported && ws < QFontDatabase::WritingSystemsCount; ++ws) {
-            if (scriptForWritingSystem[ws] != script)
-                continue;
-            if (test.family->writingSystems[ws] & QtFontFamily::Supported)
-                supported = true;
-        }
-        if (!supported) {
-            // family not supported in the script we want
+        // Check if family is supported in the script we want
+        if (script != QChar::Script_Common && !(test.family->writingSystems[writingSystem] & QtFontFamily::Supported))
             continue;
-        }
 
         // as we know the script is supported, we can be sure
         // to find a matching font here.
@@ -1084,15 +917,16 @@ static void match(int script, const QFontDef &request,
                                    QString(), styleKey, request.pixelSize,
                                    pitch, &test, force_encoding_id);
         }
-        newscore += score_adjust;
 
         if (newscore < score) {
+            result = x;
             score = newscore;
             *desc = test;
         }
         if (newscore < 10) // xlfd instead of FT... just accept it
             break;
     }
+    return result;
 }
 
 static QString styleStringHelper(int weight, QFont::Style style)
@@ -1245,6 +1079,17 @@ QFontDatabase::QFontDatabase()
     \omitvalue WritingSystemsCount
 */
 
+/*!
+    \enum QFontDatabase::SystemFont
+
+    \value GeneralFont              The default system font.
+    \value FixedFont                The fixed font that the system recommends.
+    \value TitleFont                The system standard font for titles.
+    \value SmallestReadableFont     The smallest readable system font.
+
+    \since 5.2
+*/
+
 /*!
     Returns a sorted list of the available writing systems. This is
     list generated from information about all installed fonts on the
@@ -1271,7 +1116,7 @@ QList<QFontDatabase::WritingSystem> QFontDatabase::writingSystems() const
                 list.append(writingSystem);
         }
     }
-    qSort(list);
+    std::sort(list.begin(), list.end());
     return list;
 }
 
@@ -1389,8 +1234,8 @@ QStringList QFontDatabase::styles(const QString &family) const
 }
 
 /*!
-    Returns true if the font that has family \a family and style \a
-    style is fixed pitch; otherwise returns false.
+    Returns \c true if the font that has family \a family and style \a
+    style is fixed pitch; otherwise returns \c false.
 */
 
 bool QFontDatabase::isFixedPitch(const QString &family,
@@ -1406,15 +1251,12 @@ bool QFontDatabase::isFixedPitch(const QString &family,
     QT_PREPEND_NAMESPACE(load)(familyName);
 
     QtFontFamily *f = d->family(familyName);
-#if !defined(QWS) && defined(Q_OS_MAC)
-    qt_mac_get_fixed_pitch(f);
-#endif
     return (f && f->fixedPitch);
 }
 
 /*!
-    Returns true if the font that has family \a family and style \a
-    style is a scalable bitmap font; otherwise returns false. Scaling
+    Returns \c true if the font that has family \a family and style \a
+    style is a scalable bitmap font; otherwise returns \c false. Scaling
     a bitmap font usually produces an unattractive hardly readable
     result, because the pixels of the font are scaled. If you need to
     scale a bitmap font it is better to scale it to one of the fixed
@@ -1457,9 +1299,9 @@ bool QFontDatabase::isBitmapScalable(const QString &family,
 
 
 /*!
-    Returns true if the font that has family \a family and style \a
-    style is smoothly scalable; otherwise returns false. If this
-    function returns true, it's safe to scale this font to any size,
+    Returns \c true if the font that has family \a family and style \a
+    style is smoothly scalable; otherwise returns \c false. If this
+    function returns \c true, it's safe to scale this font to any size,
     and the result will always look attractive.
 
     \sa isScalable(), isBitmapScalable()
@@ -1496,8 +1338,8 @@ bool QFontDatabase::isSmoothlyScalable(const QString &family, const QString &sty
 }
 
 /*!
-    Returns true if the font that has family \a family and style \a
-    style is scalable; otherwise returns false.
+    Returns \c true if the font that has family \a family and style \a
+    style is scalable; otherwise returns \c false.
 
     \sa isBitmapScalable(), isSmoothlyScalable()
 */
@@ -1566,7 +1408,7 @@ QList<int> QFontDatabase::pointSizes(const QString &family,
     if (smoothScalable)
         return standardSizes();
 
-    qSort(sizes);
+    std::sort(sizes.begin(), sizes.end());
     return sizes;
 }
 
@@ -1669,7 +1511,7 @@ QList<int> QFontDatabase::smoothSizes(const QString &family,
     if (smoothScalable)
         return QFontDatabase::standardSizes();
 
-    qSort(sizes);
+    std::sort(sizes.begin(), sizes.end());
     return sizes;
 }
 
@@ -1686,8 +1528,8 @@ QList<int> QFontDatabase::standardSizes()
 
 
 /*!
-    Returns true if the font that has family \a family and style \a
-    style is italic; otherwise returns false.
+    Returns \c true if the font that has family \a family and style \a
+    style is italic; otherwise returns \c false.
 
     \sa weight(), bold()
 */
@@ -1719,8 +1561,8 @@ bool QFontDatabase::italic(const QString &family, const QString &style) const
 
 
 /*!
-    Returns true if the font that has family \a family and style \a
-    style is bold; otherwise returns false.
+    Returns \c true if the font that has family \a family and style \a
+    style is bold; otherwise returns \c false.
 
     \sa italic(), weight()
 */
@@ -2256,13 +2098,50 @@ QStringList QFontDatabase::applicationFontFamilies(int id)
     return privateDb()->applicationFonts.value(id).families;
 }
 
+/*!
+    \since 5.2
+
+    Returns the most adequate font for a given \a type case for proper integration
+    with the system's look and feel.
+
+    \sa QGuiApplication::font()
+*/
+
+QFont QFontDatabase::systemFont(QFontDatabase::SystemFont type)
+{
+    const QFont *font = 0;
+    if (const QPlatformTheme *theme = QGuiApplicationPrivate::platformTheme()) {
+        switch (type) {
+            case GeneralFont:
+                font = theme->font(QPlatformTheme::SystemFont);
+                break;
+            case FixedFont:
+                font = theme->font(QPlatformTheme::FixedFont);
+                break;
+            case TitleFont:
+                font = theme->font(QPlatformTheme::TitleBarFont);
+                break;
+            case SmallestReadableFont:
+                font = theme->font(QPlatformTheme::MiniFont);
+                break;
+        }
+    }
+
+    if (font)
+        return *font;
+    else if (QPlatformIntegration *integration = QGuiApplicationPrivate::platformIntegration())
+        return integration->fontDatabase()->defaultFont();
+    else
+        return QFont();
+}
+
 /*!
     \fn bool QFontDatabase::removeApplicationFont(int id)
     \since 4.2
 
     Removes the previously loaded application font identified by \a
-    id. Returns true if unloading of the font succeeded; otherwise
-    returns false.
+    id. Returns \c true if unloading of the font succeeded; otherwise
+    returns \c false.
 
     \sa removeAllApplicationFonts(), addApplicationFont(),
         addApplicationFontFromData()
@@ -2275,8 +2154,8 @@ QStringList QFontDatabase::applicationFontFamilies(int id)
     Removes all application-local fonts previously added using addApplicationFont()
     and addApplicationFontFromData().
 
-    Returns true if unloading of the fonts succeeded; otherwise
-    returns false.
+    Returns \c true if unloading of the fonts succeeded; otherwise
+    returns \c false.
 
     \sa removeApplicationFont(), addApplicationFont(), addApplicationFontFromData()
 */
@@ -2284,12 +2163,15 @@ QStringList QFontDatabase::applicationFontFamilies(int id)
 /*!
     \fn bool QFontDatabase::supportsThreadedFontRendering()
     \since 4.4
+    \deprecated
 
-    Returns true if font rendering is supported outside the GUI
+    Returns \c true if font rendering is supported outside the GUI
     thread, false otherwise. In other words, a return value of false
     means that all QPainter::drawText() calls outside the GUI thread
     will not produce readable output.
 
+    As of 5.0, always returns \c true.
+
     \sa {Thread-Support in Qt Modules#Painting In Threads}{Painting In Threads}
 */
 
