@@ -48,8 +48,15 @@
 #include "qvarlengtharray.h"
 #include <qmath.h>
 #include <qendian.h>
+
+#ifdef QT_ENABLE_HARFBUZZ_NG
+#  include "qharfbuzzng_p.h"
+#  include <harfbuzz/hb-ot.h>
+#endif
 #include <private/qharfbuzz_p.h>
 
+#include <algorithm>
+
 QT_BEGIN_NAMESPACE
 
 static inline bool qtransform_equals_no_translate(const QTransform &a, const QTransform &b)
@@ -71,11 +78,20 @@ static inline bool qtransform_equals_no_translate(const QTransform &a, const QTr
 
 // Harfbuzz helper functions
 
+#ifdef QT_ENABLE_HARFBUZZ_NG
+bool useHarfbuzzNG = qgetenv("QT_HARFBUZZ") != "old";
+#endif
+
+Q_STATIC_ASSERT(sizeof(HB_Glyph) == sizeof(glyph_t));
+Q_STATIC_ASSERT(sizeof(HB_Fixed) == sizeof(QFixed));
+
 static HB_Bool hb_stringToGlyphs(HB_Font font, const HB_UChar16 *string, hb_uint32 length, HB_Glyph *glyphs, hb_uint32 *numGlyphs, HB_Bool rightToLeft)
 {
     QFontEngine *fe = (QFontEngine *)font->userData;
 
-    QVarLengthGlyphLayoutArray qglyphs(*numGlyphs);
+    QGlyphLayout qglyphs;
+    qglyphs.numGlyphs = *numGlyphs;
+    qglyphs.glyphs = glyphs;
 
     QFontEngine::ShaperFlags shaperFlags(QFontEngine::GlyphIndicesOnly);
     if (rightToLeft)
@@ -84,28 +100,23 @@ static HB_Bool hb_stringToGlyphs(HB_Font font, const HB_UChar16 *string, hb_uint
     int nGlyphs = *numGlyphs;
     bool result = fe->stringToCMap(reinterpret_cast<const QChar *>(string), length, &qglyphs, &nGlyphs, shaperFlags);
     *numGlyphs = nGlyphs;
-    if (!result)
-        return false;
 
-    for (hb_uint32 i = 0; i < *numGlyphs; ++i)
-        glyphs[i] = qglyphs.glyphs[i];
-
-    return true;
+    return result;
 }
 
 static void hb_getAdvances(HB_Font font, const HB_Glyph *glyphs, hb_uint32 numGlyphs, HB_Fixed *advances, int flags)
 {
     QFontEngine *fe = (QFontEngine *)font->userData;
 
-    QVarLengthGlyphLayoutArray qglyphs(numGlyphs);
+    QVarLengthArray<QFixed> advances_y(numGlyphs);
 
-    for (hb_uint32 i = 0; i < numGlyphs; ++i)
-        qglyphs.glyphs[i] = glyphs[i];
+    QGlyphLayout qglyphs;
+    qglyphs.numGlyphs = numGlyphs;
+    qglyphs.glyphs = const_cast<glyph_t *>(glyphs);
+    qglyphs.advances_x = reinterpret_cast<QFixed *>(advances);
+    qglyphs.advances_y = advances_y.data(); // not used
 
     fe->recalcAdvances(&qglyphs, (flags & HB_ShaperFlag_UseDesignMetrics) ? QFontEngine::DesignMetrics : QFontEngine::ShaperFlags(0));
-
-    for (hb_uint32 i = 0; i < numGlyphs; ++i)
-        advances[i] = qglyphs.advances_x[i].value();
 }
 
 static HB_Bool hb_canRender(HB_Font font, const HB_UChar16 *string, hb_uint32 length)
@@ -135,7 +146,7 @@ static HB_Fixed hb_getFontMetric(HB_Font font, HB_FontMetric metric)
     return 0;
 }
 
-HB_Error QFontEngine::getPointInOutline(HB_Glyph glyph, int flags, hb_uint32 point, HB_Fixed *xpos, HB_Fixed *ypos, hb_uint32 *nPoints)
+int QFontEngine::getPointInOutline(glyph_t glyph, int flags, quint32 point, QFixed *xpos, QFixed *ypos, quint32 *nPoints)
 {
     Q_UNUSED(glyph)
     Q_UNUSED(flags)
@@ -143,13 +154,13 @@ HB_Error QFontEngine::getPointInOutline(HB_Glyph glyph, int flags, hb_uint32 poi
     Q_UNUSED(xpos)
     Q_UNUSED(ypos)
     Q_UNUSED(nPoints)
-    return HB_Err_Not_Covered;
+    return Err_Not_Covered;
 }
 
 static HB_Error hb_getPointInOutline(HB_Font font, HB_Glyph glyph, int flags, hb_uint32 point, HB_Fixed *xpos, HB_Fixed *ypos, hb_uint32 *nPoints)
 {
     QFontEngine *fe = (QFontEngine *)font->userData;
-    return fe->getPointInOutline(glyph, flags, point, xpos, ypos, nPoints);
+    return (HB_Error)fe->getPointInOutline(glyph, flags, point, (QFixed *)xpos, (QFixed *)ypos, (quint32 *)nPoints);
 }
 
 static const HB_FontClass hb_fontClass = {
@@ -160,32 +171,86 @@ static const HB_FontClass hb_fontClass = {
 static HB_Error hb_getSFntTable(void *font, HB_Tag tableTag, HB_Byte *buffer, HB_UInt *length)
 {
     QFontEngine *fe = (QFontEngine *)font;
-    if (!fe->getSfntTableData(tableTag, buffer, length))
+    Q_ASSERT(fe->faceData.get_font_table);
+    if (!fe->faceData.get_font_table(fe->faceData.user_data, tableTag, buffer, length))
         return HB_Err_Invalid_Argument;
     return HB_Err_Ok;
 }
 
+static void hb_freeFace(void *face)
+{
+    qHBFreeFace((HB_Face)face);
+}
+
+
+static bool qt_get_font_table_default(void *user_data, uint tag, uchar *buffer, uint *length)
+{
+    QFontEngine *fe = (QFontEngine *)user_data;
+    return fe->getSfntTableData(tag, buffer, length);
+}
+
+
+#ifdef QT_BUILD_INTERNAL
+// for testing purpose only, not thread-safe!
+static QList<QFontEngine *> *enginesCollector = 0;
+
+Q_AUTOTEST_EXPORT void QFontEngine_startCollectingEngines()
+{
+    delete enginesCollector;
+    enginesCollector = new QList<QFontEngine *>();
+}
+
+Q_AUTOTEST_EXPORT QList<QFontEngine *> QFontEngine_stopCollectingEngines()
+{
+    Q_ASSERT(enginesCollector);
+    QList<QFontEngine *> ret = *enginesCollector;
+    delete enginesCollector;
+    enginesCollector = 0;
+    return ret;
+}
+#endif // QT_BUILD_INTERNAL
+
+
 // QFontEngine
 
 QFontEngine::QFontEngine()
-    : QObject(), ref(0)
+    : ref(0),
+      font_(0), font_destroy_func(0),
+      face_(0), face_destroy_func(0)
 {
-    cache_count = 0;
+    faceData.user_data = this;
+    faceData.get_font_table = qt_get_font_table_default;
+
+    cache_cost = 0;
     fsType = 0;
     symbol = false;
-    memset(&hbFont, 0, sizeof(hbFont));
-    hbFont.klass = &hb_fontClass;
-    hbFont.userData = this;
 
-    hbFace = 0;
     glyphFormat = -1;
     m_subPixelPositionCount = 0;
+
+#ifdef QT_BUILD_INTERNAL
+    if (enginesCollector)
+        enginesCollector->append(this);
+#endif
 }
 
 QFontEngine::~QFontEngine()
 {
     m_glyphCaches.clear();
-    qHBFreeFace(hbFace);
+
+    if (font_ && font_destroy_func) {
+        font_destroy_func(font_);
+        font_ = 0;
+    }
+    if (face_ && face_destroy_func) {
+        face_destroy_func(face_);
+        face_ = 0;
+    }
+
+#ifdef QT_BUILD_INTERNAL
+    if (enginesCollector)
+        enginesCollector->removeOne(this);
+#endif
 }
 
 QFixed QFontEngine::lineThickness() const
@@ -206,34 +271,85 @@ QFixed QFontEngine::underlinePosition() const
     return ((lineThickness() * 2) + 3) / 6;
 }
 
-HB_Font QFontEngine::harfbuzzFont() const
+void *QFontEngine::harfbuzzFont() const
 {
-    if (!hbFont.x_ppem) {
-        QFixed emSquare = emSquareSize();
-        hbFont.x_ppem = fontDef.pixelSize;
-        hbFont.y_ppem = fontDef.pixelSize * fontDef.stretch / 100;
-        hbFont.x_scale = (QFixed(hbFont.x_ppem * (1 << 16)) / emSquare).value();
-        hbFont.y_scale = (QFixed(hbFont.y_ppem * (1 << 16)) / emSquare).value();
+#ifdef QT_ENABLE_HARFBUZZ_NG
+    if (useHarfbuzzNG)
+        return hb_qt_font_get_for_engine(const_cast<QFontEngine *>(this));
+#endif
+    if (!font_) {
+        HB_FontRec *hbFont = (HB_FontRec *) malloc(sizeof(HB_FontRec));
+        Q_CHECK_PTR(hbFont);
+        hbFont->klass = &hb_fontClass;
+        hbFont->userData = const_cast<QFontEngine *>(this);
+
+        qint64 emSquare = emSquareSize().truncate();
+        Q_ASSERT(emSquare == emSquareSize().toInt()); // ensure no truncation
+        if (emSquare == 0)
+            emSquare = 1000; // a fallback value suitable for Type1 fonts
+        hbFont->y_ppem = fontDef.pixelSize;
+        hbFont->x_ppem = fontDef.pixelSize * fontDef.stretch / 100;
+        // same as QFixed(x)/QFixed(emSquare) but without int32 overflow for x
+        hbFont->x_scale = (((qint64)hbFont->x_ppem << 6) * 0x10000L + (emSquare >> 1)) / emSquare;
+        hbFont->y_scale = (((qint64)hbFont->y_ppem << 6) * 0x10000L + (emSquare >> 1)) / emSquare;
+
+        font_ = (void *)hbFont;
+        font_destroy_func = free;
     }
-    return &hbFont;
+    return font_;
 }
 
-HB_Face QFontEngine::harfbuzzFace() const
+void *QFontEngine::harfbuzzFace() const
 {
-    if (!hbFace) {
-        hbFace = qHBNewFace(const_cast<QFontEngine *>(this), hb_getSFntTable);
+#ifdef QT_ENABLE_HARFBUZZ_NG
+    if (useHarfbuzzNG)
+        return hb_qt_face_get_for_engine(const_cast<QFontEngine *>(this));
+#endif
+    if (!face_) {
+        HB_Face hbFace = qHBNewFace(const_cast<QFontEngine *>(this), hb_getSFntTable);
         Q_CHECK_PTR(hbFace);
+        if (hbFace->font_for_init != 0)
+            hbFace = qHBLoadFace(hbFace);
+        hbFace->isSymbolFont = symbol;
+
+        face_ = (void *)hbFace;
+        face_destroy_func = hb_freeFace;
     }
-    return hbFace;
+    return face_;
 }
 
-HB_Face QFontEngine::initializedHarfbuzzFace() const
+bool QFontEngine::supportsScript(QChar::Script script) const
 {
-    HB_Face face = harfbuzzFace();
-    if (face != 0 && face->font_for_init != 0)
-        face = qHBLoadFace(face);
+    // ### TODO: This only works for scripts that require OpenType. More generally
+    // for scripts that do not require OpenType we should just look at the list of
+    // supported writing systems in the font's OS/2 table.
+    if (!((script >= QChar::Script_Syriac && script <= QChar::Script_Sinhala)
+          || script == QChar::Script_Khmer || script == QChar::Script_Nko)) {
+        return true;
+    }
 
-    return face;
+#ifdef QT_ENABLE_HARFBUZZ_NG
+    if (useHarfbuzzNG) {
+        bool ret = false;
+        if (hb_face_t *face = hb_qt_face_get_for_engine(const_cast<QFontEngine *>(this))) {
+            hb_tag_t script_tag_1, script_tag_2;
+            hb_ot_tags_from_script(hb_qt_script_to_script(script), &script_tag_1, &script_tag_2);
+
+            unsigned int script_index = -1;
+            ret = hb_ot_layout_table_find_script(face, HB_OT_TAG_GSUB, script_tag_1, &script_index);
+            if (!ret) {
+                ret = hb_ot_layout_table_find_script(face, HB_OT_TAG_GSUB, script_tag_2, &script_index);
+                if (!ret && script_tag_2 != HB_OT_TAG_DEFAULT_SCRIPT)
+                    ret = hb_ot_layout_table_find_script(face, HB_OT_TAG_GSUB, HB_OT_TAG_DEFAULT_SCRIPT, &script_index);
+            }
+
+            hb_face_destroy(face);
+        }
+        return ret;
+    }
+#endif
+    HB_Face hbFace = (HB_Face)harfbuzzFace();
+    return hbFace->supported_scripts[script_to_hbscript(script)];
 }
 
 glyph_metrics_t QFontEngine::boundingBox(glyph_t glyph, const QTransform &matrix)
@@ -268,9 +384,9 @@ QFixed QFontEngine::averageCharWidth() const
     return bb.xoff;
 }
 
-bool QFontEngine::supportsTransformations(const QTransform &transform) const
+bool QFontEngine::supportsTransformation(const QTransform &transform) const
 {
-    return (transform.type() >= QTransform::TxProject);
+    return transform.type() < QTransform::TxProject;
 }
 
 void QFontEngine::getGlyphPositions(const QGlyphLayout &glyphs, const QTransform &matrix, QTextItem::RenderFlags flags,
@@ -649,6 +765,13 @@ QImage QFontEngine::alphaRGBMapForGlyph(glyph_t glyph, QFixed /*subPixelPosition
     return rgbMask;
 }
 
+QImage QFontEngine::bitmapForGlyph(glyph_t, QFixed subPixelPosition, const QTransform&)
+{
+    Q_UNUSED(subPixelPosition);
+
+    return QImage();
+}
+
 QFixed QFontEngine::subPixelPositionForX(QFixed x) const
 {
     if (m_subPixelPositionCount <= 1 || !supportsSubPixelPositions())
@@ -779,6 +902,16 @@ QByteArray QFontEngine::getSfntTable(uint tag) const
     return table;
 }
 
+void QFontEngine::clearGlyphCache(const void *key)
+{
+    for (QLinkedList<GlyphCacheEntry>::iterator it = m_glyphCaches.begin(), end = m_glyphCaches.end(); it != end; ) {
+        if (it->context == key)
+            it = m_glyphCaches.erase(it);
+        else
+            ++it;
+    }
+}
+
 void QFontEngine::setGlyphCache(const void *key, QFontEngineGlyphCache *data)
 {
     Q_ASSERT(data);
@@ -904,7 +1037,7 @@ void QFontEngine::loadKerningPairs(QFixed scalingFactor)
         }
     }
 end:
-    qSort(kerning_pairs);
+    std::sort(kerning_pairs.begin(), kerning_pairs.end());
 //    for (int i = 0; i < kerning_pairs.count(); ++i)
 //        qDebug() << 'i' << i << "left_right" << hex << kerning_pairs.at(i).left_right;
 }
@@ -1194,7 +1327,7 @@ bool QFontEngineBox::stringToCMap(const QChar *, int len, QGlyphLayout *glyphs,
         return false;
     }
 
-    memset(glyphs->glyphs, 0, len * sizeof(HB_Glyph));
+    memset(glyphs->glyphs, 0, len * sizeof(glyph_t));
 
     *nglyphs = len;
     glyphs->numGlyphs = len;
@@ -1352,11 +1485,8 @@ QFontEngineMulti::~QFontEngineMulti()
 {
     for (int i = 0; i < engines.size(); ++i) {
         QFontEngine *fontEngine = engines.at(i);
-        if (fontEngine) {
-            fontEngine->ref.deref();
-            if (fontEngine->cache_count == 0 && fontEngine->ref.load() == 0)
-                delete fontEngine;
-        }
+        if (fontEngine && !fontEngine->ref.deref())
+            delete fontEngine;
     }
 }
 
@@ -1373,10 +1503,12 @@ bool QFontEngineMulti::stringToCMap(const QChar *str, int len,
         bool surrogate = (str[i].isHighSurrogate() && i < len-1 && str[i+1].isLowSurrogate());
         uint ucs4 = surrogate ? QChar::surrogateToUcs4(str[i], str[i+1]) : str[i].unicode();
         if (glyphs->glyphs[glyph_pos] == 0 && str[i].category() != QChar::Separator_Line) {
-            QGlyphLayoutInstance tmp;
-            if (!(flags & GlyphIndicesOnly))
-                tmp = glyphs->instance(glyph_pos);
-            for (int x=1; x < engines.size(); ++x) {
+            QFixedPoint tmpAdvance;
+            if (!(flags & GlyphIndicesOnly)) {
+                tmpAdvance.x = glyphs->advances_x[glyph_pos];
+                tmpAdvance.y = glyphs->advances_y[glyph_pos];
+            }
+            for (int x = 1, n = qMin(engines.size(), 256); x < n; ++x) {
                 if (engines.at(x) == 0 && !shouldLoadFontEngineForCharacter(x, ucs4))
                     continue;
 
@@ -1389,10 +1521,8 @@ bool QFontEngineMulti::stringToCMap(const QChar *str, int len,
                 if (engine->type() == Box)
                     continue;
 
-                if (!(flags & GlyphIndicesOnly)) {
+                if (!(flags & GlyphIndicesOnly))
                     glyphs->advances_x[glyph_pos] = glyphs->advances_y[glyph_pos] = 0;
-                    glyphs->offsets[glyph_pos] = QFixedPoint();
-                }
                 int num = 2;
                 QGlyphLayout offs = glyphs->mid(glyph_pos, num);
                 engine->stringToCMap(str + i, surrogate ? 2 : 1, &offs, &num, flags);
@@ -1405,8 +1535,10 @@ bool QFontEngineMulti::stringToCMap(const QChar *str, int len,
             }
 
             // ensure we use metrics from the 1st font when we use the fallback image.
-            if (!(flags & GlyphIndicesOnly) && !glyphs->glyphs[glyph_pos])
-                glyphs->setInstance(glyph_pos, tmp);
+            if (!(flags & GlyphIndicesOnly) && glyphs->glyphs[glyph_pos] == 0) {
+                glyphs->advances_x[glyph_pos] = tmpAdvance.x;
+                glyphs->advances_y[glyph_pos] = tmpAdvance.y;
+            }
         }
 
         if (surrogate)
@@ -1708,22 +1840,27 @@ bool QFontEngineMulti::canRender(const QChar *string, int len)
     if (engine(0)->canRender(string, len))
         return true;
 
-    QVarLengthGlyphLayoutArray glyphs(len);
     int nglyphs = len;
-    if (!stringToCMap(string, len, &glyphs, &nglyphs, GlyphIndicesOnly)) {
+
+    QVarLengthArray<glyph_t> glyphs(nglyphs);
+
+    QGlyphLayout g;
+    g.numGlyphs = nglyphs;
+    g.glyphs = glyphs.data();
+    if (!stringToCMap(string, len, &g, &nglyphs, GlyphIndicesOnly)) {
         glyphs.resize(nglyphs);
-        stringToCMap(string, len, &glyphs, &nglyphs, GlyphIndicesOnly);
+        g.numGlyphs = nglyphs;
+        g.glyphs = glyphs.data();
+        if (!stringToCMap(string, len, &g, &nglyphs, GlyphIndicesOnly))
+            Q_ASSERT_X(false, Q_FUNC_INFO, "stringToCMap shouldn't fail twice");
     }
 
-    bool allExist = true;
     for (int i = 0; i < nglyphs; i++) {
-        if (!glyphs.glyphs[i]) {
-            allExist = false;
-            break;
-        }
+        if (g.glyphs[i] == 0)
+            return false;
     }
 
-    return allExist;
+    return true;
 }
 
 /* Implement alphaMapForGlyph() which is called by Lighthouse/Windows code.
@@ -1764,4 +1901,13 @@ QImage QFontEngineMulti::alphaRGBMapForGlyph(glyph_t glyph, QFixed subPixelPosit
     return engine(which)->alphaRGBMapForGlyph(stripped(glyph), subPixelPosition, t);
 }
 
+QTestFontEngine::QTestFontEngine(int size)
+    : QFontEngineBox(size)
+{}
+
+QFontEngine::Type QTestFontEngine::type() const
+{
+    return TestFontEngine;
+}
+
 QT_END_NAMESPACE
