@@ -112,28 +112,25 @@ QT_BEGIN_NAMESPACE
 #endif
 
 #define FLOOR(x)    ((x) & -64)
-#define CEIL(x)	    (((x)+63) & -64)
+#define CEIL(x)     (((x)+63) & -64)
 #define TRUNC(x)    ((x) >> 6)
 #define ROUND(x)    (((x)+32) & -64)
 
-static HB_Error hb_getSFntTable(void *font, HB_Tag tableTag, HB_Byte *buffer, HB_UInt *length)
+static bool ft_getSfntTable(void *user_data, uint tag, uchar *buffer, uint *length)
 {
-#if (FREETYPE_MAJOR*10000 + FREETYPE_MINOR*100 + FREETYPE_PATCH) > 20103
-    FT_Face face = (FT_Face)font;
-    FT_ULong ftlen = *length;
-    FT_Error error = 0;
+    FT_Face face = (FT_Face)user_data;
 
-    if ( !FT_IS_SFNT(face) )
-        return HB_Err_Invalid_Argument;
+    bool result = false;
+    if (FT_IS_SFNT(face)) {
+        FT_ULong len = *length;
+        result = FT_Load_Sfnt_Table(face, tag, 0, buffer, &len) == FT_Err_Ok;
+        *length = len;
+    }
 
-    error = FT_Load_Sfnt_Table(face, tableTag, 0, buffer, &ftlen);
-    *length = ftlen;
-    return (HB_Error)error;
-#else
-    return HB_Err_Invalid_Argument;
-#endif
+    return result;
 }
 
+
 // -------------------------- Freetype support ------------------------------
 
 class QtFreetypeData
@@ -142,11 +139,21 @@ public:
     QtFreetypeData()
         : library(0)
     { }
+    ~QtFreetypeData();
 
     FT_Library library;
     QHash<QFontEngine::FaceId, QFreetypeFace *> faces;
 };
 
+QtFreetypeData::~QtFreetypeData()
+{
+    for (QHash<QFontEngine::FaceId, QFreetypeFace *>::ConstIterator iter = faces.begin(); iter != faces.end(); ++iter)
+        iter.value()->cleanup();
+    faces.clear();
+    FT_Done_FreeType(library);
+    library = 0;
+}
+
 #ifdef QT_NO_THREAD
 Q_GLOBAL_STATIC(QtFreetypeData, theFreetypeData)
 
@@ -183,25 +190,25 @@ int QFreetypeFace::fsType() const
     return fsType;
 }
 
-HB_Error QFreetypeFace::getPointInOutline(HB_Glyph glyph, int flags, hb_uint32 point, HB_Fixed *xpos, HB_Fixed *ypos, hb_uint32 *nPoints)
+int QFreetypeFace::getPointInOutline(glyph_t glyph, int flags, quint32 point, QFixed *xpos, QFixed *ypos, quint32 *nPoints)
 {
-    if (HB_Error error = (HB_Error)FT_Load_Glyph(face, glyph, flags))
+    if (int error = FT_Load_Glyph(face, glyph, flags))
         return error;
 
     if (face->glyph->format != FT_GLYPH_FORMAT_OUTLINE)
-        return HB_Err_Invalid_SubTable;
+        return Err_Invalid_SubTable;
 
     *nPoints = face->glyph->outline.n_points;
     if (!(*nPoints))
-        return HB_Err_Ok;
+        return Err_Ok;
 
     if (point > *nPoints)
-        return HB_Err_Invalid_SubTable;
+        return Err_Invalid_SubTable;
 
-    *xpos = face->glyph->outline.points[point].x;
-    *ypos = face->glyph->outline.points[point].y;
+    *xpos = QFixed::fromFixed(face->glyph->outline.points[point].x);
+    *ypos = QFixed::fromFixed(face->glyph->outline.points[point].y);
 
-    return HB_Err_Ok;
+    return Err_Ok;
 }
 
 extern QByteArray qt_fontdata_from_index(int);
@@ -230,7 +237,7 @@ QFreetypeFace *QFreetypeFace::getFace(const QFontEngine::FaceId &face_id,
         QScopedPointer<QFreetypeFace> newFreetype(new QFreetypeFace);
         FT_Face face;
         if (!face_id.filename.isEmpty()) {
-            QString fileName = QString::fromUtf8(face_id.filename);
+            QString fileName = QFile::decodeName(face_id.filename);
             if (face_id.filename.startsWith(":qmemoryfonts/")) {
                 // from qfontdatabase.cpp
                 QByteArray idx = face_id.filename;
@@ -258,8 +265,9 @@ QFreetypeFace *QFreetypeFace::getFace(const QFontEngine::FaceId &face_id,
         }
         newFreetype->face = face;
 
-        newFreetype->hbFace = qHBNewFace(face, hb_getSFntTable);
-        Q_CHECK_PTR(newFreetype->hbFace);
+        newFreetype->hbFace = 0;
+        newFreetype->hbFace_destroy_func = 0;
+
         newFreetype->ref.store(1);
         newFreetype->xsize = 0;
         newFreetype->ysize = 0;
@@ -309,20 +317,35 @@ QFreetypeFace *QFreetypeFace::getFace(const QFontEngine::FaceId &face_id,
     return freetype;
 }
 
+void QFreetypeFace::cleanup()
+{
+    if (hbFace && hbFace_destroy_func) {
+        hbFace_destroy_func(hbFace);
+        hbFace = 0;
+    }
+    FT_Done_Face(face);
+    face = 0;
+}
+
 void QFreetypeFace::release(const QFontEngine::FaceId &face_id)
 {
-    QtFreetypeData *freetypeData = qt_getFreetypeData();
     if (!ref.deref()) {
-        qHBFreeFace(hbFace);
-        FT_Done_Face(face);
-        if(freetypeData->faces.contains(face_id))
-            freetypeData->faces.take(face_id);
+        if (face) {
+            QtFreetypeData *freetypeData = qt_getFreetypeData();
+
+            cleanup();
+
+            if (freetypeData->faces.contains(face_id))
+                freetypeData->faces.take(face_id);
+
+            if (freetypeData->faces.isEmpty()) {
+                FT_Done_FreeType(freetypeData->library);
+                freetypeData->library = 0;
+            }
+        }
+
         delete this;
     }
-    if (freetypeData->faces.isEmpty()) {
-        FT_Done_FreeType(freetypeData->library);
-        freetypeData->library = 0;
-    }
 }
 
 
@@ -400,15 +423,7 @@ QFontEngine::Properties QFreetypeFace::properties() const
 
 bool QFreetypeFace::getSfntTable(uint tag, uchar *buffer, uint *length) const
 {
-    bool result = false;
-#if (FREETYPE_MAJOR*10000 + FREETYPE_MINOR*100 + FREETYPE_PATCH) > 20103
-    if (FT_IS_SFNT(face)) {
-        FT_ULong len = *length;
-        result = FT_Load_Sfnt_Table(face, tag, 0, buffer, &len) == FT_Err_Ok;
-        *length = len;
-    }
-#endif
-    return result;
+    return ft_getSfntTable(face, tag, buffer, length);
 }
 
 /* Some fonts (such as MingLiu rely on hinting to scale different
@@ -654,7 +669,6 @@ QFontEngineFT::~QFontEngineFT()
 {
     if (freetype)
         freetype->release(face_id);
-    hbFace = 0; // we share the face in QFreeTypeFace, don't let ~QFontEngine delete it
 }
 
 bool QFontEngineFT::init(FaceId faceId, bool antialias, GlyphFormat format,
@@ -690,8 +704,6 @@ bool QFontEngineFT::init(FaceId faceId, bool antialias, GlyphFormat format,
     if (FT_Get_PS_Font_Info(freetype->face, &psrec) == FT_Err_Ok) {
         symbol = bool(fontDef.family.contains(QLatin1String("symbol"), Qt::CaseInsensitive));
     }
-    // #####
-    freetype->hbFace->isSymbolFont = symbol;
 
     lbearing = rbearing = SHRT_MIN;
     freetype->computeSize(fontDef, &xsize, &ysize, &defaultGlyphSet.outline_drawing);
@@ -717,7 +729,6 @@ bool QFontEngineFT::init(FaceId faceId, bool antialias, GlyphFormat format,
         line_thickness =  QFixed::fromFixed(FT_MulFix(face->underline_thickness, face->size->metrics.y_scale));
         underline_position = QFixed::fromFixed(-FT_MulFix(face->underline_position, face->size->metrics.y_scale));
     } else {
-        // copied from QFontEngineQPF
         // ad hoc algorithm
         int score = fontDef.weight * fontDef.pixelSize;
         line_thickness = score / 700;
@@ -729,13 +740,6 @@ bool QFontEngineFT::init(FaceId faceId, bool antialias, GlyphFormat format,
     if (line_thickness < 1)
         line_thickness = 1;
 
-    hbFont.x_ppem  = face->size->metrics.x_ppem;
-    hbFont.y_ppem  = face->size->metrics.y_ppem;
-    hbFont.x_scale = face->size->metrics.x_scale;
-    hbFont.y_scale = face->size->metrics.y_scale;
-
-    hbFace = freetype->hbFace;
-
     metrics = face->size->metrics;
 
     /*
@@ -763,6 +767,17 @@ bool QFontEngineFT::init(FaceId faceId, bool antialias, GlyphFormat format,
 
     fontDef.styleName = QString::fromUtf8(face->style_name);
 
+    if (!freetype->hbFace) {
+        faceData.user_data = face;
+        faceData.get_font_table = ft_getSfntTable;
+        freetype->hbFace = harfbuzzFace();
+        freetype->hbFace_destroy_func = face_destroy_func;
+    } else {
+        Q_ASSERT(!face_);
+        face_ = freetype->hbFace;
+    }
+    face_destroy_func = 0; // we share the HB face in QFreeTypeFace, so do not let ~QFontEngine() destroy it
+
     unlockFace();
 
     fsType = freetype->fsType();
@@ -889,18 +904,28 @@ QFontEngineFT::Glyph *QFontEngineFT::loadGlyph(QGlyphSet *set, uint glyph,
     info.yOff = 0;
 
     if ((set && set->outline_drawing) || fetchMetricsOnly) {
-        // If the advance doesn't fit in signed char, don't cache it
-        if (qAbs(info.xOff) >= 128)
-            return 0;
-        g = new Glyph;
-        g->data = 0;
-        g->linearAdvance = slot->linearHoriAdvance >> 10;
         int left  = FLOOR(slot->metrics.horiBearingX);
         int right = CEIL(slot->metrics.horiBearingX + slot->metrics.width);
         int top    = CEIL(slot->metrics.horiBearingY);
         int bottom = FLOOR(slot->metrics.horiBearingY - slot->metrics.height);
-        g->width = TRUNC(right-left);
-        g->height = TRUNC(top-bottom);
+        int width = right-left;
+        int height = top-bottom;
+
+        // If any of the metrics are too large to fit, don't cache them
+        if (qAbs(info.xOff) >= 128
+                || qAbs(TRUNC(top)) >= 128
+                || TRUNC(width) >= 256
+                || TRUNC(height) >= 256
+                || qAbs(TRUNC(left)) >= 128
+                || qAbs(TRUNC(ROUND(slot->advance.x))) >= 128) {
+            return 0;
+        }
+
+        g = new Glyph;
+        g->data = 0;
+        g->linearAdvance = slot->linearHoriAdvance >> 10;
+        g->width = TRUNC(width);
+        g->height = TRUNC(height);
         g->x = TRUNC(left);
         g->y = TRUNC(top);
         g->advance = TRUNC(ROUND(slot->advance.x));
@@ -1163,7 +1188,7 @@ QFixed QFontEngineFT::emSquareSize() const
 
 bool QFontEngineFT::getSfntTableData(uint tag, uchar *buffer, uint *length) const
 {
-    return freetype->getSfntTable(tag, buffer, length);
+    return ft_getSfntTable(freetype->face, tag, buffer, length);
 }
 
 int QFontEngineFT::synthesized() const
@@ -1296,6 +1321,12 @@ void QFontEngineFT::doKerning(QGlyphLayout *g, QFontEngine::ShaperFlags flags) c
             unlockFace();
         }
     }
+
+    if (shouldUseDesignMetrics(flags) && !(fontDef.styleStrategy & QFont::ForceIntegerMetrics))
+        flags |= DesignMetrics;
+    else
+        flags &= ~DesignMetrics;
+
     QFontEngine::doKerning(g, flags);
 }
 
@@ -1411,6 +1442,14 @@ void QFontEngineFT::getUnscaledGlyph(glyph_t glyph, QPainterPath *path, glyph_me
     unlockFace();
 }
 
+bool QFontEngineFT::supportsTransformation(const QTransform &transform) const
+{
+    // The freetype engine falls back to QFontEngine for tranformed glyphs,
+    // which uses fast-tranform and produces very ugly results, so we claim
+    // to support just translations.
+    return transform.type() <= QTransform::TxTranslate;
+}
+
 static inline unsigned int getChar(const QChar *str, int &i, const int len)
 {
     uint ucs4 = str[i].unicode();
@@ -1551,12 +1590,18 @@ bool QFontEngineFT::stringToCMap(const QChar *str, int len, QGlyphLayout *glyphs
     return true;
 }
 
+bool QFontEngineFT::shouldUseDesignMetrics(QFontEngine::ShaperFlags flags) const
+{
+    if (!FT_IS_SCALABLE(freetype->face))
+        return false;
+
+    return default_hint_style == HintNone || default_hint_style == HintLight || (flags & DesignMetrics);
+}
+
 void QFontEngineFT::recalcAdvances(QGlyphLayout *glyphs, QFontEngine::ShaperFlags flags) const
 {
     FT_Face face = 0;
-    bool design = (default_hint_style == HintNone ||
-                   default_hint_style == HintLight ||
-                   (flags & DesignMetrics)) && FT_IS_SCALABLE(freetype->face);
+    bool design = shouldUseDesignMetrics(flags);
     for (int i = 0; i < glyphs->numGlyphs; i++) {
         Glyph *g = cacheEnabled ? defaultGlyphSet.getGlyph(glyphs->glyphs[i]) : 0;
         // Since we are passing Format_None to loadGlyph, use same default format logic as loadGlyph
@@ -1569,13 +1614,18 @@ void QFontEngineFT::recalcAdvances(QGlyphLayout *glyphs, QFontEngine::ShaperFlag
             g = loadGlyph(cacheEnabled ? &defaultGlyphSet : 0, glyphs->glyphs[i], 0, Format_None, true);
             glyphs->advances_x[i] = design ? QFixed::fromFixed(face->glyph->linearHoriAdvance >> 10)
                                            : QFixed::fromFixed(face->glyph->metrics.horiAdvance).round();
+            if (!cacheEnabled)
+                delete g;
         }
-        if (fontDef.styleStrategy & QFont::ForceIntegerMetrics)
-            glyphs->advances_x[i] = glyphs->advances_x[i].round();
         glyphs->advances_y[i] = 0;
     }
     if (face)
         unlockFace();
+
+    if (fontDef.styleStrategy & QFont::ForceIntegerMetrics) {
+        for (int i = 0; i < glyphs->numGlyphs; ++i)
+            glyphs->advances_x[i] = glyphs->advances_x[i].round();
+    }
 }
 
 glyph_metrics_t QFontEngineFT::boundingBox(const QGlyphLayout &glyphs)
@@ -1604,6 +1654,8 @@ glyph_metrics_t QFontEngineFT::boundingBox(const QGlyphLayout &glyphs)
             xmax = qMax(xmax, x + g->width);
             ymax = qMax(ymax, y + g->height);
             overall.xoff += g->advance;
+            if (!cacheEnabled)
+                delete g;
         } else {
             int left  = FLOOR(face->glyph->metrics.horiBearingX);
             int right = CEIL(face->glyph->metrics.horiBearingX + face->glyph->metrics.width);
@@ -1645,6 +1697,8 @@ glyph_metrics_t QFontEngineFT::boundingBox(glyph_t glyph)
         overall.xoff = g->advance;
         if (fontDef.styleStrategy & QFont::ForceIntegerMetrics)
             overall.xoff = overall.xoff.round();
+        if (!cacheEnabled)
+            delete g;
     } else {
         int left  = FLOOR(face->glyph->metrics.horiBearingX);
         int right = CEIL(face->glyph->metrics.horiBearingX + face->glyph->metrics.width);
@@ -1719,7 +1773,7 @@ glyph_metrics_t QFontEngineFT::alphaMapBoundingBox(glyph_t glyph, QFixed subPixe
             glyphSet = &defaultGlyphSet;
         }
     }
-    Glyph * g = glyphSet ? glyphSet->getGlyph(glyph) : 0;
+    Glyph * g = glyphSet ? glyphSet->getGlyph(glyph, subPixelPosition) : 0;
     if (!g || g->format != format) {
         face = lockFace();
         FT_Matrix m = this->matrix;
@@ -1734,6 +1788,8 @@ glyph_metrics_t QFontEngineFT::alphaMapBoundingBox(glyph_t glyph, QFixed subPixe
         overall.width = g->width;
         overall.height = g->height;
         overall.xoff = g->advance;
+        if (!glyphSet)
+            delete g;
     } else {
         int left  = FLOOR(face->glyph->metrics.horiBearingX);
         int right = CEIL(face->glyph->metrics.horiBearingX + face->glyph->metrics.width);
@@ -1782,6 +1838,7 @@ QImage *QFontEngineFT::lockedAlphaMapForGlyph(glyph_t glyphIndex, QFixed subPixe
     };
 
     QFontEngineFT::Glyph *glyph;
+    QScopedPointer<QFontEngineFT::Glyph> glyphGuard;
     if (cacheEnabled) {
         QFontEngineFT::QGlyphSet *gset = &defaultGlyphSet;
         QFontEngine::HintStyle hintStyle = default_hint_style;
@@ -1817,6 +1874,7 @@ QImage *QFontEngineFT::lockedAlphaMapForGlyph(glyph_t glyphIndex, QFixed subPixe
         FT_Set_Transform(freetype->face, &m, 0);
         freetype->matrix = m;
         glyph = loadGlyph(0, glyphIndex, subPixelPosition, neededFormat);
+        glyphGuard.reset(glyph);
     }
 
     if (glyph == 0 || glyph->data == 0 || glyph->width == 0 || glyph->height == 0) {
@@ -1844,6 +1902,8 @@ QImage *QFontEngineFT::lockedAlphaMapForGlyph(glyph_t glyphIndex, QFixed subPixe
         *offset = QPoint(glyph->x, -glyph->y);
 
     currentlyLockedAlphaMap = QImage(glyph->data, glyph->width, glyph->height, pitch, format);
+    if (!glyphGuard.isNull())
+        currentlyLockedAlphaMap = currentlyLockedAlphaMap.copy();
     Q_ASSERT(!currentlyLockedAlphaMap.isNull());
 
     QImageData *data = currentlyLockedAlphaMap.data_ptr();
@@ -1869,7 +1929,9 @@ QImage QFontEngineFT::alphaMapForGlyph(glyph_t g, QFixed subPixelPosition)
 {
     lockFace();
 
-    Glyph *glyph = loadGlyphFor(g, subPixelPosition, antialias ? Format_A8 : Format_Mono);
+    QScopedPointer<Glyph> glyph(loadGlyphFor(g, subPixelPosition, antialias ? Format_A8 : Format_Mono));
+    if (cacheEnabled)
+        glyph.take();
     if (!glyph || !glyph->data) {
         unlockFace();
         return QFontEngine::alphaMapForGlyph(g);
@@ -1906,7 +1968,9 @@ QImage QFontEngineFT::alphaRGBMapForGlyph(glyph_t g, QFixed subPixelPosition, co
 
     lockFace();
 
-    Glyph *glyph = loadGlyphFor(g, subPixelPosition, Format_A32);
+    QScopedPointer<Glyph> glyph(loadGlyphFor(g, subPixelPosition, Format_A32));
+    if (cacheEnabled)
+        glyph.take();
     if (!glyph || !glyph->data) {
         unlockFace();
         return QFontEngine::alphaRGBMapForGlyph(g, subPixelPosition, t);
@@ -2026,13 +2090,13 @@ void QFontEngineFT::QGlyphSet::setGlyph(glyph_t index, QFixed subPixelPosition,
     }
 }
 
-HB_Error QFontEngineFT::getPointInOutline(HB_Glyph glyph, int flags, hb_uint32 point, HB_Fixed *xpos, HB_Fixed *ypos, hb_uint32 *nPoints)
+int QFontEngineFT::getPointInOutline(glyph_t glyph, int flags, quint32 point, QFixed *xpos, QFixed *ypos, quint32 *nPoints)
 {
     lockFace();
     bool hsubpixel = true;
     int vfactor = 1;
     int load_flags = loadFlags(0, Format_A8, flags, hsubpixel, vfactor);
-    HB_Error result = freetype->getPointInOutline(glyph, load_flags, point, xpos, ypos, nPoints);
+    int result = freetype->getPointInOutline(glyph, load_flags, point, xpos, ypos, nPoints);
     unlockFace();
     return result;
 }
