@@ -60,16 +60,10 @@
 #include "private/qtextengine_p.h"
 #include "private/qfont_p.h"
 
-
-
 #include <private/qfontengineglyphcache_p.h>
 
-struct glyph_metrics_t;
-typedef unsigned int glyph_t;
-
 QT_BEGIN_NAMESPACE
 
-class QChar;
 class QPainterPath;
 
 struct QGlyphLayout;
@@ -81,10 +75,20 @@ struct QGlyphLayout;
     ((quint32)(ch4)) \
    )
 
+// ### this only used in getPointInOutline(), refactor it and then remove these magic numbers
+enum HB_Compat_Error {
+    Err_Ok                           = 0x0000,
+    Err_Not_Covered                  = 0xFFFF,
+    Err_Invalid_Argument             = 0x1A66,
+    Err_Invalid_SubTable_Format      = 0x157F,
+    Err_Invalid_SubTable             = 0x1570
+};
 
-class Q_GUI_EXPORT QFontEngine : public QObject
+typedef void (*qt_destroy_func_t) (void *user_data);
+typedef bool (*qt_get_font_table_func_t) (void *user_data, uint tag, uchar *buffer, uint *length);
+
+class Q_GUI_EXPORT QFontEngine
 {
-    Q_OBJECT
 public:
     enum Type {
         Box,
@@ -112,7 +116,8 @@ public:
         Format_Render = Format_None,
         Format_Mono,
         Format_A8,
-        Format_A32
+        Format_A32,
+        Format_ARGB
     };
 
     enum ShaperFlag {
@@ -165,11 +170,6 @@ public:
 
     /* returns 0 as glyph index for non existent glyphs */
     virtual bool stringToCMap(const QChar *str, int len, QGlyphLayout *glyphs, int *nglyphs, ShaperFlags flags) const = 0;
-
-    /**
-     * This is a callback from harfbuzz. The font engine uses the font-system in use to find out the
-     * advances of each glyph and set it on the layout.
-     */
     virtual void recalcAdvances(QGlyphLayout *, ShaperFlags) const {}
     virtual void doKerning(QGlyphLayout *, ShaperFlags) const;
 
@@ -191,6 +191,7 @@ public:
     virtual QImage alphaMapForGlyph(glyph_t, const QTransform &t);
     virtual QImage alphaMapForGlyph(glyph_t, QFixed subPixelPosition, const QTransform &t);
     virtual QImage alphaRGBMapForGlyph(glyph_t, QFixed subPixelPosition, const QTransform &t);
+    virtual QImage bitmapForGlyph(glyph_t, QFixed subPixelPosition, const QTransform &t);
     virtual QImage *lockedAlphaMapForGlyph(glyph_t glyph, QFixed subPixelPosition,
                                            GlyphFormat neededFormat,
                                            const QTransform &t = QTransform(),
@@ -241,7 +242,7 @@ public:
         return canRender(utf16, utf16len);
     }
 
-    virtual bool supportsTransformations(const QTransform &transform) const;
+    virtual bool supportsTransformation(const QTransform &transform) const;
 
     virtual Type type() const = 0;
 
@@ -250,12 +251,13 @@ public:
 
     virtual QFontEngine *cloneWithSize(qreal /*pixelSize*/) const { return 0; }
 
-    HB_Font harfbuzzFont() const;
-    HB_Face harfbuzzFace() const;
-    HB_Face initializedHarfbuzzFace() const;
+    void *harfbuzzFont() const;
+    void *harfbuzzFace() const;
+    bool supportsScript(QChar::Script script) const;
 
-    virtual HB_Error getPointInOutline(HB_Glyph glyph, int flags, hb_uint32 point, HB_Fixed *xpos, HB_Fixed *ypos, hb_uint32 *nPoints);
+    virtual int getPointInOutline(glyph_t glyph, int flags, quint32 point, QFixed *xpos, QFixed *ypos, quint32 *nPoints);
 
+    void clearGlyphCache(const void *key);
     void setGlyphCache(const void *key, QFontEngineGlyphCache *data);
     QFontEngineGlyphCache *glyphCache(const void *key, QFontEngineGlyphCache::Type type, const QTransform &transform) const;
 
@@ -274,12 +276,19 @@ public:
 
     QAtomicInt ref;
     QFontDef fontDef;
+
+    mutable void *font_;
+    mutable qt_destroy_func_t font_destroy_func;
+    mutable void *face_;
+    mutable qt_destroy_func_t face_destroy_func;
+    struct FaceData {
+        void *user_data;
+        qt_get_font_table_func_t get_font_table;
+    } faceData;
+
     uint cache_cost; // amount of mem used in kb by the font
-    int cache_count;
     uint fsType : 16;
     bool symbol;
-    mutable HB_FontRec hbFont;
-    mutable HB_Face hbFace;
     struct KernPair {
         uint left_right;
         QFixed adjust;
@@ -296,9 +305,13 @@ public:
     QImage currentlyLockedAlphaMap;
     int m_subPixelPositionCount; // Number of positions within a single pixel for this cache
 
+    inline QVariant userData() const { return m_userData; }
+
 protected:
     QFixed lastRightBearing(const QGlyphLayout &glyphs, bool round = false);
 
+    inline void setUserData(const QVariant &userData) { m_userData = userData; }
+
 private:
     struct GlyphCacheEntry {
         const void *context;
@@ -307,6 +320,9 @@ private:
     };
 
     mutable QLinkedList<GlyphCacheEntry> m_glyphCaches;
+
+private:
+    QVariant m_userData;
 };
 
 Q_DECLARE_OPERATORS_FOR_FLAGS(QFontEngine::ShaperFlags)
@@ -364,7 +380,6 @@ private:
 
 class Q_GUI_EXPORT QFontEngineMulti : public QFontEngine
 {
-    Q_OBJECT
 public:
     explicit QFontEngineMulti(int engineCount);
     ~QFontEngineMulti();
@@ -416,8 +431,6 @@ public:
     virtual void setFallbackFamiliesList(const QStringList &) {}
 
 protected:
-    friend class QPSPrintEnginePrivate;
-    friend class QPSPrintEngineFontMulti;
     friend class QRawFont;
     virtual void loadEngine(int at) = 0;
     virtual void ensureFallbackFamiliesQueried() {}
@@ -427,8 +440,8 @@ protected:
 class QTestFontEngine : public QFontEngineBox
 {
 public:
-    QTestFontEngine(int size) : QFontEngineBox(size) {}
-    virtual Type type() const { return TestFontEngine; }
+    QTestFontEngine(int size);
+    virtual Type type() const;
 };
 
 QT_END_NAMESPACE
