@@ -169,6 +169,7 @@ const QFontEngineQPA::Glyph *QFontEngineQPA::findGlyph(glyph_t g) const
 
 bool QFontEngineQPA::verifyHeader(const uchar *data, int size)
 {
+    VERIFY(quintptr(data) % Q_ALIGNOF(Header) == 0);
     VERIFY(size >= int(sizeof(Header)));
     const Header *header = reinterpret_cast<const Header *>(data);
     if (header->magic[0] != 'Q'
@@ -687,7 +688,7 @@ void QFontEngineMultiQPA::init(QFontEngine *fe)
     engines[0] = fe;
     fe->ref.ref();
     fontDef = engines[0]->fontDef;
-    setObjectName(QStringLiteral("QFontEngineMultiQPA"));
+    cache_cost = fe->cache_cost;
 }
 
 void QFontEngineMultiQPA::loadEngine(int at)
@@ -710,7 +711,7 @@ void QFontEngineMultiQPA::ensureFallbackFamiliesQueried()
     if (fallbacksQueried)
         return;
     QStringList fallbacks = QGuiApplicationPrivate::instance()->platformIntegration()->fontDatabase()->fallbacksForFamily(engine(0)->fontDef.family, QFont::Style(engine(0)->fontDef.style)
-                                                                                                                     , QFont::AnyStyle, QUnicodeTables::Script(script));
+                                                                                                                          , QFont::AnyStyle, QChar::Script(script));
     setFallbackFamiliesList(fallbacks);
 }
 
@@ -753,7 +754,9 @@ QFontEngine* QFontEngineMultiQPA::createMultiFontEngine(QFontEngine *fe, int scr
     QFontCache::EngineCache::Iterator it = fc->engineCache.find(key),
             end = fc->engineCache.end();
     while (it != end && it.key() == key) {
-        QFontEngineMulti *cachedEngine = qobject_cast<QFontEngineMulti *>(it.value().data);
+        QFontEngineMulti *cachedEngine = 0;
+        if (it.value().data->type() == QFontEngine::Multi)
+            cachedEngine = static_cast<QFontEngineMulti *>(it.value().data);
         if (faceIsLocal || (cachedEngine && fe == cachedEngine->engine(0))) {
             engine = cachedEngine;
             fc->updateHitCountAndTimeStamp(it.value());
@@ -762,7 +765,7 @@ QFontEngine* QFontEngineMultiQPA::createMultiFontEngine(QFontEngine *fe, int scr
         it++;
     }
     if (!engine) {
-        engine = QGuiApplicationPrivate::instance()->platformIntegration()->fontDatabase()->fontEngineMulti(fe, QUnicodeTables::Script(script));
+        engine = QGuiApplicationPrivate::instance()->platformIntegration()->fontDatabase()->fontEngineMulti(fe, QChar::Script(script));
         QFontCache::instance()->insertEngine(key, engine, /* insertMulti */ !faceIsLocal);
     }
     Q_ASSERT(engine);
