@@ -45,7 +45,6 @@
 
 #include "qfont_p.h"
 #include "qfontengine_p.h"
-#include <private/qunicodetables_p.h>
 
 #include <math.h>
 
@@ -213,6 +212,14 @@ QFontMetrics &QFontMetrics::operator=(const QFontMetrics &fm)
     return *this;
 }
 
+/*!
+    \fn QFontMetrics &QFontMetrics::operator=(QFontMetrics &&other)
+
+    Move-assigns \a other to this QFontMetrics instance.
+
+    \since 5.2
+*/
+
 /*!
     \fn void QFontMetrics::swap(QFontMetrics &other)
     \since 5.0
@@ -222,8 +229,8 @@ QFontMetrics &QFontMetrics::operator=(const QFontMetrics &fm)
 */
 
 /*!
-    Returns true if \a other is equal to this object; otherwise
-    returns false.
+    Returns \c true if \a other is equal to this object; otherwise
+    returns \c false.
 
     Two font metrics are considered equal if they were constructed
     from the same QFont and the paint devices they were constructed
@@ -239,7 +246,7 @@ bool QFontMetrics::operator ==(const QFontMetrics &other) const
 /*!
     \fn bool QFontMetrics::operator !=(const QFontMetrics &other) const
 
-    Returns true if \a other is not equal to this object; otherwise returns false.
+    Returns \c true if \a other is not equal to this object; otherwise returns \c false.
 
     Two font metrics are considered equal if they were constructed
     from the same QFont and the paint devices they were constructed
@@ -262,7 +269,7 @@ bool QFontMetrics::operator ==(const QFontMetrics &other) const
 */
 int QFontMetrics::ascent() const
 {
-    QFontEngine *engine = d->engineForScript(QUnicodeTables::Common);
+    QFontEngine *engine = d->engineForScript(QChar::Script_Common);
     Q_ASSERT(engine != 0);
     return qRound(engine->ascent());
 }
@@ -280,7 +287,7 @@ int QFontMetrics::ascent() const
 */
 int QFontMetrics::descent() const
 {
-    QFontEngine *engine = d->engineForScript(QUnicodeTables::Common);
+    QFontEngine *engine = d->engineForScript(QChar::Script_Common);
     Q_ASSERT(engine != 0);
     return qRound(engine->descent());
 }
@@ -295,7 +302,7 @@ int QFontMetrics::descent() const
 */
 int QFontMetrics::height() const
 {
-    QFontEngine *engine = d->engineForScript(QUnicodeTables::Common);
+    QFontEngine *engine = d->engineForScript(QChar::Script_Common);
     Q_ASSERT(engine != 0);
     return qRound(engine->ascent()) + qRound(engine->descent());
 }
@@ -309,7 +316,7 @@ int QFontMetrics::height() const
 */
 int QFontMetrics::leading() const
 {
-    QFontEngine *engine = d->engineForScript(QUnicodeTables::Common);
+    QFontEngine *engine = d->engineForScript(QChar::Script_Common);
     Q_ASSERT(engine != 0);
     return qRound(engine->leading());
 }
@@ -323,7 +330,7 @@ int QFontMetrics::leading() const
 */
 int QFontMetrics::lineSpacing() const
 {
-    QFontEngine *engine = d->engineForScript(QUnicodeTables::Common);
+    QFontEngine *engine = d->engineForScript(QChar::Script_Common);
     Q_ASSERT(engine != 0);
     return qRound(engine->leading()) + qRound(engine->ascent()) + qRound(engine->descent());
 }
@@ -340,7 +347,7 @@ int QFontMetrics::lineSpacing() const
 */
 int QFontMetrics::minLeftBearing() const
 {
-    QFontEngine *engine = d->engineForScript(QUnicodeTables::Common);
+    QFontEngine *engine = d->engineForScript(QChar::Script_Common);
     Q_ASSERT(engine != 0);
     return qRound(engine->minLeftBearing());
 }
@@ -357,7 +364,7 @@ int QFontMetrics::minLeftBearing() const
 */
 int QFontMetrics::minRightBearing() const
 {
-    QFontEngine *engine = d->engineForScript(QUnicodeTables::Common);
+    QFontEngine *engine = d->engineForScript(QChar::Script_Common);
     Q_ASSERT(engine != 0);
     return qRound(engine->minRightBearing());
 }
@@ -367,7 +374,7 @@ int QFontMetrics::minRightBearing() const
 */
 int QFontMetrics::maxWidth() const
 {
-    QFontEngine *engine = d->engineForScript(QUnicodeTables::Common);
+    QFontEngine *engine = d->engineForScript(QChar::Script_Common);
     Q_ASSERT(engine != 0);
     return qRound(engine->maxCharWidth());
 }
@@ -378,10 +385,10 @@ int QFontMetrics::maxWidth() const
 */
 int QFontMetrics::xHeight() const
 {
-    QFontEngine *engine = d->engineForScript(QUnicodeTables::Common);
+    QFontEngine *engine = d->engineForScript(QChar::Script_Common);
     Q_ASSERT(engine != 0);
     if (d->capital == QFont::SmallCaps)
-        return qRound(d->smallCapsFontPrivate()->engineForScript(QUnicodeTables::Common)->ascent());
+        return qRound(d->smallCapsFontPrivate()->engineForScript(QChar::Script_Common)->ascent());
     return qRound(engine->xHeight());
 }
 
@@ -392,18 +399,18 @@ int QFontMetrics::xHeight() const
 */
 int QFontMetrics::averageCharWidth() const
 {
-    QFontEngine *engine = d->engineForScript(QUnicodeTables::Common);
+    QFontEngine *engine = d->engineForScript(QChar::Script_Common);
     Q_ASSERT(engine != 0);
     return qRound(engine->averageCharWidth());
 }
 
 /*!
-    Returns true if character \a ch is a valid character in the font;
-    otherwise returns false.
+    Returns \c true if character \a ch is a valid character in the font;
+    otherwise returns \c false.
 */
 bool QFontMetrics::inFont(QChar ch) const
 {
-    const int script = QUnicodeTables::script(ch);
+    const int script = ch.script();
     QFontEngine *engine = d->engineForScript(script);
     Q_ASSERT(engine != 0);
     if (engine->type() == QFontEngine::Box)
@@ -412,12 +419,12 @@ bool QFontMetrics::inFont(QChar ch) const
 }
 
 /*!
-   Returns true if the character \a ucs4 encoded in UCS-4/UTF-32 is a valid
-   character in the font; otherwise returns false.
+   Returns \c true if the character \a ucs4 encoded in UCS-4/UTF-32 is a valid
+   character in the font; otherwise returns \c false.
 */
 bool QFontMetrics::inFontUcs4(uint ucs4) const
 {
-    const int script = QUnicodeTables::script(ucs4);
+    const int script = QChar::script(ucs4);
     QFontEngine *engine = d->engineForScript(script);
     Q_ASSERT(engine != 0);
     if (engine->type() == QFontEngine::Box)
@@ -439,7 +446,7 @@ bool QFontMetrics::inFontUcs4(uint ucs4) const
 */
 int QFontMetrics::leftBearing(QChar ch) const
 {
-    const int script = QUnicodeTables::script(ch);
+    const int script = ch.script();
     QFontEngine *engine;
     if (d->capital == QFont::SmallCaps && ch.isLower())
         engine = d->smallCapsFontPrivate()->engineForScript(script);
@@ -474,7 +481,7 @@ int QFontMetrics::leftBearing(QChar ch) const
 */
 int QFontMetrics::rightBearing(QChar ch) const
 {
-    const int script = QUnicodeTables::script(ch);
+    const int script = ch.script();
     QFontEngine *engine;
     if (d->capital == QFont::SmallCaps && ch.isLower())
         engine = d->smallCapsFontPrivate()->engineForScript(script);
@@ -527,10 +534,10 @@ int QFontMetrics::width(const QString &text, int len, int flags) const
         return 0;
 
     if (flags & Qt::TextBypassShaping) {
-        // Skip harfbuzz complex shaping, only use advances
+        // Skip complex shaping, only use advances
         int numGlyphs = len;
         QVarLengthGlyphLayoutArray glyphs(numGlyphs);
-        QFontEngine *engine = d->engineForScript(QUnicodeTables::Common);
+        QFontEngine *engine = d->engineForScript(QChar::Script_Common);
         if (!engine->stringToCMap(text.data(), len, &glyphs, &numGlyphs, 0)) {
             glyphs.resize(numGlyphs);
             if (!engine->stringToCMap(text.data(), len, &glyphs, &numGlyphs, 0))
@@ -577,7 +584,7 @@ int QFontMetrics::width(QChar ch) const
     if (QChar::category(ch.unicode()) == QChar::Mark_NonSpacing)
         return 0;
 
-    const int script = QUnicodeTables::script(ch);
+    const int script = ch.script();
     QFontEngine *engine;
     if (d->capital == QFont::SmallCaps && ch.isLower())
         engine = d->smallCapsFontPrivate()->engineForScript(script);
@@ -607,14 +614,13 @@ int QFontMetrics::width(QChar ch) const
 */
 int QFontMetrics::charWidth(const QString &text, int pos) const
 {
+    int width = 0;
     if (pos < 0 || pos > (int)text.length())
-        return 0;
+        return width;
 
-    QChar ch = text.unicode()[pos];
-    const int script = QUnicodeTables::script(ch);
-    int width;
-
-    if (script != QUnicodeTables::Common) {
+    QChar ch = text.at(pos);
+    const int script = ch.script();
+    if (script != QChar::Script_Common) {
         // complex script shaping. Have to do some hard work
         int from = qMax(0, pos - 8);
         int to = qMin(text.length(), pos + 8);
@@ -623,9 +629,7 @@ int QFontMetrics::charWidth(const QString &text, int pos) const
         layout.ignoreBidi = true;
         layout.itemize();
         width = qRound(layout.width(pos-from, 1));
-    } else if (QChar::category(ch.unicode()) == QChar::Mark_NonSpacing) {
-        width = 0;
-    } else {
+    } else if (ch.category() != QChar::Mark_NonSpacing) {
         QFontEngine *engine;
         if (d->capital == QFont::SmallCaps && ch.isLower())
             engine = d->smallCapsFontPrivate()->engineForScript(script);
@@ -694,7 +698,7 @@ QRect QFontMetrics::boundingRect(const QString &text) const
 */
 QRect QFontMetrics::boundingRect(QChar ch) const
 {
-    const int script = QUnicodeTables::script(ch);
+    const int script = ch.script();
     QFontEngine *engine;
     if (d->capital == QFont::SmallCaps && ch.isLower())
         engine = d->smallCapsFontPrivate()->engineForScript(script);
@@ -862,11 +866,11 @@ QRect QFontMetrics::tightBoundingRect(const QString &text) const
     The \a flags argument is optional and currently only supports
     Qt::TextShowMnemonic as value.
 
-    The elide mark will follow the \l{Qt::LayoutDirection}{layout
-    direction}; it will be on the right side of the text for
-    right-to-left layouts, and on the left side for right-to-left
-    layouts. Note that this behavior is independent of the text
-    language.
+    The elide mark follows the \l{Qt::LayoutDirection}{layoutdirection}.
+    For example, it will be on the right side of the text for right-to-left
+    layouts if the \a mode is \c{Qt::ElideLeft}, and on the left side of the
+    text if the \a mode is \c{Qt::ElideRight}.
+
 */
 QString QFontMetrics::elidedText(const QString &text, Qt::TextElideMode mode, int width, int flags) const
 {
@@ -895,7 +899,7 @@ QString QFontMetrics::elidedText(const QString &text, Qt::TextElideMode mode, in
 */
 int QFontMetrics::underlinePos() const
 {
-    QFontEngine *engine = d->engineForScript(QUnicodeTables::Common);
+    QFontEngine *engine = d->engineForScript(QChar::Script_Common);
     Q_ASSERT(engine != 0);
     return qRound(engine->underlinePosition());
 }
@@ -931,7 +935,7 @@ int QFontMetrics::strikeOutPos() const
 */
 int QFontMetrics::lineWidth() const
 {
-    QFontEngine *engine = d->engineForScript(QUnicodeTables::Common);
+    QFontEngine *engine = d->engineForScript(QChar::Script_Common);
     Q_ASSERT(engine != 0);
     return qRound(engine->lineThickness());
 }
@@ -1093,8 +1097,8 @@ QFontMetricsF &QFontMetricsF::operator=(const QFontMetricsF &fm)
 }
 
 /*!
-  Returns true if the font metrics are equal to the \a other font
-  metrics; otherwise returns false.
+  Returns \c true if the font metrics are equal to the \a other font
+  metrics; otherwise returns \c false.
 
   Two font metrics are considered equal if they were constructed from the
   same QFont and the paint devices they were constructed for are
@@ -1109,8 +1113,8 @@ bool QFontMetricsF::operator ==(const QFontMetricsF &other) const
     \fn bool QFontMetricsF::operator !=(const QFontMetricsF &other) const
     \overload
 
-    Returns true if the font metrics are not equal to the \a other font
-    metrics; otherwise returns false.
+    Returns \c true if the font metrics are not equal to the \a other font
+    metrics; otherwise returns \c false.
 
     \sa operator==()
 */
@@ -1129,7 +1133,7 @@ bool QFontMetricsF::operator ==(const QFontMetricsF &other) const
 */
 qreal QFontMetricsF::ascent() const
 {
-    QFontEngine *engine = d->engineForScript(QUnicodeTables::Common);
+    QFontEngine *engine = d->engineForScript(QChar::Script_Common);
     Q_ASSERT(engine != 0);
     return engine->ascent().toReal();
 }
@@ -1148,7 +1152,7 @@ qreal QFontMetricsF::ascent() const
 */
 qreal QFontMetricsF::descent() const
 {
-    QFontEngine *engine = d->engineForScript(QUnicodeTables::Common);
+    QFontEngine *engine = d->engineForScript(QChar::Script_Common);
     Q_ASSERT(engine != 0);
     return engine->descent().toReal();
 }
@@ -1163,7 +1167,7 @@ qreal QFontMetricsF::descent() const
 */
 qreal QFontMetricsF::height() const
 {
-    QFontEngine *engine = d->engineForScript(QUnicodeTables::Common);
+    QFontEngine *engine = d->engineForScript(QChar::Script_Common);
     Q_ASSERT(engine != 0);
 
     return (engine->ascent() + engine->descent()).toReal();
@@ -1178,7 +1182,7 @@ qreal QFontMetricsF::height() const
 */
 qreal QFontMetricsF::leading() const
 {
-    QFontEngine *engine = d->engineForScript(QUnicodeTables::Common);
+    QFontEngine *engine = d->engineForScript(QChar::Script_Common);
     Q_ASSERT(engine != 0);
     return engine->leading().toReal();
 }
@@ -1192,7 +1196,7 @@ qreal QFontMetricsF::leading() const
 */
 qreal QFontMetricsF::lineSpacing() const
 {
-    QFontEngine *engine = d->engineForScript(QUnicodeTables::Common);
+    QFontEngine *engine = d->engineForScript(QChar::Script_Common);
     Q_ASSERT(engine != 0);
     return (engine->leading() + engine->ascent() + engine->descent()).toReal();
 }
@@ -1209,7 +1213,7 @@ qreal QFontMetricsF::lineSpacing() const
 */
 qreal QFontMetricsF::minLeftBearing() const
 {
-    QFontEngine *engine = d->engineForScript(QUnicodeTables::Common);
+    QFontEngine *engine = d->engineForScript(QChar::Script_Common);
     Q_ASSERT(engine != 0);
     return engine->minLeftBearing();
 }
@@ -1226,7 +1230,7 @@ qreal QFontMetricsF::minLeftBearing() const
 */
 qreal QFontMetricsF::minRightBearing() const
 {
-    QFontEngine *engine = d->engineForScript(QUnicodeTables::Common);
+    QFontEngine *engine = d->engineForScript(QChar::Script_Common);
     Q_ASSERT(engine != 0);
     return engine->minRightBearing();
 }
@@ -1236,7 +1240,7 @@ qreal QFontMetricsF::minRightBearing() const
 */
 qreal QFontMetricsF::maxWidth() const
 {
-    QFontEngine *engine = d->engineForScript(QUnicodeTables::Common);
+    QFontEngine *engine = d->engineForScript(QChar::Script_Common);
     Q_ASSERT(engine != 0);
     return engine->maxCharWidth();
 }
@@ -1247,10 +1251,10 @@ qreal QFontMetricsF::maxWidth() const
 */
 qreal QFontMetricsF::xHeight() const
 {
-    QFontEngine *engine = d->engineForScript(QUnicodeTables::Common);
+    QFontEngine *engine = d->engineForScript(QChar::Script_Common);
     Q_ASSERT(engine != 0);
     if (d->capital == QFont::SmallCaps)
-        return d->smallCapsFontPrivate()->engineForScript(QUnicodeTables::Common)->ascent().toReal();
+        return d->smallCapsFontPrivate()->engineForScript(QChar::Script_Common)->ascent().toReal();
     return engine->xHeight().toReal();
 }
 
@@ -1261,18 +1265,18 @@ qreal QFontMetricsF::xHeight() const
 */
 qreal QFontMetricsF::averageCharWidth() const
 {
-    QFontEngine *engine = d->engineForScript(QUnicodeTables::Common);
+    QFontEngine *engine = d->engineForScript(QChar::Script_Common);
     Q_ASSERT(engine != 0);
     return engine->averageCharWidth().toReal();
 }
 
 /*!
-    Returns true if character \a ch is a valid character in the font;
-    otherwise returns false.
+    Returns \c true if character \a ch is a valid character in the font;
+    otherwise returns \c false.
 */
 bool QFontMetricsF::inFont(QChar ch) const
 {
-    const int script = QUnicodeTables::script(ch);
+    const int script = ch.script();
     QFontEngine *engine = d->engineForScript(script);
     Q_ASSERT(engine != 0);
     if (engine->type() == QFontEngine::Box)
@@ -1283,12 +1287,12 @@ bool QFontMetricsF::inFont(QChar ch) const
 /*!
     \fn bool QFontMetricsF::inFontUcs4(uint ch) const
 
-    Returns true if the character given by \a ch, encoded in UCS-4/UTF-32,
-    is a valid character in the font; otherwise returns false.
+    Returns \c true if the character given by \a ch, encoded in UCS-4/UTF-32,
+    is a valid character in the font; otherwise returns \c false.
 */
 bool QFontMetricsF::inFontUcs4(uint ucs4) const
 {
-    const int script = QUnicodeTables::script(ucs4);
+    const int script = QChar::script(ucs4);
     QFontEngine *engine = d->engineForScript(script);
     Q_ASSERT(engine != 0);
     if (engine->type() == QFontEngine::Box)
@@ -1310,7 +1314,7 @@ bool QFontMetricsF::inFontUcs4(uint ucs4) const
 */
 qreal QFontMetricsF::leftBearing(QChar ch) const
 {
-    const int script = QUnicodeTables::script(ch);
+    const int script = ch.script();
     QFontEngine *engine;
     if (d->capital == QFont::SmallCaps && ch.isLower())
         engine = d->smallCapsFontPrivate()->engineForScript(script);
@@ -1345,7 +1349,7 @@ qreal QFontMetricsF::leftBearing(QChar ch) const
 */
 qreal QFontMetricsF::rightBearing(QChar ch) const
 {
-    const int script = QUnicodeTables::script(ch);
+    const int script = ch.script();
     QFontEngine *engine;
     if (d->capital == QFont::SmallCaps && ch.isLower())
         engine = d->smallCapsFontPrivate()->engineForScript(script);
@@ -1414,10 +1418,10 @@ qreal QFontMetricsF::width(const QString &text) const
 */
 qreal QFontMetricsF::width(QChar ch) const
 {
-    if (QChar::category(ch.unicode()) == QChar::Mark_NonSpacing)
+    if (ch.category() == QChar::Mark_NonSpacing)
         return 0.;
 
-    const int script = QUnicodeTables::script(ch);
+    const int script = ch.script();
     QFontEngine *engine;
     if (d->capital == QFont::SmallCaps && ch.isLower())
         engine = d->smallCapsFontPrivate()->engineForScript(script);
@@ -1482,7 +1486,7 @@ QRectF QFontMetricsF::boundingRect(const QString &text) const
 */
 QRectF QFontMetricsF::boundingRect(QChar ch) const
 {
-    const int script = QUnicodeTables::script(ch);
+    const int script = ch.script();
     QFontEngine *engine;
     if (d->capital == QFont::SmallCaps && ch.isLower())
         engine = d->smallCapsFontPrivate()->engineForScript(script);
@@ -1583,7 +1587,7 @@ QRectF QFontMetricsF::boundingRect(const QRectF &rect, int flags, const QString&
     \li Qt::TextWordBreak breaks the text to fit the rectangle.
     \endlist
 
-    These flags are defined in \l{Qt::TextFlags}.
+    These flags are defined in the \l{Qt::TextFlag} enum.
 
     If Qt::TextExpandTabs is set in \a flags, the following behavior is
     used to interpret tab characters in the text:
@@ -1647,13 +1651,18 @@ QRectF QFontMetricsF::tightBoundingRect(const QString &text) const
     Otherwise, returns the original string.
 
     The \a mode parameter specifies whether the text is elided on the
-    left (e.g., "...tech"), in the middle (e.g., "Tr...ch"), or on
-    the right (e.g., "Trol...").
+    left (for example, "...tech"), in the middle (for example, "Tr...ch"), or
+    on the right (for example, "Trol...").
 
     The \a width is specified in pixels, not characters.
 
     The \a flags argument is optional and currently only supports
     Qt::TextShowMnemonic as value.
+
+    The elide mark follows the \l{Qt::LayoutDirection}{layoutdirection}.
+    For example, it will be on the right side of the text for right-to-left
+    layouts if the \a mode is \c{Qt::ElideLeft}, and on the left side of the
+    text if the \a mode is \c{Qt::ElideRight}.
 */
 QString QFontMetricsF::elidedText(const QString &text, Qt::TextElideMode mode, qreal width, int flags) const
 {
@@ -1682,7 +1691,7 @@ QString QFontMetricsF::elidedText(const QString &text, Qt::TextElideMode mode, q
 */
 qreal QFontMetricsF::underlinePos() const
 {
-    QFontEngine *engine = d->engineForScript(QUnicodeTables::Common);
+    QFontEngine *engine = d->engineForScript(QChar::Script_Common);
     Q_ASSERT(engine != 0);
     return engine->underlinePosition().toReal();
 }
@@ -1717,7 +1726,7 @@ qreal QFontMetricsF::strikeOutPos() const
 */
 qreal QFontMetricsF::lineWidth() const
 {
-    QFontEngine *engine = d->engineForScript(QUnicodeTables::Common);
+    QFontEngine *engine = d->engineForScript(QChar::Script_Common);
     Q_ASSERT(engine != 0);
     return engine->lineThickness().toReal();
 }
