@@ -47,11 +47,13 @@
 
 QT_BEGIN_NAMESPACE
 
-extern void qt_registerFont(const QString &familyname, const QString &stylename,
-                            const QString &foundryname, int weight,
-                            QFont::Style style, int stretch, bool antialiased,
-                            bool scalable, int pixelSize, bool fixedPitch,
-                            const QSupportedWritingSystems &writingSystems, void *hanlde);
+void qt_registerFont(const QString &familyname, const QString &stylename,
+                     const QString &foundryname, int weight,
+                     QFont::Style style, int stretch, bool antialiased,
+                     bool scalable, int pixelSize, bool fixedPitch,
+                     const QSupportedWritingSystems &writingSystems, void *hanlde);
+
+void qt_registerAliasToFontFamily(const QString &familyName, const QString &alias);
 
 /*!
     \fn void QPlatformFontDatabase::registerQPF2Font(const QByteArray &dataArray, void *handle)
@@ -215,8 +217,8 @@ void QSupportedWritingSystems::setSupported(QFontDatabase::WritingSystem writing
 }
 
 /*!
-    Returns true if the writing system specified by \a writingSystem is
-    supported; otherwise returns false.
+    Returns \c true if the writing system specified by \a writingSystem is
+    supported; otherwise returns \c false.
 */
 bool QSupportedWritingSystems::supported(QFontDatabase::WritingSystem writingSystem) const
 {
@@ -278,8 +280,7 @@ void QPlatformFontDatabase::populateFontDatabase()
     option to fall back to the fonts given by \a fallbacks if \a fontEngine does not support
     a certain character.
 */
-QFontEngineMulti *QPlatformFontDatabase::fontEngineMulti(QFontEngine *fontEngine,
-                                                         QUnicodeTables::Script script)
+QFontEngineMulti *QPlatformFontDatabase::fontEngineMulti(QFontEngine *fontEngine, QChar::Script script)
 {
     return new QFontEngineMultiQPA(fontEngine, script);
 }
@@ -288,7 +289,7 @@ QFontEngineMulti *QPlatformFontDatabase::fontEngineMulti(QFontEngine *fontEngine
     Returns the font engine that can be used to render the font described by
     the font definition, \a fontDef, in the specified \a script.
 */
-QFontEngine *QPlatformFontDatabase::fontEngine(const QFontDef &fontDef, QUnicodeTables::Script script, void *handle)
+QFontEngine *QPlatformFontDatabase::fontEngine(const QFontDef &fontDef, QChar::Script script, void *handle)
 {
     Q_UNUSED(script);
     Q_UNUSED(handle);
@@ -312,7 +313,7 @@ QFontEngine *QPlatformFontDatabase::fontEngine(const QByteArray &fontData, qreal
     Returns a list of alternative fonts for the specified \a family and
     \a style and \a script using the \a styleHint given.
 */
-QStringList QPlatformFontDatabase::fallbacksForFamily(const QString family, const QFont::Style &style, const QFont::StyleHint &styleHint, const QUnicodeTables::Script &script) const
+QStringList QPlatformFontDatabase::fallbacksForFamily(const QString &family, QFont::Style style, QFont::StyleHint styleHint, QChar::Script script) const
 {
     Q_UNUSED(family);
     Q_UNUSED(style);
@@ -380,9 +381,11 @@ QFont QPlatformFontDatabase::defaultFont() const
     \since 5.0
  */
 
+QString qt_resolveFontFamilyAlias(const QString &alias);
+
 QString QPlatformFontDatabase::resolveFontFamilyAlias(const QString &family) const
 {
-    return family;
+    return qt_resolveFontFamilyAlias(family);
 }
 
 /*!
@@ -414,6 +417,158 @@ bool QPlatformFontDatabase::fontsAlwaysScalable() const
     return ret;
 }
 
+
+// ### copied to tools/makeqpf/qpf2.cpp
+
+// see the Unicode subset bitfields in the MSDN docs
+static const ushort requiredUnicodeBits[QFontDatabase::WritingSystemsCount][2] = {
+    { 127, 127 }, // Any
+    { 0, 127 },   // Latin
+    { 7, 127 },   // Greek
+    { 9, 127 },   // Cyrillic
+    { 10, 127 },  // Armenian
+    { 11, 127 },  // Hebrew
+    { 13, 127 },  // Arabic
+    { 71, 127 },  // Syriac
+    { 72, 127 },  // Thaana
+    { 15, 127 },  // Devanagari
+    { 16, 127 },  // Bengali
+    { 17, 127 },  // Gurmukhi
+    { 18, 127 },  // Gujarati
+    { 19, 127 },  // Oriya
+    { 20, 127 },  // Tamil
+    { 21, 127 },  // Telugu
+    { 22, 127 },  // Kannada
+    { 23, 127 },  // Malayalam
+    { 73, 127 },  // Sinhala
+    { 24, 127 },  // Thai
+    { 25, 127 },  // Lao
+    { 70, 127 },  // Tibetan
+    { 74, 127 },  // Myanmar
+    { 26, 127 },  // Georgian
+    { 80, 127 },  // Khmer
+    { 126, 127 }, // SimplifiedChinese
+    { 126, 127 }, // TraditionalChinese
+    { 126, 127 }, // Japanese
+    { 56, 127 },  // Korean
+    { 0, 127 },   // Vietnamese (same as latin1)
+    { 126, 127 }, // Other
+    { 78, 127 },  // Ogham
+    { 79, 127 },  // Runic
+    { 14, 127 },  // Nko
+};
+
+enum {
+    Latin1CsbBit = 0,
+    CentralEuropeCsbBit = 1,
+    TurkishCsbBit = 4,
+    BalticCsbBit = 7,
+    CyrillicCsbBit = 2,
+    GreekCsbBit = 3,
+    HebrewCsbBit = 5,
+    ArabicCsbBit = 6,
+    VietnameseCsbBit = 8,
+    SimplifiedChineseCsbBit = 18,
+    TraditionalChineseCsbBit = 20,
+    JapaneseCsbBit = 17,
+    KoreanCsbBit = 19,
+    KoreanJohabCsbBit = 21
+};
+
+/*!
+    Helper function that determines the writing systems support by a given
+    \a unicodeRange and \a codePageRange.
+
+    \since 5.1
+*/
+QSupportedWritingSystems QPlatformFontDatabase::writingSystemsFromTrueTypeBits(quint32 unicodeRange[4], quint32 codePageRange[2])
+{
+    QSupportedWritingSystems writingSystems;
+
+    bool hasScript = false;
+    for (int i = 0; i < QFontDatabase::WritingSystemsCount; ++i) {
+        int bit = requiredUnicodeBits[i][0];
+        int index = bit/32;
+        int flag = 1 << (bit&31);
+        if (bit != 126 && (unicodeRange[index] & flag)) {
+            bit = requiredUnicodeBits[i][1];
+            index = bit/32;
+
+            flag = 1 << (bit&31);
+            if (bit == 127 || (unicodeRange[index] & flag)) {
+                writingSystems.setSupported(QFontDatabase::WritingSystem(i));
+                hasScript = true;
+                // qDebug("font %s: index=%d, flag=%8x supports script %d", familyName.latin1(), index, flag, i);
+            }
+        }
+    }
+    if (codePageRange[0] & ((1 << Latin1CsbBit) | (1 << CentralEuropeCsbBit) | (1 << TurkishCsbBit) | (1 << BalticCsbBit))) {
+        writingSystems.setSupported(QFontDatabase::Latin);
+        hasScript = true;
+        //qDebug("font %s supports Latin", familyName.latin1());
+    }
+    if (codePageRange[0] & (1 << CyrillicCsbBit)) {
+        writingSystems.setSupported(QFontDatabase::Cyrillic);
+        hasScript = true;
+        //qDebug("font %s supports Cyrillic", familyName.latin1());
+    }
+    if (codePageRange[0] & (1 << GreekCsbBit)) {
+        writingSystems.setSupported(QFontDatabase::Greek);
+        hasScript = true;
+        //qDebug("font %s supports Greek", familyName.latin1());
+    }
+    if (codePageRange[0] & (1 << HebrewCsbBit)) {
+        writingSystems.setSupported(QFontDatabase::Hebrew);
+        hasScript = true;
+        //qDebug("font %s supports Hebrew", familyName.latin1());
+    }
+    if (codePageRange[0] & (1 << ArabicCsbBit)) {
+        writingSystems.setSupported(QFontDatabase::Arabic);
+        hasScript = true;
+        //qDebug("font %s supports Arabic", familyName.latin1());
+    }
+    if (codePageRange[0] & (1 << VietnameseCsbBit)) {
+        writingSystems.setSupported(QFontDatabase::Vietnamese);
+        hasScript = true;
+        //qDebug("font %s supports Vietnamese", familyName.latin1());
+    }
+    if (codePageRange[0] & (1 << SimplifiedChineseCsbBit)) {
+        writingSystems.setSupported(QFontDatabase::SimplifiedChinese);
+        hasScript = true;
+        //qDebug("font %s supports Simplified Chinese", familyName.latin1());
+    }
+    if (codePageRange[0] & (1 << TraditionalChineseCsbBit)) {
+        writingSystems.setSupported(QFontDatabase::TraditionalChinese);
+        hasScript = true;
+        //qDebug("font %s supports Traditional Chinese", familyName.latin1());
+    }
+    if (codePageRange[0] & (1 << JapaneseCsbBit)) {
+        writingSystems.setSupported(QFontDatabase::Japanese);
+        hasScript = true;
+        //qDebug("font %s supports Japanese", familyName.latin1());
+    }
+    if (codePageRange[0] & ((1 << KoreanCsbBit) | (1 << KoreanJohabCsbBit))) {
+        writingSystems.setSupported(QFontDatabase::Korean);
+        hasScript = true;
+        //qDebug("font %s supports Korean", familyName.latin1());
+    }
+    if (!hasScript)
+        writingSystems.setSupported(QFontDatabase::Symbol);
+
+    return writingSystems;
+}
+
+/*!
+    Helper function that register the \a alias for the \a familyName.
+
+    \since 5.2
+*/
+
+void QPlatformFontDatabase::registerAliasToFontFamily(const QString &familyName, const QString &alias)
+{
+    qt_registerAliasToFontFamily(familyName, alias);
+}
+
 /*!
     \class QPlatformFontDatabase
     \since 5.0
