@@ -51,8 +51,6 @@
 #include <QtGui/qbrush.h>
 #include <QtGui/qtextoption.h>
 
-QT_BEGIN_HEADER
-
 QT_BEGIN_NAMESPACE
 
 
@@ -1009,6 +1007,4 @@ inline void QTextTableCellFormat::setPadding(qreal padding)
 
 QT_END_NAMESPACE
 
-QT_END_HEADER
-
 #endif // QTEXTFORMAT_H
