@@ -44,6 +44,7 @@
 
 #include <qthread.h>
 #include <qfont.h>
+#include <qmath.h>
 #include <qpainter.h>
 #include <qvarlengtharray.h>
 #include <qtextformat.h>
@@ -130,7 +131,7 @@ QT_BEGIN_NAMESPACE
 /*!
     \fn bool QTextInlineObject::isValid() const
 
-    Returns true if this inline object is valid; otherwise returns
+    Returns \c true if this inline object is valid; otherwise returns
     false.
 */
 
@@ -207,7 +208,7 @@ void QTextInlineObject::setAscent(qreal a)
 }
 
 /*!
-    Sets the inline object's decent to \a d.
+    Sets the inline object's descent to \a d.
 
     \sa descent(), setAscent(), width(), rect()
 */
@@ -463,24 +464,10 @@ const QTextOption &QTextLayout::textOption() const
 */
 void QTextLayout::setPreeditArea(int position, const QString &text)
 {
-    if (text.isEmpty()) {
-        if (!d->specialData)
-            return;
-        if (d->specialData->addFormats.isEmpty()) {
-            delete d->specialData;
-            d->specialData = 0;
-        } else {
-            d->specialData->preeditText = QString();
-            d->specialData->preeditPosition = -1;
-        }
-    } else {
-        if (!d->specialData)
-            d->specialData = new QTextEngine::SpecialData;
-        d->specialData->preeditPosition = position;
-        d->specialData->preeditText = text;
-    }
-    d->invalidate();
-    d->clearLineData();
+    if (d->preeditAreaPosition() == position && d->preeditAreaText() == text)
+        return;
+    d->setPreeditArea(position, text);
+
     if (d->block.docHandle())
         d->block.docHandle()->documentChange(d->block.position(), d->block.length());
 }
@@ -493,7 +480,7 @@ void QTextLayout::setPreeditArea(int position, const QString &text)
 */
 int QTextLayout::preeditAreaPosition() const
 {
-    return d->specialData ? d->specialData->preeditPosition : -1;
+    return d->preeditAreaPosition();
 }
 
 /*!
@@ -503,7 +490,7 @@ int QTextLayout::preeditAreaPosition() const
 */
 QString QTextLayout::preeditAreaText() const
 {
-    return d->specialData ? d->specialData->preeditText : QString();
+    return d->preeditAreaText();
 }
 
 
@@ -515,27 +502,10 @@ QString QTextLayout::preeditAreaText() const
 */
 void QTextLayout::setAdditionalFormats(const QList<FormatRange> &formatList)
 {
-    if (formatList.isEmpty()) {
-        if (!d->specialData)
-            return;
-        if (d->specialData->preeditText.isEmpty()) {
-            delete d->specialData;
-            d->specialData = 0;
-        } else {
-            d->specialData->addFormats = formatList;
-            d->specialData->addFormatIndices.clear();
-        }
-    } else {
-        if (!d->specialData) {
-            d->specialData = new QTextEngine::SpecialData;
-            d->specialData->preeditPosition = -1;
-        }
-        d->specialData->addFormats = formatList;
-        d->indexAdditionalFormats();
-    }
+    d->setAdditionalFormats(formatList);
+
     if (d->block.docHandle())
         d->block.docHandle()->documentChange(d->block.position(), d->block.length());
-    d->resetFontEngineCache();
 }
 
 /*!
@@ -545,21 +515,7 @@ void QTextLayout::setAdditionalFormats(const QList<FormatRange> &formatList)
 */
 QList<QTextLayout::FormatRange> QTextLayout::additionalFormats() const
 {
-    QList<FormatRange> formats;
-    if (!d->specialData)
-        return formats;
-
-    formats = d->specialData->addFormats;
-
-    if (d->specialData->addFormatIndices.isEmpty())
-        return formats;
-
-    const QTextFormatCollection *collection = d->formats();
-
-    for (int i = 0; i < d->specialData->addFormatIndices.count(); ++i)
-        formats[i].format = collection->charFormat(d->specialData->addFormatIndices.at(i));
-
-    return formats;
+    return d->additionalFormats();
 }
 
 /*!
@@ -588,8 +544,8 @@ void QTextLayout::setCacheEnabled(bool enable)
 }
 
 /*!
-    Returns true if the complete layout information is cached; otherwise
-    returns false.
+    Returns \c true if the complete layout information is cached; otherwise
+    returns \c false.
 
     \sa setCacheEnabled()
 */
@@ -773,7 +729,7 @@ int QTextLayout::leftCursorPosition(int oldPos) const
 }
 
 /*!/
-    Returns true if position \a pos is a valid cursor position.
+    Returns \c true if position \a pos is a valid cursor position.
 
     In a Unicode context some positions in the text are not valid
     cursor positions, because the position is inside a Unicode
@@ -991,15 +947,23 @@ static void addSelectedRegionsToPath(QTextEngine *eng, int lineNumber, const QPo
                 continue;
             }
 
-            if (lastSelectionWidth > 0)
-                region->addRect(boundingRect & QRectF(lastSelectionX.toReal(), selectionY, lastSelectionWidth.toReal(), lineHeight));
+            if (lastSelectionWidth > 0) {
+                QRectF rect = boundingRect & QRectF(lastSelectionX.toReal(), selectionY, lastSelectionWidth.toReal(), lineHeight);
+                rect.moveLeft(qFloor(rect.left()));
+                rect.moveTop(qFloor(rect.top()));
+                region->addRect(rect);
+            }
 
             lastSelectionX = selectionX;
             lastSelectionWidth = selectionWidth;
         }
     }
-    if (lastSelectionWidth > 0)
-        region->addRect(boundingRect & QRectF(lastSelectionX.toReal(), selectionY, lastSelectionWidth.toReal(), lineHeight));
+    if (lastSelectionWidth > 0) {
+        QRectF rect = boundingRect & QRectF(lastSelectionX.toReal(), selectionY, lastSelectionWidth.toReal(), lineHeight);
+        rect.moveLeft(qFloor(rect.left()));
+        rect.moveTop(qFloor(rect.top()));
+        region->addRect(rect);
+    }
 }
 
 static inline QRectF clipIfValid(const QRectF &rect, const QRectF &clip)
@@ -1022,7 +986,7 @@ static inline QRectF clipIfValid(const QRectF &rect, const QRectF &clip)
 */
 #if !defined(QT_NO_RAWFONT)
 QList<QGlyphRun> QTextLayout::glyphRuns(int from, int length) const
-{    
+{
     if (from < 0)
         from = 0;
     if (length < 0)
@@ -1333,7 +1297,7 @@ void QTextLayout::drawCursor(QPainter *p, const QPointF &pos, int cursorPosition
     After being created, the line can be filled using the setLineWidth()
     or setNumColumns() functions. A line has a number of attributes including the
     rectangle it occupies, rect(), its coordinates, x() and y(), its
-    textLength(), width() and naturalTextWidth(), and its ascent() and decent()
+    textLength(), width() and naturalTextWidth(), and its ascent() and descent()
     relative to the text. The position of the cursor in terms of the
     line is available from cursorToX() and its inverse from
     xToCursor(). A line can be moved with setPosition().
@@ -1370,7 +1334,7 @@ void QTextLayout::drawCursor(QPainter *p, const QPointF &pos, int cursorPosition
 /*!
     \fn bool QTextLine::isValid() const
 
-    Returns true if this text line is valid; otherwise returns false.
+    Returns \c true if this text line is valid; otherwise returns \c false.
 */
 
 /*!
@@ -1504,8 +1468,8 @@ void QTextLine::setLeadingIncluded(bool included)
 /*!
     \since 4.6
 
-    Returns true if positive leading is included into the line's height;
-    otherwise returns false.
+    Returns \c true if positive leading is included into the line's height;
+    otherwise returns \c false.
 
     By default, leading is not included.
 
@@ -1651,7 +1615,7 @@ namespace {
         }
 
         inline glyph_t currentGlyph() const
-        {            
+        {
             Q_ASSERT(currentPosition > 0);
             Q_ASSERT(logClusters[currentPosition - 1] < glyphs.numGlyphs);
 
@@ -1833,6 +1797,9 @@ void QTextLine::layout_helper(int maxGlyphs)
             if (!line.length && !lbh.tmpData.length)
                 line.setDefaultHeight(eng);
             if (eng->option.flags() & QTextOption::ShowLineAndParagraphSeparators) {
+                if (lbh.checkFullOtherwiseExtend(line))
+                    goto found;
+
                 addNextCluster(lbh.currentPosition, end, lbh.tmpData, lbh.glyphCount,
                                current, lbh.logClusters, lbh.glyphs);
             } else {
@@ -2119,7 +2086,7 @@ static void setPenAndDrawBackground(QPainter *p, const QPen &defaultPen, const Q
 
     QBrush bg = chf.background();
     if (bg.style() != Qt::NoBrush && !chf.property(SuppressBackground).toBool())
-        p->fillRect(r, bg);
+        p->fillRect(QRectF(qFloor(r.x()), qFloor(r.y()), r.width(), r.height()), bg);
     if (c.style() != Qt::NoBrush) {
         p->setPen(QPen(c, 0));
     }
