@@ -61,6 +61,7 @@ QTextOption::QTextOption()
       wordWrap(QTextOption::WordWrap),
       design(false),
       unused(0),
+      unused2(0),
       f(0),
       tab(-1),
       d(0)
@@ -78,6 +79,7 @@ QTextOption::QTextOption(Qt::Alignment alignment)
       wordWrap(QTextOption::WordWrap),
       design(false),
       unused(0),
+      unused2(0),
       f(0),
       tab(-1),
       d(0)
@@ -104,6 +106,7 @@ QTextOption::QTextOption(const QTextOption &o)
       design(o.design),
       direction(o.direction),
       unused(o.unused),
+      unused2(o.unused2),
       f(o.f),
       tab(o.tab),
       d(0)
@@ -115,8 +118,8 @@ QTextOption::QTextOption(const QTextOption &o)
 /*!
     \fn QTextOption &QTextOption::operator=(const QTextOption &other)
 
-    Returns true if the text option is the same as the \a other text option;
-    otherwise returns false.
+    Returns \c true if the text option is the same as the \a other text option;
+    otherwise returns \c false.
 */
 QTextOption &QTextOption::operator=(const QTextOption &o)
 {
@@ -245,8 +248,8 @@ QList<QTextOption::Tab> QTextOption::tabs() const
 /*!
   \fn bool QTextOption::useDesignMetrics() const
 
-    Returns true if the layout uses design rather than device metrics;
-    otherwise returns false.
+    Returns \c true if the layout uses design rather than device metrics;
+    otherwise returns \c false.
 
     \sa setUseDesignMetrics()
 */
@@ -394,7 +397,7 @@ QList<QTextOption::Tab> QTextOption::tabs() const
 
 /*!
     \fn Tab::Tab(qreal pos, TabType tabType, QChar delim = QChar())
-    
+
     Creates a tab with the given position, tab type, and delimiter
     (\a pos, \a tabType, \a delim).
 
@@ -406,15 +409,15 @@ QList<QTextOption::Tab> QTextOption::tabs() const
 /*!
     \fn bool Tab::operator==(const Tab &other) const
 
-    Returns true if tab \a other is equal to this tab;
-    otherwise returns false.
+    Returns \c true if tab \a other is equal to this tab;
+    otherwise returns \c false.
 */
 
 /*!
     \fn bool Tab::operator!=(const Tab &other) const
 
-    Returns true if tab \a other is not equal to this tab;
-    otherwise returns false.
+    Returns \c true if tab \a other is not equal to this tab;
+    otherwise returns \c false.
 */
 
 /*!
