@@ -899,7 +899,7 @@ QIODevice* QZipReader::device() const
 }
 
 /*!
-    Returns true if the user can read the file; otherwise returns false.
+    Returns \c true if the user can read the file; otherwise returns \c false.
 */
 bool QZipReader::isReadable() const
 {
@@ -907,7 +907,7 @@ bool QZipReader::isReadable() const
 }
 
 /*!
-    Returns true if the file exists; otherwise returns false.
+    Returns \c true if the file exists; otherwise returns \c false.
 */
 bool QZipReader::exists() const
 {
@@ -1194,7 +1194,7 @@ QIODevice* QZipWriter::device() const
 }
 
 /*!
-    Returns true if the user can write to the archive; otherwise returns false.
+    Returns \c true if the user can write to the archive; otherwise returns \c false.
 */
 bool QZipWriter::isWritable() const
 {
@@ -1202,7 +1202,7 @@ bool QZipWriter::isWritable() const
 }
 
 /*!
-    Returns true if the file exists; otherwise returns false.
+    Returns \c true if the file exists; otherwise returns \c false.
 */
 bool QZipWriter::exists() const
 {
@@ -1302,7 +1302,7 @@ QFile::Permissions QZipWriter::creationPermissions() const
 */
 void QZipWriter::addFile(const QString &fileName, const QByteArray &data)
 {
-    d->addEntry(QZipWriterPrivate::File, fileName, data);
+    d->addEntry(QZipWriterPrivate::File, QDir::fromNativeSeparators(fileName), data);
 }
 
 /*!
@@ -1324,7 +1324,7 @@ void QZipWriter::addFile(const QString &fileName, QIODevice *device)
             return;
         }
     }
-    d->addEntry(QZipWriterPrivate::File, fileName, device->readAll());
+    d->addEntry(QZipWriterPrivate::File, QDir::fromNativeSeparators(fileName), device->readAll());
     if (opened)
         device->close();
 }
@@ -1335,10 +1335,10 @@ void QZipWriter::addFile(const QString &fileName, QIODevice *device)
 */
 void QZipWriter::addDirectory(const QString &dirName)
 {
-    QString name = dirName;
+    QString name(QDir::fromNativeSeparators(dirName));
     // separator is mandatory
-    if (!name.endsWith(QDir::separator()))
-        name.append(QDir::separator());
+    if (!name.endsWith(QLatin1Char('/')))
+        name.append(QLatin1Char('/'));
     d->addEntry(QZipWriterPrivate::Directory, name, QByteArray());
 }
 
@@ -1349,7 +1349,7 @@ void QZipWriter::addDirectory(const QString &dirName)
 */
 void QZipWriter::addSymLink(const QString &fileName, const QString &destination)
 {
-    d->addEntry(QZipWriterPrivate::Symlink, fileName, QFile::encodeName(destination));
+    d->addEntry(QZipWriterPrivate::Symlink, QDir::fromNativeSeparators(fileName), QFile::encodeName(destination));
 }
 
 /*!
