@@ -47,8 +47,6 @@
 #include <QtCore/QIODevice>
 #include <QtNetwork/QNetworkRequest>
 
-QT_BEGIN_HEADER
-
 QT_BEGIN_NAMESPACE
 
 
@@ -119,6 +117,4 @@ private:
 
 QT_END_NAMESPACE
 
-QT_END_HEADER
-
 #endif // QHTTPMULTIPART_H
