@@ -183,6 +183,9 @@ void QHttpNetworkConnectionChannel::close()
     else
         state = QHttpNetworkConnectionChannel::ClosingState;
 
+    // pendingEncrypt must only be true in between connected and encrypted states
+    pendingEncrypt = false;
+
     if (socket)
         socket->close();
 }
@@ -205,6 +208,17 @@ bool QHttpNetworkConnectionChannel::sendRequest()
             // _q_connected or _q_encrypted
             return false;
         }
+        QString scheme = request.url().scheme();
+        if (scheme == QLatin1String("preconnect-http")
+            || scheme == QLatin1String("preconnect-https")) {
+            state = QHttpNetworkConnectionChannel::IdleState;
+            reply->d_func()->state = QHttpNetworkReplyPrivate::AllDoneState;
+            allDone();
+            connection->preConnectFinished(); // will only decrease the counter
+            reply = 0; // so we can reuse this channel
+            return true; // we have a working connection and are done
+        }
+
         written = 0; // excluding the header
         bytesTotal = 0;
 
@@ -222,8 +236,8 @@ bool QHttpNetworkConnectionChannel::sendRequest()
             QAuthenticator &auth = authenticator;
             if (url.userName() != auth.user()
                 || (!url.password().isEmpty() && url.password() != auth.password())) {
-                auth.setUser(url.userName());
-                auth.setPassword(url.password());
+                auth.setUser(url.userName(QUrl::FullyDecoded));
+                auth.setPassword(url.password(QUrl::FullyDecoded));
                 connection->d_func()->copyCredentials(connection->d_func()->indexOf(socket), &auth, false);
             }
             // clear the userinfo,  since we use the same request for resending
@@ -622,6 +636,13 @@ bool QHttpNetworkConnectionChannel::ensureConnection()
         if (ssl) {
 #ifndef QT_NO_SSL
             QSslSocket *sslSocket = qobject_cast<QSslSocket*>(socket);
+
+            // check whether we can re-use an existing SSL session
+            // (meaning another socket in this connection has already
+            // performed a full handshake)
+            if (!connection->sslContext().isNull())
+                QSslSocketPrivate::checkSettingSslContext(sslSocket, connection->sslContext());
+
             sslSocket->connectToHostEncrypted(connectHost, connectPort, QIODevice::ReadWrite, networkLayerPreference);
             if (ignoreAllSslErrors)
                 sslSocket->ignoreSslErrors();
@@ -1024,7 +1045,7 @@ void QHttpNetworkConnectionChannel::_q_disconnected()
 void QHttpNetworkConnectionChannel::_q_connected()
 {
     // For the Happy Eyeballs we need to check if this is the first channel to connect.
-    if (connection->d_func()->networkLayerState == QHttpNetworkConnectionPrivate::InProgress) {
+    if (connection->d_func()->networkLayerState == QHttpNetworkConnectionPrivate::HostLookupPending || connection->d_func()->networkLayerState == QHttpNetworkConnectionPrivate::IPv4or6) {
         if (connection->d_func()->delayedConnectionTimer.isActive())
             connection->d_func()->delayedConnectionTimer.stop();
         if (networkLayerPreference == QAbstractSocket::IPv4Protocol)
@@ -1065,7 +1086,17 @@ void QHttpNetworkConnectionChannel::_q_connected()
 
     // ### FIXME: if the server closes the connection unexpectedly, we shouldn't send the same broken request again!
     //channels[i].reconnectAttempts = 2;
-    if (!pendingEncrypt) {
+    if (pendingEncrypt) {
+#ifndef QT_NO_SSL
+        if (connection->sslContext().isNull()) {
+            // this socket is making the 1st handshake for this connection,
+            // we need to set the SSL context so new sockets can reuse it
+            QSharedPointer<QSslContext> socketSslContext = QSslSocketPrivate::sslContext(static_cast<QSslSocket*>(socket));
+            if (!socketSslContext.isNull())
+                connection->setSslContext(socketSslContext);
+        }
+#endif
+    } else {
         state = QHttpNetworkConnectionChannel::IdleState;
         if (!reply)
             connection->d_func()->dequeueRequest(socket);
@@ -1099,6 +1130,9 @@ void QHttpNetworkConnectionChannel::_q_error(QAbstractSocket::SocketError socket
                 errorCode = QNetworkReply::RemoteHostClosedError;
             }
         } else if (state == QHttpNetworkConnectionChannel::ReadingState) {
+            if (!reply)
+                break;
+
             if (!reply->d_func()->expectContent()) {
                 // No content expected, this is a valid way to have the connection closed by the server
                 return;
@@ -1154,6 +1188,22 @@ void QHttpNetworkConnectionChannel::_q_error(QAbstractSocket::SocketError socket
     case QAbstractSocket::SslHandshakeFailedError:
         errorCode = QNetworkReply::SslHandshakeFailedError;
         break;
+    case QAbstractSocket::ProxyConnectionClosedError:
+        // try to reconnect/resend before sending an error.
+        if (reconnectAttempts-- > 0) {
+            closeAndResendCurrentRequest();
+            return;
+        }
+        errorCode = QNetworkReply::ProxyConnectionClosedError;
+        break;
+    case QAbstractSocket::ProxyConnectionTimeoutError:
+        // try to reconnect/resend before sending an error.
+        if (reconnectAttempts-- > 0) {
+            closeAndResendCurrentRequest();
+            return;
+        }
+        errorCode = QNetworkReply::ProxyTimeoutError;
+        break;
     default:
         // all other errors are treated as NetworkError
         errorCode = QNetworkReply::UnknownNetworkError;
@@ -1162,7 +1212,7 @@ void QHttpNetworkConnectionChannel::_q_error(QAbstractSocket::SocketError socket
     QPointer<QHttpNetworkConnection> that = connection;
     QString errorString = connection->d_func()->errorDetail(errorCode, socket, socket->errorString());
 
-    // In the InProgress state the channel should not emit the error.
+    // In the HostLookupPending state the channel should not emit the error.
     // This will instead be handled by the connection.
     if (!connection->d_func()->shouldEmitChannelError(socket))
         return;
@@ -1207,6 +1257,8 @@ void QHttpNetworkConnectionChannel::_q_encrypted()
     pendingEncrypt = false;
     if (!reply)
         connection->d_func()->dequeueRequest(socket);
+    if (reply)
+        emit reply->encrypted();
     if (reply)
         sendRequest();
 }
