@@ -42,6 +42,7 @@
 #include "qnetworkaccessdebugpipebackend_p.h"
 #include "QtCore/qdatastream.h"
 #include <QCoreApplication>
+#include <QStringList>
 #include <QUrlQuery>
 #include "private/qnoncontiguousbytedevice_p.h"
 
@@ -54,6 +55,11 @@ enum {
     WriteBufferSize = ReadBufferSize
 };
 
+QStringList QNetworkAccessDebugPipeBackendFactory::supportedSchemes() const
+{
+    return QStringList(QStringLiteral("debugpipe"));
+}
+
 QNetworkAccessBackend *
 QNetworkAccessDebugPipeBackendFactory::create(QNetworkAccessManager::Operation op,
                                               const QNetworkRequest &request) const
@@ -143,7 +149,7 @@ void QNetworkAccessDebugPipeBackend::pushFromSocketToDownstream()
 
         buffer.resize(ReadBufferSize);
         qint64 haveRead = socket.read(buffer.data(), ReadBufferSize);
-        
+
         if (haveRead == -1) {
             hasDownloadFinished = true;
             // this ensures a good last downloadProgress is emitted
