@@ -194,6 +194,31 @@ QNetworkReplyPrivate::QNetworkReplyPrivate()
     \sa error()
 */
 
+/*!
+    \fn void QNetworkReply::encrypted()
+    \since 5.1
+
+    This signal is emitted when an SSL/TLS session has successfully
+    completed the initial handshake. At this point, no user data
+    has been transmitted. The signal can be used to perform
+    additional checks on the certificate chain, for example to
+    notify users when the certificate for a website has changed.
+    If the reply does not match the expected criteria then it should
+    be aborted by calling QNetworkReply::abort() by a slot connected
+    to this signal. The SSL configuration in use can be inspected
+    using the QNetworkReply::sslConfiguration() method.
+
+    Internally, QNetworkAccessManager may open multiple connections
+    to a server, in order to allow it process requests in parallel.
+    These connections may be reused, which means that the encrypted()
+    signal would not be emitted. This means that you are only
+    guaranteed to receive this signal for the first connection to a
+    site in the lifespan of the QNetworkAccessManager.
+
+    \sa QSslSocket::encrypted()
+    \sa QNetworkAccessManager::encrypted()
+*/
+
 /*!
     \fn void QNetworkReply::sslErrors(const QList<QSslError> &errors)
 
@@ -465,7 +490,7 @@ QNetworkReply::NetworkError QNetworkReply::error() const
 /*!
     \since 4.6
 
-    Returns true when the reply has finished or was aborted.
+    Returns \c true when the reply has finished or was aborted.
 
     \sa isRunning()
 */
@@ -477,7 +502,7 @@ bool QNetworkReply::isFinished() const
 /*!
     \since 4.6
 
-    Returns true when the request is still processing and the
+    Returns \c true when the request is still processing and the
     reply has not finished or was aborted yet.
 
     \sa isFinished()
@@ -511,7 +536,7 @@ QVariant QNetworkReply::header(QNetworkRequest::KnownHeaders header) const
 }
 
 /*!
-    Returns true if the raw header of name \a headerName was sent by
+    Returns \c true if the raw header of name \a headerName was sent by
     the remote server
 
     \sa rawHeader()
