@@ -47,8 +47,6 @@
 #include <QtCore/QUrl>
 #include <QtCore/QVariant>
 
-QT_BEGIN_HEADER
-
 QT_BEGIN_NAMESPACE
 
 
@@ -159,6 +157,4 @@ QT_END_NAMESPACE
 
 Q_DECLARE_METATYPE(QNetworkRequest)
 
-QT_END_HEADER
-
 #endif
