@@ -60,8 +60,6 @@
 
 #ifndef QT_NO_BEARERMANAGEMENT
 
-QT_BEGIN_HEADER
-
 QT_BEGIN_NAMESPACE
 
 
@@ -79,8 +77,6 @@ public:
 
 QT_END_NAMESPACE
 
-QT_END_HEADER
-
 #endif // QT_NO_BEARERMANAGEMENT
 
 #endif // QBEARERPLUGIN_P_H
