@@ -55,10 +55,14 @@
 QT_BEGIN_NAMESPACE
 
 static QBasicAtomicPointer<QNetworkConfigurationManagerPrivate> connManager_ptr;
+static QBasicAtomicInt appShutdown;
 
 static void connManager_cleanup()
 {
     // this is not atomic or thread-safe!
+    int shutdown = appShutdown.fetchAndStoreAcquire(1);
+    Q_ASSERT(shutdown == 0);
+    Q_UNUSED(shutdown);
     QNetworkConfigurationManagerPrivate *cmp = connManager_ptr.fetchAndStoreAcquire(0);
     if (cmp)
         cmp->cleanup();
@@ -72,7 +76,8 @@ void QNetworkConfigurationManagerPrivate::addPostRoutine()
 QNetworkConfigurationManagerPrivate *qNetworkConfigurationManagerPrivate()
 {
     QNetworkConfigurationManagerPrivate *ptr = connManager_ptr.loadAcquire();
-    if (!ptr) {
+    int shutdown = appShutdown.loadAcquire();
+    if (!ptr && !shutdown) {
         static QBasicMutex connManager_mutex;
         QMutexLocker locker(&connManager_mutex);
         if (!(ptr = connManager_ptr.loadAcquire())) {
@@ -134,7 +139,7 @@ QNetworkConfigurationManagerPrivate *qNetworkConfigurationManagerPrivate()
     \sa QNetworkConfiguration
 */
 
-/*! 
+/*!
     \fn void QNetworkConfigurationManager::configurationAdded(const QNetworkConfiguration &config)
 
     This signal is emitted whenever a new network configuration is added to the system. The new
@@ -316,8 +321,8 @@ QNetworkConfiguration QNetworkConfigurationManager::configurationFromIdentifier(
 }
 
 /*!
-    Returns true if the system is considered to be connected to another device via an active
-    network interface; otherwise returns false.
+    Returns \c true if the system is considered to be connected to another device via an active
+    network interface; otherwise returns \c false.
 
     This is equivalent to the following code snippet:
 
