@@ -47,8 +47,6 @@
 
 #ifndef QT_NO_BEARERMANAGEMENT
 
-QT_BEGIN_HEADER
-
 QT_BEGIN_NAMESPACE
 
 class QNetworkConfigurationManagerPrivate;
@@ -98,8 +96,6 @@ Q_DECLARE_OPERATORS_FOR_FLAGS(QNetworkConfigurationManager::Capabilities)
 
 QT_END_NAMESPACE
 
-QT_END_HEADER
-
 #endif // QT_NO_BEARERMANAGEMENT
 
 #endif // QNETWORKCONFIGMANAGER_H
