@@ -63,6 +63,10 @@
 #  include <resolv.h>
 #endif
 
+#ifdef __GNU_LIBRARY__
+#  include <gnu/lib-names.h>
+#endif
+
 #if defined (QT_NO_GETADDRINFO)
 static QBasicMutex getHostByNameMutex;
 #endif
@@ -92,10 +96,17 @@ static res_state_ptr local_res = 0;
 
 static void resolveLibrary()
 {
-#ifndef QT_NO_LIBRARY
-    QLibrary lib(QLatin1String("resolv"));
+#if !defined(QT_NO_LIBRARY) && !defined(Q_OS_QNX)
+    QLibrary lib;
+#ifdef LIBRESOLV_SO
+    lib.setFileName(QStringLiteral(LIBRESOLV_SO));
     if (!lib.load())
-        return;
+#endif
+    {
+        lib.setFileName(QLatin1String("resolv"));
+        if (!lib.load())
+            return;
+    }
 
     local_res_init = res_init_proto(lib.resolve("__res_init"));
     if (!local_res_init)
@@ -323,7 +334,7 @@ QString QHostInfo::localHostName()
 
 QString QHostInfo::localDomainName()
 {
-#if !defined(Q_OS_VXWORKS) && !defined(Q_OS_LINUX_ANDROID)
+#if !defined(Q_OS_VXWORKS) && !defined(Q_OS_ANDROID)
     resolveLibrary();
     if (local_res_ninit) {
         // using thread-safe version
