@@ -61,7 +61,7 @@
 #  define QT_NO_GETIFADDRS
 #endif
 
-#ifdef Q_OS_LINUX_ANDROID
+#ifdef Q_OS_ANDROID
 // android lacks if_nameindex
 # define QT_NO_IPV6IFNAME
 #endif
@@ -81,7 +81,7 @@
 
 QT_BEGIN_NAMESPACE
 
-static QHostAddress addressFromSockaddr(sockaddr *sa)
+static QHostAddress addressFromSockaddr(sockaddr *sa, int ifindex = 0, const QString &ifname = QString())
 {
     QHostAddress address;
     if (!sa)
@@ -92,7 +92,11 @@ static QHostAddress addressFromSockaddr(sockaddr *sa)
     else if (sa->sa_family == AF_INET6) {
         address.setAddress(((sockaddr_in6 *)sa)->sin6_addr.s6_addr);
         int scope = ((sockaddr_in6 *)sa)->sin6_scope_id;
-        if (scope) {
+        if (scope && scope == ifindex) {
+            // this is the most likely scenario:
+            // a scope ID in a socket is that of the interface this address came from
+            address.setScopeId(ifname);
+        } else  if (scope) {
 #ifndef QT_NO_IPV6IFNAME
             char scopeid[IFNAMSIZ];
             if (::if_indextoname(scope, scopeid)) {
@@ -273,18 +277,18 @@ static QList<QNetworkInterfacePrivate *> interfaceListing()
             }
         }
 
-        // Get the interface netmask
-        if (qt_safe_ioctl(socket, SIOCGIFNETMASK, &req) >= 0) {
-            sockaddr *sa = &req.ifr_addr;
-	    entry.setNetmask(addressFromSockaddr(sa));
-        }
-
         // Get the address of the interface
         if (qt_safe_ioctl(socket, SIOCGIFADDR, &req) >= 0) {
             sockaddr *sa = &req.ifr_addr;
             entry.setIp(addressFromSockaddr(sa));
         }
 
+        // Get the interface netmask
+        if (qt_safe_ioctl(socket, SIOCGIFNETMASK, &req) >= 0) {
+            sockaddr *sa = &req.ifr_addr;
+            entry.setNetmask(addressFromSockaddr(sa));
+        }
+
         iface->addressEntries << entry;
     }
 
@@ -308,38 +312,38 @@ QT_END_INCLUDE_NAMESPACE
 static QList<QNetworkInterfacePrivate *> createInterfaces(ifaddrs *rawList)
 {
     QList<QNetworkInterfacePrivate *> interfaces;
+    QSet<QString> seenInterfaces;
 
+    // on Linux, AF_PACKET addresses carry the hardware address and interface index;
+    // scan for them first (they're usually first, but we have no guarantee this
+    // will be the case forever)
     for (ifaddrs *ptr = rawList; ptr; ptr = ptr->ifa_next) {
-        if ( !ptr->ifa_addr )
-            continue;
-
-        // Get the interface index
-        int ifindex = if_nametoindex(ptr->ifa_name);
-
-        // on Linux we use AF_PACKET and sockaddr_ll to obtain hHwAddress
-        QList<QNetworkInterfacePrivate *>::Iterator if_it = interfaces.begin();
-        for ( ; if_it != interfaces.end(); ++if_it)
-            if ((*if_it)->index == ifindex) {
-                // this one has been added already
-                if ( ptr->ifa_addr->sa_family == AF_PACKET
-                        && (*if_it)->hardwareAddress.isEmpty()) {
-                    sockaddr_ll *sll = (sockaddr_ll *)ptr->ifa_addr;
-                    (*if_it)->hardwareAddress = (*if_it)->makeHwAddress(sll->sll_halen, (uchar*)sll->sll_addr);
-                }
-                break;
-            }
-        if ( if_it != interfaces.end() ) 
-            continue;
-        
-        QNetworkInterfacePrivate *iface = new QNetworkInterfacePrivate;
-        interfaces << iface;
-        iface->index = ifindex;
-        iface->name = QString::fromLatin1(ptr->ifa_name);
-        iface->flags = convertFlags(ptr->ifa_flags);
-
-        if ( ptr->ifa_addr->sa_family == AF_PACKET ) {
+        if (ptr->ifa_addr && ptr->ifa_addr->sa_family == AF_PACKET) {
             sockaddr_ll *sll = (sockaddr_ll *)ptr->ifa_addr;
+            QNetworkInterfacePrivate *iface = new QNetworkInterfacePrivate;
+            interfaces << iface;
+            iface->index = sll->sll_ifindex;
+            iface->name = QString::fromLatin1(ptr->ifa_name);
+            iface->flags = convertFlags(ptr->ifa_flags);
             iface->hardwareAddress = iface->makeHwAddress(sll->sll_halen, (uchar*)sll->sll_addr);
+
+            seenInterfaces.insert(iface->name);
+        }
+    }
+
+    // see if we missed anything:
+    // virtual interfaces with no HW address have no AF_PACKET
+    for (ifaddrs *ptr = rawList; ptr; ptr = ptr->ifa_next) {
+        if (ptr->ifa_addr && ptr->ifa_addr->sa_family != AF_PACKET) {
+            QString name = QString::fromLatin1(ptr->ifa_name);
+            if (seenInterfaces.contains(name))
+                continue;
+
+            QNetworkInterfacePrivate *iface = new QNetworkInterfacePrivate;
+            interfaces << iface;
+            iface->name = name;
+            iface->flags = convertFlags(ptr->ifa_flags);
+            iface->index = if_nametoindex(ptr->ifa_name);
         }
     }
 
@@ -410,25 +414,20 @@ static QList<QNetworkInterfacePrivate *> interfaceListing()
 {
     QList<QNetworkInterfacePrivate *> interfaces;
 
-    int socket;
-    if ((socket = qt_safe_socket(AF_INET, SOCK_STREAM, IPPROTO_IP)) == -1)
-        return interfaces;      // error
-
     ifaddrs *interfaceListing;
     if (getifaddrs(&interfaceListing) == -1) {
         // error
-        ::close(socket);
         return interfaces;
     }
 
     interfaces = createInterfaces(interfaceListing);
     for (ifaddrs *ptr = interfaceListing; ptr; ptr = ptr->ifa_next) {
         // Get the interface index
-        int ifindex = if_nametoindex(ptr->ifa_name);
+        QString name = QString::fromLatin1(ptr->ifa_name);
         QNetworkInterfacePrivate *iface = 0;
         QList<QNetworkInterfacePrivate *>::Iterator if_it = interfaces.begin();
         for ( ; if_it != interfaces.end(); ++if_it)
-            if ((*if_it)->index == ifindex) {
+            if ((*if_it)->name == name) {
                 // found this interface already
                 iface = *if_it;
                 break;
@@ -439,20 +438,19 @@ static QList<QNetworkInterfacePrivate *> interfaceListing()
         }
 
         QNetworkAddressEntry entry;
-        entry.setIp(addressFromSockaddr(ptr->ifa_addr));
+        entry.setIp(addressFromSockaddr(ptr->ifa_addr, iface->index, iface->name));
         if (entry.ip().isNull())
             // could not parse the address
             continue;
 
-        entry.setNetmask(addressFromSockaddr(ptr->ifa_netmask));
+        entry.setNetmask(addressFromSockaddr(ptr->ifa_netmask, iface->index, iface->name));
         if (iface->flags & QNetworkInterface::CanBroadcast)
-            entry.setBroadcast(addressFromSockaddr(ptr->ifa_broadaddr));
+            entry.setBroadcast(addressFromSockaddr(ptr->ifa_broadaddr, iface->index, iface->name));
 
         iface->addressEntries << entry;
     }
 
     freeifaddrs(interfaceListing);
-    ::close(socket);
     return interfaces;
 }
 #endif
