@@ -297,9 +297,9 @@ void QLocalServerPrivate::waitForNewConnection(int msec, bool *timedOut)
     FD_ZERO(&readfds);
     FD_SET(listenSocket, &readfds);
 
-    timeval timeout;
+    struct timespec timeout;
     timeout.tv_sec = msec / 1000;
-    timeout.tv_usec = (msec % 1000) * 1000;
+    timeout.tv_nsec = (msec % 1000) * 1000 * 1000;
 
     int result = -1;
     result = qt_safe_select(listenSocket + 1, &readfds, 0, 0, (msec == -1) ? 0 : &timeout);
