@@ -71,18 +71,33 @@ QT_BEGIN_NAMESPACE
 */
 
 /*!
-    \fn void QLocalSocket::connectToServer(const QString &name, OpenMode openMode)
+    \fn void QLocalSocket::connectToServer(OpenMode openMode)
+    \since 5.1
 
-    Attempts to make a connection to \a name.
+    Attempts to make a connection to serverName().
+    setServerName() must be called before you open the connection.
+    Alternatively you can use connectToServer(const QString &name, OpenMode openMode);
 
     The socket is opened in the given \a openMode and first enters ConnectingState.
-    It then attempts to connect to the address or addresses returned by the lookup.
-    Finally, if a connection is established, QLocalSocket enters ConnectedState
-    and emits connected().
+    If a connection is established, QLocalSocket enters ConnectedState and emits connected().
 
-    At any point, the socket can emit error() to signal that an error occurred.
+    After calling this function, the socket can emit error() to signal that an error occurred.
 
-    See also state(), serverName(), and waitForConnected().
+    \sa state(), serverName(), waitForConnected()
+*/
+
+/*!
+    \fn void QLocalSocket::open(OpenMode openMode)
+
+    Equivalent to connectToServer(OpenMode mode).
+    The socket is opened in the given \a openMode to the server defined by setServerName().
+
+    Note that unlike in most other QIODevice subclasses, open() may not open the device directly.
+    The function return false if the socket was already connected or if the server to connect
+    to was not defined and true in any other case. The connected() or error() signals will be
+    emitted once the device is actualy open (or the connection failed).
+
+    See connectToServer() for more details.
 */
 
 /*!
@@ -99,8 +114,8 @@ QT_BEGIN_NAMESPACE
         LocalSocketState socketState, OpenMode openMode)
 
     Initializes QLocalSocket with the native socket descriptor
-    \a socketDescriptor. Returns true if socketDescriptor is accepted
-    as a valid socket descriptor; otherwise returns false. The socket is
+    \a socketDescriptor. Returns \c true if socketDescriptor is accepted
+    as a valid socket descriptor; otherwise returns \c false. The socket is
     opened in the mode specified by \a openMode, and enters the socket state
     specified by \a socketState.
 
@@ -172,7 +187,7 @@ QT_BEGIN_NAMESPACE
 
     This function writes as much as possible from the internal write buffer
     to the socket, without blocking.  If any data was written, this function
-    returns true; otherwise false is returned.
+    returns \c true; otherwise false is returned.
 
     Call this function if you need QLocalSocket to start sending buffered data
     immediately. The number of bytes successfully written depends on the
@@ -206,8 +221,8 @@ QT_BEGIN_NAMESPACE
 /*!
     \fn bool QLocalSocket::isValid() const
 
-    Returns true if the socket is valid and ready for use; otherwise
-    returns false.
+    Returns \c true if the socket is valid and ready for use; otherwise
+    returns \c false.
 
     \note The socket's state must be ConnectedState before reading
     and writing can occur.
@@ -248,8 +263,8 @@ QT_BEGIN_NAMESPACE
     \fn bool QLocalSocket::waitForConnected(int msecs)
 
     Waits until the socket is connected, up to \a msecs milliseconds. If the
-    connection has been established, this function returns true; otherwise
-    it returns false. In the case where it returns false, you can call
+    connection has been established, this function returns \c true; otherwise
+    it returns \c false. In the case where it returns \c false, you can call
     error() to determine the cause of the error.
 
     The following example waits up to one second for a connection
@@ -267,8 +282,8 @@ QT_BEGIN_NAMESPACE
 
     Waits until the socket has disconnected, up to \a msecs
     milliseconds. If the connection has been disconnected, this
-    function returns true; otherwise it returns false. In the case
-    where it returns false, you can call error() to determine
+    function returns \c true; otherwise it returns \c false. In the case
+    where it returns \c false, you can call error() to determine
     the cause of the error.
 
     The following example waits up to one second for a connection
@@ -289,8 +304,8 @@ QT_BEGIN_NAMESPACE
     will timeout after \a msecs milliseconds; the default timeout is
     30000 milliseconds.
 
-    The function returns true if data is available for reading;
-    otherwise it returns false (if an error occurred or the
+    The function returns \c true if data is available for reading;
+    otherwise it returns \c false (if an error occurred or the
     operation timed out).
 
     \sa waitForBytesWritten()
@@ -353,9 +368,50 @@ QLocalSocket::~QLocalSocket()
 #endif
 }
 
+bool QLocalSocket::open(OpenMode openMode)
+{
+    connectToServer(openMode);
+    return isOpen();
+}
+
+/*! \overload
+
+    Set the server \a name and attempts to make a connection to it.
+
+    The socket is opened in the given \a openMode and first enters ConnectingState.
+    If a connection is established, QLocalSocket enters ConnectedState and emits connected().
+
+    After calling this function, the socket can emit error() to signal that an error occurred.
+
+    \sa state(), serverName(), waitForConnected()
+*/
+void QLocalSocket::connectToServer(const QString &name, OpenMode openMode)
+{
+    setServerName(name);
+    connectToServer(openMode);
+}
+
 /*!
-    Returns the name of the peer as specified by connectToServer(), or an
-    empty QString if connectToServer() has not been called or it failed.
+    \since 5.1
+
+    Set the \a name of the peer to connect to.
+    On Windows name is the name of a named pipe; on Unix name is the name of a local domain socket.
+
+    This function must be called when the socket is not connected.
+*/
+void QLocalSocket::setServerName(const QString & name)
+{
+    Q_D(QLocalSocket);
+    if (d->state != UnconnectedState) {
+        qWarning("QLocalSocket::setServerName() called while not in unconnected state");
+        return;
+    }
+    d->serverName = name;
+}
+
+/*!
+    Returns the name of the peer as specified by setServerName(), or an
+    empty QString if setServerName() has not been called or connectToServer() failed.
 
     \sa connectToServer(), fullServerName()
 
