@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2012 Digia Plc and/or its subsidiary(-ies).
+** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
 ** This file is part of the QtNetwork module of the Qt Toolkit.
@@ -200,8 +200,8 @@ QSslCertificate &QSslCertificate::operator=(const QSslCertificate &other)
 */
 
 /*!
-    Returns true if this certificate is the same as \a other; otherwise
-    returns false.
+    Returns \c true if this certificate is the same as \a other; otherwise
+    returns \c false.
 */
 bool QSslCertificate::operator==(const QSslCertificate &other) const
 {
@@ -217,13 +217,13 @@ bool QSslCertificate::operator==(const QSslCertificate &other) const
 /*!
     \fn bool QSslCertificate::operator!=(const QSslCertificate &other) const
 
-    Returns true if this certificate is not the same as \a other; otherwise
-    returns false.
+    Returns \c true if this certificate is not the same as \a other; otherwise
+    returns \c false.
 */
 
 /*!
-    Returns true if this is a null certificate (i.e., a certificate
-    with no contents); otherwise returns false.
+    Returns \c true if this is a null certificate (i.e., a certificate
+    with no contents); otherwise returns \c false.
 
     By default, QSslCertificate constructs a null certificate.
 
@@ -254,8 +254,8 @@ bool QSslCertificate::isNull() const
 #endif
 
 /*!
-    Returns true if this certificate is blacklisted; otherwise
-    returns false.
+    Returns \c true if this certificate is blacklisted; otherwise
+    returns \c false.
 
     \sa isNull()
 */
@@ -876,8 +876,11 @@ QList<QSslCertificate> QSslCertificate::fromPath(const QString &path,
         // Check if the path is a file.
         if (QFileInfo(sourcePath).isFile()) {
             QFile file(sourcePath);
-            if (file.open(QIODevice::ReadOnly | QIODevice::Text))
-                return QSslCertificate::fromData(file.readAll(),format);
+            QIODevice::OpenMode openMode = QIODevice::ReadOnly;
+            if (format == QSsl::Pem)
+                openMode |= QIODevice::Text;
+            if (file.open(openMode))
+                return QSslCertificate::fromData(file.readAll(), format);
             return QList<QSslCertificate>();
         }
     }
@@ -899,8 +902,11 @@ QList<QSslCertificate> QSslCertificate::fromPath(const QString &path,
             continue;
 
         QFile file(filePath);
-        if (file.open(QIODevice::ReadOnly | QIODevice::Text))
-            certs += QSslCertificate::fromData(file.readAll(),format);
+        QIODevice::OpenMode openMode = QIODevice::ReadOnly;
+        if (format == QSsl::Pem)
+            openMode |= QIODevice::Text;
+        if (file.open(openMode))
+            certs += QSslCertificate::fromData(file.readAll(), format);
     }
     return certs;
 }
@@ -1208,6 +1214,8 @@ static const char *certificate_blacklist[] = {
 
     "08:27",                                           "*.EGO.GOV.TR", // Turktrust mis-issued intermediate certificate
     "08:64",                                           "e-islem.kktcmerkezbankasi.org", // Turktrust mis-issued intermediate certificate
+
+    "03:1d:a7",                                        "AC DG Tr\xC3\xA9sor SSL", // intermediate certificate linking back to ANSSI French National Security Agency
     0
 };
 
