@@ -47,8 +47,6 @@
 #include <QtCore/qscopedpointer.h>
 #include <QtNetwork/qssl.h>
 
-QT_BEGIN_HEADER
-
 QT_BEGIN_NAMESPACE
 
 
@@ -97,7 +95,5 @@ Q_NETWORK_EXPORT QDebug operator<<(QDebug debug, const QSslCipher &cipher);
 
 QT_END_NAMESPACE
 
-QT_END_HEADER
-
 #endif
 
