@@ -46,8 +46,6 @@
 #include <QtCore/qvariant.h>
 #include <QtNetwork/qsslcertificate.h>
 
-QT_BEGIN_HEADER
-
 QT_BEGIN_NAMESPACE
 
 
@@ -106,7 +104,7 @@ public:
     SslError error() const;
     QString errorString() const;
     QSslCertificate certificate() const;
-    
+
 private:
     QScopedPointer<QSslErrorPrivate> d;
 };
@@ -126,6 +124,4 @@ QT_END_NAMESPACE
 Q_DECLARE_METATYPE(QList<QSslError>)
 #endif
 
-QT_END_HEADER
-
 #endif
