@@ -69,6 +69,7 @@
 //
 
 #include "qsslsocket_openssl_p.h"
+#include <QtCore/qglobal.h>
 
 QT_BEGIN_NAMESPACE
 
@@ -82,8 +83,8 @@ QT_BEGIN_NAMESPACE
     typedef ret (*_q_PTR_##func)(arg);					\
     static _q_PTR_##func _q_##func = 0;					\
     ret q_##func(arg) {						\
-        if (!_q_##func) {				\
-            qWarning("QSslSocket: cannot call unresolved function "#func);	\
+        if (Q_UNLIKELY(!_q_##func)) { \
+            qsslSocketUnresolvedSymbolWarning(#func); \
             err;								\
         } \
         funcret _q_##func(a); \
@@ -94,8 +95,8 @@ QT_BEGIN_NAMESPACE
     typedef ret (*_q_PTR_##func)(arg1, arg2);         \
     static _q_PTR_##func _q_##func = 0;               \
     ret q_##func(arg1, arg2) { \
-        if (!_q_##func) { \
-            qWarning("QSslSocket: cannot call unresolved function "#func);\
+        if (Q_UNLIKELY(!_q_##func)) { \
+            qsslSocketUnresolvedSymbolWarning(#func);\
             err; \
         } \
         funcret _q_##func(a, b); \
@@ -106,8 +107,8 @@ QT_BEGIN_NAMESPACE
     typedef ret (*_q_PTR_##func)(arg1, arg2, arg3);            \
     static _q_PTR_##func _q_##func = 0;                        \
     ret q_##func(arg1, arg2, arg3) { \
-        if (!_q_##func) { \
-            qWarning("QSslSocket: cannot call unresolved function "#func); \
+        if (Q_UNLIKELY(!_q_##func)) { \
+            qsslSocketUnresolvedSymbolWarning(#func); \
             err; \
         } \
         funcret _q_##func(a, b, c); \
@@ -118,8 +119,8 @@ QT_BEGIN_NAMESPACE
     typedef ret (*_q_PTR_##func)(arg1, arg2, arg3, arg4);               \
     static _q_PTR_##func _q_##func = 0;                                 \
     ret q_##func(arg1, arg2, arg3, arg4) { \
-         if (!_q_##func) { \
-             qWarning("QSslSocket: cannot call unresolved function "#func); \
+         if (Q_UNLIKELY(!_q_##func)) { \
+             qsslSocketUnresolvedSymbolWarning(#func); \
              err; \
          } \
          funcret _q_##func(a, b, c, d); \
@@ -130,8 +131,8 @@ QT_BEGIN_NAMESPACE
     typedef ret (*_q_PTR_##func)(arg1, arg2, arg3, arg4, arg5);         \
     static _q_PTR_##func _q_##func = 0;                                 \
     ret q_##func(arg1, arg2, arg3, arg4, arg5) { \
-        if (!_q_##func) { \
-            qWarning("QSslSocket: cannot call unresolved function "#func); \
+        if (Q_UNLIKELY(!_q_##func)) { \
+            qsslSocketUnresolvedSymbolWarning(#func); \
             err; \
         } \
         funcret _q_##func(a, b, c, d, e); \
@@ -142,8 +143,8 @@ QT_BEGIN_NAMESPACE
     typedef ret (*_q_PTR_##func)(arg1, arg2, arg3, arg4, arg5, arg6);   \
     static _q_PTR_##func _q_##func = 0;                                 \
     ret q_##func(arg1, arg2, arg3, arg4, arg5, arg6) { \
-        if (!_q_##func) { \
-            qWarning("QSslSocket: cannot call unresolved function "#func); \
+        if (Q_UNLIKELY(!_q_##func)) { \
+            qsslSocketUnresolvedSymbolWarning(#func); \
             err; \
         } \
         funcret _q_##func(a, b, c, d, e, f); \
@@ -154,8 +155,8 @@ QT_BEGIN_NAMESPACE
     typedef ret (*_q_PTR_##func)(arg1, arg2, arg3, arg4, arg5, arg6, arg7);   \
     static _q_PTR_##func _q_##func = 0;                                       \
     ret q_##func(arg1, arg2, arg3, arg4, arg5, arg6, arg7) { \
-        if (!_q_##func) { \
-            qWarning("QSslSocket: cannot call unresolved function "#func); \
+        if (Q_UNLIKELY(!_q_##func)) { \
+            qsslSocketUnresolvedSymbolWarning(#func); \
             err; \
         } \
         funcret _q_##func(a, b, c, d, e, f, g); \
@@ -166,8 +167,8 @@ QT_BEGIN_NAMESPACE
     typedef ret (*_q_PTR_##func)(arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9);   \
     static _q_PTR_##func _q_##func = 0;                                                   \
     ret q_##func(arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9) { \
-        if (_q_##func) { \
-            qWarning("QSslSocket: cannot call unresolved function "#func); \
+        if (Q_UNLIKELY(!_q_##func)) { \
+            qsslSocketUnresolvedSymbolWarning(#func); \
             err; \
         }   \
         funcret _q_##func(a, b, c, d, e, f, g, h, i); \
@@ -285,7 +286,7 @@ void q_sk_free(_STACK *a);
 void * q_sk_value(STACK *a, int b);
 #else
 STACK *q_sk_new_null();
-void q_sk_push(STACK *st, void *data);
+void q_sk_push(STACK *st, char *data);
 void q_sk_free(STACK *a);
 char * q_sk_value(STACK *a, int b);
 #endif
@@ -347,6 +348,10 @@ void q_SSL_set_bio(SSL *a, BIO *b, BIO *c);
 void q_SSL_set_accept_state(SSL *a);
 void q_SSL_set_connect_state(SSL *a);
 int q_SSL_shutdown(SSL *a);
+int q_SSL_set_session(SSL *to, SSL_SESSION *session);
+void q_SSL_SESSION_free(SSL_SESSION *ses);
+SSL_SESSION *q_SSL_get1_session(SSL *ssl);
+SSL_SESSION *q_SSL_get_session(const SSL *ssl);
 #if OPENSSL_VERSION_NUMBER >= 0x10000000L
 const SSL_METHOD *q_SSLv2_client_method();
 const SSL_METHOD *q_SSLv3_client_method();
@@ -395,7 +400,11 @@ int q_X509_EXTENSION_get_critical(X509_EXTENSION *a);
 ASN1_OCTET_STRING *q_X509_EXTENSION_get_data(X509_EXTENSION *a);
 void q_BASIC_CONSTRAINTS_free(BASIC_CONSTRAINTS *a);
 void q_AUTHORITY_KEYID_free(AUTHORITY_KEYID *a);
+#if OPENSSL_VERSION_NUMBER >= 0x10000000L
 int q_ASN1_STRING_print(BIO *a, const ASN1_STRING *b);
+#else
+int q_ASN1_STRING_print(BIO *a, ASN1_STRING *b);
+#endif
 X509_NAME *q_X509_get_issuer_name(X509 *a);
 X509_NAME *q_X509_get_subject_name(X509 *a);
 int q_X509_verify_cert(X509_STORE_CTX *ctx);
@@ -461,6 +470,8 @@ void q_OPENSSL_add_all_algorithms_conf();
 int q_SSL_CTX_load_verify_locations(SSL_CTX *ctx, const char *CAfile, const char *CApath);
 long q_SSLeay();
 const char *q_SSLeay_version(int type);
+int q_i2d_SSL_SESSION(SSL_SESSION *in, unsigned char **pp);
+SSL_SESSION *q_d2i_SSL_SESSION(SSL_SESSION **a, const unsigned char **pp, long length);
 
 // Helper function
 class QDateTime;
