@@ -68,8 +68,6 @@
 #  include <syslog.h>
 #endif
 
-QT_BEGIN_HEADER
-
 /*
     This cache stores internal Qt shader programs in shared memory.
 
@@ -450,7 +448,5 @@ QT_END_NAMESPACE
 
 #endif
 
-QT_END_HEADER
-
 #endif
 #endif
