@@ -52,8 +52,6 @@
 #include <QtOpenGL/qgl.h>
 #include <QtGui/qopenglcontext.h>
 
-QT_BEGIN_HEADER
-
 QT_BEGIN_NAMESPACE
 
 
@@ -2298,6 +2296,4 @@ inline void QGLFunctions::glVertexAttribPointer(GLuint indx, GLint size, GLenum
 
 QT_END_NAMESPACE
 
-QT_END_HEADER
-
 #endif
