@@ -49,23 +49,46 @@
 
 static inline void bindApi(const QSurfaceFormat &format)
 {
-    if (format.renderableType() == QSurfaceFormat::OpenVG)
+    switch (format.renderableType()) {
+    case QSurfaceFormat::OpenVG:
         eglBindAPI(EGL_OPENVG_API);
+        break;
 #ifdef EGL_VERSION_1_4
-    else if (format.renderableType() == QSurfaceFormat::OpenGL)
+#  if !defined(QT_OPENGL_ES_2)
+    case QSurfaceFormat::DefaultRenderableType:
+#  endif
+    case QSurfaceFormat::OpenGL:
         eglBindAPI(EGL_OPENGL_API);
+        break;
 #endif
-    else
+    case QSurfaceFormat::OpenGLES:
+    default:
         eglBindAPI(EGL_OPENGL_ES_API);
+        break;
+    }
 }
 
 QEGLPlatformContext::QEGLPlatformContext(const QSurfaceFormat &format, QPlatformOpenGLContext *share, EGLDisplay display,
                                          EGLenum eglApi)
     : m_eglDisplay(display)
-    , m_eglApi(eglApi)
-    , m_eglConfig(q_configFromGLFormat(display, format, true))
-    , m_format(q_glFormatFromConfig(display, m_eglConfig))
+    , m_eglConfig(q_configFromGLFormat(display, format))
 {
+    init(format, share);
+    Q_UNUSED(eglApi);
+}
+
+QEGLPlatformContext::QEGLPlatformContext(const QSurfaceFormat &format, QPlatformOpenGLContext *share, EGLDisplay display,
+                                         EGLConfig config, EGLenum eglApi)
+    : m_eglDisplay(display)
+    , m_eglConfig(config)
+{
+    init(format, share);
+    Q_UNUSED(eglApi);
+}
+
+void QEGLPlatformContext::init(const QSurfaceFormat &format, QPlatformOpenGLContext *share)
+{
+    m_format = q_glFormatFromConfig(m_eglDisplay, m_eglConfig);
     m_shareContext = share ? static_cast<QEGLPlatformContext *>(share)->m_eglContext : 0;
 
     QVector<EGLint> contextAttrs;
@@ -92,6 +115,14 @@ bool QEGLPlatformContext::makeCurrent(QPlatformSurface *surface)
 
     EGLSurface eglSurface = eglSurfaceForPlatformSurface(surface);
 
+    // shortcut: on some GPUs, eglMakeCurrent is not a cheap operation
+    if (eglGetCurrentContext() == m_eglContext &&
+        eglGetCurrentDisplay() == m_eglDisplay &&
+        eglGetCurrentSurface(EGL_READ) == eglSurface &&
+        eglGetCurrentSurface(EGL_DRAW) == eglSurface) {
+        return true;
+    }
+
     bool ok = eglMakeCurrent(m_eglDisplay, eglSurface, eglSurface, m_eglContext);
     if (!ok)
         qWarning("QEGLPlatformContext::makeCurrent: eglError: %x, this: %p \n", eglGetError(), this);
