@@ -51,7 +51,10 @@ QT_BEGIN_NAMESPACE
 QFbBackingStore::QFbBackingStore(QWindow *window)
     : QPlatformBackingStore(window)
 {
-    (static_cast<QFbWindow *>(window->handle()))->setBackingStore(this);
+    if (window->handle())
+        (static_cast<QFbWindow *>(window->handle()))->setBackingStore(this);
+    else
+        (static_cast<QFbScreen *>(window->screen()->handle()))->addBackingStore(this);
 }
 
 QFbBackingStore::~QFbBackingStore()
@@ -74,14 +77,5 @@ void QFbBackingStore::resize(const QSize &size, const QRegion &staticContents)
         mImage = QImage(size, window()->screen()->handle()->format());
 }
 
-void QFbBackingStore::beginPaint(const QRegion &region)
-{
-    Q_UNUSED(region);
-}
-
-void QFbBackingStore::endPaint()
-{
-}
-
 QT_END_NAMESPACE
 
