@@ -120,7 +120,7 @@ void QFbCursor::setCursor(const uchar *data, const uchar *mask, int width, int h
 void QFbCursor::changeCursor(QCursor * widgetCursor, QWindow *window)
 {
     Q_UNUSED(window);
-    Qt::CursorShape shape = widgetCursor->shape();
+    const Qt::CursorShape shape = widgetCursor ? widgetCursor->shape() : Qt::ArrowCursor;
 
     if (shape == Qt::BitmapCursor) {
         // application supplied cursor
@@ -137,5 +137,12 @@ void QFbCursor::changeCursor(QCursor * widgetCursor, QWindow *window)
 }
 #endif
 
-QT_END_NAMESPACE
+void QFbCursor::setDirty()
+{
+    if (!mDirty) {
+        mDirty = true;
+        mScreen->scheduleUpdate();
+    }
+}
 
+QT_END_NAMESPACE
