@@ -45,12 +45,14 @@
 #include <qpa/qplatformscreen.h>
 #include <QtCore/QTimer>
 #include <QtCore/QSize>
+#include "qfbcursor_p.h"
 
 QT_BEGIN_NAMESPACE
 
 class QFbWindow;
 class QFbCursor;
 class QPainter;
+class QFbBackingStore;
 
 class QFbScreen : public QObject, public QPlatformScreen
 {
@@ -59,19 +61,30 @@ public:
     QFbScreen();
     ~QFbScreen();
 
-    virtual QRect geometry() const { return mGeometry; }
-    virtual int depth() const { return mDepth; }
-    virtual QImage::Format format() const { return mFormat; }
-    virtual QSizeF physicalSize() const { return mPhysicalSize; }
+    QRect geometry() const Q_DECL_OVERRIDE { return mGeometry; }
+    int depth() const Q_DECL_OVERRIDE { return mDepth; }
+    QImage::Format format() const Q_DECL_OVERRIDE { return mFormat; }
+    QSizeF physicalSize() const Q_DECL_OVERRIDE { return mPhysicalSize; }
+    QPlatformCursor *cursor() const Q_DECL_OVERRIDE { return mCursor; }
 
-    virtual QWindow *topLevelAt(const QPoint & p) const;
+    QWindow *topWindow() const;
+    QWindow *topLevelAt(const QPoint & p) const Q_DECL_OVERRIDE;
 
     // compositor api
     virtual void addWindow(QFbWindow *window);
     virtual void removeWindow(QFbWindow *window);
     virtual void raise(QFbWindow *window);
     virtual void lower(QFbWindow *window);
+    virtual void topWindowChanged(QWindow *) {}
+
+    void addBackingStore(QFbBackingStore *bs) {mBackingStores << bs;}
+
+    void scheduleUpdate();
+
+public slots:
     virtual void setDirty(const QRect &rect);
+    void setPhysicalSize(const QSize &size);
+    void setGeometry(const QRect &rect);
 
 protected slots:
     virtual QRegion doRedraw();
@@ -96,6 +109,7 @@ private:
 
     QPainter *mCompositePainter;
     QList<QPair<QRect, int> > mCachedRects;
+    QList <QFbBackingStore*> mBackingStores;
 
     friend class QFbWindow;
     bool mIsUpToDate;
@@ -104,4 +118,3 @@ private:
 QT_END_NAMESPACE
 
 #endif // QFBSCREEN_P_H
-
