@@ -43,21 +43,19 @@
 #include "qfbscreen_p.h"
 
 #include <QtGui/QScreen>
+#include <qpa/qwindowsysteminterface.h>
 
 QT_BEGIN_NAMESPACE
 
 QFbWindow::QFbWindow(QWindow *window)
-    : QPlatformWindow(window), mBackingStore(0)
+    : QPlatformWindow(window), mBackingStore(0), mWindowState(Qt::WindowNoState)
 {
     static QAtomicInt winIdGenerator(1);
     mWindowId = winIdGenerator.fetchAndAddRelaxed(1);
-
-    platformScreen()->addWindow(this);
 }
 
 QFbWindow::~QFbWindow()
 {
-    platformScreen()->removeWindow(this);
 }
 
 QFbScreen *QFbWindow::platformScreen() const
@@ -71,11 +69,36 @@ void QFbWindow::setGeometry(const QRect &rect)
     mOldGeometry = geometry();
 
     platformScreen()->invalidateRectCache();
-    //### QWindowSystemInterface::handleGeometryChange(window(), rect);
+    QWindowSystemInterface::handleGeometryChange(window(), rect);
 
     QPlatformWindow::setGeometry(rect);
 }
 
+void QFbWindow::setVisible(bool visible)
+{
+    if (visible) {
+        if (mWindowState & Qt::WindowFullScreen)
+            setGeometry(platformScreen()->geometry());
+        else if (mWindowState & Qt::WindowMaximized)
+            setGeometry(platformScreen()->availableGeometry());
+    }
+    QPlatformWindow::setVisible(visible);
+
+    if (visible)
+        platformScreen()->addWindow(this);
+    else
+        platformScreen()->removeWindow(this);
+}
+
+
+void QFbWindow::setWindowState(Qt::WindowState state)
+{
+    QPlatformWindow::setWindowState(state);
+    mWindowState = state;
+    platformScreen()->invalidateRectCache();
+}
+
+
 void QFbWindow::setWindowFlags(Qt::WindowFlags flags)
 {
     mWindowFlags = flags;
