@@ -43,31 +43,24 @@
 #include "qfontenginemultifontconfig_p.h"
 
 #include <QtCore/QList>
-#include <QtGui/private/qfont_p.h>
-
 #include <QtCore/QElapsedTimer>
 
+#include <qpa/qplatformnativeinterface.h>
 #include <qpa/qplatformscreen.h>
+#include <qpa/qplatformintegration.h>
+#include <qpa/qplatformservices.h>
 
 #include <QtGui/private/qfontengine_ft_p.h>
-#include <QtGui/private/qfontengine_p.h>
 #include <QtGui/private/qfontengine_qpa_p.h>
+#include <QtGui/private/qguiapplication_p.h>
 
-#include <ft2build.h>
-#include FT_TRUETYPE_TABLES_H
+#include <QtGui/qguiapplication.h>
 
 #include <fontconfig/fontconfig.h>
-#include FT_FREETYPE_H
-
 #if FC_VERSION >= 20402
 #include <fontconfig/fcfreetype.h>
 #endif
 
-#define SimplifiedChineseCsbBit 18
-#define TraditionalChineseCsbBit 20
-#define JapaneseCsbBit 17
-#define KoreanCsbBit 21
-
 QT_BEGIN_NAMESPACE
 
 static inline bool requiresOpenType(int writingSystem)
@@ -75,11 +68,6 @@ static inline bool requiresOpenType(int writingSystem)
     return ((writingSystem >= QFontDatabase::Syriac && writingSystem <= QFontDatabase::Sinhala)
             || writingSystem == QFontDatabase::Khmer || writingSystem == QFontDatabase::Nko);
 }
-static inline bool scriptRequiresOpenType(int script)
-{
-    return ((script >= QUnicodeTables::Syriac && script <= QUnicodeTables::Sinhala)
-            || script == QUnicodeTables::Khmer || script == QUnicodeTables::Nko);
-}
 
 static int getFCWeight(int fc_weight)
 {
@@ -97,14 +85,17 @@ static int getFCWeight(int fc_weight)
 }
 
 static const char *specialLanguages[] = {
-    "en", // Common
+    "", // Unknown
+    "", // Inherited
+    "", // Common
+    "en", // Latin
     "el", // Greek
     "ru", // Cyrillic
     "hy", // Armenian
     "he", // Hebrew
     "ar", // Arabic
     "syr", // Syriac
-    "div", // Thaana
+    "dv", // Thaana
     "hi", // Devanagari
     "bn", // Bengali
     "pa", // Gurmukhi
@@ -121,44 +112,84 @@ static const char *specialLanguages[] = {
     "my", // Myanmar
     "ka", // Georgian
     "ko", // Hangul
-    "", // Ogham
-    "", // Runic
+    "am", // Ethiopic
+    "chr", // Cherokee
+    "cr", // CanadianAboriginal
+    "sga", // Ogham
+    "non", // Runic
     "km", // Khmer
-    "" // N'Ko
+    "mn", // Mongolian
+    "ja", // Hiragana
+    "ja", // Katakana
+    "zh-TW", // Bopomofo
+    "", // Han
+    "ii", // Yi
+    "ett", // OldItalic
+    "got", // Gothic
+    "en", // Deseret
+    "fil", // Tagalog
+    "hnn", // Hanunoo
+    "bku", // Buhid
+    "tbw", // Tagbanwa
+    "cop", // Coptic
+    "lif", // Limbu
+    "tdd", // TaiLe
+    "grc", // LinearB
+    "uga", // Ugaritic
+    "en", // Shavian
+    "so", // Osmanya
+    "grc", // Cypriot
+    "", // Braille
+    "bug", // Buginese
+    "khb", // NewTaiLue
+    "cu", // Glagolitic
+    "shi", // Tifinagh
+    "syl", // SylotiNagri
+    "peo", // OldPersian
+    "pra", // Kharoshthi
+    "ban", // Balinese
+    "akk", // Cuneiform
+    "phn", // Phoenician
+    "lzh", // PhagsPa
+    "man", // Nko
+    "su", // Sundanese
+    "lep", // Lepcha
+    "sat", // OlChiki
+    "vai", // Vai
+    "saz", // Saurashtra
+    "eky", // KayahLi
+    "rej", // Rejang
+    "xlc", // Lycian
+    "xcr", // Carian
+    "xld", // Lydian
+    "cjm", // Cham
+    "nod", // TaiTham
+    "blt", // TaiViet
+    "ae", // Avestan
+    "egy", // EgyptianHieroglyphs
+    "smp", // Samaritan
+    "lis", // Lisu
+    "bax", // Bamum
+    "jv", // Javanese
+    "mni", // MeeteiMayek
+    "arc", // ImperialAramaic
+    "xsa", // OldSouthArabian
+    "xpr", // InscriptionalParthian
+    "pal", // InscriptionalPahlavi
+    "otk", // OldTurkic
+    "bh", // Kaithi
+    "bbc", // Batak
+    "pra", // Brahmi
+    "myz", // Mandaic
+    "ccp", // Chakma
+    "xmr", // MeroiticCursive
+    "xmr", // MeroiticHieroglyphs
+    "hmd", // Miao
+    "sa", // Sharada
+    "srb", // SoraSompeng
+    "doi"  // Takri
 };
-enum { SpecialLanguageCount = sizeof(specialLanguages) / sizeof(const char *) };
-
-static const ushort specialChars[] = {
-    0, // English
-    0, // Greek
-    0, // Cyrillic
-    0, // Armenian
-    0, // Hebrew
-    0, // Arabic
-    0, // Syriac
-    0, // Thaana
-    0, // Devanagari
-    0, // Bengali
-    0, // Gurmukhi
-    0, // Gujarati
-    0, // Oriya
-    0, // Tamil
-    0xc15, // Telugu
-    0xc95, // Kannada
-    0xd15, // Malayalam
-    0xd9a, // Sinhala
-    0, // Thai
-    0, // Lao
-    0, // Tibetan
-    0x1000, // Myanmar
-    0, // Georgian
-    0, // Hangul
-    0x1681, // Ogham
-    0x16a0, // Runic
-    0,  // Khmer
-    0x7ca // N'Ko
-};
-enum { SpecialCharCount = sizeof(specialChars) / sizeof(ushort) };
+Q_STATIC_ASSERT(sizeof(specialLanguages) / sizeof(const char *) == QChar::ScriptCount);
 
 // this could become a list of all languages used for each writing
 // system, instead of using the single most common language.
@@ -194,52 +225,13 @@ static const char *languageForWritingSystem[] = {
     "ko",  // Korean
     "vi",  // Vietnamese
     0, // Symbol
-    0, // Ogham
-    0, // Runic
-    0 // N'Ko
+    "sga", // Ogham
+    "non", // Runic
+    "man" // N'Ko
 };
-enum { LanguageCount = sizeof(languageForWritingSystem) / sizeof(const char *) };
-
-// Unfortunately FontConfig doesn't know about some languages. We have to test these through the
-// charset. The lists below contain the systems where we need to do this.
-static const ushort sampleCharForWritingSystem[] = {
-    0,     // Any
-    0,  // Latin
-    0,  // Greek
-    0,  // Cyrillic
-    0,  // Armenian
-    0,  // Hebrew
-    0,  // Arabic
-    0, // Syriac
-    0, // Thaana
-    0,  // Devanagari
-    0,  // Bengali
-    0,  // Gurmukhi
-    0,  // Gujarati
-    0,  // Oriya
-    0,  // Tamil
-    0xc15,  // Telugu
-    0xc95,  // Kannada
-    0xd15,  // Malayalam
-    0xd9a,  // Sinhala
-    0,  // Thai
-    0,  // Lao
-    0,  // Tibetan
-    0x1000,  // Myanmar
-    0,  // Georgian
-    0,  // Khmer
-    0, // SimplifiedChinese
-    0, // TraditionalChinese
-    0,  // Japanese
-    0,  // Korean
-    0,  // Vietnamese
-    0, // Symbol
-    0x1681, // Ogham
-    0x16a0, // Runic
-    0x7ca // N'Ko
-};
-enum { SampleCharCount = sizeof(sampleCharForWritingSystem) / sizeof(ushort) };
+Q_STATIC_ASSERT(sizeof(languageForWritingSystem) / sizeof(const char *) == QFontDatabase::WritingSystemsCount);
 
+#if FC_VERSION >= 20297
 // Newer FontConfig let's us sort out fonts that contain certain glyphs, but no
 // open type tables for is directly. Do this so we don't pick some strange
 // pseudo unicode font
@@ -279,6 +271,8 @@ static const char *openType[] = {
     0, // Runic
     "nko " // N'Ko
 };
+Q_STATIC_ASSERT(sizeof(openType) / sizeof(const char *) == QFontDatabase::WritingSystemsCount);
+#endif
 
 static const char *getFcFamilyForStyleHint(const QFont::StyleHint style)
 {
@@ -381,14 +375,20 @@ void QFontconfigDatabase::populateFontDatabase()
         FcLangSet *langset = 0;
         FcResult res = FcPatternGetLangSet(fonts->fonts[i], FC_LANG, 0, &langset);
         if (res == FcResultMatch) {
-            for (int i = 1; i < LanguageCount; ++i) {
-                const FcChar8 *lang = (const FcChar8*) languageForWritingSystem[i];
+            bool hasLang = false;
+            for (int j = 1; j < QFontDatabase::WritingSystemsCount; ++j) {
+                const FcChar8 *lang = (const FcChar8*) languageForWritingSystem[j];
                 if (lang) {
                     FcLangResult langRes = FcLangSetHasLang(langset, lang);
-                    if (langRes != FcLangDifferentLang)
-                        writingSystems.setSupported(QFontDatabase::WritingSystem(i));
+                    if (langRes != FcLangDifferentLang) {
+                        writingSystems.setSupported(QFontDatabase::WritingSystem(j));
+                        hasLang = true;
+                    }
                 }
             }
+            if (!hasLang)
+                // none of our known languages, add it to the other set
+                writingSystems.setSupported(QFontDatabase::Other);
         } else {
             // we set Other to supported for symbol fonts. It makes no
             // sense to merge these with other ones, as they are
@@ -396,21 +396,8 @@ void QFontconfigDatabase::populateFontDatabase()
             writingSystems.setSupported(QFontDatabase::Other);
         }
 
-        FcCharSet *cs = 0;
-        res = FcPatternGetCharSet(fonts->fonts[i], FC_CHARSET, 0, &cs);
-        if (res == FcResultMatch) {
-            // some languages are not supported by FontConfig, we rather check the
-            // charset to detect these
-            for (int i = 1; i < SampleCharCount; ++i) {
-                if (!sampleCharForWritingSystem[i])
-                    continue;
-                if (FcCharSetHasChar(cs, sampleCharForWritingSystem[i]))
-                    writingSystems.setSupported(QFontDatabase::WritingSystem(i));
-            }
-        }
-
 #if FC_VERSION >= 20297
-        for (int j = 1; j < LanguageCount; ++j) {
+        for (int j = 1; j < QFontDatabase::WritingSystemsCount; ++j) {
             if (writingSystems.supported(QFontDatabase::WritingSystem(j))
                 && requiresOpenType(j) && openType[j]) {
                 FcChar8 *cap;
@@ -444,6 +431,9 @@ void QFontconfigDatabase::populateFontDatabase()
         QString styleName = style_value ? QString::fromUtf8((const char *) style_value) : QString();
         QPlatformFontDatabase::registerFont(familyName,styleName,QLatin1String((const char *)foundry_value),weight,style,stretch,antialias,scalable,pixel_size,fixedPitch,writingSystems,fontFile);
 //        qDebug() << familyName << (const char *)foundry_value << weight << style << &writingSystems << scalable << true << pixel_size;
+
+        for (int k = 1; FcPatternGetString(fonts->fonts[i], FC_FAMILY, k, &value) == FcResultMatch; ++k)
+            QPlatformFontDatabase::registerAliasToFontFamily(familyName, QString::fromUtf8((const char *)value));
     }
 
     FcFontSetDestroy (fonts);
@@ -481,13 +471,12 @@ void QFontconfigDatabase::populateFontDatabase()
 //    QApplication::setFont(font);
 }
 
-QFontEngineMulti *QFontconfigDatabase::fontEngineMulti(QFontEngine *fontEngine,
-                                                       QUnicodeTables::Script script)
+QFontEngineMulti *QFontconfigDatabase::fontEngineMulti(QFontEngine *fontEngine, QChar::Script script)
 {
     return new QFontEngineMultiFontConfig(fontEngine, script);
 }
 
-QFontEngine *QFontconfigDatabase::fontEngine(const QFontDef &f, QUnicodeTables::Script script, void *usrPtr)
+QFontEngine *QFontconfigDatabase::fontEngine(const QFontDef &f, QChar::Script script, void *usrPtr)
 {
     if (!usrPtr)
         return 0;
@@ -496,7 +485,7 @@ QFontEngine *QFontconfigDatabase::fontEngine(const QFontDef &f, QUnicodeTables::
     QFontEngineFT *engine;
     FontFile *fontfile = static_cast<FontFile *> (usrPtr);
     QFontEngine::FaceId fid;
-    fid.filename = fontfile->fileName.toLocal8Bit();
+    fid.filename = QFile::encodeName(fontfile->fileName);
     fid.index = fontfile->indexValue;
 
     bool antialias = !(fontDef.styleStrategy & QFont::NoAntialias);
@@ -555,6 +544,27 @@ QFontEngine *QFontconfigDatabase::fontEngine(const QFontDef &f, QUnicodeTables::
                 break;
             }
         }
+
+        if (antialias) {
+            // If antialiasing is not fully disabled, fontconfig may still disable it on a font match basis.
+            FcBool fc_antialias;
+            if (FcPatternGetBool(match, FC_ANTIALIAS,0, &fc_antialias) != FcResultMatch)
+                fc_antialias = true;
+            antialias = fc_antialias;
+        }
+
+        if (f.hintingPreference == QFont::PreferDefaultHinting) {
+            const QPlatformServices *services = QGuiApplicationPrivate::platformIntegration()->services();
+            if (services && (services->desktopEnvironment() == "GNOME" || services->desktopEnvironment() == "UNITY")) {
+                void *hintStyleResource =
+                        QGuiApplication::platformNativeInterface()->nativeResourceForScreen("hintstyle",
+                                                                                            QGuiApplication::primaryScreen());
+                int hintStyle = int(reinterpret_cast<qintptr>(hintStyleResource));
+                if (hintStyle > 0)
+                    default_hint_style = QFontEngine::HintStyle(hintStyle - 1);
+            }
+        }
+
         engine->setDefaultHintStyle(default_hint_style);
 
         if (antialias) {
@@ -594,18 +604,16 @@ QFontEngine *QFontconfigDatabase::fontEngine(const QFontDef &f, QUnicodeTables::
     if (engine->invalid()) {
         delete engine;
         engine = 0;
-    } else if (scriptRequiresOpenType(script)) {
-        HB_Face hbFace = engine->initializedHarfbuzzFace();
-        if (!hbFace || !hbFace->supported_scripts[script]) {
-            delete engine;
-            engine = 0;
-        }
+    } else if (!engine->supportsScript(script)) {
+        qWarning("  OpenType support missing for script %d", int(script));
+        delete engine;
+        engine = 0;
     }
 
     return engine;
 }
 
-QStringList QFontconfigDatabase::fallbacksForFamily(const QString family, const QFont::Style &style, const QFont::StyleHint &styleHint, const QUnicodeTables::Script &script) const
+QStringList QFontconfigDatabase::fallbacksForFamily(const QString &family, QFont::Style style, QFont::StyleHint styleHint, QChar::Script script) const
 {
     QStringList fallbackFamilies;
     FcPattern *pattern = FcPatternCreate();
@@ -625,18 +633,18 @@ QStringList QFontconfigDatabase::fallbacksForFamily(const QString family, const
         slant_value = FC_SLANT_OBLIQUE;
     FcPatternAddInteger(pattern, FC_SLANT, slant_value);
 
-    if (script != QUnicodeTables::Common && *specialLanguages[script] != '\0') {
-        Q_ASSERT(script < QUnicodeTables::ScriptCount);
+    Q_ASSERT(uint(script) < QChar::ScriptCount);
+    if (*specialLanguages[script] != '\0') {
         FcLangSet *ls = FcLangSetCreate();
         FcLangSetAdd(ls, (const FcChar8*)specialLanguages[script]);
         FcPatternAddLangSet(pattern, FC_LANG, ls);
         FcLangSetDestroy(ls);
     } else if (!family.isEmpty()) {
-        // If script is common then it may include languages like CJK,
+        // If script is Common or Han, then it may include languages like CJK,
         // we should attach system default language set to the pattern
         // to obtain correct font fallback list (i.e. if LANG=zh_CN
         // then we normally want to use a Chinese font for CJK text;
-        // while a Japanese font should be use for that if LANG=ja)
+        // while a Japanese font should be used for that if LANG=ja)
         FcPattern *dummy = FcPatternCreate();
         FcDefaultSubstitute(dummy);
         FcChar8 *lang = 0;
@@ -747,6 +755,9 @@ QStringList QFontconfigDatabase::addApplicationFont(const QByteArray &fontData,
 
 QString QFontconfigDatabase::resolveFontFamilyAlias(const QString &family) const
 {
+    QString resolved = QBasicFontDatabase::resolveFontFamilyAlias(family);
+    if (!resolved.isEmpty() && resolved != family)
+        return resolved;
     FcPattern *pattern = FcPatternCreate();
     if (!pattern)
         return family;
@@ -760,7 +771,7 @@ QString QFontconfigDatabase::resolveFontFamilyAlias(const QString &family) const
 
     FcChar8 *familyAfterSubstitution = 0;
     FcPatternGetString(pattern, FC_FAMILY, 0, &familyAfterSubstitution);
-    QString resolved = QString::fromUtf8((const char *) familyAfterSubstitution);
+    resolved = QString::fromUtf8((const char *) familyAfterSubstitution);
     FcPatternDestroy(pattern);
 
     return resolved;
