@@ -51,9 +51,9 @@ class QFontconfigDatabase : public QBasicFontDatabase
 {
 public:
     void populateFontDatabase();
-    QFontEngineMulti *fontEngineMulti(QFontEngine *fontEngine, QUnicodeTables::Script script);
-    QFontEngine *fontEngine(const QFontDef &fontDef, QUnicodeTables::Script script, void *handle);
-    QStringList fallbacksForFamily(const QString family, const QFont::Style &style, const QFont::StyleHint &styleHint, const QUnicodeTables::Script &script) const;
+    QFontEngineMulti *fontEngineMulti(QFontEngine *fontEngine, QChar::Script script);
+    QFontEngine *fontEngine(const QFontDef &fontDef, QChar::Script script, void *handle);
+    QStringList fallbacksForFamily(const QString &family, QFont::Style style, QFont::StyleHint styleHint, QChar::Script script) const;
     QStringList addApplicationFont(const QByteArray &fontData, const QString &fileName);
     QString resolveFontFamilyAlias(const QString &family) const;
     QFont defaultFont() const;
