@@ -39,7 +39,9 @@
 **
 ****************************************************************************/
 
-#ifndef Q_OS_IOS
+#include "qglobal.h"
+
+#ifdef Q_OS_MACX
 #import <Cocoa/Cocoa.h>
 #import <IOKit/graphics/IOGraphicsLib.h>
 #endif
@@ -85,28 +87,12 @@ static const char *languageForWritingSystem[] = {
     "ko",  // Korean
     "vi",  // Vietnamese
     0, // Symbol
-    0, // Ogham
-    0, // Runic
-    0 // N'Ko
+    "sga", // Ogham
+    "non", // Runic
+    "man" // N'Ko
 };
 enum { LanguageCount = sizeof(languageForWritingSystem) / sizeof(const char *) };
 
-QFont::StyleHint styleHintFromNSString(NSString *style)
-{
-    if ([style isEqual: @"sans-serif"])
-        return QFont::SansSerif;
-    else if ([style isEqual: @"monospace"])
-        return QFont::Monospace;
-    else if ([style isEqual: @"cursive"])
-        return QFont::Cursive;
-    else if ([style isEqual: @"serif"])
-        return QFont::Serif;
-    else if ([style isEqual: @"fantasy"])
-        return QFont::Fantasy;
-    else // if ([style isEqual: @"default"])
-        return QFont::AnyStyle;
-}
-
 static NSInteger languageMapSort(id obj1, id obj2, void *context)
 {
     NSArray *map1 = (NSArray *) obj1;
@@ -121,7 +107,7 @@ static NSInteger languageMapSort(id obj1, id obj2, void *context)
 
 QCoreTextFontDatabase::QCoreTextFontDatabase()
 {
-#ifndef Q_OS_IOS
+#ifdef Q_OS_MACX
     QSettings appleSettings(QLatin1String("apple.com"));
     QVariant appleValue = appleSettings.value(QLatin1String("AppleAntiAliasingThreshold"));
     if (appleValue.isValid())
@@ -188,29 +174,12 @@ QCoreTextFontDatabase::~QCoreTextFontDatabase()
 {
 }
 
-static QString familyNameFromPostScriptName(QHash<QString, QString> &psNameToFamily,
-                                            NSString *psName)
-{
-    QString name = QCFString::toQString(psName);
-    if (psNameToFamily.contains(name))
-        return psNameToFamily[name];
-    else {
-        // Some of the font name in DefaultFontFallbacks.plist are hidden fonts like AquaHiraKaku,
-        // their family name begins with a dot, like ".AquaHiraKaku" or ".Apple Symbols Fallback",
-        // the only way (I've found) to get it are actually creating a CTFont with them. We only
-        // need to do it once though.
-        QCFType<CTFontRef> font = CTFontCreateWithName((CFStringRef) psName, 12.0, NULL);
-        if (font) {
-            QCFString family = CTFontCopyFamilyName(font);
-            psNameToFamily[name] = family;
-            return family;
-        }
-        return name;
-    }
-}
-
 void QCoreTextFontDatabase::populateFontDatabase()
 {
+    // The caller (QFontDB) expects the db to be populate only with system fonts, so we need
+    // to make sure that any previously registered app fonts become invisible.
+    removeApplicationFonts();
+
     NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
 
     QCFType<CTFontCollectionRef> collection = CTFontCollectionCreateFromAvailableFonts(0);
@@ -221,112 +190,95 @@ void QCoreTextFontDatabase::populateFontDatabase()
     if (! fonts)
         return;
 
-    QString foundryName = QLatin1String("CoreText");
     const int numFonts = CFArrayGetCount(fonts);
-    QHash<QString, QString> psNameToFamily;
     for (int i = 0; i < numFonts; ++i) {
         CTFontDescriptorRef font = (CTFontDescriptorRef) CFArrayGetValueAtIndex(fonts, i);
-        QCFString familyName = (CFStringRef) CTFontDescriptorCopyLocalizedAttribute(font, kCTFontFamilyNameAttribute, NULL);
-        QCFString styleName = (CFStringRef)CTFontDescriptorCopyLocalizedAttribute(font, kCTFontStyleNameAttribute, NULL);
-        QCFType<CFDictionaryRef> styles = (CFDictionaryRef) CTFontDescriptorCopyAttribute(font, kCTFontTraitsAttribute);
-        QFont::Weight weight = QFont::Normal;
-        QFont::Style style = QFont::StyleNormal;
-        QFont::Stretch stretch = QFont::Unstretched;
-        bool fixedPitch = false;
-
-        if (styles) {
-            if (CFNumberRef weightValue = (CFNumberRef) CFDictionaryGetValue(styles, kCTFontWeightTrait)) {
-                Q_ASSERT(CFNumberIsFloatType(weightValue));
-                double d;
-                if (CFNumberGetValue(weightValue, kCFNumberDoubleType, &d))
-                    weight = (d > 0.0) ? QFont::Bold : QFont::Normal;
-            }
-            if (CFNumberRef italic = (CFNumberRef) CFDictionaryGetValue(styles, kCTFontSlantTrait)) {
-                Q_ASSERT(CFNumberIsFloatType(italic));
-                double d;
-                if (CFNumberGetValue(italic, kCFNumberDoubleType, &d)) {
-                    if (d > 0.0)
-                        style = QFont::StyleItalic;
-                }
-            }
-            if (CFNumberRef symbolic = (CFNumberRef) CFDictionaryGetValue(styles, kCTFontSymbolicTrait)) {
-                int d;
-                if (CFNumberGetValue(symbolic, kCFNumberSInt32Type, &d)) {
-                    if (d & kCTFontMonoSpaceTrait)
-                        fixedPitch = true;
-                    if (d & kCTFontExpandedTrait)
-                        stretch = QFont::Expanded;
-                    else if (d & kCTFontCondensedTrait)
-                        stretch = QFont::Condensed;
-                }
-            }
-        }
-
-        int pixelSize = 0;
-        if (QCFType<CFNumberRef> size = (CFNumberRef) CTFontDescriptorCopyAttribute(font, kCTFontSizeAttribute)) {
-            if (CFNumberIsFloatType(size)) {
-                double d;
-                CFNumberGetValue(size, kCFNumberDoubleType, &d);
-                pixelSize = d;
-            } else {
-                CFNumberGetValue(size, kCFNumberIntType, &pixelSize);
-            }
-        }
-
-        QSupportedWritingSystems writingSystems;
-        if (QCFType<CFArrayRef> languages = (CFArrayRef) CTFontDescriptorCopyAttribute(font, kCTFontLanguagesAttribute)) {
-            CFIndex length = CFArrayGetCount(languages);
-            for (int i = 1; i < LanguageCount; ++i) {
-                if (!languageForWritingSystem[i])
-                    continue;
-                QCFString lang = CFStringCreateWithCString(NULL, languageForWritingSystem[i], kCFStringEncodingASCII);
-                if (CFArrayContainsValue(languages, CFRangeMake(0, length), lang))
-                    writingSystems.setSupported(QFontDatabase::WritingSystem(i));
-            }
-        }
-
-        CFRetain(font);
-        QPlatformFontDatabase::registerFont(familyName, styleName, foundryName, weight, style, stretch,
-                                            true /* antialiased */, true /* scalable */,
-                                            pixelSize, fixedPitch, writingSystems, (void *) font);
-        CFStringRef psName = (CFStringRef) CTFontDescriptorCopyAttribute(font, kCTFontNameAttribute);
-        // we need PostScript Name to family name mapping for fallback list construction
-        psNameToFamily[QCFString::toQString((NSString *) psName)] = familyName;
-        CFRelease(psName);
-    }
-
-    NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
-    NSArray *languages = [defaults stringArrayForKey: @"AppleLanguages"];
-
-    NSDictionary *fallbackDict = [NSDictionary dictionaryWithContentsOfFile: @"/System/Library/Frameworks/ApplicationServices.framework/Frameworks/CoreText.framework/Resources/DefaultFontFallbacks.plist"];
-
-    for (NSString *style in [fallbackDict allKeys]) {
-        NSArray *list = [fallbackDict valueForKey: style];
-        QFont::StyleHint styleHint = styleHintFromNSString(style);
-        QStringList fallbackList;
-        for (id item in list) {
-            // sort the array based on system language preferences
-            if ([item isKindOfClass: [NSArray class]]) {
-                NSArray *langs = [(NSArray *) item sortedArrayUsingFunction: languageMapSort
-                                                                    context: languages];
-                for (NSArray *map in langs)
-                    fallbackList.append(familyNameFromPostScriptName(psNameToFamily, [map objectAtIndex: 1]));
-            }
-            else if ([item isKindOfClass: [NSString class]])
-                fallbackList.append(familyNameFromPostScriptName(psNameToFamily, item));
-        }
-        fallbackLists[styleHint] = fallbackList;
+        populateFromDescriptor(font);
     }
 
     [pool release];
 }
 
+void QCoreTextFontDatabase::populateFromDescriptor(CTFontDescriptorRef font)
+{
+    QString foundryName = QStringLiteral("CoreText");
+    QCFString familyName = (CFStringRef) CTFontDescriptorCopyLocalizedAttribute(font, kCTFontFamilyNameAttribute, NULL);
+    QCFString styleName = (CFStringRef)CTFontDescriptorCopyLocalizedAttribute(font, kCTFontStyleNameAttribute, NULL);
+    QCFType<CFDictionaryRef> styles = (CFDictionaryRef) CTFontDescriptorCopyAttribute(font, kCTFontTraitsAttribute);
+    QFont::Weight weight = QFont::Normal;
+    QFont::Style style = QFont::StyleNormal;
+    QFont::Stretch stretch = QFont::Unstretched;
+    bool fixedPitch = false;
+
+    if (styles) {
+        if (CFNumberRef weightValue = (CFNumberRef) CFDictionaryGetValue(styles, kCTFontWeightTrait)) {
+            Q_ASSERT(CFNumberIsFloatType(weightValue));
+            double d;
+            if (CFNumberGetValue(weightValue, kCFNumberDoubleType, &d))
+                weight = (d > 0.0) ? QFont::Bold : QFont::Normal;
+        }
+        if (CFNumberRef italic = (CFNumberRef) CFDictionaryGetValue(styles, kCTFontSlantTrait)) {
+            Q_ASSERT(CFNumberIsFloatType(italic));
+            double d;
+            if (CFNumberGetValue(italic, kCFNumberDoubleType, &d)) {
+                if (d > 0.0)
+                    style = QFont::StyleItalic;
+            }
+        }
+        if (CFNumberRef symbolic = (CFNumberRef) CFDictionaryGetValue(styles, kCTFontSymbolicTrait)) {
+            int d;
+            if (CFNumberGetValue(symbolic, kCFNumberSInt32Type, &d)) {
+                if (d & kCTFontMonoSpaceTrait)
+                    fixedPitch = true;
+                if (d & kCTFontExpandedTrait)
+                    stretch = QFont::Expanded;
+                else if (d & kCTFontCondensedTrait)
+                    stretch = QFont::Condensed;
+            }
+        }
+    }
+
+    int pixelSize = 0;
+    if (QCFType<CFNumberRef> size = (CFNumberRef) CTFontDescriptorCopyAttribute(font, kCTFontSizeAttribute)) {
+        if (CFNumberIsFloatType(size)) {
+            double d;
+            CFNumberGetValue(size, kCFNumberDoubleType, &d);
+            pixelSize = d;
+        } else {
+            CFNumberGetValue(size, kCFNumberIntType, &pixelSize);
+        }
+    }
+
+    QSupportedWritingSystems writingSystems;
+    if (QCFType<CFArrayRef> languages = (CFArrayRef) CTFontDescriptorCopyAttribute(font, kCTFontLanguagesAttribute)) {
+        CFIndex length = CFArrayGetCount(languages);
+        for (int i = 1; i < LanguageCount; ++i) {
+            if (!languageForWritingSystem[i])
+                continue;
+            QCFString lang = CFStringCreateWithCString(NULL, languageForWritingSystem[i], kCFStringEncodingASCII);
+            if (CFArrayContainsValue(languages, CFRangeMake(0, length), lang))
+                writingSystems.setSupported(QFontDatabase::WritingSystem(i));
+        }
+    }
+
+    CFRetain(font);
+    QPlatformFontDatabase::registerFont(familyName, styleName, foundryName, weight, style, stretch,
+            true /* antialiased */, true /* scalable */,
+            pixelSize, fixedPitch, writingSystems, (void *) font);
+
+    // We need to map back and forth between PostScript-names and family-names for fallback list construction
+    CFStringRef psName = (CFStringRef) CTFontDescriptorCopyAttribute(font, kCTFontNameAttribute);
+    psNameToFamily[QCFString::toQString((NSString *) psName)] = familyName;
+    familyNameToPsName[familyName] = QCFString::toQString((NSString *) psName);
+    CFRelease(psName);
+}
+
 void QCoreTextFontDatabase::releaseHandle(void *handle)
 {
     CFRelease(CTFontDescriptorRef(handle));
 }
 
-QFontEngine *QCoreTextFontDatabase::fontEngine(const QFontDef &f, QUnicodeTables::Script script, void *usrPtr)
+QFontEngine *QCoreTextFontDatabase::fontEngine(const QFontDef &f, QChar::Script script, void *usrPtr)
 {
     Q_UNUSED(script);
 
@@ -353,12 +305,20 @@ QFontEngine *QCoreTextFontDatabase::fontEngine(const QFontDef &f, QUnicodeTables
     return NULL;
 }
 
+static void releaseFontData(void* info, const void* data, size_t size)
+{
+    Q_UNUSED(data);
+    Q_UNUSED(size);
+    delete (QByteArray*)info;
+}
+
 QFontEngine *QCoreTextFontDatabase::fontEngine(const QByteArray &fontData, qreal pixelSize, QFont::HintingPreference hintingPreference)
 {
     Q_UNUSED(hintingPreference);
 
-    QCFType<CGDataProviderRef> dataProvider = CGDataProviderCreateWithData(NULL,
-            fontData.constData(), fontData.size(), NULL);
+    QByteArray* fontDataCopy = new QByteArray(fontData);
+    QCFType<CGDataProviderRef> dataProvider = CGDataProviderCreateWithData(fontDataCopy,
+            fontDataCopy->constData(), fontDataCopy->size(), releaseFontData);
 
     CGFontRef cgFont = CGFontCreateWithDataProvider(dataProvider);
 
@@ -376,17 +336,145 @@ QFontEngine *QCoreTextFontDatabase::fontEngine(const QByteArray &fontData, qreal
     return fontEngine;
 }
 
-QStringList QCoreTextFontDatabase::fallbacksForFamily(const QString family, const QFont::Style &style, const QFont::StyleHint &styleHint, const QUnicodeTables::Script &script) const
+QFont::StyleHint styleHintFromNSString(NSString *style)
 {
-    Q_UNUSED(family);
-    Q_UNUSED(style);
-    Q_UNUSED(script);
-    if (fallbackLists.isEmpty())
-        const_cast<QCoreTextFontDatabase*>(this)->populateFontDatabase();
-    return fallbackLists[styleHint];
+    if ([style isEqual: @"sans-serif"])
+        return QFont::SansSerif;
+    else if ([style isEqual: @"monospace"])
+        return QFont::Monospace;
+    else if ([style isEqual: @"cursive"])
+        return QFont::Cursive;
+    else if ([style isEqual: @"serif"])
+        return QFont::Serif;
+    else if ([style isEqual: @"fantasy"])
+        return QFont::Fantasy;
+    else // if ([style isEqual: @"default"])
+        return QFont::AnyStyle;
 }
 
-#ifndef Q_OS_IOS
+static QString familyNameFromPostScriptName(QHash<QString, QString> &psNameToFamily,
+                                            NSString *psName)
+{
+    QString name = QCFString::toQString(psName);
+    if (psNameToFamily.contains(name))
+        return psNameToFamily[name];
+    else {
+        // Some of the font name in DefaultFontFallbacks.plist are hidden fonts like AquaHiraKaku,
+        // their family name begins with a dot, like ".AquaHiraKaku" or ".Apple Symbols Fallback",
+        // the only way (I've found) to get it are actually creating a CTFont with them. We only
+        // need to do it once though.
+        QCFType<CTFontRef> font = CTFontCreateWithName((CFStringRef) psName, 12.0, NULL);
+        if (font) {
+            QCFString family = CTFontCopyFamilyName(font);
+            psNameToFamily[name] = family;
+            return family;
+        }
+        return name;
+    }
+}
+
+QStringList QCoreTextFontDatabase::fallbacksForFamily(const QString &family, QFont::Style style, QFont::StyleHint styleHint, QChar::Script script) const
+{
+    Q_UNUSED(style);
+    Q_UNUSED(script);
+
+    static QHash<QString, QStringList> fallbackLists;
+
+    if (!family.isEmpty()) {
+#if QT_MAC_PLATFORM_SDK_EQUAL_OR_ABOVE(__MAC_10_8, __IPHONE_6_0)
+      // CTFontCopyDefaultCascadeListForLanguages is available in the SDK
+  #if QT_MAC_DEPLOYMENT_TARGET_BELOW(__MAC_10_8, __IPHONE_6_0)
+        // But we have to feature check at runtime
+        if (&CTFontCopyDefaultCascadeListForLanguages)
+  #endif
+        {
+            if (fallbackLists.contains(family))
+                return fallbackLists.value(family);
+
+            if (!familyNameToPsName.contains(family))
+                const_cast<QCoreTextFontDatabase*>(this)->populateFontDatabase();
+
+            QCFType<CTFontRef> font = CTFontCreateWithName(QCFString(familyNameToPsName[family]), 12.0, NULL);
+            if (font) {
+                NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
+                NSArray *languages = [defaults stringArrayForKey: @"AppleLanguages"];
+
+                QCFType<CFArrayRef> cascadeList = (CFArrayRef) CTFontCopyDefaultCascadeListForLanguages(font, (CFArrayRef) languages);
+                if (cascadeList) {
+                    QStringList fallbackList;
+                    const int numCascades = CFArrayGetCount(cascadeList);
+                    for (int i = 0; i < numCascades; ++i) {
+                        CTFontDescriptorRef fontFallback = (CTFontDescriptorRef) CFArrayGetValueAtIndex(cascadeList, i);
+                        QCFString fallbackFamilyName = (CFStringRef) CTFontDescriptorCopyLocalizedAttribute(fontFallback, kCTFontFamilyNameAttribute, NULL);
+                        fallbackList.append(QCFString::toQString(fallbackFamilyName));
+                    }
+                    fallbackLists[family] = fallbackList;
+                }
+            }
+
+            if (fallbackLists.contains(family))
+                return fallbackLists.value(family);
+        }
+#endif
+    }
+
+    // We were not able to find a fallback for the specific family,
+    // so we fall back to the stylehint.
+
+    static const QString styleLookupKey = QString::fromLatin1(".QFontStyleHint_%1");
+
+    static bool didPopulateStyleFallbacks = false;
+    if (!didPopulateStyleFallbacks) {
+#if defined(Q_OS_MACX)
+        // Ensure we have the psNameToFamily mapping set up
+        const_cast<QCoreTextFontDatabase*>(this)->populateFontDatabase();
+
+        NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
+        NSArray *languages = [defaults stringArrayForKey: @"AppleLanguages"];
+
+        NSDictionary *fallbackDict = [NSDictionary dictionaryWithContentsOfFile: @"/System/Library/Frameworks/ApplicationServices.framework/Frameworks/CoreText.framework/Resources/DefaultFontFallbacks.plist"];
+
+        for (NSString *style in [fallbackDict allKeys]) {
+            NSArray *list = [fallbackDict valueForKey: style];
+            QFont::StyleHint fallbackStyleHint = styleHintFromNSString(style);
+            QStringList fallbackList;
+            for (id item in list) {
+                // sort the array based on system language preferences
+                if ([item isKindOfClass: [NSArray class]]) {
+                    NSArray *langs = [(NSArray *) item sortedArrayUsingFunction: languageMapSort
+                                                                        context: languages];
+                    for (NSArray *map in langs)
+                        fallbackList.append(familyNameFromPostScriptName(psNameToFamily, [map objectAtIndex: 1]));
+                }
+                else if ([item isKindOfClass: [NSString class]])
+                    fallbackList.append(familyNameFromPostScriptName(psNameToFamily, item));
+            }
+
+            if (QCoreTextFontEngine::supportsColorGlyphs())
+                fallbackList.append(QLatin1String("Apple Color Emoji"));
+
+            fallbackLists[styleLookupKey.arg(fallbackStyleHint)] = fallbackList;
+        }
+#else
+        QStringList staticFallbackList;
+        staticFallbackList << QString::fromLatin1("Helvetica,Apple Color Emoji,Geeza Pro,Arial Hebrew,Thonburi,Kailasa"
+            "Hiragino Kaku Gothic ProN,.Heiti J,Apple SD Gothic Neo,.Heiti K,Heiti SC,Heiti TC"
+            "Bangla Sangam MN,Devanagari Sangam MN,Gujarati Sangam MN,Gurmukhi MN,Kannada Sangam MN"
+            "Malayalam Sangam MN,Oriya Sangam MN,Sinhala Sangam MN,Tamil Sangam MN,Telugu Sangam MN"
+            "Euphemia UCAS,.PhoneFallback").split(QLatin1String(","));
+
+        for (int i = QFont::Helvetica; i <= QFont::Fantasy; ++i)
+            fallbackLists[styleLookupKey.arg(i)] = staticFallbackList;
+#endif
+
+        didPopulateStyleFallbacks = true;
+    }
+
+    Q_ASSERT(!fallbackLists.isEmpty());
+    return fallbackLists[styleLookupKey.arg(styleHint)];
+}
+
+#ifdef Q_OS_MACX
 QStringList QCoreTextFontDatabase::addApplicationFont(const QByteArray &fontData, const QString &fileName)
 {
 #if MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_8
@@ -394,14 +482,16 @@ QStringList QCoreTextFontDatabase::addApplicationFont(const QByteArray &fontData
         CTFontRef font = NULL;
 
         if (!fontData.isEmpty()) {
-            QCFType<CGDataProviderRef> dataProvider = CGDataProviderCreateWithData(NULL,
-                fontData.constData(), fontData.size(), NULL);
+            QByteArray* fontDataCopy = new QByteArray(fontData);
+            QCFType<CGDataProviderRef> dataProvider = CGDataProviderCreateWithData(fontDataCopy,
+                    fontDataCopy->constData(), fontDataCopy->size(), releaseFontData);
             CGFontRef cgFont = CGFontCreateWithDataProvider(dataProvider);
             if (cgFont) {
                 CFErrorRef error;
                 bool success = CTFontManagerRegisterGraphicsFont(cgFont, &error);
                 if (success) {
                     font = CTFontCreateWithGraphicsFont(cgFont, 0.0, NULL, NULL);
+                    m_applicationGraphicsFonts.append(QCFType<CGFontRef>::constructFromGet(cgFont));
                 } else {
                     NSLog(@"Unable to register font: %@", error);
                     CFRelease(error);
@@ -413,12 +503,13 @@ QStringList QCoreTextFontDatabase::addApplicationFont(const QByteArray &fontData
             QCFType<CFURLRef> fontURL = CFURLCreateWithFileSystemPath(NULL, QCFString(fileName), kCFURLPOSIXPathStyle, false);
             bool success = CTFontManagerRegisterFontsForURL(fontURL, kCTFontManagerScopeProcess, &error);
             if (success) {
-                const void *keys[] = { fontURL };
-                const void *values[] = { kCTFontURLAttribute };
+                const void *keys[] = { kCTFontURLAttribute };
+                const void *values[] = { fontURL };
                 QCFType<CFDictionaryRef> attributes = CFDictionaryCreate(NULL, keys, values, 1,
                     &kCFTypeDictionaryKeyCallBacks, &kCFTypeDictionaryValueCallBacks);
                 QCFType<CTFontDescriptorRef> descriptor = CTFontDescriptorCreateWithAttributes(attributes);
                 font = CTFontCreateWithFontDescriptor(descriptor, 0.0, NULL);
+                m_applicationURLFonts.append(QCFType<CFURLRef>::constructFromGet(fontURL));
             } else {
                 NSLog(@"Unable to register font: %@", error);
                 CFRelease(error);
@@ -428,11 +519,16 @@ QStringList QCoreTextFontDatabase::addApplicationFont(const QByteArray &fontData
         if (font) {
             QStringList families;
             families.append(QCFString(CTFontCopyFamilyName(font)));
+
+            QCFType<CTFontDescriptorRef> descriptor = CTFontCopyFontDescriptor(font);
+            populateFromDescriptor(descriptor);
+
             CFRelease(font);
             return families;
         }
-    } else {
-#else
+    } else
+#endif
+    {
     ATSFontContainerRef fontContainer;
     OSStatus e;
 
@@ -463,15 +559,15 @@ QStringList QCoreTextFontDatabase::addApplicationFont(const QByteArray &fontData
         QStringList families;
         for (int i = 0; i < containedFonts.size(); ++i) {
             QCFType<CTFontRef> font = CTFontCreateWithPlatformFont(containedFonts[i], 12.0, NULL, NULL);
+            QCFType<CTFontDescriptorRef> descriptor = CTFontCopyFontDescriptor(font);
+            populateFromDescriptor(descriptor);
             families.append(QCFString(CTFontCopyFamilyName(font)));
         }
 
+        m_applicationFonts.append(fontContainer);
         return families;
     }
-#endif
-#if MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_8
     }
-#endif
 
     return QStringList();
 }
@@ -498,5 +594,29 @@ QList<int> QCoreTextFontDatabase::standardSizes() const
     return ret;
 }
 
+void QCoreTextFontDatabase::removeApplicationFonts()
+{
+#ifdef Q_OS_MACX
+#if MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_8
+    if (QSysInfo::QSysInfo::MacintoshVersion >= QSysInfo::MV_10_8) {
+        CFErrorRef error;
+        for (int i = 0; i < m_applicationGraphicsFonts.count(); ++i)
+            CTFontManagerUnregisterGraphicsFont(m_applicationGraphicsFonts[i], &error);
+        m_applicationGraphicsFonts.clear();
+
+        for (int i = 0; i < m_applicationURLFonts.count(); ++i)
+            CTFontManagerUnregisterFontsForURL(m_applicationURLFonts[i], kCTFontManagerScopeProcess, &error);
+        m_applicationURLFonts.clear();
+    } else
+#endif
+    {
+    for (int i = 0; i < m_applicationFonts.count(); ++i)
+        ATSFontDeactivate(m_applicationFonts[i], 0, kATSOptionFlagsDoNotNotify);
+    m_applicationFonts.clear();
+    ATSFontNotify(kATSFontNotifyActionFontsChanged, 0);
+    }
+#endif
+}
+
 QT_END_NAMESPACE
 
