@@ -43,6 +43,14 @@
 #define QCORETEXTFONTDATABASE_H
 
 #include <qpa/qplatformfontdatabase.h>
+#include <private/qcore_mac_p.h>
+
+#ifndef Q_OS_IOS
+#include <ApplicationServices/ApplicationServices.h>
+#else
+#include <CoreText/CoreText.h>
+#include <CoreGraphics/CoreGraphics.h>
+#endif
 
 QT_BEGIN_NAMESPACE
 
@@ -52,10 +60,10 @@ public:
     QCoreTextFontDatabase();
     ~QCoreTextFontDatabase();
     void populateFontDatabase();
-    QFontEngine *fontEngine(const QFontDef &fontDef, QUnicodeTables::Script script, void *handle);
+    QFontEngine *fontEngine(const QFontDef &fontDef, QChar::Script script, void *handle);
     QFontEngine *fontEngine(const QByteArray &fontData, qreal pixelSize, QFont::HintingPreference hintingPreference);
-    QStringList fallbacksForFamily(const QString family, const QFont::Style &style, const QFont::StyleHint &styleHint, const QUnicodeTables::Script &script) const;
-#ifndef Q_OS_IOS
+    QStringList fallbacksForFamily(const QString &family, QFont::Style style, QFont::StyleHint styleHint, QChar::Script script) const;
+#ifdef Q_OS_MACX
     QStringList addApplicationFont(const QByteArray &fontData, const QString &fileName);
 #endif
     void releaseHandle(void *handle);
@@ -63,8 +71,20 @@ public:
     QList<int> standardSizes() const;
 
 private:
+    void populateFromDescriptor(CTFontDescriptorRef font);
+
     mutable QString defaultFontName;
-    QHash<QFont::StyleHint, QStringList> fallbackLists;
+    mutable QHash<QString, QString> psNameToFamily;
+    mutable QHash<QString, QString> familyNameToPsName;
+
+    void removeApplicationFonts();
+#ifdef Q_OS_MACX
+#if MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_8
+    QVector<QCFType<CGFontRef> > m_applicationGraphicsFonts;
+    QVector<QCFType<CFURLRef> > m_applicationURLFonts;
+#endif
+    QVector<ATSFontContainerRef> m_applicationFonts;
+#endif
 };
 
 QT_END_NAMESPACE
