@@ -57,7 +57,13 @@ QEvdevKeyboardManager::QEvdevKeyboardManager(const QString &key, const QString &
 {
     Q_UNUSED(key);
 
-    QStringList args = specification.split(QLatin1Char(':'));
+
+    QString spec = QString::fromLocal8Bit(qgetenv("QT_QPA_EVDEV_KEYBOARD_PARAMETERS"));
+
+    if (spec.isEmpty())
+        spec = specification;
+
+    QStringList args = spec.split(QLatin1Char(':'));
     QStringList devices;
 
     foreach (const QString &arg, args) {
