@@ -50,8 +50,6 @@
 #include <QHash>
 #include <QSocketNotifier>
 
-QT_BEGIN_HEADER
-
 QT_BEGIN_NAMESPACE
 
 class QEvdevKeyboardManager : public QObject
@@ -71,8 +69,6 @@ private:
     QDeviceDiscovery *m_deviceDiscovery;
 };
 
-QT_END_HEADER
-
 QT_END_NAMESPACE
 
 #endif // QEVDEVKEYBOARDMANAGER_P_H
