@@ -49,7 +49,7 @@
 #include <QtPlatformSupport/private/qdevicediscovery_p.h>
 #include <linux/input.h>
 
-#ifdef USE_MTDEV
+#if !defined(QT_NO_MTDEV)
 extern "C" {
 #include <mtdev.h>
 }
@@ -114,6 +114,7 @@ public:
     int findClosestContact(const QHash<int, Contact> &contacts, int x, int y, int *dist);
     void reportPoints();
     void registerDevice();
+    void addTouchPoint(const Contact &contact, Qt::TouchPointStates *combinedStates);
 
     int hw_range_x_min;
     int hw_range_x_max;
@@ -125,6 +126,8 @@ public:
     bool m_forceToActiveWindow;
     QTouchDevice *m_device;
     bool m_typeB;
+    QTransform m_rotate;
+    bool m_singleTouch;
 };
 
 QEvdevTouchScreenData::QEvdevTouchScreenData(QEvdevTouchScreenHandler *q_ptr, const QStringList &args)
@@ -134,7 +137,7 @@ QEvdevTouchScreenData::QEvdevTouchScreenData(QEvdevTouchScreenHandler *q_ptr, co
       hw_range_x_min(0), hw_range_x_max(0),
       hw_range_y_min(0), hw_range_y_max(0),
       hw_pressure_min(0), hw_pressure_max(0),
-      m_device(0), m_typeB(false)
+      m_device(0), m_typeB(false), m_singleTouch(false)
 {
     m_forceToActiveWindow = args.contains(QLatin1String("force_window"));
 }
@@ -159,22 +162,43 @@ static inline bool testBit(long bit, const long *array)
     return (array[bit / LONG_BITS] >> bit % LONG_BITS) & 1;
 }
 
-QEvdevTouchScreenHandler::QEvdevTouchScreenHandler(const QString &spec, QObject *parent)
+QEvdevTouchScreenHandler::QEvdevTouchScreenHandler(const QString &specification, QObject *parent)
     : QObject(parent), m_notify(0), m_fd(-1), d(0)
-#ifdef USE_MTDEV
+#if !defined(QT_NO_MTDEV)
       , m_mtdev(0)
 #endif
 {
     setObjectName(QLatin1String("Evdev Touch Handler"));
 
-    QString dev;
+    bool printDeviceInfo = qgetenv("QT_QPA_EVDEV_DEBUG").toInt();
 
     // only the first device argument is used for now
+    QString spec = QString::fromLocal8Bit(qgetenv("QT_QPA_EVDEV_TOUCHSCREEN_PARAMETERS"));
+
+    if (spec.isEmpty())
+        spec = specification;
+
     QStringList args = spec.split(QLatin1Char(':'));
+
+    QString dev;
+    int rotationAngle = 0;
     for (int i = 0; i < args.count(); ++i) {
-        if (args.at(i).startsWith(QLatin1String("/dev/"))) {
+        if (args.at(i).startsWith(QLatin1String("/dev/")) && dev.isEmpty()) {
             dev = args.at(i);
-            break;
+        } else if (args.at(i).startsWith(QLatin1String("rotate"))) {
+            QString rotateArg = args.at(i).section(QLatin1Char('='), 1, 1);
+            bool ok;
+            uint argValue = rotateArg.toUInt(&ok);
+            if (ok) {
+                switch (argValue) {
+                case 90:
+                case 180:
+                case 270:
+                    rotationAngle = argValue;
+                default:
+                    break;
+                }
+            }
         }
     }
 
@@ -190,25 +214,30 @@ QEvdevTouchScreenHandler::QEvdevTouchScreenHandler(const QString &spec, QObject
         }
     }
 
-    if (dev.isEmpty())
+    if (dev.isEmpty()) {
+        if (printDeviceInfo)
+            qDebug("evdevtouch: No touch devices found");
         return;
+    }
+
+    if (printDeviceInfo)
+        qDebug("evdevtouch: Using device %s", qPrintable(dev));
 
-    qDebug("evdevtouch: Using device %s", qPrintable(dev));
     m_fd = QT_OPEN(dev.toLocal8Bit().constData(), O_RDONLY | O_NDELAY, 0);
 
     if (m_fd >= 0) {
         m_notify = new QSocketNotifier(m_fd, QSocketNotifier::Read, this);
         connect(m_notify, SIGNAL(activated(int)), this, SLOT(readData()));
     } else {
-        qErrnoWarning(errno, "Cannot open input device %s", qPrintable(dev));
+        qErrnoWarning(errno, "evdevtouch: Cannot open input device %s", qPrintable(dev));
         return;
     }
 
-#ifdef USE_MTDEV
+#if !defined(QT_NO_MTDEV)
     m_mtdev = static_cast<mtdev *>(calloc(1, sizeof(mtdev)));
     int mtdeverr = mtdev_open(m_mtdev, m_fd);
     if (mtdeverr) {
-        qWarning("mtdev_open failed: %d", mtdeverr);
+        qWarning("evdevtouch: mtdev_open failed: %d", mtdeverr);
         QT_CLOSE(m_fd);
         return;
     }
@@ -216,55 +245,91 @@ QEvdevTouchScreenHandler::QEvdevTouchScreenHandler(const QString &spec, QObject
 
     d = new QEvdevTouchScreenData(this, args);
 
+#if !defined(QT_NO_MTDEV)
+    const char *mtdevStr = "(mtdev)";
+    d->m_typeB = true;
+#else
+    const char *mtdevStr = "";
+    long absbits[NUM_LONGS(ABS_CNT)];
+    if (ioctl(m_fd, EVIOCGBIT(EV_ABS, sizeof(absbits)), absbits) >= 0) {
+        d->m_typeB = testBit(ABS_MT_SLOT, absbits);
+        d->m_singleTouch = !testBit(ABS_MT_POSITION_X, absbits);
+    }
+#endif
+
+    if (printDeviceInfo)
+        qDebug("evdevtouch: Protocol type %c %s (%s)", d->m_typeB ? 'B' : 'A',
+               mtdevStr, d->m_singleTouch ? "single" : "multi");
+
     input_absinfo absInfo;
     memset(&absInfo, 0, sizeof(input_absinfo));
-    if (ioctl(m_fd, EVIOCGABS(ABS_MT_POSITION_X), &absInfo) >= 0) {
-        qDebug("min X: %d max X: %d", absInfo.minimum, absInfo.maximum);
+    bool has_x_range = false, has_y_range = false;
+
+    if (ioctl(m_fd, EVIOCGABS((d->m_singleTouch ? ABS_X : ABS_MT_POSITION_X)), &absInfo) >= 0) {
+        if (printDeviceInfo)
+            qDebug("evdevtouch: min X: %d max X: %d", absInfo.minimum, absInfo.maximum);
         d->hw_range_x_min = absInfo.minimum;
         d->hw_range_x_max = absInfo.maximum;
+        has_x_range = true;
     }
-    if (ioctl(m_fd, EVIOCGABS(ABS_MT_POSITION_Y), &absInfo) >= 0) {
-        qDebug("min Y: %d max Y: %d", absInfo.minimum, absInfo.maximum);
+
+    if (ioctl(m_fd, EVIOCGABS((d->m_singleTouch ? ABS_Y : ABS_MT_POSITION_Y)), &absInfo) >= 0) {
+        if (printDeviceInfo)
+            qDebug("evdevtouch: min Y: %d max Y: %d", absInfo.minimum, absInfo.maximum);
         d->hw_range_y_min = absInfo.minimum;
         d->hw_range_y_max = absInfo.maximum;
+        has_y_range = true;
     }
+
+    if (!has_x_range || !has_y_range)
+        qWarning("evdevtouch: Invalid ABS limits, behavior unspecified");
+
     if (ioctl(m_fd, EVIOCGABS(ABS_PRESSURE), &absInfo) >= 0) {
-        qDebug("min pressure: %d max pressure: %d", absInfo.minimum, absInfo.maximum);
+        if (printDeviceInfo)
+            qDebug("evdevtouch: min pressure: %d max pressure: %d", absInfo.minimum, absInfo.maximum);
         if (absInfo.maximum > absInfo.minimum) {
             d->hw_pressure_min = absInfo.minimum;
             d->hw_pressure_max = absInfo.maximum;
         }
     }
+
     char name[1024];
     if (ioctl(m_fd, EVIOCGNAME(sizeof(name) - 1), name) >= 0) {
         d->hw_name = QString::fromLocal8Bit(name);
-        qDebug("device name: %s", name);
+        if (printDeviceInfo)
+            qDebug("evdevtouch: device name: %s", name);
+    }
+
+    // Fix up the coordinate ranges for am335x in case the kernel driver does not have them fixed.
+    if (d->hw_name == QLatin1String("ti-tsc")) {
+        if (d->hw_range_x_min == 0 && d->hw_range_x_max == 4095) {
+            d->hw_range_x_min = 165;
+            d->hw_range_x_max = 4016;
+        }
+        if (d->hw_range_y_min == 0 && d->hw_range_y_max == 4095) {
+            d->hw_range_y_min = 220;
+            d->hw_range_y_max = 3907;
+        }
+        if (printDeviceInfo)
+            qDebug("evdevtouch: found ti-tsc, overriding: min X: %d max X: %d min Y: %d max Y: %d",
+                   d->hw_range_x_min, d->hw_range_x_max, d->hw_range_y_min, d->hw_range_y_max);
     }
 
     bool grabSuccess = !ioctl(m_fd, EVIOCGRAB, (void *) 1);
     if (grabSuccess)
         ioctl(m_fd, EVIOCGRAB, (void *) 0);
     else
-        qWarning("ERROR: The device is grabbed by another process. No events will be read.");
+        qWarning("evdevtouch: The device is grabbed by another process. No events will be read.");
 
-#ifdef USE_MTDEV
-    const char *mtdevStr = "(mtdev)";
-    d->m_typeB = true;
-#else
-    const char *mtdevStr = "";
-    d->m_typeB = false;
-    long absbits[NUM_LONGS(ABS_CNT)];
-    if (ioctl(m_fd, EVIOCGBIT(EV_ABS, sizeof(absbits)), absbits) >= 0)
-        d->m_typeB = testBit(ABS_MT_SLOT, absbits);
-#endif
-    qDebug("Protocol type %c %s", d->m_typeB ? 'B' : 'A', mtdevStr);
+    if (rotationAngle)
+        d->m_rotate = QTransform::fromTranslate(0.5, 0.5).rotate(rotationAngle).translate(-0.5, -0.5);
 
     d->registerDevice();
 }
 
 QEvdevTouchScreenHandler::~QEvdevTouchScreenHandler()
 {
-#ifdef USE_MTDEV
+#if !defined(QT_NO_MTDEV)
     if (m_mtdev) {
         mtdev_close(m_mtdev);
         free(m_mtdev);
@@ -282,7 +347,7 @@ void QEvdevTouchScreenHandler::readData()
     ::input_event buffer[32];
     int n = 0;
     for (; ;) {
-#ifdef USE_MTDEV
+#if !defined(QT_NO_MTDEV)
         int result = mtdev_get(m_mtdev, m_fd, buffer, sizeof(buffer) / sizeof(::input_event));
         if (result > 0)
             result *= sizeof(::input_event);
@@ -290,11 +355,11 @@ void QEvdevTouchScreenHandler::readData()
         int result = QT_READ(m_fd, reinterpret_cast<char*>(buffer) + n, sizeof(buffer) - n);
 #endif
         if (!result) {
-            qWarning("Got EOF from input device");
+            qWarning("evdevtouch: Got EOF from input device");
             return;
         } else if (result < 0) {
             if (errno != EINTR && errno != EAGAIN) {
-                qWarning("Could not read from input device: %s", strerror(errno));
+                qErrnoWarning(errno, "evdevtouch: Could not read from input device");
                 if (errno == ENODEV) { // device got disconnected -> stop reading
                     delete m_notify;
                     m_notify = 0;
@@ -316,17 +381,42 @@ void QEvdevTouchScreenHandler::readData()
         d->processInputEvent(&buffer[i]);
 }
 
+void QEvdevTouchScreenData::addTouchPoint(const Contact &contact, Qt::TouchPointStates *combinedStates)
+{
+    QWindowSystemInterface::TouchPoint tp;
+    tp.id = contact.trackingId;
+    tp.flags = contact.flags;
+    tp.state = contact.state;
+    *combinedStates |= tp.state;
+
+    // Store the HW coordinates for now, will be updated later.
+    tp.area = QRectF(0, 0, contact.maj, contact.maj);
+    tp.area.moveCenter(QPoint(contact.x, contact.y));
+    tp.pressure = contact.pressure;
+
+    // Get a normalized position in range 0..1.
+    tp.normalPosition = QPointF((contact.x - hw_range_x_min) / qreal(hw_range_x_max - hw_range_x_min),
+                                (contact.y - hw_range_y_min) / qreal(hw_range_y_max - hw_range_y_min));
+
+    if (!m_rotate.isIdentity())
+        tp.normalPosition = m_rotate.map(tp.normalPosition);
+
+    tp.rawPositions.append(QPointF(contact.x, contact.y));
+
+    m_touchPoints.append(tp);
+}
+
 void QEvdevTouchScreenData::processInputEvent(input_event *data)
 {
     if (data->type == EV_ABS) {
 
-        if (data->code == ABS_MT_POSITION_X) {
+        if (data->code == ABS_MT_POSITION_X || (m_singleTouch && data->code == ABS_X)) {
             m_currentData.x = qBound(hw_range_x_min, data->value, hw_range_x_max);
-            if (m_typeB)
+            if (m_typeB || m_singleTouch)
                 m_contacts[m_currentSlot].x = m_currentData.x;
-        } else if (data->code == ABS_MT_POSITION_Y) {
+        } else if (data->code == ABS_MT_POSITION_Y || (m_singleTouch && data->code == ABS_Y)) {
             m_currentData.y = qBound(hw_range_y_min, data->value, hw_range_y_max);
-            if (m_typeB)
+            if (m_typeB || m_singleTouch)
                 m_contacts[m_currentSlot].y = m_currentData.y;
         } else if (data->code == ABS_MT_TRACKING_ID) {
             m_currentData.trackingId = data->value;
@@ -344,7 +434,7 @@ void QEvdevTouchScreenData::processInputEvent(input_event *data)
                 m_contacts[m_currentSlot].maj = m_currentData.maj;
         } else if (data->code == ABS_PRESSURE) {
             m_currentData.pressure = qBound(hw_pressure_min, data->value, hw_pressure_max);
-            if (m_typeB)
+            if (m_typeB || m_singleTouch)
                 m_contacts[m_currentSlot].pressure = m_currentData.pressure;
         } else if (data->code == ABS_MT_SLOT) {
             m_currentSlot = data->value;
@@ -352,9 +442,7 @@ void QEvdevTouchScreenData::processInputEvent(input_event *data)
 
     } else if (data->type == EV_KEY && !m_typeB) {
         if (data->code == BTN_TOUCH && data->value == 0)
-          {
             m_contacts[m_currentSlot].state = Qt::TouchPointReleased;
-          }
     } else if (data->type == EV_SYN && data->code == SYN_MT_REPORT && m_lastEventType != EV_SYN) {
 
         // If there is no tracking id, one will be generated later.
@@ -374,13 +462,11 @@ void QEvdevTouchScreenData::processInputEvent(input_event *data)
 
         m_touchPoints.clear();
         Qt::TouchPointStates combinedStates;
+
         QMutableHashIterator<int, Contact> it(m_contacts);
         while (it.hasNext()) {
             it.next();
-            QWindowSystemInterface::TouchPoint tp;
             Contact &contact(it.value());
-            tp.id = contact.trackingId;
-            tp.flags = contact.flags;
 
             int key = m_typeB ? it.key() : contact.trackingId;
             if (m_lastContacts.contains(key)) {
@@ -403,26 +489,32 @@ void QEvdevTouchScreenData::processInputEvent(input_event *data)
                 continue;
             }
 
-            tp.state = contact.state;
-            combinedStates |= tp.state;
+            addTouchPoint(contact, &combinedStates);
+        }
 
-            // Store the HW coordinates for now, will be updated later.
-            tp.area = QRectF(0, 0, contact.maj, contact.maj);
-            tp.area.moveCenter(QPoint(contact.x, contact.y));
-            tp.pressure = contact.pressure;
-
-            // Get a normalized position in range 0..1.
-            tp.normalPosition = QPointF((contact.x - hw_range_x_min) / qreal(hw_range_x_max - hw_range_x_min),
-                                        (contact.y - hw_range_y_min) / qreal(hw_range_y_max - hw_range_y_min));
-
-            m_touchPoints.append(tp);
+        // Now look for contacts that have disappeared since the last sync.
+        it = m_lastContacts;
+        while (it.hasNext()) {
+            it.next();
+            Contact &contact(it.value());
+            int key = m_typeB ? it.key() : contact.trackingId;
+            if (!m_contacts.contains(key)) {
+                contact.state = Qt::TouchPointReleased;
+                addTouchPoint(contact, &combinedStates);
+            }
+        }
 
+        // Remove contacts that have just been reported as released.
+        it = m_contacts;
+        while (it.hasNext()) {
+            it.next();
+            Contact &contact(it.value());
             if (contact.state == Qt::TouchPointReleased)
                 it.remove();
         }
 
         m_lastContacts = m_contacts;
-        if (!m_typeB)
+        if (!m_typeB && !m_singleTouch)
             m_contacts.clear();
 
         if (!m_touchPoints.isEmpty() && combinedStates != Qt::TouchPointStationary)
@@ -507,9 +599,10 @@ void QEvdevTouchScreenData::reportPoints()
         QWindowSystemInterface::TouchPoint &tp(m_touchPoints[i]);
 
         // Generate a screen position that is always inside the active window
-        // or the primary screen.
-        const qreal wx = winRect.left() + tp.normalPosition.x() * winRect.width();
-        const qreal wy = winRect.top() + tp.normalPosition.y() * winRect.height();
+        // or the primary screen.  Even though we report this as a QRectF, internally
+        // Qt uses QRect/QPoint so we need to bound the size to winRect.size() - QSize(1, 1)
+        const qreal wx = winRect.left() + tp.normalPosition.x() * (winRect.width() - 1);
+        const qreal wy = winRect.top() + tp.normalPosition.y() * (winRect.height() - 1);
         const qreal sizeRatio = (winRect.width() + winRect.height()) / qreal(hw_w + hw_h);
         if (tp.area.width() == -1) // touch major was not provided
             tp.area = QRectF(0, 0, 8, 8);
