@@ -48,22 +48,21 @@
 #include <QThread>
 #include <qpa/qwindowsysteminterface.h>
 
-QT_BEGIN_HEADER
+#if !defined(QT_NO_MTDEV)
+struct mtdev;
+#endif
 
 QT_BEGIN_NAMESPACE
 
 class QSocketNotifier;
 class QEvdevTouchScreenData;
-#ifdef USE_MTDEV
-struct mtdev;
-#endif
 
 class QEvdevTouchScreenHandler : public QObject
 {
     Q_OBJECT
 
 public:
-    explicit QEvdevTouchScreenHandler(const QString &spec = QString(), QObject *parent = 0);
+    explicit QEvdevTouchScreenHandler(const QString &specification = QString(), QObject *parent = 0);
     ~QEvdevTouchScreenHandler();
 
 private slots:
@@ -73,7 +72,7 @@ private:
     QSocketNotifier *m_notify;
     int m_fd;
     QEvdevTouchScreenData *d;
-#ifdef USE_MTDEV
+#if !defined(QT_NO_MTDEV)
     mtdev *m_mtdev;
 #endif
 };
@@ -93,6 +92,4 @@ private:
 
 QT_END_NAMESPACE
 
-QT_END_HEADER
-
 #endif // QEVDEVTOUCH_P_H
