@@ -47,7 +47,6 @@
 #include <QtDBus/QDBusConnection>
 #include <QtDBus/QDBusVariant>
 
-QT_BEGIN_HEADER
 QT_BEGIN_NAMESPACE
 
 class QDBusServiceWatcher;
@@ -68,8 +67,6 @@ Q_SIGNALS:
 private Q_SLOTS:
     void serviceRegistered();
     void serviceUnregistered();
-    void enabledStateCallback(const QDBusVariant &enabled);
-//    void enabledStateChanged(const QDBusVariant &);
     void connectA11yBus(const QString &address);
 
     void dbusError(const QDBusError &error);
@@ -83,6 +80,5 @@ private:
 };
 
 QT_END_NAMESPACE
-QT_END_HEADER
 
 #endif // DBUSCONNECTION_H
