@@ -42,6 +42,8 @@
 #include "qgenericunixthemes_p.h"
 #include "../../services/genericunix/qgenericunixservices_p.h"
 
+#include "qpa/qplatformtheme_p.h"
+
 #include <QtGui/QPalette>
 #include <QtGui/QFont>
 #include <QtGui/QGuiApplication>
@@ -56,20 +58,22 @@
 #include <qpa/qplatformintegration.h>
 #include <qpa/qplatformservices.h>
 
+#include <algorithm>
+
 QT_BEGIN_NAMESPACE
 
 ResourceHelper::ResourceHelper()
 {
-    qFill(palettes, palettes + QPlatformTheme::NPalettes, static_cast<QPalette *>(0));
-    qFill(fonts, fonts + QPlatformTheme::NFonts, static_cast<QFont *>(0));
+    std::fill(palettes, palettes + QPlatformTheme::NPalettes, static_cast<QPalette *>(0));
+    std::fill(fonts, fonts + QPlatformTheme::NFonts, static_cast<QFont *>(0));
 }
 
 void ResourceHelper::clear()
 {
     qDeleteAll(palettes, palettes + QPlatformTheme::NPalettes);
     qDeleteAll(fonts, fonts + QPlatformTheme::NFonts);
-    qFill(palettes, palettes + QPlatformTheme::NPalettes, static_cast<QPalette *>(0));
-    qFill(fonts, fonts + QPlatformTheme::NFonts, static_cast<QFont *>(0));
+    std::fill(palettes, palettes + QPlatformTheme::NPalettes, static_cast<QPalette *>(0));
+    std::fill(fonts, fonts + QPlatformTheme::NFonts, static_cast<QFont *>(0));
 }
 
 /*!
@@ -87,16 +91,37 @@ const char *QGenericUnixTheme::name = "generic";
 static const char defaultSystemFontNameC[] = "Sans Serif";
 enum { defaultSystemFontSize = 9 };
 
+class QGenericUnixThemePrivate : public QPlatformThemePrivate
+{
+public:
+    QGenericUnixThemePrivate()
+        : QPlatformThemePrivate()
+        , systemFont(QLatin1String(defaultSystemFontNameC), defaultSystemFontSize)
+        , fixedFont(QStringLiteral("monospace"), systemFont.pointSize())
+    {
+        fixedFont.setStyleHint(QFont::TypeWriter);
+    }
+
+    const QFont systemFont;
+    QFont fixedFont;
+};
+
 QGenericUnixTheme::QGenericUnixTheme()
-    : m_systemFont(QLatin1String(defaultSystemFontNameC), defaultSystemFontSize)
+    : QPlatformTheme(new QGenericUnixThemePrivate())
 {
 }
 
 const QFont *QGenericUnixTheme::font(Font type) const
 {
-    if (type == QPlatformTheme::SystemFont)
-        return &m_systemFont;
-    return 0;
+    Q_D(const QGenericUnixTheme);
+    switch (type) {
+    case QPlatformTheme::SystemFont:
+        return &d->systemFont;
+    case QPlatformTheme::FixedFont:
+        return &d->fixedFont;
+    default:
+        return 0;
+    }
 }
 
 // Helper to return the icon theme paths from XDG.
@@ -142,6 +167,104 @@ QVariant QGenericUnixTheme::themeHint(ThemeHint hint) const
 }
 
 #ifndef QT_NO_SETTINGS
+class QKdeThemePrivate : public QPlatformThemePrivate
+{
+public:
+    QKdeThemePrivate(const QString &kdeHome, int kdeVersion)
+        : kdeHome(kdeHome)
+        , kdeVersion(kdeVersion)
+        , toolButtonStyle(Qt::ToolButtonTextBesideIcon)
+        , toolBarIconSize(0)
+        , singleClick(true)
+    { }
+
+    QString globalSettingsFile() const
+    {
+        return kdeHome + QStringLiteral("/share/config/kdeglobals");
+    }
+
+    void refresh();
+    static void readKdeSystemPalette(const QSettings &kdeSettings, QPalette *pal);
+    static QFont *readKdeFontSetting(const QSettings &settings, const QString &key);
+    static QStringList kdeIconThemeSearchPaths(const QString &kdeHome);
+
+
+    const QString kdeHome;
+    const int kdeVersion;
+
+    ResourceHelper resources;
+    QString iconThemeName;
+    QString iconFallbackThemeName;
+    QStringList styleNames;
+    int toolButtonStyle;
+    int toolBarIconSize;
+    bool singleClick;
+};
+
+void QKdeThemePrivate::refresh()
+{
+    resources.clear();
+
+    toolButtonStyle = Qt::ToolButtonTextBesideIcon;
+    toolBarIconSize = 0;
+    styleNames.clear();
+    styleNames << QStringLiteral("Oxygen") << QStringLiteral("fusion") << QStringLiteral("windows");
+    iconFallbackThemeName = iconThemeName = QStringLiteral("oxygen");
+
+    // Read settings file.
+    const QString settingsFile = globalSettingsFile();
+    if (!QFileInfo(settingsFile).isReadable())
+        return;
+
+    const QSettings kdeSettings(settingsFile, QSettings::IniFormat);
+
+    QPalette systemPalette = QPalette();
+    readKdeSystemPalette(kdeSettings, &systemPalette);
+    resources.palettes[QPlatformTheme::SystemPalette] = new QPalette(systemPalette);
+    //## TODO tooltip color
+
+    const QVariant styleValue = kdeSettings.value(QStringLiteral("widgetStyle"));
+    if (styleValue.isValid()) {
+        const QString style = styleValue.toString();
+        if (style != styleNames.front())
+            styleNames.push_front(style);
+    }
+
+    singleClick = kdeSettings.value(QStringLiteral("KDE/SingleClick"), true).toBool();
+
+    const QVariant themeValue = kdeSettings.value(QStringLiteral("Icons/Theme"));
+    if (themeValue.isValid())
+        iconThemeName = themeValue.toString();
+
+    const QVariant toolBarIconSizeValue = kdeSettings.value(QStringLiteral("ToolbarIcons/Size"));
+    if (toolBarIconSizeValue.isValid())
+        toolBarIconSize = toolBarIconSizeValue.toInt();
+
+    const QVariant toolbarStyleValue = kdeSettings.value(QStringLiteral("ToolButtonStyle"));
+    if (toolbarStyleValue.isValid()) {
+        const QString toolBarStyle = toolbarStyleValue.toString();
+        if (toolBarStyle == QStringLiteral("TextBesideIcon"))
+            toolButtonStyle =  Qt::ToolButtonTextBesideIcon;
+        else if (toolBarStyle == QStringLiteral("TextOnly"))
+            toolButtonStyle = Qt::ToolButtonTextOnly;
+        else if (toolBarStyle == QStringLiteral("TextUnderIcon"))
+            toolButtonStyle = Qt::ToolButtonTextUnderIcon;
+    }
+
+    // Read system font, ignore 'smallestReadableFont'
+    if (QFont *systemFont = readKdeFontSetting(kdeSettings, QStringLiteral("font")))
+        resources.fonts[QPlatformTheme::SystemFont] = systemFont;
+    else
+        resources.fonts[QPlatformTheme::SystemFont] = new QFont(QLatin1String(defaultSystemFontNameC), defaultSystemFontSize);
+
+    if (QFont *fixedFont = readKdeFontSetting(kdeSettings, QStringLiteral("fixed"))) {
+        resources.fonts[QPlatformTheme::FixedFont] = fixedFont;
+    } else {
+        fixedFont = new QFont(QLatin1String(defaultSystemFontNameC), defaultSystemFontSize);
+        fixedFont->setStyleHint(QFont::TypeWriter);
+        resources.fonts[QPlatformTheme::FixedFont] = fixedFont;
+    }
+}
 
 // Reads the color from the KDE configuration, and store it in the
 // palette with the given color role if found.
@@ -158,8 +281,16 @@ static inline bool kdeColor(QPalette *pal, QPalette::ColorRole role,
     return true;
 }
 
-static inline void readKdeSystemPalette(const QSettings &kdeSettings, QPalette *pal)
+void QKdeThemePrivate::readKdeSystemPalette(const QSettings &kdeSettings, QPalette *pal)
 {
+    if (!kdeSettings.contains(QStringLiteral("Colors:Button/BackgroundNormal"))) {
+        // kcolorscheme.cpp: SetDefaultColors
+        const QColor defaultWindowBackground(214, 210, 208);
+        const QColor defaultButtonBackground(223, 220, 217);
+        *pal = QPalette(defaultButtonBackground, defaultWindowBackground);
+        return;
+    }
+
     kdeColor(pal, QPalette::Button, kdeSettings, QStringLiteral("Colors:Button/BackgroundNormal"));
     kdeColor(pal, QPalette::Window, kdeSettings, QStringLiteral("Colors:Window/BackgroundNormal"));
     kdeColor(pal, QPalette::Text, kdeSettings, QStringLiteral("Colors:View/ForegroundNormal"));
@@ -171,6 +302,34 @@ static inline void readKdeSystemPalette(const QSettings &kdeSettings, QPalette *
     kdeColor(pal, QPalette::ButtonText, kdeSettings, QStringLiteral("Colors:Button/ForegroundNormal"));
     kdeColor(pal, QPalette::Link, kdeSettings, QStringLiteral("Colors:View/ForegroundLink"));
     kdeColor(pal, QPalette::LinkVisited, kdeSettings, QStringLiteral("Colors:View/ForegroundVisited"));
+    kdeColor(pal, QPalette::ToolTipBase, kdeSettings, QStringLiteral("Colors:Tooltip/BackgroundNormal"));
+    kdeColor(pal, QPalette::ToolTipText, kdeSettings, QStringLiteral("Colors:Tooltip/ForegroundNormal"));
+
+    // The above code sets _all_ color roles to "normal" colors. In KDE, the disabled
+    // color roles are calculated by applying various effects described in kdeglobals.
+    // We use a bit simpler approach here, similar logic than in qt_palette_from_color().
+    const QColor button = pal->color(QPalette::Button);
+    int h, s, v;
+    button.getHsv(&h, &s, &v);
+
+    const QBrush whiteBrush = QBrush(Qt::white);
+    const QBrush buttonBrush = QBrush(button);
+    const QBrush buttonBrushDark = QBrush(button.darker(v > 128 ? 200 : 50));
+    const QBrush buttonBrushDark150 = QBrush(button.darker(v > 128 ? 150 : 75));
+    const QBrush buttonBrushLight150 = QBrush(button.lighter(v > 128 ? 150 : 75));
+
+    pal->setBrush(QPalette::Disabled, QPalette::WindowText, buttonBrushDark);
+    pal->setBrush(QPalette::Disabled, QPalette::ButtonText, buttonBrushDark);
+    pal->setBrush(QPalette::Disabled, QPalette::Button, buttonBrush);
+    pal->setBrush(QPalette::Disabled, QPalette::Light, buttonBrushLight150);
+    pal->setBrush(QPalette::Disabled, QPalette::Dark, buttonBrushDark);
+    pal->setBrush(QPalette::Disabled, QPalette::Mid, buttonBrushDark150);
+    pal->setBrush(QPalette::Disabled, QPalette::Text, buttonBrushDark);
+    pal->setBrush(QPalette::Disabled, QPalette::BrightText, whiteBrush);
+    pal->setBrush(QPalette::Disabled, QPalette::Base, buttonBrush);
+    pal->setBrush(QPalette::Disabled, QPalette::Window, buttonBrush);
+    pal->setBrush(QPalette::Disabled, QPalette::Highlight, buttonBrushDark150);
+    pal->setBrush(QPalette::Disabled, QPalette::HighlightedText, buttonBrushLight150);
 }
 
 /*!
@@ -183,14 +342,13 @@ static inline void readKdeSystemPalette(const QSettings &kdeSettings, QPalette *
 
 const char *QKdeTheme::name = "kde";
 
-QKdeTheme::QKdeTheme(const QString &kdeHome, int kdeVersion) :
-    m_kdeHome(kdeHome), m_kdeVersion(kdeVersion),
-    m_toolButtonStyle(Qt::ToolButtonTextBesideIcon), m_toolBarIconSize(0)
+QKdeTheme::QKdeTheme(const QString &kdeHome, int kdeVersion)
+    : QPlatformTheme(new QKdeThemePrivate(kdeHome,kdeVersion))
 {
-    refresh();
+    d_func()->refresh();
 }
 
-static inline QFont *readKdeFontSetting(const QSettings &settings, const QString &key)
+QFont *QKdeThemePrivate::readKdeFontSetting(const QSettings &settings, const QString &key)
 {
     const QVariant fontValue = settings.value(key);
     if (fontValue.isValid()) {
@@ -218,68 +376,8 @@ static inline QFont *readKdeFontSetting(const QSettings &settings, const QString
     return 0;
 }
 
-void QKdeTheme::refresh()
-{
-    m_resources.clear();
 
-    m_toolButtonStyle = Qt::ToolButtonTextBesideIcon;
-    m_toolBarIconSize = 0;
-    m_styleNames.clear();
-    m_styleNames << QStringLiteral("Oxygen") << QStringLiteral("fusion") << QStringLiteral("windows");
-    m_iconFallbackThemeName = m_iconThemeName = QStringLiteral("oxygen");
-
-    // Read settings file.
-    const QString settingsFile = globalSettingsFile();
-    if (!QFileInfo(settingsFile).isReadable())
-        return;
-
-    const QSettings kdeSettings(settingsFile, QSettings::IniFormat);
-
-    QPalette systemPalette = QPalette();
-    readKdeSystemPalette(kdeSettings, &systemPalette);
-    m_resources.palettes[SystemPalette] = new QPalette(systemPalette);
-    //## TODO tooltip color
-
-    const QVariant styleValue = kdeSettings.value(QStringLiteral("widgetStyle"));
-    if (styleValue.isValid()) {
-        const QString style = styleValue.toString();
-        if (style != m_styleNames.front())
-            m_styleNames.push_front(style);
-    }
-
-    const QVariant themeValue = kdeSettings.value(QStringLiteral("Icons/Theme"));
-    if (themeValue.isValid())
-        m_iconThemeName = themeValue.toString();
-
-    const QVariant toolBarIconSizeValue = kdeSettings.value(QStringLiteral("ToolbarIcons/Size"));
-    if (toolBarIconSizeValue.isValid())
-        m_toolBarIconSize = toolBarIconSizeValue.toInt();
-
-    const QVariant toolbarStyleValue = kdeSettings.value(QStringLiteral("ToolButtonStyle"));
-    if (toolbarStyleValue.isValid()) {
-        const QString toolBarStyle = toolbarStyleValue.toString();
-        if (toolBarStyle == QStringLiteral("TextBesideIcon"))
-            m_toolButtonStyle =  Qt::ToolButtonTextBesideIcon;
-        else if (toolBarStyle == QStringLiteral("TextOnly"))
-            m_toolButtonStyle = Qt::ToolButtonTextOnly;
-        else if (toolBarStyle == QStringLiteral("TextUnderIcon"))
-            m_toolButtonStyle = Qt::ToolButtonTextUnderIcon;
-    }
-
-    // Read system font, ignore 'fixed' 'smallestReadableFont'
-    if (QFont *systemFont = readKdeFontSetting(kdeSettings, QStringLiteral("font"))) {
-        m_resources.fonts[SystemFont] = systemFont;
-    } else {
-        m_resources.fonts[SystemFont] = new QFont(QLatin1String(defaultSystemFontNameC), defaultSystemFontSize);
-    }
-}
-
-QString QKdeTheme::globalSettingsFile() const
-{
-    return m_kdeHome + QStringLiteral("/share/config/kdeglobals");
-}
-
-static QStringList kdeIconThemeSearchPaths(const QString &kdeHome)
+QStringList QKdeThemePrivate::kdeIconThemeSearchPaths(const QString &kdeHome)
 {
     QStringList candidates = QStringList(kdeHome);
     const QString kdeDirs = QFile::decodeName(qgetenv("KDEDIRS"));
@@ -298,6 +396,7 @@ static QStringList kdeIconThemeSearchPaths(const QString &kdeHome)
 
 QVariant QKdeTheme::themeHint(QPlatformTheme::ThemeHint hint) const
 {
+    Q_D(const QKdeTheme);
     switch (hint) {
     case QPlatformTheme::UseFullScreenForPopupMenu:
         return QVariant(true);
@@ -306,25 +405,39 @@ QVariant QKdeTheme::themeHint(QPlatformTheme::ThemeHint hint) const
     case QPlatformTheme::DialogButtonBoxLayout:
         return QVariant(2); // QDialogButtonBox::KdeLayout
     case QPlatformTheme::ToolButtonStyle:
-        return QVariant(m_toolButtonStyle);
+        return QVariant(d->toolButtonStyle);
     case QPlatformTheme::ToolBarIconSize:
-        return QVariant(m_toolBarIconSize);
+        return QVariant(d->toolBarIconSize);
     case QPlatformTheme::SystemIconThemeName:
-        return QVariant(m_iconThemeName);
+        return QVariant(d->iconThemeName);
     case QPlatformTheme::SystemIconFallbackThemeName:
-        return QVariant(m_iconFallbackThemeName);
+        return QVariant(d->iconFallbackThemeName);
     case QPlatformTheme::IconThemeSearchPaths:
-        return QVariant(kdeIconThemeSearchPaths(m_kdeHome));
+        return QVariant(d->kdeIconThemeSearchPaths(d->kdeHome));
     case QPlatformTheme::StyleNames:
-        return QVariant(m_styleNames);
+        return QVariant(d->styleNames);
     case QPlatformTheme::KeyboardScheme:
         return QVariant(int(KdeKeyboardScheme));
+    case QPlatformTheme::ItemViewActivateItemOnSingleClick:
+        return QVariant(d->singleClick);
     default:
         break;
     }
     return QPlatformTheme::themeHint(hint);
 }
 
+const QPalette *QKdeTheme::palette(Palette type) const
+{
+    Q_D(const QKdeTheme);
+    return d->resources.palettes[type];
+}
+
+const QFont *QKdeTheme::font(Font type) const
+{
+    Q_D(const QKdeTheme);
+    return d->resources.fonts[type];
+}
+
 QPlatformTheme *QKdeTheme::createKdeTheme()
 {
     // Check for version >= 4 and determine home folder from environment,
@@ -361,8 +474,22 @@ QPlatformTheme *QKdeTheme::createKdeTheme()
 
 const char *QGnomeTheme::name = "gnome";
 
+class QGnomeThemePrivate : public QPlatformThemePrivate
+{
+public:
+    QGnomeThemePrivate()
+        : systemFont(QLatin1Literal(defaultSystemFontNameC), defaultSystemFontSize)
+        , fixedFont(QStringLiteral("monospace"), systemFont.pointSize())
+    {
+        fixedFont.setStyleHint(QFont::TypeWriter);
+    }
+
+    const QFont systemFont;
+    QFont fixedFont;
+};
+
 QGnomeTheme::QGnomeTheme()
-   : m_systemFont(QLatin1String(defaultSystemFontNameC), defaultSystemFontSize)
+    : QPlatformTheme(new QGnomeThemePrivate())
 {
 }
 
@@ -380,11 +507,13 @@ QVariant QGnomeTheme::themeHint(QPlatformTheme::ThemeHint hint) const
         return QVariant(QGenericUnixTheme::xdgIconThemePaths());
     case QPlatformTheme::StyleNames: {
         QStringList styleNames;
-        styleNames << QStringLiteral("GTK+") << QStringLiteral("cleanlooks") << QStringLiteral("windows");
+        styleNames << QStringLiteral("GTK+") << QStringLiteral("fusion") << QStringLiteral("windows");
         return QVariant(styleNames);
     }
     case QPlatformTheme::KeyboardScheme:
         return QVariant(int(GnomeKeyboardScheme));
+    case QPlatformTheme::PasswordMaskCharacter:
+        return QVariant(QChar(0x2022));
     default:
         break;
     }
@@ -393,9 +522,15 @@ QVariant QGnomeTheme::themeHint(QPlatformTheme::ThemeHint hint) const
 
 const QFont *QGnomeTheme::font(Font type) const
 {
-    if (type == QPlatformTheme::SystemFont)
-        return &m_systemFont;
-    return 0;
+    Q_D(const QGnomeTheme);
+    switch (type) {
+    case QPlatformTheme::SystemFont:
+        return  &d->systemFont;
+    case QPlatformTheme::FixedFont:
+        return &d->fixedFont;
+    default:
+        return 0;
+    }
 }
 
 /*!
@@ -425,7 +560,14 @@ QStringList QGenericUnixTheme::themeNames()
 #ifndef QT_NO_SETTINGS
             result.push_back(QLatin1String(QKdeTheme::name));
 #endif
-        } else { // Gnome, Unity, other Gtk-based desktops like XFCE.
+        } else if (desktopEnvironment == QByteArrayLiteral("GNOME") ||
+                desktopEnvironment == QByteArrayLiteral("UNITY") ||
+                desktopEnvironment == QByteArrayLiteral("MATE") ||
+                desktopEnvironment == QByteArrayLiteral("XFCE") ||
+                desktopEnvironment == QByteArrayLiteral("LXDE")) { // Gtk-based desktops
+            // prefer the GTK2 theme implementation with native dialogs etc.
+            result.push_back(QStringLiteral("gtk2"));
+            // fallback to the generic Gnome theme if loading the GTK2 theme fails
             result.push_back(QLatin1String(QGnomeTheme::name));
         }
         const QString session = QString::fromLocal8Bit(qgetenv("DESKTOP_SESSION"));
