@@ -96,7 +96,6 @@ public:
     QAccessible::State state() const {
         QAccessibleInterface *parentInterface = parent();
         QAccessible::State state = parentInterface->state();
-        delete parentInterface;
         return state;
     }
     QRect rect() const {
@@ -139,9 +138,12 @@ public:
         return QStringList();
     }
 
+    int index() const { return m_index; }
+
 private:
     QPointer<QTabBar> m_parent;
     int m_index;
+
 };
 
 /*!
@@ -153,6 +155,12 @@ QAccessibleTabBar::QAccessibleTabBar(QWidget *w)
     Q_ASSERT(tabBar());
 }
 
+QAccessibleTabBar::~QAccessibleTabBar()
+{
+    foreach (QAccessible::Id id, m_childInterfaces.values())
+        QAccessible::deleteAccessibleInterface(id);
+}
+
 /*! Returns the QTabBar. */
 QTabBar *QAccessibleTabBar::tabBar() const
 {
@@ -161,9 +169,14 @@ QTabBar *QAccessibleTabBar::tabBar() const
 
 QAccessibleInterface* QAccessibleTabBar::child(int index) const
 {
+    if (QAccessible::Id id = m_childInterfaces.value(index))
+        return QAccessible::accessibleInterface(id);
+
     // first the tabs, then 2 buttons
     if (index < tabBar()->count()) {
         QAccessibleTabButton *button = new QAccessibleTabButton(tabBar(), index);
+        QAccessible::registerAccessibleInterface(button);
+        m_childInterfaces.insert(index, QAccessible::uniqueId(button));
         return button;
     } else if (index >= tabBar()->count()) {
         // left button
@@ -180,12 +193,17 @@ QAccessibleInterface* QAccessibleTabBar::child(int index) const
 
 int QAccessibleTabBar::indexOfChild(const QAccessibleInterface *child) const
 {
-    // FIXME this looks broken
-
     if (child->object() && child->object() == tabBar()->d_func()->leftB)
         return tabBar()->count();
     if (child->object() && child->object() == tabBar()->d_func()->rightB)
         return tabBar()->count() + 1;
+    if (child->role() == QAccessible::PageTab) {
+        QAccessibleInterface *parent = child->parent();
+        if (parent == this) {
+            const QAccessibleTabButton *tabButton = static_cast<const QAccessibleTabButton *>(child);
+            return tabButton->index();
+        }
+    }
     return -1;
 }
 
@@ -203,41 +221,6 @@ QString QAccessibleTabBar::text(QAccessible::Text t) const
     return QString();
 }
 
-/*!
-    Selects the item with index \a child if \a on is true; otherwise
-    unselects it. If \a extend is true and the selection mode is not
-    \c Single and there is an existing selection, the selection is
-    extended to include all the items from the existing selection up
-    to and including the item with index \a child. Returns true if a
-    selection was made or extended; otherwise returns false.
-
-    \sa selection(), clearSelection()
-*/
-bool QAccessibleTabBar::setSelected(int child, bool on, bool extend)
-{
-    if (!child || !on || extend || child > tabBar()->count())
-        return false;
-
-    if (!tabBar()->isTabEnabled(child - 1))
-        return false;
-    tabBar()->setCurrentIndex(child - 1);
-    return true;
-}
-
-/*!
-    Returns a (possibly empty) list of indexes of the items selected
-    in the list box.
-
-    \sa setSelected(), clearSelection()
-*/
-QVector<int> QAccessibleTabBar::selection() const
-{
-    QVector<int> array;
-    if (tabBar()->currentIndex() != -1)
-        array +=tabBar()->currentIndex() + 1;
-    return array;
-}
-
 #endif // QT_NO_TABBAR
 
 #ifndef QT_NO_COMBOBOX
@@ -360,18 +343,6 @@ QStringList QAccessibleComboBox::keyBindingsForAction(const QString &/*actionNam
 
 #endif // QT_NO_COMBOBOX
 
-static inline void removeInvisibleWidgetsFromList(QWidgetList *list)
-{
-    if (!list || list->isEmpty())
-        return;
-
-    for (int i = 0; i < list->count(); ++i) {
-        QWidget *widget = list->at(i);
-        if (!widget->isVisible())
-            list->removeAt(i);
-    }
-}
-
 #ifndef QT_NO_SCROLLAREA
 // ======================= QAccessibleAbstractScrollArea =======================
 QAccessibleAbstractScrollArea::QAccessibleAbstractScrollArea(QWidget *widget)
