@@ -46,8 +46,6 @@
 #include <qlistview.h>
 #include <qtreeview.h>
 #include <private/qtreewidget_p.h>
-#include <qaccessible2.h>
-#include <QDebug>
 
 #ifndef QT_NO_ACCESSIBILITY
 
@@ -82,43 +80,6 @@ int QAccessibleTable::logicalIndex(const QModelIndex &index) const
     return (index.row() + hHeader)*(index.model()->columnCount() + vHeader) + (index.column() + vHeader);
 }
 
-QAccessibleInterface *QAccessibleTable::childFromLogical(int logicalIndex) const
-{
-    if (!view()->model())
-        return 0;
-
-    int vHeader = verticalHeader() ? 1 : 0;
-    int hHeader = horizontalHeader() ? 1 : 0;
-
-    int columns = view()->model()->columnCount() + vHeader;
-
-    int row = logicalIndex / columns;
-    int column = logicalIndex % columns;
-
-    if (vHeader) {
-        if (column == 0) {
-            if (row == 0) {
-                return new QAccessibleTableCornerButton(view());
-            }
-            return new QAccessibleTableHeaderCell(view(), row-1, Qt::Vertical);
-        }
-        --column;
-    }
-    if (hHeader) {
-        if (row == 0) {
-            return new QAccessibleTableHeaderCell(view(), column, Qt::Horizontal);
-        }
-        --row;
-    }
-
-    QModelIndex index = view()->model()->index(row, column, view()->rootIndex());
-    if (!index.isValid()) {
-        qWarning() << "QAccessibleTable::childFromLogical: Invalid index at: " << row << column;
-        return 0;
-    }
-    return new QAccessibleTableCell(view(), index, cellRole());
-}
-
 QAccessibleTable::QAccessibleTable(QWidget *w)
     : QAccessibleObject(w)
 {
@@ -136,8 +97,15 @@ QAccessibleTable::QAccessibleTable(QWidget *w)
     }
 }
 
+bool QAccessibleTable::isValid() const
+{
+    return (view() && !qobject_cast<QWidget*>(view())->d_func()->data.in_destructor);
+}
+
 QAccessibleTable::~QAccessibleTable()
 {
+    Q_FOREACH (QAccessible::Id id, childToId.values())
+        QAccessible::deleteAccessibleInterface(id);
 }
 
 QHeaderView *QAccessibleTable::horizontalHeader() const
@@ -168,13 +136,6 @@ QHeaderView *QAccessibleTable::verticalHeader() const
     return header;
 }
 
-QAccessibleTableCell *QAccessibleTable::cell(const QModelIndex &index) const
-{
-    if (index.isValid())
-        return new QAccessibleTableCell(view(), index, cellRole());
-    return 0;
-}
-
 QAccessibleInterface *QAccessibleTable::cellAt(int row, int column) const
 {
     if (!view()->model())
@@ -185,7 +146,7 @@ QAccessibleInterface *QAccessibleTable::cellAt(int row, int column) const
         qWarning() << "QAccessibleTable::cellAt: invalid index: " << index << " for " << view();
         return 0;
     }
-    return cell(index);
+    return child(logicalIndex(index));
 }
 
 QAccessibleInterface *QAccessibleTable::caption() const
@@ -247,9 +208,8 @@ QList<QAccessibleInterface *> QAccessibleTable::selectedCells() const
     QList<QAccessibleInterface*> cells;
     if (!view()->selectionModel())
         return cells;
-    Q_FOREACH (const QModelIndex &index, view()->selectionModel()->selectedIndexes()) {
-        cells.append(cell(index));
-    }
+    Q_FOREACH (const QModelIndex &index, view()->selectionModel()->selectedIndexes())
+        cells.append(child(logicalIndex(index)));
     return cells;
 }
 
@@ -299,9 +259,28 @@ bool QAccessibleTable::selectRow(int row)
     if (!view()->model() || !view()->selectionModel())
         return false;
     QModelIndex index = view()->model()->index(row, 0, view()->rootIndex());
-    if (!index.isValid() || view()->selectionMode() & QAbstractItemView::NoSelection)
+
+    if (!index.isValid() || view()->selectionBehavior() == QAbstractItemView::SelectColumns)
         return false;
-    view()->selectionModel()->select(index, QItemSelectionModel::Select);
+
+    switch (view()->selectionMode()) {
+    case QAbstractItemView::NoSelection:
+        return false;
+    case QAbstractItemView::SingleSelection:
+        if (view()->selectionBehavior() != QAbstractItemView::SelectRows && columnCount() > 1 )
+            return false;
+        view()->clearSelection();
+        break;
+    case QAbstractItemView::ContiguousSelection:
+        if ((!row || !view()->selectionModel()->isRowSelected(row - 1, view()->rootIndex()))
+            && !view()->selectionModel()->isRowSelected(row + 1, view()->rootIndex()))
+            view()->clearSelection();
+        break;
+    default:
+        break;
+    }
+
+    view()->selectionModel()->select(index, QItemSelectionModel::Select | QItemSelectionModel::Rows);
     return true;
 }
 
@@ -310,9 +289,26 @@ bool QAccessibleTable::selectColumn(int column)
     if (!view()->model() || !view()->selectionModel())
         return false;
     QModelIndex index = view()->model()->index(0, column, view()->rootIndex());
-    if (!index.isValid() || view()->selectionMode() & QAbstractItemView::NoSelection)
+
+    if (!index.isValid() || view()->selectionBehavior() == QAbstractItemView::SelectRows)
         return false;
-    view()->selectionModel()->select(index, QItemSelectionModel::Select);
+
+    switch (view()->selectionMode()) {
+    case QAbstractItemView::NoSelection:
+        return false;
+    case QAbstractItemView::SingleSelection:
+        if (view()->selectionBehavior() != QAbstractItemView::SelectColumns && rowCount() > 1)
+            return false;
+    case QAbstractItemView::ContiguousSelection:
+        if ((!column || !view()->selectionModel()->isColumnSelected(column - 1, view()->rootIndex()))
+            && !view()->selectionModel()->isColumnSelected(column + 1, view()->rootIndex()))
+            view()->clearSelection();
+        break;
+    default:
+        break;
+    }
+
+    view()->selectionModel()->select(index, QItemSelectionModel::Select | QItemSelectionModel::Columns);
     return true;
 }
 
@@ -320,10 +316,35 @@ bool QAccessibleTable::unselectRow(int row)
 {
     if (!view()->model() || !view()->selectionModel())
         return false;
+
     QModelIndex index = view()->model()->index(row, 0, view()->rootIndex());
-    if (!index.isValid() || view()->selectionMode() & QAbstractItemView::NoSelection)
+    if (!index.isValid())
         return false;
-    view()->selectionModel()->select(index, QItemSelectionModel::Deselect);
+
+    QItemSelection selection(index, index);
+
+    switch (view()->selectionMode()) {
+    case QAbstractItemView::SingleSelection:
+        //In SingleSelection and ContiguousSelection once an item
+        //is selected, there's no way for the user to unselect all items
+        if (selectedRowCount() == 1)
+            return false;
+        break;
+    case QAbstractItemView::ContiguousSelection:
+        if (selectedRowCount() == 1)
+            return false;
+
+        if ((!row || view()->selectionModel()->isRowSelected(row - 1, view()->rootIndex()))
+            && view()->selectionModel()->isRowSelected(row + 1, view()->rootIndex())) {
+            //If there are rows selected both up the current row and down the current rown,
+            //the ones which are down the current row will be deselected
+            selection = QItemSelection(index, view()->model()->index(rowCount() - 1, 0, view()->rootIndex()));
+        }
+    default:
+        break;
+    }
+
+    view()->selectionModel()->select(selection, QItemSelectionModel::Deselect | QItemSelectionModel::Rows);
     return true;
 }
 
@@ -331,10 +352,35 @@ bool QAccessibleTable::unselectColumn(int column)
 {
     if (!view()->model() || !view()->selectionModel())
         return false;
+
     QModelIndex index = view()->model()->index(0, column, view()->rootIndex());
-    if (!index.isValid() || view()->selectionMode() & QAbstractItemView::NoSelection)
+    if (!index.isValid())
         return false;
-    view()->selectionModel()->select(index, QItemSelectionModel::Columns & QItemSelectionModel::Deselect);
+
+    QItemSelection selection(index, index);
+
+    switch (view()->selectionMode()) {
+    case QAbstractItemView::SingleSelection:
+        //In SingleSelection and ContiguousSelection once an item
+        //is selected, there's no way for the user to unselect all items
+        if (selectedColumnCount() == 1)
+            return false;
+        break;
+    case QAbstractItemView::ContiguousSelection:
+        if (selectedColumnCount() == 1)
+            return false;
+
+        if ((!column || view()->selectionModel()->isColumnSelected(column - 1, view()->rootIndex()))
+            && view()->selectionModel()->isColumnSelected(column + 1, view()->rootIndex())) {
+            //If there are columns selected both at the left of the current row and at the right
+            //of the current rown, the ones which are at the right will be deselected
+            selection = QItemSelection(index, view()->model()->index(0, columnCount() - 1, view()->rootIndex()));
+        }
+    default:
+        break;
+    }
+
+    view()->selectionModel()->select(selection, QItemSelectionModel::Deselect | QItemSelectionModel::Columns);
     return true;
 }
 
@@ -356,7 +402,7 @@ QAccessibleInterface *QAccessibleTable::childAt(int x, int y) const
 
     QModelIndex index = view()->indexAt(indexPosition);
     if (index.isValid()) {
-        return childFromLogical(logicalIndex(index));
+        return child(logicalIndex(index));
     }
     return 0;
 }
@@ -374,7 +420,7 @@ int QAccessibleTable::indexOfChild(const QAccessibleInterface *iface) const
 {
     if (!view()->model())
         return -1;
-    QSharedPointer<QAccessibleInterface> parent(iface->parent());
+    QAccessibleInterface *parent = iface->parent();
     if (parent->object() != view())
         return -1;
 
@@ -387,7 +433,7 @@ int QAccessibleTable::indexOfChild(const QAccessibleInterface *iface) const
         return cell->index + (verticalHeader() ? 1 : 0);
     } else if (iface->role() == QAccessible::RowHeader){
         const QAccessibleTableHeaderCell* cell = static_cast<const QAccessibleTableHeaderCell*>(iface);
-        return (cell->index + 1) * (view()->model()->rowCount() + 1);
+        return (cell->index + 1) * (view()->model()->columnCount() + 1);
     } else if (iface->role() == QAccessible::Pane) {
         return 0; // corner button
     } else {
@@ -415,7 +461,7 @@ QRect QAccessibleTable::rect() const
 
 QAccessibleInterface *QAccessibleTable::parent() const
 {
-    if (view()->parent()) {
+    if (view() && view()->parent()) {
         if (qstrcmp("QComboBoxPrivateContainer", view()->parent()->metaObject()->className()) == 0) {
             return QAccessible::queryAccessibleInterface(view()->parent()->parent());
         }
@@ -424,9 +470,55 @@ QAccessibleInterface *QAccessibleTable::parent() const
     return 0;
 }
 
-QAccessibleInterface *QAccessibleTable::child(int index) const
+QAccessibleInterface *QAccessibleTable::child(int logicalIndex) const
 {
-    return childFromLogical(index);
+    if (!view()->model())
+        return 0;
+
+    if (childToId.contains(logicalIndex)) {
+        QAccessible::Id id = childToId.value(logicalIndex);
+        return QAccessible::accessibleInterface(id);
+    }
+
+    int vHeader = verticalHeader() ? 1 : 0;
+    int hHeader = horizontalHeader() ? 1 : 0;
+
+    int columns = view()->model()->columnCount() + vHeader;
+
+    int row = logicalIndex / columns;
+    int column = logicalIndex % columns;
+
+    QAccessibleInterface *iface = 0;
+
+    if (vHeader) {
+        if (column == 0) {
+            if (hHeader && row == 0) {
+                iface = new QAccessibleTableCornerButton(view());
+            } else {
+                iface = new QAccessibleTableHeaderCell(view(), row - hHeader, Qt::Vertical);
+            }
+        }
+        --column;
+    }
+    if (!iface && hHeader) {
+        if (row == 0) {
+            iface = new QAccessibleTableHeaderCell(view(), column, Qt::Horizontal);
+        }
+        --row;
+    }
+
+    if (!iface) {
+        QModelIndex index = view()->model()->index(row, column, view()->rootIndex());
+        if (!index.isValid()) {
+            qWarning() << "QAccessibleTable::child: Invalid index at: " << row << column;
+            return 0;
+        }
+        iface = new QAccessibleTableCell(view(), index, cellRole());
+    }
+
+    QAccessible::registerAccessibleInterface(iface);
+    childToId.insert(logicalIndex, QAccessible::uniqueId(iface));
+    return iface;
 }
 
 void *QAccessibleTable::interface_cast(QAccessible::InterfaceType t)
@@ -436,6 +528,140 @@ void *QAccessibleTable::interface_cast(QAccessible::InterfaceType t)
    return 0;
 }
 
+void QAccessibleTable::modelChange(QAccessibleTableModelChangeEvent *event)
+{
+    // if there is no cache yet, we don't update anything
+    if (childToId.isEmpty())
+        return;
+
+    switch (event->modelChangeType()) {
+    case QAccessibleTableModelChangeEvent::ModelReset:
+        Q_FOREACH (QAccessible::Id id, childToId.values())
+            QAccessible::deleteAccessibleInterface(id);
+        childToId.clear();
+        break;
+
+    // rows are inserted: move every row after that
+    case QAccessibleTableModelChangeEvent::RowsInserted:
+    case QAccessibleTableModelChangeEvent::ColumnsInserted: {
+        int newRows = event->lastRow() - event->firstRow() + 1;
+        int newColumns = event->lastColumn() - event->firstColumn() + 1;
+
+        ChildCache newCache;
+        ChildCache::ConstIterator iter = childToId.constBegin();
+
+        while (iter != childToId.constEnd()) {
+            QAccessible::Id id = iter.value();
+            QAccessibleInterface *iface = QAccessible::accessibleInterface(id);
+            Q_ASSERT(iface);
+            if (iface->role() == QAccessible::Cell || iface->role() == QAccessible::ListItem) {
+                Q_ASSERT(iface->tableCellInterface());
+                QAccessibleTableCell *cell = static_cast<QAccessibleTableCell*>(iface->tableCellInterface());
+                if (event->modelChangeType() == QAccessibleTableModelChangeEvent::RowsInserted
+                        && cell->m_index.row() >= event->firstRow()) {
+                    int newRow = cell->m_index.row() + newRows;
+                    cell->m_index = cell->m_index.sibling(newRow, cell->m_index.column());
+                } else if (event->modelChangeType() == QAccessibleTableModelChangeEvent::ColumnsInserted
+                        && cell->m_index.column() >= event->firstColumn()) {
+                    int newColumn = cell->m_index.column() + newColumns;
+                    cell->m_index = cell->m_index.sibling(cell->m_index.row(), newColumn);
+                }
+            } else if (event->modelChangeType() == QAccessibleTableModelChangeEvent::RowsInserted
+                       && iface->role() == QAccessible::RowHeader) {
+                QAccessibleTableHeaderCell *cell = static_cast<QAccessibleTableHeaderCell*>(iface);
+                if (cell->index >= event->firstRow()) {
+                    cell->index += newRows;
+                }
+            } else if (event->modelChangeType() == QAccessibleTableModelChangeEvent::ColumnsInserted
+                   && iface->role() == QAccessible::ColumnHeader) {
+                QAccessibleTableHeaderCell *cell = static_cast<QAccessibleTableHeaderCell*>(iface);
+                if (cell->index >= event->firstColumn()) {
+                    cell->index += newColumns;
+                }
+            }
+            if (indexOfChild(iface) >= 0) {
+                newCache.insert(indexOfChild(iface), id);
+            } else {
+                // ### This should really not happen,
+                // but it might if the view has a root index set.
+                // This needs to be fixed.
+                QAccessible::deleteAccessibleInterface(id);
+            }
+            ++iter;
+        }
+        childToId = newCache;
+        break;
+    }
+
+    case QAccessibleTableModelChangeEvent::ColumnsRemoved:
+    case QAccessibleTableModelChangeEvent::RowsRemoved: {
+        int deletedColumns = event->lastColumn() - event->firstColumn() + 1;
+        int deletedRows = event->lastRow() - event->firstRow() + 1;
+        ChildCache newCache;
+        ChildCache::ConstIterator iter = childToId.constBegin();
+        while (iter != childToId.constEnd()) {
+            QAccessible::Id id = iter.value();
+            QAccessibleInterface *iface = QAccessible::accessibleInterface(id);
+            Q_ASSERT(iface);
+            if (iface->role() == QAccessible::Cell || iface->role() == QAccessible::ListItem) {
+                Q_ASSERT(iface->tableCellInterface());
+                QAccessibleTableCell *cell = static_cast<QAccessibleTableCell*>(iface->tableCellInterface());
+                if (event->modelChangeType() == QAccessibleTableModelChangeEvent::RowsRemoved) {
+                    if (cell->m_index.row() < event->firstRow()) {
+                        newCache.insert(indexOfChild(cell), id);
+                    } else if (cell->m_index.row() > event->lastRow()) {
+                        int newRow = cell->m_index.row() - deletedRows;
+                        cell->m_index = cell->m_index.sibling(newRow, cell->m_index.column());
+                        newCache.insert(indexOfChild(cell), id);
+                    } else {
+                        QAccessible::deleteAccessibleInterface(id);
+                    }
+                } else if (event->modelChangeType() == QAccessibleTableModelChangeEvent::ColumnsRemoved) {
+                    if (cell->m_index.column() < event->firstColumn()) {
+                        newCache.insert(indexOfChild(cell), id);
+                    } else if (cell->m_index.column() > event->lastColumn()) {
+                        int newColumn = cell->m_index.column() - deletedColumns;
+                        cell->m_index = cell->m_index.sibling(cell->m_index.row(), newColumn);
+                        newCache.insert(indexOfChild(cell), id);
+                    } else {
+                        QAccessible::deleteAccessibleInterface(id);
+                    }
+                }
+            } else if (event->modelChangeType() == QAccessibleTableModelChangeEvent::RowsRemoved
+                       && iface->role() == QAccessible::RowHeader) {
+                QAccessibleTableHeaderCell *cell = static_cast<QAccessibleTableHeaderCell*>(iface);
+                if (cell->index < event->firstRow()) {
+                    newCache.insert(indexOfChild(cell), id);
+                } else if (cell->index > event->lastRow()) {
+                    cell->index -= deletedRows;
+                    newCache.insert(indexOfChild(cell), id);
+                } else {
+                    QAccessible::deleteAccessibleInterface(id);
+                }
+            } else if (event->modelChangeType() == QAccessibleTableModelChangeEvent::ColumnsRemoved
+                   && iface->role() == QAccessible::ColumnHeader) {
+                QAccessibleTableHeaderCell *cell = static_cast<QAccessibleTableHeaderCell*>(iface);
+                if (cell->index < event->firstColumn()) {
+                    newCache.insert(indexOfChild(cell), id);
+                } else if (cell->index > event->lastColumn()) {
+                    cell->index -= deletedColumns;
+                    newCache.insert(indexOfChild(cell), id);
+                } else {
+                    QAccessible::deleteAccessibleInterface(id);
+                }
+            }
+            ++iter;
+        }
+        childToId = newCache;
+        break;
+    }
+
+    case QAccessibleTableModelChangeEvent::DataChanged:
+        // nothing to do in this case
+        break;
+    }
+}
+
 // TREE VIEW
 
 QModelIndex QAccessibleTree::indexFromLogical(int row, int column) const
@@ -487,27 +713,33 @@ int QAccessibleTree::childCount() const
 }
 
 
-QAccessibleInterface *QAccessibleTree::child(int index) const
+QAccessibleInterface *QAccessibleTree::child(int logicalIndex) const
 {
-    if (index < 0 || !view()->model() || !view()->model()->columnCount())
+    if (logicalIndex < 0 || !view()->model() || !view()->model()->columnCount())
         return 0;
-    int hHeader = horizontalHeader() ? 1 : 0;
 
-    if (hHeader) {
+    QAccessibleInterface *iface = 0;
+    int index = logicalIndex;
+
+    if (horizontalHeader()) {
         if (index < view()->model()->columnCount()) {
-            return new QAccessibleTableHeaderCell(view(), index, Qt::Horizontal);
+            iface = new QAccessibleTableHeaderCell(view(), index, Qt::Horizontal);
         } else {
             index -= view()->model()->columnCount();
         }
     }
 
-    int row = index / view()->model()->columnCount();
-    int column = index % view()->model()->columnCount();
-    QModelIndex modelIndex = indexFromLogical(row, column);
-    if (modelIndex.isValid()) {
-        return cell(modelIndex);
+    if (!iface) {
+        int row = index / view()->model()->columnCount();
+        int column = index % view()->model()->columnCount();
+        QModelIndex modelIndex = indexFromLogical(row, column);
+        if (!modelIndex.isValid())
+            return 0;
+        iface = new QAccessibleTableCell(view(), modelIndex, cellRole());
     }
-    return 0;
+    QAccessible::registerAccessibleInterface(iface);
+    // ### FIXME: get interfaces from the cache instead of re-creating them
+    return iface;
 }
 
 int QAccessibleTree::rowCount() const
@@ -521,7 +753,7 @@ int QAccessibleTree::indexOfChild(const QAccessibleInterface *iface) const
 {
     if (!view()->model())
         return -1;
-    QSharedPointer<QAccessibleInterface> parent(iface->parent());
+    QAccessibleInterface *parent = iface->parent();
     if (parent->object() != view())
         return -1;
 
@@ -533,12 +765,9 @@ int QAccessibleTree::indexOfChild(const QAccessibleInterface *iface) const
         int column = cell->m_index.column();
 
         int index = row * view()->model()->columnCount() + column;
-        //qDebug() << "QAccessibleTree::indexOfChild r " << row << " c " << column << "index " << index;
-        Q_ASSERT(index >= treeView->model()->columnCount());
         return index;
     } else if (iface->role() == QAccessible::ColumnHeader){
         const QAccessibleTableHeaderCell* cell = static_cast<const QAccessibleTableHeaderCell*>(iface);
-        //qDebug() << "QAccessibleTree::indexOfChild header " << cell->index;
         return cell->index;
     } else {
         qWarning() << "WARNING QAccessibleTable::indexOfChild invalid child"
@@ -555,7 +784,11 @@ QAccessibleInterface *QAccessibleTree::cellAt(int row, int column) const
         qWarning() << "Requested invalid tree cell: " << row << column;
         return 0;
     }
-    return new QAccessibleTableCell(view(), index, cellRole());
+    const QTreeView *treeView = qobject_cast<const QTreeView*>(view());
+    Q_ASSERT(treeView);
+    int logicalIndex = treeView->d_func()->accessibleTable2Index(index);
+
+    return child(logicalIndex); // FIXME ### new QAccessibleTableCell(view(), index, cellRole());
 }
 
 QString QAccessibleTree::rowDescription(int) const
@@ -576,9 +809,28 @@ bool QAccessibleTree::selectRow(int row)
     if (!view()->selectionModel())
         return false;
     QModelIndex index = indexFromLogical(row);
-    if (!index.isValid() || view()->selectionMode() & QAbstractItemView::NoSelection)
+
+    if (!index.isValid() || view()->selectionBehavior() == QAbstractItemView::SelectColumns)
         return false;
-    view()->selectionModel()->select(index, QItemSelectionModel::Select);
+
+    switch (view()->selectionMode()) {
+    case QAbstractItemView::NoSelection:
+        return false;
+    case QAbstractItemView::SingleSelection:
+        if ((view()->selectionBehavior() != QAbstractItemView::SelectRows) && (columnCount() > 1))
+            return false;
+        view()->clearSelection();
+        break;
+    case QAbstractItemView::ContiguousSelection:
+        if ((!row || !view()->selectionModel()->isRowSelected(row - 1, view()->rootIndex()))
+            && !view()->selectionModel()->isRowSelected(row + 1, view()->rootIndex()))
+            view()->clearSelection();
+        break;
+    default:
+        break;
+    }
+
+    view()->selectionModel()->select(index, QItemSelectionModel::Select | QItemSelectionModel::Rows);
     return true;
 }
 
@@ -595,6 +847,8 @@ void *QAccessibleTableCell::interface_cast(QAccessible::InterfaceType t)
 {
     if (t == QAccessible::TableCellInterface)
         return static_cast<QAccessibleTableCellInterface*>(this);
+    if (t == QAccessible::ActionInterface)
+        return static_cast<QAccessibleActionInterface*>(this);
     return 0;
 }
 
@@ -605,6 +859,7 @@ QList<QAccessibleInterface*> QAccessibleTableCell::rowHeaderCells() const
 {
     QList<QAccessibleInterface*> headerCell;
     if (verticalHeader()) {
+        // FIXME
         headerCell.append(new QAccessibleTableHeaderCell(view, m_index.row(), Qt::Vertical));
     }
     return headerCell;
@@ -614,6 +869,7 @@ QList<QAccessibleInterface*> QAccessibleTableCell::columnHeaderCells() const
 {
     QList<QAccessibleInterface*> headerCell;
     if (horizontalHeader()) {
+        // FIXME
         headerCell.append(new QAccessibleTableHeaderCell(view, m_index.column(), Qt::Horizontal));
     }
     return headerCell;
@@ -668,13 +924,87 @@ bool QAccessibleTableCell::isSelected() const
     return view->selectionModel()->isSelected(m_index);
 }
 
-void QAccessibleTableCell::rowColumnExtents(int *row, int *column, int *rowExtents, int *columnExtents, bool *selected) const
+QStringList QAccessibleTableCell::actionNames() const
 {
-    *row = m_index.row();
-    *column = m_index.column();
-    *rowExtents = 1;
-    *columnExtents = 1;
-    *selected = isSelected();
+    QStringList names;
+    names << toggleAction();
+    return names;
+}
+
+void QAccessibleTableCell::doAction(const QString& actionName)
+{
+    if (actionName == toggleAction()) {
+        if (isSelected())
+            unselectCell();
+        else
+            selectCell();
+    }
+}
+
+QStringList QAccessibleTableCell::keyBindingsForAction(const QString &) const
+{
+    return QStringList();
+}
+
+
+void QAccessibleTableCell::selectCell()
+{
+    QAbstractItemView::SelectionMode selectionMode = view->selectionMode();
+    if (!m_index.isValid() || (selectionMode == QAbstractItemView::NoSelection))
+        return;
+    Q_ASSERT(table());
+    QAccessibleTableInterface *cellTable = table()->tableInterface();
+
+    switch (view->selectionBehavior()) {
+    case QAbstractItemView::SelectItems:
+        break;
+    case QAbstractItemView::SelectColumns:
+        if (cellTable)
+            cellTable->selectColumn(m_index.column());
+        return;
+    case QAbstractItemView::SelectRows:
+        if (cellTable)
+            cellTable->selectRow(m_index.row());
+        return;
+    }
+
+    if (selectionMode == QAbstractItemView::SingleSelection) {
+        view->clearSelection();
+    }
+
+    view->selectionModel()->select(m_index, QItemSelectionModel::Select);
+}
+
+void QAccessibleTableCell::unselectCell()
+{
+
+    QAbstractItemView::SelectionMode selectionMode = view->selectionMode();
+    if (!m_index.isValid() || (selectionMode & QAbstractItemView::NoSelection))
+        return;
+
+    QAccessibleTableInterface *cellTable = table()->tableInterface();
+
+    switch (view->selectionBehavior()) {
+    case QAbstractItemView::SelectItems:
+        break;
+    case QAbstractItemView::SelectColumns:
+        if (cellTable)
+            cellTable->unselectColumn(m_index.column());
+        return;
+    case QAbstractItemView::SelectRows:
+        if (cellTable)
+            cellTable->unselectRow(m_index.row());
+        return;
+    }
+
+    //If the mode is not MultiSelection or ExtendedSelection and only
+    //one cell is selected it cannot be unselected by the user
+    if ((selectionMode != QAbstractItemView::MultiSelection)
+        && (selectionMode != QAbstractItemView::ExtendedSelection)
+        && (view->selectionModel()->selectedIndexes().count() <= 1))
+        return;
+
+    view->selectionModel()->select(m_index, QItemSelectionModel::Deselect);
 }
 
 QAccessibleInterface *QAccessibleTableCell::table() const
@@ -767,10 +1097,7 @@ bool QAccessibleTableCell::isValid() const
 
 QAccessibleInterface *QAccessibleTableCell::parent() const
 {
-    if (m_role == QAccessible::TreeItem)
-        return new QAccessibleTree(view);
-
-    return new QAccessibleTable(view);
+    return QAccessible::queryAccessibleInterface(view);
 }
 
 QAccessibleInterface *QAccessibleTableCell::child(int) const
@@ -793,7 +1120,12 @@ QAccessible::Role QAccessibleTableHeaderCell::role() const
 
 QAccessible::State QAccessibleTableHeaderCell::state() const
 {
-    return QAccessible::State();
+    QAccessible::State s;
+    if (QHeaderView *h = headerView()) {
+        s.invisible = !h->testAttribute(Qt::WA_WState_Visible);
+        s.disabled = !h->isEnabled();
+    }
+    return s;
 }
 
 QRect QAccessibleTableHeaderCell::rect() const
@@ -856,14 +1188,7 @@ bool QAccessibleTableHeaderCell::isValid() const
 
 QAccessibleInterface *QAccessibleTableHeaderCell::parent() const
 {
-    if (false) {
-#ifndef QT_NO_TREEVIEW
-    } else if (qobject_cast<const QTreeView*>(view)) {
-        return new QAccessibleTree(view);
-#endif
-    } else {
-        return new QAccessibleTable(view);
-    }
+    return QAccessible::queryAccessibleInterface(view);
 }
 
 QAccessibleInterface *QAccessibleTableHeaderCell::child(int) const
@@ -871,6 +1196,26 @@ QAccessibleInterface *QAccessibleTableHeaderCell::child(int) const
     return 0;
 }
 
+QHeaderView *QAccessibleTableHeaderCell::headerView() const
+{
+    QHeaderView *header = 0;
+    if (false) {
+#ifndef QT_NO_TABLEVIEW
+    } else if (const QTableView *tv = qobject_cast<const QTableView*>(view)) {
+        if (orientation == Qt::Horizontal) {
+            header = tv->horizontalHeader();
+        } else {
+            header = tv->verticalHeader();
+        }
+#endif
+#ifndef QT_NO_TREEVIEW
+    } else if (const QTreeView *tv = qobject_cast<const QTreeView*>(view)) {
+        header = tv->header();
+#endif
+    }
+    return header;
+}
+
 #endif // QT_NO_ITEMVIEWS
 
 QT_END_NAMESPACE
