@@ -72,8 +72,6 @@
 
 QT_BEGIN_NAMESPACE
 
-using namespace QAccessible2;
-
 QString Q_GUI_EXPORT qt_accStripAmp(const QString &text);
 QString Q_GUI_EXPORT qt_accHotKey(const QString &text);
 
@@ -432,8 +430,9 @@ QAccessible::State QAccessibleMdiSubWindow::state() const
         state.focused = true;
     if (!mdiSubWindow()->isVisible())
         state.invisible = true;
-    if (!mdiSubWindow()->parentWidget()->contentsRect().contains(mdiSubWindow()->geometry()))
-        state.offscreen = true;
+    if (const QWidget *parent = mdiSubWindow()->parentWidget())
+        if (!parent->contentsRect().contains(mdiSubWindow()->geometry()))
+            state.offscreen = true;
     if (!mdiSubWindow()->isEnabled())
         state.disabled = true;
     return state;
@@ -559,37 +558,56 @@ QWidget *QAccessibleCalendarWidget::navigationBar() const
 #endif // QT_NO_CALENDARWIDGET
 
 #ifndef QT_NO_DOCKWIDGET
+
+// Dock Widget - order of children:
+// - Content widget
+// - Float button
+// - Close button
+// If there is a custom title bar widget, that one becomes child 1, after the content 0
+// (in that case the buttons are ignored)
 QAccessibleDockWidget::QAccessibleDockWidget(QWidget *widget)
     : QAccessibleWidget(widget, QAccessible::Window)
 {
-
 }
 
-QAccessibleInterface *QAccessibleDockWidget::child(int index) const
+QDockWidgetLayout *QAccessibleDockWidget::dockWidgetLayout() const
 {
-    if (index == 0) {
-        return new QAccessibleTitleBar(dockWidget());
-    } else if (index == 1 && dockWidget()->widget()) {
-        return QAccessible::queryAccessibleInterface(dockWidget()->widget());
-    }
-    return 0;
+    return qobject_cast<QDockWidgetLayout*>(dockWidget()->layout());
 }
 
 int QAccessibleDockWidget::childCount() const
 {
-    return dockWidget()->widget() ? 2 : 1;
+    if (dockWidget()->titleBarWidget()) {
+        return dockWidget()->widget() ? 2 : 1;
+    }
+    return dockWidgetLayout()->count();
+}
+
+QAccessibleInterface *QAccessibleDockWidget::child(int index) const
+{
+    if (dockWidget()->titleBarWidget()) {
+        if ((!dockWidget()->widget() && index == 0) || (index == 1))
+            return QAccessible::queryAccessibleInterface(dockWidget()->titleBarWidget());
+        if (index == 0)
+            return QAccessible::queryAccessibleInterface(dockWidget()->widget());
+    } else {
+        QLayoutItem *item = dockWidgetLayout()->itemAt(index);
+        if (item)
+            return QAccessible::queryAccessibleInterface(item->widget());
+    }
+    return 0;
 }
 
 int QAccessibleDockWidget::indexOfChild(const QAccessibleInterface *child) const
 {
-    if (child) {
-        if (child->role() == QAccessible::TitleBar) {
-            return 0;
-        } else {
-            return 1;   // FIXME
-        }
+    if (!child || !child->object() || child->object()->parent() != object())
+        return -1;
+
+    if (dockWidget()->titleBarWidget() == child->object()) {
+        return dockWidget()->widget() ? 1 : 0;
     }
-    return -1;
+
+    return dockWidgetLayout()->indexOf(qobject_cast<QWidget*>(child->object()));
 }
 
 QRect QAccessibleDockWidget::rect() const
@@ -611,191 +629,13 @@ QDockWidget *QAccessibleDockWidget::dockWidget() const
     return static_cast<QDockWidget *>(object());
 }
 
-////
-//      QAccessibleTitleBar
-////
-QAccessibleTitleBar::QAccessibleTitleBar(QDockWidget *widget)
-    : m_dockWidget(widget)
-{
-
-}
-
-QAccessibleInterface *QAccessibleTitleBar::parent() const
-{
-    return new QAccessibleDockWidget(dockWidget());
-}
-
-QAccessibleInterface *QAccessibleTitleBar::child(int index) const
-{
-    if (index >= 0) {
-        QDockWidgetLayout *layout = dockWidgetLayout();
-        int role;
-        int currentIndex = 0;
-        for (role = QDockWidgetLayout::CloseButton; role <= QDockWidgetLayout::FloatButton; ++role) {
-            QWidget *w = layout->widgetForRole((QDockWidgetLayout::Role)role);
-            if (!w || !w->isVisible())
-                continue;
-            if (currentIndex == index)
-                return QAccessible::queryAccessibleInterface(w);
-            ++currentIndex;
-        }
-    }
-    return 0;
-}
-
-int QAccessibleTitleBar::indexOfChild(const QAccessibleInterface * /*child*/) const
-{
-    return -1;
-}
-
-int QAccessibleTitleBar::childCount() const
-{
-    QDockWidgetLayout *layout = dockWidgetLayout();
-    int count = 0;
-    for (int role = QDockWidgetLayout::CloseButton; role <= QDockWidgetLayout::FloatButton; ++role) {
-        QWidget *w = layout->widgetForRole((QDockWidgetLayout::Role)role);
-        if (w && w->isVisible())
-            ++count;
-    }
-    return count;
-}
-
-QString QAccessibleTitleBar::text(QAccessible::Text t) const
+QString QAccessibleDockWidget::text(QAccessible::Text t) const
 {
     if (t == QAccessible::Name || t == QAccessible::Value) {
         return qt_accStripAmp(dockWidget()->windowTitle());
     }
     return QString();
 }
-
-QAccessible::State QAccessibleTitleBar::state() const
-{
-    QAccessible::State state;
-
-    QDockWidget *w = dockWidget();
-    if (w->testAttribute(Qt::WA_WState_Visible) == false)
-        state.invisible = true;
-    if (w->focusPolicy() != Qt::NoFocus && w->isActiveWindow())
-        state.focusable = true;
-    if (w->hasFocus())
-        state.focused = true;
-    if (!w->isEnabled())
-        state.disabled = true;
-
-    return state;
-}
-
-QRect QAccessibleTitleBar::rect() const
-{
-    bool mapToGlobal = true;
-    QRect rect;
-
-    if (dockWidget()->isFloating()) {
-        rect = dockWidget()->frameGeometry();
-        if (dockWidget()->widget()) {
-            QPoint globalPos = dockWidget()->mapToGlobal(dockWidget()->widget()->rect().topLeft());
-            globalPos.ry()--;
-            rect.setBottom(globalPos.y());
-            mapToGlobal = false;
-        }
-    } else {
-        QDockWidgetLayout *layout = qobject_cast<QDockWidgetLayout*>(dockWidget()->layout());
-        rect = layout->titleArea();
-    }
-
-    if (rect.isNull())
-        return rect;
-
-    if (mapToGlobal)
-        rect.moveTopLeft(dockWidget()->mapToGlobal(rect.topLeft()));
-    return rect;
-}
-
-QAccessibleInterface *QAccessibleTitleBar::childAt(int x, int y) const
-{
-    for (int i = 0; i < childCount(); ++i) {
-        QAccessibleInterface *childIface = child(i);
-        if (childIface->rect().contains(x,y)) {
-            return childIface;
-        }
-        delete childIface;
-    }
-    return 0;
-}
-
-QObject *QAccessibleTitleBar::object() const
-{
-    return 0;
-}
-
-QDockWidgetLayout *QAccessibleTitleBar::dockWidgetLayout() const
-{
-    return qobject_cast<QDockWidgetLayout*>(dockWidget()->layout());
-}
-
-QDockWidget *QAccessibleTitleBar::dockWidget() const
-{
-    return m_dockWidget;
-}
-
-//QString QAccessibleTitleBar::actionText(int action, Text t, int child) const
-//{
-//    QString str;
-//    if (child >= 1 && child <= childCount()) {
-//        if (t == Name) {
-//            switch (action) {
-//            case Press:
-//            case DefaultAction:
-//                if (child == QDockWidgetLayout::CloseButton) {
-//                    str = QDockWidget::tr("Close");
-//                } else if (child == QDockWidgetLayout::FloatButton) {
-//                    str = dockWidget()->isFloating() ? QDockWidget::tr("Dock")
-//                                                     : QDockWidget::tr("Float");
-//                }
-//                break;
-//            default:
-//                break;
-//            }
-//        }
-//    }
-//    return str;
-//}
-
-//bool QAccessibleTitleBar::doAction(int action, int child, const QVariantList& /*params*/)
-//{
-//    if (!child || !dockWidget()->isEnabled())
-//        return false;
-
-//    switch (action) {
-//    case DefaultAction:
-//    case Press: {
-//        QDockWidgetLayout *layout = dockWidgetLayout();
-//        QAbstractButton *btn = static_cast<QAbstractButton *>(layout->widgetForRole((QDockWidgetLayout::Role)child));
-//        if (btn)
-//            btn->animateClick();
-//        return true;
-//        break;}
-//    default:
-//        break;
-//    }
-
-//    return false;
-//}
-
-QAccessible::Role QAccessibleTitleBar::role() const
-{
-    return QAccessible::TitleBar;
-}
-
-void QAccessibleTitleBar::setText(QAccessible::Text /*t*/, const QString &/*text*/)
-{
-}
-
-bool QAccessibleTitleBar::isValid() const
-{
-    return dockWidget();
-}
-
 #endif // QT_NO_DOCKWIDGET
 
 #ifndef QT_NO_CURSOR
@@ -998,83 +838,18 @@ QPoint QAccessibleTextWidget::scrollBarPosition() const
     return QPoint(0, 0);
 }
 
-QPair< int, int > QAccessibleTextWidget::getBoundaries(int offset, BoundaryType boundaryType) const
-{
-    if (offset >= characterCount())
-        return QPair<int, int>(characterCount(), characterCount());
-    if (offset < 0)
-        return QPair<int, int>(0, 0);
 
-    QTextCursor cursor = textCursor();
-    QPair<int, int> result;
-
-    cursor.setPosition(offset);
-    switch (boundaryType) {
-    case CharBoundary:
-        result.first = cursor.position();
-        cursor.movePosition(QTextCursor::NextCharacter, QTextCursor::KeepAnchor);
-        result.second = cursor.position();
-        break;
-    case WordBoundary:
-        cursor.movePosition(QTextCursor::StartOfWord, QTextCursor::MoveAnchor);
-        result.first = cursor.position();
-        cursor.movePosition(QTextCursor::EndOfWord, QTextCursor::KeepAnchor);
-        result.second = cursor.position();
-        break;
-    case SentenceBoundary: {
-        // QCursor does not provide functionality to move to next sentence.
-        // We therefore find the current block, then go through the block using
-        // QTextBoundaryFinder and find the sentence the \offset represents
-        cursor.movePosition(QTextCursor::StartOfBlock, QTextCursor::MoveAnchor);
-        result.first = cursor.position();
-        cursor.movePosition(QTextCursor::EndOfBlock, QTextCursor::KeepAnchor);
-        result.second = cursor.position();
-        QString blockText = cursor.selectedText();
-        const int offsetWithinBlockText = offset - result.first;
-        QTextBoundaryFinder sentenceFinder(QTextBoundaryFinder::Sentence, blockText);
-        sentenceFinder.setPosition(offsetWithinBlockText);
-        int prevBoundary = offsetWithinBlockText;
-        int nextBoundary = offsetWithinBlockText;
-        if (!(sentenceFinder.boundaryReasons() & QTextBoundaryFinder::StartOfItem))
-            prevBoundary = sentenceFinder.toPreviousBoundary();
-        nextBoundary = sentenceFinder.toNextBoundary();
-        if (nextBoundary != -1)
-            result.second = result.first + nextBoundary;
-        if (prevBoundary != -1)
-            result.first += prevBoundary;
-        break; }
-    case LineBoundary:
-        cursor.movePosition(QTextCursor::StartOfLine, QTextCursor::MoveAnchor);
-        result.first = cursor.position();
-        cursor.movePosition(QTextCursor::EndOfLine, QTextCursor::KeepAnchor);
-        result.second = cursor.position();
-        break;
-    case ParagraphBoundary:
-        cursor.movePosition(QTextCursor::StartOfBlock, QTextCursor::MoveAnchor);
-        result.first = cursor.position();
-        cursor.movePosition(QTextCursor::EndOfBlock, QTextCursor::KeepAnchor);
-        result.second = cursor.position();
-        break;
-    case NoBoundary:
-        result.first = 0;
-        result.second = characterCount();
-        break;
-    default:
-        qWarning("QAccessibleTextWidget::getBoundaries: Unknown boundary type %d", boundaryType);
-        result.first = -1;
-        result.second = -1;
-    }
-    return result;
-}
-
-QString QAccessibleTextWidget::textBeforeOffset(int offset, BoundaryType boundaryType,
+QString QAccessibleTextWidget::textBeforeOffset(int offset, QAccessible::TextBoundaryType boundaryType,
                                                 int *startOffset, int *endOffset) const
 {
     Q_ASSERT(startOffset);
     Q_ASSERT(endOffset);
 
-    QPair<int, int> boundaries = getBoundaries(offset, boundaryType);
-    boundaries = getBoundaries(boundaries.first - 1, boundaryType);
+    QTextCursor cursor = textCursor();
+    cursor.setPosition(offset);
+    QPair<int, int> boundaries = QAccessible::qAccessibleTextBoundaryHelper(cursor, boundaryType);
+    cursor.setPosition(boundaries.first - 1);
+    boundaries = QAccessible::qAccessibleTextBoundaryHelper(cursor, boundaryType);
 
     *startOffset = boundaries.first;
     *endOffset = boundaries.second;
@@ -1083,14 +858,17 @@ QString QAccessibleTextWidget::textBeforeOffset(int offset, BoundaryType boundar
  }
 
 
-QString QAccessibleTextWidget::textAfterOffset(int offset, BoundaryType boundaryType,
+QString QAccessibleTextWidget::textAfterOffset(int offset, QAccessible::TextBoundaryType boundaryType,
                                               int *startOffset, int *endOffset) const
 {
     Q_ASSERT(startOffset);
     Q_ASSERT(endOffset);
 
-    QPair<int, int> boundaries = getBoundaries(offset, boundaryType);
-    boundaries = getBoundaries(boundaries.second, boundaryType);
+    QTextCursor cursor = textCursor();
+    cursor.setPosition(offset);
+    QPair<int, int> boundaries = QAccessible::qAccessibleTextBoundaryHelper(cursor, boundaryType);
+    cursor.setPosition(boundaries.second);
+    boundaries = QAccessible::qAccessibleTextBoundaryHelper(cursor, boundaryType);
 
     *startOffset = boundaries.first;
     *endOffset = boundaries.second;
@@ -1098,13 +876,15 @@ QString QAccessibleTextWidget::textAfterOffset(int offset, BoundaryType boundary
     return text(boundaries.first, boundaries.second);
 }
 
-QString QAccessibleTextWidget::textAtOffset(int offset, BoundaryType boundaryType,
+QString QAccessibleTextWidget::textAtOffset(int offset, QAccessible::TextBoundaryType boundaryType,
                                             int *startOffset, int *endOffset) const
 {
     Q_ASSERT(startOffset);
     Q_ASSERT(endOffset);
 
-    QPair<int, int> boundaries = getBoundaries(offset, boundaryType);
+    QTextCursor cursor = textCursor();
+    cursor.setPosition(offset);
+    QPair<int, int> boundaries = QAccessible::qAccessibleTextBoundaryHelper(cursor, boundaryType);
 
     *startOffset = boundaries.first;
     *endOffset = boundaries.second;
@@ -1190,7 +970,7 @@ QAccessibleMainWindow::QAccessibleMainWindow(QWidget *widget)
 QAccessibleInterface *QAccessibleMainWindow::child(int index) const
 {
     QList<QWidget*> kids = childWidgets(mainWindow(), true);
-    if (index < kids.count()) {
+    if (index >= 0 && index < kids.count()) {
         return QAccessible::queryAccessibleInterface(kids.at(index));
     }
     return 0;
