@@ -67,7 +67,6 @@ QT_BEGIN_NAMESPACE
 
 #ifndef QT_NO_ACCESSIBILITY
 
-using namespace QAccessible2;
 extern QList<QWidget*> childWidgets(const QWidget *widget, bool includeTopLevel = false);
 
 QString Q_GUI_EXPORT qt_accStripAmp(const QString &text);
@@ -236,7 +235,7 @@ QToolButton *QAccessibleToolButton::toolButton() const
 }
 
 /*!
-    Returns true if this tool button is a split button.
+    Returns \c true if this tool button is a split button.
 */
 bool QAccessibleToolButton::isSplitButton() const
 {
@@ -475,16 +474,16 @@ QSize QAccessibleDisplay::imageSize() const
 }
 
 /*! \internal */
-QRect QAccessibleDisplay::imagePosition() const
+QPoint QAccessibleDisplay::imagePosition() const
 {
     QLabel *label = qobject_cast<QLabel *>(widget());
     if (!label)
-        return QRect();
+        return QPoint();
     const QPixmap *pixmap = label->pixmap();
     if (!pixmap)
-        return QRect();
+        return QPoint();
 
-    return QRect(label->mapToGlobal(label->pos()), label->size());
+    return QPoint(label->mapToGlobal(label->pos()));
 }
 
 #ifndef QT_NO_GROUPBOX
@@ -606,7 +605,7 @@ QString QAccessibleLineEdit::text(QAccessible::Text t) const
         break;
     }
     if (str.isEmpty())
-        str = QAccessibleWidget::text(t);;
+        str = QAccessibleWidget::text(t);
     return qt_accStripAmp(str);
 }
 
@@ -724,7 +723,7 @@ QString QAccessibleLineEdit::text(int startOffset, int endOffset) const
     return lineEdit()->text().mid(startOffset, endOffset - startOffset);
 }
 
-QString QAccessibleLineEdit::textBeforeOffset(int offset, BoundaryType boundaryType,
+QString QAccessibleLineEdit::textBeforeOffset(int offset, QAccessible::TextBoundaryType boundaryType,
         int *startOffset, int *endOffset) const
 {
     if (lineEdit()->echoMode() != QLineEdit::Normal) {
@@ -734,7 +733,7 @@ QString QAccessibleLineEdit::textBeforeOffset(int offset, BoundaryType boundaryT
     return QAccessibleTextInterface::textBeforeOffset(offset, boundaryType, startOffset, endOffset);
 }
 
-QString QAccessibleLineEdit::textAfterOffset(int offset, BoundaryType boundaryType,
+QString QAccessibleLineEdit::textAfterOffset(int offset, QAccessible::TextBoundaryType boundaryType,
         int *startOffset, int *endOffset) const
 {
     if (lineEdit()->echoMode() != QLineEdit::Normal) {
@@ -744,7 +743,7 @@ QString QAccessibleLineEdit::textAfterOffset(int offset, BoundaryType boundaryTy
     return QAccessibleTextInterface::textAfterOffset(offset, boundaryType, startOffset, endOffset);
 }
 
-QString QAccessibleLineEdit::textAtOffset(int offset, BoundaryType boundaryType,
+QString QAccessibleLineEdit::textAtOffset(int offset, QAccessible::TextBoundaryType boundaryType,
         int *startOffset, int *endOffset) const
 {
     if (lineEdit()->echoMode() != QLineEdit::Normal) {
