@@ -74,13 +74,14 @@ const QDBusArgument &operator>>(const QDBusArgument &argument, ConnmanMap &map)
     return argument;
 }
 
-static QDBusConnection dbusConnection = QDBusConnection::systemBus();
 QConnmanManagerInterface::QConnmanManagerInterface( QObject *parent)
         : QDBusAbstractInterface(QLatin1String(CONNMAN_SERVICE),
                                  QLatin1String(CONNMAN_MANAGER_PATH),
                                  CONNMAN_MANAGER_INTERFACE,
                                  QDBusConnection::systemBus(), parent)
 {
+    qDBusRegisterMetaType<ConnmanMap>();
+    qDBusRegisterMetaType<ConnmanMapList>();
 }
 
 QConnmanManagerInterface::~QConnmanManagerInterface()
@@ -96,7 +97,7 @@ void QConnmanManagerInterface::connectNotify(const QMetaMethod &signal)
                                QLatin1String(CONNMAN_MANAGER_INTERFACE),
                                QLatin1String("PropertyChanged"),
                                this,SIGNAL(propertyChanged(QString,QDBusVariant)))) {
-            qWarning() << "PropertyCHanged not connected";
+            qWarning() << "PropertyChanged not connected";
         }
     }
 
@@ -116,7 +117,7 @@ void QConnmanManagerInterface::connectNotify(const QMetaMethod &signal)
         QConnmanDBusHelper *helper;
         helper = new QConnmanDBusHelper(this);
 
-        dbusConnection.connect(QLatin1String(CONNMAN_SERVICE),
+        QDBusConnection::systemBus().connect(QLatin1String(CONNMAN_SERVICE),
                                QLatin1String(CONNMAN_MANAGER_PATH),
                                QLatin1String(CONNMAN_MANAGER_INTERFACE),
                                QLatin1String("PropertyChanged"),
@@ -126,8 +127,24 @@ void QConnmanManagerInterface::connectNotify(const QMetaMethod &signal)
         QObject::connect(helper,SIGNAL(propertyChangedContext(QString,QString,QDBusVariant)),
                 this,SIGNAL(propertyChangedContext(QString,QString,QDBusVariant)), Qt::UniqueConnection);
     }
+    static const QMetaMethod servicesChangedSignal = QMetaMethod::fromSignal(&QConnmanManagerInterface::servicesChanged);
+    if (signal == servicesChangedSignal) {
+        if (!connection().connect(QLatin1String(CONNMAN_SERVICE),
+                               QLatin1String(CONNMAN_MANAGER_PATH),
+                               QLatin1String(CONNMAN_MANAGER_INTERFACE),
+                               QLatin1String("ServicesChanged"),
+                               this,SLOT(onServicesChanged(ConnmanMapList, QList<QDBusObjectPath>)))) {
+            qWarning() << "servicesChanged not connected";
+        }
+    }
 }
 
+void QConnmanManagerInterface::onServicesChanged(const ConnmanMapList &changed, const QList<QDBusObjectPath> &removed)
+{
+    emit servicesChanged(changed, removed);
+}
+
+
 void QConnmanManagerInterface::disconnectNotify(const QMetaMethod &signal)
 {
     static const QMetaMethod propertyChangedSignal = QMetaMethod::fromSignal(&QConnmanManagerInterface::propertyChanged);
@@ -377,7 +394,7 @@ void QConnmanProfileInterface::connectNotify(const QMetaMethod &signal)
 {
     static const QMetaMethod propertyChangedSignal = QMetaMethod::fromSignal(&QConnmanProfileInterface::propertyChanged);
     if (signal == propertyChangedSignal) {
-        dbusConnection.connect(QLatin1String(CONNMAN_SERVICE),
+        QDBusConnection::systemBus().connect(QLatin1String(CONNMAN_SERVICE),
                                this->path(),
                                QLatin1String(CONNMAN_PROFILE_INTERFACE),
                                QLatin1String("PropertyChanged"),
@@ -405,7 +422,7 @@ QVariant QConnmanProfileInterface::getProperty(const QString &property)
     QVariantMap map = getProperties();
     if (map.contains(property)) {
         var = map.value(property);
-    } 
+    }
     return var;
 }
 
@@ -447,7 +464,7 @@ void QConnmanServiceInterface::connectNotify(const QMetaMethod &signal)
 {
     static const QMetaMethod propertyChangedSignal = QMetaMethod::fromSignal(&QConnmanServiceInterface::propertyChanged);
     if (signal == propertyChangedSignal) {
-        dbusConnection.connect(QLatin1String(CONNMAN_SERVICE),
+        QDBusConnection::systemBus().connect(QLatin1String(CONNMAN_SERVICE),
                                this->path(),
                                QLatin1String(CONNMAN_SERVICE_INTERFACE),
                                QLatin1String("PropertyChanged"),
@@ -458,7 +475,7 @@ void QConnmanServiceInterface::connectNotify(const QMetaMethod &signal)
         QConnmanDBusHelper *helper;
         helper = new QConnmanDBusHelper(this);
 
-        dbusConnection.connect(QLatin1String(CONNMAN_SERVICE),
+        QDBusConnection::systemBus().connect(QLatin1String(CONNMAN_SERVICE),
                                this->path(),
                                QLatin1String(CONNMAN_SERVICE_INTERFACE),
                                QLatin1String("PropertyChanged"),
@@ -493,7 +510,7 @@ QVariant QConnmanServiceInterface::getProperty(const QString &property)
     QVariantMap map = getProperties();
     if (map.contains(property)) {
         var = map.value(property);
-    } 
+    }
     return var;
 }
 
@@ -512,9 +529,6 @@ void QConnmanServiceInterface::remove()
     QDBusReply<QVariantMap> reply = this->call(QLatin1String("Remove"));
 }
 
-// void moveBefore(QDBusObjectPath &service);
-// void moveAfter(QDBusObjectPath &service);
-
 // properties
 QString QConnmanServiceInterface::getState()
 {
@@ -777,7 +791,7 @@ void QConnmanTechnologyInterface::connectNotify(const QMetaMethod &signal)
 {
     static const QMetaMethod propertyChangedSignal = QMetaMethod::fromSignal(&QConnmanTechnologyInterface::propertyChanged);
     if (signal == propertyChangedSignal) {
-        dbusConnection.connect(QLatin1String(CONNMAN_SERVICE),
+        QDBusConnection::systemBus().connect(QLatin1String(CONNMAN_SERVICE),
                                this->path(),
                                QLatin1String(CONNMAN_TECHNOLOGY_INTERFACE),
                                QLatin1String("PropertyChanged"),
@@ -788,7 +802,7 @@ void QConnmanTechnologyInterface::connectNotify(const QMetaMethod &signal)
         QConnmanDBusHelper *helper;
         helper = new QConnmanDBusHelper(this);
 
-        dbusConnection.connect(QLatin1String(CONNMAN_SERVICE),
+        QDBusConnection::systemBus().connect(QLatin1String(CONNMAN_SERVICE),
                                this->path(),
                                QLatin1String(CONNMAN_TECHNOLOGY_INTERFACE),
                                QLatin1String("PropertyChanged"),
@@ -859,23 +873,11 @@ QConnmanAgentInterface::~QConnmanAgentInterface()
 void QConnmanAgentInterface::connectNotify(const QMetaMethod &signal)
 {
     Q_UNUSED(signal);
-//    static const QMetaMethod propertyChangedSignal = QMetaMethod::fromSignal(&QConnmanAgentInterface::propertyChanged);
-//    if (signal == propertyChangedSignal) {
-//        dbusConnection.connect(QLatin1String(CONNMAN_SERVICE),
-//                               this->path(),
-//                               QLatin1String(CONNMAN_NETWORK_INTERFACE),
-//                               QLatin1String("PropertyChanged"),
-//                               this,SIGNAL(propertyChanged(QString,QVariant&)));
-//    }
 }
 
 void QConnmanAgentInterface::disconnectNotify(const QMetaMethod &signal)
 {
     Q_UNUSED(signal);
-//    static const QMetaMethod propertyChangedSignal = QMetaMethod::fromSignal(&QConnmanAgentInterface::propertyChanged);
-//    if (signal == propertyChangedSignal) {
-
-//    }
 }
 
 
@@ -887,10 +889,6 @@ void QConnmanAgentInterface::reportError(QDBusObjectPath &/*path*/, const QStrin
 {
 }
 
-//dict QConnmanAgentInterface::requestInput(QDBusObjectPath &path, dict fields)
-//{
-//}
-
 void QConnmanAgentInterface::cancel()
 {
 }
