@@ -39,7 +39,7 @@
 **
 ****************************************************************************/
 
-/*! 
+/*!
     \class QtIcoHandler
     \since 4.4
     \brief The QtIcoHandler class provides support for the ICO image format.
@@ -448,7 +448,7 @@ void ICOReader::read4BitBMP(QImage & image)
                 image = QImage();
                 break;
             }
-            register uchar *p = image.scanLine(h);
+            uchar *p = image.scanLine(h);
             uchar *b = buf;
             for (int i=0; i<icoAttrib.w/2; i++) {   // convert nibbles to bytes
                 *p++ = *b >> 4;
@@ -487,7 +487,7 @@ void ICOReader::read16_24_32BMP(QImage & image)
 {
     if (iod) {
         int h = icoAttrib.h;
-        register QRgb *p;
+        QRgb *p;
         QRgb  *end;
         uchar *buf = new uchar[image.bytesPerLine()];
         int    bpl = ((icoAttrib.w*icoAttrib.nbits+31)/32)*4;
@@ -629,8 +629,8 @@ QList<QImage> ICOReader::read(QIODevice * device)
 
 /*!
     Writes all the QImages in the \a images list to the given \a
-    device. Returns true if the images are written successfully;
-    otherwise returns false.
+    device. Returns \c true if the images are written successfully;
+    otherwise returns \c false.
 
     The first image in the list is stored as the first icon in the
     device, and is therefore used as the default icon by applications.
