@@ -44,16 +44,13 @@
 #include <Cocoa/Cocoa.h>
 
 #include <QtGui>
-
-#ifndef QT_NO_COCOA_ACCESSIBILITY
-
 #include <qpa/qplatformaccessibility.h>
 
-class QCococaAccessibility : public QPlatformAccessibility
+class QCocoaAccessibility : public QPlatformAccessibility
 {
 public:
-    QCococaAccessibility();
-    ~QCococaAccessibility();
+    QCocoaAccessibility();
+    ~QCocoaAccessibility();
     void notifyAccessibilityUpdate(QAccessibleEvent *event);
     void setRootObject(QObject *o);
     void initialize();
@@ -81,7 +78,8 @@ namespace QCocoaAccessible {
 */
 
 NSString *macRole(QAccessibleInterface *interface);
-bool shouldBeIgnrored(QAccessibleInterface *interface);
+bool shouldBeIgnored(QAccessibleInterface *interface);
+NSArray *unignoredChildren(id parentObject, QAccessibleInterface *interface);
 NSString *getTranslatedAction(const QString &qtAction);
 NSMutableArray *createTranslatedActionsList(const QStringList &qtActions);
 QString translateAction(NSString *nsAction);
@@ -90,6 +88,4 @@ id getValueAttribute(QAccessibleInterface *interface);
 
 }
 
-#endif // QT_NO_COCOA_ACCESSIBILITY
-
 #endif
