@@ -40,30 +40,23 @@
 ****************************************************************************/
 #include "qcocoaaccessibility.h"
 #include "qcocoaaccessibilityelement.h"
-#include <qaccessible.h>
-#include <qaccessible2.h>
+#include <QtGui/qaccessible.h>
 #include <private/qcore_mac_p.h>
 
-#ifndef QT_NO_COCOA_ACCESSIBILITY
-
-QCococaAccessibility::QCococaAccessibility()
+QCocoaAccessibility::QCocoaAccessibility()
 {
 
 }
 
-QCococaAccessibility::~QCococaAccessibility()
+QCocoaAccessibility::~QCocoaAccessibility()
 {
 
 }
 
-void QCococaAccessibility::notifyAccessibilityUpdate(QAccessibleEvent *event)
+void QCocoaAccessibility::notifyAccessibilityUpdate(QAccessibleEvent *event)
 {
-    QObject *object = event->object();
-    if (!object)
-        return;
-
-    QAccessibleInterface *interface = QAccessible::queryAccessibleInterface(object);
-    if (!interface)
+    QAccessible::Id interfaceId = event->uniqueId();
+    if (!interfaceId)
         return;
 
     switch (event->type()) {
@@ -71,28 +64,26 @@ void QCococaAccessibility::notifyAccessibilityUpdate(QAccessibleEvent *event)
         case QAccessible::TextInserted :
         case QAccessible::TextRemoved :
         case QAccessible::TextUpdated : {
-            QCocoaAccessibleElement *element = [QCocoaAccessibleElement createElementWithInterface : interface parent : nil];
+            QCocoaAccessibleElement *element = [QCocoaAccessibleElement createElementWithId : interfaceId parent : nil];
             [element autorelease];
             NSAccessibilityPostNotification(element, NSAccessibilityValueChangedNotification);
         break; }
-
-        default:
-            delete interface;
+    default:
         break;
     }
 }
 
-void QCococaAccessibility::setRootObject(QObject *o)
+void QCocoaAccessibility::setRootObject(QObject *o)
 {
     Q_UNUSED(o)
 }
 
-void QCococaAccessibility::initialize()
+void QCocoaAccessibility::initialize()
 {
 
 }
 
-void QCococaAccessibility::cleanup()
+void QCocoaAccessibility::cleanup()
 {
 
 }
@@ -169,8 +160,11 @@ NSString *macRole(QAccessibleInterface *interface)
         return roleMap[qtRole];
     }
 
-    // MAC_ACCESSIBILTY_DEBUG() << "return NSAccessibilityUnknownRole";
-    return NSAccessibilityUnknownRole;
+    // Treat unknown Qt roles as generic group container items. Returning
+    // NSAccessibilityUnknownRole is also possible but makes the screen
+    // reader focus on the item instead of passing focus to child items.
+    // MAC_ACCESSIBILTY_DEBUG() << "return NSAccessibilityGroupRole for unknown Qt role";
+    return NSAccessibilityGroupRole;
 }
 
 /*
@@ -178,13 +172,14 @@ NSString *macRole(QAccessibleInterface *interface)
     the elements are still present in the accessibility tree but is
     not used by the screen reader.
 */
-bool shouldBeIgnrored(QAccessibleInterface *interface)
+bool shouldBeIgnored(QAccessibleInterface *interface)
 {
     // Mac accessibility does not have an attribute that corresponds to the Invisible/Offscreen
     // state. Ignore interfaces with those flags set.
     const QAccessible::State state = interface->state();
     if (state.invisible ||
-        state.offscreen)
+        state.offscreen ||
+        state.invalid)
         return true;
 
     // Some roles are not interesting. In particular, container roles should be
@@ -193,12 +188,13 @@ bool shouldBeIgnrored(QAccessibleInterface *interface)
     if (role == QAccessible::Border ||      // QFrame
         role == QAccessible::Application || // We use the system-provided application element.
         role == QAccessible::MenuItem ||    // The system also provides the menu items.
-        role == QAccessible::ToolBar)       // Access the tool buttons directly.
+        role == QAccessible::ToolBar ||     // Access the tool buttons directly.
+        role == QAccessible::Pane ||        // Scroll areas.
+        role == QAccessible::Client)        // The default for QWidget.
         return true;
 
     NSString *mac_role = macRole(interface);
     if (mac_role == NSAccessibilityWindowRole || // We use the system-provided window elements.
-        mac_role == NSAccessibilityGroupRole ||
         mac_role == NSAccessibilityUnknownRole)
         return true;
 
@@ -222,6 +218,25 @@ bool shouldBeIgnrored(QAccessibleInterface *interface)
     return false;
 }
 
+NSArray *unignoredChildren(id parentObject, QAccessibleInterface *interface)
+{
+    int numKids = interface->childCount();
+    // qDebug() << "Children for: " << axid << iface << " are: " << numKids;
+
+    NSMutableArray *kids = [NSMutableArray arrayWithCapacity:numKids];
+    for (int i = 0; i < numKids; ++i) {
+        QAccessibleInterface *child = interface->child(i);
+        if (!child || !child->isValid() || child->state().invalid || child->state().invisible)
+            continue;
+
+        QAccessible::Id childId = QAccessible::uniqueId(child);
+        //qDebug() << "    kid: " << childId << child;
+        QCocoaAccessibleElement *element = [QCocoaAccessibleElement createElementWithId:childId parent:parentObject];
+        [kids addObject: element];
+        [element release];
+    }
+    return NSAccessibilityUnignoredChildren(kids);
+}
 /*
     Translates a predefined QAccessibleActionInterface action to a Mac action constant.
     Returns 0 if the Qt Action has no mac equivalent. Ownership of the NSString is
@@ -280,6 +295,7 @@ QString translateAction(NSString *nsAction)
 
 bool hasValueAttribute(QAccessibleInterface *interface)
 {
+    Q_ASSERT(interface);
     const QAccessible::Role qtrole = interface->role();
     if (qtrole == QAccessible::EditableText
             || interface->valueInterface()) {
@@ -318,5 +334,3 @@ id getValueAttribute(QAccessibleInterface *interface)
 }
 
 } // namespace QCocoaAccessible
-
-#endif // QT_NO_COCOA_ACCESSIBILITY
