@@ -88,23 +88,36 @@
 
 QT_USE_NAMESPACE
 
-static QT_MANGLE_NAMESPACE(QCocoaApplicationDelegate) *sharedCocoaApplicationDelegate = nil;
+static QCocoaApplicationDelegate *sharedCocoaApplicationDelegate = nil;
 
 static void cleanupCocoaApplicationDelegate()
 {
     [sharedCocoaApplicationDelegate release];
 }
 
-@implementation QT_MANGLE_NAMESPACE(QCocoaApplicationDelegate)
+@implementation QCocoaApplicationDelegate
 
 - (id)init
 {
     self = [super init];
-    if (self)
+    if (self) {
         inLaunch = true;
+        [[NSNotificationCenter defaultCenter]
+                addObserver:self
+                   selector:@selector(updateScreens:)
+                       name:NSApplicationDidChangeScreenParametersNotification
+                     object:NSApp];
+    }
     return self;
 }
 
+- (void)updateScreens:(NSNotification *)notification
+{
+    Q_UNUSED(notification);
+    if (QCocoaIntegration *ci = dynamic_cast<QCocoaIntegration *>(QGuiApplicationPrivate::platformIntegration()))
+        ci->updateScreens();
+}
+
 - (void)dealloc
 {
     sharedCocoaApplicationDelegate = nil;
@@ -114,6 +127,8 @@ static void cleanupCocoaApplicationDelegate()
         [NSApp setDelegate:reflectionDelegate];
         [reflectionDelegate release];
     }
+    [[NSNotificationCenter defaultCenter] removeObserver:self];
+
     [super dealloc];
 }
 
@@ -129,7 +144,7 @@ static void cleanupCocoaApplicationDelegate()
     return nil;
 }
 
-+ (QT_MANGLE_NAMESPACE(QCocoaApplicationDelegate)*)sharedDelegate
++ (QCocoaApplicationDelegate *)sharedDelegate
 {
     @synchronized(self) {
         if (sharedCocoaApplicationDelegate == nil)
@@ -145,19 +160,20 @@ static void cleanupCocoaApplicationDelegate()
     dockMenu = newMenu;
 }
 
-- (NSMenu *)applicationDockMenu
+- (NSMenu *)applicationDockMenu:(NSApplication *)sender
 {
+    Q_UNUSED(sender);
     return [[dockMenu retain] autorelease];
 }
 
-- (void)setMenuLoader:(QT_MANGLE_NAMESPACE(QCocoaMenuLoader) *)menuLoader
+- (void)setMenuLoader:(QCocoaMenuLoader *)menuLoader
 {
     [menuLoader retain];
     [qtMenuLoader release];
     qtMenuLoader = menuLoader;
 }
 
-- (QT_MANGLE_NAMESPACE(QCocoaMenuLoader) *)menuLoader
+- (QCocoaMenuLoader *)menuLoader
 {
     return [[qtMenuLoader retain] autorelease];
 }
@@ -167,7 +183,7 @@ static void cleanupCocoaApplicationDelegate()
     [[NSApp mainMenu] cancelTracking];
 
     bool handle_quit = true;
-    NSMenuItem *quitMenuItem = [[[QT_MANGLE_NAMESPACE(QCocoaApplicationDelegate) sharedDelegate] menuLoader] quitMenuItem];
+    NSMenuItem *quitMenuItem = [[[QCocoaApplicationDelegate sharedDelegate] menuLoader] quitMenuItem];
     if (!QGuiApplicationPrivate::instance()->modalWindowList.isEmpty()
         && [quitMenuItem isEnabled]) {
         int visible = 0;
@@ -194,9 +210,10 @@ static void cleanupCocoaApplicationDelegate()
 - (NSApplicationTerminateReply)applicationShouldTerminate:(NSApplication *)sender
 {
     // The reflection delegate gets precedence
-    if (reflectionDelegate
-        && [reflectionDelegate respondsToSelector:@selector(applicationShouldTerminate:)]) {
-        return [reflectionDelegate applicationShouldTerminate:sender];
+    if (reflectionDelegate) {
+        if ([reflectionDelegate respondsToSelector:@selector(applicationShouldTerminate:)])
+            return [reflectionDelegate applicationShouldTerminate:sender];
+        return NSTerminateNow;
     }
 
     if ([self canQuit]) {
@@ -206,7 +223,6 @@ static void cleanupCocoaApplicationDelegate()
             // events while the event loop is still running.
             const QWindowList topLevels = QGuiApplication::topLevelWindows();
             for (int i = 0; i < topLevels.size(); ++i) {
-                QWindow *window = topLevels.at(i);
                 topLevels.at(i)->close();
             }
 
@@ -312,12 +328,11 @@ static void cleanupCocoaApplicationDelegate()
 
 - (void)applicationDidBecomeActive:(NSNotification *)notification
 {
-    Q_UNUSED(notification);
-/*
     if (reflectionDelegate
         && [reflectionDelegate respondsToSelector:@selector(applicationDidBecomeActive:)])
         [reflectionDelegate applicationDidBecomeActive:notification];
 
+/*
     onApplicationChangedActivation(true);
 
     if (!QWidget::mouseGrabber()){
@@ -336,12 +351,11 @@ static void cleanupCocoaApplicationDelegate()
 
 - (void)applicationDidResignActive:(NSNotification *)notification
 {
-    Q_UNUSED(notification);
-/*
     if (reflectionDelegate
         && [reflectionDelegate respondsToSelector:@selector(applicationDidResignActive:)])
         [reflectionDelegate applicationDidResignActive:notification];
 
+/*
     onApplicationChangedActivation(false);
 
     if (!QWidget::mouseGrabber())
