@@ -73,8 +73,13 @@ QPaintDevice *QCocoaBackingStore::paintDevice()
         }
 #endif
 
-        m_qImage = QImage(m_requestedSize * scaleFactor, QImage::Format_ARGB32_Premultiplied);
+        QCocoaWindow *cocoaWindow = static_cast<QCocoaWindow *>(window()->handle());
+        QImage::Format format = (window()->format().hasAlpha() || cocoaWindow->m_drawContentBorderGradient)
+                ? QImage::Format_ARGB32_Premultiplied : QImage::Format_RGB32;
+        m_qImage = QImage(m_requestedSize * scaleFactor, format);
         m_qImage.setDevicePixelRatio(scaleFactor);
+        if (format == QImage::Format_ARGB32_Premultiplied)
+            m_qImage.fill(Qt::transparent);
     }
     return &m_qImage;
 }
@@ -90,8 +95,10 @@ void QCocoaBackingStore::flush(QWindow *win, const QRegion &region, const QPoint
     // m_cgImage is only a reference to the data inside m_qImage, it is not a copy.
     CGImageRelease(m_cgImage);
     m_cgImage = 0;
-    if (QCocoaWindow *cocoaWindow = static_cast<QCocoaWindow *>(win->handle()))
-        [cocoaWindow->m_contentView flushBackingStore:this region:region offset:offset];
+    if (!m_qImage.isNull()) {
+        if (QCocoaWindow *cocoaWindow = static_cast<QCocoaWindow *>(win->handle()))
+            [cocoaWindow->m_qtView flushBackingStore:this region:region offset:offset];
+    }
 }
 
 void QCocoaBackingStore::resize(const QSize &size, const QRegion &)
