@@ -45,7 +45,7 @@
 #include <Cocoa/Cocoa.h>
 #include <QtGui>
 #include <qpa/qplatformdrag.h>
-#include <QtPlatformSupport/private/qsimpledrag_p.h>
+#include <private/qsimpledrag_p.h>
 
 #include <QtGui/private/qdnd_p.h>
 
@@ -55,6 +55,7 @@ class QCocoaDrag : public QPlatformDrag
 {
 public:
     QCocoaDrag();
+    virtual ~QCocoaDrag();
 
     virtual QMimeData *platformDropData();
     virtual Qt::DropAction drag(QDrag *m_drag);
