@@ -53,9 +53,15 @@ QCocoaDrag::QCocoaDrag() :
     m_lastView = 0;
 }
 
+QCocoaDrag::~QCocoaDrag()
+{
+    [m_lastEvent release];
+}
+
 void QCocoaDrag::setLastMouseEvent(NSEvent *event, NSView *view)
 {
-    m_lastEvent = event;
+    [m_lastEvent release];
+    m_lastEvent = [event copy];
     m_lastView = view;
 }
 
@@ -123,9 +129,9 @@ Qt::DropAction QCocoaDrag::drag(QDrag *o)
     if (pm.isNull())
         pm = defaultPixmap();
 
-    NSImage *nsimage = static_cast<NSImage *>(qt_mac_create_nsimage(pm));
+    NSImage *nsimage = qt_mac_create_nsimage(pm);
 
-    QMacPasteboard dragBoard((CFStringRef) NSDragPboard, QMacPasteboardMime::MIME_DND);
+    QMacPasteboard dragBoard((CFStringRef) NSDragPboard, QMacInternalPasteboardMime::MIME_DND);
     m_drag->mimeData()->setData(QLatin1String("application/x-qt-mime-type-name"), QByteArray("dummy"));
     dragBoard.setMimeData(m_drag->mimeData());
 
@@ -145,6 +151,8 @@ Qt::DropAction QCocoaDrag::drag(QDrag *o)
         source:m_lastView
         slideBack:YES];
 
+    [nsimage release];
+
     m_drag = 0;
     return m_executed_drop_action;
 }
@@ -173,7 +181,7 @@ QStringList QCocoaDropData::formats_sys() const
         qDebug("DnD: Cannot get PasteBoard!");
         return formats;
     }
-    formats = QMacPasteboard(board, QMacPasteboardMime::MIME_DND).formats();
+    formats = QMacPasteboard(board, QMacInternalPasteboardMime::MIME_DND).formats();
     return formats;
 }
 
@@ -185,7 +193,7 @@ QVariant QCocoaDropData::retrieveData_sys(const QString &mimeType, QVariant::Typ
         qDebug("DnD: Cannot get PasteBoard!");
         return data;
     }
-    data = QMacPasteboard(board, QMacPasteboardMime::MIME_DND).retrieveData(mimeType, type);
+    data = QMacPasteboard(board, QMacInternalPasteboardMime::MIME_DND).retrieveData(mimeType, type);
     CFRelease(board);
     return data;
 }
@@ -198,7 +206,7 @@ bool QCocoaDropData::hasFormat_sys(const QString &mimeType) const
         qDebug("DnD: Cannot get PasteBoard!");
         return has;
     }
-    has = QMacPasteboard(board, QMacPasteboardMime::MIME_DND).hasFormat(mimeType);
+    has = QMacPasteboard(board, QMacInternalPasteboardMime::MIME_DND).hasFormat(mimeType);
     CFRelease(board);
     return has;
 }
