@@ -75,8 +75,11 @@ public:
     bool isSharing() const;
     bool isValid() const;
 
+    void windowWasHidden();
+
 private:
     void setActiveWindow(QWindow *window);
+    void updateSurfaceFormat();
 
     NSOpenGLContext *m_context;
     NSOpenGLContext *m_shareContext;
