@@ -47,11 +47,14 @@
 #include <QtGui>
 #include <qpa/qplatformscreen.h>
 #include <private/qguiapplication_p.h>
+#include <private/qwindow_p.h>
 
 #ifndef QT_NO_WIDGETS
 #include <QtWidgets/QWidget>
 #endif
 
+#include <algorithm>
+
 QT_BEGIN_NAMESPACE
 
 //
@@ -83,11 +86,8 @@ static void drawImageReleaseData (void *info, const void *, size_t)
 
 CGImageRef qt_mac_image_to_cgimage(const QImage &img)
 {
-    if (img.width() <= 0 || img.height() <= 0) {
-        qWarning() << Q_FUNC_INFO <<
-            "trying to set" << img.width() << "x" << img.height() << "size for CGImage";
+    if (img.isNull())
         return 0;
-    }
 
     QImage *image;
     if (img.depth() != 32)
@@ -123,16 +123,7 @@ CGImageRef qt_mac_image_to_cgimage(const QImage &img)
 
 NSImage *qt_mac_cgimage_to_nsimage(CGImageRef image)
 {
-    QCocoaAutoReleasePool pool;
-    NSImage *newImage = 0;
-    NSRect imageRect = NSMakeRect(0.0, 0.0, CGImageGetWidth(image), CGImageGetHeight(image));
-    newImage = [[NSImage alloc] initWithSize:imageRect.size];
-    [newImage lockFocus];
-    {
-        CGContextRef imageContext = (CGContextRef)[[NSGraphicsContext currentContext] graphicsPort];
-        CGContextDrawImage(imageContext, *(CGRect*)&imageRect, image);
-    }
-    [newImage unlockFocus];
+    NSImage *newImage = [[NSImage alloc] initWithCGImage:image size:NSZeroSize];
     return newImage;
 }
 
@@ -147,6 +138,24 @@ NSImage *qt_mac_create_nsimage(const QPixmap &pm)
     return nsImage;
 }
 
+NSImage *qt_mac_create_nsimage(const QIcon &icon)
+{
+    if (icon.isNull())
+        return nil;
+
+    NSImage *nsImage = [[NSImage alloc] init];
+    foreach (QSize size, icon.availableSizes()) {
+        QPixmap pm = icon.pixmap(size);
+        QImage image = pm.toImage();
+        CGImageRef cgImage = qt_mac_image_to_cgimage(image);
+        NSBitmapImageRep *imageRep = [[NSBitmapImageRep alloc] initWithCGImage:cgImage];
+        [nsImage addRepresentation:imageRep];
+        [imageRep release];
+        CGImageRelease(cgImage);
+    }
+    return nsImage;
+}
+
 HIMutableShapeRef qt_mac_QRegionToHIMutableShape(const QRegion &region)
 {
     HIMutableShapeRef shape = HIShapeCreateMutable();
@@ -261,7 +270,7 @@ static const KeyPair entries[NumEntries] = {
     { NSF6FunctionKey, Qt::Key_F6 },
     { NSF7FunctionKey, Qt::Key_F7 },
     { NSF8FunctionKey, Qt::Key_F8 },
-    { NSF9FunctionKey, Qt::Key_F8 },
+    { NSF9FunctionKey, Qt::Key_F9 },
     { NSF10FunctionKey, Qt::Key_F10 },
     { NSF11FunctionKey, Qt::Key_F11 },
     { NSF12FunctionKey, Qt::Key_F12 },
@@ -313,19 +322,19 @@ QChar qt_mac_qtKey2CocoaKey(Qt::Key key)
         mustInit = false;
         for (int i=0; i<NumEntries; ++i)
             rev_entries[i] = entries[i];
-        qSort(rev_entries.begin(), rev_entries.end(), qtKey2CocoaKeySortLessThan);
+        std::sort(rev_entries.begin(), rev_entries.end(), qtKey2CocoaKeySortLessThan);
     }
     const QVector<KeyPair>::iterator i
-            = qBinaryFind(rev_entries.begin(), rev_entries.end(), key);
-    if (i == rev_entries.end())
+            = std::lower_bound(rev_entries.begin(), rev_entries.end(), key);
+    if ((i == rev_entries.end()) || (key < *i))
         return QChar();
     return i->cocoaKey;
 }
 
 Qt::Key qt_mac_cocoaKey2QtKey(QChar keyCode)
 {
-    const KeyPair *i = qBinaryFind(entries, end, keyCode);
-    if (i == end)
+    const KeyPair *i = std::lower_bound(entries, end, keyCode);
+    if ((i == end) || (keyCode < *i))
         return Qt::Key(keyCode.toUpper().unicode());
     return i->qtKey;
 }
@@ -474,11 +483,9 @@ CGColorSpaceRef qt_mac_genericColorSpace()
 {
 #if 0
     if (!m_genericColorSpace) {
-#if MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_4
         if (QSysInfo::MacintoshVersion >= QSysInfo::MV_10_4) {
             m_genericColorSpace = CGColorSpaceCreateWithName(kCGColorSpaceGenericRGB);
         } else
-#endif
         {
             m_genericColorSpace = CGColorSpaceCreateDeviceRGB();
         }
@@ -503,7 +510,6 @@ CGColorSpaceRef qt_mac_displayColorSpace(const QWidget *widget)
     CGColorSpaceRef colorSpace;
 
     CGDirectDisplayID displayID;
-    CMProfileRef displayProfile = 0;
     if (widget == 0) {
         displayID = CGMainDisplayID();
     } else {
@@ -521,18 +527,11 @@ CGColorSpaceRef qt_mac_displayColorSpace(const QWidget *widget)
     if ((colorSpace = m_displayColorSpaceHash.value(displayID)))
         return colorSpace;
 
-    CMError err = CMGetProfileByAVID((CMDisplayIDType)displayID, &displayProfile);
-    if (err == noErr) {
-        colorSpace = CGColorSpaceCreateWithPlatformColorSpace(displayProfile);
-    } else if (widget) {
-        return qt_mac_displayColorSpace(0); // fall back on main display
-    }
-
+    colorSpace = CGDisplayCopyColorSpace(displayID);
     if (colorSpace == 0)
         colorSpace = CGColorSpaceCreateDeviceRGB();
 
     m_displayColorSpaceHash.insert(displayID, colorSpace);
-    CMCloseProfile(displayProfile);
     if (!m_postRoutineRegistered) {
         m_postRoutineRegistered = true;
         void qt_mac_cleanUpMacColorSpaces();
@@ -586,10 +585,38 @@ QString qt_mac_applicationName()
     return appName;
 }
 
+/*
+    Mac window coordinates are in the first quadrant: 0, 0 is at the lower-left
+    corner of the primary screen. This function converts the given rect to an
+    NSRect for the window geometry, flipping from 4th quadrant to 1st quadrant
+    and simultaneously ensuring that as much of the window as possible will be
+    onscreen. If the rect is too tall for the screen, the OS will reduce the
+    window's height anyway; but by moving the window upwards we can have more
+    of it onscreen.  But the application can still control the y coordinate
+    in case it really wants the window to be positioned partially offscreen.
+*/
 NSRect qt_mac_flipRect(const QRect &rect, QWindow *window)
 {
     QPlatformScreen *onScreen = QPlatformScreen::platformScreenForWindow(window);
-    int flippedY = onScreen->geometry().height() - rect.y() - rect.height();
+    int flippedY = onScreen->geometry().height() - (rect.y() + rect.height());
+    QList<QScreen *> screens = QGuiApplication::screens();
+    if (screens.size() > 1) {
+        int height = 0;
+        foreach (QScreen *scr, screens)
+            height = qMax(height, scr->size().height());
+        int difference = height - onScreen->geometry().height();
+        if (difference > 0)
+            flippedY += difference;
+        else
+            flippedY -= difference;
+    }
+    // In case of automatic positioning, try to put as much of the window onscreen as possible.
+    if (window->isTopLevel() && qt_window_private(const_cast<QWindow*>(window))->positionAutomatic && flippedY < 0)
+        flippedY = onScreen->geometry().height() - onScreen->availableGeometry().height() - onScreen->availableGeometry().y();
+#ifdef QT_COCOA_ENABLE_WINDOW_DEBUG
+    qDebug() << Q_FUNC_INFO << rect << "flippedY" << flippedY <<
+                "screen" << onScreen->geometry() << "available" << onScreen->availableGeometry();
+#endif
     return NSMakeRect(rect.x(), flippedY, rect.width(), rect.height());
 }
 
@@ -615,11 +642,6 @@ InvalidContext:
         return err;
 }
 
-CGFloat qt_mac_get_scalefactor()
-{
-    return [[NSScreen mainScreen] userSpaceScaleFactor];
-}
-
 Qt::MouseButton cocoaButton2QtButton(NSInteger buttonNum)
 {
     if (buttonNum == 0)
@@ -752,6 +774,9 @@ CGContextRef qt_mac_cg_context(QPaintDevice *pdev)
     return ret;
 }
 
+// qpaintengine_mac.mm
+extern void qt_mac_cgimage_data_free(void *, const void *memoryToFree, size_t);
+
 CGImageRef qt_mac_toCGImage(const QImage &qImage, bool isMask, uchar **dataCopy)
 {
     int width = qImage.width();
@@ -765,8 +790,7 @@ CGImageRef qt_mac_toCGImage(const QImage &qImage, bool isMask, uchar **dataCopy)
 
     const uchar *imageData = qImage.bits();
     if (dataCopy) {
-        delete[] *dataCopy;
-        *dataCopy = new uchar[qImage.byteCount()];
+        *dataCopy = static_cast<uchar *>(malloc(qImage.byteCount()));
         memcpy(*dataCopy, imageData, qImage.byteCount());
     }
     int bitDepth = qImage.depth();
@@ -777,7 +801,7 @@ CGImageRef qt_mac_toCGImage(const QImage &qImage, bool isMask, uchar **dataCopy)
                 NULL,
                 dataCopy ? *dataCopy : imageData,
                 qImage.byteCount(),
-                NULL);
+                dataCopy ? qt_mac_cgimage_data_free : NULL);
 
     CGImageRef cgImage = 0;
     if (isMask) {
@@ -790,20 +814,7 @@ CGImageRef qt_mac_toCGImage(const QImage &qImage, bool isMask, uchar **dataCopy)
                                     NULL,
                                     false);
     } else {
-        // Try get a device color space. Using the device color space means
-        // that the CGImage can be drawn to screen without per-pixel color
-        // space conversion, at the cost of less color accuracy.
-        CGColorSpaceRef cgColourSpaceRef = 0;
-        CMProfileRef sysProfile;
-        if (CMGetSystemProfile(&sysProfile) == noErr)
-        {
-            cgColourSpaceRef = CGColorSpaceCreateWithPlatformColorSpace(sysProfile);
-            CMCloseProfile(sysProfile);
-        }
-
-        // Fall back to Generic RGB if a profile was not found.
-        if (!cgColourSpaceRef)
-            cgColourSpaceRef = CGColorSpaceCreateWithName(kCGColorSpaceGenericRGB);
+        CGColorSpaceRef cgColourSpaceRef = qt_mac_displayColorSpace(0);
 
         // Create a CGBitmapInfo contiaining the image format.
         // Support the 8-bit per component (A)RGB formats.
@@ -837,7 +848,6 @@ CGImageRef qt_mac_toCGImage(const QImage &qImage, bool isMask, uchar **dataCopy)
                                 NULL,
                                 false,
                                 kCGRenderingIntentDefault);
-        CGColorSpaceRelease(cgColourSpaceRef);
     }
     CGDataProviderRelease(cgDataProviderRef);
     return cgImage;
