@@ -63,7 +63,6 @@
 
 static void initResources()
 {
-    Q_INIT_RESOURCE_EXTERN(qcocoaresources)
     Q_INIT_RESOURCE(qcocoaresources);
 }
 
@@ -90,11 +89,28 @@ void QCocoaScreen::updateGeometry()
 {
     NSScreen *nsScreen = osScreen();
     NSRect frameRect = [nsScreen frame];
-    m_geometry = QRect(frameRect.origin.x, frameRect.origin.y, frameRect.size.width, frameRect.size.height);
-    NSRect visibleRect = [nsScreen visibleFrame];
-    m_availableGeometry = QRect(visibleRect.origin.x,
-                                frameRect.size.height - (visibleRect.origin.y + visibleRect.size.height), // invert y
-                                visibleRect.size.width, visibleRect.size.height);
+
+    if (m_screenIndex == 0) {
+        m_geometry = QRect(frameRect.origin.x, frameRect.origin.y, frameRect.size.width, frameRect.size.height);
+        // This is the primary screen, the one that contains the menubar. Its origin should be
+        // (0, 0), and it's the only one whose available geometry differs from its full geometry.
+        NSRect visibleRect = [nsScreen visibleFrame];
+        m_availableGeometry = QRect(visibleRect.origin.x,
+                                    frameRect.size.height - (visibleRect.origin.y + visibleRect.size.height), // invert y
+                                    visibleRect.size.width, visibleRect.size.height);
+    } else {
+        // NSScreen origin is at the bottom-left corner, QScreen is at the top-left corner.
+        // When we get the NSScreen frame rect, we need to re-align its origin y coordinate
+        // w.r.t. the primary screen, whose origin is (0, 0).
+        NSRect r = [[[NSScreen screens] objectAtIndex:0] frame];
+        QRect referenceScreenGeometry = QRect(r.origin.x, r.origin.y, r.size.width, r.size.height);
+        m_geometry = QRect(frameRect.origin.x,
+                           referenceScreenGeometry.height() - (frameRect.origin.y + frameRect.size.height),
+                           frameRect.size.width, frameRect.size.height);
+
+        // Not primary screen. See above.
+        m_availableGeometry = m_geometry;
+    }
 
     m_format = QImage::Format_RGB32;
     m_depth = NSBitsPerPixelFromDepth([nsScreen depth]);
@@ -105,7 +121,11 @@ void QCocoaScreen::updateGeometry()
     m_physicalSize = QSizeF(size.width, size.height);
     m_logicalDpi.first = 72;
     m_logicalDpi.second = 72;
-    m_refreshRate = CGDisplayModeGetRefreshRate(CGDisplayCopyDisplayMode(dpy));
+    CGDisplayModeRef displayMode = CGDisplayCopyDisplayMode(dpy);
+    float refresh = CGDisplayModeGetRefreshRate(displayMode);
+    CGDisplayModeRelease(displayMode);
+    if (refresh > 0)
+        m_refreshRate = refresh;
 
     // Get m_name (brand/model of the monitor)
     NSDictionary *deviceInfo = (NSDictionary *)IODisplayCreateInfoDictionary(CGDisplayIOServicePort(dpy), kIODisplayOnlyPreferredName);
@@ -170,14 +190,15 @@ QPixmap QCocoaScreen::grabWindow(WId window, int x, int y, int width, int height
             windowSize.setHeight(windowRect.height());
     }
 
-    QPixmap windowPixmap(windowSize);
+    QPixmap windowPixmap(windowSize * devicePixelRatio());
     windowPixmap.fill(Qt::transparent);
 
     for (uint i = 0; i < displayCount; ++i) {
         const CGRect bounds = CGDisplayBounds(displays[i]);
-        int w = (width < 0 ? bounds.size.width : width);
-        int h = (height < 0 ? bounds.size.height : height);
+        int w = (width < 0 ? bounds.size.width : width) * devicePixelRatio();
+        int h = (height < 0 ? bounds.size.height : height) * devicePixelRatio();
         QRect displayRect = QRect(x, y, w, h);
+        displayRect = displayRect.translated(qRound(-bounds.origin.x), qRound(-bounds.origin.y));
         QCFType<CGImageRef> image = CGDisplayCreateImageForRect(displays[i],
             CGRectMake(displayRect.x(), displayRect.y(), displayRect.width(), displayRect.height()));
         QPixmap pix(w, h);
@@ -188,17 +209,16 @@ QPixmap QCocoaScreen::grabWindow(WId window, int x, int y, int width, int height
         CGContextRelease(ctx);
 
         QPainter painter(&windowPixmap);
-        painter.drawPixmap(bounds.origin.x, bounds.origin.y, pix);
+        painter.drawPixmap(0, 0, pix);
     }
     return windowPixmap;
 }
 
 QCocoaIntegration::QCocoaIntegration()
     : mFontDb(new QCoreTextFontDatabase())
-    , mEventDispatcher(new QCocoaEventDispatcher())
     , mInputContext(new QCocoaInputContext)
-#ifndef QT_NO_COCOA_ACCESSIBILITY
-    , mAccessibility(new QCococaAccessibility)
+#ifndef QT_NO_ACCESSIBILITY
+    , mAccessibility(new QCocoaAccessibility)
 #endif
     , mCocoaClipboard(new QCocoaClipboard)
     , mCocoaDrag(new QCocoaDrag)
@@ -211,7 +231,7 @@ QCocoaIntegration::QCocoaIntegration()
 
     qApp->setAttribute(Qt::AA_DontUseNativeMenuBar, false);
 
-    NSApplication *cocoaApplication = [QT_MANGLE_NAMESPACE(QNSApplication) sharedApplication];
+    NSApplication *cocoaApplication = [QNSApplication sharedApplication];
     qt_redirectNSApplicationSendEvent();
 
     if (qEnvironmentVariableIsEmpty("QT_MAC_DISABLE_FOREGROUND_APPLICATION_TRANSFORM")) {
@@ -235,12 +255,12 @@ QCocoaIntegration::QCocoaIntegration()
     if (!QCoreApplication::testAttribute(Qt::AA_MacPluginApplication)) {
 
         // Set app delegate, link to the current delegate (if any)
-        QT_MANGLE_NAMESPACE(QCocoaApplicationDelegate) *newDelegate = [QT_MANGLE_NAMESPACE(QCocoaApplicationDelegate) sharedDelegate];
+        QCocoaApplicationDelegate *newDelegate = [QCocoaApplicationDelegate sharedDelegate];
         [newDelegate setReflectionDelegate:[cocoaApplication delegate]];
         [cocoaApplication setDelegate:newDelegate];
 
         // Load the application menu. This menu contains Preferences, Hide, Quit.
-        QT_MANGLE_NAMESPACE(QCocoaMenuLoader) *qtMenuLoader = [[QT_MANGLE_NAMESPACE(QCocoaMenuLoader) alloc] init];
+        QCocoaMenuLoader *qtMenuLoader = [[QCocoaMenuLoader alloc] init];
         qt_mac_loadMenuNib(qtMenuLoader);
         [cocoaApplication setMenu:[qtMenuLoader menu]];
         [newDelegate setMenuLoader:qtMenuLoader];
@@ -248,7 +268,7 @@ QCocoaIntegration::QCocoaIntegration()
 
     updateScreens();
 
-    QMacPasteboardMime::initializeMimeTypes();
+    QMacInternalPasteboardMime::initializeMimeTypes();
 }
 
 QCocoaIntegration::~QCocoaIntegration()
@@ -258,7 +278,7 @@ QCocoaIntegration::~QCocoaIntegration()
     QCocoaAutoReleasePool pool;
     if (!QCoreApplication::testAttribute(Qt::AA_MacPluginApplication)) {
         // remove the apple event handlers installed by QCocoaApplicationDelegate
-        QT_MANGLE_NAMESPACE(QCocoaApplicationDelegate) *delegate = [QT_MANGLE_NAMESPACE(QCocoaApplicationDelegate) sharedDelegate];
+        QCocoaApplicationDelegate *delegate = [QCocoaApplicationDelegate sharedDelegate];
         [delegate removeAppleEventHandlers];
         // reset the application delegate
         [[NSApplication sharedApplication] setDelegate: 0];
@@ -268,7 +288,7 @@ QCocoaIntegration::~QCocoaIntegration()
     // Deleting the clipboard integration flushes promised pastes using
     // the mime converters - the ordering here is important.
     delete mCocoaClipboard;
-    QMacPasteboardMime::destroyMimeTypes();
+    QMacInternalPasteboardMime::destroyMimeTypes();
 
     // Delete screens in reverse order to avoid crash in case of multiple screens
     while (!mScreens.isEmpty()) {
@@ -322,6 +342,14 @@ void QCocoaIntegration::updateScreens()
         screen->setVirtualSiblings(siblings);
 }
 
+QCocoaScreen *QCocoaIntegration::screenAtIndex(int index)
+{
+    if (index >= mScreens.count())
+        updateScreens();
+
+    return mScreens.at(index);
+}
+
 bool QCocoaIntegration::hasCapability(QPlatformIntegration::Capability cap) const
 {
     switch (cap) {
@@ -331,6 +359,7 @@ bool QCocoaIntegration::hasCapability(QPlatformIntegration::Capability cap) cons
     case BufferQueueingOpenGL:
     case WindowMasks:
     case MultipleWindows:
+    case ForeignWindows:
         return true;
     default:
         return QPlatformIntegration::hasCapability(cap);
@@ -354,9 +383,9 @@ QPlatformBackingStore *QCocoaIntegration::createPlatformBackingStore(QWindow *wi
     return new QCocoaBackingStore(window);
 }
 
-QAbstractEventDispatcher *QCocoaIntegration::guiThreadEventDispatcher() const
+QAbstractEventDispatcher *QCocoaIntegration::createEventDispatcher() const
 {
-    return mEventDispatcher;
+    return new QCocoaEventDispatcher;
 }
 
 QPlatformFontDatabase *QCocoaIntegration::fontDatabase() const
@@ -376,7 +405,7 @@ QPlatformInputContext *QCocoaIntegration::inputContext() const
 
 QPlatformAccessibility *QCocoaIntegration::accessibility() const
 {
-#ifndef QT_NO_COCOA_ACCESSIBILITY
+#ifndef QT_NO_ACCESSIBILITY
     return mAccessibility.data();
 #else
     return 0;
