@@ -47,14 +47,13 @@
 #include <qpa/qplatformmenu.h>
 #include "qcocoamenu.h"
 
-@class NSMenu;
-
 QT_BEGIN_NAMESPACE
 
 class QCocoaWindow;
 
 class QCocoaMenuBar : public QPlatformMenuBar
 {
+    Q_OBJECT
 public:
     QCocoaMenuBar();
     virtual ~QCocoaMenuBar();
@@ -76,6 +75,8 @@ private:
     static QCocoaMenuBar *findGlobalMenubar();
 
     bool shouldDisable(QCocoaWindow *active) const;
+    void insertNativeMenu(QCocoaMenu *menu, QCocoaMenu *beforeMenu);
+    void removeNativeMenu(QCocoaMenu *menu);
 
     QList<QCocoaMenu*> m_menus;
     NSMenu *m_nativeMenu;
