@@ -45,6 +45,7 @@
 #include <ApplicationServices/ApplicationServices.h>
 
 #include <qpa/qplatformnativeinterface.h>
+#include <QtGui/qpixmap.h>
 
 QT_BEGIN_NAMESPACE
 
@@ -52,6 +53,7 @@ class QWidget;
 class QPlatformPrinterSupport;
 class QPrintEngine;
 class QPlatformMenu;
+class QPlatformMenuBar;
 
 class QCocoaNativeInterface : public QPlatformNativeInterface
 {
@@ -64,6 +66,8 @@ public:
 
     NativeResourceForIntegrationFunction nativeResourceFunctionForIntegration(const QByteArray &resource) Q_DECL_OVERRIDE;
 
+    Q_INVOKABLE void beep();
+
     static void *cglContextForContext(QOpenGLContext *context);
     static void *nsOpenGLContextForContext(QOpenGLContext* context);
 
@@ -88,6 +92,11 @@ private:
         Needed by the native print dialog in the Qt Print Support module.
     */
     Q_INVOKABLE void *NSPrintInfoForPrintEngine(QPrintEngine *printEngine);
+    /*
+        Function to return the default background pixmap.
+        Needed by QWizard in the Qt widget module.
+    */
+    Q_INVOKABLE QPixmap defaultBackgroundPixmapForQWizard();
 
     // QMacPastebardMime support. The mac pasteboard void pointers are
     // QMacPastebardMime instances from the cocoa plugin or qtmacextras
@@ -99,9 +108,33 @@ private:
     // Dock menu support
     static void setDockMenu(QPlatformMenu *platformMenu);
 
+    // Function to return NSMenu * from QPlatformMenu
+    static void *qMenuToNSMenu(QPlatformMenu *platformMenu);
+
+    // Function to return NSMenu * from QPlatformMenuBar
+    static void *qMenuBarToNSMenu(QPlatformMenuBar *platformMenuBar);
+
     // QImage <-> CGImage conversion functions
     static CGImageRef qImageToCGImage(const QImage &image);
     static QImage cgImageToQImage(CGImageRef image);
+
+    // Embedding NSViews as child QWindows
+    static void setWindowContentView(QPlatformWindow *window, void *nsViewContentView);
+
+    // Set a QWindow as a "guest" (subwindow) of a non-QWindow
+    static void setEmbeddedInForeignView(QPlatformWindow *window, bool embedded);
+
+    // Register if a window should deliver touch events. Enabling
+    // touch events has implications for delivery of other events,
+    // for example by causing scrolling event lag.
+    //
+    // The registration is ref-counted: multiple widgets can enable
+    // touch events, which then will be delivered until the widget
+    // deregisters.
+    static void registerTouchWindow(QWindow *window,  bool enable);
+
+    // Request a unified title and toolbar look for the window.
+    static void setContentBorderThickness(QWindow *window, int topThickness, int bottomThickness);
 };
 
 #endif // QCOCOANATIVEINTERFACE_H
