@@ -46,9 +46,11 @@
 #include "qcocoamenubar.h"
 #include "qmacmime.h"
 #include "qcocoahelpers.h"
+#include "qcocoaapplication.h"
 
 #include <qbytearray.h>
 #include <qwindow.h>
+#include <qpixmap.h>
 #include <qpa/qplatformwindow.h>
 #include "qsurfaceformat.h"
 #include <qpa/qplatformopenglcontext.h>
@@ -84,10 +86,8 @@ void *QCocoaNativeInterface::nativeResourceForContext(const QByteArray &resource
 
 void *QCocoaNativeInterface::nativeResourceForWindow(const QByteArray &resourceString, QWindow *window)
 {
-    if (!window->handle()) {
-        qWarning("QCocoaNativeInterface::nativeResourceForWindow: Native window has not been created.");
+    if (!window->handle())
         return 0;
-    }
 
     if (resourceString == "nsopenglcontext") {
         return static_cast<QCocoaWindow *>(window->handle())->currentContext()->nsOpenGLContext();
@@ -109,17 +109,34 @@ QPlatformNativeInterface::NativeResourceForIntegrationFunction QCocoaNativeInter
         return NativeResourceForIntegrationFunction(QCocoaNativeInterface::registerDraggedTypes);
     if (resource.toLower() == "setdockmenu")
         return NativeResourceForIntegrationFunction(QCocoaNativeInterface::setDockMenu);
+    if (resource.toLower() == "qmenutonsmenu")
+        return NativeResourceForIntegrationFunction(QCocoaNativeInterface::qMenuToNSMenu);
+    if (resource.toLower() == "qmenubartonsmenu")
+        return NativeResourceForIntegrationFunction(QCocoaNativeInterface::qMenuBarToNSMenu);
     if (resource.toLower() == "qimagetocgimage")
         return NativeResourceForIntegrationFunction(QCocoaNativeInterface::qImageToCGImage);
     if (resource.toLower() == "cgimagetoqimage")
         return NativeResourceForIntegrationFunction(QCocoaNativeInterface::cgImageToQImage);
+    if (resource.toLower() == "setwindowcontentview")
+        return NativeResourceForIntegrationFunction(QCocoaNativeInterface::setWindowContentView);
+    if (resource.toLower() == "registertouchwindow")
+        return NativeResourceForIntegrationFunction(QCocoaNativeInterface::registerTouchWindow);
+    if (resource.toLower() == "setembeddedinforeignview")
+        return NativeResourceForIntegrationFunction(QCocoaNativeInterface::setEmbeddedInForeignView);
+    if (resource.toLower() == "setcontentborderthickness")
+        return NativeResourceForIntegrationFunction(QCocoaNativeInterface::setContentBorderThickness);
 
     return 0;
 }
 
+void QCocoaNativeInterface::beep()
+{
+    NSBeep();
+}
+
 QPlatformPrinterSupport *QCocoaNativeInterface::createPlatformPrinterSupport()
 {
-#ifndef QT_NO_WIDGETS
+#if !defined(QT_NO_WIDGETS) && !defined(QT_NO_PRINTER)
     return new QCocoaPrinterSupport();
 #else
     qFatal("Printing is not supported when Qt is configured with -no-widgets");
@@ -129,7 +146,7 @@ QPlatformPrinterSupport *QCocoaNativeInterface::createPlatformPrinterSupport()
 
 void *QCocoaNativeInterface::NSPrintInfoForPrintEngine(QPrintEngine *printEngine)
 {
-#ifndef QT_NO_WIDGETS
+#if !defined(QT_NO_WIDGETS) && !defined(QT_NO_PRINTER)
     QMacPrintEnginePrivate *macPrintEnginePriv = static_cast<QMacPrintEngine *>(printEngine)->d_func();
     if (macPrintEnginePriv->state == QPrinter::Idle && !macPrintEnginePriv->isPrintSessionInitialized())
         macPrintEnginePriv->initialize();
@@ -140,6 +157,31 @@ void *QCocoaNativeInterface::NSPrintInfoForPrintEngine(QPrintEngine *printEngine
 #endif
 }
 
+QPixmap QCocoaNativeInterface::defaultBackgroundPixmapForQWizard()
+{
+    QCFType<CFURLRef> url;
+    const int ExpectedImageWidth = 242;
+    const int ExpectedImageHeight = 414;
+    if (LSFindApplicationForInfo(kLSUnknownCreator, CFSTR("com.apple.KeyboardSetupAssistant"),
+                                 0, 0, &url) == noErr) {
+        QCFType<CFBundleRef> bundle = CFBundleCreate(kCFAllocatorDefault, url);
+        if (bundle) {
+            url = CFBundleCopyResourceURL(bundle, CFSTR("Background"), CFSTR("png"), 0);
+            if (url) {
+                QCFType<CGImageSourceRef> imageSource = CGImageSourceCreateWithURL(url, 0);
+                QCFType<CGImageRef> image = CGImageSourceCreateImageAtIndex(imageSource, 0, 0);
+                if (image) {
+                    int width = CGImageGetWidth(image);
+                    int height = CGImageGetHeight(image);
+                    if (width == ExpectedImageWidth && height == ExpectedImageHeight)
+                        return QPixmap::fromImage(qt_mac_toQImage(image));
+                }
+            }
+        }
+    }
+    return QPixmap();
+}
+
 void QCocoaNativeInterface::onAppFocusWindowChanged(QWindow *window)
 {
     Q_UNUSED(window);
@@ -167,12 +209,12 @@ void *QCocoaNativeInterface::nsOpenGLContextForContext(QOpenGLContext* context)
 
 void QCocoaNativeInterface::addToMimeList(void *macPasteboardMime)
 {
-    qt_mac_addToGlobalMimeList(reinterpret_cast<QMacPasteboardMime *>(macPasteboardMime));
+    qt_mac_addToGlobalMimeList(reinterpret_cast<QMacInternalPasteboardMime *>(macPasteboardMime));
 }
 
 void QCocoaNativeInterface::removeFromMimeList(void *macPasteboardMime)
 {
-    qt_mac_removeFromGlobalMimeList(reinterpret_cast<QMacPasteboardMime *>(macPasteboardMime));
+    qt_mac_removeFromGlobalMimeList(reinterpret_cast<QMacInternalPasteboardMime *>(macPasteboardMime));
 }
 
 void QCocoaNativeInterface::registerDraggedTypes(const QStringList &types)
@@ -184,8 +226,21 @@ void QCocoaNativeInterface::setDockMenu(QPlatformMenu *platformMenu)
 {
     QCocoaMenu *cocoaPlatformMenu = static_cast<QCocoaMenu *>(platformMenu);
     NSMenu *menu = cocoaPlatformMenu->nsMenu();
-    // setDockMenu seems to be undocumented, but this is what Qt 4 did.
-    [NSApp setDockMenu: menu];
+    [NSApp QT_MANGLE_NAMESPACE(qt_setDockMenu): menu];
+}
+
+void *QCocoaNativeInterface::qMenuToNSMenu(QPlatformMenu *platformMenu)
+{
+    QCocoaMenu *cocoaPlatformMenu = static_cast<QCocoaMenu *>(platformMenu);
+    NSMenu *menu = cocoaPlatformMenu->nsMenu();
+    return reinterpret_cast<void *>(menu);
+}
+
+void *QCocoaNativeInterface::qMenuBarToNSMenu(QPlatformMenuBar *platformMenuBar)
+{
+    QCocoaMenuBar *cocoaPlatformMenuBar = static_cast<QCocoaMenuBar *>(platformMenuBar);
+    NSMenu *menu = cocoaPlatformMenuBar->nsMenu();
+    return reinterpret_cast<void *>(menu);
 }
 
 CGImageRef QCocoaNativeInterface::qImageToCGImage(const QImage &image)
@@ -198,5 +253,36 @@ QImage QCocoaNativeInterface::cgImageToQImage(CGImageRef image)
     return qt_mac_toQImage(image);
 }
 
+void QCocoaNativeInterface::setWindowContentView(QPlatformWindow *window, void *contentView)
+{
+    QCocoaWindow *cocoaPlatformWindow = static_cast<QCocoaWindow *>(window);
+    cocoaPlatformWindow->setContentView(reinterpret_cast<NSView *>(contentView));
+}
+
+void QCocoaNativeInterface::setEmbeddedInForeignView(QPlatformWindow *window, bool embedded)
+{
+    QCocoaWindow *cocoaPlatformWindow = static_cast<QCocoaWindow *>(window);
+    cocoaPlatformWindow->setEmbeddedInForeignView(embedded);
+}
+
+void QCocoaNativeInterface::registerTouchWindow(QWindow *window,  bool enable)
+{
+    if (!window)
+        return;
+
+    QCocoaWindow *cocoaWindow = static_cast<QCocoaWindow *>(window->handle());
+    if (cocoaWindow)
+        cocoaWindow->registerTouch(enable);
+}
+
+void QCocoaNativeInterface::setContentBorderThickness(QWindow *window, int topThickness, int bottomThickness)
+{
+    if (!window)
+        return;
+
+    QCocoaWindow *cocoaWindow = static_cast<QCocoaWindow *>(window->handle());
+    if (cocoaWindow)
+        cocoaWindow->setContentBorderThickness(topThickness, bottomThickness);
+}
 
 QT_END_NAMESPACE
