@@ -43,6 +43,7 @@
 #define QCOCOAPRINTERSUPPORT_H
 
 #include <qpa/qplatformprintersupport.h>
+#ifndef QT_NO_PRINTER
 
 #include "qt_mac_p.h"
 
@@ -55,6 +56,7 @@ public:
     QPrintEngine *createNativePrintEngine(QPrinter::PrinterMode printerMode) Q_DECL_OVERRIDE;
     QPaintEngine *createPaintEngine(QPrintEngine *, QPrinter::PrinterMode printerMode) Q_DECL_OVERRIDE;
     QList<QPrinter::PaperSize> supportedPaperSizes(const QPrinterInfo &) const Q_DECL_OVERRIDE;
+    QList<QPair<QString, QSizeF> > supportedSizesWithNames(const QPrinterInfo &) const Q_DECL_OVERRIDE;
 
     QList<QPrinterInfo> availablePrinters() Q_DECL_OVERRIDE;
     QPrinterInfo printerInfo(const QString &printerName) Q_DECL_OVERRIDE;
@@ -63,4 +65,5 @@ private:
     QPrinterInfo printerInfoFromPMPrinter(const PMPrinter &printer);
 };
 
+#endif // QT_NO_PRINTER
 #endif // QCOCOAPRINTERSUPPORT_H
