@@ -40,6 +40,8 @@
 ****************************************************************************/
 
 #include "qcocoaprintersupport.h"
+
+#ifndef QT_NO_PRINTER
 #include "qprintengine_mac_p.h"
 
 #include <QtPrintSupport/QPrinter>
@@ -138,3 +140,36 @@ QPrinterInfo QCocoaPrinterSupport::printerInfoFromPMPrinter(const PMPrinter &pri
 
     return createPrinterInfo(name, description, location, makeAndModel, isDefault, 0);
 }
+
+QList<QPair<QString, QSizeF> > QCocoaPrinterSupport::supportedSizesWithNames(const QPrinterInfo &printerInfo) const
+{
+    QList<QPair<QString, QSizeF> > returnValue;
+    if (printerInfo.isNull())
+        return returnValue;
+
+    PMPrinter printer = PMPrinterCreateFromPrinterID(QCFString::toCFStringRef(printerInfo.printerName()));
+    if (!printer)
+        return returnValue;
+
+    CFArrayRef array;
+    if (PMPrinterGetPaperList(printer, &array) != noErr) {
+        PMRelease(printer);
+        return returnValue;
+    }
+
+    int count = CFArrayGetCount(array);
+    for (int i = 0; i < count; ++i) {
+        PMPaper paper = static_cast<PMPaper>(const_cast<void *>(CFArrayGetValueAtIndex(array, i)));
+        double width, height;
+        if (PMPaperGetWidth(paper, &width) == noErr && PMPaperGetHeight(paper, &height) == noErr) {
+            static const double OnePointInMillimeters = 1.0 / 72.0 * 25.4;
+            QCFString paperName;
+            if (PMPaperCreateLocalizedName(paper, printer, &paperName) == noErr)
+                returnValue.append(qMakePair(QString(paperName), QSizeF(width * OnePointInMillimeters, height * OnePointInMillimeters)));
+        }
+    }
+    PMRelease(printer);
+    return returnValue;
+}
+
+#endif  //QT_NO_PRINTER
