@@ -61,7 +61,7 @@ QT_FORWARD_DECLARE_CLASS(QCocoaWindow)
 @end
 
 @interface QNSPanel : NSPanel {
-    @public QT_PREPEND_NAMESPACE(QCocoaWindow) *m_cocoaPlatformWindow;
+    @public QCocoaWindow *m_cocoaPlatformWindow;
 }
 - (void)clearPlatformWindow;
 - (BOOL)canBecomeKeyWindow;
@@ -106,27 +106,38 @@ public:
     void setWindowTitle(const QString &title);
     void setWindowFilePath(const QString &filePath);
     void setWindowIcon(const QIcon &icon);
+    void setAlertState(bool enabled);
+    bool isAlertState() const;
     void raise();
     void lower();
     bool isExposed() const;
+    bool isOpaque() const;
     void propagateSizeHints();
     void setOpacity(qreal level);
     void setMask(const QRegion &region);
     bool setKeyboardGrabEnabled(bool grab);
     bool setMouseGrabEnabled(bool grab);
     QMargins frameMargins() const;
+    QSurfaceFormat format() const;
+
+    void requestActivateWindow();
 
     WId winId() const;
     void setParent(const QPlatformWindow *window);
 
     NSView *contentView() const;
+    void setContentView(NSView *contentView);
+
+    void setEmbeddedInForeignView(bool subwindow);
 
     void windowWillMove();
     void windowDidMove();
     void windowDidResize();
-    void windowWillClose();
+    bool windowShouldClose();
     bool windowIsPopupType(Qt::WindowType type = Qt::Widget) const;
 
+    void setSynchedWindowStateFromWindow();
+
     NSInteger windowLevel(Qt::WindowFlags flags);
     NSUInteger windowStyleMask(Qt::WindowFlags flags);
     void setWindowShadow(Qt::WindowFlags flags);
@@ -143,9 +154,18 @@ public:
     void setMenubar(QCocoaMenuBar *mb);
     QCocoaMenuBar *menubar() const;
 
+    void setWindowCursor(NSCursor *cursor);
+
+    void registerTouch(bool enable);
+    void setContentBorderThickness(int topThickness, int bottomThickness);
+    void applyContentBorderThickness(NSWindow *window);
+
     qreal devicePixelRatio() const;
+    bool isWindowExposable();
     void exposeWindow();
     void obscureWindow();
+    void updateExposedGeometry();
+    QWindow *childWindowAt(QPoint windowPoint);
 protected:
     // NSWindow handling. The QCocoaWindow/QNSView can either be displayed
     // in an existing NSWindow or in one created by Qt.
@@ -163,21 +183,42 @@ public: // for QNSView
     friend class QCocoaBackingStore;
     friend class QCocoaNativeInterface;
 
-    QNSView *m_contentView;
+    NSView *m_contentView;
+    QNSView *m_qtView;
     NSWindow *m_nsWindow;
+
+    // TODO merge to one variable if possible
+    bool m_contentViewIsEmbedded; // true if the m_contentView is actually embedded in a "foreign" NSView hiearchy
+    bool m_contentViewIsToBeEmbedded; // true if the m_contentView is intended to be embedded in a "foreign" NSView hiearchy
+
     QNSWindowDelegate *m_nsWindowDelegate;
     Qt::WindowFlags m_windowFlags;
     Qt::WindowState m_synchedWindowState;
     Qt::WindowModality m_windowModality;
     QPointer<QWindow> m_activePopupWindow;
+    QPointer<QWindow> m_enterLeaveTargetWindow;
+    bool m_windowUnderMouse;
 
     bool m_inConstructor;
     QCocoaGLContext *m_glContext;
     QCocoaMenuBar *m_menubar;
+    NSCursor *m_windowCursor;
 
     bool m_hasModalSession;
     bool m_frameStrutEventsEnabled;
+    bool m_geometryUpdateExposeAllowed;
     bool m_isExposed;
+    QRect m_exposedGeometry;
+    int m_registerTouchCount;
+    bool m_resizableTransientParent;
+
+    static const int NoAlertRequest;
+    NSInteger m_alertRequest;
+    id monitor;
+
+    bool m_drawContentBorderGradient;
+    int m_topContentBorderThickness;
+    int m_bottomContentBorderThickness;
 };
 
 QT_END_NAMESPACE
