@@ -58,6 +58,11 @@
 
 #include <QDebug>
 
+enum {
+    defaultWindowWidth = 160,
+    defaultWindowHeight = 160
+};
+
 static bool isMouseEvent(NSEvent *ev)
 {
     switch ([ev type]) {
@@ -113,7 +118,7 @@ static bool isMouseEvent(NSEvent *ev)
 
     // Windows with a transient parent (such as combobox popup windows)
     // cannot become the main window:
-    if (m_cocoaPlatformWindow->window()->transientParent())
+    if (!m_cocoaPlatformWindow || m_cocoaPlatformWindow->window()->transientParent())
         canBecomeMain = NO;
 
     return canBecomeMain;
@@ -150,8 +155,12 @@ static bool isMouseEvent(NSEvent *ev)
 
 - (BOOL)canBecomeKeyWindow
 {
+    if (!m_cocoaPlatformWindow)
+        return NO;
+
     // Only tool or dialog windows should become key:
-    if (m_cocoaPlatformWindow->window()->type() == Qt::Tool || m_cocoaPlatformWindow->window()->type() == Qt::Dialog)
+    if (m_cocoaPlatformWindow
+        && (m_cocoaPlatformWindow->window()->type() == Qt::Tool || m_cocoaPlatformWindow->window()->type() == Qt::Dialog))
         return YES;
     return NO;
 }
@@ -183,29 +192,62 @@ static bool isMouseEvent(NSEvent *ev)
 
 @end
 
+const int QCocoaWindow::NoAlertRequest = -1;
+
 QCocoaWindow::QCocoaWindow(QWindow *tlw)
     : QPlatformWindow(tlw)
+    , m_contentView(nil)
+    , m_qtView(nil)
     , m_nsWindow(0)
+    , m_contentViewIsEmbedded(false)
+    , m_contentViewIsToBeEmbedded(false)
     , m_nsWindowDelegate(0)
     , m_synchedWindowState(Qt::WindowActive)
     , m_windowModality(Qt::NonModal)
+    , m_windowUnderMouse(false)
     , m_inConstructor(true)
     , m_glContext(0)
     , m_menubar(0)
+    , m_windowCursor(0)
     , m_hasModalSession(false)
     , m_frameStrutEventsEnabled(false)
+    , m_geometryUpdateExposeAllowed(false)
     , m_isExposed(false)
+    , m_registerTouchCount(0)
+    , m_resizableTransientParent(false)
+    , m_alertRequest(NoAlertRequest)
+    , monitor(nil)
+    , m_drawContentBorderGradient(false)
+    , m_topContentBorderThickness(0)
+    , m_bottomContentBorderThickness(0)
 {
 #ifdef QT_COCOA_ENABLE_WINDOW_DEBUG
     qDebug() << "QCocoaWindow::QCocoaWindow" << this;
 #endif
     QCocoaAutoReleasePool pool;
 
-    m_contentView = [[QNSView alloc] initWithQWindow:tlw platformWindow:this];
+    if (tlw->type() == Qt::ForeignWindow) {
+        NSView *foreignView = (NSView *)WId(tlw->property("_q_foreignWinId").value<WId>());
+        setContentView(foreignView);
+    } else {
+        m_qtView = [[QNSView alloc] initWithQWindow:tlw platformWindow:this];
+        m_contentView = m_qtView;
+#if MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_7
+        // Enable high-dpi OpenGL for retina displays. Enabling has the side
+        // effect that Cocoa will start calling glViewport(0, 0, width, height),
+        // overriding any glViewport calls in application code. This is usually not a
+        // problem, except if the appilcation wants to have a "custom" viewport.
+        // (like the hellogl example)
+        if (QSysInfo::MacintoshVersion >= QSysInfo::MV_10_7
+            && tlw->surfaceType() == QSurface::OpenGLSurface)
+            [m_contentView setWantsBestResolutionOpenGLSurface:YES];
+#endif
+    }
     setGeometry(tlw->geometry());
-
     recreateWindow(parent());
-
+    tlw->setGeometry(geometry());
+    if (tlw->isTopLevel())
+        setWindowIcon(tlw->icon());
     m_inConstructor = false;
 }
 
@@ -222,6 +264,12 @@ QCocoaWindow::~QCocoaWindow()
     [m_contentView release];
     [m_nsWindow release];
     [m_nsWindowDelegate release];
+    [m_windowCursor release];
+}
+
+QSurfaceFormat QCocoaWindow::format() const
+{
+    return window()->requestedFormat();
 }
 
 void QCocoaWindow::setGeometry(const QRect &rect)
@@ -231,25 +279,34 @@ void QCocoaWindow::setGeometry(const QRect &rect)
 #ifdef QT_COCOA_ENABLE_WINDOW_DEBUG
     qDebug() << "QCocoaWindow::setGeometry" << this << rect;
 #endif
-    QPlatformWindow::setGeometry(rect);
     setCocoaGeometry(rect);
 }
 
 void QCocoaWindow::setCocoaGeometry(const QRect &rect)
 {
     QCocoaAutoReleasePool pool;
+
+    if (m_contentViewIsEmbedded) {
+        QPlatformWindow::setGeometry(rect);
+        return;
+    }
+
     if (m_nsWindow) {
         NSRect bounds = qt_mac_flipRect(rect, window());
-        [m_nsWindow setContentSize : bounds.size];
-        [m_nsWindow setFrameOrigin : bounds.origin];
+        [m_nsWindow setFrame:[m_nsWindow frameRectForContentRect:bounds] display:YES animate:NO];
     } else {
         [m_contentView setFrame : NSMakeRect(rect.x(), rect.y(), rect.width(), rect.height())];
     }
+
+    // will call QPlatformWindow::setGeometry(rect) during resize confirmation (see qnsview.mm)
 }
 
 void QCocoaWindow::setVisible(bool visible)
 {
     QCocoaAutoReleasePool pool;
+    QCocoaWindow *parentCocoaWindow = 0;
+    if (window()->transientParent())
+        parentCocoaWindow = static_cast<QCocoaWindow *>(window()->transientParent()->handle());
 #ifdef QT_COCOA_ENABLE_WINDOW_DEBUG
     qDebug() << "QCocoaWindow::setVisible" << window() << visible;
 #endif
@@ -257,10 +314,7 @@ void QCocoaWindow::setVisible(bool visible)
         // We need to recreate if the modality has changed as the style mask will need updating
         if (m_windowModality != window()->modality())
             recreateWindow(parent());
-        QCocoaWindow *parentCocoaWindow = 0;
-        if (window()->transientParent()) {
-            parentCocoaWindow = static_cast<QCocoaWindow *>(window()->transientParent()->handle());
-
+        if (parentCocoaWindow) {
             // The parent window might have moved while this window was hidden,
             // update the window geometry if there is a parent.
             setGeometry(window()->geometry());
@@ -270,6 +324,16 @@ void QCocoaWindow::setVisible(bool visible)
             if (window()->type() == Qt::Popup) {
                 // qDebug() << "transientParent and popup" << window()->type() << Qt::Popup << (window()->type() & Qt::Popup);
                 parentCocoaWindow->m_activePopupWindow = window();
+                // QTBUG-30266: a window should not be resizable while a transient popup is open
+                // Since this isn't a native popup, the window manager doesn't close the popup when you click outside
+                NSUInteger parentStyleMask = [parentCocoaWindow->m_nsWindow styleMask];
+                if ((m_resizableTransientParent = (parentStyleMask & NSResizableWindowMask))
+#if MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_7
+                    && QSysInfo::QSysInfo::MacintoshVersion >= QSysInfo::MV_10_7
+                    && !([parentCocoaWindow->m_nsWindow styleMask] & NSFullScreenWindowMask)
+#endif
+                    )
+                    [parentCocoaWindow->m_nsWindow setStyleMask:parentStyleMask & ~NSResizableWindowMask];
             }
 
         }
@@ -282,6 +346,8 @@ void QCocoaWindow::setVisible(bool visible)
         exposeWindow();
 
         if (m_nsWindow) {
+            QWindowSystemInterface::flushWindowSystemEvents();
+
             // setWindowState might have been called while the window was hidden and
             // will not change the NSWindow state in that case. Sync up here:
             syncWindowState(window()->windowState());
@@ -309,13 +375,23 @@ void QCocoaWindow::setVisible(bool visible)
                 if ((window()->type() == Qt::Popup || window()->type() == Qt::Dialog || window()->type() == Qt::Tool)
                     && [m_nsWindow isKindOfClass:[NSPanel class]]) {
                     [(NSPanel *)m_nsWindow setWorksWhenModal:YES];
+                    if (!(parentCocoaWindow && window()->transientParent()->isActive()) && window()->type() == Qt::Popup) {
+                        monitor = [NSEvent addGlobalMonitorForEventsMatchingMask:NSLeftMouseDownMask|NSRightMouseDownMask|NSOtherMouseDown handler:^(NSEvent *) {
+                            QWindowSystemInterface::handleMouseEvent(window(), QPointF(-1, -1), QPointF(window()->framePosition() - QPointF(1, 1)), Qt::LeftButton);
+                        }];
+                    }
                 }
             }
-        } else {
-            [m_contentView setHidden:NO];
         }
+        // In some cases, e.g. QDockWidget, the content view is hidden before moving to its own
+        // Cocoa window, and then shown again. Therefore, we test for the view being hidden even
+        // if it's attached to an NSWindow.
+        if ([m_contentView isHidden])
+            [m_contentView setHidden:NO];
     } else {
         // qDebug() << "close" << this;
+        if (m_glContext)
+            m_glContext->windowWasHidden();
         if (m_nsWindow) {
             if (m_hasModalSession) {
                 QCocoaEventDispatcher *cocoaEventDispatcher = qobject_cast<QCocoaEventDispatcher *>(QGuiApplication::instance()->eventDispatcher());
@@ -323,14 +399,40 @@ void QCocoaWindow::setVisible(bool visible)
                 QCocoaEventDispatcherPrivate *cocoaEventDispatcherPrivate = static_cast<QCocoaEventDispatcherPrivate *>(QObjectPrivate::get(cocoaEventDispatcher));
                 cocoaEventDispatcherPrivate->endModalSession(window());
                 m_hasModalSession = false;
+
+                [m_nsWindow orderOut:m_nsWindow];
+                if (m_nsWindow == [NSApp keyWindow] && !cocoaEventDispatcherPrivate->currentModalSession()) {
+                    // Probably because we call runModalSession: outside [NSApp run] in QCocoaEventDispatcher
+                    // (e.g., when show()-ing a modal QDialog instead of exec()-ing it), it can happen that
+                    // the current NSWindow is still key after being ordered out. Then, after checking we
+                    // don't have any other modal session left, it's safe to make the main window key again.
+                    NSWindow *mainWindow = [NSApp mainWindow];
+                    if (mainWindow && [mainWindow canBecomeKeyWindow])
+                        [mainWindow makeKeyWindow];
+                }
             } else {
                 if ([m_nsWindow isSheet])
                     [NSApp endSheet:m_nsWindow];
+                [m_nsWindow orderOut:m_nsWindow];
             }
-            [m_nsWindow orderOut:m_nsWindow];
         } else {
             [m_contentView setHidden:YES];
         }
+        if (monitor && window()->type() == Qt::Popup) {
+            [NSEvent removeMonitor:monitor];
+            monitor = nil;
+        }
+        if (parentCocoaWindow && window()->type() == Qt::Popup) {
+            parentCocoaWindow->m_activePopupWindow = 0;
+            if (m_resizableTransientParent
+#if MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_7
+                && QSysInfo::QSysInfo::MacintoshVersion >= QSysInfo::MV_10_7
+                && !([parentCocoaWindow->m_nsWindow styleMask] & NSFullScreenWindowMask)
+#endif
+                )
+                // QTBUG-30266: a window should not be resizable while a transient popup is open
+                [parentCocoaWindow->m_nsWindow setStyleMask:[parentCocoaWindow->m_nsWindow styleMask] | NSResizableWindowMask];
+        }
     }
 }
 
@@ -352,11 +454,13 @@ NSInteger QCocoaWindow::windowLevel(Qt::WindowFlags flags)
     if (type == Qt::ToolTip)
         windowLevel = NSScreenSaverWindowLevel;
 
-    // A window should be in at least the same level as its parent.
-    const QWindow * const transientParent = window()->transientParent();
-    const QCocoaWindow * const transientParentWindow = transientParent ? static_cast<QCocoaWindow *>(transientParent->handle()) : 0;
-    if (transientParentWindow)
-        windowLevel = qMax([transientParentWindow->m_nsWindow level], windowLevel);
+    // Any "special" window should be in at least the same level as its parent.
+    if (type != Qt::Window) {
+        const QWindow * const transientParent = window()->transientParent();
+        const QCocoaWindow * const transientParentWindow = transientParent ? static_cast<QCocoaWindow *>(transientParent->handle()) : 0;
+        if (transientParentWindow)
+            windowLevel = qMax([transientParentWindow->m_nsWindow level], windowLevel);
+    }
 
     return windowLevel;
 }
@@ -365,21 +469,30 @@ NSUInteger QCocoaWindow::windowStyleMask(Qt::WindowFlags flags)
 {
     Qt::WindowType type = static_cast<Qt::WindowType>(int(flags & Qt::WindowType_Mask));
     NSInteger styleMask = NSBorderlessWindowMask;
-
+    if (flags & Qt::FramelessWindowHint)
+        return styleMask;
     if ((type & Qt::Popup) == Qt::Popup) {
         if (!windowIsPopupType(type))
             styleMask = (NSUtilityWindowMask | NSResizableWindowMask | NSClosableWindowMask |
                          NSMiniaturizableWindowMask | NSTitledWindowMask);
     } else {
-        // Filter flags for supported properties
-        flags &= Qt::WindowType_Mask | Qt::FramelessWindowHint | Qt::WindowTitleHint |
-                 Qt::WindowMinMaxButtonsHint | Qt::WindowCloseButtonHint;
-        if (flags == Qt::Window) {
+        if (type == Qt::Window && !(flags & Qt::CustomizeWindowHint)) {
             styleMask = (NSResizableWindowMask | NSClosableWindowMask | NSMiniaturizableWindowMask | NSTitledWindowMask);
-        } else if ((flags & Qt::Dialog) && (window()->modality() != Qt::NonModal)) {
-            styleMask = NSResizableWindowMask | NSTitledWindowMask;
-        } else if (!(flags & Qt::FramelessWindowHint)) {
-            if ((flags & Qt::Dialog) || (flags & Qt::WindowMaximizeButtonHint))
+        } else if (type == Qt::Dialog) {
+            if (flags & Qt::CustomizeWindowHint) {
+                if (flags & Qt::WindowMaximizeButtonHint)
+                    styleMask = NSResizableWindowMask;
+                if (flags & Qt::WindowTitleHint)
+                    styleMask |= NSTitledWindowMask;
+                if (flags & Qt::WindowCloseButtonHint)
+                    styleMask |= NSClosableWindowMask;
+                if (flags & Qt::WindowMinimizeButtonHint)
+                    styleMask |= NSMiniaturizableWindowMask;
+            } else {
+                styleMask = NSResizableWindowMask | NSClosableWindowMask | NSTitledWindowMask;
+            }
+        } else {
+            if (flags & Qt::WindowMaximizeButtonHint)
                 styleMask |= NSResizableWindowMask;
             if (flags & Qt::WindowTitleHint)
                 styleMask |= NSTitledWindowMask;
@@ -390,6 +503,9 @@ NSUInteger QCocoaWindow::windowStyleMask(Qt::WindowFlags flags)
         }
     }
 
+    if (m_drawContentBorderGradient)
+        styleMask |= NSTexturedBackgroundWindowMask;
+
 #ifdef QT_COCOA_ENABLE_WINDOW_DEBUG
     qDebug("windowStyleMask of '%s': flags %X -> styleMask %lX", qPrintable(window()->title()), (int)flags, styleMask);
 #endif
@@ -410,6 +526,23 @@ void QCocoaWindow::setWindowFlags(Qt::WindowFlags flags)
         [m_nsWindow setStyleMask:styleMask];
         [m_nsWindow setLevel:level];
         setWindowShadow(flags);
+        if (!(styleMask & NSBorderlessWindowMask)) {
+            setWindowTitle(window()->title());
+        }
+
+#if MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_7
+        if (QSysInfo::QSysInfo::MacintoshVersion >= QSysInfo::MV_10_7) {
+            Qt::WindowType type = window()->type();
+            if ((type & Qt::Popup) != Qt::Popup && (type & Qt::Dialog) != Qt::Dialog) {
+                NSWindowCollectionBehavior behavior = [m_nsWindow collectionBehavior];
+                if (flags & Qt::WindowFullscreenButtonHint)
+                    behavior |= NSWindowCollectionBehaviorFullScreenPrimary;
+                else
+                    behavior &= ~NSWindowCollectionBehaviorFullScreenPrimary;
+                [m_nsWindow setCollectionBehavior:behavior];
+            }
+        }
+#endif
     }
 
     m_windowFlags = flags;
@@ -417,7 +550,7 @@ void QCocoaWindow::setWindowFlags(Qt::WindowFlags flags)
 
 void QCocoaWindow::setWindowState(Qt::WindowState state)
 {
-    if ([m_nsWindow isVisible])
+    if (window()->isVisible())
         syncWindowState(state);  // Window state set for hidden windows take effect when show() is called.
 }
 
@@ -448,6 +581,8 @@ void QCocoaWindow::setWindowIcon(const QIcon &icon)
 
     NSButton *iconButton = [m_nsWindow standardWindowButton:NSWindowDocumentIconButton];
     if (iconButton == nil) {
+        if (icon.isNull())
+            return;
         NSString *title = QCFString::toNSString(window()->title());
         [m_nsWindow setRepresentedURL:[NSURL fileURLWithPath:title]];
         iconButton = [m_nsWindow standardWindowButton:NSWindowDocumentIconButton];
@@ -462,6 +597,21 @@ void QCocoaWindow::setWindowIcon(const QIcon &icon)
     }
 }
 
+void QCocoaWindow::setAlertState(bool enabled)
+{
+    if (m_alertRequest == NoAlertRequest && enabled) {
+        m_alertRequest = [NSApp requestUserAttention:NSCriticalRequest];
+    } else if (m_alertRequest != NoAlertRequest && !enabled) {
+        [NSApp cancelUserAttentionRequest:m_alertRequest];
+        m_alertRequest = NoAlertRequest;
+    }
+}
+
+bool QCocoaWindow::isAlertState() const
+{
+    return m_alertRequest != NoAlertRequest;
+}
+
 void QCocoaWindow::raise()
 {
     //qDebug() << "raise" << this;
@@ -485,6 +635,14 @@ bool QCocoaWindow::isExposed() const
     return m_isExposed;
 }
 
+bool QCocoaWindow::isOpaque() const
+{
+    bool translucent = (window()->format().alphaBufferSize() > 0
+                        || window()->opacity() < 1
+                        || (m_qtView && [m_qtView hasMask]));
+    return !translucent;
+}
+
 void QCocoaWindow::propagateSizeHints()
 {
     QCocoaAutoReleasePool pool;
@@ -525,18 +683,19 @@ void QCocoaWindow::propagateSizeHints()
 
 void QCocoaWindow::setOpacity(qreal level)
 {
-    if (m_nsWindow)
+    if (m_nsWindow) {
         [m_nsWindow setAlphaValue:level];
+        [m_nsWindow setOpaque: isOpaque()];
+    }
 }
 
 void QCocoaWindow::setMask(const QRegion &region)
 {
-    if (m_nsWindow) {
-        [m_nsWindow setOpaque:NO];
+    if (m_nsWindow)
         [m_nsWindow setBackgroundColor:[NSColor clearColor]];
-    }
 
-    [m_contentView setMaskRegion:&region];
+    [m_qtView setMaskRegion:&region];
+    [m_nsWindow setOpaque: isOpaque()];
 }
 
 bool QCocoaWindow::setKeyboardGrabEnabled(bool grab)
@@ -571,7 +730,10 @@ WId QCocoaWindow::winId() const
 void QCocoaWindow::setParent(const QPlatformWindow *parentWindow)
 {
     // recreate the window for compatibility
+    bool unhideAfterRecreate = parentWindow && !m_contentViewIsToBeEmbedded && ![m_contentView isHidden];
     recreateWindow(parentWindow);
+    if (unhideAfterRecreate)
+        [m_contentView setHidden:NO];
     setCocoaGeometry(geometry());
 }
 
@@ -580,6 +742,25 @@ NSView *QCocoaWindow::contentView() const
     return m_contentView;
 }
 
+void QCocoaWindow::setContentView(NSView *contentView)
+{
+    // Remove and release the previous content view
+    [m_contentView removeFromSuperview];
+    [m_contentView release];
+
+    // Insert and retain the new content view
+    [contentView retain];
+    m_contentView = contentView;
+    m_qtView = 0; // The new content view is not a QNSView.
+    recreateWindow(parent()); // Adds the content view to parent NSView
+}
+
+void QCocoaWindow::setEmbeddedInForeignView(bool embedded)
+{
+    m_contentViewIsToBeEmbedded = embedded;
+    recreateWindow(0); // destroy what was already created
+}
+
 void QCocoaWindow::windowWillMove()
 {
     // Close any open popups on window move
@@ -592,7 +773,7 @@ void QCocoaWindow::windowWillMove()
 
 void QCocoaWindow::windowDidMove()
 {
-    [m_contentView updateGeometry];
+    [m_qtView updateGeometry];
 }
 
 void QCocoaWindow::windowDidResize()
@@ -600,13 +781,21 @@ void QCocoaWindow::windowDidResize()
     if (!m_nsWindow)
         return;
 
-    [m_contentView updateGeometry];
+    [m_qtView updateGeometry];
 }
 
-void QCocoaWindow::windowWillClose()
+bool QCocoaWindow::windowShouldClose()
 {
-    QWindowSystemInterface::handleCloseEvent(window());
+    bool accepted = false;
+    QWindowSystemInterface::handleCloseEvent(window(), &accepted);
     QWindowSystemInterface::flushWindowSystemEvents();
+    return accepted;
+}
+
+void QCocoaWindow::setSynchedWindowStateFromWindow()
+{
+    if (QWindow *w = window())
+        m_synchedWindowState = w->windowState();
 }
 
 bool QCocoaWindow::windowIsPopupType(Qt::WindowType type) const
@@ -641,7 +830,9 @@ void QCocoaWindow::recreateWindow(const QPlatformWindow *parentWindow)
         m_nsWindowDelegate = 0;
     }
 
-    if (!parentWindow) {
+    if (m_contentViewIsToBeEmbedded) {
+        // An embedded window doesn't have its own NSWindow.
+    } else if (!parentWindow) {
         // Create a new NSWindow if this is a top-level window.
         m_nsWindow = createNSWindow();
         setNSWindow(m_nsWindow);
@@ -658,6 +849,7 @@ void QCocoaWindow::recreateWindow(const QPlatformWindow *parentWindow)
         QRect rect = window()->geometry();
         NSRect frame = NSMakeRect(rect.x(), rect.y(), rect.width(), rect.height());
         [m_contentView setFrame:frame];
+        [m_contentView setHidden: YES];
     }
 
     const qreal opacity = qt_window_private(window())->opacity;
@@ -665,11 +857,19 @@ void QCocoaWindow::recreateWindow(const QPlatformWindow *parentWindow)
         setOpacity(opacity);
 }
 
+void QCocoaWindow::requestActivateWindow()
+{
+    NSWindow *window = [m_contentView window];
+    [ window makeFirstResponder : m_contentView ];
+    [ window makeKeyWindow ];
+}
+
 NSWindow * QCocoaWindow::createNSWindow()
 {
     QCocoaAutoReleasePool pool;
 
-    NSRect frame = qt_mac_flipRect(window()->geometry(), window());
+    QRect rect = initialGeometry(window(), window()->geometry(), defaultWindowWidth, defaultWindowHeight);
+    NSRect frame = qt_mac_flipRect(rect, window());
 
     Qt::WindowType type = window()->type();
     Qt::WindowFlags flags = window()->flags();
@@ -688,13 +888,15 @@ NSWindow * QCocoaWindow::createNSWindow()
                                                     // before the window is shown and needs a proper window.).
         if ((type & Qt::Popup) == Qt::Popup)
             [window setHasShadow:YES];
-        else
-             setWindowShadow(flags);
+        [window setHidesOnDeactivate: NO];
 
 #if MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_7
         if (QSysInfo::QSysInfo::MacintoshVersion >= QSysInfo::MV_10_7) {
             // Make popup winows show on the same desktop as the parent full-screen window.
             [window setCollectionBehavior:NSWindowCollectionBehaviorFullScreenAuxiliary];
+
+            if ((type & Qt::Popup) == Qt::Popup)
+                [window setAnimationBehavior:NSWindowAnimationBehaviorUtilityWindow];
         }
 #endif
         window->m_cocoaPlatformWindow = this;
@@ -707,14 +909,6 @@ NSWindow * QCocoaWindow::createNSWindow()
                                          defer:NO]; // Deferring window creation breaks OpenGL (the GL context is set up
                                                     // before the window is shown and needs a proper window.).
         window->m_cocoaPlatformWindow = this;
-        setWindowShadow(flags);
-
-#if MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_7
-        if (QSysInfo::QSysInfo::MacintoshVersion >= QSysInfo::MV_10_7) {
-            if (flags & Qt::WindowFullscreenButtonHint)
-                [window setCollectionBehavior:NSWindowCollectionBehaviorFullScreenPrimary];
-        }
-#endif
 
         createdWindow = window;
     }
@@ -726,7 +920,16 @@ NSWindow * QCocoaWindow::createNSWindow()
 
     NSInteger level = windowLevel(flags);
     [createdWindow setLevel:level];
+
+    if (window()->format().alphaBufferSize() > 0) {
+        [createdWindow setBackgroundColor:[NSColor clearColor]];
+        [createdWindow setOpaque:NO];
+    }
+
     m_windowModality = window()->modality();
+
+    applyContentBorderThickness(createdWindow);
+
     return createdWindow;
 }
 
@@ -734,7 +937,6 @@ void QCocoaWindow::setNSWindow(NSWindow *window)
 {
     m_nsWindowDelegate = [[QNSWindowDelegate alloc] initWithQCocoaWindow:this];
     [window setDelegate:m_nsWindowDelegate];
-    [window setAcceptsMouseMovedEvents:YES];
 
     // Prevent Cocoa from releasing the window on close. Qt
     // handles the close event asynchronously and we want to
@@ -743,10 +945,11 @@ void QCocoaWindow::setNSWindow(NSWindow *window)
     [window setReleasedWhenClosed : NO];
 
 
-    [[NSNotificationCenter defaultCenter] addObserver:m_contentView
-                                          selector:@selector(windowNotification:)
-                                          name:nil // Get all notifications
-                                          object:m_nsWindow];
+    if (m_qtView)
+        [[NSNotificationCenter defaultCenter] addObserver:m_qtView
+                                              selector:@selector(windowNotification:)
+                                              name:nil // Get all notifications
+                                              object:m_nsWindow];
 
     [m_contentView setPostsFrameChangedNotifications: NO];
     [window setContentView:m_contentView];
@@ -758,7 +961,8 @@ void QCocoaWindow::clearNSWindow(NSWindow *window)
     [window setContentView:nil];
     [window setDelegate:nil];
     [window clearPlatformWindow];
-    [[NSNotificationCenter defaultCenter] removeObserver:m_contentView];
+    [[NSNotificationCenter defaultCenter] removeObserver:m_contentView
+                                          name:nil object:window];
 }
 
 // Returns the current global screen geometry for the nswindow associated with this window.
@@ -842,11 +1046,77 @@ QCocoaMenuBar *QCocoaWindow::menubar() const
     return m_menubar;
 }
 
+void QCocoaWindow::setWindowCursor(NSCursor *cursor)
+{
+    // This function is called (via QCocoaCursor) by Qt to set
+    // the cursor for this window. It can be called for a window
+    // that is not currenly under the mouse pointer (for example
+    // for a popup window.) Qt expects the set cursor to "stick":
+    // it should be accociated with the window until a different
+    // cursor is set.
+
+    // Cocoa has different abstractions. We can set the cursor *now*:
+    if (m_windowUnderMouse)
+        [cursor set];
+    // or we can set the cursor on mouse enter/leave using tracking
+    // areas. This is done in QNSView, save the cursor:
+    if (m_windowCursor != cursor) {
+        [m_windowCursor release];
+        m_windowCursor = [cursor retain];
+    }
+}
+
+void QCocoaWindow::registerTouch(bool enable)
+{
+    m_registerTouchCount += enable ? 1 : -1;
+    if (enable && m_registerTouchCount == 1)
+        [m_contentView setAcceptsTouchEvents:YES];
+    else if (m_registerTouchCount == 0)
+        [m_contentView setAcceptsTouchEvents:NO];
+}
+
+void QCocoaWindow::setContentBorderThickness(int topThickness, int bottomThickness)
+{
+    m_topContentBorderThickness = topThickness;
+    m_bottomContentBorderThickness = bottomThickness;
+    bool enable = (topThickness > 0 || bottomThickness > 0);
+    m_drawContentBorderGradient = enable;
+
+    applyContentBorderThickness(m_nsWindow);
+}
+
+void QCocoaWindow::applyContentBorderThickness(NSWindow *window)
+{
+    if (!window)
+        return;
+
+    if (m_drawContentBorderGradient)
+        [window setStyleMask:[window styleMask] | NSTexturedBackgroundWindowMask];
+    else
+        [window setStyleMask:[window styleMask] & ~NSTexturedBackgroundWindowMask];
+
+    if (m_topContentBorderThickness > 0) {
+        [window setContentBorderThickness:m_topContentBorderThickness forEdge:NSMaxYEdge];
+        [window setAutorecalculatesContentBorderThickness:NO forEdge:NSMaxYEdge];
+    }
+
+    if (m_bottomContentBorderThickness > 0) {
+        [window setContentBorderThickness:m_topContentBorderThickness forEdge:NSMinYEdge];
+        [window setAutorecalculatesContentBorderThickness:NO forEdge:NSMinYEdge];
+    }
+}
+
+
 qreal QCocoaWindow::devicePixelRatio() const
 {
 #if MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_7
     if (QSysInfo::MacintoshVersion >= QSysInfo::MV_10_7) {
-        return qreal([[m_contentView window] backingScaleFactor]);
+        NSWindow* window = [m_contentView window];
+        if (window) {
+          return qreal([window backingScaleFactor]);
+        } else {
+          return 1.0;
+        }
     } else
 #endif
     {
@@ -854,22 +1124,73 @@ qreal QCocoaWindow::devicePixelRatio() const
     }
 }
 
+// Returns whether the window can be expose, which it can
+// if it is on screen and has a valid geometry.
+bool QCocoaWindow::isWindowExposable()
+{
+    QSize size = geometry().size();
+    bool validGeometry = (size.width() > 0 && size.height() > 0);
+    bool validScreen = ([[m_contentView window] screen] != 0);
+    bool nonHiddenSuperView = ![[m_contentView superview] isHidden];
+    return (validGeometry && validScreen && nonHiddenSuperView);
+}
+
+// Exposes the window by posting an expose event to QWindowSystemInterface
 void QCocoaWindow::exposeWindow()
 {
+    m_geometryUpdateExposeAllowed = true;
+
+    if (!isWindowExposable())
+        return;
+
     if (!m_isExposed) {
         m_isExposed = true;
+        m_exposedGeometry = geometry();
         QWindowSystemInterface::handleExposeEvent(window(), QRegion(geometry()));
     }
 }
 
+// Obscures the window by posting an empty expose event to QWindowSystemInterface
 void QCocoaWindow::obscureWindow()
 {
     if (m_isExposed) {
+        m_geometryUpdateExposeAllowed = false;
         m_isExposed = false;
         QWindowSystemInterface::handleExposeEvent(window(), QRegion());
     }
 }
 
+// Updates window geometry by posting an expose event to QWindowSystemInterface
+void QCocoaWindow::updateExposedGeometry()
+{
+    // updateExposedGeometry is not allowed to send the initial expose. If you want
+    // that call exposeWindow();
+    if (!m_geometryUpdateExposeAllowed)
+        return;
+
+    if (!isWindowExposable())
+        return;
+
+    if (m_exposedGeometry == geometry())
+        return;
+
+    m_isExposed = true;
+    m_exposedGeometry = geometry();
+    QWindowSystemInterface::handleExposeEvent(window(), QRegion(geometry()));
+}
+
+QWindow *QCocoaWindow::childWindowAt(QPoint windowPoint)
+{
+    QWindow *targetWindow = window();
+    foreach (QObject *child, targetWindow->children())
+        if (QWindow *childWindow = qobject_cast<QWindow *>(child))
+            if (QPlatformWindow *handle = childWindow->handle())
+                if (handle->isExposed() && childWindow->geometry().contains(windowPoint))
+                    targetWindow = static_cast<QCocoaWindow*>(handle)->childWindowAt(windowPoint - childWindow->position());
+
+    return targetWindow;
+}
+
 QMargins QCocoaWindow::frameMargins() const
 {
     NSRect frameW = [m_nsWindow frame];
