@@ -79,15 +79,11 @@
  */
 
 /* This is just many defines. Therefore it doesn't need things like:
-QT_BEGIN_HEADER
-
 QT_BEGIN_NAMESPACE
 
 
 QT_END_NAMESPACE
 
-QT_END_HEADER
-
 Yes, it is an informative comment ;-)
 */
 
