@@ -45,14 +45,13 @@
 #include "qcocoahelpers.h"
 #include "qcocoaaccessibility.h"
 #include "qcocoaaccessibilityelement.h"
+#include <qpa/qplatformintegration.h>
 
-#include "QAccessibleActionInterface"
+#include <QtGui/qaccessible.h>
 #include <QtCore/QDebug>
 
 #import <AppKit/NSAccessibility.h>
 
-#ifndef QT_NO_COCOA_ACCESSIBILITY
-
 @implementation QNSView (QNSViewAccessibility)
 
 // The QNSView is a container that the user does not interact directly with:
@@ -62,49 +61,58 @@
 }
 
 - (id)accessibilityAttributeValue:(NSString *)attribute {
+
+    // activate accessibility updates
+    QGuiApplicationPrivate::platformIntegration()->accessibility()->setActive(true);
+
     if ([attribute isEqualToString:NSAccessibilityRoleAttribute]) {
-        if (m_accessibleRoot)
-            return QCocoaAccessible::macRole(m_accessibleRoot);
+        if (m_window->accessibleRoot())
+            return QCocoaAccessible::macRole(m_window->accessibleRoot());
         return NSAccessibilityUnknownRole;
     } else if ([attribute isEqualToString:NSAccessibilityRoleDescriptionAttribute]) {
         return NSAccessibilityRoleDescriptionForUIElement(self);
     } else if ([attribute isEqualToString:NSAccessibilityChildrenAttribute]) {
-        if (!m_accessibleRoot)
+        if (!m_window->accessibleRoot())
             return [super accessibilityAttributeValue:attribute];
-
-        // Create QCocoaAccessibleElements for each child if the
-        // root accessible interface.
-        int numKids = m_accessibleRoot->childCount();
-        NSMutableArray *kids = [NSMutableArray arrayWithCapacity:numKids];
-        for (int i = 0; i < numKids; ++i) {
-            QCocoaAccessibleElement *element = [QCocoaAccessibleElement createElementWithInterface: m_accessibleRoot->child(i) parent:self ];
-            [kids addObject: element];
-            [element release];
-        }
-
-        return kids;
+        return QCocoaAccessible::unignoredChildren(self, m_window->accessibleRoot());
     } else {
         return [super accessibilityAttributeValue:attribute];
     }
 }
 
 - (id)accessibilityHitTest:(NSPoint)point {
-    if (!m_accessibleRoot)
+    if (!m_window->accessibleRoot())
         return [super accessibilityHitTest:point];
 
-    QAccessibleInterface *childInterface = m_accessibleRoot->childAt(point.x, qt_mac_flipYCoordinate(point.y));
+    QAccessibleInterface *childInterface = m_window->accessibleRoot()->childAt(point.x, qt_mac_flipYCoordinate(point.y));
     // No child found, meaning we hit the NSView
     if (!childInterface) {
         return [super accessibilityHitTest:point];
     }
 
     // Hit a child, forward to child accessible interface.
-
-    QCocoaAccessibleElement *accessibleElement = [QCocoaAccessibleElement createElementWithInterface: childInterface parent:self ];
+    QAccessible::Id childAxid = QAccessible::uniqueId(childInterface);
+    // FIXME: parent could be wrong
+    QCocoaAccessibleElement *accessibleElement = [QCocoaAccessibleElement createElementWithId:childAxid parent:self ];
     [accessibleElement autorelease];
     return [accessibleElement accessibilityHitTest:point];
 }
 
-@end
+- (id)accessibilityFocusedUIElement {
+    if (!m_window->accessibleRoot())
+        return [super accessibilityFocusedUIElement];
 
-#endif // QT_NO_COCOA_ACCESSIBILITY
+    QAccessibleInterface *childInterface = m_window->accessibleRoot()->focusChild();
+    if (childInterface) {
+        QAccessible::Id childAxid = QAccessible::uniqueId(childInterface);
+        // FIXME: parent could be wrong
+        QCocoaAccessibleElement *accessibleElement = [QCocoaAccessibleElement createElementWithId:childAxid parent:self];
+        [accessibleElement autorelease];
+        return accessibleElement;
+    }
+
+    // should not happen
+    return nil;
+}
+
+@end
