@@ -56,6 +56,17 @@
     return self;
 }
 
+- (void)windowDidBecomeKey:(NSNotification *)notification
+{
+    Q_UNUSED(notification);
+    if (m_cocoaWindow->m_windowUnderMouse) {
+        QPointF windowPoint;
+        QPointF screenPoint;
+        [m_cocoaWindow->m_qtView convertFromScreen:[NSEvent mouseLocation] toWindowPoint:&windowPoint andScreenPoint:&screenPoint];
+        QWindowSystemInterface::handleEnterEvent(m_cocoaWindow->m_enterLeaveTargetWindow, windowPoint, screenPoint);
+    }
+}
+
 - (void)windowDidResize:(NSNotification *)notification
 {
     Q_UNUSED(notification);
@@ -80,12 +91,22 @@
     }
 }
 
-- (void)windowWillClose:(NSNotification *)notification
+- (BOOL)windowShouldClose:(NSNotification *)notification
 {
     Q_UNUSED(notification);
     if (m_cocoaWindow) {
-        m_cocoaWindow->windowWillClose();
+        return m_cocoaWindow->windowShouldClose();
     }
+
+    return YES;
+}
+
+- (BOOL)windowShouldZoom:(NSWindow *)window toFrame:(NSRect)newFrame
+{
+    Q_UNUSED(newFrame);
+    if (m_cocoaWindow && m_cocoaWindow->m_qtView)
+        [m_cocoaWindow->m_qtView notifyWindowWillZoom:![window isZoomed]];
+    return YES;
 }
 
 @end
