@@ -90,14 +90,11 @@ static void qt_mac_clip_cg(CGContextRef hd, const QRegion &rgn, CGAffineTransfor
     if (rgn.isEmpty()) {
         CGContextAddRect(hd, CGRectMake(0, 0, 0, 0));
     } else {
-#if (MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_5)
         if (QSysInfo::MacintoshVersion >= QSysInfo::MV_10_5) {
             QCFType<HIMutableShapeRef> shape = qt_mac_QRegionToHIMutableShape(rgn);
             Q_ASSERT(!HIShapeIsEmpty(shape));
             HIShapeReplacePathInCGContext(shape, hd);
-        } else
-#endif
-        {
+        } else {
             QVector<QRect> rects = rgn.rects();
             const int count = rects.size();
             for (int i = 0; i < count; i++) {
@@ -120,8 +117,10 @@ static void qt_mac_clip_cg(CGContextRef hd, const QRegion &rgn, CGAffineTransfor
 QMacCGContext::QMacCGContext(QPainter *p)
 {
     QPaintEngine *pe = p->paintEngine();
+#ifndef QT_NO_PRINTER
     if (pe->type() == QPaintEngine::MacPrinter)
         pe = static_cast<QMacPrintEngine*>(pe)->paintEngine();
+#endif
     pe->syncState();
     context = 0;
     if (pe->type() == QPaintEngine::CoreGraphics)
@@ -338,11 +337,9 @@ CGColorSpaceRef QCoreGraphicsPaintEngine::macGenericColorSpace()
 {
 #if 0
     if (!m_genericColorSpace) {
-#if MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_4
         if (QSysInfo::MacintoshVersion >= QSysInfo::MV_10_4) {
             m_genericColorSpace = CGColorSpaceCreateWithName(kCGColorSpaceGenericRGB);
         } else
-#endif
         {
             m_genericColorSpace = CGColorSpaceCreateDeviceRGB();
         }
@@ -367,7 +364,6 @@ CGColorSpaceRef QCoreGraphicsPaintEngine::macDisplayColorSpace(const QWidget *wi
     CGColorSpaceRef colorSpace;
 
     CGDirectDisplayID displayID;
-    CMProfileRef displayProfile = 0;
     if (widget == 0) {
         displayID = CGMainDisplayID();
     } else {
@@ -381,18 +377,11 @@ CGColorSpaceRef QCoreGraphicsPaintEngine::macDisplayColorSpace(const QWidget *wi
     if ((colorSpace = m_displayColorSpaceHash.value(displayID)))
         return colorSpace;
 
-    CMError err = CMGetProfileByAVID((CMDisplayIDType)displayID, &displayProfile);
-    if (err == noErr) {
-        colorSpace = CGColorSpaceCreateWithPlatformColorSpace(displayProfile);
-    } else if (widget) {
-        return macDisplayColorSpace(0); // fall back on main display
-    }
-
+    colorSpace = CGDisplayCopyColorSpace(displayID);
     if (colorSpace == 0)
         colorSpace = CGColorSpaceCreateDeviceRGB();
 
     m_displayColorSpaceHash.insert(displayID, colorSpace);
-    CMCloseProfile(displayProfile);
     if (!m_postRoutineRegistered) {
         m_postRoutineRegistered = true;
         qAddPostRoutine(QCoreGraphicsPaintEngine::cleanUpMacColorSpaces);
@@ -1185,7 +1174,6 @@ extern "C" {
 void
 QCoreGraphicsPaintEngine::updateCompositionMode(QPainter::CompositionMode mode)
 {
-#if (MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_5)
     if (QSysInfo::MacintoshVersion >= QSysInfo::MV_10_5) {
         int cg_mode = kCGBlendModeNormal;
         switch (mode) {
@@ -1267,11 +1255,9 @@ QCoreGraphicsPaintEngine::updateCompositionMode(QPainter::CompositionMode mode)
         if (cg_mode > -1) {
             CGContextSetBlendMode(d_func()->hd, CGBlendMode(cg_mode));
         }
-    } else
-#endif
-    // The standard porter duff ops.
-    if (QSysInfo::MacintoshVersion >= QSysInfo::MV_10_3
+    } else if (QSysInfo::MacintoshVersion >= QSysInfo::MV_10_3
             && mode <= QPainter::CompositionMode_Xor) {
+        // The standard porter duff ops.
         int cg_mode = kCGCompositeModeCopy;
         switch (mode) {
         case QPainter::CompositionMode_SourceOver:
@@ -1317,7 +1303,6 @@ QCoreGraphicsPaintEngine::updateCompositionMode(QPainter::CompositionMode mode)
         if (cg_mode > -1)
             CGContextSetCompositeOperation(d_func()->hd, CGCompositeMode(cg_mode));
     } else {
-#if (MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_4)
         bool needPrivateAPI = false;
         if (QSysInfo::MacintoshVersion >= QSysInfo::MV_10_4) {
             int cg_mode = kCGBlendModeNormal;
@@ -1367,7 +1352,6 @@ QCoreGraphicsPaintEngine::updateCompositionMode(QPainter::CompositionMode mode)
             else
                 CGContextSetCompositeOperation(d_func()->hd, CGCompositeMode(cg_mode));
         }
-#endif
     }
 }
 
@@ -1376,13 +1360,8 @@ QCoreGraphicsPaintEngine::updateRenderHints(QPainter::RenderHints hints)
 {
     Q_D(QCoreGraphicsPaintEngine);
     CGContextSetShouldAntialias(d->hd, hints & QPainter::Antialiasing);
-    static const CGFloat ScaleFactor = qt_mac_get_scalefactor();
-    if (ScaleFactor > 1.) {
-        CGContextSetInterpolationQuality(d->hd, kCGInterpolationHigh);
-    } else {
-        CGContextSetInterpolationQuality(d->hd, (hints & QPainter::SmoothPixmapTransform) ?
-                                         kCGInterpolationHigh : kCGInterpolationNone);
-    }
+    CGContextSetInterpolationQuality(d->hd, (hints & QPainter::SmoothPixmapTransform) ?
+                                     kCGInterpolationHigh : kCGInterpolationNone);
     bool textAntialiasing = (hints & QPainter::TextAntialiasing) == QPainter::TextAntialiasing;
     if (!textAntialiasing || d->disabledSmoothFonts) {
         d->disabledSmoothFonts = !textAntialiasing;
