@@ -53,6 +53,8 @@
 // We mean it.
 //
 
+#include <QtCore/qglobal.h>
+
 #ifndef QT_NO_PRINTER
 
 #include <QtPrintSupport/qprinter.h>
@@ -133,6 +135,7 @@ public:
     qreal rightMargin;
     qreal bottomMargin;
     QHash<QMacPrintEngine::PrintEnginePropertyKey, QVariant> valueCache;
+    PMPaper customPaper;
     QMacPrintEnginePrivate() : mode(QPrinter::ScreenResolution), state(QPrinter::Idle),
                                orient(QPrinter::Portrait), printInfo(0), paintEngine(0),
                                hasCustomPaperSize(false), hasCustomPageMargins(false) {}
@@ -142,6 +145,7 @@ public:
     bool newPage_helper();
     void setPaperSize(QPrinter::PaperSize ps);
     QPrinter::PaperSize paperSize() const;
+    void setPaperName(const QString &name);
     QList<QVariant> supportedResolutions() const;
     inline bool isPrintSessionInitialized() const
     {
