@@ -282,21 +282,35 @@ QDirectFbKeyMap::QDirectFbKeyMap()
     insert(DIKS_MENU                  , Qt::Key_Menu);
     insert(DIKS_HELP                  , Qt::Key_Help);
 
+    insert(DIKS_CD                    , Qt::Key_CD);
     insert(DIKS_INTERNET              , Qt::Key_HomePage);
     insert(DIKS_MAIL                  , Qt::Key_LaunchMail);
     insert(DIKS_FAVORITES             , Qt::Key_Favorites);
+    insert(DIKS_PHONE                 , Qt::Key_Phone);
+    insert(DIKS_TIME                  , Qt::Key_Time);
+
+    insert(DIKS_RED                   , Qt::Key_Red);
+    insert(DIKS_GREEN                 , Qt::Key_Green);
+    insert(DIKS_YELLOW                , Qt::Key_Yellow);
+    insert(DIKS_BLUE                  , Qt::Key_Blue);
+
+    insert(DIKS_CHANNEL_UP            , Qt::Key_ChannelUp);
+    insert(DIKS_CHANNEL_DOWN          , Qt::Key_ChannelDown);
 
     insert(DIKS_BACK                  , Qt::Key_Back);
     insert(DIKS_FORWARD               , Qt::Key_Forward);
     insert(DIKS_VOLUME_UP             , Qt::Key_VolumeUp);
     insert(DIKS_VOLUME_DOWN           , Qt::Key_VolumeDown);
     insert(DIKS_MUTE                  , Qt::Key_VolumeMute);
-    insert(DIKS_PLAYPAUSE             , Qt::Key_Pause);
+    insert(DIKS_PLAYPAUSE             , Qt::Key_MediaTogglePlayPause);
     insert(DIKS_PLAY                  , Qt::Key_MediaPlay);
     insert(DIKS_STOP                  , Qt::Key_MediaStop);
     insert(DIKS_RECORD                , Qt::Key_MediaRecord);
     insert(DIKS_PREVIOUS              , Qt::Key_MediaPrevious);
     insert(DIKS_NEXT                  , Qt::Key_MediaNext);
+    insert(DIKS_REWIND                , Qt::Key_AudioRewind);
+    insert(DIKS_FASTFORWARD           , Qt::Key_AudioForward);
+    insert(DIKS_SUBTITLE              , Qt::Key_Subtitle);
 
     insert(DIKS_F1                    , Qt::Key_F1);
     insert(DIKS_F2                    , Qt::Key_F2);
