@@ -43,35 +43,60 @@
 #include "qeglfswindow.h"
 #include "qeglfscursor.h"
 #include "qeglfshooks.h"
+#include "qeglfsintegration.h"
 
+#include <QtPlatformSupport/private/qeglpbuffer_p.h>
+#include <QtGui/QSurface>
 #include <QtDebug>
 
 QT_BEGIN_NAMESPACE
 
 QEglFSContext::QEglFSContext(const QSurfaceFormat &format, QPlatformOpenGLContext *share,
                              EGLDisplay display, EGLenum eglApi)
-    : QEGLPlatformContext(hooks->surfaceFormatFor(format), share, display, eglApi)
+    : QEGLPlatformContext(QEglFSHooks::hooks()->surfaceFormatFor(format), share, display,
+                          QEglFSIntegration::chooseConfig(display, QEglFSHooks::hooks()->surfaceFormatFor(format)), eglApi),
+      m_swapIntervalSet(false)
 {
 }
 
 bool QEglFSContext::makeCurrent(QPlatformSurface *surface)
 {
-    return QEGLPlatformContext::makeCurrent(surface);
+    bool success = QEGLPlatformContext::makeCurrent(surface);
+
+    if (success && !m_swapIntervalSet) {
+        m_swapIntervalSet = true;
+        int swapInterval = 1;
+        QByteArray swapIntervalString = qgetenv("QT_QPA_EGLFS_SWAPINTERVAL");
+        if (!swapIntervalString.isEmpty()) {
+            bool ok;
+            swapInterval = swapIntervalString.toInt(&ok);
+            if (!ok)
+                swapInterval = 1;
+        }
+        eglSwapInterval(eglDisplay(), swapInterval);
+    }
+
+    return success;
 }
 
 EGLSurface QEglFSContext::eglSurfaceForPlatformSurface(QPlatformSurface *surface)
 {
-    QEglFSWindow *window = static_cast<QEglFSWindow *>(surface);
-    return window->surface();
+    if (surface->surface()->surfaceClass() == QSurface::Window)
+        return static_cast<QEglFSWindow *>(surface)->surface();
+    else
+        return static_cast<QEGLPbuffer *>(surface)->pbuffer();
 }
 
 void QEglFSContext::swapBuffers(QPlatformSurface *surface)
 {
-    QEglFSWindow *window = static_cast<QEglFSWindow *>(surface);
-    // draw the cursor
-    if (QEglFSCursor *cursor = static_cast<QEglFSCursor *>(window->screen()->cursor()))
-        cursor->paintOnScreen();
+    if (surface->surface()->surfaceClass() == QSurface::Window) {
+        QEglFSWindow *window = static_cast<QEglFSWindow *>(surface);
+        // draw the cursor
+        if (QEglFSCursor *cursor = static_cast<QEglFSCursor *>(window->screen()->cursor()))
+            cursor->paintOnScreen();
+    }
 
+    QEglFSHooks::hooks()->waitForVSync();
     QEGLPlatformContext::swapBuffers(surface);
 }
 
