@@ -62,15 +62,24 @@ QEglFSCursor::QEglFSCursor(QEglFSScreen *screen)
 }
 
 QEglFSCursor::~QEglFSCursor()
+{
+    resetResources();
+}
+
+void QEglFSCursor::resetResources()
 {
     if (QOpenGLContext::currentContext()) {
         glDeleteProgram(m_program);
         glDeleteTextures(1, &m_cursor.customCursorTexture);
         glDeleteTextures(1, &m_cursorAtlas.texture);
     }
+    m_program = 0;
+    m_cursor.customCursorTexture = 0;
+    m_cursor.customCursorPending = !m_cursor.customCursorImage.isNull();
+    m_cursorAtlas.texture = 0;
 }
 
-static GLuint createShader(GLenum shaderType, const char *program)
+GLuint QEglFSCursor::createShader(GLenum shaderType, const char *program)
 {
     GLuint shader = glCreateShader(shaderType);
     glShaderSource(shader, 1 /* count */, &program, NULL /* lengths */);
@@ -89,7 +98,7 @@ static GLuint createShader(GLenum shaderType, const char *program)
     return 0;
 }
 
-static GLuint createProgram(GLuint vshader, GLuint fshader)
+GLuint QEglFSCursor::createProgram(GLuint vshader, GLuint fshader)
 {
     GLuint program = glCreateProgram();
     glAttachShader(program, vshader);
@@ -196,15 +205,16 @@ void QEglFSCursor::changeCursor(QCursor *cursor, QWindow *window)
 
 bool QEglFSCursor::setCurrentCursor(QCursor *cursor)
 {
-    if (m_cursor.shape == cursor->shape() && cursor->shape() != Qt::BitmapCursor)
+    const Qt::CursorShape newShape = cursor ? cursor->shape() : Qt::ArrowCursor;
+    if (m_cursor.shape == newShape && newShape != Qt::BitmapCursor)
         return false;
 
     if (m_cursor.shape == Qt::BitmapCursor) {
-        m_cursor.customCursorImage = QImage(); // in case render() never uploaded it
+        m_cursor.customCursorImage = QImage();
+        m_cursor.customCursorPending = false;
     }
-
-    m_cursor.shape = cursor->shape();
-    if (cursor->shape() != Qt::BitmapCursor) { // standard cursor
+    m_cursor.shape = newShape;
+    if (newShape != Qt::BitmapCursor) { // standard cursor
         const float ws = (float)m_cursorAtlas.cursorWidth / m_cursorAtlas.width,
                     hs = (float)m_cursorAtlas.cursorHeight / m_cursorAtlas.height;
         m_cursor.textureRect = QRectF(ws * (m_cursor.shape % m_cursorAtlas.cursorsPerRow),
@@ -220,6 +230,7 @@ bool QEglFSCursor::setCurrentCursor(QCursor *cursor)
         m_cursor.texture = 0; // will get updated in the next render()
         m_cursor.size = image.size();
         m_cursor.customCursorImage = image;
+        m_cursor.customCursorPending = true;
     }
 
     return true;
@@ -254,7 +265,7 @@ void QEglFSCursor::pointerEvent(const QMouseEvent &event)
     if (event.type() != QEvent::MouseMove)
         return;
     const QRect oldCursorRect = cursorRect();
-    m_cursor.pos = event.pos();
+    m_cursor.pos = event.screenPos().toPoint();
     update(oldCursorRect | cursorRect());
 }
 
@@ -275,22 +286,22 @@ void QEglFSCursor::draw(const QRectF &r)
 {
     if (!m_program) {
         // one time initialization
+        initializeOpenGLFunctions();
         createShaderPrograms();
 
         if (!m_cursorAtlas.texture) {
             createCursorTexture(&m_cursorAtlas.texture, m_cursorAtlas.image);
-            m_cursorAtlas.image = QImage();
 
             if (m_cursor.shape != Qt::BitmapCursor)
                 m_cursor.texture = m_cursorAtlas.texture;
         }
     }
 
-    if (m_cursor.shape == Qt::BitmapCursor && !m_cursor.customCursorImage.isNull()) {
+    if (m_cursor.shape == Qt::BitmapCursor && m_cursor.customCursorPending) {
         // upload the custom cursor
         createCursorTexture(&m_cursor.customCursorTexture, m_cursor.customCursorImage);
         m_cursor.texture = m_cursor.customCursorTexture;
-        m_cursor.customCursorImage = QImage();
+        m_cursor.customCursorPending = false;
     }
 
     Q_ASSERT(m_cursor.texture);
@@ -327,7 +338,7 @@ void QEglFSCursor::draw(const QRectF &r)
     glVertexAttribPointer(m_vertexCoordEntry, 2, GL_FLOAT, GL_FALSE, 0, cursorCoordinates);
     glVertexAttribPointer(m_textureCoordEntry, 2, GL_FLOAT, GL_FALSE, 0, textureCoordinates);
 
-    glUniform1f(m_textureEntry, 0);
+    glUniform1i(m_textureEntry, 0);
 
     glEnable(GL_BLEND);
     glBlendFunc(GL_ONE, GL_ONE_MINUS_SRC_ALPHA);
