@@ -48,8 +48,6 @@
 #include <qpa/qplatformnativeinterface.h>
 #include <qpa/qplatformscreen.h>
 
-QT_BEGIN_HEADER
-
 QT_BEGIN_NAMESPACE
 
 class QEglFSIntegration : public QPlatformIntegration, public QPlatformNativeInterface
@@ -63,26 +61,45 @@ public:
     QPlatformWindow *createPlatformWindow(QWindow *window) const;
     QPlatformBackingStore *createPlatformBackingStore(QWindow *window) const;
     QPlatformOpenGLContext *createPlatformOpenGLContext(QOpenGLContext *context) const;
+    QPlatformOffscreenSurface *createPlatformOffscreenSurface(QOffscreenSurface *surface) const;
     QPlatformNativeInterface *nativeInterface() const;
 
     QPlatformFontDatabase *fontDatabase() const;
+    QPlatformServices *services() const;
 
-    QAbstractEventDispatcher *guiThreadEventDispatcher() const;
+    QAbstractEventDispatcher *createEventDispatcher() const;
+    void initialize();
 
     QVariant styleHint(QPlatformIntegration::StyleHint hint) const;
 
     // QPlatformNativeInterface
     void *nativeResourceForIntegration(const QByteArray &resource);
+    void *nativeResourceForWindow(const QByteArray &resource, QWindow *window) Q_DECL_OVERRIDE;
     void *nativeResourceForContext(const QByteArray &resource, QOpenGLContext *context);
 
+    NativeResourceForContextFunction nativeResourceFunctionForContext(const QByteArray &resource) Q_DECL_OVERRIDE;
+
+    QPlatformScreen *screen() const { return mScreen; }
+    static EGLConfig chooseConfig(EGLDisplay display, const QSurfaceFormat &format);
+
+    EGLDisplay display() const { return mDisplay; }
+
+    QPlatformInputContext *inputContext() const { return mInputContext; }
+
+protected:
+    virtual QEglFSScreen *createScreen() const;
+
 private:
+    void createInputHandlers();
+
     EGLDisplay mDisplay;
-    QAbstractEventDispatcher *mEventDispatcher;
-    QPlatformFontDatabase *mFontDb;
-    QPlatformScreen *mScreen;
+    QScopedPointer<QPlatformFontDatabase> mFontDb;
+    QScopedPointer<QPlatformServices> mServices;
+    QEglFSScreen *mScreen;
+    QPlatformInputContext *mInputContext;
+    bool mDisableInputHandlers;
 };
 
 QT_END_NAMESPACE
-QT_END_HEADER
 
 #endif // QEGLFSINTEGRATION_H
