@@ -50,10 +50,11 @@
 
 QT_BEGIN_NAMESPACE
 
-class QPlatformOpenGLContext;
 class QEglFSCursor;
+class QEglFSWindow;
+class QOpenGLContext;
 
-class QEglFSScreen : public QPlatformScreen //huh: FullScreenScreen ;) just to follow namespace
+class QEglFSScreen : public QPlatformScreen
 {
 public:
     QEglFSScreen(EGLDisplay display);
@@ -63,13 +64,37 @@ public:
     int depth() const;
     QImage::Format format() const;
 
+    QSizeF physicalSize() const;
+    QDpi logicalDpi() const;
+    Qt::ScreenOrientation nativeOrientation() const;
+    Qt::ScreenOrientation orientation() const;
+
     QPlatformCursor *cursor() const;
 
     EGLDisplay display() const { return m_dpy; }
+    EGLSurface primarySurface() const { return m_surface; }
+
+    QList<QEglFSWindow *> windows() const { return m_windows; }
+    void addWindow(QEglFSWindow *window);
+    void removeWindow(QEglFSWindow *window);
+    void moveToTop(QEglFSWindow *window);
+    void changeWindowIndex(QEglFSWindow *window, int newIdx);
+    QEglFSWindow *rootWindow();
+    QOpenGLContext *rootContext() { return m_rootContext; }
+    void setRootContext(QOpenGLContext *context) { m_rootContext = context; }
+
+protected:
+    void setPrimarySurface(EGLSurface surface);
+    virtual void topWindowChanged(QPlatformWindow *window);
 
 private:
+    friend class QEglFSWindow;
+
     EGLDisplay m_dpy;
+    EGLSurface m_surface;
     QEglFSCursor *m_cursor;
+    QList<QEglFSWindow *> m_windows;
+    QOpenGLContext *m_rootContext;
 };
 
 QT_END_NAMESPACE
