@@ -124,9 +124,13 @@ void QKmsScreen::initializeScreenMode()
 
     drmModeConnector *connector = drmModeGetConnector(m_device->fd(), m_connectorId);
     drmModeModeInfo *mode = 0;
-    if (connector->count_modes > 0)
-        mode = &connector->modes[0];
-    else
+    for (int i = 0; i < connector->count_modes; ++i) {
+        if (connector->modes[i].type & DRM_MODE_TYPE_PREFERRED) {
+            mode = &connector->modes[i];
+            break;
+        }
+    }
+    if (!mode)
         mode = &builtin_1024x768;
 
     drmModeEncoder *encoder = drmModeGetEncoder(m_device->fd(), connector->encoders[0]);
