@@ -55,8 +55,7 @@ extern "C" {
 
 #include <EGL/egl.h>
 #include <EGL/eglext.h>
-#include <GLES2/gl2.h>
-#include <GLES2/gl2ext.h>
+#include <QtGui/qopengl.h>
 
 #include <qpa/qplatformscreen.h>
 
