@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2012 Digia Plc and/or its subsidiary(-ies).
+** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
 ** This file is part of the plugins of the Qt Toolkit.
@@ -53,15 +53,16 @@ class QLinuxFbScreen : public QFbScreen
 {
     Q_OBJECT
 public:
-    QLinuxFbScreen();
+    QLinuxFbScreen(const QStringList &args);
     ~QLinuxFbScreen();
 
-    bool initialize(const QStringList &args);
+    bool initialize();
 
 public slots:
     QRegion doRedraw();
 
 private:
+    QStringList mArgs;
     int mFbFd;
     int mTtyFd;
 
