@@ -1,6 +1,6 @@
 /***************************************************************************
 **
-** Copyright (C) 2011 - 2012 Research In Motion
+** Copyright (C) 2011 - 2013 BlackBerry Limited. All rights reserved.
 ** Contact: http://www.qt-project.org/legal
 **
 ** This file is part of the plugins of the Qt Toolkit.
@@ -40,9 +40,8 @@
 ****************************************************************************/
 
 #include "qqnxglcontext.h"
-#include "qqnxrootwindow.h"
 #include "qqnxscreen.h"
-#include "qqnxwindow.h"
+#include "qqnxeglwindow.h"
 
 #include "private/qeglconvenience_p.h"
 
@@ -50,7 +49,7 @@
 #include <QtGui/QOpenGLContext>
 #include <QtGui/QScreen>
 
-#ifdef QQNXGLCONTEXT_DEBUG
+#if defined(QQNXGLCONTEXT_DEBUG)
 #define qGLContextDebug qDebug
 #else
 #define qGLContextDebug QT_NO_QDEBUG_MACRO
@@ -60,45 +59,18 @@ QT_BEGIN_NAMESPACE
 
 EGLDisplay QQnxGLContext::ms_eglDisplay = EGL_NO_DISPLAY;
 
-static EGLenum checkEGLError(const char *msg)
-{
-    static const char *errmsg[] =
-    {
-        "EGL function succeeded",
-        "EGL is not initialized, or could not be initialized, for the specified display",
-        "EGL cannot access a requested resource",
-        "EGL failed to allocate resources for the requested operation",
-        "EGL fail to access an unrecognized attribute or attribute value was passed in an attribute list",
-        "EGLConfig argument does not name a valid EGLConfig",
-        "EGLContext argument does not name a valid EGLContext",
-        "EGL current surface of the calling thread is no longer valid",
-        "EGLDisplay argument does not name a valid EGLDisplay",
-        "EGL arguments are inconsistent",
-        "EGLNativePixmapType argument does not refer to a valid native pixmap",
-        "EGLNativeWindowType argument does not refer to a valid native window",
-        "EGL one or more argument values are invalid",
-        "EGLSurface argument does not name a valid surface configured for rendering",
-        "EGL power management event has occurred",
-    };
-    EGLenum error = eglGetError();
-    fprintf(stderr, "%s: %s\n", msg, errmsg[error - EGL_SUCCESS]);
-    return error;
-}
-
 QQnxGLContext::QQnxGLContext(QOpenGLContext *glContext)
     : QPlatformOpenGLContext(),
       m_glContext(glContext),
-      m_eglSurface(EGL_NO_SURFACE),
-      m_newSurfaceRequested(true)   // Create a surface the first time makeCurrent() is called
+      m_currentEglSurface(EGL_NO_SURFACE)
 {
     qGLContextDebug() << Q_FUNC_INFO;
     QSurfaceFormat format = m_glContext->format();
 
     // Set current rendering API
     EGLBoolean eglResult = eglBindAPI(EGL_OPENGL_ES_API);
-    if (eglResult != EGL_TRUE) {
+    if (eglResult != EGL_TRUE)
         qFatal("QQNX: failed to set EGL API, err=%d", eglGetError());
-    }
 
     // Get colour channel sizes from window format
     int alphaSize = format.alphaBufferSize();
@@ -149,11 +121,18 @@ QQnxGLContext::QQnxGLContext(QOpenGLContext *glContext)
 
     // Select EGL config based on requested window format
     m_eglConfig = q_configFromGLFormat(ms_eglDisplay, format);
-    if (m_eglConfig == 0) {
+    if (m_eglConfig == 0)
         qFatal("QQnxGLContext: failed to find EGL config");
+
+    EGLContext shareContext = EGL_NO_CONTEXT;
+    if (m_glContext) {
+        QQnxGLContext *qshareContext = dynamic_cast<QQnxGLContext*>(m_glContext->shareHandle());
+        if (qshareContext) {
+            shareContext = qshareContext->m_eglContext;
+        }
     }
 
-    m_eglContext = eglCreateContext(ms_eglDisplay, m_eglConfig, EGL_NO_CONTEXT, contextAttrs());
+    m_eglContext = eglCreateContext(ms_eglDisplay, m_eglConfig, shareContext, contextAttrs());
     if (m_eglContext == EGL_NO_CONTEXT) {
         checkEGLError("eglCreateContext");
         qFatal("QQnxGLContext: failed to create EGL context, err=%d", eglGetError());
@@ -168,12 +147,33 @@ QQnxGLContext::~QQnxGLContext()
     qGLContextDebug() << Q_FUNC_INFO;
 
     // Cleanup EGL context if it exists
-    if (m_eglContext != EGL_NO_CONTEXT) {
+    if (m_eglContext != EGL_NO_CONTEXT)
         eglDestroyContext(ms_eglDisplay, m_eglContext);
-    }
+}
 
-    // Cleanup EGL surface if it exists
-    destroySurface();
+EGLenum QQnxGLContext::checkEGLError(const char *msg)
+{
+    static const char *errmsg[] =
+    {
+        "EGL function succeeded",
+        "EGL is not initialized, or could not be initialized, for the specified display",
+        "EGL cannot access a requested resource",
+        "EGL failed to allocate resources for the requested operation",
+        "EGL fail to access an unrecognized attribute or attribute value was passed in an attribute list",
+        "EGLConfig argument does not name a valid EGLConfig",
+        "EGLContext argument does not name a valid EGLContext",
+        "EGL current surface of the calling thread is no longer valid",
+        "EGLDisplay argument does not name a valid EGLDisplay",
+        "EGL arguments are inconsistent",
+        "EGLNativePixmapType argument does not refer to a valid native pixmap",
+        "EGLNativeWindowType argument does not refer to a valid native window",
+        "EGL one or more argument values are invalid",
+        "EGLSurface argument does not name a valid surface configured for rendering",
+        "EGL power management event has occurred",
+    };
+    EGLenum error = eglGetError();
+    fprintf(stderr, "%s: %s\n", msg, errmsg[error - EGL_SUCCESS]);
+    return error;
 }
 
 void QQnxGLContext::initialize()
@@ -202,12 +202,6 @@ void QQnxGLContext::shutdown()
     eglTerminate(ms_eglDisplay);
 }
 
-void QQnxGLContext::requestSurfaceChange()
-{
-    qGLContextDebug() << Q_FUNC_INFO;
-    m_newSurfaceRequested.testAndSetRelease(false, true);
-}
-
 bool QQnxGLContext::makeCurrent(QPlatformSurface *surface)
 {
     qGLContextDebug() << Q_FUNC_INFO;
@@ -216,18 +210,21 @@ bool QQnxGLContext::makeCurrent(QPlatformSurface *surface)
 
     // Set current rendering API
     EGLBoolean eglResult = eglBindAPI(EGL_OPENGL_ES_API);
-    if (eglResult != EGL_TRUE) {
+    if (eglResult != EGL_TRUE)
         qFatal("QQnxGLContext: failed to set EGL API, err=%d", eglGetError());
-    }
 
-    if (m_newSurfaceRequested.testAndSetOrdered(true, false)) {
-        qGLContextDebug() << "New EGL surface requested";
+    QQnxEglWindow *platformWindow = dynamic_cast<QQnxEglWindow*>(surface);
+    if (!platformWindow)
+        return false;
+
+    platformWindow->setPlatformOpenGLContext(this);
+
+    if (m_currentEglSurface == EGL_NO_SURFACE || m_currentEglSurface != platformWindow->getSurface()) {
+        m_currentEglSurface = platformWindow->getSurface();
         doneCurrent();
-        destroySurface();
-        createSurface(surface);
     }
 
-    eglResult = eglMakeCurrent(ms_eglDisplay, m_eglSurface, m_eglSurface, m_eglContext);
+    eglResult = eglMakeCurrent(ms_eglDisplay, m_currentEglSurface, m_currentEglSurface, m_eglContext);
     if (eglResult != EGL_TRUE) {
         checkEGLError("eglMakeCurrent");
         qFatal("QQNX: failed to set current EGL context, err=%d", eglGetError());
@@ -241,33 +238,23 @@ void QQnxGLContext::doneCurrent()
 
     // set current rendering API
     EGLBoolean eglResult = eglBindAPI(EGL_OPENGL_ES_API);
-    if (eglResult != EGL_TRUE) {
+    if (eglResult != EGL_TRUE)
         qFatal("QQNX: failed to set EGL API, err=%d", eglGetError());
-    }
 
     // clear curent EGL context and unbind EGL surface
     eglResult = eglMakeCurrent(ms_eglDisplay, EGL_NO_SURFACE, EGL_NO_SURFACE, EGL_NO_CONTEXT);
-    if (eglResult != EGL_TRUE) {
+    if (eglResult != EGL_TRUE)
         qFatal("QQNX: failed to clear current EGL context, err=%d", eglGetError());
-    }
 }
 
 void QQnxGLContext::swapBuffers(QPlatformSurface *surface)
 {
-    Q_UNUSED(surface);
     qGLContextDebug() << Q_FUNC_INFO;
+    QQnxEglWindow *platformWindow = dynamic_cast<QQnxEglWindow*>(surface);
+    if (!platformWindow)
+        return;
 
-    // Set current rendering API
-    EGLBoolean eglResult = eglBindAPI(EGL_OPENGL_ES_API);
-    if (eglResult != EGL_TRUE) {
-        qFatal("QQNX: failed to set EGL API, err=%d", eglGetError());
-    }
-
-    // Post EGL surface to window
-    eglResult = eglSwapBuffers(ms_eglDisplay, m_eglSurface);
-    if (eglResult != EGL_TRUE) {
-        qFatal("QQNX: failed to swap EGL buffers, err=%d", eglGetError());
-    }
+    platformWindow->swapEGLBuffers();
 }
 
 QFunctionPointer QQnxGLContext::getProcAddress(const QByteArray &procName)
@@ -276,14 +263,17 @@ QFunctionPointer QQnxGLContext::getProcAddress(const QByteArray &procName)
 
     // Set current rendering API
     EGLBoolean eglResult = eglBindAPI(EGL_OPENGL_ES_API);
-    if (eglResult != EGL_TRUE) {
+    if (eglResult != EGL_TRUE)
         qFatal("QQNX: failed to set EGL API, err=%d", eglGetError());
-    }
 
     // Lookup EGL extension function pointer
     return static_cast<QFunctionPointer>(eglGetProcAddress(procName.constData()));
 }
 
+EGLDisplay QQnxGLContext::getEglDisplay() {
+    return ms_eglDisplay;
+}
+
 EGLint *QQnxGLContext::contextAttrs()
 {
     qGLContextDebug() << Q_FUNC_INFO;
@@ -297,64 +287,4 @@ EGLint *QQnxGLContext::contextAttrs()
 #endif
 }
 
-bool QQnxGLContext::isCurrent() const
-{
-    qGLContextDebug() << Q_FUNC_INFO;
-    return (eglGetCurrentContext() == m_eglContext);
-}
-
-void QQnxGLContext::createSurface(QPlatformSurface *surface)
-{
-    qGLContextDebug() << Q_FUNC_INFO;
-
-    // Get a pointer to the corresponding platform window
-    QQnxWindow *platformWindow = dynamic_cast<QQnxWindow*>(surface);
-    if (!platformWindow) {
-        qFatal("QQNX: unable to create EGLSurface without a QQnxWindow");
-    }
-
-    // Link the window and context
-    platformWindow->setPlatformOpenGLContext(this);
-
-    // Fetch the surface size from the window and update
-    // the window's buffers before we create the EGL surface
-    const QSize surfaceSize = platformWindow->requestedBufferSize();
-    if (!surfaceSize.isValid()) {
-        qFatal("QQNX: Trying to create 0 size EGL surface. "
-               "Please set a valid window size before calling QOpenGLContext::makeCurrent()");
-    }
-    platformWindow->setBufferSize(surfaceSize);
-
-    // Obtain the native handle for our window
-    screen_window_t handle = platformWindow->nativeHandle();
-
-    const EGLint eglSurfaceAttrs[] =
-    {
-        EGL_RENDER_BUFFER, EGL_BACK_BUFFER,
-        EGL_NONE
-    };
-
-    // Create EGL surface
-    m_eglSurface = eglCreateWindowSurface(ms_eglDisplay, m_eglConfig, (EGLNativeWindowType) handle, eglSurfaceAttrs);
-    if (m_eglSurface == EGL_NO_SURFACE) {
-        checkEGLError("eglCreateWindowSurface");
-        qFatal("QQNX: failed to create EGL surface, err=%d", eglGetError());
-    }
-}
-
-void QQnxGLContext::destroySurface()
-{
-    qGLContextDebug() << Q_FUNC_INFO;
-
-    // Destroy EGL surface if it exists
-    if (m_eglSurface != EGL_NO_SURFACE) {
-        EGLBoolean eglResult = eglDestroySurface(ms_eglDisplay, m_eglSurface);
-        if (eglResult != EGL_TRUE) {
-            qFatal("QQNX: failed to destroy EGL surface, err=%d", eglGetError());
-        }
-    }
-
-    m_eglSurface = EGL_NO_SURFACE;
-}
-
 QT_END_NAMESPACE
