@@ -1,6 +1,6 @@
 /***************************************************************************
 **
-** Copyright (C) 2011 - 2012 Research In Motion
+** Copyright (C) 2011 - 2013 BlackBerry Limited. All rights reserved.
 ** Contact: http://www.qt-project.org/legal
 **
 ** This file is part of the plugins of the Qt Toolkit.
@@ -40,7 +40,9 @@
 ****************************************************************************/
 
 #include "qqnxintegration.h"
+#if defined(QQNX_SCREENEVENTTHREAD)
 #include "qqnxscreeneventthread.h"
+#endif
 #include "qqnxnativeinterface.h"
 #include "qqnxrasterbackingstore.h"
 #include "qqnxscreen.h"
@@ -51,6 +53,11 @@
 #include "qqnxabstractvirtualkeyboard.h"
 #include "qqnxservices.h"
 
+#include "qqnxrasterwindow.h"
+#if !defined(QT_NO_OPENGL)
+#include "qqnxeglwindow.h"
+#endif
+
 #if defined(Q_OS_BLACKBERRY)
 #include "qqnxbpseventfilter.h"
 #include "qqnxnavigatorbps.h"
@@ -84,19 +91,21 @@
 #include <qpa/qplatformwindow.h>
 #include <qpa/qwindowsysteminterface.h>
 
+#include <QtGui/private/qguiapplication_p.h>
+
 #if !defined(QT_NO_OPENGL)
 #include "qqnxglcontext.h"
 #include <QtGui/QOpenGLContext>
 #endif
 
-#include <QtPlatformSupport/private/qsimpledrag_p.h>
+#include <private/qsimpledrag_p.h>
 
 #include <QtCore/QDebug>
 #include <QtCore/QHash>
 
 #include <errno.h>
 
-#ifdef QQNXINTEGRATION_DEBUG
+#if defined(QQNXINTEGRATION_DEBUG)
 #define qIntegrationDebug qDebug
 #else
 #define qIntegrationDebug QT_NO_QDEBUG_MACRO
@@ -107,9 +116,31 @@ QT_BEGIN_NAMESPACE
 QQnxWindowMapper QQnxIntegration::ms_windowMapper;
 QMutex QQnxIntegration::ms_windowMapperMutex;
 
-QQnxIntegration::QQnxIntegration()
+static inline QQnxIntegration::Options parseOptions(const QStringList &paramList)
+{
+    QQnxIntegration::Options options = QQnxIntegration::NoOptions;
+    if (!paramList.contains(QLatin1String("no-fullscreen"))) {
+        options |= QQnxIntegration::FullScreenApplication;
+    }
+
+// On Blackberry the first window is treated as a root window
+#ifdef Q_OS_BLACKBERRY
+    if (!paramList.contains(QLatin1String("no-rootwindow"))) {
+        options |= QQnxIntegration::RootWindow;
+    }
+#else
+    if (paramList.contains(QLatin1String("rootwindow"))) {
+        options |= QQnxIntegration::RootWindow;
+    }
+#endif
+    return options;
+}
+
+QQnxIntegration::QQnxIntegration(const QStringList &paramList)
     : QPlatformIntegration()
+#if defined(QQNX_SCREENEVENTTHREAD)
     , m_screenEventThread(0)
+#endif
     , m_navigatorEventHandler(new QQnxNavigatorEventHandler())
     , m_virtualKeyboard(0)
 #if defined(QQNX_PPS)
@@ -134,14 +165,14 @@ QQnxIntegration::QQnxIntegration()
 #if !defined(QT_NO_DRAGANDDROP)
     , m_drag(new QSimpleDrag())
 #endif
+    , m_options(parseOptions(paramList))
 {
     qIntegrationDebug() << Q_FUNC_INFO;
     // Open connection to QNX composition manager
     errno = 0;
     int result = screen_create_context(&m_screenContext, SCREEN_APPLICATION_CONTEXT);
-    if (result != 0) {
+    if (result != 0)
         qFatal("QQnx: failed to connect to composition manager, errno=%d", errno);
-    }
 
     // Not on BlackBerry, it has specialized event dispatcher which also handles navigator events
 #if !defined(Q_OS_BLACKBERRY) && defined(QQNX_PPS)
@@ -186,8 +217,13 @@ QQnxIntegration::QQnxIntegration()
 
 #if defined(Q_OS_BLACKBERRY)
     QQnxVirtualKeyboardBps* virtualKeyboardBps = new QQnxVirtualKeyboardBps;
-    m_bpsEventFilter = new QQnxBpsEventFilter(m_navigatorEventHandler,
-            (m_screenEventThread ? 0 : m_screenEventHandler), virtualKeyboardBps);
+
+#if defined(QQNX_SCREENEVENTTHREAD)
+    m_bpsEventFilter = new QQnxBpsEventFilter(m_navigatorEventHandler, 0, virtualKeyboardBps);
+#else
+    m_bpsEventFilter = new QQnxBpsEventFilter(m_navigatorEventHandler, m_screenEventHandler, virtualKeyboardBps);
+#endif
+
     m_bpsEventFilter->installOnEventDispatcher(m_eventDispatcher);
 
     m_virtualKeyboard = virtualKeyboardBps;
@@ -269,6 +305,9 @@ QQnxIntegration::~QQnxIntegration()
     delete m_bpsEventFilter;
 #endif
 
+    // In case the event-dispatcher was never transferred to QCoreApplication
+    delete m_eventDispatcher;
+
     delete m_screenEventHandler;
 
     // Destroy all displays
@@ -295,9 +334,10 @@ bool QQnxIntegration::hasCapability(QPlatformIntegration::Capability cap) const
 {
     qIntegrationDebug() << Q_FUNC_INFO;
     switch (cap) {
+    case MultipleWindows:
     case ThreadedPixmaps:
         return true;
-#if defined(QT_OPENGL_ES)
+#if !defined(QT_NO_OPENGL)
     case OpenGL:
     case ThreadedOpenGL:
     case BufferQueueingOpenGL:
@@ -311,7 +351,19 @@ bool QQnxIntegration::hasCapability(QPlatformIntegration::Capability cap) const
 QPlatformWindow *QQnxIntegration::createPlatformWindow(QWindow *window) const
 {
     qIntegrationDebug() << Q_FUNC_INFO;
-    return new QQnxWindow(window, m_screenContext);
+    QSurface::SurfaceType surfaceType = window->surfaceType();
+    const bool needRootWindow = options() & RootWindow;
+    switch (surfaceType) {
+    case QSurface::RasterSurface:
+        return new QQnxRasterWindow(window, m_screenContext, needRootWindow);
+#if !defined(QT_NO_OPENGL)
+    case QSurface::OpenGLSurface:
+        return new QQnxEglWindow(window, m_screenContext, needRootWindow);
+#endif
+    default:
+        qFatal("QQnxWindow: unsupported window API");
+    }
+    return 0;
 }
 
 QPlatformBackingStore *QQnxIntegration::createPlatformBackingStore(QWindow *window) const
@@ -350,10 +402,15 @@ void QQnxIntegration::moveToScreen(QWindow *window, int screen)
     platformWindow->setScreen(platformScreen);
 }
 
-QAbstractEventDispatcher *QQnxIntegration::guiThreadEventDispatcher() const
+QAbstractEventDispatcher *QQnxIntegration::createEventDispatcher() const
 {
     qIntegrationDebug() << Q_FUNC_INFO;
-    return m_eventDispatcher;
+
+    // We transfer ownersip of the event-dispatcher to QtCoreApplication
+    QAbstractEventDispatcher *eventDispatcher = m_eventDispatcher;
+    m_eventDispatcher = 0;
+
+    return eventDispatcher;
 }
 
 QPlatformNativeInterface *QQnxIntegration::nativeInterface() const
@@ -367,9 +424,8 @@ QPlatformClipboard *QQnxIntegration::clipboard() const
     qIntegrationDebug() << Q_FUNC_INFO;
 
 #if defined(QQNX_PPS)
-    if (!m_clipboard) {
+    if (!m_clipboard)
         m_clipboard = new QQnxClipboard;
-    }
 #endif
     return m_clipboard;
 }
@@ -385,7 +441,7 @@ QPlatformDrag *QQnxIntegration::drag() const
 QVariant QQnxIntegration::styleHint(QPlatformIntegration::StyleHint hint) const
 {
     qIntegrationDebug() << Q_FUNC_INFO;
-    if (hint == ShowIsFullScreen)
+    if ((hint == ShowIsFullScreen) && (m_options & FullScreenApplication))
         return true;
 
     return QPlatformIntegration::styleHint(hint);
@@ -442,9 +498,8 @@ void QQnxIntegration::createDisplays()
     errno = 0;
     int displayCount;
     int result = screen_get_context_property_iv(m_screenContext, SCREEN_PROPERTY_DISPLAY_COUNT, &displayCount);
-    if (result != 0) {
+    if (result != 0)
         qFatal("QQnxIntegration: failed to query display count, errno=%d", errno);
-    }
 
     if (displayCount < 1) {
         // Never happens, even if there's no display, libscreen returns 1
@@ -455,9 +510,8 @@ void QQnxIntegration::createDisplays()
     errno = 0;
     screen_display_t *displays = (screen_display_t *)alloca(sizeof(screen_display_t) * displayCount);
     result = screen_get_context_property_pv(m_screenContext, SCREEN_PROPERTY_DISPLAYS, (void **)displays);
-    if (result != 0) {
+    if (result != 0)
         qFatal("QQnxIntegration: failed to query displays, errno=%d", errno);
-    }
 
     // If it's primary, we create a QScreen for it even if it's not attached
     // since Qt will dereference QGuiApplication::primaryScreen()
@@ -486,6 +540,7 @@ void QQnxIntegration::createDisplay(screen_display_t display, bool isPrimary)
     QQnxScreen *screen = new QQnxScreen(m_screenContext, display, isPrimary);
     m_screens.append(screen);
     screenAdded(screen);
+    screen->adjustOrientation();
 
     QObject::connect(m_screenEventHandler, SIGNAL(newWindowCreated(void*)),
                      screen, SLOT(newWindowCreated(void*)));
@@ -495,6 +550,8 @@ void QQnxIntegration::createDisplay(screen_display_t display, bool isPrimary)
     QObject::connect(m_navigatorEventHandler, SIGNAL(rotationChanged(int)), screen, SLOT(setRotation(int)));
     QObject::connect(m_navigatorEventHandler, SIGNAL(windowGroupActivated(QByteArray)), screen, SLOT(activateWindowGroup(QByteArray)));
     QObject::connect(m_navigatorEventHandler, SIGNAL(windowGroupDeactivated(QByteArray)), screen, SLOT(deactivateWindowGroup(QByteArray)));
+    QObject::connect(m_navigatorEventHandler, SIGNAL(windowGroupStateChanged(QByteArray,Qt::WindowState)),
+            screen, SLOT(windowGroupStateChanged(QByteArray,Qt::WindowState)));
 }
 
 void QQnxIntegration::removeDisplay(QQnxScreen *screen)
@@ -527,6 +584,11 @@ QQnxScreen *QQnxIntegration::primaryDisplay() const
     return m_screens.first();
 }
 
+QQnxIntegration::Options QQnxIntegration::options() const
+{
+    return m_options;
+}
+
 bool QQnxIntegration::supportsNavigatorEvents() const
 {
     // If QQNX_PPS or Q_OS_BLACKBERRY is defined then we have navigator
