@@ -44,7 +44,7 @@
 
 #include <qpa/qplatformscreen.h>
 
-#include "qqnxrootwindow.h"
+#include "qqnxwindow.h"
 
 #include <QtCore/QObject>
 #include <QtCore/QScopedPointer>
@@ -91,14 +91,22 @@ public:
     void lowerWindow(QQnxWindow *window);
     void updateHierarchy();
 
-    void onWindowPost(QQnxWindow *window);
+    void adjustOrientation();
 
-    QSharedPointer<QQnxRootWindow> rootWindow() const;
+    QQnxWindow *rootWindow() const;
+    void setRootWindow(QQnxWindow*);
+
+    QPlatformCursor *cursor() const;
+
+Q_SIGNALS:
+    void foreignWindowCreated(void *window);
+    void foreignWindowClosed(void *window);
 
 public Q_SLOTS:
     void setRotation(int rotation);
     void newWindowCreated(void *window);
     void windowClosed(void *window);
+    void windowGroupStateChanged(const QByteArray &id, Qt::WindowState state);
     void activateWindowGroup(const QByteArray &id);
     void deactivateWindowGroup(const QByteArray &id);
 
@@ -110,13 +118,16 @@ private:
     void resizeTopLevelWindow(QQnxWindow *w, const QRect &previousScreenGeometry) const;
     void resizeWindows(const QRect &previousScreenGeometry);
     void addOverlayWindow(screen_window_t window);
-    void removeOverlayWindow(screen_window_t window);
+    void addUnderlayWindow(screen_window_t window);
+    void addMultimediaWindow(const QByteArray &id, screen_window_t window);
+    void removeOverlayOrUnderlayWindow(screen_window_t window);
+
+    QWindow *topMostChildWindow() const;
 
     screen_context_t m_screenContext;
     screen_display_t m_display;
-    mutable QSharedPointer<QQnxRootWindow> m_rootWindow;
+    QQnxWindow *m_rootWindow;
     const bool m_primaryScreen;
-    bool m_posted;
 
     int m_initialRotation;
     int m_currentRotation;
@@ -126,10 +137,13 @@ private:
     Qt::ScreenOrientation m_nativeOrientation;
     QRect m_initialGeometry;
     QRect m_currentGeometry;
-    QPlatformOpenGLContext *m_platformContext;
 
     QList<QQnxWindow *> m_childWindows;
+    QQnxWindow *m_coverWindow;
     QList<screen_window_t> m_overlays;
+    QList<screen_window_t> m_underlays;
+
+    QPlatformCursor *m_cursor;
 };
 
 QT_END_NAMESPACE
