@@ -40,6 +40,9 @@
 ****************************************************************************/
 
 #include "qqnxscreeneventhandler.h"
+#if defined(QQNX_SCREENEVENTTHREAD)
+#include "qqnxscreeneventthread.h"
+#endif
 #include "qqnxintegration.h"
 #include "qqnxkeytranslator.h"
 #include "qqnxscreen.h"
@@ -50,7 +53,7 @@
 #include <errno.h>
 #include <sys/keycodes.h>
 
-#ifdef QQNXSCREENEVENT_DEBUG
+#if defined(QQNXSCREENEVENT_DEBUG)
 #define qScreenEventDebug qDebug
 #else
 #define qScreenEventDebug QT_NO_QDEBUG_MACRO
@@ -63,6 +66,9 @@ QQnxScreenEventHandler::QQnxScreenEventHandler(QQnxIntegration *integration)
     , m_lastButtonState(Qt::NoButton)
     , m_lastMouseWindow(0)
     , m_touchDevice(0)
+#if defined(QQNX_SCREENEVENTTHREAD)
+    , m_eventThread(0)
+#endif
 {
     // Create a touch device
     m_touchDevice = new QTouchDevice;
@@ -90,9 +96,8 @@ bool QQnxScreenEventHandler::handleEvent(screen_event_t event)
     errno = 0;
     int qnxType;
     int result = screen_get_event_property_iv(event, SCREEN_PROPERTY_TYPE, &qnxType);
-    if (result) {
+    if (result)
         qFatal("QQNX: failed to query event type, errno=%d", errno);
-    }
 
     return handleEvent(event, qnxType);
 }
@@ -126,6 +131,10 @@ bool QQnxScreenEventHandler::handleEvent(screen_event_t event, int qnxType)
         handleDisplayEvent(event);
         break;
 
+    case SCREEN_EVENT_PROPERTY:
+        handlePropertyEvent(event);
+        break;
+
     default:
         // event ignored
         qScreenEventDebug() << Q_FUNC_INFO << "unknown event" << qnxType;
@@ -161,9 +170,8 @@ void QQnxScreenEventHandler::injectKeyboardEvent(int flags, int sym, int modifie
             if ( qtMod & Qt::ControlModifier ) {
                 keyStr = QChar((int)(key & 0x3f));
             } else {
-                if (flags & KEY_SYM_VALID) {
+                if (flags & KEY_SYM_VALID)
                     keyStr = QChar(sym);
-                }
             }
         } else if ((cap > 0x0ff && cap < UNICODE_PRIVATE_USE_AREA_FIRST) || cap > UNICODE_PRIVATE_USE_AREA_LAST) {
             key = (Qt::Key)cap;
@@ -180,41 +188,73 @@ void QQnxScreenEventHandler::injectKeyboardEvent(int flags, int sym, int modifie
     }
 }
 
+#if defined(QQNX_SCREENEVENTTHREAD)
+void QQnxScreenEventHandler::setScreenEventThread(QQnxScreenEventThread *eventThread)
+{
+    m_eventThread = eventThread;
+}
+
+void QQnxScreenEventHandler::processEventsFromScreenThread()
+{
+    if (!m_eventThread)
+        return;
+
+    QQnxScreenEventArray *events = m_eventThread->lock();
+
+    for (int i = 0; i < events->size(); ++i) {
+        screen_event_t event = events->at(i);
+        if (!event)
+            continue;
+        (*events)[i] = 0;
+
+        m_eventThread->unlock();
+
+        long result = 0;
+        QAbstractEventDispatcher* dispatcher = QAbstractEventDispatcher::instance();
+        bool handled = dispatcher && dispatcher->filterNativeEvent(QByteArrayLiteral("screen_event_t"), event, &result);
+        if (!handled)
+            handleEvent(event);
+        screen_destroy_event(event);
+
+        m_eventThread->lock();
+    }
+
+    events->clear();
+
+    m_eventThread->unlock();
+}
+#endif
+
 void QQnxScreenEventHandler::handleKeyboardEvent(screen_event_t event)
 {
     // get flags of key event
     errno = 0;
     int flags;
     int result = screen_get_event_property_iv(event, SCREEN_PROPERTY_KEY_FLAGS, &flags);
-    if (result) {
+    if (result)
         qFatal("QQNX: failed to query event flags, errno=%d", errno);
-    }
 
     // get key code
     errno = 0;
     int sym;
     result = screen_get_event_property_iv(event, SCREEN_PROPERTY_KEY_SYM, &sym);
-    if (result) {
+    if (result)
         qFatal("QQNX: failed to query event sym, errno=%d", errno);
-    }
 
     int modifiers;
     result = screen_get_event_property_iv(event, SCREEN_PROPERTY_KEY_MODIFIERS, &modifiers);
-    if (result) {
+    if (result)
         qFatal("QQNX: failed to query event modifiers, errno=%d", errno);
-    }
 
     int scan;
     result = screen_get_event_property_iv(event, SCREEN_PROPERTY_KEY_SCAN, &scan);
-    if (result) {
+    if (result)
         qFatal("QQNX: failed to query event modifiers, errno=%d", errno);
-    }
 
     int cap;
     result = screen_get_event_property_iv(event, SCREEN_PROPERTY_KEY_CAP, &cap);
-    if (result) {
+    if (result)
         qFatal("QQNX: failed to query event cap, errno=%d", errno);
-    }
 
     injectKeyboardEvent(flags, sym, modifiers, scan, cap);
 }
@@ -227,38 +267,34 @@ void QQnxScreenEventHandler::handlePointerEvent(screen_event_t event)
     screen_window_t qnxWindow;
     void *handle;
     int result = screen_get_event_property_pv(event, SCREEN_PROPERTY_WINDOW, &handle);
-    if (result) {
+    if (result)
         qFatal("QQNX: failed to query event window, errno=%d", errno);
-    }
+
     qnxWindow = static_cast<screen_window_t>(handle);
 
     // Query the button states
     int buttonState = 0;
     result = screen_get_event_property_iv(event, SCREEN_PROPERTY_BUTTONS, &buttonState);
-    if (result) {
+    if (result)
         qFatal("QQNX: failed to query event button state, errno=%d", errno);
-    }
 
     // Query the window position
     int windowPos[2];
     result = screen_get_event_property_iv(event, SCREEN_PROPERTY_SOURCE_POSITION, windowPos);
-    if (result) {
+    if (result)
         qFatal("QQNX: failed to query event window position, errno=%d", errno);
-    }
 
     // Query the screen position
     int pos[2];
     result = screen_get_event_property_iv(event, SCREEN_PROPERTY_POSITION, pos);
-    if (result) {
+    if (result)
         qFatal("QQNX: failed to query event position, errno=%d", errno);
-    }
 
     // Query the wheel delta
     int wheelDelta = 0;
     result = screen_get_event_property_iv(event, SCREEN_PROPERTY_MOUSE_WHEEL, &wheelDelta);
-    if (result) {
+    if (result)
         qFatal("QQNX: failed to query event wheel delta, errno=%d", errno);
-    }
 
     // Map window handle to top-level QWindow
     QWindow *w = QQnxIntegration::window(qnxWindow);
@@ -343,33 +379,32 @@ void QQnxScreenEventHandler::handleTouchEvent(screen_event_t event, int qnxType)
     errno = 0;
     int pos[2];
     int result = screen_get_event_property_iv(event, SCREEN_PROPERTY_POSITION, pos);
-    if (result) {
+    if (result)
         qFatal("QQNX: failed to query event position, errno=%d", errno);
-    }
+
+    QCursor::setPos(pos[0], pos[1]);
 
     // get window coordinates of touch
     errno = 0;
     int windowPos[2];
     result = screen_get_event_property_iv(event, SCREEN_PROPERTY_SOURCE_POSITION, windowPos);
-    if (result) {
+    if (result)
         qFatal("QQNX: failed to query event window position, errno=%d", errno);
-    }
 
     // determine which finger touched
     errno = 0;
     int touchId;
     result = screen_get_event_property_iv(event, SCREEN_PROPERTY_TOUCH_ID, &touchId);
-    if (result) {
+    if (result)
         qFatal("QQNX: failed to query event touch id, errno=%d", errno);
-    }
 
     // determine which window was touched
     errno = 0;
     void *handle;
     result = screen_get_event_property_pv(event, SCREEN_PROPERTY_WINDOW, &handle);
-    if (result) {
+    if (result)
         qFatal("QQNX: failed to query event window, errno=%d", errno);
-    }
+
     screen_window_t qnxWindow = static_cast<screen_window_t>(handle);
 
     // check if finger is valid
@@ -397,11 +432,20 @@ void QQnxScreenEventHandler::handleTouchEvent(screen_event_t event, int qnxType)
         if (w) {
             // get size of screen which contains window
             QPlatformScreen *platformScreen = QPlatformScreen::platformScreenForWindow(w);
-            QSizeF screenSize = platformScreen->physicalSize();
+            QSizeF screenSize = platformScreen->geometry().size();
 
             // update cached position of current touch point
-            m_touchPoints[touchId].normalPosition = QPointF( static_cast<qreal>(pos[0]) / screenSize.width(), static_cast<qreal>(pos[1]) / screenSize.height() );
-            m_touchPoints[touchId].area = QRectF( pos[0], pos[1], 0.0, 0.0 );
+            m_touchPoints[touchId].normalPosition =
+                            QPointF(static_cast<qreal>(pos[0]) / screenSize.width(),
+                                    static_cast<qreal>(pos[1]) / screenSize.height());
+
+            m_touchPoints[touchId].area = QRectF(w->geometry().left() + windowPos[0],
+                                                 w->geometry().top()  + windowPos[1], 0.0, 0.0);
+            QWindow *parent = w->parent();
+            while (parent) {
+                m_touchPoints[touchId].area.translate(parent->geometry().topLeft());
+                parent = parent->parent();
+            }
 
             // determine event type and update state of current touch point
             QEvent::Type type = QEvent::None;
@@ -436,8 +480,8 @@ void QQnxScreenEventHandler::handleTouchEvent(screen_event_t event, int qnxType)
             // inject event into Qt
             QWindowSystemInterface::handleTouchEvent(w, m_touchDevice, pointList);
             qScreenEventDebug() << Q_FUNC_INFO << "Qt touch, w =" << w
-                                << ", p=(" << pos[0] << "," << pos[1]
-                                << "), t=" << type;
+                                << ", p=" << m_touchPoints[touchId].area.topLeft()
+                                << ", t=" << type;
         }
     }
 }
@@ -452,9 +496,8 @@ void QQnxScreenEventHandler::handleCloseEvent(screen_event_t event)
 
     // Map window handle to top-level QWindow
     QWindow *w = QQnxIntegration::window(window);
-    if (w != 0) {
+    if (w != 0)
         QWindowSystemInterface::handleCloseEvent(w);
-    }
 }
 
 void QQnxScreenEventHandler::handleCreateEvent(screen_event_t event)
@@ -482,8 +525,14 @@ void QQnxScreenEventHandler::handleDisplayEvent(screen_event_t event)
 
     qScreenEventDebug() << Q_FUNC_INFO << "display attachment is now:" << isAttached;
     QQnxScreen *screen = m_qnxIntegration->screenForNative(nativeDisplay);
+
     if (!screen) {
         if (isAttached) {
+            int val[2];
+            screen_get_display_property_iv(nativeDisplay, SCREEN_PROPERTY_SIZE, val);
+            if (val[0] == 0 && val[1] == 0) //If screen size is invalid, wait for the next event
+                return;
+
             qScreenEventDebug() << "creating new QQnxScreen for newly attached display";
             m_qnxIntegration->createDisplay(nativeDisplay, false /* not primary, we assume */);
         }
@@ -503,6 +552,48 @@ void QQnxScreenEventHandler::handleDisplayEvent(screen_event_t event)
     }
 }
 
+void QQnxScreenEventHandler::handlePropertyEvent(screen_event_t event)
+{
+    errno = 0;
+    int objectType;
+    if (screen_get_event_property_iv(event, SCREEN_PROPERTY_OBJECT_TYPE, &objectType) != 0)
+        qFatal("QQNX: failed to query object type property, errno=%d", errno);
+
+    if (objectType != SCREEN_OBJECT_TYPE_WINDOW)
+        return;
+
+    errno = 0;
+    screen_window_t window = 0;
+    if (screen_get_event_property_pv(event, SCREEN_PROPERTY_WINDOW, (void**)&window) != 0)
+        qFatal("QQnx: failed to query window property, errno=%d", errno);
+
+    errno = 0;
+    int property;
+    if (screen_get_event_property_iv(event, SCREEN_PROPERTY_NAME, &property) != 0)
+        qFatal("QQnx: failed to query window property, errno=%d", errno);
+
+    switch (property) {
+    case SCREEN_PROPERTY_KEYBOARD_FOCUS:
+        handleKeyboardFocusPropertyEvent(window);
+        break;
+    default:
+        // event ignored
+        qScreenEventDebug() << Q_FUNC_INFO << "Ignore property event for property: " << property;
+    }
+}
+
+void QQnxScreenEventHandler::handleKeyboardFocusPropertyEvent(screen_window_t window)
+{
+    errno = 0;
+    int focus = 0;
+    if (window && screen_get_window_property_iv(window, SCREEN_PROPERTY_KEYBOARD_FOCUS, &focus) != 0)
+        qFatal("QQnx: failed to query keyboard focus property, errno=%d", errno);
+
+    QWindow *w = focus ? QQnxIntegration::window(window) : 0;
+
+    QWindowSystemInterface::handleWindowActivated(w);
+}
+
 #include "moc_qqnxscreeneventhandler.cpp"
 
 QT_END_NAMESPACE
