@@ -1,6 +1,6 @@
 /***************************************************************************
 **
-** Copyright (C) 2011 - 2012 Research In Motion
+** Copyright (C) 2011 - 2013 BlackBerry Limited. All rights reserved.
 ** Contact: http://www.qt-project.org/legal
 **
 ** This file is part of the plugins of the Qt Toolkit.
@@ -40,20 +40,29 @@
 ****************************************************************************/
 
 #include "qqnxwindow.h"
-#ifndef QT_NO_OPENGL
-#include "qqnxglcontext.h"
-#endif
 #include "qqnxintegration.h"
 #include "qqnxscreen.h"
 
+#include <QUuid>
+
 #include <QtGui/QWindow>
 #include <qpa/qwindowsysteminterface.h>
 
+#include "private/qguiapplication_p.h"
+
 #include <QtCore/QDebug>
 
+#if defined(Q_OS_BLACKBERRY)
+#if !defined(Q_OS_BLACKBERRY_TABLET)
+#include "qqnxnavigatorcover.h"
+#endif
+#include <sys/pps.h>
+#include <bps/navigator.h>
+#endif
+
 #include <errno.h>
 
-#ifdef QQNXWINDOW_DEBUG
+#if defined(QQNXWINDOW_DEBUG)
 #define qWindowDebug qDebug
 #else
 #define qWindowDebug QT_NO_QDEBUG_MACRO
@@ -61,87 +70,39 @@
 
 QT_BEGIN_NAMESPACE
 
-QQnxWindow::QQnxWindow(QWindow *window, screen_context_t context)
+QQnxWindow::QQnxWindow(QWindow *window, screen_context_t context, bool needRootWindow)
     : QPlatformWindow(window),
       m_screenContext(context),
-      m_window(0),
-      m_currentBufferIndex(-1),
-      m_previousBufferIndex(-1),
-#ifndef QT_NO_OPENGL
-      m_platformOpenGLContext(0),
-#endif
-      m_screen(0),
       m_parentWindow(0),
-      m_visible(true),
+      m_window(0),
+      m_screen(0),
+      m_visible(false),
+      m_exposed(true),
       m_windowState(Qt::WindowNoState),
-      m_requestedBufferSize(window->geometry().size())
+      m_mmRendererWindow(0)
 {
     qWindowDebug() << Q_FUNC_INFO << "window =" << window << ", size =" << window->size();
     int result;
 
-    // Create child QNX window
+    QQnxScreen *platformScreen = static_cast<QQnxScreen *>(window->screen()->handle());
+
+    m_isTopLevel = ( needRootWindow && !platformScreen->rootWindow())
+                || (!needRootWindow && !parent())
+                || window->type() == Qt::CoverWindow;
+
     errno = 0;
-    result = screen_create_window_type(&m_window, m_screenContext, SCREEN_CHILD_WINDOW);
-    if (result != 0) {
+    if (m_isTopLevel) {
+        result = screen_create_window(&m_window, m_screenContext); // Creates an application window
+        if (window->type() != Qt::CoverWindow) {
+            if (needRootWindow)
+                platformScreen->setRootWindow(this);
+            createWindowGroup();
+        }
+    } else {
+        result = screen_create_window_type(&m_window, m_screenContext, SCREEN_CHILD_WINDOW);
+    }
+    if (result != 0)
         qFatal("QQnxWindow: failed to create window, errno=%d", errno);
-    }
-
-    // Set window buffer usage based on rendering API
-    int val;
-    QSurface::SurfaceType surfaceType = window->surfaceType();
-    switch (surfaceType) {
-    case QSurface::RasterSurface:
-        val = SCREEN_USAGE_NATIVE | SCREEN_USAGE_READ | SCREEN_USAGE_WRITE;
-        break;
-    case QSurface::OpenGLSurface:
-        val = SCREEN_USAGE_OPENGL_ES2;
-        break;
-    default:
-        qFatal("QQnxWindow: unsupported window API");
-        break;
-    }
-
-    errno = 0;
-    result = screen_set_window_property_iv(m_window, SCREEN_PROPERTY_USAGE, &val);
-    if (result != 0) {
-        qFatal("QQnxWindow: failed to set window buffer usage, errno=%d", errno);
-    }
-
-    // Alpha channel is always pre-multiplied if present
-    errno = 0;
-    val = SCREEN_PRE_MULTIPLIED_ALPHA;
-    result = screen_set_window_property_iv(m_window, SCREEN_PROPERTY_ALPHA_MODE, &val);
-    if (result != 0) {
-        qFatal("QQnxWindow: failed to set window alpha mode, errno=%d", errno);
-    }
-
-    // Make the window opaque
-    errno = 0;
-    val = SCREEN_TRANSPARENCY_NONE;
-    result = screen_set_window_property_iv(m_window, SCREEN_PROPERTY_TRANSPARENCY, &val);
-    if (result != 0) {
-        qFatal("QQnxWindow: failed to set window transparency, errno=%d", errno);
-    }
-
-    // Set the window swap interval
-    errno = 0;
-    val = 1;
-    result = screen_set_window_property_iv(m_window, SCREEN_PROPERTY_SWAP_INTERVAL, &val);
-    if (result != 0) {
-        qFatal("QQnxWindow: failed to set window swap interval, errno=%d", errno);
-    }
-
-    setScreen(static_cast<QQnxScreen *>(window->screen()->handle()));
-
-    // Add window to plugin's window mapper
-    QQnxIntegration::addWindow(m_window, window);
-
-    // Qt never calls these setters after creating the window, so we need to do that ourselves here
-    setWindowState(window->windowState());
-    if (window->parent() && window->parent()->handle())
-        setParent(window->parent()->handle());
-    setGeometryHelper(window->geometry());
-    setVisible(window->isVisible());
 }
 
 QQnxWindow::~QQnxWindow()
@@ -156,7 +117,8 @@ QQnxWindow::~QQnxWindow()
 
     // Remove from parent's Hierarchy.
     removeFromParent();
-    m_screen->updateHierarchy();
+    if (m_screen)
+        m_screen->updateHierarchy();
 
     // Cleanup QNX window and its buffers
     screen_destroy_window(m_window);
@@ -164,32 +126,24 @@ QQnxWindow::~QQnxWindow()
 
 void QQnxWindow::setGeometry(const QRect &rect)
 {
-    const QRect oldGeometry = setGeometryHelper(rect);
+    QRect newGeometry = rect;
+    if (screen()->rootWindow() == this) //If this is the root window, it has to be shown fullscreen
+        newGeometry = screen()->geometry();
 
-    // If this is an OpenGL window we need to request that the GL context updates
-    // the EGLsurface on which it is rendering. The surface will be recreated the
-    // next time QQnxGLContext::makeCurrent() is called.
-    {
-        // We want the setting of the atomic bool in the GL context to be atomic with
-        // setting m_requestedBufferSize and therefore extended the scope to include
-        // that test.
-        const QMutexLocker locker(&m_mutex);
-        m_requestedBufferSize = rect.size();
-        if (m_platformOpenGLContext != 0 && bufferSize() != rect.size())
-            m_platformOpenGLContext->requestSurfaceChange();
-    }
+    const QRect oldGeometry = setGeometryHelper(newGeometry);
 
     // Send a geometry change event to Qt (triggers resizeEvent() in QWindow/QWidget).
 
     // Calling flushWindowSystemEvents() here would flush input events which
     // could result in re-entering QQnxWindow::setGeometry() again.
     QWindowSystemInterface::setSynchronousWindowsSystemEvents(true);
-    QWindowSystemInterface::handleGeometryChange(window(), rect);
+    QWindowSystemInterface::handleGeometryChange(window(), newGeometry);
+    QWindowSystemInterface::handleExposeEvent(window(), newGeometry);
     QWindowSystemInterface::setSynchronousWindowsSystemEvents(false);
 
     // Now move all children.
     if (!oldGeometry.isEmpty()) {
-        const QPoint offset = rect.topLeft() - oldGeometry.topLeft();
+        const QPoint offset = newGeometry.topLeft() - oldGeometry.topLeft();
         Q_FOREACH (QQnxWindow *childWindow, m_childWindows)
             childWindow->setOffset(offset);
     }
@@ -211,24 +165,21 @@ QRect QQnxWindow::setGeometryHelper(const QRect &rect)
     val[0] = rect.x();
     val[1] = rect.y();
     int result = screen_set_window_property_iv(m_window, SCREEN_PROPERTY_POSITION, val);
-    if (result != 0) {
+    if (result != 0)
         qFatal("QQnxWindow: failed to set window position, errno=%d", errno);
-    }
 
     errno = 0;
     val[0] = rect.width();
     val[1] = rect.height();
     result = screen_set_window_property_iv(m_window, SCREEN_PROPERTY_SIZE, val);
-    if (result != 0) {
+    if (result != 0)
         qFatal("QQnxWindow: failed to set window size, errno=%d", errno);
-    }
 
     // Set viewport size equal to window size
     errno = 0;
     result = screen_set_window_property_iv(m_window, SCREEN_PROPERTY_SOURCE_SIZE, val);
-    if (result != 0) {
+    if (result != 0)
         qFatal("QQnxWindow: failed to set window source size, errno=%d", errno);
-    }
 
     return oldGeometry;
 }
@@ -249,9 +200,8 @@ void QQnxWindow::setOffset(const QPoint &offset)
     val[0] = newGeometry.x();
     val[1] = newGeometry.y();
     int result = screen_set_window_property_iv(m_window, SCREEN_PROPERTY_POSITION, val);
-    if (result != 0) {
+    if (result != 0)
         qFatal("QQnxWindow: failed to set window position, errno=%d", errno);
-    }
 
     Q_FOREACH (QQnxWindow *childWindow, m_childWindows)
         childWindow->setOffset(offset);
@@ -261,6 +211,9 @@ void QQnxWindow::setVisible(bool visible)
 {
     qWindowDebug() << Q_FUNC_INFO << "window =" << window() << "visible =" << visible;
 
+    if (m_visible == visible)
+        return;
+
     m_visible = visible;
 
     QQnxWindow *root = this;
@@ -271,13 +224,13 @@ void QQnxWindow::setVisible(bool visible)
 
     window()->requestActivate();
 
-    if (window()->isTopLevel()) {
-        if (visible) {
-            QWindowSystemInterface::handleExposeEvent(window(), window()->geometry());
-        } else {
-            // Flush the context, otherwise it won't disappear immediately
-            screen_flush_context(m_screenContext, 0);
-        }
+    QWindowSystemInterface::handleExposeEvent(window(), window()->geometry());
+
+    if (visible) {
+        applyWindowState();
+    } else {
+        // Flush the context, otherwise it won't disappear immediately
+        screen_flush_context(m_screenContext, 0);
     }
 }
 
@@ -288,9 +241,8 @@ void QQnxWindow::updateVisibility(bool parentVisible)
     errno = 0;
     int val = (m_visible && parentVisible) ? 1 : 0;
     int result = screen_set_window_property_iv(m_window, SCREEN_PROPERTY_VISIBLE, &val);
-    if (result != 0) {
+    if (result != 0)
         qFatal("QQnxWindow: failed to set window visibility, errno=%d", errno);
-    }
 
     Q_FOREACH (QQnxWindow *childWindow, m_childWindows)
         childWindow->updateVisibility(m_visible && parentVisible);
@@ -303,30 +255,26 @@ void QQnxWindow::setOpacity(qreal level)
     errno = 0;
     int val = (int)(level * 255);
     int result = screen_set_window_property_iv(m_window, SCREEN_PROPERTY_GLOBAL_ALPHA, &val);
-    if (result != 0) {
+    if (result != 0)
         qFatal("QQnxWindow: failed to set window global alpha, errno=%d", errno);
-    }
 
     // TODO: How to handle children of this window? If we change all the visibilities, then
     //       the transparency will look wrong...
 }
 
+void QQnxWindow::setExposed(bool exposed)
+{
+    qWindowDebug() << Q_FUNC_INFO << "window =" << window() << "expose =" << exposed;
+
+    if (m_exposed != exposed) {
+        m_exposed = exposed;
+        QWindowSystemInterface::handleExposeEvent(window(), window()->geometry());
+    }
+}
+
 bool QQnxWindow::isExposed() const
 {
-    return m_visible;
-}
-
-QSize QQnxWindow::requestedBufferSize() const
-{
-    const QMutexLocker locker(&m_mutex);
-    return m_requestedBufferSize;
-}
-
-void QQnxWindow::adjustBufferSize()
-{
-    const QSize windowSize = window()->size();
-    if (windowSize != bufferSize())
-        setBufferSize(windowSize);
+    return m_visible && m_exposed;
 }
 
 void QQnxWindow::setBufferSize(const QSize &size)
@@ -341,27 +289,19 @@ void QQnxWindow::setBufferSize(const QSize &size)
 
     int val[2] = { nonEmptySize.width(), nonEmptySize.height() };
     int result = screen_set_window_property_iv(m_window, SCREEN_PROPERTY_BUFFER_SIZE, val);
-    if (result != 0) {
+    if (result != 0)
         qFatal("QQnxWindow: failed to set window buffer size, errno=%d", errno);
-    }
 
     // Create window buffers if they do not exist
     if (m_bufferSize.isEmpty()) {
-#ifndef QT_NO_OPENGL
-        // Get pixel format from EGL config if using OpenGL;
-        // otherwise inherit pixel format of window's screen
-        if (m_platformOpenGLContext != 0) {
-            val[0] = platformWindowFormatToNativeFormat(m_platformOpenGLContext->format());
-        } else {
-            val[0] = m_screen->nativeFormat();
-        }
-#endif
+        val[0] = pixelFormat();
+        if (val[0] == -1) // The platform GL context was not set yet on the window, so we can't procede
+            return;
 
         errno = 0;
         result = screen_set_window_property_iv(m_window, SCREEN_PROPERTY_FORMAT, val);
-        if (result != 0) {
+        if (result != 0)
             qFatal("QQnxWindow: failed to set window pixel format, errno=%d", errno);
-        }
 
         errno = 0;
         result = screen_create_window_buffers(m_window, MAX_BUFFER_COUNT);
@@ -374,9 +314,8 @@ void QQnxWindow::setBufferSize(const QSize &size)
         int bufferCount = 0;
         result = screen_get_window_property_iv(m_window, SCREEN_PROPERTY_RENDER_BUFFER_COUNT, &bufferCount);
 
-        if (result != 0) {
+        if (result != 0)
             qFatal("QQnxWindow: failed to query window buffer count, errno=%d", errno);
-        }
 
         if (bufferCount != MAX_BUFFER_COUNT) {
             qFatal("QQnxWindow: invalid buffer count. Expected = %d, got = %d. You might experience problems.",
@@ -384,147 +323,85 @@ void QQnxWindow::setBufferSize(const QSize &size)
         }
     }
 
+    // Set the transparency. According to QNX technical support, setting the window
+    // transparency property should always be done *after* creating the window
+    // buffers in order to guarantee the property is paid attention to.
+    if (window()->requestedFormat().alphaBufferSize() == 0) {
+        // To avoid overhead in the composition manager, disable blending
+        // when the underlying window buffer doesn't have an alpha channel.
+        val[0] = SCREEN_TRANSPARENCY_NONE;
+    } else {
+        // Normal alpha blending. This doesn't commit us to translucency; the
+        // normal backfill during the painting will contain a fully opaque
+        // alpha channel unless the user explicitly intervenes to make something
+        // transparent.
+        val[0] = SCREEN_TRANSPARENCY_SOURCE_OVER;
+    }
+
+    errno = 0;
+    result = screen_set_window_property_iv(m_window, SCREEN_PROPERTY_TRANSPARENCY, val);
+    if (result != 0)
+        qFatal("QQnxWindow: failed to set window transparency, errno=%d", errno);
+
     // Cache new buffer size
     m_bufferSize = nonEmptySize;
-
-    // Buffers were destroyed; reacquire them
-    m_currentBufferIndex = -1;
-    m_previousDirty = QRegion();
-    m_scrolled = QRegion();
-
-    const QMutexLocker locker(&m_mutex);
-    m_requestedBufferSize = QSize();
-}
-
-QQnxBuffer &QQnxWindow::renderBuffer()
-{
-    qWindowDebug() << Q_FUNC_INFO << "window =" << window();
-
-    // Check if render buffer is invalid
-    if (m_currentBufferIndex == -1) {
-        // Get all buffers available for rendering
-        errno = 0;
-        screen_buffer_t buffers[MAX_BUFFER_COUNT];
-        const int result = screen_get_window_property_pv(m_window, SCREEN_PROPERTY_RENDER_BUFFERS, (void **)buffers);
-        if (result != 0) {
-            qFatal("QQnxWindow: failed to query window buffers, errno=%d", errno);
-        }
-
-        // Wrap each buffer
-        for (int i = 0; i < MAX_BUFFER_COUNT; ++i) {
-            m_buffers[i] = QQnxBuffer(buffers[i]);
-        }
-
-        // Use the first available render buffer
-        m_currentBufferIndex = 0;
-        m_previousBufferIndex = -1;
-    }
-
-    return m_buffers[m_currentBufferIndex];
-}
-
-void QQnxWindow::scroll(const QRegion &region, int dx, int dy, bool flush)
-{
-    qWindowDebug() << Q_FUNC_INFO << "window =" << window();
-    blitPreviousToCurrent(region, dx, dy, flush);
-    m_scrolled += region;
-}
-
-void QQnxWindow::post(const QRegion &dirty)
-{
-    // How double-buffering works
-    // --------------------------
-    //
-    // The are two buffers, the previous one and the current one.
-    // The previous buffer always contains the complete, full image of the whole window when it
-    // was last posted.
-    // The current buffer starts with the complete, full image of the second to last posting
-    // of the window.
-    //
-    // During painting, Qt paints on the current buffer. Thus, when Qt has finished painting, the
-    // current buffer contains the second to last image plus the newly painted regions.
-    // Since the second to last image is too old, we copy over the image from the previous buffer, but
-    // only for those regions that Qt didn't paint (because that would overwrite what Qt has just
-    // painted). This is the copyPreviousToCurrent() call below.
-    //
-    // After the call to copyPreviousToCurrent(), the current buffer contains the complete, full image of the
-    // whole window in its current state, and we call screen_post_window() to make the new buffer
-    // available to libscreen (called "posting"). There, only the regions that Qt painted on are
-    // posted, as nothing else has changed.
-    //
-    // After that, the previous and the current buffers are swapped, and the whole cycle starts anew.
-
-    // Check if render buffer exists and something was rendered
-    if (m_currentBufferIndex != -1 && !dirty.isEmpty()) {
-        qWindowDebug() << Q_FUNC_INFO << "window =" << window();
-        QQnxBuffer &currentBuffer = m_buffers[m_currentBufferIndex];
-
-        // Copy unmodified region from old render buffer to new render buffer;
-        // required to allow partial updates
-        QRegion preserve = m_previousDirty - dirty - m_scrolled;
-        blitPreviousToCurrent(preserve, 0, 0);
-
-        // Calculate region that changed
-        QRegion modified = preserve + dirty + m_scrolled;
-        QRect rect = modified.boundingRect();
-        int dirtyRect[4] = { rect.x(), rect.y(), rect.x() + rect.width(), rect.y() + rect.height() };
-
-        // Update the display with contents of render buffer
-        errno = 0;
-        int result = screen_post_window(m_window, currentBuffer.nativeBuffer(), 1, dirtyRect, 0);
-        if (result != 0) {
-            qFatal("QQnxWindow: failed to post window buffer, errno=%d", errno);
-        }
-
-        // Advance to next nender buffer
-        m_previousBufferIndex = m_currentBufferIndex++;
-        if (m_currentBufferIndex >= MAX_BUFFER_COUNT) {
-            m_currentBufferIndex = 0;
-        }
-
-        // Save modified region and clear scrolled region
-        m_previousDirty = dirty;
-        m_scrolled = QRegion();
-
-        // Notify screen that window posted
-        if (m_screen != 0) {
-            m_screen->onWindowPost(this);
-        }
-    }
+    resetBuffers();
 }
 
 void QQnxWindow::setScreen(QQnxScreen *platformScreen)
 {
     qWindowDebug() << Q_FUNC_INFO << "window =" << window() << "platformScreen =" << platformScreen;
 
+    if (platformScreen == 0) { // The screen has been destroyed
+        m_screen = 0;
+        Q_FOREACH (QQnxWindow *childWindow, m_childWindows) {
+            childWindow->setScreen(0);
+        }
+        return;
+    }
+
     if (m_screen == platformScreen)
         return;
 
-    if (m_screen)
+    if (m_screen) {
+        qWindowDebug() << Q_FUNC_INFO << "Moving window to different screen";
         m_screen->removeWindow(this);
-    platformScreen->addWindow(this);
+        QQnxIntegration *platformIntegration = static_cast<QQnxIntegration*>(QGuiApplicationPrivate::platformIntegration());
+
+        if ((platformIntegration->options() & QQnxIntegration::RootWindow)) {
+            screen_leave_window_group(m_window);
+        }
+    }
+
     m_screen = platformScreen;
-
-    // Move window to proper screen/display
-    errno = 0;
-    screen_display_t display = platformScreen->nativeDisplay();
-    int result = screen_set_window_property_pv(m_window, SCREEN_PROPERTY_DISPLAY, (void **)&display);
-    if (result != 0) {
-        qFatal("QQnxWindow: failed to set window display, errno=%d", errno);
+    if (!m_parentWindow) {
+        platformScreen->addWindow(this);
     }
+    if (m_isTopLevel) {
+          // Move window to proper screen/display
+        errno = 0;
+        screen_display_t display = platformScreen->nativeDisplay();
+        int result = screen_set_window_property_pv(m_window, SCREEN_PROPERTY_DISPLAY, (void **)&display);
+        if (result != 0)
+            qFatal("QQnxWindow: failed to set window display, errno=%d", errno);
+    } else {
+        errno = 0;
+        int result;
+        if (!parent()) {
+            result = screen_join_window_group(m_window, platformScreen->windowGroupName());
+            if (result != 0)
+                qFatal("QQnxWindow: failed to join window group, errno=%d", errno);
+        } else {
+            result = screen_join_window_group(m_window, static_cast<QQnxWindow*>(parent())->groupName().constData());
+            if (result != 0)
+                qFatal("QQnxWindow: failed to join window group, errno=%d", errno);
+        }
 
-    // Add window to display's window group
-    errno = 0;
-    result = screen_join_window_group(m_window, platformScreen->windowGroupName());
-    if (result != 0) {
-        qFatal("QQnxWindow: failed to join window group, errno=%d", errno);
-    }
-
-    Q_FOREACH (QQnxWindow *childWindow, m_childWindows) {
-        // Only subwindows and tooltips need necessarily be moved to another display with the window.
-        if ((window()->type() & Qt::WindowType_Mask) == Qt::SubWindow ||
-            (window()->type() & Qt::WindowType_Mask) == Qt::ToolTip)
-            childWindow->setScreen(platformScreen);
+        Q_FOREACH (QQnxWindow *childWindow, m_childWindows) {
+            // Only subwindows and tooltips need necessarily be moved to another display with the window.
+            if (window()->type() == Qt::SubWindow || window()->type() == Qt::ToolTip)
+                childWindow->setScreen(platformScreen);
+        }
     }
 
     m_screen->updateHierarchy();
@@ -539,7 +416,7 @@ void QQnxWindow::removeFromParent()
             m_parentWindow = 0;
         else
             qFatal("QQnxWindow: Window Hierarchy broken; window has parent, but parent hasn't got child.");
-    } else {
+    } else if (m_screen) {
         m_screen->removeWindow(this);
     }
 }
@@ -566,6 +443,8 @@ void QQnxWindow::setParent(const QPlatformWindow *window)
         m_screen->addWindow(this);
     }
 
+    adjustBufferSize();
+
     m_screen->updateHierarchy();
 }
 
@@ -616,27 +495,16 @@ void QQnxWindow::setWindowState(Qt::WindowState state)
     if (m_windowState == state)
         return;
 
-    switch (state) {
-
-    // WindowMinimized is not supported - navigator does not have an API to minimize a window
-    // WindowActive is not an accepted parameter according to the docs
-    case Qt::WindowMinimized:
-    case Qt::WindowActive:
-        return;
-
-    case Qt::WindowMaximized:
-    case Qt::WindowFullScreen:
-        m_unmaximizedGeometry = geometry();
-        setGeometry(state == Qt::WindowMaximized ? m_screen->availableGeometry() : m_screen->geometry());
-        break;
-
-    case Qt::WindowNoState:
-        if (m_unmaximizedGeometry.isValid())
-            setGeometry(m_unmaximizedGeometry);
-        break;
-    }
-
     m_windowState = state;
+
+    if (m_visible)
+        applyWindowState();
+}
+
+void QQnxWindow::propagateSizeHints()
+{
+    // nothing to do; silence base class warning
+    qWindowDebug() << Q_FUNC_INFO << ": ignored";
 }
 
 void QQnxWindow::gainedFocus()
@@ -647,14 +515,21 @@ void QQnxWindow::gainedFocus()
     QWindowSystemInterface::handleWindowActivated(window());
 }
 
-#ifndef QT_NO_OPENGL
-void QQnxWindow::setPlatformOpenGLContext(QQnxGLContext *platformOpenGLContext)
+void QQnxWindow::setMMRendererWindowName(const QString &name)
 {
-    // This function does not take ownership of the platform gl context.
-    // It is owned by the frontend QOpenGLContext
-    m_platformOpenGLContext = platformOpenGLContext;
+    m_mmRendererWindowName = name;
+}
+
+void QQnxWindow::setMMRendererWindow(screen_window_t handle)
+{
+    m_mmRendererWindow = handle;
+}
+
+void QQnxWindow::clearMMRendererWindow()
+{
+    m_mmRendererWindowName.clear();
+    m_mmRendererWindow = 0;
 }
-#endif
 
 QQnxWindow *QQnxWindow::findWindow(screen_window_t windowHandle)
 {
@@ -670,118 +545,150 @@ QQnxWindow *QQnxWindow::findWindow(screen_window_t windowHandle)
     return 0;
 }
 
-void QQnxWindow::blitFrom(QQnxWindow *sourceWindow, const QPoint &sourceOffset, const QRegion &targetRegion)
+void QQnxWindow::minimize()
 {
-    if (!sourceWindow || sourceWindow->m_previousBufferIndex == -1 || targetRegion.isEmpty())
-        return;
+#if defined(Q_OS_BLACKBERRY) && !defined(Q_OS_BLACKBERRY_TABLET)
+    qWindowDebug() << Q_FUNC_INFO;
 
-    qWindowDebug() << Q_FUNC_INFO << window() << sourceWindow->window() << sourceOffset << targetRegion;
+    pps_encoder_t encoder;
 
-    QQnxBuffer &sourceBuffer = sourceWindow->m_buffers[sourceWindow->m_previousBufferIndex];
-    QQnxBuffer &targetBuffer = renderBuffer();
+    pps_encoder_initialize(&encoder, false);
+    pps_encoder_add_string(&encoder, "msg", "minimizeWindow");
 
-    blitHelper(sourceBuffer, targetBuffer, sourceOffset, QPoint(0, 0), targetRegion, true);
+    if (navigator_raw_write(pps_encoder_buffer(&encoder),
+                pps_encoder_length(&encoder)) != BPS_SUCCESS) {
+        qWindowDebug() << Q_FUNC_INFO << "navigator_raw_write failed:" << strerror(errno);
+    }
+
+    pps_encoder_cleanup(&encoder);
+#else
+    qWarning("Qt::WindowMinimized is not supported by this OS version");
+#endif
+}
+
+void QQnxWindow::setRotation(int rotation)
+{
+    qWindowDebug() << Q_FUNC_INFO << "angle =" << rotation;
+    errno = 0;
+    int result = screen_set_window_property_iv(m_window, SCREEN_PROPERTY_ROTATION, &rotation);
+    if (result != 0)
+        qFatal("QQnxRootWindow: failed to set window rotation, errno=%d", errno);
+}
+
+void QQnxWindow::initWindow()
+{
+    // Alpha channel is always pre-multiplied if present
+    errno = 0;
+    int val = SCREEN_PRE_MULTIPLIED_ALPHA;
+    int result = screen_set_window_property_iv(m_window, SCREEN_PROPERTY_ALPHA_MODE, &val);
+    if (result != 0)
+        qFatal("QQnxWindow: failed to set window alpha mode, errno=%d", errno);
+
+    // Set the window swap interval
+    errno = 0;
+    val = 1;
+    result = screen_set_window_property_iv(m_window, SCREEN_PROPERTY_SWAP_INTERVAL, &val);
+    if (result != 0)
+        qFatal("QQnxWindow: failed to set window swap interval, errno=%d", errno);
+
+    if (window()->flags() & Qt::WindowDoesNotAcceptFocus) {
+        errno = 0;
+        val = SCREEN_SENSITIVITY_NO_FOCUS;
+        result = screen_set_window_property_iv(m_window, SCREEN_PROPERTY_SENSITIVITY, &val);
+        if (result != 0)
+            qFatal("QQnxWindow: failed to set window sensitivity, errno=%d", errno);
+    }
+
+    setScreen(static_cast<QQnxScreen *>(window()->screen()->handle()));
+
+    if (window()->type() == Qt::CoverWindow) {
+#if defined(Q_OS_BLACKBERRY) && !defined(Q_OS_BLACKBERRY_TABLET)
+        screen_set_window_property_pv(m_screen->rootWindow()->nativeHandle(),
+                                      SCREEN_PROPERTY_ALTERNATE_WINDOW, (void**)&m_window);
+        m_cover.reset(new QQnxNavigatorCover);
+#endif
+        m_exposed = false;
+    }
+
+    // Add window to plugin's window mapper
+    QQnxIntegration::addWindow(m_window, window());
+
+    // Qt never calls these setters after creating the window, so we need to do that ourselves here
+    setWindowState(window()->windowState());
+    if (window()->parent() && window()->parent()->handle())
+        setParent(window()->parent()->handle());
+
+    if (screen()->rootWindow() == this) {
+        setGeometryHelper(screen()->geometry());
+        QWindowSystemInterface::handleGeometryChange(window(), screen()->geometry());
+    } else {
+        setGeometryHelper(window()->geometry());
+    }
+}
+
+void QQnxWindow::createWindowGroup()
+{
+    // Generate a random window group name
+    m_windowGroupName = QUuid::createUuid().toString().toLatin1();
+
+    // Create window group so child windows can be parented by container window
+    errno = 0;
+    int result = screen_create_window_group(m_window, m_windowGroupName.constData());
+    if (result != 0)
+        qFatal("QQnxRootWindow: failed to create app window group, errno=%d", errno);
 }
 
 void QQnxWindow::updateZorder(int &topZorder)
 {
-    errno = 0;
-    int result = screen_set_window_property_iv(m_window, SCREEN_PROPERTY_ZORDER, &topZorder);
-    topZorder++;
+    updateZorder(m_window, topZorder);
 
-    if (result != 0)
-        qFatal("QQnxWindow: failed to set window z-order=%d, errno=%d, mWindow=%p", topZorder, errno, m_window);
+    if (m_mmRendererWindow)
+        updateZorder(m_mmRendererWindow, topZorder);
 
     Q_FOREACH (QQnxWindow *childWindow, m_childWindows)
         childWindow->updateZorder(topZorder);
 }
 
-void QQnxWindow::blitHelper(QQnxBuffer &source, QQnxBuffer &target, const QPoint &sourceOffset,
-                            const QPoint &targetOffset, const QRegion &region, bool flush)
+void QQnxWindow::updateZorder(screen_window_t window, int &topZorder)
 {
-    // Break down region into non-overlapping rectangles
-    const QVector<QRect> rects = region.rects();
-    for (int i = rects.size() - 1; i >= 0; i--) {
-        // Clip rectangle to bounds of target
-        const QRect rect = rects[i].intersected(target.rect());
+    errno = 0;
+    int result = screen_set_window_property_iv(window, SCREEN_PROPERTY_ZORDER, &topZorder);
+    topZorder++;
 
-        if (rect.isEmpty())
-            continue;
-
-        // Setup blit operation
-        int attribs[] = { SCREEN_BLIT_SOURCE_X, rect.x() + sourceOffset.x(),
-                          SCREEN_BLIT_SOURCE_Y, rect.y() + sourceOffset.y(),
-                          SCREEN_BLIT_SOURCE_WIDTH, rect.width(),
-                          SCREEN_BLIT_SOURCE_HEIGHT, rect.height(),
-                          SCREEN_BLIT_DESTINATION_X, rect.x() + targetOffset.x(),
-                          SCREEN_BLIT_DESTINATION_Y, rect.y() + targetOffset.y(),
-                          SCREEN_BLIT_DESTINATION_WIDTH, rect.width(),
-                          SCREEN_BLIT_DESTINATION_HEIGHT, rect.height(),
-                          SCREEN_BLIT_END };
-
-        // Queue blit operation
-        errno = 0;
-        const int result = screen_blit(m_screenContext, target.nativeBuffer(),
-                                       source.nativeBuffer(), attribs);
-        if (result != 0)
-            qFatal("QQnxWindow: failed to blit buffers, errno=%d", errno);
-    }
-
-    // Check if flush requested
-    if (flush) {
-        // Wait for all blits to complete
-        errno = 0;
-        const int result = screen_flush_blits(m_screenContext, SCREEN_WAIT_IDLE);
-        if (result != 0)
-            qFatal("QQnxWindow: failed to flush blits, errno=%d", errno);
-
-        // Buffer was modified outside the CPU
-        target.invalidateInCache();
-    }
+    if (result != 0)
+        qFatal("QQnxWindow: failed to set window z-order=%d, errno=%d, mWindow=%p", topZorder, errno, window);
 }
 
-void QQnxWindow::blitPreviousToCurrent(const QRegion &region, int dx, int dy, bool flush)
+void QQnxWindow::applyWindowState()
 {
-    qWindowDebug() << Q_FUNC_INFO << "window =" << window();
+    switch (m_windowState) {
 
-    // Abort if previous buffer is invalid or if nothing to copy
-    if (m_previousBufferIndex == -1 || region.isEmpty())
+    // WindowActive is not an accepted parameter according to the docs
+    case Qt::WindowActive:
         return;
 
-    QQnxBuffer &currentBuffer = m_buffers[m_currentBufferIndex];
-    QQnxBuffer &previousBuffer = m_buffers[m_previousBufferIndex];
+    case Qt::WindowMinimized:
+        minimize();
 
-    blitHelper(previousBuffer, currentBuffer, QPoint(0, 0), QPoint(dx, dy), region, flush);
-}
+        if (m_unmaximizedGeometry.isValid())
+            setGeometry(m_unmaximizedGeometry);
+        else
+            setGeometry(m_screen->geometry());
 
-int QQnxWindow::platformWindowFormatToNativeFormat(const QSurfaceFormat &format)
-{
-    qWindowDebug() << Q_FUNC_INFO;
-    // Extract size of colour channels from window format
-    int redSize = format.redBufferSize();
-    if (redSize == -1) {
-        qFatal("QQnxWindow: red size not defined");
-    }
+        break;
 
-    int greenSize = format.greenBufferSize();
-    if (greenSize == -1) {
-        qFatal("QQnxWindow: green size not defined");
-    }
+    case Qt::WindowMaximized:
+    case Qt::WindowFullScreen:
+        m_unmaximizedGeometry = geometry();
+        setGeometry(m_windowState == Qt::WindowMaximized ? m_screen->availableGeometry() : m_screen->geometry());
+        break;
 
-    int blueSize = format.blueBufferSize();
-    if (blueSize == -1) {
-        qFatal("QQnxWindow: blue size not defined");
-    }
-
-    // select matching native format
-    if (redSize == 5 && greenSize == 6 && blueSize == 5) {
-        return SCREEN_FORMAT_RGB565;
-    } else if (redSize == 8 && greenSize == 8 && blueSize == 8) {
-        return SCREEN_FORMAT_RGBA8888;
-    } else {
-        qFatal("QQnxWindow: unsupported pixel format");
-        return 0;
+    case Qt::WindowNoState:
+        if (m_unmaximizedGeometry.isValid())
+            setGeometry(m_unmaximizedGeometry);
+        break;
     }
 }
 
+
 QT_END_NAMESPACE
