@@ -44,7 +44,7 @@
 #include "iaccessible2.h"
 #include "qwindowsaccessibility.h"
 
-#include <QtGui/qaccessible2.h>
+#include <QtGui/qaccessible.h>
 #include <QtGui/qclipboard.h>
 #include <QtWidgets/qapplication.h>
 #include <QtCore/qdebug.h>
@@ -230,6 +230,10 @@ HRESULT STDMETHODCALLTYPE AccessibleRelation::get_targets(
  **************************************************************/
 HRESULT STDMETHODCALLTYPE QWindowsIA2Accessible::QueryInterface(REFIID id, LPVOID *iface)
 {
+    QAccessibleInterface *accessible = accessibleInterface();
+    if (!accessible)
+        return E_NOINTERFACE;
+
     HRESULT hr = QWindowsMsaaAccessible::QueryInterface(id, iface);
     if (!SUCCEEDED(hr)) {
         if (id == IID_IServiceProvider) {
@@ -301,10 +305,11 @@ ULONG STDMETHODCALLTYPE QWindowsIA2Accessible::Release()
  **************************************************************/
 HRESULT STDMETHODCALLTYPE QWindowsIA2Accessible::get_nRelations(long *nRelations)
 {
+    QAccessibleInterface *accessible = accessibleInterface();
     accessibleDebugClientCalls(accessible);
     if (!nRelations)
       return E_INVALIDARG;
-    if (!accessible->isValid())
+    if (!accessible)
         return E_FAIL;
 
     return getRelationsHelper(0, 0, 0, nRelations);
@@ -312,10 +317,11 @@ HRESULT STDMETHODCALLTYPE QWindowsIA2Accessible::get_nRelations(long *nRelations
 
 HRESULT STDMETHODCALLTYPE QWindowsIA2Accessible::get_relation(long relationIndex, IAccessibleRelation **relation)
 {
+    QAccessibleInterface *accessible = accessibleInterface();
     accessibleDebugClientCalls(accessible);
     if (!relation)
       return E_INVALIDARG;
-    if (!accessible->isValid())
+    if (!accessible)
         return E_FAIL;
 
     return getRelationsHelper(relation, relationIndex,  1);
@@ -330,8 +336,9 @@ HRESULT STDMETHODCALLTYPE QWindowsIA2Accessible::get_relations(long maxRelations
                                         IAccessibleRelation **relations,
                                         long *nRelations)
 {
+    QAccessibleInterface *accessible = accessibleInterface();
     accessibleDebugClientCalls(accessible);
-    if (!accessible->isValid())
+    if (!accessible)
         return E_FAIL;
 
     return getRelationsHelper(relations, 0, maxRelations, nRelations);
@@ -340,8 +347,9 @@ HRESULT STDMETHODCALLTYPE QWindowsIA2Accessible::get_relations(long maxRelations
 
 HRESULT STDMETHODCALLTYPE QWindowsIA2Accessible::role(long *ia2role)
 {
+    QAccessibleInterface *accessible = accessibleInterface();
     accessibleDebugClientCalls(accessible);
-    if (!accessible->isValid())
+    if (!accessible)
         return E_FAIL;
 
     long r = accessible->role();
@@ -386,8 +394,9 @@ HRESULT STDMETHODCALLTYPE QWindowsIA2Accessible::get_groupPosition(long *groupLe
 
 HRESULT STDMETHODCALLTYPE QWindowsIA2Accessible::get_states(AccessibleStates *states)
 {
+    QAccessibleInterface *accessible = accessibleInterface();
     accessibleDebugClientCalls(accessible);
-    if (!accessible->isValid())
+    if (!accessible)
         return E_FAIL;
     if (!states)
         return E_POINTER;
@@ -462,23 +471,23 @@ HRESULT STDMETHODCALLTYPE QWindowsIA2Accessible::get_localizedExtendedStates(lon
 
 HRESULT STDMETHODCALLTYPE QWindowsIA2Accessible::get_uniqueID(long *outUniqueID)
 {
+    QAccessibleInterface *accessible = accessibleInterface();
     accessibleDebugClientCalls(accessible);
-    if (!accessible->isValid())
+    if (!accessible)
         return E_FAIL;
-    // ### FIXME SERIOUSLY, NOT A STABLE SOLUTION IF NODES ARE DELETED ETC
-    // Return 0 if no object and no parent. This is really an error case.
-    uint uid = uniqueID();
-    accessibleDebug("uniqueID: %08x", uid);
 
-    *outUniqueID = (long)uid;
-    return uid ? S_OK : S_FALSE;
+    accessibleDebug("uniqueID: %08x", id);
+
+    *outUniqueID = (long)id;
+    return int(id) < 0 ? S_OK : S_FALSE;
 }
 
 
 HRESULT STDMETHODCALLTYPE QWindowsIA2Accessible::get_windowHandle(HWND *windowHandle)
 {
+    QAccessibleInterface *accessible = accessibleInterface();
     accessibleDebugClientCalls(accessible);
-    if (!accessible->isValid())
+    if (!accessible)
         return E_FAIL;
     return GetWindow(windowHandle);
 }
@@ -486,8 +495,9 @@ HRESULT STDMETHODCALLTYPE QWindowsIA2Accessible::get_windowHandle(HWND *windowHa
 
 HRESULT STDMETHODCALLTYPE QWindowsIA2Accessible::get_indexInParent(long *indexInParent)
 {
+    QAccessibleInterface *accessible = accessibleInterface();
     accessibleDebugClientCalls(accessible);
-    if (!accessible->isValid())
+    if (!accessible)
         return E_FAIL;
     if (!indexInParent)
       return E_INVALIDARG;
@@ -497,7 +507,6 @@ HRESULT STDMETHODCALLTYPE QWindowsIA2Accessible::get_indexInParent(long *indexIn
         return S_FALSE;
     }
     int indexOfChild = par->indexOfChild(accessible);
-    delete par;
     Q_ASSERT(indexOfChild >= 0);
     *indexInParent = indexOfChild;
     return S_OK;
@@ -505,8 +514,9 @@ HRESULT STDMETHODCALLTYPE QWindowsIA2Accessible::get_indexInParent(long *indexIn
 
 HRESULT STDMETHODCALLTYPE QWindowsIA2Accessible::get_locale(IA2Locale *locale)
 {
+    QAccessibleInterface *accessible = accessibleInterface();
     accessibleDebugClientCalls(accessible);
-    if (!accessible->isValid())
+    if (!accessible)
         return E_FAIL;
     IA2Locale res;
     QLocale l;
@@ -519,8 +529,9 @@ HRESULT STDMETHODCALLTYPE QWindowsIA2Accessible::get_locale(IA2Locale *locale)
 
 HRESULT STDMETHODCALLTYPE QWindowsIA2Accessible::get_attributes(BSTR *attributes)
 {
+    QAccessibleInterface *accessible = accessibleInterface();
     accessibleDebugClientCalls(accessible);
-    if (!accessible->isValid())
+    if (!accessible)
         return E_FAIL;
     *attributes = 0;//QStringToBSTR(QString());
     return S_FALSE;
@@ -531,8 +542,9 @@ HRESULT STDMETHODCALLTYPE QWindowsIA2Accessible::get_attributes(BSTR *attributes
  **************************************************************/
 HRESULT STDMETHODCALLTYPE QWindowsIA2Accessible::nActions(long *nActions)
 {
+    QAccessibleInterface *accessible = accessibleInterface();
     accessibleDebugClientCalls(accessible);
-    if (!accessible->isValid())
+    if (!accessible)
         return E_FAIL;
     *nActions = 0;
 
@@ -543,8 +555,9 @@ HRESULT STDMETHODCALLTYPE QWindowsIA2Accessible::nActions(long *nActions)
 
 HRESULT STDMETHODCALLTYPE QWindowsIA2Accessible::doAction(long actionIndex)
 {
+    QAccessibleInterface *accessible = accessibleInterface();
     accessibleDebugClientCalls(accessible);
-    if (!accessible->isValid())
+    if (!accessible)
         return E_FAIL;
     if (QAccessibleActionInterface *actionIface = actionInterface()) {
         const QStringList actionNames = actionIface->actionNames();
@@ -559,8 +572,9 @@ HRESULT STDMETHODCALLTYPE QWindowsIA2Accessible::doAction(long actionIndex)
 
 HRESULT STDMETHODCALLTYPE QWindowsIA2Accessible::get_description(long actionIndex, BSTR *description)
 {
+    QAccessibleInterface *accessible = accessibleInterface();
     accessibleDebugClientCalls(accessible);
-    if (!accessible->isValid())
+    if (!accessible)
         return E_FAIL;
     *description = 0;
     if (QAccessibleActionInterface *actionIface = actionInterface()) {
@@ -575,8 +589,9 @@ HRESULT STDMETHODCALLTYPE QWindowsIA2Accessible::get_description(long actionInde
 
 HRESULT STDMETHODCALLTYPE QWindowsIA2Accessible::get_keyBinding(long actionIndex, long nMaxBindings, BSTR **keyBindings, long *nBindings)
 {
+    QAccessibleInterface *accessible = accessibleInterface();
     accessibleDebugClientCalls(accessible);
-    if (!accessible->isValid())
+    if (!accessible)
         return E_FAIL;
     Q_UNUSED(nMaxBindings);
     BSTR *arrayOfBindingsToReturn = 0;
@@ -603,8 +618,9 @@ HRESULT STDMETHODCALLTYPE QWindowsIA2Accessible::get_keyBinding(long actionIndex
 
 HRESULT STDMETHODCALLTYPE QWindowsIA2Accessible::get_name(long actionIndex, BSTR *name)
 {
+    QAccessibleInterface *accessible = accessibleInterface();
     accessibleDebugClientCalls(accessible);
-    if (!accessible->isValid())
+    if (!accessible)
         return E_FAIL;
     *name = 0;
     if (QAccessibleActionInterface *actionIface = actionInterface()) {
@@ -619,8 +635,9 @@ HRESULT STDMETHODCALLTYPE QWindowsIA2Accessible::get_name(long actionIndex, BSTR
 
 HRESULT STDMETHODCALLTYPE QWindowsIA2Accessible::get_localizedName(long actionIndex, BSTR *localizedName)
 {
+    QAccessibleInterface *accessible = accessibleInterface();
     accessibleDebugClientCalls(accessible);
-    if (!accessible->isValid())
+    if (!accessible)
         return E_FAIL;
     *localizedName = 0;
     if (QAccessibleActionInterface *actionIface = actionInterface()) {
@@ -639,13 +656,15 @@ HRESULT STDMETHODCALLTYPE QWindowsIA2Accessible::get_localizedName(long actionIn
  **************************************************************/
 HRESULT STDMETHODCALLTYPE QWindowsIA2Accessible::get_locationInParent(long *x, long *y)
 {
+    QAccessibleInterface *accessible = accessibleInterface();
     accessibleDebugClientCalls(accessible);
-    if (!accessible->isValid())
+    if (!accessible)
         return E_FAIL;
 
     QPoint topLeft = accessible->rect().topLeft();
 
-    if (QAccessibleInterface *parentIface = accessible->parent())
+    QAccessibleInterface *parentIface = accessible->parent();
+    if (parentIface && parentIface->isValid())
         topLeft -= parentIface->rect().topLeft();
 
     *x = topLeft.x();
@@ -655,8 +674,9 @@ HRESULT STDMETHODCALLTYPE QWindowsIA2Accessible::get_locationInParent(long *x, l
 
 HRESULT STDMETHODCALLTYPE QWindowsIA2Accessible::get_foreground(IA2Color *foreground)
 {
+    QAccessibleInterface *accessible = accessibleInterface();
     accessibleDebugClientCalls(accessible);
-    if (!accessible->isValid())
+    if (!accessible)
         return E_FAIL;
 
     // IA2Color is a typedef for long
@@ -666,8 +686,9 @@ HRESULT STDMETHODCALLTYPE QWindowsIA2Accessible::get_foreground(IA2Color *foregr
 
 HRESULT STDMETHODCALLTYPE QWindowsIA2Accessible::get_background(IA2Color *background)
 {
+    QAccessibleInterface *accessible = accessibleInterface();
     accessibleDebugClientCalls(accessible);
-    if (!accessible->isValid())
+    if (!accessible)
         return E_FAIL;
 
     // IA2Color is a typedef for long
@@ -686,6 +707,8 @@ HRESULT STDMETHODCALLTYPE QWindowsIA2Accessible::get_background(IA2Color *backgr
 */
 QString QWindowsIA2Accessible::textForRange(int startOffset, int endOffset) const
 {
+    QAccessibleInterface *accessible = accessibleInterface();
+
     if (QAccessibleTextInterface *textIface = accessible->textInterface()) {
         if (endOffset == IA2_TEXT_OFFSET_LENGTH)
             endOffset = textIface->characterCount();
@@ -703,6 +726,7 @@ QString QWindowsIA2Accessible::textForRange(int startOffset, int endOffset) cons
 */
 void QWindowsIA2Accessible::replaceTextFallback(long startOffset, long endOffset, const QString &txt)
 {
+    QAccessibleInterface *accessible = accessibleInterface();
     QString t = textForRange(0, -1);
     if (endOffset == IA2_TEXT_OFFSET_LENGTH)
         endOffset = t.length();
@@ -716,6 +740,7 @@ void QWindowsIA2Accessible::replaceTextFallback(long startOffset, long endOffset
 
 HRESULT STDMETHODCALLTYPE QWindowsIA2Accessible::copyText(long startOffset, long endOffset)
 {
+    QAccessibleInterface *accessible = accessibleInterface();
     accessibleDebugClientCalls(accessible);
 #ifndef QT_NO_CLIPBOARD
     const QString t = textForRange(startOffset, endOffset);
@@ -728,6 +753,7 @@ HRESULT STDMETHODCALLTYPE QWindowsIA2Accessible::copyText(long startOffset, long
 
 HRESULT STDMETHODCALLTYPE QWindowsIA2Accessible::deleteText(long startOffset, long endOffset)
 {
+    QAccessibleInterface *accessible = accessibleInterface();
     accessibleDebugClientCalls(accessible);
     if (QAccessibleEditableTextInterface *editableTextIface = accessible->editableTextInterface())
         editableTextIface->deleteText(startOffset, endOffset);
@@ -738,6 +764,7 @@ HRESULT STDMETHODCALLTYPE QWindowsIA2Accessible::deleteText(long startOffset, lo
 
 HRESULT STDMETHODCALLTYPE QWindowsIA2Accessible::insertText(long offset, BSTR *text)
 {
+    QAccessibleInterface *accessible = accessibleInterface();
     accessibleDebugClientCalls(accessible);
     const QString txt(BSTRToQString(*text));
     if (QAccessibleEditableTextInterface *editableTextIface = accessible->editableTextInterface())
@@ -749,6 +776,7 @@ HRESULT STDMETHODCALLTYPE QWindowsIA2Accessible::insertText(long offset, BSTR *t
 
 HRESULT STDMETHODCALLTYPE QWindowsIA2Accessible::cutText(long startOffset, long endOffset)
 {
+    QAccessibleInterface *accessible = accessibleInterface();
     accessibleDebugClientCalls(accessible);
 #ifndef QT_NO_CLIPBOARD
     const QString t = textForRange(startOffset, endOffset);
@@ -765,6 +793,7 @@ HRESULT STDMETHODCALLTYPE QWindowsIA2Accessible::cutText(long startOffset, long
 
 HRESULT STDMETHODCALLTYPE QWindowsIA2Accessible::pasteText(long offset)
 {
+    QAccessibleInterface *accessible = accessibleInterface();
     accessibleDebugClientCalls(accessible);
 #ifndef QT_NO_CLIPBOARD
     const QString txt = QGuiApplication::clipboard()->text();
@@ -780,6 +809,7 @@ HRESULT STDMETHODCALLTYPE QWindowsIA2Accessible::pasteText(long offset)
 
 HRESULT STDMETHODCALLTYPE QWindowsIA2Accessible::replaceText(long startOffset, long endOffset, BSTR *text)
 {
+    QAccessibleInterface *accessible = accessibleInterface();
     accessibleDebugClientCalls(accessible);
     const QString txt(BSTRToQString(*text));
     if (QAccessibleEditableTextInterface *editableTextIface = accessible->editableTextInterface())
@@ -800,8 +830,9 @@ HRESULT STDMETHODCALLTYPE QWindowsIA2Accessible::setAttributes(long /*startOffse
  **************************************************************/
 HRESULT STDMETHODCALLTYPE QWindowsIA2Accessible::get_cellAt( long row, long column, IUnknown **cell)
 {
+    QAccessibleInterface *accessible = accessibleInterface();
     accessibleDebugClientCalls(accessible);
-    if (!accessible->isValid())
+    if (!accessible)
         return E_FAIL;
 
     *cell = 0;
@@ -816,8 +847,9 @@ HRESULT STDMETHODCALLTYPE QWindowsIA2Accessible::get_cellAt( long row, long colu
 
 HRESULT STDMETHODCALLTYPE QWindowsIA2Accessible::get_caption( IUnknown **captionInterface)
 {
+    QAccessibleInterface *accessible = accessibleInterface();
     accessibleDebugClientCalls(accessible);
-    if (!accessible->isValid())
+    if (!accessible)
         return E_FAIL;
 
     *captionInterface = 0;
@@ -830,8 +862,9 @@ HRESULT STDMETHODCALLTYPE QWindowsIA2Accessible::get_caption( IUnknown **caption
 
 HRESULT STDMETHODCALLTYPE QWindowsIA2Accessible::get_columnDescription( long column, BSTR *description)
 {
+    QAccessibleInterface *accessible = accessibleInterface();
     accessibleDebugClientCalls(accessible);
-    if (!accessible->isValid())
+    if (!accessible)
         return E_FAIL;
 
     *description = 0;
@@ -845,8 +878,9 @@ HRESULT STDMETHODCALLTYPE QWindowsIA2Accessible::get_columnDescription( long col
 
 HRESULT STDMETHODCALLTYPE QWindowsIA2Accessible::get_nColumns( long *columnCount)
 {
+    QAccessibleInterface *accessible = accessibleInterface();
     accessibleDebugClientCalls(accessible);
-    if (!accessible->isValid())
+    if (!accessible)
         return E_FAIL;
 
     if (QAccessibleTableInterface *tableIface = tableInterface()) {
@@ -858,8 +892,9 @@ HRESULT STDMETHODCALLTYPE QWindowsIA2Accessible::get_nColumns( long *columnCount
 
 HRESULT STDMETHODCALLTYPE QWindowsIA2Accessible::get_nRows(long *rowCount)
 {
+    QAccessibleInterface *accessible = accessibleInterface();
     accessibleDebugClientCalls(accessible);
-    if (!accessible->isValid())
+    if (!accessible)
         return E_FAIL;
 
     if (QAccessibleTableInterface *tableIface = tableInterface()) {
@@ -871,8 +906,9 @@ HRESULT STDMETHODCALLTYPE QWindowsIA2Accessible::get_nRows(long *rowCount)
 
 HRESULT STDMETHODCALLTYPE QWindowsIA2Accessible::get_nSelectedCells(long *cellCount)
 {
+    QAccessibleInterface *accessible = accessibleInterface();
     accessibleDebugClientCalls(accessible);
-    if (!accessible->isValid())
+    if (!accessible)
         return E_FAIL;
 
     if (QAccessibleTableInterface *tableIface = tableInterface()) {
@@ -884,8 +920,9 @@ HRESULT STDMETHODCALLTYPE QWindowsIA2Accessible::get_nSelectedCells(long *cellCo
 
 HRESULT STDMETHODCALLTYPE QWindowsIA2Accessible::get_nSelectedColumns(long *columnCount)
 {
+    QAccessibleInterface *accessible = accessibleInterface();
     accessibleDebugClientCalls(accessible);
-    if (!accessible->isValid())
+    if (!accessible)
         return E_FAIL;
 
     if (QAccessibleTableInterface *tableIface = tableInterface()) {
@@ -897,8 +934,9 @@ HRESULT STDMETHODCALLTYPE QWindowsIA2Accessible::get_nSelectedColumns(long *colu
 
 HRESULT STDMETHODCALLTYPE QWindowsIA2Accessible::get_nSelectedRows(long *rowCount)
 {
+    QAccessibleInterface *accessible = accessibleInterface();
     accessibleDebugClientCalls(accessible);
-    if (!accessible->isValid())
+    if (!accessible)
         return E_FAIL;
 
     if (QAccessibleTableInterface *tableIface = tableInterface()) {
@@ -910,13 +948,14 @@ HRESULT STDMETHODCALLTYPE QWindowsIA2Accessible::get_nSelectedRows(long *rowCoun
 
 HRESULT STDMETHODCALLTYPE QWindowsIA2Accessible::get_rowDescription(long row, BSTR *description)
 {
+    QAccessibleInterface *accessible = accessibleInterface();
     accessibleDebugClientCalls(accessible);
-    if (!accessible->isValid())
+    if (!accessible)
         return E_FAIL;
 
     *description = 0;
     if (QAccessibleTableInterface *tableIface = tableInterface()) {
-        const QString qtDesc = tableIface->columnDescription(row);
+        const QString qtDesc = tableIface->rowDescription(row);
         if (!qtDesc.isEmpty())
             *description = QStringToBSTR(qtDesc);
     }
@@ -925,10 +964,11 @@ HRESULT STDMETHODCALLTYPE QWindowsIA2Accessible::get_rowDescription(long row, BS
 
 HRESULT STDMETHODCALLTYPE QWindowsIA2Accessible::get_selectedCells(IUnknown ***cells, long *nSelectedCells)
 {
+    QAccessibleInterface *accessible = accessibleInterface();
     accessibleDebugClientCalls(accessible);
     Q_UNUSED(cells);
     Q_UNUSED(nSelectedCells);
-    if (!accessible->isValid())
+    if (!accessible)
         return E_FAIL;
 
     QList<QAccessibleInterface*> selectedCells = tableInterface()->selectedCells();
@@ -937,8 +977,9 @@ HRESULT STDMETHODCALLTYPE QWindowsIA2Accessible::get_selectedCells(IUnknown ***c
 
 HRESULT STDMETHODCALLTYPE QWindowsIA2Accessible::get_selectedColumns(long **selectedColumns, long *nColumns)
 {
+    QAccessibleInterface *accessible = accessibleInterface();
     accessibleDebugClientCalls(accessible);
-    if (!accessible->isValid())
+    if (!accessible)
         return E_FAIL;
 
     if (QAccessibleTableInterface *tableIface = tableInterface()) {
@@ -957,8 +998,9 @@ HRESULT STDMETHODCALLTYPE QWindowsIA2Accessible::get_selectedColumns(long **sele
 
 HRESULT STDMETHODCALLTYPE QWindowsIA2Accessible::get_selectedRows(long **selectedRows, long *nRows)
 {
+    QAccessibleInterface *accessible = accessibleInterface();
     accessibleDebugClientCalls(accessible);
-    if (!accessible->isValid())
+    if (!accessible)
         return E_FAIL;
 
     if (QAccessibleTableInterface *tableIface = tableInterface()) {
@@ -977,8 +1019,9 @@ HRESULT STDMETHODCALLTYPE QWindowsIA2Accessible::get_selectedRows(long **selecte
 
 HRESULT STDMETHODCALLTYPE QWindowsIA2Accessible::get_summary(IUnknown **summaryInterface)
 {
+    QAccessibleInterface *accessible = accessibleInterface();
     accessibleDebugClientCalls(accessible);
-    if (!accessible->isValid())
+    if (!accessible)
         return E_FAIL;
 
     *summaryInterface = 0;
@@ -991,8 +1034,9 @@ HRESULT STDMETHODCALLTYPE QWindowsIA2Accessible::get_summary(IUnknown **summaryI
 
 HRESULT STDMETHODCALLTYPE QWindowsIA2Accessible::get_isColumnSelected(long column, boolean *isSelected)
 {
+    QAccessibleInterface *accessible = accessibleInterface();
     accessibleDebugClientCalls(accessible);
-    if (!accessible->isValid())
+    if (!accessible)
         return E_FAIL;
 
     if (QAccessibleTableInterface *tableIface = tableInterface()) {
@@ -1004,8 +1048,9 @@ HRESULT STDMETHODCALLTYPE QWindowsIA2Accessible::get_isColumnSelected(long colum
 
 HRESULT STDMETHODCALLTYPE QWindowsIA2Accessible::get_isRowSelected(long row, boolean *isSelected)
 {
+    QAccessibleInterface *accessible = accessibleInterface();
     accessibleDebugClientCalls(accessible);
-    if (!accessible->isValid())
+    if (!accessible)
         return E_FAIL;
 
     if (QAccessibleTableInterface *tableIface = tableInterface()) {
@@ -1017,8 +1062,9 @@ HRESULT STDMETHODCALLTYPE QWindowsIA2Accessible::get_isRowSelected(long row, boo
 
 HRESULT STDMETHODCALLTYPE QWindowsIA2Accessible::selectRow(long row)
 {
+    QAccessibleInterface *accessible = accessibleInterface();
     accessibleDebugClientCalls(accessible);
-    if (!accessible->isValid())
+    if (!accessible)
         return E_FAIL;
 
     if (QAccessibleTableInterface *tableIface = tableInterface()) {
@@ -1030,8 +1076,9 @@ HRESULT STDMETHODCALLTYPE QWindowsIA2Accessible::selectRow(long row)
 
 HRESULT STDMETHODCALLTYPE QWindowsIA2Accessible::selectColumn(long column)
 {
+    QAccessibleInterface *accessible = accessibleInterface();
     accessibleDebugClientCalls(accessible);
-    if (!accessible->isValid())
+    if (!accessible)
         return E_FAIL;
 
     if (QAccessibleTableInterface *tableIface = tableInterface()) {
@@ -1043,8 +1090,9 @@ HRESULT STDMETHODCALLTYPE QWindowsIA2Accessible::selectColumn(long column)
 
 HRESULT STDMETHODCALLTYPE QWindowsIA2Accessible::unselectRow(long row)
 {
+    QAccessibleInterface *accessible = accessibleInterface();
     accessibleDebugClientCalls(accessible);
-    if (!accessible->isValid())
+    if (!accessible)
         return E_FAIL;
 
     if (QAccessibleTableInterface *tableIface = tableInterface()) {
@@ -1056,8 +1104,9 @@ HRESULT STDMETHODCALLTYPE QWindowsIA2Accessible::unselectRow(long row)
 
 HRESULT STDMETHODCALLTYPE QWindowsIA2Accessible::unselectColumn(long column)
 {
+    QAccessibleInterface *accessible = accessibleInterface();
     accessibleDebugClientCalls(accessible);
-    if (!accessible->isValid())
+    if (!accessible)
         return E_FAIL;
 
     if (QAccessibleTableInterface *tableIface = tableInterface()) {
@@ -1069,8 +1118,9 @@ HRESULT STDMETHODCALLTYPE QWindowsIA2Accessible::unselectColumn(long column)
 
 HRESULT STDMETHODCALLTYPE QWindowsIA2Accessible::get_modelChange( IA2TableModelChange * /*modelChange*/)
 {
+    QAccessibleInterface *accessible = accessibleInterface();
     accessibleDebugClientCalls(accessible);
-    if (!accessible->isValid())
+    if (!accessible)
         return E_FAIL;
     return E_NOTIMPL;
 }
@@ -1080,8 +1130,9 @@ HRESULT STDMETHODCALLTYPE QWindowsIA2Accessible::get_modelChange( IA2TableModelC
 \**************************************************************/
 HRESULT STDMETHODCALLTYPE QWindowsIA2Accessible::get_columnExtent(long *nColumnsSpanned)
 {
+    QAccessibleInterface *accessible = accessibleInterface();
     accessibleDebugClientCalls(accessible);
-    if (!accessible->isValid())
+    if (!accessible)
         return E_FAIL;
 
     *nColumnsSpanned = tableCellInterface()->columnExtent();
@@ -1091,8 +1142,9 @@ HRESULT STDMETHODCALLTYPE QWindowsIA2Accessible::get_columnExtent(long *nColumns
 HRESULT STDMETHODCALLTYPE QWindowsIA2Accessible::get_columnHeaderCells(IUnknown ***cellAccessibles,
                                                                     long *nColumnHeaderCells)
 {
+    QAccessibleInterface *accessible = accessibleInterface();
     accessibleDebugClientCalls(accessible);
-    if (!accessible->isValid())
+    if (!accessible)
         return E_FAIL;
     const QList<QAccessibleInterface*> headerCells = tableCellInterface()->columnHeaderCells();
     return wrapListOfCells(headerCells, cellAccessibles, nColumnHeaderCells);
@@ -1100,8 +1152,9 @@ HRESULT STDMETHODCALLTYPE QWindowsIA2Accessible::get_columnHeaderCells(IUnknown
 
 HRESULT STDMETHODCALLTYPE QWindowsIA2Accessible::get_columnIndex(long *columnIndex)
 {
+    QAccessibleInterface *accessible = accessibleInterface();
     accessibleDebugClientCalls(accessible);
-    if (!accessible->isValid())
+    if (!accessible)
         return E_FAIL;
     *columnIndex = tableCellInterface()->columnIndex();
     return S_OK;
@@ -1109,8 +1162,9 @@ HRESULT STDMETHODCALLTYPE QWindowsIA2Accessible::get_columnIndex(long *columnInd
 
 HRESULT STDMETHODCALLTYPE QWindowsIA2Accessible::get_rowExtent(long *nRowsSpanned)
 {
+    QAccessibleInterface *accessible = accessibleInterface();
     accessibleDebugClientCalls(accessible);
-    if (!accessible->isValid())
+    if (!accessible)
         return E_FAIL;
     *nRowsSpanned = tableCellInterface()->rowExtent();
     return S_OK;
@@ -1119,8 +1173,9 @@ HRESULT STDMETHODCALLTYPE QWindowsIA2Accessible::get_rowExtent(long *nRowsSpanne
 HRESULT STDMETHODCALLTYPE QWindowsIA2Accessible::get_rowHeaderCells(IUnknown ***cellAccessibles,
                                                                  long *nRowHeaderCells)
 {
+    QAccessibleInterface *accessible = accessibleInterface();
     accessibleDebugClientCalls(accessible);
-    if (!accessible->isValid())
+    if (!accessible)
         return E_FAIL;
     const QList<QAccessibleInterface*> headerCells = tableCellInterface()->rowHeaderCells();
     return wrapListOfCells(headerCells, cellAccessibles, nRowHeaderCells);
@@ -1128,8 +1183,9 @@ HRESULT STDMETHODCALLTYPE QWindowsIA2Accessible::get_rowHeaderCells(IUnknown ***
 
 HRESULT STDMETHODCALLTYPE QWindowsIA2Accessible::get_rowIndex(long *rowIndex)
 {
+    QAccessibleInterface *accessible = accessibleInterface();
     accessibleDebugClientCalls(accessible);
-    if (!accessible->isValid())
+    if (!accessible)
         return E_FAIL;
     *rowIndex = tableCellInterface()->rowIndex();
     return S_OK;
@@ -1137,8 +1193,9 @@ HRESULT STDMETHODCALLTYPE QWindowsIA2Accessible::get_rowIndex(long *rowIndex)
 
 HRESULT STDMETHODCALLTYPE QWindowsIA2Accessible::get_isSelected( boolean *isSelected)
 {
+    QAccessibleInterface *accessible = accessibleInterface();
     accessibleDebugClientCalls(accessible);
-    if (!accessible->isValid())
+    if (!accessible)
         return E_FAIL;
     *isSelected = tableCellInterface()->isSelected();
     return S_OK;
@@ -1148,18 +1205,24 @@ HRESULT STDMETHODCALLTYPE QWindowsIA2Accessible::get_rowColumnExtents(long *row,
                                                long *rowExtents, long *columnExtents,
                                                boolean *isSelected)
 {
+    QAccessibleInterface *accessible = accessibleInterface();
     accessibleDebugClientCalls(accessible);
-    if (!accessible->isValid())
+    if (!accessible || !tableCellInterface())
         return E_FAIL;
 
-    tableCellInterface()->rowColumnExtents((int*)row, (int*)column, (int*)rowExtents, (int*)columnExtents, (bool*)isSelected);
+    *row = (long)tableCellInterface()->rowIndex();
+    *column = (long)tableCellInterface()->columnIndex();
+    *rowExtents = (long)tableCellInterface()->rowExtent();
+    *columnExtents = (long)tableCellInterface()->columnExtent();
+    *isSelected = tableCellInterface()->isSelected();
     return S_OK;
 }
 
 HRESULT STDMETHODCALLTYPE QWindowsIA2Accessible::get_table(IUnknown **table)
 {
+    QAccessibleInterface *accessible = accessibleInterface();
     accessibleDebugClientCalls(accessible);
-    if (!accessible->isValid())
+    if (!accessible)
         return E_FAIL;
 
     QAccessibleInterface *tableIface = tableCellInterface()->table();
@@ -1174,6 +1237,7 @@ HRESULT STDMETHODCALLTYPE QWindowsIA2Accessible::get_table(IUnknown **table)
 HRESULT STDMETHODCALLTYPE QWindowsIA2Accessible::addSelection(long startOffset,
                                                            long endOffset)
 {
+    QAccessibleInterface *accessible = accessibleInterface();
     accessibleDebugClientCalls(accessible);
     if (QAccessibleTextInterface *text = textInterface()) {
         text->addSelection(startOffset, endOffset);
@@ -1187,6 +1251,7 @@ HRESULT STDMETHODCALLTYPE QWindowsIA2Accessible::get_attributes(long offset,
                                                              long *endOffset,
                                                              BSTR *textAttributes)
 {
+    QAccessibleInterface *accessible = accessibleInterface();
     accessibleDebugClientCalls(accessible);
     if (QAccessibleTextInterface *text = textInterface()) {
         const QString attrs = text->attributes(offset, (int*)startOffset, (int*)endOffset);
@@ -1198,6 +1263,7 @@ HRESULT STDMETHODCALLTYPE QWindowsIA2Accessible::get_attributes(long offset,
 
 HRESULT STDMETHODCALLTYPE QWindowsIA2Accessible::get_caretOffset(long *offset)
 {
+    QAccessibleInterface *accessible = accessibleInterface();
     accessibleDebugClientCalls(accessible);
     if (QAccessibleTextInterface *text = textInterface()) {
         *offset = text->cursorPosition();
@@ -1214,6 +1280,7 @@ HRESULT STDMETHODCALLTYPE QWindowsIA2Accessible::get_characterExtents(long offse
                                                                    long *width,
                                                                    long *height)
 {
+    QAccessibleInterface *accessible = accessibleInterface();
     accessibleDebugClientCalls(accessible);
     if (QAccessibleTextInterface *text = textInterface()) {
         QRect rect = text->characterRect(offset);
@@ -1227,6 +1294,7 @@ HRESULT STDMETHODCALLTYPE QWindowsIA2Accessible::get_characterExtents(long offse
 
 HRESULT STDMETHODCALLTYPE QWindowsIA2Accessible::get_nSelections(long *nSelections)
 {
+    QAccessibleInterface *accessible = accessibleInterface();
     accessibleDebugClientCalls(accessible);
     if (QAccessibleTextInterface *text = textInterface()) {
         *nSelections = text->selectionCount();
@@ -1240,6 +1308,7 @@ HRESULT STDMETHODCALLTYPE QWindowsIA2Accessible::get_offsetAtPoint(long x,
                                                                 enum IA2CoordinateType coordType,
                                                                 long *offset)
 {
+    QAccessibleInterface *accessible = accessibleInterface();
     accessibleDebugClientCalls(accessible);
     if (QAccessibleTextInterface *text = textInterface()) {
         QPoint screenPos = mapToScreenPos(coordType, x, y);
@@ -1254,6 +1323,7 @@ HRESULT STDMETHODCALLTYPE QWindowsIA2Accessible::get_selection(long selectionInd
                                                             long *startOffset,
                                                             long *endOffset)
 {
+    QAccessibleInterface *accessible = accessibleInterface();
     accessibleDebugClientCalls(accessible);
     if (QAccessibleTextInterface *text = textInterface()) {
         text->selection(selectionIndex, (int*)startOffset, (int*)endOffset);
@@ -1266,6 +1336,7 @@ HRESULT STDMETHODCALLTYPE QWindowsIA2Accessible::get_text(long startOffset,
                                                        long endOffset,
                                                        BSTR *text)
 {
+    QAccessibleInterface *accessible = accessibleInterface();
     accessibleDebugClientCalls(accessible);
     if (QAccessibleTextInterface *textif = textInterface()) {
         const QString t = textif->text(startOffset, endOffset);
@@ -1284,9 +1355,10 @@ HRESULT STDMETHODCALLTYPE QWindowsIA2Accessible::get_textBeforeOffset(long offse
                                                long *endOffset,
                                                BSTR *text)
 {
+    QAccessibleInterface *accessible = accessibleInterface();
     accessibleDebugClientCalls(accessible);
     if (QAccessibleTextInterface *textIface = textInterface()) {
-        const QString txt = textIface->textBeforeOffset(offset, (QAccessible2::BoundaryType)boundaryType, (int*)startOffset, (int*)endOffset);
+        const QString txt = textIface->textBeforeOffset(offset, (QAccessible::TextBoundaryType)boundaryType, (int*)startOffset, (int*)endOffset);
         if (!txt.isEmpty()) {
             *text = QStringToBSTR(txt);
             return S_OK;
@@ -1303,9 +1375,10 @@ HRESULT STDMETHODCALLTYPE QWindowsIA2Accessible::get_textAfterOffset(
     long *endOffset,
     BSTR *text)
 {
+    QAccessibleInterface *accessible = accessibleInterface();
     accessibleDebugClientCalls(accessible);
     if (QAccessibleTextInterface *textIface = textInterface()) {
-        const QString txt = textIface->textAfterOffset(offset, (QAccessible2::BoundaryType)boundaryType, (int*)startOffset, (int*)endOffset);
+        const QString txt = textIface->textAfterOffset(offset, (QAccessible::TextBoundaryType)boundaryType, (int*)startOffset, (int*)endOffset);
         if (!txt.isEmpty()) {
             *text = QStringToBSTR(txt);
             return S_OK;
@@ -1321,9 +1394,10 @@ HRESULT STDMETHODCALLTYPE QWindowsIA2Accessible::get_textAtOffset(long offset,
                                                                long *endOffset,
                                                                BSTR *text)
 {
+    QAccessibleInterface *accessible = accessibleInterface();
     accessibleDebugClientCalls(accessible);
     if (QAccessibleTextInterface *textIface = textInterface()) {
-        const QString txt = textIface->textAtOffset(offset, (QAccessible2::BoundaryType)boundaryType, (int*)startOffset, (int*)endOffset);
+        const QString txt = textIface->textAtOffset(offset, (QAccessible::TextBoundaryType)boundaryType, (int*)startOffset, (int*)endOffset);
         if (!txt.isEmpty()) {
             *text = QStringToBSTR(txt);
             return S_OK;
@@ -1335,6 +1409,7 @@ HRESULT STDMETHODCALLTYPE QWindowsIA2Accessible::get_textAtOffset(long offset,
 
 HRESULT STDMETHODCALLTYPE QWindowsIA2Accessible::removeSelection(long selectionIndex)
 {
+    QAccessibleInterface *accessible = accessibleInterface();
     accessibleDebugClientCalls(accessible);
     if (QAccessibleTextInterface *textIface = textInterface()) {
         textIface->removeSelection(selectionIndex);
@@ -1345,6 +1420,7 @@ HRESULT STDMETHODCALLTYPE QWindowsIA2Accessible::removeSelection(long selectionI
 
 HRESULT STDMETHODCALLTYPE QWindowsIA2Accessible::setCaretOffset(long offset)
 {
+    QAccessibleInterface *accessible = accessibleInterface();
     accessibleDebugClientCalls(accessible);
     if (QAccessibleTextInterface *textIface = textInterface()) {
         textIface->setCursorPosition(offset);
@@ -1357,6 +1433,7 @@ HRESULT STDMETHODCALLTYPE QWindowsIA2Accessible::setSelection(long selectionInde
                                                            long startOffset,
                                                            long endOffset)
 {
+    QAccessibleInterface *accessible = accessibleInterface();
     accessibleDebugClientCalls(accessible);
     if (QAccessibleTextInterface *textIface = textInterface()) {
         textIface->setSelection(selectionIndex, startOffset, endOffset);
@@ -1367,6 +1444,7 @@ HRESULT STDMETHODCALLTYPE QWindowsIA2Accessible::setSelection(long selectionInde
 
 HRESULT STDMETHODCALLTYPE QWindowsIA2Accessible::get_nCharacters(long *nCharacters)
 {
+    QAccessibleInterface *accessible = accessibleInterface();
     accessibleDebugClientCalls(accessible);
     if (QAccessibleTextInterface *textIface = textInterface()) {
         *nCharacters = textIface->characterCount();
@@ -1379,6 +1457,7 @@ HRESULT STDMETHODCALLTYPE QWindowsIA2Accessible::scrollSubstringTo(long startInd
                                                                 long endIndex,
                                                                 enum IA2ScrollType scrollType)
 {
+    QAccessibleInterface *accessible = accessibleInterface();
     accessibleDebugClientCalls(accessible);
     if (QAccessibleTextInterface *textIface = textInterface()) {
         Q_UNUSED(scrollType);   //###
@@ -1420,8 +1499,9 @@ HRESULT STDMETHODCALLTYPE QWindowsIA2Accessible::get_oldText(IA2TextSegment *old
  **************************************************************/
 HRESULT STDMETHODCALLTYPE QWindowsIA2Accessible::get_currentValue(VARIANT *currentValue)
 {
+    QAccessibleInterface *accessible = accessibleInterface();
     accessibleDebugClientCalls(accessible);
-    if (!accessible->isValid())
+    if (!accessible)
         return E_FAIL;
     if (QAccessibleValueInterface *valueIface = valueInterface()) {
         const QVariant var = valueIface->currentValue();
@@ -1435,8 +1515,9 @@ HRESULT STDMETHODCALLTYPE QWindowsIA2Accessible::get_currentValue(VARIANT *curre
 
 HRESULT STDMETHODCALLTYPE QWindowsIA2Accessible::setCurrentValue(VARIANT value)
 {
+    QAccessibleInterface *accessible = accessibleInterface();
     accessibleDebugClientCalls(accessible);
-    if (!accessible->isValid())
+    if (!accessible)
         return E_FAIL;
     HRESULT hr = S_FALSE;
     if (QAccessibleValueInterface *valueIface = valueInterface()) {
@@ -1451,8 +1532,9 @@ HRESULT STDMETHODCALLTYPE QWindowsIA2Accessible::setCurrentValue(VARIANT value)
 
 HRESULT STDMETHODCALLTYPE QWindowsIA2Accessible::get_maximumValue(VARIANT *maximumValue)
 {
+    QAccessibleInterface *accessible = accessibleInterface();
     accessibleDebugClientCalls(accessible);
-    if (!accessible->isValid())
+    if (!accessible)
         return E_FAIL;
     if (QAccessibleValueInterface *valueIface = valueInterface()) {
         const QVariant var = valueIface->maximumValue();
@@ -1465,8 +1547,9 @@ HRESULT STDMETHODCALLTYPE QWindowsIA2Accessible::get_maximumValue(VARIANT *maxim
 
 HRESULT STDMETHODCALLTYPE QWindowsIA2Accessible::get_minimumValue(VARIANT *minimumValue)
 {
+    QAccessibleInterface *accessible = accessibleInterface();
     accessibleDebugClientCalls(accessible);
-    if (!accessible->isValid())
+    if (!accessible)
         return E_FAIL;
     if (QAccessibleValueInterface *valueIface = valueInterface()) {
         const QVariant var = valueIface->minimumValue();
@@ -1538,6 +1621,7 @@ HRESULT STDMETHODCALLTYPE QWindowsIA2Accessible::QueryService(REFGUID guidServic
 */
 HRESULT QWindowsIA2Accessible::getRelationsHelper(IAccessibleRelation **relations, int startIndex, long maxRelations, long *nRelations /* = 0*/)
 {
+    QAccessibleInterface *accessible = accessibleInterface();
     if (nRelations)
         *nRelations = 0;
     typedef QPair<QAccessibleInterface *, QAccessible::Relation> RelationEntry;
@@ -1587,37 +1671,6 @@ HRESULT QWindowsIA2Accessible::wrapListOfCells(const QList<QAccessibleInterface*
     return count > 0 ? S_OK : S_FALSE;
 }
 
-uint QWindowsIA2Accessible::uniqueID() const
-{
-    uint uid = 0;
-    if (QObject *obj = accessible->object())
-        uid = qHash(obj);
-
-    if (!uid) {
-        QAccessibleInterface *acc = accessible;
-        QVector<int> indexOfNodes;
-        while (acc && !acc->object()) {
-            QAccessibleInterface *par = acc->parent();
-            indexOfNodes.append(par->indexOfChild(acc));
-            if (acc != accessible)
-                delete acc;
-            acc = par;
-        }
-        if (acc) {
-            if (acc->object()) {
-                uid = qHash(acc->object());
-                for (int i = 0; i < indexOfNodes.count(); ++i)
-                    uid = qHash(uid + indexOfNodes.at(i));
-
-            }
-            if (acc != accessible)
-                delete acc;
-        }
-    }
-    return uid;
-}
-
-
 #define IF_EQUAL_RETURN_IIDSTRING(id, iid) if (id == iid) return QByteArray(#iid)
 
 QByteArray QWindowsIA2Accessible::IIDToString(REFIID id)
