@@ -49,6 +49,14 @@
 #    define WM_THEMECHANGED 0x031A
 #endif
 
+#ifndef WM_DWMCOMPOSITIONCHANGED
+#    define WM_DWMCOMPOSITIONCHANGED 0x31E
+#endif
+
+#ifndef GWL_HWNDPARENT
+#    define GWL_HWNDPARENT (-8)
+#endif
+
 /* Complement the definitions and declarations missing
  * when using MinGW or older Windows SDKs. */
 
@@ -126,6 +134,10 @@ typedef struct tagUPDATELAYEREDWINDOWINFO {
 // IME.
 #define IMR_CONFIRMRECONVERTSTRING      0x0005
 
+#ifndef MAPVK_VK_TO_CHAR
+#  define MAPVK_VK_TO_CHAR 2
+#endif
+
 #endif // if defined(Q_CC_MINGW)
 
 /* Touch is supported from Windows 7 onwards and data structures
