@@ -1,5 +1,6 @@
 /****************************************************************************
 **
+** Copyright (C) 2013 Samuel Gaist <samuel.gaist@edeltech.ch>
 ** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
@@ -47,8 +48,14 @@
 #include "qtwindowsglobal.h"
 #include "qwindowsmime.h"
 #include "qwindowsinputcontext.h"
+#include "qwindowstabletsupport.h"
 #ifndef QT_NO_ACCESSIBILITY
-#include "accessible/qwindowsaccessibility.h"
+# include "accessible/qwindowsaccessibility.h"
+#endif
+#if !defined(Q_OS_WINCE) && !defined(QT_NO_SESSIONMANAGER)
+# include <private/qguiapplication_p.h>
+# include <private/qsessionmanager_p.h>
+# include "qwindowssessionmanager.h"
 #endif
 #include "qwindowsscreen.h"
 #include "qwindowstheme.h"
@@ -83,6 +90,7 @@ int QWindowsContext::verboseOLE = 0;
 int QWindowsContext::verboseInputMethods = 0;
 int QWindowsContext::verboseDialogs = 0;
 int QWindowsContext::verboseTheming = 0;
+int QWindowsContext::verboseTablet = 0;
 
 // Get verbosity of components from "foo:2,bar:3"
 static inline int componentVerbose(const char *v, const char *keyWord)
@@ -149,6 +157,14 @@ static inline bool useRTL_Extensions(QSysInfo::WinVersion ver)
 #endif
 }
 
+#if !defined(Q_OS_WINCE) && !defined(QT_NO_SESSIONMANAGER)
+static inline QWindowsSessionManager *platformSessionManager() {
+    QGuiApplicationPrivate *guiPrivate = static_cast<QGuiApplicationPrivate*>(QObjectPrivate::get(qApp));
+    QSessionManagerPrivate *managerPrivate = static_cast<QSessionManagerPrivate*>(QObjectPrivate::get(guiPrivate->session_manager));
+    return static_cast<QWindowsSessionManager *>(managerPrivate->platformSessionManager);
+}
+#endif
+
 /*!
     \class QWindowsUser32DLL
     \brief Struct that contains dynamically resolved symbols of User32.dll.
@@ -216,6 +232,7 @@ bool QWindowsUser32DLL::initTouch()
 QWindowsShell32DLL::QWindowsShell32DLL()
     : sHCreateItemFromParsingName(0)
     , sHGetStockIconInfo(0)
+    , sHGetImageList(0)
 {
 }
 
@@ -224,6 +241,7 @@ void QWindowsShell32DLL::init()
     QSystemLibrary library(QStringLiteral("shell32"));
     sHCreateItemFromParsingName = (SHCreateItemFromParsingName)(library.resolve("SHCreateItemFromParsingName"));
     sHGetStockIconInfo = (SHGetStockIconInfo)library.resolve("SHGetStockIconInfo");
+    sHGetImageList = (SHGetImageList)library.resolve("SHGetImageList");
 }
 
 QWindowsUser32DLL QWindowsContext::user32dll;
@@ -259,34 +277,34 @@ struct QWindowsContextPrivate {
     QWindowsMimeConverter m_mimeConverter;
     QWindowsScreenManager m_screenManager;
     QSharedPointer<QWindowCreationContext> m_creationContext;
+#if !defined(QT_NO_TABLETEVENT) && !defined(Q_OS_WINCE)
+    QScopedPointer<QWindowsTabletSupport> m_tabletSupport;
+#endif
     const HRESULT m_oleInitializeResult;
     const QByteArray m_eventType;
     QWindow *m_lastActiveWindow;
     bool m_asyncExpose;
 };
 
-QWindowsContextPrivate::QWindowsContextPrivate() :
-    m_systemInfo(0),
-    m_oleInitializeResult(OleInitialize(NULL)),
-    m_eventType(QByteArrayLiteral("windows_generic_MSG")),
-    m_lastActiveWindow(0), m_asyncExpose(0)
+QWindowsContextPrivate::QWindowsContextPrivate()
+    : m_systemInfo(0)
+    , m_oleInitializeResult(OleInitialize(NULL))
+    , m_eventType(QByteArrayLiteral("windows_generic_MSG"))
+    , m_lastActiveWindow(0), m_asyncExpose(0)
 {
+    const QSysInfo::WinVersion ver = QSysInfo::windowsVersion();
 #ifndef Q_OS_WINCE
     QWindowsContext::user32dll.init();
     QWindowsContext::shell32dll.init();
-#endif
     // Ensure metrics functions report correct data, QTBUG-30063.
     if (QWindowsContext::user32dll.setProcessDPIAware)
         QWindowsContext::user32dll.setProcessDPIAware();
-    m_displayContext = GetDC(0);
-    m_defaultDPI = GetDeviceCaps(m_displayContext, LOGPIXELSY);
 
-    const QSysInfo::WinVersion ver = QSysInfo::windowsVersion();
-#ifndef Q_OS_WINCE
     if (hasTouchSupport(ver) && QWindowsContext::user32dll.initTouch())
         m_systemInfo |= QWindowsContext::SI_SupportsTouch;
-#endif
-
+#endif // !Q_OS_WINCE
+    m_displayContext = GetDC(0);
+    m_defaultDPI = GetDeviceCaps(m_displayContext, LOGPIXELSY);
     if (useRTL_Extensions(ver)) {
         m_systemInfo |= QWindowsContext::SI_RTL_Extensions;
         m_keyMapper.setUseRTLExtensions(true);
@@ -313,7 +331,13 @@ QWindowsContext::QWindowsContext() :
         QWindowsContext::verboseInputMethods = componentVerbose(v, "im");
         QWindowsContext::verboseDialogs = componentVerbose(v, "dialogs");
         QWindowsContext::verboseTheming = componentVerbose(v, "theming");
+        QWindowsContext::verboseTablet = componentVerbose(v, "tablet");
     }
+#if !defined(QT_NO_TABLETEVENT) && !defined(Q_OS_WINCE)
+    d->m_tabletSupport.reset(QWindowsTabletSupport::create());
+    if (QWindowsContext::verboseTablet)
+        qDebug() << "Tablet support: " << (d->m_tabletSupport.isNull() ? QStringLiteral("None") : d->m_tabletSupport->description());
+#endif
 }
 
 QWindowsContext::~QWindowsContext()
@@ -372,58 +396,52 @@ void QWindowsContext::setKeyGrabber(QWindow *w)
 }
 
 // Window class registering code (from qapplication_win.cpp)
-// If 0 is passed as the widget pointer, register a window class
-// for QWidget as default. This is used in QGLTemporaryContext
-// during GL initialization, where we don't want to use temporary
-// QWidgets or QGLWidgets, neither do we want to have separate code
-// to register window classes.
 
 QString QWindowsContext::registerWindowClass(const QWindow *w, bool isGL)
 {
-    const Qt::WindowFlags flags = w ? w->flags() : (Qt::WindowFlags)0;
+    Q_ASSERT(w);
+    const Qt::WindowFlags flags = w->flags();
     const Qt::WindowFlags type = flags & Qt::WindowType_Mask;
-
-    uint style = 0;
-    bool icon = false;
-    QString cname = QStringLiteral("Qt5");
-    if (w && isGL) {
-        cname += QStringLiteral("QGLWindow");
-        style = CS_DBLCLKS|CS_OWNDC;
-        icon  = true;
-    } else if (w && (flags & Qt::MSWindowsOwnDC)) {
-        cname += QStringLiteral("QWindowOwnDC");
-        style = CS_DBLCLKS|CS_OWNDC;
-        icon  = true;
-    } else if (w && (type == Qt::Tool || type == Qt::ToolTip)) {
-        style = CS_DBLCLKS;
-        if (w->inherits("QTipLabel") || w->inherits("QAlphaWidget")) {
-            if ((QSysInfo::WindowsVersion >= QSysInfo::WV_XP
-                && (QSysInfo::WindowsVersion & QSysInfo::WV_NT_based))) {
-                style |= CS_DROPSHADOW;
-            }
-            cname += QStringLiteral("QToolTip");
-        } else {
-            cname += QStringLiteral("QTool");
-        }
-        style |= CS_SAVEBITS;
-        icon = false;
-    } else if (w && (type == Qt::Popup)) {
-        cname += QStringLiteral("QPopup");
-        style = CS_DBLCLKS|CS_SAVEBITS;
-        if ((QSysInfo::WindowsVersion >= QSysInfo::WV_XP
-            && (QSysInfo::WindowsVersion & QSysInfo::WV_NT_based)))
-            style |= CS_DROPSHADOW;
-        icon = false;
-    } else {
-        cname += QStringLiteral("QWindow");
-        style = CS_DBLCLKS;
-        icon  = true;
+    // Determine style and icon.
+    uint style = CS_DBLCLKS;
+    bool icon = true;
+    if (isGL || (flags & Qt::MSWindowsOwnDC))
+        style |= CS_OWNDC;
+    if ((QSysInfo::WindowsVersion & QSysInfo::WV_NT_based)
+        && (type == Qt::Popup || w->property("_q_windowsDropShadow").toBool())) {
+        style |= CS_DROPSHADOW;
     }
+    if (type == Qt::Tool || type == Qt::ToolTip || type == Qt::Popup) {
+        style |= CS_SAVEBITS; // Save/restore background
+        icon = false;
+    }
+    // Create a unique name for the flag combination
+    QString cname = QStringLiteral("Qt5QWindow");
+    switch (type) {
+    case Qt::Tool:
+        cname += QStringLiteral("Tool");
+        break;
+    case Qt::ToolTip:
+        cname += QStringLiteral("ToolTip");
+        break;
+    case Qt::Popup:
+        cname += QStringLiteral("Popup");
+        break;
+    default:
+        break;
+    }
+    if (isGL)
+        cname += QStringLiteral("GL");
+    if (style & CS_DROPSHADOW)
+        cname += QStringLiteral("DropShadow");
+    if (style & CS_SAVEBITS)
+        cname += QStringLiteral("SaveBits");
+    if (style & CS_OWNDC)
+        cname += QStringLiteral("OwnDC");
+    if (icon)
+        cname += QStringLiteral("Icon");
 
-    HBRUSH brush = 0;
-    if (w && !isGL)
-        brush = GetSysColorBrush(COLOR_WINDOW);
-    return registerWindowClass(cname, qWindowsWndProc, style, brush, icon);
+    return registerWindowClass(cname, qWindowsWndProc, style, GetSysColorBrush(COLOR_WINDOW), icon);
 }
 
 QString QWindowsContext::registerWindowClass(QString cname,
@@ -591,6 +609,11 @@ QWindow *QWindowsContext::windowUnderMouse() const
     return d->m_mouseHandler.windowUnderMouse();
 }
 
+void QWindowsContext::clearWindowUnderMouse()
+{
+    d->m_mouseHandler.clearWindowUnderMouse();
+}
+
 /*!
     \brief Find a child window at a screen point.
 
@@ -635,6 +658,15 @@ QWindowsScreenManager &QWindowsContext::screenManager()
     return d->m_screenManager;
 }
 
+QWindowsTabletSupport *QWindowsContext::tabletSupport() const
+{
+#if !defined(QT_NO_TABLETEVENT) && !defined(Q_OS_WINCE)
+    return d->m_tabletSupport.data();
+#else
+    return 0;
+#endif
+}
+
 /*!
     \brief Convenience to create a non-visible, message-only dummy
     window for example used as clipboard watcher or for GL.
@@ -751,9 +783,15 @@ bool QWindowsContext::windowsProc(HWND hwnd, UINT message,
     case QtWindows::InputMethodCloseCandidateWindowEvent:
         // TODO: Release/regrab mouse if a popup has mouse grab.
         return false;
-    case QtWindows::ClipboardEvent:
     case QtWindows::DestroyEvent:
-
+        if (!platformWindow->testFlag(QWindowsWindow::WithinDestroy)) {
+            qWarning() << "External WM_DESTROY received for " << platformWindow->window()
+                       << ", parent: " << platformWindow->window()->parent()
+                       << ", transient parent: " << platformWindow->window()->transientParent();
+            }
+        return false;
+    case QtWindows::ClipboardEvent:
+        return false;
     case QtWindows::UnknownEvent:
         return false;
     case QtWindows::AccessibleObjectFromWindowRequest:
@@ -787,7 +825,7 @@ bool QWindowsContext::windowsProc(HWND hwnd, UINT message,
             d->m_creationContext->obtainedGeometry.moveTo(GET_X_LPARAM(lParam), GET_Y_LPARAM(lParam));
             return true;
         case QtWindows::CalculateSize:
-            return false;
+            return QWindowsGeometryHint::handleCalculateSize(d->m_creationContext->customMargins, msg, result);
         default:
             break;
         }
@@ -810,7 +848,12 @@ bool QWindowsContext::windowsProc(HWND hwnd, UINT message,
     case QtWindows::KeyEvent:
     case QtWindows::InputMethodKeyEvent:
     case QtWindows::InputMethodKeyDownEvent:
+    case QtWindows::KeyboardLayoutChangeEvent:
+#if !defined(Q_OS_WINCE) && !defined(QT_NO_SESSIONMANAGER)
+        return platformSessionManager()->isInteractionBlocked() ? true : d->m_keyMapper.translateKeyEvent(platformWindow->window(), hwnd, msg, result);
+#else
         return d->m_keyMapper.translateKeyEvent(platformWindow->window(), hwnd, msg, result);
+#endif
     case QtWindows::MoveEvent:
         platformWindow->handleMoved();
         return true;
@@ -822,18 +865,19 @@ bool QWindowsContext::windowsProc(HWND hwnd, UINT message,
         platformWindow->getSizeHints(reinterpret_cast<MINMAXINFO *>(lParam));
         return true;// maybe available on some SDKs revisit WM_NCCALCSIZE
     case QtWindows::CalculateSize:
-        // NCCALCSIZE_PARAMS structure if wParam==TRUE
-        if (wParam && QWindowsContext::verboseWindows) {
-            const NCCALCSIZE_PARAMS *ncp = reinterpret_cast<NCCALCSIZE_PARAMS *>(lParam);
-            qDebug() << platformWindow->window() << *ncp;
-        }
-        break;
-#endif
+        return QWindowsGeometryHint::handleCalculateSize(platformWindow->customMargins(), msg, result);
+    case QtWindows::NonClientHitTest:
+        return platformWindow->handleNonClientHitTest(QPoint(msg.pt.x, msg.pt.y), result);
+#endif // !Q_OS_WINCE
     case QtWindows::ExposeEvent:
         return platformWindow->handleWmPaint(hwnd, message, wParam, lParam);
     case QtWindows::NonClientMouseEvent:
         if (platformWindow->frameStrutEventsEnabled())
+#if !defined(Q_OS_WINCE) && !defined(QT_NO_SESSIONMANAGER)
+            return platformSessionManager()->isInteractionBlocked() ? true : d->m_mouseHandler.translateMouseEvent(platformWindow->window(), hwnd, et, msg, result);
+#else
             return d->m_mouseHandler.translateMouseEvent(platformWindow->window(), hwnd, et, msg, result);
+#endif
         break;
 /* the mouse tracking on windows already handles the reset of the cursor
  * and does not like somebody else handling it.
@@ -849,28 +893,45 @@ bool QWindowsContext::windowsProc(HWND hwnd, UINT message,
     case QtWindows::MouseWheelEvent:
     case QtWindows::MouseEvent:
     case QtWindows::LeaveEvent:
+#if !defined(Q_OS_WINCE) && !defined(QT_NO_SESSIONMANAGER)
+        return platformSessionManager()->isInteractionBlocked() ? true : d->m_mouseHandler.translateMouseEvent(platformWindow->window(), hwnd, et, msg, result);
+#else
         return d->m_mouseHandler.translateMouseEvent(platformWindow->window(), hwnd, et, msg, result);
+#endif
     case QtWindows::TouchEvent:
+#if !defined(Q_OS_WINCE) && !defined(QT_NO_SESSIONMANAGER)
+        return platformSessionManager()->isInteractionBlocked() ? true : d->m_mouseHandler.translateTouchEvent(platformWindow->window(), hwnd, et, msg, result);
+#else
         return d->m_mouseHandler.translateTouchEvent(platformWindow->window(), hwnd, et, msg, result);
+#endif
     case QtWindows::FocusInEvent: // see QWindowsWindow::requestActivateWindow().
     case QtWindows::FocusOutEvent:
         handleFocusEvent(et, platformWindow);
         return true;
-    case QtWindows::ShowEvent:
-        platformWindow->handleShown();
-        return true;
     case QtWindows::HideEvent:
         platformWindow->handleHidden();
-        return true;
+        return false;// Indicate transient children should be hidden by windows (SW_PARENTCLOSING)
     case QtWindows::CloseEvent:
         QWindowSystemInterface::handleCloseEvent(platformWindow->window());
         return true;
-    case QtWindows::ThemeChanged: // ### fixme: Compress these events?
+    case QtWindows::ThemeChanged: {
+        // Switch from Aero to Classic changes margins.
+        const Qt::WindowFlags flags = platformWindow->window()->flags();
+        if ((flags & Qt::WindowType_Mask) != Qt::Desktop && !(flags & Qt::FramelessWindowHint))
+            platformWindow->setFlag(QWindowsWindow::FrameDirty);
         if (QWindowsTheme *theme = QWindowsTheme::instance())
             theme->windowsThemeChanged(platformWindow->window());
         return true;
+    }
+    case QtWindows::CompositionSettingsChanged:
+        platformWindow->handleCompositionSettingsChanged();
+        return true;
 #ifndef Q_OS_WINCE
     case QtWindows::ActivateWindowEvent:
+#ifndef QT_NO_TABLETEVENT
+        if (!d->m_tabletSupport.isNull())
+            d->m_tabletSupport->notifyActivate();
+#endif // !QT_NO_TABLETEVENT
         if (platformWindow->testFlag(QWindowsWindow::BlockedByModal))
             if (const QWindow *modalWindow = QGuiApplication::modalWindow())
                 QWindowsWindow::baseWindowOf(modalWindow)->alertWindow();
@@ -880,6 +941,50 @@ bool QWindowsContext::windowsProc(HWND hwnd, UINT message,
     case QtWindows::ContextMenu:
         return handleContextMenuEvent(platformWindow->window(), msg);
 #endif
+    case QtWindows::WhatsThisEvent: {
+#ifndef QT_NO_WHATSTHIS
+        QWindowSystemInterface::handleEnterWhatsThisEvent();
+        return true;
+#endif
+    }   break;
+#if !defined(Q_OS_WINCE) && !defined(QT_NO_SESSIONMANAGER)
+    case QtWindows::QueryEndSessionApplicationEvent: {
+        QWindowsSessionManager *sessionManager = platformSessionManager();
+        if (sessionManager->isActive()) // bogus message from windows
+            return true;
+
+        sessionManager->setActive(true);
+        sessionManager->blocksInteraction();
+        sessionManager->clearCancellation();
+
+        QGuiApplicationPrivate *qGuiAppPriv = static_cast<QGuiApplicationPrivate*>(QObjectPrivate::get(qApp));
+        qGuiAppPriv->commitData();
+
+        if (lParam & ENDSESSION_LOGOFF)
+            fflush(NULL);
+
+        return !sessionManager->wasCanceled();
+    }
+    case QtWindows::EndSessionApplicationEvent: {
+        QWindowsSessionManager *sessionManager = platformSessionManager();
+
+        sessionManager->setActive(false);
+        sessionManager->allowsInteraction();
+        bool endsession = (bool) wParam;
+
+        // we receive the message for each toplevel window included internal hidden ones,
+        // but the aboutToQuit signal should be emitted only once.
+        QGuiApplicationPrivate *qGuiAppPriv = static_cast<QGuiApplicationPrivate*>(QObjectPrivate::get(qApp));
+        if (endsession && !qGuiAppPriv->aboutToQuitEmitted) {
+            qGuiAppPriv->aboutToQuitEmitted = true;
+            int index = QGuiApplication::staticMetaObject.indexOfSignal("aboutToQuit()");
+            qApp->qt_metacall(QMetaObject::InvokeMetaMethod, index,0);
+            // since the process will be killed immediately quit() has no real effect
+            QGuiApplication::quit();
+        }
+        return true;
+    }
+#endif // !defined(Q_OS_WINCE) && !defined(QT_NO_SESSIONMANAGER)
     default:
         break;
     }
