@@ -45,7 +45,7 @@
 #include "qwindowswindow.h"
 #include "qwindowsscreen.h"
 
-#include <QtGui/QPixmap>
+#include <QtGui/QBitmap>
 #include <QtGui/QImage>
 #include <QtGui/QBitmap>
 #include <QtGui/QGuiApplication>
@@ -60,6 +60,30 @@ QT_BEGIN_NAMESPACE
 Q_GUI_EXPORT HBITMAP qt_pixmapToWinHBITMAP(const QPixmap &p, int hbitmapFormat = 0);
 Q_GUI_EXPORT HBITMAP qt_createIconMask(const QBitmap &bitmap);
 
+/*!
+    \class QWindowsCursorCacheKey
+    \brief Cache key for storing values in a QHash with a QCursor as key.
+
+    \internal
+    \ingroup qt-lighthouse-win
+*/
+
+QWindowsCursorCacheKey::QWindowsCursorCacheKey(const QCursor &c)
+    : shape(c.shape()), bitmapCacheKey(0), maskCacheKey(0)
+{
+    if (shape == Qt::BitmapCursor) {
+        const qint64 pixmapCacheKey = c.pixmap().cacheKey();
+        if (pixmapCacheKey) {
+            bitmapCacheKey = pixmapCacheKey;
+        } else {
+            Q_ASSERT(c.bitmap());
+            Q_ASSERT(c.mask());
+            bitmapCacheKey = c.bitmap()->cacheKey();
+            maskCacheKey = c.mask()->cacheKey();
+        }
+    }
+}
+
 /*!
     \class QWindowsCursor
     \brief Platform cursor implementation
@@ -388,9 +412,23 @@ HCURSOR QWindowsCursor::createSystemCursor(const QCursor &c)
 
 QWindowsWindowCursor QWindowsCursor::standardWindowCursor(Qt::CursorShape shape)
 {
-    StandardCursorCache::iterator it = m_standardCursorCache.find(shape);
-    if (it == m_standardCursorCache.end())
-        it = m_standardCursorCache.insert(shape, QWindowsWindowCursor(QCursor(shape)));
+    const QWindowsCursorCacheKey key(shape);
+    CursorCache::iterator it = m_cursorCache.find(key);
+    if (it == m_cursorCache.end())
+        it = m_cursorCache.insert(key, QWindowsWindowCursor(QCursor(shape)));
+    return it.value();
+}
+
+/*!
+    \brief Return cached pixmap cursor or create new one.
+*/
+
+QWindowsWindowCursor QWindowsCursor::pixmapWindowCursor(const QCursor &c)
+{
+    const  QWindowsCursorCacheKey cacheKey(c);
+    CursorCache::iterator it = m_cursorCache.find(cacheKey);
+    if (it == m_cursorCache.end())
+        it = m_cursorCache.insert(cacheKey, QWindowsWindowCursor(c));
     return it.value();
 }
 
@@ -406,11 +444,15 @@ void QWindowsCursor::changeCursor(QCursor *cursorIn, QWindow *window)
 
     if (QWindowsContext::verboseWindows > 1)
         qDebug() << __FUNCTION__ <<  cursorIn << window;
-    if (!cursorIn || !window)
+    if (!window)
         return;
+    if (!cursorIn) {
+        QWindowsWindow::baseWindowOf(window)->setCursor(QWindowsWindowCursor());
+        return;
+    }
     const QWindowsWindowCursor wcursor =
         cursorIn->shape() == Qt::BitmapCursor ?
-        QWindowsWindowCursor(*cursorIn) : standardWindowCursor(cursorIn->shape());
+        pixmapWindowCursor(*cursorIn) : standardWindowCursor(cursorIn->shape());
     if (wcursor.handle()) {
         QWindowsWindow::baseWindowOf(window)->setCursor(wcursor);
     } else {
@@ -449,6 +491,7 @@ void QWindowsCursor::setPos(const QPoint &pos)
 class QWindowsWindowCursorData : public QSharedData
 {
 public:
+    QWindowsWindowCursorData() : m_cursor(Qt::ArrowCursor), m_handle(0) {}
     explicit QWindowsWindowCursorData(const QCursor &c);
     ~QWindowsWindowCursorData();
 
@@ -464,7 +507,13 @@ QWindowsWindowCursorData::QWindowsWindowCursorData(const QCursor &c) :
 
 QWindowsWindowCursorData::~QWindowsWindowCursorData()
 {
-    DestroyCursor(m_handle);
+    if (m_handle)
+        DestroyCursor(m_handle);
+}
+
+QWindowsWindowCursor::QWindowsWindowCursor() :
+    m_data(new QWindowsWindowCursorData)
+{
 }
 
 QWindowsWindowCursor::QWindowsWindowCursor(const QCursor &c) :
@@ -488,6 +537,11 @@ QWindowsWindowCursor & QWindowsWindowCursor::operator =(const QWindowsWindowCurs
     return *this;
 }
 
+bool QWindowsWindowCursor::isNull() const
+{
+    return m_data->m_handle == 0;
+}
+
 QCursor QWindowsWindowCursor::cursor() const
 {
     return m_data->m_cursor;
