@@ -52,14 +52,37 @@ QT_BEGIN_NAMESPACE
 
 class QWindowsWindowCursorData;
 
+struct QWindowsCursorCacheKey
+{
+    explicit QWindowsCursorCacheKey(const QCursor &c);
+    explicit QWindowsCursorCacheKey(Qt::CursorShape s) : shape(s), bitmapCacheKey(0), maskCacheKey(0) {}
+    QWindowsCursorCacheKey() : shape(Qt::CustomCursor), bitmapCacheKey(0), maskCacheKey(0) {}
+
+    Qt::CursorShape shape;
+    qint64 bitmapCacheKey;
+    qint64 maskCacheKey;
+};
+
+inline bool operator==(const QWindowsCursorCacheKey &k1, const QWindowsCursorCacheKey &k2)
+{
+    return k1.shape == k2.shape && k1.bitmapCacheKey == k2.bitmapCacheKey && k1.maskCacheKey == k2.maskCacheKey;
+}
+
+inline uint qHash(const QWindowsCursorCacheKey &k, uint seed) Q_DECL_NOTHROW
+{
+    return (uint(k.shape) + uint(k.bitmapCacheKey) + uint(k.maskCacheKey)) ^ seed;
+}
+
 class QWindowsWindowCursor
 {
 public:
+    QWindowsWindowCursor();
     explicit QWindowsWindowCursor(const QCursor &c);
     ~QWindowsWindowCursor();
     QWindowsWindowCursor(const QWindowsWindowCursor &c);
     QWindowsWindowCursor &operator=(const QWindowsWindowCursor &c);
 
+    bool isNull() const;
     QCursor cursor() const;
     HCURSOR handle() const;
 
@@ -81,11 +104,12 @@ public:
     static QPoint mousePosition();
 
     QWindowsWindowCursor standardWindowCursor(Qt::CursorShape s = Qt::ArrowCursor);
+    QWindowsWindowCursor pixmapWindowCursor(const QCursor &c);
 
 private:
-    typedef QHash<Qt::CursorShape, QWindowsWindowCursor> StandardCursorCache;
+    typedef QHash<QWindowsCursorCacheKey, QWindowsWindowCursor>  CursorCache;
 
-    StandardCursorCache m_standardCursorCache;
+    CursorCache m_cursorCache;
 };
 
 QT_END_NAMESPACE
