@@ -46,6 +46,7 @@
 #include <qpa/qplatformdialoghelper.h>
 #include <qpa/qplatformtheme.h>
 #include <QtCore/QStringList>
+#include <QtCore/QSharedPointer>
 
 QT_BEGIN_NAMESPACE
 
@@ -64,28 +65,33 @@ namespace QWindowsDialogs
 template <class BaseClass>
 class QWindowsDialogHelperBase : public BaseClass
 {
+    Q_DISABLE_COPY(QWindowsDialogHelperBase)
 public:
+    typedef QSharedPointer<QWindowsNativeDialogBase> QWindowsNativeDialogBasePtr;
 
     virtual void exec();
     virtual bool show(Qt::WindowFlags windowFlags,
                           Qt::WindowModality windowModality,
                           QWindow *parent);
     virtual void hide();
-    virtual QVariant styleHint(QPlatformDialogHelper::StyleHint) const;
 
-    virtual bool supportsNonModalDialog() const { return true; }
+    virtual bool supportsNonModalDialog(const QWindow * /* parent */ = 0) const { return true; }
 
 protected:
     QWindowsDialogHelperBase();
-    ~QWindowsDialogHelperBase();
     QWindowsNativeDialogBase *nativeDialog() const;
+    inline bool hasNativeDialog() const { return m_nativeDialog; }
+    void timerEvent(QTimerEvent *);
 
 private:
     virtual QWindowsNativeDialogBase *createNativeDialog() = 0;
     inline QWindowsNativeDialogBase *ensureNativeDialog();
+    inline void startDialogThread();
+    inline void stopTimer();
 
-    QWindowsNativeDialogBase *m_nativeDialog;
+    QWindowsNativeDialogBasePtr m_nativeDialog;
     HWND m_ownerWindow;
+    int m_timerId;
 };
 
 QT_END_NAMESPACE
