@@ -54,42 +54,12 @@
 #include <QtGui/QFontDatabase>
 
 #include <wchar.h>
+#ifdef Q_OS_WINCE
+#include <QtEndian>
+#endif
 
 QT_BEGIN_NAMESPACE
 
-static inline QFontDatabase::WritingSystem writingSystemFromScript(const QString &scriptName)
-{
-    if (scriptName == QStringLiteral("Western")
-            || scriptName == QStringLiteral("Baltic")
-            || scriptName == QStringLiteral("Central European")
-            || scriptName == QStringLiteral("Turkish")
-            || scriptName == QStringLiteral("Vietnamese")
-            || scriptName == QStringLiteral("OEM/Dos"))
-        return QFontDatabase::Latin;
-    if (scriptName == QStringLiteral("Thai"))
-        return QFontDatabase::Thai;
-    if (scriptName == QStringLiteral("Symbol")
-        || scriptName == QStringLiteral("Other"))
-        return QFontDatabase::Symbol;
-    if (scriptName == QStringLiteral("CHINESE_GB2312"))
-        return QFontDatabase::SimplifiedChinese;
-    if (scriptName == QStringLiteral("CHINESE_BIG5"))
-        return QFontDatabase::TraditionalChinese;
-    if (scriptName == QStringLiteral("Cyrillic"))
-        return QFontDatabase::Cyrillic;
-    if (scriptName == QStringLiteral("Hangul"))
-        return QFontDatabase::Korean;
-    if (scriptName == QStringLiteral("Hebrew"))
-        return QFontDatabase::Hebrew;
-    if (scriptName == QStringLiteral("Greek"))
-        return QFontDatabase::Greek;
-    if (scriptName == QStringLiteral("Japanese"))
-        return QFontDatabase::Japanese;
-    if (scriptName == QStringLiteral("Arabic"))
-        return QFontDatabase::Arabic;
-    return QFontDatabase::Any;
-}
-
 // convert 0 ~ 1000 integer to QFont::Weight
 static inline QFont::Weight weightFromInteger(long weight)
 {
@@ -104,6 +74,43 @@ static inline QFont::Weight weightFromInteger(long weight)
     return QFont::Black;
 }
 
+static inline QFontDatabase::WritingSystem writingSystemFromCharSet(uchar charSet)
+{
+    switch (charSet) {
+    case ANSI_CHARSET:
+    case EASTEUROPE_CHARSET:
+    case BALTIC_CHARSET:
+    case TURKISH_CHARSET:
+        return QFontDatabase::Latin;
+    case GREEK_CHARSET:
+        return QFontDatabase::Greek;
+    case RUSSIAN_CHARSET:
+        return QFontDatabase::Cyrillic;
+    case HEBREW_CHARSET:
+        return QFontDatabase::Hebrew;
+    case ARABIC_CHARSET:
+        return QFontDatabase::Arabic;
+    case THAI_CHARSET:
+        return QFontDatabase::Thai;
+    case GB2312_CHARSET:
+        return QFontDatabase::SimplifiedChinese;
+    case CHINESEBIG5_CHARSET:
+        return QFontDatabase::TraditionalChinese;
+    case SHIFTJIS_CHARSET:
+        return QFontDatabase::Japanese;
+    case HANGUL_CHARSET:
+    case JOHAB_CHARSET:
+        return QFontDatabase::Korean;
+    case VIETNAMESE_CHARSET:
+        return QFontDatabase::Vietnamese;
+    case SYMBOL_CHARSET:
+        return QFontDatabase::Symbol;
+    default:
+        break;
+    }
+    return QFontDatabase::Any;
+}
+
 static FontFile * createFontFile(const QString &fileName, int index)
 {
     FontFile *fontFile = new FontFile;
@@ -115,9 +122,7 @@ static FontFile * createFontFile(const QString &fileName, int index)
 extern bool localizedName(const QString &name);
 extern QString getEnglishName(const QString &familyName);
 
-Q_GUI_EXPORT void qt_registerAliasToFontFamily(const QString &familyName, const QString &alias);
-
-static bool addFontToDatabase(QString familyName, const QString &scriptName,
+static bool addFontToDatabase(const QString &familyName, uchar charSet,
                               const TEXTMETRIC *textmetric,
                               const FONTSIGNATURE *signature,
                               int type)
@@ -148,7 +153,7 @@ static bool addFontToDatabase(QString familyName, const QString &scriptName,
 #ifndef QT_NO_DEBUG_OUTPUT
     if (QWindowsContext::verboseFonts > 2) {
         QDebug nospace = qDebug().nospace();
-        nospace << __FUNCTION__ << faceName << fullName << scriptName
+        nospace << __FUNCTION__ << familyName << faceName << fullName << charSet
                  << "TTF=" << ttf;
         if (type & DEVICE_FONTTYPE)
             nospace << " DEVICE";
@@ -168,6 +173,7 @@ static bool addFontToDatabase(QString familyName, const QString &scriptName,
 
     QSupportedWritingSystems writingSystems;
     if (type & TRUETYPE_FONTTYPE) {
+        Q_ASSERT(signature);
         quint32 unicodeRange[4] = {
             signature->fsUsb[0], signature->fsUsb[1],
             signature->fsUsb[2], signature->fsUsb[3]
@@ -175,7 +181,7 @@ static bool addFontToDatabase(QString familyName, const QString &scriptName,
         quint32 codePageRange[2] = {
             signature->fsCsb[0], signature->fsCsb[1]
         };
-        writingSystems = QBasicFontDatabase::determineWritingSystemsFromTrueTypeBits(unicodeRange, codePageRange);
+        writingSystems = QPlatformFontDatabase::writingSystemsFromTrueTypeBits(unicodeRange, codePageRange);
         // ### Hack to work around problem with Thai text on Windows 7. Segoe UI contains
         // the symbol for Baht, and Windows thus reports that it supports the Thai script.
         // Since it's the default UI font on this platform, most widgets will be unable to
@@ -185,7 +191,7 @@ static bool addFontToDatabase(QString familyName, const QString &scriptName,
                 faceName == QStringLiteral("Segoe UI"))
             writingSystems.setSupported(QFontDatabase::Thai, false);
     } else {
-        const QFontDatabase::WritingSystem ws = writingSystemFromScript(scriptName);
+        const QFontDatabase::WritingSystem ws = writingSystemFromCharSet(charSet);
         if (ws != QFontDatabase::Any)
             writingSystems.setSupported(ws);
     }
@@ -265,6 +271,12 @@ static bool addFontToDatabase(QString familyName, const QString &scriptName,
                 continue;
             fontCache.insert(fontName, fontFile);
             settings.setValue(fontName, fontFile);
+
+            if (localizedName(fontName)) {
+                QString englishFontName = getEnglishName(fontName);
+                fontCache.insert(englishFontName, fontFile);
+                settings.setValue(englishFontName, fontFile);
+            }
         }
 
         value = fontCache.value(faceName);
@@ -277,7 +289,11 @@ static bool addFontToDatabase(QString familyName, const QString &scriptName,
         return false;
 
     if (!QDir::isAbsolutePath(value))
-        value.prepend(QString::fromLocal8Bit(qgetenv("windir") + "\\Fonts\\"));
+#ifndef Q_OS_WINCE
+        value.prepend(QFile::decodeName(qgetenv("windir") + "\\Fonts\\"));
+#else
+        value.prepend(QFile::decodeName("/Windows/"));
+#endif
 
     QPlatformFontDatabase::registerFont(faceName, QString(), foundryName, weight, style, stretch,
         antialias, scalable, size, fixed, writingSystems, createFontFile(value, index));
@@ -296,11 +312,29 @@ static bool addFontToDatabase(QString familyName, const QString &scriptName,
                                             antialias, scalable, size, fixed, writingSystems, createFontFile(value, index));
 
     if (!englishName.isEmpty())
-        qt_registerAliasToFontFamily(faceName, englishName);
+        QPlatformFontDatabase::registerAliasToFontFamily(faceName, englishName);
 
     return true;
 }
 
+#ifdef Q_OS_WINCE
+static QByteArray getFntTable(HFONT hfont, uint tag)
+{
+    HDC hdc = GetDC(0);
+    HGDIOBJ oldFont = SelectObject(hdc, hfont);
+    quint32 t = qFromBigEndian<quint32>(tag);
+    QByteArray buffer;
+
+    DWORD length = GetFontData(hdc, t, 0, NULL, 0);
+    if (length != GDI_ERROR) {
+        buffer.resize(length);
+        GetFontData(hdc, t, 0, reinterpret_cast<uchar *>(buffer.data()), length);
+    }
+    SelectObject(hdc, oldFont);
+    return buffer;
+}
+#endif
+
 static int CALLBACK storeFont(ENUMLOGFONTEX* f, NEWTEXTMETRICEX *textmetric,
                               int type, LPARAM namesSetIn)
 {
@@ -308,14 +342,40 @@ static int CALLBACK storeFont(ENUMLOGFONTEX* f, NEWTEXTMETRICEX *textmetric,
     const QString familyName = QString::fromWCharArray(f->elfLogFont.lfFaceName)
                                + QStringLiteral("::")
                                + QString::fromWCharArray(f->elfFullName);
-    const QString script = QString::fromWCharArray(f->elfScript);
+    const uchar charSet = f->elfLogFont.lfCharSet;
 
+#ifndef Q_OS_WINCE
     const FONTSIGNATURE signature = textmetric->ntmFontSig;
+#else
+    FONTSIGNATURE signature;
+    QByteArray table;
+
+    if (type & TRUETYPE_FONTTYPE) {
+        HFONT hfont = CreateFontIndirect(&f->elfLogFont);
+        table = getFntTable(hfont, MAKE_TAG('O', 'S', '/', '2'));
+        DeleteObject((HGDIOBJ)hfont);
+    }
+
+    if (table.length() >= 86) {
+        // See also qfontdatabase_mac.cpp, offsets taken from OS/2 table in the TrueType spec
+        uchar *tableData = reinterpret_cast<uchar *>(table.data());
+
+        signature.fsUsb[0] = qFromBigEndian<quint32>(tableData + 42);
+        signature.fsUsb[1] = qFromBigEndian<quint32>(tableData + 46);
+        signature.fsUsb[2] = qFromBigEndian<quint32>(tableData + 50);
+        signature.fsUsb[3] = qFromBigEndian<quint32>(tableData + 54);
+
+        signature.fsCsb[0] = qFromBigEndian<quint32>(tableData + 78);
+        signature.fsCsb[1] = qFromBigEndian<quint32>(tableData + 82);
+    } else {
+        memset(&signature, 0, sizeof(signature));
+    }
+#endif
 
     // NEWTEXTMETRICEX is a NEWTEXTMETRIC, which according to the documentation is
     // identical to a TEXTMETRIC except for the last four members, which we don't use
     // anyway
-    if (addFontToDatabase(familyName, script, (TEXTMETRIC *)textmetric, &signature, type))
+    if (addFontToDatabase(familyName, charSet, (TEXTMETRIC *)textmetric, &signature, type))
         reinterpret_cast<StringSet *>(namesSetIn)->insert(familyName);
 
     // keep on enumerating
@@ -362,7 +422,7 @@ void QWindowsFontDatabaseFT::populate(const QString &family)
     ReleaseDC(0, dummy);
 }
 
-QFontEngine * QWindowsFontDatabaseFT::fontEngine(const QFontDef &fontDef, QUnicodeTables::Script script, void *handle)
+QFontEngine * QWindowsFontDatabaseFT::fontEngine(const QFontDef &fontDef, QChar::Script script, void *handle)
 {
     QFontEngine *fe = QBasicFontDatabase::fontEngine(fontDef, script, handle);
     if (QWindowsContext::verboseFonts)
@@ -430,9 +490,9 @@ static const char *kr_tryFonts[] = {
 
 static const char **tryFonts = 0;
 
-QStringList QWindowsFontDatabaseFT::fallbacksForFamily(const QString family, const QFont::Style &style, const QFont::StyleHint &styleHint, const QUnicodeTables::Script &script) const
+QStringList QWindowsFontDatabaseFT::fallbacksForFamily(const QString &family, QFont::Style style, QFont::StyleHint styleHint, QChar::Script script) const
 {
-    if(script == QUnicodeTables::Common) {
+    if (script == QChar::Script_Common || script == QChar::Script_Han) {
 //            && !(request.styleStrategy & QFont::NoFontMerging)) {
         QFontDatabase db;
         if (!db.writingSystems(family).contains(QFontDatabase::Symbol)) {
@@ -495,6 +555,17 @@ QStringList QWindowsFontDatabaseFT::fallbacksForFamily(const QString family, con
         default:
             result << QString::fromLatin1("Arial");
     }
+
+#ifdef Q_OS_WINCE
+    QSettings settings(QLatin1String("HKEY_LOCAL_MACHINE\\SOFTWARE\\Microsoft\\FontLink\\SystemLink"), QSettings::NativeFormat);
+    const QStringList fontList = settings.value(family).toStringList();
+    foreach (const QString &fallback, fontList) {
+        const int sep = fallback.indexOf(QLatin1Char(','));
+        if (sep > 0)
+            result << fallback.mid(sep + 1);
+    }
+#endif
+
     if (QWindowsContext::verboseFonts)
         qDebug() << __FUNCTION__ << family << style << styleHint
                  << script << result << m_families;
@@ -516,12 +587,6 @@ HFONT QWindowsFontDatabaseFT::systemFont()
 
 // Creation functions
 
-static inline bool scriptRequiresOpenType(int script)
-{
-    return ((script >= QUnicodeTables::Syriac && script <= QUnicodeTables::Sinhala)
-            || script == QUnicodeTables::Khmer || script == QUnicodeTables::Nko);
-}
-
 static inline int verticalDPI()
 {
     return GetDeviceCaps(QWindowsContext::instance()->displayContext(), LOGPIXELSY);
