@@ -66,7 +66,6 @@
 #include <QtCore/QThreadStorage>
 #include <QtCore/private/qsystemlibrary_p.h>
 
-#include <QtCore/private/qunicodetables_p.h>
 #include <QtCore/QDebug>
 
 #include <limits.h>
@@ -155,9 +154,26 @@ static OUTLINETEXTMETRIC *getOutlineTextMetric(HDC hdc)
     return otm;
 }
 
+bool QWindowsFontEngine::hasCFFTable() const
+{
+    HDC hdc = m_fontEngineData->hdc;
+    SelectObject(hdc, hfont);
+    return GetFontData(hdc, MAKE_TAG('C', 'F', 'F', ' '), 0, 0, 0) != GDI_ERROR;
+}
+
+bool QWindowsFontEngine::hasCMapTable() const
+{
+    HDC hdc = m_fontEngineData->hdc;
+    SelectObject(hdc, hfont);
+    return GetFontData(hdc, MAKE_TAG('c', 'm', 'a', 'p'), 0, 0, 0) != GDI_ERROR;
+}
+
 void QWindowsFontEngine::getCMap()
 {
-    ttf = (bool)(tm.tmPitchAndFamily & TMPF_TRUETYPE);
+    ttf = (bool)(tm.tmPitchAndFamily & TMPF_TRUETYPE) || hasCMapTable();
+
+    cffTable = hasCFFTable();
+
     HDC hdc = m_fontEngineData->hdc;
     SelectObject(hdc, hfont);
     bool symb = false;
@@ -180,7 +196,7 @@ void QWindowsFontEngine::getCMap()
         unitsPerEm = otm->otmEMSquare;
         x_height = (int)otm->otmsXHeight;
         loadKerningPairs(designToDevice);
-        _faceId.filename = QString::fromWCharArray((wchar_t *)((char *)otm + (quintptr)otm->otmpFullName)).toLatin1();
+        _faceId.filename = QFile::encodeName(QString::fromWCharArray((wchar_t *)((char *)otm + (quintptr)otm->otmpFullName)));
         lineWidth = otm->otmsUnderscoreSize;
         fsType = otm->otmfsType;
         free(otm);
@@ -326,6 +342,13 @@ QWindowsFontEngine::QWindowsFontEngine(const QString &name,
 
     if (!resolvedGetCharWidthI)
         resolveGetCharWidthI();
+
+    // ### Properties accessed by QWin32PrintEngine (QtPrintSupport)
+    QVariantMap userData;
+    userData.insert(QStringLiteral("logFont"), QVariant::fromValue(m_logfont));
+    userData.insert(QStringLiteral("hFont"), QVariant::fromValue(hfont));
+    userData.insert(QStringLiteral("trueType"), QVariant(bool(ttf)));
+    setUserData(userData);
 }
 
 QWindowsFontEngine::~QWindowsFontEngine()
@@ -357,6 +380,7 @@ HGDIOBJ QWindowsFontEngine::selectDesignFont() const
 {
     LOGFONT f = m_logfont;
     f.lfHeight = unitsPerEm;
+    f.lfWidth = 0;
     HFONT designFont = CreateFontIndirect(&f);
     return SelectObject(m_fontEngineData->hdc, designFont);
 }
@@ -1041,7 +1065,7 @@ void QWindowsFontEngine::getUnscaledGlyph(glyph_t glyph, QPainterPath *path, gly
 
 bool QWindowsFontEngine::getSfntTableData(uint tag, uchar *buffer, uint *length) const
 {
-    if (!ttf)
+    if (!ttf && !cffTable)
         return false;
     HDC hdc = m_fontEngineData->hdc;
     SelectObject(hdc, hfont);
@@ -1085,7 +1109,7 @@ QWindowsNativeImage *QWindowsFontEngine::drawGDIGlyph(HFONT font, glyph_t glyph,
         xform.eDx = margin;
         xform.eDy = margin;
 
-        HDC hdc = CreateCompatibleDC(QWindowsContext::instance()->displayContext());
+        const HDC hdc = m_fontEngineData->hdc;
 
         SetGraphicsMode(hdc, GM_ADVANCED);
         SetWorldTransform(hdc, &xform);
@@ -1097,8 +1121,16 @@ QWindowsNativeImage *QWindowsFontEngine::drawGDIGlyph(HFONT font, glyph_t glyph,
         memset(&mat, 0, sizeof(mat));
         mat.eM11.value = mat.eM22.value = 1;
 
-        if (GetGlyphOutline(hdc, glyph, ggo_options, &tgm, 0, 0, &mat) == GDI_ERROR) {
-            qWarning("QWinFontEngine: unable to query transformed glyph metrics...");
+        const DWORD result = GetGlyphOutline(hdc, glyph, ggo_options, &tgm, 0, 0, &mat);
+
+        XFORM identity = {1, 0, 0, 1, 0, 0};
+        SetWorldTransform(hdc, &identity);
+        SetGraphicsMode(hdc, GM_COMPATIBLE);
+        SelectObject(hdc, old_font);
+
+        if (result == GDI_ERROR) {
+            const int errorCode = GetLastError();
+            qErrnoWarning(errorCode, "QWinFontEngine: unable to query transformed glyph metrics (GetGlyphOutline() failed, error %d)...", errorCode);
             return 0;
         }
 
@@ -1107,21 +1139,18 @@ QWindowsNativeImage *QWindowsFontEngine::drawGDIGlyph(HFONT font, glyph_t glyph,
 
         xform.eDx -= tgm.gmptGlyphOrigin.x;
         xform.eDy += tgm.gmptGlyphOrigin.y;
-
-        SetGraphicsMode(hdc, GM_COMPATIBLE);
-        SelectObject(hdc, old_font);
-        DeleteDC(hdc);
     }
 #else // else wince
     unsigned int options = 0;
-#ifdef DEBUG
-    Q_ASSERT(!has_transformation);
-#else
-    Q_UNUSED(has_transformation);
-#endif
+    if (has_transformation) {
+        qWarning() << "QWindowsFontEngine is unable to apply transformations other than translations for fonts on Windows CE."
+                   << "If you need them anyway, start your application with -platform windows:fontengine=freetype.";
+   }
 #endif // wince
-    QWindowsNativeImage *ni = new QWindowsNativeImage(iw + 2 * margin + 4,
-                                                      ih + 2 * margin + 4,
+
+    // The padding here needs to be kept in sync with the values in alphaMapBoundingBox.
+    QWindowsNativeImage *ni = new QWindowsNativeImage(iw + 2 * margin,
+                                                      ih + 2 * margin,
                                                       QWindowsNativeImage::systemFormat());
 
     /*If cleartype is enabled we use the standard system format even on Windows CE
@@ -1154,6 +1183,17 @@ QWindowsNativeImage *QWindowsFontEngine::drawGDIGlyph(HFONT font, glyph_t glyph,
     return ni;
 }
 
+glyph_metrics_t QWindowsFontEngine::alphaMapBoundingBox(glyph_t glyph, QFixed, const QTransform &matrix, GlyphFormat format)
+{
+    int margin = 0;
+    if (format == QFontEngine::Format_A32 || format == QFontEngine::Format_ARGB)
+        margin = glyphMargin(QFontEngineGlyphCache::Raster_RGBMask);
+    glyph_metrics_t gm = boundingBox(glyph, matrix);
+    gm.width += margin * 2;
+    gm.height += margin * 2;
+    return gm;
+}
+
 QImage QWindowsFontEngine::alphaMapForGlyph(glyph_t glyph, const QTransform &xform)
 {
     HFONT font = hfont;
@@ -1216,13 +1256,13 @@ QImage QWindowsFontEngine::alphaRGBMapForGlyph(glyph_t glyph, QFixed, const QTra
 {
     HFONT font = hfont;
 
-    int contrast;
+    UINT contrast;
     SystemParametersInfo(SPI_GETFONTSMOOTHINGCONTRAST, 0, &contrast, 0);
     SystemParametersInfo(SPI_SETFONTSMOOTHINGCONTRAST, 0, (void *) 1000, 0);
 
     int margin = glyphMargin(QFontEngineGlyphCache::Raster_RGBMask);
     QWindowsNativeImage *mask = drawGDIGlyph(font, glyph, margin, t, QImage::Format_RGB32);
-    SystemParametersInfo(SPI_SETFONTSMOOTHINGCONTRAST, 0, (void *) contrast, 0);
+    SystemParametersInfo(SPI_SETFONTSMOOTHINGCONTRAST, 0, (void *) quintptr(contrast), 0);
 
     if (mask == 0)
         return QImage();
@@ -1258,7 +1298,7 @@ QFontEngine *QWindowsFontEngine::cloneWithSize(qreal pixelSize) const
     request.styleStrategy |= QFont::NoFontMerging;
 
     QFontEngine *fontEngine =
-        QWindowsFontDatabase::createEngine(QUnicodeTables::Common, request, 0,
+        QWindowsFontDatabase::createEngine(QChar::Script_Common, request, 0,
                                            QWindowsContext::instance()->defaultDPI(),
                                            false,
                                            QStringList(), m_fontEngineData);
@@ -1340,7 +1380,7 @@ void QWindowsMultiFontEngine::loadEngine(int at)
 #endif
     {
         QWindowsFontEngine *fe = static_cast<QWindowsFontEngine*>(fontEngine);
-        lf = fe->logFont();
+        lf = fe->m_logfont;
 
         data = fe->fontEngineData();
     }
@@ -1365,8 +1405,6 @@ void QWindowsMultiFontEngine::loadEngine(int at)
                 QWindowsFontEngineDirectWrite *fedw = new QWindowsFontEngineDirectWrite(directWriteFontFace,
                                                                                         fontEngine->fontDef.pixelSize,
                                                                                         data);
-                fedw->setObjectName(QStringLiteral("QWindowsFontEngineDirectWrite_") + fontEngine->fontDef.family);
-
                 fedw->fontDef = fontDef;
                 fedw->ref.ref();
                 engines[at] = fedw;
@@ -1402,5 +1440,11 @@ void QWindowsMultiFontEngine::loadEngine(int at)
     // TODO: increase cost in QFontCache for the font engine loaded here
 }
 
+bool QWindowsFontEngine::supportsTransformation(const QTransform &transform) const
+{
+    // Support all transformations for ttf files, and translations for raster fonts
+    return ttf || transform.type() <= QTransform::TxTranslate;
+}
+
 QT_END_NAMESPACE
 
