@@ -68,10 +68,7 @@ class QWindowsFontEngineData;
 
 class QWindowsFontEngine : public QFontEngine
 {
-    Q_OBJECT
-    Q_PROPERTY(HFONT hFont READ hFont STORED false)
-    Q_PROPERTY(LOGFONT logFont READ logFont STORED false)
-    Q_PROPERTY(bool trueType READ trueType STORED false)
+    friend class QWindowsMultiFontEngine;
 
 public:
     QWindowsFontEngine(const QString &name, HFONT, bool, LOGFONT,
@@ -121,8 +118,10 @@ public:
     virtual QImage alphaMapForGlyph(glyph_t t) { return alphaMapForGlyph(t, QTransform()); }
     virtual QImage alphaMapForGlyph(glyph_t, const QTransform &xform);
     virtual QImage alphaRGBMapForGlyph(glyph_t t, QFixed subPixelPosition, const QTransform &xform);
+    virtual glyph_metrics_t alphaMapBoundingBox(glyph_t glyph, QFixed, const QTransform &matrix, GlyphFormat);
 
     virtual QFontEngine *cloneWithSize(qreal pixelSize) const;
+    virtual bool supportsTransformation(const QTransform &transform) const;
 
 #ifndef Q_CC_MINGW
     virtual void getGlyphBearings(glyph_t glyph, qreal *leftBearing = 0, qreal *rightBearing = 0);
@@ -135,16 +134,13 @@ public:
 
     const QSharedPointer<QWindowsFontEngineData> &fontEngineData() const { return m_fontEngineData; }
 
-    // Properties accessed by QWin32PrintEngine (Qt Print Support)
-    LOGFONT logFont() const { return m_logfont; }
-    HFONT hFont() const     { return hfont; }
-    bool trueType() const   { return ttf; }
-
     void setUniqueFamilyName(const QString &newName) { uniqueFamilyName = newName; }
 
 private:
     QWindowsNativeImage *drawGDIGlyph(HFONT font, glyph_t, int margin, const QTransform &xform,
                                       QImage::Format mask_format);
+    bool hasCFFTable() const;
+    bool hasCMapTable() const;
 
     const QSharedPointer<QWindowsFontEngineData> m_fontEngineData;
 
@@ -155,6 +151,7 @@ private:
     uint        stockFont  : 1;
     uint        ttf        : 1;
     uint        hasOutline : 1;
+    uint        cffTable   : 1;
     TEXTMETRIC  tm;
     int         lw;
     const unsigned char *cmap;
