@@ -42,6 +42,8 @@
 #ifndef QWINDOWSFONTENGINEDIRECTWRITE_H
 #define QWINDOWSFONTENGINEDIRECTWRITE_H
 
+#include <QtCore/qglobal.h>
+
 #ifndef QT_NO_DIRECTWRITE
 
 #include <QtGui/private/qfontengine_p.h>
@@ -59,7 +61,6 @@ QT_BEGIN_NAMESPACE
 
 class QWindowsFontEngineDirectWrite : public QFontEngine
 {
-    Q_OBJECT
 public:
     explicit QWindowsFontEngineDirectWrite(IDWriteFontFace *directWriteFontFace,
                                     qreal pixelSize,
@@ -105,8 +106,6 @@ public:
     static QString fontNameSubstitute(const QString &familyName);
 
 private:
-    friend class QRawFontPrivate;
-
     QImage imageForGlyph(glyph_t t, QFixed subPixelPosition, int margin, const QTransform &xform);
     void collectMetrics();
 
