@@ -49,6 +49,8 @@
 #include <QtGui/QGuiApplication>
 #include <qpa/qplatformnativeinterface.h>
 
+#include <algorithm>
+
 #include <wingdi.h>
 #include <GL/gl.h>
 
@@ -232,6 +234,7 @@ static QSurfaceFormat
                                          QWindowsOpenGLAdditionalFormat *additionalIn = 0)
 {
     QSurfaceFormat format;
+    format.setRenderableType(QSurfaceFormat::OpenGL);
     if (pfd.dwFlags & PFD_DOUBLEBUFFER)
         format.setSwapBehavior(QSurfaceFormat::DoubleBuffer);
     format.setDepthBufferSize(pfd.cDepthBits);
@@ -278,7 +281,7 @@ static PIXELFORMATDESCRIPTOR
 
     if (format.stereo())
         pfd.dwFlags |= PFD_STEREO;
-    if (format.swapBehavior() == QSurfaceFormat::DoubleBuffer && !isPixmap)
+    if (format.swapBehavior() != QSurfaceFormat::SingleBuffer && !isPixmap)
         pfd.dwFlags |= PFD_DOUBLEBUFFER;
     pfd.cDepthBits =
         format.depthBufferSize() >= 0 ? format.depthBufferSize() : 32;
@@ -376,7 +379,7 @@ static int choosePixelFormat(HDC hdc,
         return 0;
 
     int iAttributes[attribSize];
-    qFill(iAttributes, iAttributes + attribSize, int(0));
+    std::fill(iAttributes, iAttributes + attribSize, int(0));
     int i = 0;
     iAttributes[i++] = WGL_ACCELERATION_ARB;
     iAttributes[i++] = testFlag(additional.formatFlags, QWindowsGLDirectRendering) ?
@@ -388,12 +391,11 @@ static int choosePixelFormat(HDC hdc,
     iAttributes[i++] = WGL_COLOR_BITS_ARB;
     iAttributes[i++] = 24;
     switch (format.swapBehavior()) {
-    case QSurfaceFormat::DefaultSwapBehavior:
-        break;
     case QSurfaceFormat::SingleBuffer:
         iAttributes[i++] = WGL_DOUBLE_BUFFER_ARB;
         iAttributes[i++] = FALSE;
         break;
+    case QSurfaceFormat::DefaultSwapBehavior:
     case QSurfaceFormat::DoubleBuffer:
     case QSurfaceFormat::TripleBuffer:
         iAttributes[i++] = WGL_DOUBLE_BUFFER_ARB;
@@ -500,12 +502,13 @@ static QSurfaceFormat
     enum { attribSize =40 };
 
     QSurfaceFormat result;
+    result.setRenderableType(QSurfaceFormat::OpenGL);
     if (!staticContext.hasExtensions())
         return result;
     int iAttributes[attribSize];
     int iValues[attribSize];
-    qFill(iAttributes, iAttributes + attribSize, int(0));
-    qFill(iValues, iValues + attribSize, int(0));
+    std::fill(iAttributes, iAttributes + attribSize, int(0));
+    std::fill(iValues, iValues + attribSize, int(0));
 
     int i = 0;
     const bool hasSampleBuffers = testFlag(staticContext.extensions, QOpenGLStaticContext::SampleBuffers);
@@ -566,7 +569,7 @@ static HGLRC createContext(const QOpenGLStaticContext &staticContext,
         return 0;
     int attributes[attribSize];
     int attribIndex = 0;
-    qFill(attributes, attributes + attribSize, int(0));
+    std::fill(attributes, attributes + attribSize, int(0));
 
     // We limit the requested version by the version of the static context as
     // wglCreateContextAttribsARB fails and returns NULL if the requested context
@@ -583,15 +586,17 @@ static HGLRC createContext(const QOpenGLStaticContext &staticContext,
         attributes[attribIndex++] = WGL_CONTEXT_MINOR_VERSION_ARB;
         attributes[attribIndex++] = minorVersion;
     }
+
+    int flags = 0;
+    if (format.testOption(QSurfaceFormat::DebugContext))
+        flags |= WGL_CONTEXT_DEBUG_BIT_ARB;
     if (requestedVersion >= 0x0300) {
-        attributes[attribIndex++] = WGL_CONTEXT_FLAGS_ARB;
-        attributes[attribIndex] = 0;
-        if (format.testOption(QSurfaceFormat::DeprecatedFunctions))
-             attributes[attribIndex] |= WGL_CONTEXT_FORWARD_COMPATIBLE_BIT_ARB;
-        if (format.testOption(QSurfaceFormat::DebugContext))
-            attributes[attribIndex] |= WGL_CONTEXT_DEBUG_BIT_ARB;
-        attribIndex++;
+        if (!format.testOption(QSurfaceFormat::DeprecatedFunctions))
+            flags |= WGL_CONTEXT_FORWARD_COMPATIBLE_BIT_ARB;
     }
+    attributes[attribIndex++] = WGL_CONTEXT_FLAGS_ARB;
+    attributes[attribIndex++] = flags;
+
     if (requestedVersion >= 0x0302) {
         switch (format.profile()) {
         case QSurfaceFormat::NoProfile:
@@ -612,8 +617,12 @@ static HGLRC createContext(const QOpenGLStaticContext &staticContext,
 
     const HGLRC result =
         staticContext.wglCreateContextAttribsARB(hdc, shared, attributes);
-    if (!result)
-        qErrnoWarning("%s: wglCreateContextAttribsARB() failed.", __FUNCTION__);
+    if (!result) {
+        QString message;
+        QDebug(&message).nospace() << __FUNCTION__ << ": wglCreateContextAttribsARB() failed (GL error code: 0x"
+            << hex << glGetError() << dec << ") for format: " << format << ", shared context: " << shared;
+        qErrnoWarning("%s", qPrintable(message));
+    }
     return result;
 }
 
@@ -875,6 +884,12 @@ QWindowsGLContext::QWindowsGLContext(const QOpenGLStaticContextPtr &staticContex
     m_renderingContext(0),
     m_pixelFormat(0), m_extensionsUsed(false)
 {
+    QSurfaceFormat format = context->format();
+    if (format.renderableType() == QSurfaceFormat::DefaultRenderableType)
+        format.setRenderableType(QSurfaceFormat::OpenGL);
+    if (format.renderableType() != QSurfaceFormat::OpenGL)
+        return;
+
     // workaround for matrox driver:
     // make a cheap call to opengl to force loading of DLL
     static bool opengl32dll = false;
@@ -912,7 +927,7 @@ QWindowsGLContext::QWindowsGLContext(const QOpenGLStaticContextPtr &staticContex
         QWindowsOpenGLAdditionalFormat obtainedAdditional;
         if (tryExtensions) {
             m_pixelFormat =
-                ARB::choosePixelFormat(hdc, *m_staticContext, context->format(),
+                ARB::choosePixelFormat(hdc, *m_staticContext, format,
                                        requestedAdditional, &m_obtainedPixelFormatDescriptor);
             if (m_pixelFormat > 0) {
                 m_obtainedFormat =
@@ -922,7 +937,7 @@ QWindowsGLContext::QWindowsGLContext(const QOpenGLStaticContextPtr &staticContex
             }
         } // tryExtensions
         if (!m_pixelFormat) { // Failed, try GDI
-            m_pixelFormat = GDI::choosePixelFormat(hdc, context->format(), requestedAdditional,
+            m_pixelFormat = GDI::choosePixelFormat(hdc, format, requestedAdditional,
                                                    &m_obtainedPixelFormatDescriptor);
             if (m_pixelFormat)
                 m_obtainedFormat =
@@ -945,7 +960,7 @@ QWindowsGLContext::QWindowsGLContext(const QOpenGLStaticContextPtr &staticContex
         if (m_extensionsUsed)
             m_renderingContext =
                 ARB::createContext(*m_staticContext, hdc,
-                                   context->format(),
+                                   format,
                                    requestedAdditional,
                                    sharingRenderingContext);
         if (!m_renderingContext)
@@ -1045,8 +1060,16 @@ bool QWindowsGLContext::makeCurrent(QPlatformSurface *surface)
     // Do we already have a DC entry for that window?
     QWindowsWindow *window = static_cast<QWindowsWindow *>(surface);
     const HWND hwnd = window->handle();
-    if (const QOpenGLContextData *contextData = findByHWND(m_windowContexts, hwnd))
+    if (const QOpenGLContextData *contextData = findByHWND(m_windowContexts, hwnd)) {
+        // Repeated calls to wglMakeCurrent when vsync is enabled in the driver will
+        // often result in 100% cpuload. This check is cheap and avoids the problem.
+        // This is reproducable on NVidia cards and Intel onboard chips.
+        if (wglGetCurrentContext() == contextData->renderingContext
+                && wglGetCurrentDC() == contextData->hdc) {
+            return true;
+        }
         return wglMakeCurrent(contextData->hdc, contextData->renderingContext);
+    }
     // Create a new entry.
     const QOpenGLContextData newContext(m_renderingContext, hwnd, GetDC(hwnd));
     if (!newContext.hdc)
