@@ -1,5 +1,6 @@
 /****************************************************************************
 **
+** Copyright (C) 2013 Samuel Gaist <samuel.gaist@edeltech.ch>
 ** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
@@ -65,18 +66,22 @@
 #endif
 #include "qwindowsinputcontext.h"
 #include "qwindowskeymapper.h"
-#  ifndef QT_NO_ACCESSIBILITY
-#include "accessible/qwindowsaccessibility.h"
+#ifndef QT_NO_ACCESSIBILITY
+#  include "accessible/qwindowsaccessibility.h"
 #endif
 
 #include <qpa/qplatformnativeinterface.h>
 #include <qpa/qwindowsysteminterface.h>
+#if !defined(Q_OS_WINCE) && !defined(QT_NO_SESSIONMANAGER)
+#  include "qwindowssessionmanager.h"
+#endif
 #include <QtGui/QBackingStore>
 #include <QtGui/private/qpixmap_raster_p.h>
 #include <QtGui/private/qguiapplication_p.h>
 
 #include <QtCore/private/qeventdispatcher_win_p.h>
 #include <QtCore/QDebug>
+#include <QtCore/QVariant>
 
 QT_BEGIN_NAMESPACE
 
@@ -112,8 +117,15 @@ public:
 
     Q_INVOKABLE QString registerWindowClass(const QString &classNameIn, void *eventProc) const;
 
+    Q_INVOKABLE void beep() { MessageBeep(MB_OK); } // For QApplication
+
     bool asyncExpose() const;
     void setAsyncExpose(bool value);
+
+    QVariantMap windowProperties(QPlatformWindow *window) const;
+    QVariant windowProperty(QPlatformWindow *window, const QString &name) const;
+    QVariant windowProperty(QPlatformWindow *window, const QString &name, const QVariant &defaultValue) const;
+    void setWindowProperty(QPlatformWindow *window, const QString &name, const QVariant &value);
 };
 
 void *QWindowsNativeInterface::nativeResourceForWindow(const QByteArray &resource, QWindow *window)
@@ -150,6 +162,37 @@ void *QWindowsNativeInterface::nativeResourceForBackingStore(const QByteArray &r
     return 0;
 }
 
+static const char customMarginPropertyC[] = "WindowsCustomMargins";
+
+QVariant QWindowsNativeInterface::windowProperty(QPlatformWindow *window, const QString &name) const
+{
+    QWindowsWindow *platformWindow = static_cast<QWindowsWindow *>(window);
+    if (name == QLatin1String(customMarginPropertyC))
+        return qVariantFromValue(platformWindow->customMargins());
+    return QVariant();
+}
+
+QVariant QWindowsNativeInterface::windowProperty(QPlatformWindow *window, const QString &name, const QVariant &defaultValue) const
+{
+    const QVariant result = windowProperty(window, name);
+    return result.isValid() ? result : defaultValue;
+}
+
+void QWindowsNativeInterface::setWindowProperty(QPlatformWindow *window, const QString &name, const QVariant &value)
+{
+    QWindowsWindow *platformWindow = static_cast<QWindowsWindow *>(window);
+    if (name == QLatin1String(customMarginPropertyC))
+        platformWindow->setCustomMargins(qvariant_cast<QMargins>(value));
+}
+
+QVariantMap QWindowsNativeInterface::windowProperties(QPlatformWindow *window) const
+{
+    QVariantMap result;
+    const QString customMarginProperty = QLatin1String(customMarginPropertyC);
+    result.insert(customMarginProperty, windowProperty(window, customMarginProperty));
+    return result;
+}
+
 #ifndef QT_NO_OPENGL
 void *QWindowsNativeInterface::nativeResourceForContext(const QByteArray &resource, QOpenGLContext *context)
 {
@@ -272,7 +315,6 @@ struct QWindowsIntegrationPrivate
     QWindowsDrag m_drag;
 #  endif
 #endif
-    QWindowsGuiEventDispatcher *m_eventDispatcher;
 #if defined(QT_OPENGL_ES_2)
     QEGLStaticContextPtr m_staticEGLContext;
 #elif !defined(QT_NO_OPENGL)
@@ -303,6 +345,8 @@ static inline unsigned parseOptions(const QStringList &paramList)
             }
         } else if (param == QLatin1String("gl=gdi")) {
             options |= QWindowsIntegration::DisableArb;
+        } else if (param == QLatin1String("mousefromtouch")) {
+            options |= QWindowsIntegration::PassOsMouseEventsSynthesizedFromTouch;
         }
     }
     return options;
@@ -311,7 +355,6 @@ static inline unsigned parseOptions(const QStringList &paramList)
 QWindowsIntegrationPrivate::QWindowsIntegrationPrivate(const QStringList &paramList)
     : m_options(parseOptions(paramList))
     , m_fontDatabase(0)
-    , m_eventDispatcher(new QWindowsGuiEventDispatcher)
 {
 }
 
@@ -324,7 +367,6 @@ QWindowsIntegrationPrivate::~QWindowsIntegrationPrivate()
 QWindowsIntegration::QWindowsIntegration(const QStringList &paramList) :
     d(new QWindowsIntegrationPrivate(paramList))
 {
-    QGuiApplicationPrivate::instance()->setEventDispatcher(d->m_eventDispatcher);
 #ifndef QT_NO_CLIPBOARD
     d->m_clipboard.registerViewer();
 #endif
@@ -356,6 +398,8 @@ bool QWindowsIntegration::hasCapability(QPlatformIntegration::Capability cap) co
         return true;
     case MultipleWindows:
         return true;
+    case ForeignWindows:
+        return true;
     default:
         return QPlatformIntegration::hasCapability(cap);
     }
@@ -372,6 +416,11 @@ QPlatformWindow *QWindowsIntegration::createPlatformWindow(QWindow *window) cons
     QWindowsWindow::WindowData requested;
     requested.flags = window->flags();
     requested.geometry = window->geometry();
+    // Apply custom margins (see  QWindowsWindow::setCustomMargins())).
+    const QVariant customMarginsV = window->property("_q_windowsCustomMargins");
+    if (customMarginsV.isValid())
+        requested.customMargins = qvariant_cast<QMargins>(customMarginsV);
+
     const QWindowsWindow::WindowData obtained
             = QWindowsWindow::WindowData::create(window, requested, window->title());
     if (QWindowsContext::verboseIntegration || QWindowsContext::verboseWindows)
@@ -389,6 +438,9 @@ QPlatformWindow *QWindowsIntegration::createPlatformWindow(QWindow *window) cons
         return 0;
     if (requested.flags != obtained.flags)
         window->setFlags(obtained.flags);
+    // Trigger geometry change signals of QWindow.
+    if ((obtained.flags & Qt::Desktop) != Qt::Desktop && requested.geometry != obtained.geometry)
+        QWindowSystemInterface::handleGeometryChange(window, obtained.geometry);
     return new QWindowsWindow(window, obtained);
 }
 
@@ -499,7 +551,6 @@ QVariant QWindowsIntegration::styleHint(QPlatformIntegration::StyleHint hint) co
     case QPlatformIntegration::ShowIsFullScreen:
     case QPlatformIntegration::PasswordMaskDelay:
     case QPlatformIntegration::StartDragVelocity:
-    case QPlatformIntegration::SynthesizeMouseFromTouchEvents:
         break; // Not implemented
     case QPlatformIntegration::FontSmoothingGamma:
         return QVariant(QWindowsFontDatabase::fontSmoothingGamma());
@@ -509,6 +560,17 @@ QVariant QWindowsIntegration::styleHint(QPlatformIntegration::StyleHint hint) co
         break;
     case QPlatformIntegration::UseRtlExtensions:
         return QVariant(d->m_context.useRTLExtensions());
+    case QPlatformIntegration::SynthesizeMouseFromTouchEvents:
+#ifdef Q_OS_WINCE
+        // We do not want Qt to synthesize mouse events as Windows also does that.
+        // Alternatively, Windows-generated touch mouse events can be identified and
+        // ignored by checking GetMessageExtraInfo() for MI_WP_SIGNATURE (0xFF515700).
+       return false;
+#else // Q_OS_WINCE
+        return QVariant(!(d->m_options & PassOsMouseEventsSynthesizedFromTouch));
+#endif // !Q_OS_WINCE
+    default:
+        break;
     }
     return QPlatformIntegration::styleHint(hint);
 }
@@ -563,9 +625,16 @@ unsigned QWindowsIntegration::options() const
     return d->m_options;
 }
 
-QAbstractEventDispatcher * QWindowsIntegration::guiThreadEventDispatcher() const
+#if !defined(Q_OS_WINCE) && !defined(QT_NO_SESSIONMANAGER)
+QPlatformSessionManager *QWindowsIntegration::createPlatformSessionManager(const QString &id, const QString &key) const
 {
-    return d->m_eventDispatcher;
+    return new QWindowsSessionManager(id, key);
+}
+#endif
+
+QAbstractEventDispatcher * QWindowsIntegration::createEventDispatcher() const
+{
+    return new QWindowsGuiEventDispatcher;
 }
 
 QStringList QWindowsIntegration::themeNames() const
