@@ -277,7 +277,7 @@ static bool qt_read_dibv5(QDataStream &s, QImage &image)
 
     int  bpl = image.bytesPerLine();
     uchar *data = image.bits();
-    register QRgb *p;
+    QRgb *p;
     QRgb  *end;
     uchar *buf24 = new uchar[bpl];
     int    bpl24 = ((w*nbits+31)/32)*4;
@@ -469,7 +469,7 @@ int QWindowsMime::registerMimeType(const QString &mime)
 /*!
 \fn bool QWindowsMime::canConvertFromMime(const FORMATETC &formatetc, const QMimeData *mimeData) const
 
-  Returns true if the converter can convert from the \a mimeData to
+  Returns \c true if the converter can convert from the \a mimeData to
   the format specified in \a formatetc.
 
   All subclasses must reimplement this pure virtual function.
@@ -478,7 +478,7 @@ int QWindowsMime::registerMimeType(const QString &mime)
 /*!
   \fn bool QWindowsMime::canConvertToMime(const QString &mimeType, IDataObject *pDataObj) const
 
-  Returns true if the converter can convert to the \a mimeType from
+  Returns \c true if the converter can convert to the \a mimeType from
   the available formats in \a pDataObj.
 
   All subclasses must reimplement this pure virtual function.
@@ -703,14 +703,14 @@ QWindowsMimeURI::QWindowsMimeURI()
 
 bool QWindowsMimeURI::canConvertFromMime(const FORMATETC &formatetc, const QMimeData *mimeData) const
 {
-    if (getCf(formatetc) == CF_HDROP) {
+    if (mimeData->hasUrls() && getCf(formatetc) == CF_HDROP) {
         QList<QUrl> urls = mimeData->urls();
         for (int i=0; i<urls.size(); i++) {
             if (!urls.at(i).toLocalFile().isEmpty())
                 return true;
         }
     }
-    return (getCf(formatetc) == CF_INETURL_W || getCf(formatetc) == CF_INETURL) && mimeData->hasFormat(QStringLiteral("text/uri-list"));
+    return (getCf(formatetc) == CF_INETURL_W || getCf(formatetc) == CF_INETURL) && mimeData->hasUrls();
 }
 
 bool QWindowsMimeURI::convertFromMime(const FORMATETC &formatetc, const QMimeData *mimeData, STGMEDIUM *pmedium) const
@@ -917,11 +917,11 @@ QVariant QWindowsMimeHtml::convertToMime(const QString &mime, IDataObject *pData
         qDebug("raw :");
         qDebug(html);
 #endif
-        int start = html.indexOf("StartFragment:");
-        int end = html.indexOf("EndFragment:");
+        int start = html.indexOf("StartHTML:");
+        int end = html.indexOf("EndHTML:");
 
         if (start != -1) {
-            int startOffset = start + 14;
+            int startOffset = start + 10;
             int i = startOffset;
             while (html.at(i) != '\r' && html.at(i) != '\n')
                 ++i;
@@ -930,7 +930,7 @@ QVariant QWindowsMimeHtml::convertToMime(const QString &mime, IDataObject *pData
         }
 
         if (end != -1) {
-            int endOffset = end + 12;
+            int endOffset = end + 8;
             int i = endOffset ;
             while (html.at(i) != '\r' && html.at(i) != '\n')
                 ++i;
@@ -939,8 +939,7 @@ QVariant QWindowsMimeHtml::convertToMime(const QString &mime, IDataObject *pData
         }
 
         if (end > start && start > 0) {
-            html = "<!--StartFragment-->" + html.mid(start, end - start);
-            html += "<!--EndFragment-->";
+            html = html.mid(start, end - start);
             html.replace('\r', "");
             result = QString::fromUtf8(html);
         }
@@ -954,10 +953,10 @@ bool QWindowsMimeHtml::convertFromMime(const FORMATETC &formatetc, const QMimeDa
         QByteArray data = mimeData->html().toUtf8();
         QByteArray result =
             "Version:1.0\r\n"                    // 0-12
-            "StartHTML:0000000105\r\n"            // 13-35
-            "EndHTML:0000000000\r\n"            // 36-55
-            "StartFragment:0000000000\r\n"            // 58-86
-            "EndFragment:0000000000\r\n\r\n";   // 87-105
+            "StartHTML:0000000107\r\n"           // 13-35
+            "EndHTML:0000000000\r\n"             // 36-55
+            "StartFragment:0000000000\r\n"       // 56-81
+            "EndFragment:0000000000\r\n\r\n";    // 82-107
 
         if (data.indexOf("<!--StartFragment-->") == -1)
             result += "<!--StartFragment-->";
