@@ -49,6 +49,7 @@
 #include <qpa/qwindowsysteminterface.h>
 #include <QtGui/QGuiApplication>
 #include <QtGui/QScreen>
+#include <QtGui/QWindow>
 
 #include <QtCore/QDebug>
 #include <QtCore/QScopedArrayPointer>
@@ -156,9 +157,29 @@ bool QWindowsMouseHandler::translateMouseEvent(QWindow *window, HWND hwnd,
                                                QtWindows::WindowsEventType et,
                                                MSG msg, LRESULT *result)
 {
+#ifdef Q_COMPILER_CLASS_ENUM
+    enum : quint64 { signatureMask = 0xffffff00, miWpSignature = 0xff515700 };
+#else
+    static const quint64 signatureMask = 0xffffff00;
+    static const quint64 miWpSignature = 0xff515700;
+#endif // !Q_COMPILER_CLASS_ENUM
+
     if (et == QtWindows::MouseWheelEvent)
         return translateMouseWheelEvent(window, hwnd, msg, result);
 
+#ifndef Q_OS_WINCE
+    static const bool passSynthesizedMouseEvents = QWindowsIntegration::instance()->options() & QWindowsIntegration::PassOsMouseEventsSynthesizedFromTouch;
+    if (!passSynthesizedMouseEvents) {
+        // Check for events synthesized from touch. Lower 7 bits are touch/pen index, bit 8 indicates touch.
+        // However, when tablet support is active, extraInfo is a packet serial number. This is not a problem
+        // since we do not want to ignore mouse events coming from a tablet.
+        const quint64 extraInfo = GetMessageExtraInfo();
+        const bool fromTouch = (extraInfo & signatureMask) == miWpSignature && (extraInfo & 0x80);
+        if (fromTouch)
+            return false;
+    }
+#endif // !Q_OS_WINCE
+
     const QPoint winEventPosition(GET_X_LPARAM(msg.lParam), GET_Y_LPARAM(msg.lParam));
     if (et & QtWindows::NonClientEventFlag) {
         const QPoint globalPosition = winEventPosition;
@@ -221,8 +242,10 @@ bool QWindowsMouseHandler::translateMouseEvent(QWindow *window, HWND hwnd,
     }
 
     const QPoint globalPosition = QWindowsGeometryHint::mapToGlobal(hwnd, winEventPosition);
+    // In this context, neither an invisible nor a transparent window (transparent regarding mouse
+    // events, "click-through") can be considered as the window under mouse.
     QWindow *currentWindowUnderMouse = platformWindow->hasMouseCapture() ?
-        QWindowsScreen::windowAt(globalPosition) : window;
+        QWindowsScreen::windowAt(globalPosition, CWP_SKIPINVISIBLE | CWP_SKIPTRANSPARENT) : window;
 
     compressMouseMove(&msg);
     // Qt expects the platform plugin to capture the mouse on
@@ -236,6 +259,9 @@ bool QWindowsMouseHandler::translateMouseEvent(QWindow *window, HWND hwnd,
         platformWindow->setFlag(QWindowsWindow::AutoMouseCapture);
         if (QWindowsContext::verboseEvents)
             qDebug() << "Automatic mouse capture " << window;
+        // Implement "Click to focus" for native child windows (unless it is a native widget window).
+        if (!window->isTopLevel() && !window->inherits("QWidgetWindow") && QGuiApplication::focusWindow() != window)
+            window->requestActivate();
     } else if (platformWindow->hasMouseCapture()
                && platformWindow->testFlag(QWindowsWindow::AutoMouseCapture)
                && (msg.message == WM_LBUTTONUP || msg.message == WM_MBUTTONUP
