@@ -72,6 +72,7 @@ public:
 
     static Qt::MouseButtons queryMouseButtons();
     QWindow *windowUnderMouse() const { return m_windowUnderMouse.data(); }
+    void clearWindowUnderMouse() { m_windowUnderMouse = 0; }
 
 private:
     inline bool translateMouseWheelEvent(QWindow *window, HWND hwnd,
@@ -109,6 +110,8 @@ Qt::KeyboardModifiers QWindowsMouseHandler::keyStateToModifiers(int wParam)
       mods |= Qt::ControlModifier;
     if (wParam & MK_SHIFT)
       mods |= Qt::ShiftModifier;
+    if (GetKeyState(VK_MENU) < 0)
+      mods |= Qt::AltModifier;
     return mods;
 }
 
