@@ -67,8 +67,9 @@ class QWindowsEGLStaticContext;
 struct QWindowsGeometryHint
 {
     QWindowsGeometryHint() {}
-    explicit QWindowsGeometryHint(const QWindow *w);
+    explicit QWindowsGeometryHint(const QWindow *w, const QMargins &customMargins);
     static QMargins frame(DWORD style, DWORD exStyle);
+    static bool handleCalculateSize(const QMargins &customMargins, const MSG &msg, LRESULT *result);
 #ifndef Q_OS_WINCE //MinMax maybe define struct if not available
     void applyToMinMaxInfo(DWORD style, DWORD exStyle, MINMAXINFO *mmi) const;
     void applyToMinMaxInfo(HWND hwnd, MINMAXINFO *mmi) const;
@@ -84,11 +85,13 @@ struct QWindowsGeometryHint
 
     QSize minimumSize;
     QSize maximumSize;
+    QMargins customMargins;
 };
 
 struct QWindowCreationContext
 {
     QWindowCreationContext(const QWindow *w, const QRect &r,
+                           const QMargins &customMargins,
                            DWORD style, DWORD exStyle);
 #ifndef Q_OS_WINCE //MinMax maybe define struct if not available
     void applyToMinMaxInfo(MINMAXINFO *mmi) const
@@ -101,6 +104,7 @@ struct QWindowCreationContext
     QRect requestedGeometry;
     QRect obtainedGeometry;
     QMargins margins;
+    QMargins customMargins;  // User-defined, additional frame for WM_NCCALCSIZE
     int frameX; // Passed on to CreateWindowEx(), including frame.
     int frameY;
     int frameWidth;
@@ -120,14 +124,20 @@ public:
         WithinSetParent = 0x2,
         FrameDirty = 0x4,            //! Frame outdated by setStyle, recalculate in next query.
         OpenGLSurface = 0x10,
-        OpenGLDoubleBuffered = 0x20,
-        OpenGlPixelFormatInitialized = 0x40,
-        BlockedByModal = 0x80,
-        SizeGripOperation = 0x100,
-        FrameStrutEventsEnabled = 0x200,
-        SynchronousGeometryChangeEvent = 0x400,
-        WithinSetStyle = 0x800,
-        WithinDestroy = 0x1000
+        OpenGL_ES2 = 0x20,
+        OpenGLDoubleBuffered = 0x40,
+        OpenGlPixelFormatInitialized = 0x80,
+        BlockedByModal = 0x100,
+        SizeGripOperation = 0x200,
+        FrameStrutEventsEnabled = 0x400,
+        SynchronousGeometryChangeEvent = 0x800,
+        WithinSetStyle = 0x1000,
+        WithinDestroy = 0x2000,
+        TouchRegistered = 0x4000,
+        AlertState = 0x8000,
+        Exposed = 0x10000,
+        WithinCreate = 0x20000,
+        WithinMaximize = 0x40000
     };
 
     struct WindowData
@@ -137,6 +147,7 @@ public:
         Qt::WindowFlags flags;
         QRect geometry;
         QMargins frame; // Do not use directly for windows, see FrameDirty.
+        QMargins customMargins; // User-defined, additional frame for NCCALCSIZE
         HWND hwnd;
         bool embedded;
 
@@ -154,7 +165,7 @@ public:
 
     virtual void setVisible(bool visible);
     bool isVisible() const;
-    virtual bool isExposed() const { return m_windowState != Qt::WindowMinimized && isVisible(); }
+    virtual bool isExposed() const { return testFlag(Exposed); }
     virtual bool isActive() const;
     virtual bool isEmbedded(const QPlatformWindow *parentWindow) const;
     virtual QPoint mapToGlobal(const QPoint &pos) const;
@@ -191,6 +202,9 @@ public:
     void setFrameStrutEventsEnabled(bool enabled);
     bool frameStrutEventsEnabled() const { return testFlag(FrameStrutEventsEnabled); }
 
+    QMargins customMargins() const { return m_data.customMargins; }
+    void setCustomMargins(const QMargins &m);
+
 #ifdef QT_OPENGL_ES_2
     EGLSurface eglSurfaceHandle() const { return m_eglSurface;}
     EGLSurface ensureEglSurfaceHandle(const QWindowsEGLStaticContextPtr &staticContext, EGLConfig config);
@@ -207,8 +221,8 @@ public:
 
     void handleMoved();
     void handleResized(int wParam);
-    void handleShown();
     void handleHidden();
+    void handleCompositionSettingsChanged();
 
     static inline HWND handleOf(const QWindow *w);
     static inline QWindowsWindow *baseWindowOf(const QWindow *w);
@@ -217,12 +231,14 @@ public:
     static inline void setUserDataOf(HWND hwnd, void *ud);
 
     static bool setWindowLayered(HWND hwnd, Qt::WindowFlags flags, bool hasAlpha, qreal opacity);
+    bool isLayered() const;
 
     HDC getDC();
     void releaseDC();
 #ifndef Q_OS_WINCE // maybe available on some SDKs revisit WM_GETMINMAXINFO
     void getSizeHints(MINMAXINFO *mmi) const;
-#endif
+    bool handleNonClientHitTest(const QPoint &globalPos, LRESULT *result) const;
+#endif // !Q_OS_WINCE
 
 #ifndef QT_NO_CURSOR
     QWindowsWindowCursor cursor() const { return m_cursor; }
@@ -246,6 +262,8 @@ public:
     void setWindowIcon(const QIcon &icon);
 
 #ifndef Q_OS_WINCE
+    void setAlertState(bool enabled);
+    bool isAlertState() const { return testFlag(AlertState); }
     void alertWindow(int durationMs = 0);
     void stopAlertWindow();
 #endif
@@ -260,12 +278,14 @@ private:
     inline bool isFullScreen_sys() const;
     inline void setWindowState_sys(Qt::WindowState newState);
     inline void setParent_sys(const QPlatformWindow *parent) const;
+    inline void updateTransientParent() const;
     void destroyWindow();
     void registerDropSite();
     void unregisterDropSite();
     void handleGeometryChange();
     void handleWindowStateChange(Qt::WindowState state);
     inline void destroyIcon();
+    void fireExpose(const QRegion &region, bool force=false);
 
     mutable WindowData m_data;
     mutable unsigned m_flags;
@@ -290,17 +310,6 @@ private:
     HICON m_iconBig;
 };
 
-// Conveniences for window frames.
-inline QRect operator+(const QRect &r, const QMargins &m)
-{
-    return r.adjusted(-m.left(), -m.top(), m.right(), m.bottom());
-}
-
-inline QRect operator-(const QRect &r, const QMargins &m)
-{
-    return r.adjusted(m.left(), m.top(), -m.right(), -m.bottom());
-}
-
 // Debug
 QDebug operator<<(QDebug d, const RECT &r);
 #ifndef Q_OS_WINCE // maybe available on some SDKs revisit WM_GETMINMAXINFO/WM_NCCALCSIZE
@@ -369,6 +378,17 @@ inline void QWindowsWindow::destroyIcon()
     }
 }
 
+inline bool QWindowsWindow::isLayered() const
+{
+#ifndef Q_OS_WINCE
+    return GetWindowLongPtr(m_data.hwnd, GWL_EXSTYLE) & WS_EX_LAYERED;
+#else
+    return false;
+#endif
+}
+
 QT_END_NAMESPACE
 
+Q_DECLARE_METATYPE(QMargins)
+
 #endif // QWINDOWSWINDOW_H
