@@ -53,6 +53,14 @@
 #include <QVarLengthArray>
 #include <qpa/qwindowsysteminterface.h>
 
+// This is needed to make Qt compile together with XKB. xkb.h is using a variable
+// which is called 'explicit', this is a reserved keyword in c++ */
+#ifndef QT_NO_XKB
+#define explicit dont_use_cxx_explicit
+#include <xcb/xkb.h>
+#undef explicit
+#endif
+
 #ifndef QT_NO_TABLETEVENT
 #include <QTabletEvent>
 #endif
@@ -79,8 +87,7 @@ class QXcbKeyboard;
 class QXcbClipboard;
 class QXcbWMSupport;
 class QXcbNativeInterface;
-
-typedef QHash<xcb_window_t, QXcbWindow *> WindowMapper;
+class QXcbSystemTrayTracker;
 
 namespace QXcbAtom {
     enum Atom {
@@ -92,10 +99,13 @@ namespace QXcbAtom {
         _NET_WM_CONTEXT_HELP,
         _NET_WM_SYNC_REQUEST,
         _NET_WM_SYNC_REQUEST_COUNTER,
+        MANAGER, // System tray notification
+        _NET_SYSTEM_TRAY_OPCODE, // System tray operation
 
         // ICCCM window state
         WM_STATE,
         WM_CHANGE_STATE,
+        WM_CLASS,
 
         // Session management
         WM_CLIENT_LEADER,
@@ -249,6 +259,8 @@ namespace QXcbAtom {
         AbsMTPressure,
         AbsMTTrackingID,
         MaxContacts,
+        RelX,
+        RelY,
         // XInput2 tablet
         AbsX,
         AbsY,
@@ -263,6 +275,11 @@ namespace QXcbAtom {
 #if XCB_USE_MAEMO_WINDOW_PROPERTIES
         MeegoTouchOrientationAngle,
 #endif
+        _XSETTINGS_SETTINGS,
+
+        _COMPIZ_DECOR_PENDING,
+        _COMPIZ_DECOR_REQUEST,
+        _COMPIZ_DECOR_DELETE_PIXMAP,
 
         NPredefinedAtoms,
 
@@ -285,11 +302,14 @@ public:
     QXcbEventArray *lock();
     void unlock();
 
-    bool startThread();
+    void start();
 
 signals:
     void eventPending();
 
+private slots:
+    void registerForEvents();
+
 private:
     void addEvent(xcb_generic_event_t *event);
 
@@ -301,12 +321,37 @@ private:
     XcbPollForQueuedEventFunctionPointer m_xcb_poll_for_queued_event;
 };
 
+class QXcbWindowEventListener
+{
+public:
+    virtual bool handleGenericEvent(xcb_generic_event_t *, long *) { return false; }
+
+    virtual void handleExposeEvent(const xcb_expose_event_t *) {}
+    virtual void handleClientMessageEvent(const xcb_client_message_event_t *) {}
+    virtual void handleConfigureNotifyEvent(const xcb_configure_notify_event_t *) {}
+    virtual void handleMapNotifyEvent(const xcb_map_notify_event_t *) {}
+    virtual void handleUnmapNotifyEvent(const xcb_unmap_notify_event_t *) {}
+    virtual void handleDestroyNotifyEvent(const xcb_destroy_notify_event_t *) {}
+    virtual void handleButtonPressEvent(const xcb_button_press_event_t *) {}
+    virtual void handleButtonReleaseEvent(const xcb_button_release_event_t *) {}
+    virtual void handleMotionNotifyEvent(const xcb_motion_notify_event_t *) {}
+    virtual void handleEnterNotifyEvent(const xcb_enter_notify_event_t *) {}
+    virtual void handleLeaveNotifyEvent(const xcb_leave_notify_event_t *) {}
+    virtual void handleFocusInEvent(const xcb_focus_in_event_t *) {}
+    virtual void handleFocusOutEvent(const xcb_focus_out_event_t *) {}
+    virtual void handlePropertyNotifyEvent(const xcb_property_notify_event_t *) {}
+
+    virtual QXcbWindow *toWindow() { return 0; }
+};
+
+typedef QHash<xcb_window_t, QXcbWindowEventListener *> WindowMapper;
+
 class QAbstractEventDispatcher;
 class QXcbConnection : public QObject
 {
     Q_OBJECT
 public:
-    QXcbConnection(QXcbNativeInterface *nativeInterface, const char *displayName = 0);
+    QXcbConnection(QXcbNativeInterface *nativeInterface, bool canGrabServer, const char *displayName = 0);
     ~QXcbConnection();
 
     QXcbConnection *connection() const { return const_cast<QXcbConnection *>(this); }
@@ -314,7 +359,8 @@ public:
     const QList<QXcbScreen *> &screens() const { return m_screens; }
     int primaryScreen() const { return m_primaryScreen; }
 
-    xcb_atom_t atom(QXcbAtom::Atom atom);
+    inline xcb_atom_t atom(QXcbAtom::Atom atom) const { return m_allAtoms[atom]; }
+    QXcbAtom::Atom qatom(xcb_atom_t atom) const;
     xcb_atom_t internAtom(const char *name);
     QByteArray atomName(xcb_atom_t atom);
 
@@ -333,7 +379,7 @@ public:
 #endif
 
     QXcbWMSupport *wmSupport() const { return m_wmSupport.data(); }
-
+    xcb_window_t rootWindow();
 #ifdef XCB_USE_XLIB
     void *xlib_display() const { return m_xlib_display; }
 #endif
@@ -356,15 +402,16 @@ public:
     void handleXcbError(xcb_generic_error_t *error);
     void handleXcbEvent(xcb_generic_event_t *event);
 
-    void addWindow(xcb_window_t id, QXcbWindow *window);
-    void removeWindow(xcb_window_t id);
+    void addWindowEventListener(xcb_window_t id, QXcbWindowEventListener *eventListener);
+    void removeWindowEventListener(xcb_window_t id);
+    QXcbWindowEventListener *windowEventListenerFromId(xcb_window_t id);
     QXcbWindow *platformWindowFromId(xcb_window_t id);
 
     xcb_generic_event_t *checkEvent(int type);
     template<typename T>
     inline xcb_generic_event_t *checkEvent(T &checker);
 
-    typedef bool (*PeekFunc)(xcb_generic_event_t *);
+    typedef bool (*PeekFunc)(QXcbConnection *, xcb_generic_event_t *);
     void addPeekFunc(PeekFunc f);
 
     inline xcb_timestamp_t time() const { return m_time; }
@@ -378,6 +425,8 @@ public:
     bool hasXShape() const { return has_shape_extension; }
     bool hasXRandr() const { return has_randr_extension; }
     bool hasInputShape() const { return has_input_shape; }
+    bool hasTouchWithoutMouseEmulation() const { return has_touch_without_mouse_emulation; }
+    bool hasXKB() const { return has_xkb; }
 
     bool supportsThreadedRendering() const { return m_reader->isRunning(); }
 
@@ -388,6 +437,16 @@ public:
     QXcbWindow *focusWindow() const { return m_focusWindow; }
     void setFocusWindow(QXcbWindow *);
 
+    QByteArray startupId() const { return m_startupId; }
+    void clearStartupId() { m_startupId.clear(); }
+
+    void grabServer();
+    void ungrabServer();
+
+    QXcbNativeInterface *nativeInterface() const { return m_nativeInterface; }
+
+    QXcbSystemTrayTracker *systemTrayTracker();
+
 private slots:
     void processXcbEvents();
 
@@ -399,6 +458,7 @@ private:
     void initializeXRender();
     void initializeXRandr();
     void initializeXShape();
+    void initializeXKB();
 #ifdef XCB_USE_XINPUT2_MAEMO
     void initializeXInput2Maemo();
     void finalizeXInput2Maemo();
@@ -435,8 +495,6 @@ private:
         };
         QHash<int, ValuatorClassInfo> valuatorInfo;
     };
-    void xi2QueryTabletData(void *dev, TabletData *tabletData); // use no XI stuff in headers
-    void xi2SetupTabletDevices();
     bool xi2HandleTabletEvent(void *event, TabletData *tabletData);
     void xi2ReportTabletEvent(const TabletData &tabletData, void *event);
     QVector<TabletData> m_tabletData;
@@ -451,6 +509,7 @@ private:
 
     xcb_connection_t *m_connection;
     const xcb_setup_t *m_setup;
+    bool m_canGrabServer;
 
     QList<QXcbScreen *> m_screens;
     int m_primaryScreen;
@@ -507,15 +566,25 @@ private:
 
     uint32_t xfixes_first_event;
     uint32_t xrandr_first_event;
+    uint32_t xkb_first_event;
 
     bool has_glx_extension;
     bool has_shape_extension;
     bool has_randr_extension;
     bool has_input_shape;
+    bool has_touch_without_mouse_emulation;
+    bool has_xkb;
+    bool debug_xinput_devices;
+    bool debug_xinput;
 
     Qt::MouseButtons m_buttons;
 
     QXcbWindow *m_focusWindow;
+
+    QByteArray m_startupId;
+    QXcbSystemTrayTracker *m_systemTrayTracker;
+
+    friend class QXcbEventReader;
 };
 
 #define DISPLAY_FROM_XCB(object) ((Display *)(object->connection()->xlib_display()))
@@ -537,6 +606,15 @@ xcb_generic_event_t *QXcbConnection::checkEvent(T &checker)
     return 0;
 }
 
+class QXcbConnectionGrabber
+{
+public:
+    QXcbConnectionGrabber(QXcbConnection *connection);
+    ~QXcbConnectionGrabber();
+    void release();
+private:
+    QXcbConnection *m_connection;
+};
 
 #ifdef Q_XCB_DEBUG
 template <typename cookie_t>
