@@ -43,6 +43,8 @@
 #include "qxcbconnection.h"
 #include "qxcbwindow.h"
 #include "qxcbimage.h"
+#include "qxcbxsettings.h"
+
 #include <QtCore/QLibrary>
 #include <QtGui/QWindow>
 #include <QtGui/QBitmap>
@@ -54,9 +56,17 @@
 QT_BEGIN_NAMESPACE
 
 typedef int (*PtrXcursorLibraryLoadCursor)(void *, const char *);
+typedef char *(*PtrXcursorLibraryGetTheme)(void *);
+typedef int (*PtrXcursorLibrarySetTheme)(void *, const char *);
+typedef int (*PtrXcursorLibraryGetDefaultSize)(void *);
+
 #ifdef XCB_USE_XLIB
 static PtrXcursorLibraryLoadCursor ptrXcursorLibraryLoadCursor = 0;
+static PtrXcursorLibraryGetTheme ptrXcursorLibraryGetTheme = 0;
+static PtrXcursorLibrarySetTheme ptrXcursorLibrarySetTheme = 0;
+static PtrXcursorLibraryGetDefaultSize ptrXcursorLibraryGetDefaultSize = 0;
 #endif
+
 static xcb_font_t cursorFont = 0;
 static int cursorCount = 0;
 
@@ -262,8 +272,28 @@ static const char * const cursorNames[] = {
     "link"
 };
 
+#ifndef QT_NO_CURSOR
+
+QXcbCursorCacheKey::QXcbCursorCacheKey(const QCursor &c)
+    : shape(c.shape()), bitmapCacheKey(0), maskCacheKey(0)
+{
+    if (shape == Qt::BitmapCursor) {
+        const qint64 pixmapCacheKey = c.pixmap().cacheKey();
+        if (pixmapCacheKey) {
+            bitmapCacheKey = pixmapCacheKey;
+        } else {
+            Q_ASSERT(c.bitmap());
+            Q_ASSERT(c.mask());
+            bitmapCacheKey = c.bitmap()->cacheKey();
+            maskCacheKey = c.mask()->cacheKey();
+        }
+    }
+}
+
+#endif // !QT_NO_CURSOR
+
 QXcbCursor::QXcbCursor(QXcbConnection *conn, QXcbScreen *screen)
-    : QXcbObject(conn), m_screen(screen)
+    : QXcbObject(conn), m_screen(screen), m_gtkCursorThemeInitialized(false)
 {
     if (cursorCount++)
         return;
@@ -273,22 +303,43 @@ QXcbCursor::QXcbCursor(QXcbConnection *conn, QXcbScreen *screen)
     xcb_open_font(xcb_connection(), cursorFont, strlen(cursorStr), cursorStr);
 
 #ifdef XCB_USE_XLIB
-    QLibrary xcursorLib(QLatin1String("Xcursor"), 1);
-    bool xcursorFound = xcursorLib.load();
-    if (!xcursorFound) { // try without the version number
-        xcursorLib.setFileName(QLatin1String("Xcursor"));
-        xcursorFound = xcursorLib.load();
+    static bool function_ptrs_not_initialized = true;
+    if (function_ptrs_not_initialized) {
+        QLibrary xcursorLib(QLatin1String("Xcursor"), 1);
+        bool xcursorFound = xcursorLib.load();
+        if (!xcursorFound) { // try without the version number
+            xcursorLib.setFileName(QLatin1String("Xcursor"));
+            xcursorFound = xcursorLib.load();
+        }
+        if (xcursorFound) {
+            ptrXcursorLibraryLoadCursor =
+                (PtrXcursorLibraryLoadCursor) xcursorLib.resolve("XcursorLibraryLoadCursor");
+            ptrXcursorLibraryGetTheme =
+                    (PtrXcursorLibraryGetTheme) xcursorLib.resolve("XcursorGetTheme");
+            ptrXcursorLibrarySetTheme =
+                    (PtrXcursorLibrarySetTheme) xcursorLib.resolve("XcursorSetTheme");
+            ptrXcursorLibraryGetDefaultSize =
+                    (PtrXcursorLibraryGetDefaultSize) xcursorLib.resolve("XcursorGetDefaultSize");
+        }
+        function_ptrs_not_initialized = false;
     }
-    if (xcursorFound)
-        ptrXcursorLibraryLoadCursor =
-            (PtrXcursorLibraryLoadCursor) xcursorLib.resolve("XcursorLibraryLoadCursor");
+
 #endif
 }
 
 QXcbCursor::~QXcbCursor()
 {
+    xcb_connection_t *conn = xcb_connection();
+
+    if (m_gtkCursorThemeInitialized) {
+        m_screen->xSettings()->removeCallbackForHandle(this);
+    }
+
     if (!--cursorCount)
-        xcb_close_font(xcb_connection(), cursorFont);
+        xcb_close_font(conn, cursorFont);
+
+    foreach (xcb_cursor_t cursor, m_cursorHash)
+        xcb_free_cursor(conn, cursor);
 }
 
 #ifndef QT_NO_CURSOR
@@ -300,18 +351,16 @@ void QXcbCursor::changeCursor(QCursor *cursor, QWindow *widget)
     else
         // No X11 cursor control when there is no widget under the cursor
         return;
-    
-    xcb_cursor_t c;
-    if (cursor->shape() == Qt::BitmapCursor) {
-        qint64 id = cursor->pixmap().cacheKey();
-        if (!m_bitmapCursorMap.contains(id))
-            m_bitmapCursorMap.insert(id, createBitmapCursor(cursor));
-        c = m_bitmapCursorMap.value(id);
-    } else {
-        int id = cursor->shape();
-        if (!m_shapeCursorMap.contains(id))
-            m_shapeCursorMap.insert(id, createFontCursor(cursor->shape()));
-        c = m_shapeCursorMap.value(id);
+
+    xcb_cursor_t c = XCB_CURSOR_NONE;
+    if (cursor) {
+        const QXcbCursorCacheKey key(*cursor);
+        CursorHash::iterator it = m_cursorHash.find(key);
+        if (it == m_cursorHash.end()) {
+            const Qt::CursorShape shape = cursor->shape();
+            it = m_cursorHash.insert(key, shape == Qt::BitmapCursor ? createBitmapCursor(cursor) : createFontCursor(shape));
+        }
+        c = it.value();
     }
 
     w->setCursor(c);
@@ -434,12 +483,60 @@ xcb_cursor_t QXcbCursor::createNonStandardCursor(int cshape)
             xcb_pixmap_t pmm = qt_xcb_XPixmapFromBitmap(m_screen, image.createAlphaMask());
             cursor = xcb_generate_id(conn);
             xcb_create_cursor(conn, cursor, pm, pmm, 0, 0, 0, 0xFFFF, 0xFFFF, 0xFFFF, 8, 8);
+            xcb_free_pixmap(conn, pm);
+            xcb_free_pixmap(conn, pmm);
         }
     }
 
     return cursor;
 }
 
+#ifdef XCB_USE_XLIB
+bool updateCursorTheme(void *dpy, const QByteArray theme) {
+    if (!ptrXcursorLibraryGetTheme
+            || !ptrXcursorLibrarySetTheme)
+        return false;
+    QByteArray oldTheme = ptrXcursorLibraryGetTheme(dpy);
+    if (oldTheme == theme)
+        return false;
+
+    int setTheme = ptrXcursorLibrarySetTheme(dpy,theme.constData());
+    return setTheme;
+}
+
+ void QXcbCursor::cursorThemePropertyChanged(QXcbScreen *screen, const QByteArray &name, const QVariant &property, void *handle)
+{
+    Q_UNUSED(screen);
+    Q_UNUSED(name);
+    QXcbCursor *self = static_cast<QXcbCursor *>(handle);
+    updateCursorTheme(self->connection()->xlib_display(),property.toByteArray());
+}
+
+static xcb_cursor_t loadCursor(void *dpy, int cshape)
+{
+    xcb_cursor_t cursor = XCB_NONE;
+    if (!ptrXcursorLibraryLoadCursor || !dpy)
+        return cursor;
+    switch (cshape) {
+    case Qt::DragCopyCursor:
+        cursor = ptrXcursorLibraryLoadCursor(dpy, "dnd-copy");
+        break;
+    case Qt::DragMoveCursor:
+        cursor = ptrXcursorLibraryLoadCursor(dpy, "dnd-move");
+        break;
+    case Qt::DragLinkCursor:
+        cursor = ptrXcursorLibraryLoadCursor(dpy, "dnd-link");
+        break;
+    default:
+        break;
+    }
+    if (!cursor) {
+        cursor = ptrXcursorLibraryLoadCursor(dpy, cursorNames[cshape]);
+    }
+    return cursor;
+}
+#endif //XCB_USE_XLIB
+
 xcb_cursor_t QXcbCursor::createFontCursor(int cshape)
 {
     xcb_connection_t *conn = xcb_connection();
@@ -448,24 +545,18 @@ xcb_cursor_t QXcbCursor::createFontCursor(int cshape)
 
     // Try Xcursor first
 #ifdef XCB_USE_XLIB
-    if (ptrXcursorLibraryLoadCursor && cshape >= 0 && cshape < Qt::LastCursor) {
+    if (cshape >= 0 && cshape < Qt::LastCursor) {
         void *dpy = connection()->xlib_display();
         // special case for non-standard dnd-* cursors
-        switch (cshape) {
-        case Qt::DragCopyCursor:
-            cursor = ptrXcursorLibraryLoadCursor(dpy, "dnd-copy");
-            break;
-        case Qt::DragMoveCursor:
-            cursor = ptrXcursorLibraryLoadCursor(dpy, "dnd-move");
-            break;
-        case Qt::DragLinkCursor:
-            cursor = ptrXcursorLibraryLoadCursor(dpy, "dnd-link");
-            break;
-        default:
-            break;
+        cursor = loadCursor(dpy, cshape);
+        if (!cursor && !m_gtkCursorThemeInitialized) {
+            QByteArray gtkCursorTheme = m_screen->xSettings()->setting("Gtk/CursorThemeName").toByteArray();
+            m_screen->xSettings()->registerCallbackForProperty("Gtk/CursorThemeName",cursorThemePropertyChanged,this);
+            if (updateCursorTheme(dpy,gtkCursorTheme)) {
+                cursor = loadCursor(dpy, cshape);
+            }
+            m_gtkCursorThemeInitialized = true;
         }
-        if (!cursor)
-            cursor = ptrXcursorLibraryLoadCursor(dpy, cursorNames[cshape]);
     }
     if (cursor)
         return cursor;
