@@ -50,6 +50,10 @@
 #include "qxcbclipboard.h"
 #include "qxcbdrag.h"
 
+#ifndef QT_NO_SESSIONMANAGER
+#include "qxcbsessionmanager.h"
+#endif
+
 #include <xcb/xcb.h>
 
 #include <QtPlatformSupport/private/qgenericunixeventdispatcher_p.h>
@@ -78,10 +82,12 @@
 #elif defined(XCB_USE_EGL)
 #include "qxcbeglsurface.h"
 #include <QtPlatformSupport/private/qeglplatformcontext_p.h>
+#include <QtPlatformSupport/private/qeglpbuffer_p.h>
 #endif
 
 #include <QtGui/QOpenGLContext>
 #include <QtGui/QScreen>
+#include <QtGui/QOffscreenSurface>
 #ifndef QT_NO_ACCESSIBILITY
 #include <qpa/qplatformaccessibility.h>
 #ifndef QT_NO_ACCESSIBILITY_ATSPI_BRIDGE
@@ -89,20 +95,73 @@
 #endif
 #endif
 
+#include <QtCore/QFileInfo>
+
 QT_BEGIN_NAMESPACE
 
-QXcbIntegration::QXcbIntegration(const QStringList &parameters)
-    : m_eventDispatcher(createUnixEventDispatcher())
-    ,  m_services(new QGenericUnixServices)
+#if defined(QT_DEBUG) && defined(Q_OS_LINUX)
+// Find out if our parent process is gdb by looking at the 'exe' symlink under /proc,.
+// or, for older Linuxes, read out 'cmdline'.
+static bool runningUnderDebugger()
 {
-    QGuiApplicationPrivate::instance()->setEventDispatcher(m_eventDispatcher);
+    const QString parentProc = QLatin1String("/proc/") + QString::number(getppid());
+    const QFileInfo parentProcExe(parentProc + QLatin1String("/exe"));
+    if (parentProcExe.isSymLink())
+        return parentProcExe.symLinkTarget().endsWith(QLatin1String("/gdb"));
+    QFile f(parentProc + QLatin1String("/cmdline"));
+    if (!f.open(QIODevice::ReadOnly))
+        return false;
+    QByteArray s;
+    char c;
+    while (f.getChar(&c) && c) {
+        if (c == '/')
+            s.clear();
+        else
+            s += c;
+    }
+    return s == "gdb";
+}
+#endif
 
+QXcbIntegration::QXcbIntegration(const QStringList &parameters, int &argc, char **argv)
+    : m_services(new QGenericUnixServices)
+    , m_instanceName(0)
+{
 #ifdef XCB_USE_XLIB
     XInitThreads();
 #endif
     m_nativeInterface.reset(new QXcbNativeInterface);
 
-    m_connections << new QXcbConnection(m_nativeInterface.data());
+    bool canGrab = true;
+    #if defined(QT_DEBUG) && defined(Q_OS_LINUX)
+    canGrab = !runningUnderDebugger();
+    #endif
+    static bool canNotGrabEnv = qgetenv("QT_XCB_NO_GRAB_SERVER").length();
+    if (canNotGrabEnv)
+        canGrab = false;
+
+    // Parse arguments
+    const char *displayName = 0;
+    if (argc) {
+        int j = 1;
+        for (int i = 1; i < argc; i++) {
+            char *arg = argv[i];
+            if (arg) {
+                if (!strcmp(arg, "-display") && i < argc - 1) {
+                    displayName = argv[++i];
+                    arg = 0;
+                } else if (!strcmp(arg, "-name") && i < argc - 1) {
+                    m_instanceName = argv[++i];
+                    arg = 0;
+                }
+            }
+            if (arg)
+                argv[j++] = arg;
+        }
+        argc = j;
+    } // argc
+
+    m_connections << new QXcbConnection(m_nativeInterface.data(), canGrab, displayName);
 
     for (int i = 0; i < parameters.size() - 1; i += 2) {
 #ifdef Q_XCB_DEBUG
@@ -113,17 +172,10 @@ QXcbIntegration::QXcbIntegration(const QStringList &parameters)
     }
 
     m_fontDatabase.reset(new QGenericUnixFontDatabase());
-    m_inputContext.reset(QPlatformInputContextFactory::create());
-#if !defined(QT_NO_ACCESSIBILITY) && !defined(QT_NO_ACCESSIBILITY_ATSPI_BRIDGE)
-    m_accessibility.reset(new QSpiAccessibleBridge());
-#endif
 }
 
 QXcbIntegration::~QXcbIntegration()
 {
-#if !defined(QT_NO_OPENGL) && defined(XCB_USE_GLX)
-    qDeleteAll(m_defaultContextInfos);
-#endif
     qDeleteAll(m_connections);
 }
 
@@ -168,7 +220,10 @@ public:
 
     EGLSurface eglSurfaceForPlatformSurface(QPlatformSurface *surface)
     {
-        return static_cast<QXcbWindow *>(surface)->eglSurface()->surface();
+        if (surface->surface()->surfaceClass() == QSurface::Window)
+            return static_cast<QXcbWindow *>(surface)->eglSurface()->surface();
+        else
+            return static_cast<QEGLPbuffer *>(surface)->pbuffer();
     }
 
 private:
@@ -181,14 +236,7 @@ QPlatformOpenGLContext *QXcbIntegration::createPlatformOpenGLContext(QOpenGLCont
 {
     QXcbScreen *screen = static_cast<QXcbScreen *>(context->screen()->handle());
 #if defined(XCB_USE_GLX)
-    QOpenGLDefaultContextInfo *defaultContextInfo;
-    if (m_defaultContextInfos.contains(screen)) {
-        defaultContextInfo = m_defaultContextInfos.value(screen);
-    } else {
-        defaultContextInfo = QOpenGLDefaultContextInfo::create(screen);
-        m_defaultContextInfos.insert(screen, defaultContextInfo);
-    }
-    return new QGLXContext(screen, context->format(), context->shareHandle(), defaultContextInfo);
+    return new QGLXContext(screen, context->format(), context->shareHandle());
 #elif defined(XCB_USE_EGL)
     return new QEGLXcbPlatformContext(context->format(), context->shareHandle(),
         screen->connection()->egl_display(), screen->connection());
@@ -205,6 +253,20 @@ QPlatformBackingStore *QXcbIntegration::createPlatformBackingStore(QWindow *wind
     return new QXcbBackingStore(window);
 }
 
+QPlatformOffscreenSurface *QXcbIntegration::createPlatformOffscreenSurface(QOffscreenSurface *surface) const
+{
+#if defined(XCB_USE_GLX)
+    return new QGLXPbuffer(surface);
+#elif defined(XCB_USE_EGL)
+    QXcbScreen *screen = static_cast<QXcbScreen *>(surface->screen()->handle());
+    return new QEGLPbuffer(screen->connection()->egl_display(), surface->requestedFormat(), surface);
+#else
+    Q_UNUSED(surface);
+    qWarning("QXcbIntegration: Cannot create platform offscreen surface, neither GLX nor EGL are enabled");
+    return 0;
+#endif
+}
+
 bool QXcbIntegration::hasCapability(QPlatformIntegration::Capability cap) const
 {
     switch (cap) {
@@ -216,16 +278,29 @@ bool QXcbIntegration::hasCapability(QPlatformIntegration::Capability cap) const
 #else
     case OpenGL: return false;
 #endif
+#if defined(XCB_USE_GLX)
+    case ThreadedOpenGL: return m_connections.at(0)->supportsThreadedRendering() && QGLXContext::supportsThreading();
+#else
     case ThreadedOpenGL: return m_connections.at(0)->supportsThreadedRendering();
+#endif
     case WindowMasks: return true;
     case MultipleWindows: return true;
+    case ForeignWindows: return true;
+    case SyncState: return true;
     default: return QPlatformIntegration::hasCapability(cap);
     }
 }
 
-QAbstractEventDispatcher *QXcbIntegration::guiThreadEventDispatcher() const
+QAbstractEventDispatcher *QXcbIntegration::createEventDispatcher() const
 {
-    return m_eventDispatcher;
+    return createUnixEventDispatcher();
+}
+
+void QXcbIntegration::initialize()
+{
+    // Perform everything that may potentially need the event dispatcher (timers, socket
+    // notifiers) here instead of the constructor.
+    m_inputContext.reset(QPlatformInputContextFactory::create());
 }
 
 void QXcbIntegration::moveToScreen(QWindow *window, int screen)
@@ -266,6 +341,14 @@ QPlatformInputContext *QXcbIntegration::inputContext() const
 #ifndef QT_NO_ACCESSIBILITY
 QPlatformAccessibility *QXcbIntegration::accessibility() const
 {
+#if !defined(QT_NO_ACCESSIBILITY_ATSPI_BRIDGE)
+    if (!m_accessibility) {
+        Q_ASSERT_X(QCoreApplication::eventDispatcher(), "QXcbIntegration",
+            "Initializing accessibility without event-dispatcher!");
+        m_accessibility.reset(new QSpiAccessibleBridge());
+    }
+#endif
+
     return m_accessibility.data();
 }
 #endif
@@ -283,6 +366,11 @@ Qt::KeyboardModifiers QXcbIntegration::queryKeyboardModifiers() const
     return conn->keyboard()->translateModifiers(keybMask);
 }
 
+QList<int> QXcbIntegration::possibleKeys(const QKeyEvent *e) const
+{
+    return m_connections.at(0)->keyboard()->possibleKeys(e);
+}
+
 QStringList QXcbIntegration::themeNames() const
 {
     return QGenericUnixTheme::themeNames();
@@ -293,21 +381,93 @@ QPlatformTheme *QXcbIntegration::createPlatformTheme(const QString &name) const
     return QGenericUnixTheme::createUnixTheme(name);
 }
 
-/*!
-  Called by QXcbConnection prior to a QQnxScreen being deleted.
-
-  Destroys and cleans up any default OpenGL context info for this screen.
-*/
-void QXcbIntegration::removeDefaultOpenGLContextInfo(QXcbScreen *screen)
+QVariant QXcbIntegration::styleHint(QPlatformIntegration::StyleHint hint) const
 {
-#if !defined(QT_NO_OPENGL) && defined(XCB_USE_GLX)
-    if (!m_defaultContextInfos.contains(screen))
-        return;
-    QOpenGLDefaultContextInfo* info = m_defaultContextInfos.take(screen);
-    delete info;
-#else
-    Q_UNUSED(screen);
+    switch (hint) {
+    case QPlatformIntegration::CursorFlashTime:
+    case QPlatformIntegration::KeyboardInputInterval:
+    case QPlatformIntegration::MouseDoubleClickInterval:
+    case QPlatformIntegration::StartDragDistance:
+    case QPlatformIntegration::StartDragTime:
+    case QPlatformIntegration::KeyboardAutoRepeatRate:
+    case QPlatformIntegration::PasswordMaskDelay:
+    case QPlatformIntegration::FontSmoothingGamma:
+    case QPlatformIntegration::StartDragVelocity:
+    case QPlatformIntegration::UseRtlExtensions:
+    case QPlatformIntegration::PasswordMaskCharacter:
+        // TODO using various xcb, gnome or KDE settings
+        break; // Not implemented, use defaults
+    case QPlatformIntegration::ShowIsFullScreen:
+        // X11 always has support for windows, but the
+        // window manager could prevent it (e.g. matchbox)
+        return false;
+    case QPlatformIntegration::SynthesizeMouseFromTouchEvents:
+        // We do not want Qt to synthesize mouse events if X11 already does it.
+        return m_connections.at(0)->hasTouchWithoutMouseEmulation();
+    default:
+        break;
+    }
+    return QPlatformIntegration::styleHint(hint);
+}
+
+static QString argv0BaseName()
+{
+    QString result;
+    const QStringList arguments = QCoreApplication::arguments();
+    if (!arguments.isEmpty() && !arguments.front().isEmpty()) {
+        result = arguments.front();
+        const int lastSlashPos = result.lastIndexOf(QLatin1Char('/'));
+        if (lastSlashPos != -1)
+            result.remove(0, lastSlashPos + 1);
+    }
+    return result;
+}
+
+static const char resourceNameVar[] = "RESOURCE_NAME";
+
+QByteArray QXcbIntegration::wmClass() const
+{
+    if (m_wmClass.isEmpty()) {
+        // Instance name according to ICCCM 4.1.2.5
+        QString name;
+        if (m_instanceName)
+            name = QString::fromLocal8Bit(m_instanceName);
+        if (name.isEmpty() && qEnvironmentVariableIsSet(resourceNameVar))
+            name = QString::fromLocal8Bit(qgetenv(resourceNameVar));
+        if (name.isEmpty())
+            name = argv0BaseName();
+
+        // Note: QCoreApplication::applicationName() cannot be called from the QGuiApplication constructor,
+        // hence this delayed initialization.
+        QString className = QCoreApplication::applicationName();
+        if (className.isEmpty()) {
+            className = argv0BaseName();
+            if (!className.isEmpty() && className.at(0).isLower())
+                className[0] = className.at(0).toUpper();
+        }
+
+        if (!name.isEmpty() && !className.isEmpty()) {
+            m_wmClass = name.toLocal8Bit();
+            m_wmClass.append('\0');
+            m_wmClass.append(className.toLocal8Bit());
+            m_wmClass.append('\0');
+        }
+    }
+    return m_wmClass;
+}
+
+#if !defined(QT_NO_SESSIONMANAGER) && defined(XCB_USE_SM)
+QPlatformSessionManager *QXcbIntegration::createPlatformSessionManager(const QString &id, const QString &key) const
+{
+    return new QXcbSessionManager(id, key);
+}
 #endif
+
+void QXcbIntegration::sync()
+{
+    for (int i = 0; i < m_connections.size(); i++) {
+        m_connections.at(i)->sync();
+    }
 }
 
 QT_END_NAMESPACE
