@@ -45,6 +45,8 @@
 #include <qpa/qplatformnativeinterface.h>
 #include <xcb/xcb.h>
 
+#include <QtCore/QRect>
+
 QT_BEGIN_NAMESPACE
 
 class QWidget;
@@ -53,21 +55,27 @@ class QXcbConnection;
 
 class QXcbNativeInterface : public QPlatformNativeInterface
 {
+    Q_OBJECT
 public:
     enum ResourceType {
         Display,
         EglDisplay,
         Connection,
         Screen,
-        GraphicsDevice,
         EglContext,
         GLXContext,
         AppTime,
-        AppUserTime
+        AppUserTime,
+        ScreenHintStyle,
+        StartupId,
+        TrayWindow,
+        GetTimestamp,
+        X11Screen
     };
 
     QXcbNativeInterface();
 
+    void *nativeResourceForIntegration(const QByteArray &resource) Q_DECL_OVERRIDE;
     void *nativeResourceForContext(const QByteArray &resourceString, QOpenGLContext *context);
     void *nativeResourceForScreen(const QByteArray &resource, QScreen *screen);
     void *nativeResourceForWindow(const QByteArray &resourceString, QWindow *window);
@@ -81,14 +89,24 @@ public:
     void *eglDisplayForWindow(QWindow *window);
     void *connectionForWindow(QWindow *window);
     void *screenForWindow(QWindow *window);
-    void *graphicsDeviceForWindow(QWindow *window);
     void *appTime(const QXcbScreen *screen);
     void *appUserTime(const QXcbScreen *screen);
+    void *getTimestamp(const QXcbScreen *screen);
+    void *startupId();
+    void *x11Screen();
     static void setAppTime(QScreen *screen, xcb_timestamp_t time);
     static void setAppUserTime(QScreen *screen, xcb_timestamp_t time);
     static void *eglContextForContext(QOpenGLContext *context);
     static void *glxContextForContext(QOpenGLContext *context);
 
+    Q_INVOKABLE void beep();
+    Q_INVOKABLE bool systemTrayAvailable(const QScreen *screen) const;
+    Q_INVOKABLE bool requestSystemTrayWindowDock(const QWindow *window);
+    Q_INVOKABLE QRect systemTrayWindowGlobalGeometry(const QWindow *window);
+
+signals:
+    void systemTrayWindowChanged(QScreen *screen);
+
 private:
     const QByteArray m_genericEventFilterType;
 
