@@ -44,6 +44,7 @@
 #include "qxcbcursor.h"
 #include "qxcbimage.h"
 #include "qnamespace.h"
+#include "qxcbxsettings.h"
 
 #include <stdio.h>
 
@@ -67,6 +68,8 @@ QXcbScreen::QXcbScreen(QXcbConnection *connection, xcb_screen_t *scr,
     , m_number(number)
     , m_refreshRate(60)
     , m_forcedDpi(-1)
+    , m_hintStyle(QFontEngine::HintStyle(-1))
+    , m_xSettings(0)
 {
     if (connection->hasXRandr())
         xcb_randr_select_input(xcb_connection(), screen()->root, true);
@@ -109,6 +112,7 @@ QXcbScreen::QXcbScreen(QXcbConnection *connection, xcb_screen_t *scr,
         XCB_EVENT_MASK_ENTER_WINDOW
         | XCB_EVENT_MASK_LEAVE_WINDOW
         | XCB_EVENT_MASK_PROPERTY_CHANGE
+        | XCB_EVENT_MASK_STRUCTURE_NOTIFY // for the "MANAGER" atom (system tray notification).
     };
 
     xcb_change_window_attributes(xcb_connection(), screen()->root, mask, values);
@@ -145,7 +149,7 @@ QXcbScreen::QXcbScreen(QXcbConnection *connection, xcb_screen_t *scr,
     if (!sync_reply || !sync_reply->present)
         m_syncRequestSupported = false;
     else
-        m_syncRequestSupported = m_windowManagerName != QLatin1String("KWin");
+        m_syncRequestSupported = true;
 
     m_clientLeader = xcb_generate_id(xcb_connection());
     Q_XCB_CALL2(xcb_create_window(xcb_connection(),
@@ -157,6 +161,18 @@ QXcbScreen::QXcbScreen(QXcbConnection *connection, xcb_screen_t *scr,
                                   XCB_WINDOW_CLASS_INPUT_OUTPUT,
                                   screen()->root_visual,
                                   0, 0), connection);
+#ifndef QT_NO_DEBUG
+    QByteArray ba("Qt client leader window for screen ");
+    ba += m_outputName.toUtf8();
+    Q_XCB_CALL2(xcb_change_property(xcb_connection(),
+                                   XCB_PROP_MODE_REPLACE,
+                                   m_clientLeader,
+                                   atom(QXcbAtom::_NET_WM_NAME),
+                                   atom(QXcbAtom::UTF8_STRING),
+                                   8,
+                                   ba.length(),
+                                   ba.constData()), connection);
+#endif
 
     Q_XCB_CALL2(xcb_change_property(xcb_connection(),
                                     XCB_PROP_MODE_REPLACE,
@@ -232,6 +248,40 @@ QWindow *QXcbScreen::topLevelAt(const QPoint &p) const
     return 0;
 }
 
+void QXcbScreen::windowShown(QXcbWindow *window)
+{
+    // Freedesktop.org Startup Notification
+    if (!connection()->startupId().isEmpty() && window->window()->isTopLevel()) {
+        sendStartupMessage(QByteArrayLiteral("remove: ID=") + connection()->startupId());
+        connection()->clearStartupId();
+    }
+}
+
+void QXcbScreen::sendStartupMessage(const QByteArray &message) const
+{
+    xcb_window_t rootWindow = root();
+
+    xcb_client_message_event_t ev;
+    ev.response_type = XCB_CLIENT_MESSAGE;
+    ev.format = 8;
+    ev.type = connection()->atom(QXcbAtom::_NET_STARTUP_INFO_BEGIN);
+    ev.window = rootWindow;
+    int sent = 0;
+    int length = message.length() + 1; // include NUL byte
+    const char *data = message.constData();
+    do {
+        if (sent == 20)
+            ev.type = connection()->atom(QXcbAtom::_NET_STARTUP_INFO);
+
+        const int start = sent;
+        const int numBytes = qMin(length - start, 20);
+        memcpy(ev.data.data8, data + start, numBytes);
+        xcb_send_event(connection()->xcb_connection(), false, rootWindow, XCB_EVENT_MASK_PROPERTY_CHANGE, (const char *) &ev);
+
+        sent += numBytes;
+    } while (sent < length);
+}
+
 const xcb_visualtype_t *QXcbScreen::visualForId(xcb_visualid_t visualid) const
 {
     QMap<xcb_visualid_t, xcb_visualtype_t>::const_iterator it = m_visuals.find(visualid);
@@ -323,9 +373,9 @@ void QXcbScreen::handleScreenChange(xcb_randr_screen_change_notify_event_t *chan
     QWindowSystemInterface::handleScreenGeometryChange(QPlatformScreen::screen(), geometry());
     QWindowSystemInterface::handleScreenAvailableGeometryChange(QPlatformScreen::screen(), availableGeometry());
     QWindowSystemInterface::handleScreenOrientationChange(QPlatformScreen::screen(), m_orientation);
-    QWindowSystemInterface::handleScreenLogicalDotsPerInchChange(QPlatformScreen::screen(),
-        Q_MM_PER_INCH * m_virtualSize.width() / m_virtualSizeMillimeters.width(),
-        Q_MM_PER_INCH * m_virtualSize.height() / m_virtualSizeMillimeters.height());
+
+    QDpi ldpi = logicalDpi();
+    QWindowSystemInterface::handleScreenLogicalDotsPerInchChange(QPlatformScreen::screen(), ldpi.first, ldpi.second);
 }
 
 void QXcbScreen::updateGeometry(xcb_timestamp_t timestamp)
@@ -481,6 +531,35 @@ QPixmap QXcbScreen::grabWindow(WId window, int x, int y, int width, int height)
     return result;
 }
 
+bool QXcbScreen::xResource(const QByteArray &identifier,
+                           const QByteArray &expectedIdentifier,
+                           int *value)
+{
+    Q_ASSERT(value != 0);
+    if (identifier.startsWith(expectedIdentifier)) {
+        QByteArray stringValue = identifier.mid(expectedIdentifier.size());
+
+        bool ok;
+        *value = stringValue.toInt(&ok);
+        if (!ok) {
+            if (stringValue == "hintfull")
+                *value = QFontEngine::HintFull;
+            else if (stringValue == "hintnone")
+                *value = QFontEngine::HintNone;
+            else if (stringValue == "hintmedium")
+                *value = QFontEngine::HintMedium;
+            else if (stringValue == "hintslight")
+                *value = QFontEngine::HintLight;
+
+            return *value != 0;
+        }
+
+        return true;
+    }
+
+    return false;
+}
+
 void QXcbScreen::readXResources()
 {
     int offset = 0;
@@ -508,14 +587,20 @@ void QXcbScreen::readXResources()
     QList<QByteArray> split = resources.split('\n');
     for (int i = 0; i < split.size(); ++i) {
         const QByteArray &r = split.at(i);
-        if (r.startsWith("Xft.dpi:\t")) {
-            bool ok;
-            int dpi = r.mid(sizeof("Xft.dpi:")).toInt(&ok);
-            if (ok)
-                m_forcedDpi = dpi;
-            break;
-        }
+        int value;
+        if (xResource(r, "Xft.dpi:\t", &value))
+            m_forcedDpi = value;
+        else if (xResource(r, "Xft.hintstyle:\t", &value))
+            m_hintStyle = QFontEngine::HintStyle(value);
     }
 }
 
+QXcbXSettings *QXcbScreen::xSettings() const
+{
+    if (!m_xSettings) {
+        QXcbScreen *self = const_cast<QXcbScreen *>(this);
+        self->m_xSettings = new QXcbXSettings(self);
+    }
+    return m_xSettings;
+}
 QT_END_NAMESPACE
