@@ -56,8 +56,7 @@ QT_BEGIN_NAMESPACE
 class QXcbScreen;
 class QXcbEGLSurface;
 class QIcon;
-
-class QXcbWindow : public QXcbObject, public QPlatformWindow
+class QXcbWindow : public QXcbObject, public QXcbWindowEventListener, public QPlatformWindow
 {
 public:
     enum NetWmState {
@@ -87,6 +86,9 @@ public:
     void setParent(const QPlatformWindow *window);
 
     bool isExposed() const;
+    bool isEmbedded(const QPlatformWindow *parentWindow) const;
+    QPoint mapToGlobal(const QPoint &pos) const;
+    QPoint mapFromGlobal(const QPoint &pos) const;
 
     void setWindowTitle(const QString &title);
     void setWindowIcon(const QIcon &icon);
@@ -107,6 +109,8 @@ public:
 
     QSurfaceFormat format() const;
 
+    void windowEvent(QEvent *event);
+
     bool startSystemResize(const QPoint &pos, Qt::Corner corner);
 
     void setOpacity(qreal level);
@@ -115,24 +119,31 @@ public:
     void setMask(const QRegion &region);
 #endif // !QT_NO_SHAPE
 
+    void setAlertState(bool enabled);
+    bool isAlertState() const { return m_alertState; }
+
     xcb_window_t xcb_window() const { return m_window; }
     uint depth() const { return m_depth; }
     QImage::Format imageFormat() const { return m_imageFormat; }
 
-    void handleExposeEvent(const xcb_expose_event_t *event);
-    void handleClientMessageEvent(const xcb_client_message_event_t *event);
-    void handleConfigureNotifyEvent(const xcb_configure_notify_event_t *event);
-    void handleMapNotifyEvent(const xcb_map_notify_event_t *event);
-    void handleUnmapNotifyEvent(const xcb_unmap_notify_event_t *event);
-    void handleButtonPressEvent(const xcb_button_press_event_t *event);
-    void handleButtonReleaseEvent(const xcb_button_release_event_t *event);
-    void handleMotionNotifyEvent(const xcb_motion_notify_event_t *event);
+    bool handleGenericEvent(xcb_generic_event_t *event, long *result)  Q_DECL_OVERRIDE;
 
-    void handleEnterNotifyEvent(const xcb_enter_notify_event_t *event);
-    void handleLeaveNotifyEvent(const xcb_leave_notify_event_t *event);
-    void handleFocusInEvent(const xcb_focus_in_event_t *event);
-    void handleFocusOutEvent(const xcb_focus_out_event_t *event);
-    void handlePropertyNotifyEvent(const xcb_property_notify_event_t *event);
+    void handleExposeEvent(const xcb_expose_event_t *event) Q_DECL_OVERRIDE;
+    void handleClientMessageEvent(const xcb_client_message_event_t *event) Q_DECL_OVERRIDE;
+    void handleConfigureNotifyEvent(const xcb_configure_notify_event_t *event) Q_DECL_OVERRIDE;
+    void handleMapNotifyEvent(const xcb_map_notify_event_t *event) Q_DECL_OVERRIDE;
+    void handleUnmapNotifyEvent(const xcb_unmap_notify_event_t *event) Q_DECL_OVERRIDE;
+    void handleButtonPressEvent(const xcb_button_press_event_t *event) Q_DECL_OVERRIDE;
+    void handleButtonReleaseEvent(const xcb_button_release_event_t *event) Q_DECL_OVERRIDE;
+    void handleMotionNotifyEvent(const xcb_motion_notify_event_t *event) Q_DECL_OVERRIDE;
+
+    void handleEnterNotifyEvent(const xcb_enter_notify_event_t *event) Q_DECL_OVERRIDE;
+    void handleLeaveNotifyEvent(const xcb_leave_notify_event_t *event) Q_DECL_OVERRIDE;
+    void handleFocusInEvent(const xcb_focus_in_event_t *event) Q_DECL_OVERRIDE;
+    void handleFocusOutEvent(const xcb_focus_out_event_t *event) Q_DECL_OVERRIDE;
+    void handlePropertyNotifyEvent(const xcb_property_notify_event_t *event) Q_DECL_OVERRIDE;
+
+    QXcbWindow *toWindow() Q_DECL_OVERRIDE;
 
     void handleMouseEvent(xcb_timestamp_t time, const QPoint &local, const QPoint &global, Qt::KeyboardModifiers modifiers);
 
@@ -158,6 +169,9 @@ private:
     void updateDoesNotAcceptFocus(bool doesNotAcceptFocus);
 
     QRect windowToWmGeometry(QRect r) const;
+    void sendXEmbedMessage(xcb_window_t window, quint32 message,
+                           quint32 detail = 0, quint32 data1 = 0, quint32 data2 = 0);
+    void handleXEmbedMessage(const xcb_client_message_event_t *event);
 
     void create();
     void destroy();
@@ -181,9 +195,12 @@ private:
 
     bool m_mapped;
     bool m_transparent;
+    bool m_usingSyncProtocol;
     bool m_deferredActivation;
     bool m_deferredExpose;
     bool m_configureNotifyPending;
+    bool m_embedded;
+    bool m_alertState;
     xcb_window_t m_netWmUserTimeWindow;
 
     QSurfaceFormat m_format;
