@@ -66,8 +66,7 @@ QCupsPrinterSupport::QCupsPrinterSupport() : QPlatformPrinterSupport(),
 
 QCupsPrinterSupport::~QCupsPrinterSupport()
 {
-    if (cupsFreeDests)
-        cupsFreeDests(m_cupsPrintersCount, m_cupsPrinters);
+    freeCupsPrinters();
 }
 
 QPrintEngine *QCupsPrinterSupport::createNativePrintEngine(QPrinter::PrinterMode printerMode)
@@ -86,6 +85,11 @@ QList<QPrinter::PaperSize> QCupsPrinterSupport::supportedPaperSizes(const QPrint
     return QCUPSSupport::getCupsPrinterPaperSizes(printerIndex(printerInfo));
 }
 
+QList<QPair<QString, QSizeF> > QCupsPrinterSupport::supportedSizesWithNames(const QPrinterInfo &printerInfo) const
+{
+    return QCUPSSupport::getCupsPrinterPaperSizesWithNames(printerIndex(printerInfo));
+}
+
 void QCupsPrinterSupport::loadCups()
 {
     cupsGetDests = (CupsGetDests) m_cups.resolve("cupsGetDests");
@@ -93,14 +97,20 @@ void QCupsPrinterSupport::loadCups()
     cupsGetOption = (CupsGetOption) m_cups.resolve("cupsGetOption");
 }
 
+void QCupsPrinterSupport::freeCupsPrinters()
+{
+    if (cupsFreeDests && m_cupsPrintersCount) {
+        cupsFreeDests(m_cupsPrintersCount, m_cupsPrinters);
+        m_cupsPrintersCount = 0;
+        m_cupsPrinters = 0;
+    }
+}
+
 void QCupsPrinterSupport::loadCupsPrinters()
 {
-    m_cupsPrintersCount = 0;
+    freeCupsPrinters();
     m_printers.clear();
 
-    if (cupsFreeDests)
-        cupsFreeDests(m_cupsPrintersCount, m_cupsPrinters);
-
     if (cupsGetDests)
         m_cupsPrintersCount = cupsGetDests(&m_cupsPrinters);
 
@@ -117,6 +127,12 @@ void QCupsPrinterSupport::loadCupsPrinters()
     }
 }
 
+QList<QPrinterInfo> QCupsPrinterSupport::availablePrinters()
+{
+    loadCupsPrinters();
+    return QPlatformPrinterSupport::availablePrinters();
+}
+
 QString QCupsPrinterSupport::printerOption(const QPrinterInfo &printer, const QString &key) const
 {
     return cupsOption(printerIndex(printer), key);
