@@ -155,7 +155,7 @@ void QPrintDialog::setOption(PrintDialogOption option, bool on)
 }
 
 /*!
-    Returns true if the given \a option is enabled; otherwise, returns
+    Returns \c true if the given \a option is enabled; otherwise, returns
     false.
 
     \sa options, setOption()
@@ -404,8 +404,7 @@ void QAbstractPrintDialogPrivate::setPrinter(QPrinter *newPrinter)
     Mac OS X. This is no longer supported in Qt 4.5. If you want this
     functionality, use QPrintDialog::open().
 
-    \sa QPageSetupDialog, QPrinter, {Pixelator Example}, {Order Form Example},
-        {Image Viewer Example}, {Scribble Example}
+    \sa QPageSetupDialog, QPrinter
 */
 
 /*!
@@ -461,6 +460,10 @@ void QAbstractPrintDialog::setOptionTabs(const QList<QWidget*> &tabs)
   is shown with exec(), done() causes the local event loop to finish,
   and exec() to return \a result.
 
+  \note This function does not apply to the Native Print Dialog on the Mac
+  OS X and Windows platforms, because the dialog is required to be modal
+  and only the user can close it.
+
   \sa QDialog::done()
 */
 void QPrintDialog::done(int result)
