@@ -158,6 +158,13 @@ public:
         update();
     }
 
+    void setPagePreviewLayout(int columns, int rows)
+    {
+      m_pagePreviewColumns = columns;
+      m_pagePreviewRows = rows;
+      update();
+    }
+
 protected:
     void paintEvent(QPaintEvent *)
     {
@@ -202,13 +209,27 @@ protected:
             QString text(QLatin1String("Lorem ipsum dolor sit amet, consectetuer adipiscing elit, sed diam nonummy nibh euismod tincidunt ut laoreet dolore magna aliquam erat volutpat. Ut wisi enim ad minim veniam, quis nostrud exerci tation ullamcorper suscipit lobortis nisl ut aliquip ex ea commodo consequat. Duis autem vel eum iriure dolor in hendrerit in vulputate velit esse molestie consequat, vel illum dolore eu feugiat nulla facilisis at vero eros et accumsan et iusto odio dignissim qui blandit praesent luptatum zzril delenit augue duis dolore te feugait nulla facilisi."));
             for (int i=0; i<3; ++i)
                 text += text;
-            p.drawText(marginRect, Qt::TextWordWrap|Qt::AlignVCenter, text);
+
+            const int spacing = pageRect.width() * 0.1;
+            const int textWidth = (marginRect.width() - (spacing * (m_pagePreviewColumns-1))) / m_pagePreviewColumns;
+            const int textHeight = (marginRect.height() - (spacing * (m_pagePreviewRows-1))) / m_pagePreviewRows;
+
+            for (int x = 0 ; x < m_pagePreviewColumns; ++x) {
+                for (int y = 0 ; y < m_pagePreviewRows; ++y) {
+                    QRect textRect(marginRect.left() + x * (textWidth + spacing),
+                                   marginRect.top() + y * (textHeight + spacing),
+                                   textWidth, textHeight);
+                    p.drawText(textRect, Qt::TextWordWrap|Qt::AlignVCenter, text);
+                }
+            }
         }
     }
 
 private:
     // all these are in points
     qreal m_left, m_top, m_right, m_bottom;
+    // specify width / height of one page in preview
+    int m_pagePreviewColumns, m_pagePreviewRows;
     QSizeF m_size;
 };
 
@@ -272,6 +293,8 @@ QPageSetupWidget::QPageSetupWidget(QWidget *parent)
     QVBoxLayout *lay = new QVBoxLayout(widget.preview);
     widget.preview->setLayout(lay);
     m_pagePreview = new QPagePreview(widget.preview);
+    m_pagePreview->setPagePreviewLayout(1, 1);
+
     lay->addWidget(m_pagePreview);
 
     setAttribute(Qt::WA_WState_Polished, false);
@@ -288,7 +311,7 @@ QPageSetupWidget::QPageSetupWidget(QWidget *parent)
     widget.reversePortrait->setVisible(false);
 
     populatePaperSizes(widget.paperSize);
-
+    initPagesPerSheet();
     QStringList units;
     units << tr("Centimeters (cm)") << tr("Millimeters (mm)") << tr("Inches (in)") << tr("Points (pt)");
     widget.unit->addItems(units);
@@ -306,6 +329,8 @@ QPageSetupWidget::QPageSetupWidget(QWidget *parent)
 
     connect(widget.portrait, SIGNAL(clicked()), this, SLOT(_q_pageOrientationChanged()));
     connect(widget.landscape, SIGNAL(clicked()), this, SLOT(_q_pageOrientationChanged()));
+    connect(widget.pagesPerSheetCombo, SIGNAL(currentIndexChanged(int)), this, SLOT(_q_pagesPerSheetChanged()));
+
 }
 
 void QPageSetupWidget::setPrinter(QPrinter *printer)
@@ -360,12 +385,20 @@ void QPageSetupWidget::setupPrinter() const
     else {
         m_printer->setPaperSize(static_cast<QPrinter::PaperSize>(ps));
     }
-
+    m_printer->setPaperName(widget.paperSize->currentText());
 #ifdef PSD_ENABLE_PAPERSOURCE
     m_printer->setPaperSource((QPrinter::PaperSource)widget.paperSource->currentIndex());
 #endif
     m_printer->setPageMargins(m_leftMargin, m_topMargin, m_rightMargin, m_bottomMargin, QPrinter::Point);
 
+#if !defined(QT_NO_CUPS) && !defined(QT_NO_LIBRARY)
+    QCUPSSupport::PagesPerSheet pagesPerSheet = widget.pagesPerSheetCombo->currentData()
+                                                        .value<QCUPSSupport::PagesPerSheet>();
+    QCUPSSupport::PagesPerSheetLayout pagesPerSheetLayout = widget.pagesPerSheetLayoutCombo->currentData()
+                                                        .value<QCUPSSupport::PagesPerSheetLayout>();
+
+    QCUPSSupport::setPagesPerSheetLayout(m_printer, pagesPerSheet, pagesPerSheetLayout);
+#endif
 }
 
 void QPageSetupWidget::selectPrinter()
@@ -375,21 +408,28 @@ void QPageSetupWidget::selectPrinter()
     if (QCUPSSupport::isAvailable()) {
         m_cups = true;
         QCUPSSupport cups;
+        cups.setCurrentPrinter(m_printer->printerName());
         const ppd_option_t* pageSizes = cups.pageSizes();
         const int numChoices = pageSizes ? pageSizes->num_choices : 0;
 
         int cupsDefaultSize = 0;
         QSize qtPreferredSize = m_printer->paperSize(QPrinter::Point).toSize();
+        QString qtPaperName = m_printer->paperName();
         bool preferredSizeMatched = false;
         for (int i = 0; i < numChoices; ++i) {
             widget.paperSize->addItem(QString::fromLocal8Bit(pageSizes->choices[i].text), QByteArray(pageSizes->choices[i].choice));
             if (static_cast<int>(pageSizes->choices[i].marked) == 1)
                 cupsDefaultSize = i;
-            QRect cupsPaperSize = cups.paperRect(pageSizes->choices[i].choice);
-            QSize diff = cupsPaperSize.size() - qtPreferredSize;
-            if (qAbs(diff.width()) < 5 && qAbs(diff.height()) < 5) {
+            if (qtPaperName == QString::fromLocal8Bit(pageSizes->choices[i].text)) {
                 widget.paperSize->setCurrentIndex(i);
                 preferredSizeMatched = true;
+            } else {
+                QRect cupsPaperSize = cups.paperRect(pageSizes->choices[i].choice);
+                QSize diff = cupsPaperSize.size() - qtPreferredSize;
+                if (qAbs(diff.width()) < 5 && qAbs(diff.height()) < 5) {
+                    widget.paperSize->setCurrentIndex(i);
+                    preferredSizeMatched = true;
+                }
             }
         }
         if (!preferredSizeMatched)
@@ -417,9 +457,6 @@ void QPageSetupWidget::selectPrinter()
 
     unitChanged(widget.unit->currentIndex());
     m_pagePreview->setMargins(m_leftMargin, m_topMargin, m_rightMargin, m_bottomMargin);
-
-    // setup printer here the first time
-    setupPrinter();
 }
 
 void QPageSetupWidget::selectPdfPsPrinter(const QPrinter *p)
@@ -435,58 +472,68 @@ void QPageSetupWidget::selectPdfPsPrinter(const QPrinter *p)
 // Updates size/preview after the combobox has been changed.
 void QPageSetupWidget::_q_paperSizeChanged()
 {
-    QVariant val = widget.paperSize->itemData(widget.paperSize->currentIndex());
-    int index = m_printer->pageSize();
-    if (val.type() == QVariant::Int) {
-        index = val.toInt();
-    }
-
     if (m_blockSignals) return;
     m_blockSignals = true;
 
-    QPrinter::PaperSize size = QPrinter::PaperSize(index);
-    QPrinter::Orientation orientation = widget.portrait->isChecked()
-                                        ? QPrinter::Portrait
-                                        : QPrinter::Landscape;
+    bool custom = false;
+    QVariant val = widget.paperSize->itemData(widget.paperSize->currentIndex());
+    QPrinter::Orientation orientation = widget.portrait->isChecked() ? QPrinter::Portrait : QPrinter::Landscape;
 
-    bool custom = size == QPrinter::Custom;
-
-#if !defined(QT_NO_CUPS) && !defined(QT_NO_LIBRARY)
-    custom = custom ? m_cups : custom;
+    if (m_cups) {
+        // OutputFormat == NativeFormat, data is QString Cups paper name
+        QByteArray cupsPageSize = val.toByteArray();
+        custom = (cupsPageSize == QByteArrayLiteral("Custom"));
+#ifndef QT_NO_CUPS
+        if (!custom) {
+            QCUPSSupport cups;
+            cups.setCurrentPrinter(m_printer->printerName());
+            m_paperSize = sizeForOrientation(orientation, cups.paperRect(cupsPageSize).size());
+        }
 #endif
+    } else {
+        // OutputFormat == PdfFormat, data is QPrinter::PageSize
+        QPrinter::PaperSize size = QPrinter::PaperSize(val.toInt());
+        custom = size == QPrinter::Custom;
+        if (!custom)
+            m_paperSize = qt_printerPaperSize(orientation, size, QPrinter::Point, 1);
+    }
+
+    if (custom) {
+        // Convert input custom size Units to Points
+        m_paperSize = QSizeF(widget.paperWidth->value() * m_currentMultiplier,
+                             widget.paperHeight->value() * m_currentMultiplier);
+    } else {
+        // Display standard size Points as Units
+        widget.paperWidth->setValue(m_paperSize.width() / m_currentMultiplier);
+        widget.paperHeight->setValue(m_paperSize.height() / m_currentMultiplier);
+    }
+
+    m_pagePreview->setPaperSize(m_paperSize);
 
     widget.paperWidth->setEnabled(custom);
     widget.paperHeight->setEnabled(custom);
     widget.widthLabel->setEnabled(custom);
     widget.heightLabel->setEnabled(custom);
-    if (custom) {
-        m_paperSize.setWidth( widget.paperWidth->value() * m_currentMultiplier);
-        m_paperSize.setHeight( widget.paperHeight->value() * m_currentMultiplier);
-        m_pagePreview->setPaperSize(m_paperSize);
-    } else {
-        Q_ASSERT(m_printer);
-#if !defined(QT_NO_CUPS) && !defined(QT_NO_LIBRARY)
-        if (m_cups && QCUPSSupport::isAvailable()) { // combobox is filled with cups based data
-            QCUPSSupport cups;
-            QByteArray cupsPageSize = widget.paperSize->itemData(widget.paperSize->currentIndex()).toByteArray();
-            m_paperSize = cups.paperRect(cupsPageSize).size();
-            if (orientation == QPrinter::Landscape)
-                m_paperSize = QSizeF(m_paperSize.height(), m_paperSize.width()); // swap
-        }
-        else
-#endif
-            m_paperSize = qt_printerPaperSize(orientation, size, QPrinter::Point, 1);
 
-        m_pagePreview->setPaperSize(m_paperSize);
-        widget.paperWidth->setValue(m_paperSize.width() / m_currentMultiplier);
-        widget.paperHeight->setValue(m_paperSize.height() / m_currentMultiplier);
-    }
     m_blockSignals = false;
 }
 
 void QPageSetupWidget::_q_pageOrientationChanged()
 {
-    if (QPrinter::PaperSize(widget.paperSize->currentIndex()) == QPrinter::Custom) {
+    bool custom = false;
+    QVariant val = widget.paperSize->itemData(widget.paperSize->currentIndex());
+
+    if (m_cups) {
+        // OutputFormat == NativeFormat, data is QString Cups paper name
+        QByteArray cupsPageSize = val.toByteArray();
+        custom = (cupsPageSize == QByteArrayLiteral("Custom"));
+    } else {
+        // OutputFormat == PdfFormat, data is QPrinter::PageSize
+        QPrinter::PaperSize size = QPrinter::PaperSize(val.toInt());
+        custom = size == QPrinter::Custom;
+    }
+
+    if (custom) {
         double tmp = widget.paperWidth->value();
         widget.paperWidth->setValue(widget.paperHeight->value());
         widget.paperHeight->setValue(tmp);
@@ -494,6 +541,36 @@ void QPageSetupWidget::_q_pageOrientationChanged()
     _q_paperSizeChanged();
 }
 
+void QPageSetupWidget::_q_pagesPerSheetChanged()
+{
+#if !defined(QT_NO_CUPS) && !defined(QT_NO_LIBRARY)
+    QCUPSSupport::PagesPerSheet pagesPerSheet = widget.pagesPerSheetCombo->currentData()
+    .value<QCUPSSupport::PagesPerSheet>();
+
+    switch (pagesPerSheet) {
+    case QCUPSSupport::TwoPagesPerSheet:
+        m_pagePreview->setPagePreviewLayout(1, 2);
+        break;
+    case QCUPSSupport::FourPagesPerSheet:
+        m_pagePreview->setPagePreviewLayout(2, 2);
+        break;
+    case QCUPSSupport::SixPagesPerSheet:
+        m_pagePreview->setPagePreviewLayout(3, 2);
+        break;
+    case QCUPSSupport::NinePagesPerSheet:
+        m_pagePreview->setPagePreviewLayout(3, 3);
+        break;
+    case QCUPSSupport::SixteenPagesPerSheet:
+        m_pagePreview->setPagePreviewLayout(4, 4);
+        break;
+    case QCUPSSupport::OnePagePerSheet:
+    default:
+        m_pagePreview->setPagePreviewLayout(1, 1);
+        break;
+    }
+#endif
+}
+
 extern double qt_multiplierForUnit(QPrinter::Unit unit, int resolution);
 
 void QPageSetupWidget::unitChanged(int item)
@@ -568,14 +645,18 @@ void QPageSetupWidget::setRightMargin(double newValue)
 QPageSetupDialog::QPageSetupDialog(QPrinter *printer, QWidget *parent)
     : QDialog(*(new QUnixPageSetupDialogPrivate(printer)), parent)
 {
+    Q_D(QPageSetupDialog);
     setWindowTitle(QCoreApplication::translate("QPrintPreviewDialog", "Page Setup"));
+    static_cast<QUnixPageSetupDialogPrivate *>(d)->init();
 }
 
 
 QPageSetupDialog::QPageSetupDialog(QWidget *parent)
     : QDialog(*(new QUnixPageSetupDialogPrivate(0)), parent)
 {
+    Q_D(QPageSetupDialog);
     setWindowTitle(QCoreApplication::translate("QPrintPreviewDialog", "Page Setup"));
+    static_cast<QUnixPageSetupDialogPrivate *>(d)->init();
 }
 
 int QPageSetupDialog::exec()
@@ -588,6 +669,34 @@ int QPageSetupDialog::exec()
     return ret;
 }
 
+void QPageSetupWidget::initPagesPerSheet()
+{
+#if !defined(QT_NO_CUPS) && !defined(QT_NO_LIBRARY)
+    widget.pagesPerSheetLayoutCombo->addItem(QPrintDialog::tr("Left to Right, Top to Bottom"), QVariant::fromValue(QCUPSSupport::LeftToRightTopToBottom));
+    widget.pagesPerSheetLayoutCombo->addItem(QPrintDialog::tr("Left to Right, Bottom to Top"), QVariant::fromValue(QCUPSSupport::LeftToRightBottomToTop));
+    widget.pagesPerSheetLayoutCombo->addItem(QPrintDialog::tr("Right to Left, Bottom to Top"), QVariant::fromValue(QCUPSSupport::RightToLeftBottomToTop));
+    widget.pagesPerSheetLayoutCombo->addItem(QPrintDialog::tr("Right to Left, Top to Bottom"), QVariant::fromValue(QCUPSSupport::RightToLeftTopToBottom));
+    widget.pagesPerSheetLayoutCombo->addItem(QPrintDialog::tr("Bottom to Top, Left to Right"), QVariant::fromValue(QCUPSSupport::BottomToTopLeftToRight));
+    widget.pagesPerSheetLayoutCombo->addItem(QPrintDialog::tr("Bottom to Top, Right to Left"), QVariant::fromValue(QCUPSSupport::BottomToTopRightToLeft));
+    widget.pagesPerSheetLayoutCombo->addItem(QPrintDialog::tr("Top to Bottom, Left to Right"), QVariant::fromValue(QCUPSSupport::TopToBottomLeftToRight));
+    widget.pagesPerSheetLayoutCombo->addItem(QPrintDialog::tr("Top to Bottom, Right to Left"), QVariant::fromValue(QCUPSSupport::TopToBottomRightToLeft));
+
+    widget.pagesPerSheetCombo->addItem(QPrintDialog::tr("1 (1x1)"), QVariant::fromValue(QCUPSSupport::OnePagePerSheet));
+    widget.pagesPerSheetCombo->addItem(QPrintDialog::tr("2 (2x1)"), QVariant::fromValue(QCUPSSupport::TwoPagesPerSheet));
+    widget.pagesPerSheetCombo->addItem(QPrintDialog::tr("4 (2x2)"), QVariant::fromValue(QCUPSSupport::FourPagesPerSheet));
+    widget.pagesPerSheetCombo->addItem(QPrintDialog::tr("6 (2x3)"), QVariant::fromValue(QCUPSSupport::SixPagesPerSheet));
+    widget.pagesPerSheetCombo->addItem(QPrintDialog::tr("9 (3x3)"), QVariant::fromValue(QCUPSSupport::NinePagesPerSheet));
+    widget.pagesPerSheetCombo->addItem(QPrintDialog::tr("16 (4x4)"), QVariant::fromValue(QCUPSSupport::SixteenPagesPerSheet));
+
+    // Set the combo to "1 (1x1)" -- QCUPSSupport::OnePagePerSheet
+    widget.pagesPerSheetCombo->setCurrentIndex(0);
+    // Set the layout combo to QCUPSSupport::LeftToRightTopToBottom
+    widget.pagesPerSheetLayoutCombo->setCurrentIndex(0);
+#else
+    // Disable if CUPS wasn't found
+    widget.pagesPerSheetButtonGroup->hide();
+#endif
+}
 
 QT_END_NAMESPACE
 
