@@ -67,6 +67,8 @@ QT_BEGIN_NAMESPACE
 
 Q_GUI_EXPORT HBITMAP qt_pixmapToWinHBITMAP(const QPixmap &p, int hbitmapFormat = 0);
 extern QPainterPath qt_regionToPath(const QRegion &region);
+Q_PRINTSUPPORT_EXPORT QSizeF qt_SizeFromUnitToMillimeter(const QSizeF &, QPrinter::Unit, double);
+Q_PRINTSUPPORT_EXPORT double qt_multiplierForUnit(QPrinter::Unit unit, int resolution);
 
 // #define QT_DEBUG_DRAW
 
@@ -89,7 +91,7 @@ static const struct {
     { DMPAPER_A5,                 QPrinter::A5 },
     { DMPAPER_B4,                 QPrinter::B4 },
     { DMPAPER_B5,                 QPrinter::B5 },
-    { DMPAPER_FOLIO,              QPrinter::Folio },
+    { DMPAPER_A4_PLUS,            QPrinter::Folio },
     { DMPAPER_ENV_10,             QPrinter::Comm10E },
     { DMPAPER_ENV_DL,             QPrinter::DLE },
     { DMPAPER_ENV_C3,             QPrinter::C5E },
@@ -103,7 +105,6 @@ static const struct {
     { DMPAPER_A_PLUS,             QPrinter::A4 },
     { DMPAPER_B_PLUS,             QPrinter::A3 },
     { DMPAPER_LETTER_PLUS,        QPrinter::Letter },
-    { DMPAPER_A4_PLUS,            QPrinter::A4 },
     { DMPAPER_A5_TRANSVERSE,      QPrinter::A5 },
     { DMPAPER_B5_TRANSVERSE,      QPrinter::B5 },
     { DMPAPER_A3_EXTRA,           QPrinter::A3 },
@@ -115,6 +116,52 @@ static const struct {
     { 0, QPrinter::Custom }
 };
 
+// Return a list of printer paper sizes in millimeters with the corresponding dmPaperSize value
+static QList<QPair<QSizeF, int> > printerPaperSizes(const QString &printerName)
+{
+    QList<QPair<QSizeF, int> > result;
+    const wchar_t *name = reinterpret_cast<const wchar_t*>(printerName.utf16());
+    DWORD paperNameCount = DeviceCapabilities(name, NULL, DC_PAPERS, NULL, NULL);
+    if ((int)paperNameCount > 0) {
+        // If they are not equal, then there seems to be a problem with the driver
+        if (paperNameCount != DeviceCapabilities(name, NULL, DC_PAPERSIZE, NULL, NULL))
+            return result;
+        QScopedArrayPointer<wchar_t> papersNames(new wchar_t[paperNameCount]);
+        paperNameCount = DeviceCapabilities(name, NULL, DC_PAPERS, papersNames.data(), NULL);
+        result.reserve(paperNameCount);
+        QScopedArrayPointer<POINT> paperSizes(new POINT[paperNameCount]);
+        paperNameCount = DeviceCapabilities(name, NULL, DC_PAPERSIZE, (wchar_t *)paperSizes.data(), NULL);
+        for (int i=0; i <(int)paperNameCount; i++)
+            result.push_back(qMakePair(QSizeF(paperSizes[i].x / 10, paperSizes[i].y / 10), papersNames[i]));
+    }
+    return result;
+}
+
+// Find the best-matching printer paper for size in millimeters.
+static inline int findCustomPaperSize(const QSizeF &needlePt, const QString &printerName)
+{
+    const QList<QPair<QSizeF, int> > sizes = printerPaperSizes(printerName);
+    const qreal nw = needlePt.width();
+    const qreal nh = needlePt.height();
+    for (int i = 0; i < sizes.size(); ++i) {
+        if (qAbs(nw - sizes.at(i).first.width()) <= 1 && qAbs(nh - sizes.at(i).first.height()) <= 1)
+            return sizes.at(i).second;
+    }
+    return -1;
+}
+
+static inline void setDevModePaperFlags(DEVMODE *devMode, bool custom)
+{
+    if (custom) {
+        devMode->dmPaperSize = DMPAPER_USER;
+        devMode->dmFields |= DM_PAPERLENGTH | DM_PAPERWIDTH;
+    } else {
+        devMode->dmFields &= ~(DM_PAPERLENGTH | DM_PAPERWIDTH);
+        devMode->dmPaperLength = 0;
+        devMode->dmPaperWidth = 0;
+    }
+}
+
 QPrinter::PaperSize mapDevmodePaperSize(int s)
 {
     int i = 0;
@@ -167,6 +214,16 @@ static int mapPaperSourceDevmode(QPrinter::PaperSource s)
     return sources[i].winSourceName ? sources[i].winSourceName : s;
 }
 
+static inline uint qwcsnlen(const wchar_t *str, uint maxlen)
+{
+    uint length = 0;
+    if (str) {
+        while (length < maxlen && *str++)
+            length++;
+    }
+    return length;
+}
+
 QWin32PrintEngine::QWin32PrintEngine(QPrinter::PrinterMode mode)
     : QAlphaPaintEngine(*(new QWin32PrintEnginePrivate),
                    PaintEngineFeatures(PrimitiveTransform
@@ -355,8 +412,9 @@ void QWin32PrintEngine::drawTextItem(const QPointF &p, const QTextItem &textItem
                     || ti.fontEngine->type() != QFontEngine::Win;
 
     if (!fallBack) {
-        const QVariant hFontV = ti.fontEngine->property("hFont");
-        const QVariant logFontV = ti.fontEngine->property("logFont");
+        const QVariantMap userData = ti.fontEngine->userData().toMap();
+        const QVariant hFontV = userData.value(QStringLiteral("hFont"));
+        const QVariant logFontV = userData.value(QStringLiteral("logFont"));
         if (hFontV.canConvert<HFONT>() && logFontV.canConvert<LOGFONT>()) {
             const HFONT hfont = hFontV.value<HFONT>();
             const LOGFONT logFont = logFontV.value<LOGFONT>();
@@ -1283,9 +1341,52 @@ void QWin32PrintEngine::setProperty(PrintEnginePropertyKey key, const QVariant &
             break;
         d->devMode->dmPaperSize = mapPaperSizeDevmode(QPrinter::PaperSize(value.toInt()));
         d->has_custom_paper_size = (QPrinter::PaperSize(value.toInt()) == QPrinter::Custom);
+        setDevModePaperFlags(d->devMode, d->has_custom_paper_size);
         d->doReinit();
         break;
+    case PPK_PaperName:
+        {
+            if (!d->devMode)
+                break;
+            DWORD size = DeviceCapabilities(reinterpret_cast<const wchar_t*>(d->name.utf16()),
+                                            NULL, DC_PAPERNAMES, NULL, NULL);
+            if ((int)size > 0) {
+                wchar_t *paperNames = new wchar_t[size*64];
+                size = DeviceCapabilities(reinterpret_cast<const wchar_t*>(d->name.utf16()),
+                                          NULL, DC_PAPERNAMES, paperNames, NULL);
+                int paperPos = -1;
+                for (int i=0;i<(int)size;i++) {
+                    wchar_t *copyOfPaper = paperNames + (i * 64);
+                    if (value.toString() == QString::fromWCharArray(copyOfPaper, qwcsnlen(copyOfPaper, 64))) {
+                        paperPos = i;
+                        break;
+                    }
+                }
+                delete [] paperNames;
+                size = DeviceCapabilities(reinterpret_cast<const wchar_t*>(d->name.utf16()),
+                                          NULL, DC_PAPERS, NULL, NULL);
+                if ((int)size > 0) {
+                    wchar_t *papers = new wchar_t[size];
+                    size = DeviceCapabilities(reinterpret_cast<const wchar_t*>(d->name.utf16()),
+                                              NULL, DC_PAPERS, papers, NULL);
+                    QScopedArrayPointer<POINT> paperSizes(new POINT[size]);
+                    DWORD paperNameCount = DeviceCapabilities(reinterpret_cast<const wchar_t*>(d->name.utf16()), NULL, DC_PAPERSIZE, (wchar_t *)paperSizes.data(), NULL);
+                    if (paperNameCount == size) {
+                        const double multiplier = qt_multiplierForUnit(QPrinter::Millimeter, d->resolution);
+                        d->paper_size = QSizeF((paperSizes[paperPos].x / 10.0) * multiplier, (paperSizes[paperPos].y / 10.0) * multiplier);
+                        // Our sizes may not match the paper name's size exactly
+                        // So we treat it as custom so we know the paper size is correct
+                        d->has_custom_paper_size = true;
+                        d->devMode->dmPaperSize = papers[paperPos];
+                        setDevModePaperFlags(d->devMode, false);
+                        d->doReinit();
+                    }
 
+                    delete [] papers;
+                 }
+            }
+        }
+        break;
     case PPK_PaperSource:
         {
             if (!d->devMode)
@@ -1331,6 +1432,7 @@ void QWin32PrintEngine::setProperty(PrintEnginePropertyKey key, const QVariant &
             break;
         d->has_custom_paper_size = false;
         d->devMode->dmPaperSize = value.toInt();
+        setDevModePaperFlags(d->devMode, d->has_custom_paper_size);
         d->doReinit();
         break;
 
@@ -1340,30 +1442,18 @@ void QWin32PrintEngine::setProperty(PrintEnginePropertyKey key, const QVariant &
         d->paper_size = value.toSizeF();
         if (!d->devMode)
             break;
-        int orientation = d->devMode->dmOrientation;
-        DWORD needed = 0;
-        DWORD returned = 0;
-        if (!EnumForms(d->hPrinter, 1, 0, 0, &needed, &returned)) {
-            BYTE *forms = (BYTE *) malloc(needed);
-            if (EnumForms(d->hPrinter, 1, forms, needed, &needed, &returned)) {
-                for (DWORD i=0; i< returned; ++i) {
-                    FORM_INFO_1 *formArray = reinterpret_cast<FORM_INFO_1 *>(forms);
-                    // the form sizes are specified in 1000th of a mm,
-                    // convert the size to Points
-                    QSizeF size((formArray[i].Size.cx * 72/25.4)/1000.0,
-                                (formArray[i].Size.cy * 72/25.4)/1000.0);
-                    if (qAbs(d->paper_size.width() - size.width()) <= 2
-                        && qAbs(d->paper_size.height() - size.height()) <= 2)
-                    {
-                        d->devMode->dmPaperSize = i + 1;
-                        break;
-                    }
-                }
-            }
-            free(forms);
+        const QSizeF sizeMM = qt_SizeFromUnitToMillimeter(d->paper_size, QPrinter::Point, d->resolution);
+        const int match = findCustomPaperSize(sizeMM, d->name);
+        setDevModePaperFlags(d->devMode, (match >= 0) ? false : true);
+        if (match >= 0) {
+            d->devMode->dmPaperSize = match;
+            if (d->devMode->dmOrientation != DMORIENT_PORTRAIT)
+                qSwap(d->paper_size.rwidth(), d->paper_size.rheight());
+        } else {
+            d->devMode->dmPaperLength = qRound(sizeMM.height() * 10.0);
+            d->devMode->dmPaperWidth = qRound(sizeMM.width() * 10.0);
         }
-        if (orientation != DMORIENT_PORTRAIT)
-            d->paper_size = QSizeF(d->paper_size.height(), d->paper_size.width());
+        d->doReinit();
         break;
     }
 
@@ -1479,7 +1569,40 @@ QVariant QWin32PrintEngine::property(PrintEnginePropertyKey key) const
             value = QTransform(1/d->stretch_x, 0, 0, 1/d->stretch_y, 0, 0).mapRect(d->devPaperRect);
         }
         break;
+    case PPK_PaperName:
+        if (!d->devMode) {
+            value = QLatin1String("A4");
+        } else {
+            DWORD size = DeviceCapabilities(reinterpret_cast<const wchar_t*>(d->name.utf16()),
+                                        NULL, DC_PAPERS, NULL, NULL);
+            int paperSizePos = -1;
+            if ((int)size > 0) {
+                wchar_t *papers = new wchar_t[size];
+                size = DeviceCapabilities(reinterpret_cast<const wchar_t*>(d->name.utf16()),
+                                           NULL, DC_PAPERS, papers, NULL);
+                for (int i=0;i<(int)size;i++) {
+                    if (papers[i] == d->devMode->dmPaperSize) {
+                        paperSizePos = i;
+                        break;
+                    }
+                }
+                delete [] papers;
 
+            }
+            if (paperSizePos != -1) {
+                size = DeviceCapabilities(reinterpret_cast<const wchar_t*>(d->name.utf16()),
+                    NULL, DC_PAPERNAMES, NULL, NULL);
+                if ((int)size > 0) {
+                    wchar_t *papers = new wchar_t[size*64];
+                    size = DeviceCapabilities(reinterpret_cast<const wchar_t*>(d->name.utf16()),
+                        NULL, DC_PAPERNAMES, papers, NULL);
+                    wchar_t *copyOfPaper = papers + (paperSizePos * 64);
+                    value = QString::fromWCharArray(copyOfPaper, qwcsnlen(copyOfPaper, 64));
+                    delete [] papers;
+                }
+            }
+        }
+        break;
     case PPK_PaperSource:
         if (!d->devMode) {
             value = QPrinter::Auto;
@@ -1596,6 +1719,36 @@ QList<QPrinter::PaperSize> QWin32PrintEngine::supportedPaperSizes(const QPrinter
     return returnList;
 }
 
+QList<QPair<QString, QSizeF> > QWin32PrintEngine::supportedSizesWithNames(const QPrinterInfo &printerInfo)
+{
+    QList<QPair<QString, QSizeF> > paperSizes;
+    if (printerInfo.isNull())
+        return paperSizes;
+    DWORD size = DeviceCapabilities(reinterpret_cast<const wchar_t*>(printerInfo.printerName().utf16()),
+                                    NULL, DC_PAPERNAMES, NULL, NULL);
+    if ((int)size > 0) {
+        wchar_t *papers = new wchar_t[size*64];
+        size = DeviceCapabilities(reinterpret_cast<const wchar_t*>(printerInfo.printerName().utf16()),
+                                  NULL, DC_PAPERNAMES, papers, NULL);
+        DWORD size2 = DeviceCapabilities(reinterpret_cast<const wchar_t*>(printerInfo.printerName().utf16()),
+                                         NULL, DC_PAPERSIZE, NULL, NULL);
+        if ((int)size2 > 0) {
+            POINT *points = new POINT[size2*sizeof(POINT)];
+
+            size2 = DeviceCapabilities(reinterpret_cast<const wchar_t*>(printerInfo.printerName().utf16()),
+                                       NULL, DC_PAPERSIZE, (wchar_t *)points, NULL);
+            for (int i=0;i<(int)size;i++) {
+                wchar_t *paper = papers + (i * 64);
+                QString str = QString::fromWCharArray(paper, qwcsnlen(paper, 64));
+                paperSizes << qMakePair(str, QSizeF(points[i].x / 10.0, points[i].y / 10.0));
+            }
+            delete [] points;
+        }
+        delete [] papers;
+    }
+    return paperSizes;
+}
+
 void QWin32PrintEngine::queryDefaultPrinter(QString &name, QString &program, QString &port)
 {
     /* Read the default printer name, driver and port with the intuitive function
@@ -1702,8 +1855,9 @@ static void draw_text_item_win(const QPointF &pos, const QTextItemInt &ti, HDC h
     bool ttf = false;
 
     if (ti.fontEngine->type() == QFontEngine::Win) {
-        const QVariant hfontV = ti.fontEngine->property("hFont");
-        const QVariant ttfV = ti.fontEngine->property("trueType");
+        const QVariantMap userData = ti.fontEngine->userData().toMap();
+        const QVariant hfontV = userData.value(QStringLiteral("hFont"));
+        const QVariant ttfV = userData.value(QStringLiteral("trueType"));
         if (ttfV.type() == QVariant::Bool && hfontV.canConvert<HFONT>()) {
             hfont = hfontV.value<HFONT>();
             ttf = ttfV.toBool();
@@ -1802,30 +1956,20 @@ static void draw_text_item_win(const QPointF &pos, const QTextItemInt &ti, HDC h
     SelectObject(hdc, old_font);
 }
 
-
 void QWin32PrintEnginePrivate::updateCustomPaperSize()
 {
-    uint paperSize = devMode->dmPaperSize;
+    const uint paperSize = devMode->dmPaperSize;
+    has_custom_paper_size = false;
     if (paperSize > 0 && mapDevmodePaperSize(paperSize) == QPrinter::Custom) {
         has_custom_paper_size = true;
-        DWORD needed = 0;
-        DWORD returned = 0;
-        if (!EnumForms(hPrinter, 1, 0, 0, &needed, &returned)) {
-            BYTE *forms = (BYTE *) malloc(needed);
-            if (EnumForms(hPrinter, 1, forms, needed, &needed, &returned)) {
-                if (paperSize <= returned) {
-                    FORM_INFO_1 *formArray = (FORM_INFO_1 *) forms;
-                    int width = formArray[paperSize - 1].Size.cx; // 1/1000 of a mm
-                    int height = formArray[paperSize - 1].Size.cy; // 1/1000 of a mm
-                    paper_size = QSizeF((width * 72 /25.4) / 1000.0, (height * 72 / 25.4) / 1000.0);
-                } else {
-                    has_custom_paper_size = false;
-                }
+        const QList<QPair<QSizeF, int> > paperSizes = printerPaperSizes(name);
+        for (int i=0; i<paperSizes.size(); i++) {
+            if ((uint)paperSizes.at(i).second == paperSize) {
+                paper_size = paperSizes.at(paperSize).first;
+                has_custom_paper_size = false;
+                break;
             }
-            free(forms);
         }
-    } else {
-        has_custom_paper_size = false;
     }
 }
 
