@@ -58,6 +58,7 @@
 #ifndef QT_NO_PRINTER
 
 #include "QtCore/qlist.h"
+#include "QtCore/qpair.h"
 
 QT_BEGIN_NAMESPACE
 
@@ -65,13 +66,12 @@ class QPrinterInfoPrivate
 {
 public:
     QPrinterInfoPrivate(const QString& name = QString()) :
-        name(name), isDefault(false), index(-1), hasPaperSizes(false)
+        name(name), isDefault(false), index(-1), hasPaperSizes(false),
+        hasPaperNames(false)
     {}
     ~QPrinterInfoPrivate()
     {}
 
-    static QPrinterInfoPrivate shared_null;
-
     QString name;
     QString description;
     QString location;
@@ -81,17 +81,8 @@ public:
 
     mutable bool hasPaperSizes;
     mutable QList<QPrinter::PaperSize> paperSizes;
-};
-
-
-class QPrinterInfoPrivateDeleter
-{
-public:
-    static inline void cleanup(QPrinterInfoPrivate *d)
-    {
-        if (d != &QPrinterInfoPrivate::shared_null)
-            delete d;
-    }
+    mutable bool hasPaperNames;
+    mutable QList<QPair<QString, QSizeF> > paperNames;
 };
 
 QT_END_NAMESPACE
