@@ -162,7 +162,9 @@ signals:
 protected:
     void resizeEvent(QResizeEvent* e)
     {
+        const bool blocked = verticalScrollBar()->blockSignals(true); // Don't change page, QTBUG-14517
         QGraphicsView::resizeEvent(e);
+        verticalScrollBar()->blockSignals(blocked);
         emit resized();
     }
 
