@@ -39,7 +39,8 @@
 **
 ****************************************************************************/
 
-#include "qsql_db2.h"
+#include "qsql_db2_p.h"
+#include <QtSql/private/qsqldriver_p.h>
 #include <qcoreapplication.h>
 #include <qdatetime.h>
 #include <qsqlfield.h>
@@ -70,10 +71,10 @@ QT_BEGIN_NAMESPACE
 static const int COLNAMESIZE = 255;
 static const SQLSMALLINT qParamType[4] = { SQL_PARAM_INPUT, SQL_PARAM_INPUT, SQL_PARAM_OUTPUT, SQL_PARAM_INPUT_OUTPUT };
 
-class QDB2DriverPrivate
+class QDB2DriverPrivate : public QSqlDriverPrivate
 {
 public:
-    QDB2DriverPrivate(): hEnv(0), hDbc(0) {}
+    QDB2DriverPrivate() : QSqlDriverPrivate(), hEnv(0), hDbc(0) { dbmsType = DB2; }
     SQLHANDLE hEnv;
     SQLHANDLE hDbc;
     QString user;
@@ -632,7 +633,7 @@ bool QDB2Result::exec()
                 dt->day = qdt.day();
                 r = SQLBindParameter(d->hStmt,
                                      i + 1,
-                                     qParamType[(QFlag)(bindValueType(i)) & 3],
+                                     qParamType[bindValueType(i) & 3],
                                      SQL_C_DATE,
                                      SQL_DATE,
                                      0,
@@ -652,7 +653,7 @@ bool QDB2Result::exec()
                 dt->second = qdt.second();
                 r = SQLBindParameter(d->hStmt,
                                       i + 1,
-                                      qParamType[(QFlag)(bindValueType(i)) & 3],
+                                      qParamType[bindValueType(i) & 3],
                                       SQL_C_TIME,
                                       SQL_TIME,
                                       0,
@@ -676,7 +677,7 @@ bool QDB2Result::exec()
                 dt->fraction = qdt.time().msec() * 1000000;
                 r = SQLBindParameter(d->hStmt,
                                       i + 1,
-                                      qParamType[(QFlag)(bindValueType(i)) & 3],
+                                      qParamType[bindValueType(i) & 3],
                                       SQL_C_TIMESTAMP,
                                       SQL_TIMESTAMP,
                                       0,
@@ -689,7 +690,7 @@ bool QDB2Result::exec()
             case QVariant::Int:
                 r = SQLBindParameter(d->hStmt,
                                       i + 1,
-                                      qParamType[(QFlag)(bindValueType(i)) & 3],
+                                      qParamType[bindValueType(i) & 3],
                                       SQL_C_SLONG,
                                       SQL_INTEGER,
                                       0,
@@ -701,7 +702,7 @@ bool QDB2Result::exec()
             case QVariant::Double:
                 r = SQLBindParameter(d->hStmt,
                                       i + 1,
-                                      qParamType[(QFlag)(bindValueType(i)) & 3],
+                                      qParamType[bindValueType(i) & 3],
                                       SQL_C_DOUBLE,
                                       SQL_DOUBLE,
                                       0,
@@ -716,7 +717,7 @@ bool QDB2Result::exec()
                     *ind = len;
                 r = SQLBindParameter(d->hStmt,
                                       i + 1,
-                                      qParamType[(QFlag)(bindValueType(i)) & 3],
+                                      qParamType[bindValueType(i) & 3],
                                       SQL_C_BINARY,
                                       SQL_LONGVARBINARY,
                                       len,
@@ -734,7 +735,7 @@ bool QDB2Result::exec()
                     QByteArray ba((char*)str.utf16(), str.capacity() * sizeof(QChar));
                     r = SQLBindParameter(d->hStmt,
                                         i + 1,
-                                        qParamType[(QFlag)(bindValueType(i)) & 3],
+                                        qParamType[bindValueType(i) & 3],
                                         SQL_C_WCHAR,
                                         SQL_WVARCHAR,
                                         str.length(),
@@ -748,7 +749,7 @@ bool QDB2Result::exec()
                     int len = str.length();
                     r = SQLBindParameter(d->hStmt,
                                         i + 1,
-                                        qParamType[(QFlag)(bindValueType(i)) & 3],
+                                        qParamType[bindValueType(i) & 3],
                                         SQL_C_WCHAR,
                                         SQL_WVARCHAR,
                                         len,
@@ -766,7 +767,7 @@ bool QDB2Result::exec()
                     *ind = ba.length();
                 r = SQLBindParameter(d->hStmt,
                                       i + 1,
-                                      qParamType[(QFlag)(bindValueType(i)) & 3],
+                                      qParamType[bindValueType(i) & 3],
                                       SQL_C_CHAR,
                                       SQL_VARCHAR,
                                       len,
@@ -1131,15 +1132,14 @@ void QDB2Result::detachFromResultSet()
 /************************************/
 
 QDB2Driver::QDB2Driver(QObject* parent)
-    : QSqlDriver(parent)
+    : QSqlDriver(*new QDB2DriverPrivate, parent)
 {
-    d = new QDB2DriverPrivate;
 }
 
 QDB2Driver::QDB2Driver(Qt::HANDLE env, Qt::HANDLE con, QObject* parent)
-    : QSqlDriver(parent)
+    : QSqlDriver(*new QDB2DriverPrivate, parent)
 {
-    d = new QDB2DriverPrivate;
+    Q_D(QDB2Driver);
     d->hEnv = (SQLHANDLE)env;
     d->hDbc = (SQLHANDLE)con;
     if (env && con) {
@@ -1151,12 +1151,12 @@ QDB2Driver::QDB2Driver(Qt::HANDLE env, Qt::HANDLE con, QObject* parent)
 QDB2Driver::~QDB2Driver()
 {
     close();
-    delete d;
 }
 
 bool QDB2Driver::open(const QString& db, const QString& user, const QString& password, const QString& host, int port,
                        const QString& connOpts)
 {
+    Q_D(QDB2Driver);
     if (isOpen())
       close();
     SQLRETURN r;
@@ -1259,6 +1259,7 @@ bool QDB2Driver::open(const QString& db, const QString& user, const QString& pas
 
 void QDB2Driver::close()
 {
+    Q_D(QDB2Driver);
     SQLRETURN r;
     if (d->hDbc) {
         // Open statements/descriptors handles are automatically cleaned up by SQLDisconnect
@@ -1285,11 +1286,13 @@ void QDB2Driver::close()
 
 QSqlResult *QDB2Driver::createResult() const
 {
+    Q_D(const QDB2Driver);
     return new QDB2Result(this, d);
 }
 
 QSqlRecord QDB2Driver::record(const QString& tableName) const
 {
+    Q_D(const QDB2Driver);
     QSqlRecord fil;
     if (!isOpen())
         return fil;
@@ -1363,6 +1366,7 @@ QSqlRecord QDB2Driver::record(const QString& tableName) const
 
 QStringList QDB2Driver::tables(QSql::TableType type) const
 {
+    Q_D(const QDB2Driver);
     QStringList tl;
     if (!isOpen())
         return tl;
@@ -1434,6 +1438,7 @@ QStringList QDB2Driver::tables(QSql::TableType type) const
 
 QSqlIndex QDB2Driver::primaryIndex(const QString& tablename) const
 {
+    Q_D(const QDB2Driver);
     QSqlIndex index(tablename);
     if (!isOpen())
         return index;
@@ -1535,6 +1540,7 @@ bool QDB2Driver::beginTransaction()
 
 bool QDB2Driver::commitTransaction()
 {
+    Q_D(QDB2Driver);
     if (!isOpen()) {
         qWarning("QDB2Driver::commitTransaction: Database not open");
         return false;
@@ -1552,6 +1558,7 @@ bool QDB2Driver::commitTransaction()
 
 bool QDB2Driver::rollbackTransaction()
 {
+    Q_D(QDB2Driver);
     if (!isOpen()) {
         qWarning("QDB2Driver::rollbackTransaction: Database not open");
         return false;
@@ -1569,6 +1576,7 @@ bool QDB2Driver::rollbackTransaction()
 
 bool QDB2Driver::setAutoCommit(bool autoCommit)
 {
+    Q_D(QDB2Driver);
     SQLUINTEGER ac = autoCommit ? SQL_AUTOCOMMIT_ON : SQL_AUTOCOMMIT_OFF;
     SQLRETURN r  = SQLSetConnectAttr(d->hDbc,
                                       SQL_ATTR_AUTOCOMMIT,
@@ -1627,6 +1635,7 @@ QString QDB2Driver::formatValue(const QSqlField &field, bool trimStrings) const
 
 QVariant QDB2Driver::handle() const
 {
+    Q_D(const QDB2Driver);
     return QVariant(qRegisterMetaType<SQLHANDLE>("SQLHANDLE"), &d->hDbc);
 }
 
