@@ -39,7 +39,7 @@
 **
 ****************************************************************************/
 
-#include "qsql_ibase.h"
+#include "qsql_ibase_p.h"
 #include <qcoreapplication.h>
 #include <qdatetime.h>
 #include <qvariant.h>
@@ -47,6 +47,8 @@
 #include <qsqlfield.h>
 #include <qsqlindex.h>
 #include <qsqlquery.h>
+#include <QtSql/private/qsqlcachedresult_p.h>
+#include <QtSql/private/qsqldriver_p.h>
 #include <qlist.h>
 #include <qvector.h>
 #include <qtextcodec.h>
@@ -238,7 +240,7 @@ static ISC_TIMESTAMP toTimeStamp(const QDateTime &dt)
 static QDateTime fromTimeStamp(char *buffer)
 {
     static const QDate bd(1858, 11, 17);
-    QTime t;
+    QTime t(0, 0);
     QDate d;
 
     // have to demangle the structure ourselves because isc_decode_time
@@ -308,13 +310,15 @@ struct QIBaseEventBuffer {
     QIBaseSubscriptionState subscriptionState;
 };
 
-class QIBaseDriverPrivate
+class QIBaseDriverPrivate : public QSqlDriverPrivate
 {
+    Q_DECLARE_PUBLIC(QIBaseDriver)
 public:
-    QIBaseDriverPrivate(QIBaseDriver *d) : q(d), ibase(0), trans(0), tc(0) {}
+    QIBaseDriverPrivate() : QSqlDriverPrivate(), ibase(0), trans(0), tc(0) { dbmsType = Interbase; }
 
     bool isError(const char *msg, QSqlError::ErrorType typ = QSqlError::UnknownError)
     {
+        Q_Q(QIBaseDriver);
         QString imsg;
         ISC_LONG sqlcode;
         if (!getIBaseError(imsg, status, sqlcode, tc))
@@ -326,7 +330,6 @@ public:
     }
 
 public:
-    QIBaseDriver* q;
     isc_db_handle ibase;
     isc_tr_handle trans;
     QTextCodec *tc;
@@ -346,6 +349,31 @@ static void qFreeEventBuffer(QIBaseEventBuffer* eBuffer)
     delete eBuffer;
 }
 
+class QIBaseResultPrivate;
+
+class QIBaseResult : public QSqlCachedResult
+{
+    friend class QIBaseResultPrivate;
+
+public:
+    explicit QIBaseResult(const QIBaseDriver* db);
+    virtual ~QIBaseResult();
+
+    bool prepare(const QString& query);
+    bool exec();
+    QVariant handle() const;
+
+protected:
+    bool gotoNext(QSqlCachedResult::ValueCache& row, int rowIdx);
+    bool reset (const QString& query);
+    int size();
+    int numRowsAffected();
+    QSqlRecord record() const;
+
+private:
+    QIBaseResultPrivate* d;
+};
+
 class QIBaseResultPrivate
 {
 public:
@@ -391,9 +419,9 @@ public:
 
 
 QIBaseResultPrivate::QIBaseResultPrivate(QIBaseResult *d, const QIBaseDriver *ddb):
-    q(d), db(ddb), trans(0), stmt(0), ibase(ddb->d->ibase), sqlda(0), inda(0), queryType(-1), tc(ddb->d->tc)
+    q(d), db(ddb), trans(0), stmt(0), ibase(ddb->d_func()->ibase), sqlda(0), inda(0), queryType(-1), tc(ddb->d_func()->tc)
 {
-    localTransaction = (ddb->d->ibase == 0);
+    localTransaction = (ddb->d_func()->ibase == 0);
 }
 
 void QIBaseResultPrivate::cleanup()
@@ -834,9 +862,9 @@ bool QIBaseResultPrivate::transaction()
 {
     if (trans)
         return true;
-    if (db->d->trans) {
+    if (db->d_func()->trans) {
         localTransaction = false;
-        trans = db->d->trans;
+        trans = db->d_func()->trans;
         return true;
     }
     localTransaction = true;
@@ -1134,6 +1162,9 @@ bool QIBaseResult::gotoNext(QSqlCachedResult::ValueCache& row, int rowIdx)
                 case QSql::HighPrecision:
                     v.convert(QVariant::String);
                     break;
+                case QSql::LowPrecisionDouble:
+                    // no conversion
+                    break;
                 }
             }
             row[idx] = v;
@@ -1369,15 +1400,14 @@ QVariant QIBaseResult::handle() const
 /*********************************/
 
 QIBaseDriver::QIBaseDriver(QObject * parent)
-    : QSqlDriver(parent)
+    : QSqlDriver(*new QIBaseDriverPrivate, parent)
 {
-    d = new QIBaseDriverPrivate(this);
 }
 
 QIBaseDriver::QIBaseDriver(isc_db_handle connection, QObject *parent)
-    : QSqlDriver(parent)
+    : QSqlDriver(*new QIBaseDriverPrivate, parent)
 {
-    d = new QIBaseDriverPrivate(this);
+    Q_D(QIBaseDriver);
     d->ibase = connection;
     setOpen(true);
     setOpenError(false);
@@ -1385,7 +1415,6 @@ QIBaseDriver::QIBaseDriver(isc_db_handle connection, QObject *parent)
 
 QIBaseDriver::~QIBaseDriver()
 {
-    delete d;
 }
 
 bool QIBaseDriver::hasFeature(DriverFeature f) const
@@ -1398,6 +1427,7 @@ bool QIBaseDriver::hasFeature(DriverFeature f) const
     case SimpleLocking:
     case FinishQuery:
     case MultipleResultSets:
+    case CancelQuery:
         return false;
     case Transactions:
     case PreparedQueries:
@@ -1415,9 +1445,10 @@ bool QIBaseDriver::open(const QString & db,
           const QString & user,
           const QString & password,
           const QString & host,
-          int /*port*/,
+          int port,
           const QString & connOpts)
 {
+    Q_D(QIBaseDriver);
     if (isOpen())
         close();
 
@@ -1482,9 +1513,13 @@ bool QIBaseDriver::open(const QString & db,
         i += role.length();
     }
 
+    QString portString;
+    if (port != -1)
+        portString = QStringLiteral("/%1").arg(port);
+
     QString ldb;
     if (!host.isEmpty())
-        ldb += host + QLatin1Char(':');
+        ldb += host + portString + QLatin1Char(':');
     ldb += db;
     isc_attach_database(d->status, 0, const_cast<char *>(ldb.toLocal8Bit().constData()),
                         &d->ibase, i, ba.data());
@@ -1495,11 +1530,13 @@ bool QIBaseDriver::open(const QString & db,
     }
 
     setOpen(true);
+    setOpenError(false);
     return true;
 }
 
 void QIBaseDriver::close()
 {
+    Q_D(QIBaseDriver);
     if (isOpen()) {
 
         if (d->eventBuffers.size()) {
@@ -1536,6 +1573,7 @@ QSqlResult *QIBaseDriver::createResult() const
 
 bool QIBaseDriver::beginTransaction()
 {
+    Q_D(QIBaseDriver);
     if (!isOpen() || isOpenError())
         return false;
     if (d->trans)
@@ -1548,6 +1586,7 @@ bool QIBaseDriver::beginTransaction()
 
 bool QIBaseDriver::commitTransaction()
 {
+    Q_D(QIBaseDriver);
     if (!isOpen() || isOpenError())
         return false;
     if (!d->trans)
@@ -1561,6 +1600,7 @@ bool QIBaseDriver::commitTransaction()
 
 bool QIBaseDriver::rollbackTransaction()
 {
+    Q_D(QIBaseDriver);
     if (!isOpen() || isOpenError())
         return false;
     if (!d->trans)
@@ -1723,6 +1763,7 @@ QString QIBaseDriver::formatValue(const QSqlField &field, bool trimStrings) cons
 
 QVariant QIBaseDriver::handle() const
 {
+    Q_D(const QIBaseDriver);
     return QVariant(qRegisterMetaType<isc_db_handle>("isc_db_handle"), &d->ibase);
 }
 
@@ -1751,6 +1792,7 @@ static isc_callback qEventCallback(char *result, short length, char *updated)
 
 bool QIBaseDriver::subscribeToNotification(const QString &name)
 {
+    Q_D(QIBaseDriver);
     if (!isOpen()) {
         qWarning("QIBaseDriver::subscribeFromNotificationImplementation: database not open.");
         return false;
@@ -1800,6 +1842,7 @@ bool QIBaseDriver::subscribeToNotification(const QString &name)
 
 bool QIBaseDriver::unsubscribeFromNotification(const QString &name)
 {
+    Q_D(QIBaseDriver);
     if (!isOpen()) {
         qWarning("QIBaseDriver::unsubscribeFromNotificationImplementation: database not open.");
         return false;
@@ -1829,11 +1872,13 @@ bool QIBaseDriver::unsubscribeFromNotification(const QString &name)
 
 QStringList QIBaseDriver::subscribedToNotifications() const
 {
+    Q_D(const QIBaseDriver);
     return QStringList(d->eventBuffers.keys());
 }
 
 void QIBaseDriver::qHandleEventNotification(void *updatedResultBuffer)
 {
+    Q_D(QIBaseDriver);
     QMap<QString, QIBaseEventBuffer *>::const_iterator i;
     for (i = d->eventBuffers.constBegin(); i != d->eventBuffers.constEnd(); ++i) {
         QIBaseEventBuffer* eBuffer = i.value();
