@@ -39,7 +39,7 @@
 **
 ****************************************************************************/
 
-#include "qsql_odbc.h"
+#include "qsql_odbc_p.h"
 #include <qsqlrecord.h>
 
 #if defined (Q_OS_WIN32)
@@ -57,6 +57,7 @@
 #include <qmath.h>
 #include <QDebug>
 #include <QSqlQuery>
+#include <QtSql/private/qsqldriver_p.h>
 
 QT_BEGIN_NAMESPACE
 
@@ -111,14 +112,13 @@ inline static QVarLengthArray<SQLTCHAR> toSQLTCHAR(const QString &input)
     return result;
 }
 
-class QODBCDriverPrivate
+class QODBCDriverPrivate : public QSqlDriverPrivate
 {
 public:
     enum DefaultCase{Lower, Mixed, Upper, Sensitive};
     QODBCDriverPrivate()
-    : hEnv(0), hDbc(0), unicode(false), useSchema(false), disconnectCount(0), datetime_precision(19), isMySqlServer(false),
-           isMSSqlServer(false), isFreeTDSDriver(false), hasSQLFetchScroll(true),
-           hasMultiResultSets(false), isQuoteInitialized(false), quote(QLatin1Char('"'))
+    : QSqlDriverPrivate(), hEnv(0), hDbc(0), unicode(false), useSchema(false), disconnectCount(0), datetime_precision(19),
+      isFreeTDSDriver(false), hasSQLFetchScroll(true), hasMultiResultSets(false), isQuoteInitialized(false), quote(QLatin1Char('"'))
     {
     }
 
@@ -129,15 +129,13 @@ public:
     bool useSchema;
     int disconnectCount;
     int datetime_precision;
-    bool isMySqlServer;
-    bool isMSSqlServer;
     bool isFreeTDSDriver;
     bool hasSQLFetchScroll;
     bool hasMultiResultSets;
 
     bool checkDriver() const;
     void checkUnicode();
-    void checkSqlServer();
+    void checkDBMS();
     void checkHasSQLFetchScroll();
     void checkHasMultiResults();
     void checkSchemaUsage();
@@ -190,13 +188,13 @@ public:
 bool QODBCPrivate::isStmtHandleValid(const QSqlDriver *driver)
 {
     const QODBCDriver *odbcdriver = static_cast<const QODBCDriver*> (driver);
-    return disconnectCount == odbcdriver->d->disconnectCount;
+    return disconnectCount == odbcdriver->d_func()->disconnectCount;
 }
 
 void QODBCPrivate::updateStmtHandleState(const QSqlDriver *driver)
 {
     const QODBCDriver *odbcdriver = static_cast<const QODBCDriver*> (driver);
-    disconnectCount = odbcdriver->d->disconnectCount;
+    disconnectCount = odbcdriver->d_func()->disconnectCount;
 }
 
 static QString qWarnODBCHandle(int handleType, SQLHANDLE handle, int *nativeCode = 0)
@@ -960,14 +958,8 @@ bool QODBCResult::reset (const QString& query)
         return false;
     }
 
-    if(r == SQL_NO_DATA) {
-        setSelect(false);
-        return true;
-    }
-
-    SQLINTEGER bufferLength;
-    SQLULEN isScrollable;
-    r = SQLGetStmtAttr(d->hStmt, SQL_ATTR_CURSOR_SCROLLABLE, &isScrollable, SQL_IS_INTEGER, &bufferLength);
+    SQLULEN isScrollable = 0;
+    r = SQLGetStmtAttr(d->hStmt, SQL_ATTR_CURSOR_SCROLLABLE, &isScrollable, SQL_IS_INTEGER, 0);
     if(r == SQL_SUCCESS || r == SQL_SUCCESS_WITH_INFO)
         QSqlResult::setForwardOnly(isScrollable==SQL_NONSCROLLABLE);
 
@@ -1113,7 +1105,7 @@ bool QODBCResult::fetchLast()
                 "Unable to fetch last"), QSqlError::ConnectionError, d));
         return false;
     }
-    SQLINTEGER currRow;
+    SQLULEN currRow = 0;
     r = SQLGetStmtAttr(d->hStmt,
                         SQL_ROW_NUMBER,
                         &currRow,
@@ -1357,7 +1349,7 @@ bool QODBCResult::exec()
                 dt->day = qdt.day();
                 r = SQLBindParameter(d->hStmt,
                                       i + 1,
-                                      qParamType[(QFlag)(bindValueType(i)) & QSql::InOut],
+                                      qParamType[bindValueType(i) & QSql::InOut],
                                       SQL_C_DATE,
                                       SQL_DATE,
                                       0,
@@ -1377,7 +1369,7 @@ bool QODBCResult::exec()
                 dt->second = qdt.second();
                 r = SQLBindParameter(d->hStmt,
                                       i + 1,
-                                      qParamType[(QFlag)(bindValueType(i)) & QSql::InOut],
+                                      qParamType[bindValueType(i) & QSql::InOut],
                                       SQL_C_TIME,
                                       SQL_TIME,
                                       0,
@@ -1407,12 +1399,12 @@ bool QODBCResult::exec()
 
                     // (How many leading digits do we want to keep?  With SQL Server 2005, this should be 3: 123000000)
                     int keep = (int)qPow(10.0, 9 - qMin(9, precision));
-                    dt->fraction /= keep * keep;
+                    dt->fraction = (dt->fraction / keep) * keep;
                 }
 
                 r = SQLBindParameter(d->hStmt,
                                       i + 1,
-                                      qParamType[(QFlag)(bindValueType(i)) & QSql::InOut],
+                                      qParamType[bindValueType(i) & QSql::InOut],
                                       SQL_C_TIMESTAMP,
                                       SQL_TIMESTAMP,
                                       d->driverPrivate->datetime_precision,
@@ -1425,7 +1417,7 @@ bool QODBCResult::exec()
             case QVariant::Int:
                 r = SQLBindParameter(d->hStmt,
                                       i + 1,
-                                      qParamType[(QFlag)(bindValueType(i)) & QSql::InOut],
+                                      qParamType[bindValueType(i) & QSql::InOut],
                                       SQL_C_SLONG,
                                       SQL_INTEGER,
                                       0,
@@ -1437,7 +1429,7 @@ bool QODBCResult::exec()
             case QVariant::UInt:
                 r = SQLBindParameter(d->hStmt,
                                       i + 1,
-                                      qParamType[(QFlag)(bindValueType(i)) & QSql::InOut],
+                                      qParamType[bindValueType(i) & QSql::InOut],
                                       SQL_C_ULONG,
                                       SQL_NUMERIC,
                                       15,
@@ -1449,7 +1441,7 @@ bool QODBCResult::exec()
             case QVariant::Double:
                 r = SQLBindParameter(d->hStmt,
                                       i + 1,
-                                      qParamType[(QFlag)(bindValueType(i)) & QSql::InOut],
+                                      qParamType[bindValueType(i) & QSql::InOut],
                                       SQL_C_DOUBLE,
                                       SQL_DOUBLE,
                                       0,
@@ -1461,7 +1453,7 @@ bool QODBCResult::exec()
             case QVariant::LongLong:
                 r = SQLBindParameter(d->hStmt,
                                       i + 1,
-                                      qParamType[(QFlag)(bindValueType(i)) & QSql::InOut],
+                                      qParamType[bindValueType(i) & QSql::InOut],
                                       SQL_C_SBIGINT,
                                       SQL_BIGINT,
                                       0,
@@ -1473,7 +1465,7 @@ bool QODBCResult::exec()
             case QVariant::ULongLong:
                 r = SQLBindParameter(d->hStmt,
                                       i + 1,
-                                      qParamType[(QFlag)(bindValueType(i)) & QSql::InOut],
+                                      qParamType[bindValueType(i) & QSql::InOut],
                                       SQL_C_UBIGINT,
                                       SQL_BIGINT,
                                       0,
@@ -1488,7 +1480,7 @@ bool QODBCResult::exec()
                 }
                 r = SQLBindParameter(d->hStmt,
                                       i + 1,
-                                      qParamType[(QFlag)(bindValueType(i)) & QSql::InOut],
+                                      qParamType[bindValueType(i) & QSql::InOut],
                                       SQL_C_BINARY,
                                       SQL_LONGVARBINARY,
                                       val.toByteArray().size(),
@@ -1500,7 +1492,7 @@ bool QODBCResult::exec()
             case QVariant::Bool:
                 r = SQLBindParameter(d->hStmt,
                                       i + 1,
-                                      qParamType[(QFlag)(bindValueType(i)) & QSql::InOut],
+                                      qParamType[bindValueType(i) & QSql::InOut],
                                       SQL_C_BIT,
                                       SQL_BIT,
                                       0,
@@ -1521,7 +1513,7 @@ bool QODBCResult::exec()
                         QByteArray ba((const char *)a.constData(), a.size() * sizeof(SQLTCHAR));
                         r = SQLBindParameter(d->hStmt,
                                             i + 1,
-                                            qParamType[(QFlag)(bindValueType(i)) & QSql::InOut],
+                                            qParamType[bindValueType(i) & QSql::InOut],
                                             SQL_C_TCHAR,
                                             strSize > 254 ? SQL_WLONGVARCHAR : SQL_WVARCHAR,
                                             0, // god knows... don't change this!
@@ -1535,7 +1527,7 @@ bool QODBCResult::exec()
                     QByteArray strba((const char *)toSQLTCHAR(str).constData(), str.size()*sizeof(SQLTCHAR));
                     r = SQLBindParameter(d->hStmt,
                                           i + 1,
-                                          qParamType[(QFlag)(bindValueType(i)) & QSql::InOut],
+                                          qParamType[bindValueType(i) & QSql::InOut],
                                           SQL_C_TCHAR,
                                           strSize > 254 ? SQL_WLONGVARCHAR : SQL_WVARCHAR,
                                           strSize,
@@ -1555,7 +1547,7 @@ bool QODBCResult::exec()
 
                     r = SQLBindParameter(d->hStmt,
                                           i + 1,
-                                          qParamType[(QFlag)(bindValueType(i)) & QSql::InOut],
+                                          qParamType[bindValueType(i) & QSql::InOut],
                                           SQL_C_CHAR,
                                           strSize > 254 ? SQL_LONGVARCHAR : SQL_VARCHAR,
                                           strSize,
@@ -1573,7 +1565,7 @@ bool QODBCResult::exec()
                     *ind = ba.size();
                 r = SQLBindParameter(d->hStmt,
                                       i + 1,
-                                      qParamType[(QFlag)(bindValueType(i)) & QSql::InOut],
+                                      qParamType[bindValueType(i) & QSql::InOut],
                                       SQL_C_BINARY,
                                       SQL_VARBINARY,
                                       ba.length() + 1,
@@ -1592,16 +1584,15 @@ bool QODBCResult::exec()
         }
     }
     r = SQLExecute(d->hStmt);
-    if (r != SQL_SUCCESS && r != SQL_SUCCESS_WITH_INFO) {
+    if (r != SQL_SUCCESS && r != SQL_SUCCESS_WITH_INFO && r != SQL_NO_DATA) {
         qWarning() << "QODBCResult::exec: Unable to execute statement:" << qODBCWarn(d);
         setLastError(qMakeError(QCoreApplication::translate("QODBCResult",
                      "Unable to execute statement"), QSqlError::StatementError, d));
         return false;
     }
 
-    SQLINTEGER bufferLength;
-    SQLULEN isScrollable;
-    r = SQLGetStmtAttr(d->hStmt, SQL_ATTR_CURSOR_SCROLLABLE, &isScrollable, SQL_IS_INTEGER, &bufferLength);
+    SQLULEN isScrollable = 0;
+    r = SQLGetStmtAttr(d->hStmt, SQL_ATTR_CURSOR_SCROLLABLE, &isScrollable, SQL_IS_INTEGER, 0);
     if(r == SQL_SUCCESS || r == SQL_SUCCESS_WITH_INFO)
         QSqlResult::setForwardOnly(isScrollable==SQL_NONSCROLLABLE);
 
@@ -1677,6 +1668,38 @@ QSqlRecord QODBCResult::record() const
     return d->rInf;
 }
 
+QVariant QODBCResult::lastInsertId() const
+{
+    QString sql;
+
+    switch (d->driverPrivate->dbmsType) {
+    case QODBCDriverPrivate::MSSqlServer:
+    case QODBCDriverPrivate::Sybase:
+        sql = QLatin1String("SELECT @@IDENTITY;");
+        break;
+    case QODBCDriverPrivate::MySqlServer:
+        sql = QLatin1String("SELECT LAST_INSERT_ID();");
+        break;
+    case QODBCDriverPrivate::PostgreSQL:
+        sql = QLatin1String("SELECT lastval();");
+        break;
+    default:
+        break;
+    }
+
+    if (!sql.isEmpty()) {
+        QSqlQuery qry(driver()->createResult());
+        if (qry.exec(sql) && qry.next())
+            return qry.value(0);
+
+        qSqlWarning(QLatin1String("QODBCResult::lastInsertId: Unable to get lastInsertId"), d);
+    } else {
+        qSqlWarning(QLatin1String("QODBCResult::lastInsertId: not implemented for this DBMS"), d);
+    }
+
+    return QVariant();
+}
+
 QVariant QODBCResult::handle() const
 {
     return QVariant(qRegisterMetaType<SQLHANDLE>("SQLHANDLE"), &d->hStmt);
@@ -1742,15 +1765,14 @@ void QODBCResult::setForwardOnly(bool forward)
 
 
 QODBCDriver::QODBCDriver(QObject *parent)
-    : QSqlDriver(parent)
+    : QSqlDriver(*new QODBCDriverPrivate, parent)
 {
-    init();
 }
 
-QODBCDriver::QODBCDriver(SQLHANDLE env, SQLHANDLE con, QObject * parent)
-    : QSqlDriver(parent)
+QODBCDriver::QODBCDriver(SQLHANDLE env, SQLHANDLE con, QObject *parent)
+    : QSqlDriver(*new QODBCDriverPrivate, parent)
 {
-    init();
+    Q_D(QODBCDriver);
     d->hEnv = env;
     d->hDbc = con;
     if (env && con) {
@@ -1759,19 +1781,14 @@ QODBCDriver::QODBCDriver(SQLHANDLE env, SQLHANDLE con, QObject * parent)
     }
 }
 
-void QODBCDriver::init()
-{
-    d = new QODBCDriverPrivate();
-}
-
 QODBCDriver::~QODBCDriver()
 {
     cleanup();
-    delete d;
 }
 
 bool QODBCDriver::hasFeature(DriverFeature f) const
 {
+    Q_D(const QODBCDriver);
     switch (f) {
     case Transactions: {
         if (!d->hDbc)
@@ -1797,16 +1814,20 @@ bool QODBCDriver::hasFeature(DriverFeature f) const
         return true;
     case QuerySize:
     case NamedPlaceholders:
-    case LastInsertId:
     case BatchOperations:
     case SimpleLocking:
     case EventNotifications:
     case CancelQuery:
         return false;
+    case LastInsertId:
+        return (d->dbmsType == QODBCDriverPrivate::MSSqlServer)
+                || (d->dbmsType == QODBCDriverPrivate::Sybase)
+                || (d->dbmsType == QODBCDriverPrivate::MySqlServer)
+                || (d->dbmsType == QODBCDriverPrivate::PostgreSQL);
     case MultipleResultSets:
         return d->hasMultiResultSets;
     case BLOB: {
-        if(d->isMySqlServer)
+        if (d->dbmsType == QODBCDriverPrivate::MySqlServer)
             return true;
         else
             return false;
@@ -1822,6 +1843,7 @@ bool QODBCDriver::open(const QString & db,
                         int,
                         const QString& connOpts)
 {
+    Q_D(QODBCDriver);
     if (isOpen())
       close();
     SQLRETURN r;
@@ -1896,13 +1918,13 @@ bool QODBCDriver::open(const QString & db,
 
     d->checkUnicode();
     d->checkSchemaUsage();
-    d->checkSqlServer();
+    d->checkDBMS();
     d->checkHasSQLFetchScroll();
     d->checkHasMultiResults();
     d->checkDateTimePrecision();
     setOpen(true);
     setOpenError(false);
-    if(d->isMSSqlServer) {
+    if (d->dbmsType == QODBCDriverPrivate::MSSqlServer) {
         QSqlQuery i(createResult());
         i.exec(QLatin1String("SET QUOTED_IDENTIFIER ON"));
     }
@@ -1918,9 +1940,8 @@ void QODBCDriver::close()
 
 void QODBCDriver::cleanup()
 {
+    Q_D(QODBCDriver);
     SQLRETURN r;
-    if (!d)
-        return;
 
     if(d->hDbc) {
         // Open statements/descriptors handles are automatically cleaned up by SQLDisconnect
@@ -2069,7 +2090,7 @@ void QODBCDriverPrivate::checkSchemaUsage()
         useSchema = (val != 0);
 }
 
-void QODBCDriverPrivate::checkSqlServer()
+void QODBCDriverPrivate::checkDBMS()
 {
     SQLRETURN   r;
     QVarLengthArray<SQLTCHAR> serverString(200);
@@ -2088,8 +2109,16 @@ void QODBCDriverPrivate::checkSqlServer()
 #else
         serverType = QString::fromUtf8((const char *)serverString.constData(), t);
 #endif
-        isMySqlServer = serverType.contains(QLatin1String("mysql"), Qt::CaseInsensitive);
-        isMSSqlServer = serverType.contains(QLatin1String("Microsoft SQL Server"), Qt::CaseInsensitive);
+        if (serverType.contains(QLatin1String("PostgreSQL"), Qt::CaseInsensitive))
+            dbmsType = PostgreSQL;
+        else if (serverType.contains(QLatin1String("Oracle"), Qt::CaseInsensitive))
+            dbmsType = Oracle;
+        else if (serverType.contains(QLatin1String("MySql"), Qt::CaseInsensitive))
+            dbmsType = MySqlServer;
+        else if (serverType.contains(QLatin1String("Microsoft SQL Server"), Qt::CaseInsensitive))
+            dbmsType = MSSqlServer;
+        else if (serverType.contains(QLatin1String("Sybase"), Qt::CaseInsensitive))
+            dbmsType = Sybase;
     }
     r = SQLGetInfo(hDbc,
                    SQL_DRIVER_NAME,
@@ -2160,11 +2189,13 @@ void QODBCDriverPrivate::checkDateTimePrecision()
 
 QSqlResult *QODBCDriver::createResult() const
 {
-    return new QODBCResult(this, d);
+    Q_D(const QODBCDriver);
+    return new QODBCResult(this, const_cast<QODBCDriverPrivate*>(d));
 }
 
 bool QODBCDriver::beginTransaction()
 {
+    Q_D(QODBCDriver);
     if (!isOpen()) {
         qWarning() << "QODBCDriver::beginTransaction: Database not open";
         return false;
@@ -2184,6 +2215,7 @@ bool QODBCDriver::beginTransaction()
 
 bool QODBCDriver::commitTransaction()
 {
+    Q_D(QODBCDriver);
     if (!isOpen()) {
         qWarning() << "QODBCDriver::commitTransaction: Database not open";
         return false;
@@ -2201,6 +2233,7 @@ bool QODBCDriver::commitTransaction()
 
 bool QODBCDriver::rollbackTransaction()
 {
+    Q_D(QODBCDriver);
     if (!isOpen()) {
         qWarning() << "QODBCDriver::rollbackTransaction: Database not open";
         return false;
@@ -2218,6 +2251,7 @@ bool QODBCDriver::rollbackTransaction()
 
 bool QODBCDriver::endTrans()
 {
+    Q_D(QODBCDriver);
     SQLUINTEGER ac(SQL_AUTOCOMMIT_ON);
     SQLRETURN r  = SQLSetConnectAttr(d->hDbc,
                                       SQL_ATTR_AUTOCOMMIT,
@@ -2232,6 +2266,7 @@ bool QODBCDriver::endTrans()
 
 QStringList QODBCDriver::tables(QSql::TableType type) const
 {
+    Q_D(const QODBCDriver);
     QStringList tl;
     if (!isOpen())
         return tl;
@@ -2309,6 +2344,7 @@ QStringList QODBCDriver::tables(QSql::TableType type) const
 
 QSqlIndex QODBCDriver::primaryIndex(const QString& tablename) const
 {
+    Q_D(const QODBCDriver);
     QSqlIndex index(tablename);
     if (!isOpen())
         return index;
@@ -2324,7 +2360,7 @@ QSqlIndex QODBCDriver::primaryIndex(const QString& tablename) const
         return index;
     }
     QString catalog, schema, table;
-    d->splitTableQualifier(tablename, catalog, schema, table);
+    const_cast<QODBCDriverPrivate*>(d)->splitTableQualifier(tablename, catalog, schema, table);
 
     if (isIdentifierEscaped(catalog, QSqlDriver::TableName))
         catalog = stripDelimiters(catalog, QSqlDriver::TableName);
@@ -2436,13 +2472,14 @@ QSqlIndex QODBCDriver::primaryIndex(const QString& tablename) const
 
 QSqlRecord QODBCDriver::record(const QString& tablename) const
 {
+    Q_D(const QODBCDriver);
     QSqlRecord fil;
     if (!isOpen())
         return fil;
 
     SQLHANDLE hStmt;
     QString catalog, schema, table;
-    d->splitTableQualifier(tablename, catalog, schema, table);
+    const_cast<QODBCDriverPrivate*>(d)->splitTableQualifier(tablename, catalog, schema, table);
 
     if (isIdentifierEscaped(catalog, QSqlDriver::TableName))
         catalog = stripDelimiters(catalog, QSqlDriver::TableName);
@@ -2561,12 +2598,14 @@ QString QODBCDriver::formatValue(const QSqlField &field,
 
 QVariant QODBCDriver::handle() const
 {
+    Q_D(const QODBCDriver);
     return QVariant(qRegisterMetaType<SQLHANDLE>("SQLHANDLE"), &d->hDbc);
 }
 
 QString QODBCDriver::escapeIdentifier(const QString &identifier, IdentifierType) const
 {
-    QChar quote = d->quoteChar();
+    Q_D(const QODBCDriver);
+    QChar quote = const_cast<QODBCDriverPrivate*>(d)->quoteChar();
     QString res = identifier;
     if(!identifier.isEmpty() && !identifier.startsWith(quote) && !identifier.endsWith(quote) ) {
         res.replace(quote, QString(quote)+QString(quote));
@@ -2578,7 +2617,8 @@ QString QODBCDriver::escapeIdentifier(const QString &identifier, IdentifierType)
 
 bool QODBCDriver::isIdentifierEscaped(const QString &identifier, IdentifierType) const
 {
-    QChar quote = d->quoteChar();
+    Q_D(const QODBCDriver);
+    QChar quote = const_cast<QODBCDriverPrivate*>(d)->quoteChar();
     return identifier.size() > 2
         && identifier.startsWith(quote) //left delimited
         && identifier.endsWith(quote); //right delimited
