@@ -39,7 +39,7 @@
 **
 ****************************************************************************/
 
-#include "qsql_tds.h"
+#include "qsql_tds_p.h"
 
 #include <qglobal.h>
 #ifdef Q_OS_WIN32    // We assume that MS SQL Server is used. Set Q_USE_SYBASE to force Sybase.
@@ -58,6 +58,8 @@
 #include <qsqlfield.h>
 #include <qsqlindex.h>
 #include <qsqlquery.h>
+#include <QtSql/private/qsqlcachedresult_p.h>
+#include <QtSql/private/qsqldriver_p.h>
 #include <qstringlist.h>
 #include <qvector.h>
 
@@ -135,16 +137,43 @@ QSqlError qMakeError(const QString& err, QSqlError::ErrorType type, int errNo =
     return QSqlError(QLatin1String("QTDS: ") + err, QString(), type, errNo);
 }
 
-class QTDSDriverPrivate
+class QTDSDriverPrivate : public QSqlDriverPrivate
 {
 public:
-    QTDSDriverPrivate(): login(0), initialized(false) {}
+    QTDSDriverPrivate() : QSqlDriverPrivate(), login(0), initialized(false) { dbmsType = Sybase; }
     LOGINREC* login;  // login information
     QString hostName;
     QString db;
     bool initialized;
 };
 
+struct QTDSColumnData
+{
+    void *data;
+    DBINT nullbind;
+};
+Q_DECLARE_TYPEINFO(QTDSColumnData, Q_MOVABLE_TYPE);
+
+class QTDSResultPrivate;
+
+class QTDSResult : public QSqlCachedResult
+{
+public:
+    explicit QTDSResult(const QTDSDriver* db);
+    ~QTDSResult();
+    QVariant handle() const;
+
+protected:
+    void cleanup();
+    bool reset (const QString& query);
+    int size();
+    int numRowsAffected();
+    bool gotoNext(QSqlCachedResult::ValueCache &values, int index);
+    QSqlRecord record() const;
+
+private:
+    QTDSResultPrivate* d;
+};
 
 class QTDSResultPrivate
 {
@@ -156,7 +185,7 @@ public:
     void addErrorMsg(QString& errMsg) { errorMsgs.append(errMsg); }
     QString getErrorMsgs() { return errorMsgs.join(QLatin1String("\n")); }
     void clearErrorMsgs() { errorMsgs.clear(); }
-    QVector<void *> buffer;
+    QVector<QTDSColumnData> buffer;
     QSqlRecord rec;
 
 private:
@@ -298,12 +327,12 @@ QTDSResult::QTDSResult(const QTDSDriver* db)
     : QSqlCachedResult(db)
 {
     d = new QTDSResultPrivate();
-    d->login = db->d->login;
+    d->login = db->d_func()->login;
 
-    d->dbproc = dbopen(d->login, const_cast<char*>(db->d->hostName.toLatin1().constData()));
+    d->dbproc = dbopen(d->login, const_cast<char*>(db->d_func()->hostName.toLatin1().constData()));
     if (!d->dbproc)
         return;
-    if (dbuse(d->dbproc, const_cast<char*>(db->d->db.toLatin1().constData())) == FAIL)
+    if (dbuse(d->dbproc, const_cast<char*>(db->d_func()->db.toLatin1().constData())) == FAIL)
         return;
 
     // insert d in error handler dict
@@ -325,8 +354,8 @@ void QTDSResult::cleanup()
 {
     d->clearErrorMsgs();
     d->rec.clear();
-    for (int i = 0; i < d->buffer.size() / 2; ++i)
-        free(d->buffer.at(i * 2));
+    for (int i = 0; i < d->buffer.size(); ++i)
+        free(d->buffer.at(i).data);
     d->buffer.clear();
     // "can" stands for "cancel"... very clever.
     dbcanquery(d->dbproc);
@@ -340,9 +369,9 @@ QVariant QTDSResult::handle() const
     return QVariant(qRegisterMetaType<DBPROCESS *>("DBPROCESS*"), &d->dbproc);
 }
 
-static inline bool qIsNull(const void *ind)
+static inline bool qIsNull(const QTDSColumnData &p)
 {
-    return *reinterpret_cast<const DBINT *>(&ind) == -1;
+    return p.nullbind == -1;
 }
 
 bool QTDSResult::gotoNext(QSqlCachedResult::ValueCache &values, int index)
@@ -364,33 +393,33 @@ bool QTDSResult::gotoNext(QSqlCachedResult::ValueCache &values, int index)
         int idx = index + i;
         switch (d->rec.field(i).type()) {
             case QVariant::DateTime:
-                if (qIsNull(d->buffer.at(i * 2 + 1))) {
+                if (qIsNull(d->buffer.at(i))) {
                     values[idx] = QVariant(QVariant::DateTime);
                 } else {
-                    DBDATETIME *bdt = (DBDATETIME*) d->buffer.at(i * 2);
+                    DBDATETIME *bdt = (DBDATETIME*) d->buffer.at(i).data;
                     QDate date = QDate::fromString(QLatin1String("1900-01-01"), Qt::ISODate);
                     QTime time = QTime::fromString(QLatin1String("00:00:00"), Qt::ISODate);
                     values[idx] = QDateTime(date.addDays(bdt->dtdays), time.addMSecs(int(bdt->dttime / 0.3)));
                 }
                 break;
             case QVariant::Int:
-                if (qIsNull(d->buffer.at(i * 2 + 1)))
+                if (qIsNull(d->buffer.at(i)))
                     values[idx] = QVariant(QVariant::Int);
                 else
-                    values[idx] = *((int*)d->buffer.at(i * 2));
+                    values[idx] = *((int*)d->buffer.at(i).data);
                 break;
             case QVariant::Double:
             case QVariant::String:
-                if (qIsNull(d->buffer.at(i * 2 + 1)))
+                if (qIsNull(d->buffer.at(i)))
                     values[idx] = QVariant(QVariant::String);
                 else
-                    values[idx] = QString::fromLocal8Bit((const char*)d->buffer.at(i * 2)).trimmed();
+                    values[idx] = QString::fromLocal8Bit((const char*)d->buffer.at(i).data).trimmed();
                 break;
             case QVariant::ByteArray: {
-                if (qIsNull(d->buffer.at(i * 2 + 1)))
+                if (qIsNull(d->buffer.at(i)))
                     values[idx] = QVariant(QVariant::ByteArray);
                 else
-                    values[idx] = QByteArray((const char*)d->buffer.at(i * 2));
+                    values[idx] = QByteArray((const char*)d->buffer.at(i).data);
                 break;
             }
             default:
@@ -430,7 +459,7 @@ bool QTDSResult::reset (const QString& query)
     setSelect((DBCMDROW(d->dbproc) == SUCCEED)); // decide whether or not we are dealing with a SELECT query
     int numCols = dbnumcols(d->dbproc);
     if (numCols > 0) {
-        d->buffer.resize(numCols * 2);
+        d->buffer.resize(numCols);
         init(numCols);
     }
     for (int i = 0; i < numCols; ++i) {
@@ -470,11 +499,11 @@ bool QTDSResult::reset (const QString& query)
             break;
         }
         if (ret == SUCCEED) {
-            d->buffer[i * 2] = p;
-            ret = dbnullbind(d->dbproc, i+1, (DBINT*)(&d->buffer[i * 2 + 1]));
+            d->buffer[i].data = p;
+            ret = dbnullbind(d->dbproc, i+1, &d->buffer[i].nullbind);
         } else {
-            d->buffer[i * 2] = 0;
-            d->buffer[i * 2 + 1] = 0;
+            d->buffer[i].data = 0;
+            d->buffer[i].nullbind = 0;
             free(p);
         }
         if ((ret != SUCCEED) && (ret != -1)) {
@@ -512,14 +541,15 @@ QSqlRecord QTDSResult::record() const
 ///////////////////////////////////////////////////////////////////
 
 QTDSDriver::QTDSDriver(QObject* parent)
-    : QSqlDriver(parent)
+    : QSqlDriver(*new QTDSDriverPrivate, parent)
 {
     init();
 }
 
 QTDSDriver::QTDSDriver(LOGINREC* rec, const QString& host, const QString &db, QObject* parent)
-    : QSqlDriver(parent)
+    : QSqlDriver(*new QTDSDriverPrivate, parent)
 {
+    Q_D(QTDSDriver);
     init();
     d->login = rec;
     d->hostName = host;
@@ -532,12 +562,13 @@ QTDSDriver::QTDSDriver(LOGINREC* rec, const QString& host, const QString &db, QO
 
 QVariant QTDSDriver::handle() const
 {
+    Q_D(const QTDSDriver);
     return QVariant(qRegisterMetaType<LOGINREC *>("LOGINREC*"), &d->login);
 }
 
 void QTDSDriver::init()
 {
-    d = new QTDSDriverPrivate();
+    Q_D(QTDSDriver);
     d->initialized = (dbinit() == SUCCEED);
     // the following two code-lines will fail compilation on some FreeTDS versions
     // just comment them out if you have FreeTDS (you won't get any errors and warnings then)
@@ -551,7 +582,6 @@ QTDSDriver::~QTDSDriver()
     dbmsghandle(0);
     // dbexit also calls dbclose if necessary
     dbexit();
-    delete d;
 }
 
 bool QTDSDriver::hasFeature(DriverFeature f) const
@@ -578,6 +608,7 @@ bool QTDSDriver::open(const QString & db,
                        int /*port*/,
                        const QString& /*connOpts*/)
 {
+    Q_D(QTDSDriver);
     if (isOpen())
         close();
     if (!d->initialized) {
@@ -617,6 +648,7 @@ bool QTDSDriver::open(const QString & db,
 
 void QTDSDriver::close()
 {
+    Q_D(QTDSDriver);
     if (isOpen()) {
 #ifdef Q_USE_SYBASE
         dbloginfree(d->login);
