@@ -48,31 +48,31 @@
 #endif
 
 #ifdef QT_SQL_PSQL
-#include "../drivers/psql/qsql_psql.h"
+#include "../drivers/psql/qsql_psql_p.h"
 #endif
 #ifdef QT_SQL_MYSQL
-#include "../drivers/mysql/qsql_mysql.h"
+#include "../drivers/mysql/qsql_mysql_p.h"
 #endif
 #ifdef QT_SQL_ODBC
-#include "../drivers/odbc/qsql_odbc.h"
+#include "../drivers/odbc/qsql_odbc_p.h"
 #endif
 #ifdef QT_SQL_OCI
-#include "../drivers/oci/qsql_oci.h"
+#include "../drivers/oci/qsql_oci_p.h"
 #endif
 #ifdef QT_SQL_TDS
 // conflicting RETCODE typedef between odbc and freetds
 #define RETCODE DBRETCODE
-#include "../drivers/tds/qsql_tds.h"
+#include "../drivers/tds/qsql_tds_p.h"
 #undef RETCODE
 #endif
 #ifdef QT_SQL_DB2
-#include "../drivers/db2/qsql_db2.h"
+#include "../drivers/db2/qsql_db2_p.h"
 #endif
 #ifdef QT_SQL_SQLITE
-#include "../drivers/sqlite/qsql_sqlite.h"
+#include "../drivers/sqlite/qsql_sqlite_p.h"
 #endif
 #ifdef QT_SQL_SQLITE2
-#include "../drivers/sqlite2/qsql_sqlite2.h"
+#include "../drivers/sqlite2/qsql_sqlite2_p.h"
 #endif
 #ifdef QT_SQL_IBASE
 #undef SQL_FLOAT  // avoid clash with ODBC
@@ -82,7 +82,7 @@
 #undef SQL_TYPE_DATE
 #undef SQL_DATE
 #define SCHAR IBASE_SCHAR  // avoid clash with ODBC (older versions of ibase.h with Firebird)
-#include "../drivers/ibase/qsql_ibase.h"
+#include "../drivers/ibase/qsql_ibase_p.h"
 #undef SCHAR
 #endif
 
@@ -454,7 +454,7 @@ void QSqlDatabasePrivate::disable()
     The database connection is referred to by \a connectionName. The
     newly added database connection is returned.
 
-    If \a type is not available or could not be loaded, isValid() returns false.
+    If \a type is not available or could not be loaded, isValid() returns \c false.
 
     If \a connectionName is not specified, the new connection becomes
     the default connection for the application, and subsequent calls
@@ -620,8 +620,8 @@ void QSqlDatabase::registerSqlDriver(const QString& name, QSqlDriverCreatorBase
 /*!
     \threadsafe
 
-    Returns true if the list of database connections contains \a
-    connectionName; otherwise returns false.
+    Returns \c true if the list of database connections contains \a
+    connectionName; otherwise returns \c false.
 
     \sa connectionNames(), database(), {Threads and the SQL Module}
 */
@@ -826,7 +826,7 @@ QSqlQuery QSqlDatabase::exec(const QString & query) const
 
 /*!
     Opens the database connection using the current connection
-    values. Returns true on success; otherwise returns false. Error
+    values. Returns \c true on success; otherwise returns \c false. Error
     information can be retrieved using lastError().
 
     \sa lastError(), setDatabaseName(), setUserName(), setPassword(),
@@ -843,7 +843,7 @@ bool QSqlDatabase::open()
     \overload
 
     Opens the database connection using the given \a user name and \a
-    password. Returns true on success; otherwise returns false. Error
+    password. Returns \c true on success; otherwise returns \c false. Error
     information can be retrieved using the lastError() function.
 
     This function does not store the password it is given. Instead,
@@ -876,8 +876,8 @@ void QSqlDatabase::close()
 }
 
 /*!
-    Returns true if the database connection is currently open;
-    otherwise returns false.
+    Returns \c true if the database connection is currently open;
+    otherwise returns \c false.
 */
 
 bool QSqlDatabase::isOpen() const
@@ -886,8 +886,8 @@ bool QSqlDatabase::isOpen() const
 }
 
 /*!
-    Returns true if there was an error opening the database
-    connection; otherwise returns false. Error information can be
+    Returns \c true if there was an error opening the database
+    connection; otherwise returns \c false. Error information can be
     retrieved using the lastError() function.
 */
 
@@ -1264,6 +1264,7 @@ QSqlRecord QSqlDatabase::record(const QString& tablename) const
     \list
     \li QSQLITE_BUSY_TIMEOUT
     \li QSQLITE_OPEN_READONLY
+    \li QSQLITE_OPEN_URI
     \li QSQLITE_ENABLE_SHARED_CACHE
     \endlist
 
@@ -1302,8 +1303,8 @@ QString QSqlDatabase::connectOptions() const
 }
 
 /*!
-    Returns true if a driver called \a name is available; otherwise
-    returns false.
+    Returns \c true if a driver called \a name is available; otherwise
+    returns \c false.
 
     \sa drivers()
 */
@@ -1418,7 +1419,7 @@ QSqlDatabase QSqlDatabase::addDatabase(QSqlDriver* driver, const QString& connec
 }
 
 /*!
-    Returns true if the QSqlDatabase has a valid driver.
+    Returns \c true if the QSqlDatabase has a valid driver.
 
     Example:
     \snippet code/src_sql_kernel_qsqldatabase.cpp 8
