@@ -45,8 +45,6 @@
 #include <QtCore/qstring.h>
 #include <QtSql/qsql.h>
 
-QT_BEGIN_HEADER
-
 QT_BEGIN_NAMESPACE
 
 
@@ -146,6 +144,4 @@ Q_SQL_EXPORT QDebug operator<<(QDebug, const QSqlDatabase &);
 
 QT_END_NAMESPACE
 
-QT_END_HEADER
-
 #endif // QSQLDATABASE_H
