@@ -46,8 +46,6 @@
 #include <QtCore/qstring.h>
 #include <QtCore/qlist.h>
 
-QT_BEGIN_HEADER
-
 QT_BEGIN_NAMESPACE
 
 
@@ -78,6 +76,4 @@ private:
 
 QT_END_NAMESPACE
 
-QT_END_HEADER
-
 #endif // QSQLINDEX_H
