@@ -114,11 +114,11 @@ QSqlQueryPrivate::~QSqlQueryPrivate()
     standard SQL (e.g. \c{SET DATESTYLE=ISO} for PostgreSQL).
 
     Successfully executed SQL statements set the query's state to
-    active so that isActive() returns true. Otherwise the query's
+    active so that isActive() returns \c true. Otherwise the query's
     state is set to inactive. In either case, when executing a new SQL
     statement, the query is positioned on an invalid record. An active
     query must be navigated to a valid record (so that isValid()
-    returns true) before values can be retrieved.
+    returns \c true) before values can be retrieved.
 
     For some databases, if an active query that is a \c{SELECT}
     statement exists when you call \l{QSqlDatabase::}{commit()} or
@@ -309,7 +309,7 @@ QSqlQuery& QSqlQuery::operator=(const QSqlQuery& other)
 }
 
 /*!
-  Returns true if the query is \l{isActive()}{active} and positioned
+  Returns \c true if the query is \l{isActive()}{active} and positioned
   on a valid record and the \a field is NULL; otherwise returns
   false. Note that for some drivers, isNull() will not return accurate
   information until after an attempt is made to retrieve data.
@@ -325,10 +325,10 @@ bool QSqlQuery::isNull(int field) const
 }
 
 /*!
-  
-  Executes the SQL in \a query. Returns true and sets the query state
+
+  Executes the SQL in \a query. Returns \c true and sets the query state
   to \l{isActive()}{active} if the query was successful; otherwise
-  returns false. The \a query string must use syntax appropriate for
+  returns \c false. The \a query string must use syntax appropriate for
   the SQL database being queried (for example, standard SQL).
 
   After the query is executed, the query is positioned on an \e
@@ -339,7 +339,7 @@ bool QSqlQuery::isNull(int field) const
   called.
 
   For SQLite, the query string can contain only one statement at a time.
-  If more than one statement is given, the function returns false.
+  If more than one statement is given, the function returns \c false.
 
   Example:
 
@@ -579,7 +579,7 @@ bool QSqlQuery::seek(int index, bool relative)
 }
 
 /*!
-  
+
   Retrieves the next record in the result, if available, and positions
   the query on the retrieved record. Note that the result must be in
   the \l{isActive()}{active} state and isSelect() must return true
@@ -686,7 +686,7 @@ bool QSqlQuery::previous()
   positions the query on the retrieved record. Note that the result
   must be in the \l{isActive()}{active} state and isSelect() must
   return true before calling this function or it will do nothing and
-  return false.  Returns true if successful. If unsuccessful the query
+  return false.  Returns \c true if successful. If unsuccessful the query
   position is set to an invalid position and false is returned.
 
   \sa next(), previous(), last(), seek(), at(), isActive(), isValid()
@@ -710,7 +710,7 @@ bool QSqlQuery::first()
   the query on the retrieved record. Note that the result must be in
   the \l{isActive()}{active} state and isSelect() must return true
   before calling this function or it will do nothing and return false.
-  Returns true if successful. If unsuccessful the query position is
+  Returns \c true if successful. If unsuccessful the query position is
   set to an invalid position and false is returned.
 
   \sa next(), previous(), first(), seek(), at(), isActive(), isValid()
@@ -729,8 +729,8 @@ bool QSqlQuery::last()
   Returns the size of the result (number of rows returned), or -1 if
   the size cannot be determined or if the database does not support
   reporting information about query sizes. Note that for non-\c SELECT
-  statements (isSelect() returns false), size() will return -1. If the
-  query is not active (isActive() returns false), -1 is returned.
+  statements (isSelect() returns \c false), size() will return -1. If the
+  query is not active (isActive() returns \c false), -1 is returned.
 
   To determine the number of rows affected by a non-\c SELECT
   statement, use numRowsAffected().
@@ -773,8 +773,8 @@ QSqlError QSqlQuery::lastError() const
 }
 
 /*!
-  Returns true if the query is currently positioned on a valid
-  record; otherwise returns false.
+  Returns \c true if the query is currently positioned on a valid
+  record; otherwise returns \c false.
 */
 
 bool QSqlQuery::isValid() const
@@ -783,8 +783,8 @@ bool QSqlQuery::isValid() const
 }
 
 /*!
-  
-  Returns true if the query is \e{active}. An active QSqlQuery is one
+
+  Returns \c true if the query is \e{active}. An active QSqlQuery is one
   that has been \l{QSqlQuery::exec()} {exec()'d} successfully but not
   yet finished with.  When you are finished with an active query, you
   can make the query inactive by calling finish() or clear(), or
@@ -805,8 +805,8 @@ bool QSqlQuery::isActive() const
 }
 
 /*!
-  Returns true if the current query is a \c SELECT statement;
-  otherwise returns false.
+  Returns \c true if the current query is a \c SELECT statement;
+  otherwise returns \c false.
 */
 
 bool QSqlQuery::isSelect() const
@@ -815,8 +815,8 @@ bool QSqlQuery::isSelect() const
 }
 
 /*!
-  Returns true if you can only scroll forward through a result set;
-  otherwise returns false.
+  Returns \c true if you can only scroll forward through a result set;
+  otherwise returns \c false.
 
   \sa setForwardOnly(), next()
 */
@@ -894,8 +894,8 @@ void QSqlQuery::clear()
 }
 
 /*!
-  Prepares the SQL query \a query for execution. Returns true if the
-  query is prepared successfully; otherwise returns false.
+  Prepares the SQL query \a query for execution. Returns \c true if the
+  query is prepared successfully; otherwise returns \c false.
 
   The query may contain placeholders for binding values. Both Oracle
   style colon-name (e.g., \c{:surname}), and ODBC style (\c{?})
@@ -909,7 +909,7 @@ void QSqlQuery::clear()
   will fail.
 
   For SQLite, the query string can contain only one statement at a time.
-  If more than one statement is given, the function returns false.
+  If more than one statement is given, the function returns \c false.
 
   Example:
 
@@ -949,8 +949,8 @@ bool QSqlQuery::prepare(const QString& query)
 }
 
 /*!
-  Executes a previously prepared SQL query. Returns true if the query
-  executed successfully; otherwise returns false.
+  Executes a previously prepared SQL query. Returns \c true if the query
+  executed successfully; otherwise returns \c false.
 
   Note that the last error for this query is reset when exec() is
   called.
@@ -981,8 +981,8 @@ bool QSqlQuery::exec()
   support batch executions, the driver will simulate it using
   conventional exec() calls.
 
-  Returns true if the query is executed successfully; otherwise
-  returns false.
+  Returns \c true if the query is executed successfully; otherwise
+  returns \c false.
 
   Example:
 
@@ -1143,7 +1143,7 @@ QString QSqlQuery::executedQuery() const
   behavior is undefined.
 
   For MySQL databases the row's auto-increment field will be returned.
-    
+
   \note For this function to work in PSQL, the table table must
   contain OIDs, which may not have been created by default.  Check the
   \c default_with_oids configuration variable to be sure.
@@ -1199,7 +1199,7 @@ QSql::NumericalPrecisionPolicy QSqlQuery::numericalPrecisionPolicy() const
   call this function, but it may be helpful in order to free resources
   such as locks or cursors if you intend to re-use the query at a
   later time.
-    
+
   Sets the query to inactive. Bound values retain their values.
 
   \sa prepare(), exec(), isActive()
@@ -1216,7 +1216,7 @@ void QSqlQuery::finish()
 
 /*!
   \since 4.4
- 
+
   Discards the current result set and navigates to the next if available.
 
   Some databases are capable of returning multiple result sets for
@@ -1224,12 +1224,12 @@ void QSqlQuery::finish()
   multiple statements). If multiple result sets are available after
   executing a query this function can be used to navigate to the next
   result set(s).
-    
+
   If a new result set is available this function will return true.
   The query will be repositioned on an \e invalid record in the new
   result set and must be navigated to a valid record before data
   values can be retrieved. If a new result set isn't available the
-  function returns false and the query is set to inactive. In any
+  function returns \c false and the query is set to inactive. In any
   case the old result set will be discarded.
 
   When one of the statements is a non-select statement a count of
