@@ -51,6 +51,7 @@
 #include "qsqldriver.h"
 #include "qpointer.h"
 #include "qsqlresult_p.h"
+#include "private/qsqldriver_p.h"
 #include <QDebug>
 
 QT_BEGIN_NAMESPACE
@@ -61,7 +62,7 @@ QString QSqlResultPrivate::holderAt(int index) const
 }
 
 // return a unique id for bound names
-QString QSqlResultPrivate::fieldSerial(int i)
+QString QSqlResultPrivate::fieldSerial(int i) const
 {
     ushort arr[] = { ':', 'f', 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };
     ushort *ptr = &arr[1];
@@ -81,7 +82,7 @@ static bool qIsAlnum(QChar ch)
     return u - 'a' < 26 || u - 'A' < 26 || u - '0' < 10 || u == '_';
 }
 
-QString QSqlResultPrivate::positionalToNamedBinding(const QString &query, QString (fieldSerialFunc)(int idx))
+QString QSqlResultPrivate::positionalToNamedBinding(const QString &query) const
 {
     int n = query.size();
 
@@ -89,6 +90,7 @@ QString QSqlResultPrivate::positionalToNamedBinding(const QString &query, QStrin
     result.reserve(n * 5 / 4);
     QChar closingQuote;
     int count = 0;
+    bool ignoreBraces = (sqldriver->d_func()->dbmsType == QSqlDriverPrivate::PostgreSQL);
 
     for (int i = 0; i < n; ++i) {
         QChar ch = query.at(i);
@@ -106,11 +108,11 @@ QString QSqlResultPrivate::positionalToNamedBinding(const QString &query, QStrin
             result += ch;
         } else {
             if (ch == QLatin1Char('?')) {
-                result += fieldSerialFunc(count++);
+                result += fieldSerial(count++);
             } else {
                 if (ch == QLatin1Char('\'') || ch == QLatin1Char('"') || ch == QLatin1Char('`'))
                     closingQuote = ch;
-                else if (ch == QLatin1Char('['))
+                else if (!ignoreBraces && ch == QLatin1Char('['))
                     closingQuote = QLatin1Char(']');
                 result += ch;
             }
@@ -129,6 +131,7 @@ QString QSqlResultPrivate::namedToPositionalBinding(const QString &query)
     QChar closingQuote;
     int count = 0;
     int i = 0;
+    bool ignoreBraces = (sqldriver->d_func()->dbmsType == QSqlDriverPrivate::PostgreSQL);
 
     while (i < n) {
         QChar ch = query.at(i);
@@ -160,7 +163,7 @@ QString QSqlResultPrivate::namedToPositionalBinding(const QString &query)
             } else {
                 if (ch == QLatin1Char('\'') || ch == QLatin1Char('"') || ch == QLatin1Char('`'))
                     closingQuote = ch;
-                else if (ch == QLatin1Char('['))
+                else if (!ignoreBraces && ch == QLatin1Char('['))
                     closingQuote = QLatin1Char(']');
                 result += ch;
                 ++i;
@@ -218,11 +221,24 @@ QString QSqlResultPrivate::namedToPositionalBinding(const QString &query)
 
 QSqlResult::QSqlResult(const QSqlDriver *db)
 {
-    d = new QSqlResultPrivate(this);
+    d_ptr = new QSqlResultPrivate;
+    Q_D(QSqlResult);
+    d->q_ptr = this;
     d->sqldriver = const_cast<QSqlDriver *>(db);
-    if(db) {
-        setNumericalPrecisionPolicy(db->numericalPrecisionPolicy());
-    }
+    if (d->sqldriver)
+        setNumericalPrecisionPolicy(d->sqldriver->numericalPrecisionPolicy());
+}
+
+/*!  \internal
+*/
+QSqlResult::QSqlResult(QSqlResultPrivate &dd, const QSqlDriver *db)
+{
+    d_ptr = &dd;
+    Q_D(QSqlResult);
+    d->q_ptr = this;
+    d->sqldriver = const_cast<QSqlDriver *>(db);
+    if (d->sqldriver)
+        setNumericalPrecisionPolicy(d->sqldriver->numericalPrecisionPolicy());
 }
 
 /*!
@@ -231,6 +247,7 @@ QSqlResult::QSqlResult(const QSqlDriver *db)
 
 QSqlResult::~QSqlResult()
 {
+    Q_D(QSqlResult);
     delete d;
 }
 
@@ -243,6 +260,7 @@ QSqlResult::~QSqlResult()
 
 void QSqlResult::setQuery(const QString& query)
 {
+    Q_D(QSqlResult);
     d->sql = query;
 }
 
@@ -255,6 +273,7 @@ void QSqlResult::setQuery(const QString& query)
 
 QString QSqlResult::lastQuery() const
 {
+    Q_D(const QSqlResult);
     return d->sql;
 }
 
@@ -267,37 +286,40 @@ QString QSqlResult::lastQuery() const
 */
 int QSqlResult::at() const
 {
+    Q_D(const QSqlResult);
     return d->idx;
 }
 
 
 /*!
-    Returns true if the result is positioned on a valid record (that
+    Returns \c true if the result is positioned on a valid record (that
     is, the result is not positioned before the first or after the
-    last record); otherwise returns false.
+    last record); otherwise returns \c false.
 
     \sa at()
 */
 
 bool QSqlResult::isValid() const
 {
+    Q_D(const QSqlResult);
     return d->idx != QSql::BeforeFirstRow && d->idx != QSql::AfterLastRow;
 }
 
 /*!
     \fn bool QSqlResult::isNull(int index)
 
-    Returns true if the field at position \a index in the current row
-    is null; otherwise returns false.
+    Returns \c true if the field at position \a index in the current row
+    is null; otherwise returns \c false.
 */
 
 /*!
-    Returns true if the result has records to be retrieved; otherwise
-    returns false.
+    Returns \c true if the result has records to be retrieved; otherwise
+    returns \c false.
 */
 
 bool QSqlResult::isActive() const
 {
+    Q_D(const QSqlResult);
     return d->active;
 }
 
@@ -310,6 +332,7 @@ bool QSqlResult::isActive() const
 
 void QSqlResult::setAt(int index)
 {
+    Q_D(QSqlResult);
     d->idx = index;
 }
 
@@ -325,18 +348,20 @@ void QSqlResult::setAt(int index)
 
 void QSqlResult::setSelect(bool select)
 {
+    Q_D(QSqlResult);
     d->isSel = select;
 }
 
 /*!
-    Returns true if the current result is from a \c SELECT statement;
-    otherwise returns false.
+    Returns \c true if the current result is from a \c SELECT statement;
+    otherwise returns \c false.
 
     \sa setSelect()
 */
 
 bool QSqlResult::isSelect() const
 {
+    Q_D(const QSqlResult);
     return d->isSel;
 }
 
@@ -347,6 +372,7 @@ bool QSqlResult::isSelect() const
 
 const QSqlDriver *QSqlResult::driver() const
 {
+    Q_D(const QSqlResult);
     return d->sqldriver;
 }
 
@@ -360,6 +386,7 @@ const QSqlDriver *QSqlResult::driver() const
 
 void QSqlResult::setActive(bool active)
 {
+    Q_D(QSqlResult);
     if (active && d->executedQuery.isEmpty())
         d->executedQuery = d->sql;
 
@@ -375,6 +402,7 @@ void QSqlResult::setActive(bool active)
 
 void QSqlResult::setLastError(const QSqlError &error)
 {
+    Q_D(QSqlResult);
     d->error = error;
 }
 
@@ -385,6 +413,7 @@ void QSqlResult::setLastError(const QSqlError &error)
 
 QSqlError QSqlResult::lastError() const
 {
+    Q_D(const QSqlResult);
     return d->error;
 }
 
@@ -512,13 +541,14 @@ bool QSqlResult::fetchPrevious()
 }
 
 /*!
-    Returns true if you can only scroll forward through the result
-    set; otherwise returns false.
+    Returns \c true if you can only scroll forward through the result
+    set; otherwise returns \c false.
 
     \sa setForwardOnly()
 */
 bool QSqlResult::isForwardOnly() const
 {
+    Q_D(const QSqlResult);
     return d->forwardOnly;
 }
 
@@ -540,13 +570,14 @@ bool QSqlResult::isForwardOnly() const
 */
 void QSqlResult::setForwardOnly(bool forward)
 {
+    Q_D(QSqlResult);
     d->forwardOnly = forward;
 }
 
 /*!
     Prepares the given \a query, using the underlying database
-    functionality where possible. Returns true if the query is
-    prepared successfully; otherwise returns false.
+    functionality where possible. Returns \c true if the query is
+    prepared successfully; otherwise returns \c false.
 
     Note: This method should have been called "safePrepare()".
 
@@ -554,6 +585,7 @@ void QSqlResult::setForwardOnly(bool forward)
 */
 bool QSqlResult::savePrepare(const QString& query)
 {
+    Q_D(QSqlResult);
     if (!driver())
         return false;
     d->clear();
@@ -565,7 +597,7 @@ bool QSqlResult::savePrepare(const QString& query)
     d->executedQuery = d->namedToPositionalBinding(query);
 
     if (driver()->hasFeature(QSqlDriver::NamedPlaceholders))
-        d->executedQuery = QSqlResultPrivate::positionalToNamedBinding(query);
+        d->executedQuery = d->positionalToNamedBinding(query);
 
     return prepare(d->executedQuery);
 }
@@ -573,12 +605,13 @@ bool QSqlResult::savePrepare(const QString& query)
 /*!
     Prepares the given \a query for execution; the query will normally
     use placeholders so that it can be executed repeatedly. Returns
-    true if the query is prepared successfully; otherwise returns false.
+    true if the query is prepared successfully; otherwise returns \c false.
 
     \sa exec()
 */
 bool QSqlResult::prepare(const QString& query)
 {
+    Q_D(QSqlResult);
     d->sql = query;
     if (d->holders.isEmpty()) {
         // parse the query to memorize parameter location
@@ -595,6 +628,7 @@ bool QSqlResult::prepare(const QString& query)
 */
 bool QSqlResult::exec()
 {
+    Q_D(QSqlResult);
     bool ret;
     // fake preparation - just replace the placeholders..
     QString query = lastQuery();
@@ -647,8 +681,9 @@ bool QSqlResult::exec()
 */
 void QSqlResult::bindValue(int index, const QVariant& val, QSql::ParamType paramType)
 {
+    Q_D(QSqlResult);
     d->binds = PositionalBinding;
-    d->indexes[QSqlResultPrivate::fieldSerial(index)].append(index);
+    d->indexes[d->fieldSerial(index)].append(index);
     if (d->values.count() <= index)
         d->values.resize(index + 1);
     d->values[index] = val;
@@ -675,6 +710,7 @@ void QSqlResult::bindValue(int index, const QVariant& val, QSql::ParamType param
 void QSqlResult::bindValue(const QString& placeholder, const QVariant& val,
                            QSql::ParamType paramType)
 {
+    Q_D(QSqlResult);
     d->binds = NamedBinding;
     // if the index has already been set when doing emulated named
     // bindings - don't reset it
@@ -696,6 +732,7 @@ void QSqlResult::bindValue(const QString& placeholder, const QVariant& val,
 */
 void QSqlResult::addBindValue(const QVariant& val, QSql::ParamType paramType)
 {
+    Q_D(QSqlResult);
     d->binds = PositionalBinding;
     bindValue(d->bindCount, val, paramType);
     ++d->bindCount;
@@ -709,6 +746,7 @@ void QSqlResult::addBindValue(const QVariant& val, QSql::ParamType paramType)
 */
 QVariant QSqlResult::boundValue(int index) const
 {
+    Q_D(const QSqlResult);
     return d->values.value(index);
 }
 
@@ -722,6 +760,7 @@ QVariant QSqlResult::boundValue(int index) const
 */
 QVariant QSqlResult::boundValue(const QString& placeholder) const
 {
+    Q_D(const QSqlResult);
     QList<int> indexes = d->indexes.value(placeholder);
     return d->values.value(indexes.value(0,-1));
 }
@@ -733,6 +772,7 @@ QVariant QSqlResult::boundValue(const QString& placeholder) const
 */
 QSql::ParamType QSqlResult::bindValueType(int index) const
 {
+    Q_D(const QSqlResult);
     return d->types.value(index, QSql::In);
 }
 
@@ -744,6 +784,7 @@ QSql::ParamType QSqlResult::bindValueType(int index) const
 */
 QSql::ParamType QSqlResult::bindValueType(const QString& placeholder) const
 {
+    Q_D(const QSqlResult);
     return d->types.value(d->indexes.value(placeholder).value(0,-1), QSql::In);
 }
 
@@ -754,6 +795,7 @@ QSql::ParamType QSqlResult::bindValueType(const QString& placeholder) const
 */
 int QSqlResult::boundValueCount() const
 {
+    Q_D(const QSqlResult);
     return d->values.count();
 }
 
@@ -765,7 +807,8 @@ int QSqlResult::boundValueCount() const
 */
 QVector<QVariant>& QSqlResult::boundValues() const
 {
-    return d->values;
+    Q_D(const QSqlResult);
+    return const_cast<QSqlResultPrivate *>(d)->values;
 }
 
 /*!
@@ -773,6 +816,7 @@ QVector<QVariant>& QSqlResult::boundValues() const
 */
 QSqlResult::BindingSyntax QSqlResult::bindingSyntax() const
 {
+    Q_D(const QSqlResult);
     return d->binds;
 }
 
@@ -782,6 +826,7 @@ QSqlResult::BindingSyntax QSqlResult::bindingSyntax() const
 */
 void QSqlResult::clear()
 {
+    Q_D(QSqlResult);
     d->clear();
 }
 
@@ -795,11 +840,13 @@ void QSqlResult::clear()
 */
 QString QSqlResult::executedQuery() const
 {
+    Q_D(const QSqlResult);
     return d->executedQuery;
 }
 
 void QSqlResult::resetBindCount()
 {
+    Q_D(QSqlResult);
     d->resetBindCount();
 }
 
@@ -811,17 +858,19 @@ void QSqlResult::resetBindCount()
 */
 QString QSqlResult::boundValueName(int index) const
 {
+    Q_D(const QSqlResult);
     return d->holderAt(index);
 }
 
 /*!
-    Returns true if at least one of the query's bound values is a \c
-    QSql::Out or a QSql::InOut; otherwise returns false.
+    Returns \c true if at least one of the query's bound values is a \c
+    QSql::Out or a QSql::InOut; otherwise returns \c false.
 
     \sa bindValueType()
 */
 bool QSqlResult::hasOutValues() const
 {
+    Q_D(const QSqlResult);
     if (d->types.isEmpty())
         return false;
     QHash<int, QSql::ParamType>::ConstIterator it;
@@ -901,6 +950,7 @@ void QSqlResult::virtual_hook(int, void *)
 bool QSqlResult::execBatch(bool arrayBind)
 {
     Q_UNUSED(arrayBind);
+    Q_D(QSqlResult);
 
     QVector<QVariant> values = d->values;
     if (values.count() == 0)
@@ -924,6 +974,7 @@ void QSqlResult::detachFromResultSet()
  */
 void QSqlResult::setNumericalPrecisionPolicy(QSql::NumericalPrecisionPolicy policy)
 {
+    Q_D(QSqlResult);
     d->precisionPolicy = policy;
 }
 
@@ -931,6 +982,7 @@ void QSqlResult::setNumericalPrecisionPolicy(QSql::NumericalPrecisionPolicy poli
  */
 QSql::NumericalPrecisionPolicy QSqlResult::numericalPrecisionPolicy() const
 {
+    Q_D(const QSqlResult);
     return d->precisionPolicy;
 }
 
