@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2012 Digia Plc and/or its subsidiary(-ies).
+** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
 ** This file is part of the QtSql module of the Qt Toolkit.
@@ -95,6 +95,13 @@ void QSqlQueryModelPrivate::initColOffsets(int size)
     memset(colOffsets.data(), 0, colOffsets.size() * sizeof(int));
 }
 
+int QSqlQueryModelPrivate::columnInQuery(int modelColumn) const
+{
+    if (modelColumn < 0 || modelColumn >= rec.count() || !rec.isGenerated(modelColumn) || modelColumn >= colOffsets.size())
+        return -1;
+    return modelColumn - colOffsets[modelColumn];
+}
+
 /*!
     \class QSqlQueryModel
     \brief The QSqlQueryModel class provides a read-only data model for SQL
@@ -193,7 +200,7 @@ void QSqlQueryModel::fetchMore(const QModelIndex &parent)
 /*!
     \since 4.1
 
-    Returns true if it is possible to read more rows from the database.
+    Returns \c true if it is possible to read more rows from the database.
     This only affects databases that don't report back the size of a query
     (see QSqlDriver::hasFeature()).
 
@@ -370,11 +377,7 @@ QVariant QSqlQueryModel::headerData(int section, Qt::Orientation orientation, in
             val = d->headers.value(section).value(Qt::EditRole);
         if (val.isValid())
             return val;
-
-        // See if it's an inserted column (iiq.column() != -1)
-        QModelIndex dItem = indexInQuery(createIndex(0, section));
-
-        if (role == Qt::DisplayRole && d->rec.count() > section && dItem.column() != -1)
+        if (role == Qt::DisplayRole && d->rec.count() > section && d->columnInQuery(section) != -1)
             return d->rec.fieldName(section);
     }
     return QAbstractItemModel::headerData(section, orientation, role);
@@ -490,7 +493,7 @@ void QSqlQueryModel::clear()
     \a value. This is useful if the model is used to
     display data in a view (e.g., QTableView).
 
-    Returns true if \a orientation is Qt::Horizontal and
+    Returns \c true if \a orientation is Qt::Horizontal and
     the \a section refers to a valid section; otherwise returns
     false.
 
@@ -591,7 +594,7 @@ QSqlRecord QSqlQueryModel::record() const
     \a parent parameter must always be an invalid QModelIndex, since
     the model does not support parent-child relationships.
 
-    Returns true if \a column is within bounds; otherwise returns false.
+    Returns \c true if \a column is within bounds; otherwise returns \c false.
 
     By default, inserted columns are empty. To fill them with data,
     reimplement data() and handle any inserted column separately:
@@ -633,7 +636,7 @@ bool QSqlQueryModel::insertColumns(int column, int count, const QModelIndex &par
     Removing columns effectively hides them. It does not affect the
     underlying QSqlQuery.
 
-    Returns true if the columns were removed; otherwise returns false.
+    Returns \c true if the columns were removed; otherwise returns \c false.
  */
 bool QSqlQueryModel::removeColumns(int column, int count, const QModelIndex &parent)
 {
@@ -668,12 +671,10 @@ bool QSqlQueryModel::removeColumns(int column, int count, const QModelIndex &par
 QModelIndex QSqlQueryModel::indexInQuery(const QModelIndex &item) const
 {
     Q_D(const QSqlQueryModel);
-    if (item.column() < 0 || item.column() >= d->rec.count()
-        || !d->rec.isGenerated(item.column())
-        || item.column() >= d->colOffsets.size())
+    int modelColumn = d->columnInQuery(item.column());
+    if (modelColumn < 0)
         return QModelIndex();
-    return createIndex(item.row(), item.column() - d->colOffsets[item.column()],
-                       item.internalPointer());
+    return createIndex(item.row(), modelColumn, item.internalPointer());
 }
 
 QT_END_NAMESPACE
