@@ -44,8 +44,6 @@
 
 #include <QtSql/qsqltablemodel.h>
 
-QT_BEGIN_HEADER
-
 QT_BEGIN_NAMESPACE
 
 
@@ -112,6 +110,4 @@ private:
 
 QT_END_NAMESPACE
 
-QT_END_HEADER
-
 #endif // QSQLRELATIONALTABLEMODEL_H
