@@ -44,8 +44,6 @@
 
 #include <QtTest/qtest_global.h>
 
-QT_BEGIN_HEADER
-
 QT_BEGIN_NAMESPACE
 
 
@@ -60,13 +58,32 @@ enum QBenchmarkMetric {
     InstructionReads,
     Events,
     WalltimeNanoseconds,
-    BytesAllocated
+    BytesAllocated,
+    CPUMigrations,
+    CPUCycles,
+    BusCycles,
+    StalledCycles,
+    Instructions,
+    BranchInstructions,
+    BranchMisses,
+    CacheReferences,
+    CacheReads,
+    CacheWrites,
+    CachePrefetches,
+    CacheMisses,
+    CacheReadMisses,
+    CacheWriteMisses,
+    CachePrefetchMisses,
+    ContextSwitches,
+    PageFaults,
+    MinorPageFaults,
+    MajorPageFaults,
+    AlignmentFaults,
+    EmulationFaults
 };
 
 }
 
 QT_END_NAMESPACE
 
-QT_END_HEADER
-
 #endif // QBENCHMARK_H
