@@ -56,8 +56,6 @@
 #include <QtTest/qtest_global.h>
 #include <QtTest/qbenchmarkmetric.h>
 
-QT_BEGIN_HEADER
-
 QT_BEGIN_NAMESPACE
 
 
@@ -68,6 +66,4 @@ namespace QTest {
 
 QT_END_NAMESPACE
 
-QT_END_HEADER
-
 #endif // QBENCHMARK_H
