@@ -64,6 +64,9 @@
 #include <QtTest/private/qbenchmark_p.h>
 #include <QtTest/private/cycle_p.h>
 
+#include <numeric>
+#include <algorithm>
+
 #include <stdarg.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -82,7 +85,7 @@
 #include <time.h>
 #endif
 
-#ifdef Q_OS_MAC
+#if defined(Q_OS_MACX)
 #include <IOKit/pwr_mgt/IOPMLib.h>
 #endif
 
@@ -861,7 +864,7 @@ QT_BEGIN_NAMESPACE
     \since 5.0
 
     Waits for \a timeout milliseconds or until the \a window is exposed.
-    Returns true if \c window is exposed within \a timeout milliseconds, otherwise returns false.
+    Returns \c true if \c window is exposed within \a timeout milliseconds, otherwise returns \c false.
 
     This is mainly useful for asynchronous systems like X11, where a window will be mapped to screen some
     time after being asked to show itself on the screen.
@@ -874,7 +877,7 @@ QT_BEGIN_NAMESPACE
 
     Waits for \a timeout milliseconds or until the \a window is active.
 
-    Returns true if \c window is active within \a timeout milliseconds, otherwise returns false.
+    Returns \c true if \c window is active within \a timeout milliseconds, otherwise returns \c false.
 
     \sa QTest::qWaitForWindowExposed(), QWindow::isActive()
 */
@@ -883,7 +886,7 @@ QT_BEGIN_NAMESPACE
     \since 5.0
 
     Waits for \a timeout milliseconds or until the \a widget's window is exposed.
-    Returns true if \c widget's window is exposed within \a timeout milliseconds, otherwise returns false.
+    Returns \c true if \c widget's window is exposed within \a timeout milliseconds, otherwise returns \c false.
 
     This is mainly useful for asynchronous systems like X11, where a window will be mapped to screen some
     time after being asked to show itself on the screen.
@@ -896,7 +899,7 @@ QT_BEGIN_NAMESPACE
 
     Waits for \a timeout milliseconds or until the \a widget's window is active.
 
-    Returns true if \c widget's window is active within \a timeout milliseconds, otherwise returns false.
+    Returns \c true if \c widget's window is active within \a timeout milliseconds, otherwise returns \c false.
 
     \sa QTest::qWaitForWindowExposed(), QWidget::isActiveWindow()
 */
@@ -906,7 +909,7 @@ QT_BEGIN_NAMESPACE
     \deprecated
 
     Waits for \a timeout milliseconds or until the \a widget's window is exposed.
-    Returns true if \c widget's window is exposed within \a timeout milliseconds, otherwise returns false.
+    Returns \c true if \c widget's window is exposed within \a timeout milliseconds, otherwise returns \c false.
 
     This function does the same as qWaitForWindowExposed().
 
@@ -1128,9 +1131,7 @@ namespace QTest
     static int keyDelay = -1;
     static int mouseDelay = -1;
     static int eventDelay = -1;
-#if defined(Q_OS_UNIX)
     static bool noCrashHandler = false;
-#endif
 
 /*! \internal
     Invoke a method of the object without generating warning if the method does not exist
@@ -1333,19 +1334,23 @@ Q_TESTLIB_EXPORT void qtest_qParseArgs(int argc, char *argv[], bool qml)
          " -mousedelay ms      : Set default delay for mouse simulation to ms milliseconds\n"
          " -maxwarnings n      : Sets the maximum amount of messages to output.\n"
          "                       0 means unlimited, default: 2000\n"
-#if defined(Q_OS_UNIX)
          " -nocrashhandler     : Disables the crash handler\n"
-#endif
          "\n"
          " Benchmarking options:\n"
 #ifdef QTESTLIB_USE_VALGRIND
          " -callgrind          : Use callgrind to time benchmarks\n"
 #endif
+#ifdef QTESTLIB_USE_PERF_EVENTS
+         " -perf               : Use Linux perf events to time benchmarks\n"
+         " -perfcounter name   : Use the counter named 'name'\n"
+         " -perfcounterlist    : Lists the counters available\n"
+#endif
 #ifdef HAVE_TICK_COUNTER
          " -tickcounter        : Use CPU tick counters to time benchmarks\n"
 #endif
          " -eventcounter       : Counts events received during benchmarks\n"
          " -minimumvalue n     : Sets the minimum acceptable measurement value\n"
+         " -minimumtotal n     : Sets the minimum acceptable total for repeated executions of a test function\n"
          " -iterations  n      : Sets the number of accumulation iterations.\n"
          " -median  n          : Sets the number of median iterations.\n"
          " -vb                 : Print out verbose benchmarking information.\n";
@@ -1460,10 +1465,8 @@ Q_TESTLIB_EXPORT void qtest_qParseArgs(int argc, char *argv[], bool qml)
             } else {
                 QTestLog::setMaxWarnings(qToInt(argv[++i]));
             }
-#if defined(Q_OS_UNIX)
         } else if (strcmp(argv[i], "-nocrashhandler") == 0) {
             QTest::noCrashHandler = true;
-#endif
 #ifdef QTESTLIB_USE_VALGRIND
         } else if (strcmp(argv[i], "-callgrind") == 0) {
             if (QBenchmarkValgrindUtils::haveValgrind())
@@ -1481,6 +1484,25 @@ Q_TESTLIB_EXPORT void qtest_qParseArgs(int argc, char *argv[], bool qml)
             QBenchmarkGlobalData::current->callgrindOutFileBase =
                 QBenchmarkValgrindUtils::outFileBase();
 #endif
+#ifdef QTESTLIB_USE_PERF_EVENTS
+        } else if (strcmp(argv[i], "-perf") == 0) {
+            if (QBenchmarkPerfEventsMeasurer::isAvailable()) {
+                // perf available
+                QBenchmarkGlobalData::current->setMode(QBenchmarkGlobalData::PerfCounter);
+            } else {
+                fprintf(stderr, "WARNING: Linux perf events not available. Using the walltime measurer.\n");
+            }
+        } else if (strcmp(argv[i], "-perfcounter") == 0) {
+            if (i + 1 >= argc) {
+                fprintf(stderr, "-perfcounter needs an extra parameter with the name of the counter\n");
+                exit(1);
+            } else {
+                QBenchmarkPerfEventsMeasurer::setCounter(argv[++i]);
+            }
+        } else if (strcmp(argv[i], "-perfcounterlist") == 0) {
+            QBenchmarkPerfEventsMeasurer::listCounters();
+            exit(0);
+#endif
 #ifdef HAVE_TICK_COUNTER
         } else if (strcmp(argv[i], "-tickcounter") == 0) {
             QBenchmarkGlobalData::current->setMode(QBenchmarkGlobalData::TickCounter);
@@ -1494,6 +1516,13 @@ Q_TESTLIB_EXPORT void qtest_qParseArgs(int argc, char *argv[], bool qml)
             } else {
                 QBenchmarkGlobalData::current->walltimeMinimum = qToInt(argv[++i]);
             }
+        } else if (strcmp(argv[i], "-minimumtotal") == 0) {
+            if (i + 1 >= argc) {
+                fprintf(stderr, "-minimumtotal needs an extra parameter to indicate the minimum total measurement\n");
+                exit(1);
+            } else {
+                QBenchmarkGlobalData::current->minimumTotal = qToInt(argv[++i]);
+            }
         } else if (strcmp(argv[i], "-iterations") == 0) {
             if (i + 1 >= argc) {
                 fprintf(stderr, "-iterations needs an extra parameter to indicate the number of iterations\n");
@@ -1583,7 +1612,7 @@ Q_TESTLIB_EXPORT void qtest_qParseArgs(int argc, char *argv[], bool qml)
         }
     }
 
-    bool installedTestCoverage = installCoverageTool(QTestResult::currentAppname(), QTestResult::currentTestObjectName());
+    bool installedTestCoverage = installCoverageTool(QTestResult::currentAppName(), QTestResult::currentTestObjectName());
     QTestLog::setInstalledTestCoverage(installedTestCoverage);
 
     // If no loggers were created by the long version of the -o command-line
@@ -1600,10 +1629,10 @@ QBenchmarkResult qMedian(const QList<QBenchmarkResult> &container)
         return QBenchmarkResult();
 
     if (count == 1)
-        return container.at(0);
+        return container.front();
 
     QList<QBenchmarkResult> containerCopy = container;
-    qSort(containerCopy);
+    std::sort(containerCopy.begin(), containerCopy.end());
 
     const int middle = count / 2;
 
@@ -1623,6 +1652,15 @@ struct QTestDataSetter
     }
 };
 
+namespace {
+
+qreal addResult(qreal current, const QBenchmarkResult& r)
+{
+    return current + r.value;
+}
+
+}
+
 static void qInvokeTestMethodDataEntry(char *slot)
 {
     /* Benchmarking: for each median iteration*/
@@ -1631,6 +1669,7 @@ static void qInvokeTestMethodDataEntry(char *slot)
     int i = (QBenchmarkGlobalData::current->measurer->needsWarmupIteration()) ? -1 : 0;
 
     QList<QBenchmarkResult> results;
+    bool minimumTotalReached = false;
     do {
         QBenchmarkTestMethodData::current->beginDataRun();
 
@@ -1689,8 +1728,16 @@ static void qInvokeTestMethodDataEntry(char *slot)
                 }
             }
         }
+
+        // Verify if the minimum total measurement is reached, if it was specified:
+        if (QBenchmarkGlobalData::current->minimumTotal == -1) {
+            minimumTotalReached = true;
+        } else {
+            const qreal total = std::accumulate(results.begin(), results.end(), 0.0, addResult);
+            minimumTotalReached = (total >= QBenchmarkGlobalData::current->minimumTotal);
+        }
     } while (isBenchmark
-             && (++i < QBenchmarkGlobalData::current->adjustMedianIterationCount())
+             && ((++i < QBenchmarkGlobalData::current->adjustMedianIterationCount()) || !minimumTotalReached)
              && !QTestResult::skipCurrentTest() && !QTestResult::currentTestFailed());
 
     // If the test is a benchmark, finalize the result after all iterations have finished.
@@ -2031,6 +2078,18 @@ FatalSignalHandler::~FatalSignalHandler()
 
 } // namespace
 
+#if defined(Q_OS_WIN) && !defined(Q_OS_WINCE) && !defined(Q_OS_WINRT)
+static LONG WINAPI windowsFaultHandler(struct _EXCEPTION_POINTERS *exInfo)
+{
+    char appName[MAX_PATH];
+    if (!GetModuleFileNameA(NULL, appName, MAX_PATH))
+        appName[0] = 0;
+    fprintf(stderr, "A crash occurred in %s (exception code 0x%lx).",
+            appName, exInfo->ExceptionRecord->ExceptionCode);
+    return EXCEPTION_EXECUTE_HANDLER;
+}
+#endif // Q_OS_WIN) && !Q_OS_WINCE && !Q_OS_WINRT
+
 /*!
     Executes tests declared in \a testObject. In addition, the private slots
     \c{initTestCase()}, \c{cleanupTestCase()}, \c{init()} and \c{cleanup()}
@@ -2078,7 +2137,7 @@ int QTest::qExec(QObject *testObject, int argc, char **argv)
     int callgrindChildExitCode = 0;
 #endif
 
-#ifdef Q_OS_MAC
+#if defined(Q_OS_MACX)
     bool macNeedsActivate = qApp && (qstrcmp(qApp->metaObject()->className(), "QApplication") == 0);
     IOPMAssertionID powerID;
 #endif
@@ -2086,14 +2145,7 @@ int QTest::qExec(QObject *testObject, int argc, char **argv)
     try {
 #endif
 
-#if defined(Q_OS_WIN) && !defined(Q_OS_WINCE)
-# if !defined(Q_CC_MINGW)
-    _CrtSetReportMode(_CRT_ERROR, _CRTDBG_MODE_DEBUG);
-# endif
-    SetErrorMode(SetErrorMode(0) | SEM_NOGPFAULTERRORBOX);
-#endif
-
-#ifdef Q_OS_MAC
+#if defined(Q_OS_MACX)
     if (macNeedsActivate) {
         CFStringRef reasonForActivity= CFSTR("No Display Sleep");
         IOReturn ok = IOPMAssertionCreateWithName(kIOPMAssertionTypeNoDisplaySleep, kIOPMAssertionLevelOn, reasonForActivity, &powerID);
@@ -2114,10 +2166,20 @@ int QTest::qExec(QObject *testObject, int argc, char **argv)
 
     QTestResult::setCurrentTestObject(metaObject->className());
     if (argc > 0)
-        QTestResult::setCurrentAppname(argv[0]);
+        QTestResult::setCurrentAppName(argv[0]);
 
     qtest_qParseArgs(argc, argv, false);
 
+#if defined(Q_OS_WIN) && !defined(Q_OS_WINCE) && !defined(Q_OS_WINRT)
+    if (!noCrashHandler) {
+# ifndef Q_CC_MINGW
+        _CrtSetReportMode(_CRT_ERROR, _CRTDBG_MODE_DEBUG);
+# endif
+        SetErrorMode(SetErrorMode(0) | SEM_NOGPFAULTERRORBOX);
+        SetUnhandledExceptionFilter(windowsFaultHandler);
+    } // !noCrashHandler
+#endif // Q_OS_WIN) && !Q_OS_WINCE && !Q_OS_WINRT
+
 #ifdef QTESTLIB_USE_VALGRIND
     if (QBenchmarkGlobalData::current->mode() == QBenchmarkGlobalData::CallgrindParentProcess) {
         const QStringList origAppArgs(QCoreApplication::arguments());
@@ -2146,7 +2208,7 @@ int QTest::qExec(QObject *testObject, int argc, char **argv)
          }
 
         QTestLog::stopLogging();
-#ifdef Q_OS_MAC
+#if defined(Q_OS_MACX)
          if (macNeedsActivate) {
              IOPMAssertionRelease(powerID);
          }
@@ -2163,7 +2225,7 @@ int QTest::qExec(QObject *testObject, int argc, char **argv)
 
     QSignalDumper::endDump();
 
-#ifdef Q_OS_MAC
+#if defined(Q_OS_MACX)
      if (macNeedsActivate) {
          IOPMAssertionRelease(powerID);
      }
@@ -2457,6 +2519,14 @@ QTestData &QTest::newRow(const char *dataTag)
     \sa QTest::newRow(), QFETCH(), QMetaType
 */
 
+/*!
+    Returns the name of the binary that is currently executed.
+*/
+const char *QTest::currentAppName()
+{
+    return QTestResult::currentAppName();
+}
+
 /*!
     Returns the name of the test function that is currently executed.
 
@@ -2479,7 +2549,7 @@ const char *QTest::currentDataTag()
 }
 
 /*!
-    Returns true if the current test function failed, otherwise false.
+    Returns \c true if the current test function failed, otherwise false.
 */
 bool QTest::currentTestFailed()
 {
@@ -2507,7 +2577,9 @@ void QTest::qSleep(int ms)
 {
     QTEST_ASSERT(ms > 0);
 
-#ifdef Q_OS_WIN
+#if defined(Q_OS_WINRT)
+    WaitForSingleObjectEx(GetCurrentThread(), ms, true);
+#elif defined(Q_OS_WIN)
     Sleep(uint(ms));
 #else
     struct timespec ts = { ms / 1000, (ms % 1000) * 1000 * 1000 };
@@ -2684,6 +2756,10 @@ bool QTest::compare_string_helper(const char *t1, const char *t2, const char *ac
     \internal
 */
 
+/*! \fn bool QTest::qCompare(QList<T> const &t1, QList<T> const &t2, const char *actual, const char *expected, const char *file, int line)
+    \internal
+*/
+
 /*! \fn bool QTest::qCompare(QFlags<T> const &t1, T const &t2, const char *actual, const char *expected, const char *file, int line)
     \internal
 */
@@ -2692,6 +2768,30 @@ bool QTest::compare_string_helper(const char *t1, const char *t2, const char *ac
     \internal
 */
 
+/*! \fn bool QTest::qCompare(qint64 const &t1, qint32 const &t2, const char *actual, const char *expected, const char *file, int line)
+    \internal
+*/
+
+/*! \fn bool QTest::qCompare(qint64 const &t1, quint32 const &t2, const char *actual, const char *expected, const char *file, int line)
+    \internal
+*/
+
+/*! \fn bool QTest::qCompare(quint64 const &t1, quint32 const &t2, const char *actual, const char *expected, const char *file, int line)
+    \internal
+*/
+
+/*! \fn bool QTest::qCompare(qint32 const &t1, qint64 const &t2, const char *actual, const char *expected, const char *file, int line)
+    \internal
+*/
+
+/*! \fn bool QTest::qCompare(quint32 const &t1, qint64 const &t2, const char *actual, const char *expected, const char *file, int line)
+    \internal
+*/
+
+/*! \fn bool QTest::qCompare(quint32 const &t1, quint64 const &t2, const char *actual, const char *expected, const char *file, int line)
+    \internal
+*/
+
 /*! \fn bool QTest::qCompare(bool const &t1, int const &t2, const char *actual, const char *expected, const char *file, int line)
   \internal
  */
