@@ -51,8 +51,6 @@
 
 #include <string.h>
 
-QT_BEGIN_HEADER
-
 QT_BEGIN_NAMESPACE
 
 
@@ -202,6 +200,8 @@ namespace QTest
     Q_TESTLIB_EXPORT void *qElementData(const char *elementName, int metaTypeId);
     Q_TESTLIB_EXPORT QObject *testObject();
 
+    Q_TESTLIB_EXPORT const char *currentAppName();
+
     Q_TESTLIB_EXPORT const char *currentTestFunction();
     Q_TESTLIB_EXPORT const char *currentDataTag();
     Q_TESTLIB_EXPORT bool currentTestFailed();
@@ -352,6 +352,4 @@ namespace QTest
 
 QT_END_NAMESPACE
 
-QT_END_HEADER
-
 #endif
