@@ -56,8 +56,6 @@
 #include <QtTest/private/qtestcorelist_p.h>
 #include <QtTest/private/qtestelementattribute_p.h>
 
-QT_BEGIN_HEADER
-
 QT_BEGIN_NAMESPACE
 
 
@@ -174,6 +172,4 @@ const QTestElementAttribute *QTestCoreElement<ElementType>::attribute(QTest::Att
 
 QT_END_NAMESPACE
 
-QT_END_HEADER
-
 #endif
