@@ -54,13 +54,14 @@
 #include <QtCore/qpoint.h>
 #include <QtCore/qstring.h>
 #include <QtGui/qevent.h>
+#include <QtGui/qwindow.h>
 
 #ifdef QT_WIDGETS_LIB
 #include <QtWidgets/qapplication.h>
 #include <QtWidgets/qwidget.h>
 #endif
 
-QT_BEGIN_HEADER
+#include <QtCore/QDebug>
 
 QT_BEGIN_NAMESPACE
 
@@ -85,6 +86,11 @@ namespace QTest
         QTEST_ASSERT(window);
         extern int Q_TESTLIB_EXPORT defaultMouseDelay();
 
+        // pos is in window local coordinates
+        if (window->geometry().width() <= pos.x() || window->geometry().height() <= pos.y()) {
+            QTest::qWarn("Mouse event occurs outside of target window.");
+        }
+
          static Qt::MouseButton lastButton = Qt::NoButton;
 
         if (delay == -1 || delay < defaultMouseDelay())
@@ -227,6 +233,4 @@ namespace QTest
 
 QT_END_NAMESPACE
 
-QT_END_HEADER
-
 #endif // QTESTMOUSE_H
