@@ -1,7 +1,8 @@
 /****************************************************************************
 **
 ** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/
+** Copyright (C) 2013 Olivier Goffart <ogoffart@woboq.com>
+** Contact: http://www.qt-project.org/legal
 **
 ** This file is part of the tools applications of the Qt Toolkit.
 **
@@ -64,7 +65,7 @@ uint nameToBuiltinType(const QByteArray &name)
 }
 
 /*
-  Returns true if the type is a built-in type.
+  Returns \c true if the type is a built-in type.
 */
 bool isBuiltinType(const QByteArray &type)
  {
@@ -171,18 +172,30 @@ bool Generator::registerableMetaType(const QByteArray &propertyType)
 #undef STREAM_1ARG_TEMPLATE
     ;
     foreach (const QByteArray &oneArgTemplateType, oneArgTemplates)
-        if (propertyType.startsWith(oneArgTemplateType + "<") && !propertyType.endsWith("&")) {
+        if (propertyType.startsWith(oneArgTemplateType + "<") && propertyType.endsWith(">")) {
             const int argumentSize = propertyType.size() - oneArgTemplateType.size() - 1
                                      // The closing '>'
                                      - 1
                                      // templates inside templates have an extra whitespace char to strip.
-                                     - (propertyType.at(propertyType.size() - 2) == '>' ? 1 : 0 );
+                                     - (propertyType.at(propertyType.size() - 2) == ' ' ? 1 : 0 );
             const QByteArray templateArg = propertyType.mid(oneArgTemplateType.size() + 1, argumentSize);
             return isBuiltinType(templateArg) || registerableMetaType(templateArg);
         }
     return false;
 }
 
+/* returns \c true if name and qualifiedName refers to the same name.
+ * If qualified name is "A::B::C", it returns \c true for "C", "B::C" or "A::B::C" */
+static bool qualifiedNameEquals(const QByteArray &qualifiedName, const QByteArray &name)
+{
+    if (qualifiedName == name)
+        return true;
+    int index = qualifiedName.indexOf("::");
+    if (index == -1)
+        return false;
+    return qualifiedNameEquals(qualifiedName.mid(index+2), name);
+}
+
 void Generator::generateCode()
 {
     bool isQt = (cdef->classname == "Qt");
@@ -297,7 +310,7 @@ void Generator::generateCode()
                 int escapeLen = lengthOfEscapeSequence(s, backSlashPos);
                 spanLen = qBound(spanLen, backSlashPos + escapeLen - idx, s.length() - idx);
             }
-            fwrite(s.constData() + idx, 1, spanLen, out);
+            fprintf(out, "%.*s", spanLen, s.constData() + idx);
             idx += spanLen;
             col += spanLen;
         }
@@ -430,7 +443,7 @@ void Generator::generateCode()
             int s = p.type.lastIndexOf("::");
             if (s > 0) {
                 QByteArray scope = p.type.left(s);
-                if (scope != "Qt" && scope != cdef->classname && !extraList.contains(scope))
+                if (scope != "Qt" && !qualifiedNameEquals(cdef->qualified, scope)  && !extraList.contains(scope))
                     extraList += scope;
             }
         }
@@ -445,7 +458,7 @@ void Generator::generateCode()
         int s = enumKey.lastIndexOf("::");
         if (s > 0) {
             QByteArray scope = enumKey.left(s);
-            if (scope != "Qt" && scope != cdef->classname && !extraList.contains(scope))
+            if (scope != "Qt" && !qualifiedNameEquals(cdef->qualified, scope) && !extraList.contains(scope))
                 extraList += scope;
         }
     }
@@ -522,11 +535,6 @@ void Generator::generateCode()
     }
     if (!purestSuperClass.isEmpty() && !isQObject) {
         QByteArray superClass = purestSuperClass;
-        // workaround for VC6
-        if (superClass.contains("::")) {
-            fprintf(out, "    typedef %s QMocSuperClass;\n", superClass.constData());
-            superClass = "QMocSuperClass";
-        }
         fprintf(out, "    return %s::qt_metacast(_clname);\n", superClass.constData());
     } else {
         fprintf(out, "    return 0;\n");
@@ -719,7 +727,9 @@ void Generator::generateProperties()
         uint flags = Invalid;
         if (!isBuiltinType(p.type))
             flags |= EnumOrFlag;
-        if (!p.read.isEmpty())
+        if (!p.member.isEmpty() && !p.constant)
+            flags |= Writable;
+        if (!p.read.isEmpty() || !p.member.isEmpty())
             flags |= Readable;
         if (!p.write.isEmpty()) {
             flags |= Writable;
@@ -846,11 +856,6 @@ void Generator::generateMetacall()
 
     if (!purestSuperClass.isEmpty() && !isQObject) {
         QByteArray superClass = purestSuperClass;
-        // workaround for VC6
-        if (superClass.contains("::")) {
-            fprintf(out, "    typedef %s QMocSuperClass;\n", superClass.constData());
-            superClass = "QMocSuperClass";
-        }
         fprintf(out, "    _id = %s::qt_metacall(_c, _id, _a);\n", superClass.constData());
     }
 
@@ -893,12 +898,12 @@ void Generator::generateMetacall()
         bool needUser = false;
         for (int i = 0; i < cdef->propertyList.size(); ++i) {
             const PropertyDef &p = cdef->propertyList.at(i);
-            needGet |= !p.read.isEmpty();
-            if (!p.read.isEmpty())
+            needGet |= !p.read.isEmpty() || !p.member.isEmpty();
+            if (!p.read.isEmpty() || !p.member.isEmpty())
                 needTempVarForGet |= (p.gspec != PropertyDef::PointerSpec
                                       && p.gspec != PropertyDef::ReferenceSpec);
 
-            needSet |= !p.write.isEmpty();
+            needSet |= !p.write.isEmpty() || (!p.member.isEmpty() && !p.constant);
             needReset |= !p.reset.isEmpty();
             needDesignable |= p.designable.endsWith(')');
             needScriptable |= p.scriptable.endsWith(')');
@@ -917,7 +922,7 @@ void Generator::generateMetacall()
             fprintf(out, "        switch (_id) {\n");
             for (int propindex = 0; propindex < cdef->propertyList.size(); ++propindex) {
                 const PropertyDef &p = cdef->propertyList.at(propindex);
-                if (p.read.isEmpty())
+                if (p.read.isEmpty() && p.member.isEmpty())
                     continue;
                 QByteArray prefix;
                 if (p.inPrivateClass.size()) {
@@ -933,9 +938,12 @@ void Generator::generateMetacall()
                 else if (cdef->enumDeclarations.value(p.type, false))
                     fprintf(out, "        case %d: *reinterpret_cast<int*>(_v) = QFlag(%s%s()); break;\n",
                             propindex, prefix.constData(), p.read.constData());
-                else
+                else if (!p.read.isEmpty())
                     fprintf(out, "        case %d: *reinterpret_cast< %s*>(_v) = %s%s(); break;\n",
                             propindex, p.type.constData(), prefix.constData(), p.read.constData());
+                else
+                    fprintf(out, "        case %d: *reinterpret_cast< %s*>(_v) = %s%s; break;\n",
+                            propindex, p.type.constData(), prefix.constData(), p.member.constData());
             }
             fprintf(out, "        }\n");
         }
@@ -952,7 +960,9 @@ void Generator::generateMetacall()
             fprintf(out, "        switch (_id) {\n");
             for (int propindex = 0; propindex < cdef->propertyList.size(); ++propindex) {
                 const PropertyDef &p = cdef->propertyList.at(propindex);
-                if (p.write.isEmpty())
+                if (p.constant)
+                    continue;
+                if (p.write.isEmpty() && p.member.isEmpty())
                     continue;
                 QByteArray prefix;
                 if (p.inPrivateClass.size()) {
@@ -962,9 +972,25 @@ void Generator::generateMetacall()
                 if (cdef->enumDeclarations.value(p.type, false)) {
                     fprintf(out, "        case %d: %s%s(QFlag(*reinterpret_cast<int*>(_v))); break;\n",
                             propindex, prefix.constData(), p.write.constData());
-                } else {
+                } else if (!p.write.isEmpty()) {
                     fprintf(out, "        case %d: %s%s(*reinterpret_cast< %s*>(_v)); break;\n",
                             propindex, prefix.constData(), p.write.constData(), p.type.constData());
+                } else {
+                    fprintf(out, "        case %d:\n", propindex);
+                    fprintf(out, "            if (%s%s != *reinterpret_cast< %s*>(_v)) {\n",
+                            prefix.constData(), p.member.constData(), p.type.constData());
+                    fprintf(out, "                %s%s = *reinterpret_cast< %s*>(_v);\n",
+                            prefix.constData(), p.member.constData(), p.type.constData());
+                    if (!p.notify.isEmpty() && p.notifyId != -1) {
+                        const FunctionDef &f = cdef->signalList.at(p.notifyId);
+                        if (f.arguments.size() == 0)
+                            fprintf(out, "                Q_EMIT %s();\n", p.notify.constData());
+                        else if (f.arguments.size() == 1 && f.arguments.at(0).normalizedType == p.type)
+                            fprintf(out, "                Q_EMIT %s(%s%s);\n",
+                                    p.notify.constData(), prefix.constData(), p.member.constData());
+                    }
+                    fprintf(out, "            }\n");
+                    fprintf(out, "            break;\n");
                 }
             }
             fprintf(out, "        }\n");
@@ -1436,6 +1462,10 @@ void Generator::generatePluginMetaData()
     data.insert(debugKey, QJsonValue(false));
     data.insert(QStringLiteral("MetaData"), cdef->pluginData.metaData.object());
 
+    // Add -M args from the command line:
+    foreach (const QString &key, cdef->pluginData.metaArgs.keys())
+        data.insert(key, cdef->pluginData.metaArgs.value(key));
+
     fputs("\nQT_PLUGIN_METADATA_SECTION const uint qt_section_alignment_dummy = 42;\n\n"
           "#ifdef QT_NO_DEBUG\n", out);
     writePluginMetaData(out, data);
