@@ -787,6 +787,10 @@ void Moc::parse()
             if (!def.hasQObject && !def.hasQGadget)
                 error("Class declarations lacks Q_OBJECT macro.");
 
+            // Add meta tags to the plugin meta data:
+            if (!def.pluginData.iid.isEmpty())
+                def.pluginData.metaArgs = metaArgs;
+
             checkSuperClasses(&def);
             checkProperties(&def);
 
@@ -870,7 +874,12 @@ void Moc::generate(FILE *out)
         findRequiredContainers(&classList[i], &requiredQtContainers);
     }
 
-    foreach (const QByteArray &qtContainer, requiredQtContainers) {
+    // after finding the containers, we sort them into a list to avoid
+    // non-deterministic behavior which may cause rebuilds unnecessarily.
+    QList<QByteArray> requiredContainerList = requiredQtContainers.toList();
+    std::sort(requiredContainerList.begin(), requiredContainerList.end());
+
+    foreach (const QByteArray &qtContainer, requiredContainerList) {
         fprintf(out, "#include <QtCore/%s>\n", qtContainer.constData());
     }
 
@@ -983,7 +992,7 @@ void Moc::parseSignals(ClassDef *def)
             prev();
         }
         FunctionDef funcDef;
-        funcDef.access = FunctionDef::Protected;
+        funcDef.access = FunctionDef::Public;
         parseFunction(&funcDef);
         if (funcDef.isVirtual)
             warning("Signals cannot be declared virtual");
@@ -1059,6 +1068,12 @@ void Moc::createPropertyDef(PropertyDef &propDef)
                 v2 = "()";
         }
         switch (l[0]) {
+        case 'M':
+            if (l == "MEMBER")
+                propDef.member = v;
+            else
+                error(2);
+            break;
         case 'R':
             if (l == "READ")
                 propDef.read = v;
@@ -1099,11 +1114,11 @@ void Moc::createPropertyDef(PropertyDef &propDef)
             error(2);
         }
     }
-    if (propDef.read.isNull()) {
+    if (propDef.read.isNull() && propDef.member.isNull()) {
         QByteArray msg;
         msg += "Property declaration ";
         msg += propDef.name;
-        msg += " has no READ accessor function. The property will be invalid.";
+        msg += " has no READ accessor function or associated MEMBER variable. The property will be invalid.";
         warning(msg.constData());
     }
     if (propDef.constant && !propDef.write.isNull()) {
@@ -1443,6 +1458,11 @@ bool Moc::until(Token target) {
             --index;
             break;
         }
+
+        if (braceCount <= 0 && t == SEMIC) {
+            // Abort on semicolon. Allow recovering bad template parsing (QTBUG-31218)
+            break;
+        }
     }
 
     if(target == COMMA && angleCount != 0 && possible != -1) {
@@ -1515,7 +1535,7 @@ void Moc::checkProperties(ClassDef *cdef)
     //
     for (int i = 0; i < cdef->propertyList.count(); ++i) {
         PropertyDef &p = cdef->propertyList[i];
-        if (p.read.isEmpty())
+        if (p.read.isEmpty() && p.member.isEmpty())
             continue;
         for (int j = 0; j < cdef->publicList.count(); ++j) {
             const FunctionDef &f = cdef->publicList.at(j);
