@@ -47,6 +47,8 @@
 #include <qmap.h>
 #include <qpair.h>
 #include <qjsondocument.h>
+#include <qjsonarray.h>
+#include <qjsonobject.h>
 #include <stdio.h>
 #include <ctype.h>
 
@@ -127,7 +129,7 @@ struct FunctionDef
 struct PropertyDef
 {
     PropertyDef():notifyId(-1), constant(false), final(false), gspec(ValueSpec), revision(0){}
-    QByteArray name, type, read, write, reset, designable, scriptable, editable, stored, user, notify, inPrivateClass;
+    QByteArray name, type, member, read, write, reset, designable, scriptable, editable, stored, user, notify, inPrivateClass;
     int notifyId;
     bool constant;
     bool final;
@@ -171,6 +173,7 @@ struct ClassDef {
 
     struct PluginData {
         QByteArray iid;
+        QMap<QString, QJsonArray> metaArgs;
         QJsonDocument metaData;
     } pluginData;
 
@@ -213,6 +216,7 @@ public:
     QMap<QByteArray, QByteArray> interface2IdMap;
     QList<QByteArray> metaTypes;
     QSet<QByteArray> knownQObjectClasses;
+    QMap<QString, QJsonArray> metaArgs;
 
     void parse();
     void generate(FILE *out);
