@@ -55,8 +55,80 @@
 
 QT_BEGIN_NAMESPACE
 
-/*
-  An entry on the MetaStack.
+QString ConfigStrings::ALIAS = QStringLiteral("alias");
+QString ConfigStrings::BASE = QStringLiteral("base");
+QString ConfigStrings::BASEDIR = QStringLiteral("basedir");
+QString ConfigStrings::BUILDVERSION = QStringLiteral("buildversion");
+QString ConfigStrings::CODEINDENT = QStringLiteral("codeindent");
+QString ConfigStrings::CPPCLASSESPAGE = QStringLiteral("cppclassespage");
+QString ConfigStrings::DEFINES = QStringLiteral("defines");
+QString ConfigStrings::DEPENDS = QStringLiteral("depends");
+QString ConfigStrings::DESCRIPTION = QStringLiteral("description");
+QString ConfigStrings::EDITION = QStringLiteral("edition");
+QString ConfigStrings::ENDHEADER = QStringLiteral("endheader");
+QString ConfigStrings::EXAMPLEDIRS = QStringLiteral("exampledirs");
+QString ConfigStrings::EXAMPLES = QStringLiteral("examples");
+QString ConfigStrings::EXAMPLESINSTALLPATH = QStringLiteral("examplesinstallpath");
+QString ConfigStrings::EXCLUDEDIRS = QStringLiteral("excludedirs");
+QString ConfigStrings::EXCLUDEFILES = QStringLiteral("excludefiles");
+QString ConfigStrings::EXTRAIMAGES = QStringLiteral("extraimages");
+QString ConfigStrings::FALSEHOODS = QStringLiteral("falsehoods");
+QString ConfigStrings::FORMATTING = QStringLiteral("formatting");
+QString ConfigStrings::GENERATEINDEX = QStringLiteral("generateindex");
+QString ConfigStrings::HEADERDIRS = QStringLiteral("headerdirs");
+QString ConfigStrings::HEADERS = QStringLiteral("headers");
+QString ConfigStrings::HEADERSCRIPTS = QStringLiteral("headerscripts");
+QString ConfigStrings::HEADERSTYLES = QStringLiteral("headerstyles");
+QString ConfigStrings::HOMEPAGE = QStringLiteral("homepage");
+QString ConfigStrings::IGNOREDIRECTIVES = QStringLiteral("ignoredirectives");
+QString ConfigStrings::IGNORETOKENS = QStringLiteral("ignoretokens");
+QString ConfigStrings::IMAGEDIRS = QStringLiteral("imagedirs");
+QString ConfigStrings::IMAGES = QStringLiteral("images");
+QString ConfigStrings::INDEXES = QStringLiteral("indexes");
+QString ConfigStrings::LANDINGPAGE = QStringLiteral("landingpage");
+QString ConfigStrings::LANGUAGE = QStringLiteral("language");
+QString ConfigStrings::MACRO = QStringLiteral("macro");
+QString ConfigStrings::MANIFESTMETA = QStringLiteral("manifestmeta");
+QString ConfigStrings::NATURALLANGUAGE = QStringLiteral("naturallanguage");
+QString ConfigStrings::NAVIGATION = QStringLiteral("navigation");
+QString ConfigStrings::NOLINKERRORS = QStringLiteral("nolinkerrors");
+QString ConfigStrings::OBSOLETELINKS = QStringLiteral("obsoletelinks");
+QString ConfigStrings::OUTPUTDIR = QStringLiteral("outputdir");
+QString ConfigStrings::OUTPUTENCODING = QStringLiteral("outputencoding");
+QString ConfigStrings::OUTPUTLANGUAGE = QStringLiteral("outputlanguage");
+QString ConfigStrings::OUTPUTFORMATS = QStringLiteral("outputformats");
+QString ConfigStrings::OUTPUTPREFIXES = QStringLiteral("outputprefixes");
+QString ConfigStrings::PROJECT = QStringLiteral("project");
+QString ConfigStrings::REDIRECTDOCUMENTATIONTODEVNULL = QStringLiteral("redirectdocumentationtodevnull");
+QString ConfigStrings::QHP = QStringLiteral("qhp");
+QString ConfigStrings::QUOTINGINFORMATION = QStringLiteral("quotinginformation");
+QString ConfigStrings::SCRIPTDIRS = QStringLiteral("scriptdirs");
+QString ConfigStrings::SCRIPTS = QStringLiteral("scripts");
+QString ConfigStrings::SHOWINTERNAL = QStringLiteral("showinternal");
+QString ConfigStrings::SOURCEDIRS = QStringLiteral("sourcedirs");
+QString ConfigStrings::SOURCEENCODING = QStringLiteral("sourceencoding");
+QString ConfigStrings::SOURCES = QStringLiteral("sources");
+QString ConfigStrings::SPURIOUS = QStringLiteral("spurious");
+QString ConfigStrings::STYLEDIRS = QStringLiteral("styledirs");
+QString ConfigStrings::STYLE = QStringLiteral("style");
+QString ConfigStrings::STYLES = QStringLiteral("styles");
+QString ConfigStrings::STYLESHEETS = QStringLiteral("stylesheets");
+QString ConfigStrings::SYNTAXHIGHLIGHTING = QStringLiteral("syntaxhighlighting");
+QString ConfigStrings::TEMPLATEDIR = QStringLiteral("templatedir");
+QString ConfigStrings::TABSIZE = QStringLiteral("tabsize");
+QString ConfigStrings::TAGFILE = QStringLiteral("tagfile");
+QString ConfigStrings::TRANSLATORS = QStringLiteral("translators");
+QString ConfigStrings::URL = QStringLiteral("url");
+QString ConfigStrings::VERSION = QStringLiteral("version");
+QString ConfigStrings::VERSIONSYM = QStringLiteral("versionsym");
+QString ConfigStrings::FILEEXTENSIONS = QStringLiteral("fileextensions");
+QString ConfigStrings::IMAGEEXTENSIONS = QStringLiteral("imageextensions");
+QString ConfigStrings::QMLONLY = QStringLiteral("qmlonly");
+QString ConfigStrings::QMLTYPESPAGE = QStringLiteral("qmltypespage");
+
+/*!
+  An entry in a stack, where each entry is a list
+  of string values.
  */
 class MetaStackEntry
 {
@@ -68,14 +140,19 @@ public:
     QStringList next;
 };
 
-/*
+/*!
+  Start accumulating values in a list by appending an empty
+  string to the list.
  */
 void MetaStackEntry::open()
 {
     next.append(QString());
 }
 
-/*
+/*!
+  Stop accumulating values and append the list of accumulated
+  values to the complete list of accumulated values.
+
  */
 void MetaStackEntry::close()
 {
@@ -83,8 +160,10 @@ void MetaStackEntry::close()
     next.clear();
 }
 
-/*
-  ###
+/*!
+  \class MetaStack
+
+  This class maintains a stack of values of config file variables.
 */
 class MetaStack : private QStack<MetaStackEntry>
 {
@@ -97,19 +176,27 @@ public:
     QStringList getExpanded(const Location& location);
 };
 
+/*!
+  The default constructor pushes a new stack entry and
+  opens it.
+ */
 MetaStack::MetaStack()
 {
     push(MetaStackEntry());
     top().open();
 }
 
+/*!
+  Processes the character \a ch using the \a location.
+  It really just builds up a name by appending \a ch to
+  it.
+ */
 void MetaStack::process(QChar ch, const Location& location)
 {
     if (ch == QLatin1Char('{')) {
         push(MetaStackEntry());
         top().open();
-    }
-    else if (ch == QLatin1Char('}')) {
+    } else if (ch == QLatin1Char('}')) {
         if (count() == 1)
             location.fatal(tr("Unexpected '}'"));
 
@@ -127,12 +214,13 @@ void MetaStack::process(QChar ch, const Location& location)
             }
             ++pre;
         }
-    }
-    else if (ch == QLatin1Char(',') && count() > 1) {
+    } else if (ch == QLatin1Char(',') && count() > 1) {
         top().close();
         top().open();
-    }
-    else {
+    } else {
+        /*
+          This is where all the processing is done.
+         */
         QStringList::Iterator pre = top().next.begin();
         while (pre != top().next.end()) {
             *pre += ch;
@@ -141,6 +229,9 @@ void MetaStack::process(QChar ch, const Location& location)
     }
 }
 
+/*!
+  Returns the accumulated string values.
+ */
 QStringList MetaStack::getExpanded(const Location& location)
 {
     if (count() > 1)
@@ -158,6 +249,7 @@ QSet<QString> Config::overrideOutputFormats;
 QMap<QString, QString> Config::extractedDirs;
 int Config::numInstances;
 QStack<QString> Config::workingDirs_;
+QMap<QString, QStringList> Config::includeFilesMap_;
 
 /*!
   \class Config
@@ -176,10 +268,9 @@ Config::Config(const QString& programName)
 {
     loc = Location::null;
     lastLocation_ = Location::null;
-    locMap.clear();
-    stringPairMap.clear();
-    stringListPairMap.clear();
+    configVars_.clear();
     numInstances++;
+    includeFilesMap_.clear();
 }
 
 /*!
@@ -187,6 +278,7 @@ Config::Config(const QString& programName)
  */
 Config::~Config()
 {
+    includeFilesMap_.clear();
 }
 
 /*!
@@ -201,28 +293,13 @@ Config::~Config()
 void Config::load(const QString& fileName)
 {
     load(Location::null, fileName);
-    if (loc.isEmpty()) {
+    if (loc.isEmpty())
         loc = Location(fileName);
-    }
-    else {
+    else
         loc.setEtc(true);
-    }
     lastLocation_ = Location::null;
 }
 
-/*!
-  Writes the qdoc configuration data to the named file.
-  The previous contents of the file are overwritten.
- */
-void Config::unload(const QString& fileName)
-{
-    QStringPairMap::ConstIterator v = stringPairMap.constBegin();
-    while (v != stringPairMap.constEnd()) {
-        qDebug() << v.key() << " = " << v.value().second;
-        ++v;
-    }
-    qDebug() << "fileName:" << fileName;
-}
 /*!
   Joins all the strings in \a values into a single string with the
   individual \a values separated by ' '. Then it inserts the result
@@ -233,10 +310,7 @@ void Config::unload(const QString& fileName)
  */
 void Config::setStringList(const QString& var, const QStringList& values)
 {
-    stringPairMap[var].first = QDir::currentPath();
-    stringPairMap[var].second = values.join(QLatin1Char(' '));
-    stringListPairMap[var].first = QDir::currentPath();
-    stringListPairMap[var].second = values;
+    configVars_.insert(var,ConfigVar(var, values, QDir::currentPath()));
 }
 
 /*!
@@ -274,10 +348,19 @@ int Config::getInt(const QString& var) const
   */
 QString Config::getOutputDir() const
 {
+    QString t;
     if (overrideOutputDir.isNull())
-        return getString(QLatin1String(CONFIG_OUTPUTDIR));
+        t = getString(CONFIG_OUTPUTDIR);
     else
-        return overrideOutputDir;
+        t = overrideOutputDir;
+    if (!Generator::useOutputSubdirs()) {
+        t = t.left(t.lastIndexOf('/'));
+        QString singleOutputSubdir = getString("HTML.outputsubdir");
+        if (singleOutputSubdir.isEmpty())
+            singleOutputSubdir = "html";
+        t += QLatin1Char('/') + singleOutputSubdir;
+    }
+    return t;
 }
 
 /*!
@@ -288,7 +371,7 @@ QString Config::getOutputDir() const
 QSet<QString> Config::getOutputFormats() const
 {
     if (overrideOutputFormats.isEmpty())
-        return getStringSet(QLatin1String(CONFIG_OUTPUTFORMATS));
+        return getStringSet(CONFIG_OUTPUTFORMATS);
     else
         return overrideOutputFormats;
 }
@@ -303,30 +386,27 @@ QSet<QString> Config::getOutputFormats() const
  */
 QString Config::getString(const QString& var) const
 {
-    if (!locMap[var].isEmpty())
-        (Location&) lastLocation_ = locMap[var];
-    return stringPairMap[var].second;
-}
-
-/*!
-  This function looks up the variable \a var in the location map
-  and, if found, sets the internal variable \c{lastLocation_} to the
-  location that \a var maps to.
-
-  Then it looks up \a var in the configuration variable map and,
-  if found, constructs a path from the pair value, which consists
-  of the directory path of the configuration file where the value
-  came from, and the value itself. The constructed path is returned.
- */
-QString Config::getPath(const QString& var) const
-{
-    if (!locMap[var].isEmpty())
-        (Location&) lastLocation_ = locMap[var];
-    QString path;
-    if (stringPairMap.contains(var)) {
-        path = QDir(stringPairMap[var].first + "/" + stringPairMap[var].second).absolutePath();
+    QList<ConfigVar> configVars = configVars_.values(var);
+    QString value;
+    if (!configVars.empty()) {
+        int i = configVars.size() - 1;
+        while (i >= 0) {
+            const ConfigVar& cv = configVars[i];
+            if (!cv.location_.isEmpty())
+                (Location&) lastLocation_ = cv.location_;
+            if (!cv.values_.isEmpty()) {
+                if (!cv.plus_)
+                    value.clear();
+                for (int j=0; j<cv.values_.size(); ++j) {
+                    if (!value.isEmpty() && !value.endsWith(QChar('\n')))
+                        value.append(QChar(' '));
+                    value.append(cv.values_[j]);
+                }
+            }
+            --i;
+        }
     }
-    return path;
+    return value;
 }
 
 /*!
@@ -341,40 +421,64 @@ QSet<QString> Config::getStringSet(const QString& var) const
 
 /*!
   First, this function looks up the configuration variable \a var
-  in the location map and, if found, sets the internal variable
+  in the location map. If found, it sets the internal variable
   \c{lastLocation_} to the Location that \a var maps to.
 
-  Then it looks up the configuration variable \a var in the string
-  list map, and returns the string list that \a var maps to.
+  Then it looks up the configuration variable \a var in the map of
+  configuration variable records. If found, it gets a list of all
+  the records for \a var. Then it appends all the values for \a var
+  to a list and returns the list. As it appends the values from each
+  record, if the \a var used '=' instead of '+=' the list is cleared
+  before the values are appended. \note '+=' should always be used.
+  The final list is returned.
  */
 QStringList Config::getStringList(const QString& var) const
 {
-    if (!locMap[var].isEmpty())
-        (Location&) lastLocation_ = locMap[var];
-    return stringListPairMap[var].second;
+    QList<ConfigVar> configVars = configVars_.values(var);
+    QStringList values;
+    if (!configVars.empty()) {
+        int i = configVars.size() - 1;
+        while (i >= 0) {
+            if (!configVars[i].location_.isEmpty())
+                (Location&) lastLocation_ = configVars[i].location_;
+            if (configVars[i].plus_)
+                values.append(configVars[i].values_);
+            else
+                values = configVars[i].values_;
+            --i;
+        }
+    }
+    return values;
 }
 
-
 /*!
    \brief Returns the a path list where all paths are canonicalized, then
           made relative to the config file.
    \param var The variable containing the list of paths.
    \see   Location::canonicalRelativePath()
  */
-QStringList Config::getCanonicalRelativePathList(const QString& var) const
+QStringList Config::getCanonicalPathList(const QString& var) const
 {
-    if (!locMap[var].isEmpty())
-        (Location&) lastLocation_ = locMap[var];
     QStringList t;
-    QStringListPairMap::const_iterator it = stringListPairMap.constFind(var);
-    if (it != stringListPairMap.constEnd()) {
-        const QStringList& sl = it.value().second;
-        if (!sl.isEmpty()) {
-            t.reserve(sl.size());
-            for (int i=0; i<sl.size(); ++i) {
-                const QString &canonicalized = location().canonicalRelativePath(sl[i]);
-                t.append(canonicalized);
+    QList<ConfigVar> configVars = configVars_.values(var);
+    if (!configVars.empty()) {
+        int i = configVars.size() - 1;
+        while (i >= 0) {
+            const ConfigVar& cv = configVars[i];
+            if (!cv.location_.isEmpty())
+                (Location&) lastLocation_ = cv.location_;
+            if (!cv.plus_)
+                t.clear();
+            const QString d = cv.currentPath_;
+            const QStringList& sl = cv.values_;
+            if (!sl.isEmpty()) {
+                t.reserve(t.size() + sl.size());
+                for (int i=0; i<sl.size(); ++i) {
+                    QDir dir(d + "/" + sl[i]);
+                    t.append(dir.canonicalPath());
+                }
             }
+            --i;
         }
     }
     return t;
@@ -382,32 +486,30 @@ QStringList Config::getCanonicalRelativePathList(const QString& var) const
 
 /*!
   This function should only be called when the configuration
-  variable \a var maps to a string list that contains file paths.
+  variable \a var maps to string lists that contain file paths.
   It cleans the paths with QDir::cleanPath() before returning
   them.
-
-  First, this function looks up the configuration variable \a var
-  in the location map and, if found, sets the internal variable
-  \c{lastLocation_} the Location that \a var maps to.
-
-  Then it looks up the configuration variable \a var in the string
-  list map, which maps to a string list that contains file paths.
-  These paths might not be clean, so QDir::cleanPath() is called
-  for each one. The string list returned contains cleaned paths.
  */
 QStringList Config::getCleanPathList(const QString& var) const
 {
-    if (!locMap[var].isEmpty())
-        (Location&) lastLocation_ = locMap[var];
     QStringList t;
-    QStringListPairMap::const_iterator it = stringListPairMap.constFind(var);
-    if (it != stringListPairMap.constEnd()) {
-        const QStringList& sl = it.value().second;
-        if (!sl.isEmpty()) {
-            t.reserve(sl.size());
-            for (int i=0; i<sl.size(); ++i) {
-                t.append(QDir::cleanPath(sl[i]));
+    QList<ConfigVar> configVars = configVars_.values(var);
+    if (!configVars.empty()) {
+        int i = configVars.size() - 1;
+        while (i >= 0) {
+            const ConfigVar& cv = configVars[i];
+            if (!cv.plus_)
+                t.clear();
+            if (!cv.location_.isEmpty())
+                (Location&) lastLocation_ = cv.location_;
+            const QStringList& sl = cv.values_;
+            if (!sl.isEmpty()) {
+                t.reserve(t.size() + sl.size());
+                for (int i=0; i<sl.size(); ++i) {
+                    t.append(QDir::cleanPath(sl[i].simplified()));
+                }
             }
+            --i;
         }
     }
     return t;
@@ -415,7 +517,7 @@ QStringList Config::getCleanPathList(const QString& var) const
 
 /*!
   This function should only be called when the configuration
-  variable \a var maps to a string list that contains file paths.
+  variable \a var maps to string lists that contain file paths.
   It cleans the paths with QDir::cleanPath() before returning
   them.
 
@@ -424,30 +526,39 @@ QStringList Config::getCleanPathList(const QString& var) const
   \c{lastLocation_} the Location that \a var maps to.
 
   Then it looks up the configuration variable \a var in the string
-  list map, which maps to a string list that contains file paths.
+  list map, which maps to one or more records that each contains a
+  list of file paths.
+
   These paths might not be clean, so QDir::cleanPath() is called
   for each one. The string list returned contains cleaned paths.
  */
 QStringList Config::getPathList(const QString& var) const
 {
-    if (!locMap[var].isEmpty())
-        (Location&) lastLocation_ = locMap[var];
     QStringList t;
-    QStringListPairMap::const_iterator it = stringListPairMap.constFind(var);
-    if (it != stringListPairMap.constEnd()) {
-        const QStringList& sl = it.value().second;
-        const QString d = it.value().first;
-        if (!sl.isEmpty()) {
-            t.reserve(sl.size());
-            for (int i=0; i<sl.size(); ++i) {
-                QFileInfo fileInfo;
-                QString path = d + "/" + QDir::cleanPath(sl[i]);
-                fileInfo.setFile(path);
-                if (!fileInfo.exists())
-                    lastLocation_.warning(tr("File '%1' does not exist").arg(path));
-                else
-                    t.append(path);
+    QList<ConfigVar> configVars = configVars_.values(var);
+    if (!configVars.empty()) {
+        int i = configVars.size() - 1;
+        while (i >= 0) {
+            const ConfigVar& cv = configVars[i];
+            if (!cv.plus_)
+                t.clear();
+            if (!cv.location_.isEmpty())
+                (Location&) lastLocation_ = cv.location_;
+            const QString d = cv.currentPath_;
+            const QStringList& sl = cv.values_;
+            if (!sl.isEmpty()) {
+                t.reserve(t.size() + sl.size());
+                for (int i=0; i<sl.size(); ++i) {
+                    QFileInfo fileInfo;
+                    QString path = d + "/" + QDir::cleanPath(sl[i].simplified());
+                    fileInfo.setFile(path);
+                    if (!fileInfo.exists())
+                        lastLocation_.warning(tr("File '%1' does not exist").arg(path));
+                    else
+                        t.append(path);
+                }
             }
+            --i;
         }
     }
     return t;
@@ -509,30 +620,31 @@ QSet<QString> Config::subVars(const QString& var) const
 {
     QSet<QString> result;
     QString varDot = var + QLatin1Char('.');
-    QStringPairMap::ConstIterator v = stringPairMap.constBegin();
-    while (v != stringPairMap.constEnd()) {
-        if (v.key().startsWith(varDot)) {
-            QString subVar = v.key().mid(varDot.length());
+    ConfigVarMultimap::ConstIterator i = configVars_.constBegin();
+    while (i != configVars_.constEnd()) {
+        if (i.key().startsWith(varDot)) {
+            QString subVar = i.key().mid(varDot.length());
             int dot = subVar.indexOf(QLatin1Char('.'));
             if (dot != -1)
                 subVar.truncate(dot);
-            result.insert(subVar);
+            if (!result.contains(subVar))
+                result.insert(subVar);
         }
-        ++v;
+        ++i;
     }
     return result;
 }
 
 /*!
-  Same as subVars(), but in this case we return a string map
-  with the matching keys (stripped of the prefix \a var and
-  mapped to their values. The pairs are inserted into \a t
+  Same as subVars(), but in this case we return a config var
+  multimap with the matching keys (stripped of the prefix \a var
+  and mapped to their values. The pairs are inserted into \a t
  */
-void Config::subVarsAndValues(const QString& var, QStringPairMap& t) const
+void Config::subVarsAndValues(const QString& var, ConfigVarMultimap& t) const
 {
     QString varDot = var + QLatin1Char('.');
-    QStringPairMap::ConstIterator v = stringPairMap.constBegin();
-    while (v != stringPairMap.constEnd()) {
+    ConfigVarMultimap::ConstIterator v = configVars_.constBegin();
+    while (v != configVars_.constEnd()) {
         if (v.key().startsWith(varDot)) {
             QString subVar = v.key().mid(varDot.length());
             int dot = subVar.indexOf(QLatin1Char('.'));
@@ -544,6 +656,33 @@ void Config::subVarsAndValues(const QString& var, QStringPairMap& t) const
     }
 }
 
+/*!
+  Get all .qdocinc files.
+ */
+QString Config::getIncludeFilePath(const QString& fileName) const
+{
+    QString ext = fileName.mid(fileName.lastIndexOf('.'));
+    ext.prepend('*');
+
+    if (!includeFilesMap_.contains(ext)) {
+        QSet<QString> t;
+        QStringList result;
+        QStringList dirs = getCanonicalPathList(CONFIG_SOURCEDIRS);
+        QStringList::ConstIterator d = dirs.constBegin();
+        while (d != dirs.constEnd()) {
+            result += getFilesHere(*d, ext, location(), t, t);
+            ++d;
+        }
+        includeFilesMap_.insert(ext, result);
+    }
+    const QStringList& paths = (*includeFilesMap_.find(ext));
+    for (int i=0; i<paths.size(); ++i) {
+        if (paths[i].endsWith(fileName))
+            return paths[i];
+    }
+    return QString();
+}
+
 /*!
   Builds and returns a list of file pathnames for the file
   type specified by \a filesVar (e.g. "headers" or "sources").
@@ -558,10 +697,10 @@ QStringList Config::getAllFiles(const QString &filesVar,
                                 const QSet<QString> &excludedDirs,
                                 const QSet<QString> &excludedFiles)
 {
-    QStringList result = getStringList(filesVar);
-    QStringList dirs = getCanonicalRelativePathList(dirsVar);
+    QStringList result = getCanonicalPathList(filesVar);
+    QStringList dirs = getCanonicalPathList(dirsVar);
 
-    QString nameFilter = getString(filesVar + dot + QLatin1String(CONFIG_FILEEXTENSIONS));
+    QString nameFilter = getString(filesVar + dot + CONFIG_FILEEXTENSIONS);
 
     QStringList::ConstIterator d = dirs.constBegin();
     while (d != dirs.constEnd()) {
@@ -575,7 +714,7 @@ QStringList Config::getExampleQdocFiles(const QSet<QString> &excludedDirs,
                                         const QSet<QString> &excludedFiles)
 {
     QStringList result;
-    QStringList dirs = getCanonicalRelativePathList("exampledirs");
+    QStringList dirs = getCanonicalPathList("exampledirs");
     QString nameFilter = " *.qdoc";
 
     QStringList::ConstIterator d = dirs.constBegin();
@@ -590,8 +729,8 @@ QStringList Config::getExampleImageFiles(const QSet<QString> &excludedDirs,
                                          const QSet<QString> &excludedFiles)
 {
     QStringList result;
-    QStringList dirs = getCanonicalRelativePathList("exampledirs");
-    QString nameFilter = getString(CONFIG_EXAMPLES + dot + QLatin1String(CONFIG_IMAGEEXTENSIONS));
+    QStringList dirs = getCanonicalPathList("exampledirs");
+    QString nameFilter = getString(CONFIG_EXAMPLES + dot + CONFIG_IMAGEEXTENSIONS);
 
     QStringList::ConstIterator d = dirs.constBegin();
     while (d != dirs.constEnd()) {
@@ -641,9 +780,8 @@ QString Config::findFile(const Location& location,
         QStringList::ConstIterator d = dirs.constBegin();
         while (d != dirs.constEnd()) {
             fileInfo.setFile(QDir(*d), firstComponent);
-            if (fileInfo.exists()) {
+            if (fileInfo.exists())
                 break;
-            }
             ++d;
         }
     }
@@ -663,9 +801,9 @@ QString Config::findFile(const Location& location,
             QString extracted = extractedDirs[fileInfo.filePath()];
             ++c;
             fileInfo.setFile(QDir(extracted), *c);
-        }
-        else
+        } else {
             break;
+        }
 
         userFriendlyFilePath += QLatin1Char('?');
     }
@@ -731,9 +869,8 @@ QString Config::copyFile(const Location& location,
 
     char buffer[1024];
     int len;
-    while ((len = inFile.read(buffer, sizeof(buffer))) > 0) {
+    while ((len = inFile.read(buffer, sizeof(buffer))) > 0)
         outFile.write(buffer, len);
-    }
     return outFileName;
 }
 
@@ -787,7 +924,7 @@ bool Config::removeDirContents(const QString& dir)
 }
 
 /*!
-  Returns true if \a ch is a letter, number, '_', '.',
+  Returns \c true if \a ch is a letter, number, '_', '.',
   '{', '}', or ','.
  */
 bool Config::isMetaKeyChar(QChar ch)
@@ -842,7 +979,9 @@ void Config::load(Location location, const QString& fileName)
     }
 
     QTextStream stream(&fin);
+#ifndef QT_NO_TEXTCODEC
     stream.setCodec("UTF-8");
+#endif
     QString text = stream.readAll();
     text += QLatin1String("\n\n");
     text += QLatin1Char('\0');
@@ -855,21 +994,19 @@ void Config::load(Location location, const QString& fileName)
     QChar c = text.at(0);
     uint cc = c.unicode();
     while (i < (int) text.length()) {
-        if (cc == 0)
+        if (cc == 0) {
             ++i;
-        else if (c.isSpace()) {
+        } else if (c.isSpace()) {
             SKIP_CHAR();
-        }
-        else if (cc == '#') {
+        } else if (cc == '#') {
             do {
                 SKIP_CHAR();
             } while (cc != '\n');
-        }
-        else if (isMetaKeyChar(c)) {
+        } else if (isMetaKeyChar(c)) {
             Location keyLoc = location;
             bool plus = false;
             QString stringValue;
-            QStringList stringListValue;
+            QStringList rhsValues;
             QString word;
             bool inQuote = false;
             bool prevWordQuoted = true;
@@ -931,6 +1068,7 @@ void Config::load(Location location, const QString& fileName)
             else {
                 /*
                   It wasn't an include statement, so it's something else.
+                  We must see either '=' or '+=' next. If not, fatal error.
                  */
                 if (cc == '+') {
                     plus = true;
@@ -972,7 +1110,12 @@ void Config::load(Location location, const QString& fileName)
                                 if (metWord)
                                     stringValue += QLatin1Char(' ');
                                 stringValue += word;
-                                stringListValue << word;
+#if 0
+                                if (metWord)
+                                    rhsValues << QString(" " + word);
+                                else
+#endif
+                                rhsValues << word;
                                 metWord = true;
                                 word.clear();
                                 prevWordQuoted = false;
@@ -988,7 +1131,7 @@ void Config::load(Location location, const QString& fileName)
                                 stringValue += QLatin1Char(' ');
                             stringValue += word;
                             if (!word.isEmpty())
-                                stringListValue << word;
+                                rhsValues << word;
                             metWord = true;
                             word.clear();
                             prevWordQuoted = true;
@@ -1025,41 +1168,19 @@ void Config::load(Location location, const QString& fileName)
                     if (!keySyntax.exactMatch(*key))
                         keyLoc.fatal(tr("Invalid key '%1'").arg(*key));
 
-                    if (plus) {
-                        if (locMap[*key].isEmpty()) {
-                            locMap[*key] = keyLoc;
-                        }
-                        else {
-                            locMap[*key].setEtc(true);
-                        }
-                        if (stringPairMap[*key].second.isEmpty()) {
-                            stringPairMap[*key].first = QDir::currentPath();
-                            stringPairMap[*key].second = stringValue;
-                        }
-                        else {
-                            stringPairMap[*key].second += QLatin1Char(' ') + stringValue;
-                        }
-                        stringListPairMap[*key].first = QDir::currentPath();
-                        stringListPairMap[*key].second += stringListValue;
-                    }
-                    else {
-                        locMap[*key] = keyLoc;
-                        stringPairMap[*key].first = QDir::currentPath();
-                        stringPairMap[*key].second = stringValue;
-                        stringListPairMap[*key].first = QDir::currentPath();
-                        stringListPairMap[*key].second = stringListValue;
-                    }
+                    ConfigVarMultimap::Iterator i;
+                    i = configVars_.insert(*key, ConfigVar(*key, rhsValues, QDir::currentPath(), keyLoc));
+                    i.value().plus_ = (plus ? true : false);
                     ++key;
                 }
             }
-        }
-        else {
+        } else {
             location.fatal(tr("Unexpected character '%1' at beginning of line").arg(c));
         }
     }
     popWorkingDir();
     if (!workingDirs_.isEmpty())
-        QDir::setCurrent(QFileInfo(workingDirs_.top()).path());
+        QDir::setCurrent(workingDirs_.top());
 }
 
 QStringList Config::getFilesHere(const QString& uncleanDir,
@@ -1068,8 +1189,7 @@ QStringList Config::getFilesHere(const QString& uncleanDir,
                                  const QSet<QString> &excludedDirs,
                                  const QSet<QString> &excludedFiles)
 {
-    //
-    QString dir = location.isEmpty() ? QDir::cleanPath(uncleanDir) : location.canonicalRelativePath(uncleanDir);
+    QString dir = location.isEmpty() ? QDir::cleanPath(uncleanDir) : QDir(uncleanDir).canonicalPath();
     QStringList result;
     if (excludedDirs.contains(dir))
         return result;
@@ -1087,9 +1207,8 @@ QStringList Config::getFilesHere(const QString& uncleanDir,
         if (!fn->startsWith(QLatin1Char('~'))) {
             QString s = dirInfo.filePath(*fn);
             QString c = QDir::cleanPath(s);
-            if (!excludedFiles.contains(c)) {
+            if (!excludedFiles.contains(c))
                 result.append(c);
-            }
         }
         ++fn;
     }
@@ -1119,9 +1238,9 @@ void Config::pushWorkingDir(const QString& dir)
  */
 QString Config::popWorkingDir()
 {
-    if (!workingDirs_.isEmpty()) {
+    if (!workingDirs_.isEmpty())
         return workingDirs_.pop();
-    }
+
     qDebug() << "RETURNED EMPTY WORKING DIR";
     return QString();
 }
