@@ -71,7 +71,10 @@ struct Topic
 {
     QString topic;
     QString args;
+    Topic() { }
     Topic(QString& t, QString a) : topic(t), args(a) { }
+    bool isEmpty() const { return topic.isEmpty(); }
+    void clear() { topic.clear(); args.clear(); }
 };
 typedef QList<Topic> TopicList;
 
@@ -136,10 +139,6 @@ public:
     };
 
     Doc() : priv(0) {}
-    Doc(const Location &start_loc,
-        const Location &end_loc,
-        const QString &source,
-        const QSet<QString> &metaCommandSet);
     Doc(const Location& start_loc,
         const Location& end_loc,
         const QString& source,
@@ -165,7 +164,6 @@ public:
     Text briefText(bool inclusive = false) const;
     Text trimmedBriefText(const QString &className) const;
     Text legaleseText() const;
-    const QString& baseName() const;
     Sections granularity() const;
     const QSet<QString> &parameterNames() const;
     const QStringList &enumItemNames() const;
@@ -191,11 +189,14 @@ public:
                                      Quoter &quoter,
                                      const QString &fileName);
     static QString canonicalTitle(const QString &title);
+    static const Config* config() { return config_; }
 
 private:
     void detach();
     DocPrivate *priv;
+    static const Config* config_;
 };
+typedef QList<Doc> DocList;
 
 QT_END_NAMESPACE
 
