@@ -40,7 +40,6 @@
 ****************************************************************************/
 
 #include <qglobal.h>
-#include <qlibraryinfo.h>
 #include <stdlib.h>
 #include "codemarker.h"
 #include "codeparser.h"
@@ -65,29 +64,10 @@
 #  include "qcoreapplication.h"
 #endif
 
+#include <algorithm>
+
 QT_BEGIN_NAMESPACE
 
-/*
-  The default indent for code is 4.
-  The default value for false is 0.
-  The default supported file extensions are cpp, h, qdoc and qml.
-  The default language is c++.
-  The default output format is html.
-  The default tab size is 8.
-  And those are all the default values for configuration variables.
- */
-static const struct {
-    const char *key;
-    const char *value;
-} defaults[] = {
-    { CONFIG_CODEINDENT, "4" },
-    { CONFIG_FALSEHOODS, "0" },
-    { CONFIG_FILEEXTENSIONS, "*.cpp *.h *.qdoc *.qml"},
-    { CONFIG_LANGUAGE, "Cpp" },
-    { CONFIG_OUTPUTFORMATS, "HTML" },
-    { CONFIG_TABSIZE, "8" },
-    { 0, 0 }
-};
 
 bool creationTimeBefore(const QFileInfo &fi1, const QFileInfo &fi2)
 {
@@ -96,6 +76,7 @@ bool creationTimeBefore(const QFileInfo &fi1, const QFileInfo &fi2)
 
 static bool highlighting = false;
 static bool showInternal = false;
+static bool redirectDocumentationToDevNull = false;
 static bool noLinkErrors = false;
 static bool obsoleteLinks = false;
 static QStringList defines;
@@ -140,6 +121,8 @@ static void printHelp()
                              "Run qdoc to read the index files and generate the docs\n"
                              "    -showinternal  "
                              "Include content marked internal\n"
+                             "    -redirect-documentation-to-dev-null "
+                             "Save all documentation content to /dev/null. Useful if someone is interested in qdoc errors only.\n"
                              "    -version       "
                              "Display version of qdoc and exit\n") );
 }
@@ -163,6 +146,15 @@ static void loadIndexFiles(Config& config)
 
     dependModules += config.getStringList(CONFIG_DEPENDS);
 
+    bool noOutputSubdirs = false;
+    QString singleOutputSubdir;
+    if (config.getBool(QString("HTML.nosubdirs"))) {
+        noOutputSubdirs = true;
+        singleOutputSubdir = config.getString("HTML.outputsubdir");
+        if (singleOutputSubdir.isEmpty())
+            singleOutputSubdir = "html";
+    }
+
     // Allow modules and third-party application/libraries to link
     // to the Qt docs without having to explicitly pass --indexdir.
     if (!indexDirs.contains(documentationPath))
@@ -197,15 +189,19 @@ static void loadIndexFiles(Config& config)
                 QString indexToAdd;
                 QList<QFileInfo> foundIndices;
                 for (int j = 0; j < indexDirs.size(); j++) {
-                    QString fileToLookFor = indexDirs[j] + QLatin1Char('/') + dependModules[i] +
-                            QLatin1Char('/') + dependModules[i] + QLatin1String(".index");
+                    QString fileToLookFor = indexDirs[j] + QLatin1Char('/');
+                    if (noOutputSubdirs)
+                        fileToLookFor += singleOutputSubdir + QLatin1Char('/');
+                    else
+                        fileToLookFor += dependModules[i] + QLatin1Char('/');
+                    fileToLookFor += dependModules[i] + QLatin1String(".index");
                     if (QFile::exists(fileToLookFor)) {
                         QFileInfo tempFileInfo(fileToLookFor);
                         if (!foundIndices.contains(tempFileInfo))
                             foundIndices.append(tempFileInfo);
                     }
                 }
-                qSort(foundIndices.begin(), foundIndices.end(), creationTimeBefore);
+                std::sort(foundIndices.begin(), foundIndices.end(), creationTimeBefore);
                 if (foundIndices.size() > 1) {
                     /*
                         QDoc should always use the last entry in the multimap when there are
@@ -222,8 +218,15 @@ static void loadIndexFiles(Config& config)
                 else if (foundIndices.size() == 1) {
                     indexToAdd = foundIndices[0].absoluteFilePath();
                 }
-                if (!indexToAdd.isEmpty() && !indexFiles.contains(indexToAdd))
-                    indexFiles << indexToAdd;
+                if (!indexToAdd.isEmpty()) {
+                    if (!indexFiles.contains(indexToAdd))
+                        indexFiles << indexToAdd;
+                }
+                else if (Generator::runGenerateOnly()) {
+                    qDebug() << "warning:" << config.getString(CONFIG_PROJECT)
+                             << "Cannot locate index file for dependency"
+                             << dependModules[i];
+                }
             }
         }
         else {
@@ -251,27 +254,36 @@ static void processQdocconfFile(const QString &fileName)
       initialize the configuration with some default values.
      */
     Config config(QCoreApplication::translate("QDoc", "qdoc"));
-    int i = 0;
-    while (defaults[i].key) {
-        config.setStringList(defaults[i].key, QStringList() << defaults[i].value);
-        ++i;
-    }
-    config.setStringList(CONFIG_SYNTAXHIGHLIGHTING, QStringList(highlighting ? "true" : "false"));
-    config.setStringList(CONFIG_SHOWINTERNAL, QStringList(showInternal ? "true" : "false"));
-    config.setStringList(CONFIG_NOLINKERRORS, QStringList(noLinkErrors ? "true" : "false"));
-    config.setStringList(CONFIG_OBSOLETELINKS, QStringList(obsoleteLinks ? "true" : "false"));
 
     /*
-      If QT_INSTALL_DOCS is not set, set it here so it can be used from
-      the qdocconf files.
-    */
-    QString qt_install_docs = qgetenv("QT_INSTALL_DOCS");
-    if (qt_install_docs.isEmpty()) {
-        documentationPath = QLibraryInfo::rawLocation(QLibraryInfo::DocumentationPath,
-                                                      QLibraryInfo::EffectivePaths);
-        qputenv("QT_INSTALL_DOCS", documentationPath.toLatin1());
+      The default indent for code is 4.
+      The default value for false is 0.
+      The default supported file extensions are cpp, h, qdoc and qml.
+      The default language is c++.
+      The default output format is html.
+      The default tab size is 8.
+      And those are all the default values for configuration variables.
+     */
+    static QHash<QString,QString> defaults;
+    if (defaults.isEmpty()) {
+        defaults.insert(CONFIG_CODEINDENT, QLatin1String("4"));
+        defaults.insert(CONFIG_FALSEHOODS, QLatin1String("0"));
+        defaults.insert(CONFIG_FILEEXTENSIONS, QLatin1String("*.cpp *.h *.qdoc *.qml"));
+        defaults.insert(CONFIG_LANGUAGE, QLatin1String("Cpp"));
+        defaults.insert(CONFIG_OUTPUTFORMATS, QLatin1String("HTML"));
+        defaults.insert(CONFIG_TABSIZE, QLatin1String("8"));
     }
 
+    QHash<QString,QString>::iterator iter;
+    for (iter = defaults.begin(); iter != defaults.end(); ++iter)
+        config.setStringList(iter.key(), QStringList() << iter.value());
+
+    config.setStringList(CONFIG_SYNTAXHIGHLIGHTING, QStringList(highlighting ? "true" : "false"));
+    config.setStringList(CONFIG_SHOWINTERNAL, QStringList(showInternal ? "true" : "false"));
+    config.setStringList(CONFIG_REDIRECTDOCUMENTATIONTODEVNULL, QStringList(redirectDocumentationToDevNull ? "true" : "false"));
+    config.setStringList(CONFIG_NOLINKERRORS, QStringList(noLinkErrors ? "true" : "false"));
+    config.setStringList(CONFIG_OBSOLETELINKS, QStringList(obsoleteLinks ? "true" : "false"));
+
     /*
       With the default configuration values in place, load
       the qdoc configuration file. Note that the configuration
@@ -321,7 +333,7 @@ static void processQdocconfFile(const QString &fileName)
       Load the language translators, if the configuration specifies any.
      */
     QStringList fileNames = config.getStringList(CONFIG_TRANSLATORS);
-    QStringList::Iterator fn = fileNames.constBegin();
+    QStringList::ConstIterator fn = fileNames.constBegin();
     while (fn != fileNames.constEnd()) {
         QTranslator *translator = new QTranslator(0);
         if (!translator->load(*fn))
@@ -350,7 +362,7 @@ static void processQdocconfFile(const QString &fileName)
      */
     QDocDatabase* qdb = QDocDatabase::qdocDB();
     qdb->setVersion(config.getString(CONFIG_VERSION));
-
+    qdb->setShowInternal(config.getBool(CONFIG_SHOWINTERNAL));
     /*
       By default, the only output format is HTML.
      */
@@ -368,10 +380,12 @@ static void processQdocconfFile(const QString &fileName)
     QStringList excludedFilesList;
 
     Generator::debugSegfault("Reading excludedirs");
-    excludedDirsList = config.getCanonicalRelativePathList(CONFIG_EXCLUDEDIRS);
+    excludedDirsList = config.getCanonicalPathList(CONFIG_EXCLUDEDIRS);
     foreach (const QString &excludeDir, excludedDirsList) {
         QString p = QDir::fromNativeSeparators(excludeDir);
-        excludedDirs.insert(p);
+        QDir tmp(p);
+        if (tmp.exists())
+            excludedDirs.insert(p);
     }
 
     Generator::debugSegfault("Reading excludefiles");
@@ -386,6 +400,10 @@ static void processQdocconfFile(const QString &fileName)
     QMap<QString,QString> headers;
     QMultiMap<QString,QString> headerFileNames;
     for (int i=0; i<headerList.size(); ++i) {
+        if (headerList[i].contains(QString("doc/snippets")))
+            continue;
+        if (headers.contains(headerList[i]))
+            continue;
         headers.insert(headerList[i],headerList[i]);
         QString t = headerList[i].mid(headerList[i].lastIndexOf('/')+1);
         headerFileNames.insert(t,t);
@@ -396,6 +414,10 @@ static void processQdocconfFile(const QString &fileName)
     QMap<QString,QString> sources;
     QMultiMap<QString,QString> sourceFileNames;
     for (int i=0; i<sourceList.size(); ++i) {
+        if (sourceList[i].contains(QString("doc/snippets")))
+            continue;
+        if (sources.contains(sourceList[i]))
+            continue;
         sources.insert(sourceList[i],sourceList[i]);
         QString t = sourceList[i].mid(sourceList[i].lastIndexOf('/')+1);
         sourceFileNames.insert(t,t);
@@ -583,6 +605,9 @@ int main(int argc, char **argv)
         else if (opt == "-showinternal") {
             showInternal = true;
         }
+        else if (opt == "-redirect-documentation-to-dev-null") {
+            redirectDocumentationToDevNull = true;
+        }
         else if (opt == "-no-examples") {
             Config::generateExamples = false;
         }
