@@ -96,15 +96,15 @@ class QDocDatabase
     DocNode* getGroup(const QString& name);
     DocNode* findGroup(const QString& name);
     DocNode* findModule(const QString& name);
-    DocNode* findQmlModule(const QString& name);
+    QmlModuleNode* findQmlModule(const QString& name);
 
     DocNode* addGroup(const QString& name);
     DocNode* addModule(const QString& name);
-    DocNode* addQmlModule(const QString& name);
+    QmlModuleNode* addQmlModule(const QString& name);
 
     DocNode* addToGroup(const QString& name, Node* node);
     DocNode* addToModule(const QString& name, Node* node);
-    DocNode* addToQmlModule(const QString& name, Node* node);
+    void addToQmlModule(const QString& name, Node* node);
 
     QmlClassNode* findQmlType(const QString& qmid, const QString& name) const;
     QmlClassNode* findQmlType(const ImportRec& import, const QString& name) const;
@@ -113,6 +113,7 @@ class QDocDatabase
     void findAllFunctions(const InnerNode *node);
     void findAllLegaleseTexts(const InnerNode *node);
     void findAllNamespaces(const InnerNode *node);
+    void findAllObsoleteThings(const InnerNode* node);
     void findAllSince(const InnerNode *node);
     void buildCollections();
 
@@ -121,6 +122,9 @@ class QDocDatabase
     NodeMap& getMainClasses() { return mainClasses_; }
     NodeMap& getCompatibilityClasses() { return compatClasses_; }
     NodeMap& getObsoleteClasses() { return obsoleteClasses_; }
+    NodeMap& getClassesWithObsoleteMembers() { return classesWithObsoleteMembers_; }
+    NodeMap& getObsoleteQmlTypes() { return obsoleteQmlTypes_; }
+    NodeMap& getQmlTypesWithObsoleteMembers() { return qmlTypesWithObsoleteMembers_; }
     NodeMap& getNamespaces() { return namespaceIndex_; }
     NodeMap& getServiceClasses() { return serviceClasses_; }
     NodeMap& getQmlTypes() { return qmlClasses_; }
@@ -192,6 +196,12 @@ class QDocDatabase
                        Generator* g,
                        bool generateInternalNodes = false);
 
+    void clearOpenNamespaces() { openNamespaces_.clear(); }
+    void insertOpenNamespace(const QString& path) { openNamespaces_.insert(path); }
+    FunctionNode* findNodeInOpenNamespace(const QStringList& parentPath, const FunctionNode* clone);
+    Node* findNodeInOpenNamespace(QStringList& path, Node::Type type, Node::SubType subtype);
+    void setShowInternal(bool value) { showInternal_ = value; }
+
     /* debugging functions */
     void printModules() const;
     void printQmlModules() const;
@@ -211,6 +221,7 @@ class QDocDatabase
 
  private:
     static QDocDatabase*    qdocDB_;
+    bool                    showInternal_;
     QString                 version_;
     QDocMultiMap            masterMap_;
     Tree*                   tree_;
@@ -223,6 +234,9 @@ class QDocDatabase
     NodeMap                 mainClasses_;
     NodeMap                 compatClasses_;
     NodeMap                 obsoleteClasses_;
+    NodeMap                 classesWithObsoleteMembers_;
+    NodeMap                 obsoleteQmlTypes_;
+    NodeMap                 qmlTypesWithObsoleteMembers_;
     NodeMap                 namespaceIndex_;
     NodeMap                 serviceClasses_;
     NodeMap                 qmlClasses_;
@@ -233,6 +247,7 @@ class QDocDatabase
     TextToNodeMap           legaleseTexts_;
     DocNodeMultiMap         docNodesByTitle_;
     TargetRecMultiMap       targetRecMultiMap_;
+    QSet<QString>           openNamespaces_;
 };
 
 QT_END_NAMESPACE
