@@ -71,7 +71,8 @@ public:
 
     // Ui
     virtual bool visit(UiProgram *) { return true; }
-    virtual bool visit(UiImportList *) { return true; }
+    virtual bool visit(UiHeaderItemList *) { return true; }
+    virtual bool visit(UiPragma *) { return true; }
     virtual bool visit(UiImport *) { return true; }
     virtual bool visit(UiPublicMember *) { return true; }
     virtual bool visit(UiSourceElement *) { return true; }
@@ -80,13 +81,16 @@ public:
     virtual bool visit(UiObjectBinding *) { return true; }
     virtual bool visit(UiScriptBinding *) { return true; }
     virtual bool visit(UiArrayBinding *) { return true; }
+    virtual bool visit(UiParameterList *) { return true; }
     virtual bool visit(UiObjectMemberList *) { return true; }
     virtual bool visit(UiArrayMemberList *) { return true; }
     virtual bool visit(UiQualifiedId *) { return true; }
+    virtual bool visit(UiQualifiedPragmaId *) { return true; }
 
     virtual void endVisit(UiProgram *) {}
-    virtual void endVisit(UiImportList *) {}
     virtual void endVisit(UiImport *) {}
+    virtual void endVisit(UiHeaderItemList *) {}
+    virtual void endVisit(UiPragma *) {}
     virtual void endVisit(UiPublicMember *) {}
     virtual void endVisit(UiSourceElement *) {}
     virtual void endVisit(UiObjectDefinition *) {}
@@ -94,9 +98,11 @@ public:
     virtual void endVisit(UiObjectBinding *) {}
     virtual void endVisit(UiScriptBinding *) {}
     virtual void endVisit(UiArrayBinding *) {}
+    virtual void endVisit(UiParameterList *) {}
     virtual void endVisit(UiObjectMemberList *) {}
     virtual void endVisit(UiArrayMemberList *) {}
     virtual void endVisit(UiQualifiedId *) {}
+    virtual void endVisit(UiQualifiedPragmaId *) {}
 
     // QQmlJS
     virtual bool visit(ThisExpression *) { return true; }
@@ -135,8 +141,14 @@ public:
     virtual bool visit(Elision *) { return true; }
     virtual void endVisit(Elision *) {}
 
-    virtual bool visit(PropertyNameAndValueList *) { return true; }
-    virtual void endVisit(PropertyNameAndValueList *) {}
+    virtual bool visit(PropertyAssignmentList *) { return true; }
+    virtual void endVisit(PropertyAssignmentList *) {}
+
+    virtual bool visit(PropertyNameAndValue *) { return true; }
+    virtual void endVisit(PropertyNameAndValue *) {}
+
+    virtual bool visit(PropertyGetterSetter *) { return true; }
+    virtual void endVisit(PropertyGetterSetter *) {}
 
     virtual bool visit(NestedExpression *) { return true; }
     virtual void endVisit(NestedExpression *) {}
