@@ -169,9 +169,9 @@ const Node* Tree::findNode(const QStringList& path,
         if (node && i == path.size()
                 && (!(findFlags & NonFunction) || node->type() != Node::Function
                     || ((FunctionNode*)node)->metaness() == FunctionNode::MacroWithoutParams)) {
-            if ((node != self) && (node->subType() != Node::QmlPropertyGroup)) {
+            if ((node != self) && (node->type() != Node::QmlPropertyGroup)) {
                 if (node->subType() == Node::Collision) {
-                    node = node->applyModuleIdentifier(start);
+                    node = node->applyModuleName(start);
                 }
                 return node;
             }
@@ -367,7 +367,11 @@ const FunctionNode* Tree::findFunctionNode(const QStringList& parentPath,
                                            const Node* relative,
                                            int findFlags) const
 {
-    const Node* parent = findNode(parentPath, relative, findFlags);
+    const Node* parent = findNamespaceNode(parentPath);
+    if (parent == 0)
+        parent = findClassNode(parentPath, 0);
+    if (parent == 0)
+        parent = findNode(parentPath, relative, findFlags);
     if (parent == 0 || !parent->isInnerNode())
         return 0;
     return ((InnerNode*)parent)->findFunctionNode(clone);
@@ -658,7 +662,7 @@ Node* Tree::findNodeRecursive(const QStringList& path,
                               Node* start,
                               Node::Type type,
                               Node::SubType subtype,
-                              bool acceptCollision)
+                              bool acceptCollision) const
 {
     if (!start || path.isEmpty())
         return 0; // no place to start, or nothing to search for.
@@ -736,7 +740,7 @@ EnumNode* Tree::findEnumNode(const QStringList& path, Node* start)
   at the root of the tree. Only a C++ class node named \a path is
   acceptible. If one is not found, 0 is returned.
  */
-ClassNode* Tree::findClassNode(const QStringList& path, Node* start)
+ClassNode* Tree::findClassNode(const QStringList& path, Node* start) const
 {
     if (!start)
         start = const_cast<NamespaceNode*>(root());
@@ -748,23 +752,10 @@ ClassNode* Tree::findClassNode(const QStringList& path, Node* start)
   the root of the tree. Only a Namespace node named \a path
   is acceptible. If one is not found, 0 is returned.
  */
-NamespaceNode* Tree::findNamespaceNode(const QStringList& path)
+NamespaceNode* Tree::findNamespaceNode(const QStringList& path) const
 {
     Node* start = const_cast<NamespaceNode*>(root());
     return static_cast<NamespaceNode*>(findNodeRecursive(path, 0, start, Node::Namespace, Node::NoSubType));
 }
 
-/*!
-  Find the Qml module node named \a path. Begin the search at the
-  \a start node. If the \a start node is 0, begin the search
-  at the root of the tree. Only a Qml module node named \a path is
-  acceptible. If one is not found, 0 is returned.
- */
-DocNode* Tree::findQmlModuleNode(const QStringList& path, Node* start)
-{
-    if (!start)
-        start = const_cast<NamespaceNode*>(root());
-    return static_cast<DocNode*>(findNodeRecursive(path, 0, start, Node::Document, Node::QmlModule));
-}
-
 QT_END_NAMESPACE
