@@ -82,10 +82,9 @@ class Tree
     ~Tree();
 
     EnumNode* findEnumNode(const QStringList& path, Node* start = 0);
-    ClassNode* findClassNode(const QStringList& path, Node* start = 0);
+    ClassNode* findClassNode(const QStringList& path, Node* start = 0) const;
     QmlClassNode* findQmlTypeNode(const QStringList& path);
-    NamespaceNode* findNamespaceNode(const QStringList& path);
-    DocNode* findQmlModuleNode(const QStringList& path, Node* start = 0);
+    NamespaceNode* findNamespaceNode(const QStringList& path) const;
 
     Node* findNodeByNameAndType(const QStringList& path,
                                 Node::Type type,
@@ -98,7 +97,7 @@ class Tree
                             Node* start,
                             Node::Type type,
                             Node::SubType subtype,
-                            bool acceptCollision = false);
+                            bool acceptCollision = false) const;
 
     const Node* findNode(const QStringList &path,
                          const Node* relative = 0,
@@ -128,7 +127,6 @@ class Tree
     void addPropertyFunction(PropertyNode *property,
                              const QString &funcName,
                              PropertyNode::FunctionRole funcRole);
-    void addToQmlModule(Node* node);
     void resolveInheritance(NamespaceNode *rootNode = 0);
     void resolveProperties();
     void resolveCppToQmlLinks();
