@@ -556,24 +556,6 @@ QT_CLASS_LIB(QSqlRelationalDelegate, QtSql, qsqlrelationaldelegate.h)
 QT_CLASS_LIB(QSqlRelation, QtSql, qsqlrelationaltablemodel.h)
 QT_CLASS_LIB(QSqlRelationalTableModel, QtSql, qsqlrelationaltablemodel.h)
 QT_CLASS_LIB(QSqlTableModel, QtSql, qsqltablemodel.h)
-QT_CLASS_LIB(QDB2Result, QtSql, qsql_db2.h)
-QT_CLASS_LIB(QDB2Driver, QtSql, qsql_db2.h)
-QT_CLASS_LIB(QIBaseResult, QtSql, qsql_ibase.h)
-QT_CLASS_LIB(QIBaseDriver, QtSql, qsql_ibase.h)
-QT_CLASS_LIB(QMYSQLResult, QtSql, qsql_mysql.h)
-QT_CLASS_LIB(QMYSQLDriver, QtSql, qsql_mysql.h)
-QT_CLASS_LIB(QOCIResult, QtSql, qsql_oci.h)
-QT_CLASS_LIB(QOCIDriver, QtSql, qsql_oci.h)
-QT_CLASS_LIB(QODBCResult, QtSql, qsql_odbc.h)
-QT_CLASS_LIB(QODBCDriver, QtSql, qsql_odbc.h)
-QT_CLASS_LIB(QPSQLResult, QtSql, qsql_psql.h)
-QT_CLASS_LIB(QPSQLDriver, QtSql, qsql_psql.h)
-QT_CLASS_LIB(QSQLiteResult, QtSql, qsql_sqlite.h)
-QT_CLASS_LIB(QSQLiteDriver, QtSql, qsql_sqlite.h)
-QT_CLASS_LIB(QSQLite2Result, QtSql, qsql_sqlite2.h)
-QT_CLASS_LIB(QSQLite2Driver, QtSql, qsql_sqlite2.h)
-QT_CLASS_LIB(QTDSResult, QtSql, qsql_tds.h)
-QT_CLASS_LIB(QTDSDriver, QtSql, qsql_tds.h)
 QT_CLASS_LIB(QAccessible, QtWidgets, qaccessible.h)
 QT_CLASS_LIB(QAccessibleInterface, QtWidgets, qaccessible.h)
 QT_CLASS_LIB(QAccessibleInterfaceEx, QtWidgets, qaccessible.h)
@@ -605,8 +587,8 @@ QT_CLASS_LIB(QFontDialog, QtWidgets, qfontdialog.h)
 QT_CLASS_LIB(QInputDialog, QtWidgets, qinputdialog.h)
 QT_CLASS_LIB(QMessageBox, QtWidgets, qmessagebox.h)
 QT_CLASS_LIB(QPageSetupDialog, QtWidgets, qpagesetupdialog.h)
-QT_CLASS_LIB(QPrintDialog, QtWidgets, qprintdialog.h)
-QT_CLASS_LIB(QPrintPreviewDialog, QtWidgets, qprintpreviewdialog.h)
+QT_CLASS_LIB(QPrintDialog, QtPrintSupport, qprintdialog.h)
+QT_CLASS_LIB(QPrintPreviewDialog, QtPrintSupport, qprintpreviewdialog.h)
 QT_CLASS_LIB(QProgressDialog, QtWidgets, qprogressdialog.h)
 QT_CLASS_LIB(QWizard, QtWidgets, qwizard.h)
 QT_CLASS_LIB(QWizardPage, QtWidgets, qwizard.h)
@@ -949,6 +931,7 @@ QT_CLASS_LIB(QFocusFrame, QtWidgets, qfocusframe.h)
 QT_CLASS_LIB(QFontComboBox, QtWidgets, qfontcombobox.h)
 QT_CLASS_LIB(QFrame, QtWidgets, qframe.h)
 QT_CLASS_LIB(QGroupBox, QtWidgets, qgroupbox.h)
+QT_CLASS_LIB(QKeySequenceEdit, QtWidgets, qkeysequenceedit.h)
 QT_CLASS_LIB(QLabel, QtWidgets, qlabel.h)
 QT_CLASS_LIB(QLCDNumber, QtWidgets, qlcdnumber.h)
 QT_CLASS_LIB(QLineEdit, QtWidgets, qlineedit.h)
@@ -962,7 +945,7 @@ QT_CLASS_LIB(QMenuBar, QtWidgets, qmenubar.h)
 QT_CLASS_LIB(QMenuItem, QtWidgets, qmenudata.h)
 QT_CLASS_LIB(QPlainTextEdit, QtWidgets, qplaintextedit.h)
 QT_CLASS_LIB(QPlainTextDocumentLayout, QtWidgets, qplaintextedit.h)
-QT_CLASS_LIB(QPrintPreviewWidget, QtWidgets, qprintpreviewwidget.h)
+QT_CLASS_LIB(QPrintPreviewWidget, QtPrintSupport, qprintpreviewwidget.h)
 QT_CLASS_LIB(QProgressBar, QtWidgets, qprogressbar.h)
 QT_CLASS_LIB(QPushButton, QtWidgets, qpushbutton.h)
 QT_CLASS_LIB(QRadioButton, QtWidgets, qradiobutton.h)
