@@ -3,7 +3,7 @@
 ** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
-** This file is part of the QtGui module of the Qt Toolkit.
+** This file is part of the QtWidgets module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
@@ -63,6 +63,8 @@
 #include "qmimedata.h"
 #include "qspinbox.h"
 #include "qdialogbuttonbox.h"
+#include "qscreen.h"
+#include "qcursor.h"
 
 QT_BEGIN_NAMESPACE
 
@@ -134,6 +136,7 @@ public:
 
 signals:
     void selected(int row, int col);
+    void currentChanged(int row, int col);
 
 protected:
     virtual void paintCell(QPainter *, int row, int col, const QRect&);
@@ -309,6 +312,8 @@ void QWellArray::setCurrent(int row, int col)
 
     updateCell(oldRow, oldCol);
     updateCell(curRow, curCol);
+
+    emit currentChanged(curRow, curCol);
 }
 
 /*
@@ -342,6 +347,7 @@ void QWellArray::setSelected(int row, int col)
 void QWellArray::focusInEvent(QFocusEvent*)
 {
     updateCell(curRow, curCol);
+    emit currentChanged(curRow, curCol);
 }
 
 void QWellArray::setCellBrush(int row, int col, const QBrush &b)
@@ -421,6 +427,30 @@ void QWellArray::keyPressEvent(QKeyEvent* e)
 
 //////////// QWellArray END
 
+// Event filter to be installed on the dialog while in color-picking mode.
+class QColorPickingEventFilter : public QObject {
+public:
+    explicit QColorPickingEventFilter(QColorDialogPrivate *dp, QObject *parent = 0) : QObject(parent), m_dp(dp) {}
+
+    bool eventFilter(QObject *, QEvent *event) Q_DECL_OVERRIDE
+    {
+        switch (event->type()) {
+        case QEvent::MouseMove:
+            return m_dp->handleColorPickingMouseMove(static_cast<QMouseEvent *>(event));
+        case QEvent::MouseButtonRelease:
+            return m_dp->handleColorPickingMouseButtonRelease(static_cast<QMouseEvent *>(event));
+        case QEvent::KeyPress:
+            return m_dp->handleColorPickingKeyPress(static_cast<QKeyEvent *>(event));
+        default:
+            break;
+        }
+        return false;
+    }
+
+private:
+    QColorDialogPrivate *m_dp;
+};
+
 /*!
     Returns the number of custom colors supported by QColorDialog. All
     color dialogs share the same custom colors.
@@ -916,6 +946,8 @@ signals:
 private slots:
     void rgbEd();
     void hsvEd();
+    void htmlEd();
+
 private:
     void showCurrentColor();
     int hue, sat, val;
@@ -927,6 +959,7 @@ private:
     QLabel *lblRed;
     QLabel *lblGreen;
     QLabel *lblBlue;
+    QLabel *lblHtml;
     QColSpinBox *hEd;
     QColSpinBox *sEd;
     QColSpinBox *vEd;
@@ -935,6 +968,7 @@ private:
     QColSpinBox *bEd;
     QColSpinBox *alphaEd;
     QLabel *alphaLab;
+    QLineEdit *htEd;
     QColorShowLabel *lab;
     bool rgbOriginal;
     QColorDialog *colorDialog;
@@ -1226,6 +1260,19 @@ QColorShower::QColorShower(QColorDialog *parent)
 #endif
     alphaEd->hide();
     alphaLab->hide();
+    lblHtml = new QLabel(this);
+    htEd = new QLineEdit(this);
+#ifndef QT_NO_SHORTCUT
+    lblHtml->setBuddy(htEd);
+#endif
+
+    QRegularExpression regExp(QStringLiteral("#([A-Fa-f0-9]{6}|[A-Fa-f0-9]{3})"));
+    QRegularExpressionValidator *validator = new QRegularExpressionValidator(regExp, this);
+    htEd->setValidator(validator);
+
+    lblHtml->setAlignment(Qt::AlignRight|Qt::AlignVCenter);
+    gl->addWidget(lblHtml, 5, 1);
+    gl->addWidget(htEd, 5, 2);
 
     connect(hEd, SIGNAL(valueChanged(int)), this, SLOT(hsvEd()));
     connect(sEd, SIGNAL(valueChanged(int)), this, SLOT(hsvEd()));
@@ -1235,6 +1282,7 @@ QColorShower::QColorShower(QColorDialog *parent)
     connect(gEd, SIGNAL(valueChanged(int)), this, SLOT(rgbEd()));
     connect(bEd, SIGNAL(valueChanged(int)), this, SLOT(rgbEd()));
     connect(alphaEd, SIGNAL(valueChanged(int)), this, SLOT(rgbEd()));
+    connect(htEd, SIGNAL(textEdited(QString)), this, SLOT(htmlEd()));
 
     retranslateStrings();
 }
@@ -1247,7 +1295,7 @@ inline bool QColorDialogPrivate::isAlphaVisible() const { return cs->isAlphaVisi
 
 QColor QColorDialogPrivate::currentQColor() const
 {
-    if (!options->testOption(QColorDialogOptions::DontUseNativeDialog) && nativeDialogInUse)
+    if (nativeDialogInUse)
         return platformColorDialogHelper()->currentColor();
     return cs->currentQColor();
 }
@@ -1269,6 +1317,8 @@ void QColorShower::rgbEd()
     sEd->setValue(sat);
     vEd->setValue(val);
 
+    htEd->setText(QColor(curCol).name());
+
     showCurrentColor();
     emit newCol(currentColor());
     updateQColor();
@@ -1289,6 +1339,31 @@ void QColorShower::hsvEd()
     gEd->setValue(qGreen(currentColor()));
     bEd->setValue(qBlue(currentColor()));
 
+    htEd->setText(c.name());
+
+    showCurrentColor();
+    emit newCol(currentColor());
+    updateQColor();
+}
+
+void QColorShower::htmlEd()
+{
+    QColor c;
+    QString t = htEd->text();
+    c.setNamedColor(t);
+    if (!c.isValid())
+        return;
+    curCol = qRgba(c.red(), c.green(), c.blue(), currentAlpha());
+    rgb2hsv(curCol, hue, sat, val);
+
+    hEd->setValue(hue);
+    sEd->setValue(sat);
+    vEd->setValue(val);
+
+    rEd->setValue(qRed(currentColor()));
+    gEd->setValue(qGreen(currentColor()));
+    bEd->setValue(qBlue(currentColor()));
+
     showCurrentColor();
     emit newCol(currentColor());
     updateQColor();
@@ -1309,6 +1384,8 @@ void QColorShower::setRgb(QRgb rgb)
     gEd->setValue(qGreen(currentColor()));
     bEd->setValue(qBlue(currentColor()));
 
+    htEd->setText(QColor(rgb).name());
+
     showCurrentColor();
     updateQColor();
 }
@@ -1332,6 +1409,8 @@ void QColorShower::setHsv(int h, int s, int v)
     gEd->setValue(qGreen(currentColor()));
     bEd->setValue(qBlue(currentColor()));
 
+    htEd->setText(c.name());
+
     showCurrentColor();
     updateQColor();
 }
@@ -1345,6 +1424,7 @@ void QColorShower::retranslateStrings()
     lblGreen->setText(QColorDialog::tr("&Green:"));
     lblBlue->setText(QColorDialog::tr("Bl&ue:"));
     alphaLab->setText(QColorDialog::tr("A&lpha channel:"));
+    lblHtml->setText(QColorDialog::tr("&HTML:"));
 }
 
 void QColorShower::updateQColor()
@@ -1358,16 +1438,20 @@ void QColorShower::updateQColor()
 //sets all widgets to display h,s,v
 void QColorDialogPrivate::_q_newHsv(int h, int s, int v)
 {
-    cs->setHsv(h, s, v);
-    cp->setCol(h, s);
-    lp->setCol(h, s, v);
+    if (!nativeDialogInUse) {
+        cs->setHsv(h, s, v);
+        cp->setCol(h, s);
+        lp->setCol(h, s, v);
+    }
 }
 
 //sets all widgets to display rgb
 void QColorDialogPrivate::setCurrentColor(QRgb rgb)
 {
-    cs->setRgb(rgb);
-    _q_newColorTypedIn(rgb);
+    if (!nativeDialogInUse) {
+        cs->setRgb(rgb);
+        _q_newColorTypedIn(rgb);
+    }
 }
 
 // hack; doesn't keep curCol in sync, so use with care
@@ -1417,20 +1501,35 @@ bool QColorDialogPrivate::selectColor(const QColor &col)
     return false;
 }
 
+QColor QColorDialogPrivate::grabScreenColor(const QPoint &p)
+{
+    const QDesktopWidget *desktop = QApplication::desktop();
+    const QPixmap pixmap = QGuiApplication::screens().at(desktop->screenNumber())->grabWindow(desktop->winId(),
+                                                                                              p.x(), p.y(), 1, 1);
+    QImage i = pixmap.toImage();
+    return i.pixel(0, 0);
+}
+
 //sets all widgets except cs to display rgb
 void QColorDialogPrivate::_q_newColorTypedIn(QRgb rgb)
 {
-    int h, s, v;
-    rgb2hsv(rgb, h, s, v);
-    cp->setCol(h, s);
-    lp->setCol(h, s, v);
+    if (!nativeDialogInUse) {
+        int h, s, v;
+        rgb2hsv(rgb, h, s, v);
+        cp->setCol(h, s);
+        lp->setCol(h, s, v);
+    }
+}
+
+void QColorDialogPrivate::_q_nextCustom(int r, int c)
+{
+    nextCust = r + 2 * c;
 }
 
 void QColorDialogPrivate::_q_newCustom(int r, int c)
 {
     const int i = r + 2 * c;
     setCurrentColor(QColorDialogOptions::customColor(i));
-    nextCust = i;
     if (standard)
         standard->setSelected(-1,-1);
 }
@@ -1442,6 +1541,54 @@ void QColorDialogPrivate::_q_newStandard(int r, int c)
         custom->setSelected(-1,-1);
 }
 
+void QColorDialogPrivate::_q_pickScreenColor()
+{
+    Q_Q(QColorDialog);
+    if (!colorPickingEventFilter)
+        colorPickingEventFilter = new QColorPickingEventFilter(this);
+    q->installEventFilter(colorPickingEventFilter);
+    // If user pushes Escape, the last color before picking will be restored.
+    beforeScreenColorPicking = cs->currentColor();
+    /*For some reason, q->grabMouse(Qt::CrossCursor) doesn't change
+     * the cursor, and therefore I have to change it manually.
+     */
+    q->grabMouse();
+#ifndef QT_NO_CURSOR
+    q->setCursor(Qt::CrossCursor);
+#endif
+    q->grabKeyboard();
+    /* With setMouseTracking(true) the desired color can be more precisedly picked up,
+     * and continuously pushing the mouse button is not necessary.
+     */
+    q->setMouseTracking(true);
+
+    addCusBt->setDisabled(true);
+    buttons->setDisabled(true);
+    screenColorPickerButton->setDisabled(true);
+
+    q->setCurrentColor(grabScreenColor(QCursor::pos()));
+    lblScreenColorInfo->setText(QColorDialog::tr("Cursor at %1, %2, color: %3\nPress ESC to cancel")
+                                .arg(QCursor::pos().x())
+                                .arg(QCursor::pos().y())
+                                .arg(q->currentColor().name()));
+}
+
+void QColorDialogPrivate::releaseColorPicking()
+{
+    Q_Q(QColorDialog);
+    q->removeEventFilter(colorPickingEventFilter);
+    q->releaseMouse();
+    q->releaseKeyboard();
+#ifndef QT_NO_CURSOR
+    q->setCursor(Qt::ArrowCursor);
+#endif
+    q->setMouseTracking(false);
+    lblScreenColorInfo->setText(QLatin1String("\n"));
+    addCusBt->setDisabled(false);
+    buttons->setDisabled(false);
+    screenColorPickerButton->setDisabled(false);
+}
+
 void QColorDialogPrivate::init(const QColor &initial)
 {
     Q_Q(QColorDialog);
@@ -1449,9 +1596,24 @@ void QColorDialogPrivate::init(const QColor &initial)
     q->setSizeGripEnabled(false);
     q->setWindowTitle(QColorDialog::tr("Select Color"));
 
+    // default: use the native dialog if possible.  Can be overridden in setOptions()
     nativeDialogInUse = (platformColorDialogHelper() != 0);
-
+    colorPickingEventFilter = 0;
     nextCust = 0;
+
+    if (!nativeDialogInUse)
+        initWidgets();
+
+#ifdef Q_WS_MAC
+    delegate = 0;
+#endif
+
+    q->setCurrentColor(initial);
+}
+
+void QColorDialogPrivate::initWidgets()
+{
+    Q_Q(QColorDialog);
     QVBoxLayout *mainLay = new QVBoxLayout(q);
     // there's nothing in this dialog that benefits from sizing up
     mainLay->setSizeConstraint(QLayout::SetFixedSize);
@@ -1486,6 +1648,15 @@ void QColorDialogPrivate::init(const QColor &initial)
         leftLay->addWidget(lblBasicColors);
         leftLay->addWidget(standard);
 
+#if !defined(Q_OS_WINCE) && !defined(QT_SMALL_COLORDIALOG)
+        // The screen color picker button
+        screenColorPickerButton = new QPushButton(QColorDialog::tr("Pick Screen Color"));
+        leftLay->addWidget(screenColorPickerButton);
+        lblScreenColorInfo = new QLabel(QLatin1String("\n"));
+        leftLay->addWidget(lblScreenColorInfo);
+        q->connect(screenColorPickerButton, SIGNAL(clicked()), SLOT(_q_pickScreenColor()));
+#endif
+
 #if !defined(Q_OS_WINCE)
         leftLay->addStretch();
 #endif
@@ -1494,6 +1665,7 @@ void QColorDialogPrivate::init(const QColor &initial)
         custom->setAcceptDrops(true);
 
         q->connect(custom, SIGNAL(selected(int,int)), SLOT(_q_newCustom(int,int)));
+        q->connect(custom, SIGNAL(currentChanged(int,int)), SLOT(_q_nextCustom(int,int)));
         lblCustomColors = new QLabel(q);
 #ifndef QT_NO_SHORTCUT
         lblCustomColors->setBuddy(custom);
@@ -1588,12 +1760,6 @@ void QColorDialogPrivate::init(const QColor &initial)
     QObject::connect(cancel, SIGNAL(clicked()), q, SLOT(reject()));
 
     retranslateStrings();
-
-#ifdef Q_WS_MAC
-    delegate = 0;
-#endif
-
-    q->setCurrentColor(initial);
 }
 
 void QColorDialogPrivate::initHelper(QPlatformDialogHelper *h)
@@ -1658,6 +1824,11 @@ static const Qt::WindowFlags DefaultWindowFlags =
     during the execution of the program. Use setCustomColor() to set
     the custom colors, and use customColor() to get them.
 
+    When pressing the "Pick Screen Color" button, the cursor changes to a haircross
+    and the colors on the screen are scanned. The user can pick up one by clicking
+    the mouse or the Enter button. Pressing Escape restores the last color selected
+    before entering this mode.
+
     The \l{dialogs/standarddialogs}{Standard Dialogs} example shows
     how to use QColorDialog as well as other built-in Qt dialogs.
 
@@ -1699,12 +1870,13 @@ QColorDialog::QColorDialog(const QColor &initial, QWidget *parent)
 void QColorDialog::setCurrentColor(const QColor &color)
 {
     Q_D(QColorDialog);
-    d->setCurrentColor(color.rgb());
-    d->selectColor(color);
-    d->setCurrentAlpha(color.alpha());
-
-    if (!testOption(QColorDialog::DontUseNativeDialog) && d->nativeDialogInUse)
+    if (d->nativeDialogInUse)
         d->platformColorDialogHelper()->setCurrentColor(color);
+    else {
+        d->setCurrentColor(color.rgb());
+        d->selectColor(color);
+        d->setCurrentAlpha(color.alpha());
+    }
 }
 
 QColor QColorDialog::currentColor() const
@@ -1743,7 +1915,7 @@ void QColorDialog::setOption(ColorDialogOption option, bool on)
 /*!
     \since 4.5
 
-    Returns true if the given \a option is enabled; otherwise, returns
+    Returns \c true if the given \a option is enabled; otherwise, returns
     false.
 
     \sa options, setOption()
@@ -1774,8 +1946,14 @@ void QColorDialog::setOptions(ColorDialogOptions options)
         return;
 
     d->options->setOptions(QColorDialogOptions::ColorDialogOptions(int(options)));
-    d->buttons->setVisible(!(options & NoButtons));
-    d->showAlpha(options & ShowAlphaChannel);
+    if ((options & DontUseNativeDialog) && d->nativeDialogInUse) {
+        d->nativeDialogInUse = false;
+        d->initWidgets();
+    }
+    if (!d->nativeDialogInUse) {
+        d->buttons->setVisible(!(options & NoButtons));
+        d->showAlpha(options & ShowAlphaChannel);
+    }
 }
 
 QColorDialog::ColorDialogOptions QColorDialog::options() const
@@ -1794,8 +1972,8 @@ QColorDialog::ColorDialogOptions QColorDialog::options() const
 
     \value ShowAlphaChannel Allow the user to select the alpha component of a color.
     \value NoButtons Don't display \uicontrol{OK} and \uicontrol{Cancel} buttons. (Useful for "live dialogs".)
-    \value DontUseNativeDialog Use Qt's standard color dialog on the Mac instead of Apple's
-                               native color panel.
+    \value DontUseNativeDialog  Use Qt's standard color dialog instead of the operating system
+                                native color dialog.
 
     \sa options, setOption(), testOption(), windowModality()
 */
@@ -1857,15 +2035,12 @@ void QColorDialog::setVisible(bool visible)
     }
 #else
 
-    if (!(options() & DontUseNativeDialog))
-        d->setNativeDialogVisible(visible);
-
     if (d->nativeDialogInUse) {
+        d->setNativeDialogVisible(visible);
         // Set WA_DontShowOnScreen so that QDialog::setVisible(visible) below
         // updates the state correctly, but skips showing the non-native version:
         setAttribute(Qt::WA_DontShowOnScreen);
     } else {
-        d->nativeDialogInUse = false;
         setAttribute(Qt::WA_DontShowOnScreen, false);
     }
 #endif
@@ -1974,6 +2149,43 @@ void QColorDialog::changeEvent(QEvent *e)
     QDialog::changeEvent(e);
 }
 
+bool QColorDialogPrivate::handleColorPickingMouseMove(QMouseEvent *e)
+{
+    Q_Q(QColorDialog);
+    const QPoint globalPos = e->globalPos();
+    const QColor color = grabScreenColor(globalPos);
+    q->setCurrentColor(color);
+    lblScreenColorInfo->setText(QColorDialog::tr("Cursor at %1, %2, color: %3\nPress ESC to cancel")
+                                .arg(globalPos.x()).arg(globalPos.y()).arg(color.name()));
+    return true;
+}
+
+bool QColorDialogPrivate::handleColorPickingMouseButtonRelease(QMouseEvent *e)
+{
+    Q_Q(QColorDialog);
+    q->setCurrentColor(grabScreenColor(e->globalPos()));
+    releaseColorPicking();
+    return true;
+}
+
+bool QColorDialogPrivate::handleColorPickingKeyPress(QKeyEvent *e)
+{
+    Q_Q(QColorDialog);
+    switch (e->key()) {
+    case Qt::Key_Escape:
+        releaseColorPicking();
+        q->setCurrentColor(beforeScreenColorPicking);
+        break;
+    case Qt::Key_Return:
+    case Qt::Key_Enter:
+        q->setCurrentColor(grabScreenColor(QCursor::pos()));
+        releaseColorPicking();
+        break;
+    }
+    e->accept();
+    return true;
+}
+
 /*!
   Closes the dialog and sets its result code to \a result. If this dialog
   is shown with exec(), done() causes the local event loop to finish,
