@@ -3,7 +3,7 @@
 ** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
-** This file is part of the QtGui module of the Qt Toolkit.
+** This file is part of the QtWidgets module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
@@ -52,6 +52,8 @@
 #include "qwhatsthis.h"
 #include "qmenu.h"
 #include "qcursor.h"
+#include "qmessagebox.h"
+#include "qerrormessage.h"
 #include <qpa/qplatformtheme.h>
 #include "private/qdialog_p.h"
 #include "private/qguiapplication_p.h"
@@ -74,6 +76,14 @@ static inline int themeDialogType(const QDialog *dialog)
 #ifndef QT_NO_FONTDIALOG
     if (qobject_cast<const QFontDialog *>(dialog))
         return QPlatformTheme::FontDialog;
+#endif
+#ifndef QT_NO_MESSAGEBOX
+    if (qobject_cast<const QMessageBox *>(dialog))
+        return QPlatformTheme::MessageDialog;
+#endif
+#ifndef QT_NO_ERRORMESSAGE
+    if (qobject_cast<const QErrorMessage *>(dialog))
+        return QPlatformTheme::MessageDialog;
 #endif
     return -1;
 }
@@ -100,6 +110,17 @@ QPlatformDialogHelper *QDialogPrivate::platformHelper() const
     return m_platformHelper;
 }
 
+bool QDialogPrivate::canBeNativeDialog() const
+{
+    QDialogPrivate *ncThis = const_cast<QDialogPrivate *>(this);
+    QDialog *dialog = ncThis->q_func();
+    const int type = themeDialogType(dialog);
+    if (type >= 0)
+        return QGuiApplicationPrivate::platformTheme()
+                ->usePlatformNativeDialog(static_cast<QPlatformTheme::DialogType>(type));
+    return false;
+}
+
 QWindow *QDialogPrivate::parentWindow() const
 {
     if (const QWidget *parent = q_func()->nativeParentWidget())
@@ -150,12 +171,11 @@ void QDialogPrivate::deletePlatformHelper()
     provide a \l{#return}{return value}, and they can have \l{#default}{default buttons}. QDialogs can also have a QSizeGrip in their
     lower-right corner, using setSizeGripEnabled().
 
-    Note that QDialog (an any other widget that has type Qt::Dialog) uses
-    the parent widget slightly differently from other classes in Qt. A
-    dialog is always a top-level widget, but if it has a parent, its
-    default location is centered on top of the parent's top-level widget
-    (if it is not top-level itself). It will also share the parent's
-    taskbar entry.
+    Note that QDialog (and any other widget that has type \c Qt::Dialog) uses
+    the parent widget slightly differently from other classes in Qt. A dialog is
+    always a top-level widget, but if it has a parent, its default location is
+    centered on top of the parent's top-level widget (if it is not top-level
+    itself). It will also share the parent's taskbar entry.
 
     Use the overload of the QWidget::setParent() function to change
     the ownership of a QDialog widget. This function allows you to
@@ -698,6 +718,9 @@ void QDialog::closeEvent(QCloseEvent *e)
 void QDialog::setVisible(bool visible)
 {
     Q_D(QDialog);
+    if (!testAttribute(Qt::WA_DontShowOnScreen) && d->canBeNativeDialog() && d->setNativeDialogVisible(visible))
+        return;
+
     if (visible) {
         if (testAttribute(Qt::WA_WState_ExplicitShowHide) && !testAttribute(Qt::WA_WState_Hidden))
             return;
@@ -768,8 +791,10 @@ void QDialog::setVisible(bool visible)
         if (d->eventLoop)
             d->eventLoop->exit();
     }
+
+    const QPlatformTheme *theme = QGuiApplicationPrivate::platformTheme();
     if (d->mainDef && isActiveWindow()
-        && d->styleHint(QPlatformDialogHelper::SnapToDefaultButton).toBool())
+        && theme->themeHint(QPlatformTheme::DialogSnapToDefaultButton).toBool())
         QCursor::setPos(d->mainDef->mapToGlobal(d->mainDef->rect().center()));
 }
 
@@ -1040,7 +1065,7 @@ QSize QDialog::minimumSizeHint() const
     \property QDialog::modal
     \brief whether show() should pop up the dialog as modal or modeless
 
-    By default, this property is false and show() pops up the dialog
+    By default, this property is \c false and show() pops up the dialog
     as modeless. Setting his property to true is equivalent to setting
     QWidget::windowModality to Qt::ApplicationModal.
 
