@@ -3,7 +3,7 @@
 ** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
-** This file is part of the QtGui module of the Qt Toolkit.
+** This file is part of the QtWidgets module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
@@ -108,7 +108,13 @@ public:
         return QExtendedInformation::System;
     }
 
-    bool isSymLink() const {
+    bool isSymLink(bool ignoreNtfsSymLinks = false) const
+    {
+        if (ignoreNtfsSymLinks) {
+#ifdef Q_OS_WIN
+            return !mFileInfo.suffix().compare(QLatin1String("lnk"), Qt::CaseInsensitive);
+#endif
+        }
         return mFileInfo.isSymLink();
     }
 
