@@ -3,7 +3,7 @@
 ** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
-** This file is part of the QtGui module of the Qt Toolkit.
+** This file is part of the QtWidgets module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
@@ -356,6 +356,9 @@ QWidget *QGraphicsProxyWidgetPrivate::findFocusChild(QWidget *child, bool next)
         }
     }
 
+    if (!child)
+        return 0;
+
     QWidget *oldChild = child;
     uint focus_flag = qt_tab_all_widgets() ? Qt::TabFocus : Qt::StrongFocus;
     do {
@@ -825,7 +828,9 @@ bool QGraphicsProxyWidget::event(QEvent *event)
     }
     case QEvent::InputMethod: {
         inputMethodEvent(static_cast<QInputMethodEvent *>(event));
-        break;
+        if (event->isAccepted())
+            return true;
+        return false;
     }
     case QEvent::ShortcutOverride: {
         QWidget *focusWidget = d->widget->focusWidget();
