@@ -3,7 +3,7 @@
 ** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
-** This file is part of the QtGui module of the Qt Toolkit.
+** This file is part of the QtWidgets module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
@@ -127,6 +127,8 @@ public:
     void doDelayedItemsLayout(int delay = 0);
     void interruptDelayedItemsLayout() const;
 
+    void updateGeometry();
+
     void startAutoScroll()
     {   // ### it would be nice to make this into a style hint one day
         int scrollInterval = (verticalScrollMode == QAbstractItemView::ScrollPerItem) ? 150 : 50;
@@ -345,7 +347,7 @@ public:
 
     QModelIndexList selectedDraggableIndexes() const;
 
-    QStyleOptionViewItem viewOptions() const;
+    QStyleOptionViewItem viewOptionsV1() const;
 
     void doDelayedReset()
     {
