@@ -3,7 +3,7 @@
 ** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
-** This file is part of the QtGui module of the Qt Toolkit.
+** This file is part of the QtWidgets module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
@@ -44,8 +44,6 @@
 
 #include <QtWidgets/qabstractitemview.h>
 
-QT_BEGIN_HEADER
-
 QT_BEGIN_NAMESPACE
 
 
@@ -63,6 +61,7 @@ class Q_WIDGETS_EXPORT QHeaderView : public QAbstractItemView
     Q_PROPERTY(bool cascadingSectionResizes READ cascadingSectionResizes WRITE setCascadingSectionResizes)
     Q_PROPERTY(int defaultSectionSize READ defaultSectionSize WRITE setDefaultSectionSize)
     Q_PROPERTY(int minimumSectionSize READ minimumSectionSize WRITE setMinimumSectionSize)
+    Q_PROPERTY(int maximumSectionSize READ maximumSectionSize WRITE setMaximumSectionSize)
     Q_PROPERTY(Qt::Alignment defaultAlignment READ defaultAlignment WRITE setDefaultAlignment)
     Q_ENUMS(ResizeMode)
 
@@ -86,6 +85,7 @@ public:
     int offset() const;
     int length() const;
     QSize sizeHint() const;
+    void setVisible(bool v);
     int sectionSizeHint(int logicalIndex) const;
 
     int visualIndexAt(int position) const;
@@ -134,6 +134,10 @@ public:
     ResizeMode sectionResizeMode(int logicalIndex) const;
     void setSectionResizeMode(ResizeMode mode);
     void setSectionResizeMode(int logicalIndex, ResizeMode mode);
+
+    void setResizeContentsPrecision(int precision);
+    int  resizeContentsPrecision() const;
+
 #if QT_DEPRECATED_SINCE(5, 0)
     inline QT_DEPRECATED void setResizeMode(ResizeMode mode)
         { setSectionResizeMode(mode); }
@@ -163,6 +167,8 @@ public:
 
     int minimumSectionSize() const;
     void setMinimumSectionSize(int size);
+    int maximumSectionSize() const;
+    void setMaximumSectionSize(int size);
 
     Qt::Alignment defaultAlignment() const;
     void setDefaultAlignment(Qt::Alignment alignment);
@@ -263,6 +269,4 @@ inline void QHeaderView::showSection(int alogicalIndex)
 
 QT_END_NAMESPACE
 
-QT_END_HEADER
-
 #endif // QHEADERVIEW_H
