@@ -3,7 +3,7 @@
 ** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
-** This file is part of the QtGui module of the Qt Toolkit.
+** This file is part of the QtWidgets module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
@@ -113,6 +113,7 @@ public:
 
     static QString valueToText(const QVariant &value, const QStyleOptionViewItem &option);
 
+    bool tryFixup(QWidget *editor);
     void _q_commitDataAndCloseEditor(QWidget *editor);
 
     QItemEditorFactory *f;
@@ -386,6 +387,24 @@ QString QItemDelegatePrivate::valueToText(const QVariant &value, const QStyleOpt
     return text;
 }
 
+bool QItemDelegatePrivate::tryFixup(QWidget *editor)
+{
+#ifndef QT_NO_LINEEDIT
+    if (QLineEdit *e = qobject_cast<QLineEdit*>(editor)) {
+        if (!e->hasAcceptableInput()) {
+            if (const QValidator *validator = e->validator()) {
+                QString text = e->text();
+                validator->fixup(text);
+                e->setText(text);
+            }
+            return e->hasAcceptableInput();
+        }
+    }
+#endif // QT_NO_LINEEDIT
+
+    return true;
+}
+
 /*!
     Renders the delegate using the given \a painter and style \a option for
     the item specified by \a index.
@@ -523,7 +542,10 @@ QWidget *QItemDelegate::createEditor(QWidget *parent,
     const QItemEditorFactory *factory = d->f;
     if (factory == 0)
         factory = QItemEditorFactory::defaultFactory();
-    return factory->createEditor(index.data(Qt::EditRole).userType(), parent);
+    QWidget *w = factory->createEditor(index.data(Qt::EditRole).userType(), parent);
+    if (w)
+        w->setFocusPolicy(Qt::WheelFocus);
+    return w;
 }
 
 /*!
@@ -1124,8 +1146,8 @@ QRect QItemDelegate::textRectangle(QPainter * /*painter*/, const QRect &rect,
 /*!
     \fn bool QItemDelegate::eventFilter(QObject *editor, QEvent *event)
 
-    Returns true if the given \a editor is a valid QWidget and the
-    given \a event is handled; otherwise returns false. The following
+    Returns \c true if the given \a editor is a valid QWidget and the
+    given \a event is handled; otherwise returns \c false. The following
     key press events are handled by default:
 
     \list
@@ -1151,18 +1173,24 @@ QRect QItemDelegate::textRectangle(QPainter * /*painter*/, const QRect &rect,
 
 bool QItemDelegate::eventFilter(QObject *object, QEvent *event)
 {
+    Q_D(QItemDelegate);
+
     QWidget *editor = qobject_cast<QWidget*>(object);
     if (!editor)
         return false;
     if (event->type() == QEvent::KeyPress) {
         switch (static_cast<QKeyEvent *>(event)->key()) {
         case Qt::Key_Tab:
-            emit commitData(editor);
-            emit closeEditor(editor, QAbstractItemDelegate::EditNextItem);
+            if (d->tryFixup(editor)) {
+                emit commitData(editor);
+                emit closeEditor(editor, EditNextItem);
+            }
             return true;
         case Qt::Key_Backtab:
-            emit commitData(editor);
-            emit closeEditor(editor, QAbstractItemDelegate::EditPreviousItem);
+            if (d->tryFixup(editor)) {
+                emit commitData(editor);
+                emit closeEditor(editor, EditPreviousItem);
+            }
             return true;
         case Qt::Key_Enter:
         case Qt::Key_Return:
@@ -1173,11 +1201,9 @@ bool QItemDelegate::eventFilter(QObject *object, QEvent *event)
             // before committing the data (e.g. so it can do
             // validation/fixup of the input).
 #endif // QT_NO_TEXTEDIT
-#ifndef QT_NO_LINEEDIT
-            if (QLineEdit *e = qobject_cast<QLineEdit*>(editor))
-                if (!e->hasAcceptableInput())
-                    return false;
-#endif // QT_NO_LINEEDIT
+            if (!d->tryFixup(editor))
+                return true;
+
             QMetaObject::invokeMethod(this, "_q_commitDataAndCloseEditor",
                                       Qt::QueuedConnection, Q_ARG(QWidget*, editor));
             return false;
@@ -1208,8 +1234,9 @@ bool QItemDelegate::eventFilter(QObject *object, QEvent *event)
                 return false;
             }
 #endif
+            if (d->tryFixup(editor))
+                emit commitData(editor);
 
-            emit commitData(editor);
             emit closeEditor(editor, NoHint);
         }
     } else if (event->type() == QEvent::ShortcutOverride) {
@@ -1268,8 +1295,11 @@ bool QItemDelegate::editorEvent(QEvent *event,
         return false;
     }
 
-    Qt::CheckState state = (static_cast<Qt::CheckState>(value.toInt()) == Qt::Checked
-                            ? Qt::Unchecked : Qt::Checked);
+    Qt::CheckState state = static_cast<Qt::CheckState>(value.toInt());
+    if (flags & Qt::ItemIsTristate)
+        state = ((Qt::CheckState)((state + 1) % 3));
+    else
+        state = (state == Qt::Checked) ? Qt::Unchecked : Qt::Checked;
     return model->setData(index, state, Qt::CheckStateRole);
 }
 
@@ -1299,6 +1329,9 @@ QStyleOptionViewItem QItemDelegate::setOptions(const QModelIndex &index,
     if (value.canConvert<QBrush>())
         opt.palette.setBrush(QPalette::Text, qvariant_cast<QBrush>(value));
 
+    // disable style animations for checkboxes etc. within itemviews (QTBUG-30146)
+    opt.styleObject = 0;
+
     return opt;
 }
 
