@@ -3,7 +3,7 @@
 ** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
-** This file is part of the QtGui module of the Qt Toolkit.
+** This file is part of the QtWidgets module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
@@ -304,7 +304,7 @@ void QListModel::sort(int column, Qt::SortOrder order)
     }
 
     LessThan compare = (order == Qt::AscendingOrder ? &itemLessThan : &itemGreaterThan);
-    qSort(sorting.begin(), sorting.end(), compare);
+    std::sort(sorting.begin(), sorting.end(), compare);
     QModelIndexList fromIndexes;
     QModelIndexList toIndexes;
     for (int r = 0; r < sorting.count(); ++r) {
@@ -338,7 +338,7 @@ void QListModel::ensureSorted(int column, Qt::SortOrder order, int start, int en
     }
 
     LessThan compare = (order == Qt::AscendingOrder ? &itemLessThan : &itemGreaterThan);
-    qSort(sorting.begin(), sorting.end(), compare);
+    std::sort(sorting.begin(), sorting.end(), compare);
 
     QModelIndexList oldPersistentIndexes = persistentIndexList();
     QModelIndexList newPersistentIndexes = oldPersistentIndexes;
@@ -347,7 +347,11 @@ void QListModel::ensureSorted(int column, Qt::SortOrder order, int start, int en
     bool changed = false;
     for (int i = 0; i < count; ++i) {
         int oldRow = sorting.at(i).second;
+        int tmpitepos = lit - tmp.begin();
         QListWidgetItem *item = tmp.takeAt(oldRow);
+        if (tmpitepos > tmp.size())
+            --tmpitepos;
+        lit = tmp.begin() + tmpitepos;
         lit = sortedInsertionIterator(lit, tmp.end(), order, item);
         int newRow = qMax(lit - tmp.begin(), 0);
         lit = tmp.insert(lit, item);
@@ -551,7 +555,7 @@ Qt::DropActions QListModel::supportedDropActions() const
     \fn bool QListWidgetItem::isSelected() const
     \since 4.2
 
-    Returns true if the item is selected; otherwise returns false.
+    Returns \c true if the item is selected; otherwise returns \c false.
 
     \sa setSelected()
 */
@@ -569,7 +573,7 @@ Qt::DropActions QListModel::supportedDropActions() const
     \fn bool QListWidgetItem::isHidden() const
     \since 4.2
 
-    Returns true if the item is hidden; otherwise returns false.
+    Returns \c true if the item is hidden; otherwise returns \c false.
 
     \sa setHidden()
 */
@@ -721,8 +725,8 @@ QVariant QListWidgetItem::data(int role) const
 }
 
 /*!
-    Returns true if this item's text is less then \a other item's text;
-    otherwise returns false.
+    Returns \c true if this item's text is less then \a other item's text;
+    otherwise returns \c false.
 */
 bool QListWidgetItem::operator<(const QListWidgetItem &other) const
 {
@@ -1452,7 +1456,7 @@ QListWidgetItem *QListWidget::currentItem() const
     Sets the current item to \a item.
 
     Unless the selection mode is \l{QAbstractItemView::}{NoSelection},
-    the item is also be selected.
+    the item is also selected.
 */
 void QListWidget::setCurrentItem(QListWidgetItem *item)
 {
@@ -1551,7 +1555,7 @@ void QListWidget::sortItems(Qt::SortOrder order)
     \property QListWidget::sortingEnabled
     \brief whether sorting is enabled
 
-    If this property is true, sorting is enabled for the list; if the property
+    If this property is \c true, sorting is enabled for the list; if the property
     is false, sorting is not enabled.
 
     The default value is false.
@@ -1644,7 +1648,7 @@ void QListWidget::setItemWidget(QListWidgetItem *item, QWidget *widget)
 }
 
 /*!
-    Returns true if \a item is selected; otherwise returns false.
+    Returns \c true if \a item is selected; otherwise returns \c false.
 
     \obsolete
 
@@ -1713,7 +1717,7 @@ QList<QListWidgetItem*> QListWidget::findItems(const QString &text, Qt::MatchFla
 }
 
 /*!
-    Returns true if the \a item is explicitly hidden; otherwise returns false.
+    Returns \c true if the \a item is explicitly hidden; otherwise returns \c false.
 
     \obsolete
 
@@ -1788,8 +1792,8 @@ QMimeData *QListWidget::mimeData(const QList<QListWidgetItem*>) const
 #ifndef QT_NO_DRAGANDDROP
 /*!
     Handles \a data supplied by an external drag and drop operation that ended
-    with the given \a action in the given \a index. Returns true if \a data and
-    \a action can be handled by the model; otherwise returns false.
+    with the given \a action in the given \a index. Returns \c true if \a data and
+    \a action can be handled by the model; otherwise returns \c false.
 
     \sa supportedDropActions()
 */
@@ -1828,7 +1832,7 @@ void QListWidget::dropEvent(QDropEvent *event) {
 
             if (persIndexes.contains(topIndex))
                 return;
-            qSort(persIndexes); // The dropped items will remain in the same visual order.
+            std::sort(persIndexes.begin(), persIndexes.end()); // The dropped items will remain in the same visual order.
 
             QPersistentModelIndex dropRow = model()->index(row, col, topIndex);
 
