@@ -3,7 +3,7 @@
 ** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
-** This file is part of the QtGui module of the Qt Toolkit.
+** This file is part of the QtWidgets module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
@@ -47,8 +47,6 @@
 #include <QtCore/qvector.h>
 //#include <QtWidgets/qitemselectionmodel.h>
 
-QT_BEGIN_HEADER
-
 QT_BEGIN_NAMESPACE
 
 
@@ -371,6 +369,4 @@ inline bool QTableWidgetItem::isSelected() const
 
 QT_END_NAMESPACE
 
-QT_END_HEADER
-
 #endif // QTABLEWIDGET_H
