@@ -3,7 +3,7 @@
 ** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
-** This file is part of the QtGui module of the Qt Toolkit.
+** This file is part of the QtWidgets module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
@@ -44,8 +44,6 @@
 
 #include <QtWidgets/qabstractitemview.h>
 
-QT_BEGIN_HEADER
-
 QT_BEGIN_NAMESPACE
 
 
@@ -130,6 +128,9 @@ public:
     void setWordWrap(bool on);
     bool wordWrap() const;
 
+    void setTreePosition(int logicalIndex);
+    int treePosition() const;
+
     void keyboardSearch(const QString &search);
 
     QRect visualRect(const QModelIndex &index) const;
@@ -205,6 +206,8 @@ protected:
 
     void updateGeometries();
 
+    QSize viewportSizeHint() const Q_DECL_OVERRIDE;
+
     int sizeHintForColumn(int column) const;
     int indexRowSizeHint(const QModelIndex &index) const;
     int rowHeight(const QModelIndex &index) const;
@@ -235,6 +238,4 @@ private:
 
 QT_END_NAMESPACE
 
-QT_END_HEADER
-
 #endif // QTREEVIEW_H
