@@ -3,7 +3,7 @@
 ** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
-** This file is part of the QtGui module of the Qt Toolkit.
+** This file is part of the QtWidgets module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
@@ -52,8 +52,8 @@
 QT_BEGIN_NAMESPACE
 
 /*
-    Returns true if the \a widget can be added to the \a layout;
-    otherwise returns false.
+    Returns \c true if the \a widget can be added to the \a layout;
+    otherwise returns \c false.
 */
 static bool checkWidget(QLayout *layout, QWidget *widget)
 {
@@ -140,6 +140,7 @@ public:
     void calcHfw(int);
 
     void effectiveMargins(int *left, int *top, int *right, int *bottom) const;
+    QLayoutItem* replaceAt(int index, QLayoutItem*) Q_DECL_OVERRIDE;
 };
 
 QBoxLayoutPrivate::~QBoxLayoutPrivate()
@@ -248,7 +249,7 @@ void QBoxLayoutPrivate::effectiveMargins(int *left, int *top, int *right, int *b
                 if (right)
                     r = qMax(r, wr.right() - lir.right());
             }
-        }        
+        }
     }
 #endif
     if (left)
@@ -444,6 +445,21 @@ void QBoxLayoutPrivate::calcHfw(int w)
     hfwMinHeight = mh;
 }
 
+QLayoutItem* QBoxLayoutPrivate::replaceAt(int index, QLayoutItem *item)
+{
+    Q_Q(QBoxLayout);
+    if (!item)
+        return 0;
+    QBoxLayoutItem *b = list.value(index);
+    if (!b)
+        return 0;
+    QLayoutItem *r = b->item;
+
+    b->item = item;
+    q->invalidate();
+    return r;
+}
+
 
 /*!
     \class QBoxLayout
@@ -509,7 +525,7 @@ void QBoxLayoutPrivate::calcHfw(int w)
 
     \list
     \li setContentsMargins() sets the width of the outer border on
-       each side of the widget. This is the width of the reserved space 
+       each side of the widget. This is the width of the reserved space
        along each of the QBoxLayout's four sides.
     \li setSpacing() sets the width between neighboring boxes. (You
        can use addSpacing() to get more space at a particular spot.)
@@ -595,7 +611,7 @@ int QBoxLayout::spacing() const
 
 /*!
   Reimplements QLayout::setSpacing(). Sets the spacing
-  property to \a spacing. 
+  property to \a spacing.
 
   \sa QLayout::setSpacing(), spacing()
  */
@@ -942,7 +958,8 @@ void QBoxLayout::insertSpacerItem(int index, QSpacerItem *spacerItem)
 void QBoxLayout::insertLayout(int index, QLayout *layout, int stretch)
 {
     Q_D(QBoxLayout);
-    addChildLayout(layout);
+    if (!adoptLayout(layout))
+        return;
     if (index < 0)                                // append
         index = d->list.count();
     QBoxLayoutItem *it = new QBoxLayoutItem(layout, stretch);
@@ -1095,7 +1112,7 @@ void QBoxLayout::addStrut(int size)
 /*!
     Sets the stretch factor for \a widget to \a stretch and returns
     true if \a widget is found in this layout (not including child
-    layouts); otherwise returns false.
+    layouts); otherwise returns \c false.
 
     \sa setAlignment()
 */
@@ -1119,8 +1136,8 @@ bool QBoxLayout::setStretchFactor(QWidget *widget, int stretch)
     \overload
 
     Sets the stretch factor for the layout \a layout to \a stretch and
-    returns true if \a layout is found in this layout (not including
-    child layouts); otherwise returns false.
+    returns \c true if \a layout is found in this layout (not including
+    child layouts); otherwise returns \c false.
 */
 bool QBoxLayout::setStretchFactor(QLayout *layout, int stretch)
 {
