@@ -3,7 +3,7 @@
 ** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
-** This file is part of the QtGui module of the Qt Toolkit.
+** This file is part of the QtWidgets module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
@@ -52,8 +52,6 @@
 
 #ifndef QT_NO_GESTURES
 
-QT_BEGIN_HEADER
-
 Q_DECLARE_METATYPE(Qt::GestureState)
 Q_DECLARE_METATYPE(Qt::GestureType)
 
@@ -328,8 +326,6 @@ private:
 QT_END_NAMESPACE
 
 Q_DECLARE_METATYPE(QGesture::GestureCancelPolicy)
-QT_END_HEADER
-
 #endif // QT_NO_GESTURES
 
 #endif // QGESTURE_H
