@@ -3,7 +3,7 @@
 ** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
-** This file is part of the QtGui module of the Qt Toolkit.
+** This file is part of the QtWidgets module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
@@ -85,6 +85,7 @@ public:
     void setGeometry(const QRect &r) { item_->setGeometry(r); }
     Qt::Alignment alignment() const { return item_->alignment(); }
     QLayoutItem *item() { return item_; }
+    void setItem(QLayoutItem *newitem) { item_ = newitem; }
     QLayoutItem *takeItem() { QLayoutItem *i = item_; item_ = 0; return i; }
 
     int hStretch() { return item_->widget() ?
@@ -171,6 +172,18 @@ public:
         }
         return 0;
     }
+    QLayoutItem* replaceAt(int index, QLayoutItem *newitem) Q_DECL_OVERRIDE
+    {
+        if (!newitem)
+            return 0;
+        QLayoutItem *item = 0;
+        QGridBox *b = things.value(index);
+        if (b) {
+            item = b->takeItem();
+            b->setItem(newitem);
+        }
+        return item;
+    }
 
     void getItemPosition(int index, int *row, int *column, int *rowSpan, int *columnSpan) const {
         if (index < things.count()) {
@@ -1418,8 +1431,8 @@ void QGridLayout::addItem(QLayoutItem *item, int row, int column, int rowSpan, i
 }
 
 /*
-  Returns true if the widget \a w can be added to the layout \a l;
-  otherwise returns false.
+  Returns \c true if the widget \a w can be added to the layout \a l;
+  otherwise returns \c false.
 */
 static bool checkWidget(QLayout *l, QWidget *w)
 {
@@ -1505,7 +1518,8 @@ void QGridLayout::addWidget(QWidget *widget, int fromRow, int fromColumn,
 void QGridLayout::addLayout(QLayout *layout, int row, int column, Qt::Alignment alignment)
 {
     Q_D(QGridLayout);
-    addChildLayout(layout);
+    if (!adoptLayout(layout))
+        return;
     QGridBox *b = new QGridBox(layout);
     b->setAlignment(alignment);
     d->add(b, row, column);
@@ -1524,7 +1538,8 @@ void QGridLayout::addLayout(QLayout *layout, int row, int column,
                                       int rowSpan, int columnSpan, Qt::Alignment alignment)
 {
     Q_D(QGridLayout);
-    addChildLayout(layout);
+    if (!adoptLayout(layout))
+        return;
     QGridBox *b = new QGridBox(layout);
     b->setAlignment(alignment);
     d->add(b, row, (rowSpan < 0) ? -1 : row + rowSpan - 1, column, (columnSpan < 0) ? -1 : column + columnSpan - 1);
