@@ -3,7 +3,7 @@
 ** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
-** This file is part of the QtGui module of the Qt Toolkit.
+** This file is part of the QtWidgets module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
@@ -51,6 +51,7 @@
 #include <private/qapplication_p.h>
 #include <private/qshortcutmap_p.h>
 #include <private/qaction_p.h>
+#include <private/qwidgetwindow_qpa_p.h>
 
 QT_BEGIN_NAMESPACE
 
@@ -71,7 +72,7 @@ static bool correctActionContext(Qt::ShortcutContext context, QAction *a, QWidge
 
 
 /*! \internal
-    Returns true if the widget \a w is a logical sub window of the current
+    Returns \c true if the widget \a w is a logical sub window of the current
     top-level widget.
 */
 bool qWidgetShortcutContextMatcher(QObject *object, Qt::ShortcutContext context)
@@ -86,6 +87,20 @@ bool qWidgetShortcutContextMatcher(QObject *object, Qt::ShortcutContext context)
     if (QApplication::activePopupWidget())
         active_window = QApplication::activePopupWidget();
 
+    if (!active_window) {
+        QWindow *qwindow = QGuiApplication::focusWindow();
+        if (qwindow && qwindow->isActive()) {
+            while (qwindow) {
+                QWidgetWindow *widgetWindow = qobject_cast<QWidgetWindow *>(qwindow);
+                if (widgetWindow) {
+                    active_window = widgetWindow->widget();
+                    break;
+                }
+                qwindow = qwindow->parent();
+            }
+        }
+    }
+
     if (!active_window)
         return false;
 
@@ -106,6 +121,18 @@ bool qWidgetShortcutContextMatcher(QObject *object, Qt::ShortcutContext context)
             w = s->parentWidget();
     }
 
+    if (!w) {
+        QWindow *qwindow = qobject_cast<QWindow *>(object);
+        while (qwindow) {
+            QWidgetWindow *widget_window = qobject_cast<QWidgetWindow *>(qwindow);
+            if (widget_window) {
+                w = widget_window->widget();
+                break;
+            }
+            qwindow = qwindow->parent();
+        }
+    }
+
     if (!w)
         return false;
 
@@ -131,7 +158,7 @@ static bool correctWidgetContext(Qt::ShortcutContext context, QWidget *w, QWidge
 
     if (context == Qt::WidgetWithChildrenShortcut) {
         const QWidget *tw = QApplication::focusWidget();
-        while (tw && tw != w && (tw->windowType() == Qt::Widget || tw->windowType() == Qt::Popup))
+        while (tw && tw != w && (tw->windowType() == Qt::Widget || tw->windowType() == Qt::Popup || tw->windowType() == Qt::SubWindow))
             tw = tw->parentWidget();
         return tw == w;
     }
@@ -246,6 +273,15 @@ static bool correctActionContext(Qt::ShortcutContext context, QAction *a, QWidge
         QWidget *w = widgets.at(i);
 #ifndef QT_NO_MENU
         if (QMenu *menu = qobject_cast<QMenu *>(w)) {
+#ifdef Q_OS_MAC
+            // On Mac, menu item shortcuts are processed before reaching any window.
+            // That means that if a menu action shortcut has not been already processed
+            // (and reaches this point), then the menu item itself has been disabled.
+            // This occurs at the QPA level on Mac, were we disable all the Cocoa menus
+            // when showing a modal window.
+            if (a->shortcut().count() <= 1)
+                continue;
+#endif
             QAction *a = menu->menuAction();
             if (correctActionContext(context, a, active_window))
                 return true;
@@ -480,7 +516,7 @@ QKeySequence QShortcut::key() const
     If the application is in \c WhatsThis mode the shortcut will not emit
     the signals, but will show the "What's This?" text instead.
 
-    By default, this property is true.
+    By default, this property is \c true.
 
     \sa whatsThis
 */
