@@ -3,7 +3,7 @@
 ** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
-** This file is part of the QtGui module of the Qt Toolkit.
+** This file is part of the QtWidgets module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
@@ -52,6 +52,8 @@
 #include <qpa/qplatformopenglcontext.h>
 #include <qpa/qplatformintegration.h>
 #include "QtGui/private/qwindow_p.h"
+#include "QtGui/private/qguiapplication_p.h"
+#include <private/qwindowcontainer_p.h>
 
 #include <qpa/qplatformcursor.h>
 #include <QtGui/QGuiApplication>
@@ -68,7 +70,7 @@ void q_createNativeChildrenAndSetParent(const QWidget *parentWidget)
             const QWidget *childWidget = qobject_cast<const QWidget *>(children.at(i));
             if (childWidget) { // should not be necessary
                 if (childWidget->testAttribute(Qt::WA_NativeWindow)) {
-                    if (!childWidget->windowHandle())
+                    if (!childWidget->internalWinId())
                         childWidget->winId();
                     if (childWidget->windowHandle()) {
                         QWindow *parentWindow = childWidget->nativeParentWidget()->windowHandle();
@@ -109,14 +111,19 @@ void QWidgetPrivate::create_sys(WId window, bool initializeWindow, bool destroyO
 
     win->setFlags(data.window_flags);
     fixPosIncludesFrame();
-    win->setGeometry(q->geometry());
+    if (q->testAttribute(Qt::WA_Moved)
+        || !QGuiApplicationPrivate::platformIntegration()->hasCapability(QPlatformIntegration::WindowManagement))
+        win->setGeometry(q->geometry());
+    else
+        win->resize(q->size());
     win->setScreen(QGuiApplication::screens().value(topData()->screenIndex, 0));
 
-    if (q->testAttribute(Qt::WA_TranslucentBackground)) {
-        QSurfaceFormat format;
+    QSurfaceFormat format = win->requestedFormat();
+    if ((flags & Qt::Window) && win->surfaceType() != QSurface::OpenGLSurface
+            && q->testAttribute(Qt::WA_TranslucentBackground)) {
         format.setAlphaBufferSize(8);
-        win->setFormat(format);
     }
+    win->setFormat(format);
 
     if (QWidget *nativeParent = q->nativeParentWidget()) {
         if (nativeParent->windowHandle()) {
@@ -133,20 +140,28 @@ void QWidgetPrivate::create_sys(WId window, bool initializeWindow, bool destroyO
     qt_window_private(win)->positionPolicy = topData()->posIncludesFrame ?
         QWindowPrivate::WindowFrameInclusive : QWindowPrivate::WindowFrameExclusive;
     win->create();
+    // Enable nonclient-area events for QDockWidget and other NonClientArea-mouse event processing.
+    if ((flags & Qt::Desktop) == Qt::Window)
+        win->handle()->setFrameStrutEventsEnabled(true);
 
     data.window_flags = win->flags();
 
     QBackingStore *store = q->backingStore();
 
     if (!store) {
-        if (win && q->windowType() != Qt::Desktop)
-            q->setBackingStore(new QBackingStore(win));
-        else
+        if (win && q->windowType() != Qt::Desktop) {
+            if (q->isTopLevel())
+                q->setBackingStore(new QBackingStore(win));
+        } else {
             q->setAttribute(Qt::WA_PaintOnScreen, true);
+        }
     }
 
     setWindowModified_helper();
-    setWinId(win->winId());
+    WId id = win->winId();
+    // See the QPlatformWindow::winId() documentation
+    Q_ASSERT(id != WId(0));
+    setWinId(id);
 
     // Check children and create windows for them if necessary
     q_createNativeChildrenAndSetParent(q);
@@ -168,7 +183,7 @@ void QWidget::destroy(bool destroyWindow, bool destroySubWindows)
         parentWidget()->d_func()->invalidateBuffer(d->effectiveRectFor(geometry()));
     d->deactivateWidgetCleanup();
 
-    if ((windowType() == Qt::Popup))
+    if ((windowType() == Qt::Popup) && qApp)
         qApp->d_func()->closePopup(this);
 
     if (this == QApplicationPrivate::active_window)
@@ -256,8 +271,9 @@ void QWidgetPrivate::setParent_sys(QWidget *newparent, Qt::WindowFlags f)
     bool explicitlyHidden = q->testAttribute(Qt::WA_WState_Hidden) && q->testAttribute(Qt::WA_WState_ExplicitShowHide);
 
     // Reparenting toplevel to child
-    if (!(f&Qt::Window) && (oldFlags&Qt::Window) && !q->testAttribute(Qt::WA_NativeWindow)) {
-        //qDebug() << "setParent_sys() change from toplevel";
+    if (wasCreated && !(f & Qt::Window) && (oldFlags & Qt::Window) && !q->testAttribute(Qt::WA_NativeWindow)) {
+        if (extra && extra->hasWindowContainer)
+            QWindowContainer::toplevelAboutToBeDestroyed(q);
         q->destroy();
     }
 
@@ -466,19 +482,6 @@ void QWidget::activateWindow()
         wnd->requestActivate();
 }
 
-// Position top level windows at the center, avoid showing
-// Windows at the default 0,0 position excluding the frame.
-static inline QRect positionTopLevelWindow(QRect geometry, const QScreen *screen)
-{
-    if (screen && geometry.x() == 0 && geometry.y() == 0) {
-       const QRect availableGeometry = screen->availableGeometry();
-        if (availableGeometry.width() > geometry.width()
-            && availableGeometry.height() > geometry.height())
-            geometry.moveCenter(availableGeometry.center());
-    }
-    return geometry;
-}
-
 // move() was invoked with Qt::WA_WState_Created not set (frame geometry
 // unknown), that is, crect has a position including the frame.
 // If we can determine the frame strut, fix that and clear the flag.
@@ -529,16 +532,17 @@ void QWidgetPrivate::show_sys()
         if (q->isWindow())
             fixPosIncludesFrame();
         QRect geomRect = q->geometry();
-        if (q->isWindow()) {
-            if (!q->testAttribute(Qt::WA_Moved))
-                geomRect = positionTopLevelWindow(geomRect, window->screen());
-        } else {
+        if (!q->isWindow()) {
             QPoint topLeftOfWindow = q->mapTo(q->nativeParentWidget(),QPoint());
             geomRect.moveTopLeft(topLeftOfWindow);
         }
         const QRect windowRect = window->geometry();
         if (windowRect != geomRect) {
-            window->setGeometry(geomRect);
+            if (q->testAttribute(Qt::WA_Moved)
+                || !QGuiApplicationPrivate::platformIntegration()->hasCapability(QPlatformIntegration::WindowManagement))
+                window->setGeometry(geomRect);
+            else
+                window->resize(geomRect.size());
         }
 
         if (QBackingStore *store = q->backingStore()) {
@@ -552,6 +556,13 @@ void QWidgetPrivate::show_sys()
 #endif
         invalidateBuffer(q->rect());
         window->setVisible(true);
+        // Was the window moved by the Window system or QPlatformWindow::initialGeometry() ?
+        if (window->isTopLevel()) {
+            const QPoint crectTopLeft = q->data->crect.topLeft();
+            const QPoint windowTopLeft = window->geometry().topLeft();
+            if (crectTopLeft == QPoint(0, 0) && windowTopLeft != crectTopLeft)
+                q->data->crect.moveTopLeft(windowTopLeft);
+        }
     }
 }
 
@@ -685,6 +696,16 @@ void QWidgetPrivate::setGeometry_sys(int x, int y, int w, int h, bool isMove)
         h = qMax(h,extra->minh);
     }
 
+    if (q->isWindow() && q->windowHandle()) {
+        QPlatformIntegration *integration = QGuiApplicationPrivate::platformIntegration();
+        if (!integration->hasCapability(QPlatformIntegration::NonFullScreenWindows)) {
+            x = 0;
+            y = 0;
+            w = q->windowHandle()->width();
+            h = q->windowHandle()->height();
+        }
+    }
+
     QPoint oldp = q->geometry().topLeft();
     QSize olds = q->size();
     QRect r(x, y, w, h);
@@ -730,7 +751,7 @@ void QWidgetPrivate::setGeometry_sys(int x, int y, int w, int h, bool isMove)
                     q->windowHandle()->setGeometry(QRect(posInNativeParent,r.size()));
                 }
                 const QWidgetBackingStore *bs = maybeBackingStore();
-                if (bs->store) {
+                if (bs && bs->store) {
                     if (isResize)
                         bs->store->resize(r.size());
                 }
@@ -846,6 +867,8 @@ int QWidget::metric(PaintDeviceMetric m) const
         return qRound(screen->physicalDotsPerInchX());
     } else if (m == PdmPhysicalDpiY) {
         return qRound(screen->physicalDotsPerInchY());
+    } else if (m == PdmDevicePixelRatio) {
+        return screen->devicePixelRatio();
     } else {
         val = QPaintDevice::metric(m);// XXX
     }
@@ -876,18 +899,31 @@ void QWidgetPrivate::deleteSysExtra()
 
 }
 
+#ifdef Q_OS_WIN
+static const char activeXNativeParentHandleProperty[] = "_q_embedded_native_parent_handle";
+#endif
+
 void QWidgetPrivate::createTLSysExtra()
 {
     Q_Q(QWidget);
-    extra->topextra->screenIndex = 0;
-    extra->topextra->window = 0;
-    if (q->testAttribute(Qt::WA_NativeWindow) || q->isWindow()) {
+    if (!extra->topextra->window && (q->testAttribute(Qt::WA_NativeWindow) || q->isWindow())) {
         extra->topextra->window = new QWidgetWindow(q);
         if (extra->minw || extra->minh)
             extra->topextra->window->setMinimumSize(QSize(extra->minw, extra->minh));
         if (extra->maxw != QWIDGETSIZE_MAX || extra->maxh != QWIDGETSIZE_MAX)
             extra->topextra->window->setMaximumSize(QSize(extra->maxw, extra->maxh));
+        if (extra->topextra->opacity != 255 && q->isWindow())
+            extra->topextra->window->setOpacity(qreal(extra->topextra->opacity) / qreal(255));
+#ifdef Q_OS_WIN
+        // Pass on native parent handle for Widget embedded into Active X.
+        const QVariant activeXNativeParentHandle = q->property(activeXNativeParentHandleProperty);
+        if (activeXNativeParentHandle.isValid())
+            extra->topextra->window->setProperty(activeXNativeParentHandleProperty, activeXNativeParentHandle);
+        if (q->inherits("QTipLabel") || q->inherits("QAlphaWidget"))
+            extra->topextra->window->setProperty("_q_windowsDropShadow", QVariant(true));
+#endif
     }
+
 }
 
 void QWidgetPrivate::deleteTLSysExtra()
@@ -999,6 +1035,12 @@ static inline void applyCursor(QWidget *w, QCursor c)
         window->setCursor(c);
 }
 
+static inline void unsetCursor(QWidget *w)
+{
+    if (QWindow *window = w->windowHandle())
+        window->unsetCursor();
+}
+
 void qt_qpa_set_cursor(QWidget *w, bool force)
 {
     if (!w->testAttribute(Qt::WA_WState_Created))
@@ -1032,11 +1074,11 @@ void qt_qpa_set_cursor(QWidget *w, bool force)
         else
             // Enforce the windows behavior of clearing the cursor on
             // disabled widgets.
-            applyCursor(nativeParent, Qt::ArrowCursor);
+            unsetCursor(nativeParent);
     } else {
-        applyCursor(nativeParent, Qt::ArrowCursor);
+        unsetCursor(nativeParent);
     }
 }
-#endif //QT_NO_CURSOR 
+#endif //QT_NO_CURSOR
 
 QT_END_NAMESPACE
