@@ -3,7 +3,7 @@
 ** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
-** This file is part of the QtGui module of the Qt Toolkit.
+** This file is part of the QtWidgets module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
@@ -240,7 +240,13 @@ private:
     }
 
     inline bool hasStaticContents() const
-    { return !staticWidgets.isEmpty() && false; }
+    {
+#if defined(Q_OS_WIN)
+        return !staticWidgets.isEmpty();
+#else
+        return !staticWidgets.isEmpty() && false;
+#endif
+    }
 
     friend QRegion qt_dirtyRegion(QWidget *);
     friend class QWidgetPrivate;
