@@ -3,7 +3,7 @@
 ** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
-** This file is part of the QtGui module of the Qt Toolkit.
+** This file is part of the QtWidgets module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
@@ -49,9 +49,13 @@
 #endif
 #include <private/qwidgetbackingstore_p.h>
 #include <qpa/qwindowsysteminterface_p.h>
+#include <qpa/qplatformtheme.h>
+#include <private/qgesturemanager_p.h>
 
 QT_BEGIN_NAMESPACE
 
+Q_WIDGETS_EXPORT extern bool qt_tab_all_widgets();
+
 QWidget *qt_button_down = 0; // widget got last button-down
 static QWidget *qt_tablet_target = 0;
 
@@ -73,6 +77,13 @@ public:
         }
         return w;
     }
+
+    void clearFocusObject()
+    {
+        if (QApplicationPrivate::focus_widget)
+            QApplicationPrivate::focus_widget->clearFocus();
+    }
+
 };
 
 QWidgetWindow::QWidgetWindow(QWidget *widget)
@@ -83,6 +94,12 @@ QWidgetWindow::QWidgetWindow(QWidget *widget)
     connect(m_widget, &QObject::objectNameChanged, this, &QWidgetWindow::updateObjectName);
 }
 
+QWidgetWindow::~QWidgetWindow()
+{
+    if (m_widget == qt_tablet_target)
+        qt_tablet_target = 0;
+}
+
 #ifndef QT_NO_ACCESSIBILITY
 QAccessibleInterface *QWidgetWindow::accessibleRoot() const
 {
@@ -123,6 +140,8 @@ bool QWidgetWindow::event(QEvent *event)
     // these should not be sent to QWidget, the corresponding events
     // are sent by QApplicationPrivate::notifyActiveWindowChange()
     case QEvent::FocusIn:
+        handleFocusInEvent(static_cast<QFocusEvent *>(event));
+        // Fallthrough
     case QEvent::FocusOut: {
 #ifndef QT_NO_ACCESSIBILITY
         QAccessible::State state;
@@ -216,6 +235,13 @@ bool QWidgetWindow::event(QEvent *event)
         handleTabletEvent(static_cast<QTabletEvent *>(event));
         return true;
 #endif
+
+#ifndef QT_NO_GESTURES
+    case QEvent::NativeGesture:
+        handleGestureEvent(static_cast<QNativeGestureEvent *>(event));
+        return true;
+#endif
+
 #ifndef QT_NO_CONTEXTMENU
     case QEvent::ContextMenu:
         handleContextMenuEvent(static_cast<QContextMenuEvent *>(event));
@@ -226,7 +252,9 @@ bool QWidgetWindow::event(QEvent *event)
     case QEvent::Show:
     case QEvent::Hide:
         return QWindow::event(event);
-
+    case QEvent::WindowBlocked:
+        qt_button_down = 0;
+        break;
     default:
         break;
     }
@@ -284,6 +312,42 @@ void QWidgetWindow::handleEnterLeaveEvent(QEvent *event)
     }
 }
 
+QWidget *QWidgetWindow::getFocusWidget(FocusWidgets fw)
+{
+    QWidget *tlw = m_widget;
+    QWidget *w = tlw->nextInFocusChain();
+
+    QWidget *last = tlw;
+
+    uint focus_flag = qt_tab_all_widgets() ? Qt::TabFocus : Qt::StrongFocus;
+
+    while (w != tlw)
+    {
+        if (((w->focusPolicy() & focus_flag) == focus_flag)
+            && w->isVisibleTo(m_widget) && w->isEnabled())
+        {
+            last = w;
+            if (fw == FirstFocusWidget)
+                break;
+        }
+        w = w->nextInFocusChain();
+    }
+
+    return last;
+}
+
+void QWidgetWindow::handleFocusInEvent(QFocusEvent *e)
+{
+    QWidget *focusWidget = 0;
+    if (e->reason() == Qt::BacktabFocusReason)
+        focusWidget = getFocusWidget(LastFocusWidget);
+    else if (e->reason() == Qt::TabFocusReason)
+        focusWidget = getFocusWidget(FirstFocusWidget);
+
+    if (focusWidget != 0)
+        focusWidget->setFocus();
+}
+
 void QWidgetWindow::handleNonClientAreaMouseEvent(QMouseEvent *e)
 {
     QApplication::sendSpontaneousEvent(m_widget, e);
@@ -291,6 +355,9 @@ void QWidgetWindow::handleNonClientAreaMouseEvent(QMouseEvent *e)
 
 void QWidgetWindow::handleMouseEvent(QMouseEvent *event)
 {
+    static const QEvent::Type contextMenuTrigger =
+        QGuiApplicationPrivate::platformTheme()->themeHint(QPlatformTheme::ContextMenuOnMouseRelease).toBool() ?
+        QEvent::MouseButtonRelease : QEvent::MouseButtonPress;
     if (qApp->d_func()->inPopupMode()) {
         QWidget *activePopupWidget = qApp->activePopupWidget();
         QWidget *popup = activePopupWidget;
@@ -352,9 +419,30 @@ void QWidgetWindow::handleMouseEvent(QMouseEvent *event)
             && qt_replay_popup_mouse_event) {
             if (m_widget->windowType() != Qt::Popup)
                 qt_button_down = 0;
+            if (event->type() == QEvent::MouseButtonPress) {
+                // the popup disappeared, replay the mouse press event
+                QWidget *w = QApplication::widgetAt(event->globalPos());
+                if (w && !QApplicationPrivate::isBlockedByModal(w)) {
+                    // activate window of the widget under mouse pointer
+                    if (!w->isActiveWindow()) {
+                        w->activateWindow();
+                        w->raise();
+                    }
+
+                    QWindow *win = w->windowHandle();
+                    if (!win)
+                        win = w->nativeParentWidget()->windowHandle();
+                    if (win && win->geometry().contains(event->globalPos())) {
+                        const QPoint localPos = win->mapFromGlobal(event->globalPos());
+                        QMouseEvent e(QEvent::MouseButtonPress, localPos, localPos, event->globalPos(), event->button(), event->buttons(), event->modifiers());
+                        e.setTimestamp(event->timestamp());
+                        QApplication::sendSpontaneousEvent(win, &e);
+                    }
+                }
+            }
             qt_replay_popup_mouse_event = false;
 #ifndef QT_NO_CONTEXTMENU
-        } else if (event->type() == QEvent::MouseButtonPress
+        } else if (event->type() == contextMenuTrigger
                    && event->button() == Qt::RightButton
                    && (openPopupCount == oldOpenPopupCount)) {
             QWidget *popupEvent = popup;
@@ -385,7 +473,7 @@ void QWidgetWindow::handleMouseEvent(QMouseEvent *event)
     if (!widget)
         widget = m_widget;
 
-    if (event->type() == QEvent::MouseButtonPress && !qt_button_down)
+    if (event->type() == QEvent::MouseButtonPress)
         qt_button_down = widget;
 
     QWidget *receiver = QApplicationPrivate::pickMouseReceiver(m_widget, event->windowPos().toPoint(), &mapped, event->type(), event->buttons(),
@@ -403,7 +491,7 @@ void QWidgetWindow::handleMouseEvent(QMouseEvent *event)
                                         qt_last_mouse_receiver);
 
 #ifndef QT_NO_CONTEXTMENU
-    if (event->type() == QEvent::MouseButtonPress && event->button() == Qt::RightButton) {
+    if (event->type() == contextMenuTrigger && event->button() == Qt::RightButton) {
         QContextMenuEvent e(QContextMenuEvent::Mouse, mapped, event->globalPos(), event->modifiers());
         QGuiApplication::sendSpontaneousEvent(receiver, &e);
     }
@@ -412,10 +500,16 @@ void QWidgetWindow::handleMouseEvent(QMouseEvent *event)
 
 void QWidgetWindow::handleTouchEvent(QTouchEvent *event)
 {
-    if (event->type() == QEvent::TouchCancel)
+    if (event->type() == QEvent::TouchCancel) {
         QApplicationPrivate::translateTouchCancel(event->device(), event->timestamp());
-    else
-        QApplicationPrivate::translateRawTouchEvent(m_widget, event->device(), event->touchPoints(), event->timestamp());
+        event->accept();
+    } else if (qApp->d_func()->inPopupMode()) {
+        // Ignore touch events for popups. This will cause QGuiApplication to synthesise mouse
+        // events instead, which QWidgetWindow::handleMouseEvent will forward correctly:
+        event->ignore();
+    } else {
+        event->setAccepted(QApplicationPrivate::translateRawTouchEvent(m_widget, event->device(), event->touchPoints(), event->timestamp()));
+    }
 }
 
 void QWidgetWindow::handleKeyEvent(QKeyEvent *event)
@@ -423,14 +517,12 @@ void QWidgetWindow::handleKeyEvent(QKeyEvent *event)
     if (QApplicationPrivate::instance()->modalState() && !qt_try_modal(m_widget, event->type()))
         return;
 
-    QObject *receiver = 0;
-    if (QApplicationPrivate::inPopupMode()) {
+    QObject *receiver = QWidget::keyboardGrabber();
+    if (!receiver && QApplicationPrivate::inPopupMode()) {
         QWidget *popup = QApplication::activePopupWidget();
         QWidget *popupFocusWidget = popup->focusWidget();
         receiver = popupFocusWidget ? popupFocusWidget : popup;
     }
-    if (!receiver)
-        receiver = QWidget::keyboardGrabber();
     if (!receiver)
         receiver = focusObject();
     QGuiApplication::sendSpontaneousEvent(receiver, event);
@@ -473,9 +565,10 @@ void QWidgetWindow::handleResizeEvent(QResizeEvent *event)
     }
 }
 
-void QWidgetWindow::handleCloseEvent(QCloseEvent *)
+void QWidgetWindow::handleCloseEvent(QCloseEvent *event)
 {
-    m_widget->d_func()->close_helper(QWidgetPrivate::CloseWithSpontaneousEvent);
+    bool is_closing = m_widget->d_func()->close_helper(QWidgetPrivate::CloseWithSpontaneousEvent);
+    event->setAccepted(is_closing);
 }
 
 #ifndef QT_NO_WHEELEVENT
@@ -493,7 +586,7 @@ void QWidgetWindow::handleWheelEvent(QWheelEvent *event)
 
     QPoint mapped = widget->mapFrom(m_widget, event->pos());
 
-    QWheelEvent translated(mapped, event->globalPos(), event->pixelDelta(), event->angleDelta(), event->delta(), event->orientation(), event->buttons(), event->modifiers());
+    QWheelEvent translated(mapped, event->globalPos(), event->pixelDelta(), event->angleDelta(), event->delta(), event->orientation(), event->buttons(), event->modifiers(), event->phase());
     QGuiApplication::sendSpontaneousEvent(widget, &translated);
 }
 
@@ -666,6 +759,25 @@ void QWidgetWindow::handleTabletEvent(QTabletEvent *event)
 }
 #endif // QT_NO_TABLETEVENT
 
+#ifndef QT_NO_GESTURES
+void QWidgetWindow::handleGestureEvent(QNativeGestureEvent *e)
+{
+    // copy-pasted code to find correct widget follows:
+    QObject *receiver = 0;
+    if (QApplicationPrivate::inPopupMode()) {
+        QWidget *popup = QApplication::activePopupWidget();
+        QWidget *popupFocusWidget = popup->focusWidget();
+        receiver = popupFocusWidget ? popupFocusWidget : popup;
+    }
+    if (!receiver)
+        receiver = QApplication::widgetAt(e->globalPos());
+    if (!receiver)
+        receiver = m_widget; // last resort
+
+    QApplication::sendSpontaneousEvent(receiver, e);
+}
+#endif // QT_NO_GESTURES
+
 #ifndef QT_NO_CONTEXTMENU
 void QWidgetWindow::handleContextMenuEvent(QContextMenuEvent *e)
 {
