@@ -3,7 +3,7 @@
 ** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
-** This file is part of the QtGui module of the Qt Toolkit.
+** This file is part of the QtWidgets module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
@@ -93,6 +93,7 @@
 #include <qlibrary.h>
 #include <qdatetimeedit.h>
 #include <qmath.h>
+#include <qset.h>
 #include <QtWidgets/qgraphicsproxywidget.h>
 #include <QtWidgets/qgraphicsview.h>
 
@@ -193,10 +194,13 @@ public:
                                HIThemeButtonDrawInfo *bdi) const;
     QPixmap generateBackgroundPattern() const;
 
+    void setAutoDefaultButton(QObject *button) const;
+
 public:
     mutable QPointer<QObject> pressedButton;
     mutable QPointer<QObject> defaultButton;
     mutable QPointer<QObject> autoDefaultButton;
+    static  QSet<QPointer<QObject> > scrollBars;
 
     struct ButtonState {
         int frame;
