@@ -3,7 +3,7 @@
 ** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
-** This file is part of the QtGui module of the Qt Toolkit.
+** This file is part of the QtWidgets module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
@@ -46,13 +46,9 @@
 
 QT_BEGIN_NAMESPACE
 
-QStyleAnimation::QStyleAnimation(QObject *target) : QAbstractAnimation(),
+QStyleAnimation::QStyleAnimation(QObject *target) : QAbstractAnimation(target),
     _delay(0), _duration(-1), _startTime(QTime::currentTime())
 {
-    if (target) {
-        moveToThread(target->thread());
-        setParent(target);
-    }
 }
 
 QStyleAnimation::~QStyleAnimation()
@@ -97,7 +93,10 @@ void QStyleAnimation::setStartTime(const QTime &time)
 void QStyleAnimation::updateTarget()
 {
     QEvent event(QEvent::StyleAnimationUpdate);
+    event.setAccepted(false);
     QCoreApplication::sendEvent(target(), &event);
+    if (!event.isAccepted())
+        stop();
 }
 
 bool QStyleAnimation::isUpdateNeeded() const
@@ -107,16 +106,8 @@ bool QStyleAnimation::isUpdateNeeded() const
 
 void QStyleAnimation::updateCurrentTime(int)
 {
-    if (QObject *tgt = target()) {
-        if (tgt->isWidgetType()) {
-            QWidget *widget = static_cast<QWidget *>(tgt);
-            if (!widget->isVisible() || widget->window()->isMinimized())
-                stop();
-        }
-
-        if (isUpdateNeeded())
-            updateTarget();
-    }
+    if (target() && isUpdateNeeded())
+        updateTarget();
 }
 
 QProgressStyleAnimation::QProgressStyleAnimation(int speed, QObject *target) :
