@@ -3,7 +3,7 @@
 ** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
-** This file is part of the QtGui module of the Qt Toolkit.
+** This file is part of the QtWidgets module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
@@ -100,7 +100,7 @@ QT_BEGIN_NAMESPACE
     \fn bool QStylePainter::begin(QWidget *widget)
 
     Begin painting operations on the specified \a widget.
-    Returns true if the painter is ready to use; otherwise returns false.
+    Returns \c true if the painter is ready to use; otherwise returns \c false.
 
     This is automatically called by the constructor that takes a QWidget.
 */
