@@ -3,7 +3,7 @@
 ** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
-** This file is part of the QtGui module of the Qt Toolkit.
+** This file is part of the QtWidgets module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
@@ -48,8 +48,6 @@
 #include <QtCore/qabstractitemmodel.h>
 #include <QtCore/qrect.h>
 
-QT_BEGIN_HEADER
-
 QT_BEGIN_NAMESPACE
 
 
@@ -65,6 +63,7 @@ class Q_WIDGETS_EXPORT QCompleter : public QObject
     Q_OBJECT
     Q_PROPERTY(QString completionPrefix READ completionPrefix WRITE setCompletionPrefix)
     Q_PROPERTY(ModelSorting modelSorting READ modelSorting WRITE setModelSorting)
+    Q_PROPERTY(Qt::MatchFlags filterMode READ filterMode WRITE setFilterMode)
     Q_PROPERTY(CompletionMode completionMode READ completionMode WRITE setCompletionMode)
     Q_PROPERTY(int completionColumn READ completionColumn WRITE setCompletionColumn)
     Q_PROPERTY(int completionRole READ completionRole WRITE setCompletionRole)
@@ -101,6 +100,9 @@ public:
     void setCompletionMode(CompletionMode mode);
     CompletionMode completionMode() const;
 
+    void setFilterMode(Qt::MatchFlags filterMode);
+    Qt::MatchFlags filterMode() const;
+
     QAbstractItemView *popup() const;
     void setPopup(QAbstractItemView *popup);
 
@@ -165,6 +167,4 @@ private:
 
 QT_END_NAMESPACE
 
-QT_END_HEADER
-
 #endif // QCOMPLETER_H
