@@ -3,7 +3,7 @@
 ** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
-** This file is part of the QtGui module of the Qt Toolkit.
+** This file is part of the QtWidgets module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
@@ -237,7 +237,7 @@ private:
     \since 5.0
 
     \inmodule QtWidgets
- 
+
     With kinetic scrolling, the user can push the widget in a given
     direction and it will continue to scroll in this direction until it is
     stopped either by the user or by friction.  Aspects of inertia, friction
@@ -1429,7 +1429,7 @@ void QScrollerPrivate::createScrollingSegments(const QPointF &v,
 
 /*! \internal
     Prepares scrolling by sending a QScrollPrepareEvent to the receiver widget.
-    Returns true if the scrolling was accepted and a target was returned.
+    Returns \c true if the scrolling was accepted and a target was returned.
 */
 bool QScrollerPrivate::prepareScrolling(const QPointF &position)
 {
