@@ -3,7 +3,7 @@
 ** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
-** This file is part of the QtGui module of the Qt Toolkit.
+** This file is part of the QtWidgets module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
@@ -94,7 +94,7 @@ QScrollerPropertiesPrivate *QScrollerPropertiesPrivate::defaults()
     \since 4.8
 
     \inmodule QtWidgets
- 
+
     The QScrollerProperties class stores the parameters used by QScroller.
 
     The default settings are platform dependent so that Qt emulates the
@@ -139,7 +139,7 @@ QScrollerProperties::~QScrollerProperties()
 }
 
 /*!
-    Returns true if these scroller properties are equal to \a sp; otherwise returns false.
+    Returns \c true if these scroller properties are equal to \a sp; otherwise returns \c false.
 */
 bool QScrollerProperties::operator==(const QScrollerProperties &sp) const
 {
@@ -147,7 +147,7 @@ bool QScrollerProperties::operator==(const QScrollerProperties &sp) const
 }
 
 /*!
-    Returns true if these scroller properties are different from \a sp; otherwise returns false.
+    Returns \c true if these scroller properties are different from \a sp; otherwise returns \c false.
 */
 bool QScrollerProperties::operator!=(const QScrollerProperties &sp) const
 {
