@@ -3,7 +3,7 @@
 ** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
-** This file is part of the QtGui module of the Qt Toolkit.
+** This file is part of the QtWidgets module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
@@ -294,7 +294,7 @@ void QUndoGroup::redo()
     Returns the value of the active stack's QUndoStack::canUndo().
 
     If none of the stacks are active, or if the group is empty, this function
-    returns false.
+    returns \c false.
 
     \sa canRedo(), setActiveStack()
 */
@@ -309,7 +309,7 @@ bool QUndoGroup::canUndo() const
     Returns the value of the active stack's QUndoStack::canRedo().
 
     If none of the stacks are active, or if the group is empty, this function
-    returns false.
+    returns \c false.
 
     \sa canUndo(), setActiveStack()
 */
@@ -354,7 +354,7 @@ QString QUndoGroup::redoText() const
     Returns the value of the active stack's QUndoStack::isClean().
 
     If none of the stacks are active, or if the group is empty, this function
-    returns true.
+    returns \c true.
 
     \sa setActiveStack()
 */
