@@ -3,7 +3,7 @@
 ** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
-** This file is part of the QtGui module of the Qt Toolkit.
+** This file is part of the QtWidgets module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
@@ -549,7 +549,7 @@ void QAbstractSlider::setValue(int value)
     \property QAbstractSlider::invertedAppearance
     \brief whether or not a slider shows its values inverted.
 
-    If this property is false (the default), the minimum and maximum will
+    If this property is \c false (the default), the minimum and maximum will
     be shown in its classic position for the inherited widget. If the
     value is true, the minimum and maximum appear at their opposite location.
 
@@ -577,7 +577,7 @@ void QAbstractSlider::setInvertedAppearance(bool invert)
     \property QAbstractSlider::invertedControls
     \brief whether or not the slider inverts its wheel and key events.
 
-    If this property is false, scrolling the mouse wheel "up" and using keys
+    If this property is \c false, scrolling the mouse wheel "up" and using keys
     like page up will increase the slider's value towards its maximum. Otherwise
     pressing page up will move value towards the slider's minimum.
 */
@@ -724,7 +724,7 @@ bool QAbstractSliderPrivate::scrollByDelta(Qt::Orientation orientation, Qt::Keyb
 #else
         // Native UI-elements on Mac can scroll hundreds of lines at a time as
         // a result of acceleration. So keep the same behaviour in Qt, and
-        // don't restrict stepsToScroll to certain maximum (pageStep): 
+        // don't restrict stepsToScroll to certain maximum (pageStep):
         stepsToScroll = int(offset_accumulated);
 #endif
         offset_accumulated -= int(offset_accumulated);
