@@ -3,7 +3,7 @@
 ** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
-** This file is part of the QtGui module of the Qt Toolkit.
+** This file is part of the QtWidgets module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
@@ -46,6 +46,7 @@
 #include <QtWidgets/qdialog.h>
 #include <QtWidgets/qapplication.h>
 #include <private/qwidget_p.h>
+#include <QtGui/qpa/qplatformdialoghelper.h>
 #include <QtWidgets/qaction.h>
 
 #include "qdialogbuttonbox.h"
@@ -166,46 +167,8 @@ enum {
 
 static QDialogButtonBox::ButtonRole roleFor(QDialogButtonBox::StandardButton button)
 {
-    switch (button) {
-    case QDialogButtonBox::Ok:
-    case QDialogButtonBox::Save:
-    case QDialogButtonBox::Open:
-    case QDialogButtonBox::SaveAll:
-    case QDialogButtonBox::Retry:
-    case QDialogButtonBox::Ignore:
-        return QDialogButtonBox::AcceptRole;
-
-    case QDialogButtonBox::Cancel:
-    case QDialogButtonBox::Close:
-    case QDialogButtonBox::Abort:
-        return QDialogButtonBox::RejectRole;
-
-    case QDialogButtonBox::Discard:
-        return QDialogButtonBox::DestructiveRole;
-
-    case QDialogButtonBox::Help:
-        return QDialogButtonBox::HelpRole;
-
-    case QDialogButtonBox::Apply:
-        return QDialogButtonBox::ApplyRole;
-
-    case QDialogButtonBox::Yes:
-    case QDialogButtonBox::YesToAll:
-        return QDialogButtonBox::YesRole;
-
-    case QDialogButtonBox::No:
-    case QDialogButtonBox::NoToAll:
-        return QDialogButtonBox::NoRole;
-
-    case QDialogButtonBox::RestoreDefaults:
-    case QDialogButtonBox::Reset:
-        return QDialogButtonBox::ResetRole;
-
-    case QDialogButtonBox::NoButton:    // NoButton means zero buttons, not "No" button
-        ;
-    }
-
-    return QDialogButtonBox::InvalidRole;
+    return static_cast<QDialogButtonBox::ButtonRole>(QMessageDialogOptions::buttonRole(
+        static_cast<QMessageDialogOptions::StandardButton>(button)));
 }
 
 static const uint layouts[2][5][14] =
@@ -668,6 +631,21 @@ QDialogButtonBox::QDialogButtonBox(Qt::Orientation orientation, QWidget *parent)
     d_func()->initLayout();
 }
 
+/*!
+    \since 5.2
+
+    Constructs a horizontal button box with the given \a parent, containing
+    the standard buttons specified by \a buttons.
+
+    \sa orientation, addButton()
+*/
+QDialogButtonBox::QDialogButtonBox(StandardButtons buttons, QWidget *parent)
+    : QWidget(*new QDialogButtonBoxPrivate(Qt::Horizontal), parent, 0)
+{
+    d_func()->initLayout();
+    d_func()->createStandardButtons(buttons);
+}
+
 /*!
     Constructs a button box with the given \a orientation and \a parent, containing
     the standard buttons specified by \a buttons.
@@ -1060,7 +1038,7 @@ void QDialogButtonBoxPrivate::_q_handleButtonDestroyed()
     \property QDialogButtonBox::centerButtons
     \brief whether the buttons in the button box are centered
 
-    By default, this property is false. This behavior is appopriate
+    By default, this property is \c false. This behavior is appopriate
     for most types of dialogs. A notable exception is message boxes
     on most platforms (e.g. Windows), where the button box is
     centered horizontally.
