@@ -3,7 +3,7 @@
 ** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
-** This file is part of the QtGui module of the Qt Toolkit.
+** This file is part of the QtWidgets module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
@@ -49,13 +49,13 @@
 #include <qevent.h>
 #include <qfontmetrics.h>
 #include <qwindow.h>
+#include <qscreen.h>
 #include <qmainwindow.h>
 #include <qrubberband.h>
 #include <qstylepainter.h>
 #include <qtoolbutton.h>
 #include <qdebug.h>
 
-#include <qpa/qplatformwindow.h>
 #include <private/qwidgetresizehandler_p.h>
 
 #include "qdockwidget_p.h"
@@ -455,14 +455,6 @@ int QDockWidgetLayout::titleHeight() const
                             perp(verticalTitleBar, floatSize));
 
     QFontMetrics titleFontMetrics = q->fontMetrics();
-#ifdef Q_WS_MAC
-    if (qobject_cast<QMacStyle *>(q->style())) {
-        //### this breaks on proxy styles.  (But is this code still called?)
-        QFont font = qt_app_fonts_hash()->value("QToolButton", q->font());
-        titleFontMetrics = QFontMetrics(font);
-    }
-#endif
-
     int mw = q->style()->pixelMetric(QStyle::PM_DockWidgetTitleMargin, 0, q);
 
     return qMax(buttonHeight + 2, titleFontMetrics.height() + 2*mw);
@@ -674,12 +666,18 @@ void QDockWidgetPrivate::updateButtons()
         = qobject_cast<QAbstractButton*>(dwLayout->widgetForRole(QDockWidgetLayout::FloatButton));
     button->setIcon(q->style()->standardIcon(QStyle::SP_TitleBarNormalButton, &opt, q));
     button->setVisible(canFloat && !hideButtons);
-
+#ifndef QT_NO_ACCESSIBILITY
+    button->setAccessibleName(QDockWidget::tr("Float"));
+    button->setAccessibleDescription(QDockWidget::tr("Undocks and re-attaches the dock widget"));
+#endif
     button
         = qobject_cast <QAbstractButton*>(dwLayout->widgetForRole(QDockWidgetLayout::CloseButton));
     button->setIcon(q->style()->standardIcon(QStyle::SP_TitleBarCloseButton, &opt, q));
     button->setVisible(canClose && !hideButtons);
-
+#ifndef QT_NO_ACCESSIBILITY
+    button->setAccessibleName(QDockWidget::tr("Close"));
+    button->setAccessibleDescription(QDockWidget::tr("Closes the dock widget"));
+#endif
     q->setAttribute(Qt::WA_ContentsPropagated,
                     (canFloat || canClose) && !hideButtons);
 
@@ -874,7 +872,7 @@ bool QDockWidgetPrivate::mouseMoveEvent(QMouseEvent *event)
         QPoint pos = event->globalPos() - state->pressPos;
         q->move(pos);
 
-        if (!state->ctrlDrag)
+        if (state && !state->ctrlDrag)
             mwlayout->hover(state->widgetItem, event->globalPos());
 
         ret = true;
@@ -1000,14 +998,6 @@ void QDockWidgetPrivate::plug(const QRect &rect)
     setWindowState(false, false, rect);
 }
 
-static void setFrameStrutEventsEnabled(const QWidget *w, bool enabled)
-{
-    if (const QWindow *window = w->windowHandle())
-        if (QPlatformWindow *platformWindow = window->handle())
-            if (platformWindow->frameStrutEventsEnabled() != enabled)
-                platformWindow->setFrameStrutEventsEnabled(enabled);
-}
-
 void QDockWidgetPrivate::setWindowState(bool floating, bool unplug, const QRect &rect)
 {
     Q_Q(QDockWidget);
@@ -1060,9 +1050,6 @@ void QDockWidgetPrivate::setWindowState(bool floating, bool unplug, const QRect
         }
     }
 
-    if (floating && nativeDeco)
-        setFrameStrutEventsEnabled(q, true);
-
     resizer->setActive(QWidgetResizeHandler::Resize, !unplug && floating && !nativeDeco);
 }
 
@@ -1258,7 +1245,7 @@ QDockWidget::DockWidgetFeatures QDockWidget::features() const
     window "on top" of its parent QMainWindow, instead of being
     docked in the QMainWindow.
 
-    By default, this property is true.
+    By default, this property is \c true.
 
     \sa isWindow()
 */
@@ -1271,6 +1258,9 @@ void QDockWidget::setFloating(bool floating)
         d->endDrag(true);
 
     QRect r = d->undockedGeometry;
+    // Keep position when undocking for the first time.
+    if (floating && isVisible() && !r.isValid())
+        r = QRect(mapToGlobal(QPoint(0, 0)), size());
 
     d->setWindowState(floating, false, floating ? r : QRect());
 
@@ -1309,8 +1299,8 @@ Qt::DockWidgetAreas QDockWidget::allowedAreas() const
 /*!
     \fn bool QDockWidget::isAreaAllowed(Qt::DockWidgetArea area) const
 
-    Returns true if this dock widget can be placed in the given \a area;
-    otherwise returns false.
+    Returns \c true if this dock widget can be placed in the given \a area;
+    otherwise returns \c false.
 */
 
 /*! \reimp */
@@ -1396,11 +1386,17 @@ bool QDockWidget::event(QEvent *event)
         d->toggleViewAction->setChecked(false);
         emit visibilityChanged(false);
         break;
-    case QEvent::Show:
-        if (static_cast<QDockWidgetLayout *>(QDockWidget::layout())->nativeWindowDeco(isFloating()))
-            setFrameStrutEventsEnabled(this, true);
+    case QEvent::Show: {
         d->toggleViewAction->setChecked(true);
-        emit visibilityChanged(geometry().right() >= 0 && geometry().bottom() >= 0);
+        QPoint parentTopLeft(0, 0);
+        if (isWindow()) {
+            if (const QWindow *window = windowHandle())
+                parentTopLeft = window->screen()->availableVirtualGeometry().topLeft();
+            else
+                parentTopLeft = QGuiApplication::primaryScreen()->availableVirtualGeometry().topLeft();
+        }
+        emit visibilityChanged(geometry().right() >= parentTopLeft.x() && geometry().bottom() >= parentTopLeft.y());
+}
         break;
 #endif
     case QEvent::ApplicationLayoutDirectionChange:
