@@ -3,7 +3,7 @@
 ** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
-** This file is part of the QtGui module of the Qt Toolkit.
+** This file is part of the QtWidgets module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
@@ -121,7 +121,7 @@ void QGroupBox::initStyleOption(QStyleOptionGroupBox *option) const
             option->state |= QStyle::State_Sunken;
     }
 
-    if (!option->palette.isBrushSet(isEnabled() ? QPalette::Active : 
+    if (!option->palette.isBrushSet(isEnabled() ? QPalette::Active :
                                     QPalette::Disabled, QPalette::WindowText))
         option->textColor = QColor(style()->styleHint(QStyle::SH_GroupBox_TextLabelColor,
                                    option, this));
@@ -229,7 +229,7 @@ void QGroupBoxPrivate::init()
     overCheckBox = false;
     pressedControl = QStyle::SC_None;
     calculateFrame();
-    q->setSizePolicy(QSizePolicy(QSizePolicy::Preferred, QSizePolicy::Preferred, 
+    q->setSizePolicy(QSizePolicy(QSizePolicy::Preferred, QSizePolicy::Preferred,
                      QSizePolicy::GroupBox));
 }
 
@@ -543,7 +543,7 @@ void QGroupBox::setFlat(bool b)
     \property QGroupBox::checkable
     \brief whether the group box has a checkbox in its title
 
-    If this property is true, the group box displays its title using
+    If this property is \c true, the group box displays its title using
     a checkbox in place of an ordinary label. If the checkbox is checked,
     the group box's children are enabled; otherwise, they are disabled and
     inaccessible.
@@ -645,8 +645,8 @@ void QGroupBox::setChecked(bool b)
 #ifndef QT_NO_ACCESSIBILITY
         QAccessible::State st;
         st.checked = true;
-        QAccessibleStateChangeEvent *ev = new QAccessibleStateChangeEvent(this, st);
-        QAccessible::updateAccessibility(ev);
+        QAccessibleStateChangeEvent e(this, st);
+        QAccessible::updateAccessibility(&e);
 #endif
         emit toggled(b);
     }
