@@ -3,7 +3,7 @@
 ** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
-** This file is part of the QtGui module of the Qt Toolkit.
+** This file is part of the QtWidgets module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
@@ -429,7 +429,7 @@ void QLCDNumber::setDigitCount(int numDigits)
         bool doDisplay = d->ndigits == 0;
         if (numDigits == d->ndigits)             // no change
             return;
-        register int i;
+        int i;
         int dif;
         if (numDigits > d->ndigits) {            // expand
             dif = numDigits - d->ndigits;
@@ -468,8 +468,8 @@ int QLCDNumber::digitCount() const
 /*!
     \overload
 
-    Returns true if \a num is too big to be displayed in its entirety;
-    otherwise returns false.
+    Returns \c true if \a num is too big to be displayed in its entirety;
+    otherwise returns \c false.
 
     \sa display(), digitCount(), smallDecimalPoint()
 */
@@ -484,8 +484,8 @@ bool QLCDNumber::checkOverflow(int num) const
 
 
 /*!
-    Returns true if \a num is too big to be displayed in its entirety;
-    otherwise returns false.
+    Returns \c true if \a num is too big to be displayed in its entirety;
+    otherwise returns \c false.
 
     \sa display(), digitCount(), smallDecimalPoint()
 */
