@@ -3,7 +3,7 @@
 ** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
-** This file is part of the QtGui module of the Qt Toolkit.
+** This file is part of the QtWidgets module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
@@ -1016,19 +1016,6 @@ void QMainWindowLayout::toggleToolBarsVisible()
 
 #ifndef QT_NO_DOCKWIDGET
 
-static inline void validateDockWidgetArea(Qt::DockWidgetArea &area)
-{
-    switch (area) {
-    case Qt::LeftDockWidgetArea:
-    case Qt::RightDockWidgetArea:
-    case Qt::TopDockWidgetArea:
-    case Qt::BottomDockWidgetArea:
-        break;
-    default:
-        area = Qt::LeftDockWidgetArea;
-    }
-}
-
 static QInternal::DockPosition toDockPos(Qt::DockWidgetArea area)
 {
     switch (area) {
