@@ -3,7 +3,7 @@
 ** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
-** This file is part of the QtGui module of the Qt Toolkit.
+** This file is part of the QtWidgets module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
@@ -1211,7 +1211,7 @@ void QPlainTextEditPrivate::ensureViewportLayouted()
     Users are only able to undo or redo actions if this property is
     true, and if there is an action that can be undone (or redone).
 
-    By default, this property is true.
+    By default, this property is \c true.
 */
 
 /*!
@@ -2223,11 +2223,57 @@ void QPlainTextEdit::changeEvent(QEvent *e)
 #ifndef QT_NO_WHEELEVENT
 void QPlainTextEdit::wheelEvent(QWheelEvent *e)
 {
+    Q_D(QPlainTextEdit);
+    if (!(d->control->textInteractionFlags() & Qt::TextEditable)) {
+        if (e->modifiers() & Qt::ControlModifier) {
+            const int delta = e->delta();
+            if (delta < 0)
+                zoomOut();
+            else if (delta > 0)
+                zoomIn();
+            return;
+        }
+    }
     QAbstractScrollArea::wheelEvent(e);
     updateMicroFocus();
 }
 #endif
 
+/*!
+    \fn QPlainTextEdit::zoomIn(int range)
+
+    Zooms in on the text by making the base font size \a range
+    points larger and recalculating all font sizes to be the new size.
+    This does not change the size of any images.
+
+    \sa zoomOut()
+*/
+void QPlainTextEdit::zoomIn(int range)
+{
+    QFont f = font();
+    const int newSize = f.pointSize() + range;
+    if (newSize <= 0)
+        return;
+    f.setPointSize(newSize);
+    setFont(f);
+}
+
+/*!
+    \fn QPlainTextEdit::zoomOut(int range)
+
+    \overload
+
+    Zooms out on the text by making the base font size \a range points
+    smaller and recalculating all font sizes to be the new size. This
+    does not change the size of any images.
+
+    \sa zoomIn()
+*/
+void QPlainTextEdit::zoomOut(int range)
+{
+    zoomIn(-range);
+}
+
 #ifndef QT_NO_CONTEXTMENU
 /*!  This function creates the standard context menu which is shown
   when the user clicks on the line edit with the right mouse
@@ -2286,11 +2332,11 @@ QRect QPlainTextEdit::cursorRect() const
    As with many text editors, the plain text editor widget can be configured
    to insert or overwrite existing text with new text entered by the user.
 
-   If this property is true, existing text is overwritten, character-for-character
+   If this property is \c true, existing text is overwritten, character-for-character
    by new text; otherwise, text is inserted at the cursor position, displacing
    existing text.
 
-   By default, this property is false (new text does not overwrite existing text).
+   By default, this property is \c false (new text does not overwrite existing text).
 */
 
 bool QPlainTextEdit::overwriteMode() const
@@ -2389,7 +2435,7 @@ QMimeData *QPlainTextEdit::createMimeDataFromSelection() const
 }
 
 /*!
-    This function returns true if the contents of the MIME data object, specified
+    This function returns \c true if the contents of the MIME data object, specified
     by \a source, can be decoded and inserted into the document. It is called
     for example when during a drag operation the mouse enters this widget and it
     is necessary to determine whether it is possible to accept the drag.
@@ -2701,8 +2747,8 @@ void QPlainTextEdit::setCenterOnScroll(bool enabled)
 
 /*!
     Finds the next occurrence of the string, \a exp, using the given
-    \a options. Returns true if \a exp was found and changes the
-    cursor to select the match; otherwise returns false.
+    \a options. Returns \c true if \a exp was found and changes the
+    cursor to select the match; otherwise returns \c false.
 */
 bool QPlainTextEdit::find(const QString &exp, QTextDocument::FindFlags options)
 {
