@@ -3,7 +3,7 @@
 ** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
-** This file is part of the QtGui module of the Qt Toolkit.
+** This file is part of the QtWidgets module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
@@ -240,7 +240,7 @@ QWidget *QScrollArea::widget() const
     The \a widget becomes a child of the scroll area, and will be
     destroyed when the scroll area is deleted or when a new widget is
     set.
-    
+
     The widget's \l{QWidget::setAutoFillBackground()}{autoFillBackground}
     property will be set to \c{true}.
 
@@ -407,6 +407,18 @@ QSize QScrollArea::sizeHint() const
     return sz.boundedTo(QSize(36 * h, 24 * h));
 }
 
+/*!
+    \reimp
+ */
+QSize QScrollArea::viewportSizeHint() const
+{
+    Q_D(const QScrollArea);
+    if (d->widget) {
+        return d->resizable ? d->widget->sizeHint() : d->widget->size();
+    }
+    const int h = fontMetrics().height();
+    return QSize(6 * h, 4 * h);
+}
 
 
 /*!
@@ -499,6 +511,15 @@ void QScrollArea::ensureWidgetVisible(QWidget *childWidget, int xmargin, int yma
     \brief the alignment of the scroll area's widget
     \since 4.2
 
+    A valid alignment is a combination of the following flags:
+    \list
+    \li \c Qt::AlignLeft
+    \li \c Qt::AlignHCenter
+    \li \c Qt::AlignRight
+    \li \c Qt::AlignTop
+    \li \c Qt::AlignVCenter
+    \li \c Qt::AlignBottom
+    \endlist
     By default, the widget stays rooted to the top-left corner of the
     scroll area.
 */
