@@ -3,7 +3,7 @@
 ** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
-** This file is part of the QtGui module of the Qt Toolkit.
+** This file is part of the QtWidgets module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
@@ -44,8 +44,6 @@
 
 #include <QtWidgets/qabstractscrollarea.h>
 
-QT_BEGIN_HEADER
-
 QT_BEGIN_NAMESPACE
 
 
@@ -71,6 +69,7 @@ public:
     void setWidgetResizable(bool resizable);
 
     QSize sizeHint() const;
+
     bool focusNextPrevChild(bool next);
 
     Qt::Alignment alignment() const;
@@ -86,6 +85,8 @@ protected:
     void resizeEvent(QResizeEvent *);
     void scrollContentsBy(int dx, int dy);
 
+    QSize viewportSizeHint() const Q_DECL_OVERRIDE;
+
 private:
     Q_DECLARE_PRIVATE(QScrollArea)
     Q_DISABLE_COPY(QScrollArea)
@@ -95,6 +96,4 @@ private:
 
 QT_END_NAMESPACE
 
-QT_END_HEADER
-
 #endif // QSCROLLAREA_H
