@@ -3,7 +3,7 @@
 ** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
-** This file is part of the QtGui module of the Qt Toolkit.
+** This file is part of the QtWidgets module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
@@ -510,6 +510,11 @@ void QScrollBar::wheelEvent(QWheelEvent *event)
     Q_D(QScrollBar);
     if (d->scrollByDelta(event->orientation(), event->modifiers(), delta))
         event->accept();
+
+    if (event->phase() == Qt::ScrollBegin)
+        d->setTransient(false);
+    else if (event->phase() == Qt::ScrollEnd)
+        d->setTransient(true);
 }
 #endif
 
