@@ -3,7 +3,7 @@
 ** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
-** This file is part of the QtGui module of the Qt Toolkit.
+** This file is part of the QtWidgets module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
@@ -534,7 +534,7 @@ void QStatusBar::reformat()
 }
 
 /*!
-  
+
   Hides the normal status indications and displays the given \a
   message for the specified number of milli-seconds (\a{timeout}). If
   \a{timeout} is 0 (default), the \a {message} remains displayed until
@@ -551,8 +551,6 @@ void QStatusBar::showMessage(const QString &message, int timeout)
 {
     Q_D(QStatusBar);
 
-    d->tempItem = message;
-
     if (timeout > 0) {
         if (!d->timer) {
             d->timer = new QTimer(this);
@@ -563,6 +561,9 @@ void QStatusBar::showMessage(const QString &message, int timeout)
         delete d->timer;
         d->timer = 0;
     }
+    if (d->tempItem == message)
+        return;
+    d->tempItem = message;
 
     hideOrShow();
 }
@@ -744,7 +745,7 @@ bool QStatusBar::event(QEvent *e)
             }
         }
     }
-    
+
 // On Mac OS X Leopard it is possible to drag the window by clicking
 // on the tool bar on most applications.
 #ifndef Q_WS_MAC
@@ -755,7 +756,7 @@ bool QStatusBar::event(QEvent *e)
 
     // Enable drag-click only if the status bar is the status bar for a
     // QMainWindow with a unifed toolbar.
-    if (parent() == 0 || qobject_cast<QMainWindow *>(parent()) == 0 || 
+    if (parent() == 0 || qobject_cast<QMainWindow *>(parent()) == 0 ||
         qobject_cast<QMainWindow *>(parent())->unifiedTitleAndToolBarOnMac() == false )
         return QWidget::event(e);
 
