@@ -3,7 +3,7 @@
 ** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
-** This file is part of the QtGui module of the Qt Toolkit.
+** This file is part of the QtWidgets module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
@@ -172,6 +172,26 @@ QT_BEGIN_NAMESPACE
     \sa setTabsClosable()
 */
 
+/*!
+    \fn void QTabWidget::tabBarClicked(int index)
+
+    This signal is emitted when user clicks on a tab at an \a index.
+
+    \a index refers to the tab clicked, or -1 if no tab is under the cursor.
+
+    \since 5.2
+*/
+
+/*!
+    \fn void QTabWidget::tabBarDoubleClicked(int index)
+
+    This signal is emitted when the user double clicks on a tab at an \a index.
+
+    \a index is the index of a clicked tab, or -1 if no tab is under the cursor.
+
+    \since 5.2
+*/
+
 class QTabWidgetPrivate : public QWidgetPrivate
 {
     Q_DECLARE_PUBLIC(QTabWidget)
@@ -503,7 +523,7 @@ QIcon QTabWidget::tabIcon(int index) const
 }
 
 /*!
-    Returns true if the page at position \a index is enabled; otherwise returns false.
+    Returns \c true if the page at position \a index is enabled; otherwise returns \c false.
 
     \sa setTabEnabled(), QWidget::isEnabled()
 */
@@ -693,6 +713,10 @@ void QTabWidget::setTabBar(QTabBar* tb)
             this, SLOT(_q_showTab(int)));
     connect(d->tabs, SIGNAL(tabMoved(int,int)),
             this, SLOT(_q_tabMoved(int,int)));
+    connect(d->tabs, SIGNAL(tabBarClicked(int)),
+            this, SIGNAL(tabBarClicked(int)));
+    connect(d->tabs, SIGNAL(tabBarDoubleClicked(int)),
+            this, SIGNAL(tabBarDoubleClicked(int)));
     if (d->tabs->tabsClosable())
         connect(d->tabs, SIGNAL(tabCloseRequested(int)),
                 this, SIGNAL(tabCloseRequested(int)));
@@ -996,7 +1020,7 @@ void QTabWidget::setTabsClosable(bool closeable)
 
     \since 4.5
 
-    By default, this property is false;
+    By default, this property is \c false;
 */
 
 bool QTabWidget::isMovable() const
