@@ -3,7 +3,7 @@
 ** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
-** This file is part of the QtGui module of the Qt Toolkit.
+** This file is part of the QtWidgets module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
@@ -151,12 +151,18 @@ public:
 QString QTextBrowserPrivate::findFile(const QUrl &name) const
 {
     QString fileName;
-    if (name.scheme() == QLatin1String("qrc"))
+    if (name.scheme() == QLatin1String("qrc")) {
         fileName = QLatin1String(":/") + name.path();
-    else if (name.scheme().isEmpty())
+    } else if (name.scheme().isEmpty()) {
         fileName = name.path();
-    else
-        fileName = name.toLocalFile();
+    } else {
+#if defined(Q_OS_ANDROID)
+        if (name.scheme() == QLatin1String("assets"))
+            fileName = QLatin1String("assets:") + name.path();
+        else
+#endif
+            fileName = name.toLocalFile();
+    }
 
     if (QFileInfo(fileName).isAbsolute())
         return fileName;
@@ -217,13 +223,14 @@ void QTextBrowserPrivate::_q_activateAnchor(const QString &href)
     textOrSourceChanged = false;
 
 #ifndef QT_NO_DESKTOPSERVICES
-    if ((openExternalLinks
-         && url.scheme() != QLatin1String("file")
-         && url.scheme() != QLatin1String("qrc")
-         && !url.isRelative())
-        || (url.isRelative() && !currentURL.isRelative()
-            && currentURL.scheme() != QLatin1String("file")
-            && currentURL.scheme() != QLatin1String("qrc"))) {
+    bool isFileScheme =
+            url.scheme() == QLatin1String("file")
+#if defined(Q_OS_ANDROID)
+            || url.scheme() == QLatin1String("assets")
+#endif
+            || url.scheme() == QLatin1String("qrc");
+    if ((openExternalLinks && !isFileScheme && !url.isRelative())
+        || (url.isRelative() && !currentURL.isRelative() && !isFileScheme)) {
         QDesktopServices::openUrl(url);
         return;
     }
@@ -636,14 +643,14 @@ void QTextBrowserPrivate::restoreHistoryEntry(const HistoryEntry entry)
     \property QTextBrowser::readOnly
     \brief whether the text browser is read-only
 
-    By default, this property is true.
+    By default, this property is \c true.
 */
 
 /*!
     \property QTextBrowser::undoRedoEnabled
     \brief whether the text browser supports undo/redo operations
 
-    By default, this property is false.
+    By default, this property is \c false.
 */
 
 void QTextBrowserPrivate::init()
@@ -1096,7 +1103,7 @@ QVariant QTextBrowser::loadResource(int /*type*/, const QUrl &name)
 /*!
     \since 4.2
 
-    Returns true if the text browser can go backward in the document history
+    Returns \c true if the text browser can go backward in the document history
     using backward().
 
     \sa backwardAvailable(), backward()
@@ -1110,7 +1117,7 @@ bool QTextBrowser::isBackwardAvailable() const
 /*!
     \since 4.2
 
-    Returns true if the text browser can go forward in the document history
+    Returns \c true if the text browser can go forward in the document history
     using forward().
 
     \sa forwardAvailable(), forward()
