@@ -3,7 +3,7 @@
 ** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
-** This file is part of the QtGui module of the Qt Toolkit.
+** This file is part of the QtWidgets module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
@@ -182,7 +182,7 @@ void QTextEditPrivate::init(const QString &html)
     q->setFocusPolicy(Qt::WheelFocus);
     q->setAttribute(Qt::WA_KeyCompression);
     q->setAttribute(Qt::WA_InputMethodEnabled);
-
+    q->setInputMethodHints(Qt::ImhMultiLine);
 #ifndef QT_NO_CURSOR
     viewport->setCursor(Qt::IBeamCursor);
 #endif
@@ -445,6 +445,8 @@ void QTextEditPrivate::_q_ensureVisible(const QRectF &_rect)
     example, the documentTitle() function will return the text from
     within HTML \c{<title>} tags.
 
+    \note Zooming into HTML documents only works if the font-size is not set to a fixed size.
+
     \section1 Using QTextEdit as an Editor
 
     All the information about using QTextEdit as a display widget also
@@ -662,7 +664,7 @@ int QTextEdit::fontWeight() const
 }
 
 /*!
-    Returns true if the font of the current format is underlined; otherwise returns
+    Returns \c true if the font of the current format is underlined; otherwise returns
     false.
 
     \sa setFontUnderline()
@@ -674,7 +676,7 @@ bool QTextEdit::fontUnderline() const
 }
 
 /*!
-    Returns true if the font of the current format is italic; otherwise returns
+    Returns \c true if the font of the current format is italic; otherwise returns
     false.
 
     \sa setFontItalic()
@@ -770,6 +772,35 @@ QTextDocument *QTextEdit::document() const
     return d->control->document();
 }
 
+/*!
+    \since 5.2
+
+    \property QTextEdit::placeholderText
+    \brief the editor placeholder text
+
+    Setting this property makes the editor display a grayed-out
+    placeholder text as long as the document() is empty.
+
+    By default, this property contains an empty string.
+
+    \sa document()
+*/
+QString QTextEdit::placeholderText() const
+{
+    Q_D(const QTextEdit);
+    return d->placeholderText;
+}
+
+void QTextEdit::setPlaceholderText(const QString &placeholderText)
+{
+    Q_D(QTextEdit);
+    if (d->placeholderText != placeholderText) {
+        d->placeholderText = placeholderText;
+        if (d->control->document()->isEmpty())
+            d->viewport->update();
+    }
+}
+
 /*!
     Sets the visible \a cursor.
 */
@@ -1482,6 +1513,14 @@ void QTextEditPrivate::paint(QPainter *p, QPaintEvent *e)
 
     if (layout)
         layout->setViewport(QRect());
+
+    if (!placeholderText.isEmpty() && doc->isEmpty()) {
+        QColor col = control->palette().text().color();
+        col.setAlpha(128);
+        p->setPen(col);
+        const int margin = int(doc->documentMargin());
+        p->drawText(viewport->rect().adjusted(margin, margin, -margin, -margin), Qt::AlignTop | Qt::TextWordWrap, placeholderText);
+    }
 }
 
 /*! \fn void QTextEdit::paintEvent(QPaintEvent *event)
@@ -1868,11 +1907,11 @@ QString QTextEdit::anchorAt(const QPoint& pos) const
    As with many text editors, the text editor widget can be configured
    to insert or overwrite existing text with new text entered by the user.
 
-   If this property is true, existing text is overwritten, character-for-character
+   If this property is \c true, existing text is overwritten, character-for-character
    by new text; otherwise, text is inserted at the cursor position, displacing
    existing text.
 
-   By default, this property is false (new text does not overwrite existing text).
+   By default, this property is \c false (new text does not overwrite existing text).
 */
 
 bool QTextEdit::overwriteMode() const
@@ -2016,7 +2055,7 @@ QMimeData *QTextEdit::createMimeDataFromSelection() const
 }
 
 /*!
-    This function returns true if the contents of the MIME data object, specified
+    This function returns \c true if the contents of the MIME data object, specified
     by \a source, can be decoded and inserted into the document. It is called
     for example when during a drag operation the mouse enters this widget and it
     is necessary to determine whether it is possible to accept the drag and drop
@@ -2407,8 +2446,8 @@ void QTextEdit::setWordWrapMode(QTextOption::WrapMode mode)
 
 /*!
     Finds the next occurrence of the string, \a exp, using the given
-    \a options. Returns true if \a exp was found and changes the
-    cursor to select the match; otherwise returns false.
+    \a options. Returns \c true if \a exp was found and changes the
+    cursor to select the match; otherwise returns \c false.
 */
 bool QTextEdit::find(const QString &exp, QTextDocument::FindFlags options)
 {
