@@ -3,7 +3,7 @@
 ** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
-** This file is part of the QtGui module of the Qt Toolkit.
+** This file is part of the QtWidgets module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
@@ -75,7 +75,7 @@ public:
           allowedAreas(Qt::AllToolBarAreas), orientation(Qt::Horizontal),
           toolButtonStyle(Qt::ToolButtonIconOnly),
           layout(0), state(0)
-#ifdef Q_WS_MAC
+#ifdef Q_OS_OSX
         , macWindowDragging(false)
 #endif
     { }
@@ -107,7 +107,7 @@ public:
     };
     DragState *state;
 
-#ifdef Q_WS_MAC
+#ifdef Q_OS_OSX
     bool macWindowDragging;
     QPoint macWindowDragPressPosition;
 #endif
