@@ -3,7 +3,7 @@
 ** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
-** This file is part of the QtGui module of the Qt Toolkit.
+** This file is part of the QtWidgets module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
@@ -680,6 +680,10 @@ void QToolButton::showMenu()
         d->menuButtonDown = false;
         return; // no menu to show
     }
+    // prevent recursions spinning another event loop
+    if (d->menuButtonDown)
+        return;
+
 
     d->menuButtonDown = true;
     repaint();
