@@ -50,8 +50,6 @@
 #include <QtCore/qlist.h>
 #include <QtCore/qscopedpointer.h>
 
-QT_BEGIN_HEADER
-
 QT_BEGIN_NAMESPACE
 
 
@@ -119,8 +117,8 @@ private:
 class Q_XML_EXPORT QXmlAttributes
 {
 public:
-    QXmlAttributes() {}
-    virtual ~QXmlAttributes() {}
+    QXmlAttributes();
+    virtual ~QXmlAttributes();
 
     int index(const QString& qName) const;
     int index(QLatin1String qName) const;
@@ -367,8 +365,8 @@ public:
 class Q_XML_EXPORT QXmlDefaultHandler : public QXmlContentHandler, public QXmlErrorHandler, public QXmlDTDHandler, public QXmlEntityResolver, public QXmlLexicalHandler, public QXmlDeclHandler
 {
 public:
-    QXmlDefaultHandler() { }
-    virtual ~QXmlDefaultHandler() { }
+    QXmlDefaultHandler();
+    virtual ~QXmlDefaultHandler();
 
     void setDocumentLocator(QXmlLocator* locator);
     bool startDocument();
@@ -417,6 +415,4 @@ inline int QXmlAttributes::count() const
 
 QT_END_NAMESPACE
 
-QT_END_HEADER
-
 #endif // QXML_H
