@@ -64,9 +64,11 @@ private slots:
 #endif
 
     void qMessagePattern();
+    void qMessagePatternIf();
 
 private:
     QString m_appDir;
+    QStringList m_baseEnvironment;
 };
 
 static QtMsgType s_type;
@@ -98,6 +100,16 @@ void tst_qmessagehandler::initTestCase()
     m_appDir = QFINDTESTDATA("app");
     QVERIFY2(!m_appDir.isEmpty(), qPrintable(
         QString::fromLatin1("Couldn't find helper app dir starting from %1.").arg(QDir::currentPath())));
+
+#ifndef QT_NO_PROCESS
+    m_baseEnvironment = QProcess::systemEnvironment();
+    for (int i = 0; i < m_baseEnvironment.count(); ++i) {
+        if (m_baseEnvironment.at(i).startsWith("QT_MESSAGE_PATTERN=")) {
+            m_baseEnvironment.removeAt(i);
+            break;
+        }
+    }
+#endif // !QT_NO_PROCESS
 }
 
 void tst_qmessagehandler::cleanup()
@@ -631,13 +643,16 @@ void tst_qmessagehandler::cleanupFuncinfo()
 
 void tst_qmessagehandler::qMessagePattern()
 {
+#ifdef QT_NO_PROCESS
+    QSKIP("This test requires QProcess support");
+#else
     QProcess process;
     const QString appExe = m_appDir + "/app";
 
     //
     // test QT_MESSAGE_PATTERN
     //
-    QStringList environment = QProcess::systemEnvironment();
+    QStringList environment = m_baseEnvironment;
     // %{file} is tricky because of shadow builds
     environment.prepend("QT_MESSAGE_PATTERN=\"%{type} %{appname} %{line} %{function} %{message}\"");
     process.setEnvironment(environment);
@@ -651,15 +666,16 @@ void tst_qmessagehandler::qMessagePattern()
 //    qDebug() << output;
     QVERIFY(!output.isEmpty());
 
-    QVERIFY(output.contains("debug  45 T::T static constructor"));
+    QVERIFY(output.contains("debug  46 T::T static constructor"));
     //  we can't be sure whether the QT_MESSAGE_PATTERN is already destructed
     QVERIFY(output.contains("static destructor"));
-    QVERIFY(output.contains("debug tst_qlogging 56 main qDebug"));
-    QVERIFY(output.contains("warning tst_qlogging 57 main qWarning"));
-    QVERIFY(output.contains("critical tst_qlogging 58 main qCritical"));
-    QVERIFY(output.contains("debug tst_qlogging 62 main qDebug2"));
+    QVERIFY(output.contains("debug tst_qlogging 57 main qDebug"));
+    QVERIFY(output.contains("warning tst_qlogging 58 main qWarning"));
+    QVERIFY(output.contains("critical tst_qlogging 59 main qCritical"));
+    QVERIFY(output.contains("warning tst_qlogging 62 main qDebug with category "));
+    QVERIFY(output.contains("debug tst_qlogging 66 main qDebug2"));
 
-    environment = QProcess::systemEnvironment();
+    environment = m_baseEnvironment;
     environment.prepend("QT_MESSAGE_PATTERN=\"PREFIX: %{unknown} %{message}\"");
     process.setEnvironment(environment);
 
@@ -695,11 +711,81 @@ void tst_qmessagehandler::qMessagePattern()
     QByteArray expected = "static constructor\n"
             "[debug] qDebug\n"
             "[warning] qWarning\n"
-            "[critical] qCritical\n";
+            "[critical] qCritical\n"
+            "[warning] qDebug with category \n";
 #ifdef Q_OS_WIN
     output.replace("\r\n", "\n");
 #endif
     QCOMPARE(QString::fromLatin1(output), QString::fromLatin1(expected));
+#endif // !QT_NO_PROCESS
+}
+
+void tst_qmessagehandler::qMessagePatternIf()
+{
+#ifdef QT_NO_PROCESS
+    QSKIP("This test requires QProcess support");
+#else
+    QProcess process;
+    const QString appExe = m_appDir + "/app";
+
+    QStringList environment = m_baseEnvironment;
+    environment.prepend("QT_MESSAGE_PATTERN=\"[%{if-debug}D%{endif}%{if-warning}W%{endif}%{if-critical}C%{endif}%{if-fatal}F%{endif}] %{if-category}%{category}: %{endif}%{message}\"");
+    process.setEnvironment(environment);
+    process.start(appExe);
+    QVERIFY2(process.waitForStarted(), qPrintable(
+        QString::fromLatin1("Could not start %1: %2").arg(appExe, process.errorString())));
+    process.waitForFinished();
+
+    QByteArray output = process.readAllStandardError();
+    //    qDebug() << output;
+    QVERIFY(!output.isEmpty());
+    QVERIFY(!output.contains("QT_MESSAGE_PATTERN"));
+
+    QVERIFY(output.contains("[D] static constructor"));
+    //  we can't be sure whether the QT_MESSAGE_PATTERN is already destructed
+    QVERIFY(output.contains("static destructor"));
+    QVERIFY(output.contains("[D] qDebug"));
+    QVERIFY(output.contains("[W] qWarning"));
+    QVERIFY(output.contains("[C] qCritical"));
+    QVERIFY(output.contains("[W] category: qDebug with category"));
+    QVERIFY(output.contains("[D] qDebug2"));
+
+    //
+    // Tests some errors
+    //
+    environment = m_baseEnvironment;
+    environment.prepend("QT_MESSAGE_PATTERN=\"PREFIX: %{unknown} %{endif}  %{if-warning}\"");
+    process.setEnvironment(environment);
+
+    process.start(appExe);
+    QVERIFY2(process.waitForStarted(), qPrintable(
+        QString::fromLatin1("Could not start %1: %2").arg(appExe, process.errorString())));
+    process.waitForFinished();
+
+    output = process.readAllStandardError();
+    //     qDebug() << output;
+    QVERIFY(!output.isEmpty());
+    QVERIFY(output.contains("QT_MESSAGE_PATTERN: Unknown placeholder %{unknown}"));
+    QVERIFY(output.contains("QT_MESSAGE_PATTERN: %{endif} without an %{if-*}"));
+    QVERIFY(output.contains("QT_MESSAGE_PATTERN: missing %{endif}"));
+
+
+    environment = m_baseEnvironment;
+    environment.prepend("QT_MESSAGE_PATTERN=\"A %{if-debug}DEBUG%{if-warning}WARNING%{endif} %{message}  \"");
+    process.setEnvironment(environment);
+
+    process.start(appExe);
+    QVERIFY2(process.waitForStarted(), qPrintable(
+        QString::fromLatin1("Could not start %1: %2").arg(appExe, process.errorString())));
+    process.waitForFinished();
+
+    output = process.readAllStandardError();
+    //     qDebug() << output;
+    QVERIFY(!output.isEmpty());
+    QVERIFY(output.contains("QT_MESSAGE_PATTERN: %{if-*} cannot be nested"));
+    QVERIFY(output.contains("A DEBUG qDebug"));
+    QVERIFY(output.contains("A  qWarning"));
+#endif // !QT_NO_PROCESS
 }
 
 QTEST_MAIN(tst_qmessagehandler)
