@@ -79,6 +79,8 @@ private slots:
     void QTBUG2331();
     void QTBUG2331_data() { basicTest_data(); }
 
+    void signalsEmittedAfterFileMoved();
+
 private:
     QString m_tempDirPattern;
 };
@@ -94,20 +96,31 @@ tst_QFileSystemWatcher::tst_QFileSystemWatcher()
 void tst_QFileSystemWatcher::basicTest_data()
 {
     QTest::addColumn<QString>("backend");
-    QTest::newRow("native backend") << "native";
-    QTest::newRow("poller backend") << "poller";
+    QTest::addColumn<QString>("testFileName");
+    const QString testFile = QStringLiteral("testfile.txt");
+    // QTBUG-31341: Test the UNICODE capabilities; ensure no QString::toLower()
+    // is in the code path since that will lower case for example
+    // LATIN_CAPITAL_LETTER_I_WITH_DOT_ABOVE with context, whereas the Windows file
+    // system will not.
+    const QString specialCharacterFile =
+        QString(QChar(ushort(0x130))) // LATIN_CAPITAL_LETTER_I_WITH_DOT_ABOVE
+        + QChar(ushort(0x00DC)) // LATIN_CAPITAL_LETTER_U_WITH_DIAERESIS
+        + QStringLiteral(".txt");
+
+    QTest::newRow("native backend-testfile") << "native" << testFile;
+    QTest::newRow("poller backend-testfile") << "poller" << testFile;
+    QTest::newRow("native backend-specialchars") << "native" << specialCharacterFile;
 }
 
 void tst_QFileSystemWatcher::basicTest()
 {
     QFETCH(QString, backend);
+    QFETCH(QString, testFileName);
 
     // create test file
     QTemporaryDir temporaryDirectory(m_tempDirPattern);
     QVERIFY(temporaryDirectory.isValid());
-    QFile testFile(temporaryDirectory.path() + QStringLiteral("/testfile.txt"));
-    testFile.setPermissions(QFile::ReadOwner | QFile::WriteOwner);
-    testFile.remove();
+    QFile testFile(temporaryDirectory.path() + QLatin1Char('/') + testFileName);
     QVERIFY(testFile.open(QIODevice::WriteOnly | QIODevice::Truncate));
     testFile.write(QByteArray("hello"));
     testFile.close();
@@ -457,7 +470,12 @@ void tst_QFileSystemWatcher::watchFileAndItsDirectory()
 
     timer.start(3000);
     eventLoop.exec();
-    QCOMPARE(fileChangedSpy.count(), 0);
+    int fileChangedSpyCount = fileChangedSpy.count();
+#ifdef Q_OS_WIN64
+    if (fileChangedSpyCount != 0)
+        QEXPECT_FAIL("", "See QTBUG-30943", Continue);
+#endif
+    QCOMPARE(fileChangedSpyCount, 0);
 #ifdef Q_OS_WINCE
     QEXPECT_FAIL("poller", "Directory does not get updated on file removal(See #137910)", Abort);
 #endif
@@ -580,5 +598,84 @@ void tst_QFileSystemWatcher::QTBUG2331()
     QCOMPARE(watcher.directories(), QStringList());
 }
 
+class SignalReceiver : public QObject
+{
+    Q_OBJECT
+public:
+    SignalReceiver(const QDir &moveSrcDir,
+                   const QString &moveDestination,
+                   QFileSystemWatcher *watcher,
+                   QObject *parent = 0)
+        : QObject(parent),
+          added(false),
+          moveSrcDir(moveSrcDir),
+          moveDestination(QDir(moveDestination)),
+          watcher(watcher)
+    {}
+
+public slots:
+    void fileChanged(const QString &path)
+    {
+        QFileInfo finfo(path);
+
+        QCOMPARE(finfo.absolutePath(), moveSrcDir.absolutePath());
+
+        if (!added) {
+            foreach (const QFileInfo &fi, moveDestination.entryInfoList(QDir::Files | QDir::NoSymLinks))
+                watcher->addPath(fi.absoluteFilePath());
+            added = true;
+        }
+    }
+
+private:
+    bool added;
+    QDir moveSrcDir;
+    QDir moveDestination;
+    QFileSystemWatcher *watcher;
+};
+
+// regression test for QTBUG-33211.
+// using inotify backend if a file is moved and then added to the watcher
+// before all the fileChanged signals are emitted the remaining signals are
+// emitted with the destination path instead of the starting path
+void tst_QFileSystemWatcher::signalsEmittedAfterFileMoved()
+{
+    QTemporaryDir temporaryDirectory(m_tempDirPattern);
+    QVERIFY(temporaryDirectory.isValid());
+    QDir testDir(temporaryDirectory.path());
+    QVERIFY(testDir.mkdir("movehere"));
+    QString movePath = testDir.filePath("movehere");
+
+    for (int i = 0; i < 10; i++) {
+        QFile f(testDir.filePath(QString("test%1.txt").arg(i)));
+        QVERIFY(f.open(QIODevice::WriteOnly));
+        f.write(QByteArray("i am ") + QByteArray::number(i));
+        f.close();
+    }
+
+    QFileSystemWatcher watcher;
+    QVERIFY(watcher.addPath(testDir.path()));
+    QVERIFY(watcher.addPath(movePath));
+
+    // add files to watcher
+    QFileInfoList files = testDir.entryInfoList(QDir::Files | QDir::NoSymLinks);
+    foreach (const QFileInfo &finfo, files)
+        QVERIFY(watcher.addPath(finfo.absoluteFilePath()));
+
+    // create the signal receiver
+    SignalReceiver signalReceiver(testDir, movePath, &watcher);
+    connect(&watcher, SIGNAL(fileChanged(QString)), &signalReceiver, SLOT(fileChanged(QString)));
+
+    // watch signals
+    QSignalSpy changedSpy(&watcher, SIGNAL(fileChanged(QString)));
+    QVERIFY(changedSpy.isValid());
+
+    // move files to second directory
+    foreach (const QFileInfo &finfo, files)
+        QVERIFY(testDir.rename(finfo.fileName(), QString("movehere/%2").arg(finfo.fileName())));
+
+    QTRY_COMPARE(changedSpy.count(), 10);
+}
+
 QTEST_MAIN(tst_QFileSystemWatcher)
 #include "tst_qfilesystemwatcher.moc"
