@@ -81,6 +81,8 @@ private slots:
     void getSetCheck();
     void constructing();
     void simpleStart();
+    void startWithOpen();
+    void startWithOldOpen();
     void execute();
     void startDetached();
     void crashTest();
@@ -91,6 +93,7 @@ private slots:
     void echoTest2();
 #ifdef Q_OS_WIN
     void echoTestGui();
+    void testSetNamedPipeHandleState();
     void batFiles_data();
     void batFiles();
 #endif
@@ -105,8 +108,8 @@ private slots:
     void softExitInSlots_data();
     void softExitInSlots();
     void mergedChannels();
+    void forwardedChannels_data();
     void forwardedChannels();
-    void forwardedChannelsOutput();
     void atEnd();
     void atEnd2();
     void waitForFinishedWithTimeout();
@@ -120,6 +123,7 @@ private slots:
     void setStandardInputFile();
     void setStandardOutputFile_data();
     void setStandardOutputFile();
+    void setStandardOutputFile2();
     void setStandardOutputProcess_data();
     void setStandardOutputProcess();
     void removeFileWhileProcessIsRunning();
@@ -151,6 +155,8 @@ private slots:
     void invalidProgramString_data();
     void invalidProgramString();
     void onlyOneStartedSignal();
+    void finishProcessBeforeReadingDone();
+    void waitForStartedWithoutStart();
 
     // keep these at the end, since they use lots of processes and sometimes
     // caused obscure failures to occur in tests that followed them (esp. on the Mac)
@@ -265,15 +271,7 @@ void tst_QProcess::simpleStart()
         QCOMPARE(process->state(), QProcess::Running);
     QVERIFY2(process->waitForStarted(5000), qPrintable(process->errorString()));
     QCOMPARE(process->state(), QProcess::Running);
-#if defined(Q_OS_WINCE)
-    // Note: This actually seems incorrect, it will only exit the while loop when finishing fails
-    while (process->waitForFinished(5000))
-    { }
-#else
-    while (process->waitForReadyRead(5000))
-    { }
-#endif
-    QCOMPARE(int(process->state()), int(QProcess::NotRunning));
+    QTRY_COMPARE(process->state(), QProcess::NotRunning);
 
     delete process;
     process = 0;
@@ -283,6 +281,44 @@ void tst_QProcess::simpleStart()
     QCOMPARE(qvariant_cast<QProcess::ProcessState>(spy.at(1).at(0)), QProcess::Running);
     QCOMPARE(qvariant_cast<QProcess::ProcessState>(spy.at(2).at(0)), QProcess::NotRunning);
 }
+
+//-----------------------------------------------------------------------------
+void tst_QProcess::startWithOpen()
+{
+    QProcess p;
+    QTest::ignoreMessage(QtWarningMsg, "QProcess::start: program not set");
+    QCOMPARE(p.open(QIODevice::ReadOnly), false);
+
+    p.setProgram("testProcessNormal/testProcessNormal");
+    QCOMPARE(p.program(), QString("testProcessNormal/testProcessNormal"));
+
+    p.setArguments(QStringList() << "arg1" << "arg2");
+    QCOMPARE(p.arguments().size(), 2);
+
+    QVERIFY(p.open(QIODevice::ReadOnly));
+    QCOMPARE(p.openMode(), QIODevice::ReadOnly);
+    QVERIFY(p.waitForFinished(5000));
+}
+
+//-----------------------------------------------------------------------------
+void tst_QProcess::startWithOldOpen()
+{
+    // similar to the above, but we start with start() actually
+    // while open() is overridden to call QIODevice::open().
+    // This tests the BC requirement that "it works with the old implementation"
+    class OverriddenOpen : public QProcess
+    {
+    public:
+        virtual bool open(OpenMode mode) Q_DECL_OVERRIDE
+        { return QIODevice::open(mode); }
+    };
+
+    OverriddenOpen p;
+    p.start("testProcessNormal/testProcessNormal");
+    QVERIFY(p.waitForStarted(5000));
+    QVERIFY(p.waitForFinished(5000));
+}
+
 //-----------------------------------------------------------------------------
 void tst_QProcess::execute()
 {
@@ -503,7 +539,6 @@ void tst_QProcess::echoTest2()
 #endif
 
 #if defined(Q_OS_WIN) && !defined(Q_OS_WINCE)
-//Batch files are not supported on Winfows CE
 // Reading and writing to a process is not supported on Qt/CE
 //-----------------------------------------------------------------------------
 void tst_QProcess::echoTestGui()
@@ -521,11 +556,22 @@ void tst_QProcess::echoTestGui()
     QCOMPARE(process.readAllStandardOutput(), QByteArray("Hello"));
     QCOMPARE(process.readAllStandardError(), QByteArray("Hello"));
 }
+
+void tst_QProcess::testSetNamedPipeHandleState()
+{
+    QProcess process;
+    process.setProcessChannelMode(QProcess::SeparateChannels);
+    process.start("testSetNamedPipeHandleState/testSetNamedPipeHandleState");
+    QVERIFY2(process.waitForStarted(5000), qPrintable(process.errorString()));
+    QVERIFY(process.waitForFinished(5000));
+    QCOMPARE(process.exitCode(), 0);
+    QCOMPARE(process.exitStatus(), QProcess::NormalExit);
+}
 #endif // !Q_OS_WINCE && Q_OS_WIN
 
 //-----------------------------------------------------------------------------
 #if defined(Q_OS_WIN) && !defined(Q_OS_WINCE)
-//Batch files are not supported on Winfows CE
+// Batch files are not supported on Windows CE
 void tst_QProcess::batFiles_data()
 {
     QTest::addColumn<QString>("batFile");
@@ -1055,36 +1101,55 @@ void tst_QProcess::mergedChannels()
 //-----------------------------------------------------------------------------
 #ifndef Q_OS_WINCE
 // Reading and writing to a process is not supported on Qt/CE
+
+void tst_QProcess::forwardedChannels_data()
+{
+    QTest::addColumn<int>("mode");
+    QTest::addColumn<int>("inmode");
+    QTest::addColumn<QByteArray>("outdata");
+    QTest::addColumn<QByteArray>("errdata");
+
+    QTest::newRow("separate") << int(QProcess::SeparateChannels) << int(QProcess::ManagedInputChannel)
+                              << QByteArray() << QByteArray();
+    QTest::newRow("forwarded") << int(QProcess::ForwardedChannels) << int(QProcess::ManagedInputChannel)
+                               << QByteArray("forwarded") << QByteArray("forwarded");
+    QTest::newRow("stdout") << int(QProcess::ForwardedOutputChannel) << int(QProcess::ManagedInputChannel)
+                            << QByteArray("forwarded") << QByteArray();
+    QTest::newRow("stderr") << int(QProcess::ForwardedErrorChannel) << int(QProcess::ManagedInputChannel)
+                            << QByteArray() << QByteArray("forwarded");
+    QTest::newRow("fwdinput") << int(QProcess::ForwardedErrorChannel) << int(QProcess::ForwardedInputChannel)
+                            << QByteArray() << QByteArray("input");
+}
+
 void tst_QProcess::forwardedChannels()
 {
+    QFETCH(int, mode);
+    QFETCH(int, inmode);
+    QFETCH(QByteArray, outdata);
+    QFETCH(QByteArray, errdata);
+
     QProcess process;
-    process.setReadChannelMode(QProcess::ForwardedChannels);
-    QCOMPARE(process.readChannelMode(), QProcess::ForwardedChannels);
-
-    process.start("testProcessEcho2/testProcessEcho2");
-
+    process.start("testForwarding/testForwarding", QStringList() << QString::number(mode) << QString::number(inmode));
     QVERIFY(process.waitForStarted(5000));
-    QCOMPARE(process.write("forwarded\n"), qlonglong(10));
-    QVERIFY(!process.waitForReadyRead(250));
-    QCOMPARE(process.bytesAvailable(), qlonglong(0));
-
+    QCOMPARE(process.write("input"), 5);
     process.closeWriteChannel();
     QVERIFY(process.waitForFinished(5000));
-}
-#endif
-
-#ifndef Q_OS_WINCE
-// Reading and writing to a process is not supported on Qt/CE
-void tst_QProcess::forwardedChannelsOutput()
-{
-    QProcess process;
-    process.start("testForwarding/testForwarding");
-    QVERIFY(process.waitForStarted(5000));
-    QVERIFY(process.waitForFinished(5000));
-    QVERIFY(!process.exitCode());
-    QByteArray data = process.readAll();
-    QVERIFY(!data.isEmpty());
-    QVERIFY(data.contains("forwarded"));
+    const char *err;
+    switch (process.exitCode()) {
+    case 0: err = "ok"; break;
+    case 1: err = "processChannelMode is wrong"; break;
+    case 11: err = "inputChannelMode is wrong"; break;
+    case 2: err = "failed to start"; break;
+    case 3: err = "failed to write"; break;
+    case 4: err = "did not finish"; break;
+    case 5: err = "unexpected stdout"; break;
+    case 6: err = "unexpected stderr"; break;
+    case 13: err = "parameter error"; break;
+    default: err = "unknown exit code"; break;
+    }
+    QVERIFY2(!process.exitCode(), err);
+    QCOMPARE(process.readAllStandardOutput(), outdata);
+    QCOMPARE(process.readAllStandardError(), errdata);
 }
 #endif
 
@@ -1256,7 +1321,6 @@ void tst_QProcess::waitForBytesWrittenInABytesWrittenSlot()
     process->start("testProcessEcho/testProcessEcho");
     QVERIFY(process->waitForStarted(5000));
 
-    qRegisterMetaType<qint64>("qint64");
     QSignalSpy spy(process, SIGNAL(bytesWritten(qint64)));
     QVERIFY(spy.isValid());
     process->write("f");
@@ -1835,6 +1899,13 @@ void tst_QProcess::setStandardInputFile()
         QByteArray all = process.readAll();
     QCOMPARE(all.size(), int(sizeof data) - 1); // testProcessEcho drops the ending \0
     QVERIFY(all == data);
+
+    QProcess process2;
+    process2.setStandardInputFile(QProcess::nullDevice());
+    process2.start("testProcessEcho/testProcessEcho");
+    QPROCESS_VERIFY(process2, waitForFinished());
+    all = process2.readAll();
+    QCOMPARE(all.size(), 0);
 }
 #endif
 
@@ -1869,6 +1940,23 @@ void tst_QProcess::setStandardOutputFile_data()
                                    << true;
 }
 
+//-----------------------------------------------------------------------------
+#ifndef Q_OS_WINCE
+void tst_QProcess::setStandardOutputFile2()
+{
+    static const char testdata[] = "Test data.";
+
+    QProcess process;
+    process.setStandardOutputFile(QProcess::nullDevice());
+    process.start("testProcessEcho2/testProcessEcho2");
+    process.write(testdata, sizeof testdata);
+    QPROCESS_VERIFY(process,waitForFinished());
+    QVERIFY(!process.bytesAvailable());
+
+    QVERIFY(!QFileInfo(QProcess::nullDevice()).isFile());
+}
+#endif
+
 void tst_QProcess::setStandardOutputFile()
 {
     static const char data[] = "Original data. ";
@@ -2149,6 +2237,45 @@ void tst_QProcess::onlyOneStartedSignal()
     QCOMPARE(spyFinished.count(), 1);
 }
 
+//-----------------------------------------------------------------------------
+
+class BlockOnReadStdOut : public QObject
+{
+    Q_OBJECT
+public:
+    BlockOnReadStdOut(QProcess *process)
+    {
+        connect(process, SIGNAL(readyReadStandardOutput()), SLOT(block()));
+    }
+
+public slots:
+    void block()
+    {
+        QThread::sleep(1);
+    }
+};
+
+void tst_QProcess::finishProcessBeforeReadingDone()
+{
+    QProcess process;
+    BlockOnReadStdOut blocker(&process);
+    QEventLoop loop;
+    connect(&process, SIGNAL(finished(int)), &loop, SLOT(quit()));
+    process.start("testProcessOutput/testProcessOutput");
+    QVERIFY(process.waitForStarted());
+    loop.exec();
+    QStringList lines = QString::fromLocal8Bit(process.readAllStandardOutput()).split(
+            QRegExp(QStringLiteral("[\r\n]")), QString::SkipEmptyParts);
+    QVERIFY(!lines.isEmpty());
+    QCOMPARE(lines.last(), QStringLiteral("10239 -this is a number"));
+}
+
+void tst_QProcess::waitForStartedWithoutStart()
+{
+    QProcess process;
+    QVERIFY(!process.waitForStarted(5000));
+}
+
 #endif //QT_NO_PROCESS
 
 QTEST_MAIN(tst_QProcess)
