@@ -43,10 +43,6 @@
 #include <QObject>
 #include <QProcessEnvironment>
 
-#ifdef QT_NO_PROCESS
-QTEST_NOOP_MAIN
-#else
-
 class tst_QProcessEnvironment: public QObject
 {
     Q_OBJECT
@@ -322,4 +318,3 @@ void tst_QProcessEnvironment::putenv()
 QTEST_MAIN(tst_QProcessEnvironment)
 
 #include "tst_qprocessenvironment.moc"
-#endif
