@@ -56,10 +56,11 @@
 #define STRINGPREP_BIDI_LEADTRAIL_NOT_RAL 5
 
 struct ushortarray {
-    ushortarray(unsigned short *array = 0)
+    ushortarray() {}
+    template <size_t N>
+    ushortarray(unsigned short (&array)[N])
     {
-        if (array)
-            memcpy(points, array, sizeof(points));
+        memcpy(points, array, N*sizeof(unsigned short));
     }
 
     unsigned short points[100];
@@ -351,7 +352,7 @@ void tst_QUrlInternal::nameprep_testsuite_data()
         << QString() << 0 << 0;
 
     QTest::newRow("Non-ASCII multibyte space character U+1680")
-        << QString::fromUtf8("\xE1\x9A\x80")
+        << QString::fromUtf8("x\xE1\x9A\x80x")
         << QString()
         << QString("Nameprep") << 0 << STRINGPREP_CONTAINS_PROHIBITED;
 
@@ -375,13 +376,18 @@ void tst_QUrlInternal::nameprep_testsuite_data()
         << QString::fromUtf8("\x10\x7F")
         << QString() << 0 << 0;
 
+    QTest::newRow("Non-ASCII 8bit control character U+0080")
+        << QString::fromUtf8("x\xC2\x80x")
+        << QString()
+        << QString("Nameprep") << 0 << STRINGPREP_CONTAINS_PROHIBITED;
+
     QTest::newRow("Non-ASCII 8bit control character U+0085")
-        << QString::fromUtf8("\xC2\x85")
+        << QString::fromUtf8("x\xC2\x85x")
         << QString()
         << QString("Nameprep") << 0 << STRINGPREP_CONTAINS_PROHIBITED;
 
     QTest::newRow("Non-ASCII multibyte control character U+180E")
-        << QString::fromUtf8("\xE1\xA0\x8E")
+        << QString::fromUtf8("x\xE1\xA0\x8Ex")
         << QString()
         << QString("Nameprep") << 0 << STRINGPREP_CONTAINS_PROHIBITED;
 
@@ -391,47 +397,47 @@ void tst_QUrlInternal::nameprep_testsuite_data()
         << QString() << 0 << 0;
 
     QTest::newRow("Non-ASCII control character U+1D175")
-        << QString::fromUtf8("\xF0\x9D\x85\xB5")
+        << QString::fromUtf8("x\xF0\x9D\x85\xB5x")
         << QString()
         << QString("Nameprep") << 0 << STRINGPREP_CONTAINS_PROHIBITED;
 
     QTest::newRow("Plane 0 private use character U+F123")
-        << QString::fromUtf8("\xEF\x84\xA3")
+        << QString::fromUtf8("x\xEF\x84\xA3x")
         << QString()
         << QString("Nameprep") << 0 << STRINGPREP_CONTAINS_PROHIBITED;
 
     QTest::newRow("Plane 15 private use character U+F1234")
-        << QString::fromUtf8("\xF3\xB1\x88\xB4")
+        << QString::fromUtf8("x\xF3\xB1\x88\xB4x")
         << QString()
         << QString("Nameprep") << 0 << STRINGPREP_CONTAINS_PROHIBITED;
 
     QTest::newRow("Plane 16 private use character U+10F234")
-        << QString::fromUtf8("\xF4\x8F\x88\xB4")
+        << QString::fromUtf8("x\xF4\x8F\x88\xB4x")
         << QString()
         << QString("Nameprep") << 0 << STRINGPREP_CONTAINS_PROHIBITED;
 
     QTest::newRow("Non-character code point U+8FFFE")
-        << QString::fromUtf8("\xF2\x8F\xBF\xBE")
+        << QString::fromUtf8("x\xF2\x8F\xBF\xBEx")
         << QString()
         << QString("Nameprep") << 0 << STRINGPREP_CONTAINS_PROHIBITED;
 
     QTest::newRow("Non-character code point U+10FFFF")
-        << QString::fromUtf8("\xF4\x8F\xBF\xBF")
+        << QString::fromUtf8("x\xF4\x8F\xBF\xBFx")
         << QString()
         << QString("Nameprep") << 0 << STRINGPREP_CONTAINS_PROHIBITED;
 
     QTest::newRow("Surrogate code U+DF42")
-        << QString::fromUtf8("\xED\xBD\x82")
+        << QString::fromUtf8("x\xED\xBD\x82x")
         << QString()
         << QString("Nameprep") << 0 <<  STRINGPREP_CONTAINS_PROHIBITED;
 
     QTest::newRow("Non-plain text character U+FFFD")
-        << QString::fromUtf8("\xEF\xBF\xBD")
+        << QString::fromUtf8("x\xEF\xBF\xBDx")
         << QString()
         << QString("Nameprep") << 0 << STRINGPREP_CONTAINS_PROHIBITED;
 
     QTest::newRow("Ideographic description character U+2FF5")
-        << QString::fromUtf8("\xE2\xBF\xB5")
+        << QString::fromUtf8("x\xE2\xBF\xB5x")
         << QString()
         << QString("Nameprep") << 0 << STRINGPREP_CONTAINS_PROHIBITED;
 
@@ -441,22 +447,22 @@ void tst_QUrlInternal::nameprep_testsuite_data()
         << QString() << 0 << 0;
 
     QTest::newRow("Left-to-right mark U+200E")
-        << QString::fromUtf8("\xE2\x80\x8E")
-        << QString::fromUtf8("\xCC\x81")
+        << QString::fromUtf8("x\xE2\x80\x8Ex")
+        << QString()
         << QString("Nameprep") << 0 << STRINGPREP_CONTAINS_PROHIBITED;
 
     QTest::newRow("Deprecated U+202A")
-        << QString::fromUtf8("\xE2\x80\xAA")
-        << QString::fromUtf8("\xCC\x81")
+        << QString::fromUtf8("x\xE2\x80\xAA")
+        << QString()
         << QString("Nameprep") << 0 << STRINGPREP_CONTAINS_PROHIBITED;
 
     QTest::newRow("Language tagging character U+E0001")
-        << QString::fromUtf8("\xF3\xA0\x80\x81")
-        << QString::fromUtf8("\xCC\x81")
+        << QString::fromUtf8("x\xF3\xA0\x80\x81x")
+        << QString()
         << QString("Nameprep") << 0 << STRINGPREP_CONTAINS_PROHIBITED;
 
     QTest::newRow("Language tagging character U+E0042")
-        << QString::fromUtf8("\xF3\xA0\x81\x82")
+        << QString::fromUtf8("x\xF3\xA0\x81\x82x")
         << QString()
         << QString("Nameprep") << 0 << STRINGPREP_CONTAINS_PROHIBITED;
 
@@ -512,12 +518,6 @@ void tst_QUrlInternal::nameprep_testsuite()
     QFETCH(QString, out);
     QFETCH(QString, profile);
 
-    QEXPECT_FAIL("Left-to-right mark U+200E",
-                 "Investigate further", Continue);
-    QEXPECT_FAIL("Deprecated U+202A",
-                 "Investigate further", Continue);
-    QEXPECT_FAIL("Language tagging character U+E0001",
-                 "Investigate further", Continue);
     qt_nameprep(&in, 0);
     QCOMPARE(in, out);
 }
@@ -549,9 +549,9 @@ void tst_QUrlInternal::nameprep_highcodes_data()
             << QString() << 0 << 0;
     }
     {
-        QChar st[] = { 'D', 0xdb40, 0xdc20, 'o', 0xd834, 0xdd7a, '\'', 0x2060, 'h' };
+        QChar st[] = { 'D', 'o', '\'', 0x2060, 'h' };
         QChar se[] = { 'd', 'o', '\'', 'h' };
-        QTest::newRow("highcodes (D, U+E0020, o, U+1D17A, ', U+2060, h)")
+        QTest::newRow("highcodes (D, o, ', U+2060, h)")
             << QString(st, sizeof(st)/sizeof(st[0]))
             << QString(se, sizeof(se)/sizeof(se[0]))
             << QString() << 0 << 0;
@@ -649,6 +649,20 @@ void tst_QUrlInternal::ace_testsuite_data()
         << "xn----rmckbbajlc6dj7bxne2c.xn--wgbh1c"
         << "."
         << egyptianIDN;
+
+    QString russianIDN = QString::fromUtf8("\321\217\320\275\320\264\320\265\320\272\321\201.\321\200\321\204");
+    QTest::newRow("russian-tld-ace")
+        << "xn--d1acpjx3f.xn--p1ai"
+        << "xn--d1acpjx3f.xn--p1ai"
+        << "."
+        << russianIDN;
+
+    QString taiwaneseIDN = QString::fromUtf8("\345\217\260\345\214\227\346\214\211\346\221\251.\345\217\260\347\201\243");
+    QTest::newRow("taiwanese-tld-ace")
+        << "xn--djrptm67aikb.xn--kpry57d"
+        << "xn--djrptm67aikb.xn--kpry57d"
+        << "."
+        << taiwaneseIDN;
 }
 
 void tst_QUrlInternal::ace_testsuite()
@@ -811,12 +825,20 @@ void tst_QUrlInternal::correctEncodedMistakes()
     QFETCH(QString, expected);
 
     // prepend some data to be sure that it remains there
-    QString output = QTest::currentDataTag();
-    expected.prepend(output);
+    QString dataTag = QTest::currentDataTag();
+    QString output = dataTag;
 
     if (!qt_urlRecode(output, input.constData(), input.constData() + input.length(), 0))
         output += input;
-    QCOMPARE(output, expected);
+    QCOMPARE(output, dataTag + expected);
+
+    // now try the full decode mode
+    output = dataTag;
+    QString expected2 = QUrl::fromPercentEncoding(expected.toLatin1());
+
+    if (!qt_urlRecode(output, input.constData(), input.constData() + input.length(), QUrl::FullyDecoded))
+        output += input;
+    QCOMPARE(output, dataTag + expected2);
 }
 
 static void addUtf8Data(const char *name, const char *data)
@@ -942,8 +964,10 @@ void tst_QUrlInternal::encodingRecode_data()
     addUtf8Data("utf8-string-2", "\xDF\xBF\xE0\xA0\x80""A");
     addUtf8Data("utf8-string-3", "\xE0\xA0\x80\xDF\xBF...");
 
+    QTest::newRow("encode-unicode-noncharacter") << QString(QChar(0xffff)) << F(QUrl::FullyEncoded) << "%EF%BF%BF";
+    QTest::newRow("decode-unicode-noncharacter") << QString(QChar(0xffff)) << F(QUrl::PrettyDecoded) << QString::fromUtf8("\xEF\xBF\xBF");
+
     // special cases: stuff we can encode, but not decode
-    QTest::newRow("unicode-noncharacter") << QString(QChar(0xffff)) << F(QUrl::FullyEncoded) << "%EF%BF%BF";
     QTest::newRow("unicode-lo-surrogate") << QString(QChar(0xD800)) << F(QUrl::FullyEncoded) << "%ED%A0%80";
     QTest::newRow("unicode-hi-surrogate") << QString(QChar(0xDC00)) << F(QUrl::FullyEncoded) << "%ED%B0%80";
 
@@ -989,9 +1013,6 @@ void tst_QUrlInternal::encodingRecodeInvalidUtf8_data()
     extern void loadInvalidUtf8Rows();
     loadInvalidUtf8Rows();
 
-    extern void loadNonCharactersRows();
-    loadNonCharactersRows();
-
     QTest::newRow("utf8-mix-4") << QByteArray("\xE0.A2\x80");
     QTest::newRow("utf8-mix-5") << QByteArray("\xE0\xA2.80");
     QTest::newRow("utf8-mix-6") << QByteArray("\xE0\xA2\x33");
@@ -1014,6 +1035,15 @@ void tst_QUrlInternal::encodingRecodeInvalidUtf8()
     if (!qt_urlRecode(output, input.constData(), input.constData() + input.length(), QUrl::FullyEncoded))
         output += input;
     QCOMPARE(output, QTest::currentDataTag() + input);
+
+    // verify for security reasons that all bad UTF-8 data got replaced by QChar::ReplacementCharacter
+    output = QTest::currentDataTag();
+    if (!qt_urlRecode(output, input.constData(), input.constData() + input.length(), QUrl::FullyEncoded))
+        output += input;
+    for (int i = strlen(QTest::currentDataTag()); i < output.length(); ++i) {
+        QVERIFY2(output.at(i).unicode() < 0x80 || output.at(i) == QChar::ReplacementCharacter,
+                 qPrintable(QString("Character at i == %1 was U+%2").arg(i).arg(output.at(i).unicode(), 4, 16, QLatin1Char('0'))));
+    }
 }
 
 void tst_QUrlInternal::recodeByteArray_data()
