@@ -39,6 +39,8 @@
 **
 ****************************************************************************/
 
+#include "tst_qcoreapplication.h"
+
 #include <QtCore/QtCore>
 #include <QtTest/QtTest>
 
@@ -46,30 +48,12 @@
 #include <private/qeventloop_p.h>
 #include <private/qthread_p.h>
 
-class tst_QCoreApplication: public QObject
-{
-    Q_OBJECT
-private slots:
-    void sendEventsOnProcessEvents(); // this must be the first test
-    void getSetCheck();
-    void qAppName();
-#ifndef Q_OS_WIN
-    void argc();
+#ifdef QT_GUI_LIB
+#include <QtGui/QGuiApplication>
+typedef QGuiApplication TestApplication;
+#else
+typedef QCoreApplication TestApplication;
 #endif
-    void postEvent();
-    void removePostedEvents();
-#ifndef QT_NO_THREAD
-    void deliverInDefinedOrder();
-#endif
-    void applicationPid();
-    void globalPostedEventsCount();
-    void processEventsAlwaysSendsPostedEvents();
-    void reexec();
-    void execAfterExit();
-    void eventLoopExecAfterExit();
-    void customEventDispatcher();
-    void testQuitLock();
-};
 
 class EventSpy : public QObject
 {
@@ -87,8 +71,8 @@ public:
 void tst_QCoreApplication::sendEventsOnProcessEvents()
 {
     int argc = 1;
-    char *argv[] = { const_cast<char*>("tst_qcoreapplication") };
-    QCoreApplication app(argc, argv);
+    char *argv[] = { const_cast<char*>(QTest::currentAppName()) };
+    TestApplication app(argc, argv);
 
     EventSpy spy;
     app.installEventFilter(&spy);
@@ -109,8 +93,8 @@ void tst_QCoreApplication::getSetCheck()
     // Test the property
     {
         int argc = 1;
-        char *argv[] = { const_cast<char*>("tst_qcoreapplication") };
-        QCoreApplication app(argc, argv);
+        char *argv[] = { const_cast<char*>(QTest::currentAppName()) };
+        TestApplication app(argc, argv);
         QCOMPARE(app.property("applicationVersion").toString(), v);
     }
     v = QString();
@@ -120,11 +104,17 @@ void tst_QCoreApplication::getSetCheck()
 
 void tst_QCoreApplication::qAppName()
 {
+#ifdef QT_GUI_LIB
+    const char* appName = "tst_qguiapplication";
+#else
+    const char* appName = "tst_qcoreapplication";
+#endif
+
     int argc = 1;
-    char *argv[] = { const_cast<char*>("tst_qcoreapplication") };
-    QCoreApplication app(argc, argv);
-    QCOMPARE(::qAppName(), QString::fromLatin1("tst_qcoreapplication"));
-    QCOMPARE(QCoreApplication::applicationName(), QString::fromLatin1("tst_qcoreapplication"));
+    char *argv[] = { const_cast<char*>(appName) };
+    TestApplication app(argc, argv);
+    QCOMPARE(::qAppName(), QString::fromLatin1(appName));
+    QCOMPARE(QCoreApplication::applicationName(), QString::fromLatin1(appName));
 }
 
 // "QCoreApplication::arguments() always parses arguments from actual command line on Windows
@@ -134,19 +124,19 @@ void tst_QCoreApplication::argc()
 {
     {
         int argc = 1;
-        char *argv[] = { const_cast<char*>("tst_qcoreapplication") };
-        QCoreApplication app(argc, argv);
+        char *argv[] = { const_cast<char*>(QTest::currentAppName()) };
+        TestApplication app(argc, argv);
         QCOMPARE(argc, 1);
         QCOMPARE(app.arguments().count(), 1);
     }
 
     {
         int argc = 4;
-        char *argv[] = { const_cast<char*>("tst_qcoreapplication"),
+        char *argv[] = { const_cast<char*>(QTest::currentAppName()),
                          const_cast<char*>("arg1"),
                          const_cast<char*>("arg2"),
                          const_cast<char*>("arg3") };
-        QCoreApplication app(argc, argv);
+        TestApplication app(argc, argv);
         QCOMPARE(argc, 4);
         QCOMPARE(app.arguments().count(), 4);
     }
@@ -154,16 +144,16 @@ void tst_QCoreApplication::argc()
     {
         int argc = 0;
         char **argv = 0;
-        QCoreApplication app(argc, argv);
+        TestApplication app(argc, argv);
         QCOMPARE(argc, 0);
         QCOMPARE(app.arguments().count(), 0);
     }
 
     {
         int argc = 2;
-        char *argv[] = { const_cast<char*>("tst_qcoreapplication"),
+        char *argv[] = { const_cast<char*>(QTest::currentAppName()),
                          const_cast<char*>("-qmljsdebugger=port:3768,block") };
-        QCoreApplication app(argc, argv);
+        TestApplication app(argc, argv);
         QCOMPARE(argc, 1);
         QCOMPARE(app.arguments().count(), 1);
     }
@@ -195,8 +185,8 @@ public:
 void tst_QCoreApplication::postEvent()
 {
     int argc = 1;
-    char *argv[] = { const_cast<char*>("tst_qcoreapplication") };
-    QCoreApplication app(argc, argv);
+    char *argv[] = { const_cast<char*>(QTest::currentAppName()) };
+    TestApplication app(argc, argv);
 
     EventSpy spy;
     EventGenerator odd, even;
@@ -280,8 +270,8 @@ void tst_QCoreApplication::postEvent()
 void tst_QCoreApplication::removePostedEvents()
 {
     int argc = 1;
-    char *argv[] = { const_cast<char*>("tst_qcoreapplication") };
-    QCoreApplication app(argc, argv);
+    char *argv[] = { const_cast<char*>(QTest::currentAppName()) };
+    TestApplication app(argc, argv);
 
     EventSpy spy;
     QObject one, two;
@@ -459,8 +449,8 @@ public:
 void tst_QCoreApplication::deliverInDefinedOrder()
 {
     int argc = 1;
-    char *argv[] = { const_cast<char*>("tst_qcoreapplication") };
-    QCoreApplication app(argc, argv);
+    char *argv[] = { const_cast<char*>(QTest::currentAppName()) };
+    TestApplication app(argc, argv);
 
     DeliverInDefinedOrderObject obj(&app);
     // causes sendPostedEvents() to recurse twice
@@ -499,8 +489,8 @@ public:
 void tst_QCoreApplication::globalPostedEventsCount()
 {
     int argc = 1;
-    char *argv[] = { const_cast<char*>("tst_qcoreapplication") };
-    QCoreApplication app(argc, argv);
+    char *argv[] = { const_cast<char*>(QTest::currentAppName()) };
+    TestApplication app(argc, argv);
 
     QCoreApplication::sendPostedEvents();
     QCOMPARE(qGlobalPostedEventsCount(), 0u);
@@ -545,8 +535,8 @@ public:
 void tst_QCoreApplication::processEventsAlwaysSendsPostedEvents()
 {
     int argc = 1;
-    char *argv[] = { const_cast<char*>("tst_qcoreapplication") };
-    QCoreApplication app(argc, argv);
+    char *argv[] = { const_cast<char*>(QTest::currentAppName()) };
+    TestApplication app(argc, argv);
 
     ProcessEventsAlwaysSendsPostedEventsObject object;
     QTime t;
@@ -563,8 +553,8 @@ void tst_QCoreApplication::processEventsAlwaysSendsPostedEvents()
 void tst_QCoreApplication::reexec()
 {
     int argc = 1;
-    char *argv[] = { const_cast<char*>("tst_qcoreapplication") };
-    QCoreApplication app(argc, argv);
+    char *argv[] = { const_cast<char*>(QTest::currentAppName()) };
+    TestApplication app(argc, argv);
 
     // exec once
     QMetaObject::invokeMethod(&app, "quit", Qt::QueuedConnection);
@@ -578,8 +568,8 @@ void tst_QCoreApplication::reexec()
 void tst_QCoreApplication::execAfterExit()
 {
     int argc = 1;
-    char *argv[] = { const_cast<char*>("tst_qcoreapplication") };
-    QCoreApplication app(argc, argv);
+    char *argv[] = { const_cast<char*>(QTest::currentAppName()) };
+    TestApplication app(argc, argv);
 
     app.exit(1);
     QMetaObject::invokeMethod(&app, "quit", Qt::QueuedConnection);
@@ -589,8 +579,8 @@ void tst_QCoreApplication::execAfterExit()
 void tst_QCoreApplication::eventLoopExecAfterExit()
 {
     int argc = 1;
-    char *argv[] = { const_cast<char*>("tst_qcoreapplication") };
-    QCoreApplication app(argc, argv);
+    char *argv[] = { const_cast<char*>(QTest::currentAppName()) };
+    TestApplication app(argc, argv);
 
     // exec once and exit
     QMetaObject::invokeMethod(&app, "quit", Qt::QueuedConnection);
@@ -643,12 +633,12 @@ void tst_QCoreApplication::customEventDispatcher()
     QCOMPARE(QCoreApplication::eventDispatcher(), ed);
     // test the alternative API of QAbstractEventDispatcher
     QCOMPARE(QAbstractEventDispatcher::instance(), ed);
-    QWeakPointer<DummyEventDispatcher> weak_ed(ed);
+    QPointer<DummyEventDispatcher> weak_ed(ed);
     QVERIFY(!weak_ed.isNull());
     {
         int argc = 1;
-        char *argv[] = { const_cast<char*>("tst_qcoreapplication") };
-        QCoreApplication app(argc, argv);
+        char *argv[] = { const_cast<char*>(QTest::currentAppName()) };
+        TestApplication app(argc, argv);
         // instantiating app should not overwrite the ED
         QCOMPARE(QCoreApplication::eventDispatcher(), ed);
         QMetaObject::invokeMethod(&app, "quit", Qt::QueuedConnection);
@@ -762,13 +752,55 @@ private slots:
 void tst_QCoreApplication::testQuitLock()
 {
     int argc = 1;
-    char *argv[] = { const_cast<char*>("tst_qcoreapplication") };
-    QCoreApplication app(argc, argv);
+    char *argv[] = { const_cast<char*>(QTest::currentAppName()) };
+    TestApplication app(argc, argv);
 
     QuitTester tester;
     app.exec();
 }
 
 
+void tst_QCoreApplication::QTBUG31606_QEventDestructorDeadLock()
+{
+    class MyEvent : public QEvent
+    { public:
+        MyEvent() : QEvent(QEvent::Type(QEvent::User + 1)) {}
+        ~MyEvent() {
+            QCoreApplication::postEvent(qApp, new QEvent(QEvent::Type(QEvent::User+2)));
+        }
+    };
+
+    int argc = 1;
+    char *argv[] = { const_cast<char*>(QTest::currentAppName()) };
+    TestApplication app(argc, argv);
+
+    EventSpy spy;
+    app.installEventFilter(&spy);
+
+    QCoreApplication::postEvent(&app, new MyEvent);
+    QCoreApplication::processEvents();
+    QVERIFY(spy.recordedEvents.contains(QEvent::User + 1));
+    QVERIFY(!spy.recordedEvents.contains(QEvent::User + 2));
+    QCoreApplication::processEvents();
+    QVERIFY(spy.recordedEvents.contains(QEvent::User + 2));
+}
+
+
+static void createQObjectOnDestruction()
+{
+    // Make sure that we can create a QObject after the last QObject has been
+    // destroyed (especially after QCoreApplication has).
+    //
+    // Before the fixes, this would cause a dangling pointer dereference. If
+    // the problem comes back, it's possible that the following causes no
+    // effect.
+    QObject obj;
+    obj.thread()->setProperty("testing", 1);
+}
+Q_DESTRUCTOR_FUNCTION(createQObjectOnDestruction)
+
+#ifndef QT_GUI_LIB
 QTEST_APPLESS_MAIN(tst_QCoreApplication)
+#endif
+
 #include "tst_qcoreapplication.moc"
