@@ -78,6 +78,7 @@ private slots:
     /* void registerEventNotifiier(); */ // Not implemented here, see tst_QWinEventNotifier instead
     void sendPostedEvents_data();
     void sendPostedEvents();
+    void processEventsOnlySendsQueuedEvents();
 };
 
 bool tst_QEventDispatcher::event(QEvent *e)
@@ -148,8 +149,7 @@ void tst_QEventDispatcher::registerTimer()
     // process events, waiting for the next event... this should only fire the precise timer
     receivedEventType = -1;
     timerIdFromEvent = -1;
-    QVERIFY(eventDispatcher->processEvents(QEventLoop::WaitForMoreEvents));
-    QCOMPARE(receivedEventType, int(QEvent::Timer));
+    QTRY_COMPARE_WITH_TIMEOUT(receivedEventType, int(QEvent::Timer), PreciseTimerInterval * 2);
     QCOMPARE(timerIdFromEvent, preciseTimerId);
     // now unregister it and make sure it's gone
     eventDispatcher->unregisterTimer(preciseTimerId);
@@ -161,8 +161,7 @@ void tst_QEventDispatcher::registerTimer()
     // do the same again for the coarse timer
     receivedEventType = -1;
     timerIdFromEvent = -1;
-    QVERIFY(eventDispatcher->processEvents(QEventLoop::WaitForMoreEvents));
-    QCOMPARE(receivedEventType, int(QEvent::Timer));
+    QTRY_COMPARE_WITH_TIMEOUT(receivedEventType, int(QEvent::Timer), CoarseTimerInterval * 2);
     QCOMPARE(timerIdFromEvent, coarseTimerId);
     // now unregister it and make sure it's gone
     eventDispatcher->unregisterTimer(coarseTimerId);
@@ -209,5 +208,49 @@ void tst_QEventDispatcher::sendPostedEvents()
     }
 }
 
+class ProcessEventsOnlySendsQueuedEvents : public QObject
+{
+    Q_OBJECT
+public:
+    int eventsReceived;
+
+    inline ProcessEventsOnlySendsQueuedEvents() : eventsReceived(0) {}
+
+    bool event(QEvent *event)
+    {
+        ++eventsReceived;
+
+        if (event->type() == QEvent::User)
+             QCoreApplication::postEvent(this, new QEvent(QEvent::Type(QEvent::User + 1)));
+
+        return QObject::event(event);
+    }
+public slots:
+    void timerFired()
+    {
+        QCoreApplication::postEvent(this, new QEvent(QEvent::Type(QEvent::User + 1)));
+    }
+};
+
+void tst_QEventDispatcher::processEventsOnlySendsQueuedEvents()
+{
+    ProcessEventsOnlySendsQueuedEvents object;
+
+    // Posted events during event processing should be handled on
+    // the next processEvents iteration.
+    QCoreApplication::postEvent(&object, new QEvent(QEvent::User));
+    QCoreApplication::processEvents();
+    QCOMPARE(object.eventsReceived, 1);
+    QCoreApplication::processEvents();
+    QCOMPARE(object.eventsReceived, 2);
+
+    // The same goes for posted events during timer processing
+    QTimer::singleShot(0, &object, SLOT(timerFired()));
+    QCoreApplication::processEvents();
+    QCOMPARE(object.eventsReceived, 3);
+    QCoreApplication::processEvents();
+    QCOMPARE(object.eventsReceived, 4);
+}
+
 QTEST_MAIN(tst_QEventDispatcher)
 #include "tst_qeventdispatcher.moc"
