@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2012 Digia Plc and/or its subsidiary(-ies).
+** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
 ** This file is part of the test suite of the Qt Toolkit.
@@ -43,7 +43,7 @@
 
 #include <qobject.h>
 #include <qmetaobject.h>
-#include <qwindow.h>
+#include <qabstractproxymodel.h>
 #include <private/qmetaobject_p.h>
 
 Q_DECLARE_METATYPE(const QMetaObject *)
@@ -155,6 +155,7 @@ private slots:
     void invokeCustomTypes();
     void invokeMetaConstructor();
     void invokeTypedefTypes();
+    void invokeException();
     void qtMetaObjectInheritance();
     void normalizedSignature_data();
     void normalizedSignature();
@@ -301,6 +302,19 @@ void tst_QMetaObject::connectSlotsByName()
 
 struct MyUnregisteredType { };
 
+static int countedStructObjectsCount = 0;
+struct CountedStruct
+{
+    CountedStruct() { ++countedStructObjectsCount; }
+    CountedStruct(const CountedStruct &) { ++countedStructObjectsCount; }
+    CountedStruct &operator=(const CountedStruct &) { return *this; }
+    ~CountedStruct() { --countedStructObjectsCount; }
+};
+
+#ifndef QT_NO_EXCEPTIONS
+class ObjectException : public std::exception { };
+#endif
+
 class QtTestObject: public QObject
 {
     friend class tst_QMetaObject;
@@ -328,6 +342,7 @@ public slots:
     QObject *sl11();
     const char *sl12();
     QList<QString> sl13(QList<QString> l1);
+    qint64 sl14();
     void testSender();
 
     void testReference(QString &str);
@@ -339,6 +354,13 @@ public slots:
 
     void slotWithUnregisteredParameterType(MyUnregisteredType);
 
+    CountedStruct throwingSlot(const CountedStruct &, CountedStruct s2) {
+#ifndef QT_NO_EXCEPTIONS
+        throw ObjectException();
+#endif
+        return s2;
+    }
+
 signals:
     void sig0();
     QString sig1(QString s1);
@@ -395,6 +417,9 @@ const char *QtTestObject::sl12()
 { slotResult = "sl12"; return "foo"; }
 QList<QString> QtTestObject::sl13(QList<QString> l1)
 { slotResult = "sl13"; return l1; }
+qint64 QtTestObject::sl14()
+{ slotResult = "sl14"; return Q_INT64_C(123456789)*123456789; }
+
 void QtTestObject::testReference(QString &str)
 { slotResult = "testReference:" + str; str = "gotcha"; }
 
@@ -513,6 +538,13 @@ void tst_QMetaObject::invokeMetaMember()
     QCOMPARE(returnValue, argument);
     QCOMPARE(obj.slotResult, QString("sl13"));
 
+    // return qint64
+    qint64 return64;
+    QVERIFY(QMetaObject::invokeMethod(&obj, "sl14",
+                                      Q_RETURN_ARG(qint64, return64)));
+    QCOMPARE(return64, Q_INT64_C(123456789)*123456789);
+    QCOMPARE(obj.slotResult, QString("sl14"));
+
     //test signals
     QVERIFY(QMetaObject::invokeMethod(&obj, "sig0"));
     QCOMPARE(obj.slotResult, QString("sl0"));
@@ -735,13 +767,13 @@ void tst_QMetaObject::invokeBlockingQueuedMetaMember()
 void tst_QMetaObject::qtMetaObjectInheritance()
 {
     QVERIFY(QObject::staticMetaObject.superClass() == 0);
-    QCOMPARE(QWindow::staticMetaObject.indexOfEnumerator("Qt::ScreenOrientation"), -1);
-    QCOMPARE(QWindow::staticMetaObject.indexOfEnumerator("ScreenOrientation"), -1);
-    int indexOfContentOrientation = QWindow::staticMetaObject.indexOfProperty("contentOrientation");
-    QVERIFY(indexOfContentOrientation != -1);
-    QMetaProperty contentOrientation = QWindow::staticMetaObject.property(indexOfContentOrientation);
-    QVERIFY(contentOrientation.isValid());
-    QCOMPARE(contentOrientation.enumerator().name(), "ScreenOrientation");
+    QCOMPARE(QSortFilterProxyModel::staticMetaObject.indexOfEnumerator("Qt::CaseSensitivity"), -1);
+    QCOMPARE(QSortFilterProxyModel::staticMetaObject.indexOfEnumerator("CaseSensitivity"), -1);
+    int indexOfSortCaseSensitivity = QSortFilterProxyModel::staticMetaObject.indexOfProperty("sortCaseSensitivity");
+    QVERIFY(indexOfSortCaseSensitivity != -1);
+    QMetaProperty sortCaseSensitivity = QSortFilterProxyModel::staticMetaObject.property(indexOfSortCaseSensitivity);
+    QVERIFY(sortCaseSensitivity.isValid());
+    QCOMPARE(sortCaseSensitivity.enumerator().name(), "CaseSensitivity");
 }
 
 struct MyType
@@ -836,6 +868,23 @@ void tst_QMetaObject::invokeTypedefTypes()
     QCOMPARE(spy.at(0).at(0), QVariant(arg));
 }
 
+void tst_QMetaObject::invokeException()
+{
+#ifndef QT_NO_EXCEPTIONS
+    QtTestObject obj;
+    QCOMPARE(countedStructObjectsCount, 0);
+    try {
+        CountedStruct s;
+        QVERIFY(QMetaObject::invokeMethod(&obj, "throwingSlot", Q_RETURN_ARG(CountedStruct, s),
+                                          Q_ARG(CountedStruct, s), Q_ARG(CountedStruct, s)));
+        QFAIL("Did not throw");
+    } catch(ObjectException &) {}
+    QCOMPARE(countedStructObjectsCount, 0);
+#else
+    QSKIP("Needs exceptions");
+#endif
+}
+
 void tst_QMetaObject::normalizedSignature_data()
 {
     QTest::addColumn<QString>("signature");
