@@ -106,6 +106,8 @@ private slots:
 
     void customEventDispatcher();
 
+    void requestTermination();
+
 #ifndef Q_OS_WINCE
     void stressTest();
 #endif
@@ -1026,7 +1028,7 @@ void tst_QThread::exitAndExec()
     thread.sem2.acquire();
     int v = thread.value;
     QCOMPARE(v, 556);
-    
+
     //test that the thread is running by executing queued connected signal there
     Syncronizer sync1;
     sync1.moveToThread(&thread);
@@ -1221,9 +1223,9 @@ QT_END_NAMESPACE
 
 class DummyEventDispatcher : public QAbstractEventDispatcher {
 public:
-    DummyEventDispatcher() : QAbstractEventDispatcher(), visited(false) {}
+    DummyEventDispatcher() : QAbstractEventDispatcher() {}
     bool processEvents(QEventLoop::ProcessEventsFlags) {
-        visited = true;
+        visited.store(true);
         emit awake();
         QCoreApplication::sendPostedEvents();
         return false;
@@ -1247,7 +1249,7 @@ public:
     void unregisterEventNotifier(QWinEventNotifier *) { }
 #endif
 
-    bool visited;
+    QBasicAtomicInt visited; // bool
 };
 
 class ThreadObj : public QObject
@@ -1285,7 +1287,7 @@ void tst_QThread::customEventDispatcher()
     QMetaObject::invokeMethod(&obj, "visit", Qt::QueuedConnection);
     loop.exec();
     // test that the ED has really been used
-    QVERIFY(ed->visited);
+    QVERIFY(ed->visited.load());
 
     QPointer<DummyEventDispatcher> weak_ed(ed);
     QVERIFY(!weak_ed.isNull());
@@ -1345,5 +1347,43 @@ void tst_QThread::quitLock()
     QVERIFY(exitThreadCalled);
 }
 
+class StopableJob : public QObject
+{
+    Q_OBJECT
+public:
+    StopableJob (QSemaphore &sem) : sem(sem) {}
+    QSemaphore &sem;
+public Q_SLOTS:
+    void run() {
+        sem.release();
+        while (!thread()->isInterruptionRequested())
+            QTest::qSleep(10);
+        sem.release();
+        Q_EMIT finished();
+    }
+Q_SIGNALS:
+    void finished();
+};
+
+void tst_QThread::requestTermination()
+{
+    QThread thread;
+    QVERIFY(!thread.isInterruptionRequested());
+    QSemaphore sem;
+    StopableJob *j  = new StopableJob(sem);
+    j->moveToThread(&thread);
+    connect(&thread, &QThread::started, j, &StopableJob::run);
+    connect(j, &StopableJob::finished, &thread, &QThread::quit, Qt::DirectConnection);
+    connect(&thread, &QThread::finished, j, &QObject::deleteLater);
+    thread.start();
+    QVERIFY(!thread.isInterruptionRequested());
+    sem.acquire();
+    QVERIFY(!thread.wait(1000));
+    thread.requestInterruption();
+    sem.acquire();
+    QVERIFY(thread.wait(1000));
+    QVERIFY(!thread.isInterruptionRequested());
+}
+
 QTEST_MAIN(tst_QThread)
 #include "tst_qthread.moc"
