@@ -64,6 +64,12 @@ QRunnable *createTask(FunctionPointer pointer)
 class tst_QThreadPool : public QObject
 {
     Q_OBJECT
+public:
+    tst_QThreadPool();
+    ~tst_QThreadPool();
+
+    static QMutex *functionTestMutex;
+
 private slots:
     void runFunction();
     void createThreadRunFunction();
@@ -84,16 +90,36 @@ private slots:
     void reserveThread();
     void releaseThread_data();
     void releaseThread();
+    void reserveAndStart();
     void start();
     void tryStart();
     void tryStartPeakThreadCount();
     void tryStartCount();
+    void priorityStart_data();
+    void priorityStart();
     void waitForDone();
+    void clear();
     void waitForDoneTimeout();
     void destroyingWaitsForTasksToFinish();
     void stressTest();
+
+private:
+    QMutex m_functionTestMutex;
 };
 
+
+QMutex *tst_QThreadPool::functionTestMutex = 0;
+
+tst_QThreadPool::tst_QThreadPool()
+{
+    tst_QThreadPool::functionTestMutex = &m_functionTestMutex;
+}
+
+tst_QThreadPool::~tst_QThreadPool()
+{
+    tst_QThreadPool::functionTestMutex = 0;
+}
+
 int testFunctionCount;
 
 void sleepTestFunction()
@@ -114,19 +140,19 @@ void noSleepTestFunction()
 
 void sleepTestFunctionMutex()
 {
-    static QMutex testMutex;
+    Q_ASSERT(tst_QThreadPool::functionTestMutex);
     QTest::qSleep(1000);
-    testMutex.lock();
+    tst_QThreadPool::functionTestMutex->lock();
     ++testFunctionCount;
-    testMutex.unlock();
+    tst_QThreadPool::functionTestMutex->unlock();
 }
 
 void noSleepTestFunctionMutex()
 {
-    static QMutex testMutex;
-    testMutex.lock();
+    Q_ASSERT(tst_QThreadPool::functionTestMutex);
+    tst_QThreadPool::functionTestMutex->lock();
     ++testFunctionCount;
-    testMutex.unlock();
+    tst_QThreadPool::functionTestMutex->unlock();
 }
 
 void tst_QThreadPool::runFunction()
@@ -190,22 +216,22 @@ void tst_QThreadPool::waitcomplete()
     QCOMPARE(testFunctionCount, runs);
 }
 
-volatile bool ran;
+QAtomicInt ran; // bool
 class TestTask : public QRunnable
 {
 public:
     void run()
     {
-        ran = true;
+        ran.store(true);
     }
 };
 
 void tst_QThreadPool::runTask()
 {
     QThreadPool manager;
-    ran = false;
+    ran.store(false);
     manager.start(new TestTask());
-    QTRY_VERIFY(ran);
+    QTRY_VERIFY(ran.load());
 }
 
 /*
@@ -213,19 +239,19 @@ void tst_QThreadPool::runTask()
 */
 void tst_QThreadPool::singleton()
 {
-    ran = false;
+    ran.store(false);
     QThreadPool::globalInstance()->start(new TestTask());
-    QTRY_VERIFY(ran);
+    QTRY_VERIFY(ran.load());
 }
 
-int *value = 0;
+QAtomicInt *value = 0;
 class IntAccessor : public QRunnable
 {
 public:
     void run()
     {
         for (int i = 0; i < 100; ++i) {
-            ++(*value);
+            value->ref();
             QTest::qSleep(1);
         }
     }
@@ -237,7 +263,7 @@ public:
 */
 void tst_QThreadPool::destruction()
 {
-    value = new int;
+    value = new QAtomicInt;
     QThreadPool *threadManager = new QThreadPool();
     threadManager->start(new IntAccessor());
     threadManager->start(new IntAccessor());
@@ -605,6 +631,65 @@ void tst_QThreadPool::releaseThread()
     threadpool->setMaxThreadCount(savedLimit);
 }
 
+void tst_QThreadPool::reserveAndStart() // QTBUG-21051
+{
+    class WaitingTask : public QRunnable
+    {
+    public:
+        QAtomicInt count;
+        QSemaphore waitForStarted;
+        QSemaphore waitBeforeDone;
+
+        WaitingTask() { setAutoDelete(false); }
+
+        void run()
+        {
+            count.ref();
+            waitForStarted.release();
+            waitBeforeDone.acquire();
+        }
+    };
+
+    // Set up
+    QThreadPool *threadpool = QThreadPool::globalInstance();
+    int savedLimit = threadpool->maxThreadCount();
+    threadpool->setMaxThreadCount(1);
+    QCOMPARE(threadpool->activeThreadCount(), 0);
+
+    // reserve
+    threadpool->reserveThread();
+    QCOMPARE(threadpool->activeThreadCount(), 1);
+
+    // start a task, to get a running thread
+    WaitingTask *task = new WaitingTask;
+    threadpool->start(task);
+    QCOMPARE(threadpool->activeThreadCount(), 2);
+    task->waitForStarted.acquire();
+    task->waitBeforeDone.release();
+    QTRY_COMPARE(task->count.load(), 1);
+    QTRY_COMPARE(threadpool->activeThreadCount(), 1);
+
+    // now the thread is waiting, but tryStart() will fail since activeThreadCount() >= maxThreadCount()
+    QVERIFY(!threadpool->tryStart(task));
+    QTRY_COMPARE(threadpool->activeThreadCount(), 1);
+
+    // start() will therefore do a failing tryStart(), followed by enqueueTask()
+    // which will actually wake up the waiting thread.
+    threadpool->start(task);
+    QTRY_COMPARE(threadpool->activeThreadCount(), 2);
+    task->waitForStarted.acquire();
+    task->waitBeforeDone.release();
+    QTRY_COMPARE(task->count.load(), 2);
+    QTRY_COMPARE(threadpool->activeThreadCount(), 1);
+
+    threadpool->releaseThread();
+    QTRY_COMPARE(threadpool->activeThreadCount(), 0);
+
+    delete task;
+
+    threadpool->setMaxThreadCount(savedLimit);
+}
+
 QAtomicInt count;
 class CountingRunnable : public QRunnable
 {
@@ -657,8 +742,8 @@ void tst_QThreadPool::tryStart()
 }
 
 QMutex mutex;
-int activeThreads = 0;
-int peakActiveThreads = 0;
+QAtomicInt activeThreads;
+QAtomicInt peakActiveThreads;
 void tst_QThreadPool::tryStartPeakThreadCount()
 {
     class CounterTask : public QRunnable
@@ -670,14 +755,14 @@ void tst_QThreadPool::tryStartPeakThreadCount()
         {
             {
                 QMutexLocker lock(&mutex);
-                ++activeThreads;
-                peakActiveThreads = qMax(peakActiveThreads, activeThreads);
+                activeThreads.ref();
+                peakActiveThreads.store(qMax(peakActiveThreads.load(), activeThreads.load()));
             }
 
             QTest::qWait(100);
             {
                 QMutexLocker lock(&mutex);
-                --activeThreads;
+                activeThreads.deref();
             }
         }
     };
@@ -689,13 +774,13 @@ void tst_QThreadPool::tryStartPeakThreadCount()
         if (threadPool.tryStart(&task) == false)
             QTest::qWait(10);
     }
-    QCOMPARE(peakActiveThreads, QThread::idealThreadCount());
+    QCOMPARE(peakActiveThreads.load(), QThread::idealThreadCount());
 
     for (int i = 0; i < 20; ++i) {
         if (threadPool.tryStart(&task) == false)
             QTest::qWait(10);
     }
-    QCOMPARE(peakActiveThreads, QThread::idealThreadCount());
+    QCOMPARE(peakActiveThreads.load(), QThread::idealThreadCount());
 }
 
 void tst_QThreadPool::tryStartCount()
@@ -725,6 +810,57 @@ void tst_QThreadPool::tryStartCount()
     }
 }
 
+void tst_QThreadPool::priorityStart_data()
+{
+    QTest::addColumn<int>("otherCount");
+    QTest::newRow("0") << 0;
+    QTest::newRow("1") << 1;
+    QTest::newRow("2") << 2;
+}
+
+void tst_QThreadPool::priorityStart()
+{
+    class Holder : public QRunnable
+    {
+    public:
+        QSemaphore &sem;
+        Holder(QSemaphore &sem) : sem(sem) {}
+        void run()
+        {
+            sem.acquire();
+        }
+    };
+    class Runner : public QRunnable
+    {
+    public:
+        QAtomicPointer<QRunnable> &ptr;
+        Runner(QAtomicPointer<QRunnable> &ptr) : ptr(ptr) {}
+        void run()
+        {
+            ptr.testAndSetRelaxed(0, this);
+        }
+    };
+
+    QFETCH(int, otherCount);
+    QSemaphore sem;
+    QAtomicPointer<QRunnable> firstStarted;
+    QRunnable *expected;
+    QThreadPool threadPool;
+    threadPool.setMaxThreadCount(1); // start only one thread at a time
+
+    // queue the holder first
+    // We need to be sure that all threads are active when we
+    // queue the two Runners
+    threadPool.start(new Holder(sem));
+    while (otherCount--)
+        threadPool.start(new Runner(firstStarted), 0); // priority 0
+    threadPool.start(expected = new Runner(firstStarted), 1); // priority 1
+
+    sem.release();
+    QVERIFY(threadPool.waitForDone());
+    QCOMPARE(firstStarted.load(), expected);
+}
+
 void tst_QThreadPool::waitForDone()
 {
     QTime total, pass;
@@ -761,7 +897,7 @@ void tst_QThreadPool::waitForDoneTimeout()
     public:
       QMutex mutex;
       BlockedTask() { setAutoDelete(false); }
-      
+
       void run()
         {
           mutex.lock();
@@ -780,6 +916,34 @@ void tst_QThreadPool::waitForDoneTimeout()
     QVERIFY(threadPool.waitForDone(400));
 }
 
+void tst_QThreadPool::clear()
+{
+    QSemaphore sem(0);
+    class BlockingRunnable : public QRunnable
+    {
+        public:
+            QSemaphore & sem;
+            BlockingRunnable(QSemaphore & sem) : sem(sem){}
+            void run()
+            {
+                sem.acquire();
+                count.ref();
+            }
+    };
+
+    QThreadPool threadPool;
+    threadPool.setMaxThreadCount(10);
+    int runs = 2 * threadPool.maxThreadCount();
+    count.store(0);
+    for (int i = 0; i <= runs; i++) {
+        threadPool.start(new BlockingRunnable(sem));
+    }
+    threadPool.clear();
+    sem.release(threadPool.maxThreadCount());
+    threadPool.waitForDone();
+    QCOMPARE(count.load(), threadPool.maxThreadCount());
+}
+
 void tst_QThreadPool::destroyingWaitsForTasksToFinish()
 {
     QTime total, pass;
