@@ -42,12 +42,13 @@
 
 #include <QtTest/QtTest>
 #include <math.h>
-#include <qglobal.h>
+#include <qdebug.h>
 #include <qdir.h>
 #include <qfileinfo.h>
 #include <QScopedArrayPointer>
 #include <qtextcodec.h>
 #include <qdatetime.h>
+#include <qprocess.h>
 #include <float.h>
 
 #include <qlocale.h>
@@ -74,6 +75,52 @@ extern "C" DWORD GetThreadLocale(void) {
 #    include <stdlib.h>
 #endif
 
+#ifdef Q_OS_BLACKBERRY
+#include <unistd.h>
+#include <errno.h>
+#include <fcntl.h>
+#include <sys/stat.h>
+#include <sys/pps.h>
+
+static const char ppsLanguageLocalePath[] = "/pps/services/confstr/_CS_LOCALE";
+static const size_t ppsBufferSize = 256;
+
+static QByteArray readPpsValue(const char *ppsObject, int ppsFd)
+{
+    QByteArray result;
+    if (!ppsObject || ppsFd == -1)
+        return result;
+
+    char buffer[ppsBufferSize];
+
+    int bytes = read(ppsFd, buffer, ppsBufferSize - 1);
+    if (bytes == -1) {
+        qFatal("Failed to read Locale pps, errno=%d", errno);
+        return result;
+    }
+    // ensure data is null terminated
+    buffer[bytes] = '\0';
+
+    pps_decoder_t ppsDecoder;
+    pps_decoder_initialize(&ppsDecoder, 0);
+    if (pps_decoder_parse_pps_str(&ppsDecoder, buffer) == PPS_DECODER_OK) {
+        pps_decoder_push(&ppsDecoder, 0);
+        const char *ppsBuff;
+        if (pps_decoder_get_string(&ppsDecoder, ppsObject, &ppsBuff) == PPS_DECODER_OK) {
+            result = ppsBuff;
+        } else {
+            int val;
+            if (pps_decoder_get_int(&ppsDecoder, ppsObject, &val) == PPS_DECODER_OK)
+                result = QByteArray::number(val);
+        }
+    }
+
+    pps_decoder_cleanup(&ppsDecoder);
+
+    return result;
+}
+#endif // Q_OS_BLACKBERRY
+
 Q_DECLARE_METATYPE(QLocale::FormatType)
 
 class tst_QLocale : public QObject
@@ -85,6 +132,7 @@ public:
 
 private slots:
     void initTestCase();
+    void cleanupTestCase();
 #ifdef Q_OS_WIN
     void windowsDefaultLocale();
 #endif
@@ -115,6 +163,7 @@ private slots:
     void formatTime_data();
     void formatDateTime();
     void formatDateTime_data();
+    void formatTimeZone();
     void toDateTime_data();
     void toDateTime();
     void negativeNumbers();
@@ -145,14 +194,27 @@ private slots:
     void measurementSystems();
     void QTBUG_26035_positivesign();
 
+    void textDirection_data();
+    void textDirection();
+
 private:
     QString m_decimal, m_thousand, m_sdate, m_ldate, m_time;
     QString m_sysapp;
+    bool europeanTimeZone;
+
+#ifdef Q_OS_BLACKBERRY
+    int m_languageFd;
+#endif
 };
 
 tst_QLocale::tst_QLocale()
 {
     qRegisterMetaType<QLocale::FormatType>("QLocale::FormatType");
+
+    // Test if in Central European Time zone
+    uint x1 = QDateTime(QDate(1990, 1, 1), QTime()).toTime_t();
+    uint x2 = QDateTime(QDate(1990, 6, 1), QTime()).toTime_t();
+    europeanTimeZone = (x1 == 631148400 && x2 == 644191200);
 }
 
 void tst_QLocale::initTestCase()
@@ -169,6 +231,18 @@ void tst_QLocale::initTestCase()
     QVERIFY2(fi.exists() && fi.isExecutable(),
              qPrintable(QDir::toNativeSeparators(m_sysapp)
                         + QStringLiteral(" does not exist or is not executable.")));
+
+#ifdef Q_OS_BLACKBERRY
+    if ((m_languageFd = open(ppsLanguageLocalePath, O_RDONLY)) == -1)
+        QFAIL("Failed to open language pps.");
+#endif
+}
+
+void tst_QLocale::cleanupTestCase()
+{
+#ifdef Q_OS_BLACKBERRY
+    close(m_languageFd);
+#endif
 }
 
 void tst_QLocale::ctor()
@@ -346,6 +420,7 @@ void tst_QLocale::ctor()
     TEST_CTOR("es_ES", Spanish, Spain)
     TEST_CTOR("es_419", Spanish, LatinAmericaAndTheCaribbean)
     TEST_CTOR("es-419", Spanish, LatinAmericaAndTheCaribbean)
+    TEST_CTOR("fr_MA", French, Morocco)
 
     // test default countries for languages
     TEST_CTOR("zh", Chinese, China)
@@ -457,6 +532,11 @@ void tst_QLocale::emptyCtor()
     QVERIFY2(runSysApp(m_sysapp, env, &defaultLoc, &errorMessage),
              qPrintable(errorMessage));
 
+#ifdef Q_OS_BLACKBERRY
+    QString locale = readPpsValue("_CS_LOCALE", m_languageFd);
+    QVERIFY2(runSysApp(m_sysapp, env, &locale, &errorMessage),
+             qPrintable(errorMessage));
+#else
     TEST_CTOR("C", "C")
     TEST_CTOR("bla", "C")
     TEST_CTOR("zz", "C")
@@ -494,6 +574,7 @@ void tst_QLocale::emptyCtor()
     TEST_CTOR("en/", defaultLoc.toLatin1())
     TEST_CTOR("asdfghj", defaultLoc.toLatin1());
     TEST_CTOR("123456", defaultLoc.toLatin1());
+#endif // Q_OS_BLACKBERRY
 
 #undef TEST_CTOR
 }
@@ -712,6 +793,7 @@ void tst_QLocale::double_conversion()
     QFETCH(QString, num_str);
     QFETCH(bool, good);
     QFETCH(double, num);
+    QStringRef num_strRef = num_str.leftRef(-1);
 
     QLocale locale(locale_name);
     QCOMPARE(locale.name(), locale_name);
@@ -726,6 +808,16 @@ void tst_QLocale::double_conversion()
             diff = -diff;
         QVERIFY(diff <= MY_DOUBLE_EPSILON);
     }
+
+    d = locale.toDouble(num_strRef, &ok);
+    QCOMPARE(ok, good);
+
+    if (ok) {
+        double diff = d - num;
+        if (diff < 0)
+            diff = -diff;
+        QVERIFY(diff <= MY_DOUBLE_EPSILON);
+    }
 }
 
 void tst_QLocale::long_long_conversion_data()
@@ -787,6 +879,7 @@ void tst_QLocale::long_long_conversion()
     QFETCH(QString, num_str);
     QFETCH(bool, good);
     QFETCH(qlonglong, num);
+    QStringRef num_strRef = num_str.leftRef(-1);
 
     QLocale locale(locale_name);
     QCOMPARE(locale.name(), locale_name);
@@ -795,9 +888,14 @@ void tst_QLocale::long_long_conversion()
     qlonglong l = locale.toLongLong(num_str, &ok);
     QCOMPARE(ok, good);
 
-    if (ok) {
+    if (ok)
+        QCOMPARE(l, num);
+
+    l = locale.toLongLong(num_strRef, &ok);
+    QCOMPARE(ok, good);
+
+    if (ok)
         QCOMPARE(l, num);
-    }
 }
 
 void tst_QLocale::long_long_conversion_extra()
@@ -1107,6 +1205,59 @@ void tst_QLocale::formatDateTime_data()
     QTest::newRow("28no_NO") << "no_NO" << QDateTime()
                              << "'\"yymm\"'" << "";
 
+    QDateTime testLongHour(QDate(1999, 12, 31), QTime(23, 59, 59, 999));
+    QDateTime testShortHour(QDate(1999, 12, 31), QTime(3, 59, 59, 999));
+    QDateTime testZeroHour(QDate(1999, 12, 31), QTime(0, 59, 59, 999));
+
+    QTest::newRow("datetime0")      << "en_US" << QDateTime()
+                                    << QString("dd-MM-yyyy hh:mm:ss") << QString();
+    QTest::newRow("datetime1")      << "en_US" << testLongHour
+                                    << QString("dd-'mmddyy'MM-yyyy hh:mm:ss.zzz")
+                                    << QString("31-mmddyy12-1999 23:59:59.999");
+    QTest::newRow("datetime2")      << "en_US" << testLongHour
+                                    << QString("dd-'apAP'MM-yyyy hh:mm:ss.zzz")
+                                    << QString("31-apAP12-1999 23:59:59.999");
+    QTest::newRow("datetime3")      << "en_US" << testLongHour
+                                    << QString("Apdd-MM-yyyy hh:mm:ss.zzz")
+                                    << QString("PMp31-12-1999 11:59:59.999");
+    QTest::newRow("datetime4")      << "en_US" << testLongHour
+                                    << QString("'ap'apdd-MM-yyyy 'AP'hh:mm:ss.zzz")
+                                    << QString("appm31-12-1999 AP11:59:59.999");
+    QTest::newRow("datetime5")      << "en_US" << testLongHour
+                                    << QString("'''") << QString("'");
+    QTest::newRow("datetime6")      << "en_US" << testLongHour
+                                    << QString("'ap") << QString("ap");
+    QTest::newRow("datetime7")      << "en_US" << testLongHour
+                                    << QString("' ' 'hh' hh") << QString("  hh 23");
+    QTest::newRow("datetime8")      << "en_US" << testLongHour
+                                    << QString("d'foobar'") << QString("31foobar");
+    QTest::newRow("datetime9")      << "en_US" << testShortHour
+                                    << QString("hhhhh") << QString("03033");
+    QTest::newRow("datetime11")     << "en_US" << testLongHour
+                                    << QString("HHHhhhAaAPap") << QString("23231111PMpmPMpm");
+    QTest::newRow("datetime12")     << "en_US" << testShortHour
+                                    << QString("HHHhhhAaAPap") << QString("033033AMamAMam");
+    QTest::newRow("datetime13")     << "en_US" << QDateTime(QDate(1974, 12, 1), QTime(14, 14, 20))
+                                    << QString("hh''mm''ss dd''MM''yyyy")
+                                    << QString("14'14'20 01'12'1974");
+    QTest::newRow("AM no p")        << "en_US" << testZeroHour
+                                    << QString("hhAX") << QString("12AMX");
+    QTest::newRow("AM no p, x 2")   << "en_US" << testShortHour
+                                    << QString("hhhhhaA") << QString("03033amAM");
+    QTest::newRow("am 0 hour")      << "en_US" << testZeroHour
+                                    << QString("hAP") << QString("12AM");
+    QTest::newRow("AM zero hour")   << "en_US" << testZeroHour
+                                    << QString("hhAP") << QString("12AM");
+    QTest::newRow("dddd")           << "en_US" << testZeroHour
+                                    << QString("dddd") << QString("Friday");
+    QTest::newRow("ddd")            << "en_US" << testZeroHour
+                                    << QString("ddd") << QString("Fri");
+    QTest::newRow("MMMM")           << "en_US" << testZeroHour
+                                    << QString("MMMM") << QString("December");
+    QTest::newRow("MMM")            << "en_US" << testZeroHour
+                                    << QString("MMM") << QString("Dec");
+    QTest::newRow("empty")          << "en_US" << testZeroHour
+                                    << QString("") << QString("");
 }
 
 void tst_QLocale::formatDateTime()
@@ -1120,6 +1271,58 @@ void tst_QLocale::formatDateTime()
     QCOMPARE(l.toString(dateTime, format), result);
 }
 
+void tst_QLocale::formatTimeZone()
+{
+    QLocale enUS("en_US");
+
+    QDateTime dt1(QDate(2013, 1, 1), QTime(1, 0, 0), Qt::OffsetFromUTC, 60 * 60);
+    QCOMPARE(enUS.toString(dt1, "t"), QString("UTC+01:00"));
+
+    QDateTime dt2(QDate(2013, 1, 1), QTime(1, 0, 0), Qt::OffsetFromUTC, -60 * 60);
+    QCOMPARE(enUS.toString(dt2, "t"), QString("UTC-01:00"));
+
+    QDateTime dt3(QDate(2013, 1, 1), QTime(0, 0, 0), Qt::UTC);
+    QCOMPARE(enUS.toString(dt3, "t"), QString("UTC"));
+
+    // LocalTime should vary
+    if (europeanTimeZone) {
+        // Time definitely in Standard Time
+        QDateTime dt4(QDate(2013, 1, 1), QTime(0, 0, 0), Qt::LocalTime);
+#ifdef Q_OS_WIN
+        QEXPECT_FAIL("", "Windows only returns long name (QTBUG-32759)", Continue);
+#endif // Q_OS_WIN
+        QCOMPARE(enUS.toString(dt4, "t"), QString("CET"));
+
+        // Time definitely in Daylight Time
+        QDateTime dt5(QDate(2013, 6, 1), QTime(0, 0, 0), Qt::LocalTime);
+#ifdef Q_OS_WIN
+        QEXPECT_FAIL("", "Windows only returns long name (QTBUG-32759)", Continue);
+#endif // Q_OS_WIN
+        QCOMPARE(enUS.toString(dt5, "t"), QString("CEST"));
+    } else {
+        QSKIP("You must test using Central European (CET/CEST) time zone, e.g. TZ=Europe/Oslo");
+    }
+
+    QDateTime dt6(QDate(2013, 1, 1), QTime(0, 0, 0), QTimeZone("Europe/Berlin"));
+#ifdef Q_OS_WIN
+    QEXPECT_FAIL("", "QTimeZone windows backend only returns long name", Continue);
+#endif
+    QCOMPARE(enUS.toString(dt6, "t"), QString("CET"));
+
+    QDateTime dt7(QDate(2013, 6, 1), QTime(0, 0, 0), QTimeZone("Europe/Berlin"));
+#ifdef Q_OS_WIN
+    QEXPECT_FAIL("", "QTimeZone windows backend only returns long name", Continue);
+#endif
+    QCOMPARE(enUS.toString(dt7, "t"), QString("CEST"));
+
+    // Current datetime should return current abbreviation
+    QCOMPARE(enUS.toString(QDateTime::currentDateTime(), "t"),
+             QDateTime::currentDateTime().timeZoneAbbreviation());
+
+    // Time on its own will always be current local time zone
+    QCOMPARE(enUS.toString(QTime(1, 2, 3), "t"), QDateTime::currentDateTime().timeZoneAbbreviation());
+}
+
 void tst_QLocale::toDateTime_data()
 {
     QTest::addColumn<QString>("localeName");
@@ -1204,10 +1407,7 @@ void tst_QLocale::macDefaultLocale()
     QCOMPARE(locale.decimalPoint(), QChar('.'));
     QCOMPARE(locale.groupSeparator(), QChar(','));
     QCOMPARE(locale.dateFormat(QLocale::ShortFormat), QString("M/d/yy"));
-    if (QSysInfo::MacintoshVersion > QSysInfo::MV_10_6)
-        QCOMPARE(locale.dateFormat(QLocale::LongFormat), QString("MMMM d, y"));
-    else
-        QCOMPARE(locale.dateFormat(QLocale::LongFormat), QString("MMMM d, yyyy"));
+    QCOMPARE(locale.dateFormat(QLocale::LongFormat), QString("MMMM d, yyyy"));
     QCOMPARE(locale.timeFormat(QLocale::ShortFormat), QString("h:mm AP"));
     QCOMPARE(locale.timeFormat(QLocale::LongFormat), QString("h:mm:ss AP t"));
 
@@ -1281,7 +1481,7 @@ static QString getWinLocaleInfo(LCTYPE type)
     int cnt = GetLocaleInfo(id, type, 0, 0) * 2;
 
     if (cnt == 0) {
-        qWarning("QLocale: empty windows locale info (%d)", type);
+        qWarning().nospace() << "QLocale: empty windows locale info (" <<  type << ')';
         return QString();
     }
     cnt /= sizeof(wchar_t);
@@ -1289,7 +1489,7 @@ static QString getWinLocaleInfo(LCTYPE type)
     cnt = GetLocaleInfo(id, type, buf.data(), cnt);
 
     if (cnt == 0) {
-        qWarning("QLocale: empty windows locale info (%d)", type);
+        qWarning().nospace() << "QLocale: empty windows locale info (" << type << ')';
         return QString();
     }
     return QString::fromWCharArray(buf.data());
@@ -1533,6 +1733,11 @@ void tst_QLocale::dayName()
     QLocale ir("ga_IE");
     QCOMPARE(ir.dayName(1, QLocale::ShortFormat), QLatin1String("Luan"));
     QCOMPARE(ir.dayName(7, QLocale::ShortFormat), QLatin1String("Domh"));
+
+    QLocale gr("el_GR");
+    QCOMPARE(gr.dayName(2, QLocale::ShortFormat), QString::fromUtf8("\316\244\317\201\316\257"));
+    QCOMPARE(gr.dayName(4, QLocale::ShortFormat), QString::fromUtf8("\316\240\316\255\316\274"));
+    QCOMPARE(gr.dayName(6, QLocale::ShortFormat), QString::fromUtf8("\316\243\316\254\316\262"));
 }
 
 void tst_QLocale::standaloneDayName_data()
@@ -1616,10 +1821,10 @@ void tst_QLocale::defaultNumeringSystem()
     QCOMPARE(pa.toString(123), QLatin1String("123"));
 
     QLocale ne("ne_IN");
-    QCOMPARE(ne.toString(123), QLatin1String("123"));
+    QCOMPARE(ne.toString(123), QString::fromUtf8("१२३"));
 
     QLocale mr("mr_IN");
-    QCOMPARE(mr.toString(123), QLatin1String("123"));
+    QCOMPARE(mr.toString(123), QString::fromUtf8("१२३"));
 
     QLocale ml("ml_IN");
     QCOMPARE(ml.toString(123), QLatin1String("123"));
@@ -1680,6 +1885,9 @@ void tst_QLocale::dateFormat()
 
     const QLocale ja("ja_JP");
     QCOMPARE(ja.dateFormat(QLocale::ShortFormat), QLatin1String("yyyy/MM/dd"));
+
+    const QLocale ir("ga_IE");
+    QCOMPARE(ir.dateFormat(QLocale::ShortFormat), QLatin1String("dd/MM/yyyy"));
 }
 
 void tst_QLocale::timeFormat()
@@ -1700,6 +1908,10 @@ void tst_QLocale::timeFormat()
     const QLocale cat("ca_ES");
     QCOMPARE(cat.timeFormat(QLocale::ShortFormat), QLatin1String("H.mm"));
     QCOMPARE(cat.timeFormat(QLocale::LongFormat), QLatin1String("H.mm.ss t"));
+
+    const QLocale bra("pt_BR");
+    QCOMPARE(bra.timeFormat(QLocale::ShortFormat), QLatin1String("HH:mm"));
+    QCOMPARE(bra.timeFormat(QLocale::LongFormat), QLatin1String("HH:mm:ss t"));
 }
 
 void tst_QLocale::dateTimeFormat()
@@ -1939,5 +2151,44 @@ void tst_QLocale::QTBUG_26035_positivesign()
     QVERIFY(ok);
 }
 
+void tst_QLocale::textDirection_data()
+{
+    QTest::addColumn<int>("language");
+    QTest::addColumn<int>("script");
+    QTest::addColumn<bool>("rightToLeft");
+
+    for (int language = QLocale::C; language <= QLocale::LastLanguage; ++language) {
+        bool rightToLeft = false;
+        switch (language) {
+        case QLocale::Arabic:
+        case QLocale::Hebrew:
+        case QLocale::Kashmiri:
+        case QLocale::Persian:
+        case QLocale::Pashto:
+        case QLocale::Urdu:
+        case QLocale::Syriac:
+        case QLocale::Divehi:
+            rightToLeft = QLocale(QLocale::Language(language)).language() == QLocale::Language(language); // false if there is no locale data for language
+            break;
+        default:
+            break;
+        }
+        QString testName = QLocalePrivate::languageToCode(QLocale::Language(language));
+        QTest::newRow(testName.toLatin1().constData()) << language << int(QLocale::AnyScript) << rightToLeft;
+    }
+    QTest::newRow("pa_Arab") << int(QLocale::Punjabi) << int(QLocale::ArabicScript) << true;
+    QTest::newRow("uz_Arab") << int(QLocale::Uzbek) << int(QLocale::ArabicScript) << true;
+}
+
+void tst_QLocale::textDirection()
+{
+    QFETCH(int, language);
+    QFETCH(int, script);
+    QFETCH(bool, rightToLeft);
+
+    QLocale locale(QLocale::Language(language), QLocale::Script(script), QLocale::AnyCountry);
+    QCOMPARE(locale.textDirection() == Qt::RightToLeft, rightToLeft);
+}
+
 QTEST_MAIN(tst_QLocale)
 #include "tst_qlocale.moc"
