@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2012 Digia Plc and/or its subsidiary(-ies).
+** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
 ** This file is part of the test suite of the Qt Toolkit.
@@ -86,6 +86,7 @@ private slots:
 #endif
 
     void dotsPerMeterZero();
+    void dotsPerMeterAndDpi();
 
     void convertToFormatPreserveDotsPrMeter();
     void convertToFormatPreserveText();
@@ -727,6 +728,24 @@ void tst_QImage::convertToFormat_data()
                                             << int(QImage::Format_RGB888) << 0xffffffffu;
     QTest::newRow("semiblack pm -> rgb888") << int(QImage::Format_ARGB32_Premultiplied) << 0x7f000000u
                                             << int(QImage::Format_RGB888) << 0xff000000u;
+
+    QTest::newRow("red rgba8888 -> argb32") << int(QImage::Format_RGBA8888) << 0xffff0000
+                                      << int(QImage::Format_ARGB32) << 0xffff0000;
+    QTest::newRow("green rgba8888 -> argb32") << int(QImage::Format_RGBA8888) << 0xff00ff00
+                                        << int(QImage::Format_ARGB32) << 0xff00ff00;
+    QTest::newRow("blue rgba8888 -> argb32") << int(QImage::Format_RGBA8888) << 0xff0000ff
+                                       << int(QImage::Format_ARGB32) << 0xff0000ff;
+
+    QTest::newRow("semired rgba8888 -> argb pm") << int(QImage::Format_RGBA8888) << 0x7fff0000u
+                                       << int(QImage::Format_ARGB32_Premultiplied) << 0x7f7f0000u;
+    QTest::newRow("semigreen rgba8888 -> argb pm") << int(QImage::Format_RGBA8888) << 0x7f00ff00u
+                                         << int(QImage::Format_ARGB32_Premultiplied) << 0x7f007f00u;
+    QTest::newRow("semiblue rgba8888 -> argb pm") << int(QImage::Format_RGBA8888) << 0x7f0000ffu
+                                        << int(QImage::Format_ARGB32_Premultiplied) << 0x7f00007fu;
+    QTest::newRow("semiwhite rgba8888 -> argb pm") << int(QImage::Format_RGBA8888) << 0x7fffffffu
+                                         << int(QImage::Format_ARGB32_Premultiplied) << 0x7f7f7f7fu;
+    QTest::newRow("semiblack rgba8888 -> argb pm") << int(QImage::Format_RGBA8888) << 0x7f000000u
+                                         << int(QImage::Format_ARGB32_Premultiplied) << 0x7f000000u;
 }
 
 
@@ -899,6 +918,27 @@ void tst_QImage::dotsPerMeterZero()
 
     QCOMPARE(img.dotsPerMeterX(), defaultDpmX);
     QCOMPARE(img.dotsPerMeterY(), defaultDpmY);
+
+}
+
+// verify that setting dotsPerMeter has an effect on the dpi.
+void tst_QImage::dotsPerMeterAndDpi()
+{
+    QImage img(100, 100, QImage::Format_RGB32);
+    QVERIFY(!img.isNull());
+
+    QPoint defaultLogicalDpi(img.logicalDpiX(), img.logicalDpiY());
+    QPoint defaultPhysicalDpi(img.physicalDpiX(), img.physicalDpiY());
+
+    img.setDotsPerMeterX(100);  // set x
+    QCOMPARE(img.logicalDpiY(), defaultLogicalDpi.y()); // no effect on y
+    QCOMPARE(img.physicalDpiY(), defaultPhysicalDpi.y());
+    QVERIFY(img.logicalDpiX() != defaultLogicalDpi.x()); // x changed
+    QVERIFY(img.physicalDpiX() != defaultPhysicalDpi.x());
+
+    img.setDotsPerMeterY(200);  // set y
+    QVERIFY(img.logicalDpiY() != defaultLogicalDpi.y()); // y changed
+    QVERIFY(img.physicalDpiY() != defaultPhysicalDpi.y());
 }
 
 void tst_QImage::rotate_data()
@@ -931,6 +971,10 @@ void tst_QImage::rotate_data()
             << QImage::Format_RGB888 << d;
         QTest::newRow(qPrintable(title.arg("Format_Indexed8")))
             << QImage::Format_Indexed8 << d;
+        QTest::newRow(qPrintable(title.arg("Format_RGBX8888")))
+            << QImage::Format_RGBX8888 << d;
+        QTest::newRow(qPrintable(title.arg("Format_RGBA8888_Premultiplied")))
+            << QImage::Format_RGBA8888_Premultiplied << d;
     }
 }
 
@@ -1139,6 +1183,21 @@ void tst_QImage::setPixel_data()
                                   << 0xff00ff00 << 0x00ff00u;
     QTest::newRow("RGB888 blue") << int(QImage::Format_RGB888)
                                  << 0xff0000ff << 0x0000ffu;
+#if Q_BYTE_ORDER == Q_BIG_ENDIAN
+    QTest::newRow("RGBA8888 red") << int(QImage::Format_RGBA8888)
+                                << 0xffff0000u << 0xff0000ffu;
+    QTest::newRow("RGBA8888 green") << int(QImage::Format_RGBA8888)
+                                  << 0xff00ff00u << 0x00ff00ffu;
+    QTest::newRow("RGBA8888 blue") << int(QImage::Format_RGBA8888)
+                                 << 0xff0000ffu << 0x0000ffffu;
+#else
+    QTest::newRow("RGBA8888 red") << int(QImage::Format_RGBA8888)
+                                << 0xffff0000u << 0xff0000ffu;
+    QTest::newRow("RGBA8888 green") << int(QImage::Format_RGBA8888)
+                                  << 0xff00ff00u << 0xff00ff00u;
+    QTest::newRow("RGBA8888 blue") << int(QImage::Format_RGBA8888)
+                                 << 0xff0000ffu << 0xffff0000u;
+#endif
 }
 
 void tst_QImage::setPixel()
@@ -1162,6 +1221,9 @@ void tst_QImage::setPixel()
     case int(QImage::Format_RGB32):
     case int(QImage::Format_ARGB32):
     case int(QImage::Format_ARGB32_Premultiplied):
+    case int(QImage::Format_RGBX8888):
+    case int(QImage::Format_RGBA8888):
+    case int(QImage::Format_RGBA8888_Premultiplied):
     {
         for (int y = 0; y < h; ++y) {
             const quint32 *row = (const quint32*)(img.scanLine(y));
@@ -1879,6 +1941,8 @@ void tst_QImage::fillColor_data()
         "RGB888",
         "RGB444",
         "ARGB4444pm",
+        "RGBx8888",
+        "RGBA8888pm",
         0
     };
 
@@ -1895,7 +1959,9 @@ void tst_QImage::fillColor_data()
         QImage::Format_ARGB8555_Premultiplied,
         QImage::Format_RGB888,
         QImage::Format_RGB444,
-        QImage::Format_ARGB4444_Premultiplied
+        QImage::Format_ARGB4444_Premultiplied,
+        QImage::Format_RGBX8888,
+        QImage::Format_RGBA8888_Premultiplied,
     };
 
     for (int i=0; names[i] != 0; ++i) {
@@ -1913,6 +1979,7 @@ void tst_QImage::fillColor_data()
     QTest::newRow("RGB32, transparent") << QImage::Format_RGB32 << Qt::transparent << 0xff000000;
     QTest::newRow("ARGB32, transparent") << QImage::Format_ARGB32 << Qt::transparent << 0x00000000u;
     QTest::newRow("ARGB32pm, transparent") << QImage::Format_ARGB32_Premultiplied << Qt::transparent << 0x00000000u;
+    QTest::newRow("RGBA8888pm, transparent") << QImage::Format_RGBA8888_Premultiplied << Qt::transparent << 0x00000000u;
 }
 
 void tst_QImage::fillColor()
@@ -1991,6 +2058,8 @@ void tst_QImage::rgbSwapped_data()
     QTest::newRow("Format_ARGB8555_Premultiplied") << QImage::Format_ARGB8555_Premultiplied;
     QTest::newRow("Format_RGB888") << QImage::Format_RGB888;
     QTest::newRow("Format_RGB444") << QImage::Format_RGB444;
+    QTest::newRow("Format_RGBX8888") << QImage::Format_RGBX8888;
+    QTest::newRow("Format_RGBA8888_Premultiplied") << QImage::Format_RGBA8888_Premultiplied;
 }
 
 void tst_QImage::rgbSwapped()
@@ -2093,5 +2162,5 @@ void tst_QImage::cleanupFunctions()
 
 }
 
-QTEST_MAIN(tst_QImage)
+QTEST_GUILESS_MAIN(tst_QImage)
 #include "tst_qimage.moc"
