@@ -50,6 +50,7 @@
 #include <QImageWriter>
 #include <QPainter>
 #include <QSet>
+#include <QTemporaryDir>
 
 #ifdef Q_OS_UNIX // for geteuid()
 # include <sys/types.h>
@@ -81,8 +82,10 @@ private slots:
     void writeImage2_data();
     void writeImage2();
     void supportedFormats();
+    void supportedMimeTypes();
 
     void writeToInvalidDevice();
+    void testCanWrite();
 
     void supportsOption_data();
     void supportsOption();
@@ -352,6 +355,26 @@ void tst_QImageWriter::supportedFormats()
     QCOMPARE(formatSet.size(), formats.size());
 }
 
+void tst_QImageWriter::supportedMimeTypes()
+{
+    QList<QByteArray> mimeTypes = QImageWriter::supportedMimeTypes();
+    QList<QByteArray> sortedMimeTypes = mimeTypes;
+    qSort(sortedMimeTypes);
+
+    // check that the list is sorted
+    QCOMPARE(mimeTypes, sortedMimeTypes);
+
+    QSet<QByteArray> mimeTypeSet;
+    foreach (QByteArray mimeType, mimeTypes)
+        mimeTypeSet << mimeType;
+
+    // check the list as a minimum contains image/bmp
+    QVERIFY(mimeTypeSet.contains("image/bmp"));
+
+    // check that the list does not contain duplicates
+    QCOMPARE(mimeTypeSet.size(), mimeTypes.size());
+}
+
 void tst_QImageWriter::writeToInvalidDevice()
 {
     QLatin1String fileName("/these/directories/do/not/exist/001.png");
@@ -381,6 +404,28 @@ void tst_QImageWriter::writeToInvalidDevice()
     }
 }
 
+void tst_QImageWriter::testCanWrite()
+{
+    {
+        // device is not set
+        QImageWriter writer;
+        QVERIFY(!writer.canWrite());
+        QCOMPARE(writer.error(), QImageWriter::DeviceError);
+    }
+
+    {
+        // check if canWrite won't leave an empty file
+        QTemporaryDir dir;
+        QVERIFY(dir.isValid());
+        QString fileName(dir.path() + QLatin1String("/001.garble"));
+        QVERIFY(!QFileInfo(fileName).exists());
+        QImageWriter writer(fileName);
+        QVERIFY(!writer.canWrite());
+        QCOMPARE(writer.error(), QImageWriter::UnsupportedFormatError);
+        QVERIFY(!QFileInfo(fileName).exists());
+    }
+}
+
 void tst_QImageWriter::supportsOption_data()
 {
     QTest::addColumn<QString>("fileName");
