@@ -114,9 +114,11 @@ private slots:
 
     void writeToReadFromDataStream_data();
     void writeToReadFromDataStream();
+    void writeToReadFromOldDataStream();
 
     void colorInteger();
     void invalidQColor();
+    void validQColor();
 
     void debugStream_data();
     void debugStream();
@@ -451,8 +453,8 @@ void tst_QGuiVariant::vector2D()
     QVariant variant;
     QVector2D vector = qvariant_cast<QVector2D>(variant);
     QVERIFY(vector.isNull());
-    variant.setValue(QVector2D(0.1, 0.2));
-    QCOMPARE(QVector2D(0.1, 0.2), qvariant_cast<QVector2D>(variant));
+    variant.setValue(QVector2D(0.1f, 0.2f));
+    QCOMPARE(QVector2D(0.1f, 0.2f), qvariant_cast<QVector2D>(variant));
 
     void *pvector = QMetaType::create(QVariant::Vector2D, 0);
     QVERIFY(pvector);
@@ -525,6 +527,8 @@ void tst_QGuiVariant::writeToReadFromDataStream_data()
     QTest::newRow( "pointarray_valid" ) << QVariant::fromValue( QPolygon( QRect( 10, 10, 20, 20 ) ) ) << false;
     QTest::newRow( "region_invalid" ) << QVariant::fromValue( QRegion() ) << true;
     QTest::newRow( "region_valid" ) << QVariant::fromValue( QRegion( 10, 10, 20, 20 ) ) << false;
+    QTest::newRow("polygonf_invalid") << QVariant::fromValue(QPolygonF()) << true;
+    QTest::newRow("polygonf_valid") << QVariant::fromValue(QPolygonF(QRectF(10, 10, 20, 20))) << false;
 }
 
 void tst_QGuiVariant::invalidQColor()
@@ -537,6 +541,21 @@ void tst_QGuiVariant::invalidQColor()
     QVERIFY(!qvariant_cast<QColor>(va).isValid());
 }
 
+void tst_QGuiVariant::validQColor()
+{
+    QColor col(Qt::red);
+    QVariant va(col.name());
+    QVERIFY(va.canConvert(QVariant::Color));
+
+    QVERIFY(va.convert(QVariant::Color));
+
+    QVERIFY(col.isValid());
+
+    QVERIFY(va.convert(QVariant::String));
+
+    QCOMPARE(qvariant_cast<QString>(va), col.name());
+}
+
 void tst_QGuiVariant::colorInteger()
 {
     QVariant v = QColor(Qt::red);
@@ -609,6 +628,46 @@ void tst_QGuiVariant::writeToReadFromDataStream()
     }
 }
 
+void tst_QGuiVariant::writeToReadFromOldDataStream()
+{
+    QPolygonF polyF(QRectF(10, 10, 50, 50));
+    QVariant testVariant(polyF);
+    {
+        // Read into a variant and compare
+        QFile file(":/data/qpolygonf.bin");
+        QVERIFY(file.open(QIODevice::ReadOnly));
+        QDataStream dataFileStream(&file);
+        dataFileStream.setVersion(QDataStream::Qt_4_9);
+        QVariant readVariant;
+        dataFileStream >> readVariant;
+        QVERIFY(readVariant.type() == QMetaType::QPolygonF);
+        QCOMPARE(testVariant, readVariant);
+        file.close();
+    }
+    {
+        QByteArray variantData;
+        {
+            QDataStream varDataStream(&variantData, QIODevice::WriteOnly);
+            varDataStream << testVariant;
+        }
+        // Read into a bytearray and compare
+        QFile file(":/data/qpolygonf.bin");
+        QVERIFY(file.open(QIODevice::ReadOnly));
+        QDataStream dataFileStream(&file);
+        dataFileStream.setVersion(QDataStream::Qt_4_9);
+        int dummy;
+        dataFileStream >> dummy;
+        QByteArray polyData49;
+        dataFileStream >> polyData49;
+        file.close();
+        QByteArray polyData50;
+        QDataStream readVarData(variantData);
+        readVarData >> dummy;
+        readVarData >> polyData50;
+        QVERIFY(polyData49 == polyData50);
+    }
+}
+
 void tst_QGuiVariant::debugStream_data()
 {
     QTest::addColumn<QVariant>("variant");
@@ -636,7 +695,7 @@ void tst_QGuiVariant::implicitConstruction()
     // This is a compile-time test
     QVariant v;
 
-#define FOR_EACH_GUI_CLASS(F) \
+#define FOR_EACH_GUI_CLASS_BASE(F) \
     F(Font) \
     F(Pixmap) \
     F(Brush) \
@@ -647,7 +706,6 @@ void tst_QGuiVariant::implicitConstruction()
     F(Polygon) \
     F(Region) \
     F(Bitmap) \
-    F(Cursor) \
     F(KeySequence) \
     F(Pen) \
     F(TextLength) \
@@ -659,7 +717,16 @@ void tst_QGuiVariant::implicitConstruction()
     F(Vector3D) \
     F(Vector4D) \
     F(Quaternion) \
-    F(PolygonF) \
+    F(PolygonF)
+
+#ifndef QTEST_NO_CURSOR
+#  define FOR_EACH_GUI_CLASS(F) \
+    FOR_EACH_GUI_CLASS_BASE(F) \
+    F(Cursor)
+#else // !QTEST_NO_CURSOR
+#  define FOR_EACH_GUI_CLASS(F) \
+    FOR_EACH_GUI_CLASS_BASE(F)
+#endif // QTEST_NO_CURSOR
 
 #define CONSTRUCT(TYPE) \
     { \
