@@ -43,6 +43,7 @@
 #include <QtTest/QtTest>
 #include <qkeysequence.h>
 #include <qpa/qplatformtheme.h>
+#include <qpa/qplatformtheme_p.h>
 #include <private/qkeysequence_p.h>
 #include <private/qguiapplication_p.h>
 #include <QTranslator>
@@ -119,10 +120,8 @@ private slots:
     void symetricConstructors();
     void checkMultipleNames();
     void checkMultipleCodes();
-#ifndef Q_OS_MAC
     void mnemonic_data();
     void mnemonic();
-#endif
     void toString_data();
     void toString();
     void toStringFromKeycode_data();
@@ -133,16 +132,18 @@ private slots:
     void parseString();
     void fromString_data();
     void fromString();
+    void listToString_data();
+    void listToString();
+    void listFromString_data();
+    void listFromString();
 #ifdef QT_BUILD_INTERNAL
     void ensureSorted();
 #endif
     void standardKeys_data();
     void standardKeys();
     void keyBindings();
-#if !defined (Q_OS_MAC) && !defined (Q_OS_WINCE)
     void translated_data();
     void translated();
-#endif
     void i18nKeys_data();
     void i18nKeys();
 
@@ -302,16 +303,16 @@ void tst_QKeySequence::checkMultipleCodes()
 }
 
 /*
-* We must ensure that the keyBindings data is always sorted
+* We must ensure that the keyBindings data are always sorted by standardKey
 * so that we can safely perform binary searches.
 */
 #ifdef QT_BUILD_INTERNAL
 void tst_QKeySequence::ensureSorted()
 {
-    uint N = QKeySequencePrivate::numberOfKeyBindings;
-    uint val = QKeySequencePrivate::keyBindings[0].shortcut;
+    uint N = QPlatformThemePrivate::numberOfKeyBindings;
+    uint val = QPlatformThemePrivate::keyBindings[0].standardKey;
     for ( uint i = 1 ; i < N ; ++i) {
-        uint nextval = QKeySequencePrivate::keyBindings[i].shortcut;
+        uint nextval = QPlatformThemePrivate::keyBindings[i].standardKey;
         if (nextval < val)
             qDebug() << "Data not sorted at index " << i;
         QVERIFY(nextval >= val);
@@ -400,9 +401,11 @@ void tst_QKeySequence::keyBindings()
     QCOMPARE(bindings, expected);
 }
 
-#ifndef Q_OS_MAC
 void tst_QKeySequence::mnemonic_data()
 {
+#ifdef Q_OS_MAC
+    QSKIP("Test not applicable to Mac OS X");
+#endif
     QTest::addColumn<QString>("string");
     QTest::addColumn<QString>("key");
     QTest::addColumn<bool>("warning");
@@ -422,6 +425,7 @@ void tst_QKeySequence::mnemonic_data()
 
 void tst_QKeySequence::mnemonic()
 {
+#ifndef Q_OS_MAC
     QFETCH(QString, string);
     QFETCH(QString, key);
     QFETCH(bool, warning);
@@ -439,8 +443,8 @@ void tst_QKeySequence::mnemonic()
     QKeySequence res = QKeySequence(key);
 
     QCOMPARE(seq, res);
-}
 #endif
+}
 
 void tst_QKeySequence::toString_data()
 {
@@ -506,6 +510,11 @@ void tst_QKeySequence::toStringFromKeycode_data()
     QTest::newRow("A") << QKeySequence(Qt::Key_A) << "A";
     QTest::newRow("-1") << QKeySequence(-1) << "";
     QTest::newRow("Unknown") << QKeySequence(Qt::Key_unknown) << "";
+    QTest::newRow("Ctrl+Num+Ins") << QKeySequence(Qt::ControlModifier | Qt::KeypadModifier | Qt::Key_Insert) << "Ctrl+Num+Ins";
+    QTest::newRow("Ctrl+Num+Del") << QKeySequence(Qt::ControlModifier | Qt::KeypadModifier | Qt::Key_Delete) << "Ctrl+Num+Del";
+    QTest::newRow("Ctrl+Alt+Num+Del") << QKeySequence(Qt::ControlModifier | Qt::AltModifier | Qt::KeypadModifier | Qt::Key_Delete) << "Ctrl+Alt+Num+Del";
+    QTest::newRow("Ctrl+Ins") << QKeySequence(Qt::ControlModifier | Qt::Key_Insert) << "Ctrl+Ins";
+    QTest::newRow("Ctrl+Num+Ins(1)") << QKeySequence(Qt::Key_Insert | Qt::KeypadModifier | Qt::ControlModifier) << "Ctrl+Num+Ins";
 }
 
 void tst_QKeySequence::toStringFromKeycode()
@@ -596,11 +605,6 @@ void tst_QKeySequence::parseString()
     QFETCH( QString, strSequence );
     QFETCH( QKeySequence, keycode );
 
-#ifdef Q_OS_MAC
-    QEXPECT_FAIL("Win+A", "QTBUG-24406 - This test fails on OSX", Abort);
-    QEXPECT_FAIL("Simon+G", "QTBUG-24406 - This test fails on OSX", Abort);
-#endif
-
     QCOMPARE( QKeySequence(strSequence).toString(), keycode.toString() );
     QVERIFY( QKeySequence(strSequence) == keycode );
 }
@@ -631,9 +635,110 @@ void tst_QKeySequence::fromString()
     QCOMPARE(ks4, ks1);
 }
 
-#if !defined (Q_OS_MAC) && !defined (Q_OS_WINCE)
+void tst_QKeySequence::listToString_data()
+{
+    QTest::addColumn<QString>("strSequences");
+    QTest::addColumn<QList<QKeySequence> >("sequences");
+
+    QList<QKeySequence> sequences;
+
+    sequences << QKeySequence(Qt::CTRL + Qt::Key_Left)
+              << QKeySequence(Qt::META + Qt::Key_A);
+    QTest::newRow("Ctrl+Left; Meta+A") << "Ctrl+Left; Meta+A" << sequences;
+
+    sequences.clear();
+    sequences << QKeySequence(Qt::CTRL + Qt::Key_Semicolon)
+              << QKeySequence(Qt::META + Qt::Key_A);
+    QTest::newRow("Ctrl+;; Meta+A") << "Ctrl+;; Meta+A" << sequences;
+
+    sequences.clear();
+    sequences << QKeySequence(Qt::Key_Semicolon)
+              << QKeySequence(Qt::META + Qt::Key_A);
+    QTest::newRow(";; Meta+A") << ";; Meta+A" << sequences;
+
+    sequences.clear();
+    sequences << QKeySequence(Qt::CTRL + Qt::Key_Left)
+              << QKeySequence(Qt::META + Qt::Key_Semicolon);
+    QTest::newRow("Ctrl+Left; Meta+;") << "Ctrl+Left; Meta+;" << sequences;
+
+    sequences.clear();
+    sequences << QKeySequence(Qt::CTRL + Qt::Key_Left)
+              << QKeySequence();
+    QTest::newRow("Ctrl+Left; ") << "Ctrl+Left; " << sequences;
+
+    sequences.clear();
+    sequences << QKeySequence(Qt::CTRL + Qt::Key_Left)
+              << QKeySequence()
+              << QKeySequence(Qt::META + Qt::Key_A);
+    QTest::newRow("Ctrl+Left; ; Meta+A") << "Ctrl+Left; ; Meta+A" << sequences;
+}
+
+void tst_QKeySequence::listToString()
+{
+    QFETCH(QList<QKeySequence>, sequences);
+    QFETCH(QString, strSequences);
+
+    QCOMPARE(QKeySequence::listToString(sequences), strSequences);
+}
+
+void tst_QKeySequence::listFromString_data()
+{
+    QTest::addColumn<QString>("strSequences");
+    QTest::addColumn<QList<QKeySequence> >("sequences");
+
+    QList<QKeySequence> sequences;
+
+    sequences << QKeySequence(Qt::CTRL + Qt::Key_Left)
+              << QKeySequence(Qt::META + Qt::Key_A);
+    QTest::newRow("Ctrl+Left; Meta+A") << "Ctrl+Left; Meta+A" << sequences;
+
+    sequences.clear();
+    sequences << QKeySequence(Qt::CTRL + Qt::Key_Semicolon)
+              << QKeySequence(Qt::META + Qt::Key_A);
+    QTest::newRow("Ctrl+;; Meta+A") << "Ctrl+;; Meta+A" << sequences;
+
+    sequences.clear();
+    sequences << QKeySequence(Qt::Key_Semicolon)
+              << QKeySequence(Qt::META + Qt::Key_A);
+    QTest::newRow(";; Meta+A") << ";; Meta+A" << sequences;
+
+    sequences.clear();
+    sequences << QKeySequence(Qt::CTRL + Qt::Key_Left)
+              << QKeySequence(Qt::META + Qt::Key_Semicolon);
+    QTest::newRow("Ctrl+Left; Meta+;") << "Ctrl+Left; Meta+;" << sequences;
+
+    sequences.clear();
+    sequences << QKeySequence(Qt::CTRL + Qt::Key_Left)
+              << QKeySequence();
+    QTest::newRow("Ctrl+Left; ") << "Ctrl+Left; " << sequences;
+
+    sequences.clear();
+    sequences << QKeySequence(Qt::CTRL + Qt::Key_Left)
+              << QKeySequence()
+              << QKeySequence(Qt::META + Qt::Key_A);
+    QTest::newRow("Ctrl+Left; ; Meta+A") << "Ctrl+Left; ; Meta+A" << sequences;
+
+    sequences.clear();
+    sequences << QKeySequence(Qt::CTRL + Qt::Key_Left)
+              << QKeySequence(Qt::Key_unknown)
+              << QKeySequence(Qt::META + Qt::Key_A);
+    QTest::newRow("Ctrl+Left; 4+3=2; Meta+A") << "Ctrl+Left; 4+3=2; Meta+A" << sequences;
+}
+
+void tst_QKeySequence::listFromString()
+{
+    QFETCH(QList<QKeySequence>, sequences);
+    QFETCH(QString, strSequences);
+
+    QCOMPARE(QKeySequence::listFromString(strSequences), sequences);
+}
+
 void tst_QKeySequence::translated_data()
 {
+#if defined (Q_OS_MAC) || defined (Q_OS_WINCE)
+    QSKIP("Test not applicable");
+#endif
+
     qApp->installTranslator(ourTranslator);
     qApp->installTranslator(qtTranslator);
 
@@ -661,6 +766,7 @@ void tst_QKeySequence::translated_data()
 
 void tst_QKeySequence::translated()
 {
+#if !defined (Q_OS_MAC) && !defined (Q_OS_WINCE)
     QFETCH(QString, transKey);
     QFETCH(QString, compKey);
 
@@ -672,8 +778,8 @@ void tst_QKeySequence::translated()
 
     qApp->removeTranslator(ourTranslator);
     qApp->removeTranslator(qtTranslator);
-}
 #endif
+}
 
 void tst_QKeySequence::i18nKeys_data()
 {
