@@ -41,6 +41,8 @@
 
 #include <qwindow.h>
 #include <qpa/qwindowsysteminterface.h>
+#include <qpa/qplatformintegration.h>
+#include <private/qguiapplication_p.h>
 
 #include <QtTest/QtTest>
 
@@ -49,6 +51,7 @@
 
 // For QSignalSpy slot connections.
 Q_DECLARE_METATYPE(Qt::ScreenOrientation)
+Q_DECLARE_METATYPE(QWindow::Visibility)
 
 class tst_QWindow: public QObject
 {
@@ -56,6 +59,7 @@ class tst_QWindow: public QObject
 
 private slots:
     void eventOrderOnShow();
+    void resizeEventAfterResize();
     void mapGlobal();
     void positioning();
     void isExposed();
@@ -75,6 +79,8 @@ private slots:
     void inputReentrancy();
     void tabletEvents();
     void windowModality_QTBUG27039();
+    void visibility();
+    void mask();
 
     void initTestCase()
     {
@@ -163,13 +169,31 @@ void tst_QWindow::eventOrderOnShow()
     QVERIFY(window.eventIndex(QEvent::Resize) < window.eventIndex(QEvent::Expose));
 }
 
+void tst_QWindow::resizeEventAfterResize()
+{
+    // Some platforms enforce minimum widths for windows, which can cause extra resize
+    // events, so set the width to suitably large value to avoid those.
+    QRect geometry(QGuiApplication::primaryScreen()->availableGeometry().topLeft() + QPoint(20, 20), QSize(300, 40));
+
+    Window window;
+    window.setGeometry(geometry);
+    window.showNormal();
+
+    QTRY_COMPARE(window.received(QEvent::Resize), 1);
+
+    // QTBUG-32706
+    // Make sure we get a resizeEvent after calling resize
+    window.resize(400, 100);
+
+    QTRY_COMPARE(window.received(QEvent::Resize), 2);
+}
+
 void tst_QWindow::positioning()
 {
-#ifdef Q_OS_MAC
-    // the fullscreen animation delay on OS X Lion also causes failures in
-    // the isActive() test below, so it's best to just skip it for now
-    QSKIP("Multiple failures in this test on Mac OS X, see QTBUG-23059");
-#endif
+    if (!QGuiApplicationPrivate::platformIntegration()->hasCapability(
+                QPlatformIntegration::NonFullScreenWindows)) {
+        QSKIP("This platform does not support non-fullscreen windows");
+    }
 
     // Some platforms enforce minimum widths for windows, which can cause extra resize
     // events, so set the width to suitably large value to avoid those.
@@ -182,7 +206,8 @@ void tst_QWindow::positioning()
     QCOMPARE(window.geometry().size(), size);
     window.setGeometry(geometry);
     QCOMPARE(window.geometry(), geometry);
-    window.show();
+    //  explicitly use non-fullscreen show. show() can be fullscreen on some platforms
+    window.showNormal();
     QCoreApplication::processEvents();
 
     QTRY_COMPARE(window.received(QEvent::Resize), 1);
@@ -198,6 +223,9 @@ void tst_QWindow::positioning()
 
     window.setWindowState(Qt::WindowFullScreen);
     QCoreApplication::processEvents();
+#ifdef Q_OS_OSX
+    QEXPECT_FAIL("", "Multiple failures in this test on Mac OS X, see QTBUG-23059", Abort);
+#endif
     QTRY_COMPARE(window.received(QEvent::Resize), 2);
 
     window.setWindowState(Qt::WindowNoState);
@@ -211,7 +239,7 @@ void tst_QWindow::positioning()
     // if our positioning is actually fully respected by the window manager
     // test whether it correctly handles frame positioning as well
     if (originalPos == geometry.topLeft() && (originalMargins.top() != 0 || originalMargins.left() != 0)) {
-        QPoint framePos(40, 40);
+        QPoint framePos = QGuiApplication::primaryScreen()->availableVirtualGeometry().topLeft() + QPoint(40, 40);
 
         window.reset();
         window.setFramePosition(framePos);
@@ -426,7 +454,7 @@ void tst_QWindow::testInputEvents()
 {
     InputTestWindow window;
     window.setGeometry(80, 80, 40, 40);
-    window.show();
+    window.showNormal();
     QVERIFY(QTest::qWaitForWindowExposed(&window));
 
     QWindowSystemInterface::handleKeyEvent(&window, QEvent::KeyPress, Qt::Key_A, Qt::NoModifier);
@@ -481,6 +509,8 @@ void tst_QWindow::testInputEvents()
 
 void tst_QWindow::touchToMouseTranslation()
 {
+    if (!QGuiApplicationPrivate::platformIntegration()->styleHint(QPlatformIntegration::SynthesizeMouseFromTouchEvents).toBool())
+        QSKIP("Mouse events are synthesized by the system on this platform.");
     InputTestWindow window;
     window.ignoreTouch = true;
     window.setGeometry(80, 80, 40, 40);
@@ -681,6 +711,8 @@ void tst_QWindow::touchCancel()
 
 void tst_QWindow::touchCancelWithTouchToMouse()
 {
+    if (!QGuiApplicationPrivate::platformIntegration()->styleHint(QPlatformIntegration::SynthesizeMouseFromTouchEvents).toBool())
+        QSKIP("Mouse events are synthesized by the system on this platform.");
     InputTestWindow window;
     window.ignoreTouch = true;
     window.setGeometry(80, 80, 40, 40);
@@ -811,7 +843,10 @@ void tst_QWindow::activateAndClose()
 {
     for (int i = 0; i < 10; ++i)  {
        QWindow window;
-       window.show();
+       // qWaitForWindowActive will block for the duration of
+       // of the timeout if the window is at 0,0
+       window.setGeometry(QGuiApplication::primaryScreen()->availableGeometry().adjusted(1, 1, -1, -1));
+       window.showNormal();
        window.requestActivate();
        QVERIFY(QTest::qWaitForWindowActive(&window));
        QCOMPARE(qGuiApp->focusWindow(), &window);
@@ -1079,6 +1114,60 @@ void tst_QWindow::windowModality_QTBUG27039()
     QCOMPARE(modalA.mousePressedCount, 1);
 }
 
+void tst_QWindow::visibility()
+{
+    qRegisterMetaType<Qt::WindowModality>("QWindow::Visibility");
+
+    QWindow window;
+    QSignalSpy spy(&window, SIGNAL(visibilityChanged(QWindow::Visibility)));
+
+    window.setVisibility(QWindow::AutomaticVisibility);
+    QVERIFY(window.isVisible());
+    QVERIFY(window.visibility() != QWindow::Hidden);
+    QVERIFY(window.visibility() != QWindow::AutomaticVisibility);
+    QCOMPARE(spy.count(), 1);
+    spy.clear();
+
+    window.setVisibility(QWindow::Hidden);
+    QVERIFY(!window.isVisible());
+    QCOMPARE(window.visibility(), QWindow::Hidden);
+    QCOMPARE(spy.count(), 1);
+    spy.clear();
+
+    window.setVisibility(QWindow::FullScreen);
+    QVERIFY(window.isVisible());
+    QCOMPARE(window.windowState(), Qt::WindowFullScreen);
+    QCOMPARE(window.visibility(), QWindow::FullScreen);
+    QCOMPARE(spy.count(), 1);
+    spy.clear();
+
+    window.setWindowState(Qt::WindowNoState);
+    QCOMPARE(window.visibility(), QWindow::Windowed);
+    QCOMPARE(spy.count(), 1);
+    spy.clear();
+
+    window.setVisible(false);
+    QCOMPARE(window.visibility(), QWindow::Hidden);
+    QCOMPARE(spy.count(), 1);
+    spy.clear();
+}
+
+void tst_QWindow::mask()
+{
+    QRegion mask = QRect(10, 10, 800 - 20, 600 - 20);
+
+    QWindow window;
+    window.resize(800, 600);
+    window.setMask(mask);
+
+    QCOMPARE(window.mask(), QRegion());
+
+    window.create();
+    window.setMask(mask);
+
+    QCOMPARE(window.mask(), mask);
+}
+
 #include <tst_qwindow.moc>
 QTEST_MAIN(tst_QWindow)
 
