@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2012 Digia Plc and/or its subsidiary(-ies).
+** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
 ** This file is part of the test suite of the Qt Toolkit.
@@ -502,6 +502,10 @@ void tst_QGlyphRun::drawMultiScriptText2()
     drawGlyphs.save("drawMultiScriptText2_drawGlyphIndexes.png");
 #endif
 
+#ifdef Q_OS_OSX
+    if (drawGlyphs.toImage() != textLayoutDraw.toImage())
+        QEXPECT_FAIL("", "See QTBUG-32690", Continue);
+#endif // Q_OS_OSX
     QCOMPARE(drawGlyphs, textLayoutDraw);
 }
 
