@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2012 Digia Plc and/or its subsidiary(-ies).
+** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
 ** This file is part of the test suite of the Qt Toolkit.
@@ -54,6 +54,7 @@
 #include <qabstracttextdocumentlayout.h>
 #include <qtextlist.h>
 #include <qtextcodec.h>
+#include <qguiapplication.h>
 #include <qurl.h>
 #include <qpainter.h>
 #include <qfontmetrics.h>
@@ -113,6 +114,7 @@ private slots:
     void toHtmlBodyBgColorRgba();
     void toHtmlBodyBgColorTransparent();
     void toHtmlRootFrameProperties();
+    void toHtmlLineHeightProperties();
     void capitalizationHtmlInExport();
     void wordspacingHtmlExport();
 
@@ -185,6 +187,7 @@ private slots:
     void htmlExportImportBlockCount();
 
     void QTBUG27354_spaceAndSoftSpace();
+    void QTBUG28998_linkColor();
 
 private:
     void backgroundImage_checkExpectedHtml(const QTextDocument &doc);
@@ -1856,6 +1859,25 @@ void tst_QTextDocument::toHtmlRootFrameProperties()
     QCOMPARE(doc.toHtml(), expectedOutput);
 }
 
+void tst_QTextDocument::toHtmlLineHeightProperties()
+{
+    CREATE_DOC_AND_CURSOR();
+
+    QTextBlock block = doc.firstBlock();
+    QTextBlockFormat blockFormat = block.blockFormat();
+    blockFormat.setLineHeight(200, QTextBlockFormat::ProportionalHeight);
+    cursor.setBlockFormat(blockFormat);
+
+    cursor.insertText("Blah");
+    QString expectedOutput("<p DEFAULTBLOCKSTYLE line-height:200%;\">Blah</p>");
+
+    expectedOutput.prepend(htmlHead);
+    expectedOutput.replace("DEFAULTBLOCKSTYLE", "style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;");
+    expectedOutput.append(htmlTail);
+
+    QCOMPARE(doc.toHtml(), expectedOutput);
+}
+
 void tst_QTextDocument::capitalizationHtmlInExport()
 {
     doc->setPlainText("Test");
@@ -2925,5 +2947,29 @@ void tst_QTextDocument::QTBUG27354_spaceAndSoftSpace()
     }
 }
 
+void tst_QTextDocument::QTBUG28998_linkColor()
+{
+    QPalette pal;
+    pal.setColor(QPalette::Link, QColor("tomato"));
+    QGuiApplication::setPalette(pal);
+
+    QTextDocument doc;
+    doc.setHtml("<a href=\"http://www.qt-project.org\">Qt</a>");
+
+    QCOMPARE(doc.blockCount(), 1);
+    QTextBlock block = doc.firstBlock();
+    QVERIFY(block.isValid());
+
+    QTextFragment fragment = block.begin().fragment();
+    QVERIFY(fragment.isValid());
+
+    QTextCharFormat format = fragment.charFormat();
+    QVERIFY(format.isValid());
+    QVERIFY(format.isAnchor());
+    QCOMPARE(format.anchorHref(), QStringLiteral("http://www.qt-project.org"));
+
+    QCOMPARE(format.foreground(), pal.link());
+}
+
 QTEST_MAIN(tst_QTextDocument)
 #include "tst_qtextdocument.moc"
