@@ -78,7 +78,6 @@ private slots:
     void shareRegister();
     void textureCleanup();
 #endif
-    void graphicsViewClipping();
     void partialGLWidgetUpdates_data();
     void partialGLWidgetUpdates();
     void glWidgetWithAlpha();
@@ -98,6 +97,7 @@ private slots:
     void destroyFBOAfterContext();
     void threadImages();
     void nullRectCrash();
+    void graphicsViewClipping();
 };
 
 tst_QGL::tst_QGL()
@@ -874,7 +874,11 @@ void tst_QGL::graphicsViewClipping()
 
     scene.setSceneRect(view.viewport()->rect());
 
-    QTest::qWaitForWindowExposed(&view);
+    QVERIFY(QTest::qWaitForWindowExposed(&view));
+    #ifdef Q_OS_MAC
+        // The black rectangle jumps from the center to the upper left for some reason.
+        QTest::qWait(100);
+    #endif
 
     QTRY_VERIFY(widget->painted);
 
@@ -1284,7 +1288,7 @@ void tst_QGL::glFBOUseInGLWidget()
 
     FBOUseInGLWidget w;
     w.resize(100, 100);
-    w.show();
+    w.showNormal();
 
     QVERIFY(QTest::qWaitForWindowExposed(&w));
 
@@ -1305,14 +1309,17 @@ void tst_QGL::glWidgetReparent()
 {
     // Try it as a top-level first:
     GLWidget *widget = new GLWidget;
+    widget->setObjectName(QStringLiteral("glWidget1"));
     widget->setGeometry(0, 0, 200, 30);
     widget->show();
 
     QWidget grandParentWidget;
+    grandParentWidget.setObjectName(QStringLiteral("grandParentWidget"));
     grandParentWidget.setPalette(Qt::blue);
     QVBoxLayout grandParentLayout(&grandParentWidget);
 
     QWidget parentWidget(&grandParentWidget);
+    parentWidget.setObjectName(QStringLiteral("parentWidget"));
     grandParentLayout.addWidget(&parentWidget);
     parentWidget.setPalette(Qt::green);
     parentWidget.setAutoFillBackground(true);
@@ -1344,6 +1351,7 @@ void tst_QGL::glWidgetReparent()
     // Now do pretty much the same thing, but don't show the
     // widget first:
     widget = new GLWidget;
+    widget->setObjectName(QStringLiteral("glWidget2"));
     parentLayout.addWidget(widget);
 
     QVERIFY(QTest::qWaitForWindowExposed(&grandParentWidget));
@@ -1372,12 +1380,13 @@ void tst_QGL::glWidgetRenderPixmap()
 {
     RenderPixmapWidget *w = new RenderPixmapWidget;
 
-    QPixmap pm = w->renderPixmap(100, 100, false);
+    QSize pmSize = QSize(100, 100);
+    QPixmap pm = w->renderPixmap(pmSize.width(), pmSize.height(), false);
 
     delete w;
 
     QImage fb = pm.toImage().convertToFormat(QImage::Format_RGB32);
-    QImage reference(fb.size(), QImage::Format_RGB32);
+    QImage reference(pmSize, QImage::Format_RGB32);
     reference.fill(0xffff0000);
 
     QFUZZY_COMPARE_IMAGES(fb, reference);
@@ -1766,7 +1775,7 @@ void tst_QGL::clipTest()
 {
     ClipTestGLWidget glw;
     glw.resize(220, 220);
-    glw.show();
+    glw.showNormal();
 
     QVERIFY(QTest::qWaitForWindowExposed(&glw));
 
@@ -2003,6 +2012,7 @@ void tst_QGL::textureCleanup()
     QGLWidget w;
     w.resize(200,200);
     w.show();
+    QTest::qWaitForWindowExposed(&w);
     w.makeCurrent();
 
     // Test pixmaps which have been loaded via QPixmapCache are removed from the texture cache
