@@ -42,8 +42,10 @@
 
 #include <QtCore/QCoreApplication>
 #include <QtTest/QtTest>
+#ifdef QT_GUI_LIB
 #include <QtGui/QImage>
 #include <QtGui/QPixmap>
+#endif
 
 /* XPM test data for QPixmap, QImage tests (use drag cursors as example) */
 
@@ -136,10 +138,14 @@ private slots:
     void compare_tostring_data();
     void compareQStringLists();
     void compareQStringLists_data();
+    void compareQListInt();
+    void compareQListDouble();
+#ifdef QT_GUI_LIB
     void compareQPixmaps();
     void compareQPixmaps_data();
     void compareQImages();
     void compareQImages_data();
+#endif
 };
 
 static bool boolfunc() { return true; }
@@ -307,6 +313,21 @@ void tst_Cmptest::compareQStringLists()
     QCOMPARE(opA, opB);
 }
 
+void tst_Cmptest::compareQListInt()
+{
+    QList<int> int1; int1 << 1 << 2 << 3;
+    QList<int> int2; int2 << 1 << 2 << 4;
+    QCOMPARE(int1, int2);
+}
+
+void tst_Cmptest::compareQListDouble()
+{
+    QList<double> double1; double1 << 1.5 << 2 << 3;
+    QList<double> double2; double2 << 1 << 2 << 4;
+    QCOMPARE(double1, double2);
+}
+
+#ifdef QT_GUI_LIB
 void tst_Cmptest::compareQPixmaps_data()
 {
     QTest::addColumn<QPixmap>("opA");
@@ -358,6 +379,7 @@ void tst_Cmptest::compareQImages()
 
     QCOMPARE(opA, opB);
 }
+#endif
 
 QTEST_MAIN(tst_Cmptest)
 #include "tst_cmptest.moc"
