@@ -49,8 +49,10 @@
 #include <qstyle.h>
 #include <QVBoxLayout>
 #include <QSizeGrip>
-
-
+#include <QDesktopWidget>
+#include <private/qguiapplication_p.h>
+#include <qpa/qplatformtheme.h>
+#include <qpa/qplatformtheme_p.h>
 
 QT_FORWARD_DECLARE_CLASS(QDialog)
 
@@ -82,6 +84,7 @@ private slots:
 #endif
     void setVisible();
     void reject();
+    void snapToDefaultButton();
 
 private:
     QDialog *testWidget;
@@ -247,7 +250,7 @@ void tst_QDialog::showMaximized()
     QDialog dialog(0);
     dialog.setSizeGripEnabled(true);
 #ifndef QT_NO_SIZEGRIP
-    QSizeGrip *sizeGrip = qFindChild<QSizeGrip *>(&dialog);
+    QSizeGrip *sizeGrip = dialog.findChild<QSizeGrip *>();
     QVERIFY(sizeGrip);
 #endif
 
@@ -324,11 +327,10 @@ void tst_QDialog::showFullScreen()
     QDialog dialog(0, Qt::X11BypassWindowManagerHint);
     dialog.setSizeGripEnabled(true);
 #ifndef QT_NO_SIZEGRIP
-    QSizeGrip *sizeGrip = qFindChild<QSizeGrip *>(&dialog);
+    QSizeGrip *sizeGrip = dialog.findChild<QSizeGrip *>();
     QVERIFY(sizeGrip);
 #endif
 
-    qApp->syncX();
     dialog.showFullScreen();
     QVERIFY(dialog.isFullScreen());
     QVERIFY(dialog.isVisible());
@@ -336,7 +338,6 @@ void tst_QDialog::showFullScreen()
     QVERIFY(!sizeGrip->isVisible());
 #endif
 
-    qApp->syncX();
     dialog.showNormal();
     QVERIFY(!dialog.isFullScreen());
     QVERIFY(dialog.isVisible());
@@ -344,32 +345,26 @@ void tst_QDialog::showFullScreen()
     QVERIFY(sizeGrip->isVisible());
 #endif
 
-    qApp->syncX();
     dialog.showFullScreen();
     QVERIFY(dialog.isFullScreen());
     QVERIFY(dialog.isVisible());
 
-    qApp->syncX();
     dialog.hide();
     QVERIFY(dialog.isFullScreen());
     QVERIFY(!dialog.isVisible());
 
-    qApp->syncX();
     dialog.show();
     QVERIFY(dialog.isFullScreen());
     QVERIFY(dialog.isVisible());
 
-    qApp->syncX();
     dialog.hide();
     QVERIFY(dialog.isFullScreen());
     QVERIFY(!dialog.isVisible());
 
-    qApp->syncX();
     dialog.showFullScreen();
     QVERIFY(dialog.isFullScreen());
     QVERIFY(dialog.isVisible());
 
-    qApp->syncX();
     dialog.hide();
     QVERIFY(dialog.isFullScreen());
     QVERIFY(!dialog.isVisible());
@@ -446,7 +441,7 @@ void tst_QDialog::showSizeGrip()
     QVERIFY(!dialog.isSizeGripEnabled());
 
     dialog.setSizeGripEnabled(true);
-    QPointer<QSizeGrip> sizeGrip = qFindChild<QSizeGrip *>(&dialog);
+    QPointer<QSizeGrip> sizeGrip = dialog.findChild<QSizeGrip *>();
     QVERIFY(sizeGrip);
     QVERIFY(sizeGrip->isVisible());
     QVERIFY(dialog.isSizeGripEnabled());
@@ -464,7 +459,7 @@ void tst_QDialog::showSizeGrip()
     dialog.showExtension(false);
     QVERIFY(dialog.extension() && !dialog.extension()->isVisible());
     QVERIFY(dialog.isSizeGripEnabled());
-    sizeGrip = qFindChild<QSizeGrip *>(&dialog);
+    sizeGrip = dialog.findChild<QSizeGrip *>();
     QVERIFY(sizeGrip);
     QVERIFY(sizeGrip->isVisible());
 
@@ -482,7 +477,7 @@ void tst_QDialog::showSizeGrip()
     QVERIFY(!dialog.isSizeGripEnabled());
 
     dialog.setSizeGripEnabled(true);
-    sizeGrip = qFindChild<QSizeGrip *>(&dialog);
+    sizeGrip = dialog.findChild<QSizeGrip *>();
     QVERIFY(sizeGrip);
     QVERIFY(sizeGrip->isVisible());
     sizeGrip->hide();
@@ -562,6 +557,32 @@ void tst_QDialog::reject()
     QCOMPARE(dialog.called, 4);
 }
 
+void tst_QDialog::snapToDefaultButton()
+{
+#ifdef QT_NO_CURSOR
+    QSKIP("Test relies on there being a cursor");
+#else
+    QPoint topLeftPos = QApplication::desktop()->availableGeometry().topLeft();
+    topLeftPos = QPoint(topLeftPos.x() + 100, topLeftPos.y() + 100);
+    QPoint startingPos(topLeftPos.x() + 250, topLeftPos.y() + 250);
+    QCursor::setPos(startingPos);
+    QVERIFY(QCursor::pos() == startingPos);
+    QDialog dialog;
+    QPushButton *button = new QPushButton(&dialog);
+    button->setDefault(true);
+    dialog.setGeometry(QRect(topLeftPos, QSize(200, 200)));
+    dialog.show();
+    QVERIFY(QTest::qWaitForWindowExposed(&dialog));
+    if (const QPlatformTheme *theme = QGuiApplicationPrivate::platformTheme()) {
+        if (theme->themeHint(QPlatformTheme::DialogSnapToDefaultButton).toBool()) {
+            QPoint localPos = button->mapFromGlobal(QCursor::pos());
+            QVERIFY(button->rect().contains(localPos));
+        } else {
+            QVERIFY(startingPos == QCursor::pos());
+        }
+    }
+#endif // !QT_NO_CURSOR
+}
 
 QTEST_MAIN(tst_QDialog)
 #include "tst_qdialog.moc"
