@@ -44,6 +44,7 @@
 
 #include <qcoreapplication.h>
 #include <qdebug.h>
+#include <qsharedpointer.h>
 #include <qfiledialog.h>
 #include <qabstractitemdelegate.h>
 #include <qdirmodel.h>
@@ -65,6 +66,8 @@
 #include <private/qfilesystemmodel_p.h>
 #include <private/qfiledialog_p.h>
 #endif
+#include <private/qguiapplication_p.h>
+#include <qpa/qplatformtheme.h>
 #include <QFileDialog>
 #include <QFileSystemModel>
 
@@ -144,6 +147,7 @@ private slots:
     void clearLineEdit();
     void enableChooseButton();
     void hooks();
+    void widgetlessNativeDialog();
 #ifdef Q_OS_UNIX
 #ifdef QT_BUILD_INTERNAL
     void tildeExpansion_data();
@@ -200,7 +204,7 @@ void tst_QFiledialog::currentChangedSignal()
     fd.setViewMode(QFileDialog::List);
     QSignalSpy spyCurrentChanged(&fd, SIGNAL(currentChanged(QString)));
 
-    QListView* listView = qFindChild<QListView*>(&fd, "listView");
+    QListView* listView = fd.findChild<QListView*>("listView");
     QVERIFY(listView);
     fd.setDirectory(QDir::root());
     QModelIndex root = listView->rootIndex();
@@ -229,14 +233,14 @@ void tst_QFiledialog::directoryEnteredSignal()
     QSignalSpy spyDirectoryEntered(&fd, SIGNAL(directoryEntered(QString)));
 
     // sidebar
-    QSidebar *sidebar = qFindChild<QSidebar*>(&fd, "sidebar");
+    QSidebar *sidebar = fd.findChild<QSidebar*>("sidebar");
     sidebar->setCurrentIndex(sidebar->model()->index(1, 0));
     QTest::keyPress(sidebar->viewport(), Qt::Key_Return);
     QCOMPARE(spyDirectoryEntered.count(), 1);
     spyDirectoryEntered.clear();
 
     // lookInCombo
-    QComboBox *comboBox = qFindChild<QComboBox*>(&fd, "lookInCombo");
+    QComboBox *comboBox = fd.findChild<QComboBox*>("lookInCombo");
     comboBox->showPopup();
     QVERIFY(comboBox->view()->model()->index(1, 0).isValid());
     comboBox->view()->setCurrentIndex(comboBox->view()->model()->index(1, 0));
@@ -248,7 +252,7 @@ void tst_QFiledialog::directoryEnteredSignal()
     /*
     // platform specific
     fd.setViewMode(QFileDialog::ViewMode(QFileDialog::List));
-    QListView* listView = qFindChild<QListView*>(&fd, "listView");
+    QListView* listView = fd.findChild<QListView*>("listView");
     QVERIFY(listView);
     QModelIndex root = listView->rootIndex();
     QTRY_COMPARE(listView->model()->rowCount(root) > 0, true);
@@ -293,7 +297,7 @@ void tst_QFiledialog::filesSelectedSignal()
 
     fd.show();
     QVERIFY(QTest::qWaitForWindowExposed(&fd));
-    QListView *listView = qFindChild<QListView*>(&fd, "listView");
+    QListView *listView = fd.findChild<QListView*>("listView");
     QVERIFY(listView);
 
     QModelIndex root = listView->rootIndex();
@@ -314,7 +318,7 @@ void tst_QFiledialog::filesSelectedSignal()
     listView->selectionModel()->select(file, QItemSelectionModel::Select | QItemSelectionModel::Rows);
     listView->setCurrentIndex(file);
 
-    QDialogButtonBox *buttonBox = qFindChild<QDialogButtonBox*>(&fd, "buttonBox");
+    QDialogButtonBox *buttonBox = fd.findChild<QDialogButtonBox*>("buttonBox");
     QPushButton *button = buttonBox->button(QDialogButtonBox::Open);
     QVERIFY(button);
     QVERIFY(button->isEnabled());
@@ -338,7 +342,7 @@ void tst_QFiledialog::filterSelectedSignal()
     fd.setNameFilters(filterChoices);
     QCOMPARE(fd.nameFilters(), filterChoices);
 
-    QComboBox *filters = qFindChild<QComboBox*>(&fd, "fileTypeCombo");
+    QComboBox *filters = fd.findChild<QComboBox*>("fileTypeCombo");
     QVERIFY(filters);
     QVERIFY(filters->view());
     QCOMPARE(filters->isVisible(), true);
@@ -367,7 +371,7 @@ void tst_QFiledialog::directory()
 {
     QNonNativeFileDialog fd;
     fd.setViewMode(QFileDialog::List);
-    QFileSystemModel *model = qFindChild<QFileSystemModel*>(&fd, "qt_filesystem_model");
+    QFileSystemModel *model = fd.findChild<QFileSystemModel*>("qt_filesystem_model");
     QVERIFY(model);
     fd.setDirectory(QDir::currentPath());
     QSignalSpy spyCurrentChanged(&fd, SIGNAL(currentChanged(QString)));
@@ -392,7 +396,7 @@ void tst_QFiledialog::directory()
     QCOMPARE(spyFilterSelected.count(), 0);
 
     // Check my way
-    QList<QListView*> list = qFindChildren<QListView*>(&fd, "listView");
+    QList<QListView*> list = fd.findChildren<QListView*>("listView");
     QVERIFY(list.count() > 0);
 #ifdef Q_OS_WIN
     QCOMPARE(list.at(0)->rootIndex().data().toString().toLower(), temp.dirName().toLower());
@@ -441,33 +445,25 @@ void tst_QFiledialog::completer_data()
 
 void tst_QFiledialog::completer()
 {
+    typedef QSharedPointer<QTemporaryFile> TemporaryFilePtr;
+
     QFETCH(QString, input);
     QFETCH(QString, startPath);
     QFETCH(int, expected);
 
-    QString tempPath = QDir::tempPath() + '/' + "QFileDialogTestDir";
-    if (startPath.isEmpty())
-        startPath = tempPath;
+    QTemporaryDir tempDir;
+    QVERIFY(tempDir.isValid());
 
-    startPath = QDir::cleanPath(startPath);
+    const QString tempPath = tempDir.path();
+    startPath = startPath.isEmpty() ? tempPath : QDir::cleanPath(startPath);
 
     // make temp dir and files
-    {
-        QDir cleanup(tempPath);
-        QStringList x = cleanup.entryList();
-        for (int i = 0; i < x.count(); ++i)
-            QFile::remove(tempPath + '/' + x[i]);
-        cleanup.rmdir(tempPath);
-    }
-    QDir tmp(QDir::tempPath());
-    if (!tmp.exists(tempPath))
-        QVERIFY(tmp.mkdir("QFileDialogTestDir"));
-    QList<QTemporaryFile*> files;
+    QList<TemporaryFilePtr> files;
     QT_TRY {
     for (int i = 0; i < 10; ++i) {
-        QScopedPointer<QTemporaryFile> file(new QTemporaryFile(tempPath + "/rXXXXXX"));
-        file->open();
-        files.append(file.take());
+        TemporaryFilePtr file(new QTemporaryFile(tempPath + QStringLiteral("/rXXXXXX")));
+        QVERIFY(file->open());
+        files.append(file);
     }
 
     // ### flesh this out more
@@ -476,9 +472,9 @@ void tst_QFiledialog::completer()
     fd.show();
     QVERIFY(QTest::qWaitForWindowExposed(&fd));
     QVERIFY(fd.isVisible());
-    QFileSystemModel *model = qFindChild<QFileSystemModel*>(&fd, "qt_filesystem_model");
+    QFileSystemModel *model = fd.findChild<QFileSystemModel*>("qt_filesystem_model");
     QVERIFY(model);
-    QLineEdit *lineEdit = qFindChild<QLineEdit*>(&fd, "fileNameEdit");
+    QLineEdit *lineEdit = fd.findChild<QLineEdit*>("fileNameEdit");
     QVERIFY(lineEdit);
     QCompleter *completer = lineEdit->completer();
     QVERIFY(completer);
@@ -515,7 +511,7 @@ void tst_QFiledialog::completer()
 
     QStringList expectedFiles;
     if (expected == -1) {
-        QString fullPath = startPath.isEmpty() ? tempPath : startPath;
+        QString fullPath = startPath;
         if (!fullPath.endsWith(QLatin1Char('/')))
             fullPath.append(QLatin1Char('/'));
         fullPath.append(input);
@@ -551,10 +547,8 @@ void tst_QFiledialog::completer()
 
     QTRY_COMPARE(cModel->rowCount(), expected);
     } QT_CATCH(...) {
-        qDeleteAll(files);
         QT_RETHROW;
     }
-    qDeleteAll(files);
 }
 
 void tst_QFiledialog::completer_up()
@@ -567,7 +561,7 @@ void tst_QFiledialog::completer_up()
     QSignalSpy spyFilterSelected(&fd, SIGNAL(filterSelected(QString)));
 
     fd.show();
-    QLineEdit *lineEdit = qFindChild<QLineEdit*>(&fd, "fileNameEdit");
+    QLineEdit *lineEdit = fd.findChild<QLineEdit*>("fileNameEdit");
     QVERIFY(lineEdit);
     QCOMPARE(spyFilesSelected.count(), 0);
     int depth = QDir::currentPath().split('/').count();
@@ -586,7 +580,7 @@ void tst_QFiledialog::acceptMode()
     QNonNativeFileDialog fd;
     fd.show();
 
-    QToolButton* newButton = qFindChild<QToolButton*>(&fd, "newFolderButton");
+    QToolButton* newButton = fd.findChild<QToolButton*>("newFolderButton");
     QVERIFY(newButton);
 
     // default
@@ -621,6 +615,8 @@ void tst_QFiledialog::defaultSuffix()
     QCOMPARE(fd.defaultSuffix(), QString());
     fd.setDefaultSuffix("txt");
     QCOMPARE(fd.defaultSuffix(), QString("txt"));
+    fd.setDefaultSuffix(".txt");
+    QCOMPARE(fd.defaultSuffix(), QString("txt"));
     fd.setDefaultSuffix(QString());
     QCOMPARE(fd.defaultSuffix(), QString());
 }
@@ -658,7 +654,7 @@ void tst_QFiledialog::filters()
     QCOMPARE(fd.nameFilters(), QStringList("All Files (*)"));
 
     // effects
-    QList<QComboBox*> views = qFindChildren<QComboBox*>(&fd, "fileTypeCombo");
+    QList<QComboBox*> views = fd.findChildren<QComboBox*>("fileTypeCombo");
     QVERIFY(views.count() == 1);
     QCOMPARE(views.at(0)->isVisible(), false);
 
@@ -728,7 +724,7 @@ void tst_QFiledialog::history()
 {
     QNonNativeFileDialog fd;
     fd.setViewMode(QFileDialog::List);
-    QFileSystemModel *model = qFindChild<QFileSystemModel*>(&fd, "qt_filesystem_model");
+    QFileSystemModel *model = fd.findChild<QFileSystemModel*>("qt_filesystem_model");
     QVERIFY(model);
     QSignalSpy spyCurrentChanged(&fd, SIGNAL(currentChanged(QString)));
     QSignalSpy spyDirectoryEntered(&fd, SIGNAL(directoryEntered(QString)));
@@ -744,7 +740,7 @@ void tst_QFiledialog::history()
     if (fd.history() != history) {
         qDebug() << fd.history() << history;
         // quick and dirty output for windows failure.
-        QListView* list = qFindChild<QListView*>(&fd, "listView");
+        QListView* list = fd.findChild<QListView*>("listView");
         QVERIFY(list);
         QModelIndex root = list->rootIndex();
         while (root.isValid()) {
@@ -781,9 +777,9 @@ void tst_QFiledialog::isReadOnly()
 {
     QNonNativeFileDialog fd;
 
-    QPushButton* newButton = qFindChild<QPushButton*>(&fd, "newFolderButton");
-    QAction* renameAction = qFindChild<QAction*>(&fd, "qt_rename_action");
-    QAction* deleteAction = qFindChild<QAction*>(&fd, "qt_delete_action");
+    QPushButton* newButton = fd.findChild<QPushButton*>("newFolderButton");
+    QAction* renameAction = fd.findChild<QAction*>("qt_rename_action");
+    QAction* deleteAction = fd.findChild<QAction*>("qt_delete_action");
 
     QCOMPARE(fd.isReadOnly(), false);
 
@@ -862,7 +858,7 @@ void tst_QFiledialog::selectFile()
     QFETCH(QString, file);
     QFETCH(int, count);
     QNonNativeFileDialog fd;
-    QFileSystemModel *model = qFindChild<QFileSystemModel*>(&fd, "qt_filesystem_model");
+    QFileSystemModel *model = fd.findChild<QFileSystemModel*>("qt_filesystem_model");
     QVERIFY(model);
     fd.setDirectory(QDir::currentPath());
     // default value
@@ -888,9 +884,9 @@ void tst_QFiledialog::selectFiles()
 {
     QNonNativeFileDialog fd;
     fd.setViewMode(QFileDialog::List);
-    QString tempPath = QDir::tempPath() + '/' + "QFileDialogTestDir4SelectFiles";
-    QDir dir;
-    QVERIFY(dir.mkpath(tempPath));
+    QTemporaryDir tempDir;
+    QVERIFY(tempDir.isValid());
+    const QString tempPath = tempDir.path();
     fd.setDirectory(tempPath);
     QSignalSpy spyCurrentChanged(&fd, SIGNAL(currentChanged(QString)));
     QSignalSpy spyDirectoryEntered(&fd, SIGNAL(directoryEntered(QString)));
@@ -912,7 +908,7 @@ void tst_QFiledialog::selectFiles()
     QStringList list = fd.directory().entryList(QDir::Files);
     QModelIndexList toSelect;
     QVERIFY(list.count() > 1);
-    QListView* listView = qFindChild<QListView*>(&fd, "listView");
+    QListView* listView = fd.findChild<QListView*>("listView");
     QVERIFY(listView);
     for (int i = 0; i < list.count(); ++i) {
         fd.selectFile(fd.directory().path() + "/" + list.at(i));
@@ -934,18 +930,15 @@ void tst_QFiledialog::selectFiles()
     QCOMPARE(spyDirectoryEntered.count(), 0);
     QCOMPARE(spyFilesSelected.count(), 0);
     QCOMPARE(spyFilterSelected.count(), 0);
-    for (int i=0; i < 5; ++i)
-        QFile::remove(filesPath + QString::fromLatin1("/qfiledialog_auto_test_not_pres_%1").arg(i));
 
     //If the selection is invalid then we fill the line edit but without the /
     QNonNativeFileDialog * dialog = new QNonNativeFileDialog( 0, "Save" );
     dialog->setFileMode( QFileDialog::AnyFile );
     dialog->setAcceptMode( QFileDialog::AcceptSave );
-    QString temporary = QDir::tempPath() + QLatin1String("/blah");
-    dialog->selectFile(temporary);
+    dialog->selectFile(tempPath + QStringLiteral("/blah"));
     dialog->show();
     QVERIFY(QTest::qWaitForWindowExposed(dialog));
-    QLineEdit *lineEdit = qFindChild<QLineEdit*>(dialog, "fileNameEdit");
+    QLineEdit *lineEdit = dialog->findChild<QLineEdit*>("fileNameEdit");
     QVERIFY(lineEdit);
     QCOMPARE(lineEdit->text(),QLatin1String("blah"));
     delete dialog;
@@ -958,13 +951,13 @@ void tst_QFiledialog::viewMode()
     fd.show();
 
     // find widgets
-    QList<QTreeView*> treeView = qFindChildren<QTreeView*>(&fd, "treeView");
+    QList<QTreeView*> treeView = fd.findChildren<QTreeView*>("treeView");
     QCOMPARE(treeView.count(), 1);
-    QList<QListView*> listView = qFindChildren<QListView*>(&fd, "listView");
+    QList<QListView*> listView = fd.findChildren<QListView*>("listView");
     QCOMPARE(listView.count(), 1);
-    QList<QToolButton*> listButton = qFindChildren<QToolButton*>(&fd, "listModeButton");
+    QList<QToolButton*> listButton = fd.findChildren<QToolButton*>("listModeButton");
     QCOMPARE(listButton.count(), 1);
-    QList<QToolButton*> treeButton = qFindChildren<QToolButton*>(&fd, "detailModeButton");
+    QList<QToolButton*> treeButton = fd.findChildren<QToolButton*>("detailModeButton");
     QCOMPARE(treeButton.count(), 1);
 
     // default value
@@ -1068,9 +1061,8 @@ void tst_QFiledialog::focus()
 
     // make sure the tests work with focus follows mouse
     QCursor::setPos(fd.geometry().center());
-    QApplication::syncX();
 
-    QList<QWidget*> treeView = qFindChildren<QWidget*>(&fd, "fileNameEdit");
+    QList<QWidget*> treeView = fd.findChildren<QWidget*>("fileNameEdit");
     QCOMPARE(treeView.count(), 1);
     QVERIFY(treeView.at(0));
     QTRY_COMPARE(treeView.at(0)->hasFocus(), true);
@@ -1081,11 +1073,11 @@ void tst_QFiledialog::focus()
 void tst_QFiledialog::historyBack()
 {
     QNonNativeFileDialog fd;
-    QFileSystemModel *model = qFindChild<QFileSystemModel*>(&fd, "qt_filesystem_model");
+    QFileSystemModel *model = fd.findChild<QFileSystemModel*>("qt_filesystem_model");
     QVERIFY(model);
-    QToolButton *backButton = qFindChild<QToolButton*>(&fd, "backButton");
+    QToolButton *backButton = fd.findChild<QToolButton*>("backButton");
     QVERIFY(backButton);
-    QToolButton *forwardButton = qFindChild<QToolButton*>(&fd, "forwardButton");
+    QToolButton *forwardButton = fd.findChild<QToolButton*>("forwardButton");
     QVERIFY(forwardButton);
 
     QSignalSpy spy(model, SIGNAL(rootPathChanged(QString)));
@@ -1129,12 +1121,12 @@ void tst_QFiledialog::historyForward()
 {
     QNonNativeFileDialog fd;
     fd.setDirectory(QDir::currentPath());
-    QToolButton *backButton = qFindChild<QToolButton*>(&fd, "backButton");
+    QToolButton *backButton = fd.findChild<QToolButton*>("backButton");
     QVERIFY(backButton);
-    QToolButton *forwardButton = qFindChild<QToolButton*>(&fd, "forwardButton");
+    QToolButton *forwardButton = fd.findChild<QToolButton*>("forwardButton");
     QVERIFY(forwardButton);
 
-    QFileSystemModel *model = qFindChild<QFileSystemModel*>(&fd, "qt_filesystem_model");
+    QFileSystemModel *model = fd.findChild<QFileSystemModel*>("qt_filesystem_model");
     QVERIFY(model);
     QSignalSpy spy(model, SIGNAL(rootPathChanged(QString)));
 
@@ -1205,7 +1197,7 @@ void tst_QFiledialog::disableSaveButton()
 
     QNonNativeFileDialog fd(0, "caption", path);
     fd.setAcceptMode(QFileDialog::AcceptSave);
-    QDialogButtonBox *buttonBox = qFindChild<QDialogButtonBox*>(&fd, "buttonBox");
+    QDialogButtonBox *buttonBox = fd.findChild<QDialogButtonBox*>("buttonBox");
     QPushButton *button = buttonBox->button(QDialogButtonBox::Save);
     QVERIFY(button);
     QCOMPARE(button->isEnabled(), isEnabled);
@@ -1236,7 +1228,7 @@ void tst_QFiledialog::saveButtonText()
         fd.setLabelText(QFileDialog::Accept, label);
     fd.setDirectory(QDir::temp());
     fd.selectFile(path);
-    QDialogButtonBox *buttonBox = qFindChild<QDialogButtonBox*>(&fd, "buttonBox");
+    QDialogButtonBox *buttonBox = fd.findChild<QDialogButtonBox*>("buttonBox");
     QVERIFY(buttonBox);
     QPushButton *button = buttonBox->button(QDialogButtonBox::Save);
     QVERIFY(button);
@@ -1254,12 +1246,12 @@ void tst_QFiledialog::clearLineEdit()
     //play it really safe by creating a directory
     QDir::home().mkdir("_____aaaaaaaaaaaaaaaaaaaaaa");
 
-    QLineEdit *lineEdit = qFindChild<QLineEdit*>(&fd, "fileNameEdit");
+    QLineEdit *lineEdit = fd.findChild<QLineEdit*>("fileNameEdit");
     QVERIFY(lineEdit);
     QVERIFY(lineEdit->text() == "foo");
     fd.setDirectory(QDir::home());
 
-    QListView* list = qFindChild<QListView*>(&fd, "listView");
+    QListView* list = fd.findChild<QListView*>("listView");
     QVERIFY(list);
 
     // saving a file the text shouldn't be cleared
@@ -1309,7 +1301,7 @@ void tst_QFiledialog::enableChooseButton()
     QNonNativeFileDialog fd;
     fd.setFileMode(QFileDialog::Directory);
     fd.show();
-    QDialogButtonBox *buttonBox = qFindChild<QDialogButtonBox*>(&fd, "buttonBox");
+    QDialogButtonBox *buttonBox = fd.findChild<QDialogButtonBox*>("buttonBox");
     QPushButton *button = buttonBox->button(QDialogButtonBox::Open);
     QVERIFY(button);
     QCOMPARE(button->isEnabled(), true);
@@ -1347,6 +1339,38 @@ QString saveName(QWidget *, const QString &, const QString &, const QString &, Q
     return "saveName";
 }
 
+QT_BEGIN_NAMESPACE
+typedef QUrl (*_qt_filedialog_existing_directory_url_hook)(QWidget *parent, const QString &caption, const QUrl &dir, QFileDialog::Options options, const QStringList &supportedSchemes);
+extern Q_WIDGETS_EXPORT _qt_filedialog_existing_directory_url_hook qt_filedialog_existing_directory_url_hook;
+QT_END_NAMESPACE
+QUrl existingUrl(QWidget *, const QString &, const QUrl &, QFileDialog::Options, const QStringList &) {
+    return QUrl("http://dirUrl");
+}
+
+QT_BEGIN_NAMESPACE
+typedef QUrl (*_qt_filedialog_open_file_url_hook)(QWidget * parent, const QString &caption, const QUrl &dir, const QString &filter, QString *selectedFilter, QFileDialog::Options options, const QStringList &supportedSchemes);
+extern Q_WIDGETS_EXPORT _qt_filedialog_open_file_url_hook qt_filedialog_open_file_url_hook;
+QT_END_NAMESPACE
+QUrl openUrl(QWidget *, const QString &, const QUrl &, const QString &, QString *, QFileDialog::Options, const QStringList &) {
+    return QUrl("http://openUrl");
+}
+
+QT_BEGIN_NAMESPACE
+typedef QList<QUrl> (*_qt_filedialog_open_file_urls_hook)(QWidget * parent, const QString &caption, const QUrl &dir, const QString &filter, QString *selectedFilter, QFileDialog::Options options, const QStringList &supportedSchemes);
+extern Q_WIDGETS_EXPORT _qt_filedialog_open_file_urls_hook qt_filedialog_open_file_urls_hook;
+QT_END_NAMESPACE
+QList<QUrl> openUrls(QWidget *, const QString &, const QUrl &, const QString &, QString *, QFileDialog::Options, const QStringList &) {
+    return QList<QUrl>() << QUrl("http://openUrls");
+}
+
+QT_BEGIN_NAMESPACE
+typedef QUrl (*_qt_filedialog_save_file_url_hook)(QWidget * parent, const QString &caption, const QUrl &dir, const QString &filter, QString *selectedFilter, QFileDialog::Options options, const QStringList &supportedSchemes);
+extern Q_WIDGETS_EXPORT _qt_filedialog_save_file_url_hook qt_filedialog_save_file_url_hook;
+QT_END_NAMESPACE
+QUrl saveUrl(QWidget *, const QString &, const QUrl &, const QString &, QString *, QFileDialog::Options, const QStringList &) {
+    return QUrl("http://saveUrl");
+}
+
 
 void tst_QFiledialog::hooks()
 {
@@ -1359,6 +1383,34 @@ void tst_QFiledialog::hooks()
     QCOMPARE(QFileDialog::getOpenFileName(), QString("openName"));
     QCOMPARE(QFileDialog::getOpenFileNames(), QStringList("openNames"));
     QCOMPARE(QFileDialog::getSaveFileName(), QString("saveName"));
+    QCOMPARE(QFileDialog::getExistingDirectoryUrl(), QUrl::fromLocalFile("dir"));
+    QCOMPARE(QFileDialog::getOpenFileUrl(), QUrl::fromLocalFile("openName"));
+    QCOMPARE(QFileDialog::getOpenFileUrls(), QList<QUrl>() << QUrl::fromLocalFile("openNames"));
+    QCOMPARE(QFileDialog::getSaveFileUrl(), QUrl::fromLocalFile("saveName"));
+
+    qt_filedialog_existing_directory_url_hook = &existingUrl;
+    qt_filedialog_save_file_url_hook = &saveUrl;
+    qt_filedialog_open_file_url_hook = &openUrl;
+    qt_filedialog_open_file_urls_hook = &openUrls;
+
+    QCOMPARE(QFileDialog::getExistingDirectoryUrl(), QUrl("http://dirUrl"));
+    QCOMPARE(QFileDialog::getOpenFileUrl(), QUrl("http://openUrl"));
+    QCOMPARE(QFileDialog::getOpenFileUrls(), QList<QUrl>() << QUrl("http://openUrls"));
+    QCOMPARE(QFileDialog::getSaveFileUrl(), QUrl("http://saveUrl"));
+}
+
+void tst_QFiledialog::widgetlessNativeDialog()
+{
+    if (!QGuiApplicationPrivate::platformTheme()->usePlatformNativeDialog(QPlatformTheme::FileDialog))
+        QSKIP("This platform always uses widgets to realize its QFileDialog, instead of the native file dialog.");
+    QFileDialog fd;
+    fd.setWindowModality(Qt::ApplicationModal);
+    fd.show();
+    QTRY_VERIFY(fd.isVisible());
+    QFileSystemModel *model = fd.findChild<QFileSystemModel*>("qt_filesystem_model");
+    QVERIFY(!model);
+    QPushButton *button = fd.findChild<QPushButton*>();
+    QVERIFY(!button);
 }
 
 #ifdef Q_OS_UNIX
