@@ -48,6 +48,7 @@
 #include <QFileIconProvider>
 #include <QTreeView>
 #include <QHeaderView>
+#include <QStandardPaths>
 #include <QTime>
 #include <QStyle>
 #include <QtGlobal>
@@ -109,10 +110,12 @@ private slots:
     void setData_data();
     void setData();
 
+    void sortPersistentIndex();
     void sort_data();
     void sort();
 
     void mkdir();
+    void deleteFile();
 
     void caseSensitivity();
 
@@ -207,27 +210,37 @@ void tst_QFileSystemModel::rootPath()
     QCOMPARE(rootChanged.count(), 0);
 
     QString oldRootPath = model->rootPath();
-    root = model->setRootPath(QDir::homePath());
+    const QStringList documentPaths = QStandardPaths::standardLocations(QStandardPaths::DocumentsLocation);
+    QVERIFY(!documentPaths.isEmpty());
+    QString documentPath = documentPaths.front();
+    // In particular on Linux, ~/Documents (the first
+    // DocumentsLocation) may not exist, so choose ~ in that case:
+    if (!QFile::exists(documentPath)) {
+        documentPath = QDir::homePath();
+        qWarning("%s: first documentPath \"%s\" does not exist. Using ~ (\"%s\") instead.",
+                 Q_FUNC_INFO, qPrintable(documentPaths.front()), qPrintable(documentPath));
+    }
+    root = model->setRootPath(documentPath);
 
     QTRY_VERIFY(model->rowCount(root) >= 0);
-    QCOMPARE(model->rootPath(), QString(QDir::homePath()));
+    QCOMPARE(model->rootPath(), QString(documentPath));
     QCOMPARE(rootChanged.count(), oldRootPath == model->rootPath() ? 0 : 1);
-    QCOMPARE(model->rootDirectory().absolutePath(), QDir::homePath());
+    QCOMPARE(model->rootDirectory().absolutePath(), documentPath);
 
     model->setRootPath(QDir::rootPath());
     int oldCount = rootChanged.count();
     oldRootPath = model->rootPath();
-    root = model->setRootPath(QDir::homePath() + QLatin1String("/."));
+    root = model->setRootPath(documentPath + QLatin1String("/."));
     QTRY_VERIFY(model->rowCount(root) >= 0);
-    QCOMPARE(model->rootPath(), QDir::homePath());
+    QCOMPARE(model->rootPath(), documentPath);
     QCOMPARE(rootChanged.count(), oldRootPath == model->rootPath() ? oldCount : oldCount + 1);
-    QCOMPARE(model->rootDirectory().absolutePath(), QDir::homePath());
+    QCOMPARE(model->rootDirectory().absolutePath(), documentPath);
 
-    QDir newdir = QDir::home();
+    QDir newdir = documentPath;
     if (newdir.cdUp()) {
         oldCount = rootChanged.count();
         oldRootPath = model->rootPath();
-        root = model->setRootPath(QDir::homePath() + QLatin1String("/.."));
+        root = model->setRootPath(documentPath + QLatin1String("/.."));
         QTRY_VERIFY(model->rowCount(root) >= 0);
         QCOMPARE(model->rootPath(), newdir.path());
         QCOMPARE(rootChanged.count(), oldCount + 1);
@@ -310,9 +323,9 @@ void tst_QFileSystemModel::naturalCompare()
 void tst_QFileSystemModel::readOnly()
 {
     QCOMPARE(model->isReadOnly(), true);
-    QTemporaryFile file;
+    QTemporaryFile file(flatDirTestPath + QStringLiteral("/XXXXXX.dat"));
     file.open();
-    QModelIndex root = model->setRootPath(QDir::tempPath());
+    QModelIndex root = model->setRootPath(flatDirTestPath);
 
     QTRY_VERIFY(model->rowCount(root) > 0);
     QVERIFY(!(model->flags(model->index(file.fileName())) & Qt::ItemIsEditable));
@@ -358,7 +371,10 @@ void tst_QFileSystemModel::iconProvider()
     delete p;
 
     QFileSystemModel *myModel = new QFileSystemModel();
-    myModel->setRootPath(QDir::homePath());
+    const QStringList documentPaths = QStandardPaths::standardLocations(QStandardPaths::DocumentsLocation);
+    QVERIFY(!documentPaths.isEmpty());
+    const QString documentPath = documentPaths.front();
+    myModel->setRootPath(documentPath);
     //Let's wait to populate the model
     QTest::qWait(250);
     //We change the provider, icons must me updated
@@ -791,6 +807,19 @@ void tst_QFileSystemModel::setData()
     QTRY_COMPARE(model->rowCount(root), files.count());
 }
 
+void tst_QFileSystemModel::sortPersistentIndex()
+{
+    QTemporaryFile file(flatDirTestPath + QStringLiteral("/XXXXXX.dat"));
+    file.open();
+    QModelIndex root = model->setRootPath(flatDirTestPath);
+    QTRY_VERIFY(model->rowCount(root) > 0);
+
+    QPersistentModelIndex idx = model->index(0, 1, root);
+    model->sort(0, Qt::AscendingOrder);
+    model->sort(0, Qt::DescendingOrder);
+    QVERIFY(idx.column() != 0);
+}
+
 class MyFriendFileSystemModel : public QFileSystemModel
 {
     friend class tst_QFileSystemModel;
@@ -806,18 +835,6 @@ void tst_QFileSystemModel::sort_data()
 
 void tst_QFileSystemModel::sort()
 {
-    QTemporaryFile file;
-    file.open();
-    QModelIndex root = model->setRootPath(QDir::tempPath());
-    QTRY_VERIFY(model->rowCount(root) > 0);
-
-    QPersistentModelIndex idx = model->index(0, 1, root);
-    model->sort(0, Qt::AscendingOrder);
-    model->sort(0, Qt::DescendingOrder);
-    QVERIFY(idx.column() != 0);
-
-    model->setRootPath(QDir::homePath());
-
     QFETCH(bool, fileDialogMode);
 
     MyFriendFileSystemModel *myModel = new MyFriendFileSystemModel();
@@ -912,14 +929,31 @@ void tst_QFileSystemModel::mkdir()
     int oldRow = idx.row();
     QTest::qWait(WAITTIME);
     idx = model->index(newFolderPath);
-    QDir cleanup(tmp);
-    QVERIFY(cleanup.rmdir(QLatin1String("NewFoldermkdirtest3")));
-    QVERIFY(cleanup.rmdir(QLatin1String("NewFoldermkdirtest5")));
-    bestatic.rmdir(newFolderPath);
+    QVERIFY(model->remove(idx));
+    QVERIFY(!bestatic.exists());
     QVERIFY(0 != idx.row());
     QCOMPARE(oldRow, idx.row());
 }
 
+void tst_QFileSystemModel::deleteFile()
+{
+    QString newFilePath = QDir::temp().filePath("NewFileDeleteTest");
+    QFile newFile(newFilePath);
+    if (newFile.exists()) {
+        if (!newFile.remove())
+            qWarning() << "unable to remove" << newFilePath;
+        QTest::qWait(WAITTIME);
+    }
+    if (!newFile.open(QIODevice::WriteOnly | QIODevice::Text)) {
+        qWarning() << "unable to create" << newFilePath;
+    }
+    newFile.close();
+    QModelIndex idx = model->index(newFilePath);
+    QVERIFY(idx.isValid());
+    QVERIFY(model->remove(idx));
+    QVERIFY(!newFile.exists());
+}
+
 void tst_QFileSystemModel::caseSensitivity()
 {
     QString tmp = flatDirTestPath;
@@ -1024,6 +1058,14 @@ void tst_QFileSystemModel::roleNames()
     QVERIFY(values.contains(roleName));
 }
 
+static inline QByteArray permissionRowName(bool readOnly, int permission)
+{
+    QByteArray result = readOnly ? QByteArrayLiteral("ro") : QByteArrayLiteral("rw");
+    result += QByteArrayLiteral("-0");
+    result += QByteArray::number(permission, 16);
+    return result;
+}
+
 void tst_QFileSystemModel::permissions_data()
 {
     QTest::addColumn<int>("permissions");
@@ -1034,11 +1076,10 @@ void tst_QFileSystemModel::permissions_data()
         QFile::ReadOwner,
         QFile::WriteOwner|QFile::ReadOwner,
     };
-#define ROW_NAME(i) qPrintable(QString().sprintf("%s-0%04x", readOnly ? "ro" : "rw", permissions[i]))
-    for (int readOnly = false ; readOnly <= true; ++readOnly)
-        for (size_t i = 0; i < sizeof permissions / sizeof *permissions; ++i)
-            QTest::newRow(ROW_NAME(i)) << permissions[i] << bool(readOnly);
-#undef ROW_NAME
+    for (size_t i = 0; i < sizeof permissions / sizeof *permissions; ++i) {
+        QTest::newRow(permissionRowName(false, permissions[i]).constData()) << permissions[i] << false;
+        QTest::newRow(permissionRowName(true, permissions[i]).constData()) << permissions[i] << true;
+    }
 }
 
 void tst_QFileSystemModel::permissions() // checks QTBUG-20503
