@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2012 Digia Plc and/or its subsidiary(-ies).
+** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
 ** This file is part of the test suite of the Qt Toolkit.
@@ -114,6 +114,7 @@ private slots:
     void about();
     void detailsText();
     void detailsButtonText();
+    void expandDetails_QTBUG_32473();
 
 #ifndef Q_OS_MAC
     void shortcut();
@@ -137,6 +138,19 @@ private:
     QTimer keySendTimer;
 };
 
+class tst_ResizingMessageBox : public QMessageBox
+{
+public:
+    tst_ResizingMessageBox() : QMessageBox(), resized(false) { }
+    bool resized;
+
+protected:
+    void resizeEvent ( QResizeEvent * event ) {
+        resized = true;
+        QMessageBox::resizeEvent(event);
+    }
+};
+
 tst_QMessageBox::tst_QMessageBox() : keyToSend(-1)
 {
 }
@@ -603,6 +617,37 @@ void tst_QMessageBox::detailsButtonText()
     }
 }
 
+void tst_QMessageBox::expandDetails_QTBUG_32473()
+{
+    tst_ResizingMessageBox box;
+    box.setDetailedText("bla");
+    box.show();
+    QApplication::postEvent(&box, new QEvent(QEvent::LanguageChange));
+    QApplication::processEvents();
+    QDialogButtonBox* bb = box.findChild<QDialogButtonBox*>("qt_msgbox_buttonbox");
+    QVERIFY(bb);
+
+    QList<QAbstractButton *> list = bb->buttons();
+    QAbstractButton* moreButton = NULL;
+    foreach (QAbstractButton* btn, list)
+        if (btn && bb->buttonRole(btn) == QDialogButtonBox::ActionRole)
+            moreButton = btn;
+    QVERIFY(moreButton);
+    QVERIFY(QTest::qWaitForWindowExposed(&box));
+    QRect geom = box.geometry();
+    box.resized = false;
+    moreButton->click();
+    QTRY_VERIFY(box.resized);
+    // After we receive the expose event for a second widget, it's likely
+    // that the window manager is also done manipulating the first QMessageBox.
+    QWidget fleece;
+    fleece.show();
+    QTest::qWaitForWindowExposed(&fleece);
+    if (geom.topLeft() == box.geometry().topLeft())
+        QTest::qWait(500);
+    QCOMPARE(geom.topLeft(), box.geometry().topLeft());
+}
+
 void tst_QMessageBox::incorrectDefaultButton()
 {
     keyToSend = Qt::Key_Escape;
