@@ -46,6 +46,7 @@
 #include <qdebug.h>
 #include <qprogressdialog.h>
 #include <qlabel.h>
+#include <qthread.h>
 
 class tst_QProgressDialog : public QObject
 {
@@ -56,8 +57,11 @@ public:
     virtual ~tst_QProgressDialog();
 
 private slots:
+    void autoShow_data();
+    void autoShow();
     void getSetCheck();
     void task198202();
+    void QTBUG_31046();
 };
 
 tst_QProgressDialog::tst_QProgressDialog()
@@ -68,6 +72,34 @@ tst_QProgressDialog::~tst_QProgressDialog()
 {
 }
 
+void tst_QProgressDialog::autoShow_data()
+{
+    QTest::addColumn<int>("min");
+    QTest::addColumn<int>("max");
+    QTest::addColumn<int>("delay");
+    QTest::addColumn<bool>("expectedAutoShow");
+
+    QTest::newRow("50_to_100_long") << 50 << 100 << 100 << true; // 50*100ms = 5s
+    QTest::newRow("50_to_100_short") << 50 << 1 << 100 << false; // 50*1ms = 50ms
+
+    QTest::newRow("0_to_100_long") << 0 << 100 << 100 << true; // 100*100ms = 10s
+    QTest::newRow("0_to_10_short") << 0 << 10 << 100 << false; // 10*100ms = 1s
+}
+
+void tst_QProgressDialog::autoShow()
+{
+    QFETCH(int, min);
+    QFETCH(int, max);
+    QFETCH(int, delay);
+    QFETCH(bool, expectedAutoShow);
+
+    QProgressDialog dlg("", "", min, max);
+    dlg.setValue(0);
+    QThread::msleep(delay);
+    dlg.setValue(min+1);
+    QCOMPARE(dlg.isVisible(), expectedAutoShow);
+}
+
 // Testing get/set functions
 void tst_QProgressDialog::getSetCheck()
 {
@@ -141,7 +173,7 @@ void tst_QProgressDialog::task198202()
     QProgressDialog dlg(QLatin1String("test"),QLatin1String("test"),1,10);
     dlg.show();
     QVERIFY(QTest::qWaitForWindowExposed(&dlg));
-    int futureHeight = dlg.sizeHint().height() - qFindChild<QLabel*>(&dlg)->sizeHint().height();
+    int futureHeight = dlg.sizeHint().height() - dlg.findChild<QLabel*>()->sizeHint().height();
     dlg.setLabel(0);
     QTest::ignoreMessage(QtWarningMsg, "QProgressDialog::setBar: Cannot set a null progress bar");
     dlg.setBar(0);
@@ -149,5 +181,14 @@ void tst_QProgressDialog::task198202()
     QCOMPARE(dlg.sizeHint().height(), futureHeight);
 }
 
+void tst_QProgressDialog::QTBUG_31046()
+{
+    QProgressDialog dlg("", "", 50, 60);
+    dlg.setValue(0);
+    QThread::msleep(200);
+    dlg.setValue(50);
+    QCOMPARE(50, dlg.value());
+}
+
 QTEST_MAIN(tst_QProgressDialog)
 #include "tst_qprogressdialog.moc"
