@@ -52,6 +52,8 @@
 #include <QWizard>
 #include <QTreeWidget>
 
+Q_DECLARE_METATYPE(QWizard::WizardButton);
+
 static QImage grabWidget(QWidget *window)
 {
     return window->grab().toImage();
@@ -100,6 +102,8 @@ private slots:
     void setWizardStyle();
     void removePage();
     void sideWidget();
+    void objectNames_data();
+    void objectNames();
 
     // task-specific tests below me:
     void task177716_disableCommitButton();
@@ -439,14 +443,8 @@ void tst_QWizard::setPixmap()
     QVERIFY(wizard.pixmap(QWizard::BannerPixmap).isNull());
     QVERIFY(wizard.pixmap(QWizard::LogoPixmap).isNull());
     QVERIFY(wizard.pixmap(QWizard::WatermarkPixmap).isNull());
-#ifdef Q_OS_MAC
-    if (QSysInfo::MacintoshVersion > QSysInfo::MV_10_3) {
-        QEXPECT_FAIL("", "QTBUG-23701", Continue);
-        QVERIFY(wizard.pixmap(QWizard::BackgroundPixmap).isNull() == false);
-    } else {
-        // fall through since the image doesn't exist on a 10.3 system.
-        QVERIFY(page->pixmap(QWizard::BackgroundPixmap).isNull());
-    }
+#ifdef Q_OS_OSX
+    QVERIFY(wizard.pixmap(QWizard::BackgroundPixmap).isNull() == false);
 #else
     QVERIFY(wizard.pixmap(QWizard::BackgroundPixmap).isNull());
 #endif
@@ -454,14 +452,8 @@ void tst_QWizard::setPixmap()
     QVERIFY(page->pixmap(QWizard::BannerPixmap).isNull());
     QVERIFY(page->pixmap(QWizard::LogoPixmap).isNull());
     QVERIFY(page->pixmap(QWizard::WatermarkPixmap).isNull());
-#ifdef Q_OS_MAC
-    if (QSysInfo::MacintoshVersion > QSysInfo::MV_10_3) {
-        QEXPECT_FAIL("", "QTBUG-23701", Continue);
-        QVERIFY(wizard.pixmap(QWizard::BackgroundPixmap).isNull() == false);
-    } else {
-        // fall through since the image doesn't exist on a 10.3 system.
-        QVERIFY(page->pixmap(QWizard::BackgroundPixmap).isNull());
-    }
+#ifdef Q_OS_OSX
+    QVERIFY(wizard.pixmap(QWizard::BackgroundPixmap).isNull() == false);
 #else
     QVERIFY(page->pixmap(QWizard::BackgroundPixmap).isNull());
 #endif
@@ -611,7 +603,7 @@ void tst_QWizard::addPage()
     arguments = spy.takeFirst();
     QCOMPARE(arguments.at(0).toInt(), N + 52);
 
-    QTest::ignoreMessage(QtWarningMsg,"QWizard::setPage: Cannot insert null page");    
+    QTest::ignoreMessage(QtWarningMsg,"QWizard::setPage: Cannot insert null page");
     wizard.addPage(0); // generates a warning
     QCOMPARE(spy.count(), 0);
     delete parent;
@@ -638,7 +630,7 @@ void tst_QWizard::setPage()
     QCOMPARE(wizard.nextId(), -1);
 
     page = new QWizardPage(parent);
-    QTest::ignoreMessage(QtWarningMsg,"QWizard::setPage: Cannot insert page with ID -1");    
+    QTest::ignoreMessage(QtWarningMsg,"QWizard::setPage: Cannot insert page with ID -1");
     wizard.setPage(-1, page);   // gives a warning and does nothing
     QCOMPARE(spy.count(), 0);
     QVERIFY(!wizard.page(-2));
@@ -2396,6 +2388,44 @@ void tst_QWizard::sideWidget()
     QVERIFY(wizard.sideWidget() == 0);
 }
 
+void tst_QWizard::objectNames_data()
+{
+    QTest::addColumn<QWizard::WizardButton>("wizardButton");
+    QTest::addColumn<QString>("buttonName");
+
+    QTest::newRow("BackButton")    << QWizard::BackButton    << QStringLiteral("__qt__passive_wizardbutton0");
+    QTest::newRow("NextButton")    << QWizard::NextButton    << QStringLiteral("__qt__passive_wizardbutton1");
+    QTest::newRow("CommitButton")  << QWizard::CommitButton  << QStringLiteral("qt_wizard_commit");
+    QTest::newRow("FinishButton")  << QWizard::FinishButton  << QStringLiteral("qt_wizard_finish");
+    QTest::newRow("CancelButton")  << QWizard::CancelButton  << QStringLiteral("qt_wizard_cancel");
+    QTest::newRow("HelpButton")    << QWizard::HelpButton    << QStringLiteral("__qt__passive_wizardbutton5");
+    QTest::newRow("CustomButton1") << QWizard::CustomButton1 << QStringLiteral("__qt__passive_wizardbutton6");
+    QTest::newRow("CustomButton2") << QWizard::CustomButton2 << QStringLiteral("__qt__passive_wizardbutton7");
+    QTest::newRow("CustomButton3") << QWizard::CustomButton3 << QStringLiteral("__qt__passive_wizardbutton8");
+}
+
+void tst_QWizard::objectNames()
+{
+    QFETCH(QWizard::WizardButton, wizardButton);
+    QFETCH(QString, buttonName);
+
+    QWizard wizard;
+    QList<QWizard::WizardButton> buttons = QList<QWizard::WizardButton>()
+        << QWizard::BackButton
+        << QWizard::NextButton
+        << QWizard::CommitButton
+        << QWizard::FinishButton
+        << QWizard::CancelButton
+        << QWizard::HelpButton
+        << QWizard::CustomButton1
+        << QWizard::CustomButton2
+        << QWizard::CustomButton3
+      ;
+    QVERIFY(buttons.contains(wizardButton));
+    QVERIFY(wizard.button(wizardButton));
+    QCOMPARE(wizard.button(wizardButton)->objectName(), buttonName);
+}
+
 class task177716_CommitPage : public QWizardPage
 {
     Q_OBJECT
@@ -2453,7 +2483,7 @@ private:
 
         QLayout *layout_ = new QVBoxLayout(this);
         layout_->addWidget(treeWidget = new QTreeWidget);
- 
+
         QSizePolicy policy = sizePolicy();
         policy.setVerticalPolicy(verticalPolicy);
         treeWidget->setSizePolicy(policy);
@@ -2518,7 +2548,7 @@ void tst_QWizard::task161658_alignments()
     wizard.show();
     QVERIFY(QTest::qWaitForWindowExposed(&wizard));
 
-    foreach (QLabel *subtitleLabel, qFindChildren<QLabel *>(&wizard)) {
+    foreach (QLabel *subtitleLabel, wizard.findChildren<QLabel *>()) {
         if (subtitleLabel->text().startsWith("SUBTITLE#")) {
             QCOMPARE(lineEdit1.mapToGlobal(lineEdit1.contentsRect().bottomRight()).x(),
                      subtitleLabel->mapToGlobal(subtitleLabel->contentsRect().bottomRight()).x());
