@@ -505,11 +505,12 @@ class QMoveCursorListView : public QListView
 public:
     QMoveCursorListView() : QListView() {}
 
+    // enum CursorAction and moveCursor() are protected in QListView.
     enum CursorAction { MoveUp, MoveDown, MoveLeft, MoveRight,
         MoveHome, MoveEnd, MovePageUp, MovePageDown,
         MoveNext, MovePrevious };
 
-    QModelIndex moveCursor(QMoveCursorListView::CursorAction action, Qt::KeyboardModifiers modifiers)
+    QModelIndex doMoveCursor(QMoveCursorListView::CursorAction action, Qt::KeyboardModifiers modifiers)
     {
         return QListView::moveCursor((QListView::CursorAction)action, modifiers);
     }
@@ -540,9 +541,9 @@ void tst_QListView::moveCursor2()
     vu.selectionModel()->setCurrentIndex(model.index(0,0), QItemSelectionModel::SelectCurrent);
     QCoreApplication::processEvents();
 
-    QModelIndex idx = vu.moveCursor(QMoveCursorListView::MoveHome, Qt::NoModifier);
+    QModelIndex idx = vu.doMoveCursor(QMoveCursorListView::MoveHome, Qt::NoModifier);
     QCOMPARE(idx, model.index(0,0));
-    idx = vu.moveCursor(QMoveCursorListView::MoveDown, Qt::NoModifier);
+    idx = vu.doMoveCursor(QMoveCursorListView::MoveDown, Qt::NoModifier);
     QCOMPARE(idx, model.index(8,0));
 }
 
@@ -1468,7 +1469,15 @@ void tst_QListView::wordWrap()
     lv.setModel(&model);
     lv.setWordWrap(true);
     lv.setFixedSize(150, 150);
-    lv.show();
+#ifdef Q_OS_BLACKBERRY
+    // BB standard font size is too big, triggering
+    // a horizontal scrollbar even when word-wrapping
+    // is enabled
+    QFont font = lv.font();
+    font.setPointSize(5);
+    lv.setFont(font);
+#endif // Q_OS_BLACKBERRY
+    lv.showNormal();
     QApplication::processEvents();
 
     QTRY_COMPARE(lv.horizontalScrollBar()->isVisible(), false);
@@ -1628,7 +1637,7 @@ void tst_QListView::task196118_visualRegionForSelection()
     class MyListView : public QListView
     {
     public:
-        QRegion visualRegionForSelection() const
+        QRegion getVisualRegionForSelection() const
         { return QListView::visualRegionForSelection( selectionModel()->selection()); }
     } view;
 
@@ -1643,7 +1652,7 @@ void tst_QListView::task196118_visualRegionForSelection()
     view.selectionModel()->select(top1.index(), QItemSelectionModel::Select);
 
     QCOMPARE(view.selectionModel()->selectedIndexes().count(), 1);
-    QVERIFY(view.visualRegionForSelection().isEmpty());
+    QVERIFY(view.getVisualRegionForSelection().isEmpty());
 }
 
 void tst_QListView::task254449_draggingItemToNegativeCoordinates()
@@ -1977,11 +1986,11 @@ void tst_QListView::taskQTBUG_5877_skippingItemInPageDownUp()
         vu.selectionModel()->setCurrentIndex(model.index(currentItemIndexes[i], 0),
                                              QItemSelectionModel::SelectCurrent);
 
-        QModelIndex idx = vu.moveCursor(QMoveCursorListView::MovePageDown, Qt::NoModifier);
+        QModelIndex idx = vu.doMoveCursor(QMoveCursorListView::MovePageDown, Qt::NoModifier);
         int newCurrent = qMin(currentItemIndexes[i] + scrolledRowCount, 99);
         QCOMPARE(idx, model.index(newCurrent, 0));
 
-        idx = vu.moveCursor(QMoveCursorListView::MovePageUp, Qt::NoModifier);
+        idx = vu.doMoveCursor(QMoveCursorListView::MovePageUp, Qt::NoModifier);
         newCurrent = qMax(currentItemIndexes[i] - scrolledRowCount, 0);
         QCOMPARE(idx, model.index(newCurrent, 0));
     }
@@ -2012,7 +2021,7 @@ void tst_QListView::taskQTBUG_9455_wrongScrollbarRanges()
     w.setMovement(QListView::Static);
     const int spacing = 40;
     w.setSpacing(spacing);
-    w.show();
+    w.showNormal();
     QVERIFY(QTest::qWaitForWindowExposed(&w));
     QCOMPARE(w.verticalScrollBar()->maximum(), w.contentsSize().height() - w.viewport()->geometry().height());
 }
@@ -2116,7 +2125,7 @@ void tst_QListView::taskQTBUG_21115_scrollToAndHiddenItems()
         list << QString::number(i);
     model.setStringList(list);
     lv.setModel(&model);
-    lv.show();
+    lv.showNormal();
     QVERIFY(QTest::qWaitForWindowExposed(&lv));
 
     // Save first item rect for reference
@@ -2206,7 +2215,7 @@ void tst_QListView::taskQTBUG_21804_hiddenItemsAndScrollingWithKeys()
     lv.setSpacing(spacing);
     lv.setModel(&model);
     lv.show();
-    QTest::qWaitForWindowShown(&lv);
+    QTest::qWaitForWindowExposed(&lv);
 
     // hide every odd number row
     for (int i = 1; i < model.rowCount(); i+=2)
@@ -2282,7 +2291,7 @@ void tst_QListView::spacing()
     lv.setModel(&model);
     lv.setSpacing(spacing);
     lv.show();
-    QTest::qWaitForWindowShown(&lv);
+    QTest::qWaitForWindowExposed(&lv);
 
     // check size and position of first two items
     QRect item1 = lv.visualRect(lv.model()->index(0, 0));
