@@ -153,6 +153,7 @@ private slots:
 
     void focusChanged();
     void focusOut();
+    void focusMouseClick();
 
     void execAfterExit();
 
@@ -316,7 +317,6 @@ void tst_QApplication::alert()
     widget.activateWindow();
     QApplication::setActiveWindow(&widget);
     app.alert(&widget, 200);
-    app.syncX();
 }
 
 void tst_QApplication::multiple_data()
@@ -695,18 +695,20 @@ void tst_QApplication::quitOnLastWindowClosed()
         QSignalSpy spy(&app, SIGNAL(aboutToQuit()));
         QSignalSpy spy2(&timer, SIGNAL(timeout()));
 
-        QPointer<QMainWindow> mainWindow = new QMainWindow;
-        QPointer<QDialog> dialog = new QDialog(mainWindow);
+        QMainWindow mainWindow;
+        QDialog *dialog = new QDialog(&mainWindow);
 
         QVERIFY(app.quitOnLastWindowClosed());
-        QVERIFY(mainWindow->testAttribute(Qt::WA_QuitOnClose));
+        QVERIFY(mainWindow.testAttribute(Qt::WA_QuitOnClose));
         QVERIFY(dialog->testAttribute(Qt::WA_QuitOnClose));
 
-        mainWindow->show();
+        mainWindow.show();
+        QVERIFY(QTest::qWaitForWindowExposed(&mainWindow));
         dialog->show();
+        QVERIFY(QTest::qWaitForWindowExposed(dialog));
 
         timer.start();
-        QTimer::singleShot(1000, mainWindow, SLOT(close())); // This should quit the application
+        QTimer::singleShot(1000, &mainWindow, SLOT(close())); // This should quit the application
         QTimer::singleShot(2000, &app, SLOT(quit()));        // This makes sure we quit even if it didn't
 
         app.exec();
@@ -723,15 +725,16 @@ void tst_QApplication::quitOnLastWindowClosed()
         QSignalSpy spy(&app, SIGNAL(aboutToQuit()));
         QSignalSpy spy2(&timer, SIGNAL(timeout()));
 
-        QPointer<CloseEventTestWindow> mainWindow = new CloseEventTestWindow;
+        CloseEventTestWindow mainWindow;
 
         QVERIFY(app.quitOnLastWindowClosed());
-        QVERIFY(mainWindow->testAttribute(Qt::WA_QuitOnClose));
+        QVERIFY(mainWindow.testAttribute(Qt::WA_QuitOnClose));
 
-        mainWindow->show();
+        mainWindow.show();
+        QVERIFY(QTest::qWaitForWindowExposed(&mainWindow));
 
         timer.start();
-        QTimer::singleShot(1000, mainWindow, SLOT(close())); // This should quit the application
+        QTimer::singleShot(1000, &mainWindow, SLOT(close())); // This should quit the application
         QTimer::singleShot(2000, &app, SLOT(quit()));        // This makes sure we quit even if it didn't
 
         app.exec();
@@ -784,6 +787,31 @@ void tst_QApplication::quitOnLastWindowClosed()
 
         QVERIFY(timerSpy.count() > 15);      // Should be around 20 if closing did not caused the quit
     }
+    {   // QTBUG-31569: If the last widget with Qt::WA_QuitOnClose set is closed, other
+        // widgets that don't have the attribute set should be closed automatically.
+        int argc = 0;
+        QApplication app(argc, 0);
+        QVERIFY(app.quitOnLastWindowClosed());
+
+        QWidget w1;
+        w1.show();
+
+        QWidget w2;
+        w2.setAttribute(Qt::WA_QuitOnClose, false);
+        w2.show();
+
+        QVERIFY(QTest::qWaitForWindowExposed(&w2));
+
+        QTimer timer;
+        timer.setInterval(100);
+        timer.start();
+        QSignalSpy timerSpy(&timer, SIGNAL(timeout()));
+
+        QTimer::singleShot(100, &w1, SLOT(close()));
+        app.exec();
+
+        QVERIFY(timerSpy.count() < 10);
+    }
 }
 
 class PromptOnCloseWidget : public QWidget
@@ -1242,7 +1270,7 @@ public slots:
 
 void DeleteLaterWidget::runTest()
 {
-    QObject *stillAlive = qFindChild<QObject*>(this, "deleteLater");
+    QObject *stillAlive = this->findChild<QObject*>("deleteLater");
 
     QWidget *w = new QWidget(this);
     connect(w, SIGNAL(destroyed()), this, SLOT(childDeleted()));
@@ -1288,7 +1316,7 @@ void tst_QApplication::testDeleteLater()
     object->setObjectName("deleteLater");
     object->deleteLater();
 
-    QObject *stillAlive = qFindChild<QObject*>(wgt, "deleteLater");
+    QObject *stillAlive = wgt->findChild<QObject*>("deleteLater");
     QVERIFY(stillAlive);
 
     app.exec();
@@ -1744,6 +1772,63 @@ void tst_QApplication::focusOut()
     QTest::qWait(2000);
 }
 
+class SpontaneousEvent
+{
+    Q_GADGET
+    QDOC_PROPERTY(bool accepted READ isAccepted WRITE setAccepted)
+    Q_ENUMS(Type)
+public:
+    enum Type {
+        Void
+    };
+
+    virtual ~SpontaneousEvent() {}
+
+    QEventPrivate *d;
+    ushort t;
+
+    ushort posted : 1;
+    ushort spont : 1;
+};
+
+void tst_QApplication::focusMouseClick()
+{
+    int argc = 1;
+    QApplication app(argc, &argv0);
+
+    QWidget w;
+    w.setFocusPolicy(Qt::StrongFocus);
+    QWidget w2(&w);
+    w2.setFocusPolicy(Qt::TabFocus);
+    w.show();
+    w.setFocus();
+    QTRY_COMPARE(QApplication::focusWidget(), &w);
+
+    // front most widget has Qt::TabFocus, parent widget accepts clicks as well
+    // now send a mouse button press event and check what happens with the focus
+    // it should be given to the parent widget
+    QMouseEvent ev(QEvent::MouseButtonPress, QPointF(), Qt::LeftButton, Qt::LeftButton, Qt::NoModifier);
+    reinterpret_cast<SpontaneousEvent *>(&ev)->spont = 1;
+    QVERIFY(ev.spontaneous());
+    qApp->notify(&w2, &ev);
+    QCOMPARE(QApplication::focusWidget(), &w);
+
+    // then we give the inner widget strong focus -> it should get focus
+    w2.setFocusPolicy(Qt::StrongFocus);
+    reinterpret_cast<SpontaneousEvent *>(&ev)->spont = 1;
+    QVERIFY(ev.spontaneous());
+    qApp->notify(&w2, &ev);
+    QTRY_COMPARE(QApplication::focusWidget(), &w2);
+
+    // now back to tab focus and click again (it already had focus) -> focus should stay
+    // (focus was revoked as of QTBUG-34042)
+    w2.setFocusPolicy(Qt::TabFocus);
+    reinterpret_cast<SpontaneousEvent *>(&ev)->spont = 1;
+    QVERIFY(ev.spontaneous());
+    qApp->notify(&w2, &ev);
+    QCOMPARE(QApplication::focusWidget(), &w2);
+}
+
 void tst_QApplication::execAfterExit()
 {
     int argc = 1;
