@@ -84,6 +84,7 @@ private slots:
     void controlTypes();
     void controlTypes2();
     void adjustSizeShouldMakeSureLayoutIsActivated();
+    void testRetainSizeWhenHidden();
 };
 
 tst_QLayout::tst_QLayout()
@@ -155,7 +156,7 @@ void tst_QLayout::geometry()
 
 void tst_QLayout::smartMaxSize()
 {
-    QVector<int> expectedWidths; 
+    QVector<int> expectedWidths;
 
     QFile f(QFINDTESTDATA("baseline/smartmaxsize"));
 
@@ -170,9 +171,9 @@ void tst_QLayout::smartMaxSize()
     f.close();
 
     int sizeCombinations[] = { 0, 10, 20, QWIDGETSIZE_MAX};
-    QSizePolicy::Policy policies[] = {  QSizePolicy::Fixed, 
-                                        QSizePolicy::Minimum, 
-                                        QSizePolicy::Maximum, 
+    QSizePolicy::Policy policies[] = {  QSizePolicy::Fixed,
+                                        QSizePolicy::Minimum,
+                                        QSizePolicy::Maximum,
                                         QSizePolicy::Preferred,
                                         QSizePolicy::Expanding,
                                         QSizePolicy::MinimumExpanding,
@@ -250,7 +251,7 @@ void tst_QLayout::setContentsMargins()
     MyLayout layout;
     layout.invalidated = false;
     int left, top, right, bottom;
-    
+
     layout.setContentsMargins(52, 53, 54, 55);
     QVERIFY(layout.invalidated);
     layout.invalidated = false;
@@ -260,7 +261,7 @@ void tst_QLayout::setContentsMargins()
     QCOMPARE(top, 53);
     QCOMPARE(right, 54);
     QCOMPARE(bottom, 55);
- 
+
     layout.setContentsMargins(52, 53, 54, 55);
     QVERIFY(!layout.invalidated);
 }
@@ -350,5 +351,47 @@ void tst_QLayout::adjustSizeShouldMakeSureLayoutIsActivated()
     QCOMPARE(main.size(), QSize(200, 10));
 }
 
+void tst_QLayout::testRetainSizeWhenHidden()
+{
+    QWidget widget;
+    QBoxLayout layout(QBoxLayout::TopToBottom, &widget);
+
+    QLabel *label1 = new QLabel("label1 text", &widget);
+    layout.addWidget(label1);
+    QLabel *label2 = new QLabel("label2 text", &widget);
+    layout.addWidget(label2);
+
+    widget.show();
+    QVERIFY(QTest::qWaitForWindowExposed(&widget));
+    int normalHeight = widget.height();
+
+    // a. Verify that a removed visible will mean lesser size after adjust
+    label1->hide();
+    widget.adjustSize();
+    int heightWithoutLabel1 = widget.height();
+    QVERIFY(heightWithoutLabel1 < normalHeight);
+
+    // b restore with verify that the size is the same
+    label1->show();
+    QCOMPARE(widget.sizeHint().height(), normalHeight);
+
+    // c verify that a policy with retainSizeWhenHidden is respected
+    QSizePolicy sp_remove = label1->sizePolicy();
+    QSizePolicy sp_retain = label1->sizePolicy();
+    sp_retain.setRetainSizeWhenHidden(true);
+
+    label1->setSizePolicy(sp_retain);
+    label1->hide();
+    QCOMPARE(widget.sizeHint().height(), normalHeight);
+
+    // d check that changing the policy to not wanting size will result in lesser size
+    label1->setSizePolicy(sp_remove);
+    QCOMPARE(widget.sizeHint().height(), heightWithoutLabel1);
+
+    // e verify that changing back the hidden widget to want the hidden size will ensure that it gets more size
+    label1->setSizePolicy(sp_retain);
+    QCOMPARE(widget.sizeHint().height(), normalHeight);
+}
+
 QTEST_MAIN(tst_QLayout)
 #include "tst_qlayout.moc"
