@@ -70,6 +70,7 @@ private slots:
     void removeWidget();
     void keepFocusAfterSetCurrent();
     void heigthForWidth();
+    void replaceWidget();
 
 private:
     QWidget *testWidget;
@@ -132,7 +133,6 @@ void tst_QStackedLayout::init()
     // make sure the tests work with focus follows mouse
     QCursor::setPos(testWidget->geometry().center());
     testWidget->activateWindow();
-    QApplication::syncX();
     QTest::qWait(250);
 }
 
@@ -394,6 +394,27 @@ void tst_QStackedLayout::heigthForWidth()
 
 }
 
+void tst_QStackedLayout::replaceWidget()
+{
+    QWidget w;
+    QStackedLayout *stackLayout = new QStackedLayout(&w);
+
+    QLineEdit *replaceFrom = new QLineEdit;
+    QLineEdit *replaceTo = new QLineEdit;
+    stackLayout->addWidget(new QLineEdit());
+    stackLayout->addWidget(replaceFrom);
+    stackLayout->addWidget(new QLineEdit());
+    stackLayout->setCurrentWidget(replaceFrom);
+
+    QCOMPARE(stackLayout->indexOf(replaceFrom), 1);
+    QCOMPARE(stackLayout->indexOf(replaceTo), -1);
+    stackLayout->replaceWidget(replaceFrom, replaceTo);
+
+    QCOMPARE(stackLayout->indexOf(replaceFrom), -1);
+    QCOMPARE(stackLayout->indexOf(replaceTo), 1);
+    QCOMPARE(stackLayout->currentWidget(), replaceTo);
+}
+
 QTEST_MAIN(tst_QStackedLayout)
 #include "tst_qstackedlayout.moc"
 
