@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2012 Digia Plc and/or its subsidiary(-ies).
+** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
 ** This file is part of the test suite of the Qt Toolkit.
@@ -490,6 +490,12 @@ void tst_QAbstractButton::toggled()
 
     QTest::mouseRelease( testWidget, Qt::LeftButton );
     QVERIFY( click_count == 1 );
+
+    testWidget->setCheckable(true);
+    testWidget->toggle();
+    testWidget->toggle();
+    QCOMPARE(int(toggle_count), 2);
+    testWidget->setCheckable(false);
 }
 
 void tst_QAbstractButton::setShortcut()
