@@ -1119,7 +1119,7 @@ void tst_QPlainTextEdit::mimeDataReimplementations()
     QCOMPARE(ed.insertCallCount, 0);
 
 #ifdef QT_BUILD_INTERNAL
-    QWidgetTextControl *control = qFindChild<QWidgetTextControl *>(&ed);
+    QWidgetTextControl *control = ed.findChild<QWidgetTextControl *>();
     QVERIFY(control);
 
     control->canInsertFromMimeData(QApplication::clipboard()->mimeData());
