@@ -54,6 +54,10 @@
 # include <windows.h>
 #endif
 
+#if defined(Q_OS_QNX) && defined(open)
+#undef open
+#endif
+
 #define BUFSIZE 1024*512
 #define FACTOR 1024*512
 #define TF_SIZE FACTOR*81
@@ -184,9 +188,9 @@ void tst_qfile::readBigFile_QFSFileEngine()
     QSKIP("This test requires -developer-build.");
 #endif
 }
-void tst_qfile::readBigFile_posix() 
-{ 
-    readBigFile(); 
+void tst_qfile::readBigFile_posix()
+{
+    readBigFile();
 }
 void tst_qfile::readBigFile_Win32() { readBigFile(); }
 
@@ -509,13 +513,13 @@ void tst_qfile::readSmallFiles_QFSFileEngine()
     QSKIP("This test requires -developer-build.");
 #endif
 }
-void tst_qfile::readSmallFiles_posix() 
+void tst_qfile::readSmallFiles_posix()
 {
-    readSmallFiles(); 
+    readSmallFiles();
 }
-void tst_qfile::readSmallFiles_Win32() 
-{ 
-    readSmallFiles(); 
+void tst_qfile::readSmallFiles_Win32()
+{
+    readSmallFiles();
 }
 
 void tst_qfile::readSmallFiles_QFile_data()
