@@ -44,6 +44,8 @@
 #include <QTableView>
 #include <QImage>
 #include <QPainter>
+#include <QHeaderView>
+#include <QStandardItemModel>
 
 class QtTestTableModel: public QAbstractTableModel
 {
@@ -149,6 +151,7 @@ private slots:
     void columnInsertion();
     void columnRemoval_data();
     void columnRemoval();
+    void sizeHintForColumnWhenHidden();
 private:
     static inline void spanInit_helper(QTableView *);
 };
@@ -174,7 +177,7 @@ void tst_QTableView::spanInit_helper(QTableView *view)
     for (int i=0; i < 40; i++) {
         view->setSpan(1+i%2, 1+4*i, 1+i%3, 2);
     }
-    
+
     for (int i=1; i < 40; i++) {
         view->setSpan(6 + i*7, 4, 4, 50);
     }
@@ -185,7 +188,7 @@ void tst_QTableView::spanInit()
     QtTestTableModel model(500, 500);
     QTableView v;
     v.setModel(&model);
-    
+
     QBENCHMARK {
         spanInit_helper(&v);
     }
@@ -196,7 +199,7 @@ void tst_QTableView::spanDraw()
     QtTestTableModel model(500, 500);
     QTableView v;
     v.setModel(&model);
-    
+
     spanInit_helper(&v);
     v.show();
     v.resize(500,500);
@@ -214,11 +217,11 @@ void tst_QTableView::spanSelectAll()
     QtTestTableModel model(500, 500);
     QTableView v;
     v.setModel(&model);
-    
+
     spanInit_helper(&v);
     v.show();
     QTest::qWait(30);
-    
+
     QBENCHMARK {
         v.selectAll();
     }
@@ -229,11 +232,11 @@ void tst_QTableView::spanSelectColumn()
     QtTestTableModel model(500, 500);
     QTableView v;
     v.setModel(&model);
-    
+
     spanInit_helper(&v);
     v.show();
     QTest::qWait(30);
-    
+
     QBENCHMARK {
         v.selectColumn(22);
     }
@@ -361,5 +364,23 @@ void tst_QTableView::columnRemoval()
     }
 }
 
+void tst_QTableView::sizeHintForColumnWhenHidden()
+{
+    QTableView view;
+    QStandardItemModel model(12500, 6);
+    for (int r = 0; r < model.rowCount(); ++r)
+        for (int c = 0; c < model.columnCount(); ++c) {
+            QStandardItem *item = new QStandardItem(QString("row %0, column %1").arg(r).arg(c));
+            model.setItem(r, c, item);
+        }
+
+    view.horizontalHeader()->setSectionResizeMode(QHeaderView::ResizeToContents);
+    view.setModel(&model);
+    QBENCHMARK_ONCE {
+        view.horizontalHeader()->resizeSection(0, 10); // this force resizeSections - on a hidden view.
+    }
+
+}
+
 QTEST_MAIN(tst_QTableView)
 #include "tst_qtableview.moc"
