@@ -50,6 +50,9 @@
 #include <QtNetwork/qtcpserver.h>
 #include "../../../../auto/network-settings.h"
 
+#ifdef QT_BUILD_INTERNAL
+#include <QtNetwork/private/qhostinfo_p.h>
+#endif
 
 Q_DECLARE_METATYPE(QSharedPointer<char>)
 
@@ -460,7 +463,9 @@ private slots:
 #ifndef QT_NO_SSL
     void echoPerformance_data();
     void echoPerformance();
-#endif
+    void preConnectEncrypted();
+#endif // !QT_NO_SSL
+    void preConnectEncrypted_data();
 
     void downloadPerformance();
     void uploadPerformance();
@@ -471,6 +476,14 @@ private slots:
     void httpDownloadPerformanceDownloadBuffer_data();
     void httpDownloadPerformanceDownloadBuffer();
     void httpsRequestChain();
+    void httpsUpload();
+    void preConnect_data();
+    void preConnect();
+
+private:
+    void runHttpsUploadRequest(const QByteArray &data, const QNetworkRequest &request);
+    QPair<QNetworkReply *, qint64> runGetRequest(QNetworkAccessManager *manager,
+                                                 const QNetworkRequest &request);
 };
 
 void tst_qnetworkreply::initTestCase()
@@ -491,6 +504,19 @@ void tst_qnetworkreply::httpLatency()
     }
 }
 
+QPair<QNetworkReply *, qint64> tst_qnetworkreply::runGetRequest(
+        QNetworkAccessManager *manager, const QNetworkRequest &request)
+{
+    QElapsedTimer timer;
+    timer.start();
+    QNetworkReply *reply = manager->get(request);
+    connect(reply, SIGNAL(sslErrors(QList<QSslError>)), reply, SLOT(ignoreSslErrors()));
+    connect(reply, SIGNAL(finished()), &QTestEventLoop::instance(), SLOT(exitLoop()), Qt::QueuedConnection);
+    QTestEventLoop::instance().enterLoop(20);
+    qint64 elapsed = timer.elapsed();
+    return qMakePair(reply, elapsed);
+}
+
 #ifndef QT_NO_SSL
 void tst_qnetworkreply::echoPerformance_data()
 {
@@ -523,8 +549,61 @@ void tst_qnetworkreply::echoPerformance()
         delete reply;
     }
 }
+
+void tst_qnetworkreply::preConnectEncrypted()
+{
+    QString hostName = QLatin1String("www.google.com");
+
+    QNetworkAccessManager manager;
+    QNetworkRequest request(QUrl("https://" + hostName));
+
+    // make sure we have a full request including
+    // DNS lookup, TCP and SSL handshakes
+#ifdef QT_BUILD_INTERNAL
+    qt_qhostinfo_clear_cache();
+#else
+    qWarning("no internal build, could not clear DNS cache. Results may not be representative.");
 #endif
 
+    // first, benchmark a normal request
+    QPair<QNetworkReply *, qint64> normalResult = runGetRequest(&manager, request);
+    QNetworkReply *normalReply = normalResult.first;
+    QVERIFY(!QTestEventLoop::instance().timeout());
+    QVERIFY(normalReply->error() == QNetworkReply::NoError);
+    qint64 normalElapsed = normalResult.second;
+
+    // clear all caches again
+#ifdef QT_BUILD_INTERNAL
+    qt_qhostinfo_clear_cache();
+#else
+    qWarning("no internal build, could not clear DNS cache. Results may not be representative.");
+#endif
+    manager.clearAccessCache();
+
+    // now try to make the connection beforehand
+    QFETCH(int, sleepTime);
+    manager.connectToHostEncrypted(hostName);
+    QTestEventLoop::instance().enterLoopMSecs(sleepTime);
+
+    // now make another request and hopefully use the existing connection
+    QPair<QNetworkReply *, qint64> preConnectResult = runGetRequest(&manager, request);
+    QNetworkReply *preConnectReply = normalResult.first;
+    QVERIFY(!QTestEventLoop::instance().timeout());
+    QVERIFY(preConnectReply->error() == QNetworkReply::NoError);
+    qint64 preConnectElapsed = preConnectResult.second;
+    qDebug() << request.url().toString() << "full request:" << normalElapsed
+             << "ms, pre-connect request:" << preConnectElapsed << "ms, difference:"
+             << (normalElapsed - preConnectElapsed) << "ms";
+}
+#endif // !QT_NO_SSL
+
+void tst_qnetworkreply::preConnectEncrypted_data()
+{
+    QTest::addColumn<int>("sleepTime");
+    QTest::newRow("2secs") << 2000; // to start a new request after preconnecting is done
+    QTest::newRow("100ms") << 100; // to start a new request while preconnecting is in-flight
+}
+
 void tst_qnetworkreply::downloadPerformance()
 {
     // unlike the above function, this one tries to send as fast as possible
@@ -794,7 +873,6 @@ void tst_qnetworkreply::httpsRequestChain()
     int count = 10;
 
     QNetworkRequest request(QUrl("https://" + QtNetworkSettings::serverName() + "/fluke.gif"));
-    //QNetworkRequest request(QUrl("https://www.nokia.com/robots.txt"));
     // Disable keep-alive so we have the full re-connecting of TCP.
     request.setRawHeader("Connection", "close");
 
@@ -804,7 +882,6 @@ void tst_qnetworkreply::httpsRequestChain()
 
     // Warm up DNS cache and then immediately start HTTP
     QHostInfo::lookupHost(QtNetworkSettings::serverName(), &helper, SLOT(doNextRequest()));
-    //QHostInfo::lookupHost("www.nokia.com", &helper, SLOT(doNextRequest()));
 
     // we can use QBENCHMARK_ONCE when we find out how to make it really run once.
     // there is still a warmup-run :(
@@ -825,7 +902,79 @@ void tst_qnetworkreply::httpsRequestChain()
 
 }
 
+void tst_qnetworkreply::runHttpsUploadRequest(const QByteArray &data, const QNetworkRequest &request)
+{
+    QNetworkReply* reply = manager.post(request, data);
+    reply->ignoreSslErrors();
+    connect(reply, SIGNAL(finished()), &QTestEventLoop::instance(), SLOT(exitLoop()));
+    QTestEventLoop::instance().enterLoop(15);
+    QVERIFY(!QTestEventLoop::instance().timeout());
+    QCOMPARE(reply->error(), QNetworkReply::NoError);
+    reply->deleteLater();
+}
 
+void tst_qnetworkreply::httpsUpload()
+{
+    QByteArray data = QByteArray(2*1024*1024+1, '\177');
+    QNetworkRequest request(QUrl("https://" + QtNetworkSettings::serverName() + "/qtest/cgi-bin/md5sum.cgi"));
+    request.setHeader(QNetworkRequest::ContentTypeHeader, "application/octet-stream");
+//    for (int a = 0; a < 10; ++a)
+//        runHttpsUploadRequest(data, request); // to warmup all TCP connections
+    QBENCHMARK {
+        runHttpsUploadRequest(data, request);
+    }
+}
+
+void tst_qnetworkreply::preConnect_data()
+{
+    preConnectEncrypted_data();
+}
+
+void tst_qnetworkreply::preConnect()
+{
+    QString hostName = QLatin1String("www.google.com");
+
+    QNetworkAccessManager manager;
+    QNetworkRequest request(QUrl("http://" + hostName));
+
+    // make sure we have a full request including
+    // DNS lookup and TCP handshake
+#ifdef QT_BUILD_INTERNAL
+    qt_qhostinfo_clear_cache();
+#else
+    qWarning("no internal build, could not clear DNS cache. Results may not be representative.");
+#endif
+
+    // first, benchmark a normal request
+    QPair<QNetworkReply *, qint64> normalResult = runGetRequest(&manager, request);
+    QNetworkReply *normalReply = normalResult.first;
+    QVERIFY(!QTestEventLoop::instance().timeout());
+    QVERIFY(normalReply->error() == QNetworkReply::NoError);
+    qint64 normalElapsed = normalResult.second;
+
+    // clear all caches again
+#ifdef QT_BUILD_INTERNAL
+    qt_qhostinfo_clear_cache();
+#else
+    qWarning("no internal build, could not clear DNS cache. Results may not be representative.");
+#endif
+    manager.clearAccessCache();
+
+    // now try to make the connection beforehand
+    QFETCH(int, sleepTime);
+    manager.connectToHost(hostName);
+    QTestEventLoop::instance().enterLoopMSecs(sleepTime);
+
+    // now make another request and hopefully use the existing connection
+    QPair<QNetworkReply *, qint64> preConnectResult = runGetRequest(&manager, request);
+    QNetworkReply *preConnectReply = normalResult.first;
+    QVERIFY(!QTestEventLoop::instance().timeout());
+    QVERIFY(preConnectReply->error() == QNetworkReply::NoError);
+    qint64 preConnectElapsed = preConnectResult.second;
+    qDebug() << request.url().toString() << "full request:" << normalElapsed
+             << "ms, pre-connect request:" << preConnectElapsed << "ms, difference:"
+             << (normalElapsed - preConnectElapsed) << "ms";
+}
 
 QTEST_MAIN(tst_qnetworkreply)
 
