@@ -3,7 +3,7 @@
 ** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
-** This file is part of the examples of the Qt Toolkit.
+** This file is part of the test suite of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
@@ -40,14 +40,15 @@
 ****************************************************************************/
 
 #include "filedialogpanel.h"
+#include "utils.h"
 
 #include <QGridLayout>
 #include <QVBoxLayout>
 #include <QHBoxLayout>
+#include <QGridLayout>
 #include <QFormLayout>
 #include <QSpacerItem>
 #include <QGroupBox>
-#include <QComboBox>
 #include <QPushButton>
 #include <QCheckBox>
 #include <QLineEdit>
@@ -59,25 +60,19 @@
 #include <QTimer>
 #include <QDebug>
 
-struct ComboData
-{
-    const char *description;
-    int value;
-};
-
-const ComboData acceptModeComboData[] =
+const FlagData acceptModeComboData[] =
 {
 {"AcceptOpen", QFileDialog::AcceptOpen },
 {"AcceptSave", QFileDialog::AcceptSave }
 };
 
-const ComboData viewModeComboData[] =
+const FlagData viewModeComboData[] =
 {
     {"Detail", QFileDialog::Detail},
     {"List", QFileDialog::List}
 };
 
-const ComboData fileModeComboData[] =
+const FlagData fileModeComboData[] =
 {
     {"AnyFile", QFileDialog::AnyFile},
     {"ExistingFile", QFileDialog::ExistingFile},
@@ -86,30 +81,13 @@ const ComboData fileModeComboData[] =
     {"DirectoryOnly", QFileDialog::DirectoryOnly}
 };
 
-static QComboBox *createCombo(QWidget *parent, const ComboData *d, size_t size)
-{
-    QComboBox *c = new QComboBox(parent);
-    for (size_t i = 0; i < size; ++i)
-        c->addItem(QLatin1String(d[i].description), QVariant(d[i].value));
-    return c;
-}
-
-template <class Enum>
-Enum comboBoxValue(const QComboBox *c)
-{
-    return static_cast<Enum>(c->itemData(c->currentIndex()).toInt());
-}
-
-inline void setComboBoxValue(QComboBox *c, int v)
-{
-    c->setCurrentIndex(c->findData(QVariant(v)));
-}
-
-static inline void addButton(const QString &description, QBoxLayout *layout, QObject *receiver, const char *slotFunc)
+static inline QPushButton *addButton(const QString &description, QGridLayout *layout,
+                                     int &row, int column, QObject *receiver, const char *slotFunc)
 {
     QPushButton *button = new QPushButton(description);
     QObject::connect(button, SIGNAL(clicked()), receiver, slotFunc);
-    layout->addWidget(button);
+    layout->addWidget(button, row++, column);
+    return button;
 }
 
 // A line edit for editing the label fields of the dialog, keeping track of whether it has
@@ -146,19 +124,25 @@ private:
 
 FileDialogPanel::FileDialogPanel(QWidget *parent)
     : QWidget(parent)
+    , m_showDirsOnly(new QCheckBox(tr("Show dirs only")))
     , m_readOnly(new QCheckBox(tr("Read only")))
     , m_confirmOverWrite(new QCheckBox(tr("Confirm overwrite")))
     , m_nameFilterDetailsVisible(new QCheckBox(tr("Name filter details visible")))
     , m_resolveSymLinks(new QCheckBox(tr("Resolve symlinks")))
     , m_native(new QCheckBox(tr("Use native dialog")))
-    , m_acceptMode(createCombo(this, acceptModeComboData, sizeof(acceptModeComboData)/sizeof(ComboData)))
-    , m_fileMode(createCombo(this, fileModeComboData, sizeof(fileModeComboData)/sizeof(ComboData)))
-    , m_viewMode(createCombo(this, viewModeComboData, sizeof(viewModeComboData)/sizeof(ComboData)))
+    , m_customDirIcons(new QCheckBox(tr("Don't use custom directory icons")))
+    , m_acceptMode(createCombo(this, acceptModeComboData, sizeof(acceptModeComboData)/sizeof(FlagData)))
+    , m_fileMode(createCombo(this, fileModeComboData, sizeof(fileModeComboData)/sizeof(FlagData)))
+    , m_viewMode(createCombo(this, viewModeComboData, sizeof(viewModeComboData)/sizeof(FlagData)))
+    , m_allowedSchemes(new QLineEdit(this))
     , m_defaultSuffix(new QLineEdit(this))
     , m_directory(new QLineEdit(this))
     , m_selectedFileName(new QLineEdit(this))
+    , m_useMimeTypeFilters(new QCheckBox(this))
     , m_nameFilters(new QPlainTextEdit)
     , m_selectedNameFilter(new QLineEdit(this))
+    , m_deleteNonModalDialogButton(0)
+    , m_deleteModalDialogButton(0)
 {
     // Options
     QGroupBox *optionsGroupBox = new QGroupBox(tr("Options"));
@@ -166,18 +150,22 @@ FileDialogPanel::FileDialogPanel(QWidget *parent)
     optionsLayout->addRow(tr("AcceptMode:"), m_acceptMode);
     optionsLayout->addRow(tr("FileMode:"), m_fileMode);
     optionsLayout->addRow(tr("ViewMode:"), m_viewMode);
+    optionsLayout->addRow(tr("Allowed Schemes:"), m_allowedSchemes);
+    optionsLayout->addRow(m_showDirsOnly);
     optionsLayout->addRow(m_native);
     optionsLayout->addRow(m_confirmOverWrite);
     optionsLayout->addRow(m_nameFilterDetailsVisible);
     optionsLayout->addRow(m_resolveSymLinks);
     optionsLayout->addRow(m_readOnly);
+    optionsLayout->addRow(m_customDirIcons);
 
     // Files
     QGroupBox *filesGroupBox = new QGroupBox(tr("Files / Filters"));
-    QFormLayout *filesLayout = new QFormLayout(filesGroupBox);
+    filesLayout = new QFormLayout(filesGroupBox);
     filesLayout->addRow(tr("Default Suffix:"), m_defaultSuffix);
     filesLayout->addRow(tr("Directory:"), m_directory);
     filesLayout->addRow(tr("Selected file:"), m_selectedFileName);
+    filesLayout->addRow(tr("Use mime type filters:"), m_useMimeTypeFilters);
     m_nameFilters->setMaximumHeight(80);
     filesLayout->addRow(tr("Name filters:"), m_nameFilters);
     filesLayout->addRow(tr("Selected name filter:"), m_selectedNameFilter);
@@ -197,19 +185,28 @@ FileDialogPanel::FileDialogPanel(QWidget *parent)
     labelsLayout->addRow(tr("Reject label:"), m_labelLineEdits.back());
 
     // Buttons
-    QVBoxLayout *buttonLayout = new QVBoxLayout;
-    buttonLayout->addSpacerItem(new QSpacerItem(0, 0, QSizePolicy::Ignored, QSizePolicy::MinimumExpanding));
-    addButton(tr("Show modal"), buttonLayout, this, SLOT(showModal()));
-    addButton(tr("Show non-modal"), buttonLayout, this, SLOT(showNonModal()));
-    addButton(tr("getOpenFileName"), buttonLayout, this, SLOT(getOpenFileName()));
-    addButton(tr("getOpenFileNames"), buttonLayout, this, SLOT(getOpenFileNames()));
-    addButton(tr("getSaveFileName"), buttonLayout, this, SLOT(getSaveFileName()));
-    addButton(tr("getExistingDirectory"), buttonLayout, this, SLOT(getExistingDirectory()));
-    addButton(tr("Restore defaults"), buttonLayout, this, SLOT(restoreDefaults()));
     QGroupBox *buttonsGroupBox = new QGroupBox(tr("Show"));
-    QHBoxLayout *buttonsGroupLayout = new QHBoxLayout(buttonsGroupBox);
-    buttonsGroupLayout->addSpacerItem(new QSpacerItem(0, 0, QSizePolicy::MinimumExpanding, QSizePolicy::Ignored));
-    buttonsGroupLayout->addLayout(buttonLayout);
+    QGridLayout *buttonLayout = new QGridLayout(buttonsGroupBox);
+    int row = 0;
+    int column = 0;
+    addButton(tr("Exec modal"), buttonLayout, row, column, this, SLOT(execModal()));
+    addButton(tr("Show modal"), buttonLayout, row, column, this, SLOT(showModal()));
+    m_deleteModalDialogButton =
+        addButton(tr("Delete modal"), buttonLayout, row, column, this, SLOT(deleteModalDialog()));
+    addButton(tr("Show non-modal"), buttonLayout, row, column, this, SLOT(showNonModal()));
+    m_deleteNonModalDialogButton =
+        addButton(tr("Delete non-modal"), buttonLayout, row, column, this, SLOT(deleteNonModalDialog()));
+    row = 0;
+    column++;
+    addButton(tr("getOpenFileName"), buttonLayout, row, column, this, SLOT(getOpenFileName()));
+    addButton(tr("getOpenFileUrl"), buttonLayout, row, column, this, SLOT(getOpenFileUrl()));
+    addButton(tr("getOpenFileNames"), buttonLayout, row, column, this, SLOT(getOpenFileNames()));
+    addButton(tr("getOpenFileUrls"), buttonLayout, row, column, this, SLOT(getOpenFileUrls()));
+    addButton(tr("getSaveFileName"), buttonLayout, row, column, this, SLOT(getSaveFileName()));
+    addButton(tr("getSaveFileUrl"), buttonLayout, row, column, this, SLOT(getSaveFileUrl()));
+    addButton(tr("getExistingDirectory"), buttonLayout, row, column, this, SLOT(getExistingDirectory()));
+    addButton(tr("getExistingDirectoryUrl"), buttonLayout, row, column, this, SLOT(getExistingDirectoryUrl()));
+    addButton(tr("Restore defaults"), buttonLayout, row, column, this, SLOT(restoreDefaults()));
 
     // Main layout
     QGridLayout *gridLayout = new QGridLayout(this);
@@ -218,26 +215,78 @@ FileDialogPanel::FileDialogPanel(QWidget *parent)
     gridLayout->addWidget(labelsGroupBox, 1, 0);
     gridLayout->addWidget(buttonsGroupBox, 1, 1);
 
+    connect(m_useMimeTypeFilters, SIGNAL(toggled(bool)), this, SLOT(useMimeTypeFilters(bool)));
+
+    enableDeleteModalDialogButton();
+    enableDeleteNonModalDialogButton();
     restoreDefaults();
 }
 
-void FileDialogPanel::showModal()
+void FileDialogPanel::execModal()
 {
     QFileDialog dialog(this);
     applySettings(&dialog);
+    connect(&dialog, SIGNAL(accepted()), this, SLOT(accepted()));
     dialog.setWindowTitle(tr("Modal File Dialog Qt %1").arg(QLatin1String(QT_VERSION_STR)));
     dialog.exec();
 }
 
+void FileDialogPanel::showModal()
+{
+    if (m_modalDialog.isNull()) {
+        static int  n = 0;
+        m_modalDialog = new QFileDialog(this);
+        m_modalDialog->setModal(true);
+        connect(m_modalDialog.data(), SIGNAL(accepted()), this, SLOT(accepted()));
+        m_modalDialog->setWindowTitle(tr("Modal File Dialog #%1 Qt %2")
+                                      .arg(++n)
+                                      .arg(QLatin1String(QT_VERSION_STR)));
+        enableDeleteModalDialogButton();
+    }
+    applySettings(m_modalDialog);
+    m_modalDialog->show();
+}
+
 void FileDialogPanel::showNonModal()
 {
-    QFileDialog *dialog = new QFileDialog(this);
-    dialog->setAttribute(Qt::WA_DeleteOnClose);
-    applySettings(dialog);
-    dialog->setWindowTitle(tr("Non-Modal File Dialog Qt %1").arg(QLatin1String(QT_VERSION_STR)));
-    dialog->show();
+    if (m_nonModalDialog.isNull()) {
+        static int  n = 0;
+        m_nonModalDialog = new QFileDialog(this);
+        connect(m_nonModalDialog.data(), SIGNAL(accepted()), this, SLOT(accepted()));
+        m_nonModalDialog->setWindowTitle(tr("Non-Modal File Dialog #%1 Qt %2")
+                                         .arg(++n)
+                                         .arg(QLatin1String(QT_VERSION_STR)));
+        enableDeleteNonModalDialogButton();
+    }
+    applySettings(m_nonModalDialog);
+    m_nonModalDialog->show();
 }
 
+void FileDialogPanel::deleteNonModalDialog()
+{
+    if (!m_nonModalDialog.isNull())
+        delete m_nonModalDialog;
+    enableDeleteNonModalDialogButton();
+}
+
+void FileDialogPanel::deleteModalDialog()
+{
+    if (!m_modalDialog.isNull())
+        delete m_modalDialog;
+    enableDeleteModalDialogButton();
+}
+
+void FileDialogPanel::enableDeleteNonModalDialogButton()
+{
+    m_deleteNonModalDialogButton->setEnabled(!m_nonModalDialog.isNull());
+}
+
+void FileDialogPanel::enableDeleteModalDialogButton()
+{
+    m_deleteModalDialogButton->setEnabled(!m_modalDialog.isNull());
+}
+
+
 QString FileDialogPanel::filterString() const
 {
     return m_nameFilters->toPlainText().trimmed().replace(QLatin1String("\n"), QLatin1String(";;"));
@@ -246,6 +295,8 @@ QString FileDialogPanel::filterString() const
 QFileDialog::Options FileDialogPanel::options() const
 {
     QFileDialog::Options result;
+    if (m_showDirsOnly->isChecked())
+        result |= QFileDialog::ShowDirsOnly;
     if (!m_nameFilterDetailsVisible->isChecked())
         result |= QFileDialog::HideNameFilterDetails;
     if (!m_resolveSymLinks->isChecked())
@@ -256,9 +307,16 @@ QFileDialog::Options FileDialogPanel::options() const
         result |= QFileDialog::DontConfirmOverwrite;
     if (!m_native->isChecked())
         result |= QFileDialog::DontUseNativeDialog;
+    if (!m_customDirIcons->isChecked())
+        result |= QFileDialog::DontUseCustomDirectoryIcons;
     return result;
 }
 
+QStringList FileDialogPanel::allowedSchemes() const
+{
+    return m_allowedSchemes->text().simplified().split(' ', QString::SkipEmptyParts);
+}
+
 void FileDialogPanel::getOpenFileNames()
 {
     QString selectedFilter = m_selectedNameFilter->text().trimmed();
@@ -274,6 +332,24 @@ void FileDialogPanel::getOpenFileNames()
     }
 }
 
+void FileDialogPanel::getOpenFileUrls()
+{
+#if QT_VERSION >= 0x050000
+    QString selectedFilter = m_selectedNameFilter->text().trimmed();
+    const QList<QUrl> files =
+        QFileDialog::getOpenFileUrls(this, tr("getOpenFileNames Qt %1").arg(QLatin1String(QT_VERSION_STR)),
+                                      QUrl(m_directory->text()), filterString(), &selectedFilter, options(),
+                                      allowedSchemes());
+    if (!files.isEmpty()) {
+        QString result;
+        QDebug(&result).nospace()
+            << "Files: " << QUrl::toStringList(files)
+            << "\nName filter: " << selectedFilter;
+        QMessageBox::information(this, tr("getOpenFileNames"), result, QMessageBox::Ok);
+    }
+#endif // Qt 5
+}
+
 void FileDialogPanel::getOpenFileName()
 {
     QString selectedFilter = m_selectedNameFilter->text().trimmed();
@@ -289,6 +365,24 @@ void FileDialogPanel::getOpenFileName()
     }
 }
 
+void FileDialogPanel::getOpenFileUrl()
+{
+#if QT_VERSION >= 0x050000
+    QString selectedFilter = m_selectedNameFilter->text().trimmed();
+    const QUrl file =
+        QFileDialog::getOpenFileUrl(this, tr("getOpenFileUrl Qt %1").arg(QLatin1String(QT_VERSION_STR)),
+                                      QUrl(m_directory->text()), filterString(), &selectedFilter, options(),
+                                      allowedSchemes());
+    if (file.isValid()) {
+        QString result;
+        QDebug(&result).nospace()
+            << "File: " << file.toString()
+            << "\nName filter: " << selectedFilter;
+        QMessageBox::information(this, tr("getOpenFileName"), result, QMessageBox::Ok);
+    }
+#endif // Qt 5
+}
+
 void FileDialogPanel::getSaveFileName()
 {
     QString selectedFilter = m_selectedNameFilter->text().trimmed();
@@ -304,6 +398,24 @@ void FileDialogPanel::getSaveFileName()
     }
 }
 
+void FileDialogPanel::getSaveFileUrl()
+{
+#if QT_VERSION >= 0x050000
+    QString selectedFilter = m_selectedNameFilter->text().trimmed();
+    const QUrl file =
+        QFileDialog::getSaveFileUrl(this, tr("getSaveFileName Qt %1").arg(QLatin1String(QT_VERSION_STR)),
+                                    QUrl(m_directory->text()), filterString(), &selectedFilter, options(),
+                                    allowedSchemes());
+    if (file.isValid()) {
+        QString result;
+        QDebug(&result).nospace()
+            << "File: " << file.toString()
+            << "\nName filter: " << selectedFilter;
+        QMessageBox::information(this, tr("getSaveFileNames"), result, QMessageBox::Ok);
+    }
+#endif // Qt 5
+}
+
 void FileDialogPanel::getExistingDirectory()
 {
     const QString dir =
@@ -313,20 +425,36 @@ void FileDialogPanel::getExistingDirectory()
         QMessageBox::information(this, tr("getExistingDirectory"), QLatin1String("Directory: ") + dir, QMessageBox::Ok);
 }
 
+void FileDialogPanel::getExistingDirectoryUrl()
+{
+#if QT_VERSION >= 0x050000
+    const QUrl dir =
+        QFileDialog::getExistingDirectoryUrl(this, tr("getExistingDirectory Qt %1").arg(QLatin1String(QT_VERSION_STR)),
+                                          QUrl(m_directory->text()), options() | QFileDialog::ShowDirsOnly,
+                                          allowedSchemes());
+    if (!dir.isEmpty())
+        QMessageBox::information(this, tr("getExistingDirectory"), QLatin1String("Directory: ") + dir.toString(), QMessageBox::Ok);
+#endif // Qt 5
+}
+
 void FileDialogPanel::restoreDefaults()
 {
     QFileDialog d;
     setComboBoxValue(m_acceptMode, d.acceptMode());
     setComboBoxValue(m_fileMode, d.fileMode());
     setComboBoxValue(m_viewMode, d.viewMode());
+    m_showDirsOnly->setChecked(d.testOption(QFileDialog::ShowDirsOnly));
+    m_allowedSchemes->setText(QString());
     m_confirmOverWrite->setChecked(d.confirmOverwrite());
     m_nameFilterDetailsVisible->setChecked(d.isNameFilterDetailsVisible());
     m_resolveSymLinks->setChecked(d.resolveSymlinks());
     m_readOnly->setChecked(d.isReadOnly());
     m_native->setChecked(true);
+    m_customDirIcons->setChecked(d.testOption(QFileDialog::DontUseCustomDirectoryIcons));
     m_directory->setText(QDir::homePath());
-    m_defaultSuffix->setText(QLatin1String(".txt"));
-    m_nameFilters->setPlainText(QLatin1String("Any files (*)\nImage files (*.png *.xpm *.jpg)\nText files (*.txt)"));
+    m_defaultSuffix->setText(QLatin1String("txt"));
+    m_useMimeTypeFilters->setChecked(false);
+    useMimeTypeFilters(false);
     m_selectedFileName->setText(QString());
     m_selectedNameFilter->setText(QString());
     foreach (LabelLineEdit *l, m_labelLineEdits)
@@ -340,17 +468,42 @@ void FileDialogPanel::applySettings(QFileDialog *d) const
     d->setFileMode(comboBoxValue<QFileDialog::FileMode>(m_fileMode));
     d->setOptions(options());
     d->setDefaultSuffix(m_defaultSuffix->text().trimmed());
-    d->setDirectory(m_directory->text().trimmed());
+    const QString directory = m_directory->text().trimmed();
+    if (!directory.isEmpty())
+        d->setDirectory(directory);
     const QString file = m_selectedFileName->text().trimmed();
     if (!file.isEmpty())
        d->selectFile(file);
-    d->setNameFilters(m_nameFilters->toPlainText().trimmed().split(QLatin1Char('\n'), QString::SkipEmptyParts));
     const QString filter = m_selectedNameFilter->text().trimmed();
-    if (!filter.isEmpty())
-        d->selectNameFilter(filter);
+    const QStringList filters = m_nameFilters->toPlainText().trimmed().split(QLatin1Char('\n'), QString::SkipEmptyParts);
+    if (!m_useMimeTypeFilters->isChecked()) {
+        d->setNameFilters(filters);
+        if (!filter.isEmpty())
+            d->selectNameFilter(filter);
+    } else {
+#if QT_VERSION >= 0x050000
+        d->setMimeTypeFilters(filters);
+        if (!filter.isEmpty())
+            d->selectMimeTypeFilter(filter);
+#endif // Qt 5
+    }
     foreach (LabelLineEdit *l, m_labelLineEdits)
         l->apply(d);
-    connect(d, SIGNAL(accepted()), this, SLOT(accepted()));
+}
+
+void FileDialogPanel::useMimeTypeFilters(bool b)
+{
+    QWidget *textEdit = filesLayout->labelForField(m_nameFilters);
+    if (QLabel *label = qobject_cast<QLabel *>(textEdit))
+        label->setText(b ? tr("Mime type filters:") : tr("Name filters:"));
+    QWidget *w = filesLayout->labelForField(m_selectedNameFilter);
+    if (QLabel *label = qobject_cast<QLabel *>(w))
+        label->setText(b ? tr("Selected mime type filter:") : tr("Selected name filter:"));
+
+    if (b)
+        m_nameFilters->setPlainText(QLatin1String("image/jpeg\nimage/png\ntext/plain\napplication/octet-stream"));
+    else
+        m_nameFilters->setPlainText(QLatin1String("Any files (*)\nImage files (*.png *.xpm *.jpg)\nText files (*.txt)"));
 }
 
 void FileDialogPanel::accepted()
