@@ -3,7 +3,7 @@
 ** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
-** This file is part of the examples of the Qt Toolkit.
+** This file is part of the test suite of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
@@ -44,11 +44,16 @@
 
 #include <QGroupBox>
 #include <QFileDialog>
+#include <QPointer>
 
+QT_BEGIN_NAMESPACE
+class QPushButton;
 class QCheckBox;
 class QComboBox;
 class QLineEdit;
 class QPlainTextEdit;
+class QFormLayout;
+QT_END_NAMESPACE
 class LabelLineEdit;
 
 class FileDialogPanel : public QWidget
@@ -58,36 +63,58 @@ public:
     explicit FileDialogPanel(QWidget *parent = 0);
 
 public slots:
+    void execModal();
     void showModal();
     void showNonModal();
+    void deleteNonModalDialog();
+    void deleteModalDialog();
     void getOpenFileNames();
+    void getOpenFileUrls();
     void getOpenFileName();
+    void getOpenFileUrl();
     void getSaveFileName();
+    void getSaveFileUrl();
     void getExistingDirectory();
+    void getExistingDirectoryUrl();
     void accepted();
     void showAcceptedResult();
     void restoreDefaults();
 
+private slots:
+    void enableDeleteNonModalDialogButton();
+    void enableDeleteModalDialogButton();
+    void useMimeTypeFilters(bool);
+
 private:
     QString filterString() const;
     QFileDialog::Options options() const;
+    QStringList allowedSchemes() const;
     void applySettings(QFileDialog *d) const;
 
+    QFormLayout *filesLayout;
+    QCheckBox *m_showDirsOnly;
     QCheckBox *m_readOnly;
     QCheckBox *m_confirmOverWrite;
     QCheckBox *m_nameFilterDetailsVisible;
     QCheckBox *m_resolveSymLinks;
     QCheckBox *m_native;
+    QCheckBox *m_customDirIcons;
     QComboBox *m_acceptMode;
     QComboBox *m_fileMode;
     QComboBox *m_viewMode;
+    QLineEdit *m_allowedSchemes;
     QLineEdit *m_defaultSuffix;
     QLineEdit *m_directory;
     QLineEdit *m_selectedFileName;
     QList<LabelLineEdit *> m_labelLineEdits;
+    QCheckBox *m_useMimeTypeFilters;
     QPlainTextEdit *m_nameFilters;
     QLineEdit *m_selectedNameFilter;
+    QPushButton *m_deleteNonModalDialogButton;
+    QPushButton *m_deleteModalDialogButton;
     QString m_result;
+    QPointer<QFileDialog> m_modalDialog;
+    QPointer<QFileDialog> m_nonModalDialog;
 };
 
 #endif // FILEDIALOGPANEL_H
