@@ -3,7 +3,7 @@
 ** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
-** This file is part of the examples of the Qt Toolkit.
+** This file is part of the test suite of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
@@ -40,6 +40,11 @@
 ****************************************************************************/
 
 #include "filedialogpanel.h"
+#include "colordialogpanel.h"
+#include "fontdialogpanel.h"
+#include "printdialogpanel.h"
+#include "wizardpanel.h"
+#include "messageboxpanel.h"
 
 #include <QMainWindow>
 #include <QApplication>
@@ -50,7 +55,6 @@
 #include <QKeySequence>
 
 // Test for dialogs, allowing to play with all dialog options for implementing native dialogs.
-// Currently, only QFileDialog is implemented.
 // Compiles with Qt 4.8 and Qt 5.
 
 class MainWindow : public QMainWindow {
@@ -68,6 +72,13 @@ MainWindow::MainWindow(QWidget *parent) : QMainWindow(parent)
     connect(quitAction, SIGNAL(triggered()), qApp, SLOT(quit()));
     QTabWidget *tabWidget = new QTabWidget;
     tabWidget->addTab(new FileDialogPanel, tr("QFileDialog"));
+    tabWidget->addTab(new ColorDialogPanel, tr("QColorDialog"));
+    tabWidget->addTab(new FontDialogPanel, tr("QFontDialog"));
+    tabWidget->addTab(new WizardPanel, tr("QWizard"));
+    tabWidget->addTab(new MessageBoxPanel, tr("QMessageBox"));
+#ifndef QT_NO_PRINTER
+    tabWidget->addTab(new PrintDialogPanel, tr("QPrintDialog"));
+#endif
     setCentralWidget(tabWidget);
 }
 
