@@ -39,7 +39,8 @@
 **
 ****************************************************************************/
 
-#include <QtWidgets>
+#include <QMainWindow>
+#include <QApplication>
 #include "window.h"
 
 int main(int argc, char **argv)
