@@ -3,7 +3,7 @@
 ** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
-** This file is part of the examples of the Qt Toolkit.
+** This file is part of the test suite of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:BSD$
 ** You may use this file under the terms of the BSD license as follows:
@@ -41,7 +41,17 @@
 #ifndef CALENDAR_H
 #define CALENDAR_H
 
-#include <QtWidgets>
+#include <QWidget>
+#include <QDate>
+#include <QLocale>
+
+class QComboBox;
+class QGridLayout;
+class QGroupBox;
+class QCalendarWidget;
+class QLabel;
+class QDateEdit;
+class QCheckBox;
 
 class CalendarWidget : public QWidget
 {
