@@ -3,7 +3,7 @@
 ** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
-** This file is part of the examples of the Qt Toolkit.
+** This file is part of the test suite of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:BSD$
 ** You may use this file under the terms of the BSD license as follows:
@@ -41,7 +41,11 @@
 #ifndef CURRENCY_H
 #define CURRENCY_H
 
-#include <QtWidgets>
+#include <QWidget>
+#include <QLocale>
+
+class QLabel;
+class QLineEdit;
 
 class CurrencyWidget : public QWidget
 {
