@@ -39,7 +39,7 @@
 **
 ****************************************************************************/
 
-#include <QtWidgets/QApplication>
+#include <QApplication>
 #include "widget.h"
 
 int main(int argc, char *argv[])
