@@ -39,7 +39,11 @@
 **
 ****************************************************************************/
 
-#include <QtWidgets>
+#include <QWidget>
+#include <QPainter>
+#include <QMouseEvent>
+#include <QDebug>
+
 class StaticWidget : public QWidget
 {
 Q_OBJECT
