@@ -173,7 +173,7 @@ language_list = {
     125 : [ "Turkish",            "tr" ],
     126 : [ "Turkmen",            "tk" ],
     127 : [ "Tahitian",           "ty" ],
-    128 : [ "Uigur",              "ug" ],
+    128 : [ "Uighur",             "ug" ],
     129 : [ "Ukrainian",          "uk" ],
     130 : [ "Urdu",               "ur" ],
     131 : [ "Uzbek",              "uz" ], # macrolanguage
@@ -305,7 +305,58 @@ language_list = {
     257 : [ "Ngomba",             "jgo" ],
     258 : [ "Kako",               "kkj" ],
     259 : [ "Meta",               "mgo" ],
-    260 : [ "Ngiemboon",          "nnh" ]
+    260 : [ "Ngiemboon",          "nnh" ],
+    261 : [ "Aragonese",          "an" ],
+    262 : [ "Akkadian",           "akk" ],
+    263 : [ "AncientEgyptian",    "egy" ],
+    264 : [ "AncientGreek",       "grc" ],
+    265 : [ "Aramaic",            "arc" ],
+    266 : [ "Balinese",           "ban" ],
+    267 : [ "Bamun",              "bax" ],
+    268 : [ "BatakToba",          "bbc" ],
+    269 : [ "Buginese",           "bug" ],
+    270 : [ "Buhid",              "bku" ],
+    271 : [ "Carian",             "xcr" ],
+    272 : [ "Chakma",             "ccp" ],
+    273 : [ "ClassicalMandaic",   "myz" ],
+    274 : [ "Coptic",             "cop" ],
+    275 : [ "Dogri",              "doi" ], # macrolanguage
+    276 : [ "EasternCham",        "cjm" ],
+    277 : [ "EasternKayah",       "eky" ],
+    278 : [ "Etruscan",           "ett" ],
+    279 : [ "Gothic",             "got" ],
+    280 : [ "Hanunoo",            "hnn" ],
+    281 : [ "Ingush",             "inh" ],
+    282 : [ "LargeFloweryMiao",   "hmd" ],
+    283 : [ "Lepcha",             "lep" ],
+    284 : [ "Limbu",              "lif" ],
+    285 : [ "Lisu",               "lis" ],
+    286 : [ "Lu",                 "khb" ],
+    287 : [ "Lycian",             "xlc" ],
+    288 : [ "Lydian",             "xld" ],
+    289 : [ "Mandingo",           "man" ], # macrolanguage
+    290 : [ "Manipuri",           "mni" ],
+    291 : [ "Meroitic",           "xmr" ],
+    292 : [ "NorthernThai",       "nod" ],
+    293 : [ "OldIrish",           "sga" ],
+    294 : [ "OldNorse",           "non" ],
+    295 : [ "OldPersian",         "peo" ],
+    296 : [ "OldTurkish",         "otk" ],
+    297 : [ "Pahlavi",            "pal" ],
+    298 : [ "Parthian",           "xpr" ],
+    299 : [ "Phoenician",         "phn" ],
+    300 : [ "PrakritLanguage",    "pra" ],
+    301 : [ "Rejang",             "rej" ],
+    302 : [ "Sabaean",            "xsa" ],
+    303 : [ "Samaritan",          "smp" ],
+    304 : [ "Santali",            "sat" ],
+    305 : [ "Saurashtra",         "saz" ],
+    306 : [ "Sora",               "srb" ],
+    307 : [ "Sylheti",            "syl" ],
+    308 : [ "Tagbanwa",           "tbw" ],
+    309 : [ "TaiDam",             "blt" ],
+    310 : [ "TaiNua",             "tdd" ],
+    311 : [ "Ugaritic",           "uga" ]
 }
 
 country_list = {
@@ -565,7 +616,8 @@ country_list = {
     253 : [ "TristanDaCunha",                           "TA"  ],
     254 : [ "SouthSudan",                               "SS"  ],
     255 : [ "Bonaire",                                  "BQ"  ],
-    256 : [ "SintMaarten",                              "SX"  ]
+    256 : [ "SintMaarten",                              "SX"  ],
+    257 : [ "Kosovo",                                   "XK"  ]
 }
 
 script_list = {
@@ -602,9 +654,84 @@ script_list = {
     30  : [ "Thai",              "Thai" ],
     31  : [ "Tibetan",           "Tibt" ],
     32  : [ "Sinhala",           "Sinh" ],
-    33  : [ "Syriac",            "Syri" ],
+    33  : [ "Syriac",            "Syrc" ],
     34  : [ "Yi",                "Yiii" ],
-    35  : [ "Vai",               "Vaii" ]
+    35  : [ "Vai",               "Vaii" ],
+    36  : [ "Avestan",                "Avst" ],
+    37  : [ "Balinese",               "Bali" ],
+    38  : [ "Bamum",                  "Bamu" ],
+    39  : [ "Batak",                  "Batk" ],
+    40  : [ "Bopomofo",               "Bopo" ],
+    41  : [ "Brahmi",                 "Brah" ],
+    42  : [ "Buginese",               "Bugi" ],
+    43  : [ "Buhid",                  "Buhd" ],
+    44  : [ "CanadianAboriginal",     "Cans" ],
+    45  : [ "Carian",                 "Cari" ],
+    46  : [ "Chakma",                 "Cakm" ],
+    47  : [ "Cham",                   "Cham" ],
+    48  : [ "Coptic",                 "Copt" ],
+    49  : [ "Cypriot",                "Cprt" ],
+    50  : [ "Egyptian Hieroglyphs",   "Egyp" ],
+    51  : [ "Fraser",                 "Lisu" ],
+    52  : [ "Glagolitic",             "Glag" ],
+    53  : [ "Gothic",                 "Goth" ],
+    54  : [ "Han",                    "Hani" ],
+    55  : [ "Hangul",                 "Hang" ],
+    56  : [ "Hanunoo",                "Hano" ],
+    57  : [ "Imperial Aramaic",       "Armi" ],
+    58  : [ "Inscriptional Pahlavi",  "Phli" ],
+    59  : [ "Inscriptional Parthian", "Prti" ],
+    60  : [ "Javanese",               "Java" ],
+    61  : [ "Kaithi",                 "Kthi" ],
+    62  : [ "Katakana",               "Kana" ],
+    63  : [ "Kayah Li",               "Kali" ],
+    64  : [ "Kharoshthi",             "Khar" ],
+    65  : [ "Lanna",                  "Lana" ],
+    66  : [ "Lepcha",                 "Lepc" ],
+    67  : [ "Limbu",                  "Limb" ],
+    68  : [ "Linear B",               "Linb" ],
+    69  : [ "Lycian",                 "Lyci" ],
+    70  : [ "Lydian",                 "Lydi" ],
+    71  : [ "Mandaean",               "Mand" ],
+    72  : [ "Meitei Mayek",           "Mtei" ],
+    73  : [ "Meroitic",               "Mero" ],
+    74  : [ "Meroitic Cursive",       "Merc" ],
+    75  : [ "Nko",                    "Nkoo" ],
+    76  : [ "New Tai Lue",            "Talu" ],
+    77  : [ "Ogham",                  "Ogam" ],
+    78  : [ "Ol Chiki",               "Olck" ],
+    79  : [ "Old Italic",             "Ital" ],
+    80  : [ "Old Persian",            "Xpeo" ],
+    81  : [ "Old South Arabian",      "Sarb" ],
+    82  : [ "Orkhon",                 "Orkh" ],
+    83  : [ "Osmanya",                "Osma" ],
+    84  : [ "Phags Pa",               "Phag" ],
+    85  : [ "Phoenician",             "Phnx" ],
+    86  : [ "Pollard Phonetic",       "Plrd" ],
+    87  : [ "Rejang",                 "Rjng" ],
+    88  : [ "Runic",                  "Runr" ],
+    89  : [ "Samaritan",              "Samr" ],
+    90  : [ "Saurashtra",             "Saur" ],
+    91  : [ "Sharada",                "Shrd" ],
+    92  : [ "Shavian",                "Shaw" ],
+    93  : [ "Sora Sompeng",           "Sora" ],
+    94  : [ "Cuneiform",              "Xsux" ],
+    95  : [ "Sundanese",              "Sund" ],
+    96  : [ "Syloti Nagri",           "Sylo" ],
+    97  : [ "Tagalog",                "Tglg" ],
+    98  : [ "Tagbanwa",               "Tagb" ],
+    99  : [ "Tai Le",                 "Tale" ],
+    100 : [ "Tai Viet",               "Tavt" ],
+    101 : [ "Takri",                  "Takr" ],
+    102 : [ "Ugaritic",               "Ugar" ],
+    103 : [ "Braille",                "Brai" ],
+    104 : [ "Hiragana",               "Hira" ]
+
+#    ### : [ "Blissymbols",            "Blis" ],
+#    ### : [ "Linear A",               "Lina" ],
+#    ### : [ "Naxi Geba",              "Nkgb" ],
+#    ### : [ "Pahawh Hmong",           "Hmng" ],
+#    ### : [ "Varang Kshiti",          "Wara" ],
 }
 
 def countryCodeToId(code):
